/*
 * '(c) 2017, SAS Institute Inc.'
 */

jQuery.sap.declare("sas.analytics.riskhp.common.riskcommons.crosstab.HprCrosstabContextMenuProvider");

jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require('sas.analytics.riskhp.common.riskcommons.CrosstabUtil');
jQuery.sap.require("sas.ltjs.commons.crosstab.CrosstabPickResultEntry");
jQuery.sap.require("sas.ltjs.commons.pickResults.PickEntryType");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Compvar");
jQuery.sap.require("sas.analytics.riskhp.sensitivities.RiskFactorSensitivitiesUtil");

// noinspection JSClosureCompilerSyntax
/**
 * @name sas.analytics.riskhp.common.riskcommons.crosstab.HprCrosstabContextMenuProvider
 * @extends sap.ui.base.Object
 */
sap.ui.base.Object.extend("sas.analytics.riskhp.common.riskcommons.crosstab.HprCrosstabContextMenuProvider", {

     COMPVAR : sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR,
     BENCHVAR : sas.analytics.riskhp.models.dataitems.Benchvar.BENCHVAR,

     CROSSTAB_UTIL : sas.analytics.riskhp.common.riskcommons.CrosstabUtil,
     HPR_UTIL : sas.analytics.riskhp.utils.HprUtil,
     CrosstabPickResultEntry : sas.ltjs.commons.crosstab.CrosstabPickResultEntry,
     PickEntryType : sas.ltjs.commons.pickResults.PickEntryType,

    metadata: {
        publicMethods: [
            'buildContextMenu'
        ]
    },

    constructor: function () {
    },

    /**
     * Helper method to create a MenuItem object
     * @param label
     * @param name
     * @param selectedHandler
     * @param singleTextTitle
     * @param startsSection
     * @returns {sas.hc.ui.unified.MenuItem}
     * @private
     */
    _createMenuItem: function (label, name, selectedHandler, singleTextTitle, startsSection) {
        var item = new sas.hc.ui.unified.MenuItem({
            text: label,
            select: selectedHandler,
            enabled: true,
            tooltip: label
        });

        if (singleTextTitle !== undefined) {
            item.singleValueText = singleTextTitle;
        }

        if (startsSection !== undefined) {
            item.setStartsSection(startsSection);
        }

        return item;
    },

    _locationForNewPanel: function (crosstabId) {
        var self = this;
        // get all main contents
        var panelsView = self.HPR_UTIL.getPanelsViewForComponent(crosstabId);
        var ids = panelsView.getIdsOfMainContainers();
        // filter out crosstabs from list of panel ids
        var validPanelIds = [];
        _.each(ids, function (panelId) {
            if (panelId.indexOf("Crosstab") === -1) {
                validPanelIds.push(panelId);
            }
        });
        var associatedId = crosstabId;
        var finalLocation = sas.risk.common.panelsview.RiskPanelsViewDock.Bottom;
        // mod the list of validPanelIds on base 2
        var tempValue = validPanelIds.length % 2;

        // If there is exactly one valid panel the new panel goes to the right of it
        if (validPanelIds.length === 1) {
            associatedId = _.last(validPanelIds);
            finalLocation = sas.risk.common.panelsview.RiskPanelsViewDock.Right;
        }
        // if there is an odd number of valid panels then add new panel to bottom of last panel
        else if (tempValue !== 0) {
            associatedId = _.last(validPanelIds);
        }
        else {
            // if even number of panels AND there ARE non-crosstab panels then add it to the bottom of the next to last panel
            if (validPanelIds.length > 0) {
                associatedId = validPanelIds[validPanelIds.length-2];
            }
            else {
                // In this case we will put the new panel at the bottom of the main view.
                // Note: If there are ONLY two crosstabs in the view, I'm not sure what the right thing to do is
                associatedId = null;
            }
        }
        return {
            id: associatedId,
            location: finalLocation
        };
    },

    _locationForNewCrosstab: function (crosstabId) {
        return {
            id: crosstabId,
            location: sas.risk.common.panelsview.RiskPanelsViewDock.Bottom
        };
    },

    _buildSimulatedValuesDataObject: function (crosstab, orientation, info) {
        return {
            type: HprConstants.HPR_SIMULATED_VALUES_PANEL,
            classVariables: crosstab.context.getClassVariables(),
            drill: crosstab.context.findRowTreePath(crosstab.context.drillnode, "name"),
            tuples: [info.tuple],
            orientation: orientation, //FIXME not needed
            displayHorizons: crosstab.context.optionDisplayHorizons.value,
            showLabels: crosstab.context.optionShowLabels.value,
            dataSourceId: crosstab.context.dataSourceId,
            sorts: crosstab.context.buildSortJSON(),
            options: crosstab.context.buildOptionsJSON(),
            filters: crosstab.context.getFilters(),
            crosstabId: crosstab.context.id,
            requests: [info.expansion]
        };
    },

    _buildPortfolioDetailsDataObject: function (crosstab, orientation, info) {
        return {
            type: HprConstants.HPR_PORTFOLIO_DETAILS_PANEL,
            classVariables: crosstab.context.getClassVariables(),
            drill: crosstab.context.findRowTreePath(crosstab.context.drillnode, "name"),
            tuples: [info.tuple],
            orientation: orientation, //FIXME not needed
            displayHorizons: crosstab.context.optionDisplayHorizons.value,
            showLabels: crosstab.context.optionShowLabels.value,
            dataSourceId: crosstab.context.dataSourceId,
            cacheId: crosstab.context.id, // Must send to midtier to access the correct cache
            sorts: crosstab.context.buildSortJSON(),
            options: crosstab.context.buildOptionsJSON(),
            filters: crosstab.context.getFilters(),
            requests: [info.expansion]
        };
    },

    _buildRiskFactorSensitivitiesDataObject: function (crosstab, info) {
        return {
            type: HprConstants.HPR_RISK_FACTOR_SENSITIVITIES_PANEL,
            classVariables: crosstab.context.getClassVariables(),
            drill: crosstab.context.findRowTreePath(crosstab.context.drillnode, "name"),
            tuples: [info.tuple],
            displayHorizons: crosstab.context.optionDisplayHorizons.value,
            showLabels: crosstab.context.optionShowLabels.value,
            dataSourceId: crosstab.context.dataSourceId,
            sorts: crosstab.context.buildSortJSON(),
            options: crosstab.context.buildOptionsJSON(),
            filters: crosstab.context.getFilters(),
            requests: [info.expansion]
        };
    },

    /**
     * Builds a sub-section of a menu structure.  This is a recursive method.
     * The menuDescriptor object must be in the form:
     * {
     *   subMenuName: {title for the containing submenu}
     *   menuItems: {[menuDescriptors | MenuItems]}
     *  }
     *  Note the MenuItems are created by a call to _createMenuItemwhich include an added property of 'singleValueText'
     *  which is the text to use for the list of MenuItems if it only contains one item and must be promoted up the
     *  menu hierarchy chain.
     * @param menuDescriptor
     * @param parentMenu
     * @param startsSection
     * @returns The MenuItem that was added to the passed in parentMenu for additional configuration if needed.
     * @private
     */
    _buildLevelMenu: function (menuDescriptor, parentMenu, startsSection) {
        var self = this;

        if (_.has(menuDescriptor, "subMenuName")) {
            if (menuDescriptor.menuItems.length > 1) {
                var subMenu = new sas.hc.ui.unified.Menu();
                _.each(menuDescriptor.menuItems, function (menuDescriptor) {
                    self._buildLevelMenu(menuDescriptor, subMenu, false); // This needs to pass a menu object
                });
                var subMenuItems = subMenu.getItems();
                if (subMenuItems.length > 1) {
                    var subMenuHolder = new sas.hc.ui.unified.MenuItem({
                        text: menuDescriptor.subMenuName,
                        tooltip: menuDescriptor.subMenuName
                    });
                    subMenuHolder.setSubmenu(subMenu);
                    subMenuHolder.setStartsSection(startsSection);
                    parentMenu.addItem(subMenuHolder);
                    return subMenuHolder;
                }
                else if (subMenuItems.length === 1) {
                    var item = subMenuItems[0];
                    if (item.singleValueText !== undefined) {
                        item.setText(item.singleValueText);
                    }
                    item.setStartsSection(startsSection);
                    parentMenu.addItem(item);
                    return item;
                }
            }
            else if (menuDescriptor.menuItems.length === 1) {
                parentMenu.addItem(menuDescriptor.menuItems[0]);
                return menuDescriptor.menuItems[0];
            }
        }
        else {
            parentMenu.addItem(menuDescriptor); // must be a MenuItem
            return menuDescriptor;
        }
    },

    /**
     * Helper method to build the compare dataItem by portfolio menu item
     * @param crosstab
     * @param info
     * @param howType
     * @param compareDataItem
     * @returns {*}
     * @privates
     */
    _buildCompareDataItemPortfolioMenuItem: function (crosstab, info, howType, compareDataItem) {
        var self = this;
//        var menuItems = [];
        var context = crosstab.context;
        var crosstabId = crosstab.getId();

        // Default to statistic if compareDataItem is not given
        var whatType = HprConstants.WhatType.Statistic;
        if (compareDataItem) {
            whatType = compareDataItem.type;
        }

//        var comparePortfolioMenuItems = [];
        // Compare stat by subporfolio
        // Compare stat by subporfolio as line chart
        return self._createMenuItem(
            context.i18n.getText("HPRisk.Crosstab.Context.Menu.Compare.Subportfolio.txt"),
            "compPortfolioByLineChart",
            function () {
                var newLocation = self._locationForNewPanel(crosstabId);
                var plotType = {
                    PlotType: HprConstants.PlotType.Comparison,
                    WhatType: whatType,
                    ByType: HprConstants.ByType.Subportfolio,
                    HowType: howType
                };
                var dataObject = self.CROSSTAB_UTIL.buildPlotDataObject(crosstab, plotType, info);
                dataObject.tuples = info.tuple;
                self.HPR_UTIL.getPanelsViewForComponent(crosstabId).addPanel(newLocation.id, newLocation.location, dataObject);
            }
        );
    },

    /**
     * Helper method to build the compare stat submenu
     * @param crosstab
     * @param info
     * @param dataItemFromColumn
     * @returns {*}
     * @privates
     */
    _buildCompareStatMenu: function (crosstab, info, dataItemFromColumn) {
        var self = this;
 //       var menuItems = [];
        var context = crosstab.context;
        var crosstabId = crosstab.getId();
//        var showLabels = context.getOptionShowLabels().value;
        var howType = HprConstants.HowType.Bar;

        // What is the dataItem closest to the leaf node in the crosstab column tree that ISN'T
        // a statistic or outvar.  That dataItem is what we will be comparing against it's sibling nodes.
        var comparisonDataItemObject = self.CROSSTAB_UTIL.getComparisonDataItemObject(info.column);

        // This is a list of tuples that represent the columns in the crosstab that are to be compared.

        var templateColumn = self.CROSSTAB_UTIL.createColumnTemplate(info.column, comparisonDataItemObject, context);
        var tupleList = context.getTuplesFromColumnTemplate(templateColumn);

        // Include menu items for portfolio comparison
        var comparePortfolioMenuItem = self._buildCompareDataItemPortfolioMenuItem(crosstab, info, HprConstants.HowType.Bar);

//        var compareDataItemMenuItems = [];


        var menuLabel = "";
        if(context.isAnyHorizonType(comparisonDataItemObject.item)) {
            howType = HprConstants.HowType.Line;
            menuLabel = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Compare.Horizon.txt");
        } else if(comparisonDataItemObject.type === sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE) {
            menuLabel = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Compare.RiskCube.txt");
        } else if(comparisonDataItemObject.type === sas.analytics.riskhp.models.dataitems.Scenario.SCENARIO) {
            menuLabel = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Compare.Scenario.txt");
        }
        // Removed per defect S1416097
        // else if(comparisonDataItemObject.type === sas.analytics.riskhp.models.dataitems.ResultSet.RESULTSET) {
        //     menuLabel = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Compare.ResultSet.txt");
        // }

        // Compare stat by horizon as line chart
        var compareStatByHorizon = undefined;
        if (menuLabel.length > 0) {
            compareStatByHorizon = self._createMenuItem(menuLabel,
                "compStatByLineChart",
                function () {
                    var newLocation = self._locationForNewPanel(crosstabId);
                    var plotType = {
                        PlotType: HprConstants.PlotType.Comparison,
                        WhatType: HprConstants.WhatType.Statistic,
                        ByType: comparisonDataItemObject,
                        HowType: howType
                    };
                    var dataObject = self.CROSSTAB_UTIL.buildPlotDataObject(crosstab, plotType, info);
                    dataObject.tuples = tupleList;
                    dataObject.templateColumn = templateColumn;
                    self.HPR_UTIL.getPanelsViewForComponent(crosstabId).addPanel(newLocation.id, newLocation.location, dataObject);
                }
            );
        }

        // All scenarios and result sets across horizon
        var compareStatForAllScenariosByHorizon = undefined;
        var scenario = context.getDataItemInfoFromTuple(info.tuple, sas.analytics.riskhp.models.dataitems.Scenario.SCENARIO);
        var horizon = context.getDataItemInfoFromTuple(info.tuple, sas.analytics.riskhp.models.dataitems.Horizon.HORIZON);
        var resultSet = context.getDataItemInfoFromTuple(info.tuple, sas.analytics.riskhp.models.dataitems.ResultSet.RESULTSET);
        if (scenario && horizon && resultSet) {

            // This is a list of tuples that represent the columns in the crosstab that are to be compared.
            var columnTemplate = self.CROSSTAB_UTIL.createColumnTemplateForScenariosAndResultSets(info.column, context);
            var matchingTuples = context.getTuplesFromColumnTemplate(columnTemplate);

            // Compare stat by horizon as line chart
            var sbhMenuLabel = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Compare.AllScenariosByHorizon.txt");
            compareStatForAllScenariosByHorizon = self._createMenuItem(sbhMenuLabel,
                "compStatForAllScenariosByHorizon",
                function () {
                    var newLocation = self._locationForNewPanel(crosstabId);
                    var plotType = {
                        PlotType: HprConstants.PlotType.Comparison,
                        WhatType: HprConstants.WhatType.Statistic,
                        ByType: [sas.analytics.riskhp.models.dataitems.ResultSet.RESULTSET,sas.analytics.riskhp.models.dataitems.Scenario.SCENARIO,sas.analytics.riskhp.models.dataitems.Horizon.HORIZON],
                        HowType: HprConstants.HowType.Line
                    };
                    var dataObject = self.CROSSTAB_UTIL.buildPlotDataObject(crosstab, plotType, info);
                    dataObject.tuples = matchingTuples;
                    dataObject.templateColumn = columnTemplate;
                    self.HPR_UTIL.getPanelsViewForComponent(crosstabId).addPanel(newLocation.id, newLocation.location, dataObject);
                }
            );

        }

        if (compareStatByHorizon === undefined && compareStatForAllScenariosByHorizon === undefined) {
            var statMenuLabelName = self._getCompareStatMenuName(info, crosstab);
            var csbsMenuLabel = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Compare.Stat.Subportfolio.fmt", statMenuLabelName);
            comparePortfolioMenuItem.setText(csbsMenuLabel);
            return comparePortfolioMenuItem;
        }
        else {
            var menuDataItemName = self._getCompareStatMenuName(info, crosstab);
            return {
                subMenuName: context.i18n.getText("HPRisk.Crosstab.Context.Menu.Compare.Stat.TopLevel.fmt", menuDataItemName),
                menuItems: [
                    comparePortfolioMenuItem,
                    compareStatByHorizon,
                    compareStatForAllScenariosByHorizon
                ]
            };
        }
    },

    /**
     * Helper method to build the name for a statistic based menu item.  This is usually in the form of
     * some action (like Compare) the stat name and then how it will be used (like by Subportfolio).  For
     * a price cube this will return the Statistic's name/label, for a stress cube this will return
     * Outvar name/label (Statistic name/label).
     * @param info
     * @param crosstab
     * @returns {String|*}
     * @private
     */
    _getCompareStatMenuName: function(info, crosstab) {
        var self = this;
        var showLabels = crosstab.context.getOptionShowLabels().value;
        var menuDataItemNameProvider = null;
        var stat = crosstab.context.getDataItemInfoFromTuple(info.tuple, sas.analytics.riskhp.models.dataitems.Statistic.STATISTIC);
        if (stat) {
            menuDataItemNameProvider = stat;
        }
        else {
            // Look for a benchmark var instead
            menuDataItemNameProvider = crosstab.context.getDataItemInfoFromTuple(info.tuple, sas.analytics.riskhp.models.dataitems.Benchvar.BENCHVAR);
            // if that failed look for a  computed variable
            if (!menuDataItemNameProvider) {
                menuDataItemNameProvider = crosstab.context.getDataItemInfoFromTuple(info.tuple, sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR);
            }
        }
        var outvar = crosstab.context.getDataItemInfoFromTuple(info.tuple, sas.analytics.riskhp.models.dataitems.Outvar.OUTVAR);
        var menuDataItemName = "UNKNOWN";
        if (menuDataItemNameProvider) {
            menuDataItemName = menuDataItemNameProvider.dataItem.getDisplayLabel(showLabels);
        }
        if (crosstab.context.cubeDataModel.isStressCube() && stat && outvar) {
            menuDataItemName = crosstab.context.i18n.getText("HPRisk.Crosstab.Context.Menu.Compare.Stat.ForStress.ItemName.fmt", [outvar.dataItem.getDisplayLabel(showLabels), stat.dataItem.getDisplayLabel(showLabels)]);
        }
        return menuDataItemName;
    },

    /**
     * Helper method to build the compare distribution submenu
     * @param crosstab
     * @param info
     * @returns {*}
     * @privates
     */
    _buildCompareDistMenu: function (crosstab, info) {
        var self = this;
        var menuItems = [];
        var context = crosstab.context;
        var crosstabId = crosstab.getId();
        var statDataItemList = crosstab._getStatListForPlot();
//        var subportfolioMenuItem = null;
        var secondMenuItem = null;

        // What is the dataItem closest to the leaf node in the crosstab column tree that ISN'T
        // a statistic or outvar.  That dataItem is what we will be comparing against it's sibling nodes.
        var comparisonDataItemObject = self.CROSSTAB_UTIL.getComparisonDataItemObject(info.column);

        // This is a list of tuples that represent the columns in the crosstab that are to be compared.
        var templateColumn = self.CROSSTAB_UTIL.createColumnTemplate(info.column, comparisonDataItemObject, context);
        var tupleList = context.getTuplesFromColumnTemplate(templateColumn);

        var menuLabel = "";
        if(context.isAnyHorizonType(comparisonDataItemObject.item)) {
            menuLabel = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Compare.Horizon.txt");
        } else if(comparisonDataItemObject.type === sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE) {
            menuLabel = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Compare.RiskCube.txt");
        } else if(comparisonDataItemObject.type === sas.analytics.riskhp.models.dataitems.Scenario.SCENARIO) {
            menuLabel = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Compare.Scenario.txt");
        }
        // Removed per defect S1416097
        // else if(comparisonDataItemObject.type === sas.analytics.riskhp.models.dataitems.ResultSet.RESULTSET) {
        //     menuLabel = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Compare.ResultSet.txt");
        // }
        else if(comparisonDataItemObject.type === sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR) {
            menuLabel = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Compare.Compvar.txt");
        } else if(comparisonDataItemObject.type === sas.analytics.riskhp.models.dataitems.Benchvar.BENCHVAR) {
            menuLabel = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Compare.Benchvar.txt");
        }

        // If a menuLabel was created (and it is not blank) we need to create a MenuItem for it.
        if (menuLabel.length > 0) {
            secondMenuItem = self._createMenuItem(
                menuLabel,
                "CompareDistributionX",
                function () {
                    var newLocation = self._locationForNewPanel(crosstabId);
                    var plotType = {
                        PlotType: HprConstants.PlotType.Comparison,
                        WhatType: HprConstants.WhatType.Distribution,
                        ByType: comparisonDataItemObject,
                        HowType: HprConstants.HowType.Distribution
                    };
                    var dataObject = self.CROSSTAB_UTIL.buildPlotDataObject(crosstab, plotType, info);
                    dataObject.statistics = statDataItemList;
                    dataObject.tuples = tupleList;
                    dataObject.templateColumn = templateColumn;
                    self.HPR_UTIL.getPanelsViewForComponent(crosstabId).addPanel(newLocation.id, newLocation.location, dataObject);
                });
        }

        // Determine the distribution by subportfolio menuitem name based on whehter there will be a second menu item and therefore a submenu
        var menuName = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Compare.Subportfolio.txt");
        if (secondMenuItem == null) {
            menuName = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Compare.Distribution.TopLevel.Subportfolio.txt");
        }
        menuItems.push(self._createMenuItem(
            menuName,
            "compDistBySubport",
            function () {
                var newLocation = self._locationForNewPanel(crosstabId);
                var plotType = {
                    PlotType: HprConstants.PlotType.Comparison,
                    WhatType: HprConstants.WhatType.Distribution,
                    ByType: HprConstants.ByType.Subportfolio,
                    HowType: HprConstants.HowType.Distribution
                };
                var dataObject = self.CROSSTAB_UTIL.buildPlotDataObject(crosstab, plotType, info);
                dataObject.statistics = statDataItemList;
                self.HPR_UTIL.getPanelsViewForComponent(crosstabId).addPanel(newLocation.id, newLocation.location, dataObject);
            })
        );

        // If there was a second menu item make sure we add it to our menu item list
        if (secondMenuItem != null) {
            menuItems.push(secondMenuItem);
        }
        return {
            subMenuName: context.i18n.getText("HPRisk.Crosstab.Context.Menu.Compare.Distribution.TopLevel.txt"),
            menuItems: menuItems
        };
    },

    /**
     * Helper method to build the cashflow submenu
     * @param {HprCrosstab} crosstab
     * @param info
     * @returns {*}
     * @privates
     */
    _buildCashFlowMenu: function (crosstab, info) {
        var self = this;
        var context = crosstab.context;
        var cashflowLegs = context.getCashFlowLegs();
        var crosstabId = crosstab.getId();
        var cashflowMenuItem;

        if (!self.CROSSTAB_UTIL.isTupleColumnValidForCashflowPlot(info.tuple, crosstab)) {
            return null;
        }

        if (cashflowLegs) {
            if (cashflowLegs.length > 1) {
                var cashflowSubMenu = new sas.hc.ui.unified.Menu();
                _.each(cashflowLegs, function (cashflowLegDataItem) {
                    var cflMenuName = cashflowLegDataItem.name + "cashflowplot";
                    var cashflowLegDataItemMenuItem = self._createMenuItem(
                        cashflowLegDataItem.name,
                        cflMenuName,
                        function () {
                            var newLocation = self._locationForNewPanel(crosstabId);
                            var plotType = {PlotType: HprConstants.PlotType.CashFlow};
                            var dataObject = self.CROSSTAB_UTIL.buildPlotDataObject(crosstab, plotType, info);
                            dataObject.cashFlow = cashflowLegDataItem;
                            self.HPR_UTIL.getPanelsViewForComponent(crosstabId).addPanel(newLocation.id, newLocation.location, dataObject);
                        });
                    cashflowSubMenu.addItem(cashflowLegDataItemMenuItem);
                });

                cashflowMenuItem = self._createMenuItem(
                    context.i18n.getText("HPRisk.Crosstab.Context.Menu.Cash.Flow.Plots.txt"),
                    "CashFlowPlots",
                    function () {
                    });
                cashflowMenuItem.setStartsSection(true);
                cashflowMenuItem.setSubmenu(cashflowSubMenu);
                return cashflowMenuItem;
            }
            else {
                if (cashflowLegs[0] !== undefined) {
                    var menuName = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Cash.Flow.Plot.fmt", cashflowLegs[0].name);
                    cashflowMenuItem = self._createMenuItem(
                        menuName,
                        "CashFlowPlot",
                        function () {
                            var newLocation = self._locationForNewPanel(crosstabId);
                            var plotType = {PlotType: HprConstants.PlotType.CashFlow};
                            var dataObject = self.CROSSTAB_UTIL.buildPlotDataObject(crosstab, plotType, info);
                            dataObject.cashFlow = cashflowLegs[0];
                            self.HPR_UTIL.getPanelsViewForComponent(crosstabId).addPanel(newLocation.id, newLocation.location, dataObject);
                        });
                    cashflowMenuItem.setStartsSection(true);
                    return cashflowMenuItem;
                }
            }
        }
        return null;
    },

    _buildMoveColumnContextMenu: function(oMenu, crosstab, info, startsSection) {
        var self = this;
        var menuItems = [];
        var menuName;
        var menuItem;
        var context = crosstab.context;
        crosstab.logger.trace("HprCrosstab._buildMoveColumnContextMenu");

        if (info) {

            var currentNode = context.findColumnTreeNode(info.keys);
            var numberOfSiblings = 0;
            var pos = "middle";
            if (currentNode) {
                if (currentNode.parent) {
                    numberOfSiblings = currentNode.parent.children.length;
                    if (numberOfSiblings > 1) {
                        var currentIndex = _.findIndex(currentNode.parent.children, function (childNode) {
                            return childNode.name === currentNode.name;
                        });
                        if (currentIndex === 0) {
                            pos = "leftMost";
                        } else if (currentIndex === currentNode.parent.children.length - 1) {
                            pos = "rightMost";
                        }
                    } else {
                        // If there are no siblings or just one, don't even return a Move menu because it is not valid
                        return null;
                    }
                }
            }

            if (info.keys.length === 1) {
                // Move to beginning
                if (pos !== "leftMost") {
                    menuName = (context.getTranspose() === false
                        ? context.i18n.getText("HPRisk.Crosstab.Context.Menu.move.farLeft.txt")
                        : context.i18n.getText("HPRisk.Crosstab.Context.Menu.move.top.txt"));
                    menuItem = self._createMenuItem(
                        menuName,
                        "moveBegin",
                        function () {
                            crosstab.moveColumn("begin", info);
                        });
                    menuItems.push(menuItem);
                }

                // Move to end
                if (pos !== "rightMost") {
                    menuName = (context.getTranspose() === false
                        ? context.i18n.getText("HPRisk.Crosstab.Context.Menu.move.farRight.txt")
                        : context.i18n.getText("HPRisk.Crosstab.Context.Menu.move.bottom.txt"));
                    menuItem = this._createMenuItem(
                        menuName,
                        "moveEnd",
                        function () {
                            crosstab.moveColumn("end", info);
                        });
                    menuItems.push(menuItem);
                }
            }

            if (numberOfSiblings > 1) {
                // Move left
                if (pos !== "leftMost") {
                    menuName = (context.getTranspose() === false
                        ? context.i18n.getText("HPRisk.Crosstab.Context.Menu.move.left.txt")
                        : context.i18n.getText("HPRisk.Crosstab.Context.Menu.move.up.txt"));
                    menuItem = this._createMenuItem(
                        menuName,
                        "moveLeft",
                        function () {
                            crosstab.moveColumn("left", info);
                        }
                    );
                    menuItems.push(menuItem);
                }

                // Move right
                if (pos !== "rightMost") {
                    menuName = (context.getTranspose() === false
                        ? context.i18n.getText("HPRisk.Crosstab.Context.Menu.move.right.txt")
                        : context.i18n.getText("HPRisk.Crosstab.Context.Menu.move.down.txt"));

                    menuItem = this._createMenuItem(
                        menuName,
                        "moveRight",
                        function () {
                            crosstab.moveColumn("right", info);
                        }
                    );
                    menuItems.push(menuItem);
                }
            }

            // If there is only one menuItem, then the submenu will collapse and only parent menu will display.
            // The parent menu will be this one menuItem.  So fix the menu label.
            if (menuItems.length === 1) {
                var menuLabel = menuItem.getText();
                var newMenuLabel = context.i18n.getText("HPRisk.Crosstab.Context.Menu.move.txt") + " " + menuLabel;
                menuItem.setText(newMenuLabel);
                menuItem.setStartsSection(startsSection);
            }

            var menuStructure = {
                subMenuName: context.i18n.getText("HPRisk.Crosstab.Context.Submenu.move.txt"),
                menuItems: menuItems
            };
            self._buildLevelMenu(menuStructure, oMenu, startsSection);
        }
    },

    /**
     * Builds the appropriate menuitems for the given pickResults and crosstab and attaches them to the supplied oMenu
     * @param oMenu - The menu to add the menu items to
     * @param pickResults - The data for the cell that was selected
     * @param crosstab _ The crosstab to build the menu for
     */
    buildContextMenu: function(oMenu, pickResults, crosstab) {
        var self = this;
        var context = crosstab.context;

        // Make pagingPopover disappear
        if (crosstab.pagingPopover !== undefined) {
            crosstab.pagingPopover.close();
        }

        var entries = pickResults[0].getEntries();
        if (entries) {
            // Entries is an array but not sure where there would ever be more than one for a gesture/mouse event.
            // Only handle first one unless otherwise needed.
            var entry = entries[0];  // Should be a DataGridPickResultEntry
            if (entry) {
                if (entry instanceof self.CrosstabPickResultEntry) {

                    var info = self.CROSSTAB_UTIL.getPickInfo(pickResults, context);
                    switch(info.pickEntryType) {
                        case self.PickEntryType.CROSSTAB_ROW_HEADER:
                            self._buildRowHeaderMenu(oMenu, crosstab, info);
                            break;
                        case self.PickEntryType.CROSSTAB_MEASURE_HEADER:
                            self._buildMeasureHeaderContextMenu(oMenu, crosstab, info);
                            break;
                        case self.PickEntryType.CROSSTAB_COLUMN_HEADER:
                            self._buildColumnHeaderMenu(oMenu, crosstab, info);
                            break;
                        case self.PickEntryType.CROSSTAB_ROW_AXIS_LABEL_HEADER:
                            self._buildAxisLabelHeaderContextMenu(oMenu, crosstab, info);
                            break;
                        case self.PickEntryType.CROSSTAB_COLUMN_AXIS_LABEL_HEADER:
                         //   var info = CrosstabUtil.getPickInfo(pickResults, context);
                        // oMenu.addItem(this._createMenuItem("Menu1_COLUMN_AXIS_LABEL", "Menu1Name", true, function() {}));
                            break;
                        case self.PickEntryType.CROSSTAB_DATA_CELL:
                            self._buildDataCellContextMenu(oMenu, crosstab, info, entry);
                            break;
                        default:
                            crosstab.logger.warning("HprCrosstab._onContextMenu", "Unknown type");
                    }
                }
            }
        }

    },

    /**
     * Builds the top level menu item for the risk factor sensitivities table
     * @param crosstab
     * @param statFromColumn
     * @param info
     * @returns {*}
     * @private
     */
    _buildRiskFactorSensitivitiesMenuItem: function(crosstab, statFromColumn, info) {
        var self = this;

        // Do the capabilities tell us that we support sensitivities
        var capabilities = self.CROSSTAB_UTIL.getCapabilities(info.tuple, crosstab.context.cubeDataModel);
        var sensitivitiesSupported = capabilities.sensitivitiesSupported;

        if (!sensitivitiesSupported) {
            return null;
        }

        // Check to make sure column has a horizon in it
        // Note: The fact we have to to dereference into the returned object's dataItem attribute for the real
        // horizon data item is an artifact of the context's function I am using.  Unfortunate, but better than
        // copying the code.
        var horizon = crosstab.context.getAnyHorizonInfo(info.tuple);
        if (!horizon || !horizon.dataItem) {
            return null;
        }

        // Only support the base case horizon
        if (horizon.dataItem.horizonNumber !== -1  || horizon.dataItem.interval !== "baseCase") {
            return null;
        }

        var outvarDataItem = crosstab.context.getDataItemInfoFromTuple(info.tuple, sas.analytics.riskhp.models.dataitems.Outvar.OUTVAR).dataItem;
        if (outvarDataItem && !outvarDataItem.isSensitivityAnalysisSupported()) {
            return null;
        }

        var menuItem = self._createMenuItem(
            crosstab.context.i18n.getText("HPRisk.Crosstab.Context.Menu.RFS.txt"),
            "riskFactorSensitivities",
            function () {
                var crosstabId = crosstab.getId();
                // Open dialog
                var newLocation = self._locationForNewPanel(crosstabId);
                var dataObject = self._buildRiskFactorSensitivitiesDataObject(crosstab, info);
                sas.analytics.riskhp.sensitivities.RiskFactorSensitivitiesUtil.showRiskFactorsSelectorDialog(crosstabId, newLocation.id, newLocation.location, dataObject);
            }
        );
        menuItem.setStartsSection(true);
        return menuItem;
    },

    /**
     * Creates a submenu item based on the externalLink
     * @param context - crosstab context
     * @param pickInfo - pick results info about clicked data cell
     * @param externalLink - external link info from cubeDataModel
     * @returns {*} - menuItem for the externalLink or null if required parameters are missing
     * @private
     */
    _buildExternalLinkMenuItem:function(context, pickInfo, externalLink) {
        var self = this;
        var url = externalLink.baseUrl;
        var urlPropertiesList = "";
        var urlHasProperties = url.indexOf("?") !== -1;
        var parmsValid = true;

        if (externalLink.parms) {
            // Process the parms by using the names to find values.
            _.each(externalLink.parms, function (parm) {
                var propertyValue = "";

                // Check scenario
                if (parm.source === "_SCENARIO_") {
                    var scenarioInfo = _.find(pickInfo.tuple, function(tupleInfo){
                        return tupleInfo.type === "scenario";
                    });
                    if (scenarioInfo != null) {
                        // Found a scenario so use its name as the property value.
                        propertyValue = scenarioInfo.name;
                    }
                } else {
                    var cvInfo = _.find(pickInfo.expansion.key, function (cv) {
                        return parm.source.toUpperCase() === cv.name.toUpperCase();
                    });
                    if (cvInfo) {
                        var classVar = _.find(context.classVariables, function (cv) {
                            return cvInfo.id === cv.id;
                        });

                        propertyValue = classVar.dataType !== "string" ? cvInfo.value.trim() : cvInfo.value;
                    }
                }

                // Did we get a usable property value?  If we didn't and property is required, then we bail.
                // If we got a valid value, then add it to the URL property list that we are building.
                if (propertyValue.length === 0){
                    if (parm.required) {
                        parmsValid = false;
                    }
                } else {
                    // Add the user defined property name with the found property value to the url property list.
                    if (urlHasProperties) {
                        urlPropertiesList += "&" + parm.target + "=" + propertyValue;
                    } else {
                        urlPropertiesList += "?" + parm.target + "=" + propertyValue;
                        urlHasProperties = true;
                    }
                }
            });
        }

        if (parmsValid) {
            var theUrl = url + urlPropertiesList;
            return new sas.hc.ui.unified.MenuItem({
                text: externalLink.label,
                select: function () {
                    window.open(theUrl);
                    // window.open(encodeURI(url + urlPropertiesList));
                },
                enabled: true,
                tooltip: theUrl
            });
        } else {
            return null;
        }
    },


    /**
     * Creates the "External Link..." menu item and submenu of links based on externalLinls in the cubeDataModel
     * @param crosstab
     * @param info - pick results info about the clicked data cell
     * @returns {*|sas.hc.ui.unified.MenuItem}
     * @private
     */
    _buildExternalLinksMenu: function (crosstab, info) {
        var self = this;
        var context = crosstab.context;
        var externalLinksInfo = context.cubeDataModel.json.dataItems.externalLinks;

        var externalLinksSubMenu = new sas.hc.ui.unified.Menu();
        _.each(externalLinksInfo, function(externalLink) {
            var menuItem = self._buildExternalLinkMenuItem(context, info, externalLink);
            if (menuItem) {
                externalLinksSubMenu.addItem(menuItem);
            }
        });

        var externalLinksMenu = self._createMenuItem(
            context.i18n.getText("HPRisk.Crosstab.Context.Menu.External.Link.txt"),
            context.i18n.getText("HPRisk.Crosstab.Context.Menu.External.Link.txt"),
            function () {
            });
        externalLinksMenu.setStartsSection(true);
        externalLinksMenu.setEnabled(externalLinksSubMenu.getItems().length > 0);
        externalLinksMenu.setSubmenu(externalLinksSubMenu);

        return externalLinksMenu;
    },


    /**
     * Adds the appropriate MenuItem and sub-menus for the currently selected cell as identified by the entry and
     * info objects.
     * @param oMenu - the menu to add the menu items to
     * @param crosstab - The crosstab for this menu
     * @param info - the info object extracted from the pick results
     * @param entry
     */
    _buildDataCellContextMenu: function (oMenu, crosstab, info, entry) {
        var menuName,
        menuItem,
        riskFactorSensitivitiesMenuItem,
        menuItems,
        self = this,
        context = crosstab.context,
        crosstabId = crosstab.getId(),
//        columnIndex = entry.getColumnIndex(),
//        horizonDataItemList,
//        horizonTuples,
//        menuStructure,
        topLevelMenuItem,
//        showLabels = context.getOptionShowLabels().value,
        capabilities = self.CROSSTAB_UTIL .getCapabilities(info.tuple, context.cubeDataModel);

        /**
         * Internal method to isolate adding to the main menu, MenuItems that may be null, so we can do the null
         * check in one place
         * @param menuItem The MenuItem to be added.  If it is null it will not be added.
         * @private
         */
        var _addMenuItem = function(menuItem) {
            if (menuItem) {
                oMenu.addItem(menuItem);
            }
        };

        crosstab.logger.trace("HprCrosstab._buildDataCellContextMenu");

        if (capabilities === undefined) {
            crosstab.logger.error("HprCrosstab._buildDataCellContextMenu", "Unable to determine capabilities");
            return;
        }
        var distributionSupported = capabilities.distributionSupported;
        var densitySupported = capabilities.densitySupported;
        var rfimSupported = capabilities.rfimSupported;
        var detailsSupported = capabilities.detailsSupported;

        var vectorsSupported = capabilities.vectorsSupported;
        var cashFlowSupported = self.CROSSTAB_UTIL.hasCashflows(context.cubeDataModel);

        // ***************************************
        // Computed and Benchmark variables get this menu
        // ***************************************

        var compareDataItem = undefined;
        var compvarDataItemObject = context.getDataItemInfoFromTuple(info.tuple, self.COMPVAR);
        if (compvarDataItemObject) {
            compareDataItem = compvarDataItemObject.dataItem;
        }
        else {
            var benchvarDataItemObject = context.getDataItemInfoFromTuple(info.tuple, self.BENCHVAR);
            if (benchvarDataItemObject) {
                compareDataItem = benchvarDataItemObject.dataItem;
            }
        }

        if (compareDataItem) {
            var comparePortfolioMenuItem = self._createMenuItem(
                context.i18n.getText("HPRisk.Crosstab.Context.Menu.Compare.Stat.Subportfolio.fmt", self._getCompareStatMenuName(info, crosstab)),
                "compPortfolioByLineChart",
                function () {
                    var newLocation = self._locationForNewPanel(crosstabId);
                    var plotType = {
                        PlotType: HprConstants.PlotType.Comparison,
                        WhatType: compareDataItem.type,
                        ByType: HprConstants.ByType.Subportfolio,
                        HowType: HprConstants.HowType.Bar
                    };
                    var dataObject = self.CROSSTAB_UTIL.buildPlotDataObject(crosstab, plotType, info);
                    dataObject.tuples = info.tuple;
                    self.HPR_UTIL.getPanelsViewForComponent(crosstabId).addPanel(newLocation.id, newLocation.location, dataObject);
                }
            );

            oMenu.addItem(comparePortfolioMenuItem);

            // computed variable
            menuItem =  self._createMenuItem(
                context.i18n.getText("HPRisk.Crosstab.Context.Menu.Create.Computed.Variable.txt"),
                "createComputedVariable",
                function () {
                    var expCtrl = sas.analytics.riskhp.utils.ViewManager.getExplorationView().getController();
                    expCtrl.createComputedVariable.call(expCtrl, context.getId(), info);
                });
            _addMenuItem(menuItem);

            // display rule
            _addMenuItem( self._createMenuItem(
                context.i18n.getText("HPRisk.Crosstab.Context.Menu.Create.Display.Rule.txt"),
                context.i18n.getText("HPRisk.Crosstab.Context.Menu.Create.Display.Rule.txt"),
                function () {
                    var expCtrl = sas.analytics.riskhp.utils.ViewManager.getExplorationView().getController();
                    expCtrl.createDisplayRule.call(expCtrl, context.getId(), info);
                })
            );
        }
        else {

            var statFromColumnObject = context.getStatisticFromColumn(info.tuple);
            var statFromColumn = (statFromColumnObject !== undefined ? statFromColumnObject.dataItem : undefined);
            var hasColumnStat = (statFromColumn !== undefined);

            var columnDataItems = context.getColumn(info.columnIndex, true);
            var headerDataItemObject = self.CROSSTAB_UTIL.getHeaderDataItemObject(columnDataItems);

            var horizonDataItem = context.getAnyHorizonInfo(info.tuple);

            // ***************************************
            // Basecase columns get this menu
            // ***************************************

            if ((hasColumnStat) && ((horizonDataItem !== undefined && horizonDataItem.dataItem.isBasecase()) || ((headerDataItemObject !== undefined) && (headerDataItemObject.item.id === self.CROSSTAB_UTIL.BASECASEHEADER)))) {

                // Compare STAT by subportfolio
                var compareStatBySubport = self._buildCompareDataItemPortfolioMenuItem(crosstab, info, HprConstants.HowType.Bar);
                if (hasColumnStat && !self.CROSSTAB_UTIL.isStalenessStatistic(statFromColumn)) {
                    self._buildLevelMenu(self._buildCompareStatMenu(crosstab, info, statFromColumn), oMenu, false);
                }

                // Compare distribution...base case doesn't get this because there are no stats for this there

                // Valid for basecase?
                if (cashFlowSupported) {
                    menuItem = self._buildCashFlowMenu(crosstab, info);
                    _addMenuItem(menuItem);
                }

                riskFactorSensitivitiesMenuItem = this._buildRiskFactorSensitivitiesMenuItem(crosstab, statFromColumn, info);
                _addMenuItem(riskFactorSensitivitiesMenuItem);

                if (detailsSupported) {
                    menuItem =  self._createMenuItem(
                        context.i18n.getText("HPRisk.Crosstab.Context.Menu.Subportfolio.Details.txt"),
                        "subportfolioDetails",
                        function () {
                            var newLocation = self._locationForNewPanel(crosstabId);
                            var dataObject = self._buildPortfolioDetailsDataObject(crosstab, "column", info);
                            self.HPR_UTIL.getPanelsViewForComponent(crosstabId).addPanel(newLocation.id, newLocation.location, dataObject);
                        });
                    _addMenuItem(menuItem);
                }

                // computed variable
                menuItem =  self._createMenuItem(
                    context.i18n.getText("HPRisk.Crosstab.Context.Menu.Create.Computed.Variable.txt"),
                    "createComputedVariable",
                    function () {
                        var expCtrl = sas.analytics.riskhp.utils.ViewManager.getExplorationView().getController();
                        expCtrl.createComputedVariable.call(expCtrl, context.getId(), info);
                    });
                _addMenuItem(menuItem);

                // display rule
                _addMenuItem( self._createMenuItem(
                    context.i18n.getText("HPRisk.Crosstab.Context.Menu.Create.Display.Rule.txt"),
                    context.i18n.getText("HPRisk.Crosstab.Context.Menu.Create.Display.Rule.txt"),
                    function () {
                        var expCtrl = sas.analytics.riskhp.utils.ViewManager.getExplorationView().getController();
                        expCtrl.createDisplayRule.call(expCtrl, context.getId(), info);
                    })
                );
            }
            // ***************************************
            // else regular old column get this menu
            // ***************************************
            else {

                // Compare STAT ...
                if (hasColumnStat && !self.CROSSTAB_UTIL.isStalenessStatistic(statFromColumn)) {
                    self._buildLevelMenu(self._buildCompareStatMenu(crosstab, info, statFromColumn), oMenu, false);
                }

                // Compare distribution...
                if (distributionSupported &&
                    !self.CROSSTAB_UTIL.isStalenessStatistic(statFromColumn) &&
                    !self.CROSSTAB_UTIL.isNinstStatistic(statFromColumn)) {
                    topLevelMenuItem = self._buildLevelMenu(self._buildCompareDistMenu(crosstab, info), oMenu, false);
                    topLevelMenuItem.setStartsSection(true);
                }

                if (densitySupported &&
                    !self.CROSSTAB_UTIL.isStalenessStatistic(statFromColumn) &&
                    !self.CROSSTAB_UTIL.isNinstStatistic(statFromColumn)) {
                    menuItem = self._createMenuItem(
                        context.i18n.getText("HPRisk.Crosstab.Context.Menu.Density.Plot.txt"),
                        "densityPlot",
                        function () {
                            var newLocation = self._locationForNewPanel(crosstabId);
                            var plotType = {PlotType: HprConstants.PlotType.Density};
                            var dataObject = self.CROSSTAB_UTIL.buildPlotDataObject(crosstab, plotType, info);
                            self.HPR_UTIL.getPanelsViewForComponent(crosstabId).addPanel(newLocation.id, newLocation.location, dataObject);
                        });
                    _addMenuItem(menuItem);
                }

                if (vectorsSupported &&
                    !self.CROSSTAB_UTIL.isStalenessStatistic(statFromColumn) &&
                    !self.CROSSTAB_UTIL.isNinstStatistic(statFromColumn)) {
                    menuItem = self._createMenuItem(
                        context.i18n.getText("HPRisk.Crosstab.Context.Menu.simulatedValues.txt"),
                        "simulatedValuesTable",
                        function () {
                            var newLocation = self._locationForNewPanel(crosstabId);
                            var dataObject = self._buildSimulatedValuesDataObject(crosstab, "column", info);
                            self.HPR_UTIL.getPanelsViewForComponent(crosstabId).addPanel(newLocation.id, newLocation.location, dataObject);
                        });
                    _addMenuItem(menuItem);
                }


                // This is a priority 3 item, removing for now until we are given the green light for priority 3 items - MCA
                // if (vectorsSupported) {
                //     menuItem = self._createMenuItem(
                //         context.i18n.getText("HPRisk.Crosstab.Context.Menu.Simulated.Values.Table.txt"),
                //         "vectorPlot",
                //         function () {
                //             var newLocation = self._locationForNewPanel(crosstabId);
                //             var dataObject = self._buildPlotDataObject(crosstab, {PlotType: HprConstants.PlotType.SimulatedValues}, "column", info);
                //             HPR_UTIL.getPanelsViewForComponent(crosstabId).addPanel(newLocation.id, newLocation.location, dataObject);
                //         }
                //     );
                //     menuItem.setEnabled(true);
                //     _addMenuItem(menuItem);
                // }

                if (rfimSupported &&
                    !self.CROSSTAB_UTIL.isStalenessStatistic(statFromColumn) &&
                    !self.CROSSTAB_UTIL.isNinstStatistic(statFromColumn)) {
                    menuItem = self._createMenuItem(
                        context.i18n.getText("HPRisk.Crosstab.Context.Menu.RFIM.txt"),
                        "rfimPlot",
                        function () {
                            var newLocation = self._locationForNewPanel(crosstabId);
                            var plotType = {PlotType: HprConstants.PlotType.RiskFactorInformationMeasures};
                            var dataObject = self.CROSSTAB_UTIL.buildPlotDataObject(crosstab, plotType, info);
                            self.HPR_UTIL.getPanelsViewForComponent(crosstabId).addPanel(newLocation.id, newLocation.location, dataObject);
                        }
                    );
                    menuItem.setEnabled(true);
                    _addMenuItem(menuItem);
                }

                if (cashFlowSupported) {
                    menuItem = self._buildCashFlowMenu(crosstab, info);
                    _addMenuItem(menuItem);
                }

                if (detailsSupported) {
                    menuName = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Subportfolio.Details.txt");
                    menuItem = self._createMenuItem(
                        menuName,
                        "subportfolioDetails",
                        function () {
                            var newLocation = self._locationForNewPanel(crosstabId);
                            var dataObject = self._buildPortfolioDetailsDataObject(crosstab, "column", info);
                            self.HPR_UTIL.getPanelsViewForComponent(crosstabId).addPanel(newLocation.id, newLocation.location, dataObject);
                        }
                    );
                    menuItem.setStartsSection(true);
                    _addMenuItem(menuItem);
                }

                // computed variable
                menuItem =  self._createMenuItem(
                    context.i18n.getText("HPRisk.Crosstab.Context.Menu.Create.Computed.Variable.txt"),
                    "createComputedVariable",
                    function () {
                        var expCtrl = sas.analytics.riskhp.utils.ViewManager.getExplorationView().getController();
                        expCtrl.createComputedVariable.call(expCtrl, context.getId(), info);
                    });
                _addMenuItem(menuItem);

                // display rule
                _addMenuItem( self._createMenuItem(
                    context.i18n.getText("HPRisk.Crosstab.Context.Menu.Create.Display.Rule.txt"),
                    context.i18n.getText("HPRisk.Crosstab.Context.Menu.Create.Display.Rule.txt"),
                    function () {
                        var expCtrl = sas.analytics.riskhp.utils.ViewManager.getExplorationView().getController();
                        expCtrl.createDisplayRule.call(expCtrl, context.getId(), info);
                    })
                );
            }
        }

        // Refresh the connected plots
        menuItem = self._createMenuItem(
            context.i18n.getText("HPRisk.Crosstab.Context.Menu.UpdateLinkedPlots.txt"),
            "updateLinkedPlots",
            function () {
                sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_LINKED_PLOTS, {
                    crosstabId: crosstab.getId(),
                    info: info,
                    selectedCellChanged: true
                });
           });
        menuItem.setStartsSection(true);

        // Only enable if any plot is linked to this crosstab
        menuItem.setEnabled(self.CROSSTAB_UTIL.numLinkedPlots(crosstab.getId()) > 0);
        _addMenuItem(menuItem);

        // Copy contents of data cell to the clipboard
        var value = entry.getLabel();
        menuItem = self._createMenuItem(
            context.i18n.getText("HPRisk.Crosstab.Context.Menu.CopyToClipboard.txt"),
            "copyValue",
            function () {
                self.CROSSTAB_UTIL.copyTextToClipboard(value);
            });
        _addMenuItem(menuItem);

        // Create a new crosstab and drill to this location
        menuItem = self._createMenuItem(
            context.i18n.getText("HPRisk.Crosstab.Context.Menu.NewCrosstab.txt"),
            "newCrosstab",
            function () {
                var contextId = sas.analytics.riskhp.utils.HprConstants.HPR_CROSSTAB_COMPONENT_ID + Date.now();

                var restOfCrosstabClone = function(newContext, originalContext) {
                    var newDrillpath = _.map(info.expansion.key, function (obj) {
                        return obj.value;
                    });

                    // Fix the drillpath if we don't have enough classvars
                    if (newDrillpath.length >= originalContext.classVariables.length) {
                        newDrillpath.pop();
                    }

                    // Convert keys from the name to their label equivalent.  Drillpoint uses labels.
                    var node = originalContext.findRowTreeNode(newDrillpath, "label");
                    var newDrillpath2 = originalContext.findRowTreePath(node, "label");
                    newContext.setDrillPath(newDrillpath2);

                    var dataObject = {
                        type: HprConstants.HPR_CROSSTAB_PANEL,
                        context: newContext,
                        dataSourceId: originalContext.dataSourceId
                    };

                    var newLocation = self._locationForNewCrosstab(crosstabId);
                    self.HPR_UTIL.getPanelsViewForComponent(crosstabId).addPanel(newLocation.id, newLocation.location, dataObject);
                };

                // Clone might take a couple of seconds since we might need to hit the mid-tier, so we wait on it
                $.when(
                    context.clone(contextId)
                ).done(function(clonedContext) {
                    restOfCrosstabClone(clonedContext, context);
                });
            });
        _addMenuItem(menuItem);

        // external links
        menuItem = self._buildExternalLinksMenu(crosstab, info);
        _addMenuItem(menuItem);


        // walk over the collected menu items and add any missing separators if needed
        menuItems = oMenu.getItems();
        var riskFactorSensitivitiesMenuItemFound = false;
        _.each(menuItems, function (menuItem) {
            if (riskFactorSensitivitiesMenuItem && menuItem.getId() === riskFactorSensitivitiesMenuItem.getId()) {
                riskFactorSensitivitiesMenuItemFound = true;
            }
            else {
                if (riskFactorSensitivitiesMenuItemFound) {
                    menuItem.setStartsSection(true);
                    riskFactorSensitivitiesMenuItemFound = false;
                }
            }
        });

    },

    /**
     * Builds the menu for a row header in the crosstab.
     * NOTE: using info.displayedKeys because crosstab functions use the displayedKey (formatted)
     *       instead of info.key (unformatted) keys.
     * @param oMenu - the menu to add the menu items to
     * @param crosstab - The crosstab for this menu
     * @param info - the info object extracted from the pick results
     */
    _buildRowHeaderMenu: function(oMenu, crosstab, info){
        var context = crosstab.context;
        var self = this;
        var menuItem;
        var expandCollpaseMenuItemsAdded = false;
        var pageMenuItemsAdded = false;

        if (info
            && (info.name !== context.i18n.getText('HPRisk.Crosstab.Total.txt'))
            && (info.name !== context.i18n.getText('HPRisk.Crosstab.Subtotal.txt')) ) {

            // No need to display if cell is leaf node
            if (info.displayedKeys.length < context.classVariables.length) {
                menuItem = self._createMenuItem(
                    context.i18n.getText("HPRisk.Crosstab.Context.Menu.Expand.txt"),
                    "ExpandMenuItem",
                    function () {
                        crosstab.doExpand(info.displayedKeys);
                    }
                );
                oMenu.addItem(menuItem);

                oMenu.addItem(self._createMenuItem(
                    context.i18n.getText("HPRisk.Crosstab.Context.Menu.Collapse.txt"),
                    "CollapseMenuItem",
                    function () {
                        crosstab.doCollapse(info.displayedKeys);
                    })
                );

                expandCollpaseMenuItemsAdded = true;
            }

            var paged = crosstab.isPaging(info.displayedKeys);
            if (paged) {
                menuItem = self._createMenuItem(
                    context.i18n.getText("HPRisk.Crosstab.Context.Menu.PageUp.txt"),
                    "PageUpMenuItem",
                    function () {
                        crosstab.doPaging("up", info.displayedKeys);
                    },
                    undefined,
                    expandCollpaseMenuItemsAdded // Serves as startsSection parameter
                );
                menuItem.setEnabled((paged === "bottomPage" || paged === "middlePage"));
                oMenu.addItem(menuItem);

                menuItem = self._createMenuItem(
                    context.i18n.getText("HPRisk.Crosstab.Context.Menu.PageDown.txt"),
                    "PageDownMenuItem",
                    function () {
                        crosstab.doPaging("down", info.displayedKeys);
                    }
                );
                menuItem.setEnabled((paged === "topPage" || paged === "middlePage"));
                oMenu.addItem(menuItem);

                pageMenuItemsAdded = true;
            }

            var doCVRemove = function(oCrosstab, oContext, oCVDataItem) {
                oContext.removeClassVariable({name: oCVDataItem.name});
                oCrosstab.rebuildCrosstab();
                oCrosstab.updateHierarchiesPane();
            };

            var cvIndex = info.displayedKeys.length - 1;
            var cvArray = context.classVariables;
            // Make sure there is more than one Class Variable in the Crosstab and we have an index to work with
            if (cvArray.length > 1 && cvIndex >= 0) {
                var cvDataItem = cvArray[cvIndex];
                if (cvDataItem) {
                    oMenu.addItem(self._createMenuItem(
                        context.i18n.getText("HPRisk.Crosstab.Context.Menu.Remove.fmt", [cvDataItem.label]),
                        "RemoveMenuItem",
                        function () {
                            if (context.isForcedHierarchyMode()) {
                                var hierarchyDialog = sap.ui.jsfragment("sas.analytics.riskhp.fragments.HierarchyDialog", {
                                    messageText: context.i18n.getText("Hierarchy.forced.disable.dialog.txt"),
                                    doContinue: function() {
                                        context.setForcedHierarchyModeOff();
                                        doCVRemove(crosstab, context, cvDataItem);
                                    }
                                });
                                hierarchyDialog.open();
                            } else {
                                doCVRemove(crosstab, context, cvDataItem);
                            }
                        },
                        undefined,
                        expandCollpaseMenuItemsAdded || pageMenuItemsAdded  // Serves as startsSection parameter
                    ));
                }
            }
        }
    },

    /**
     * Builds the context menu for when the crosstab's column header is right clicked on
     * @param oMenu - the menu to add the menu items to
     * @param crosstab - The crosstab for this menu
     * @param info - the info object extracted from the pick results
     */
    _buildColumnHeaderMenu: function(oMenu, crosstab, info) {
        var self = this;
        var context = crosstab.context;
        var menuItem;

        var node = context.findColumnTreeNode(info.keys);
        var dataItem = node.item;

        var itemName = info.name;
        var allItemName = info.name;
        if (dataItem.basecase && dataItem.basecase === true) {
            itemName = dataItem.constructBaseCaseString() + " (" + itemName + ")";
            allItemName = dataItem.constructBaseCaseString();
        }

        self._buildMoveColumnContextMenu(oMenu, crosstab, info);

        var startsSection = false;
        // Quick check to see if we need a startSection separator added
        if (oMenu.getItems() && oMenu.getItems().length > 0) {
            startsSection = true;
        }

        menuItem = self._createMenuItem(
            context.i18n.getText("HPRisk.Crosstab.Context.Menu.Remove.Item.fmt", itemName),
            "RemoveItem",
            function () {
                // May have to remove multiple columns because of spanning
                context.removeColumnsWithKey(info.keys, info.dataItemType);
                crosstab.rebuildCrosstab();
            },
            undefined,
            startsSection
        );
        // Disable the menu item if the parent node does not have any more children and the total number of columns for the Crosstab is the
        // same as the number of columns for the node we right-clicked on
        if ((node.parent && node.parent.children.length <= 1) && (context.getNumberOfColumns() - context.getNumberOfColumns(node) === 0) ) {
            menuItem.setEnabled(false);
        }
        oMenu.addItem(menuItem);

        menuItem = self._createMenuItem(
            context.i18n.getText("HPRisk.Crosstab.Context.Menu.Remove.All.fmt", allItemName),
            "RemoveAll",
            function() {
                context.removeColumnsBasedOnInfo(info);
                crosstab.rebuildCrosstab();
            }
        );
        menuItem.setEnabled(false);
        // Only enable the menu item if there are more columns in the Crosstab
        if (context.getNumberOfColumns() - context.getNumberOfColumns(node) > 0) {
            menuItem.setEnabled(true);
        }
        oMenu.addItem(menuItem);
    },

    /**
     * Builds the menu for when the row axis header cell is right clicked on.
     * @param oMenu - the menu to add the menu items to
     * @param crosstab - The crosstab for this menu
     * @param info - the info object extracted from the pick results
     */
    _buildAxisLabelHeaderContextMenu: function(oMenu, crosstab, info) {
        var self = this;
        var context = crosstab.context;
        var menuName;
//        var menuItems = [];
        var menuItem;

        var doCVRemove = function(oCrosstab, oContext, info) {
            oContext.removeClassVariable({label: info.name}); // S1437562
            oCrosstab.rebuildCrosstab();
            oCrosstab.updateHierarchiesPane();
        };

        // Only display menu if there is more than one Class Variable
        if (context.classVariables.length > 1) {
            menuName = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Remove.fmt", [info.name]);
            menuItem = self._createMenuItem(
                menuName,
                "RemoveClassvar",
                function() {
                    if (context.isForcedHierarchyMode()) {
                        var hierarchyDialog = sap.ui.jsfragment("sas.analytics.riskhp.fragments.HierarchyDialog", {
                            messageText: context.i18n.getText("Hierarchy.forced.disable.dialog.txt"),
                            doContinue: function() {
                                context.setForcedHierarchyModeOff();
                                doCVRemove(crosstab, context, info);
                            }
                        });
                        hierarchyDialog.open();
                    } else {
                        doCVRemove(crosstab, context, info);
                    }
                });
            oMenu.addItem(menuItem);
        }
    },

    /**
     * Builds the menu for when a measure header cell is right clicked on.
     * @param oMenu - the menu to add the menu items to
     * @param crosstab - The crosstab for this menu
     * @param info - the info object extracted from the pick results
     */
    _buildMeasureHeaderContextMenu: function(oMenu, crosstab, info) {
        var self = this;
        var context = crosstab.context;
        var menuName;
        var menuItems = [];
        var menuItem;
        var menuStructure;

        var node = context.findColumnTreeNode(info.keys);
        var statObject = context.getStatisticFromColumn(info.column);
        var isStaleness = false;
        var isRowstat = false;
        var sortDataItem = undefined;
        if (statObject) {
            sortDataItem = statObject.dataItem;
            isStaleness = (sortDataItem.name === sas.analytics.riskhp.models.dataitems.Statistic.STALENESS);
            isRowstat = (sortDataItem.statisticType === sas.analytics.riskhp.models.dataitems.Statistic.typeRow);
        } else {
            sortDataItem = node.item;
        }

        if (!isStaleness) {
            menuName = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Sort.Ascending.txt");
            menuItem = self._createMenuItem(
                menuName,
                "SortValueAscending",
                function () {
                    context.setSortColumn(info.columnIndex,
                        sas.analytics.riskhp.models.CrosstabContext.SORT_DIRECTION_ASCENDING,
                        sas.analytics.riskhp.models.CrosstabContext.SORT_TYPE_VALUE);
                    crosstab.rebuildCrosstab(true, true);
                }
            );
            menuItems.push(menuItem);

            menuName = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Sort.Descending.txt");
            menuItem = this._createMenuItem(
                menuName,
                "SortValueDescending",
                function () {
                    context.setSortColumn(info.columnIndex,
                        sas.analytics.riskhp.models.CrosstabContext.SORT_DIRECTION_DESCENDING,
                        sas.analytics.riskhp.models.CrosstabContext.SORT_TYPE_VALUE);
                    crosstab.rebuildCrosstab(true, true);
                }
            );
            menuItems.push(menuItem);

            menuStructure = {
                subMenuName: context.i18n.getText("HPRisk.Crosstab.Context.Menu.Sort.Stat.fmt",
                    sortDataItem.getDisplayLabel(crosstab.context.optionShowLabels.value)),
                menuItems: menuItems
            };
            self._buildLevelMenu(menuStructure, oMenu, false);

            var absoluteMenuItems = [];
            menuName = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Sort.Ascending.txt");
            menuItem = self._createMenuItem(
                menuName,
                "SortAbsoluteAscending",
                function () {
                    context.setSortColumn(info.columnIndex,
                        sas.analytics.riskhp.models.CrosstabContext.SORT_DIRECTION_ASCENDING,
                        sas.analytics.riskhp.models.CrosstabContext.SORT_TYPE_ABSOLUTE);
                    crosstab.rebuildCrosstab(true, true);
                }
            );
            absoluteMenuItems.push(menuItem);

            menuName = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Sort.Descending.txt");
            menuItem = self._createMenuItem(
                menuName,
                "SortAbsoluteDescending",
                function () {
                    context.setSortColumn(info.columnIndex,
                        sas.analytics.riskhp.models.CrosstabContext.SORT_DIRECTION_DESCENDING,
                        sas.analytics.riskhp.models.CrosstabContext.SORT_TYPE_ABSOLUTE);
                    crosstab.rebuildCrosstab(true, true);
                }
            );
            absoluteMenuItems.push(menuItem);

            menuStructure = {
                subMenuName: context.i18n.getText("HPRisk.Crosstab.Context.Menu.Sort.Absolute.txt"),
                menuItems: absoluteMenuItems
            };
            self._buildLevelMenu(menuStructure, oMenu, false);

            menuName = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Sort.Clear.txt");
            menuItem = self._createMenuItem(
                menuName,
                "SortClear",
                function () {
                    context.clearSortColumn(info.columnIndex);
                    //crosstab.rebuildCrosstab();
                    crosstab.rebuildCrosstab(true, true);
                }
            );
            oMenu.addItem(menuItem);
        }

        self._buildMoveColumnContextMenu(oMenu, crosstab, info, !isStaleness);

        if (!isRowstat) {
            var format = context.getFormatFromColumn(info.column);
            if (format && (format.type === "COMMA" || format.type === "PERCENT")) {
                menuName = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Set.Format.txt");
                menuItem = self._createMenuItem(
                    menuName,
                    "SetFormat",
                    function () {
                        // pop a format chooser dialog
                        // var node = context.findColumnTreeLeafNodeFromTuple(info.column);
                        var formatDialog = sap.ui.jsfragment("sas.analytics.riskhp.fragments.FormatDialog", {
                            info: info,
                            context: context
                        });
                        formatDialog.open();
                    },
                    undefined,
                    true
                );
                //menuItem.setEnabled(false);
                oMenu.addItem(menuItem);
            }
        }

        // Remove item
        menuName = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Remove.Item.fmt", info.name);
        menuItem = self._createMenuItem(
            menuName,
            "RemoveItem",
            function() {
                context.removeColumn(info.columnIndex);
                crosstab.rebuildCrosstab();
            },
            undefined,
            true
        );
        // Disable the menu item if the parent node does not have any more children and the total number of columns for the Crosstab is the
        // same as the number of columns for the node we right-clicked on
        if ((node.parent && node.parent.children.length <= 1) && (context.getNumberOfColumns() - context.getNumberOfColumns(node) === 0) ) {
            menuItem.setEnabled(false);
        }
        oMenu.addItem(menuItem);

        menuName = context.i18n.getText("HPRisk.Crosstab.Context.Menu.Remove.All.fmt", info.name);
        menuItem = self._createMenuItem(
            menuName,
            "RemoveAll",
            function() {
                context.removeColumnsWithNameAndType(info.name, info.dataItemType);
                crosstab.rebuildCrosstab();
            }
        );
        menuItem.setEnabled(false);
        // Only enable the menu item if there are more columns in the Crosstab
        if (context.getNumberOfColumns() - context.getNumberOfColumns(node) > 0) {
            menuItem.setEnabled(true);
        }
        oMenu.addItem(menuItem);
    }
});