jQuery.sap.declare("sas.analytics.riskhp.common.riskcommons.plot.HprBarChart");
(function() {
    "use strict";
    jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils");
    jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprAbstractChart");

    // LTJS common requires
    jQuery.sap.require("sap.ui.core.Control");
    jQuery.sap.require("sas.ltjs.commons.models.data.dataset.Column");
    jQuery.sap.require("sas.ltjs.commons.models.data.dataset.DataSet");
    jQuery.sap.require("sas.ltjs.commons.models.data.dataset.DataType");
    jQuery.sap.require("sas.ltjs.gtl.models.Graph");
    jQuery.sap.require("sas.ltjs.gtl.models.data.AnalysisRole");
    jQuery.sap.require("sas.ltjs.gtl.models.data.ClassificationRole");
    jQuery.sap.require("sas.ltjs.gtl.models.data.DataSetDataProvider");
    jQuery.sap.require("sas.ltjs.gtl.models.layoutOverlay.LayoutOverlay");
    jQuery.sap.require("sas.ltjs.gtl.models.entry.EntryTitle");
    jQuery.sap.require("sas.ltjs.gtl.models.entry.EntryTextItem");
    jQuery.sap.require("sas.ltjs.gtl.models.core.HorizontalAlignment");
    jQuery.sap.require("sas.ltjs.ltjsLoader");
    jQuery.sap.require("sas.ltjs.gtl.models.data.PlotRole");
    jQuery.sap.require("sas.ltjs.gtl.models.axis.AxisLabelPosition");

    // Bar chart (extra requires)
    jQuery.sap.require("sas.ltjs.gtl.models.layoutOverlay.barChart.BarChart");
    jQuery.sap.require("sas.ltjs.gtl.models.layoutOverlay.barChart.BarChartGroupDisplay");
    jQuery.sap.require("sas.ltjs.gtl.models.core.Orientation");
    jQuery.sap.require("sas.ltjs.gtl.models.core.SortOrder");
    jQuery.sap.require("sas.ltjs.gtl.models.legend.ContinuousLegend");
    jQuery.sap.require("sas.ltjs.gtl.models.legend.LayoutGlobalLegend");
    jQuery.sap.require("sas.ltjs.gtl.models.legend.LegendTitlePosition");
    jQuery.sap.require("sas.ltjs.gtl.models.axis.DiscreteAxisOptions");
    jQuery.sap.require("sas.ltjs.gtl.models.axis.AxisTickValueFitPolicy");

    // Expose ltjs objects we want to use without having to type the full namespace
    var HprChartUtils = sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils;
    var Column = sas.ltjs.commons.models.data.dataset.Column;
    /** @name sas.ltjs.gtl.models.Graph */
    var Graph = sas.ltjs.gtl.models.Graph;
    /** @name sas.ltjs.gtl.models.data.AnalysisRole */
    var AnalysisRole = sas.ltjs.gtl.models.data.AnalysisRole;
    /** @name sas.ltjs.gtl.models.data.ClassificationRole */
    var ClassificationRole = sas.ltjs.gtl.models.data.ClassificationRole;
    var DataSet = sas.ltjs.commons.models.data.dataset.DataSet;
    /** @name sas.ltjs.gtl.models.data.DataSetDataProvider */
    var DataSetDataProvider = sas.ltjs.gtl.models.data.DataSetDataProvider;
    /** @name sas.ltjs.gtl.models.layoutOverlay */
    /** @name sas.ltjs.gtl.models.layoutOverlay.LayoutOverlay */
    var LayoutOverlay = sas.ltjs.gtl.models.layoutOverlay.LayoutOverlay;
    /** @name sas.ltjs.gtl.models.layoutOverlay.barChart */
    var BarChart = sas.ltjs.gtl.models.layoutOverlay.barChart.BarChart;
    /** @name sas.ltjs.gtl.models.layoutOverlay.barChart.BarChartGroupDisplay */
    var BarChartGroupDisplay = sas.ltjs.gtl.models.layoutOverlay.barChart.BarChartGroupDisplay;
    /** @name sas.ltjs.gtl.models.core.HorizontalAlignment */
    var HorizontalAlignment = sas.ltjs.gtl.models.core.HorizontalAlignment;
    /** @name sas.ltjs.gtl.models.core.Orientation */
    var Orientation = sas.ltjs.gtl.models.core.Orientation;
    /** @name sas.ltjs.gtl.models.legend.ContinuousLegend */
    var ContinuousLegend = sas.ltjs.gtl.models.legend.ContinuousLegend;
    /** @name sas.ltjs.gtl.models.legend.LayoutGlobalLegend */
    var LayoutGlobalLegend = sas.ltjs.gtl.models.legend.LayoutGlobalLegend;
    /** @name sas.ltjs.gtl.models.legend.LegendTitlePosition */
    var LegendTitlePosition = sas.ltjs.gtl.models.legend.LegendTitlePosition;
    /** @name sas.ltjs.gtl.models.axis.DiscreteAxisOptions */
    var DiscreteAxisOptions = sas.ltjs.gtl.models.axis.DiscreteAxisOptions;
    /** @name sas.ltjs.gtl.models.axis.AxisTickValueFitPolicy */
    var AxisTickValueFitPolicy = sas.ltjs.gtl.models.axis.AxisTickValueFitPolicy;
    /** @name sas.ltjs.gtl.models.core.SortOrder */
    var SortOrder = sas.ltjs.gtl.models.core.SortOrder;
    /** @name sas.ltjs.gtl.models.axis.AxisLabelPosition */
    var AxisLabelPosition = sas.ltjs.gtl.models.axis.AxisLabelPosition;
    /** @name sas.ltjs.gtl.models.data.PlotRole */
    var PlotRole = sas.ltjs.gtl.models.data.PlotRole;

    // Require our hprisk ltjs override objects
    jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprChartModel");
    jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprGraphView");

    /**
     * Renders a bar chart the HPRisk way.
     * Example in flex app Compare VaR -> Horizon -> Bar Chart
     *
     * @see http://sww.sas.com/saspedia/GTL_BarChart_statement
     * @see http://sww.sas.com/saspedia/GTL_ContinuousLegend_statement
     * @name sas.analytics.riskhp.common.riskcommons.plot.HprBarChart
     */
    sas.analytics.riskhp.common.riskcommons.plot.HprAbstractChart.extend("sas.analytics.riskhp.common.riskcommons.plot.HprBarChart", {
        metadata: { },

        renderer: { }, // Use the base class renderer

        /**
         * Given a model builds and returns the correct graph object
         * @param chartModel
         * @returns {sas.ltjs.gtl.models.Graph}
         * @private
         */
        _buildGraph: function(chartModel) {
            //return this._sampleGraph();
            var graph = new Graph();

            // Make sure blank values are included.
            graph.setIncludeMissingDiscrete(true);

            // set the title.  OBE: For now done in view
            // HprChartUtils.setGraphPathTitle(chartModel, graph);

            var dataSet = new DataSet(true, true);

            var charts = [];
            _.each(chartModel.categoryRoles, function(categoryRole, chartIndex) {

                // add the data to the columns and then put the columns in the dataSet
                // Category - the columns for category values (aka X)
                var categoryRoleName = HprChartUtils.getCategoryRoleName(categoryRole, chartModel);
                var categoryColumn = new Column();
                categoryColumn.setName(categoryRoleName);

                // For now we assume there is only one formatter per responseRole set
                var responseRoleFormatter = chartModel.getResponseRoleFormatter(chartIndex);

                var xValues = []; // Really is bucket groupings
                _.each(chartModel.getSeriesIds(chartIndex), function(seriesId){
                    var seriesObject = chartModel.getSeriesObject(seriesId, chartIndex);
                    // We have to push a "placeholder" into the x values array so the labels will match with the y values
                    // they are accumulated (so we ignore the actual value in the returned array from the getValues() call
                    _.each(seriesObject.getValues(), function() {
                        xValues.push(seriesObject.getName(chartModel.dataSource, chartModel.showLabels()));
                    });
                });
                HprChartUtils.addValuesToColumn(categoryColumn, xValues, responseRoleFormatter);
                dataSet.addColumn(categoryColumn);

                // Accumulate the y values values
                var yValues = [];
                _.each(chartModel.getSeriesIds(chartIndex), function(seriesId){
                    var seriesObject = chartModel.getSeriesObject(seriesId, chartIndex);
                    yValues.push(seriesObject.values[0]);
                });

                // Response - The numeric values (aka Y)
                var responseRole = chartModel.getResponseType(chartIndex);
                var responseRoleName = HprChartUtils.getResponseRoleName(chartModel, responseRole);

                // We are expecting only one series at this time, here is where we loop over the series ids to generate multiple bars (see HprTimeSeriesChart)
                var responseColumn = new Column();
                responseColumn.setName(responseRoleName);
                HprChartUtils.addValuesToColumn(responseColumn, yValues, responseRole);
                if (chartModel.isRFIMPlot()) {
                    responseColumn.setFormatName("BEST");
                }
                responseColumn.setDataType(sas.ltjs.commons.models.data.dataset.DataType.NUMBER);

                dataSet.addColumn(responseColumn);

                // BEGIN Fix to horizontal bar ordering recommended by the graph group
                // Create fake grouping column for RFIM plots
                // if (chartModel.isRFIMPlot()) {
                //     var fakeGroupingColumn = new Column();
                //     fakeGroupingColumn.setName("__fakeGroupingColumn__");
                //
                //     var constantValues = [];
                //     _.each(yValues, function() {
                //         constantValues.push(1);
                //     });
                //     fakeGroupingColumn.setNumbers(constantValues);
                //     fakeGroupingColumn.setDataType(sas.ltjs.commons.models.data.dataset.DataType.NUMBER);
                //     dataSet.addColumn(fakeGroupingColumn);
                // }
                // END Fix to horizontal bar ordering recommended by the graph group

                // setup the bar chart
                var barChart = new BarChart();
                var classificationRole = new ClassificationRole(categoryRoleName);
                barChart.setCategory(classificationRole);

                var analysisRole = new AnalysisRole(responseRoleName);
                barChart.setResponse([analysisRole]);

                barChart.selectable = true;

                // When orientation is VERTICAL the X axis is used for category and the
                // Y axis is used for RESPONSE
                if (chartModel.isRFIMPlot()) {
                    barChart.setOrientation(Orientation.HORIZONTAL);
                    // BEGIN Fix to horizontal bar ordering recommended by the graph group
                    // Not supported now, might be in a future release so the presorting in HprChartModel could be removed
                    // barChart.setCategorySortByResponse(true);
                    // var groupRole = new ClassificationRole("__fakeGroupingColumn__");
                    // barChart.setGroup(groupRole);
                    // END Fix to horizontal bar ordering recommended by the graph group

                }
                else {
                    barChart.setOrientation(Orientation.VERTICAL);
               }

                // assign the data provider to the bar chart
                barChart.setDataProvider(new DataSetDataProvider(dataSet));


                charts.push(barChart);
            });

            // add the bar chart to the layout and then set the layout on the graph
            var layout = new LayoutOverlay();

            // Build the y-axis options
            var yAxisOptions = HprChartUtils.getDefaultYAxisOptions();
            // BEGIN Fix to horizontal bar ordering recommended by the graph group
            // if (chartModel.isRFIMPlot()) {
            //      var yDiscreteAxisOptions = new DiscreteAxisOptions();
            //      yDiscreteAxisOptions.setSortOrder(SortOrder.ASCENDING);
            //      yAxisOptions.setDiscreteOptions(yDiscreteAxisOptions);
            // }
            // END Fix to horizontal bar ordering recommended by the graph group
            layout.setYAxisOptions(yAxisOptions);

            // Build the x-axis options
            var xAxisOptions = HprChartUtils.getDefaultXAxisOptions();
            var xDiscreteAxisOptions = new DiscreteAxisOptions();
            xDiscreteAxisOptions.setTickValueFitPolicy(AxisTickValueFitPolicy.ROTATETHIN);
            if (chartModel.isRFIMPlot()) {
                //var yDiscreteAxisOptions = new DiscreteAxisOptions();
                xDiscreteAxisOptions.setSortOrder(SortOrder.DESCENDING);
                //yAxisOptions.setDiscreteOptions(yDiscreteAxisOptions);
            }
            xAxisOptions.setDiscreteOptions(xDiscreteAxisOptions);
            layout.setXAxisOptions(xAxisOptions);

            layout.setOverlays(charts);
            graph.setLayout(layout);
            return graph;
        }

        // Leave here for testing purposes
        // _sampleGraph: function() {
        //     var graph = new Graph();
        //
        //     // Some vars we will use
        //     var categoryRoleName = "Risk Factor";
        //     var responseRoleName = "RFIM";
        //
        //     // Make sure blank values are included.
        //     graph.setIncludeMissingDiscrete(true);
        //
        //     var dataSet = new DataSet(true, true);
        //
        //     var categoryColumn = new Column();
        //     categoryColumn.setName(categoryRoleName);
        //     var xValues = ["#FakeLabel [rf57]", "rf51", "#FakeLabel [rf54]", "rf49", "#FakeLabel [rf43]", "rf55", "#FakeLabel [rf46]", "rf52", "#FakeLabel [rf42]", "rf47"];
        //     categoryColumn.setStrings(xValues);
        //     categoryColumn.setDataType(sas.ltjs.commons.models.data.dataset.DataType.STRING);
        //     dataSet.addColumn(categoryColumn);
        //
        //     var yValues = [0.3374129979822021, 0.1174657376483805, 0.11518670162256388, 0.11119728123729879, 0.10796806617305393, 0.10677264968965573, 0.10376575646498246, 0.10322819496985923, 0.10270285191551243, 0.10086475410342835];
        //     var responseColumn = new Column();
        //     responseColumn.setName(responseRoleName);
        //     responseColumn.setNumbers(yValues);
        //     responseColumn.setDataType(sas.ltjs.commons.models.data.dataset.DataType.NUMBER);
        //     responseColumn.setFormatName("BEST");
        //     dataSet.addColumn(responseColumn);
        //
        //     // setup the bar chart
        //     var barChart = new BarChart();
        //     var classificationRole = new ClassificationRole(categoryRoleName);
        //     barChart.setCategory(classificationRole);
        //
        //     var analysisRole = new AnalysisRole(responseRoleName);
        //     barChart.setResponse([analysisRole]);
        //
        //     barChart.setOrientation(Orientation.HORIZONTAL);
        //
        //     barChart.setDataProvider(new DataSetDataProvider(dataSet));
        //
        //     // add the bar chart to the layout and then set the layout on the graph
        //     var layout = new LayoutOverlay();
        //
        //     // Build the y-axis options
        //     var tickValueFormatOptions = new sas.ltjs.gtl.models.axis.AxisTickValueFormatOptions();
        //     tickValueFormatOptions.setExtractScale(true); // Turn on scale extraction
        //     var yAxisOptions = new sas.ltjs.gtl.models.axis.AxisOptions();
        //     var linearAxisOptions = new sas.ltjs.gtl.models.axis.LinearAxisOptions();
        //     linearAxisOptions.setTickValueFormat(tickValueFormatOptions);
        //     yAxisOptions.setLinearOptions(linearAxisOptions);
        //     yAxisOptions.setLabelPosition(sas.ltjs.gtl.models.axis.AxisLabelPosition.LEFT);
        //     layout.setYAxisOptions(yAxisOptions);
        //
        //     // Build the x-axis options
        //     tickValueFormatOptions = new sas.ltjs.gtl.models.axis.AxisTickValueFormatOptions();
        //     tickValueFormatOptions.setExtractScale(true); // Turn on scale extraction
        //     var xAxisOptions = new sas.ltjs.gtl.models.axis.AxisOptions();
        //     linearAxisOptions = new sas.ltjs.gtl.models.axis.LinearAxisOptions();
        //     linearAxisOptions.setTickValueFormat(tickValueFormatOptions);
        //     xAxisOptions.setLinearOptions(linearAxisOptions);
        //
        //     var xDiscreteAxisOptions = new DiscreteAxisOptions();
        //     xDiscreteAxisOptions.setTickValueFitPolicy(AxisTickValueFitPolicy.ROTATETHIN);
        //     xDiscreteAxisOptions.setSortOrder(SortOrder.DESCENDING);
        //     xAxisOptions.setDiscreteOptions(xDiscreteAxisOptions);
        //     layout.setXAxisOptions(xAxisOptions);
        //
        //     layout.setOverlays([barChart]);
        //     graph.setLayout(layout);
        //     return graph;
        // }

    });
})();