jQuery.sap.declare("sas.analytics.riskhp.common.riskcommons");
jQuery.sap.declare("sas.analytics.riskhp.common.riskcommons.plot");
jQuery.sap.declare("sas.analytics.riskhp.common.riskcommons.plot.HprChartModel");

(function() {
    "use strict";
    jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
    jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils");
    jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.SeriesObject");
    jQuery.sap.require("sas.ltjs.commons.pickResults.PickEntryType");

    var PickEntryType = sas.ltjs.commons.pickResults.PickEntryType;
    var ApplicationSettings = sas.analytics.riskhp.settings.ApplicationSettings;
    var HprChartUtils = sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils;
    var HprUtil = sas.analytics.riskhp.utils.HprUtil;
    var __SINGLE_SERIES__ = "__single_series__";
    var APPLY_JSON_FIX = true;

    // noinspection JSClosureCompilerSyntax
    /**
     * @description The HprChartModel holds the data to be plotted for all the plot types
     *
     * @name sas.analytics.riskhp.common.riskcommons.plot.HprChartModel
     * @class HprChartModel
     * @extends sap.ui.base.Object
     *
     * @typedef {Object} HprChartModel
     * @property {DataSource} dataSource
     * @property {PlotQueryData} plotQueryData
     * @property {Object} optionsPanelOptions
     * @property {Array} categoryRoles
     * @property {Object} [originalData] - This will be set if the model represents a new plot that is being generated
     * @property {function(number):string[]} getSeriesIds
     * @property {function(string,number):SeriesObject} getSeriesObject
     * @property {function():boolean} showLabels
     * @property {function(number,string):number[]} getResponseValues
     * @property {function():boolean} isByTypeSubportfolio
     * @property {function():boolean} isCashFlowPlot
     * @property {function():boolean} isRFIMPlot
     * @property {function():boolean} isByTypeRiskCube
     * @property {function():boolean} isByTypeResultSet
     * @property {function():boolean} isByTypeScenario
     * @property {function():boolean} isByTypeHorizon
     * @property {function(number):string} getResponseType
     */

    /**
     * @typedef {string} DataItemType - the type of a data item
     */

    /**
     * @typedef {Object} ByTypeDataItemIdObject - An object that represents a DataItemIdObject as a ByType parameter
     * @property {number} index
     * @property {DataItemIdObject} item
     * @property {string} type
     */
    /**
     * @typedef {Object} QueryResults - the results from a query for data to the midtier
     * @property {string} id
     * @property {string} name
     * @property {Object} results
     * @property {number} status
     * @property {string} type
     * @property {number} version
     */

    /**
     * @typedef {Object} PlotQueryData -
     * @property {Array} classVariables
     * @property {string} crosstabId
     * @property {string} dataSourceId
     * @property {Array} drill
     * @property {Array} filters
     * @property {Array} options
     * @property {Object} plotType
     * @property {Array} requests
     * @property {Array} sorts
     * @property {Array} tuples
     * @property {string} type
     * @property {string} [displayHorizons]
     * @property {boolean} [showLabels]
     */

    /**
     * @typedef {Object} UpdateLinkedPlotObject - An object passed to the model that gives information about why and how a linked plot needs to update
     * @property {string} crosstabId - The id of the crosstab where the update event initiated from
     * @property {boolean} selectedCellChanged - Did the selected cell in the crosstab change
     * @property {Object} info - the info object with the current state and expansion information from the crosstab
     */

    /**
     * @typedef {Object} DataItemIdObject - A dehydrated dataitem or dataitem-like representation
     * @property {string} id - the id of the dataitem
     * @property {DataItemType} type - the type of the dataitem
     */

    /**
     * @typedef {Object} QueryDataDefinitionTuple
     * @property {string} name - The name of the column as sent in the query
     * @property {DataItemIdObject[]} dataItems - the list of dataitems that make up the column that was queried for
     */

    /**
     * @typedef {Object} QueryRequest
     * @property {string} name
     * @property {Array} key
     * @property {number} maxRows
     * @property {number} startRow
     */

    /**
     * @typedef {Object} NameValueObject
     * @property {string} name
     * @property {*} value
     */

    /**
     * @typedef {Object} OptionsPanelOptions
     * @property {boolean} applyEnabled
     * @property {boolean} generalOptionsSectionCollapsed
     * @property {boolean} graphOptionsSectionCollapsed
     * @property {boolean} advancedQueryOptionsSectionCollapsed
     * @property {boolean} linkedToCrosstab
     * @property {number} ndensity
     * @property {number} kernal
     * @property {string} plotType
     * @property {NameValueObject} displayHorizons
     * @property {NameValueObject} showLabels
     * @property {NameValueObject} alpha
     * @property {NameValueObject} pctldef
     */

    /**
     * @typedef {string} PlotType
     */

    /**
     * @typedef {Object} PlotTypeObject
     * @property {PlotType} PlotType
     * @property {string} WhatType
     * @property {string} HowType
     * @property {sas.analytics.riskhp.utils.HprConstants.ByType|ByTypeDataItemIdObject|Array} ByType
     */

    /**
     * @typedef {Object} ExpansionObject
     * @property {number} maxRows
     * @property {number} startRow
     * @property {[string]} key
     */

    /**
     * @callback successCallback - function to run if the method will complete normally
     */
    /**
     * @callback failureCallback - function to run if the method fails in someway
     */

    /**
     * @typedef {Object} PlotContentData
     * @property {string} dataSourceId
     * @property {string} type
     * @property {string} plotType
     * @property {boolean} linkedToCrosstab - Flag to indicate if the model is linked to a crosstab
     * @property {string} crosstabId - Id of the crosstab the model is (or has been) linked to, may be null
     * @property {string} name
     * @property {Array} drill
     * @property {Array} sorts
     * @property {Array} options
     * @property {Array} filters
     * @property {Object} requests
     * @property {Object} statistics
     * @property {Array} horizons
     * @property {DataItem[]} tuples
     * @property {DataItem[]} classVariables
     * @property {Object} cashFlow
     * @property {Object} optionsPanelOptions
     * @property {[DataItemIdObject]} sourceColumn
     */
    sap.ui.base.Object.extend("sas.analytics.riskhp.common.riskcommons.plot.HprChartModel", {
        metadata: {
            publicMethods: [
                'setDefaultOptions',
                'updateOptions',
                'getOptions',
                'getOptionName',
                "getOptionShowLabels",
                "setOptionShowLabels",
                'getOptionALPHA',
                'getOptionPCTLDEF',
                'getOptionNDENSITY',
                'getOptionKERNEL',
                'getOptionDisplayHorizons',
                'setFilters',
                'updateFilterSections',
                'getFilters',
                'toOptionsJSON',
                'fromOptionsJSON',
                'setLinkedToCrosstab',
                'isLinkedToCrosstab',
                'areDataSourceIdsBeingReferenced',
                'removeDataSource',
                'replaceDataSource'
            ]
        },

        constructor: function (id) {
            this.id = id;

            // External dependencies
            this.plotQueryData = {};
            this.query = {};
            this.results = {};
            this.categoryRoleName = "";
            this.responseRoleName = "";
            this.responseValues = [];
            this.categoryLabels = [];
            this.customLabels = [];
            this.series = [];
            this.crosstabId = null;
            this.linkedToCrosstab = false;
            this.shouldRefreshRightPanel = false;
            this.logger = sas.log.getLogger("sas.analytics.riskhp.common.riskcommons.plot");

            this.dataSource = {};

            // Options Panel Options
            this.optionsPanelOptions ={};

            // filters
            this.filters = [];  // {collapsed, name (id?), type, values, include}

            // The intent of this is to capture the message when invalidating the model.  May be returned by getInvalidModelMessage
            // This may be a bit contrived now as I have moved to as much of a throw/catch pattern as I can.
            this.invalidReasonString = null;
        },

        /** public constants **/
        KEY_RESPONSE_VALUE: "value",

        /**
         * Returns the id for this chart
         * @returns {*}
         */
        getId: function () {
            return this.id;
        },

        /**
         * @description Retrieve the {PlotTypeObject} for this model
         * @returns {PlotTypeObject}
         */
        getPlotTypeObject: function() {
            return this.plotQueryData.plotType;
        },

        /**
         * Returns one of the HprConstants.PlotType constants indicating what type of plot this model should generate
         * @returns {*}
         */
        getPlotType: function() {
            if (this.plotQueryData.isZeroState) {
                return HprConstants.PlotType.ZeroState;
            }
            var plotType = this.plotQueryData.plotType.PlotType;
            var whatType = this.getWhatType();
            if (plotType === HprConstants.PlotType.Comparison && whatType === HprConstants.WhatType.Distribution) {
                return HprConstants.WhatType.Distribution;
            }
            return this.plotQueryData.plotType.PlotType;
        },

        /**
         * Returns the HowType portion of the PlotType object on the plotQueryData
         * @returns {*|sas.analytics.riskhp.utils.HprConstants.HowType|{Line, Bar, Distribution, Table}}
         */
        getHowType: function() {
            return this.plotQueryData.plotType.HowType;
        },

        /**
         * Returns the WhatType portion of the PlotType object on the plotQueryData
         * @returns {*|sas.analytics.riskhp.utils.HprConstants.WhatType|{Statistic, Distribution}}
         */
        getWhatType: function() {
            return this.plotQueryData.plotType.WhatType;
        },

        /**
         * Returns the ByType portion of the PlotType object on the plotQueryData
         * @returns {sas.analytics.riskhp.utils.HprConstants.ByType|ByTypeDataItemIdObject| Array} One of the ByType string constants, an object representing a DataItemId, or an array of DataItem type strings
         */
        getByType: function() {
            return this.plotQueryData.plotType.ByType;
        },

        /**
         * Returns the column that was originally used to build the plot from
         * @returns {DataItemIdObject[]}
         */
        getSourceColumn: function() {
            return this.plotQueryData.sourceColumn;
        },

        /**
         * Can this chartModel support both line and bar charts with the same data
         */
        supportsBothBarAndLinePlots: function() {
            var self = this;
            if (HprChartUtils.isRFIMPlot(self.plotQueryData)) {
                return false;
            }
            var howType = self.plotQueryData.plotType.HowType;
            return (howType === HprConstants.HowType.Bar || howType === HprConstants.HowType.Line);
        },

        /**
         * Change the type of plot this chartmodel will render to a Bar
         * @returns {*|sas.analytics.riskhp.utils.HprConstants.HowType|{Line, Bar, Distribution, Table}}
         */
        setDisplayAsBar: function() {
            this.plotQueryData.plotType.HowType = HprConstants.HowType.Bar;
        },

        /**
         * Change the type of plot this chartmodel will render to a Line
         * @returns {*|sas.analytics.riskhp.utils.HprConstants.HowType|{Line, Bar, Distribution, Table}}
         */
        setDisplayAsLine: function() {
            this.plotQueryData.plotType.HowType = HprConstants.HowType.Line;
        },

        /**
         * @description Convenience method to determine if the ByType held in the PlotType is Subportfolio
         * @returns {boolean}
         * @memberOf HprChartModel
         */
        isByTypeSubportfolio: function() {
            return HprChartUtils.isByTypeSubportfolio(this.plotQueryData.plotType);
        },

        /**
         * @description Convenience method to determine if this is a RFIM plot
         * @returns {boolean}
         * @memberOf HprChartModel
         */
        isRFIMPlot: function() {
            return HprChartUtils.isRFIMPlot(this.plotQueryData.plotType);
        },

        /**
         * @description Convenience method to determine if this is a horizon based plot
         * @returns {boolean}
         * @memberOf HprChartModel
         */
        isHorizonPlot: function() {
            HprChartUtils.isHorizonPlot(this.plotQueryData.plotType);
        },

        /**
         * @description Convenience method to determine if this is a distribution plot
         * @returns {*|boolean}
         */
        isDistributionPlot: function() {
            return HprChartUtils.isRFIMPlot(this.plotQueryData.plotType);
        },

        /**
         * Checks to see if the plot will be made up of multiple series
         * @returns {boolean}
         */
        isMultiSeriesPlot: function() {
            var self = this,
                byType = this.getByType();
            return (_.isArray(byType) && byType.length > 1) || HprChartUtils.isDistributionPlot(self.getPlotTypeObject());
        },

        /**
         * @description Convenience method to determine if the ByType held in the PlotType is for a RiskCube
         * @returns {boolean}
         */
        isByTypeRiskCube: function() {
            return HprChartUtils.isByTypeRiskCube(this.plotQueryData.plotType);
        },

        /**
         * @description Convenience method to determine if the ByType held in the PlotTypeObject is a resultSet
         * @returns {boolean}
         */
        isByTypeResultSet: function() {
            return HprChartUtils.isByTypeResultSet(this.plotQueryData.plotType);
        },

        /**
         * @description Convenience method to determine if the ByType held in the PlotTypeObject is a Horizon
         * @returns {boolean}
         */
        isByTypeHorizon: function() {
            return HprChartUtils.isByTypeHorizon(this.plotQueryData.plotType);
        },

        /**
         * @description Convenience method to determine if the plot that is supposed to be generated a Density plot
         * @returns {boolean}
         */
        isDensityPlot: function() {
            return HprChartUtils.isDensityPlot(this.plotQueryData.plotType);
        },

        isByTypeScenario: function() {
            var byType = this.getByType();
            if (_.isObject(byType)) {
                return byType.type === sas.analytics.riskhp.models.dataitems.Scenario.SCENARIO;
            }
            return byType === sas.analytics.riskhp.models.dataitems.Scenario.SCENARIO;
        },

        /**
         * @description Is the plot that is supposed to be generated a CashFlow plot
         * @returns {boolean}
         * @memberOf HprChartModel
         */
        isCashFlowPlot: function() {
            return HprChartUtils.isCashFlowPlot(this.plotQueryData.plotType);
        },

        /**
         * @description This flag will be set if the model was just changed because of a replace data source action.  It is
         * intended to be used to stop cascading actions (i.e. crosstab re-rendering updating the plot again) until the
         * plot has fully been rendered after the replace data source action.  It is expected to be a one-time thing and
         * should be "reset" by a call to clearReplacingDataSource() when you know normal processing can continue.
         * @returns {boolean}
         */
        isReplacingDataSource: function() {
            var self = this;
            return (!_.isUndefined(self.replacingDataSource) && !_.isNull(self.replacingDataSource) && self.replacingDataSource === true);
        },

        /**
         * Removes the flag indicating that the model was just modified by a replace DataSource action.
         * @see {@link isReplacingDataSource} for more information
         */
        clearReplacingDataSource: function() {
            this.replacingDataSource = undefined;
        },

        /**
         * @description Returns the id representing the type being plotted along the x-axis for the given chart index
         * @param {number} chartIndex
         * @returns {*}
         */
        getCategoryRole: function(chartIndex) {
            return this.categoryRoles[chartIndex];
        },

        /**
         * @description Returns the response type array for the given chartIndex
         * @param {number} chartIndex
         * @returns {*|DataItem}
         * @memberOf HprChartModel
         */
        getResponseType: function(chartIndex) {
            return this.responseTypes[chartIndex];
        },

        /**
         * Returns the values for the given chart index and seriesId. If the seriesId is omitted, then it is assumed to
         * be the SINGLE_SERIES id.  (This helps with backwards compatibility for charts that either will never have
         * multiple series or don't at this time)
         * @param {number} chartIndex
         * @param {(string|null)} [seriesId] - may be null or undefined
         * @returns {number[]}
         * @memberOf HprChartModel
         */
        getResponseValues: function(chartIndex, seriesId) {
            if (seriesId) {
                return this.responseValues[chartIndex][seriesId];
            }
            return this.responseValues[chartIndex][__SINGLE_SERIES__];
        },

        /**
         * returns the responseRole for the given chartIndex
         * @param chartIndex
         * @returns {*}
         */
        getResponseRole: function(chartIndex) {
            return this.responseRoles[chartIndex];
        },

        /**
         * Returns the data item that holds the format string for the given chart index.
         * @param chartIndex
         * @returns {sas.analytics.riskhp.models.dataitems.BaseDataItem}
         */
        getResponseRoleFormatter: function(chartIndex) {
            return this.dataSource.getDataItemById(this.responseRolesFormatterIds[chartIndex]);
        },

        /**
         * Returns the format string for the responseRole, may return null if none could be found.  Check on this!!
         * @param chartIndex
         * @returns {null}
         */
        getResponseRoleFormat: function(chartIndex) {
            // For now we assume there is only one formatter per responseRole set
            var formatter = this.getResponseRoleFormatter(chartIndex);
            if (formatter) {
                return formatter.getFormat();
            }
            return null;
        },

        /**
         * @description Returns the array of series ids for the given chart index. Normally there would only be one per chart, but in the
         * case of a grouped by horizons chart there could be multiple.  The list represents the ids of the data items
         * the values are grouped by.  It will match a keyed array in the response values object.
         *
         * @param {number} [chartIndex=0] - The index for the chart
         * @returns {string[]}
         * @memberOf HprChartModel
         * @function
         */
        getSeriesIds: function(chartIndex) {
            var indexToUse = 0;
            if (chartIndex) {
                indexToUse = chartIndex;
            }
            return this.seriesIds[indexToUse];
        },

        /**
         * Given a data item id being used as a series group, returns the corresponding data item
         * @param seriesId
         * @param chartIndex
         * @returns {sas.analytics.riskhp.models.dataitems.BaseDataItem}
         */
        getSeriesDataItem: function(seriesId, chartIndex) {
            if (seriesId === __SINGLE_SERIES__) {
                return this.categoryRoles[chartIndex];
            }
            return this.dataSource.getDataItemById(seriesId);
        },

        /**
         * Returns the dataitems that represent this list of series ids
         * @param seriesIds A list of ids that represent a series
         * @param chartIndex The index of the chart the series is for.
         * @returns [ dataItem, ...]
         */
        getSeriesDataItems: function(seriesIds, chartIndex) {
            var self = this;
            if (seriesIds.length === 1 && seriesIds[0] === __SINGLE_SERIES__) {
                return [self.categoryRoles[chartIndex]];
            }
            return _.map(seriesIds, function(seriesId){
                return self.dataSource.getDataItemById(seriesId);
            });
        },

        /**
         * @description Returns the SeriesObject associated with the given series id
         * @param {string} seriesId
         * @param {number} [chartIndex=0]
         * @returns {(null|SeriesObject)}
         * @memberOf HprChartModel
         */
        getSeriesObject: function(seriesId, chartIndex) {
            var indexToUse = 0;
            if (chartIndex) {
                indexToUse = chartIndex;
            }
            return this.seriesMaps[indexToUse][seriesId];
        },

        /**
         * @description Convenience method that given the chart index to pull from returns the first SeriesObject it finds
         * @param {number} chartIndex
         * @returns {*|SeriesObject}
         */
        getFirstSeriesObject: function(chartIndex) {
            var indexToUse = 0;
            if (chartIndex) {
                indexToUse = chartIndex;
            }
            return this.series[indexToUse][0];
        },

        getResponseRoles: function(chartIndex) {
            return this.responseRoles[chartIndex];
        },


        /**
         * Returns the list of tuples this model was built with.
         * @returns {*|Array}
         */
        getTuples: function() {
            return this.plotQueryData.tuples;
        },

        /**
         * Returns true if the horizons should be displayed as absolute values versus relative values.  This is mostly
         * used for label and title determination
         * @returns {boolean}
         * @private
         */
        areHorizonsAbsolute: function() {
            var horizonOptions = this.getOptionDisplayHorizons();
            if (horizonOptions) {
                return horizonOptions.value === HprConstants.DISPLAY_HORIZONS_ABSOLUTE_VALUE;
            }
            return false;
        },

        fixHorizonsBeforeSaving: function() {
            var results = HprChartUtils.switchHorizonsInColumnToBaseHorizons(this.plotQueryData.sourceColumn, this.dataSource);

            this.plotQueryData.sourceColumn = results.column;
            // If byType has a horizon in it, can you safely swap it out with the base horizon? Do it
            if (this.isByTypeHorizon()) {
                this.plotQueryData.plotType.ByType.item = HprChartUtils.dehydrateDataItem(results.baseHorizon);
            }
        },

        fixTemplateColumnBeforeSaving: function() {
            if (!this.plotQueryData.templateColumn) {
                return;
            }
            this.plotQueryData.templateColumn = HprChartUtils.sanitizeColumnTemplate(this.plotQueryData.templateColumn);
        },

        /**
         * Returns a JSON object representation of the plot this model represents.  This is suitable for restoration
         * @returns {{type: string, plotType: (*|string), name: *, dataSourceId: (*|The), requests: *, orientation: *, statistics: *, horizons: *, classVariables: *, tuples: *, cashFlow: *, optionsPanelOptions: *, filters: *}}
         */
        toJSON: function() {
            var self = this;
            if (APPLY_JSON_FIX) {
                this.fixHorizonsBeforeSaving();
                this.fixTemplateColumnBeforeSaving();
            }
            self._validateLinkedCrosstabState();
            var jsonToReturn = {
                type: HprConstants.HPR_PLOT_PANEL,
                plotType: self.plotQueryData.plotType,
                name : self.id,
                dataSourceId: self.dataSourceId,
                crosstabId: self.crosstabId,
                linkedToCrosstab: self.linkedToCrosstab,
                requests: self.plotQueryData.requests,
                sorts: self.plotQueryData.sorts,
                sourceColumn: self.plotQueryData.sourceColumn,
                templateColumn: self.plotQueryData.templateColumn,
                statistics : HprChartUtils.dehydrateDataItems(self.plotQueryData.statistics),
                classVariables : HprChartUtils.dehydrateDataItems(self.plotQueryData.classVariables),
                tuples : self._dehydrateTuples(self.plotQueryData.tuples),
                cashFlow : self.plotQueryData.cashFlow,
                optionsPanelOptions : self.toOptionsJSON(),
                options: self.plotQueryData.options,
                filters : self._toFiltersJSON()
            };
            return jsonToReturn;
        },

        /**
         * Configures this model from the passed in JSON.  After this call it should be usable to generate a plot
         * @param expJson
         * @param obj
         */
        fromJSON: function(expJson, obj) {
            var self = this;

            // Safety check to make sure we have an Object
            var exp;
            if (obj) {
                exp = obj;
            } else {
                exp = JSON.parse(expJson);
            }

            self.crosstabId = exp.crosstabId;
            self.linkedToCrosstab = exp.linkedToCrosstab;

            // Have to set the dataSourceId first as hydration will need to use it.
            self.dataSourceId = exp.dataSourceId;

            // Re-construct plotQueryData from JSON
            self.plotQueryData.type = exp.type;
            self.plotQueryData.plotType = exp.plotType;
            self.plotQueryData.classVariables = exp.classVariables; // Will be hydrated at time of query run
            self.plotQueryData.statistics = exp.statistics; // Will be hydrated at time of query run
            self.plotQueryData.tuples = exp.tuples; // Will be hydrated at time of query run
            self.plotQueryData.sorts = exp.sorts;
            self.plotQueryData.requests = exp.requests;
            self.plotQueryData.cashFlow = exp.cashFlow;
            self.plotQueryData.options = exp.options;
            self.plotQueryData.sourceColumn = exp.sourceColumn;
            self.plotQueryData.templateColumn = exp.templateColumn;
            self.replacingDataSource = exp.replacingDataSource; // This is never saved, it is just passed through when replace datasource is in progress

            // Re-construct Plot Options from JSON
            if (exp.optionsPanelOptions) {
                self.fromOptionsJSON(exp.optionsPanelOptions);
            }

            // Re-construct Plot Filters from JSON
            if (exp.filters) {
                self._fromFiltersJSON(exp.filters);
            }

            // Final sanity check on the link state
            self._validateLinkedCrosstabState();
        },

        /**************************************************************************************
         * "Private" methods go below here
         **************************************************************************************/


        /**
         * Takes the original query data from the crosstab and dehydrates any dataitems found as all we need is the
         * id and type, and ensures the tuples array is properly structured (i.e. is an array of arrays).
         * @param queryData {} - The raw query data from the crosstab or a saved exploration
         * @private
         */
        _normalizeQueryData: function(queryData) {
            var self = this;

            // Create a clone the jQuery way of the query data
            var normalizedQueryData = jQuery.extend(true, {}, queryData);


            // Dehydrate the PlotType.ByType if it is a DataItem
            var byType = queryData.plotType.ByType;
            if (_.has(byType,'item')) {
                normalizedQueryData.plotType.ByType.item = HprChartUtils.dehydrateDataItem(byType.item);
            }

            //Dehydrate the classVaraibles array
            if (_.has(queryData,'classVariables')) {
                normalizedQueryData.classVariables = HprChartUtils.dehydrateDataItems(queryData.classVariables);
            }

            // Always expect tuples to be an array of arrays, but in a column orientation mode it is not, so first
            // fix it by wrapping the single dimension tuples array in another array.
            if (_.has(queryData,'tuples')){
                normalizedQueryData.tuples = self._dehydrateQueryDataTuples(queryData.tuples);
            }

            // Dehydrate the statistics array if supplied
            if (_.has(queryData,'statistics')) {
                normalizedQueryData.statistics = HprChartUtils.dehydrateDataItems(queryData.statistics);
            }

            if (_.has(queryData,'sourceColumn')) {
                normalizedQueryData.sourceColumn = HprChartUtils.dehydrateDataItems(queryData.sourceColumn);
            }

            if (_.has(queryData,'templateColumn')) {
                normalizedQueryData.templateColumn = self._dehydrateTemplateColumn(queryData.templateColumn);
            }

            // Dehydrate the cash flow if supplied
            if (_.has(queryData,'cashflow')) {
                normalizedQueryData.cashflow = HprChartUtils.dehydrateDataItem(queryData.cashflow);
            }

            // Finally because jQuery's clone doesn't seem to always do a deep clone of arrays, use the poor man's clone
            // method we put in
            return HprUtil.hprClone(normalizedQueryData);
        },

        /**
         * Takes a templateColumn array and dehydrates the DataItems within it so it can be stored
         * @param templateColumn
         * @returns {Array}
         * @private
         */
        _dehydrateTemplateColumn: function(templateColumn) {
            var dehydratedColumn = [];
            _.each(templateColumn, function(templateColumnItem) {
               if (templateColumnItem.item) {
                   dehydratedColumn.push({
                       item: HprChartUtils.dehydrateDataItem(templateColumnItem.item),
                       type: templateColumnItem.type
                   });
               }
               else {
                   dehydratedColumn.push({
                       item: undefined,
                       type: templateColumnItem.type
                   });
               }

            });
            return dehydratedColumn;
        },

        /**
         * Dehydrates a list of tuples so they can be efficiently stored
         * @param {DataItem[]} tuples
         * @returns {DataItemIdObject[][]}
         * @private
         */
        _dehydrateQueryDataTuples: function(tuples) {
            var properlyPackagedTuples = tuples;
            // Flag to indicate whether the original array contains arrays as we expected
            var tuplesContainsArrays = false;

            // Make sure there is at least one element and if so check the first one (since they will all be the same type)
            if (properlyPackagedTuples && _.isArray(properlyPackagedTuples) && properlyPackagedTuples.length > 0) {
                if (!_.isUndefined(properlyPackagedTuples[0])) { // There is a case where the array holds an undefined
                    tuplesContainsArrays = _.isArray(properlyPackagedTuples[0]);
                }
            }

            // If the original tuples array wasn't two dimensional then make it so.
            if (!tuplesContainsArrays) {
                if (properlyPackagedTuples) {
                    properlyPackagedTuples = [properlyPackagedTuples];
                }
                else {
                    properlyPackagedTuples = [[]];
                }
            }

            var finalTuplesArray = [];
            // FYI - a single tuple is really a column
            // Dehydrate the tuple DataItems
            _.each(properlyPackagedTuples, function(tupleArray){
                finalTuplesArray.push(HprChartUtils.dehydrateDataItems(tupleArray));
            });
            return finalTuplesArray;
        },

        /**
         * Configures the model with the passed in JSON, (generally) representing a new chart model
         * @param newContentData
         */
        configure: function(newContentData) {
            var self = this;

            // Print out the raw query data as received from the crosstab for debugging
            self._logObject(newContentData);

            // Store the json into our model (after normalizing everything)
            self.originalData = newContentData;
            self.plotQueryData = self._normalizeQueryData(newContentData);

            self.crosstabId = newContentData.crosstabId;
            self.linkedToCrosstab = true; // Is this ever not true?

            // Grab the data source id and populate the options pane once we have it.  TODO: Don't we already have it here?
            self.dataSourceId = newContentData.dataSourceId;
            var deferred = DataSourceCacheManager.getDataSource(self.dataSourceId);
            deferred.done(function(dataSource) {
                var optionsModel = dataSource.getOptions();
                // Pass in data to populate the default options; The plot type is passed in so we can show/hide certain fields
                self.optionsPanelOptions = self._buildDefaultOptionsForOptionsPanel(optionsModel);
                self._validateLinkedCrosstabState();
            });

            // filters
            if (newContentData.filters) {
                self.setFilters($.extend(true, [], newContentData.filters));
            }
        },

        /**
         * Helper method to log an object and check both the debug flag and ensure the generated string will not overrun
         * the buffer allowed by console.log
         * @param {Object} object
         * @param {string} [prefix] - An optional string to prepend the string representation of the object.  Will get a ": " separator added
         * @private
         */
        _logObject: function(object, prefix) {
            if (HprConstants.ENABLE_DEBUG) {
                var stringifiedJSON = JSON.stringify(object);
                var stringToPrint = "";
                var suffix = "";
                var prefixToPrint = (prefix && prefix.length > 0) ? prefix : "";
                var separator = (prefixToPrint.length > 0) ? ":\n" : "";
                if (stringifiedJSON.length > 100000) {
                    stringToPrint = stringifiedJSON.substring(0,100000);
                    suffix = "...";
                }
                else {
                    stringToPrint = stringifiedJSON;
                }
                this.logger.trace(prefixToPrint + separator + stringToPrint + suffix);
            }
        },

        /**
         * Builds the array for the dataDefinitions.tuples property for the query. It will have the following structure:
         * [
         *  {
         *   name:column#,
         *   dataItems:[{id: string, type: string}]
         *  }
         * ]
         * @param {[dataitems]|[[dataitems]} tuples is either an array of data items or an array of arrays of data items
         * @returns {*}
         * @private
         */
        _buildTuplesArrayForQuery: function (tuples) {
            var columnIndex = -1;
            var tupleArrays = null;

            // If only one column is included in the tuples, the tuples will be the top level array
            // otherwise if multiple columns are in the query then then the top-level tuples array will be an array of arrays
            // To ensure the subsequent code can work, we normalise the tuples array so it always has the same structure regardless
            // of whether it is one or multiple columns
            if (tuples.length > 0 && _.isArray(tuples[0])){

                // Case where tuples was an array of tuple arrays
                tupleArrays = tuples;
            }
            else {

                // Case where tuples was a single dimension array (i.e. one column)
                tupleArrays = [tuples];
            }

            // Build our structure by walking the top level array
            return _.map(tupleArrays, function (tupleDataArray) {
                // Build the object for the column
                columnIndex++;
                return {
                    "name": "column" + columnIndex,
                    "dataItems": tupleDataArray
                };
            });
        },

        /**
         * Uses the dataSource to dehydrate the tuples array.  If the dataSource is not set returns an empty array
         * @param tuples
         * @private
         */
        _dehydrateTuples: function(tuples) {
            var self = this;
            var dehydratedTuples = [];

            // See if there is at least one hydrated tuple in the tuples list
            var hydratedTuple = _.find(tuples, function(tuple){
                return typeof tuple.getId === 'function';
            });

            // If there is at least one hydrated tuple then we need to dehydrate them
            if (hydratedTuple) {
                if (!_.isEmpty(self.dataSource)) {
                    dehydratedTuples = self.dataSource.dehydrateTuples(tuples);
                }
                return dehydratedTuples;
            }
            return tuples;
        },

        /**
         * Builds the array for the report.requests property for the query
         *
         * @param {Object} plotQueryData The query data passed in from the Crosstab
         * @returns {Array}
         * @private
         */
        _buildRequestArrays: function (plotQueryData) {
            var returnArray = [];
            _.each(plotQueryData.requests, function (requestData) {
                var request = {};
                request.name = requestData.name;
                request.startRow = requestData.startRow;
                request.maxRows = requestData.maxRows;
                request.key = JSON.parse(JSON.stringify(requestData.key));  // Need to clone the key array
                // If we are plotting columnar data the last value in the key should be the SAS_ALL value
                if (HprChartUtils.isColumnarPlotType(plotQueryData.plotType)) {
                    if (request.key.length > 0) {
                        request.key[request.key.length-1].value = HprConstants.CLASSVAR_SAS_ALL_VALUE;
                    }
                }
                returnArray.push(request);
            });
            return returnArray;
        },

        /**
         * Builds the query for the midtier
         * @param dataModel
         * @param {Object} plotQueryData The query data passed in from the Crosstab
         * @returns {{version: number, name: string, type: string, orientation: *, dataSources: {id: string, name: string, label: string, lib: string, description: null, type: string, lastModifiedTimeStamp: string}, dataDefinitions: {classVariables: *, tuples: *}, report: {sorts: Array, options: Array, filters: Array, requests: *}}}
         * @private
         */
        _buildPlotQuery: function (dataModel, plotQueryData) {
            var self = this;
            self._logObject(plotQueryData, "Plot Query Data");
            var query = {
                "version": 1,
                "id": self.getId(),
                "cacheId": plotQueryData.cacheId,
                "type": self.getPlotType(),
                "dataSource": {
                    id: dataModel.dataSource.id,
                    application: dataModel.dataSource.application,
                    workGroup: dataModel.dataSource.workGroup,
                    relPath: dataModel.dataSource.relPath,
                    type: dataModel.dataSource.type
                },
                "dataDefinitions": {
                    "classVariables": HprChartUtils.getIds(plotQueryData.classVariables),
                    "tuples": self._buildTuplesArrayForQuery(plotQueryData.tuples)
                },
                "report": {
                    "sorts": plotQueryData.sorts,
                    "options": self._buildQueryOptionsJSON(self.getPlotType()),
                    "filters": self._buildQueryFilterJSON(),
                    "requests": self._buildRequestArrays(plotQueryData)
                }
            };

            // For cash flows we need to add the legs
            if (self.isCashFlowPlot()) {
                query.dataDefinitions.cashFlowLegs = [plotQueryData.cashFlow.id];
            }

            return query;
        },

        /**
         * Initiates the actual query to the midtier and coordinates the retrieval of the needed data.
         * @param queryRequest
         * @param {successCallback} successCallback
         * @param {failureCallback} failureCallback
         * @private
         */
        _executeQuery: function(queryRequest,successCallback,failureCallback) {
            var self = this;
            self.clearErrorState();
            self._logObject(queryRequest, "Plot query");
            sas.analytics.riskhp.utils.HprAPI.query(queryRequest).done(function (queryResults) {
                // If there are no query results, set the chart model to an invalid state and call the failureCallBack function
                if (!queryResults || _.isEmpty(queryResults)) {
                    self._invalidateChartModel();
                    failureCallback();
                    return;
                }

                // Have to sort the queryResults so horizontal bar chart will render like flex client
                if (self.isRFIMPlot()) {
                    if (queryResults.results && queryResults.results.data) {
                        /**
                         * @typdef {object} rfimResult
                         * @property {string} name
                         * @property {string} [label]
                         * @property {number} measure
                         */

                        /* Results is [rfimResult] */
                        var results = queryResults.results.data;
                        queryResults.results.data = _.sortBy(results,'measure');
                    }
                }

                self._processQueryResults(self.plotQueryData, queryResults,successCallback,failureCallback);
            }).fail(function (jqxhr) { //, textStatus, error
                // If there are no query results, set the chart model to an invalid state and call the failureCallBack function
                var message = null;
                if (jqxhr && jqxhr.responseJSON && jqxhr.responseJSON.message) {
                    message = jqxhr.responseJSON.message;
                }
                self._invalidateChartModel(message);
                failureCallback();
            });
        },

        /**
         * @description Execute the query for plots that can use the CacheManager
         * @param {DataSource} dataSource
         * @param {successCallback} successCallback - what to run if the query succeeds
         * @param {failureCallback} failureCallback - what to run if the query fails
         * @private
         */
        _executeQueryForCacheManagerSupportedPlot: function(dataSource, successCallback, failureCallback) {
            var self = this;


            // Fix the tuples list in the case it is a distribution plot
            if (HprChartUtils.isDistributionPlot(self.getPlotTypeObject()) && HprChartUtils.shouldExpandTuplesForDistributionPlot(self.plotQueryData.tuples)) {
                // For each tuple(column) we need to change out the statistic value to the stats we need for a distribution plot
                self.plotQueryData.tuples = HprChartUtils.getExpandedTuplesForDistributionPlot(self.plotQueryData.tuples);
            }

            // Build up list of tuple ids
            var tupleIds = [];
            _.map(self.plotQueryData.tuples, function(tupleArray){
                tupleIds.push(HprChartUtils.getIds(tupleArray));
            });

            // Generate the expansions array
            var expansions = [];
            _.each(self.plotQueryData.requests, function(request){
                var value = {};
                value["maxRows"] = request.maxRows;
                value["startRow"] = request.startRow;
                var keys = [];
                _.each(request.key, function(key){
                    keys.push(key.value);
                });
                value["key"] = keys;
                expansions.push(value);
            });

            if (self.isByTypeSubportfolio()) {
                // In a sub-portfolio plot the last key of the expansion object must be SAS_ALL
                /* {ExpansionObject} expansionObject */
                _.each(expansions, function(expansionObject) {
                    var expansionKeyLength = expansionObject.key.length;
                    if (expansionKeyLength > 0) {
                        expansionObject.key[expansionKeyLength-1] = HprConstants.CLASSVAR_SAS_ALL_VALUE;
                    }
                });
            }

            self.query = self._buildPlotQuery(dataSource,self.plotQueryData);
            var cacheKey = self.getCacheKey();
            self._logObject(cacheKey, "Plot query: (for CacheManager)\nCacheKey");
            self._logObject(expansions, "Expansions");
            self._logObject(tupleIds, "TupleIds");
            var deferred = sas.analytics.riskhp.models.cache.CacheManager.getData(cacheKey, self.getId(), expansions, tupleIds);
            deferred.done(function(results) {
                // Need to update the model's data here, when done we call the success function
                self._processQueryResults(self.plotQueryData, results, successCallback, failureCallback);
            });
            deferred.fail(function() {
                failureCallback();
            });

        },
        /**
         * @description Processes the results of retrieving the data for the plot.
         * @param {PlotQueryData} plotQueryData
         * @param {QueryResults} queryResults - The results of retrieving the data
         * @param {successCallback} successCallback - function to run if the processing is successful
         * @param {failureCallback} failureCallback - function to run if the processing fails
         * @private
         */
        _processQueryResults: function(plotQueryData, queryResults, successCallback, failureCallback) {
            var self = this;

            self._logObject(queryResults, "Plot query results");
            self.queryResultsData = queryResults;

            try {
                // Get a list of categories(x-axis) to plot - one per chart
                var requests = plotQueryData.requests;
//                var classVariables = plotQueryData.classVariables;

                // for now plotQueryData.tuples always has this form:
                // [ column, ...]
                // where column = [DataItemIdObject, ...]
                // Since in the future we might support overlaying charts we need to wrap plotQueryData.tuples in an array
                // so it matches the chart (index) to render, since we only do one right now the index will always be 0.
                // so the final form of tuples MUST be [ [ [dataItemObject, ...] ] ]
                //var tuples =  [self.query.dataDefinitions.tuples];
                var queryTuples = [self.query.dataDefinitions.tuples];

                // Right now the query data tuples are a 2-d array of dataitems.  Each dataitem array represents a column and the
                // first dimension represents the entire set of columns for a chart.  In the future if we wanted to draw multiple
                // charts on the same plot this would need to be a 3-d array of dataitems, where the first dimension is the
                // chart to render, and the 2nd dimension represents the entire set of columns for that specific chart.  For now,
                // so the logic will not change we'll wrap the queryData.tuples in another array to mimic the future behaviour.
                var tuples = [plotQueryData.tuples];

                // Category role info
                self.categoryRoleIds = self._buildCategoryRoleIdsList(requests);
                self.categoryRoles = self._buildCategoryRolesList(self.categoryRoleIds, self.dataSource);

                // Series info
                self.seriesTypes = self._buildSeriesTypes(queryResults);

                // We'll keep two series collections.  The first is used for the correct ordering and is an array.
                // The second is a map and will be used for quick lookups.  In both we will have the same object instance
                self.series = self._buildSeriesObjects(tuples, self.seriesTypes, self.queryResultsData);
                self.seriesMaps = self._buildSeriesObjectMap(self.categoryRoles, self.series);
                self.seriesIds = self._buildSeriesIdsList(tuples, self.seriesTypes, self.series);

                // Response type info
                // Get a list of responseTypes(y-axis) to plot - one per chart
                self.responseTypeIds = self._buildResponseTypeIdsList(tuples, self.getWhatType());
                self.responseTypes = self._buildRolesList(self.responseTypeIds);

                // Response role info
                self.responseRoleIds = self._buildResponseRoleIdsList(tuples, queryTuples, queryResults, requests, self.seriesIds, self.dataSource);
                self.responseRoles = self._buildResponseRolesList(self.responseRoleIds);
                self.responseRolesFormatterIds = self._buildResponseRolesFormatterIdsList(self.plotQueryData, self.dataSource);

                self.responseValues = self._getResponseValues(
                    tuples,
                    self.categoryRoles,
                    self.seriesTypes,
                    self.seriesIds,
                    self.series,
                    self.seriesMaps,
                    self.responseTypeIds,
                    self.responseRoleIds,
                    self.responseRoles,
                    self.queryResultsData); //

                // For testing, print out the things we care about
                self.printModelState(true);

                self._validateResponseValuesHavePlottableData();

                successCallback();
            }
            catch (e) {
                self.logger.error("Error creating model: "+e);
                self._invalidateChartModel(e);
                failureCallback();
            }
        },

        /**
         * Helper method to print the model's state variables that will be used for plot and table generation
         * @param {bool} printAsOne - true if you want the model state printed as one thing (useful for pretty printing), false otherwise
         */
        printModelState: function(printAsOne) {
            var self = this;
            if (printAsOne) {
                var consolidatedObject = {
                    categoryRoleIds: self.categoryRoleIds,
                    seriesTypes: self.seriesTypes,
                    series: self.series,
                    seriesIds: self.seriesIds,
                    responseTypeIds: self.responseTypeIds,
                    responseRolesIds: self.responseRoleIds,
                    responseRolesFormatterIds: self.responseRolesFormatterIds,
                    responseValues: self.responseValues
                };
                self._logObject(consolidatedObject);
            }
            else {
                // Category roles info
                self._logObject(self.categoryRoleIds,"CategoryRole Ids");

                // Series information
                self._logObject(self.seriesTypes,"Series Types");
                self._logObject(self.series,"Series");
                self._logObject(self.seriesIds,"Series Ids");

                // Response type info
                self._logObject(self.responseTypeIds,"ResponseType Ids");
                self._logObject(self.responseRoleIds,"ResponseRole Ids");
                self._logObject(self.responseRolesFormatterIds,"ResponseRolesFormatter Ids");
                self._logObject(self.responseValues,"ResponseValues");
            }
        },

        /**
         * Execute the query to update this model
         * @callback successCallback
         * @callback failureCallback
         */
        executeQuery: function (successCallback, failureCallback) {
            var self = this;

            // Do not execute the query if the chart model is invalid
            var invalidReason = self.getModelInvalidReasonString();
            if (invalidReason) {
                failureCallback();
                return;
            }

            var dataSourceDeferred = DataSourceCacheManager.getDataSource(self.dataSourceId);
            dataSourceDeferred.done(function(dataSource) {
                self.dataSource = dataSource;
                if (self._isCacheManagerDependentPlot()) {
                    self._executeQueryForCacheManagerSupportedPlot(dataSource, successCallback, failureCallback);
                }
                else {
                    self.query = self._buildPlotQuery(dataSource,self.plotQueryData);
                    self._executeQuery(self.query,successCallback,failureCallback);
                }
            });
        },

        /**
         * Isolates the logic to determine if the the plot that would be rendered from this model should use the
         * CacheManager and NOT make a direct query to the midtier
         * @private
         */
        _isCacheManagerDependentPlot: function() {
            var self = this;
            var plotTypeObject = self.getPlotTypeObject();
            return HprChartUtils.isHorizonPlot(plotTypeObject) ||
                HprChartUtils.isDistributionPlot(plotTypeObject) ||
                HprChartUtils.isComparisonPlot(plotTypeObject);
        },

        /**
         * @description Uses the current state of the model to get the cache key.  Note this always
         * regenerates the cache key.
         * @returns {sas.analytics.riskhp.models.cache.CacheKey}
         */
        getCacheKey: function() {
            var self = this;
            var classvarIds = _.map(self.plotQueryData.classVariables, function(dataItemObject){
                return dataItemObject.id;
            });
            return new sas.analytics.riskhp.models.cache.CacheKey(self.dataSourceId, classvarIds, self.plotQueryData.options, self.getFilters(), self.plotQueryData.sorts);

        },

        /**
         * Call this function when a crosstab is deleted with the id of the crosstab that was deleted
         * @param crosstabId
         * @returns {boolean} true if the model was interested in the passed in crosstabId and adjusted itself.
         */
        handleCrosstabDeleted: function(crosstabId) {
            var self = this;
            if (!self.linkedToCrosstab || crosstabId !== self.crosstabId) {
                return false;
            }
            self.crosstabId = null;
            self.linkedToCrosstab = false;
            return true;
        },

        /**
         * Call this function when the chart is being relinked to it's original crosstab
         */
        handleRelinkToCrosstab: function() {
            var self = this;
            console.log("HprChartModel: handleRelinkToCrosstab called");

            // Grab the crosstab we were linked to
            var crosstab = sap.ui.getCore().byId(self.crosstabId);
            if (crosstab) {
                var dataObject = sas.analytics.riskhp.common.riskcommons.CrosstabUtil.buildPlotDataObject(crosstab, self.plotQueryData.plotType, {});

                self.plotQueryData.classVariables = dataObject.classVariables;
                // There will never be tuples when relinking
                // if (dataObject.tuples && dataObject.tuples.length > 0) {
                //     self.plotQueryData.tuples = dataObject.tuples;
                // }

                self.plotQueryData.sorts = dataObject.sorts;
                self._overridePanelOptionsWithPlotQueryDataValues(self.optionsPanelOptions, dataObject);
                self.filters = dataObject.filters;
                // Requests can come back null on the second event (for some reason, more investigation needed)
                if (dataObject.requests) {
                    self.plotQueryData.requests = dataObject.requests;
                }

                // At this point we know we are going to requery so we clear out any error state so callers can start
                // from a good position
                self.clearErrorState();
            }
        },

        /**
         * @description Helper function to isolate the code to check if the given crosstabId is linked to
         * this chart
         * @param {string} crosstabId
         * @returns {boolean}
         */
        isLinkedToCrosstabWithId: function(crosstabId) {
            return this.linkedToCrosstab && this.crosstabId && this.crosstabId === crosstabId;
        },

        /**
         * @description Checker method to determine if the passed in information (when consumed) requires this model's state to be
         * updated.  This only works for a non-cell change event.  Cell change events are assumed to require an update
         * @param {DataItemIdObject[]} newClassVariables
         * @param {UpdateLinkedPlotObject} data
         * @returns {boolean} - true if the model needs to be updated
         * @private
         */
        _crosstabDataRequiresModelUpdate: function(newClassVariables, data) {
            var self = this,
                newStateWillBeDifferentFromCurrentState = false;
            // Safety valve to see if we really need to update the plot.
            // To do this check, "unpack" what is in the data object and compare to the model's current state
            // If no change then it is assumed there is no need to update.


            // Have classVariables changed - Note: The classVariables need to be a DataItemIdObject
            if (JSON.stringify(self.plotQueryData.classVariables) !== JSON.stringify(newClassVariables)) {
                newStateWillBeDifferentFromCurrentState = true;
            }

            // Has drill changed?
            if (JSON.stringify(self.plotQueryData.drill) !== JSON.stringify(data.info.drill)) {
                newStateWillBeDifferentFromCurrentState = true;
            }
            // Has type of horizons to display changed
            if (self.plotQueryData.displayHorizons !== data.info.displayHorizons) {
                newStateWillBeDifferentFromCurrentState = true;
            }
            // Has show labels changed?
            if (self.getOptionShowLabels().value !== data.info.showLabels) {
                newStateWillBeDifferentFromCurrentState = true;
            }
            // Has dataSourceId changed
            // Has sorts changed
            if (JSON.stringify(self.plotQueryData.sorts) !== JSON.stringify(data.info.sorts)) {
                newStateWillBeDifferentFromCurrentState = true;
            }

            // has options changed
            if (JSON.stringify(self.plotQueryData.options) !== JSON.stringify(data.info.options)) {
                newStateWillBeDifferentFromCurrentState = true;
            }

            // Has filters changed
            if (JSON.stringify(self.getFilters()) !== JSON.stringify(data.info.filters)) {
                newStateWillBeDifferentFromCurrentState = true;
            }

            // Has displayHorizons changed? or should we look at the templateColumn?
            if (self.displayHorizons !== data.info.displayHorizons) {
                newStateWillBeDifferentFromCurrentState = true;
            }

            return newStateWillBeDifferentFromCurrentState;
        },

        /**
         * Another "shim" function to help with testing the JSON fix for defect S1516410
         * @param originalTemplateColumn
         * @param sourceColumn
         * @param displayHorizons
         * @param dataSource
         * @returns {*|Array}
         * @private
         */
        _getQueryTemplateColumn: function(originalTemplateColumn, sourceColumn, displayHorizons, dataSource) {
            if (APPLY_JSON_FIX) {
                return HprChartUtils.synthesizeColumnTemplate(originalTemplateColumn, sourceColumn, displayHorizons, dataSource );
            }
            else {
                return originalTemplateColumn;
            }
        },

        /**
         * @description Call this when a crosstab is updated
         * @returns {boolean} true if the model was adjusted
         * @param {UpdateLinkedPlotObject} data - contains the information about the new state of the crosstab
         */
        handleCrosstabUpdated: function(data) {
            var self = this,
                queryTemplateColumn,
                tupleList,
                CROSSTAB_UTIL = sas.analytics.riskhp.common.riskcommons.CrosstabUtil;
            console.log("HprChartModel: handleLinkedCrosstabUpdate called");

            // If we have a pickEntryType in the info object then something happened in the crosstab and
            // we can only respond to cell changes, otherwise we let it through (this happens in the case of
            // options changing)
            if (data.info.pickEntryType && data.info.pickEntryType !== PickEntryType.CROSSTAB_DATA_CELL) {
                return false;
            }
            if (self.isLinkedToCrosstabWithId(data.crosstabId)) {
                console.log("HprChartModel: updating because linked crosstab changed");

                var crosstab = sap.ui.getCore().byId(data.crosstabId);
                var dataObject = CROSSTAB_UTIL.buildPlotDataObject(crosstab, self.plotQueryData.plotType, data.info);

                // Get the cube's capabilities and get some information about what is supported
                var tupleForCapabilities = data.info.tuple ? data.info.tuple : self.plotQueryData.tuples[0];
                if (tupleForCapabilities) {
                    tupleForCapabilities = HprChartUtils.hydrateDataItems(tupleForCapabilities, self.dataSource);
                }
                var capabilities = CROSSTAB_UTIL.getCapabilities(tupleForCapabilities, crosstab.context.cubeDataModel);
                var densitySupported = capabilities.densitySupported;
                var distributionSupported = capabilities.distributionSupported;
                var rfimSupported = capabilities.rfimSupported;

                // Does the tuple list have a basecase horizon in it?
                var hasBasecaseHorizon = HprChartUtils.hasBasecaseHorizon(data.info.tuple);

                // Check to see if the column is a computed variable column.
                var hasComputedVar = HprChartUtils.hasComputedVariable(data.info.tuple);

                // Check to see if the column is a computed variable column.
                var hasBenchmarkVar = HprChartUtils.hasBenchmarkVariable(data.info.tuple);

                // Special checks for density plots, see defect S1427953
                if (self.isDensityPlot() && (!densitySupported || hasBasecaseHorizon || hasComputedVar || hasBenchmarkVar)) {
                    self._invalidateChartModel(sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.density_not_supported.txt"));
                    return true;
                }

                // Special checks for rfim plots, see defect S1427953
                if (self.isRFIMPlot() && (!rfimSupported || hasBasecaseHorizon || hasComputedVar || hasBenchmarkVar)) {
                    self._invalidateChartModel(sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.rfim_not_supported.txt"));
                    return true;
                }

                // Special checks for distribution plots, see defect S1427953
                if (HprChartUtils.isDistributionPlot(self.getPlotTypeObject()) && (!distributionSupported || hasBasecaseHorizon || hasComputedVar || hasBenchmarkVar)) {
                    self._invalidateChartModel(sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.distribution_not_supported.txt"));
                    return true;
                }

                // Special checks for cashflow plots
                if (self.isCashFlowPlot()) {
                    var cashflowLegs = crosstab.context.getCashFlowLegs();
                    if (cashflowLegs.length === 0) {
                        self._invalidateChartModel(sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.cashflow_missing_legs.txt"));
                        return true;
                    }

                    // If the options pane was changed there may be no tuple which in that case we ASSUME we had a valid tuple before . . . EEK!!!
                    if (data.info.tuple && !sas.analytics.riskhp.common.riskcommons.CrosstabUtil.isTupleColumnValidForCashflowPlot(data.info.tuple, crosstab)) {
                        self._invalidateChartModel(sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.cashflow_missing_data_items.txt"));
                        return true;
                    }
                }

                // Now we are ready to update the chart model with the parts that matter
                self.shouldRefreshRightPanel = true;

                // Dehydrate the new class variables
                var newClassVariables = HprChartUtils.dehydrateDataItems(dataObject.classVariables);

                // Case where a new cell was selected
                if (data.selectedCellChanged) {

                    var newSourceDehydratedColumn = HprChartUtils.dehydrateDataItems(dataObject.sourceColumn);
                    // If the model had a template column before, update it.
                    if (self.plotQueryData.templateColumn) {
                        var displayHorizons = HprChartUtils.convertHorizonTypeSelectValueToDataItemType(self.getOptionDisplayHorizons().value); // Could this ever be null?
                        queryTemplateColumn = self._getQueryTemplateColumn(self.plotQueryData.templateColumn, newSourceDehydratedColumn, displayHorizons, self.dataSource);
                        if (!queryTemplateColumn) {
                           self._invalidateChartModel(sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.column_not_supported.txt"));
                           return true;
                        }
                        tupleList = crosstab.context.getTuplesFromColumnTemplate(queryTemplateColumn);

                        if (HprChartUtils.isDistributionPlot(self.getPlotTypeObject()) && HprChartUtils.shouldExpandTuplesForDistributionPlot(tupleList)) {
                            tupleList = HprChartUtils.getExpandedTuplesForDistributionPlot(tupleList);
                        }
                        // Only change our tuples if we got some
                        if (tupleList.length > 0) {
                            self.plotQueryData.sourceColumn = newSourceDehydratedColumn;
                        }
                    }
                    else {
                        tupleList = data.info.tuple; //dataObject.tuples;
                        if (HprChartUtils.isDistributionPlot(self.getPlotTypeObject()) && HprChartUtils.shouldExpandTuplesForDistributionPlot([tupleList])) {
                            tupleList = HprChartUtils.getExpandedTuplesForDistributionPlot([tupleList]);
                        }
                        try {
                            HprChartUtils.fixPlotTypeForNewTuples(self.plotQueryData.plotType, tupleList);
                        }
                        catch (e) {
                            self._invalidateChartModel(e);
                            return true;

                        }
                        self.plotQueryData.sourceColumn = newSourceDehydratedColumn; // Do we need to fix the source column?
                    }
                }
                else {

                    // We need to know what type of horizons to display.  The info will have this, we'll use this in several places
                    // The crosstab context gives us the options name for what type of horizon to display. Unfortunately this
                    // isn't the actual DataItem(Horizon) type.  This is needed by the later functions so we fix it here
                    var displayHorizons = HprChartUtils.convertHorizonTypeSelectValueToDataItemType(data.info.displayHorizons);

                    // Rehydrate the templateColumn (if we have one) so we can get the new items (if any)
                    if (self.plotQueryData.templateColumn) {
                        queryTemplateColumn = self._getQueryTemplateColumn(self.plotQueryData.templateColumn, self.plotQueryData.sourceColumn, displayHorizons, self.dataSource);
                        if (!queryTemplateColumn) {
                            self._invalidateChartModel(sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.column_not_supported.txt"));
                            return true;
                        }
                        if (self.dataSource.getValidRiskCubes().length === 0) {
                            queryTemplateColumn = _.filter(queryTemplateColumn, function(item){
                                return item.type !== sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE;
                            });
                        }
                        tupleList = crosstab.context.getTuplesFromColumnTemplate(queryTemplateColumn, {ordered: false});

                        // This was the old behavior which resulted in (most of the time) the plot going into zero state
                        // Which had a side affect of making the panel title go to a default value (i.e. no column path)
                        // The new behavior with the change above uses the original template column and therfore the
                        // panel's title stays as it was but the plot does not go into a zero state.
//                        tupleList = crosstab.context.getTuplesFromColumnTemplate(hydratedTemplateColumn);
                        if (HprChartUtils.isDistributionPlot(self.getPlotTypeObject()) && HprChartUtils.shouldExpandTuplesForDistributionPlot(tupleList)) {
                            // Note that tupleList here is in the correct form unlike above
                            tupleList = HprChartUtils.getExpandedTuplesForDistributionPlot(tupleList);
                        }
                    }
                    else {
                        tupleList = HprChartUtils.fixTuplesForHorizonType(self.plotQueryData.tuples); //,displayHorizons,self.dataSource
                    }

                    // Have to dehydrate the tupleList long enough to do the "has changed" comparison
                    var dehydratedTuples = self._dehydrateQueryDataTuples(tupleList);
                    if (!self._crosstabDataRequiresModelUpdate(newClassVariables, data) && JSON.stringify(dehydratedTuples) === JSON.stringify(self.plotQueryData.tuples)) {
                        console.log("New data object would not result in changed plot so ignoring update request");
                        return false;
                    }
                }

                // Check the tuples we got and return if we got nothing
                if (tupleList && tupleList.length > 0) {
                    self.plotQueryData.tuples = self._dehydrateQueryDataTuples(tupleList);
                }
                else {
                    self.invalidReasonString = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.column_not_supported.txt");
                    return true;
                }

                // Update class variables
                self.plotQueryData.classVariables = newClassVariables;

                self.plotQueryData.sorts = dataObject.sorts;
                self._overridePanelOptionsWithPlotQueryDataValues(self.optionsPanelOptions, dataObject);
                self.plotQueryData.options = dataObject.options;
                self.filters = dataObject.filters;

                // dataObject.requests will be null if coming from the apply filter/options case so we use what we have
                if (dataObject.requests && data.selectedCellChanged) {
                    self.plotQueryData.requests = dataObject.requests;
                }

                // TODO: what about statistics
                // TODO: what about cashflow

                // At this point we know we are going to requery so we clear out any error state so callers can start
                // from a good position
                self.clearErrorState();
                return true;
            }
            if (!self.linkedToCrosstab) {
                console.log("HprChartModel: NOT updating because linkedToCrosstab flag was false");
            }
            else {
                console.log("HprChartModel: NOT updating because linked crosstab was not the one this model was interested in: " + this.crosstabId);
            }
            return false;
        },

        /**
         * @description Looks at the responseValues object in this model and determines if it has any data that could be plotted.
         * @throws An error if the responseValues are not plottable (this usually means there was no data for some reason)
         * @private
         */
        _validateResponseValuesHavePlottableData: function() {
            var self = this;

            if (self._isCacheManagerDependentPlot()) {
                var seriesWithValues = _.find(self.series, function(chartSeriesObject){
                    var plottableSeriesObject = _.find(chartSeriesObject, function(seriesObject){
                        var values = seriesObject.getValues();
                        var plottableValue = _.find(values, function(value){
                            if (_.isObject(value)) {
                                return (!_.isUndefined(value.value) && !_.isNull(value.value) &&  value.value !== ".");
                            }
                            else {
                                return (!_.isUndefined(value) && !_.isNull(value) && value !== ".");
                            }
                        });
                        return (!_.isNull(plottableValue) && !_.isUndefined(plottableValue));
                    });
                    return (!_.isNull(plottableSeriesObject) && !_.isUndefined(plottableSeriesObject));
                });
                if (!seriesWithValues) {
                    throw sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.no.model.txt");
                }
                return;
            }

            // Case where we are not dealing with a CacheManager based plot ()
            /**
             * Accumulate all the plottable values from the responseValues object.  This is a
             * recurrsive algorithm so beware.
             * @param responseValue
             * @param valueAccumulator
             */
            var accumulateValues = function(responseValue, valueAccumulator) {
                if (_.isNull(responseValue) || responseValue === "NaN" || _.isNaN(responseValue)) {
                    // no-op
                }
                else if (_.isArray(responseValue) && responseValue.length > 0) {
                    _.each(responseValue, function(responseValueObject){
                        accumulateValues(responseValueObject,valueAccumulator);
                    });
                }
                else if (_.isObject(responseValue)) {
                    var responseValueKeys = _.keys(responseValue);
                    _.each(responseValueKeys, function(key) {
                        var newResponseValue = responseValue[key];
                        accumulateValues(newResponseValue, valueAccumulator);
                    });
                }
                else {
                    valueAccumulator.push(responseValue);
                }
            };

            var accumulatedValues = [];
            accumulateValues(self.responseValues,accumulatedValues);
            var hasValues = accumulatedValues.length > 0;
            if (!hasValues) {
                throw sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.no.model.txt");
            }
        },


        /*************************
         * BEGIN Private Query extract functions
         *************************/

        /**
         * @description A Category Role is the category to use on the X-Axis.  It is used as the x-axis title (or main label)
         * among other things. For example, if we were plotting family member's ages as a bar plot
         * then the Category Role would be 'Family Member'. It indicates the type of the values (it will be something like
         * Horizon in the case of a horizon plot or the name of a class var in the case of a sub-portfolio plot) that are being plotted along the
         * X-Axis. This will return an array of ids (normally a data item id but for certain types of plots will just be
         * a string).  There will be one category role id per plot (layer) to generate.
         * @param {QueryRequest[]} requests
         * @returns {string[]}
         * @private
         */
        _buildCategoryRoleIdsList: function(requests) {
            var self = this;
            self.logger.trace("entering: _buildCategoryRoleIdsList");

            // Create the array to return
            var returnArray = [];

            // If we are plotting by subportfolio or the plot is a cashflow then the last tuple in the query data's
            // requests array holds the dataitem's id
            if (self.isByTypeSubportfolio() || self.isCashFlowPlot() || self.isRFIMPlot()) {

                // Safety check to make sure we have a requests object
                if (requests.length === 0) {
                    return returnArray;
                }

                _.each(requests, function(request, index) {
                    if (HprChartUtils.isTopLevelQuery(self)) {
                        returnArray.push(self._getTopLevelClassVarId());
                    }
                    else {
                        var keyTupleArray = request.key;
                        if (keyTupleArray.length > 0) {
                            var lastKeyTuple = _.last(keyTupleArray);
                            returnArray.push(lastKeyTuple.id);
                        }
                        else {
                            // Technically if the keyTupleArray is empty we have an error, but for this we will push a 'fake' id
                            returnArray.push('unknown ' + index);
                        }
                    }
                });
            }
            else if (self.isDensityPlot()) {
                returnArray.push('density'); // Density doesn't really have a category, but we need one to push forward.
            }
            else if (self.isMultiSeriesPlot()) {
                if (HprChartUtils.isMultiSeriesHorizonPlot(self.getPlotTypeObject())) {
                    returnArray.push('multi_series_horizon');
                }
                else {
                    returnArray.push('multi_series');
                }
            }
            else {
                // In all other cases the ByType has the data item for the category, we just need it's id
                // Note:  we are assuming there is only one chart to be rendered.  If multiple charts are needed per plot then
                // we'll need to get an array of ByTypes, which we don't today
                var byType = self.getByType();
                if (byType && _.has(byType,'item')) { // be safe about this
                    returnArray.push(byType.item.id);
                }
                else {
                    throw "No category found when getting category id";
                }
            }
            return returnArray;
        },

        /**
         * @description Given a list of categoryRoleIds returns the associated data items. The plotting code will use these to get the
         * labels along the x-axis for the category groupings.  In the case of a DensityPlot these will not be dataitems, but instead
         * will be the actual label(s) to use.
         * @param {string[]} categoryRoleIds
         * @param {DataSource} dataSource
         * @returns {(string[]| DataItem[])} - One label/DataItem for each category(chart) to be drawn
         * @private
         */
        _buildCategoryRolesList: function(categoryRoleIds, dataSource) {
            var self = this;
            var categoryRoles = [];

            console.log("entering: _buildCategoryRolesList");

            if (self.isDensityPlot()) {
                return categoryRoleIds; // Just return what we got as there are no data items for the category role in a density plot
            }
            else if (self.isMultiSeriesPlot() && !self.isByTypeSubportfolio() /* && !self.isDistributionPlot()*/) {
                return categoryRoleIds; // Just return what we got as multi-series plots  (that are not by subportfolio) won't have data item ids
            }
            _.each(categoryRoleIds, function(categoryRoleId){
                var dataItem = dataSource.getDataItemById(categoryRoleId);
                if (dataItem) {
                    categoryRoles.push(dataItem);
                }
            });
            if (categoryRoleIds.length !== categoryRoles.length) {
                throw "Could not find a corresponding dataitem for a passed in category id";
            }
            return categoryRoles;
        },

        /**
         * @description A series in a plot is made up one or more DataItem types.  This method returns the types that are to be found
         * in each of the column tuple arrays.  When walking the column tuple arrays, each tuple should be searched for
         * these types in order to find the series.  Common series should/will be grouped together.
         * @param {QueryResults} queryResults
         * @returns {string[]}
         * @private
         */
        _buildSeriesTypes: function(queryResults) {
            var self = this,
                byType = this.getByType();

            if (self.isMultiSeriesPlot()) {
                if (HprChartUtils.isDistributionPlot(self.getPlotTypeObject())) {
                    // Distribution plots can be by subportfolio, riskcube or horizon.  The plot type for them, however,
                    // won't necessarily have an array of data items in their by type (as of 4/24/2018) so we
                    // need to deduce what their series types will be.
                    if (self.isByTypeRiskCube()) {
                        return [sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE];
                    }
                    else if (self.isByTypeHorizon()) {
                        return [sas.analytics.riskhp.models.dataitems.Horizon.HORIZON];
                    }
                    // Everything else is a Sub-portfolio plot so it must be by classification variable (the last one in the requests key)
                    return [sas.analytics.riskhp.models.dataitems.Classvar.CLASSVAR];
                }
                // Since we are a multi-series plot we know the byType is an array of types, with the last type being the type we are plotting
                // so it is removed from the list
                return _.first(byType,byType.length-1);
            }
            else if (self.isCashFlowPlot()) {
                var tempSet = {};
                _.map(queryResults.results.data, function(dataObject){
                    var keys = _.keys(dataObject);
                    _.each(keys, function(key) {
                        if (key !== "date" && key !== "value") {
                            tempSet[key] = "";
                        }
                    });
                });
                return _.keys(tempSet);
            }
            else if (HprChartUtils.isComparisonPlot(self.getPlotTypeObject()) && HprChartUtils.isByTypeSubportfolio(self.getPlotTypeObject())) {
                // If we are a comparison plot AND it is be subportfolio then we know the series is a statistic
                return [sas.analytics.riskhp.models.dataitems.Statistic.STATISTIC];
            }
            else if (_.isObject(byType)) {
                return [ byType.item.type];
            }
            // Fallback case to support the older style of queries (i.e. those making direct queries to the server)
            return [__SINGLE_SERIES__];
        },

        /**
         * @description Builds the list of series objects for the plot. There will be one inner array for each chart to generated
         * @param {DataItemIdObject[][]} tuples
         * @param {string[]} seriesTypes - the DataItem types that make up each series
         * @param queryResults
         * @returns {SeriesObject[][]}
         * @private
         */
        _buildSeriesObjects: function(tuples, seriesTypes, queryResults) {
            var self = this,
                allChartsSeries = [];

            this.logger.trace("entering: _buildSeriesObjects");

            if (self.isCashFlowPlot()) {
                // For a cashflow we just create the SeriesObjects off of the seriesTypes we have
                var tempArray = [];
                _.each(seriesTypes, function(seriesType) {
                    var seriesObject = new sas.analytics.riskhp.common.riskcommons.plot.SeriesObject([seriesType], true);
                    seriesObject.setName(seriesType);
                    tempArray.push(seriesObject);
                });
                return [tempArray];
            }

            if (self.isRFIMPlot()) {
                // RFIM plot is columnar like subportfolios but it comes directly from the server and not the CacheManager
                // so we have to process the data differently
                var seenSeries = {};
                // Want to keep in row order so we use an array instead of a map
                var series = [];
                _.each(queryResults.results.data, function(queryResultData){
                    var seriesObject = new sas.analytics.riskhp.common.riskcommons.plot.SeriesObject([queryResultData.name]);
                    seriesObject.setName(queryResultData.name);
                    var label = (queryResultData.label) ? queryResultData.label : queryResultData.name;
                    seriesObject.setLabel(label);
                    seriesObject.setValues([queryResultData.measure]);

                    // Add this seriesObject if we haven't seen it before
                    var seriesId = seriesObject.getId();
                    if (!_.has(seenSeries, seriesId)) {
                        seenSeries[seriesId] = seriesObject;
                        series.push(seriesObject);
                    }
                });
                allChartsSeries.push(series);
            }

            /**
             * @typedef {Object} ResultsData
             * @param {[number]} data
             * @param {Object} value - key/value pair
             */
            /**
             * @typedef {Object} QueryResultsObject
             * @param {ExpansionObject} expansion
             * @param {[ResultsData]} data
             */
            // The goal here is to create a list of the unique buckets(series). To do this we walk each set of column
            // tuples, use the seriesTypes array to pull out the dataitems that are important, and then build our
            // buckets(series) off of that.  Making sure to not have duplicates
            else if (self.isByTypeSubportfolio() ) {
                // Subportfolio is a columnar plot so the series are row based
                var topLevelQuery = HprChartUtils.isTopLevelQuery(self);
                _.each(queryResults, function(queryResult){
                    var seenSeries = {};
                    // Want to keep in column order so we use an array instead of a map
                    var series = [];
                    _.each(queryResult.data, function(queryResultData){
                        var dataItemIds = _.values(queryResultData.value); // Will be a display name!!!
                        var seriesObject = new sas.analytics.riskhp.common.riskcommons.plot.SeriesObject(dataItemIds);
                        if (topLevelQuery) {
                            seriesObject.setName(sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.path.value.all.txt"));
                        }
                        else {
                            seriesObject.setName(dataItemIds[0]);
                        }
                        seriesObject.setValues(queryResultData.data);

                        // Add this seriesObject if we haven't seen it before
                        var seriesId = seriesObject.getId();
                        if (!_.has(seenSeries, seriesId)) {
                            seenSeries[seriesId] = seriesObject;
                            series.push(seriesObject);
                        }
                   });
                    allChartsSeries.push(series);
                });
            }
            else if (HprChartUtils.isComparisonPlot(self.getPlotTypeObject())) {
                // This is a comparison across a row
                _.each(tuples, function(chartTuplesList, chartTupleIndex) {
                    var seenSeries = {};
                    // Want to keep in column order so we use an array instead of a map
                    var series = [];
                    // Walk over each tuple array in the passed in tuples (see above for this format of this list)
                    _.each(chartTuplesList, function (tupleList, tupleIndex) {
                        var dataItemIds = [];

                        // Find the dataitems in the current tupleList object that represent the types expected to be grouped
                        // together in the series.  We also make sure the ids of those items are collected up in the correct order.
                        _.each(seriesTypes, function (seriesType) {
                            var foundElement = _.find(tupleList, function (tuple) {
                                if (HprChartUtils.isHorizonType(seriesType)) {
                                    return HprChartUtils.isAnyTypeOfHorizon(tuple);
                                }
                                else {
                                    return tuple.type === seriesType;
                                }
                            });
                            if (foundElement) {
                                dataItemIds.push(foundElement.id);
                            }
                        });
                        if (dataItemIds.length !== seriesTypes.length) {
                            // we have an error here as the tuple array we processed didn't have all the dataitem types it needed
                        }

                        var seriesObject = new sas.analytics.riskhp.common.riskcommons.plot.SeriesObject(dataItemIds);
                        // See defect S1482578 as to why this needs to be done
                        if (!HprChartUtils.isDistributionPlot(self.getPlotTypeObject())) {
                            var values = queryResults[chartTupleIndex].data[0].data[tupleIndex];
                            seriesObject.setValues([values]);  // Must be an array
                        }

                        // Add this seriesObject if we haven't seen it before
                        var seriesId = seriesObject.getId();
                        if (!_.has(seenSeries, seriesId)) {
                            seenSeries[seriesId] = seriesObject;
                            series.push(seriesObject);
                        }
                    });
                    allChartsSeries.push(series);
                });
            }
            else {
                _.each(tuples, function(chartTuplesList) {
                    var seenSeries = {};
                    // Want to keep in column order so we use an array instead of a map
                    var series = [];
                    // Walk over each tuple array in the passed in tuples (see above for this format of this list)
                    _.each(chartTuplesList, function (tupleList, index) {
                        var dataItemIds = [];

                        // Find the dataitems in the current tupleList object that represent the types expected to be grouped
                        // together in the series.  We also make sure the ids of those items are collected up in the correct order.
                        _.each(seriesTypes, function (seriesType) {
                            var foundElement = _.find(tupleList, function (tuple) {
                                if (HprChartUtils.isHorizonType(seriesType)) {
                                    return HprChartUtils.isAnyTypeOfHorizon(tuple);
                                }
                                else {
                                    return tuple.type === seriesType;
                                }
                            });
                            if (foundElement) {
                                dataItemIds.push(foundElement.id);
                            }
                        });
                        if (dataItemIds.length !== seriesTypes.length) {
                            // we have an error here as the tuple array we processed didn't have all the dataitem types it needed
                        }

                        var seriesObject = new sas.analytics.riskhp.common.riskcommons.plot.SeriesObject(dataItemIds);

                        if (dataItemIds.length === 1) {

                        }
                        // Add this seriesObject if we haven't seen it before
                        var seriesId = seriesObject.getId();
                        if (!_.has(seenSeries, seriesId)) {
                            seenSeries[seriesId] = seriesObject;
                            series.push(seriesObject);
                        }
                    });
                    allChartsSeries.push(series);
                });
            }
            return allChartsSeries;
        },

        /**
         * @description Takes the series that we have created and maps them based on their key and chart index so we can find them
         * later by key
         * @param {DataItem[]} categoryRoles
         * @param {SeriesObject[][]} series
         * @returns {{seriesId: SeriesObject}[]}
         * @private
         */
        _buildSeriesObjectMap: function(categoryRoles, series) {
            var chartSeriesMaps = [];

            console.log("entering: _buildSeriesObjectMap");

            // Walk each category role generating a map of the relevant series objects.
            _.each(categoryRoles, function(categoryRole, chartIndex) {
                var seriesMap = {};
                _.each(series[chartIndex], function(seriesObject){
                    seriesMap[seriesObject.getId()] = seriesObject;
                });
                chartSeriesMaps.push(seriesMap);
            });
            return chartSeriesMaps;
        },

        /**
         * @description Builds the series ids (aka, the buckets)  See above for the definition of a seriesId
         * @param {DataItemIdObject[][]} tuples
         * @param {string[]} seriesTypes - An array of dataitem types that represent a series.
         * @param {} series
         * @returns {string[][]}
         * @private
         */
        _buildSeriesIdsList: function(tuples, seriesTypes, series) {
            var self = this,
                allChartsSeriesIds = [];

            console.log("entering: _buildSeriesIdsList");

            if (HprChartUtils.isDistributionPlot(self.getPlotTypeObject()) && self.isByTypeSubportfolio()) {
                _.each(series, function(chartSeries) {
                    var seriesIds = [];
                    _.each(chartSeries, function(seriesObject){
                        seriesIds.push(seriesObject.getId());
                    });
                    allChartsSeriesIds.push(seriesIds);
                });
                return allChartsSeriesIds;
            }
            if (self.isByTypeSubportfolio() || self.isRFIMPlot()) {
                _.each(series, function(chartSeries) {
                    var seriesIds = [];
                    _.each(chartSeries, function(seriesObject){
                        seriesIds.push(seriesObject.getId());
                    });
                    allChartsSeriesIds.push(seriesIds);
                });
                return allChartsSeriesIds;
            }
            if (self.isDensityPlot()) {
                allChartsSeriesIds.push([__SINGLE_SERIES__]);
                return allChartsSeriesIds;
            }

            if (self.isCashFlowPlot()) {
                allChartsSeriesIds.push(seriesTypes);
                return allChartsSeriesIds;
            }

            _.each(tuples, function(chartTuplesList){
                var seenSeriesIds = [];
                var seriesIds = [];

                // Walk over each tuple array in the passed in tuples (see above for this format of this list)
                _.each(chartTuplesList, function(tupleList){
                    var dataItemIds = [];

                    // Find the dataitems in the current tupleList object that represent the types expected to be grouped
                    // together in the series.  We also make sure the ids of those items are collected up in the correct order.
                    _.each(seriesTypes, function(seriesType) {
                        var foundElement = _.find(tupleList, function(tuple){
                            if (HprChartUtils.isHorizonType(seriesType)) {
                                return HprChartUtils.isAnyTypeOfHorizon(tuple);
                            }
                            else {
                                return tuple.type === seriesType;
                            }
                        });
                        if (foundElement) {
                            dataItemIds.push(foundElement.id);
                        }
                    });
                    if (dataItemIds.length !== seriesTypes.length) {
                        // we have an error here
                    }

                    // Synthesize a unique id for this series
                    var uniqueSeriesId = HprChartUtils.generateUniqueSeriesIdFromIds(dataItemIds);
                    if (_.indexOf(seenSeriesIds,uniqueSeriesId) === -1) {
                        seenSeriesIds.push(uniqueSeriesId);
                        // Push the ids array (which represents one series) into our final seriesIds array
                        seriesIds.push(uniqueSeriesId);
                    }
                });
                allChartsSeriesIds.push(seriesIds);
            });
            return allChartsSeriesIds;
        },

        /**
         * @description Returns the id of the data item that is being plotted on the y-axis,  One per chart
         * @param {DataItemIdObject[][][]} tuples
         * @param {string} whatType - What is the DataItem type of the response ids to find
         * @returns {DataItemIdObject[]}
         * @private
         */
        _buildResponseTypeIdsList: function(tuples, whatType) {
            var self = this;
            // Right now the query data tuples are a 2-d array of dataitems.  Each dataitem array represents a column and the
            // first dimension represents the entire set of columns for a chart.  In the future if we wanted to draw multiple
            // charts on the same plot this would need to be a 3-d array of dataitems, where the first dimension is the
            // chart to render, and the 2nd dimension represents the entire set of columns for that specific chart.  For now,
            // so the logic will not change we'll wrap the queryData.tuples in another array to mimic the future behaviour.
            var responseTypes = [];

            console.log("entering: _buildResponseTypeIdsList");

            if (self.isDensityPlot()) {
                responseTypes.push(sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.DensityGraph.likelihood.txt"));
            }
            else if (self.isRFIMPlot()) {
                responseTypes.push(sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.RFIMGraph.responseType.txt"));
            }
            else if (self.isCashFlowPlot()) { // We need the stat from the tuples list
                responseTypes = self._getResponseTypeIdsForCashFlowPlot(tuples);
            }
            else if (HprChartUtils.isDistributionPlot(self.getPlotTypeObject()) && self.isByTypeSubportfolio()) {
                responseTypes = self._getResponseTypeIdsForDistributionBySubportfolioPlot(tuples);
            }
            else {
                responseTypes = self._getResponseTypeIdsForGeneralPlot(tuples, whatType);
            }

            // Final sanity check to make sure we got something
            if (responseTypes.length === 0) {
                throw "There were no response types found.";
            }
            return responseTypes;
        },

        /**
         * @description Isolates the code to get the responseTypeIds for a CashFlow plot
         * @param {DataItemIdObject[]} tuples
         * @returns {*|Array}
         * @private
         */
        _getResponseTypeIdsForCashFlowPlot: function(tuples) {
             var responseTypeObjects = [];
            // For each element in the first dimension of the tuples array find the corresponding dataitem that represents
            // the statistic as that is what Cashflow plots use
            _.each(tuples, function(chartTuples){

                // grab the first column
                var firstColumnTuples = chartTuples[0];

                // Find the id of the data item that corresponds to the statistic
                var foundElement = _.find(firstColumnTuples, function(tupleElement){
                    return tupleElement.type === sas.analytics.riskhp.models.dataitems.Statistic.STATISTIC;
                });

                // If we found the item then push it to our list otherwise fail fast
                if (foundElement) {
                    responseTypeObjects.push(foundElement);
                }
                else {
                    throw "Could not find the statistic for the cashflow chart";
                }
            });
            return responseTypeObjects;
        },

        /**
         * Isolates the code to get the responseTypeIds for a Distribution by Sub-portfolio plot
         * @param tuples
         * @returns [ DataItemObject, ...]
         * @private
         */
        _getResponseTypeIdsForDistributionBySubportfolioPlot: function(tuples) {
            var responseTypes = [];
            // distribution always uses the outvar in the tuple so just need an
            // array with equal length of the categoryRoles

            // For each element in the first dimension of the tuples array find the corresponding dataitem that represents
            // the WhatType (the type we are graphing).  Since presumably each of the tuples for a specific chart have the
            // same column types we can use the first column as our source for the WhatType we will return for each
            // particular chart
            _.each(tuples, function(chartTuples){

                // grab the first column
                var firstColumnTuples = chartTuples[0];

                // Find the id of the data item that corresponds to our OUTVAR.  This is our guy!!
                var foundElement = _.find(firstColumnTuples, function(tupleElement){
                    return tupleElement.type === sas.analytics.riskhp.models.dataitems.Outvar.OUTVAR;
                });

                // If we found the item then push it to our list otherwise fail fast
                if (foundElement) {
                    responseTypes.push(foundElement);
                }
                else {
                    throw "Could not find the outvar for the distribution chart";
                }
            });
            return responseTypes;
        },

        /**
         * Isolates the code to get the responseTypeIds for a default plot.
         * @param tuples
         * @param whatType
         * @returns [ DataItemObject, ...]
         * @private
         */
        _getResponseTypeIdsForGeneralPlot: function(tuples, whatType) {
            var responseTypes = [];

            if (whatType === HprConstants.WhatType.Distribution) {
                whatType = sas.analytics.riskhp.models.dataitems.Outvar.OUTVAR;
            }

            // For each element in the first dimension of the tuples array find the corresponding dataitem that represents
            // the WhatType (the type we are graphing).  Since presumably each of the tuples for a specific chart have the
            // same column types we can use the first column as our source for the WhatType we will return for each
            // particular chart
            _.each(tuples, function(chartTuples){

                // grab the first column
                var firstColumnTuples = chartTuples[0];

                // Find the id of the data item that corresponds to our WhatType.  This is our guy!!
                var foundElement = _.find(firstColumnTuples, function(tupleElement){
                    return tupleElement.type === whatType;
                });

                // If we found the item then push it to our list otherwise fail fast
                if (foundElement) {
                    responseTypes.push(foundElement);
                }
                else {
                    throw "Could not find the response type "+whatType +" for the chart in the first column data";
                }
            });
            return responseTypes;
        },

        /**
         * @description Helper method to take a list of DataItemIdObjects (that are acting as roles) and retrieve the corresponding DataItem.
         * If the DataItem is not found the original DataItemIdObject is pushed into the resulting array
         * @param {DataItemIdObject[]} roleIds - The DataItemIdObjects to build the roles from
         * @returns {DataItem[]}
         * @private
         */
        _buildRolesList: function(roleIds) {
            var self = this;
            var rolesArray = [];

            self.logger.trace("entering: _buildRolesList");

            _.each(roleIds, function(roleIdObject){
                var dataItem = self.dataSource.getDataItemById(roleIdObject.id);
                if (dataItem) {
                    rolesArray.push(dataItem);
                }
                else {
                    if (!roleIdObject.id) {
                        self.logger.debug("DataItemIdObject: " + roleIdObject +" did not have an id attribute pushing it as is into rolesArray");
                    }
                    else {
                        self.logger.debug("Could not find a corresponding dataitem for roleid: " + roleIdObject.id + " pushing DataItemIdObject into rolesArray");
                    }
                    rolesArray.push(roleIdObject);
                }
            });
            return rolesArray;
        },

        /**
         * @description It is the x-axis tick values (or buckets if you will).  This is the aggregated method. It defers to
         * helper methods for the different plot types.
         *
         * @param {DataItemIdObject[][]} tuples
         * @param {QueryDataDefinitionTuple[][]} queryTuples
         * @param {QueryResults} queryResults
         * @param {QueryRequest[]} requests
         * @param {string[][]} seriesIds
         * @param {DataSource} dataSource
         * @returns {(string[][] | DataItemIdObject[][])} -  The jury is still out as to whether this should be an array of objects or just the dataItem.id
         * @private
         */
        _buildResponseRoleIdsList: function(tuples, queryTuples, queryResults, requests, seriesIds, dataSource) {
            var self = this;

            console.log("entering: _buildResponseRoleIdsList");

            if (self.isDensityPlot()) {
                return self._getResponseRoleIdsForDensityPlot(queryResults);
            }
            if (self.isCashFlowPlot()) { // We need the stat from the tuples list
                return self._getResponseRoleIdsForCashFlowPlot(queryResults);
            }
            if (self.isRFIMPlot()) {
                return seriesIds;
            }

            if (HprChartUtils.isDistributionPlot(self.getPlotTypeObject()) /*&& self.isByTypeSubportfolio()*/) {
                if (self.isByTypeSubportfolio()) {
                    return seriesIds;
                }
                // This is an across distro plot, right now we have a list of ids that each series represents, so we convert them to dehydrated dataitems
                var finalList = [];
                _.each(seriesIds, function(chartSeriesIds) {
                    finalList.push(HprChartUtils.createDataItemIdObjects(chartSeriesIds, dataSource));
                });
                return finalList;
            }
            else if (self.isByTypeSubportfolio()) {
                return seriesIds;
            }
            else if (HprChartUtils.isMultiSeriesHorizonPlot(self.getPlotTypeObject())) {
                return self._getResponseRoleIdsForMultiSeriesHorizonPlot(tuples, dataSource);
            }
            else {
                return self._getResponseRoleIdsForGeneralPlot(tuples, self.getByType(), dataSource);
            }
        },

        /**
         * Generates the responseRoleIds for a Density plot
         * @param {QueryResults} queryResults
         * @returns {[DataItemIdObject[]]}
         * @private
         */
        _getResponseRoleIdsForDensityPlot: function(queryResults) {
            var tempArray = [],
                returnArray = [];
            // Convert the result's series information into the correct format even though it is a bit contrived for this plot type
            _.map(queryResults.results.series, function(seriesValue){
                tempArray.push({name: seriesValue, id: seriesValue});
            });
            returnArray.push(tempArray);
            return returnArray;
        },

        /**
         * @description Generates the responseRoleIds for a cashflow plot
         * @param {QueryResults} queryResults
         * @returns {string[]} This will be the seriesIds for each response role.  A Cashflow plot is effectively a time
         * series so the plotting code will need to retrieve the horizons from the series values objects.
         * @private
         */
        _getResponseRoleIdsForCashFlowPlot: function(queryResults) {
            var responseRoleIds,
                tempSet = {};
            _.map(queryResults.results.data, function(dataObject){
                var keys = _.keys(dataObject);
                _.each(keys, function(key) {
                    if (key !== "date" && key !== "value") {
                        tempSet[key] = "";
                    }
                });
            });
            responseRoleIds = _.keys(tempSet);
            return [responseRoleIds];
        },

        /**
         * Generates the response role ids for the distro by subportfolio plot
         * @param chartTuples
         * @param queryResults
         * @returns {Array}
         * @private
         */
        _getResponseRoleIdsForDistributionBySubportfolioPlot: function(chartTuples, queryResults) {
            var results = null,
                returnArray = [];

            // distribution always uses the outvar in the tuple so just need an
            // array with equal length of the categoryRoles

            // Capture the results data off the queryResults
            results = [queryResults.results.data];

            // For each set of tuples go to the results and grab the individual labels
            _.each(chartTuples, function(chartTuple, chartIndex) {

                // Walk the columns(tuple) of the chart
                _.each(chartTuple, function(tuple){
                    var name = tuple.name; // have column 0
                    // get data for first chart
                    var resultsData = results[chartIndex];

                    var chartRoleIds = [];
                    _.each(resultsData, function(resultObject){
                        var resultObjectLabel = resultObject.label;
                        var resultObjectTuples = resultObject.tuples;
                        var matchingResultObjectTuple = _.find(resultObjectTuples, function(resultObjectTuple) {
                            return resultObjectTuple.tuple = name;
                        });
                        if (matchingResultObjectTuple) {
                            chartRoleIds.push(resultObjectLabel);
                        }
                    });
                    returnArray.push(chartRoleIds);
                });
            });
            return returnArray;
            // Make sure we have an array of requests, otherwise return an empty array
            // if (lengthOfRequests === 0) {
            //     return [];
            // }
            //
            // // Find the outvar in the list of tuples
            // // var outVar = _.find(this.plotQueryData.tuples, function(tuple) {
            // //     return tuple.type === sas.analytics.riskhp.models.dataitems.Outvar.OUTVAR;
            // // });
            //
            // // For each value in the requests length push in the same outvar info
            // for (var i=0; i< lengthOfRequests; i++) {
            //     var outvarArray = [];
            //     // Get the correct tuples list for the index of the category
            //     var tuplesForCategory = this.plotQueryData.tuples[i];
            //
            //     // Find the outvar in the list of tuples
            //     var outVar = _.find(tuplesForCategory, function(tuple) {
            //         return tuple.type === sas.analytics.riskhp.models.dataitems.Outvar.OUTVAR;
            //     });
            //
            //     if (outVar) {  // Note: If we don't get an outvar we are HOSED!!!
            //         outvarArray.push({name: outVar.name, id: outVar.id}); // Note name is undefined here!!!, grab it from the datasource?
            //     }
            //     returnArray.push(outvarArray);
            // }

        },

        /**
         * Generates the response role ids for a subportfolio plot
         * @param tuples
         * @param queryResults
         * @returns {Array}
         * @private
         */
        _getResponseRoleIdsForSubportfolioPlot: function(tuples, queryResults) {
            var results = null,
                returnArray = [];

            // Subportfolio plot is columnar and the response roles come from the query results not the original query
            // results data is expected to be an array of arrays but it is not. So we fix it for now so we can walk it as if there were multiple charts
            results = [queryResults.results.data];
            // For each set of tuples go to the results and grab the individual labels
            _.each(tuples, function(chartTuples, chartIndex) {
                var chartRoleIds = [];
                var resultsData = results[chartIndex];
                _.each(resultsData, function(resultObject){
                    chartRoleIds.push(resultObject.label);
                });
                returnArray.push(chartRoleIds);
            });
            return returnArray;
        },

        /**
         * Generates the response role ids for a multi-series horizons plot
         * @param tuples
         * @param dataSource
         * @returns {Array}
         * @private
         */
        _getResponseRoleIdsForMultiSeriesHorizonPlot: function(tuples, dataSource) {
            var byTypesType = sas.analytics.riskhp.models.dataitems.Horizon.HORIZON,
                useRelativeHorizons = false,
                horizonSet = {},
                returnArray = [];

            // Goal is to return a unique sorted list of the horizons
            if (byTypesType === sas.analytics.riskhp.models.dataitems.RelativeHorizon.RELATIVEHORIZON) { // Huh??
                useRelativeHorizons = true;
            }

            // For each element in the first dimension of the tuples array find the corresponding dataitem that represents
            // the WhatType (the type we are graphing).  Since presumably each of the tuples for a specific chart have the
            // same column types we can use the first column as our source for the WhatType we will return for each
            // particular chart
            _.each(tuples, function(chartTuples) {

                // Walk each column in the chart's tuples and pull out the corresponding ByType elements
                _.each(chartTuples, function (columnTuple) {

                    // Find the id of the data item that corresponds to our ByType.  This is our guy!!
                    var foundItem = _.find(columnTuple, function (tupleElement) {
                        return tupleElement.type === sas.analytics.riskhp.models.dataitems.Horizon.HORIZON;
                    });

                    // need the real horizon
                    var horizon = dataSource.getDataItemById(foundItem.id);
                    if (useRelativeHorizons && foundItem && foundItem.type === sas.analytics.riskhp.models.dataitems.Horizon.HORIZON) {
                        // Now we need the corresponding relative horizon object.
                        horizon = dataSource.getRelatedRelativeHorizon(foundItem.id);
                    }
                    // If we found the item then push it to our list otherwise fail fast
                    if (foundItem && horizon) {
                        horizonSet[foundItem.id] = horizon;
                    }
                });

                // Now flatten and sort the list of horizons we accumulated
                // TODO: At this point we have the set of horizons (1 for each column) but they have not been bucketized.
                // TODO: This is not error proof yet - what if a horizon can't be retrieved, what if in relative dates
                var uniqueHorizons = _.values(horizonSet);
                uniqueHorizons.sort(function(horizon1, horizon2) {
                    var date1 = sas.analytics.riskhp.utils.HprUtil.convertISODateToLocalJSDate(horizon1.horizonDate);
                    var date2 = sas.analytics.riskhp.utils.HprUtil.convertISODateToLocalJSDate(horizon2.horizonDate);
                    return date1.getTime() - date2.getTime();
                });

                // Now collapse the horizons
                var finalHorizons = [];
                _.each(uniqueHorizons, function(horizon){
                    var horizonDate = horizon.horizonDate;
                    var foundHorizon = _.find(finalHorizons, function(finalHorizon){
                        return finalHorizon.horizonDate === horizonDate;
                    });
                    if (!foundHorizon) {
                        finalHorizons.push({ horizonDate: horizonDate});
                    }
                });
                returnArray.push(finalHorizons);
            });
            return returnArray;
        },

        /**
         * Generate the response role ids for an arbitrary(aka general) plot
         * @param {QueryDataDefinitionTuple[]} tuples
         * @param byType
         * @param {DataSource} dataSource
         * @returns {DataItemIdObject[]}
         * @private
         */
        _getResponseRoleIdsForGeneralPlot: function(tuples, byType, dataSource) {
            var self = this,
                byTypesType = byType.type, // We want the corresponding byType
                useRelativeHorizons = false,
                returnArray = [];

            // If we are plotting by any type of horizon then we are plotting by Horizon and not it's subtypes
            if (self.isByTypeHorizon()) {
                byTypesType = sas.analytics.riskhp.models.dataitems.Horizon.HORIZON;
            }

            if (byTypesType === sas.analytics.riskhp.models.dataitems.RelativeHorizon.RELATIVEHORIZON) {
                useRelativeHorizons = true;
            }

            // For each element in the first dimension of the tuples array find the corresponding dataitem that represents
            // the WhatType (the type we are graphing).  Since presumably each of the tuples for a specific chart have the
            // same column types we can use the first column as our source for the WhatType we will return for each
            // particular chart
            _.each(tuples, function(chartTuples) {

                var chartResponseRoles = [];
                // Walk each column in the chart's tuples and pull out the corresponding ByType elements
                _.each(chartTuples, function (columnTuple) {

                    // Find the id of the data item that corresponds to our ByType.  This is our guy!!
                    var foundItem = _.find(columnTuple, function (tupleElement) {
                        return tupleElement.type === byTypesType;
                    });

                    if (useRelativeHorizons && foundItem && foundItem.type === sas.analytics.riskhp.models.dataitems.Horizon.HORIZON) {
                        // Now we need the corresponding relative horizon object.
                        foundItem.id = dataSource.getRelatedRelativeHorizon(foundItem.id).getId();
                    }
                    // If we found the item then push it to our list otherwise fail fast
                    if (foundItem) {
                        chartResponseRoles.push(foundItem);
                    }
                });

                // Check lengths for sanity
                if (chartTuples.length !== chartResponseRoles.length) {
                    throw "Could not retrieve corresponding dataitem for ByType: " + byType +" for a column";
                }

                returnArray.push(chartResponseRoles);
            });
            return returnArray;
        },

        /**
         * @description Converts the ids for the response role to data items so labels can be generated.  For Density and CashFlow
         * plots just returns the original labels
         * @param {string[][]} responseRoleIds
         * @returns {string[][]}
         * @private
         */
        _buildResponseRolesList: function(responseRoleIds) {
            var self = this,
                responseRoles = [];

            console.log("entering: _buildResponseRolesList");

            _.each(responseRoleIds, function(responseRoleIdArray){
                if (self.isByTypeSubportfolio()) {
                    responseRoles.push(responseRoleIdArray); // In the subportfolio case we get the labels from the results
                }
                else {
                    responseRoles.push(self._buildRolesList(responseRoleIdArray));
                }
            });
            return responseRoles;
        },

        /**
         * @description Returns a list of DataItem ids for use during formatting of the x-axis labels.  There will be one per chart.
         *
         * @param {PlotQueryData} queryData
         * @returns {string[]}
         * @private
         */
        _buildResponseRolesFormatterIdsList: function(queryData) {
            var self = this,
                formatters = [];

            self.logger.trace("entering: _buildResponseRolesFormatterIdsList");

            if (self.isByTypeRiskCube()) {
                return formatters; // Always want strings
            }
            _.each(queryData.requests, function(requestObject){
                if (HprChartUtils.isTopLevelQuery(self)) {
                    formatters.push(self._getTopLevelClassVarId());
                }
                else {
                    if (requestObject.key[requestObject.key.length - 1] === undefined) {
                        console.error("For some reason the request key is undefined");
                    }
                    formatters.push(requestObject.key[requestObject.key.length - 1].id);
                }
            });
            return formatters;
        },

        /**
         * Given a list of responseRole ids and the results from a query builds the list of values for that role
         * [value|[values]|{}, ...] - The actual values in the 2nd dimension may be a single value, an array of data, or object
         * depending on the type of plot being generated. In any case the return is an array (possibly 2D) where each
         * 1st dimension element corresponds to the values for that particular chart.
         * @param {DataItemIdObject[][]} tuples
         * @param {DataItem[]} categoryRoles
         * @param {string[]} seriesTypes
         * @param {string[]} seriesIds
         * @param {SeriesObject[]} series
         * @param seriesMaps
         * @param {DataItemIdObject[]} responseTypeIds
         * @param {DataItemIdObject[]} responseRoleIds
         * @param {DataItem[][]} responseRoles
         * @param {QueryResults} queryResults
         * @returns {(SeriesObject[][]|*)}
         * @private
         */
        _getResponseValues: function(tuples, categoryRoles, seriesTypes, seriesIds, series, seriesMaps, responseTypeIds, responseRoleIds, responseRoles, queryResults) {
            var self = this,
                chartQueryResults,
                roleValues = [];

            self.logger.trace("entering: _getResponseValues");

            // TODO: in 191 look at refactoring this by leveraging the _isCacheManagerDependentPlot method
            // Looking to build this
            // [chartIndex [seriesId:{values}|[values],]]
            _.each(self.categoryRoles, function(categoryRole, chartIndex){
                if (self.isDensityPlot()) {
                    roleValues.push(self._getResponseValuesForDensityPlot(queryResults, seriesIds[chartIndex], chartIndex, responseRoleIds));
                }
                else if (self.isCashFlowPlot()) {
                    roleValues.push(self._getResponseValuesForCashFlowPlot(queryResults.results, seriesTypes, seriesIds[chartIndex], seriesMaps[chartIndex]));
                }
                else if (HprChartUtils.isDistributionPlot(self.getPlotTypeObject())) { // CacheManager dependent
                    if (self.isByTypeSubportfolio()) {
                        roleValues.push(series[chartIndex]);
                    }
                    else {
                        chartQueryResults = queryResults[chartIndex];
                        if (chartQueryResults) {
                            var queryResultsData = chartQueryResults.data[0];
                            if (queryResultsData) {
                                roleValues.push(self._getResponseValuesForDistributionPlot(queryResultsData, seriesIds[chartIndex], tuples[chartIndex], seriesMaps[chartIndex]));
                            }
                        }
                    }
                }
                else if (self.isByTypeSubportfolio()) { // CacheManager dependent
                    roleValues.push(series[chartIndex]);
                }
                else if (self.isRFIMPlot()) { // Not CacheManager dependent, but uses the CacheManager object
                    // (i.e SeriesObjects) structure as the underlying chart object (BarChart) is dependent on that
                    // object structure and I didn't want to do it two different ways
                    roleValues.push(series[chartIndex]);
                }
                else if(HprChartUtils.isHorizonPlot(self.getPlotTypeObject())) { // CacheManager dependent
                    var results = self._adaptRESTQueryResultsToCrosstabQueryResults(queryResults, tuples[chartIndex], seriesTypes, chartIndex);
                    roleValues.push(self._getResponseValuesForMultiSeriesHorizonPlot(results.data[chartIndex], seriesTypes, seriesIds[chartIndex], seriesMaps[chartIndex], tuples[chartIndex], chartIndex ));
                }
                else if (HprChartUtils.isComparisonPlot(self.getPlotTypeObject())) { // CacheManager dependent
                    roleValues.push(series[chartIndex]);
                }
                else {
                    // TODO: For a scenario chart we don't know what scenario the responseRoleIds(horizons) are in however
                    // the responseRoleIds does represent all the horizons for the graph.
                    // responseRoleIds is an array of arrays.  Each element corresponds to one chart.  The inner array maps
                    // to columns in the result.  We want to return an array of array or arrays
                    roleValues.push(self._getDefaultResponseValues(queryResults, seriesIds[chartIndex], chartIndex, responseRoleIds, responseTypeIds));
                }
            });

            return roleValues;
        },

        /**
         * Gets the response values for a Density plot
         * @param queryResults
         * @param chartSeriesIds
         * @param chartIndex
         * @param responseRoleIds
         * @returns {{}}
         * @private
         */
        _getResponseValuesForDensityPlot: function(queryResults, chartSeriesIds, chartIndex, responseRoleIds) {
            var seriesValuesObject = {},
                data = queryResults.results.data, // grab the returned data off the results
                responseValuesObject;

            _.each(chartSeriesIds, function(seriesId){
                responseValuesObject = {};
                // Collect the different data arrays and key them with their returned series label, so we can find them later.
                // Right now we only support the first density chart.  In the future we will need to loop the chart index here
                _.each(responseRoleIds[chartIndex], function(responseRoleIdObject, index) {
                    responseValuesObject[responseRoleIdObject.id] = data[index];
                });

                seriesValuesObject[seriesId] = responseValuesObject;
            });
            return seriesValuesObject;
        },

        _getResponseValuesForCashFlowPlot: function(chartQueryResults, seriesTypes, chartSeriesIds, chartSeriesMap) {
            var chartData = chartQueryResults.data,
                seriesObjects;

            // For each chart (categoryRole) specified by the chartIndex, we walk the responseRoles for that chart, which is a list
            // of ordered and unique horizons, and we accumulate objects that each hold one of these dates
            // Create a template value array.  We'll copy this into the values array for each seriesObject we have
            var accumulatorArray = [];
            _.each(chartData, function(dataObject){
                accumulatorArray.push({horizonDate: dataObject.date, value: null}); // Make a copy and we'll add value in a bit
            });

            // Walk our series and set the values object array to our template we just created
            seriesObjects = _.values(chartSeriesMap);
            _.each(seriesObjects, function(seriesObject) { // These are the groups
                seriesObject.setValues(JSON.parse(JSON.stringify(accumulatorArray)));
            });

            _.each(chartData, function(dataObject, dataObjectIndex){
                var keys = _.keys(dataObject);
                // Anything that isn't "date" or "value" one of our series
                _.each(keys, function(key) {
                    if (key !== "date" && key !== "value") {
                        var seriesObject = chartSeriesMap[key];
                        var valuesObject = seriesObject.values[dataObjectIndex];
                        valuesObject.value = dataObject[key];
                    }
                });
            });

            // Dereference the series objects from their map and return them as they have been properly hydrated
            return [_.values(chartSeriesMap)];
        },

        /**
         * @description Populates the seriesObjects for a distribution plot with the correct values
         * @param chartQueryResults
         * @param chartSeriesIds
         * @param tuples
         * @param seriesMap
         * @returns {Array}
         * @private
         */
        _getResponseValuesForDistributionPlot: function(chartQueryResults, chartSeriesIds, tuples, seriesMap) {
            var data = chartQueryResults.data;
            _.each(chartSeriesIds, function(seriesId) {
                var seriesObject = seriesMap[seriesId];
                if (seriesObject) {
                    _.each(tuples, function (tuple, tupleIndex) {
                        var foundDataItem = _.find(tuple, function (dataItem) {
                            return dataItem.id === seriesId;
                        });
                        if (foundDataItem) {
                            seriesObject.addValue(data[tupleIndex]);
                        }
                    });
                }
            });
            return []; // Don't care what we return as the series objects will hold the values.
        },

        _getResponseValuesForByTypeSubportfolio: function(queryResults, chartSeriesIds, chartIndex, responseTypeIds, responseRoleIds) {
            var seriesValuesObject,
                responseValuesObject;

            seriesValuesObject = {};
            _.each(chartSeriesIds, function(seriesId) {
                responseValuesObject = {};
                // Subportfolio plot is columnar and the response roles come from the query results not the original query
                // results data is expected to be an array of arrays but it is not. So we fix it for now so we can walk it as if there were multiple charts
                var resultsData = [queryResults.results.data];
                var responseTypeId = responseTypeIds[chartIndex];
                var chartData = resultsData[chartIndex];
                var chartResponseValues = [];

                // For each set of tuples go to the results and grab the individual labels
                _.each(responseRoleIds[chartIndex], function(responseRoleId) {

                    var chartDataObject = _.find(chartData, function(dataObject){
                        return dataObject.label === responseRoleId;
                    });
                    var valuesArray = [];
                    _.each(chartDataObject.tuples, function(chartDataObjectTuple){
                        var value = chartDataObjectTuple.data[responseTypeId.id];
                        // Never push undefined as it gives us a false positive during validation
                        if (!_.isUndefined(value) && !_.isNull(value)) {
                            valuesArray.push(value);
                        }
                    });

                    // We'll use null to indicate we have no data for this response value.  Not sure how this will play completely.
                    if (valuesArray.length > 0) {
                        chartResponseValues.push({value: valuesArray});
                    }
                    else {
                        chartResponseValues.push(null);
                    }
                });
                seriesValuesObject[seriesId] = chartResponseValues;

            });
            return seriesValuesObject;
        },

        /**
         * Generates the ResponseValues object for grouped horizons plots
         * @param queryResults
         * @param chartIndex
         * @param chartSeriesIds
         * @returns {{}}
         * @private
         */
        _getResponseValuesForGroupedHorizons: function(queryResults, chartSeriesIds, chartIndex ) {
            var self = this,
                data = queryResults.results.data; // grab the returned data off the results

            // For a grouped horizons plot we don't know what "bucket" (i.e. scenario, resultset, etc) the responseRoleIds(horizons)
            // are in however the responseRoleIds does represent all the horizons to be plotted.

            // For each chart (categoryRole) specified by the chartIndex, we walk the responseRoles for that chart, which is a list
            // of ordered and unique horizons, and we accumulate objects that each hold one of these dates
            var accumulatorArray = [];
            _.each(self.getResponseRole(chartIndex), function(horizonDate){
                accumulatorArray.push({horizonDate: horizonDate.horizonDate}); // Make a copy and we'll add value in a bit
            });

            // For each series we make a copy of our horizon objects held in the accumulatorArray
            var seriesValues = {};
            _.each(chartSeriesIds, function(seriesId) { // These are the groups
                seriesValues[seriesId] = JSON.parse(JSON.stringify(accumulatorArray));
            });

            // Now we pull out the data representing the values for the specific chart
            var chartData = data[chartIndex];
            var responseType = self.getResponseType(chartIndex); //responseTypeIds[chartIndex];
            if (chartData) {

                // Store the tuples of the original query (this will be the columns when the query was started
                var queryTuples = self.query.dataDefinitions.tuples;

                _.each(chartData.tuples, function (tuple) {

                    // Get the id of the relevant tuple from the response
                    var tupleId = tuple.tuple;

                    // Find the matching tuple object from the original query
                    var matchingQueryTuple = _.find(queryTuples, function(tupleObject){
                        return tupleObject.name === tupleId;
                    });

                    // Walk over the data items of the matchingQueryTuple looking for the dataitem with the type equal to the ByType (which is how we are grouping the values)
                    var groupType = self.getByType();
                    var groupingDataItem = _.find(matchingQueryTuple.dataItems, function(dataItem){
                        return dataItem.type === groupType;
                    });

                    // pluck the correct values array by the id of the groupingDataItem
                    var valuesArray = seriesValues[groupingDataItem.id];

                    // Grab the corresponding horizon id from the same matchingQueryTuple
                    var horizonIdToPlotOn = _.find(matchingQueryTuple.dataItems, function(dataItem){
                        return dataItem.type === sas.analytics.riskhp.models.dataitems.Horizon.HORIZON;
                    });

                    // Now get the horizon from the dataSource. TODO: what if this isn't found?
                    var horizonToPlotOn = self.dataSource.getDataItemById(horizonIdToPlotOn.id);

                    // Walk the valuesArray looking for an object with the same horizon date as the horizonToPlotOn
                    var foundHorizonValueObject = _.find(valuesArray, function(item){
                        return item.horizonDate === horizonToPlotOn.horizonDate;
                    });

                    if (foundHorizonValueObject) {
                        // At this point we have an object to attach our value to so we walk the tuples of the
                        // response data, looking for a tuple with the same tupleId

                        var columnData = _.find(chartData.tuples, function(tupleItem){
                            return tupleItem.tuple === tupleId;
                        });

                        // Finally we update the foundHorizonValueObject with the value in the columnData object corresponding to the responseTypeId
                        foundHorizonValueObject.value = columnData.data[responseType.id];
                    }
                });
            }
            return seriesValues;
        },

        /**
         * Generates the ResponseValues object for a multi series horizon plot
         * @param chartQueryResults
         * @param seriesTypes
         * @param chartSeriesIds
         * @param chartSeriesMap
         * @param chartTuples
         * @param chartIndex
         * @returns {Array}
         * @private
         */
        _getResponseValuesForMultiSeriesHorizonPlot: function(chartQueryResults, seriesTypes, chartSeriesIds, chartSeriesMap,  chartTuples, chartIndex ) {
            var self = this,
                chartData = chartQueryResults.data,
                seriesObjects;

            // For a multi-series horizon plot we don't know what "bucket" (i.e. scenario, resultset, etc) the responseRoleIds (i.e. horizons)
            // are in however the responseRoleIds does represent the full set of horizons (x-values), in order, to be plotted.

            // For each chart (categoryRole) specified by the chartIndex, we walk the responseRoles for that chart, which is a list
            // of ordered and unique horizons, and we accumulate objects that each hold one of these dates
            var accumulatorArray = [];
            _.each(self.getResponseRole(chartIndex), function(horizonDate){
                accumulatorArray.push({horizonDate: horizonDate.horizonDate, value: null}); // Make a copy and we'll add value in a bit
            });

            seriesObjects = _.values(chartSeriesMap);
            _.each(seriesObjects, function(seriesObject) { // These are the groups
                seriesObject.setValues(JSON.parse(JSON.stringify(accumulatorArray)));
            });

            // Here data and tuples should match in length
            if (chartData.length !== chartTuples.length) {
                throw "Retrieved chart data length did not match tuples length to build plot for";
            }

            // Walk tuples(aka columns)
            var columnValue = null;
            var seriesId = null;
            var seriesObject = null;
            var horizonTuple = null;
            _.each(chartTuples, function(tupleArray, index){
                // grab value for that tuple(column)
                columnValue = chartData[index];

                // generate the tuple's correct id
                seriesId = HprChartUtils.generateUniqueSeriesIdFromTuples(tupleArray, seriesTypes);

                // grab horizon from tuple
                horizonTuple = _.find(tupleArray, function(tuple){
                    return tuple.type === sas.analytics.riskhp.models.dataitems.Horizon.HORIZON;
                });

                var horizon = self.dataSource.getDataItemById(horizonTuple.id);
                // get corresponding SeriesObject from series map
                seriesObject = chartSeriesMap[seriesId];

                // find horizon in SeriesObject's value list
                var matchingValueObject = _.find(seriesObject.values, function(valueObject) {
                    return valueObject.horizonDate === horizon.horizonDate;
                });

                if (matchingValueObject){
                    matchingValueObject.value = columnValue;
                }
            });

            // At this point all the seriesObjects have had their values updated.  Now we need to return correct order?
            seriesObjects = [];
            _.each(chartSeriesIds, function(seriesId){
                seriesObjects.push(chartSeriesMap[seriesId]);
            });
            return seriesObjects;
         },

        /**
         * The is the default case of getting response values for the chart
         * @param queryResults - The results from running the query
         * @param {string[][]} chartSeriesIds
         * @param chartIndex - The index of the chart to be built
         * @param responseRoleIds - The DataItem ids of the response roles
         * @param responseTypeIds - The DataItem ids of the response types
         * @returns {{}|*}
         * @private
         */
        _getDefaultResponseValues: function(queryResults, chartSeriesIds, chartIndex, responseRoleIds, responseTypeIds) {
            var seriesValuesObject,
                data = queryResults.results.data; // grab the returned data off the results

            seriesValuesObject = {};
            _.each(chartSeriesIds, function(seriesId){
                _.each(responseRoleIds, function(responseRolesArray){
                    var responseTypeId = responseTypeIds[chartIndex];
                    var oneChartValuesArray = [];
                    _.each(responseRolesArray, function(responseRole, columnIndex) {
                        var oneColumnValue = {};
                        var chartData = data[chartIndex];
                        if (chartData) {
                            var chartDataTuples = chartData.tuples;
                            var column = chartDataTuples[columnIndex];
                            var columnDataValues = column.data;
                            var value = columnDataValues[responseTypeId.id];
                            if (!_.isUndefined(value)) {
                                oneColumnValue["value"] = value;
                            }
                            oneChartValuesArray.push(oneColumnValue);
                        }
                        else {
                            // no-op for now, but if we get here then we have a mismatch and things are going to go badly
                            //     throw sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.no.queryData.txt");
                        }
                    });
                    seriesValuesObject[seriesId] = oneChartValuesArray;
                });
            });
            return seriesValuesObject;
        },

        /**
         * @description A temporary method to take the queryResults from a query to the server and modify it to match the results from
         * a query to the CacheManager.  If the queryResults has the same structure as that which is returned from the
         * CacheManager then it is assumed to have originated there and is returned.  Otherwise the queryResults are
         * taken apart to a reconstructed to match what is expected.  Right now this method only supports the structure
         * needed to retrieve the values in the values construction code.  This is removeable once ALL plots use the
         * CacheManager for their data. Note this only returns the results for the given chartIndex, not the entire array.
         *
         * @param queryResults
         * @param {DataItemIdObject[][]} chartTuples
         * @param {DataItemType[]} seriesTypes
         * @param {number} chartIndex
         * @returns {*}
         * @private
         */
        _adaptRESTQueryResultsToCrosstabQueryResults: function(queryResults, chartTuples, seriesTypes, chartIndex){
            // First figure out if each results is in the correct form.  If it is just return it
            if (_.isArray(queryResults) && _.isObject(queryResults[chartIndex]) && _.isArray(queryResults[chartIndex].data)) {
                // At this point we are pretty sure we have the new type of query results but we need to see if we have any data
                if (! _.isObject(queryResults[chartIndex].data[chartIndex])) {
                    throw sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.no.model.txt");
                }
                // Hope this is good enough
                return queryResults[chartIndex];
            }
            // Create the adapted object
            var returnObject = {};
            returnObject.data = [];
            returnObject.data[chartIndex] = {};
            var finalDataArray = [];
            returnObject.data[chartIndex].data = finalDataArray;
            var tuplesArray = queryResults.results.data[chartIndex].tuples;
            var seriesType = seriesTypes[0]; // Always only have one, I think
            _.each(chartTuples,function(columnTuple, index){
                var matchingTuple = _.find(columnTuple, function(dataItemObject){
                    return dataItemObject.type === seriesType;
                });

                var tupleObject = tuplesArray[index];

                var value = tupleObject.data[matchingTuple.id];
                if (value) {
                    finalDataArray.push(value);
                }
                else {
                    finalDataArray.push(null);
                }

            });
            return returnObject;
        },

        /*************************
         * END Private Query extract functions
         *************************/

        /**
         * For a request at the "toplevel" the top level class var is the only class var in the class vars list of the
         * queryData.
         * @private
         */
        _getTopLevelClassVarId: function() {
            var self = this;
            var topLevelClassVar = self.plotQueryData.classVariables[0];
            if (topLevelClassVar) {
                return topLevelClassVar.id;
            }
            throw "Could not find top level class var id";
        },

        /**
         * Build the JSON object representing the query options, suitable for use in the query
         * @param plotType
         * @returns {Array}
         * @private
         */
        _buildQueryOptionsJSON: function(plotType) {
            var self = this;
            var queryOptions = [];

            // Create an array of options that should be sent on query based on the type of plot
            switch(plotType) {
                case HprConstants.PlotType.Comparison:
                    queryOptions.push({"name": self.getOptionALPHA().name, "value": self.getOptionALPHA().value});
                    break;
                case HprConstants.PlotType.Distribution:
                    // 'Subportfolio by Distribution' plots
                    queryOptions.push({"name": self.getOptionALPHA().name, "value": self.getOptionALPHA().value});
                    break;
                case HprConstants.PlotType.Density:
                    queryOptions.push({"name": self.getOptionALPHA().name, "value": self.getOptionALPHA().value});
                    queryOptions.push({"name": self.getOptionNDENSITY().name, "value": self.getOptionNDENSITY().value});
                    queryOptions.push({"name": self.getOptionKERNEL().name, "value": self.getOptionKERNEL().value});
                    queryOptions.push({"name": self.getOptionPCTLDEF().name, "value": self.getOptionPCTLDEF().value});
                    break;
                default:
                    break;
            }

            return queryOptions;
        },

        //*****************************
        // Filters
        //*****************************

        setFilters: function(filters) {
            this.filters = filters;
        },

        updateFilterSections: function(indexedFilterSectionData) {
            var self = this;
            var filterSection;
            _.each(self.filters, function(filter) {
                filterSection = indexedFilterSectionData[filter.id];
                if (filterSection) {
                    filter.collapsed = filterSection.collapsed;
                }
            });
        },

        getFilters: function() {
            return $.extend(true, [], this.filters);
        },

        _toFiltersJSON: function() {
            return this.filters;
        },

        _fromFiltersJSON: function(filterJSON) {
            this.filters = filterJSON;
        },

        _buildQueryFilterJSON: function() {
            var self = this;
            return _.map(self.filters, function(aFilter){ // , index, list
                return {id: aFilter.id, type: aFilter.type, includes: aFilter.includes, values: aFilter.values};
            });
        },

        /**
         * A generic validator function to ensure the list of data items (usually dehydrated) in the
         * passed in list all exist in the given dataSource.  If either the dataSource or dataItemList is not
         * supplied or null return false.
         * @param dataSource - The DataSource to validate the dataItemList against
         * @param dataItemList
         * @returns {boolean}
         */
        _dataItemValidator: function(dataSource, dataItemList) {
            var areItemsValid = true;
            if (!dataSource || !dataItemList) {
                return false;
            }
            _.each(dataItemList, function(candidateDataItem){
                var dataItem = dataSource.getDataItemById(candidateDataItem.id);
                if (!dataItem) {
                    areItemsValid = false;
                }
            });
            return areItemsValid;
        },

        /**
         * A method which checks to see if the existing content object references any of the data source ids by looking
         * at the "dataSourceId" property.
         * @param { PlotContentData } currentContentData - The JSON object representing the current state of a panel
         * @param { [string] } dataSourceIds - A list of data source ids
         */
        areDataSourceIdsBeingReferenced: function(currentContentData, dataSourceIds) {
            return _.contains(dataSourceIds, currentContentData.dataSourceId);
        },

        /**
         * pseudo static method that takes a list of Data Source ids which are being removed from an Exploration and determines
         * if this model references one of them; If it does a Zero State object is returned
         * @param { PlotContentData } currentContentData - The JSON object representing the current state of a panel
         * @param { [string] } dataSourceIdsBeingRemoved - A list of data source ids that are being removed
         * @returns {*}
         */
        removeDataSource: function(currentContentData, dataSourceIdsBeingRemoved) {
            // Create the wrapping deferred
            var returnDeferred = $.Deferred();

            // Check to make sure the current data source id is contained in the list of data source ids being replaced,
            // otherwise there is nothing to do so return the provided currentContentData.
            var currentDataSourceId = currentContentData.dataSourceId;
            if (!_.contains(dataSourceIdsBeingRemoved, currentDataSourceId)) {
                returnDeferred.resolve(currentContentData);
                return returnDeferred;
            }

            var zeroStateObject = {
                name: currentContentData.name,
                dataSourceId: "",
                type: HprConstants.HPR_PLOT_PANEL,
                plotType: HprConstants.PlotType.ZeroState,
                tuples: [],
                requests: [{}],
                optionsPanelOptions: {
                    // So the name of the ZeroState Plot will be the same
                    name: currentContentData.optionsPanelOptions.name
                }
            };

            returnDeferred.resolve(zeroStateObject);
            return returnDeferred;
        },

        /**
         * pseudo static method that takes an existing content object and changes it in a way to represent a new data source.
         * This function should select an equivalent data source id from the list of newDataSourceIds for the data source
         * id the currentContentData is representing.
         *
         * This method must return a jQuery.Deferred object that behaves as follows:
         * - Returns the JSON to represent the panel's new state regardless of whether the panel should change because of the new data source
         * - ALWAYS resolve successfully.
         * - May make subsequent/wrapped ajax calls but when it returns all processing should be complete.
         * - May not change the state of the model instance or attempt to interact with the UI.
         *
         * @param { PlotContentData } currentContentData - The JSON object representing the current state of a panel
         * @param { [string] } dataSourceIdsBeingReplaced - A list of data source ids that are being replaced
         * @param { [string] } newDataSourceIds - A list of replacement data source ids
         * @returns {*}
         */
        replaceDataSource: function(currentContentData, dataSourceIdsBeingReplaced, newDataSourceIds) {
            // Create the wrapping deferred
            var returnDeferred = $.Deferred();
            var self = this;

            // Check to make sure the current data source id is contained in the list of data source ids being replaced,
            // otherwise there is nothing to do so return the provided currentContentData.
            var currentDataSourceId = currentContentData.dataSourceId;
            if (!_.contains(dataSourceIdsBeingReplaced, currentDataSourceId)) {
                returnDeferred.resolve(currentContentData);
                return returnDeferred;
            }

            var zeroStateObject = {
                name: currentContentData.name,
                dataSourceId: "",
                type: HprConstants.HPR_PLOT_PANEL,
                plotType: HprConstants.PlotType.ZeroState,
                tuples: [],
                requests: [{}],
                optionsPanelOptions: {
                    // So the name of the ZeroState Plot will be the same
                    name: currentContentData.optionsPanelOptions.name
                }
            };

            // Get a compatible data source id from the list of newDataSourceIds.  If one is not found then there is
            // nothing to do so return the provided currentContentData
            var newDataSourceId = HprUtil.getCompatibleDataSourceId(currentDataSourceId, newDataSourceIds);
            if (!newDataSourceId) {
                returnDeferred.resolve(zeroStateObject);
                return returnDeferred;
            }

            // Initial setup checks are done, create the new content data object
            var newContentData = {};

            // type stays the same
            newContentData.type = currentContentData.type;

            // plotType stays the same
            newContentData.plotType = currentContentData.plotType;

            // name stays the same
            newContentData.name = currentContentData.name;

            // Change to use new data source id
            newContentData.dataSourceId = newDataSourceId;

            // linked to crosstab stays
            newContentData.linkedToCrosstab = currentContentData.linkedToCrosstab;

            // Crosstab id stays
            newContentData.crosstabId = currentContentData.crosstabId;

            // Drill stays the same
            newContentData.drill = currentContentData.drill;

            // Sorts stay the same
            newContentData.sorts = currentContentData.sorts;

            // Options stay the same
            newContentData.options = currentContentData.options;

            // Set a temporary flag so when first crosstab rerender occurs after replace data source we ignore it.
            newContentData.replacingDataSource = true;

            // Grab the data source - should already be loaded
            var deferred = DataSourceCacheManager.getDataSource(newDataSourceId);
            deferred.done(function(targetDataSource) {

                // flag to capture the overall status of the validation
                var passedValidation = true;

                // variable to capture the first error message for why validation failed
                var firstErrorMessage = null;

                /**
                 * Internal helper function to collect the two steps of setting the error message and setting the overall validation
                 * status when an error condition is detected
                 * @param message
                 */
                var setErrorMessage = function(message) {
                    if (firstErrorMessage === null) {
                        firstErrorMessage = message;
                        passedValidation = false;
                    }
                };

                // validate requests - There could be multiples so we have to ensure all the requests are valid
                var validRequests = true;
                _.each(currentContentData.requests, function(request){
                    if (!self._dataItemValidator(targetDataSource, request.key)){
                        validRequests = false;
                    }
                });
//                if (validRequests) {
                    newContentData.requests = currentContentData.requests;
                // }
                // else {
                //     setErrorMessage("Could not find compatible DataItem in requests object in new data source");
                // }

                // validate stats
                if (self._dataItemValidator(targetDataSource, currentContentData.statistics)) {
                    newContentData.statistics = currentContentData.statistics;
                }
                else {
                    setErrorMessage("Could not find compatible statistic DataItems in new data source");
                }

                // validate horizons
                if (self._dataItemValidator(targetDataSource, currentContentData.horizons)) {
                    newContentData.horizons = currentContentData.horizons;
                }
                else {
                    setErrorMessage("Could not find compatible horizons in new data source");
                }

                // validate classvars
                if (self._dataItemValidator(targetDataSource, currentContentData.classVariables)) {
                    newContentData.classVariables = currentContentData.classVariables;
                }
                else {
                    setErrorMessage("Could not find compatible class variables in new data source");
                }


                // For version 1 we decided not to blow away the tuples if they cannot be validated as this allows the
                // user to switch back to a valid column without loosing their plot.  This could change in a future
                // version, so I left the checking code in, but commented out.
                // If this will be a CacheManager based plot, The CacheManager cannot deal with invalid tuples so we need
                // to check the tuples for validity in the new DataSource and place the chart into an error state.
                var tempTuples = HprChartUtils.fixTuplesForRiskCube(targetDataSource, currentContentData.tuples);
                // FixMe: Can't use self._isCacheManagerDependentPlot() as that requires a full instance of this class.
                if (HprChartUtils.isHorizonPlot(newContentData.plotType) || HprChartUtils.isDistributionPlot(newContentData.plotType) || HprChartUtils.isComparisonPlot(newContentData.plotType)) {
                    // Validate all the tuples array items
                    var tuplesValid = true;
                    _.each(tempTuples, function(tuplesArray){
                        if (! self._dataItemValidator(targetDataSource, tuplesArray)) {
                            tuplesValid = false;
                        }
                    });
                    if (!tuplesValid) {
                        setErrorMessage("Could not find compatible tuples in new data source");
                    }
                }
                newContentData.tuples = tempTuples;

                // copy over cashflow, options, and filters
                newContentData.cashFlow = currentContentData.cashFlow;
                newContentData.optionsPanelOptions = currentContentData.optionsPanelOptions;
                newContentData.sourceColumn = currentContentData.sourceColumn;
                if (!newContentData.optionsPanelOptions.usingCustomTitle) {

                    // Need to regenerate the title
                    // first validate the sourceColumn from the currentContentData
                    var sourceColumnValid = true;
                    if (newContentData.sourceColumn) {
                        if (! self._dataItemValidator(targetDataSource, newContentData.sourceColumn)) {
                            sourceColumnValid = false;
                        }
                    }
                    if (!sourceColumnValid) {
                        // if the source column is invalid this means the panel title is not generatable and more than likely
                        // the query will return no data so since at this point we know we are not using a custom title
                        // we need to clear out the source column and regenerate a title for the panel
                        newContentData.sourceColumn = null;
                    }
                    else {
                    }
                }

                newContentData.filters = currentContentData.filters;

                if (!passedValidation) {
                    console.error(firstErrorMessage);
                    newContentData.tuples = [];
                    newContentData.requests = [{}];
                }

                var currentContentDataKeys = _.keys(currentContentData);
                var newContentDataKeys = _.keys(newContentData);

                _.each(currentContentDataKeys, function(key){
                   if (_.indexOf(newContentDataKeys, key) === -1) {
                       console.log("New Content Data is missing: "+key);
                   }
                });
                // Always resolve the deferred whether it passed validation or not.
                returnDeferred.resolve(newContentData);
             });
            deferred.fail(function (jqxhr) { // , textStatus, error
                // deferred could fail w/o jqxhr being passed back
                if (jqxhr && jqxhr.responseText) {
                    // Failure case where we create the zero state json and resolve the deferred successfully
                    jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
                }
                returnDeferred.resolve(zeroStateObject);
            });
            return returnDeferred;
        },

        /**
         * Resets the error status on the model so it can requery successfully
         *
         */
        clearErrorState: function() {
            this.invalidReasonString = null;
        },

        /**
         * This method performs actions that make this chart model invalid.  Currently it sets the 'plotQueryData' to an empty object
         * and after that it assigns a ZeroState type to the 'plotQueryData.type' property.  The point of the method is to make the
         * chart model invalid so a ZeroState plot is displayed as the content.
         * @param message - An optional string explaining why the model was invalidated.
         * @private
         */
        _invalidateChartModel: function(message) {
            var self = this;

            // add the passed in message to the list of invalid strings
            if (message) {
                self.invalidReasonString = message;
            }
            else {
                self.invalidReasonString = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.generic.load.error.txt");
            }
            // empty out the 'plotQueryData' object to signify a bad chart model
//            this.plotQueryData = {};

            // 'type' property is needed so we can restore the Exploration when it is saved
            // with a ZeroState plot in it
            // this.plotQueryData.isZeroState = true;
            // this.plotQueryData.type = HprConstants.PlotType.ZeroState;
            //
            // // Do this so any queries for the plot type will be ok when in this state.
            // this.plotQueryData.plotType = {PlotType: HprConstants.PlotType.ZeroState};
        },

        /**
         *
         * Checks to see if this chart model deems itself as valid or not.  If the model is invalid, a String representing
         * the reason the model is invalid is returned, otherwise null is returned.
         * @returns {null|String}} Returns the reason the model is invalid or null if the model is valid
         */
        getModelInvalidReasonString: function() {
            var self = this;
            var reasonString = null;

            if (self.invalidReasonString !== null) {
                reasonString = self.invalidReasonString;
            }
            else if (!self.dataSourceId || _.isEmpty(self.plotQueryData.requests) || _.isEmpty(self.plotQueryData.tuples)) {
                // Currently, if there is no 'dataSourceId', plotQueryData.requests, or plotQueryDate.tuples in the chart model it is deemed invalid
                reasonString = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.no.queryData.txt");
            }
            return reasonString;
        },

        //*********************************
        // Options panel support methods
        //*********************************

        toOptionsJSON: function() {
            var copiedOptions = $.extend(true, {}, this.optionsPanelOptions);
            delete copiedOptions.plotType;
            return copiedOptions;
        },

        fromOptionsJSON: function(opts) {
            this.optionsPanelOptions = $.extend(true,{},opts);
        },

        getOptionName: function() {
            return this.optionsPanelOptions.name;
        },

        setDefaultTitle: function(title){
            this.optionsPanelOptions.name = title;
            this.optionsPanelOptions.usingCustomTitle = false;
        },

        setOptionName: function(newName) {
            this.optionsPanelOptions.name = newName;
            this.optionsPanelOptions.usingCustomTitle = true;
        },

        isUsingCustomTitle: function() {
            return this.optionsPanelOptions.usingCustomTitle;
        },

        /**
         * @description Returns the showLabels option
         * @return {NameValueObject}
         */
        getOptionShowLabels: function() {
            return this.optionsPanelOptions.showLabels;
        },

        /**
         * @description Should data items be shown as labels or names.
         * @return {boolean} true if the labels should be shown
         */
        showLabels: function() {
            return this.optionsPanelOptions.showLabels.value;
        },

        /**
         * @description Sets the showLabels option.  Note the type of the object that is expected
         * to be passed in
         * @param {NameValueObject} showLabels
         */
        setOptionShowLabels: function(showLabels) {
           this.optionsPanelOptions.showLabels = showLabels;
        },

        /**
         * @description gets the alpha option.
         * @return {NameValueObject}
         */
        getOptionALPHA: function() {
            return this.optionsPanelOptions.alpha;
        },

        /**
         * @description gets the pctl option.
         * @return {NameValueObject}
         */
        getOptionPCTLDEF: function() {
            return this.optionsPanelOptions.pctldef;
        },

        getOptionNDENSITY: function() {
            return this.optionsPanelOptions.ndensity;
        },

        getOptionKERNEL: function() {
            return this.optionsPanelOptions.kernel;
        },

        getOptionDisplayHorizons: function() {
            return this.optionsPanelOptions.displayHorizons;
        },

        setOptionGeneralOptionsSectionCollapsed: function(generalOptionsSectionCollapsed) {
            this.optionsPanelOptions.generalOptionsSectionCollapsed = generalOptionsSectionCollapsed;
        },

        setOptionGraphOptionsSectionCollapsed: function(graphOptionsSectionCollapsed) {
            this.optionsPanelOptions.graphOptionsSectionCollapsed = graphOptionsSectionCollapsed;
        },

        setOptionAdvancedQueryOptionsSectionCollapsed: function(advancedQueryOptionsSectionCollapsed) {
            this.optionsPanelOptions.advancedQueryOptionsSectionCollapsed = advancedQueryOptionsSectionCollapsed;
        },

        getOptions: function() {
            var self = this;
            var optionsObject = $.extend(true, {}, self.optionsPanelOptions);
            optionsObject.id = self.id;
            optionsObject.linkedToCrosstab = self.linkedToCrosstab;
            return optionsObject;
        },

        updateOptions: function(optionsData) {
            // Get the current state of certain options
            // Case where name has changed, commented out due to decision to not track absolute/relative switch on titles
            // at 3/29/2017 UI meeting
            // BEGIN: Absolute/Relative responding code
            // var originalDisplayHorizon = this.options.displayHorizons.value;
            // var originalName = this.options.name;
            // var originalUseDefaultTitleSetting = this.options.useDefaultTitle;
            // END: Absolute/Relative responding code

            // Always take the options sent in as the new options state
            var nameChanged = this.optionsPanelOptions.name !== optionsData.name;
            this.optionsPanelOptions = $.extend(true,{},optionsData);
            if (nameChanged) {
                this.optionsPanelOptions.usingCustomTitle = true;
            }

            // Here, we 'fix' the options based on the previous state of the options
            // Case where name has changed, commented out due to decision to not track absolute/relative switch on titles
            // at 3/29/2017 UI meeting
            // BEGIN: Absolute/Relative responding code
            // if (optionsData.name !== originalName) {
            //     this.options.useDefaultTitle = false;
            //     this.options.useDefaultTitle = this._isTitleDefault(optionsData.name);
            // }
            // else {
            //     // Case where they changed the horizon display type
            //     if (optionsData.displayHorizons.value !== originalDisplayHorizon && originalUseDefaultTitleSetting) {
            //         this.options.useDefaultTitle = true;
            //     }
            //     else {
            //         // Update the option properties with data passed in
            //         this.options.useDefaultTitle = this._isTitleDefault(optionsData.name);
            //     }
            // }
            // END: Absolute/Relative responding code
        },

        _buildDefaultOptionsForOptionsPanel: function(optionsData) {
            var self = this;
            var optionsPanelOptions = {};

            // Initialize these properties so the UI components are initialized correctly
            optionsPanelOptions.applyEnabled = false; // Flag for the Apply button on each Options pane; Disabled by default
            optionsPanelOptions.generalOptionsSectionCollapsed = false; // Open by default
            optionsPanelOptions.graphOptionsSectionCollapsed = true; // Closed by default
            optionsPanelOptions.advancedQueryOptionsSectionCollapsed = true; // Closed by default

            // The ALPHA and PCTLDEF values should come from Crosstab in the plotQueryData; if not, we use the default Options in cube model data
            // First seed the objects with the default Options in cube model data (this will give us constructor, methods, etc.)
            optionsPanelOptions.alpha = optionsData.alpha;
            optionsPanelOptions.pctldef = optionsData.pctldef;

            optionsPanelOptions.linkedToCrosstab = self.linkedToCrosstab;


            // Default option properties provided by default Options cube model data
            optionsPanelOptions.ndensity = optionsData.ndensity;
            optionsPanelOptions.kernel = optionsData.kernel;

            // Other properties needed for display in Options pane (others are initialized in constructor)
            //           options.useDefaultTitle = true;
            optionsPanelOptions.plotType = self.plotQueryData.plotType;

            // UI only options
            var displayHorizonsObject = {};
            displayHorizonsObject.name = HprConstants.HPR_OPTIONS_PANE_DISPLAY_HORIZONS_NAME;
            // TODO: I think this needs to move to the _overridePanelOptionsWithPlotQueryDataValues function
            // For the Chart's default options, the default Display Horizons value should come from the metadata passed in from the Crosstab the Plot was
            // generated from; Otherwise, if that metadata was not passed in, pull the value from the Application Settings
            displayHorizonsObject.value = self.plotQueryData.displayHorizons ? self.plotQueryData.displayHorizons : ApplicationSettings.getDisplayHorizons();
            optionsPanelOptions.displayHorizons = displayHorizonsObject;

            var showLabelsObject = {};
            showLabelsObject.name = HprConstants.HPR_OPTIONS_PANE_SHOW_LABELS_NAME;
            // For the Chart's Show Labels option, the default Show Labels value should come from the metadata passed in from the Crosstab the Plot was
            // generated from; Otherwise, if that metadata was not passed in, pull the value from the Application Settings
            showLabelsObject.value = ApplicationSettings.getShowLabels();
            optionsPanelOptions.showLabels = showLabelsObject; // Should we use the setter function?

            // Update the values if the plotQueryData options exists
            self._overridePanelOptionsWithPlotQueryDataValues(optionsPanelOptions, self.plotQueryData);

            // Delete the displayHorizons and showLabels properties from the plotQueryData/metadata since it is no longer needed
            // delete self.plotQueryData.displayHorizons;
            // delete self.plotQueryData.showLabels;

            return optionsPanelOptions;
        },

        /**
         * @description Internal method to isolate the logic of overriding the default options on the options panel with those in the
         * plotQueryData.  This is needed for the handleCrosstabUpdated case as well as the initial setting of the options
         * @param oPanelOptions
         * @param {PlotQueryData} plotQueryData
         * @private
         */
        _overridePanelOptionsWithPlotQueryDataValues: function(oPanelOptions, plotQueryData) {
            // Update the values if the plotQueryData options exists
            if (plotQueryData.options) {
                var alphaPlotQueryDataObject = _.findWhere(plotQueryData.options, {name: HprConstants.HPR_OPTIONS_PANE_ALPHA_NAME});
                if (alphaPlotQueryDataObject) {
                    oPanelOptions.alpha.value = alphaPlotQueryDataObject.value;
                }

                var pctldefPlotQueryDataObject = _.findWhere(plotQueryData.options, {name: HprConstants.HPR_OPTIONS_PANE_PCTLDEF_NAME});
                if (pctldefPlotQueryDataObject) {
                    oPanelOptions.pctldef.value = pctldefPlotQueryDataObject.value;
                }
            }

            if (_.has(plotQueryData,"displayHorizons")){
                oPanelOptions.displayHorizons.value = plotQueryData.displayHorizons;
            }
            // The showLabels attribute does not come through the options attribute of the plotQueryData when the handleCrosstabUpdate method is called
            // so we need to handle that differently
            if (_.has(plotQueryData,"showLabels")){
                oPanelOptions.showLabels.value = plotQueryData.showLabels;
            }
        },

        /*****************
         * Linked Plot methods
         *********************/

        /**
         * At various points in the lifecycle of a plot it's linked crosstab could go away.  This function consolidates
         * the update code in one place.  It should be called anytime you need to ensure you have a linked crosstab.
         * @private
         */
        _validateLinkedCrosstabState: function() {
            var self = this;

            // Safety check in case the crosstabId is null
            if (!self.crosstabId) {
                self.linkedToCrosstab = false;
                return;
            }

            // Check to make sure we can still find the crosstab
            var crosstab = sap.ui.getCore().byId(self.crosstabId);
            if (!crosstab) {
                self.crosstabId = null;
                self.linkedToCrosstab = false;
            }
        },

        /**
         * Sets the linked to crosstab flag
         * @param value true if it should be linked to it's source crosstab, false otherwise
         */
        setLinkedToCrosstab: function(value) {
            var self = this;
            self.linkedToCrosstab = value;
        },

        /**
         * Is this plot currently linked to a valid crosstab.
         * @returns {boolean} true if it is linked AND the crosstab exists, false otherwise
         */
        isLinkedToCrosstab: function() {
            var self = this;
            if (self.crosstabId) {
                // Check to make sure we can still find the crosstab
                var crosstab = sap.ui.getCore().byId(self.crosstabId);
                if (!crosstab) {
                    return false;
                }
                return self.linkedToCrosstab;
            }
            return false;
        },

        /**
         * Can the chart be relinked to it's originating crosstab.
         * @returns {boolean}
         */
        canLinkToCrosstab: function() {
            var self = this;
            self._validateLinkedCrosstabState();
            return !_.isUndefined(self.crosstabId) && !_.isNull(self.crosstabId);
        }
     });
})();