/*
 * '(c) 2017, SAS Institute Inc.'
 */

jQuery.sap.declare("sas.analytics.riskhp.common.riskcommons.plot.HprChartTableUtils");

jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.ltjs.gtl.models.Graph");
jQuery.sap.require("sas.ltjs.gtl.models.entry.EntryTitle");
jQuery.sap.require("sas.ltjs.gtl.models.entry.EntryTextItem");
jQuery.sap.require("sas.ltjs.gtl.models.core.HorizontalAlignment");
jQuery.sap.require("sas.ltjs.gtl.models.core.TextFitPolicy");
jQuery.sap.require("sas.ltjs.gtl.models.axis.AxisOptions");
jQuery.sap.require("sas.ltjs.gtl.models.axis.LinearAxisOptions");
jQuery.sap.require("sas.ltjs.gtl.models.axis.AxisLabelPosition");
jQuery.sap.require("sas.ltjs.gtl.models.axis.AxisTickValueFormatOptions");
jQuery.sap.require("sas.ltjs.commons.util.Dimension");
jQuery.sap.require("sas.ltjs.sasicu.util.CalendarUtil");
jQuery.sap.require("sas.ltjs.sasicu.util.SASFormatterUtil");
jQuery.sap.require("sas.ltjs.gtl.models.legend.AutomaticLegend");
jQuery.sap.require("sas.ltjs.gtl.models.legend.LayoutGlobalLegend");
jQuery.sap.require("sas.ltjs.gtl.models.legend.LegendTitlePosition");

/**
 * @namespace sas.analytics.riskhp.common.riskcommons
 * @namespace sas.analytics.riskhp.common.riskcommons.plot
 * @namespace sas.analytics.riskhp.models.dataitems
 */

/**
 * @class HprChartTableUtils
 * @name sas.analytics.riskhp.common.riskcommons.plot.HprChartTableUtils
 */
sas.analytics.riskhp.common.riskcommons.plot.HprChartTableUtils = {

    // TODO: This class calls directly to getLabel for various data items.  This needs to be reviewed for where it is needed
    // and where it needs to call HprChartUtils.getDisplayString(dataItem,chartModel);
    KEY_SORTING_SUFFIX: "__sorting__",
    HprChartUtils:  sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils,
    SASFormatterUtil: sas.ltjs.sasicu.util.SASFormatterUtil,
    Statistic: sas.analytics.riskhp.models.dataitems.Statistic,

    /**
     * Helper method to build a model key for the sort values in a table
     * @param {string} root
     * @returns {string}
     * @private
     * @memberOf HprChartTableUtils
     */
    _buildSortKey: function(root) {
        return root+this.KEY_SORTING_SUFFIX;
    },

    /**
     * Return the correct type of alignment for the passed in DataItem
     * @param {DataItem} dataItem
     * @returns {sap.ui.core.TextAlign|string}
     * @private
     * @memberOf HprChartTableUtils
     */
    _getAlignmentValue: function(dataItem) {
        if (this.HprChartUtils.isDataItemDateType(dataItem) || this.HprChartUtils.isDataItemNumberType(dataItem) || this.HprChartUtils.isDataItemHorizonType(dataItem)) {
            return sap.ui.core.TextAlign.Right;
        }
        return sap.ui.core.TextAlign.Begin;
    },

    /**
     * returns the correct object for the value to sort by.  A date type will return a number, a horizon type will
     * return a horizonDate
     * @param {number} value
     * @param {DataItem} dataItem
     * @returns {*}
     * @private
     * @memberOf HprChartTableUtils
     */
    _getSortValue: function(value, dataItem) {
        if (this.HprChartUtils.isDataItemDateType(dataItem)) {
            return Number(value);
        }
        else if (this.HprChartUtils.isDataItemHorizonType(dataItem)) { // What if the dataItem is only a formatter? In this case all the value objects will be converted to the same value
            return dataItem.horizonDate;
        }
        return value;
    },

    /**
     * Helper method to build a column for the table of a plot in a standard way
     * @param {string} bindingKey - The binding path to the data in the model the column should use
     * @param {Object} alignmentMap - The map of bindingKeys to alignment values
     * @param {string} [columnTitle] - The title text to be shown on the column, if not supplied the bindingKey will be used
     * @returns {sas.hc.ui.table.Column}
     * @private
     * @memberOf HprChartTableUtils
     */
    _createColumn: function(bindingKey, alignmentMap, columnTitle) {
        var textProperty ={
            path: bindingKey
        };
        var sortingKey = bindingKey + this.KEY_SORTING_SUFFIX;

        var title = bindingKey;
        if (columnTitle) {
            title = columnTitle;
        }
        var columnAlign = alignmentMap[bindingKey];
        // if (sortingKey) {
        //     internalSortingKey = sortingKey;
        // }
        // if (showAsDate) {
        //     textProperty = {
        //         path: bindingKey,
        //         formatter: sas.analytics.riskhp.utils.HprUtil.formatDateTime
        //     };
        // }
        // This is a bit flakey, the titleKey is normally not a localized string so we end up just returning the string passed in
        // I'm taking this out for now to see what it looks like
        // text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText(titleKey)
        return new sas.hc.ui.table.Column({
            label: new sas.hc.m.Label({
                textAlign: columnAlign,
                text: title
            }),
            template: new sas.hc.m.Text({
                textAlign: columnAlign,
                text: textProperty,
                wrapping: false,
                width: "100%"
            }),
//                width: "10rem",
            sortProperty: sortingKey,
            flexible: false
        });
    },

    /**
     * Builds the table data model and columns for comparison plots
     * @param chartModel
     * @returns {{values: Array, columns: Array}}
     */
    buildComparisonPlotTableData: function(chartModel) {
        var responseType,
            categoryRoleName,
            responseRoleName,
            values = [],
            responseValues,
            responseRoles,
            tableItem,
            seriesName,
            self = this,
            alignmentMap = {},
            columns = [];

        // TODO: Lots of opportunities to refactor this code when we get to 191
        _.each(chartModel.categoryRoles, function(categoryRole, chartIndex){
            categoryRole = chartModel.getCategoryRole(chartIndex);
            responseType = chartModel.getResponseType(chartIndex);
            categoryRoleName = self.HprChartUtils.getCategoryRoleName(categoryRole, chartModel);
            responseRoleName = responseType.getLabel();
            responseRoles = chartModel.getResponseRole(chartIndex);
            alignmentMap = {};

            // If the model represents a grouped horizons plot we will get three columns (i.e. values, group, horizon}
            if (self.HprChartUtils.isHorizonPlot(chartModel.getPlotTypeObject())) {
                if (self.HprChartUtils.isMultiSeriesHorizonPlot(chartModel.getPlotTypeObject())) {
                    // To get category role (which has to be a horizon) we have to go back to the source column
                    var horizonDataItemIdObject = _.find(chartModel.getSourceColumn(), function(columnDataItem){
                        return self.HprChartUtils.isHorizonType(columnDataItem.type);
                    });
                    if (horizonDataItemIdObject) {
                        categoryRole = horizonDataItemIdObject;
                        categoryRoleName = self.HprChartUtils.getCategoryRoleName(horizonDataItemIdObject, chartModel);
                    }
                }
                alignmentMap[categoryRoleName] = self._getAlignmentValue(categoryRole); // CategoryRole is a horizon

                var tableItemMap = {};
                var seriesColumnNames = {};
                _.each(chartModel.getSeriesIds(chartIndex), function (seriesId, index) {

                    var seriesObject = chartModel.getSeriesObject(seriesId, chartIndex);
                    // Get the list of values for the chartIndex AND series
                    responseValues = seriesObject.getValues();
//                    seriesDataItem = seriesObject.getDataItems(chartModel.dataSource);
                    seriesName = "Series"+index;
                    seriesColumnNames[seriesId] = seriesObject.getName(chartModel.dataSource, chartModel.showLabels());
                    alignmentMap[seriesId] = self._getAlignmentValue(responseType);

                    _.each(responseValues, function(responseValue) {
                        // Get the displayable string
                        var dateString = sas.analytics.riskhp.utils.HprUtil.formatISODateString(responseValue.horizonDate);
                        tableItem = tableItemMap[dateString];
                        if (! tableItem) {
                            tableItem = {};
                            tableItem[categoryRoleName] = dateString;
                            values.push(tableItem);
                            tableItemMap[dateString] = tableItem;
                            // Get actual date value here so we can sort it
                            var sasDate = self.HprChartUtils.convertToSASDate(responseValue.horizonDate);
                            tableItem[self._buildSortKey(categoryRoleName)] = self._getSortValue(sasDate,responseType);
                        }
                        tableItem[seriesId] = self.SASFormatterUtil.format(responseValue.value, responseType.getFormat());
                        tableItem[self._buildSortKey(seriesId)] = self._getSortValue(responseValue.value,responseType);
                    });
                });
                // Create the groupingRoleName column
                // What we should do is create a column for each horizon (which lives in response values
                // Is no column for category role
                columns.push(self._createColumn(categoryRoleName, alignmentMap, categoryRoleName)); // Horizons //resultSet
                // TODO: deal with a single series (responseRoleName) I think
                _.each(chartModel.getSeriesIds(chartIndex), function (seriesId) {
                    var columnName = seriesColumnNames[seriesId];
                    columns.push(self._createColumn(seriesId, alignmentMap, columnName)); // SeriesName //Value
                });
            }
            else if (self.HprChartUtils.isByTypeSubportfolio(chartModel.getPlotTypeObject())) {
                values = self._buildSubPortfolioPlotDataDefault(chartIndex, responseType, categoryRoleName, categoryRole, alignmentMap, chartModel);
                columns.push(self._createColumn(categoryRoleName,alignmentMap));
                columns.push(self._createColumn(responseRoleName,alignmentMap));
            }
            else {
                values = self._buildComparisonPlotTableDataDefault(chartIndex, responseType, categoryRoleName, categoryRole, alignmentMap, chartModel);
                columns.push(self._createColumn(categoryRoleName,alignmentMap));
                columns.push(self._createColumn(responseRoleName,alignmentMap));
            }
        });

        return {values: values, columns: columns};
    },

    /**
     * Builds the values table for a Distribution plot
     * @param chartModel
     * @returns {{values: Array, columns: Array}}
     */
    buildDistributionPlotTableData: function(chartModel) {
        var self = this,
            responseType,
            categoryRoleName,
            responseRoleName,
            responseRoles,
            responseRoleFormatter,
            // responseValues,
            // tableItem,
            values = [],
            alignmentMap = {},
            columns = [],

            // Accumulate the labels of each of the stats in this hash as the keys, will put a dummy value in for the value
            // as we will not use it.  I think there is an issue here should we have multiple charts.
            statNameSet = {};

        _.each(chartModel.categoryRoles, function(categoryRole, chartIndex) {
            responseType = chartModel.getResponseType(chartIndex);
            categoryRoleName = self.HprChartUtils.getCategoryRoleName(categoryRole, chartModel);
            responseRoleFormatter = chartModel.getResponseRoleFormatter(chartIndex);
            responseRoleName = responseType.getLabel();

            responseRoles = chartModel.getResponseRole(chartIndex);
            values = self._buildDistributionPlotDataDefault(chartIndex, responseRoles, categoryRoleName, responseRoleFormatter, alignmentMap, statNameSet, chartModel);

            // Add the category role name to the alignment map
            alignmentMap[categoryRoleName] = self._getAlignmentValue(responseRoleFormatter);
            columns.push(self._createColumn(categoryRoleName, alignmentMap));
            var statNameSetKeys = _.keys(statNameSet);
            _.each(statNameSetKeys, function(statNameKey){
                columns.push(self._createColumn(statNameKey, alignmentMap));
            });
        });

        return {values: values, columns: columns};
    },

    /**
     * @description Builds the data for comparison plots when the model indicates it is by sub portfolio (which means down a column)
     * @param {number} chartIndex
     * @param {DataItem} responseType
     * @param {string} categoryRoleName
     * @param {DataItem} categoryRole
     * @param {Object} alignmentMap
     * @param {HprChartModel} chartModel
     * @returns {Array}
     * @private
     * @memberOf HprChartTableUtils
     */
    _buildSubPortfolioPlotDataDefault: function(chartIndex, responseType, categoryRoleName, categoryRole, alignmentMap, chartModel) {
        var self = this,
            stat,
            value,
            seriesIds,
            seriesObject,
            tableItem,
            values = [];

        seriesIds = chartModel.getSeriesIds(chartIndex);
        _.each(seriesIds, function(seriesId) {
            seriesObject = chartModel.getSeriesObject(seriesId, chartIndex);
            if (seriesObject) {
                tableItem = {};
                tableItem[categoryRoleName] = seriesObject.getName(chartModel.dataSource); // If series is a horizon when this doesn't work
                var responseValues = seriesObject.getValues();
                tableItem[self._buildSortKey(categoryRoleName)] = seriesObject.getSortValue(chartModel.dataSource);
                stat = chartModel.dataSource.getDataItemById(responseType.getId());
                if (stat) { // Note: If we don't get a stat here we are HOSED!!!!
                    value = self.SASFormatterUtil.format(responseValues[0],stat.format);
                    tableItem[stat.getLabel()] = value;
                    tableItem[self._buildSortKey(stat.getLabel())] = responseValues[0];
                    alignmentMap[stat.getLabel()] = sap.ui.core.TextAlign.Right;
                }
                values.push(tableItem);
            }
        });
        return values;
    },

    /**
     * @description Builds the data for the distribution plot table for anything other than when the distribution is by risk cube
     * @param {number} chartIndex
     * @param {string[]} responseRoles
     * @param {string} categoryRoleName
     * @param responseRoleFormatter
     * @param alignmentMap
     * @param statNameSet
     * @param {HprChartModel} chartModel
     * @returns {Array}
     * @private
     * @memberOf HprChartTableUtils
     */
    _buildDistributionPlotDataDefault: function(chartIndex, responseRoles, categoryRoleName, responseRoleFormatter, alignmentMap, statNameSet, chartModel) {
        var self = this,
            stat,
            value,
            seriesIds,
            seriesObject,
            tableItem,
            values = [];

        seriesIds = chartModel.getSeriesIds(chartIndex);
        _.each(seriesIds, function(seriesId) {
            /* {SeriesObject} seriesObject */
            seriesObject = chartModel.getSeriesObject(seriesId, chartIndex);
            if (seriesObject) {
                tableItem = {};
                tableItem[categoryRoleName] = seriesObject.getName(chartModel.dataSource); // If series is a horizon when this doesn't work
                var responseValues = seriesObject.getValues();
                tableItem[self._buildSortKey(categoryRoleName)] = seriesObject.getSortValue(chartModel.dataSource);
                _.each(self.HprChartUtils.DISTRIBUTION_STATISTIC_IDS, function(statId, index){
                    stat = chartModel.dataSource.getDataItemById(statId);
                    if (stat) { // Note: If we don't get a stat here we are HOSED!!!!
                        value = self.SASFormatterUtil.format(responseValues[index],stat.format);
                        if (stat.getName() === self.Statistic.NUMPRICE) {
                            tableItem["N"] = value;
                            tableItem[self._buildSortKey("N")] = responseValues[index];
                            alignmentMap["N"] = sap.ui.core.TextAlign.Right;
                            statNameSet["N"] = "dummy";
                        }
                        else {
                            tableItem[stat.getLabel()] = value;
                            tableItem[self._buildSortKey(stat.getLabel())] = responseValues[index];
                            alignmentMap[stat.getLabel()] = sap.ui.core.TextAlign.Right;
                            statNameSet[stat.getLabel()] = "dummy";
                        }
                    }
                });
                values.push(tableItem);
            }
        });
        return values;
    },

    /**
     * This is the default implementation to build the data for a comparison plot.  It is assumed the data runs across
     * a row (if the date is down a column use _buildSubPortfolioPlotDataDefault)
     * @param {number} chartIndex
     * @param {DataItem} responseType
     * @param {string} categoryRoleName
     * @param {DataItem} categoryRole
     * @param {Object} alignmentMap
     * @param {HprChartModel} chartModel
     * @returns {Array}
     * @private
     * @memberOf HprChartTableUtils
     */
    _buildComparisonPlotTableDataDefault: function(chartIndex, responseType, categoryRoleName, categoryRole, alignmentMap, chartModel) {
        var self = this,
            seriesIds,
            seriesObject,
            tableItem,
            responseValues,
            formattedValue,
            tableItems = [],
            stat;

        alignmentMap[categoryRoleName] = self._getAlignmentValue(categoryRole);
        seriesIds = chartModel.getSeriesIds(chartIndex);
        _.each(seriesIds, function(seriesId) {
            seriesObject = chartModel.getSeriesObject(seriesId, chartIndex);
            if (seriesObject) {
                tableItem = {};
                tableItem[categoryRoleName] = seriesObject.getName(chartModel.dataSource, chartModel.showLabels());
                responseValues = seriesObject.getValues();
                tableItem[self._buildSortKey(categoryRoleName)] = seriesObject.getSortValue(chartModel.dataSource);
                stat = chartModel.dataSource.getDataItemById(responseType.getId());
                if (stat) { // Note: If we don't get a stat here we are HOSED!!!!
                    formattedValue = self.SASFormatterUtil.format(responseValues[0],stat.format);
                    tableItem[stat.getLabel()] = formattedValue;
                    tableItem[self._buildSortKey(stat.getLabel())] = responseValues[0];
                    alignmentMap[stat.getLabel()] = sap.ui.core.TextAlign.Right;
                }
                tableItems.push(tableItem);
            }
        });
        return tableItems;
    },

    buildCashFlowPlotTableData: function(chartModel) {
        var self = this,
            responseType,
            categoryRoleName,
            responseRoleName,
            responseValues,
            tableItem,
            values = [],
            alignmentMap = {},
            tableItemMap = {},
            seriesColumnNames = {},
            seriesName,
            columns = [];

        _.each(chartModel.categoryRoles, function(categoryRole, chartIndex){
            //categoryRole = chartModel.getCategoryRole(chartIndex);
            responseType = chartModel.getResponseType(chartIndex);
            categoryRoleName = self.HprChartUtils.getCategoryRoleName(categoryRole, chartModel);
            responseRoleName = responseType.getLabel();

            var horizonDataItemIdObject = _.find(chartModel.getSourceColumn(), function(columnDataItem){
                return self.HprChartUtils.isHorizonType(columnDataItem.type);
            });
            if (horizonDataItemIdObject) {
                categoryRole = horizonDataItemIdObject;
                categoryRoleName = self.HprChartUtils.getCategoryRoleName(horizonDataItemIdObject, chartModel);
            }
            alignmentMap[categoryRoleName] = self._getAlignmentValue(categoryRole); // CategoryRole is a horizon

            _.each(chartModel.getSeriesIds(chartIndex), function (seriesId, index) {

                var seriesObject = chartModel.getSeriesObject(seriesId, chartIndex);
                // Get the list of values for the chartIndex AND series
                responseValues = seriesObject.getValues();
                //seriesDataItem = seriesObject.getDataItems(chartModel.dataSource);
                seriesName = "Series"+index;
                seriesColumnNames[seriesId] = seriesObject.getName(chartModel.dataSource, chartModel.showLabels());
                alignmentMap[seriesId] = self._getAlignmentValue(responseType);

                _.each(responseValues, function(responseValue) {
                    // Get the displayable string
                    var dateString = sas.analytics.riskhp.utils.HprUtil.formatISODateString(responseValue.horizonDate);
                    tableItem = tableItemMap[dateString];
                    if (! tableItem) {
                        tableItem = {};
                        tableItem[categoryRoleName] = dateString;
                        values.push(tableItem);
                        tableItemMap[dateString] = tableItem;
                        // Get actual date value here so we can sort it
                        var sasDate = self.HprChartUtils.convertToSASDate(responseValue.horizonDate);
                        tableItem[self._buildSortKey(categoryRoleName)] = self._getSortValue(sasDate,responseType);
                    }
                    tableItem[seriesId] = self.SASFormatterUtil.format(responseValue.value, responseType.getFormat());

                });
            });
            // Create the groupingRoleName column
            // What we should do is create a column for each horizon (which lives in response values
            // Is no column for category role
            columns.push(self._createColumn(categoryRoleName, alignmentMap, categoryRoleName)); // Horizons //resultSet
            // TODO: deal with a single series (responseRoleName) I think
            _.each(chartModel.getSeriesIds(chartIndex), function (seriesId) {
                var columnName = seriesColumnNames[seriesId];
                columns.push(self._createColumn(seriesId, alignmentMap, columnName)); // SeriesName //Value
            });
        });
        return {values: values, columns: columns};
    },

    /**
     * Builds the table data for RFIM plots
     * @param {HprChartModel} chartModel
     * @returns {{values: Array, columns: Array}}
     */
    buildRFIMPlotTableData: function(chartModel) {
        var self = this,
            responseType,
            categoryRoleName,
            responseRoleName,
            tableItem,
            values = [],
            alignmentMap = {},
            seriesObject,
            seriesIds,
            formattedValue,
            columns = [];

         _.each(chartModel.categoryRoles, function(categoryRole, chartIndex) {
             responseType = chartModel.getResponseType(chartIndex);
             categoryRoleName = self.HprChartUtils.getCategoryRoleName(categoryRole, chartModel);
             responseRoleName = responseType; // We know this is RFIM
             alignmentMap = {};

             seriesIds = chartModel.getSeriesIds(chartIndex);
             _.each(seriesIds, function (seriesId) {
                 seriesObject = chartModel.getSeriesObject(seriesId, chartIndex);
                 if (seriesObject) {
                     tableItem = {};
                     tableItem[categoryRoleName] = seriesObject.getName(chartModel.dataSource);
                     var responseValues = seriesObject.getValues();
                     tableItem[self._buildSortKey(categoryRoleName)] = seriesObject.getSortValue(chartModel.dataSource);
                     formattedValue = self.SASFormatterUtil.format(responseValues[0], "BEST");
                     tableItem[responseRoleName] = formattedValue;
                     tableItem[self._buildSortKey(responseRoleName)] = responseValues[0];
                     alignmentMap[responseRoleName] = sap.ui.core.TextAlign.Right;
                     values.push(tableItem);
                 }
             });
             columns.push(self._createColumn(categoryRoleName,alignmentMap));
             columns.push(self._createColumn(responseRoleName,alignmentMap));
         });

        return {values: values, columns: columns};
    },

    /**
     * @description Creates the data for the table for Density plots
     * @param {HprChartModel} chartModel
     * @returns {{values: Array, columns: Array}}
     * @memberOf HprChartTableUtils
     */
    buildDensityPlotTableData: function(chartModel) {
        var self = this,
            responseValues,
            tableItem,
            alignmentMap = {},
            values = [],
            columns = [];

        var likelihoodName = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.DensityGraph.likelihood.txt");
        var outVarName = self.HprChartUtils.getXAxisLabel(chartModel);

        _.each(chartModel.categoryRoles, function(categoryRole, chartIndex){
            // Grab the values
            responseValues = chartModel.getResponseValues(chartIndex,null);

            // For now we only plot the x-Values and the Likelihood value
            var outVarValues = responseValues["X-VALUES"];
            var likelihoodValues = responseValues["LIKELIHOOD"];

            // Build the data values, matching each value to the correct index.
            // Note DensityPlots have their own data so the responseRoleFormatter doesn't apply here.  So we bypass
            // the regular role formatting (self.HprChartUtils.getFormattedValue(outVarValue, responseRoleFormatter))
            // and just format the values directly.  Current format used in "BEST"
            _.each(outVarValues, function(outVarValue, index){
                tableItem = {};
                tableItem[outVarName] = sas.ltjs.sasicu.util.SASFormatterUtil.format(Number(outVarValue), "BEST");
                tableItem[self._buildSortKey(outVarName)] = outVarValue;
                alignmentMap[outVarName] = sap.ui.core.TextAlign.Right;
                tableItem[likelihoodName] = sas.ltjs.sasicu.util.SASFormatterUtil.format(Number(likelihoodValues[index]), "BEST");
                tableItem[self._buildSortKey(likelihoodName)] = likelihoodValues[index];
                alignmentMap[likelihoodName] = sap.ui.core.TextAlign.Right;
                values.push(tableItem);
            });
        });
        columns.push(self._createColumn(likelihoodName,alignmentMap));
        columns.push(self._createColumn(outVarName,alignmentMap));

        return {values: values, columns: columns};
    }

};