jQuery.sap.declare("sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils");

jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.ltjs.gtl.models.Graph");
jQuery.sap.require("sas.ltjs.gtl.models.entry.EntryTitle");
jQuery.sap.require("sas.ltjs.gtl.models.entry.EntryTextItem");
jQuery.sap.require("sas.ltjs.gtl.models.core.HorizontalAlignment");
jQuery.sap.require("sas.ltjs.gtl.models.core.TextFitPolicy");
jQuery.sap.require("sas.ltjs.gtl.models.axis.AxisOptions");
jQuery.sap.require("sas.ltjs.gtl.models.axis.LinearAxisOptions");
jQuery.sap.require("sas.ltjs.gtl.models.axis.AxisLabelPosition");
jQuery.sap.require("sas.ltjs.gtl.models.axis.AxisTickValueFormatOptions");
jQuery.sap.require("sas.ltjs.commons.util.Dimension");
jQuery.sap.require("sas.ltjs.sasicu.util.CalendarUtil");
jQuery.sap.require("sas.ltjs.sasicu.util.SASFormatterUtil");
jQuery.sap.require("sas.ltjs.gtl.models.legend.AutomaticLegend");
jQuery.sap.require("sas.ltjs.gtl.models.legend.LayoutGlobalLegend");
jQuery.sap.require("sas.ltjs.gtl.models.legend.LegendTitlePosition");

// Make ESLint happy by declaring name spaces for external objects used here (Note: this is not being used correctly according to esling.org, but . . .you do what you can)
/** @name sas.ltjs.gtl.models.legend.LegendTitlePosition */
/** @name sas.ltjs.gtl.models.axis.AxisLabelPosition */
/** @name sas.ltjs.gtl.models.core.HorizontalAlignment */

/**
 * @name sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils
 */
sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils = {

    /** @name sas.ltjs.gtl.models.core.TextFitPolicy */
    TextFitPolicy:  sas.ltjs.gtl.models.core.TextFitPolicy,

    /**
     * @typedef {[string]} DISTRIBUTION_STATISTIC_IDS
     */
    // [ "VAR", "MIN", "MAX", "Q1", "Q3", "MEAN", "MEDIAN", "STD", "NUMPRICE"]
    DISTRIBUTION_STATISTIC_IDS: ["074ma9e", "0749oq6", "0749n5k", "07mcx", "07mcz", "07ucq5bi", "0716mwwec1ny", "074i6as", "071ogzbnltzc9r9"],

    _getNameAndTypes: function(dataItemsArray) {
        var returnArray = [];
        _.each(dataItemsArray, function(dataItem) {
            if (dataItem === null || dataItem === 'undefined'){
                // no-op
            }
            else {
                returnArray.push({name:dataItem.name, type: dataItem.type});
            }
        });
        return returnArray;
    },

    /**
     * @description Given a list of objects that conform to DataItemIdObject type returns a list of their ids
     * @param {[DataItemIdObject]|[DataItem]} items
     * @returns {[number]}
     */
    getIds: function(items) {
        return _.map(items, function(item){
            return item.id;
        });
    },

    /**
     * @description Takes a list of column tuples and expands them so the result is a set of columns that can retrieve all the stats
     * required for a distribution plot
     * @param {[[DataItem]]} tuples - the tuples (columns) to be expanded
     * @returns {[[DataItem]]} Right now this will be an array of tuples 9 times as large as the original array
     */
    getExpandedTuplesForDistributionPlot: function(tuples) {
        var self = this,
            newTuplesArray = [],
            statIndex = -1;
        _.map(tuples, function(tupleArray){
            statIndex = self.indexOf(tupleArray, sas.analytics.riskhp.models.dataitems.Statistic.STATISTIC);
            if (statIndex !== -1) {
                _.map(self.DISTRIBUTION_STATISTIC_IDS, function(distStatId){
                   var newTuple = sas.analytics.riskhp.utils.HprUtil.hprClone(tupleArray);
                   newTuple[statIndex].id = distStatId;
                   newTuplesArray.push(newTuple);
                });
            }
        });
        return newTuplesArray;
    },

    /**
     * @Description given a list of tuples (columns) determines if each of the tuples contains "sister" tuples that represent
     * all the stats needed to generate a Distrobution plot
     * @param {DataItem[][]|DataItemIdObject[][]}tuples
     * @returns {boolean} true if the tuples are found lacking and should be run through the getExpandedTuplesForDistributionPlot method
     */
    shouldExpandTuplesForDistributionPlot: function(tuples) {
        var self = this;

        // Inner funtion to generate a simple string representation of a tuple
        var generateTupleIdString = function(tuple, statReplaceId) {
            var tupleIdString = "";
            _.each(tuple, function(item){
                if (statReplaceId && item.type === sas.analytics.riskhp.models.dataitems.Statistic.STATISTIC) {
                    tupleIdString = tupleIdString + statReplaceId + "|";
                }
                else {
                    tupleIdString = tupleIdString + item.id + "|";
                }
            });
            return tupleIdString;

        };

        // Convert the tuples to a list of our simple tuple ids
        var tupleIdAggregationList = _.map(tuples, function(tuple){
            return generateTupleIdString(tuple);
        });

        // For each tuple, walk the list (old school style so we can fail quickly) to ensure there is a matching tuple
        // with all the stats required for a distribution plot Note: this is an O(N^2) algorithm (so be CAREFUL!!)
        for (var index = 0; index < tuples.length; index++) {
            var tupleToCheckTemplate = tuples[index];
            for (var ii=0; ii< self.DISTRIBUTION_STATISTIC_IDS.length; ii++) {
                var tupleIdToCheckFor = generateTupleIdString(tupleToCheckTemplate, self.DISTRIBUTION_STATISTIC_IDS[ii]);
                if (_.indexOf(tupleIdAggregationList, tupleIdToCheckFor) === -1) {
                    return true;
                }
            }
        }
        return false;
    },

    /**
     * @description Will the model (if rendered now) result in a new plot or is the model representing a plot that has been stored
     * before.
     * @param {HprChartModel} model - The model to check
     * @returns {boolean|*}
     */
    willCreateNewPlot: function(model) {
        return !_.isUndefined(model.originalData) && _.isUndefined(model.originalData.name);
    },

    /**
     * @description Given a {DataItem} and a {HprChartModel} uses the state of the chart model to return the string to display
     * for the given data item.
     * @param {*|DataItem} item
     * @param {HprChartModel} chartModel
     * @returns {string} The displayable name for the item, if it is not a DataItem then the item is returned
     */
    getDisplayString: function(item, chartModel) {
        var self = this;
        if (item && (self.isDataItem(item))) {
            return item.getDisplayLabel(chartModel.showLabels());
        }
        return item;
    },

    /**
     * @description Is the passed in string that represents a dataItem's type value considered a Horizon of any type
     * @param {string} type - This is null safe
     * @returns {boolean}
     * @memberOf HprChartUtils
     */
    isHorizonType: function(type) {
        return (type && (type === sas.analytics.riskhp.models.dataitems.Horizon.HORIZON ||
        type === sas.analytics.riskhp.models.dataitems.AbsoluteHorizon.ABSOLUTEHORIZON ||
        type === sas.analytics.riskhp.models.dataitems.RelativeHorizon.RELATIVEHORIZON));
    },

    /**
     * @description Is the passed in DataItem any type of horizon object
     * @param {(DataItem|DataItemIdObject} item
     * @returns {boolean}
     */
    isAnyTypeOfHorizon: function(item) {
        var self = this,
            type;
        if (item && (self.isDataItem(item) || _.has(item,"type"))) {
            type = item.type;
        }
        return self.isHorizonType(type);
    },

    /**
     * @description Does the list of passed in dataItems include at least one horizon that is a basecase horizon
     * @param {DataItem[]} dataItems
     * @returns {boolean}
     */
    hasBasecaseHorizon: function(dataItems) {
        var self = this;
        var foundBasecaseHorizon = _.find(dataItems, function(dataItem){
            return self.isAnyTypeOfHorizon(dataItem);
        });
        if (foundBasecaseHorizon) {
            return foundBasecaseHorizon.isBasecase();
        }
        return false;
    },

    /**
     * Does the list of data items have a Computed Variable in it
     * @param {[DataItem]} dataItems
     */
    hasComputedVariable: function(dataItems) {
        var self = this;
        var tempObject = self.getMatch(dataItems, sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR);
        return (!_.isUndefined(tempObject) && !_.isNull(tempObject));
    },

    /**
     * Does the list of data items have a Benchmark Variable in it
     * @param {[DataItem]} dataItems
     */
    hasBenchmarkVariable: function(dataItems) {
        var self = this;
        var tempObject = self.getMatch(dataItems, sas.analytics.riskhp.models.dataitems.Benchvar.BENCHVAR);
        return (!_.isUndefined(tempObject) && !_.isNull(tempObject));
    },

    /**
     * @description Takes an object that might be a {DataItem} and if so converts it into a {DataItemIdObject} representation of it,
     * otherwise returns the object
     * @param {(DataItem|Object)} candidateDataItem
     * @returns {(DataItemIdObject|Object)}
     */
    dehydrateDataItem: function(candidateDataItem) {
        if(candidateDataItem && typeof candidateDataItem.getId === 'function' && typeof candidateDataItem.getType === 'function') {
            return {id: candidateDataItem.getId(), type: candidateDataItem.getType()};
        }
        return candidateDataItem;
    },

    /**
     * @description Takes an array of DataItems and converts the entire list to an array of DataItemIdObjects
     * @param {DataItem[]} dataItems
     * @returns {DataItemIdObject[]}
     */
    dehydrateDataItems: function(dataItems) {
        var self = this;
        var dehydratedItems = [];
        _.each(dataItems, function(item){
            dehydratedItems.push(self.dehydrateDataItem(item));
        });
        return dehydratedItems;
    },

    /**
     * @description Takes an array of dataItem ids and attempts to convert them to DataItemId objects.  If an id cannot
     * be found it will not be in the returned list
     * @param {string[]} dataItemIds
     * @param {DataSource} dataSource
     * @returns {DataItemIdObject[]}
     */
    createDataItemIdObjects: function(dataItemIds, dataSource) {
        var self = this,
            returnList = [];
        if (_.isEmpty(dataSource)) {
            return returnList;
        }
        _.each(dataItemIds, function(dataItemId){
            var tempDataItem = self.dehydrateDataItem(dataSource.getDataItemById(dataItemId));
            if (tempDataItem) {
                returnList.push(tempDataItem);
            }
        });
        return returnList;
    },

    /**
     * @description Takes a {DataItemIdObject} and attempts to rehydrate it
     * @param {DataItemIdObject} dehydratedItem
     * @param {DataSource} dataSource
     * @returns {(DataItem|null)}
     */
    hydrateDataItem: function(dehydratedItem, dataSource) {
        if (!_.isEmpty(dataSource) && _.has(dehydratedItem, "id")) {
            return dataSource.getDataItemById(dehydratedItem.id);
        }
        return null;
    },

    /**
     * @description Hydrates a list of dehydrated dataitems using the given cube.  If the item doesn't have an id attribute or the
     * dataitem cannot be found the item in the original list is pushed into the resultant list unchanged.
     * @param {DataItemIdObject[]} dehydratedItems
     * @param {DataSource} dataSource
     * @returns {DataItem[]}
     */
    hydrateDataItems: function(dehydratedItems, dataSource) {
        var self = this,
            dataItems = [];

        _.each(dehydratedItems, function(item){
            var dataItem = self.hydrateDataItem(item, dataSource);
            if (dataItem !== null) {
                dataItems.push(dataItem);
            }
            else {
                dataItems.push(item);
            }
        });
        return dataItems;
    },

    /**
     * Generic method to generate the "Path: XXX" title for a plot.  If ALL is the only part of the path that will be
     * generated then will return null.
     * @param chartModel
     * @private
     */
    _generateDefaultPathTitle: function(chartModel){
        var self = this,
            pathString = "",
            classVariable = null,
            plotQueryData = chartModel.plotQueryData,
            keys = plotQueryData.requests[0].key,
            HPR_UTIL = sas.analytics.riskhp.utils.HprUtil;

        // During initialization of a saved exploration there is a point where the dataSource hasn't yet been loaded!!
        // So added this safety check
        if (!chartModel || !chartModel.dataSource || _.keys(chartModel.dataSource).length === 0) {
            return "";
        }
        if (keys.length === 0) {
            classVariable = chartModel.dataSource.getDataItemById(plotQueryData.classVariables[0].id);
            pathString = HPR_UTIL.getBundle().getText("HPRisk.plot.path.allonly.fmt", [self.getDisplayString(classVariable, chartModel)]);
//            pathString = HPR_UTIL.getBundle().getText("HPRisk.plot.path.fmt", [self.getDisplayString(classVariable, chartModel),sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.path.value.all.txt")]);
        }
        else {
            var currentIndex = 0;
            var numKeys = keys.length;
            // Flag to capture when the path has a value added to it that is not the "ALL" case
            var addedMoreThanAll = false;
            _.each(keys, function(keyTuple){
                var classVarName = keyTuple.name; // You have an id here so could get the real data item
                var value = keyTuple.value;
                var hydratedClassVariables = self.hydrateDataItems(plotQueryData.classVariables, chartModel.dataSource);
                classVariable = _.find(hydratedClassVariables, function(classVar){
                   return classVar.name === classVarName;
                });
                if (classVariable) {
                    classVarName = self.getDisplayString(classVariable, chartModel);
                    // If the class variable is not a string we need to run it through the formatter, otherwise
                    // the value is the value we will use.
                    if (classVariable.getDataType() !== HprConstants.HPR_DATA_TYPE_STRING ) {
                        value = sas.ltjs.sasicu.util.SASFormatterUtil.format(Number(value), classVariable.getFormat());
                    }
               }
                // If we are on the last key in the tuple, and this is not one of the "special" charts the value at the
                // currentIndex will be the label for the x-axis so there is no need to add it to the path string.
                if (currentIndex >= (numKeys - 1) && ! chartModel.isDensityPlot() && ! chartModel.isCashFlowPlot() && !chartModel.isByTypeRiskCube() && !self.isHorizonPlot(chartModel.getPlotTypeObject())) {
                    // no-op - Never add all per Katie
                    //pathString = pathString + " " + sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.path.fmt", [classVarName, sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.path.value.all.txt")]) + " >";
                }
                else {
                    addedMoreThanAll = true;
                    pathString = pathString + " " + sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.path.fmt", [classVarName, value]) + " >";
                }
                currentIndex += 1;
            });
            if (pathString.length > 2) {
                pathString = pathString.substring(0, pathString.length - 2);
            }

            // If only the "All" value was added then we don't want to return a title. - per Katie
            if (! addedMoreThanAll) {
                return null;
            }
        }
        return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.panel.genericPath.fmt', [pathString]);
    },

    /**
     * @description Does the plotType represent a chart that will have multiple series across horizons
     * @param {PlotTypeObject} plotType
     * @returns {boolean}
     */
    isMultiSeriesHorizonPlot: function(plotType) {
        var byType = plotType.ByType;
        return _.isArray(byType) && byType.length > 1 && byType[byType.length-1] === sas.analytics.riskhp.models.dataitems.Horizon.HORIZON;
    },

    /**
     * @description Is this plot, plotting horizons across the x-axis
     * @param {PlotTypeObject} plotType
     * @returns {boolean}
     */
    isHorizonPlot: function(plotType) {
        var self = this,
            byType = plotType.ByType;

        if (self.isMultiSeriesHorizonPlot(plotType)) {
            return true;
        }
        else if (_.isObject(byType)) {
            return self.isAnyTypeOfHorizon(byType);
        }
        return false;

    },
    /**
     * Construct the title text for a plot/chart based on the type.  This will construct the title used on the plot/chart
     * itself.
     * @param chartModel The model for the chart
     * @returns {*}
     */
    constructChartTitle: function(chartModel) {
        var plotQueryData = chartModel.plotQueryData;
        if (!plotQueryData) {
            return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.invalid.chart.title.txt');
        }
        var plotType = chartModel.getPlotType();
        switch (plotType) {
            case HprConstants.PlotType.Comparison:
                return this._generateDefaultPathTitle(chartModel);
            case HprConstants.PlotType.Density:
                // Want to do something different as density is cell based??!!
                return this._generateDefaultPathTitle(chartModel);
            case HprConstants.PlotType.Distribution:
                return this._generateDefaultPathTitle(chartModel);
            case HprConstants.PlotType.CashFlow:
                return this._generateDefaultPathTitle(chartModel);
            case HprConstants.PlotType.RiskFactorInformationMeasures:
                return null; // No title on RFIM
            default:
                return plotQueryData.type;
        }
    },

    /**
     * Use the the chartModel to construct an appropriate title for the plot/chart.  This constructs the title on the plot/chart
     * itself, not the title for the DroppableContainer and Window Manager.  That title is covered by the
     * 'generateDefaultPanelTitle' method.
     * @param chartModel The model for the chart
     * @returns {*} - May return null if no title should be shown
     */
    getChartTitle: function(chartModel) {
        var textItem = new sas.ltjs.gtl.models.entry.EntryTextItem();
//        var shortTextItem = new sas.ltjs.gtl.models.entry.EntryTextItem();
        // Here is how you set the attributes
//        textItem.getTextAttributes().setFontSize(new sas.ltjs.commons.util.Dimension("12px"));
        var chartTitleText = this.constructChartTitle(chartModel);
        if (_.isNull(chartTitleText)) {
            return null;
        }
        textItem.setText(chartTitleText);
//        textItem.setHAlign(sas.ltjs.gtl.models.core.HorizontalAlignment.LEFT);
//        shortTextItem.setText("Some short text");
//        shortTextItem.setHAlign(sas.ltjs.gtl.models.core.HorizontalAlignment.LEFT);
        var entryTitle = new sas.ltjs.gtl.models.entry.EntryTitle(); // Will appear above graph
        entryTitle.setTextItems([textItem]);
        entryTitle.getTextAttributes().setFontSize(new sas.ltjs.commons.util.Dimension("16px"));
        //entryTitle.setShortText("Some short text");
        entryTitle.setHAlign(sas.ltjs.gtl.models.core.HorizontalAlignment.LEFT);
        entryTitle.setTextFitPolicy(sas.ltjs.gtl.models.core.TextFitPolicy.TRUNCATE);
        return entryTitle;
    },

    /**
     * Returns a sas.ltjs.gtl.models.axis.AxisOptions configured for the y-axis, this includes left justifying
     * the label and turning the extract scale value on.
     * @returns {*}
     */
    getDefaultYAxisOptions: function() {
        var tickValueFormatOptions = new sas.ltjs.gtl.models.axis.AxisTickValueFormatOptions();
        tickValueFormatOptions.setExtractScale(true); // Turn on scale extraction

        var axisOptions = new sas.ltjs.gtl.models.axis.AxisOptions();
        var linearAxisOptions = new sas.ltjs.gtl.models.axis.LinearAxisOptions();
        linearAxisOptions.setTickValueFormat(tickValueFormatOptions);

        axisOptions.setLinearOptions(linearAxisOptions);
        axisOptions.setLabelPosition(sas.ltjs.gtl.models.axis.AxisLabelPosition.LEFT);
        return axisOptions;
    },

    /**
     * Returns a sas.ltjs.gtl.models.axis.AxisOptions configured for the x-axis, this includes turning the extract scale value on.
     * @returns {*}
     */
    getDefaultXAxisOptions: function() {
        var tickValueFormatOptions = new sas.ltjs.gtl.models.axis.AxisTickValueFormatOptions();
        tickValueFormatOptions.setExtractScale(true); // Turn on scale extraction

        var axisOptions = new sas.ltjs.gtl.models.axis.AxisOptions();
        var linearAxisOptions = new sas.ltjs.gtl.models.axis.LinearAxisOptions();
        linearAxisOptions.setTickValueFormat(tickValueFormatOptions);

        axisOptions.setLinearOptions(linearAxisOptions);
        return axisOptions;
    },

    /**
     * Is the type of plot specified by the plotType (one of the PlotType "enums") parameter a plot of a column from the crosstab.
     * Callers should use this to determine if the SAS_ALL attribute should be appended to the end of the query key
     * @param plotTypeObject
     * @returns {boolean}
     */
    isColumnarPlotType: function(plotTypeObject) {
        switch(plotTypeObject.PlotType) {
            case HprConstants.PlotType.Comparison:
                if (plotTypeObject.WhatType === HprConstants.WhatType.Statistic || plotTypeObject.WhatType === HprConstants.WhatType.ComputedVariable || plotTypeObject.WhatType === HprConstants.WhatType.BenchmarkVariable) {
                    return plotTypeObject.ByType === HprConstants.ByType.Subportfolio;
                }
                if (plotTypeObject.WhatType === HprConstants.WhatType.Distribution) {
                    return plotTypeObject.ByType === HprConstants.ByType.Subportfolio;
                }
                break;
            case sas.analytics.riskhp.utils.HprConstants.PlotType.CashFlow:
                return false;
            default:
                return false;
        }
    },

    /**
     * Returns the label for the x axis given the chart's model
     * @param chartModel
     * @returns {*}
     */
    getXAxisLabel: function(chartModel) {
        var self = this,
            labelName = sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.unknown.chart.title.txt'),
            plotQueryData = chartModel.plotQueryData,
            dataSource = chartModel.dataSource;
        if (!plotQueryData) {
            return labelName;
        }
         switch (chartModel.getPlotType()) {
            case HprConstants.PlotType.Density:
                // Density graphs have a single tuple in the tuples array by default.  The outvar is your label name
                var firstTuple = plotQueryData.tuples[0]; // don't forget the tuples are dehydrated
                var outvar = self.getMatch(firstTuple, sas.analytics.riskhp.models.dataitems.Outvar.OUTVAR);
                if (outvar !== null) {
                    var dataItem = dataSource.getDataItemById(outvar.id);
                    if (dataItem !== null) {
                        return self.getDisplayString(dataItem,chartModel);
                    }
                }
                break;
             case HprConstants.PlotType.Comparison:
                 return this._generateDefaultPathTitle(plotQueryData);
            default:
                return plotQueryData.type;
        }
    },

    DEFAULT_DATE_FORMAT: "NLDATE10",

    /**
     * Takes an ISO date and converts it to a SAS Date
     * @param {string} dateString - Must be ISO formatted (i.e. 2017-10-24)
     * @returns {*|number}
     */
    convertToSASDate: function(dateString) {
        // dateStrings come in as an ISO formatted date string 2017-10-24
        // so we can convert them to a localized JavaScript date and then convert them to sas dates from there
        var date = sas.analytics.riskhp.utils.HprUtil.convertISODateToLocalJSDate(dateString);
        return sas.ltjs.sasicu.util.CalendarUtil.jsDateToSASDate(date);
    },

    /**
     * Takes ISO date strings and converts them to sas date numbers
     * @param dateStrings - Must be UTC date strings
     * @returns {Array} - Will be an array of numbers.  If you get all zeros in this array then you didn't pass in UTC date strings
     */
    convertToSASDates: function(dateStrings) {
        var sasDates = [];
        _.each(dateStrings, function(dateString){
            // dateStrings come in as an ISO formatted date string 2017-10-24
            // so we can convert them to a localized JavaScript date and then convert them to sas dates from there
            var date = sas.analytics.riskhp.utils.HprUtil.convertISODateToLocalJSDate(dateString);
            sasDates.push(sas.ltjs.sasicu.util.CalendarUtil.jsDateToSASDate(date));
        });
        return sasDates;
    },

    /**
     * @description Returns "true" if the query for the supplied model represents a top level query
     * @param {HprChartModel} chartModel
     * @return {boolean}
     */
    isTopLevelQuery: function(chartModel) {
        var requests = chartModel.plotQueryData.requests;

        // Safety check to make sure we have a requests object
        if (requests.length === 0) {
            return false;
        }

        // requests is an array so we filter out only the query's that are top level - Right now if the requests array
        // has more than one we assume that if any are toplvel then all are. - MCA
        var topLevelRequests = _.filter(requests, function(request) {
            return (!_.isUndefined(request.key) && request.key.length === 0);
//                return request.name === 'query>toplevel';
        });

        return (topLevelRequests !== null && topLevelRequests !== undefined && topLevelRequests.length > 0);
    },

    /**
     * @description Constructs the title for the DroppableContainer and Window Manager "tile" which holds the plot/chart.  It also constructs
     * the default options 'name' value used in the Plot Options pane, since they should be the same and can be updated via the Plot
     * Options pane.  The title will have the form {General plot name} | tuple 1| tuple 2 | ...
     * Note: What is being plotted will be removed from the list.
     * @param {DataSource} dataSource
     * @param {HprChartModel} chartModel
     * @returns {string}
     */
    generateDefaultPanelTitle: function(dataSource, chartModel) {
        var self = this;

        var usingAbsoluteHorizons = chartModel.areHorizonsAbsolute();
        var whatType;

        var value = "";
        var sourceColumn = chartModel.getSourceColumn();
        if (sourceColumn && sourceColumn.length > 0 && chartModel.getModelInvalidReasonString() === null) {
            value = "";
            if (chartModel.isCashFlowPlot()) {
                // Note: The dataSource is not set on the chartModel here!!
                var cashFlowObject = dataSource.getDataItemById(chartModel.plotQueryData.cashFlow.id);
                if (cashFlowObject){
                    value = self.getDisplayString(cashFlowObject, chartModel);
                }
            }

            // Loop over the items in the source column to build up the path string
            // Note: this assumes a RTL language - MCA
            for (var i=0; i< sourceColumn.length; i++) {
                var item = sourceColumn[i];

                // Per defect S1420105 and a discussion with Katie we remove what we are plotting from the panel's title
                whatType = chartModel.getWhatType();
                if (item.type === whatType) {
                    continue;
                }
                // Items are dehydrated so we need to get the real one as we are going to call the .getLabel() function on it
                var itemToUse = dataSource.getDataItemById(item.id);
                // We only want to use hydrated data items because we are going to call .getLabel() so check to be sure
                if (self.isDataItem(itemToUse)) {

                    // If the item is a Horizon, and we are not using absolute horizons then we need to get the
                    // correct relative horizon so we can get the correct label
                    if (itemToUse.type === sas.analytics.riskhp.models.dataitems.Horizon.HORIZON && !usingAbsoluteHorizons){
                        itemToUse = dataSource.getRelatedRelativeHorizon(itemToUse.getId());
                    }
                    else if (itemToUse.type === sas.analytics.riskhp.models.dataitems.AbsoluteHorizon.ABSOLUTEHORIZON && !usingAbsoluteHorizons){
                        itemToUse = dataSource.getRelatedRelativeHorizon(itemToUse.getId());
                    }
                    else if (itemToUse.type === sas.analytics.riskhp.models.dataitems.RelativeHorizon.RELATIVEHORIZON && usingAbsoluteHorizons) {
                        itemToUse = dataSource.getDataItemById(itemToUse.sourceHorizonsIds[0]);
                    }

                    // Handle special cases
                    if (self.isDistributionPlot(chartModel.getPlotTypeObject())) {
                        itemToUse = (itemToUse.type === sas.analytics.riskhp.models.dataitems.Statistic.STATISTIC || itemToUse.type === sas.analytics.riskhp.models.dataitems.Outvar.OUTVAR) ? null : itemToUse;
                    }

                    // If we are plotting a comparison by RiskCube and the item we are on is a RiskCube then don't add it
                    if (itemToUse && itemToUse.type === sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE &&
                        self.isComparisonPlot(chartModel.getPlotTypeObject()) && chartModel.isByTypeRiskCube()) {
                        itemToUse = null;
                    }

                    // If we are plotting any type of RiskCube plot and we are in absolute horizon mode, don't show the
                    // horizon
                    // if (chartModel.isByTypeRiskCube() && usingAbsoluteHorizons && itemToUse && self.isAnyTypeOfHorizon(itemToUse)) {
                    //     itemToUse = null;
                    // }

                    // Final check to make sure we have a valid DataItem and then append it to the path string
                    if (itemToUse) {
                        value = value + " | " + self.getDisplayString(itemToUse, chartModel);
                    }
                }
            }
        }
        return self.formatDefaultPanelTitle(value,chartModel);

    },

    formatDefaultPanelTitle: function(columnValue, chartModel) {
        // Tack on the front (non-changing part) of the title
        var self = this;
        var byType = chartModel.getByType();
        var whatType = chartModel.getWhatType();
        var plotType = chartModel.getPlotType();
        var value = "";
        if (columnValue) {
            value = columnValue;
        }

        // switch on the type of plot we are building
        switch (plotType) {
            case HprConstants.PlotType.Comparison:

                // If this is a comparison chart and we are comparing horizons, we need to use the correct prefix depending
                // on whether we are comparing by a stat or by distribution. (this is across a row)
                if (chartModel.isByTypeHorizon()) {

                    // Comparing by statistic
                    if (whatType === HprConstants.WhatType.Statistic) {
                        return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.horizons.CompareByStatistic.defaultName.fmt', [value]);
                    }

                    // Comparing by distribution
                    else if (whatType === HprConstants.WhatType.Distribution) {
                        return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.horizons.CompareByDistribution.defaultName.fmt', [value]);
                    }
                }

                if (chartModel.isByTypeRiskCube()) {
                    // Comparing by statistic
                    if (whatType === HprConstants.WhatType.Statistic) {
                        return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.riskcubes.CompareByStatistic.defaultName.fmt', [value]);
                    }

                    // Comparing by distribution
                    else if (whatType === HprConstants.WhatType.Distribution) {
                        return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.riskcubes.CompareByHorizon.defaultName.fmt', [value]);
                    }
                }

                if (chartModel.isByTypeResultSet()) {
                    // Comparing by statistic
                    if (whatType === HprConstants.WhatType.Statistic) {
                        return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.resultSets.CompareByStatistic.defaultName.fmt', [value]);
                    }

                    // Comparing by distribution
                    else if (whatType === HprConstants.WhatType.Distribution) {
                        return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.resultSets.CompareByHorizon.defaultName.fmt', [value]);
                    }
                }

                if (chartModel.isByTypeScenario()) {
                    // Comparing by statistic
                    if (whatType === HprConstants.WhatType.Statistic) {
                        return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.scenarios.CompareByStatistic.defaultName.fmt', [value]);
                    }

                    // Comparing by distribution
                    else if (whatType === HprConstants.WhatType.Distribution) {
                        return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.scenarios.CompareByHorizon.defaultName.fmt', [value]);
                    }
                }

                if (chartModel.isMultiSeriesPlot()) {
                    if (self.isMultiSeriesHorizonPlot(chartModel.getPlotTypeObject())) {
                        return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.GroupedHorizons.CompareByAllScenarioAllResultSets.txt');
                    }
                    else {
                        return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.GroupedHorizons.CompareByScenario.txt');
                    }
                }

                // If we are comparing by subportfolio then use that prefix (this is down a column)
                if (byType === HprConstants.ByType.Subportfolio) {
                    return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.SubportfolioComparison.defaultName.fmt', [value]);
                }
                break;
            case HprConstants.PlotType.Distribution:

                // If the plot type is distribution then we
                if (chartModel.isByTypeHorizon()) {

                    // comparing by stat
                    if (whatType === HprConstants.WhatType.Statistic) {
                        return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.horizons.CompareByStatistic.defaultName.fmt', [value]);
                    }

                    // comparing by distribution
                    else if (whatType === HprConstants.WhatType.Distribution) {
                        return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.horizons.CompareByDistribution.defaultName.fmt', [value]);
                    }
                }
                else if (chartModel.isByTypeRiskCube()) {
                    return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.riskcubes.CompareByDistribution.defaultName.fmt', [value]);
                }


                // If we are comparing by subportfolio then use that prefix (this is down a column)
                if (byType === HprConstants.ByType.Subportfolio) {
                    return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.SubportfolioDistribution.defaultName.fmt', [value]);
                }
                else {
                    return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.distribution.defaultName.fmt', [value]);
                }
            // Plot is a density graph
            case HprConstants.PlotType.Density:
                return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.DensityGraph.defaultName.fmt', [value]);

            // Plot is a RFIM graph
            case HprConstants.PlotType.RiskFactorInformationMeasures:
                return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.RFIMGraph.defaultName.fmt', [value]);
            // For a cashflow use the generated value as the title
            case HprConstants.PlotType.CashFlow:
                return value;
            default:
                return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.defaultName.fmt', [plotType, value]);
        }
    },

    /**
     * @description Given a category role which is really a DataItem returns the displayable string for it. If the category
     * role is of any type of horizon then the localized name for 'Horizon' is returned.  Will return 'Unknown'
     * if both of these conditions are false.
     * @param {DataItem} categoryRole
     * @param {HprChartModel} chartModel
     * @returns {string}
     */
    getCategoryRoleName: function(categoryRole, chartModel) {
        var self = this;
        if (chartModel.isByTypeSubportfolio()) {
            return self.getDisplayString(categoryRole, chartModel);// TODO: honor label setting in options
        }
        else if (chartModel.isCashFlowPlot()){
            return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.cashflow.categoryRoleName.txt');
        }
        else if (chartModel.isRFIMPlot()){
            return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.rfim.categoryRoleName.txt');
        }
        else if (self.isMultiSeriesHorizonPlot(chartModel.getPlotTypeObject())) {
            return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.horizons.categoryRoleName.txt');
        }
        else if (chartModel.isByTypeRiskCube()) {
            return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.riskcube.categoryRoleName.txt');
        }
        else if (chartModel.isByTypeResultSet()) {
            return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.resultSet.categoryRoleName.txt');
        }
        else if (chartModel.isByTypeScenario()) {
            return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.scenario.categoryRoleName.txt');
        }
        else if (chartModel.isByTypeHorizon()) {
            return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.horizons.categoryRoleName.txt');
        }
        else if (self.isHorizonType(categoryRole.type)) {
            return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.horizons.categoryRoleName.txt');
        }
        return "Unknown";
    },

    /**
     * @description Given a list of responseRoles, generates an array of displayable strings for those roles
     * @param {DataItem[]|string[]} responseRoles
     * @param {HprChartModel} chartModel
     * @returns {string[]}
     */
    getResponseRoleLabels: function(responseRoles, chartModel) {
        var self = this,
            labels = [];
        _.each(responseRoles, function(responseRole){
            if (self.isDataItem(responseRole)) {
                labels.push(self.getDisplayString(responseRole, chartModel));
            }
            else {
                labels.push(responseRole); // Response role is assumed to be the label
            }
        });
        return labels;
    },

    /**
     * Generates a displayable name for the response role.  If the cube is a stress cube this will typically be
     * outvar (stat) otherwise it will just be the stat
     * @param {HprChartModel} chartModel
     * @param {DataItem} [responseRole] Required for a price cube, not used for a stress cube.
     * @returns {*}
     */
    getResponseRoleName: function(chartModel, responseRole) {
        var self = this,
            firstColumnTuples,
            statId,
            statName = null,
            stat,
            outvarId,
            outvar,
            outvarName;
        // If we have a stress cube the name should be outvar(stat)
        if (chartModel.dataSource.isStressCube()) {
            firstColumnTuples = chartModel.plotQueryData.tuples[0];

            // Get the stat name
            statId = _.find(firstColumnTuples, function(tuple) {
                return tuple.type === sas.analytics.riskhp.models.dataitems.Statistic.STATISTIC;
            });
            if (statId) {
                stat = chartModel.dataSource.getDataItemById(statId.id);
                if (stat) {
                    statName = self.getDisplayString(stat, chartModel);
                }
            }

            // Get the outvar name
            outvarId = _.find(firstColumnTuples, function(tuple) {
                return tuple.type === sas.analytics.riskhp.models.dataitems.Outvar.OUTVAR;
            });
            if (outvarId) {
                outvar = chartModel.dataSource.getDataItemById(outvarId.id);
                if (outvar) {
                    outvarName = self.getDisplayString(outvar, chartModel);
                }
            }

            if (outvarName && statName) {
                return sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.stresscube.responseRoleName.fmt", [outvarName, statName]);
            }
            else if (statName) {
                return statName;
            }
            else if (outvarName) {
                return outvarName;
            }
            // If got here we should fall through and let the price cube rules go into play
        }

        if (self.isDataItem(responseRole)) {
            return self.getDisplayString(responseRole, chartModel);
        }

        // If it it's not a DataItem object check to see if it has a name property and return that, otherwise return the
        // responseRole object unchanged
        var nameProp = _.propertyOf(responseRole)('name');
        if (nameProp) {
            return nameProp;
        }
        return responseRole;
    },

    /**
     * @description Returns the displayable label for the given response role.  If responseRole is not a DataItem, it
     * is assummed to be a string and is directly returned.
     * @param {string|DataItem} responseRole
     * @param {HprChartModel} chartModel
     * @returns {string}
     */
    getLabelForResponseRole: function(responseRole, chartModel) {
        var self = this,
            label = "";
        if (self.isDataItem(responseRole)) {
            label = self.getDisplayString(responseRole, chartModel);
        }
        else {
            label = responseRole;
        }
        return label;
    },

    /**
     * Common place to generate and set the path title for the given graph.  May do nothing if the path title would only
     * include the ALL path.
     * @param chartModel
     * @param graph
     */
    setGraphPathTitle: function(chartModel, graph) {
        var title = sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils.getChartTitle(chartModel);
        if (title) {
            graph.setTitles([title]);
        }
    },

    /**
     * @description Given a value that is presumably a string AND a {DataItem} object that acts as/holds the formatting information,
     * return the value formatted (if the formatter is not a STRING type) otherwise the value is returned.
     * @param {(number | string)} value
     * @param {DataItem} formatter
     * @returns {string}
     */
    getFormattedValue: function(value, formatter) {
        // If the class variable is not a string we need to run it through the formatter, otherwise
        // the value is the value we will use.
        if (formatter.getDataType() !== HprConstants.HPR_DATA_TYPE_STRING) {
            return sas.ltjs.sasicu.util.SASFormatterUtil.format(Number(value), formatter.getFormat());
        }
        return value;
    },

    /**
     * Given a GTL Column, adds the passed in values to it based on the formatter.  If the formatter is null or has the
     * dataType of "string" the values are added to the column as Strings.  Otherwise the formatter is used to set the
     * format name on the column, the values are assumed to be numbers and added as such, and the dataType of the
     * formatter is used to set the column's data type.
     * @param column - The column to add the (formatted) values to
     * @param values - An array of values (usually strings or numbers)
     * @param formatter - null, DataItem or a string
     */
    addValuesToColumn: function(column, values, formatter) {

        if (formatter == null || formatter.dataType === "string") {
            //sanitize any blank strings that are length zero as the plots will not show the column in that case. so we push
            // a 1 space string in it's place.
            var santizedStrings = _.map(values, function(value){
                if (value.length === 0) {
                    return " ";
                }
                return value;
            });
            column.setStrings(santizedStrings);
            column.setDataType(sas.ltjs.commons.models.data.dataset.DataType.STRING);
        }
        else {
            var format = (_.isString(formatter)) ? formatter : formatter.getFormat();

            // We ASSume numbers!!
            column.setNumbers(values);
            column.setDataType(sas.ltjs.commons.models.data.dataset.DataType.NUMBER);
            column.setFormatName(format);
        }
    },

    // /**
    //  * Accumulates the values and stat labels for a distribution plot.  Returns a two value tuple containing
    //  * both lists.
    //  * @param categoryResponseRoles
    //  * @param chartModel
    //  * @returns {{yValues: Array, statStrings: Array}}
    //  */
    // buildValuesForDistributionPlot: function(categoryResponseRoles, chartModel) {
    //     var self = this,
    //         yValues = [],
    //         statStrings = [];
    //     _.each(categoryResponseRoles, function(responseRole) {
    //         var label = self.getLabelForResponseRole(responseRole,chartModel);
    //
    //         // Get the responseValues object for the given category label
    //         var responseValuesForCategory = chartModel.responseValues[label];
    //
    //         // walk over keys pulling out each of the values and pushing into the yValues array
    //         var responseValuesForCategoryKeys = _.keys(responseValuesForCategory);
    //         _.each(responseValuesForCategoryKeys, function(key){
    //             yValues.push(responseValuesForCategory[key]);
    //             var stat = chartModel.dataSource.getDataItemById(key);
    //             if (stat) { // Note: If we don't get a stat here we are HOSED!!!!
    //                 if (stat.getName() === sas.analytics.riskhp.models.dataitems.Statistic.NUMPRICE) {
    //                     statStrings.push("N");
    //                 }
    //                 else {
    //                     statStrings.push(stat.getName());
    //                 }
    //             }
    //         });
    //     });
    //     return {yValues: yValues,
    //             statStrings: statStrings};
    // },

    /**
     * Is the ByType held in the PlotType Subportfolio
     * @param plotType
     * @returns {boolean}
     */
    isByTypeSubportfolio: function(plotType) {
        var byType = plotType.ByType;

        // ByType can be a DataItem :-( or a string.  When plotting by subportfolio it will be a string, so protect ourselves.
        if (!_.isObject(byType)) {
            return byType === HprConstants.ByType.Subportfolio;
        }
        return false;
    },

    /**
     * @description Is the ByType held in the PlotType a RiskCube
     * @param {PlotTypeObject} plotType
     * @returns {boolean}
     */
    isByTypeRiskCube: function(plotType) {
        var byType = plotType.ByType;

        // ByType can be a DataItem :-( or a string.  When plotting by risk cube it will be a DataItem object.
        if (_.isObject(byType)) {
            return byType.type === sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE;
        }
        return false;
    },

    /**
     * @description Is the ByType held in the passed in {PlotTypeObject} a ResultSet
     * @param {PlotTypeObject} plotType
     * @returns {boolean}
     */
    isByTypeResultSet: function(plotType) {
        var byType = plotType.ByType;

        // ByType can be a DataItem :-( or a string.  When plotting by result set it will be a DataItem object.
        if (_.isObject(byType)) {
            return byType.type === sas.analytics.riskhp.models.dataitems.ResultSet.RESULTSET;
        }
        return false;
    },

    /**
     * @description Is the ByType held in the {PlotTypeObject} a Horizon
     * @param {PlotTypeObject} plotType
     * @returns {boolean}
     */
    isByTypeHorizon: function(plotType) {
        if (!plotType || !plotType.ByType || _.isArray(plotType.ByType)) {
            return false;
        }
        return this.isDataItemHorizonType(plotType.ByType);
    },

    /**
     * Is the DataItem a Horizon DataItem of any type
     * @param {DataItem} dataItem
     * @returns {boolean}
     */
    isDataItemHorizonType: function(dataItem) {
        var self = this;
        if (_.isObject(dataItem)) {
            return self.isHorizonType(dataItem.type);
//            return dataItem.type === sas.analytics.riskhp.models.dataitems.AbsoluteHorizon.ABSOLUTEHORIZON || dataItem.type === sas.analytics.riskhp.models.dataitems.Horizon.HORIZON || dataItem.type === sas.analytics.riskhp.models.dataitems.RelativeHorizon.RELATIVEHORIZON;
        }
        return false;
    },

    /**
     * Is the item passed in a DataItem object.  To be a DataItem, item must be an object and have a defined function for 'getId'
     * @param item
     * @returns {boolean}
     */
    isDataItem: function(item) {
        if (_.isObject(item)) {
            return (typeof item.getId === 'function' );
        }
        return false;
    },

    /**
     * Is the DataItem a date DataItem
     * @param dataItem
     * @returns {boolean}
     */
    isDataItemDateType: function(dataItem) {
        if (this.isDataItem(dataItem)) {
            return dataItem.dataType === HprConstants.HPR_DATA_TYPE_DATE;
        }
        return false;
    },

    /**
     * Is the DataItem a date DataItem
     * @param dataItem
     * @returns {boolean}
     */
    isDataItemNumberType: function(dataItem) {
        if (_.isObject(dataItem)) {
            return dataItem.dataType === HprConstants.HPR_DATA_TYPE_NUMBER;
        }
        return false;
    },

    /**
     * @description Is the PlotType a Density plot
     * @param {PlotTypeObject} plotType
     * @returns {boolean}
     */
    isDensityPlot: function(plotType) {
        return plotType.PlotType === HprConstants.PlotType.Density;
    },

    /**
     * @description Is the PlotType a RiskFactorInformationMeasures plot
     * @param {PlotTypeObject} plotType
     * @returns {boolean}
     */
    isRFIMPlot: function(plotType) {
        return plotType.PlotType === HprConstants.PlotType.RiskFactorInformationMeasures;
    },

    /**
     * @description Is the plot to generate a Comparison plot
     * @param {PlotTypeObject} plotTypeObject
     * @returns {boolean}
     */
    isComparisonPlot: function(plotTypeObject) {
        return plotTypeObject.PlotType === HprConstants.PlotType.Comparison;
    },

    /**
     * @description Is the plotType a CashFlow plot
     * @param {PlotTypeObject} plotType
     * @returns {boolean}
     */
    isCashFlowPlot: function(plotType) {
        return plotType.PlotType === HprConstants.PlotType.CashFlow;
    },

    /**
     * @description does the passed in plotType represent a distribution plot
     * @param {PlotTypeObject} plotType
     * @returns {boolean}
     */
    isDistributionPlot: function(plotType) {
        return plotType.WhatType === HprConstants.WhatType.Distribution;
    },

    /**
     * @description Helper function to take a list of data item ids (presumably representing a single series) and creates a unique
     * string representing those ids suitable for use as a key in a map.
     * @param {string[]} ids - A list of data item ids
     * @returns {string}
     */
    generateUniqueSeriesIdFromIds: function(ids) {
        var uniqueSeriesId = "";
        _.each(ids, function(id){
            if (uniqueSeriesId.length === 0) {
                uniqueSeriesId = id;
            }
            else {
                uniqueSeriesId = uniqueSeriesId+ "|" + id;
            }
        });
        return uniqueSeriesId;
    },

    /**
     * @description Given a list of dataItems finds the first item in the list that has the same type
     * @param {(DataItemIdObject[]|DataItem[])} tupleList
     * @param {(string|string[])} types - the allowed compatible types.  Note the order is significant, the first matching DataItem found by type will be returned.
     * @param {boolean} [matchAnyHorizonType] - true if a horizon object should match any of it's compatible types (i.e absolute, relative, or horizon)
     * @return {(DataItemIdObject|DataItem|null)} The found item or null if none was found
     */
    getMatch: function(tupleList, types, matchAnyHorizonType) {
        var self = this;
        var typesToSearch;

        if (_.isArray(types)) {
            typesToSearch = types;
        }
        else {
            typesToSearch = [types]; // ASSume we got one
        }

        for (var i=0; i< typesToSearch.length; i++) {
            var type = typesToSearch[i];
            var foundObject = _.find(tupleList, function(tuple){
                if (self.isHorizonType(type) && matchAnyHorizonType) {
                    return self.isAnyTypeOfHorizon(tuple);
                }
                else {
                    return tuple.type === type;
                }
            });
            if (foundObject) {
                return foundObject;
            }
        }
        return null;
    },

    /**
     * @description Given a list of dataItems returns the index of first item in the list that has one of the types in the passed in types list
     * @param {(DataItemIdObject[]|DataItem[])} tupleList
     * @param {(string|string[])} types - the allowed compatible types.  Note the order is significant, the index of the first matching DataItem found by type will be returned.
     * @param {boolean} [matchAnyHorizonType] - true if a horizon object should match any of it's compatible types (i.e absolute, relative, or horizon)
     * @return {Int} The index of the found item or -1 if none was found
     */
    indexOf: function(tupleList, types, matchAnyHorizonType) {
        var self = this;
        var typesToSearch;

        if (_.isArray(types)) {
            typesToSearch = types;
        }
        else {
            typesToSearch = [types]; // ASSume we got one
        }

        for (var i=0; i< typesToSearch.length; i++) {
            var type = typesToSearch[i];
            var foundIndex = _.findIndex(tupleList, function(tuple){
                if (self.isHorizonType(type) && matchAnyHorizonType) {
                    return self.isAnyTypeOfHorizon(tuple);
                }
                else {
                    return tuple.type === type;
                }
            });
            if (foundIndex > -1) {
                return foundIndex;
            }
        }
        return -1;
    },

    generateUniqueSeriesIdFromTuples: function(tuples, seriesTypes) {
        var self = this,
            tupleIdsToFormUniqueIdFrom = [];

        //First get tuple objects that should compromise the id
        _.each(seriesTypes, function(seriesType){
            var matchingTuple = self.getMatch(tuples, seriesType, true);
            tupleIdsToFormUniqueIdFrom.push(matchingTuple.id);
        });

        // Now call generateUniqueSeriesIdFromIds to generate the unique series id
        return self.generateUniqueSeriesIdFromIds(tupleIdsToFormUniqueIdFrom);
    },

    /**
     * @description Given the data items that make up a {SeriesObject} returns the displayable name for the series
     * @param {DataItem[]} dataItems
     * @param {HprChartModel} chartModel
     * @returns {string}
     */
    getSeriesName: function(dataItems, chartModel) {
        var self = this;
        if (dataItems.length === 1) {
            self.getDisplayString(dataItems[0], chartModel);
        }
        var name = "";
        _.each(dataItems, function(dataItem){
            name = name + "/" + self.getDisplayString(dataItem, chartModel);
        });
        return name;
    },

    /**
     * @description Given the ids of dataitems that compose a series, returns the displayable name for the series
     * @param {string[]} seriesDataItemIds
     * @param {number} chartIndex
     * @param {HprChartModel} chartModel
     * @returns {string}
     */
    getSeriesNameForIds: function(seriesDataItemIds, chartIndex, chartModel) {
        var self = this;
        var seriesDataItems = chartModel.getSeriesDataItems(seriesDataItemIds, chartIndex);
        return self.getSeriesName(seriesDataItems, chartModel);
    },

    /**
     * Is the plot that is supposed to be generated a Subportfolio Distribution plot
     * @param plotType
     * @returns {boolean}
     */
    isSubportfolioDistributionPlot: function(plotType) {
        return plotType.PlotType === HprConstants.PlotType.Comparison && plotType.WhatType === HprConstants.WhatType.Distribution;
    },

    /**
     * @description Builds the legend for the given chart, may return null if the chart needs no legend (which is the case when there
     * is only one series in the chart)
     *
     * @param {HprChartModel} chartModel
     * @param {Array} charts - array of the charts the legend will be for.  It will be used as the
     * legend entry provider for the legend
     * @returns {*}
     */
    buildLegend: function(chartModel, charts) {

        if (!chartModel.isMultiSeriesPlot() && !chartModel.isCashFlowPlot()) {
            return null;
        }

        // No legend if there is only one series.  Note: we assume only one chart is being rendered at this time
        if (chartModel.getSeriesIds(0).length <= 1) {
            return null;
        }

        var autoLegend = new sas.ltjs.gtl.models.legend.AutomaticLegend();
        autoLegend.setLegendEntryProvider(charts);

        var globalLegend = new sas.ltjs.gtl.models.legend.LayoutGlobalLegend();
        globalLegend.setLegendTitlePosition(sas.ltjs.gtl.models.legend.LegendTitlePosition.BOTTOM);
        globalLegend.setLegends([autoLegend]);
        return globalLegend;
    },

    /**
     * Returns a displayable string for the grouping type.  If the plot is not a grouping plot then returns null
     * @returns {string | null}
     */
    getGroupingRoleName: function(chartModel) {
        if (chartModel.isMultiSeriesPlot()) {
            if (chartModel.isByTypeScenario()) {
                return sas.analytics.riskhp.utils.HprUtil.getBundle().getText("DataItem.scenario.txt");
            }
            else if (chartModel.isByTypeResultSet()) {
                return sas.analytics.riskhp.utils.HprUtil.getBundle().getText("DataItem.resultSet.txt");
            }
            // must be all scenarios/result sets
            return sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.GroupedHorizons.CompareByAllScenarioAllResultSets.LegendTitle.txt");
        }
        else if (chartModel.isCashFlowPlot()) {
            return sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.Cashflows.LegendTitle.txt");
        }
        return null;
    },

    /**
     * Given a chartModel returns the name of the Crosstab this model is linked to.  Will return null if it couldn't
     * find the crosstab or it is not linked.  Consumers should check the model's isLinkedToCrosstab state prior to calling
     * this function.
     * @param chartModel
     * @returns {(string|null)}
     */
    getLinkedCrosstabName: function(chartModel) {
        if (chartModel.crosstabId) {
            // Check to make sure we can still find the crosstab
            var crosstab = sap.ui.getCore().byId(chartModel.crosstabId);
            if (crosstab && crosstab.context) {
                return crosstab.context.getOptionName();
            }
            return null;
        }
        return null;
    },

    /**
     * @description Takes the original template column and a new column tuple and merges the two lists
     * so wildcards are left in the resultant template but all the other DataItem's in the
     * returned list are from the newColumnTuple.
     *
     * @param {TemplateItem[]} originalTemplate
     * @param {DataItem[]} newColumnTuple
     * @returns {TemplateItem[]}
     */
    updateTemplateColumn: function(originalTemplate, newColumnTuple){
        var self = this,
            newTemplate = [];

        // Create the new template from the newColumnTuple (this is a dehydrated template)
        _.each(newColumnTuple, function(tuple){
           newTemplate.push({ item:{id: tuple.id, type: tuple.type}, type: tuple.type});
        });

        // find type of the wild carded items (if any) in the original template
        var wildcardTemplateItems = [];
        _.each(originalTemplate, function(templateItem){
            if (_.isUndefined(templateItem.item)) {
                wildcardTemplateItems.push(templateItem);
            }
        });

        // if there are wildcarded template items then find and replace matching items in the newTemplate array with the wildcards.
        var newTemplateIsCompatible = true;
        _.each(wildcardTemplateItems, function(wildcardTemplateItem){
            var indexToReplace = -1;

            // Note: we walk the newColumnTuple array as we will modify the newTemplate array
            // Note: this algorithm won't work if there are two of the same type in the newColumnTuple but as I understand it that isn't possible
            _.each(newColumnTuple, function(tuple,index){
                if (self.isHorizonType(wildcardTemplateItem.type) && self.isHorizonType(tuple.type)) {
                    indexToReplace = index;
                }
                else if (wildcardTemplateItem.type === tuple.type) {
                    indexToReplace = index;
                }
            });

            // If we found the item in the newColumnTuple replace it's representation in the newTemplate with the wildcard version.
            if (indexToReplace >= 0) {
                newTemplate[indexToReplace] = wildcardTemplateItem;
            }
            else {
                // We have an incompatible template
                newTemplateIsCompatible = false;
            }
        });

        if (!newTemplateIsCompatible) {
            return null;
//            return originalTemplate;
        }
        return newTemplate;
    },

    /**
     * Ensures all the items in the passed in template column that can be hydrated
     * are hydrated.  Wildcards will be passed through
     * @param {(DataItemIdObject[]|DataItem[])} templateColumn
     * @param {DataSource} dataSource
     * @returns {DataItem[]} - Note: This might need to return {TemplateItem} objects
     */
    getHydratedTemplateColumn: function(templateColumn, dataSource) {
        var hydratedTemplateColumn = [];
        _.each(templateColumn, function(templateColumnItem) {
            if (templateColumnItem.item) {
                hydratedTemplateColumn.push({item: sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils.hydrateDataItem(templateColumnItem.item, dataSource), type: templateColumnItem.type});
            }
            else {
                hydratedTemplateColumn.push(templateColumnItem);
            }
        });
        return hydratedTemplateColumn;
    },

    convertHorizonTypeSelectValueToDataItemType: function(optionValue) {
        if (optionValue === HprConstants.DISPLAY_HORIZONS_ABSOLUTE_VALUE) {
            return sas.analytics.riskhp.models.dataitems.AbsoluteHorizon.ABSOLUTEHORIZON;
        }
        else if (optionValue === HprConstants.DISPLAY_HORIZONS_RELATIVE_VALUE) {
            return sas.analytics.riskhp.models.dataitems.RelativeHorizon.RELATIVEHORIZON;
        }
    },

    switchHorizonsInColumnToBaseHorizons: function(column, dataSource) {
        var newColumn = this.hydrateDataItems(column, dataSource);

        // If there is a horizon in the source column get it's base horizon (and swap it out in the rehydrated list)
        var index = this.indexOf(newColumn, sas.analytics.riskhp.models.dataitems.Horizon.HORIZON, true);
        var baseHorizon = null;
        if (index !== -1) {
            baseHorizon = sas.analytics.riskhp.common.riskcommons.CrosstabUtil.findBaseHorizon(newColumn, index, dataSource);
            newColumn[index] = baseHorizon;
        }
        return { baseHorizon: baseHorizon, column: this.dehydrateDataItems(newColumn)};
    },

    /**
     * Extracts just the wildcard items from the passed in template column.  If a wildcard is of any horizon type ensures
     * the type stored for the wild card is the base horizon type
     * @param templateColumn
     * @returns {*}
     */
    sanitizeColumnTemplate: function(templateColumn) {
        var self = this;
        var wildcards =  _.filter(templateColumn, function(templateColumnObject) {
            return !_.has(templateColumnObject,"item");
        });

        return _.map(wildcards, function(item) {
            if (self.isHorizonType(item.type)) {
                return { type: sas.analytics.riskhp.models.dataitems.Horizon.HORIZON};
            }
            else {
                return item;
            }
        });
    },

    synthesizeColumnTemplate: function(templateColumn, sourceColumn, displayHorizonType, dataSource) {
        var templateToReturn = [];
        var self = this;

        var sanitizedTemplate = this.sanitizeColumnTemplate(templateColumn);
        _.each(sourceColumn, function(dataItemObject) {
            var dataItem = sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils.hydrateDataItem(dataItemObject, dataSource);
            var type = dataItemObject.type;
            // If the templateColumn has that type in it then this is wildcarded, otherwise hydrate it
            var wildcardItem = _.find(sanitizedTemplate, function(templateColumnItem) {
                // normalize the type we are checking to the base horizon so we can deal with non-fixed explorations
                // (explorations where their dataitems include children horizons in the source column)
                if (self.isHorizonType(type)) {
                    return self.isHorizonType(templateColumnItem.type);
                }
                else {
                    return templateColumnItem.type === type;
                }
            });
            // If we are on a type that is wildcarded, push the wildcard (accounting for the type of horizon to retrieve)
            if (wildcardItem) {
                if (self.isHorizonType(wildcardItem.type)) {
                    if (displayHorizonType === sas.analytics.riskhp.models.dataitems.AbsoluteHorizon.ABSOLUTEHORIZON) {
                        templateToReturn.push({ type: sas.analytics.riskhp.models.dataitems.AbsoluteHorizon.ABSOLUTEHORIZON});
                    }
                    else {
                        templateToReturn.push({ type: sas.analytics.riskhp.models.dataitems.RelativeHorizon.RELATIVEHORIZON});
                    }
                }
                else {
                    templateToReturn.push(wildcardItem);
                }
            }
            else if (dataItem) {
                // Otherwise push in the dataItem but ensure we have accounted for the type of horizon to retrieve
                if (self.isHorizonType(type)) {
                    var resolvedHorizon = null;
                    if (displayHorizonType === sas.analytics.riskhp.models.dataitems.AbsoluteHorizon.ABSOLUTEHORIZON) {
                        resolvedHorizon = dataSource.findAbsoluteHorizon(dataItem.horizonDate, dataItem.label );
                    }
                    else {
                        resolvedHorizon = dataSource.findRelativeHorizon(dataItem.interval, dataItem.horizonNumber );
                    }
                    if (resolvedHorizon) {
                        templateToReturn.push({item: resolvedHorizon, type: resolvedHorizon.type});
                    }
                }
                else {
                    templateToReturn.push({item: dataItem, type: dataItem.type});
                }
            }
        });
        return templateToReturn;
    },

    /**
     * @description Takes a tuple list and the type that was attempted to be found (but could not be) in that list. Searches
     * the tuple list for DataItems that are "compatible" with the original type.  For example if type is a statistic then
     * a benchmark variable is compatible, etc. This method is really only intended to be used for comparison plots.
     * @param {DataItem[]} tuples
     * @param {string} type
     * @returns {(DataItem|null)}
     */
    getCompatibleDataItem: function(tuples, type) {
        // Case where original type was statistic
        if (type === sas.analytics.riskhp.models.dataitems.Statistic.STATISTIC) {
            return this.getMatch(tuples, [sas.analytics.riskhp.models.dataitems.Benchvar.BENCHVAR, sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR]);
        }

        // Case where original type was benchmark
        if (type === sas.analytics.riskhp.models.dataitems.Benchvar.BENCHVAR) {
            return this.getMatch(tuples, [sas.analytics.riskhp.models.dataitems.Statistic.STATISTIC, sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR]);
        }

        // Case where original type was computedvariable
        if (type === sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR) {
            return this.getMatch(tuples, [sas.analytics.riskhp.models.dataitems.Statistic.STATISTIC, sas.analytics.riskhp.models.dataitems.Benchvar.BENCHVAR]);
        }
        return null;
    },

    /**
     * @description Uses the passed in tuple list to update the passed in {PlotType} object
     * @param {PlotTypeObject} plotType
     * @param {DataItemIdObject[]} newTupleList
     * @throws Throws an error message if the plotType could not be updated, usually because there is not a compatible
     * DataItem in the newTupleList
     */
    fixPlotTypeForNewTuples: function(plotType, newTupleList) {
        var self = this;
        if (self.isDistributionPlot(plotType)) {
            return; // Nothing to fix?
        }
        if (self.isComparisonPlot(plotType)) {
            // does the new tuple have the same DataItem type in it?
            var matchingDataItem = self.getMatch(newTupleList, plotType.WhatType);

            // If we have the same type then we are good
            if (matchingDataItem) {
                return;
            }

            // If not we need to find a compatible type and then recheck
            var compatibleDataItem = self.getCompatibleDataItem(newTupleList,plotType.WhatType);

            // if we found a compatible one then adjust the what type and return
            if (compatibleDataItem) {
                plotType.WhatType = compatibleDataItem.type;
                return;
            }

            // If we got here the new tuple list isn't compatible with the plot so log it and throw an exception
            console.log("New tuple list doesn't have a compatible data item");
            throw sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.column_not_supported.txt");
        }
    },

    /**
     * Not sure this is needed, we'll know once all the absolute vs relative horizon code paths have been exercised.
     * @param {DataItemIdObject[]} tuples
     */
    fixTuplesForHorizonType: function(tuples) { // , requiredHorizonType, dataSource
        if (_.isUndefined(tuples) || tuples.length === 0) {
            return [];
        }
        return tuples;
    },

    /**
     * Method to help the replace data source workflow.  This takes the new DataSource and the list of tuples the plot
     * holds and attempts to replace any risk cube ids in the tuple list with the valid risk cube id in the new passed
     * in DataSource.  If the dataSource has more than one RiskCube, the tuple list will be returned without modification
     * because we haven't written the AI module (to know which one to use) yet.
     * @param {DataSource} dataSource
     * @param {[[DataItemIdObject]]} tuples
     * @returns {[[DataItemIdObject]]}
     */
    fixTuplesForRiskCube: function(dataSource, tuples) {
        if (_.isUndefined(tuples) || tuples.length === 0 || !dataSource) {
            return [];
        }
        var riskCubes = dataSource.getAllRiskCubes();
        if (!riskCubes || riskCubes.length === 0 || riskCubes.length > 1) {
            return tuples;
        }
        var cubeId = riskCubes[0].getId();
        var finalList = [];
        _.each(tuples, function(tupleArray){
            finalList.push(_.map(tupleArray, function(tuple){
                if (tuple.type === sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE) {
                    tuple.id = cubeId;
                }
                return tuple;
            }));
        });
        return finalList;
    },

    /**
     * @description Ensures regardless of what is held in the chartModel we have a easily comparable representation
     * of the model's byType
     * @param {HprChartModel} chartModel
     * @returns {string} a string representation of the chartModel's current ByType
     */
    getSanitizedByType: function(chartModel) {
        // By type is special as it can be a string, a ByTypeDataItemIdObject, or an array, but for now all that
        // really matters is it's type, so we switch to it, if needed.
        var byType = chartModel.getByType();
        if (_.has(byType, "type")) { // This is a ByTypeDataItemIdObject
            return byType.type;
        }
        else if (_.isArray(byType)) { // An array of strings
            var typeString = "";
            // Must sort these in their natural order so comparisons will always work
            // Make copy then sort
            var sortedByTypeArray = byType.slice(0, byType.length);
            sortedByTypeArray.sort();
            _.each(sortedByTypeArray, function(value){
                typeString = typeString+value;
            });
            return typeString;
        }
        return byType;
    },

    /**
     * Check to see if the two "whatTypes" are equivalent based on the currently known rules
     * @param whatType
     * @param otherWhatType
     * @returns {boolean}
     */
    areCompatibleWhatTypes: function(whatType, otherWhatType) {

        // If they are the same there is no reason to check.
        if (whatType === otherWhatType) {
            return true;
        }

        // Case where original type was statistic
        if (whatType === sas.analytics.riskhp.models.dataitems.Statistic.STATISTIC) {
            return otherWhatType === sas.analytics.riskhp.models.dataitems.Benchvar.BENCHVAR || otherWhatType === sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR;
        }

        // Case where original type was benchmark
        if (whatType === sas.analytics.riskhp.models.dataitems.Benchvar.BENCHVAR) {
            return otherWhatType === sas.analytics.riskhp.models.dataitems.Statistic.STATISTIC || otherWhatType === sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR;
        }

        // Case where original type was computed variable
        if (whatType === sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR) {
            return otherWhatType === sas.analytics.riskhp.models.dataitems.Statistic.STATISTIC || otherWhatType === sas.analytics.riskhp.models.dataitems.Benchvar.BENCHVAR;
        }
        return false;
    }
};