// (c) 2017, SAS Institute Inc.

/**
 * @namespace sas.analytics.riskhp.common.riskcommons
 * @namespace sas.analytics.riskhp.common.riskcommons.plot
 */
jQuery.sap.declare("sas.analytics.riskhp.common.riskcommons.plot.HprChartView");

jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils");
jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprChartTableUtils");
jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprChartModel");
jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprBarChart");
jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprCashFlowChart");
jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprLineChart");
jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprDistributionChart");
jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprSeriesChart");
jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprTimeSeriesChart");
jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprDensityChart");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Statistic");
jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
jQuery.sap.require("sas.ltjs.sasicu.util.CalendarUtil");
jQuery.sap.require("sas.ltjs.sasicu.util.SASFormatterUtil");
jQuery.sap.require("sas.hc.x.WideTable");

jQuery.sap.require("sas.risk.common.panelsview.DivWrapper");

(function() {
    "use strict";
    /** @name sas.icons.HC */
    /** @name sas.icons.HC.ZEROSTATEGRAPH */
    /** @name sas.icons.HC.LINKBROKEN */
    /** @name sas.hc.m.MessageDialog */

    var HprChartUtils = sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils;
//    var HprChartModel = sas.analytics.riskhp.common.riskcommons.plot.HprChartModel;
    var HprBarChart = sas.analytics.riskhp.common.riskcommons.plot.HprBarChart;
    var HprCashFlowChart = sas.analytics.riskhp.common.riskcommons.plot.HprCashFlowChart;
    var HprLineChart = sas.analytics.riskhp.common.riskcommons.plot.HprLineChart;
    var HprDistributionChart = sas.analytics.riskhp.common.riskcommons.plot.HprDistributionChart;
//    var HprSeriesChart = sas.analytics.riskhp.common.riskcommons.plot.HprSeriesChart;
    var HprTimeSeriesChart = sas.analytics.riskhp.common.riskcommons.plot.HprTimeSeriesChart;
    var HprDensityChart = sas.analytics.riskhp.common.riskcommons.plot.HprDensityChart;
//    var Statistic = sas.analytics.riskhp.models.dataitems.Statistic;
//    var SASFormatterUtil = sas.ltjs.sasicu.util.SASFormatterUtil;

    /**
     * This is the "controller" class for plot panels.  It provides the ability to switch between table
     * and chart mode and coordinates the building of each.
     *
     * @name sas.analytics.riskhp.common.riskcommons.plot.HprChartView
     */
    sap.ui.core.Control.extend("sas.analytics.riskhp.common.riskcommons.plot.HprChartView", {

        metadata: {
            properties: {
                "selectedMode" : {type: "string", defaultValue: "chart"},
                "chart": {type: "any"},
                "table": {type: "any"},
                "viewLayout": {type: "any"},
                "chartModel": {type: "object", defaultValue: null },
                "width": {type: "sap.ui.core.CSSSize", defaultValue: null},
                "height": {type: "sap.ui.core.CSSSize", defaultValue: null}
            }
        },

        constructor: function(id, mSettings) {
            // construct super
            sap.ui.core.Control.call(this, id, mSettings);

            // declare local vars
            var self = this;

            // Subscribe to event that is fired when double click is detected in a crosstab
            console.log("HprChartView: subscribing to update linked plots event");
            sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_LINKED_PLOTS, self._onUpdateLinkedPlot, self);
            sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_CROSSTAB_DELETED, self._onCrosstabDeleted, self);
            sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_SCENARIO_UPDATED, self._onScenarioChanged, self);
            sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_SCENARIO_DELETED, self._onScenarioChanged, self);
            sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_SCENARIO_ADDED, self._onScenarioChanged, self);

            // Subscribe to an event that is fired when the label for a Data Item is updated
//            sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_DATA_ITEM_LABEL_UPDATED, self._onDataItemLabelUpdate, self);

            // Subscribe to an event that is fired when the 'Show labels' option field was updated
//            sas.eventBus.subscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_PORTFOLIO_DETAILS_OPTIONS_SHOW_LABELS_UPDATED, this._onShowLabelsChange, this);
            var child = self._currentContent();

            self._delinkLikePlots();

            var toolbar = self._buildPathToolbar();
            var topToolbar = null;
            if (toolbar) {
                topToolbar = new sas.hc.ui.commons.layout.BorderLayoutArea({
                    overflowX: "hidden",
                    overflowY: "hidden",
                    size: "2.0rem",
                    content: [toolbar]
                });
            }

            var borderLayout = new sas.hc.ui.commons.layout.BorderLayout({
                width: "100%",
                height: "100%",
                top: topToolbar,
                center: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    overflowX: "hidden",
                    overflowY: "hidden",
                    content: [child]
                })
            });
            self.setViewLayout(borderLayout);
        },

        /**
         * The renderer for this view
         * @param rm
         * @param control
         */
        renderer: function(rm, control) {
            if(!control.getVisible()) return;

            // Create a div to put everything in
            rm.write("<div");
            rm.writeControlData(control);

            // Make sure the right width and heights are set and also add the hprChart style
            if(control.getWidth()) {
                rm.addStyle("width", control.getWidth());
            }
            if(control.getHeight()) {
                rm.addStyle("height", control.getHeight());
            }
            rm.addClass('hprChart');

            rm.writeStyles();
            rm.writeClasses();
            rm.write(">");

            rm.renderControl(control.getViewLayout());
            rm.write("</div>");
        },

        exit: function() {
            var self = this,
                chartModel = self.getChartModel();
            console.log("HprChartView: unsubscribing from update linked plots event");
            sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_LINKED_PLOTS, self._onUpdateLinkedPlot, self);
            sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_CROSSTAB_DELETED, self._onCrosstabDeleted, self);
            sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_SCENARIO_UPDATED, self._onScenarioChanged, self);
            sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_SCENARIO_DELETED, self._onScenarioChanged, self);
            sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_SCENARIO_ADDED, self._onScenarioChanged, self);
            sas.analytics.riskhp.models.cache.CacheManager.deregister(chartModel.getCacheKey(), chartModel.getId());
        },


        /**
         * @description Isolates code to delink "like" plots when a new plot is created
         * @private
         */
        _delinkLikePlots: function() {
            var self = this;
            var sourceChartModel = self.getChartModel();
            var crosstabId = sourceChartModel.crosstabId;
            // get other plots for this crosstab
            var selectedRiskPanelsView = sas.analytics.riskhp.utils.HprUtil.getPanelsViewForComponent(crosstabId);
            if (selectedRiskPanelsView) {
                var delegate = selectedRiskPanelsView.getDelegate();

                var linkedPlotIds = _.filter(delegate.plotIds, function (plotId) {
                    var chartView = sap.ui.getCore().byId(plotId);
                    if (chartView) {
                        var chartModel = chartView.getChartModel();
                        if (chartModel) {
                            return chartModel.getId() !== sourceChartModel.getId() && crosstabId === chartModel.crosstabId && chartModel.linkedToCrosstab;
                        }
                    }
                    return false;
                });
                // Grab common information about the sourceChartModel, we MAY use in the equality comparison
                // Check equality of the two model's PlotTypeObjects
                var sourcePlotType = sourceChartModel.getPlotType();
                var sourceWhatType = sourceChartModel.getWhatType();
                var sourceByType = HprChartUtils.getSanitizedByType(sourceChartModel);
                //var sourceHowType = sourceChartModel.getHowType();

                // Now walk the ids of the plots that are also linked to the crosstab to delink any "like" plots
                _.each(linkedPlotIds, function(plotId){
                    var currentchartView = sap.ui.getCore().byId(plotId);
                    if (currentchartView) {
                        var currentChartModel = currentchartView.getChartModel();
                        if (currentChartModel) {

                            // Now get the information about the currentChartModel (the one we iterated over)
                            var currentPlotType = currentChartModel.getPlotType();
                            var currentWhatType = currentChartModel.getWhatType();
                            var currentByType = HprChartUtils.getSanitizedByType(currentChartModel);
                            //var currentHowType = currentChartModel.getHowType();

                            if (sourcePlotType === currentPlotType &&
                                HprChartUtils.areCompatibleWhatTypes(sourceWhatType, currentWhatType) &&
                                sourceByType === currentByType) {
                                currentchartView._handleLinkToCrosstab(false);
                            }
                        }
                    }
                });
            }
        },

        /**
         * @description Callback function for when the EVENT_UPDATE_LINKED_PLOTS is called
         * @param {string} channelId
         * @param {string} eventId
         * @param {UpdateLinkedPlotObject} data
         * @private
         */
        _onUpdateLinkedPlot: function(channelId, eventId, data) {
            var self = this,
                chartModel = self.getChartModel();

            console.log("HprChartView: OnUpdateLinkedPlot called");

            // Check to see if the chartModel is linked to the crosstab that just updated, if not return
            if (!chartModel.isLinkedToCrosstabWithId(data.crosstabId)) {
                return;
            }

            // If the chartModel is in the process of replacing the data source (this means it is in an intermediate state)
            // We want to ignore the update call from the crosstab and clear the replacing data source flag to avoid a
            // double query
            if (chartModel.isReplacingDataSource()) {
                chartModel.clearReplacingDataSource();
                return;
            }

            // Process the update
            if (chartModel.handleCrosstabUpdated(data)) {

                // If the model is not using a custom title then the panel title should be regenerated
                // TODO: Should we always regenerate the title?
                if (!chartModel.isUsingCustomTitle()) {
                    var title = sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils.generateDefaultPanelTitle(chartModel.dataSource, chartModel);
                    chartModel.setDefaultTitle(title);
                    var selectedRiskPanelsView = sas.analytics.riskhp.utils.HprUtil.getSelectedRiskPanelsView();
                    if (selectedRiskPanelsView) {
                        var delegate = selectedRiskPanelsView.getDelegate();
                        delegate.setTitleForContent(title,chartModel.getId());
                    }
                }
                self.rebuild();

                // If the selected cell changed and this plot is linked to the crosstab that was changed then fire the dirty state
                if (data.selectedCellChanged && chartModel.isLinkedToCrosstabWithId(data.crosstabId)) {
                    $.proxy(self._fireDirty(),self);
                }
            }
        },

        /**
         * @description Callback function for when the EVENT_SCENARIO_UPDATED or DELETED is fired.
         * Right now this causes a refresh on the plot regardless of whether it is using an affected
         * scenario or not.
         * @param {string} channelId
         * @param {string} eventId
         * @param {ScenarioDeletedObject|ScenarioUpdatedObject|ScenarioAddedObject} data
         * @private
         */
        _onScenarioChanged: function(channelId, eventId, data) {
            var self = this,
                chartModel = self.getChartModel();

            console.log("HprChartView: OnScenarioChanged called");

            // If the model is not using a custom title then the panel title should be regenerated
            if (!chartModel.isUsingCustomTitle()) {
                var title = sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils.generateDefaultPanelTitle(chartModel.dataSource, chartModel);
                chartModel.setDefaultTitle(title);
                var selectedRiskPanelsView = sas.analytics.riskhp.utils.HprUtil.getSelectedRiskPanelsView();
                if (selectedRiskPanelsView) {
                    var delegate = selectedRiskPanelsView.getDelegate();
                    delegate.setTitleForContent(title,chartModel.getId());
                }
            }
            // We are about to try to rebuild the plot so clear out the error state (if any)
            chartModel.clearErrorState();
            self._rebuild(true, null, true);
        },

        /**
         * Builds the toolbar for the plot that will show the path string
         * @returns {*}
         * @private
         */
        _buildPathToolbar: function() {
            var title = this._getCurrentPathTitle();
            if (!title || title.length === 0) {
                return null;
            }
            this.pathLabel = new sas.hc.m.Label({
                text: title  // Will show entire path in hover popup if too long
            });
            this.pathLabel.addStyleClass("hprChartPathLabel");
            //self.pathLabel = new sas.hc.m.Label();
            var toolbarContent = [];

            toolbarContent.push(this.pathLabel);
            var toolbar = new sas.hc.m.ResponsiveToolbar({content: toolbarContent});
            toolbar.addStyleClass("hprChartPathToolbar");
            return toolbar;
        },

        /**
         * @description Uses HprChartUtils to generate the title for the plot (this is the part above the plot) not the panel
         * @returns {string}
         * @private
         */
        _getCurrentPathTitle: function() {
            if (!this.isModelValid()) {
                return ""; // return nothing if model is not valid as the zero state will kick in
            }
            var chartTitleText = sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils.constructChartTitle(this.getChartModel());
            if (_.isNull(chartTitleText)) {
                return "";
            }
            return chartTitleText;
        },

        /**
         * @description Returns the current content of the view, will either be a chart or a table (TBD)
         * @returns {*}
         * @private
         */
        _currentContent: function() {
            // Before we attempt to draw a chart or table, check to see if the model is valid
            // If not, display an empty/zero state
            if (!this.isModelValid()) {
                return this._buildZeroState(this.getInvalidModelReasonString());
            }

            if (this.getSelectedMode() === "chart") {
                if (this.chart == null) {
                    this.chart = this._buildChart();
                }
                return this.chart;
            }
            else {
                if (this.table == null) {
                    this.table = this._buildTable();
                    this.table.rerender();
                }
                return this.table;
            }
        },

        /**
         * Checks to see if the chart model is valid.
         * @returns {boolean} Returns whether the underlying model is valid or not.
         */
        isModelValid: function() {
            return this.getInvalidModelReasonString() === null;
        },

        /**
         * Returns a string representation of why the model is valid, or null if the model is valid.
         * @returns {(null|String)}
         */
        getInvalidModelReasonString: function() {
            var chartModel = this.getChartModel();
            if (!chartModel) {
                return sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.no.model.txt");
            }
            // If we have a model, check the chartModel and see if deems itself as valid
            return chartModel.getModelInvalidReasonString();
        },

        /**
         * Pseudo interface method to return the dataSourceId the component is using in a common way
         * @returns String
         */
        getDataSourceId: function() {
            return this.getChartModel().dataSourceId;
        },

        /**
         * Pseudo interface method to return the type of control this is in a common way
         * @returns String
         */
        getPanelType: function() {
            return HprConstants.HPR_PLOT_PANEL;
        },

        /**
         * Force the owning panel to be refreshed for this view.  Do this, for example when the model goes into an
         * invalid state or after a rebuild (to ensure the menu actions are updated)
         * @private
         */
        _refreshPanel: function() {
            var self = this;
            var panelsView = sas.analytics.riskhp.utils.HprUtil.getSelectedRiskPanelsView();
            panelsView.refreshPanelWithId(self.getId(), false, "RELOAD_ACTIONS");
        },

        /**
         * Rebuilds the components of this view and then re-render's it so it gets updated.
         * @param success {function} callback for when the refresh is completed.
         */
        rebuild: function(success) {
            var self = this;
            return self._rebuild(true, success);
        },

        /**
         * The internal version of this method that does all the work.  When called internally, we are changing the
         * type of plot rendered (and therefore the actual chart instance object).  Because we will have already built
         * the new chart this method exists so we can avoid a needless, second query.  External consumers on the other
         * hand are updating the existing chart object so all they want to do is have the query made and repaint the
         * existing chart object.  So this method facilitates both use cases.  Note: calling this function will put the
         * view in a busy state which will be cleared when the returned deferred is resolved.  There is no need to
         * manage busy state outside of this method when using it.
         * @param {boolean} shouldRequery
         * @param {function} success
         * @param {boolean} forceRefreshOnFailure
         * @returns {*}
         * @private
         */
        _rebuild: function(shouldRequery, success, forceRefreshOnFailure) {
            var self = this,
                deferred = $.Deferred(); // captured in the success and failure callback

            self.getViewLayout().setBusy(true);
//            self.setBusyIndicatorDelay(0);
            self.setBusy(true);
            if (self.chart) {
                self.chart.setBusy(true);
            }
            if (self.table) {
                self.table.setBusy(true);
            }

            // Setup the callback when the query is successful
            var successCallback = function () {
                self.updateChartWithCurrentModel();
                if (success) {
                    success();
                }
                deferred.resolve();
            };

            // Setup the callback when there is a failure
            var failureCallback = function() {
                self.updateChartWithCurrentModel(forceRefreshOnFailure);
                deferred.reject();
            };

            // Execute the query
            if (shouldRequery) {
                var chartModel = self.getChartModel();
                // Deferred will be resolved as a consequence of one of the callback's being called
                chartModel.executeQuery(successCallback, failureCallback);
            }
            else {
                // Success callback will directly resolve the deferred
                successCallback();
            }
            deferred.always(function() {
                self.setBusy(false);
                self.getViewLayout().setBusy(false);
                if (self.chart) {
                    self.chart.setBusy(false);
                }
                if (self.table) {
                    self.table.setBusy(false);
                }
            });

            return deferred;
        },

        /**
         * Updates the view with the current model
         * @param {boolean} forceRefresh
         */
        updateChartWithCurrentModel: function(forceRefresh) {
            var self = this,
                chartModel = self.getChartModel();

            // Only rebuild the chart and table if the model is valid
            if (self.isModelValid() || forceRefresh) {
                if (self.chart) {
                    // If we already have a chart then it is possible there is also a table so go ahead and build both
                    self.chart.rebuildChart(chartModel);
                    self.table = self._buildTable();
                }
                else {
                    self.chart = this._buildChart();
                    // If there was no chart object then it is presumed we are building the chart from scratch and
                    // since the table is seen as a secondary view, it will be built on demand.
                }
            }

            // Regardless of whether the model is valid or not, always rebuild the toolbar
            var toolbar = self._buildPathToolbar();
            var newTopToolbar = null;
            if (toolbar) {
                newTopToolbar = new sas.hc.ui.commons.layout.BorderLayoutArea({
                    overflowX: "hidden",
                    overflowY: "hidden",
                    size: "2.0rem",
                    content: [toolbar]
                });
            }
            self.getViewLayout().setTop(newTopToolbar);

            // Re-render the center area
            var currentContent = self._currentContent();
            var newContentArea = new sas.hc.ui.commons.layout.BorderLayoutArea({
                overflowX: "hidden",
                overflowY: "hidden",
                content: [currentContent]
            });
            self.getViewLayout().setCenter(newContentArea);
            self.rerender();

            self._refreshPanel();
        },

        /**
         * Called from HPRiskPanelsViewDelegate.getContentMetadata() when saving the Plot/Chart.
         * If the 'chart' exists, use it's getQueryData method to build a JSON string representing
         * the Plot/Chart.
         * @returns {*} A String representing the saved Plot/Chart or returns an empty Object if
         * the 'chart' does not exist
         */
        getQueryData: function() {
            if (this.chart) {
                return this.chart.getQueryData();
            }
            // For a ZeroState plot/chart, we don't have a chart object but we do have a model, use it to generate the JSON
            // so a ZeroState plot can be saved and re-rendered correctly
            return this.getChartModel().toJSON();
        },

        /**
         * Constructs a Empty/ZeroState layout to display instead of the Chart or Table.  If a string is provided, it is
         * displayed within the ZeroState layout; otherwise, a generic message is displayed.
         * @param invalidReasonString String to display inside the ZeroState; otherwise, a default message will appear
         * @returns {sas.risk.common.widgets.ZeroStateLayout}
         * @private
         */
        _buildZeroState: function(invalidReasonString) {
            var titleToDisplay = invalidReasonString ? invalidReasonString : sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.no.data.default.txt");
            return new sas.risk.common.widgets.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATEGRAPH,
                title: titleToDisplay,
                buttons: []
            });
        },

        /**
         * Builds the chart for the view, using the stored chart model
         * @returns {*} One of the HPR Chart classes (HprBarChart, HprCashFlowChart, HprDensityChart, HprDistributionChart, HprLineChart, HprSeriesChart, HprTimeSeriesChart)
         * @private
         */
        _buildChart: function() {
            var self = this,
                chart = null,
                chartModel = this.getProperty("chartModel"),
                chartType = chartModel.getPlotType(),
                deferred = $.Deferred(); // captured in the success and failure handlers passed to execute query as well as the catch block

//            self.setBusyIndicatorDelay(0);
            self.setBusy(true);
            console.log("Building plot type: " + chartType);
            switch(chartType) {
                case HprConstants.PlotType.Density:
                    chart = new HprDensityChart({});
                    break;
                case HprConstants.PlotType.Comparison:
                    if (chartModel.isByTypeHorizon()) {
                        chart = new HprTimeSeriesChart({});
                    }
                    else if (HprChartUtils.isMultiSeriesHorizonPlot(chartModel.getPlotTypeObject())) {
                        chart = new HprTimeSeriesChart({});
                    }
                    else {
                        switch (chartModel.getHowType()) {
                            case HprConstants.HowType.Line:
                                chart = new HprLineChart({});
                                break;
                            default:
                                chart = new HprBarChart({});
                        }
                    }
                    break;
                case HprConstants.PlotType.Distribution:
                    chart = new HprDistributionChart({});
                    break;
                case HprConstants.PlotType.CashFlow:
                    chart = new HprCashFlowChart({});
                    break;
                case HprConstants.PlotType.RiskFactorInformationMeasures:
                    chart = new HprBarChart({});
                    break;
                default:
                    break;
            }
            if (chart) {
                chart.setBusy(true);
                var jsonModel = new sap.ui.model.json.JSONModel(chartModel);
                chart.setModel(jsonModel, "chartModel");

                // callback when the query execution is successful
                var successCallback = function() {
                    try {
                        chart.rebuildChart(chartModel);
                    }
                    catch (e) {
                        // For now e is the raw message from where ever it originated.  Can also pass null to get a vanilla error message.
                        console.error(e);
                        var stringToShowUser = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.no.data.default.txt");
                        self.chart = self._buildZeroState(stringToShowUser); // TODO: Replace invalid reason string with something more user friendly
                        self.rerender();
                    }
                    deferred.resolve();
                };

                // callback when the query execution fails
                var failureCallback = function() {
                    self.updateChartWithCurrentModel();
                    deferred.resolve();
                };

                // Now execute the query
                try {

                    chartModel.executeQuery(successCallback, failureCallback);
                }
                catch (e) {
                    // For now e is the raw message from where ever it originated.  Can also pass null to get a vanilla error message.
                    console.error(e);
                    var stringToShowUser = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.no.data.default.txt");
                    chart = self._buildZeroState(stringToShowUser); // TODO: Replace invalid reason string with something more user friendly
                    self.rerender();
                    deferred.resolve();
                }

                deferred.always(function() {
                    self.setBusy(false);
                    chart.setBusy(false);
                });
                return chart;
            }

            return new sas.risk.common.panelsview.DivWrapper({
                width: "100%",
                height: "100%"
            }).addStyleClass('testDivRed');
        },


        /**
         * Builds a table representation based on the state of the chartModel
         * @returns {WideTable}
         * @private
         */
        _buildTable: function() {
            var tableDataTuple;
            // Use the 'chartModel' stored with the component, chartmodel is assumed to be populated with data
            var chartModel = this.getProperty("chartModel");

            var chartType = chartModel.getPlotType();
            //var values = [];

            switch(chartType) {
                case HprConstants.PlotType.Comparison:
                    tableDataTuple = sas.analytics.riskhp.common.riskcommons.plot.HprChartTableUtils.buildComparisonPlotTableData(chartModel);
                    break;
                case HprConstants.PlotType.Distribution:
                    tableDataTuple = sas.analytics.riskhp.common.riskcommons.plot.HprChartTableUtils.buildDistributionPlotTableData(chartModel);
                    break;
                case HprConstants.PlotType.CashFlow:
                    tableDataTuple = sas.analytics.riskhp.common.riskcommons.plot.HprChartTableUtils.buildCashFlowPlotTableData(chartModel);
                    break;
                case HprConstants.PlotType.RiskFactorInformationMeasures:
                    tableDataTuple = sas.analytics.riskhp.common.riskcommons.plot.HprChartTableUtils.buildRFIMPlotTableData(chartModel);
                    break;
                case HprConstants.PlotType.Density:
                    tableDataTuple = sas.analytics.riskhp.common.riskcommons.plot.HprChartTableUtils.buildDensityPlotTableData(chartModel);
                    break;
                default:
                    break;
            }

            // Let's try the wide table for a bit to see if it will work
            //var table = new sas.hc.x.WideTable({
            var table = new sas.hc.ui.table.Table({
                //width: "800px",
                showSelectionControls: false,
                showTableOptions: false,
                enableColumnReordering: false,
                enableColumnFreeze: false,
                showColumnVisibilityMenu: false,
                selectionMode: sas.hc.ui.table.SelectionMode.None,
                visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
                minAutoRowCount: 1, // required for scrolling with < 5 rows
                allowReflow: false,
                columns: tableDataTuple.columns
            });

            var oModel = new sap.ui.model.json.JSONModel({ values:tableDataTuple.values });
            table.setModel(oModel);
            table.bindRows("/values");
            return table;
        },


        /**
         * Toggles whether the chart or the table is shown
         * @private
         */
        _toggleSelectedMode: function() {
            if (this.getSelectedMode() === "chart") {
                this.setProperty("selectedMode", "table");
            }
            else {
                this.setProperty("selectedMode", "chart");
            }
            this.updateChartWithCurrentModel();
        },

        /**
         * Notifies the system that the chart state is dirty and therefore the exploration should be saved to retain changes
         * @private
         */
        _fireDirty: function() {
            // Publish an event that will update the dirtyState of the Exploration view
            sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {dirty: true});
        },

        _onCrosstabDeleted: function(channel, event, data) {
            var self = this;
            var chartModel = this.getProperty("chartModel");
            if (chartModel.handleCrosstabDeleted(data.id)) {
                 var deferred = self._rebuild(true);
                  deferred.always(function() {
                      // Refresh the panel so the toolbar actions get rebuilt
                      self._refreshPanel();

                      sas.analytics.riskhp.utils.HprUtil.updateRightPanelsForSelectedPlot(
                          chartModel.getId(),
                          HprConstants.HPR_OPTIONS_PANE_PLOT_TYPE,
                          HprConstants.HPR_FILTERS_PANE_PLOT_TYPE,
                          true);

                      // Change the exploration's "dirty" state
                      $.proxy(self._fireDirty(),self);
                  });
            }
        },

        /**
         * @description Handles the action of de-linking or relinking a plot to it's owning crosstab
         * @param {boolean} linked - True if you want to relink, false otherwise
         * @private
         */
        _handleLinkToCrosstab: function(linked) {
            var self = this;
            var chartModel = this.getProperty("chartModel");
            chartModel.setLinkedToCrosstab(linked);

            if (linked === true) {
                chartModel.handleRelinkToCrosstab();
            }

            // Always want the right panel to refresh when linking/de-linking to crosstab
            chartModel.shouldRefreshRightPanel = true;

            // Rebuild the view, per defect S1424870 we should not requery if we are delinking
            var deferred = self._rebuild(linked);

            deferred.always( function(){
                // Refresh the panel so the toolbar actions get rebuilt
                self._refreshPanel();

                // Force the right plot options panel to refresh as enablement probably has been affected.
                var explorationPanesStateModel = sap.ui.getCore().getModel(HprConstants.MODEL_ID_EXPLORATION_PANES_STATE);

                explorationPanesStateModel.setProperty("/panes/filtersPane/componentsEnabled", !linked);

                sas.analytics.riskhp.utils.HprUtil.updateRightPanelsForSelectedPlot(chartModel.getId(),
                    HprConstants.HPR_OPTIONS_PANE_PLOT_TYPE,
                    HprConstants.HPR_FILTERS_PANE_PLOT_TYPE,
                    true);

                // Change the exploration's "dirty" state
                $.proxy(self._fireDirty(),self);
            });
            //$.proxy(self._rebuild(linked),self);


        },

        getToolbarActions: function() {
            var self = this,
                buttons = [],
                tooltip,
                button;

            var chartModel = this.getProperty("chartModel");
            tooltip = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.linked.generic.tooltip.txt");
            var crosstabName = HprChartUtils.getLinkedCrosstabName(chartModel);
            if (crosstabName) {
                tooltip = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.linked.tooltip.fmt", [crosstabName]);
            }
            if (chartModel.isLinkedToCrosstab()) {
                button = new sas.hc.m.Button({
                    //id: sBasicId + '-download',
                    icon: sas.icons.HC.LINK,
                    tooltip: tooltip,
                    press: function() {
                        self._handleLinkToCrosstab(false);
                    }
                });
                buttons.push(button);
            }
            else {
// The tooltip for use when they change their minds and want relinking back ;-)
//                Unlinked, press to relink with {0}
//                var tooltip = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.unlinked.tooltip.fmt", [crosstabName]);
                tooltip = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.unlinked.tooltip.txt");
                if (!chartModel.canLinkToCrosstab()) {
                    tooltip = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.permanently_unlinked.tooltip.txt", [crosstabName]);
                }
                button = new sas.hc.m.Button({
                    //id: sBasicId + '-download',
                    icon: sas.icons.HC.LINKBROKEN, //sas.icons.HC.DISCONNECT,
                    tooltip: tooltip,
                    enabled: false, // For now, per Katie, we don't need relink (rolleyes)
//                    enabled: chartModel.canLinkToCrosstab(),
                    press: function() {
                        self._handleLinkToCrosstab(true);
                    }
                });
                buttons.push(button);
            }
            return buttons;
        },

        /**
         * Returns the toolbar actions for the view.
         * @returns {Array}
         */
        getOverflowMenuActions: function() {
            var self = this;
            var actions = [];
            var isInTableMode = self.getSelectedMode() === "table";
            var chartModel = this.getProperty("chartModel");
            var supportsBothBarAndLinePlots = chartModel.supportsBothBarAndLinePlots();

            if (isInTableMode) {
                // If the view is in table mode and it DOESN'T support both line and bar charts push a menu item to show the chart
                if (! supportsBothBarAndLinePlots) {
                    actions.push({
                        text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.chart.action.txt"),
                        tooltip: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.chart.action.tip"),
                        press: function() {
                            self._toggleSelectedMode();

                            // Change the exploration's "dirty" state
                            $.proxy(self._fireDirty(),self);
                        }
                    });
                }
            }
            else {
                if (self.isModelValid()) {
                    // Push a menu item to show the table since the view is in chart mode
                    actions.push({
                        text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.table.action.txt"),
                        tooltip: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.table.action.tip"),
                        enabled: self.isModelValid,
                        press: function() {
                            $.proxy(self._toggleSelectedMode(),self);

                            // Refresh the panel so the toolbar actions get rebuilt
                            self._refreshPanel();

                            // Change the exploration's "dirty" state
                            $.proxy(self._fireDirty(),self);
                        }
                    });
                }
            }

            // Add actions for bar and line charts when both are supported
            if (supportsBothBarAndLinePlots && self.isModelValid()) {
                // If both bar and lines are supported then at this point no (show line/bar) menu option is in the actions
                // so add one or both depending on what view mode the View is in.
                var howType = chartModel.getHowType();
                if (isInTableMode || howType === HprConstants.HowType.Line) {
                   actions.push(
                       {
                           text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.bar.action.txt"),
                           tooltip: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.bar.action.tip"),
                           press: function() {

                               // Make sure we are showing the chart
                               self.setProperty("selectedMode", "chart");

                               // Change type of plot to show
                               chartModel.setDisplayAsBar();

                               // Build the chart object based on the new state of the model
                               self.chart = self._buildChart();

                               // Rebuild the view but don't requery as _buildChart already did that
                               var deferred = self._rebuild(false);
                               deferred.always(function(){
                                   // Change the exploration's "dirty" state
                                   $.proxy(self._fireDirty(),self);
                               });
//                               $.proxy(self._rebuild(false),self);

                           }
                       }
                   );
                }
                if (isInTableMode || howType === HprConstants.HowType.Bar) {
                    actions.push(
                        {
                            text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.line.action.txt"),
                            tooltip: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.line.action.tip"),
                            press: function() {

                                // Make sure we are showing the chart
                                self.setProperty("selectedMode", "chart");

                                // Change type of plot to show
                                chartModel.setDisplayAsLine();

                                // Build the chart object based on the new state of the model
                                self.chart = self._buildChart();

                                // Rebuild the view but don't requery as _buildChart already did that
                                var deferred = self._rebuild(false);
                                deferred.always(function(){
                                    // Change the exploration's "dirty" state
                                    $.proxy(self._fireDirty(),self);
                                });
//                                $.proxy(self._rebuild(false),self);

                            }
                        }
                    );
                }
            }

            if (!isInTableMode && self.isModelValid()) {
                // Push a menu item as the last one (grouped by itself) to export the graph
                actions.push({
                    text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.exportToPng.action.txt"),
                    tooltip: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.exportToPng.action.tip"),
                    startsSection: true,
                    press: function() {
                        var outputFileName = "chart";
                        var chartId = self.getId();
                        if (!sas.risk.common.utils.CommonUtil.exportGraphToPNG(chartId, outputFileName)) {
                            // warn graph export failed
                            sas.hc.m.MessageDialog.error({
                                title: sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.exportToPng.action.txt'),
                                text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.plot.exportToPng.error.txt')
                            });
                        }
                    }
                });
            }

            return actions;
        }
    });
})();