jQuery.sap.declare("sas.analytics.riskhp.common.riskcommons.plot.HprLineChart");
(function() {
    "use strict";
    jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils");
    jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprAbstractChart");

    // LTJS common requires
    jQuery.sap.require("sap.ui.core.Control");
    jQuery.sap.require("sas.ltjs.commons.models.data.dataset.Column");
    jQuery.sap.require("sas.ltjs.commons.models.data.dataset.DataSet");
    jQuery.sap.require("sas.ltjs.commons.models.data.dataset.DataType");
    jQuery.sap.require("sas.ltjs.gtl.models.Graph");
    jQuery.sap.require("sas.ltjs.gtl.models.data.AnalysisRole");
    jQuery.sap.require("sas.ltjs.gtl.models.data.ClassificationRole");
    jQuery.sap.require("sas.ltjs.gtl.models.data.DataSetDataProvider");
    jQuery.sap.require("sas.ltjs.gtl.models.layoutOverlay.LayoutOverlay");
    jQuery.sap.require("sas.ltjs.ltjsLoader");

    // Line chart requires
    jQuery.sap.require("sas.ltjs.gtl.models.axis.AxisOptions");
    jQuery.sap.require("sas.ltjs.gtl.models.layoutOverlay.lineChart.LineChart");
    jQuery.sap.require("sas.ltjs.gtl.models.legend.DiscreteLegend");
    jQuery.sap.require("sas.ltjs.gtl.models.layoutOverlay.lineChart.LineChartDisplay");
    jQuery.sap.require("sas.ltjs.gtl.models.axis.DiscreteAxisOptions");
    jQuery.sap.require("sas.ltjs.gtl.models.axis.AxisTickValueFitPolicy");


    // Expose ltjs objects we want to us without having to type the full namespace
    var HprChartUtils = sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils;
    var Column = sas.ltjs.commons.models.data.dataset.Column;
    var Graph = sas.ltjs.gtl.models.Graph;
    var AnalysisRole = sas.ltjs.gtl.models.data.AnalysisRole;
    var ClassificationRole = sas.ltjs.gtl.models.data.ClassificationRole;
    var DataSetDataProvider = sas.ltjs.gtl.models.data.DataSetDataProvider;
    var LayoutOverlay = sas.ltjs.gtl.models.layoutOverlay.LayoutOverlay;
    /** @name sas.ltjs.gtl.models.layoutOverlay.lineChart */
    var LineChart = sas.ltjs.gtl.models.layoutOverlay.lineChart.LineChart;
    /** @name sas.ltjs.gtl.models.layoutOverlay.lineChart.LineChartDisplay */
    var LineChartDisplay = sas.ltjs.gtl.models.layoutOverlay.lineChart.LineChartDisplay;
    var DiscreteAxisOptions = sas.ltjs.gtl.models.axis.DiscreteAxisOptions;
    var AxisTickValueFitPolicy = sas.ltjs.gtl.models.axis.AxisTickValueFitPolicy;
    /** @name sas.ltjs.commons.models.data.dataset.DataSet */
    var DataSet = sas.ltjs.commons.models.data.dataset.DataSet;
    var DataType = sas.ltjs.commons.models.data.dataset.DataType;

    // Require our hprisk ltjs override objects
    jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprChartModel");
    jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprGraphView");

    /**
     * Renders a line chart the HPRisk way.
     *
     * @name sas.analytics.riskhp.common.riskcommons.plot.HprLineChart
     */
    sas.analytics.riskhp.common.riskcommons.plot.HprAbstractChart.extend("sas.analytics.riskhp.common.riskcommons.plot.HprLineChart", {
        metadata: {
        },

        renderer: { }, // Use the base class renderer

        /**
         * Given a model builds and returns the correct graph object
         * @param chartModel
         * @returns {sas.ltjs.gtl.models.Graph}
         * @private
         */
        _buildGraph: function(chartModel) {
            var graph = new Graph();

            // Make sure blank values are included.
            graph.setIncludeMissingDiscrete(true);

            // set the title.  OBE: For now done in view
            // HprChartUtils.setGraphPathTitle(chartModel, graph);

            /** @typedef {sas.ltjs.commons.models.data.dataset.DataSet} dataSet */
            var dataSet = new DataSet(true, true);

            var charts = [];
            _.each(chartModel.categoryRoles, function(categoryRole, chartIndex){

                // Create category (think X values grouping) for this particular line chart
                var categoryRoleName = HprChartUtils.getCategoryRoleName(categoryRole, chartModel);
                var categoryColumn = new Column(); // X labels
                categoryColumn.setName(categoryRoleName);

                // For now we assume there is only one formatter per responseRole set
                var responseRoleFormatter = chartModel.getResponseRoleFormatter(chartIndex);

                // Category labels are the labels across the X-axis for the values that will be plotted
                var xValues = []; // Really is bucket groupings
                _.each(chartModel.getSeriesIds(chartIndex), function(seriesId){
                    var seriesObject = chartModel.getSeriesObject(seriesId, chartIndex);
                    // We have to push a "placeholder" into the x values array so the labels will match with the y values
                    // they are accumulated (so we ignore the actual value in the returned array from the getValues() call
                    _.each(seriesObject.getValues(), function() {
                        xValues.push(seriesObject.getName(chartModel.dataSource, chartModel.showLabels()));
                    });
                });
                HprChartUtils.addValuesToColumn(categoryColumn, xValues, responseRoleFormatter);
                dataSet.addColumn(categoryColumn);

                // Accumulate the y values values
                var yValues = [];
                _.each(chartModel.getSeriesIds(chartIndex), function(seriesId){
                    var seriesObject = chartModel.getSeriesObject(seriesId, chartIndex);
                    yValues.push(seriesObject.values[0]);
                });

                // Build the response column (think Y values) for this particular line chart
                var responseRole = chartModel.getResponseType(chartIndex);
                var responseRoleName = HprChartUtils.getResponseRoleName(chartModel, responseRole);

                // The response values are the actual Y values and match up with the categoryLabels above
                // We are expecting only one series at this time, here is where we loop over the series ids to generate multiple bars (see HprTimeSeriesChart)
                var responseColumn = new Column(); // Y values
                responseColumn.setName(responseRoleName);
                HprChartUtils.addValuesToColumn(responseColumn, yValues, responseRole);
                dataSet.addColumn(responseColumn);

                // Set up the line chart
                var lineChart = new LineChart();
                var classificationRole = new ClassificationRole(categoryRoleName);
                lineChart.setCategory(classificationRole);

                var analysisRole = new AnalysisRole(responseRoleName);
                lineChart.setResponse([analysisRole]);

                var lineChartDisplay = new LineChartDisplay();
                lineChartDisplay.setMarkers(false);
                lineChartDisplay.setLine(true);
                lineChart.setDisplay(lineChartDisplay);

                lineChart.setDataProvider(new DataSetDataProvider(dataSet));

                charts.push(lineChart);
            });

            var layout = new LayoutOverlay();

            // Build the x-axis options
            var xAxisOptions = HprChartUtils.getDefaultXAxisOptions();
            var xDiscreteAxisOptions = new DiscreteAxisOptions();
            xDiscreteAxisOptions.setTickValueFitPolicy(AxisTickValueFitPolicy.ROTATETHIN);
            xAxisOptions.setDiscreteOptions(xDiscreteAxisOptions);
            layout.setXAxisOptions(xAxisOptions);

            // Build the y-axis options
            var yAxisOptions = HprChartUtils.getDefaultYAxisOptions();
            layout.setYAxisOptions(yAxisOptions);

            layout.setOverlays(charts);
            graph.setLayout(layout);
            return graph;
        },

        /**
         * Builds a sample version of this type of graph so we can try things out without the overhead of parsing the
         * data from the server. Typically this is based off of the relevant Honeycomb example
         * @returns {sas.ltjs.gtl.models.Graph}
         * @private
         */
        _buildSample: function() {
            var graph = new Graph(),
                layout = new LayoutOverlay(),
                yAxisOpts = layout.getYAxisOptions(),
                lineChart1 = new LineChart(),
//                lineChart2 = new LineChart(),
                lineChartDisplay = new LineChartDisplay(),
                dataSet = new DataSet(true, true),
                column1 = new Column(),
                column2 = new Column(),
                column3 = new Column(),
                col1Role = new ClassificationRole("Quarter"),
                col2Role = new AnalysisRole("Revenue"),
                col3Role = new AnalysisRole("Predecessor's Revenue");
//                discreteLegend = new sas.ltjs.gtl.models.legend.DiscreteLegend();

            graph.setIncludeMissingDiscrete(true);

            // add the data to the columns and then put the columns in the dataSet
            column1.setName("Quarter");
            column1.setStrings([" ", "Q2", "Q3", "Q4"]);
            column1.setDataType(DataType.STRING);
            dataSet.addColumn(column1);

            column2.setName("Revenue");
            column2.setNumbers([46, 32, 36, 43]);
            column2.setDataType(DataType.NUMBER);
            dataSet.addColumn(column2);

            column3.setName("Predecessor's Revenue");
            column3.setNumbers([50, 32, 34, 40]);
            column3.setDataType(DataType.NUMBER);
            dataSet.addColumn(column3);

            // set the label on the Y axis
            yAxisOpts.setLabel("Revenue Percentage");

            // setup the line charts
            lineChart1.setCategory(col1Role);
            lineChart1.setResponse([col2Role, col3Role]);

            lineChartDisplay.setMarkers(true);
            lineChartDisplay.setLine(true);
            lineChart1.setDisplay(lineChartDisplay);

            // add the line chart to the layout and then set the layout on the graph
            layout.setOverlays([lineChart1]);

            graph.setLayout(layout);

            // assign the data provider to the line charts
            lineChart1.setDataProvider(new DataSetDataProvider(dataSet));
            return graph;
        }
    });
})();