/*
 * '(c) 2018, SAS Institute Inc.'
 */

// (c) 2015, SAS Institute Inc.

jQuery.sap.declare("sas.analytics.riskhp.common.riskcommons.plot.SeriesObject");
jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils");

/**
 * @name sas.analytics.riskhp.common.riskcommons.plot.SeriesObject
 * @extends sap.ui.base.Object
 */
sap.ui.base.Object.extend("sas.analytics.riskhp.common.riskcommons.plot.SeriesObject", {
    constructor: function(dataItemIds, isCashflow) {
        this.dataItemIds = dataItemIds;
        this.values = [];
        if (_.isBoolean(isCashflow)) {
            this.cashflowSeries = isCashflow;
        }
        else {
            this.cashflowSeries = false;
        }

        this.internalName = "";
        this.internalLabel = "";
        this.internalSortValue = null;
        this.id = sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils.generateUniqueSeriesIdFromIds(dataItemIds);
    }
});

sas.analytics.riskhp.common.riskcommons.plot.SeriesObject.prototype.getDataItemIds = function() {
    return this.dataItemIds;
};

/**
 * Convert the list of DataItem ids held in this object into their DataItem representations.  Note: if an id cannot
 * be found in the given dataSource it will be skipped.
 * @param {DataSource} dataSource
 * @returns {DataItem[]}
 */
sas.analytics.riskhp.common.riskcommons.plot.SeriesObject.prototype.getDataItems = function(dataSource) {
    var dataItems = [];
    // Essentially what we do here is walk the list of data item ids held and return the real list of
    // data items (without any that could not ve found).  Note if some ids are not found in the datasource
    // we probably have a problem.
    _.each(this.dataItemIds, function(id){
        var dataItem = dataSource.getDataItemById(id);
        if (dataItem) {
            dataItems.push(dataItem);
        }
    });
    return dataItems;
};

sas.analytics.riskhp.common.riskcommons.plot.SeriesObject.prototype.getId = function() {
    return this.id;
};

sas.analytics.riskhp.common.riskcommons.plot.SeriesObject.prototype.getValues = function() {
    return this.values;
};

sas.analytics.riskhp.common.riskcommons.plot.SeriesObject.prototype.addValue = function(value) {
    this.values.push(value);
};

sas.analytics.riskhp.common.riskcommons.plot.SeriesObject.prototype.setValues = function(values) {
    this.values = values;
};

sas.analytics.riskhp.common.riskcommons.plot.SeriesObject.prototype.isCashflowSeries = function() {
    return this.cashflowSeries;
};

sas.analytics.riskhp.common.riskcommons.plot.SeriesObject.prototype.setCashflowSeries = function(value) {
    this.cashflowSeries = value;
};

sas.analytics.riskhp.common.riskcommons.plot.SeriesObject.prototype.setName = function(value) {
    this.internalName = value;
};

sas.analytics.riskhp.common.riskcommons.plot.SeriesObject.prototype.setLabel = function(value) {
    this.internalLabel = value;
};

/**
 * Returns the displayable name of the SeriesObject
 * @param {DataSource} dataSource
 * @param {boolean} showLabel - whether the label or name should be shown, true for showing label, defaults to true
 * @returns {*}
 */
sas.analytics.riskhp.common.riskcommons.plot.SeriesObject.prototype.getName = function(dataSource, showLabel) {
    var shouldShowLabel = true;
    if (!_.isUndefined(showLabel) && _.isBoolean(showLabel)) {
        shouldShowLabel = showLabel;
    }
    if (this.internalName && this.internalName.length > 0) {
        if (shouldShowLabel && this.internalLabel && this.internalLabel.length > 0) {
            return this.internalLabel;
        }
        return this.internalName;
    }
    var dataItems = this.getDataItems(dataSource);
    if (dataItems.length === 1) {
        return dataItems[0].getDisplayLabel(shouldShowLabel);
    }
    var name = "";
    _.each(dataItems, function(dataItem){
        if (name.length === 0) {
            name = dataItem.getDisplayLabel(shouldShowLabel);
        }
        else {
            name = name + "/" + dataItem.getDisplayLabel(shouldShowLabel);
        }
    });
    if (name.length === 0) {
        return "Unknown";
    }
    return name;
};

sas.analytics.riskhp.common.riskcommons.plot.SeriesObject.prototype.setSortValue = function(value) {
    this.internalSortValue = value;
};

sas.analytics.riskhp.common.riskcommons.plot.SeriesObject.prototype.getSortValue = function(dataSource) {
    if (this.internalSortValue) {
        return this.internalSortValue;
    }
    var dataItems = this.getDataItems(dataSource);
    if (dataItems.length === 1) {
        var dataItem = dataItems[0];
        if (sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils.isDataItemHorizonType(dataItem)) {
            return dataItem.getHorizonValue();
        }
        if (_.isUndefined(dataItem.format)) {
            return this.getName(dataSource, false);
        //    return dataItem.get
        }
        return dataItem.getvalue();
    }
    return this.getName(dataSource, false);
};

