// (c) 2018, SAS Institute Inc.

jQuery.sap.declare("sas.analytics.riskhp.common.riskcommons.table.TableViewer");

jQuery.sap.require("sas.hc.ui.commons.dualselector.DualSelector");
jQuery.sap.require("sas.hc.ui.layout.HorizontalLayout");
jQuery.sap.require("sas.hc.ui.layout.VerticalLayout");
jQuery.sap.require("sas.hc.ui.model.PageableJSONModel");
jQuery.sap.require("sas.hc.x.WideTable");

jQuery.sap.require("sas.ltjs.sasicu.util.SASFormatterUtil");

jQuery.sap.require("sas.hc.m.Dialog");
jQuery.sap.require("sas.hc.m.Heading");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.List");
jQuery.sap.require("sas.hc.m.Menu");
jQuery.sap.require("sas.hc.m.MenuItem");
jQuery.sap.require("sas.hc.m.RadioButton");
jQuery.sap.require("sas.hc.m.RadioButtonGroup");
jQuery.sap.require("sas.hc.m.SearchField");
jQuery.sap.require("sas.hc.m.StandardListItem");
jQuery.sap.require("sas.hc.m.ResponsiveToolbar");
jQuery.sap.require("sas.hc.m.ToolbarSeparator");
jQuery.sap.require("sas.hc.m.ToolbarSpacer");

jQuery.sap.require("sas.analytics.riskhp.settings.ApplicationSettings");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Classvar");
jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.table.TableViewerColumn");
jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.table.TableViewerModel");
jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.table.TableViewerColumnMenu");
jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.table.TableViewerUtil");
jQuery.sap.require("sas.analytics.riskhp.utils.HprAPI");
jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");

jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.risk.common.panelsview.DivWrapper");
jQuery.sap.require("sas.risk.common.utils.CommonUtil");
jQuery.sap.require("sas.risk.common.widgets.RiskColumnText");
jQuery.sap.require("sas.risk.common.widgets.ZeroStateLayout");

jQuery.sap.require("sas.ltjs.sasicu.util.CalendarUtil");
jQuery.sap.require("sas.ltjs.sasicu.util.SASFormatterUtil");

/**
 * @name sas.analytics.riskhp.common.riskcommons.table.TableViewer
 */
sap.ui.core.Control.extend("sas.analytics.riskhp.common.riskcommons.table.TableViewer", {

    metadata: {
        properties: {
            width: {type: "sap.ui.core.CSSSize", defaultValue: null},
            height: {type: "sap.ui.core.CSSSize", defaultValue: null},
            tableViewerModel: {type: "sas.analytics.riskhp.common.riskcommons.table.TableViewerModel"}
        },
        events: {
            dirtyChange: {}
        }
    },

    ApplicationSettings: sas.analytics.riskhp.settings.ApplicationSettings,
    CommonUtil: sas.risk.common.utils.CommonUtil,
    HprAPI: sas.analytics.riskhp.utils.HprAPI,
    HprConstants: sas.analytics.riskhp.utils.HprConstants,
    HPRiskModelManager: sas.analytics.riskhp.models.HPRiskModelManager,
    HprUtil: sas.analytics.riskhp.utils.HprUtil,
    TableViewerUtil: sas.analytics.riskhp.common.riskcommons.table.TableViewerUtil,
    CLASSVAR: sas.analytics.riskhp.models.dataitems.Classvar.CLASSVAR,

    wrapper: null,
    table: null,
    tableLayout: null,
    tableColumnSet: [],
    tableModel: null,
    zeroStateLayout: null,
    pageSizeChangeDetected: false,
    pageSize: 10000000,

    renderer: function(rm, control) {
        if(!control.getVisible()) return;

        rm.write("<div");
        rm.writeControlData(control);
        if(control.getWidth()) {
            rm.addStyle("width", control.getWidth());
        }
        if(control.getHeight()) {
            rm.addStyle("height", control.getHeight());
        }

        rm.writeStyles();
        rm.writeClasses();
        rm.write(">");

        // render the details table
        if (!_.isUndefined(control.tableWrapper) && control.tableWrapper) {
            control.addDependent(control.tableWrapper);
            rm.renderControl(control.tableWrapper);
        }

        rm.write("</div>");
    },

    constructor: function(id, mSettings) {
        // construct super
        sap.ui.core.Control.call(this, id, mSettings);

        // declare local vars
        var self = this;
        var tableViewerModel = self.getTableViewerModel();
        self.dataObject = mSettings.dataObject;

        // mark the control as busy
        self.setBusy(true);

        self._delinkLikeTables();

        // We need the wrapper so we can properly contain the table by positioning
        // it absolutely without layering on top of the popout menu for the panel
        // (the swizzle).
        self.tableWrapper = new sas.risk.common.panelsview.DivWrapper({
            height: "100%",
            width: "100%",
            content: self._createTable()
        });
        self.tableWrapper.addStyleClass("hprBoundedControlContainer");

        // If the model is not valid there is not much we can do, so display the ZeroStateLayout
        if (!self.isModelValid()) {
            self.setBusy(false);
            self._createZeroStateLayout(self.HprUtil.getBundle().getText("HPRisk." + tableViewerModel.type + ".zeroState.query.problem.txt"));
            return;
        }

        // load the data source (required for column creation, so make sure it happens before we bind columns)
        tableViewerModel.initializeModel().done(
            function () {
                self.restoringTableState = true;

                // now that the dataSource is in the model, update path label
                self._updatePathLabel();

                // add the table columns
                self._createTableColumns(tableViewerModel.dataSetColumns);

                // configure the table model
                self.table.setModel(self._createTableDataModel());
                self.table.bindRows("/data");

                // restore sort order
                var sortColumn = tableViewerModel.getSortColumn();
                if (sortColumn) {
                    var columnId = _.find(self.table.getColumns(), function (column) {
                        var columnId = _.find(column.getCustomData(), function (customData) {
                            return customData.getKey() === "columnId";
                        }).getValue();

                        return columnId === sortColumn.columnId;
                    });
                    if (columnId === undefined)
                        columnId = sortColumn.columnId;

                    self.table.sort(columnId, sortColumn.sortOrder, false);
                }

                // rerender the table to show changes
                self._rerenderTable();

                self.restoringTableState = false;
            }
        ).fail(
            function (jqxhr, textStatus, error) {
                // No metadata, display the ZeroStateLayout
                //TODO notify user instead of log?
                console.log("Table metadata failed to load: " + jqxhr);
                self.setBusy(false);
                self._createZeroStateLayout(self.HprUtil.getBundle().getText("HPRisk." + tableViewerModel.type + ".zeroState.query.problem.txt"));
            }
        );

        // Subscribe to event that is fired when double click is detected in a crosstab
        sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_LINKED_PLOTS, self._onUpdateLinkedTable, self);

        // Subscribe to event that is fired when a Crosstab is deleted
        sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_CROSSTAB_DELETED, self._onCrosstabDeleted, self);

        // Subscribe to an event that is fired when the label for a Data Item is updated
        sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_DATA_ITEM_LABEL_UPDATED, self._onDataItemLabelUpdate, self);

        // Subscribe to an event that is fired when the 'Show labels' option field was updated
        sas.eventBus.subscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_TABLE_VIEWER_OPTIONS_SHOW_LABELS_UPDATED, self._onShowLabelsChange, self);
    },

    exit: function() {
        var self = this;
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_DATA_ITEM_LABEL_UPDATED, self._onDataItemLabelUpdate, self);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_LINKED_PLOTS, self._onUpdateLinkedTable, self);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_CROSSTAB_DELETED, self._onCrosstabDeleted, self);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_TABLE_VIEWER_OPTIONS_SHOW_LABELS_UPDATED, self._onShowLabelsChange, self);
    },

    /**
     * Notifies the system that the chart state is dirty and therefore the exploration should be saved to retain changes
     * @private
     */
    _fireDirty: function() {
        // Publish an event that will update the dirtyState of the Exploration view
        sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {dirty: true});
    },

    // public functions

    /**
     * Called from HPRiskPanelsViewDelegate.getContentMetadata() when saving the control.
     * @returns {Object} the query data for this control
     */
    getQueryData: function() {
        return this.getTableViewerModel().toJSON();
    },

    /**
     * Returns the toolbar actions for the view.
     * @returns {Array}
     */
    getToolbarActions: function() {
        var self = this,
            buttons = [],
            tooltip,
            button;

        var tableViewerModel = self.getTableViewerModel();
        var tableType = tableViewerModel.type;

        tooltip = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk." + tableType + ".linked.generic.tooltip.txt");
        var crosstabName = self.TableViewerUtil.getLinkedCrosstabName(tableViewerModel);
        if (crosstabName) {
            tooltip = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk." + tableType + ".linked.tooltip.fmt", [crosstabName]);
        }
        if (tableViewerModel.isLinkedToCrosstab()) {
            button = new sas.hc.m.Button({
                //id: sBasicId + '-download',
                icon: sas.icons.HC.LINK,
                tooltip: tooltip,
                press: function() {
                    self._handleLinkToCrosstab(false);
                }
            });
            buttons.push(button);
        }
        else {
            // The tooltip for use when they change their minds and want relinking back ;-)
            // Unlinked, press to relink with {0}
            tooltip = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk." + tableType + ".unlinked.tooltip.txt");
            if (!tableViewerModel.canLinkToCrosstab()) {
                tooltip = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk." + tableType + ".permanently_unlinked.tooltip.txt", [crosstabName]);
            }
            button = new sas.hc.m.Button({
                //id: sBasicId + '-download',
                icon: sas.icons.HC.LINKBROKEN, //sas.icons.HC.DISCONNECT,
                tooltip: tooltip,
                //enabled: false,  //FIXME  mark astin temp?
                press: function() {
                    self._handleLinkToCrosstab(true);
                }
            });
            buttons.push(button);
        }
        return buttons;
    },

    /**
     * Returns the toolbar actions for the view.
     * @returns {Array}
     */
    getOverflowMenuActions: function() {
        var self = this;
        return [
            // {
            //     text: this.HprUtil.getBundle().getText("HPRisk." + self.getTableViewerModel().type + ".OptionsMenu.Export.txt"),
            //     tooltip: this.HprUtil.getBundle().getText("HPRisk." + self.getTableViewerModel().type + ".OptionsMenu.Export.txt"),
            //     startsSection: true,
            //     press: $.proxy(this._handleExportToExcel, this),
            //     enabled : false,
            // }
        ];
    },

    handleResize: function() {
        this._rerenderTable();
    },

    /**
     * Requeries the data for the table.
     */
    rebuild: function() {
        // save the deferred on the tableViewer so it can be resolved/rejected after the model query returns
        this.rebuildDeferred = $.Deferred();

        this.table.getModel().clearData();
        if (this.getTableViewerModel().isValid()) {
            this._rerenderTable();
        }
        this._updateTitleAndPath();

        return this.rebuildDeferred;
    },


    // private functions

    _updateTitleAndPath: function() {
        this._updatePathLabel();
        this._updatePanelTitle();
        this._refreshPanel(this.getTableViewerModel().id);
    },

    /**
     * Force the owning panel to be refreshed for this view.  Do this, for example when the model goes into an
     * invalid state or after a rebuild (to ensure the menu actions are updated)
     * @private
     */
    _refreshPanel: function(id) {
        var self = this;
        var panelsView = sas.analytics.riskhp.utils.HprUtil.getSelectedRiskPanelsView();
        panelsView.refreshPanelWithId(id, false, "RELOAD_ACTIONS");
    },

    _updatePathLabel: function() {
        this.pathLabel.setText(this.getTableViewerModel().createPathString());
    },

    _updatePanelTitle: function() {
        var title = sas.analytics.riskhp.common.riskcommons.table.TableViewerUtil.generateDefaultTableTitle(this.getTableViewerModel().dataSource, this.getTableViewerModel());
        this.getTableViewerModel().setOptionName(title);
        var selectedRiskPanelsView = sas.analytics.riskhp.utils.HprUtil.getSelectedRiskPanelsView();
        if (selectedRiskPanelsView) {
            var delegate = selectedRiskPanelsView.getDelegate();
            delegate.setTitleForContent(title, this.getTableViewerModel().id);
        }
    },

    /**
     * This method performs actions that make this viewer's table model invalid.  Currently it sets the 'tuples' to an empty array.
     * The point of the method is to make the table model invalid so a ZeroStateLayout is displayed as the content.
     * @private
     */
    _invalidateTableViewerModel: function() {
        this.getTableViewerModel().invalidate();
    },

    /**
     * Checks to see if this viewer's table model is valid or not.  It currently checks the model for an empty 'tuples' array.
     * @returns {boolean} Returns true if this viewer's model is valid; otherwise, it returns false
     */
    isModelValid: function() {
        var valid = true;
        var tableViewerModel = this.getTableViewerModel();

        if (_.isUndefined(tableViewerModel) || !tableViewerModel) {
            valid = false;
        }

        if (valid && tableViewerModel.isValid() === false/*_.isEmpty(tableViewerModel.tuples)*/) {
            valid = false;
        }

        return valid;
    },

    /**
     * Pseudo interface method to return the dataSourceId the component is using in a common way
     * @returns String
     */
    getDataSourceId: function() {
        return this.getTableViewerModel().dataSourceId;
    },

    /**
     * Pseudo interface method to return the type of control this is in a common way
     * @returns String
     */
    getPanelType: function() {
        return this.getTableViewerModel().type;
    },

    /**
     * This method replaces the currently displayed WideTable with a ZeroStateLayout with a detailed message (if provided).  The 'zeroStateText'
     * string is displayed in the ZerStateLayout if it is provided; otherwise, a default message is displayed.
     * @param zeroStateText String Message to display within the ZeroStateLayout
     * @private
     */
    _createZeroStateLayout: function(zeroStateText) {

        // Determine the correct text to display in the ZeroState
        var zeroStateLayoutText = zeroStateText ? zeroStateText : this.HprUtil.getBundle().getText("HPRisk." + this.getTableViewerModel().type + ".zeroState.default.txt");

        // Construct a ZeroStateLayout to display inside the RiskPanelsView
        if (this.zeroStateLayout === undefined || this.zeroStateLayout === null) {
            this.zeroStateLayout = new sas.risk.common.widgets.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATETABLE,
                title: zeroStateLayoutText,
                buttons: []
            });
        }
        else {
            this.zeroStateLayout.setTitle(zeroStateLayoutText);
        }

       this.tableWrapper.setContent(this.zeroStateLayout);
    },

    _getResourceBundle: function() {
        return sas.analytics.riskhp.utils.HprUtil.getBundle();
    },

    /**
     * Create the toolbar for the table.
     * @private
     */
    _createTableToolbar: function() {
        var self = this;
        var toolbarContent = [];

        self.pathLabel = new sas.hc.m.Label();

        toolbarContent.push(new sas.hc.m.ToolbarSeparator());
        toolbarContent.push(self.pathLabel);
        toolbarContent.push(new sas.hc.m.ToolbarSpacer());

        return new sas.hc.m.ResponsiveToolbar({content: toolbarContent});
    },

    /**
     * Creates the table displayed by this control.
     * @private
     */
    _createTable: function() {
        this.table = new sas.hc.x.WideTable({
            showSelectionControls: false,
            showTableOptions: false,
            selectionMode: sas.hc.ui.table.SelectionMode.None,
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            minAutoRowCount: 1, // required for scrolling with < 5 rows
            allowReflow: false,
            toolbar: this._createTableToolbar(),
            sort: $.proxy(this._handleColumnSort, this),
        });
        this.table.addStyleClass("hprBoundedControl");

        this.tableLayout = new sas.hc.ui.commons.layout.BorderLayout({
            width: "100%",
            height: "100%",
            center: new sas.hc.ui.commons.layout.BorderLayoutArea({
                overflowX: "hidden",
                overflowY: "hidden",
                content: [this.table]
            }).addStyleClass("sasPropertyPaneLayout_paneArea_centerLayoutArea")
        });
        return this.tableLayout;
        //return this.table;
    },

    /**
     * Function for creating columns for the table.
     * @param dataSetColumns  array of objects defining a column
     * @returns
     * @private
     */
    _createTableColumns: function(dataSetColumns) {
        var self = this;

        if (dataSetColumns && dataSetColumns.length > 0) {
            if (self.table.getColumns().length === 0) {
                self.getTableViewerModel().saveDataSetColumns(dataSetColumns);
                for (var x = 0; x < dataSetColumns.length; x++) {
                    self._buildColumn(dataSetColumns[x]);
                }
            }
            else {
                // Turn off midtier sending column information since we already have it.
                self.getTableViewerModel().enableDataSetColumns(false);
            }
        }
    },

    /**
     * Query midtier for data and create a data model for the table.
     * @private
     */
    _createTableDataModel: function() {
        var self = this;

        var oService = {
            sURL : self.getTableViewerModel()._buildQueryURL(),
            get: function(requestOptions) {
                sas.log.debug("Service->get()", requestOptions);
                var service = this;
                return $.Deferred(function(defer) {

                    var responseData = {
                        id: "",
                        name: self.sId,
                        version: 1,
                        limit: self.pageSize,
                        count: 0,
                        start: 0,
                        data: null,
                        dataSetColumns: null
                    };

                    var tableViewerModel = self.getTableViewerModel();
                    if (tableViewerModel.isValid()) {
                        self.setBusy(true);

                        self.currentRequestData = tableViewerModel.getRequestData(requestOptions.data.start, requestOptions.data.limit, requestOptions.data.sortBy);

                        var headers = {};
                        headers["Accept"] = "application/vnd.sas.risk.query+json;charset=utf-8";
                        headers["Content-Type"] = "application/vnd.sas.risk.query.parms+json;charset=utf-8";
                        var request = sas.ajax({
                            async: true,
                            cache: false,
                            method: "POST",
                            headers: headers,
                            data: JSON.stringify(self.currentRequestData),
                            url: service.sURL
                        }).done(function (data, textStatus, jqXHR) {

                            //var responseData = {};

                            // The server returns a 204 No Content response if the query is valid but has no results.  It does
                            // not return an empty JSON model which is resulting in an exception in the PageableJSONModel (S1387411)
                            // If we get a No Content response, provide an empty JSON model
                            if (jqXHR.status === 204) {
                                // self.HprUtil.displayGenericErrorDialog(self._getResourceBundle().getText('HPRisk.ErrorCode.204.txt'));

                                responseData = {
                                    id: "",
                                    name: self.sId,
                                    version: 1,
                                    limit: self.pageSize,
                                    count: 0,
                                    start: 0,
                                    data: null,
                                    dataSetColumns: null
                                };

                            } else {
                                responseData = {
                                    id: "",
                                    name: self.sId,
                                    version: 1,
                                    limit: self.pageSize,
                                    count: data.results.data.length,
                                    start: 0,
                                    data: data.results.data,
                                    dataSetColumns: data.results.dataSetColumns
                                };
                            }
                            defer.resolve(responseData);
                        }).fail(function (XMLHttpRequest, textStatus, errorThrown) {
                            defer.reject(XMLHttpRequest, textStatus, errorThrown);
                        }).always(function () {
                            self.setBusy(false);
                        });
                    }
                    else {
                        defer.resolve(responseData);
                    }
                }).promise();
            }
        };

        self.model = new sas.hc.ui.model.PageableJSONModel({
            service: oService,
            bindId: "data",
            cacheSize: 3,
            pageSize: self.pageSize,   //Default Page Size?
        });

        // process successful request
        self.model.attachRequestCompleted(function(evt) {
            var responseData = evt.getParameter("json");
            if (responseData) {
                self._createTableColumns(responseData.dataSetColumns);
            }
            if (self.rebuildDeferred) {
                self.rebuildDeferred.resolve();
                self.rebuildDeferred = null;
            }
        });

        // handle request failure
        self.model.attachRequestFailed(function(jqxhr) {
            sas.analytics.riskhp.utils.HprUtil.displayDetailedErrorDialog(jqxhr);

            self._createZeroStateLayout(self.HprUtil.getBundle().getText("HPRisk." + self.getTableViewerModel().type + ".zeroState.query.problem.txt"));
            if (self.rebuildDeferred) {
                self.rebuildDeferred.reject();
                self.rebuildDeferred = null;
            }
        });

        return self.model;
    },

    _buildColumn: function(dataSetColumn) {
        var self = this;

        if (dataSetColumn !== undefined) {

            // Determine a display label for the column
            var displayLabel = dataSetColumn.name;
            if (dataSetColumn.label !== undefined) {
                displayLabel = dataSetColumn.label;
            }
            else {
                var msgKey = "HPRisk." + self.getTableViewerModel().type + "." + dataSetColumn.name + ".txt";
                var columnLabel = self.HprUtil.getBundle().getText(msgKey);
                // If the msgKey is not found in the resources bundle, msgKey is returned.
                if (columnLabel !== msgKey) {
                    displayLabel = columnLabel;
                }
            }

            self.table.addColumn(new sas.analytics.riskhp.common.riskcommons.table.TableViewerColumn({
                label: new sas.hc.m.Label({text: displayLabel}),
                template: new sas.risk.common.widgets.RiskColumnText({
                    text: {
                        parts: [{path: dataSetColumn.name}], // could be dataSetColumn.id ???
                        formatter: function (value) {
                            var displayText = value;
                            if (dataSetColumn.dataType === "numeric" || dataSetColumn.dataType === "date") {
                                displayText = sas.ltjs.sasicu.util.SASFormatterUtil.format(Number(value), dataSetColumn.format);
                            }
                            return displayText;
                        }
                    }
                }),
                customData: new sas.hc.ui.core.CustomData({key: "columnId", value: dataSetColumn.name}),
                sortProperty: dataSetColumn.name,
                //width: "150px"
            }));
        }
    },

    /**
     * When a label for a Data Item is updated, see if any of the column labels representing a
     * Class Variable Data Item need to be updated.
     *
     * @param channel
     * @param event
     * @param data
     * @private
     */
    _onDataItemLabelUpdate: function(channel, event, data) {
        var self = this;

        // Only try to update the label if the id's of the DataSource being updated and DataSource for this
        // tableViewer match
        if (data && data.dataSourceId) {
            if (data.dataSourceId !== self.getTableViewerModel().dataSourceId) return;
        }

       // Only try to update the column label if tableViewer is displaying labels (and not names) for Data Items
        var showLabels = self.getTableViewerModel().getOptionShowLabels().value;
        if (!showLabels) return;

        self._updateTitleAndPath();
    },

    /**
     * When the "Show labels" option is updated, update the path appropriately.
     *
     * @param channel
     * @param event
     * @param data
     * @private
     */
    _onShowLabelsChange: function(channel, event, data) {
        var self = this;

        // Only try to update the path if the id of this TableViewer matches the one that fired the event (think multiple TableViewers could be on a page)
        if (data && data.tableViewerId) {
            if (data.tableViewerId !== self.getTableViewerModel().id) return;
        }

        // Only update the path on show labels change, and not the title of the panel
        self._updatePathLabel();
        self._refreshPanel(self.getTableViewerModel().id);
    },

    /**
     * Handle the user sorting a column in the table.
     * @param event
     * @private
     */
    _handleColumnSort: function(event) {
        if (this.restoringTableState === true) {
            return;
        }

        var sortOrder = event.getParameter("sortOrder");
        var columnId = _.find(event.getParameter("column").getCustomData(), function(customData) {
            return customData.getKey() === "columnId";
        }).getValue();

        this.getTableViewerModel().setSortColumn(columnId, sortOrder);

        this.fireDirtyChange();
    },

    _handleExportToExcel: function() {
        var self = this;

        self.setBusy(true);

        // if no export request, nothing to export
        var exportRequestData = self.getTableViewerModel().getExportRequestData();
        if (!exportRequestData) {
            //TODO show message
            return;
        }

        // export the data represented by the request
        self.HprAPI.exportPanel(exportRequestData).done(
            function(exportResult) {
                console.debug(self.getTableViewerModel().type + " export successful.");
            }
        ).fail(
            function (jqxhr, textStatus, error) {
                console.debug(self.getTableViewerModel().type + " export failed.");
            }
        ).always(
            function() {
                self.setBusy(false);
            }
        );
    },


    // utilities

    /**
     * Rerenders the table and ensures that the sort decoration is reapplied.
     * @private
     */
    _rerenderTable: function() {
        var self = this;

        // If there is no table, don't try and re-render
        if (!self.table) { return; }

        var deferred = $.Deferred();
        var rerenderFunctionInternal = function() {
            // rerender table
            self.table.rerender();

            // reapply sort icon
            var sortColumn = self.getTableViewerModel().getSortColumn();
            if (sortColumn) {
                var tableColumn = _.find(self.table.getColumns(), function (column) {
                    var columnId = _.find(column.getCustomData(), function (customData) {
                        return customData.getKey() === "columnId";
                    }).getValue();

                    return columnId === sortColumn.columnId;
                });

                // resize can be fired before the table columns are set
                if (tableColumn) {
                    tableColumn._updateIcons();
                }
            }

            deferred.resolve();
        };

        // Using a 0 timeout moves this execution after the current
        // block of code that triggered this function has finished
        // processing.
        setTimeout(rerenderFunctionInternal, 0);

        return deferred;
    },

    _onCrosstabDeleted: function(channel, event, data) {
        var self = this;
        console.log("TableView: OnCrosstabDeleted called");

        var tableViewerModel = self.getTableViewerModel();
        if (tableViewerModel.handleCrosstabDeleted(data.id)) {
            $.proxy(self.rebuild(), self);
        }
    },


    //************************************************************************************
    // Linked Table methods
    //************************************************************************************

    /**
     * @description Isolates code to delink "like" tables when a new plot is created
     * @private
     */
    _delinkLikeTables: function() {
        var self = this;
        var tableViewerModel = self.getTableViewerModel();
        var crosstabId = tableViewerModel.crosstabId;
        // get other plots for this crosstab
        var selectedRiskPanelsView = sas.analytics.riskhp.utils.HprUtil.getPanelsViewForComponent(crosstabId);
        if (selectedRiskPanelsView) {
            var delegate = selectedRiskPanelsView.getDelegate();

            var linkedTableIds = _.filter(delegate.tableIds, function (plotId) {
                var tableView = sap.ui.getCore().byId(plotId);
                if (tableView) {
                    var tableModel = tableView.getTableViewerModel();
                    if (tableModel) {
                        return tableModel.id !== tableViewerModel.id && crosstabId === tableModel.crosstabId && tableModel.linkedToCrosstab;
                    }
                }
                return false;
            });
            var sourceTableType = tableViewerModel.type;

            // Now walk the ids of the plots that are also linked to the crosstab to delink any "like" plots
            _.each(linkedTableIds, function(tableId){
                var currentTableView = sap.ui.getCore().byId(tableId);
                if (currentTableView) {
                    var currentTableModel = currentTableView.getTableViewerModel();
                    if (currentTableModel) {

                        // Now get the information about the currentChartModel (the one we iterated over)
                        var currentTableType = currentTableModel.type;

                        if (sourceTableType === currentTableType) {
                            currentTableView._handleLinkToCrosstab(false);
                        }
                    }
                }
            });
        }
    },

    /**
     * @description Callback function for when the EVENT_UPDATE_LINKED_PLOTS is called
     * @param {string} channelId
     * @param {string} eventId
     * @param {UpdateLinkedPlotObject} data
     * @private
     */
    _onUpdateLinkedTable: function(channelId, eventId, data) {
        var self = this;

        console.log("TableView: _onUpdateLinkedTable called");

        var tableViewerModel = self.getTableViewerModel();
        // Check to see if the tableViewerModel is linked to the crosstab that just updated, if not return
        if (!tableViewerModel.isLinkedToCrosstabWithId(data.crosstabId)) {
            return;
        }

        // If the tableViewerModel is in the process of replacing the data source (this means it is in an intermediate state)
        // We want to ignore the update call from the crosstab and clear the replacing data source flag to avoid a
        // double query
        if (tableViewerModel.isReplacingDataSource()) {
            tableViewerModel.clearReplacingDataSource();
            return;
        }

        // Process the update
        if (tableViewerModel.handleCrosstabUpdated(data)) {

            if (self.isModelValid()) {
                self.tableWrapper.setContent(self.tableLayout);
                self._updatePanelTitle();
                self.rebuild();

                // If the selected cell changed and this plot is linked to the crosstab that was changed then fire the dirty state
                if (data.selectedCellChanged && tableViewerModel.isLinkedToCrosstabWithId(data.crosstabId)) {
                    $.proxy(self._fireDirty(), self);
                }
            }
            else {
                // If the model is not valid there is not much we can do, so display the ZeroStateLayout
                self.setBusy(false);
                self._updatePanelTitle();
                self._updateTitleAndPath();
                self._createZeroStateLayout(self.HprUtil.getBundle().getText("HPRisk." + tableViewerModel.type + ".cell_not_supported.txt"));
            }
        }
    },

    /**
     * @description Handles the action of de-linking or relinking a plot to it's owning crosstab
     * @param {boolean} linked - True if you want to relink, false otherwise
     * @private
     */
    _handleLinkToCrosstab: function(linked) {
        var self = this;

        console.log("TableView: _handleLinkToCrosstab called");

        var tableViewerModel = self.getTableViewerModel();

        // Only try to relink if the model is valid
        if (tableViewerModel.isValid()) {
            tableViewerModel.setLinkedToCrosstab(linked);

            if (linked === true) {
                tableViewerModel.handleRelinkToCrosstab();
            }

            $.proxy(self.rebuild(), self);
        }
    }
});