// (c) 2018, SAS Institute Inc.

jQuery.sap.declare("sas.analytics.riskhp.common.riskcommons.table.TableViewerModel");

jQuery.sap.require("sas.analytics.riskhp.models.DataSourceCacheManager");
jQuery.sap.require("sas.analytics.riskhp.models.HPRiskModelManager");
jQuery.sap.require("sas.analytics.riskhp.portfolio.ColumnDescriptor");
jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.table.TableViewerUtil");
jQuery.sap.require("sas.analytics.riskhp.settings.ApplicationSettings");
jQuery.sap.require("sas.analytics.riskhp.utils.HprAPI");
jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");

jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Horizon");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.RiskCube");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Statistic");

jQuery.sap.require("sas.ltjs.commons.pickResults.PickEntryType");

/**
 *
 *
 * @name sas.analytics.riskhp.common.riskcommons.table.TableViewerModel
 */
sap.ui.base.Object.extend("sas.analytics.riskhp.common.riskcommons.table.TableViewerModel", {
    metadata: {
        publicMethods: [
            "initializeModel",
            "setSortColumn",
            "getDataSourceId",
            "getDrillItemLabels",
            "getQueryURL",
            "getRequestData",
            "getOptions",
            "getOptionName",
            "setOptionName",
            "getOptionShowLabels",
            "setOptionShowLabels",
            "getSortColumn",
            "getTuples",
            "areHorizonsAbsolute",
            "toJSON",
            "areDataSourceIdsBeingReferenced",
            "removeDataSource",
            "replaceDataSource",
            "toOptionsJSON",
            "fromOptionsJSON",
            "updateOptions",
            'setFilters',
            'updateFilterSections',
            'getFilters',
            'isModelInvalid',
            'setLinkedToCrosstab',
            'isLinkedToCrosstab'
        ]
    },

    // pseudo-imports
    DataSourceCacheManager: sas.analytics.riskhp.models.DataSourceCacheManager,
    HPRiskModelManager: sas.analytics.riskhp.models.HPRiskModelManager,
    TableViewerUtil: sas.analytics.riskhp.common.riskcommons.table.TableViewerUtil,
    HprAPI: sas.analytics.riskhp.utils.HprAPI,
    HprConstants: sas.analytics.riskhp.utils.HprConstants,
    HprUtil: sas.analytics.riskhp.utils.HprUtil,
    ApplicationSettings: sas.analytics.riskhp.settings.ApplicationSettings,
    PickEntryType : sas.ltjs.commons.pickResults.PickEntryType,


constructor: function(id, contextData, isRestoration) {
        this.id = id;
        this.isNewModel = !(isRestoration === true);

        // Options properties
        this.options = {};
        this.options.id = this.id;
        this.options.name = null;
        this.options.nameChanged = false;
        this.options.showLabels = {};
        this.options.applyEnabled = false; // Flag for the Apply button on each Options pane; Disabled by default
        this.options.generalOptionsSectionCollapsed = false; // Open by default

        this.optionsQuery = []; // Options used during query for results

        // selected column options
        this.selectedColumnOption = null;
        this.selectedInstrumentTypeNames = [];
        this.selectedColumnIds = [];
        this.sortColumn = null;

        // filters: array of {collapsed, name (id?), type, values, include}
        this.filters = [];
        this.filtersQuery = []; // filters used during query for results

        // process the context data
        this.processContextData(contextData, isRestoration);
    },

    processContextData: function(contextData, isRestoration) {
        // process the context data
        if (contextData) {
            this.requests = contextData.requests;

            this.dataSourceId = contextData.dataSourceId;
            this.type = contextData.type;
            this.drill = contextData.drill;
            this.displayHorizons = contextData.displayHorizons;
            this.tuples = contextData.tuples;

            this.classVariables = contextData.classVariables;
            this.showLabels = contextData.showLabels;
            this.formats = contextData.formats;

            this.crosstabId = contextData.crosstabId;
            this.linkedToCrosstab = true;



            if (isRestoration === true) {
                this._restoreModelData(contextData);
            }
            else {
                this._setContextData(contextData);
            }
        }

        // update the valid state of the model
        this.invalid = !!!contextData || contextData.invalid === true ? true : false;

        // capture of most recent request
        this.currentRequestData = null;
    },

    _setContextData: function(contextData) {

        // options
        this.optionsQuery = contextData.options;

        var showLabelsObject = {};
        showLabelsObject.name = this.HprConstants.HPR_OPTIONS_PANE_SHOW_LABELS_NAME;
        // For the table's Show Labels option, the value should come from the Crosstab that launched it
        showLabelsObject.value = contextData.showLabels;
        this.options.showLabels = showLabelsObject;

        // default filters from cube
        if (contextData.filters) {
            this.filtersQuery = contextData.filters;
        }

        // query data
        this._buildQueryData(contextData);
    },

    _restoreModelData: function(modelData) {
        // column selections
        this.selectedColumnOption = modelData.selectedColumnOption;
        this.selectedInstrumentTypeNames = modelData.selectedInstrumentTypeNames;
        this.selectedColumnIds = modelData.selectedColumnIds;

        // column data
        this.dataSetColumns = modelData.dataSetColumns;
        this.sortColumn = modelData.sortColumn;

        // Table link to crosstab
        this.crosstabId = modelData.crosstabId;
        this.linkedToCrosstab = modelData.linkedToCrosstab;

        // options
        this.options = modelData.options;
        this.optionsQuery = modelData.optionsQuery;

        // filters
        this.filters = modelData.filters;
        this.filtersQuery = modelData.filtersQuery;

        // data query
        this._buildQueryData(modelData);

        //this.invalid = modelData.invalid;
    },

    _buildQueryData: function(contextData) {
        var self = this;

        // query data
        self.queryData = {};
        self.queryData.requestData = {};
        self.queryData.requestData.name = self.id;
        self.queryData.requestData.version = 1;
        self.queryData.requestData.type = contextData.type;
        self.queryData.requestData.orientation = "row";
        self.queryData.requestData.dataSetColumns = true;
        self.queryData.requestData.dataDefinitions = {};
        self.queryData.requestData.dataDefinitions.tuples = [];
        _.each(contextData.tuples, function(dataItems) {
            var tuple = {};
            tuple.dataItems = [];
            _.each(dataItems, function(dataItem) {
                tuple.dataItems.push({
                    id: dataItem.id,
                    type: dataItem.type
                });
            });
            self.queryData.requestData.dataDefinitions.tuples.push(tuple);
        });

        self.queryData.requestData.report = {};
        self.queryData.requestData.report.requests = contextData.requests;
        self.queryData.requestData.report.sorts = [];
        self.queryData.requestData.report.options = self.optionsQuery;
        self.queryData.requestData.report.filters = self.filtersQuery;
    },

    _validTuples: function(tuples) {
        var self = this;
        var valid = true;

        _.each(tuples, function(dataItems) {
            _.each(dataItems, function(dataItem) {
                if (dataItem.basecase) {
                    valid = false;
                }
            });
        });
        return valid;
    },

    enableDataSetColumns: function(enabled) {
        var self = this;
        self.queryData.requestData.dataSetColumns = enabled;
    },

    saveDataSetColumns: function(dsColumns) {
        var self = this;
        self.dataSetColumns = dsColumns;
    },

    /**
     * This method must be called before the model can be used to manage/access
     * columns and the associated data source. It is meant to be called once by the consumer
     * (see TableViewer). If calls to functions such as getAvailableColumns occur
     * before this function has successfully resolved,
     *
     * @returns {Deferred}
     */
    initializeModel: function() {
        var self = this;
        var deferred = $.Deferred();

        // load the data source and column metadata
        self._loadDataSource(self.dataSourceId).done(
            function (dataSource) {
                self.dataSource = dataSource;
                deferred.resolve();
            }
        ).fail(
            function (jqxhr, textStatus, error) {
                self.invalidate();
                deferred.reject(jqxhr, textStatus, error);
            }
        );

        return deferred.promise();
    },

    toJSON: function() {
        return {
            name: this.id,
            type: this.type,
            dataSourceId: this.dataSourceId,
            // Can't be sure at this point if tuples are dehydrated or not.
            // (old exploration versus flex exploration)
            // By hydrating and then dehydrating, we can be sure that the tuples are saved dehydrated
            // and no exception if tuples is already dehydrated.
            tuples: this._dehydrateTuples(this._hydrateTuples(this.tuples)),
            drill: this.drill,
            displayHorizons: this.displayHorizons,

            requests: this.requests,

            // Can't be sure at this point if classvars are dehydrated or not.
            // (old exploration versus flex exploration)
            // By hydrating and then dehydrating, we can be sure that the classvars are saved dehydrated
            // and no exception if classvar is already dehydrated.
            classVariables : this._dehydrateDataItems(this._hydrateDataItems(this.classVariables)),
            showLabels : this.showLabels,
            formats : this.formats,

            crosstabId: this.crosstabId,
            linkedToCrosstab: this.linkedToCrosstab,

            dataSetColumns: this.dataSetColumns,
            sortColumn: this.sortColumn,

            options: this.toOptionsJSON(),
            optionsQuery: this.optionsQuery,
            filters: this._toFiltersJSON(),
            filtersQuery: this.filtersQuery,

            invalid: this.invalid
        };
    },

    getDataSourceId: function() {
        return this.dataSourceId;
    },

    getDrillItemLabels: function() {
        return this.drill;
    },

    areHorizonsAbsolute: function() {
        return this.displayHorizons !== HprConstants.DISPLAY_HORIZONS_RELATIVE_VALUE
    },

    isValid: function() {
        var tuplesOK = this._validTuples(this.getTuples());
        return !(this.invalid === true ? true : false || !tuplesOK);
    },

    invalidate: function(valid) {
        this.invalid = true;
    },

    /**
     * Creates the string that represents the crosstab drill path from which this table viewer was launched.
     * @returns {String}
     * @private
     */
    createPathString: function() {
        var self = this;

        var classVariables = self.dataSource.getDataItemsForType(sas.analytics.riskhp.models.dataitems.Classvar.CLASSVAR);
        var keys = self.requests[0].key;

        var pathString = "";
        var appendToPathString = function (itemLabel) {
            if (pathString.length > 0) {
                pathString += " > ";
            }
            pathString += "[" + itemLabel + "]";
        };

        var classVariable = null;
        if (keys.length === 0) {
            classVariable = classVariables[0];
            // Use the Show Labels option value to find the correct "label" to display; Could either be label or name based on value
            appendToPathString(classVariable.getDisplayLabel(self.getOptionShowLabels().value) + ":" + self.HprUtil.getBundle().getText("HPRisk.plot.path.value.all.txt"));
        } else {
            _.each(keys, function (key) {
                classVariable = _.find(classVariables, function(classVar) {
                    return classVar.id === key.id;
                });

                if (classVariable) {
                    var valueToUse = key.value;
                    if (classVariable.getDataType() !== HprConstants.HPR_DATA_TYPE_STRING) {
                        valueToUse = sas.ltjs.sasicu.util.SASFormatterUtil.format(Number(valueToUse), classVariable.getFormat());
                    }
                    // Use the Show Labels option value to find the correct "label" to display; Could either be label or name based on value
                    appendToPathString(classVariable.getDisplayLabel(self.getOptionShowLabels().value) + ":" + valueToUse);
                }
            });
        }
        return pathString ? this.HprUtil.getBundle().getText("HPRisk.panel.genericPath.fmt", [pathString]) : null;
    },

    /**
     * A method which checks to see if the existing content object references any of the data source ids by looking
     * at the "dataSourceId" property.
     * @param { string } currentContentData - The JSON object representing the current state of a panel
     * @param { [string] } dataSourceIds - A list of data source ids
     */
    areDataSourceIdsBeingReferenced: function(currentContentData, dataSourceIds) {
        return _.contains(dataSourceIds, currentContentData.dataSourceId);
    },

    /**
     * pseudo static method that takes a list of Data Source ids which are being removed from an Exploration and determines
     * if this model references one of them; If it does a Zero State object is returned
     * @param { string } currentContentData - The JSON object representing the current state of a panel
     * @param { [string] } dataSourceIdsBeingRemoved - A list of data source ids that are being removed
     * @returns {*}
     */
    removeDataSource: function(currentContentData, dataSourceIdsBeingRemoved) {
        var deferred = $.Deferred();

        // Check to make sure the current data source id is contained in the list of data source ids being removed,
        // otherwise there is nothing to do so return the provided currentContentData.
        var currentDataSourceId = currentContentData.dataSourceId;
        if (!_.contains(dataSourceIdsBeingRemoved, currentDataSourceId)) {
            deferred.resolve(currentContentData);   // this model is unaffected as it doesn't reference the removed data sources
        } else {
            // Define the basic model to signify a ZeroState Simulated Values Table
            var updatedModelJSON = {
                name: currentContentData.name,
                dataSourceId: "",
                options: {
                    // So the name of the ZeroState Simulated Values Table will be the same
                    name: currentContentData.options.name
                },
                type: currentContentData.type,
                invalid: true
            };

            deferred.resolve(updatedModelJSON);
        }

        return deferred.promise();
    },

    /**
     * @description This flag will be set if the model was just changed because of a replace data source action.  It is
     * intended to be used to stop cascading actions (i.e. crosstab re-rendering updating the plot again) until the
     * plot has fully been rendered after the replace data source action.  It is expected to be a one-time thing and
     * should be "reset" by a call to clearReplacingDataSource() when you know normal processing can continue.
     * @returns {boolean}
     */
    isReplacingDataSource: function() {
        var self = this;
        return (!_.isUndefined(self.replacingDataSource) && !_.isNull(self.replacingDataSource) && self.replacingDataSource === true);
    },

    /**
     * Removes the flag indicating that the model was just modified by a replace DataSource action.
     * @see {@link isReplacingDataSource} for more information
     */
    clearReplacingDataSource: function() {
        this.replacingDataSource = undefined;
    },

    /**
     * pseudo static method that takes an existing content object and changes it in a way to represent a new data source.
     * This function should select an equivalent data source id from the list of newDataSourceIds for the data source
     * id the currentContentData is representing.
     *
     * This method must return a jQuery.Deferred object that behaves as follows:
     * - Returns the JSON to represent the panel's new state regardless of whether the panel should change because of the new data source
     * - ALWAYS resolve successfully.
     * - May make subsequent/wrapped ajax calls but when it returns all processing should be complete.
     * - May not change the state of the model instance or attempt to interact with the UI.
     *
     * @param { string } currentContentData - The JSON object representing the current state of a panel
     * @param { [string] } dataSourceIdsBeingReplaced - A list of data source ids that are being replaced
     * @param { [string] } newDataSourceIds - A list of replacement data source ids
     * @returns {*}
     */
    replaceDataSource: function(currentContentData, dataSourceIdsBeingReplaced, newDataSourceIds) {
        var self = this;
        var deferred = $.Deferred();

        // Check to make sure the current data source id is contained in the list of data source ids being replaced,
        // otherwise there is nothing to do so return the provided currentContentData.
        var currentDataSourceId = currentContentData.dataSourceId;
        if (!_.contains(dataSourceIdsBeingReplaced, currentDataSourceId)) {
            deferred.resolve(currentContentData);   // this model is unaffected as it doesn't reference the replaced data source
        }
        else if (currentContentData.invalid === true) {
            deferred.resolve(currentContentData);   // already invalid, so nothing we can do
        }
        else {
            // Store any deferreds from requests needed to update for the new data source here. After all items are
            // processed, this set of deferreds will be passed into a when function, whereupon their completion, the
            // master deferred will be resolved (if we get that far, of course).
            var pendingRequests = [];

            // Get a compatible data source id from the list of newDataSourceIds.  If one is not found then there is
            // nothing to do so return the provided currentContentData
            var replacementDataSourceId = self.HprUtil.getCompatibleDataSourceId(currentContentData.dataSourceId, newDataSourceIds);

            // define the basic model
            var updatedModelJSON = {

                // Set a temporary flag so when first crosstab rerender occurs after replace data source we ignore it.
                replacingDataSource: true,

                name: currentContentData.name,
                dataSourceId: replacementDataSourceId,
                linkedToCrosstab: currentContentData.linkedToCrosstab,
                crosstabId: currentContentData.crosstabId,

                displayHorizons: currentContentData.displayHorizons,
                drill: currentContentData.drill,
                optionsQuery: currentContentData.optionsQuery,
                filtersQuery: currentContentData.filtersQuery,
                type: currentContentData.type,
                requests: currentContentData.requests,

                classVariables : currentContentData.classVariables,
                showLabels : currentContentData.showLabels,

                options: currentContentData.options,
                filters: currentContentData.filters,

                formats : currentContentData.formats,

                sortColumn: currentContentData.sortColumn
            };


            self._loadDataSource(replacementDataSourceId).done(
                function (newDataSource) {
                    // validate classvars
                    if (self._dataItemValidator(newDataSource, currentContentData.classVariables)) {
                        updatedModelJSON.classVariables = currentContentData.classVariables;
                    }
                    else {
                        updatedModelJSON.invalid = true;
                        deferred.resolve(updatedModelJSON);
                        return;
                    }

                    // validate request - Currently only one, but support multiple for future use.
                    var validRequests = true;
                    _.each(currentContentData.requests, function(request){
                        if (!self._dataItemValidator(newDataSource, request.key)){
                            validRequests = false;
                        }
                    });
                    if (validRequests) {
                        updatedModelJSON.requests = currentContentData.requests;
                    }
                    else {
                        updatedModelJSON.invalid = true;
                        deferred.resolve(updatedModelJSON);
                        return;
                    }

                    // compare tuples
                    var remainingTuples = [];
                    _.each(currentContentData.tuples, function(tupleDataItems) {
                        var remainingTupleDataItems = _.filter(tupleDataItems, function(tupleDataItem) {
                            return !!newDataSource.getDataItemById(tupleDataItem.id);
                        });
                        //if (remainingTupleDataItems && remainingTupleDataItems.length > 0) {
                        if (remainingTupleDataItems && remainingTupleDataItems.length === tupleDataItems.length) {
                            remainingTuples.push(remainingTupleDataItems);
                        }
                    });
                    if (remainingTuples.length === 0) {
                        updatedModelJSON.invalid = true;
                        deferred.resolve(updatedModelJSON);
                        return;
                    }
                    updatedModelJSON.tuples = remainingTuples;


                    // check sort column
                    updatedModelJSON.sortColumn = currentContentData.sortColumn;

                    // update the title if the user has not yet changed it
                    if (updatedModelJSON.options.nameChanged !== true) {
                        updatedModelJSON.options.name = self.TableViewerUtil.generateDefaultTableTitle(newDataSource, updatedModelJSON);
                    }

                    updatedModelJSON.invalid = false;
                    deferred.resolve(updatedModelJSON);

                }
            ).fail(
                function (jqxhr, textStatus, error) {
                    updatedModelJSON.invalid = true;
                    deferred.resolve(updatedModelJSON);
                }
            );
        }

        return deferred.promise();
    },

    /**
     * A generic validator function to ensure the list of data items (usually dehydrated) in the
     * passed in list all exist in the given dataSource.  If either the dataSource or dataItemList is not
     * supplied or null return false.
     * @param dataSource - The DataSource to validate the dataItemList against
     * @param dataItemList
     * @returns {boolean}
     */
    _dataItemValidator: function(dataSource, dataItemList) {
        var areItemsValid = true;
        if (!dataSource || !dataItemList) {
            return false;
        }
        _.each(dataItemList, function(candidateDataItem){
            var dataItem = dataSource.getDataItemById(candidateDataItem.id);
            if (!dataItem) {
                areItemsValid = false;
            }
        });
        return areItemsValid;
    },



    //*****************************
    // Query
    //*****************************

    getRequestKeys: function() {
        return this.queryData.requestData.report.requests[0].key;
    },

    getQueryURL: function() {
        if (!this.queryData.url) {
            this.queryData.url = this._buildQueryURL(this.dataSourceId);
        }
        return this.queryData.url;
    },

    getRequestData: function(start, limit, sortBy) {
        var self = this;

        // clone the initial request data
        var requestData = $.extend(true, {}, this.queryData.requestData);

        // get the data source information
        var dataSourceParams = self.DataSourceCacheManager.getDataSourceParam(self.getDataSourceId());
        requestData.dataSource = {
            id: dataSourceParams.realDataSourceId,
            application: dataSourceParams.application,
            workGroup: dataSourceParams.workGroup,
            relPath: dataSourceParams.relPath,
            type: dataSourceParams.type
        };

        requestData.report.requests[0].startRow = start;
        requestData.report.requests[0].maxRows = limit;

        if (sortBy) {
            requestData.report.sorts = [this._translateSortBy(sortBy)];
        }

        // capture this request
        this.currentRequestData = requestData;

        return requestData;
    },

    getExportRequestData: function() {
        // if we have not yet created a request, nothing to export
        if (!this.currentRequestData) {
            return {};
        }

        // clone the most recent request and add export information to it
        var self = this;
        var requestData = $.extend(true, {}, self.currentRequestData);

        // create the export object
        requestData.export = {
            name: self.HprUtil.getBundle().getText("Hpr.Export.WorksheetName.label.fmt", [1]),  // always sheet 1 for now
            type: self.HprConstants.HPR_EXPORT_FORMAT_EXCEL,
            headers: []
        };

        // add the data source name
        requestData.export.headers.push({
            key: self.HprConstants.HPR_EXPORT_TABLE_NAME_KEY,
            name: self.HprUtil.getBundle().getText("Hpr.Export.TableName.label.txt"),
            value: self.options.name
        });

        // add the table type
        requestData.export.headers.push({
            key: self.HprConstants.HPR_EXPORT_TABLE_TYPE_KEY,
            name: self.HprUtil.getBundle().getText("Hpr.Export.TableType.label.txt"),
            value: self.HprUtil.getBundle().getText("Hpr.Export.TableType.Portfolio.label.txt"),
        });

        // add the data source name
        requestData.export.headers.push({
            key: self.HprConstants.HPR_EXPORT_DATA_SOURCE_NAME_KEY,
            name: self.HprUtil.getBundle().getText("Hpr.Export.DataSourceName.label.txt"),
            value: self.dataSource.getName()
        });

        // add the creation date
        requestData.export.headers.push({
            key: self.HprConstants.HPR_EXPORT_DATASOURCE_CREATION_DATE_KEY,
            name: self.HprUtil.getBundle().getText("Hpr.Export.DataSourceCreationDate.label.txt"),
            value: self.HprUtil.formatDateTime(self.dataSource.getCreatedDate())
        });

        requestData.export.headers.push({
            key: HprConstants.HPR_EXPORT_EXPORT_CREATION_DATE_KEY,
            name: self.HprUtil.getBundle().getText("Hpr.Export.ExportCreationDate.label.txt"),
            value: self.HprUtil.formatDateTime(Date.now())
        });

        // add the path value
        requestData.export.headers.push({
            key: self.HprConstants.HPR_EXPORT_PORTFOLIO_DRILL_KEY,
            name: self.HprUtil.getBundle().getText("Hpr.Export.PortfolioDrill.label.txt"),
            value: self.createPathString()
        });

        // This entry must be last.
        requestData.export.headers.push({
            key: HprConstants.HPR_EXPORT_TABLE_DATA_KEY,
            name: self.HprUtil.getBundle().getText("Hpr.Export.TableData.label.txt"),
            value: ""
        });

        return requestData;
    },

    getTuples: function() {
        if (this.tuples && this.tuples.length > 0) {
            if (this.dataSource) {
                this.tuples = this._hydrateTuples(this.tuples);
            }
            return this.tuples;
        }
        return [];
    },

    getClassVariables: function() {
        if (this.classVariables && this.classVariables.length > 0) {
            if (this.dataSource) {
                return this._hydrateDataItems(this.classVariables);
            }
        }
        return [];
    },


    //*****************************
    // Columns
    //*****************************

    setSortColumn: function(columnId, sortOrder) {
        // if no sort order, then it has been removed, so clear the metadata
        if (!sortOrder) {
            this.sortColumn = null;
        }
        else {
            this.sortColumn = {
                columnId: columnId,
                sortOrder: sortOrder
            };
        }
    },

    getSortColumn: function() {
        return this.sortColumn;
    },


    //*****************************
    // Options
    //*****************************

    setOptionGeneralOptionsSectionCollapsed: function(generalOptionsSectionCollapsed) {
        this.options.generalOptionsSectionCollapsed = generalOptionsSectionCollapsed;
    },

    getOptions: function () {
        return $.extend(true, {}, this.options);
    },

    getOptionName: function() {
        return this.options.name;
    },

    setOptionName: function(name) {
        this.options.name = name;
    },

    getOptionsNameChanged: function() {
        return this.options.nameChanged;
    },

    getOptionShowLabels: function() {
        return this.options.showLabels;
    },

    setOptionShowLabels: function(showLabels) {
        this.options.showLabels = showLabels;
    },

    toOptionsJSON: function() {
        return {
            id: this.options.id,
            name : this.options.name,
            nameChanged : this.options.nameChanged,
            showLabels : this.options.showLabels,
            applyEnabled : this.options.applyEnabled,
            generalOptionsSectionCollapsed : this.options.generalOptionsSectionCollapsed
        };
    },

    fromOptionsJSON: function(opts) {
        this.options.id = opts.id;
        this.options.name = opts.name;
        this.options.nameChanged = opts.nameChanged;
        this.options.showLabels = opts.showLabels;
        this.options.applyEnabled = opts.applyEnabled;
        this.options.generalOptionsSectionCollapsed = opts.generalOptionsSectionCollapsed;
    },

    updateOptions: function(optionsData) {
        // check if the name changed
        if (this.options.name !== optionsData.name) {
            this.options.nameChanged = true;
        }

        // Update the option properties with data passed in
        this.options.name = optionsData.name;
        this.options.showLabels = optionsData.showLabels;
        this.options.applyEnabled = optionsData.applyEnabled;
        this.options.generalOptionsSectionCollapsed = optionsData.generalOptionsSectionCollapsed;
    },


    //*****************************
    // Filters
    //*****************************

    setFilters: function(filters) {
        this.filters = this.HprUtil.hprClone(filters);
        //this.filtersModified = true;
    },

    updateFilterSections: function(indexedFilterSectionData) {
        var self = this;
        var filterSection;
        _.each(self.filters, function(filter) {
            filterSection = indexedFilterSectionData[filter.id];
            if (filterSection) {
                filter.collapsed = filterSection.collapsed;
            }
        });
    },

    getFilters: function() {
        return this.HprUtil.hprClone(this.filters);
    },

    _toFiltersJSON: function() {
        return this.filters;
    },

    _fromFiltersJSON: function(filterJSON) {
        this.filters = filterJSON;
    },

    /**
     * Returns true if the filters collection still contains any and all filters created from
     * the context request key that indicates the crosstab cell from which the user launched
     * the table.
     */
    isPathFilteringApplied: function() {
        var self = this;
        var key = self.requests[0].key;

        // optimization--if the filter set contains fewer items than the key, then the filters cannot
        // contain all filters produced by the key.
        if (self.filters.length < key.length) {
            return false;
        }

        // otherwise, check each key and ensure that there is a corresponding filter created for it
        var matchedKeyItems = _.filter(key, function(keyItem) {
            return !!_.find(self.filters, function(filter) {
                return filter.id === keyItem.id && filter.values.length === 1 && filter.values[0] === self._convertKeyItemValue(keyItem);
            });
        });
        return matchedKeyItems.length === key.length;
    },

    /**
     * Returns true if the filters have been modified such that the path filters are no longer
     * fully represented in the filter collection, or if the path filters are represented but
     * additional filtering is also present.
     */
    isAdditionalFilteringApplied: function() {
        return this.filters.length > 0 && !this.isPathFilteringApplied() || this.requests[0].key.length < this.filters.length;
    },

    /**
     * S1390667 -- ensure that numeric/date values are represented consistently. For numeric
     * and date data item types, convert the string to a number and back to a string to eliminate
     * space padding added by the server.
     *
     * Future consideration--should we just trim this?
     *
     * @param keyItem
     * @returns {*}
     * @private
     */
    _convertKeyItemValue: function(keyItem) {
        var parsedFilterValue = keyItem.value;
        var filterDataItem = this.dataSource.getDataItemById(keyItem.id);
        if (filterDataItem) {
            var filterItemDataType = filterDataItem.getDataType();
            if (filterItemDataType === this.HprConstants.HPR_DATA_TYPE_NUMBER || filterItemDataType === this.HprConstants.HPR_DATA_TYPE_DATE) {
                parsedFilterValue = Number(keyItem.value).toString();
            }
        }
        return parsedFilterValue;
    },


    //*****************************
    // Utils
    //*****************************

    _hydrateDataItem: function(dataItem) {
        if (dataItem && this.dataSource) {
            return this.dataSource.getDataItemById(dataItem);
        }
        return dataItem;
    },

    _dehydrateDataItem: function(dataItem) {
        if (dataItem) {
            return {id: dataItem.id, type: dataItem.type};
        }
        return dataItem;
    },

    _hydrateDataItems: function(dehydratedDataItems) {
        var self = this;
        if (dehydratedDataItems && dehydratedDataItems.length > 0 && this.dataSource) {
            var hydratedDataItems = [];
            _.each(dehydratedDataItems, function(dehydratedDataItem){
                var dataItem = self._hydrateDataItem(dehydratedDataItem.id);
                if (dataItem !== null) {
                    hydratedDataItems.push(dataItem);
                }
            });
            return hydratedDataItems;
        }
        return dehydratedDataItems;
    },

    _dehydrateDataItems: function(hydratedDataItems) {
        var self = this;
        if (hydratedDataItems && hydratedDataItems.length > 0) {
            var dehydratedDataItems = [];
            _.each(hydratedDataItems, function(hydratedDataItem) {
                dehydratedDataItems.push(self._dehydrateDataItem(hydratedDataItem));
            });
            return dehydratedDataItems;
        }
        return hydratedDataItems;
    },

    _hydrateTuples: function(tuples) {
        if (tuples && tuples.length > 0) {
            if (this.dataSource) {
                return this.dataSource.hydrateTuples(tuples);
            }
        }
        return tuples;
    },

    _dehydrateTuples: function(tuples) {
        if (tuples && tuples.length > 0) {
            if(this.dataSource) {
                return this.dataSource.dehydrateTuples(tuples);
            }
        }
        return tuples;
    },

    _loadDataSource: function(dataSourceId) {
        var self = this;
        var deferred = $.Deferred();

        // load the data source
        self.DataSourceCacheManager.getDataSource(dataSourceId).done(
            function (dataSource) {
                deferred.resolve(dataSource);
            }
        ).fail(
            function (jqxhr, textStatus, error) {
                deferred.reject(jqxhr, textStatus, error);
            }
        );

        return deferred.promise();
    },

    _buildQueryURL: function() {
        var self = this;
        var url = self.HprConstants.HPR_TABLE_QUERY_URL + "?" + self.HprAPI.GUID_PARAM_STRING;
        url = url.replace(self.HprConstants.HPR_CUBE_ID_TOKEN, encodeURIComponent(self.dataSourceId));
        return url;
    },

    _translateSortBy: function(sortBy) {
        var sortByProperties = sortBy.split(":");
        //var columnDescriptor = this.columnDescriptorMap[sortByProperties[0]];

        var sortObject = {};
        sortObject.tuple = {};
        sortObject.tuple.name = sortByProperties[0];
        sortObject.tuple.dataItems = [
            {
                //type: columnDescriptor.type,
                type: "dataSetColumn",
                id: sortByProperties[0]
            }
        ];
        sortObject.direction = sortByProperties[1];
        sortObject.type = "value";

        return sortObject;
    },

    /**
     * Call this function when a crosstab is deleted with the id of the crosstab that was deleted
     * @param crosstabId
     * @returns {boolean} true if the model was interested in the passed in crosstabId and adjusted itself.
     */
    handleCrosstabDeleted: function(crosstabId) {
        var self = this;
        if (!self.linkedToCrosstab || crosstabId !== self.crosstabId) {
            return false;
        }
        self.crosstabId = null;
        self.linkedToCrosstab = false;
        return true;
    },


    //************************************************************************************
    // Linked Table methods
    //************************************************************************************

    /**
     * Call this function when the chart is being relinked to it's original crosstab
     */
    handleRelinkToCrosstab: function() {
        var self = this;
        console.log("TableViewerModel: handleRelinkToCrosstab called");

        // Grab the crosstab we were linked to
        var crosstab = sap.ui.getCore().byId(self.crosstabId);
        if (crosstab) {
            //Anything to do here?
        }
    },

    /**
     * @description Helper function to isolate the code to check if the given crosstabId is linked to
     * this table
     * @param {string} crosstabId
     * @returns {boolean}
     */
    isLinkedToCrosstabWithId: function(crosstabId) {
        return this.linkedToCrosstab && this.crosstabId && this.crosstabId === crosstabId;
    },

    /**
     * At various points in the lifecycle of a plot it's linked crosstab could go away.  This function consolidates
     * the update code in one place.  It should be called anytime you need to ensure you have a linked crosstab.
     * @private
     */
    _validateLinkedCrosstabState: function() {
        var self = this;

        // Safety check in case the crosstabId is null
        if (!self.crosstabId) {
            self.linkedToCrosstab = false;
            return;
        }

        // Check to make sure we can still find the crosstab
        var crosstab = sap.ui.getCore().byId(self.crosstabId);
        if (!crosstab) {
            self.crosstabId = null;
            self.linkedToCrosstab = false;
        }
    },

    /**
     * Sets the linked to crosstab flag
     * @param value true if it should be linked to it's source crosstab, false otherwise
     */
    setLinkedToCrosstab: function(value) {
        var self = this;
        self.linkedToCrosstab = value;
    },

    /**
     * Is this table currently linked to a valid crosstab.
     * @returns {boolean} true if it is linked AND the crosstab exists, false otherwise
     */
    isLinkedToCrosstab: function() {
        var self = this;
        if (self.crosstabId) {
            // Check to make sure we can still find the crosstab
            var crosstab = sap.ui.getCore().byId(self.crosstabId);
            if (!crosstab) {
                return false;
            }
            return self.linkedToCrosstab;
        }
        return false;
    },

    /**
     * Can the chart be relinked to it's originating crosstab.
     * @returns {boolean}
     */
    canLinkToCrosstab: function() {
        var self = this;
        self._validateLinkedCrosstabState();
        return !_.isUndefined(self.crosstabId) && !_.isNull(self.crosstabId);
    },

    /**
     * @description Call this when a crosstab is updated
     * @returns {boolean} true if the model was adjusted
     * @param {Object} data - contains the information about the new state of the crosstab
     */
    handleCrosstabUpdated: function(data) {
        var self = this,
            hydratedTemplateColumn,
            newTemplateColumn,
            tupleList,
            CROSSTAB_UTIL = sas.analytics.riskhp.common.riskcommons.CrosstabUtil;
        console.log("TableViewerModel: handleCrosstabUpdated called");

        // If we have a pickEntryType in the info object then something happened in the crosstab and
        // we can only respond to cell changes, otherwise we let it through (this happens in the case of
        // options changing)
        if (data.info.pickEntryType && data.info.pickEntryType !== self.PickEntryType.CROSSTAB_DATA_CELL) {
            return false;
        }
        if (self.isLinkedToCrosstabWithId(data.crosstabId)) {
            console.log("TableViewerModel: updating because linked crosstab changed");

            var crosstab = sap.ui.getCore().byId(data.crosstabId);

            if (data.selectedCellChanged === true) {
                var contextData = self.TableViewerUtil.buildTableDataObject(self.type, crosstab, data.info);

                // process the context data
                self.processContextData(contextData, false);

                // At this point we know we are going to requery so we clear out any error state so callers can start
                // from a good position
                //self.clearErrorState();

                return true;
            }
            else {
                // Not a doubleclick in a crosstab data cell, so treat as crosstab options update.
                var rc = false;
                if (self.displayHorizons !== data.info.displayHorizons) {
                    self.displayHorizons = data.info.displayHorizons;
                    rc = true;
                }
                if (self.showLabels !== data.info.showLabels) {
                    self.showLabels = data.info.showLabels;
                    rc = true;
                }
                if (JSON.stringify(self.optionsQuery) !== JSON.stringify(data.info.options)) {
                    self.optionsQuery = data.info.options;
                    self.queryData.requestData.report.options = self.optionsQuery;
                    rc = true;
                }
                if (JSON.stringify(self.filtersQuery) !== JSON.stringify(data.info.filters)) {
                    self.filtersQuery = data.info.filters;
                    self.queryData.requestData.report.filters = self.filtersQuery;
                    rc = true;
                }
                return rc;
            }
        }
        if (!self.linkedToCrosstab) {
            console.log("TableViewerModel: NOT updating because linkedToCrosstab flag was false");
        }
        else {
            console.log("TableViewerModel: NOT updating because linked crosstab was not the one this model was interested in: " + this.crosstabId);
        }
        return false;
    },


});