/*
 * (c) 2017, SAS Institute Inc.
 */

jQuery.sap.declare("sas.analytics.riskhp.common.riskcommons.table.TableViewerUtil");

jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.RiskCube");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Statistic");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Horizon");
jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");

/**
 * @name sas.analytics.riskhp.common.riskcommons.table.TableViewerUtil
 */
sas.analytics.riskhp.common.riskcommons.table.TableViewerUtil = {

    // /**
    //  * Constructs the title for the DroppableContainer and Window Manager "tile" which holds the
    //  * table. It also constructs the default options 'name' value used in the options pane.
    //  * @returns {String}
    //  */
    // generateDefaultTableTitle: function(dataSource, riskCubeId) {
    //     var value = "",
    //         riskCube;
    //
    //     if (dataSource.isSideBySide()) {
    //         riskCube = dataSource.getDataItemById(riskCubeId);
    //         //_.find(portfolioModel.getTuples()[0], function(dataItem) {
    //         //    return dataItem.getType() === sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE;
    //         //});
    //     }
    //
    //     if (riskCube) {
    //         value += (" | " + riskCube.getLabel());
    //     }
    //
    //     return sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.PortfolioDetails.defaultName.fmt", [value]);
    // },
    /**
     * @description Constructs the title for the DroppableContainer and Window Manager "tile" which holds the table.  It also constructs
     * the default options 'name' value used in the Table Options pane, since they should be the same and can be updated via the Table
     * Options pane.  The title will have the form {General plot name} | tuple 1| tuple 2 | ...
     * Note: What is being plotted will be removed from the list.
     * @param {DataSource} dataSource
     * @param {tableViewerModel} tableViewerModel
     * @returns {string}
     */
    generateDefaultTableTitle: function(dataSource, tableViewerModel) {
        var self = this;
        var usingAbsoluteHorizons = self.areHorizonsAbsolute(tableViewerModel);

        var value = "";
        var sourceColumn = tableViewerModel.getTuples()[0];
        if (sourceColumn && sourceColumn.length > 0) {
            value = "";

            // Loop over the items in the source column to build up the path string
            // Note: this assumes a RTL language - MCA
            for (var i=0; i< sourceColumn.length; i++) {
                var item = sourceColumn[i];

                // We only want to use hydrated data items because we are going to call .getLabel() so check to be sure
                if (self.isDataItem(item)) {

                    // If the item is a Horizon, and we are not using absolute horizons then we need to get the
                    // correct relative horizon so we can get the correct label
                    if (item.type === sas.analytics.riskhp.models.dataitems.Horizon.HORIZON && !usingAbsoluteHorizons){
                        item = dataSource.getRelatedRelativeHorizon(item.getId());
                    }
                    else if (item.type === sas.analytics.riskhp.models.dataitems.AbsoluteHorizon.ABSOLUTEHORIZON && !usingAbsoluteHorizons){
                        item = dataSource.getRelatedRelativeHorizon(item.getId());
                    }
                    else if (item.type === sas.analytics.riskhp.models.dataitems.RelativeHorizon.RELATIVEHORIZON && usingAbsoluteHorizons) {
                        item = dataSource.getDataItemById(item.sourceHorizonsIds[0]);
                    }

                    // Final check to make sure we have a valid DataItem and then append it to the path string
                    if (item) {
                        value = value + " | " + self.getDisplayString(item, tableViewerModel);
                    }
                }
            }
        }
        var textKey = 'HPRisk.' + tableViewerModel.type + '.defaultName.fmt';
        return sas.analytics.riskhp.utils.HprUtil.getBundle().getText(textKey, [value]);
    },

    /**
     * Is the item passed in a DataItem object.  To be a DataItem, item must be an object and have a defined function for 'getId'
     * @param item
     * @returns {boolean}
     */
    isDataItem: function(item) {
        if (_.isObject(item)) {
            return (typeof item.getId === 'function' );
        }
        return false;
    },

    /**
     * Returns true if the horizons should be displayed as absolute values versus relative values.  This is mostly
     * used for label and title determination
     * @returns {boolean}
     * @private
     */
    areHorizonsAbsolute: function(tableViewerModel) {
        var horizonOptions = tableViewerModel.displayHorizons;
        if (horizonOptions) {
            return horizonOptions === HprConstants.DISPLAY_HORIZONS_ABSOLUTE_VALUE;
        }
        return false;
    },

    /**
     * @description Given a {DataItem} and a {dataObject} uses the state of the chart model to return the string to display
     * for the given data item.
     * @param {*|DataItem} item
     * @param {dataObject} dataObject
     * @returns {string} The displayable name for the item, if it is not a DataItem then the item is returned
     */
    getDisplayString: function(item, tableViewerModel) {
        var self = this;
        if (item && (self.isDataItem(item))) {
            return item.getDisplayLabel(tableViewerModel.showLabels);
        }
        return item;
    },

    /**
     * Replaces the page size (limit) in the queryData with the specified value.
     * @param pageSize
     */
    updateRequestPageSize: function(queryData, pageSize) {
        if (!queryData.report || !queryData.report.requests || !queryData.report.options) {
            return;
        }

        // update the maxRows property--this is the one used by the midtier
        _.each(queryData.report.requests, function(request) {
            if (request && request.hasOwnProperty("maxRows")) {
                request.maxRows = pageSize;
            }
        });

        // update the NDETAILS option--this might be used by the server
        _.each(queryData.report.options, function(option) {
            if (option.name === sas.analytics.riskhp.utils.HprConstants.HPR_OPTIONS_PANE_NDETAILS_NAME) {
                option.value = pageSize;
            }
        });
    },

    /**
     * Given a tableViewerModel returns the name of the Crosstab this model is linked to.  Will return null if it couldn't
     * find the crosstab or it is not linked.  Consumers should check the model's isLinkedToCrosstab state prior to calling
     * this function.
     * @param tableViewerModel
     * @returns {(string|null)}
     */
    getLinkedCrosstabName: function(tableViewerModel) {
        if (tableViewerModel.crosstabId) {
            // Check to make sure we can still find the crosstab
            var crosstab = sap.ui.getCore().byId(tableViewerModel.crosstabId);
            if (crosstab && crosstab.context) {
                return crosstab.context.getOptionName();
            }
            return null;
        }
        return null;
    },

    buildTableDataObject: function (tableType, crosstab, info) {
        var dataObject = undefined;

        switch (tableType) {
            case HprConstants.HPR_SIMULATED_VALUES_PANEL :
                dataObject = {
                    type: HprConstants.HPR_SIMULATED_VALUES_PANEL,
                    classVariables: crosstab.context.getClassVariables(),
                    drill: crosstab.context.findRowTreePath(crosstab.context.drillnode, "name"),
                    tuples: [info.tuple],
                    displayHorizons: crosstab.context.optionDisplayHorizons.value,
                    showLabels: crosstab.context.optionShowLabels.value,
                    dataSourceId: crosstab.context.dataSourceId,
                    formats: {value: HprConstants.HPR_SIMVAL_VALUE_DEFAULT_FORMAT,
                        date: HprConstants.HPR_SIMVAL_DATE_DEFAULT_FORMAT},
                    sorts: [],
                    options: crosstab.context.buildOptionsJSON(),
                    filters: crosstab.context.getFilters(),
                    crosstabId: crosstab.context.id,
                    requests: [info.expansion]
                };
                break;
            default :
                break;
        }
        return dataObject;
    },

};
