// (c) 2017, SAS Institute Inc.

jQuery.sap.declare("sas.analytics.riskhp.controls.filters.DiscreteValueFilter");

jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.CheckBox");
jQuery.sap.require("sas.hc.m.FlexBox");
jQuery.sap.require("sas.hc.m.HBox");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.List");
jQuery.sap.require("sas.hc.m.StandardListItem");
jQuery.sap.require("sas.hc.m.SearchField");
jQuery.sap.require("sas.hc.m.ResponsiveToolbar");
jQuery.sap.require("sas.hc.m.ToolbarSpacer");

jQuery.sap.require("sas.hc.ui.layout.VerticalLayout");
jQuery.sap.require("sas.hc.ui.model.PageableJSONModel");
jQuery.sap.require("sas.hc.ui.commons.HorizontalDivider");

jQuery.sap.require("sas.ltjs.sasicu.util.SASFormatterUtil");

jQuery.sap.require("sas.analytics.riskhp.controls.filters.DateValueSearchControl");
jQuery.sap.require("sas.analytics.riskhp.controls.filters.NumericValueSearchControl");
jQuery.sap.require("sas.analytics.riskhp.controls.filters.StringValueSearchControl");
jQuery.sap.require("sas.analytics.riskhp.models.HPRiskModelManager");
jQuery.sap.require("sas.analytics.riskhp.utils.HprAPI");
jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");

jQuery.sap.require("sas.risk.common.layout.BarLayout");
jQuery.sap.require("sas.risk.common.layout.BarLayoutSpacer");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.risk.common.utils.CommonUtil");


/**
 * @name sas.analytics.riskhp.controls.filters.DiscreteValueFilter
 */
sap.ui.core.Control.extend("sas.analytics.riskhp.controls.filters.DiscreteValueFilter", {

    // pseudo-imports
    HPRiskModelManager: sas.analytics.riskhp.models.HPRiskModelManager,
    HprAPI: sas.analytics.riskhp.utils.HprAPI,
    HprConstants: sas.analytics.riskhp.utils.HprConstants,
    HprUtil: sas.analytics.riskhp.utils.HprUtil,
    CommonUtil: sas.risk.common.utils.CommonUtil,

    // constants
    VALUE_LIST_MODEL_NAME: "ValueListModel",
    SEARCH_VALUE_START: "(value,'",
    SEARCH_VALUE_END: "')",

    // vars
    totalValuesCount: 0,
    filteredValuesCount: 0,

    metadata: {
        properties: {
            width: {type: "sap.ui.core.CSSSize", defaultValue: null},
            height: {type: "sap.ui.core.CSSSize", defaultValue: null},

            dataSourceId: {type: "string"},
            riskCubeId: {type: "string"},
            filterItem: {type: "object"},
        },
        events: {
            filterChange: {}
        }
    },

    renderer: function (rm, control) {
        if (!control.getVisible()) return;

        rm.write("<div");
        rm.writeControlData(control);
        if (control.getWidth()) {
            rm.addStyle("width", control.getWidth());
        }
        if (control.getHeight()) {
            rm.addStyle("height", control.getHeight());
        }

        rm.writeStyles();
        rm.writeClasses();
        rm.write(">");

        control.addDependent(control.filterLayout);
        rm.renderControl(control.filterLayout);

        rm.write("</div>");
    },

    constructor: function (id, mSettings) {
        var self = this;

        // construct super
        sap.ui.core.Control.call(self, id, mSettings);

        // update busy delay
        self.setBusyIndicatorDelay(0);
        self.setBusy(true);

        // layout for the filter controls
        self.filterLayout = new sas.hc.ui.layout.VerticalLayout({
            width: "100%",
            content: [
                this._createTypeSelector(),
                this._createValueSelector()
            ]
        });
    },

    /**
     * If this is a paged set of values, we need to tell the midtier to clear the cache.
     */
    exit: function() {
        if (this._isPaged() !== true) {
            return;
        }
        var filterItemRep = {
            version: 1,
            type: this.getFilterItem().getType(),
            id: this.getFilterItem().getId(),
            riskCubeId: this.getRiskCubeId(),
            clearCache: true,
        };
        this.HprAPI.updateCubeValues(this.getDataSourceId(), [filterItemRep]);
    },

    /**
     * Hook to allow the filter control to do some cleanup if being
     * removed by the user.
     */
    handleFilterRemove: function() {
        // if the class variable values are paged, tell the midtier/server to destroy the cache
        if (this._isPaged() === true) {
            this.HprAPI.destroyCubeValuesCache(this.getDataSourceId(), this.getFilterItem().getId());
        }
    },

    isValid: function() {
        var isValid = true;

        var values = this.getModel().getProperty("/values");
        var includes = this.getModel().getProperty("/includes");

        if (includes === this.HprConstants.HPR_FILTER_TYPE_INCLUDE_VALUE) {
            if (!values || values.length === 0) {
                isValid = false;
            }
        } else {
            if (values && values.length > 0 && values.length === this.totalValuesCount) {
                isValid = false;
            }
        }
        return isValid;
    },

    /**
     * Save the applied values and type selection for reset
     */
    _cacheAppliedFilterData: function() {
        this.appliedValues = $.extend([], this.getModel().getProperty("/values"));
        this.appliedIncludes = this.getModel().getProperty("/includes");
    },

    /**
     * Sets the model on the control and initiates loading of the values.
     * @param model
     */
    setModel: function(model) {
        // call super
        sap.ui.core.Control.prototype.setModel.apply(this, model);

        this._cacheAppliedFilterData();

        // load the values and populate the control
        this._loadValues();
    },

    /**
     * Handler for refreshing the value tallies when changes to the filter
     * pane are applied.
     */
    handleFiltersApplied: function() {
        this._cacheAppliedFilterData();

        if (this._isPaged()) {
            this.valueList.getModel(this.VALUE_LIST_MODEL_NAME).clearData();
            this._forcePagedModelQuery();
        }
        else {
            this._loadUnpagedModelData(this.valueList.getModel(this.VALUE_LIST_MODEL_NAME));
        }
    },


    // control creation

    _createTypeSelector: function() {
        var self = this;

        // selector label
        var filterTypeLabel = new sas.hc.m.Label({
            enabled: '{filterModel>/controlsEnabled}',
            text: self.HprUtil.getBundle().getText("HPRisk.Filters.FilterType.txt")
        });

        // selector
        var filterTypesModel = new sas.risk.common.models.RiskJSONModel([
            {
                label: self.HprUtil.getBundle().getText("HPRisk.Filters.Include.txt"),
                value: self.HprConstants.HPR_FILTER_TYPE_INCLUDE_VALUE
            },
            {
                label: self.HprUtil.getBundle().getText("HPRisk.Filters.Exclude.txt"),
                value: self.HprConstants.HPR_FILTER_TYPE_EXCLUDE_VALUE
            }
        ]);

        var setFilterType = function(includes) {
            self.getModel().setProperty("/includes", includes);

            // notify the filter pane of the change
            self.fireFilterChange();
        };

        // Select controls are replaced by menuitems when the responsive toolbar becomes
        // too narrow. This custom menuitem allows us to be able to handle the events
        // when the include/exclude submenu items are pressed.
        var filterTypeOverflowMenuItem = new sas.hc.m.MenuItem({
            text: self.HprUtil.getBundle().getText("HPRisk.Filters.FilterType.menu.action.label"),
            enabled: "{filterModel>/enableControls}",
            items: [
                new sas.hc.m.MenuItem({
                    text: self.HprUtil.getBundle().getText("HPRisk.Filters.Include.txt"),
                    selected: {
                        path: "/includes",
                        formatter: function(includes) {
                            return includes;
                        }
                    },
                    checkable: true,
                    press: function() {
                        setFilterType(self.HprConstants.HPR_FILTER_TYPE_INCLUDE_VALUE);
                    }
                }),

                new sas.hc.m.MenuItem({
                    text: self.HprUtil.getBundle().getText("HPRisk.Filters.Exclude.txt"),
                    selected: {
                        path: "/includes",
                        formatter: function(includes) {
                            return !includes;
                        }
                    },
                    checkable: true,
                    press: function() {
                        setFilterType(self.HprConstants.HPR_FILTER_TYPE_EXCLUDE_VALUE);
                    }
                })
            ]
        });

        var filterTypeSelect = new sas.hc.m.Select({
            items: {
                path: 'filterTypes>/',
                factory: function (sId, oContext) {
                    var filterType = oContext.getObject();

                    // Has to be sas.hc.ui.core.ListItem for ResponsiveToolBar's use of Select component
                    return new sas.hc.ui.core.ListItem({
                        key: filterType.value,
                        text: filterType.label
                    })
                }
            },
            enabled: "{filterModel>/enableControls}",
            selectedKey: "{/includes}",
            change: function(evt) {
                setFilterType (evt.getParameter("selectedItem").getKey() === "true");
            },
            menuItem: filterTypeOverflowMenuItem
        });
        filterTypeSelect.setModel(filterTypesModel, 'filterTypes');

        return new sas.hc.m.ResponsiveToolbar({
            content: [
                filterTypeLabel,
                new sas.hc.m.ToolbarSpacer(),
                filterTypeSelect
            ]
        });
    },

    _createValueSelector: function() {
        return new sas.hc.ui.layout.VerticalLayout({
            width: "100%",
            content: [
                this._createSearchHeader(),
                new sas.hc.ui.commons.HorizontalDivider().addStyleClass("hprFiltersHorizDivider"),
                this._createValueList(),
                new sas.hc.ui.commons.HorizontalDivider().addStyleClass("hprFiltersHorizDivider"),
                this._createValueListFooter(),
                this._createShowAllSelectionsButton()
            ]
        });
    },

    _createSearchHeader: function() {
        var self = this;
        var filterItemDataType = self.getFilterItem().getDataType();

        if (filterItemDataType === self.HprConstants.HPR_DATA_TYPE_NUMBER) {
            self.searchControl = new sas.analytics.riskhp.controls.filters.NumericValueSearchControl({
                enabled: "{filterModel>/enableControls}",
                filterItem: self.getFilterItem(),
                includeMissingValues: true,
                search: $.proxy(self._applyNumericSearch, self)
            });
        }
        else if (filterItemDataType === self.HprConstants.HPR_DATA_TYPE_DATE) {
            self.searchControl = new sas.analytics.riskhp.controls.filters.DateValueSearchControl({
                enabled: "{filterModel>/enableControls}",
                filterItem: self.getFilterItem(),
                includeMissingValues: true,
                search: $.proxy(self._applyNumericSearch, self)
            });
        }
        else {
            self.searchControl = new sas.analytics.riskhp.controls.filters.StringValueSearchControl({
                enabled: "{filterModel>/enableControls}",
                filterItemLabel: self.getFilterItem().getLabel(),
                search: $.proxy(self._applyStringSearch, self)
            });
        }

        return self.searchControl;
    },

    _applyStringSearch: function(evt) {
        var self = this;

        self.searchProperties = evt.getParameters();

        // apply the search
        if (self._isPaged()) {
            self.valueList.getModel(self.VALUE_LIST_MODEL_NAME).clearData();
            self._forcePagedModelQuery();
        }
        else {
            self.valueList.getBinding("items").filter(new sap.ui.model.Filter("value", sap.ui.model.FilterOperator.Contains, self.searchProperties.values ? self.searchProperties.values[0] : ""));
            self.filteredValuesCount = self.valueList.getItems().length;
            self._updateCheckedValuesCountLabel();
        }
    },

    _applyNumericSearch: function(evt) {
        var self = this;

        self.searchProperties = evt.getParameters();

        if (self._isPaged()) {
            self.valueList.getModel(self.VALUE_LIST_MODEL_NAME).clearData();
            self._forcePagedModelQuery();
        }
        else {
            var filteredValuesCount = 0,
                isMatchedValue = false;
            self.valueList.getBinding("items").filter(new sap.ui.model.Filter({
                path: "value",
                test: function (itemValue) {
                    if (!itemValue || Number.isNaN(Number(itemValue))) {
                        if (self.searchProperties.includeMissingValues === true) {
                            filteredValuesCount++;
                            return true;
                        }
                        else {
                            return false;
                        }
                    }
                    isMatchedValue = !self.searchProperties.values || Number(itemValue) >= self.searchProperties.values[0] && Number(itemValue) <= self.searchProperties.values[1];
                    if (isMatchedValue) {
                        filteredValuesCount++;
                    }
                    return isMatchedValue;
                }
            }));
            self.filteredValuesCount = filteredValuesCount;
            self._updateCheckedValuesCountLabel();
        }
    },

    _createValueList: function() {
        var self = this;

        self.valueList = new sas.hc.m.List({
            scrolling: true,
            enabled: "{filterModel>/enableControls}",
            mode: sap.m.ListMode.MultiSelect,
            rememberSelections: false,
            selectionChange: function(evt) {
                var listItem = evt.getParameter('listItem');
                var value = listItem.getCustomData()[0].getValue();
                var dataType = self.getFilterItem().getDataType();
                if (dataType === self.HprConstants.HPR_DATA_TYPE_NUMBER || dataType === self.HprConstants.HPR_DATA_TYPE_DATE) {
                    value = Number(value).toString();   // S1390667--represent numeric values consistently
                }
                var filterValues = self.getModel().getProperty("/values") || [];


                if (listItem.getSelected() === true) {
                    filterValues.push(value);
                }
                else {
                    filterValues = _.reject(filterValues, function(filterValue) {
                        return filterValue === value;
                    });
                }

                self.getModel().setProperty("/values", filterValues);
                self._updateCheckedValuesCountLabel();

                self.fireFilterChange();
            },

            // 5 rows (2rem per row)
            height: "10rem",
            maxHeight: "10rem"
        });
        self.valueList.addStyleClass("hprFiltersClassVarValueList");

        return self.valueList;
    },

    _createValueListFooter: function() {
        var self = this;

        this.checkedValuesCountLabel = new sas.hc.m.Label({
            enabled: "{filterModel>/enableControls}"
        });

        var resetButton = new sas.hc.m.Button({
            text: this.HprUtil.getBundle().getText("HPRisk.Filters.ResetSelections.txt"),
            enabled: "{filterModel>/enableControls}",
            design: sas.hc.m.Button.Design.LightweightNeutral,
            press: function(evt) {
                self.getModel().setProperty("/includes", self.appliedIncludes);
                self.getModel().setProperty("/values", $.extend([], self.appliedValues));
                self._updateCheckedValuesCountLabel();

                // notify the filter pane of the change
                self.fireFilterChange();
            }
        });

        return new sas.hc.m.ResponsiveToolbar({
            content: [
                this.checkedValuesCountLabel,
                new sas.hc.m.ToolbarSpacer(),
                resetButton
            ]
        }).addStyleClass("hprFilterFooter");
    },

    _createShowAllSelectionsButton: function() {
        var self = this;
        return new sas.hc.m.CheckBox({
            text: self.HprUtil.getBundle().getText("HPRisk.Filters.OnlySelected.tooltip.txt"),
            tooltip: self.HprUtil.getBundle().getText("HPRisk.Filters.OnlySelected.tooltip.txt"),
            enabled: "{filterModel>/enableControls}",
            select: function(evt) {
                var searchFilters = [];
                var filterItem = self.getFilterItem();
                var dataType = filterItem.getDataType();

                //if (evt.getSource().getPressed() === true) {
                if (evt.getSource().getSelected() === true) {
                    _.each(self.getModel().getProperty("/values"), function (filterValue) {
                        if (self._isPaged()) {
                            searchFilters.push(new sap.ui.model.Filter("value", sap.ui.model.FilterOperator.EQ, filterValue));
                        }
                        else {
                            // S1390667: provide custom filter function since numeric values are sent up as strings
                            searchFilters.push(new sap.ui.model.Filter({
                                path: "value",
                                test: function (itemValue) {
                                    if (dataType === self.HprConstants.HPR_DATA_TYPE_NUMBER || dataType === self.HprConstants.HPR_DATA_TYPE_DATE) {
                                        return Number(itemValue) === Number(filterValue);
                                    }
                                    // S1403303: itemValue is upper cased by the filtering framework (FilterProcessor.js, line 105)--upcase both to be safe
                                    return (itemValue ? itemValue.toUpperCase() : itemValue) === (filterValue ? filterValue.toUpperCase() : filterValue);
                                }
                            }));
                        }
                    });

                    // disable and clear text search
                    self.searchControl.setEnabled(false);
                }
                else {
                    self.searchControl.setEnabled(true);
                }

                self.valueList.getBinding("items").filter(searchFilters);
            }
        });
    },

    _loadValues: function() {
        var self = this;
        var filterItem = self.getFilterItem();

        var valueItemTemplate = new sas.hc.m.StandardListItem({
            title: {
                parts: [
                    {path: self.VALUE_LIST_MODEL_NAME + ">value"}
                ],
                formatter: function(value) {
                    if (!value) {
                        return "";
                    }

                    // if date, apply formatter
                    switch (filterItem.getDataType()) {
                        case self.HprConstants.HPR_DATA_TYPE_DATE:
                            return sas.ltjs.sasicu.util.SASFormatterUtil.format(Number(value), filterItem.getFormat());
                        case self.HprConstants.HPR_DATA_TYPE_NUMBER:
                            return sas.ltjs.sasicu.util.SASFormatterUtil.format(Number(value), filterItem.getFormat());
                        default:
                            return value;
                    }
                }
            },
            info: {
                parts: [
                    {path: self.VALUE_LIST_MODEL_NAME + ">filteredTallies"},
                    {path: self.VALUE_LIST_MODEL_NAME + ">tallies"}
                ],
                formatter: function (filteredTallies, tallies) {
                    // these can be undefined/null with the fix from commons for scrolling and page loading (see S1383761)
                    if (!filteredTallies && filteredTallies !== 0 && !tallies && tallies !== 0) {
                        return "";
                    }
                    else if (filteredTallies < tallies) {
                        return self.HprUtil.getBundle().getText("HPRisk.Filters.ValueCountFiltered.fmt", [filteredTallies, tallies]);
                    }
                    return String(tallies);
                }
            },
            selected: {
                parts: [
                    {path: self.VALUE_LIST_MODEL_NAME + ">value"},
                    {path: "/values"}
                ],
                formatter: function(value, filterValues) {
                    var dataType = filterItem.getDataType();
                    var valueIndex = _.findIndex(filterValues, function(filterValue) {
                        if (dataType === self.HprConstants.HPR_DATA_TYPE_NUMBER || dataType === self.HprConstants.HPR_DATA_TYPE_DATE) {
                            return Number(value) === Number(filterValue);
                        }
                        return value === filterValue;
                    });
                    return valueIndex !== -1;
                }
            }
        });
        // Keep the original value of the filter for selection handling
        var oDataTemplate = new sap.ui.core.CustomData({key:"value"});
        oDataTemplate.bindProperty("value", self.VALUE_LIST_MODEL_NAME + ">value");
        valueItemTemplate.addCustomData(oDataTemplate);

        // Set the count of class var values. Once the count is set,
        // we can create the appropriate model for the list (paged
        // or unpaged).
        var _setValueCount = function(valueCount) {
            self.totalValuesCount = valueCount;
            self.filteredValuesCount = valueCount;
            self.valueList.setModel(self._createValueListModel(), self.VALUE_LIST_MODEL_NAME);
            self.valueList.bindAggregation("items", self.VALUE_LIST_MODEL_NAME + ">/values", valueItemTemplate);
            self._updateCheckedValuesCountLabel();
        };

        if (filterItem.getType() !== self.HprConstants.CLASS_VARIABLE_DATA_ITEM_TYPE || filterItem.updateCount === true) {
            // create the object to send to midtier for class var value count query
            var filterItemRep = {
                version: 1,
                type: filterItem.getType(),
                id: filterItem.getId(),
                countOnly: true
            };

            // get the count of the class variable values then set the model for the list
            self.HprAPI.updateCubeValues(self.getDataSourceId(), [filterItemRep]).done(
                function (data) {
                    _setValueCount(data.items[0].count);

                    // if a class variable, notify data pane of count
                    if (filterItem.getType() === self.HprConstants.CLASS_VARIABLE_DATA_ITEM_TYPE) {
                        sas.eventBus.publish(self.HprConstants.CHANNEL_EXPLORATION, self.HprConstants.EVENT_DATA_ITEM_COUNT_UPDATED, {
                            dataSourceId: self.getDataSourceId(),
                            dataItemId: data.items[0].id,
                            dataItemCount: data.items[0].count
                        });
                    }
                }
            ).fail(
                function (jqxhr) {
                    //TODO log? notify?
                }
            );
        }
        else {
            _setValueCount(filterItem.count);
        }
    },

    _createValueListModel: function() {
        if (this._isPaged() === true) {
            return this._createPagedModel();
        }
        else {
            return this._createUnpagedModel();
        }
    },

    _createUnpagedModel: function() {
        var valueListModel = new sas.risk.common.models.RiskJSONModel({});
        this._loadUnpagedModelData(valueListModel);
        return valueListModel;
    },

    _loadUnpagedModelData: function(model) {
        var self = this;

        // create the object to send to midtier for class var value count query
        var filterItemRep = {
            version: 1,
            type: self.getFilterItem().getType(),
            id: self.getFilterItem().getId(),
            riskCubeId: self.getRiskCubeId(),
            countOnly: false,
            start: 0,
            limit: self.totalValuesCount
        };

        // get the count of the class variable values then set the model for the list
        this.setBusy(true);
        this.HprAPI.updateCubeValues(self.getDataSourceId(), [filterItemRep]).done(
            function(data) {
                var filterItemDataType = self.getFilterItem().getDataType();
                if (filterItemDataType === self.HprConstants.HPR_DATA_TYPE_NUMBER || filterItemDataType === self.HprConstants.HPR_DATA_TYPE_DATE) {
                    self._setSearchControlMinMax(data.items[0].min, data.items[0].max);
                }
                model.setProperty("/values", data.items[0].values);
            }
        ).fail(
            function(jqxhr) {
                //TODO
            }
        ).always(
            function() {
                self.setBusy(false);
            }
        );
    },

    _createPagedModel: function() {
        var self = this;
        var filterItemDataType = self.getFilterItem().getDataType();

        var valueListModel = new sas.hc.ui.model.PageableJSONModel({
            url: self.HprConstants.HPR_BASE_CUBES_URL + '/' + encodeURIComponent(self.getDataSourceId()) + "/values?" + self.HprAPI.GUID_PARAM_STRING,
            bindId: "values",
            cacheSize: -1,
            pageSize: self.HprConstants.HPR_FILTERS_PANE_VALUES_PAGE_SIZE
        });

        // process successful request
        var searchResultsPaged = false;
        valueListModel.attachRequestCompleted(function(evt) {
            var data = evt.getParameter("json");
            if (data) {
                // set min/max on search control if number or date
                if (filterItemDataType === self.HprConstants.HPR_DATA_TYPE_NUMBER || filterItemDataType === self.HprConstants.HPR_DATA_TYPE_DATE) {
                    self._setSearchControlMinMax(data.min, data.max);
                }

                // update the filtered total for the footer
                self.filteredValuesCount = data.count;
                self._updateCheckedValuesCountLabel();
            }

            self.setBusy(false);

            // throw up message dialog to inform user results exceed page size
            if (searchResultsPaged) {
                sas.hc.m.MessageDialog.warning({
                    text: self.HprUtil.getBundle().getText("HPRisk.Filters.SearchedValuesExceedLimit.fmt", [self.HprConstants.HPR_FILTERS_PANE_VALUES_PAGE_SIZE]),
                    title: self.HprUtil.getBundle().getText("HPRisk.Filters.SearchedValuesExceedLimit.Title.txt")
                });
            }
        });

        // handle request failure
        valueListModel.attachRequestFailed(function(evt) {
            //TODO set zero state or notify user?
            self.setBusy(false);
        });

        // create the request data template
        var requestOptionsDataTemplate = {
            version: "1",
            type: self.getFilterItem().getType(),
            id: self.getFilterItem().getId(),
            riskCubeId: self.getRiskCubeId(),
            countOnly: false
        };

        // define an intercept function to modify the request
        valueListModel.fnUpdateUrlOptions = function(requestOptions) {
            self.setBusy(true);

            // extract the model-defined request data for processing
            var originalRequestData = requestOptions.data;

            // for HPR midtier, request data is an array
            requestOptions.data = [];

            // clone the request data template and include start and limit
            var requestOptionsData = $.extend(true, {start: originalRequestData.start, limit: originalRequestData.limit}, requestOptionsDataTemplate);

            // determine if a search filter is to be applied, and if so, build up the request accordingly
            if (originalRequestData.filter) {
                // this filter came from a clicking the footer button to reduce to selected items only
                var searchValues = self.getModel().getProperty("/values");
                if (searchValues && searchValues.length > 0) {
                    if (filterItemDataType === self.HprConstants.HPR_DATA_TYPE_NUMBER || filterItemDataType === self.HprConstants.HPR_DATA_TYPE_DATE) {
                        _.each(searchValues, function (searchValue) {
                            requestOptions.data.push($.extend(true, {search: [{value: Number(searchValue), type: "equals"}]}, requestOptionsData));
                        });
                    }
                    else {
                        _.each(searchValues, function (searchValue) {
                            requestOptions.data.push($.extend(true, {search: [{value: searchValue, type: "exact"}]}, requestOptionsData));
                        });
                    }
                }
            }
            else if (self.searchProperties && self.searchProperties.values) {
                // this search came from a user entering text or selecting a range
                requestOptionsData.includeMissingValues = self.searchProperties.includeMissingValues;

                requestOptionsData.search = [];
                if (filterItemDataType === self.HprConstants.HPR_DATA_TYPE_NUMBER || filterItemDataType === self.HprConstants.HPR_DATA_TYPE_DATE) {
                    requestOptionsData.search.push({value: self.searchProperties.values[0], type: "greaterThanOrEqual"});
                    requestOptionsData.search.push({value: self.searchProperties.values[1], type: "lessThanOrEqual"});
                }
                else {
                    requestOptionsData.search.push({value: self.searchProperties.values[0], type: "contains"});
                }

                requestOptions.data.push(requestOptionsData);
            }
            else {
                requestOptions.data.push(requestOptionsData);
            }

            requestOptions.type = "POST";
            if (!requestOptions.headers) {
                requestOptions.headers = {};
            }
            requestOptions.headers["Accept-Type"] = "application/vnd.sas.collection+json;charset=utf-8";
            requestOptions.headers["Accept-Item"] = "application/vnd.sas.risk.values+json;charset=utf-8";
            requestOptions.headers["Content-Type"] = "application/json";
            requestOptions.headers["Content-Item"] = "application/vnd.sas.risk.values.parms+json;charset=utf-8";

            // process/parse data returned from the server
            requestOptions.dataFilter = function(jsonString, type) {
                if (jsonString) {
                    var data = JSON.parse(jsonString);

                    // update var for tracking search result paging (triggers warning to user if true)
                    searchResultsPaged = JSON.parse(jsonString).items[0].searchResultsPaged === true;

                    // translate the data for model consumption
                    return JSON.stringify(self.HprUtil.parseFilterValuesData(data.items));
                }

                // no data returned, so set an empty model
                return JSON.stringify({values: []});
            };

            console.log("PAGED REQUEST: " + JSON.stringify(requestOptions));
        };

        return valueListModel;
    },

    _updateCheckedValuesCountLabel: function() {
        var filterValues = this.getModel().getProperty("/values");
        var selectedValueCount = filterValues ? filterValues.length : 0;
        this.checkedValuesCountLabel.setText(this.HprUtil.getBundle().getText("HPRisk.Filters.SelectedCount.fmt", [selectedValueCount, this.filteredValuesCount]));
    },

    _isPaged: function() {
        return !!(this.totalValuesCount && this.totalValuesCount >= this.HprConstants.HPR_FILTERS_PANE_VALUES_PAGE_SIZE);
    },

    _isUserSearch: function(searchExpression) {
        return searchExpression && searchExpression.includes("contains");
    },

    /**
    * FIXME workaround until 8.0--see S1394540
    * Force the list to requery the items (normally this would happen automatically after clearing the model data).
    * @private
    */
    _forcePagedModelQuery: function() {
        var itemsBinding = this.valueList.getBindingInfo("items").binding;
        itemsBinding.getContexts(1,1);
        this.valueList.invalidate();
    },

    _setSearchControlMinMax: function(min, max) {
        if (this.searchControl.getMin() !== min) {
            this.searchControl.setMin(min);
        }
        if (this.searchControl.getMax() !== max) {
            this.searchControl.setMax(max);
        }
        if (!this.searchControl.getRangeValues()) {
            this.searchControl.setRangeValues([min, max]);
        }
    }
});