// (c) 2017, SAS Institute Inc.

jQuery.sap.declare("sas.analytics.riskhp.controls.filters.PlotFiltersContainer");

jQuery.sap.require("sas.hc.m.List");
jQuery.sap.require("sas.hc.m.ResponsivePopover");
jQuery.sap.require("sas.hc.m.SearchField");
jQuery.sap.require("sas.hc.m.StandardListItem");

jQuery.sap.require("sas.analytics.riskhp.controls.filters.ClassVariableFiltersContainer");

/**
 * @name sas.analytics.riskhp.controls.filters.PlotFiltersContainer
 */
sas.analytics.riskhp.controls.filters.ClassVariableFiltersContainer.extend("sas.analytics.riskhp.controls.filters.PlotFiltersContainer", {

    metadata: {
        properties: {}
    },

    renderer: {},   // use parent renderer

    showAddFilterSelector: function(evt) {
        var self = this;

        this.DataSourceCacheManager.getDataSource(self.getDataSourceId()).done(function(dataSource) {
            var allClassVars = dataSource.getDataItemsForType(sas.analytics.riskhp.models.dataitems.Classvar.CLASSVAR);

            var classVarPopover = new sas.hc.m.ResponsivePopover({
                placement: sap.m.PlacementType.Left,
                offsetX: -16, /* 1 rem */
                offsetY: -2, /* align with center of + icon */
                modal: false,
                showHeader: false,
                showSharkfin: true,
                autoClose: true,
                contentWidth: HprConstants.HPR_FILTERS_PANE_FILTER_ITEM_POPOVER_WIDTH,
                contentHeight: "20rem"
            });
            classVarPopover.addStyleClass("hprAddFilterPopover");

            var _populateClassVariableList = function (list, classVars) {
                var listItems = [];
                _.each(classVars, function (classVarDataItem) {
                    if (classVarDataItem.isFilterable()) {
                        listItems.push({
                            id: classVarDataItem.getId(),
                            label: classVarDataItem.getDisplayLabel(self._getShowLabels()),
                            icon: classVarDataItem.getIcon()
                        });
                    }
                });
                var listModel = new sas.risk.common.models.RiskJSONModel(listItems);
                list.setModel(listModel);
                list.bindRows( "/");

                // set selections on the table for the existing filters
                var sections = self.windowShade.getSections();
                _.each(sections, function(section) {
                    var index = _.findIndex(listItems, function(item) {
                        return item.id === self._convertSectionIdToFilterId(section.getId());
                    });
                    if (index !== -1) {
                        filterItemTable.addSelectionInterval(index, index);
                    }
                });
            };

            var _handleClassVariableSelection = function (evt) {
                // if the selected row does not have a context (short list that doesn't fill the popover), no-op
                var selectedRowContext = filterItemTable.getContextByIndex(evt.mParameters.rowIndex);
                if (!selectedRowContext) {
                    return;
                }

                // get the object for the selected row
                var selectedClassVariable = selectedRowContext.getObject();

                // create the base filter
                var filter = {
                    id: selectedClassVariable.id,
                    type: "classVariable",
                    includes: false,
                    collapsed: false,
                    values: []
                };

                // add the filter to the model
                var filters = self.getFilterModel().getProperty("/filters");
                filters.push(filter);
                self.getFilterModel().setProperty("/filters", filters);

                // add the filter to the panel
                self._addFilter(filter.id, dataSource);

                // update the panel button states
                self._handleFilterChange();

                // close the popover (this does not destroy it)
                classVarPopover.close();

                // apparently, some selection processing in the popover's list still
                // occurs after the close, so use the timeout-0 trick to destroy the
                // popover after other processing is complete.
                setTimeout(function() {
                    classVarPopover.destroy();
                }, 0);
            };

            var quickSearchField = new sas.hc.m.SearchField({
                filterMode: true,
                enabled: "{filterModel>/enableControls}",
                liveChange: function (evt) {
                    var filters = [
                        // Does not need to respond to 'Show labels' option change because this is for List "items" binding
                        new sap.ui.model.Filter("label", sap.ui.model.FilterOperator.Contains, evt.getParameter("newValue"))
                    ];
                    filterItemTable.getBinding("rows").filter(filters);
                }
            });
            // Set a custom landmark label - see S1221065
            self.CommonUtil.setSearchFieldLandmarkLabel(quickSearchField, self.HprUtil.getBundle().getText("HPRisk.Filters.SearchClassVars.landmark.txt"));
            quickSearchField.addStyleClass("riskToolbarSearchField");

            var filterItemTable = new sas.hc.ui.table.Table({
                visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
                enabled: "{filterModel>/enableControls}",
                selectionMode: sas.hc.ui.table.SelectionMode.MultiToggle,
                selectionBehavior: sap.ui.table.SelectionBehavior.RowOnly,
                allowTextSelection: false,
                columnHeaderVisible: false,
                showColumnVisibilityMenu: false,
                enableColumnFreeze: false,
                toolbar: new sas.hc.m.Toolbar({content: [quickSearchField]}),
                columns: [
                    new sas.hc.ui.table.Column({
                        template: new sas.hc.m.Label({
                            text: "{label}",
                            icon: "{icon}"
                        }).addStyleClass("filterItemLabel")
                    })
                ],
                defaultAction: function(event) {
                    // Fired when a single click, single tap, and ENTER key occurs on a row.
                    var rowBindingContext = event.getParameter("rowBindingContext");
                    if (rowBindingContext) {
                        var rowObject = rowBindingContext.getModel().getProperty(rowBindingContext.getPath());
                        if (rowObject) {
                            var id = rowObject.id;
                            var filterExists = self._isInFilterPane(id);
                            // if not already a filter for this
                            if (!filterExists) {
                                _handleClassVariableSelection(event);
                            } else {
                                setTimeout(function() {
                                    // disable unselect by reselecting the row
                                    var rowIndex = event.mParameters.rowIndex;
                                    filterItemTable.addSelectionInterval(rowIndex, rowIndex);
                                },1);
                            }
                        }
                    }
                }
            }).addStyleClass("filterItemsTable");
            _populateClassVariableList(filterItemTable, allClassVars);

            classVarPopover.addContent(filterItemTable);
            classVarPopover.openBy(evt.getSource());
        });
    },

    getFilterDataItem: function(filter, dataSource) {
        return dataSource.getDataItemById(filter.id);
    },

    getAppliedFilters: function() {
        return this.HprUtil.hprClone(this.HPRiskModelManager.getPlotFiltersModel(this.getParentPanelId()).getData().filters);
    },
    
    onParentPanelOptionsNameUpdate: function(channelId, eventId, data) {
        if (this.getParentPanelId() === data.plotId) {
            this.updateParentPanelTitle(data.newPlotName);
        }
    },

    init: function() {
        // Subscribe to an event that is published when the Name field for a Crosstab is updated on the Options pane
        sas.eventBus.subscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_PLOT_OPTIONS_NAME_UPDATED, this.onParentPanelOptionsNameUpdate, this);
    },

    exit: function() {
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_PLOT_OPTIONS_NAME_UPDATED, this.onParentPanelOptionsNameUpdate, this);
    }
});