// (c) 2017, SAS Institute Inc.

jQuery.sap.declare("sas.analytics.riskhp.controls.filters.PortfolioDetailsFiltersContainer");

jQuery.sap.require("sas.hc.m.List");
jQuery.sap.require("sas.hc.m.ResponsivePopover");
jQuery.sap.require("sas.hc.m.SearchField");
jQuery.sap.require("sas.hc.m.StandardListItem");
jQuery.sap.require("sas.hc.m.Toolbar");

jQuery.sap.require("sas.analytics.riskhp.controls.filters.FiltersContainer");
jQuery.sap.require("sas.analytics.riskhp.controls.filters.RangedDateFilter");
jQuery.sap.require("sas.analytics.riskhp.controls.filters.RangedNumericFilter");
jQuery.sap.require("sas.analytics.riskhp.controls.filters.DiscreteValueFilter");

jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
jQuery.sap.require("sas.risk.common.utils.CommonUtil");

/**
 * @name sas.analytics.riskhp.controls.filters.PortfolioDetailsFiltersContainer
 */
sas.analytics.riskhp.controls.filters.FiltersContainer.extend("sas.analytics.riskhp.controls.filters.PortfolioDetailsFiltersContainer", {

    metadata: {
        properties: {}
    },

    renderer: {},   // use parent renderer

    // pseudo-imports
    HprConstants: sas.analytics.riskhp.utils.HprConstants,
    CommonUtil: sas.risk.common.utils.CommonUtil,

    setParentPanelId: function (parentPanelId) {
        // construct super
        sas.analytics.riskhp.controls.filters.FiltersContainer.prototype.setParentPanelId.call(this, parentPanelId);

        // get the portfolio details model
        this.portfolioDetailsModel = this.getPrimeModelCallback()(parentPanelId);
    },

    showAddFilterSelector: function(evt) {
        var self = this;

        this.DataSourceCacheManager.getDataSource(self.getDataSourceId()).done(function (dataSource) {

            var filterItemPopover = new sas.hc.m.ResponsivePopover({
                placement: sap.m.PlacementType.Left,
                offsetX: -16, /* 1 rem */
                offsetY: -2, /* align with center of + icon */
                modal: false,
                showHeader: false,
                showSharkfin: true,
                autoClose: true,
                contentWidth: HprConstants.HPR_FILTERS_PANE_FILTER_ITEM_POPOVER_WIDTH,
                contentHeight: "20rem"
            });
            filterItemPopover.addStyleClass("hprAddFilterPopover");

            var _populateFilterItemList = function (list, columnDescriptors) {
                var listItems = [];

                // first add the class variables from the dataSource
                var classVariables = dataSource.getDataItemsForType(sas.analytics.riskhp.models.dataitems.Classvar.CLASSVAR);
                _.each(classVariables, function(classVarDataItem) {
                    listItems.push({
                        id: classVarDataItem.getId(),
                        label: classVarDataItem.getLabel(),
                        type: self.HprConstants.CLASS_VARIABLE_DATA_ITEM_TYPE
                    })
                });

                // then add the portfolio columns that are not class variables
                _.each(columnDescriptors, function (columnDescriptor) {
                    if (!columnDescriptor.classVariableId) {
                        listItems.push({
                            id: columnDescriptor.id,
                            label: columnDescriptor.getDisplayLabel(self._getShowLabels()),
                            type: self.HprConstants.PORTFOLIO_COLUMN_DATA_ITEM_TYPE
                        })
                    }
                });
                var listModel = new sas.risk.common.models.RiskJSONModel(listItems);
                list.setModel(listModel);
                list.bindRows("/");

                // set selections on the table for the existing filters
                var sections = self.windowShade.getSections();
                _.each(sections, function(section) {
                    var index = _.findIndex(listItems, function(item) {
                        return item.id === self._convertSectionIdToFilterId(section.getId());
                    });
                    if (index != -1) {
                        filterItemTable.addSelectionInterval(index, index);
                    }
                });
            };

            var _handleFilterItemSelection = function (evt) {
                // if the selected row does not have a context (short list that doesn't fill the popover), no-op
                var selectedRowContext = filterItemTable.getContextByIndex(evt.mParameters.rowIndex);
                if (!selectedRowContext) {
                    return;
                }

                // get the object for the selected row
                var selectedFilterItem = selectedRowContext.getObject();

                // create the base filter
                var filter = {
                    id: selectedFilterItem.id,
                    type: selectedFilterItem.type,
                    collapsed: false
                };

                // if the filter is for string values, add the include boolean and values array
                var filterDataItem = self.getFilterDataItem(selectedFilterItem, dataSource);
                if (filterDataItem.getDataType() === self.HprConstants.HPR_DATA_TYPE_STRING || filter.type === self.HprConstants.CLASS_VARIABLE_DATA_ITEM_TYPE) {
                    filter.includes = false;
                    filter.values = [];
                }

                // default to include missing values for numeric (ranged) filters
                if (filterDataItem.getDataType() !== self.HprConstants.HPR_DATA_TYPE_STRING) {
                    filter.includeMissingValues = true;
                }

                // add the filter to the model
                var filters = self.getFilterModel().getProperty("/filters");
                filters.push(filter);
                self.getFilterModel().setProperty("/filters", filters);

                // add the filter to the panel
                self._addFilter(filter.id, dataSource);

                // update the panel button states
                self._handleFilterChange();

                // close the popover (this does not destroy it)
                filterItemPopover.close();

                // apparently, some selection processing in the popover's list still
                // occurs after the close, so use the timeout-0 trick to destroy the
                // popover after other processing is complete.
                setTimeout(function () {
                    filterItemPopover.destroy();
                }, 0);
            };

            var quickSearchField = new sas.hc.m.SearchField({
                filterMode: true,
                liveChange: function (evt) {
                    var filters = [
                        // Does not need to respond to 'Show labels' option change because this is for List "items" binding
                        new sap.ui.model.Filter("label", sap.ui.model.FilterOperator.Contains, evt.getParameter("newValue"))
                    ];
                    filterItemTable.getBinding("rows").filter(filters);
                }
            });
            // Set a custom landmark label - see S1221065
            self.CommonUtil.setSearchFieldLandmarkLabel(quickSearchField, self.HprUtil.getBundle().getText("HPRisk.Filters.SearchPortfolioColumn.landmark.txt"));
            quickSearchField.addStyleClass("riskToolbarSearchField");

            // filter item variable list
            var columnDescriptors = self.portfolioDetailsModel.getColumnDescriptors();
            var filterItemTable = new sas.hc.ui.table.Table({
                visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
                selectionMode: sas.hc.ui.table.SelectionMode.MultiToggle,
                selectionBehavior: sap.ui.table.SelectionBehavior.RowOnly,
                allowTextSelection: false,
                columnHeaderVisible: false,
                showColumnVisibilityMenu: false,
                enableColumnFreeze: false,
                toolbar: new sas.hc.m.Toolbar({content: [quickSearchField]}),
                columns: [
                    new sas.hc.ui.table.Column({
                        template: new sas.hc.m.Label({
                            text: "{label}",
                            icon: "{icon}"
                        }).addStyleClass("filterItemLabel")
                    })
                ],
                defaultAction: function(event) {
                    // Fired when a single click, single tap, and ENTER key occurs on a row.
                    var rowBindingContext = event.getParameter("rowBindingContext");
                    if (rowBindingContext) {
                        var rowObject = rowBindingContext.getModel().getProperty(rowBindingContext.getPath());
                        if (rowObject) {
                            var id = rowObject.id;
                            var filterExists = self._isInFilterPane(id);
                            // if not already a filter for this
                            if (!filterExists) {
                                _handleFilterItemSelection(event);
                            } else {
                                setTimeout(function() {
                                    // disable unselect by reselecting the row
                                    var rowIndex = event.mParameters.rowIndex;
                                    filterItemTable.addSelectionInterval(rowIndex, rowIndex);
                                },1);
                            }
                        }
                    }
                }
            }).addStyleClass("filterItemsTable");
            _populateFilterItemList(filterItemTable, columnDescriptors);

            filterItemPopover.addContent(filterItemTable);
            filterItemPopover.openBy(evt.getSource());
        });
    },

    /**
     * Creates the control to place in the windowshade section.
     * @param dataSource
     * @param filter
     * @returns {sap.ui.core.Control}
     */
    createFilterControl: function(dataSource, filter) {
        if (filter.type === this.HprConstants.CLASS_VARIABLE_DATA_ITEM_TYPE) {
            var classVar = dataSource.getDataItemById(filter.id);
            return new sas.analytics.riskhp.controls.filters.DiscreteValueFilter({
                dataSourceId: dataSource.getCubeId(),
                riskCubeId: this.portfolioDetailsModel.getRiskCubeId(),
                filterItem: classVar,
                filterChange: $.proxy(this._handleFilterChange, this)
            });
        } else {

            // if not class variable column, then determine the appropriate filter based on data type
            var columnDescriptor = this.portfolioDetailsModel.getColumnDescriptor(filter.id);
            switch (columnDescriptor.getDataType()) {
                case this.HprConstants.HPR_DATA_TYPE_NUMBER:
                    return new sas.analytics.riskhp.controls.filters.RangedNumericFilter({
                        dataSourceId: dataSource.getCubeId(),
                        riskCubeId: this.portfolioDetailsModel.getRiskCubeId(),
                        filterItem: columnDescriptor,
                        filterChange: $.proxy(this._handleFilterChange, this)
                    });
                case this.HprConstants.HPR_DATA_TYPE_DATE:
                    return new sas.analytics.riskhp.controls.filters.RangedDateFilter({
                        dataSourceId: dataSource.getCubeId(),
                        riskCubeId: this.portfolioDetailsModel.getRiskCubeId(),
                        filterItem: columnDescriptor,
                        filterChange: $.proxy(this._handleFilterChange, this)
                    });
                default:
                    return new sas.analytics.riskhp.controls.filters.DiscreteValueFilter({
                        dataSourceId: dataSource.getCubeId(),
                        riskCubeId: this.portfolioDetailsModel.getRiskCubeId(),
                        filterItem: columnDescriptor,
                        filterChange: $.proxy(this._handleFilterChange, this)
                    });
            }
        }
    },

    /**
     * Returns the data item (column descriptor) that the filter is based upon.
     * @param filter
     * @returns {sas.analytics.riskhp.models.dataitems.BaseDataItem|*}
     */
    getFilterDataItem: function(filter, dataSource) {
        if (filter.type === this.HprConstants.CLASS_VARIABLE_DATA_ITEM_TYPE) {
            return dataSource.getDataItemById(filter.id);
        }
        return this.portfolioDetailsModel.getColumnDescriptor(filter.id);
    },

    getAppliedFilters: function() {
        return this.HprUtil.hprClone(this.HPRiskModelManager.getPortfolioDetailsFiltersModel(this.getParentPanelId()).getData().filters);
    },

    onParentPanelOptionsNameUpdate: function(channelId, eventId, data) {
        if (this.getParentPanelId() === data.portfolioDetailsId) {
            this.updateParentPanelTitle(data.newPortfolioDetailsName);
        }
    },

    init: function() {
        // Subscribe to an event that is published when the Name field for a Crosstab is updated on the Options pane
        sas.eventBus.subscribe(this.HprConstants.CHANNEL_OPTIONS_PANE, this.HprConstants.EVENT_PORTFOLIO_DETAILS_OPTIONS_NAME_UPDATED, this.onParentPanelOptionsNameUpdate, this);
    },

    exit: function() {
        sas.eventBus.unsubscribe(this.HprConstants.CHANNEL_OPTIONS_PANE, this.HprConstants.EVENT_PORTFOLIO_DETAILS_OPTIONS_NAME_UPDATED, this.onParentPanelOptionsNameUpdate, this);
    }
});