// (c) 2017, SAS Institute Inc.

jQuery.sap.require('sas.analytics.riskhp.utils.HprAPI');
jQuery.sap.require('sas.analytics.riskhp.utils.HprConstants');
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.analytics.riskhp.utils.ViewManager");
jQuery.sap.require("sas.analytics.riskhp.models.DataSourceCacheManager");
jQuery.sap.require("sas.analytics.riskhp.models.DataSourceParams");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.risk.common.utils.CommonConstants");

/**
 * Choose Data Source dialog.  Depending on the mode of the dialog it is used to create a new Exploration, replace Data Source, and add Data Source.
 * @name sas.analytics.riskhp.fragments.ChooseDataSourceDialog
 */
sap.ui.jsfragment("sas.analytics.riskhp.fragments.ChooseDataSourceDialog", {

    createContent: function(ctrl) {
        var self = this;
        var controller = ctrl;
        this.dataSourceParams = null; // This is a copy of the DataSourceParams stored in the DataSourceCacheManager
        this.dialogMode = null;
        this.liveChangeTimer = 0;
        this.fileCubesCount = 0; // This is the number of 'file' cubes returned in the TreeTable data which will be displayed to the user

        // Name field label
        this.nameFieldLabel = new sas.hc.m.Label(ctrl.createId("nameFieldLabel"), {
            text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("ChooseDataSourceDialog.nameFieldLabel.txt")
        }).addStyleClass("sasHPRiskCommon-largeFontSize");

        // Name field
        this.nameField = new sas.hc.m.Input(ctrl.createId("nameField"), {
            width: "400px",
            type: sap.m.InputType.Text,
            liveChange: $.proxy(self._onNameFieldLiveChange, self)
        });

        this.oldCubeNameLabel = new sas.hc.m.Label(ctrl.createId("oldCubeNameLabel"), {}).addStyleClass("sasHPRiskCommon-largeFontSizeBold");

        // Search field label
        var searchFieldLabel = new sas.hc.m.Label(ctrl.createId("searchFieldLabel"), {
            text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("ChooseDataSourceDialog.searchFieldLabel.txt")
        }).addStyleClass("sasHPRiskCommon-largeFontSize sasHPRiskCommon-topMargin");

        // Search field
        this.searchField = new sas.hc.m.SearchField(ctrl.createId("searchField"), {
            filterMode: true,
            liveChange: $.proxy(self._onSearchLiveChange, self) // Filter results when user types after slight delay
        });

        // Data source table refresh button
        this.refreshButton = new sas.hc.m.Button(ctrl.createId("refreshButton"), {
            icon: sas.icons.HC.REFRESH,
            press: function() {
                onRefresh();
            }
        });

        // Data Source table
        this.dataSourceTable = new sas.hc.ui.table.TreeTable(ctrl.createId("dataSourceTable"), {
            selectionMode: sas.hc.ui.table.SelectionMode.Single,
            selectionBehavior: sas.hc.ui.table.SelectionBehavior.RowOnly,
            toolbar: new sas.hc.m.ResponsiveToolbar({content: [self.searchField, new sas.hc.m.ToolbarSpacer(), self.refreshButton]}),
            visibleRowCount: 9,
            showTableOptions: false,
            editable: false,
            expandFirstLevel: false,
            enableColumnReordering: false,
            enableColumnFreeze: false,
            rowSelectionChange: $.proxy(self._onDataSourceTableRowSelectionChange, self),
            columns: [
                new sas.hc.ui.table.Column({
                    label: new sas.hc.m.Label({
                        text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("ChooseDataSourceDialog.DataSourceTableNameColumn.txt")
                    }),
                    template: new sas.hc.m.HBox({
                        items: [
                            new sas.hc.ui.core.Icon({
                                size: "1rem",
                                src: sas.icons.HC.OLAPCUBE,
                                visible: {
                                    path: "nodeType",
                                    formatter: function(nodeType) {
                                        return nodeType === "file";
                                    }
                                }
                            }).addStyleClass("sasHPRiskCommon-rightMargin"),
                            new sas.hc.m.Text({text: {path: "label"}})
                        ],
                        alignItems: sap.m.FlexAlignItems.Center
                    }),
                    sortProperty: "label",
                    width: "70%"
                }),
                new sas.hc.ui.table.Column({
                    label: new sas.hc.m.Label({
                        text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("ChooseDataSourceDialog.DataSourceTableLastUpdatedColumn.txt")
                    }),
                    template: new sas.hc.m.Text({
                        text: {
                            path: "lastModifiedTimeStamp",
                            formatter: sas.analytics.riskhp.utils.HprUtil.formatDateTime
                        }
                    }),
                    sortProperty: "lastModifiedTimeStamp",
                    width: "30%"
                })
            ]
        });

        // Count for data sources
        this.dataSourceCountText = new sas.hc.m.Text(ctrl.createId("dataSourceCountText"), {});

        // Data Source select label
        var dataSourceAreaSelectLabel = new sas.hc.m.Label(ctrl.createId("dataSourceAreaSelectLabel"), {
            text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("ChooseDataSourceDialog.data.source.select.label.txt")
        }).addStyleClass("sasHPRiskCommon-largeFontSize");

        // Select drop-down for allowing choice of data source type
        this.dataSourceAreaSelect = new sas.hc.m.Select(ctrl.createId("dataSourceAreaSelect"), {
            width: "300px",
            enabled: false,
            placeHolder: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("ChooseDataSourceDialog.data.source.select.placeHolder.txt"),
            required: true,
            enableUnselected: true,
            valueStateText: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("ChooseDataSourceDialog.data.source.select.required.txt"),
            change: $.proxy(self._onDataSourceAreaSelectChange, self)
        }).addStyleClass("sasHPRiskCommon-topMargin");

        // Vertical layout for DataSourceArea components
        this.dataSourceAreaSelectVerticalLayout = new sas.hc.ui.layout.VerticalLayout({
            content: [dataSourceAreaSelectLabel, this.dataSourceAreaSelect]
        });

        // Layout for the Data Source components
        var dataSourceFlexBox = new sas.hc.m.FlexBox({
            alignItems: sap.m.FlexAlignItems.Start,
            justifyContent: sap.m.FlexJustifyContent.SpaceBetween,
            items: [this.dataSourceCountText, this.dataSourceAreaSelectVerticalLayout]
        }).addStyleClass('chooseDataSourceDialogFlexBox');

        // Main dialog vertical layout
        var verticalLayout = new sas.hc.ui.layout.VerticalLayout(ctrl.createId("verticalLayout"), {
            //TODO (ryalbe): only include the correct fields based on the dialog mode. Right now dialog mode is used to control visibility,
            //               but name field label and input will never display with oldCubeNameLabel, and in the add case, neither is displayed.
            //               I would make that change now, but I don't know if there is code that expects all controls to be defined, even
            //               when not used, and since this work is for a hotfix, that will have to be addressed in a subsequent release.
            content: [this.nameFieldLabel, this.nameField, this.oldCubeNameLabel, searchFieldLabel, this.dataSourceTable, dataSourceFlexBox]
        });

        // Method which is called when the Cancel button is clicked
        var onCancel = function() {
            self.dialog.close();
        };

        // Code to perform when the Open button is clicked in 'addDataSource' mode
        var addDataSourceOnOpen = function() {
            var selectedRowIndex = self.dataSourceTable.getSelectedIndex();
            var dataSourceObject = self.dataSourceTable.getContextByIndex(selectedRowIndex).getObject();

            // Fall back to the DataPane.controller class when the dialog is in 'addDataSource' mode, find details
            // about the Data Source that is selected and re-populate the Data Source Select component on the controller
            controller.addDataSourceOnOpen(dataSourceObject);
        };

        // Code to perform when the Open button is clicked in 'replaceDataSource' mode
        var replaceDataSourceOnOpen = function() {
            var selectedRowIndex = self.dataSourceTable.getSelectedIndex();
            var dataSourceObject = self.dataSourceTable.getContextByIndex(selectedRowIndex).getObject();

            // Fall back to the DataPane.controller class when the dialog is in 'replaceDataSource' mode, find details
            // about the Data Source that is selected and try to show differences to the user so they can decide whether
            // they want to replace data source.  In addition, lots of other things do take place on replace data source.
            var dialogModel = self.dialog.getModel();
            var callback = dialogModel.getProperty("/openCallback");
            callback(dataSourceObject);
            // controller.replaceDataSourceOnOpen(dataSourceObject);
        };

        // Code to perform when the Open button is clicked in 'newExploration' mode
        var loadNewExplorationImpl = function() {
            var newExplorationName = self.nameField.getValue();

            // Retrieve the dataSourceDetails we want to load which is stored as CustomData on the dataSourceAreaSelect component
            var dataSourceAreaSelectedItem = self.dataSourceAreaSelect.getSelectedItem();
            var dataSourceAreaSelectedItemCustomData = dataSourceAreaSelectedItem.getCustomData();
            var dataSourceDetails = dataSourceAreaSelectedItemCustomData[0].getProperty('value');

            // Load a new Exploration and call the appropriate handler function when it is done
            var promise = sas.analytics.riskhp.utils.ViewManager.loadNewExploration(newExplorationName, dataSourceDetails, self.dataSourceAreaSelect.getSelectedKey());
            promise.done(sas.analytics.riskhp.utils.HprUtil.createSuccessHandler(controller.getView(), null));

            // This method is ran in the failure handler when it completes
            var newExplorationFailureHandlerCompletionFunction = function() {
                // Check to see if we are in ExplorationView, and if so route to WelcomeView when there is a failure
                var explorationView = sas.analytics.riskhp.utils.ViewManager.getExplorationView();
                if (explorationView) {
                    var explorationViewController = explorationView.getController();
                    explorationViewController.navChecker.setDirty(false); // Have to set navChecker dirty state to 'false' so router will navigate correctly
                    sas.analytics.riskhp.utils.ViewManager.loadHprWelcomeView();
                }
            };
            promise.fail(sas.analytics.riskhp.utils.HprUtil.createFailureHandler(controller.getView(), true, newExplorationFailureHandlerCompletionFunction));
        };

        // A generic method which is called when the Open button is clicked which performs different steps and decideds which impl. method to call
        var newExplorationOnOpen = function() {
            controller.getView().setBusy(true); // Set the parent view to busy

            // Release the currently opened cube resources
            var dialogModel = self.dialog.getModel();
            var releaseAllLoadedDataSources = dialogModel.getProperty("/releaseAllLoadedDataSources");
            // If this dialog is opened from Welcome screen, there are no currently opened cubes
            if (releaseAllLoadedDataSources) {
                $.when (
                    DataSourceCacheManager.releaseAllLoadedDataSources()
                ).done(function() {
                    loadNewExplorationImpl();
                }).fail(function() {
                    controller.getView().setBusy(false);
                    sas.analytics.riskhp.utils.HprUtil.displayGenericErrorDialog();
                });
            } else {
                loadNewExplorationImpl();
            }
        };

        // Main open method which decides the implementation method to call based on the dialog mode.  Called when the Open button for the dialog is clicked.
        var onOpen = function() {
            self.dialog.close();

            // Based on the dialog mode, decide which method to call
            if (self.dialogMode === HprConstants.HPR_CHOOSEDATASOURCE_DIALOG_ADD_DATA_SOURCE_MODE) {
                addDataSourceOnOpen();
            } else if (self.dialogMode === HprConstants.HPR_CHOOSEDATASOURCE_DIALOG_REPLACE_DATA_SOURCE_MODE) {
                replaceDataSourceOnOpen();
            } else {
                newExplorationOnOpen();
            }
        };

        /**
         * Create a TreeNode which represents a 'resource' for use in the TreeTable
         * @param resource Object which holds properties about a Cube resource to populate the TreeNode with
         * @returns {{}|null} A node object representing a resource or null if a node can not be created
         */
        var createResourceNode = function(resource, parentFolderLineage) {
            // For dialog modes of Add Data Source and Replace Data Source, do not build a resource node in
            // the TreeTable for data sources which are already represented in the DataSourceCacheManager.
            // We don't the user to be able to have the same DataSource loaded multiple times.  Do this
            // by matching up the required properties for a DataSourceParams object.  This essentially filters
            // out the Data Sources from the data in the TreeTable.
            if (self.dialogMode != HprConstants.HPR_CHOOSEDATASOURCE_DIALOG_NEW_EXPLORATION_MODE) {
                var index = _.findIndex(self.dataSourceParams, {
                    abstractDataSourceId: resource.id,
                    application: resource.application,
                    relPath: resource.relPath,
                    workGroup: resource.workGroup
                });

                if (index != -1) { return null; }
            }

            self.fileCubesCount++; // Keep track of the number of cubes we represent

            // For now add the appropriate link but we may not use this - MCA 19 Apr 2017
            // var link = _.find(resource.links, function(link) {
            //    return link.rel == "self";
            // });
            var resourceNode = {};
            resourceNode.application = resource.application;
            resourceNode.id = resource.id;
            resourceNode.label = resource.label;
            resourceNode.lastModifiedTimeStamp = resource.lastModifiedTimeStamp;
            resourceNode.name = resource.name;
            resourceNode.nodeType = "file";
            resourceNode.relPath = resource.relPath;
            resourceNode.type = resource.type;
            resourceNode.workGroup = resource.workGroup;
            resourceNode.lineage = parentFolderLineage + " " + resourceNode.label;

            return resourceNode;
        };

        /**
         * Create a TreeNode which represents a 'folder' or 'directory' for use in the TreeTable
         * @param folder Object which holds properties about a Cube folder to build a TreeNode with
         * @returns {Object} A node object representing a folder
         */
        var createFolderNode = function(folder, parentFolderLineage) {
            var folderNodeChildren = [];

            // Create a folder node object
            var folderNode = {};
            folderNode.name = folder.name;
            folderNode.label = folder.label ? folder.label : folder.name; // Use the label if present; otherwise, use name
            folderNode.nodeType = "dir";
            folderNode.resourceMediaType = folder.resourceMediaType;
            folderNode.lineage = parentFolderLineage ? parentFolderLineage + " " + folderNode.label : folderNode.label;

            // Folders can contain more folders
            if (folder.folders) {
                _.each(folder.folders, function(dir) {
                    folderNodeChildren.push(createFolderNode(dir, folderNode.lineage));
                });
            }

            // Folder can contain resources too
            if (folder.resources) {
                _.each(folder.resources, function(resource) {
                    folderNodeChildren.push(createResourceNode(resource, folderNode.lineage));
                });
            }

            // Finally add the children to the folder node
            folderNode.children = folderNodeChildren;

            return folderNode;
        };

        /**
         * Create a TreeNode which represents the root node for the TreeTable
         * @param cubes List of cube objects with folders to populate the Tree with
         * @returns {{}}
         */
        var createRootNode = function(cubes) {
            var rootNodeChildren = [];

            // Construct the children of the rootNode
            _.each(cubes, function(cube) {
                rootNodeChildren.push(createFolderNode(cube));
            });

            var rootNode = {};
            rootNode.name = "root";
            rootNode.description = "root";
            rootNode.children = rootNodeChildren;

            return {
                root: rootNode
            };
        };

        /**
         * Refresh the TreeTable with a new cube list
         */
        var onRefresh = function() {
            self.dialog.setBusyIndicatorDelay(0);
            self.dialog.setBusy(true);

            $.when (
                // Load the cubes
                sas.analytics.riskhp.utils.HprAPI.getCubes()
            ).done(function(cubes) {
                self.fileCubesCount = 0;
                self._disableLeftButton();

                var model = self.dialog.getModel();
                var dataSourceItemsPath = model.getProperty("/dataSourceItemsPath");
                model.setProperty(dataSourceItemsPath, cubes);
                model.refresh();

                // Build a model for the TreeTable
                var oModel = new sap.ui.model.json.JSONModel(createRootNode(cubes));
                dataSourceTable.setModel(oModel);
                dataSourceTable.bindRows("/root");

                // Update count label by re-applying filter if present
                if (cubes) {
                    var searchFieldValue = self.searchField.getValue();
                    if (searchFieldValue) {
                        // Use expandToLevel to fully expand the tree if there is a search value
                        self.dataSourceTable.expandToLevel(sas.risk.common.utils.CommonConstants.MAX_INTEGER);
                    }
                    self._filterDataSources(searchFieldValue); // Let _filterDataSources handle missing search value if needed
                }

                self.dialog.setBusy(false);
            }).fail(function() {
                self.dialog.setBusy(false);
                sas.analytics.riskhp.utils.HprUtil.displayGenericErrorDialog();
            });
        };

        var okButton = new sas.hc.m.Button({
            text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("hprisk.okButton.txt"),
            press: onOpen,
            enabled: false
        });

        this.dialog = new sas.hc.m.Dialog(ctrl.createId("dialog"), {
            content: [verticalLayout],
            contentWidth: "750px",
            defaultButton: okButton,
            leftButton: okButton,
            rightButton: new sas.hc.m.Button({
                text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("hprisk.cancelButton.txt"),
                press: onCancel
            }),
            afterClose: function(evt) {
                this.destroy();
            }
        }).addStyleClass("chooseDataSourceDialog");

        var dataSourceTable = this.dataSourceTable;
        this.dialog.attachBeforeOpen(function() {
            // Make a copy of the current DataSourceParam objects contained in the cache;
            // This will allow filtering of nodes in the TreeTable
            self.dataSourceParams = DataSourceCacheManager.getDataSourceParams();

            // Before the dialog opens, set the title and hide some components based on the dialog mode
            var model = self.dialog.getModel();
            self.dialog.setTitle(model.getProperty("/dialogTitle"));
            self.dialogMode = model.getProperty("/dialogMode"); // Set the dialog mode

            // In Add and Replace modes, hide a lot of the fields we don't need to show the user
            if (self.dialogMode === HprConstants.HPR_CHOOSEDATASOURCE_DIALOG_ADD_DATA_SOURCE_MODE ||
                self.dialogMode === HprConstants.HPR_CHOOSEDATASOURCE_DIALOG_REPLACE_DATA_SOURCE_MODE) {
                self.nameFieldLabel.setVisible(false);
                self.nameField.setVisible(false);
                self.dataSourceAreaSelectVerticalLayout.setVisible(false);
                self.dialog.setInitialFocus(self.searchField.getId());

                if (self.dialogMode === HprConstants.HPR_CHOOSEDATASOURCE_DIALOG_REPLACE_DATA_SOURCE_MODE) {
                    self.oldCubeNameLabel.setText(model.getProperty("/oldCubeName"));
                    self.oldCubeNameLabel.setVisible(true);
                }
            }
            else {
                self.dialog.setInitialFocus(self.nameField.getId());
            }
        });
        this.dialog.attachAfterOpen(function() {
            var model = self.dialog.getModel();
            var dataSourceItemsPath = model.getProperty("/dataSourceItemsPath");
            var dataSourceItems = model.getProperty(dataSourceItemsPath);
            self.fileCubesCount = 0; // Start count of cubes at 0

            // Build a model for the TreeTable
            var oModel = new sas.risk.common.models.RiskJSONModel(createRootNode(dataSourceItems));
            dataSourceTable.setModel(oModel);
            dataSourceTable.bindRows("/root");

            // Update count label
            if (dataSourceItems) {
                self._updateCountText(self.fileCubesCount);
            }

            if (self.dialogMode === HprConstants.HPR_CHOOSEDATASOURCE_DIALOG_NEW_EXPLORATION_MODE) {
                // Set input value to new default name and select all the input text
                self.nameField.setValue(sas.analytics.riskhp.utils.HprUtil.getUniqueNewExplorationName(model.getProperty("/explorations")));
                sas.analytics.riskhp.utils.HprUtil.selectAllInputText(self.nameField);
            }
        });

        return this.dialog;
    },

    _onNameFieldLiveChange: function(evt) {
        this._enableLeftButton();
    },

    _onDataSourceTableRowSelectionChange: function(evt) {
        // Reset some components on every selection
        this._disableLeftButton();
        this._disableDataSourceAreaSelect();
        this.dataSourceAreaSelect.setValueState(sap.ui.core.ValueState.None);

        var tableSelectedIndex = this.dataSourceTable.getSelectedIndex();
        var rowContext = evt.getParameter("rowContext");
        if (rowContext && tableSelectedIndex !== -1) {
            // Gather the object for the selected row and if it is a selectable cube, try and load the details about that DataSource
            var rowContextObject = rowContext.getObject();
            if (rowContextObject && rowContextObject.nodeType === 'file') {
                // When Add Data Source and Replace Data Source modes, just work with the Left Button
                if (this.dialogMode === HprConstants.HPR_CHOOSEDATASOURCE_DIALOG_ADD_DATA_SOURCE_MODE ||
                    this.dialogMode === HprConstants.HPR_CHOOSEDATASOURCE_DIALOG_REPLACE_DATA_SOURCE_MODE) {
                    this._enableLeftButton();
                } else {
                    // For the other modes, we need to go get information from the mid-tier
                    this.dataSourceAreaSelect.setBusyIndicatorDelay(0);
                    this.dataSourceAreaSelect.setBusy(true);
                    this._getDataSourceDetails(rowContextObject);
                }
            }
        }
    },

    _getDataSourceDetails: function(dataSourceObject) {
        var self = this;

        // Create a DataSourceParams object that we can use to find details about it
        var dataSourceParams = sas.analytics.riskhp.models.DataSourceParams.prototype.createRootParams(dataSourceObject);

        // Get the DataSource details so we can populate the Select component with different 'types'
        var promise = sas.analytics.riskhp.utils.HprAPI.getDataSourceDetails(dataSourceParams);
        promise.done(function(dataSourceDetails) {
            // Take 'dataSources' and load up Select component
            if (dataSourceDetails && dataSourceDetails.dataSources && dataSourceDetails.dataSources.length > 0) {
                // Remove all the items and select the first item for the Select component (should be the placeholder)
                self.dataSourceAreaSelect.removeAllItems();

                // Populate the Select drop-down with items representing the different data sources
                _.each(dataSourceDetails.dataSources, function(dataSource) {
                    self.dataSourceAreaSelect.addItem(self._constructSelectItem(dataSource, dataSourceDetails));
                });

                self.dataSourceAreaSelect.setBusy(false); // Hide the busy indicator

                // Check to see if there is only 1 DataSource
                if (dataSourceDetails.dataSources.length === 1) {
                    // If we only have 1, select it and fire the event so the Open button will be active; Keep the Select component
                    // disabled in this case, so the user can't change back to Placeholder item
                    var itemToSelect = self.dataSourceAreaSelect.getItemAt(1); // 0 is placeholder
                    self.dataSourceAreaSelect.setSelectedItem(itemToSelect, true);
                } else {
                    var dataSourceKeys = [HprConstants.HPR_CUBE_INFORMATION_TYPE_PRICE, HprConstants.HPR_CUBE_INFORMATION_TYPE_STRESS]; // TODO: Add 'portfolio' when it is ready
                    var i = 0;
                    var dataSourceToSelect = null;

                    // Try to find the desired data source type by searching for 'price' and then 'stress', etc.
                    do {
                        dataSourceToSelect = self.dataSourceAreaSelect.getItemByKey(dataSourceKeys[i]);
                        i = i + 1;
                    } while (i < dataSourceKeys.length && dataSourceToSelect === null);

                    // If we still didn't find a desired one, just use the first one in the list
                    if (dataSourceToSelect === null) {
                        dataSourceToSelect = self.dataSourceAreaSelect.getItemAt(1); // 0 is placeholder
                    }

                    // Select the item and fire the event
                    self.dataSourceAreaSelect.setSelectedItem(dataSourceToSelect, true);

                    // Enable the Select component if there is more than 1 so user can choose
                    self.dataSourceAreaSelect.setEnabled(true);
                }
            } else {
                // Turn off busy state of select control
                self.dataSourceAreaSelect.setBusy(false);

                // Display error dialog
                sas.analytics.riskhp.utils.HprUtil.displayGenericErrorDialog(sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.detailed.error.cube.information.failed.to.load.txt'));
            }
        });
        promise.fail(function() {
            self.dataSourceAreaSelect.setBusy(false);
        });
    },

    _constructSelectItem: function(dataSource, dataSourceDetails) {
        var textString = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("ChooseDataSourceDialog." + dataSource.type + ".data.source.txt");
        var dataSourceItem = new sas.hc.ui.core.Item({
            key: dataSource.type,
            tooltip: textString,
            text: textString,
            customData: new sas.hc.ui.core.CustomData({key: "dataSourceDetails", value: dataSourceDetails})
        });
        return dataSourceItem;
    },

    _onDataSourceAreaSelectChange: function(evt) {
        var self = this;
        var selectedItem = evt.getParameters().selectedItem;

        // Placeholder item does not have a 'key' property so don't enable Left button in that case
        if (selectedItem && selectedItem.getKey()) {
            self._enableLeftButton();
        } else {
            self._disableLeftButton();
        }
    },

    _enableLeftButton: function() {
        var enabled = false;

        var tableNumSelectedIndexes = this.dataSourceTable.getSelectedIndices().length;
        if (this.dialogMode === HprConstants.HPR_CHOOSEDATASOURCE_DIALOG_ADD_DATA_SOURCE_MODE ||
            this.dialogMode === HprConstants.HPR_CHOOSEDATASOURCE_DIALOG_REPLACE_DATA_SOURCE_MODE) {
            // For Add Data Source and Replace Data Source modes, we just need the table to have 1 row selected before we can enable the Left Button
            if (tableNumSelectedIndexes === 1) {
                enabled = true;
            }
        } else {
            // For other modes, we need to check the Name and Select fields for appropriate content
            var nameFieldVisible = this.nameField.getVisible();
            var nameFieldTrimmedValue = this.nameField.getValue().trim();
            var dataSourceAreaSelectKey = this.dataSourceAreaSelect.getSelectedKey();
            if ( ((nameFieldVisible && nameFieldTrimmedValue) || !nameFieldVisible) && tableNumSelectedIndexes === 1 && dataSourceAreaSelectKey ) {
                var selectedRowIndex = this.dataSourceTable.getSelectedIndex();
                var dataSourceObject = this.dataSourceTable.getContextByIndex(selectedRowIndex).getObject();
                // Only file node types are valid, not the folders
                if (dataSourceObject.nodeType === 'file') {
                    enabled = true;
                }
            }
        }

        this.dialog.getBeginButton().setEnabled(enabled);
    },

    _disableLeftButton: function() {
        this.dialog.getBeginButton().setEnabled(false);
    },

    _disableDataSourceAreaSelect: function() {
        this.dataSourceAreaSelect.setEnabled(false);
    },

    _onSearchLiveChange: function(evt) {
        var self = this;
        var searchValue = evt.getSource().getValue();
        var delay = (searchValue ? 300 : 0); // no delay if value is empty

        // Use expandToLevel to fully expand the tree when a filter starts
        this.dataSourceTable.expandToLevel(sas.risk.common.utils.CommonConstants.MAX_INTEGER);

        // Execute search after user stopped typing for 300ms
        clearTimeout(this.liveChangeTimer);
        if (delay) {
            this.liveChangeTimer = setTimeout(function () {
                self._filterDataSources(searchValue);
            }, delay);
        } else {
            self._filterDataSources(searchValue);
        }
    },

    _filterDataSources: function(searchValue) {
        if(searchValue && searchValue.length > 0) {
            this.searchFilter = new sap.ui.model.Filter({
                path: "lineage",
                operator: sap.ui.model.FilterOperator.Contains,
                value1: searchValue
            });
        } else {
            this.searchFilter = null;
        }

        this._applyFilters();
    },

    _applyFilters: function(evt) {
        var filters = [];

        if(this.searchFilter) {
            filters.push(this.searchFilter);
        }

        if(this.dataSourceTable.isBound("rows")) {
            // Apply the search filter and update the count text to reflect new number of rows displayed
            var rowsBindingObject = this.dataSourceTable.getBinding("rows");
            rowsBindingObject.filter(filters);

            // If a filter has been applied, count the number of "contexts" returned so we can
            // display an appropriate result to the user.
            if (filters && filters.length > 0) {
                // For the TreeTable, there is not a good way to use the 'rows' binding to figure out the count
                // of the filtered contexts.  Unfortunately, we have to dig deep inside the 'rows' binding to find
                // the information we need to display the result.  'filterInfo' is not documented so on an SDK upgrade
                // this is subject to problems.
                var filteredContexts = rowsBindingObject.filterInfo.aFilteredContexts;
                var filteredContextsCount = 0;
                var treeModel = this.dataSourceTable.getModel();
                _.each(filteredContexts, function(context) {
                    // Loop through the filtered contexts and count the number of 'file' cube nodes in order to display that
                    // number to the user
                    var node = treeModel.getProperty(context.getPath());
                    if (node.nodeType === 'file') {filteredContextsCount++;}
                });
                this._updateCountText(filteredContextsCount);
            }
            else {
                // When no filter is applied, return the original number of 'file' cubes
                this._updateCountText(this.fileCubesCount);
            }
        }
    },

    _updateCountText: function(newCount) {
        if (newCount > 1) {
            this.dataSourceCountText.setText(sas.analytics.riskhp.utils.HprUtil.getBundle().getText("ChooseDataSourceDialog.multipleDataSourceResult.fmt", [newCount]));
        } else if (newCount === 1) {
            this.dataSourceCountText.setText(sas.analytics.riskhp.utils.HprUtil.getBundle().getText("ChooseDataSourceDialog.oneDataSourceResult.txt"));
        } else {
            this.dataSourceCountText.setText(sas.analytics.riskhp.utils.HprUtil.getBundle().getText("ChooseDataSourceDialog.zeroDataSourceResult.txt"));
        }
    }
});