// (c) 2018, SAS Institute Inc.

jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Horizon");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.RelativeHorizon");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.Dialog");
jQuery.sap.require("sas.hc.ui.commons.formControl.Field");
jQuery.sap.require("sas.hc.ui.commons.formControl.FormControl");
jQuery.sap.require("sas.hc.ui.commons.formControl.Group");

/**
 * Data Item Properties dialog
 */
sap.ui.jsfragment("sas.analytics.riskhp.fragments.DataItemPropertiesDialog", {

    Button: sas.hc.m.Button,
    Dialog: sas.hc.m.Dialog,
    HprUtil: sas.analytics.riskhp.utils.HprUtil,
    HORIZON: sas.analytics.riskhp.models.dataitems.Horizon.HORIZON,
    RELATIVEHORIZON: sas.analytics.riskhp.models.dataitems.RelativeHorizon.RELATIVEHORIZON,
    Field: sas.hc.ui.commons.formControl.Field,
    FormControl: sas.hc.ui.commons.formControl.FormControl,
    Group: sas.hc.ui.commons.formControl.Group,

    createContent: function(ctrl) {

        var self = this;

        this.labelField = new self.Field({
            label: self.HprUtil.getBundle().getText("DataPane.data.item.properties.dialog.label.field.label"),
            editable: false
        });

        this.nameField = new self.Field({
            label: self.HprUtil.getBundle().getText("DataPane.data.item.properties.dialog.name.field.label"),
            editable: false
        });

        this.typeField = new self.Field({
            label: self.HprUtil.getBundle().getText("DataPane.data.item.properties.dialog.type.field.label"),
            editable: false
        });

        this.countField = new self.Field({
            label: self.HprUtil.getBundle().getText("DataPane.data.item.properties.dialog.count.field.label"),
            editable: false
        });

        this.formControl = new self.FormControl(ctrl.createId("-formControl"), {
            displayHeader: false,
            showGroupTitle: false,
            showResetToDefaults: false,
            rootGroup: new self.Group({
                groups: [
                    new self.Group({
                        fields: [this.labelField, this.nameField, this.typeField, this.countField]
                    })
                ]
            })
        });

        var onClose = function() {
            self.dialog.close();
        };

        var rightButton = new self.Button(ctrl.createId("-closeButton"), {
            text: self.HprUtil.getBundle().getText("hprisk.closeButton.txt"),
            press: onClose
        });

        this.dialog = new self.Dialog(ctrl.createId("-dialog"), {
            title: self.HprUtil.getBundle().getText("DataPane.data.item.properties.dialog.title"),
            content: [this.formControl],
            defaultButton: rightButton,
            initialFocus: rightButton.getId(),
            rightButton: rightButton,
            afterClose: function(evt) {
                this.destroy();
            }
        });

        // Perform a lot of actions and populate fields before the dialog opens
        this.dialog.attachBeforeOpen(function() {
            var model = self.dialog.getModel();
            var selectedDataItem = model.getProperty("/selectedDataItem");
            var selectedDataItemType = selectedDataItem.getType();
            var selectedDataItemCount = $.isFunction(selectedDataItem.getCount) ? selectedDataItem.getCount() : undefined;
            var selectedDataItemTypeUpdateCount = $.isFunction(selectedDataItem.getUpdateCount) ? selectedDataItem.getUpdateCount() : undefined;
            var numberOfFieldsDisplayed = 2; // Always display 2 (Label and Type)

            // Fill in the Label field value
            self.labelField.setValue(selectedDataItem.getLabel());

            // Name field should only be visible for Data Item objects (all types except Horizons and Relative Horizons)
            if (selectedDataItemType !== self.HORIZON && selectedDataItemType !== self.RELATIVEHORIZON) {
                self.nameField.setVisible(true);
                self.nameField.setValue(selectedDataItem.getName());
                numberOfFieldsDisplayed++;
            } else {
                self.nameField.setVisible(false);
            }

            // Figure out the correct value for the Type field
            self.typeField.setValue(self._dataItemTypeFormatter(selectedDataItemType));

            // Count field should only be visible for certain data items (i.e. Class Variables) and when it needs to be updated
            if (selectedDataItemCount !== null && selectedDataItemCount !== undefined) {
                self.countField.setVisible(true);
                numberOfFieldsDisplayed++;

                // Populate the value correctly based on the count value and the updateCount flag (must match count formatter in HprTreeTableRow)
                if (selectedDataItemTypeUpdateCount) {
                    if (selectedDataItemCount === -1) {
                        self.countField.setValue("--");
                    } else {
                        self.countField.setValue(self.HprUtil.getBundle().getText('DataPane.properties.update.count.fmt', [selectedDataItemCount]));
                    }
                } else {
                    self.countField.setValue(selectedDataItemCount);
                }
            } else {
                self.countField.setVisible(false);
            }

            // Set the width of the dialog
            self.dialog.setContentWidth("30%");

            // Based on the number of Fields displayed, set the height accordingly
            if (numberOfFieldsDisplayed === 2) {
                self.dialog.setContentHeight("35%");
            } else if (numberOfFieldsDisplayed === 3) {
                self.dialog.setContentHeight("45%");
            } else {
                self.dialog.setContentHeight("50%");
            }
        });

        return this.dialog;
    },

    // Helper function to find correct data item type string
    _dataItemTypeFormatter: function(type) {
        var i18nKey = "DataItem." + type + ".txt";
        return this.HprUtil.getBundle().getText(i18nKey);
    }
});