// (c) 2017, SAS Institute Inc.

jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.risk.common.layout.BarLayout");
jQuery.sap.require("sas.risk.common.layout.BarLayoutSpacer");
jQuery.sap.require("sas.risk.common.widgets.RiskColumnText");

/**
 * Data Source Properties dialog
 */
sap.ui.jsfragment("sas.analytics.riskhp.fragments.DataSourcePropertiesDialog", {

    HprUtil: sas.analytics.riskhp.utils.HprUtil,

    createContent: function(ctrl) {

        var self = this;

        // Grid for Field layout; Content is added later when dialog stands up
        this.grid = new sas.hc.ui.layout.Grid({
            hSpacing: 1,
            vSpacing: 1,
            defaultSpan: "L4 M6 S6",
            height: "100%"
        }).addStyleClass("dataSourcePropertiesDialogGrid");

        // Search field
        var searchField = new sas.hc.m.SearchField({
            filterMode: true,
            liveChange: $.proxy(self._onSearchLiveChange, self) // Filter results when user types after slight delay
        });

        // Sub-cube table
        this.subCubesTable = new sas.hc.ui.table.Table(ctrl.createId("subCubesTable"), {
            selectionMode: sas.hc.ui.table.SelectionMode.None,
            toolbar: new sas.hc.m.ResponsiveToolbar({content: [searchField]}),
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            minAutoRowCount: 1, // required for scrolling with < 5 rows
            editable: false,
            columns: [
                // Visible columns by default
                this._createColumn("index", "DataPane.data.source.properties.sub.cubes.table.index.column.label", "2rem"),
                this._createNameColumn("name", "staleness", "DataPane.data.source.properties.sub.cubes.table.name.column.label", "12rem"),
                this._createColumn("label", "DataPane.data.source.properties.sub.cubes.table.label.column.label", "10rem"),
                this._createColumn("runDateDisplayString", "DataPane.data.source.properties.sub.cubes.table.runDate.column.label", "3rem", true, "runDate"), // As of Date
                this._createColumn("positions", "DataPane.data.source.properties.sub.cubes.table.positions.column.label", "2em", true, "positionsAsNumber", sap.ui.core.TextAlign.End),
                this._createColumn("numSamples", "DataPane.data.source.properties.sub.cubes.table.numSamples.column.label", "2rem", true, "numSamplesAsNumber", sap.ui.core.TextAlign.End), // States
                // Hidden columns by default
                this._createColumn("analysisTypeDisplayString", "DataPane.data.source.properties.sub.cubes.table.analysisType.column.label", "4rem", false, "analysisType"),
                this._createColumn("currency", "DataPane.data.source.properties.sub.cubes.table.currency.column.label", "4rem", false),
                this._createColumn("sasCreationDateDisplayString", "DataPane.data.source.properties.sub.cubes.table.sasCreationDate.column.label", "4rem", false, "sasCreationDate"),
                this._createColumn("sasAlertDateDisplayString", "DataPane.data.source.properties.sub.cubes.table.alertDate.column.label", "4rem", false, "sasAlertDate"),
                this._createColumn("sasExpirationDateDisplayString", "DataPane.data.source.properties.sub.cubes.table.expirationDate.column.label", "4rem", false, "sasExpirationDate")
            ]
        });

        // Dialog border layout
        this.dialogBorderLayout = new sas.hc.ui.commons.layout.BorderLayout({
            height: "100%",
            width: "100%",
            top: new sas.hc.ui.commons.layout.BorderLayoutArea({
                content: [this.grid]
            }),
            center: new sas.hc.ui.commons.layout.BorderLayoutArea({
                content: [this.subCubesTable],
                overflowX: "hidden",
                overflowY: "hidden"
            })
        });

        var onClose = function() {
            self.dialog.close();
        };

        var rightButton = new sas.hc.m.Button(ctrl.createId("-closeButton"), {
            text: self.HprUtil.getBundle().getText("hprisk.closeButton.txt"),
            press: onClose
        });

        this.dialog = new sas.hc.m.Dialog(ctrl.createId("dialog"), {
            title: self.HprUtil.getBundle().getText("DataPane.data.source.properties.dialog.title"),
            content: [self.dialogBorderLayout],
            defaultButton: rightButton,
            initialFocus: rightButton.getId(),
            rightButton: rightButton,
            afterClose: function(evt) {
                this.destroy();
            }
        });

        // Perform a lot of actions and populate fields before the dialog opens
        this.dialog.attachBeforeOpen(function() {
            var model = self.dialog.getModel();
            var dataSourcePropertiesMainCube = model.getProperty("/dataSourcePropertiesMainCube");
            var dataSourcePropertiesSubCubesPath = model.getProperty("/dataSourcePropertiesSubCubesPath");
            var dataSourcePropertiesSubCubes = model.getProperty("/dataSourcePropertiesSubCubes");

            // Add the Name field
            self.grid.addContent(self._createGridElement(self.HprUtil.getBundle().getText("DataPane.data.source.properties.name.field.label"), dataSourcePropertiesMainCube.name));

            // Only display the Label field if it has a value
            var label = dataSourcePropertiesMainCube.label;
            if (null !== label && undefined !== label && label.length > 0) {
                self.grid.addContent(self._createGridElement(self.HprUtil.getBundle().getText("DataPane.data.source.properties.label.field.label"), label));
            }

            // Add the Path field
            self.grid.addContent(self._createGridElement(self.HprUtil.getBundle().getText("DataPane.data.source.properties.label.path.label"), dataSourcePropertiesMainCube.cubePath));

            // Format and add Creation Date field
            self.grid.addContent(self._createGridElement(self.HprUtil.getBundle().getText("DataPane.data.source.properties.creation.date.field.label"),
                self.HprUtil.formatDateTime(new Date(dataSourcePropertiesMainCube.sasCreationDate))));

            // Format and add the Run Date field; Do not include Run Date for Side by Side cubes
            if (dataSourcePropertiesMainCube.joinType !== HprConstants.HPR_CUBE_INFORMATION_JOINTYPE_SIDEBYSIDE) {
                var runDateValue = self.HprUtil.formatISODateString(dataSourcePropertiesMainCube.runDate); // Format to 25Jan2015
                self.grid.addContent(self._createGridElement(self.HprUtil.getBundle().getText("DataPane.data.source.properties.run.date.field.label"), runDateValue));
            }

            // NOTE: Hide these fields for now per Katie
            // Check to see if staleness dates are needed. If so, format and then add them
            // if (self._hasStaleness(dataSourcePropertiesMainCube)) {
            //     var sasAlertDate = new Date(dataSourcePropertiesMainCube.sasAlertDate);
            //     var sasExpirationDate = new Date(dataSourcePropertiesMainCube.sasExpirationDate);
            //     self.grid.addContent(self._createGridElement(self.HprUtil.getBundle().getText("DataPane.data.source.properties.alert.date.field.label"), self.HprUtil.formatDateTime(sasAlertDate)));
            //     self.grid.addContent(self._createExpDateGridElement(self.HprUtil.getBundle().getText("DataPane.data.source.properties.expiration.date.field.label"),
            //         self.HprUtil.formatDateTime(sasExpirationDate), dataSourcePropertiesMainCube));
            // }

            // Add the States field;  Should only be shown for non-stress cubes and non-Side by Side
            if (!dataSourcePropertiesMainCube.isStress &&
                dataSourcePropertiesMainCube.joinType !== HprConstants.HPR_CUBE_INFORMATION_JOINTYPE_SIDEBYSIDE) {
                self.grid.addContent(self._createGridElement(self.HprUtil.getBundle().getText("DataPane.data.source.properties.states.field.label"), dataSourcePropertiesMainCube.numSamples));
            }

            // Add the Positions field
            self.grid.addContent(self._createGridElement(self.HprUtil.getBundle().getText("DataPane.data.source.properties.position.field.label"), dataSourcePropertiesMainCube.positions));

            // Add the Currency field
            self.grid.addContent(self._createGridElement(self.HprUtil.getBundle().getText("DataPane.data.source.properties.currency.field.label"), dataSourcePropertiesMainCube.currency));

            // Add Cube Type field
            var cubeTypeValue = self.HprUtil.getBundle().getText("DataPane.data.source.properties.cube.type.price.txt");
            if (dataSourcePropertiesMainCube.isStress) {
                cubeTypeValue = self.HprUtil.getBundle().getText("DataPane.data.source.properties.cube.type.stress.txt");
            }
            self.grid.addContent(self._createGridElement(self.HprUtil.getBundle().getText("DataPane.data.source.properties.cube.type.field.label"), cubeTypeValue));

            // Add Join Type field
            var joinTypeValue = null;
            switch(dataSourcePropertiesMainCube.joinType) {
                case HprConstants.HPR_CUBE_INFORMATION_JOINTYPE_NONE:
                    joinTypeValue = self.HprUtil.getBundle().getText("DataPane.data.source.properties.joinType.none.txt");
                    break;
                case HprConstants.HPR_CUBE_INFORMATION_JOINTYPE_AGGREGATE:
                    joinTypeValue =  self.HprUtil.getBundle().getText("DataPane.data.source.properties.joinType.aggregate.txt");
                    break;
                default:
                    joinTypeValue = self.HprUtil.getBundle().getText("DataPane.data.source.properties.joinType.comparison.txt");
                    break;
            }
            self.grid.addContent(self._createGridElement(self.HprUtil.getBundle().getText("DataPane.data.source.properties.join.type.field.label"), joinTypeValue));

            // Add the Analysis Type field; Only display for non-joined Stress cubes
            if (dataSourcePropertiesMainCube.analysisType &&
                dataSourcePropertiesMainCube.joinType === HprConstants.HPR_CUBE_INFORMATION_JOINTYPE_NONE &&
                !dataSourcePropertiesMainCube.isStress) {
                var analysisTypeValue = self.HprUtil.getBundle().getText("DataPane.data.source.properties.analysis.type." + dataSourcePropertiesMainCube.analysisType + ".txt");
                self.grid.addContent(self._createGridElement(self.HprUtil.getBundle().getText("DataPane.data.source.properties.analysis.type.field.label"), analysisTypeValue));
            }

            // Based on the number of sub-cubes and fields to display, decide how big the dialog should be, whether to display the sub-cube table and how big the border layout areas should be
            var numSubCubes = dataSourcePropertiesMainCube.numSubCubes;
            if (numSubCubes > 0) {
                // Add Sub Cubes field
                self.grid.addContent(self._createGridElement(self.HprUtil.getBundle().getText("DataPane.data.source.properties.sub.cubes.field.label"), numSubCubes));

                // Display the Sub Cubes table
                self.subCubesTable.setModel(model);
                self.subCubesTable.bindRows(dataSourcePropertiesSubCubesPath);

                // Adjust the border layout sizes
                if (numSubCubes <= 5) {
                    self.dialogBorderLayout.getTop().setSize("40%");
                    self.dialogBorderLayout.getCenter().setSize("60%");
                } else {
                    self.dialogBorderLayout.getTop().setSize("30%");
                    self.dialogBorderLayout.getCenter().setSize("70%");
                }

                // Adjust dialog size
                self.dialog.setContentWidth("95%");
                self.dialog.setContentHeight("75%");
            } else {
                // Hide Sub Cubes table
                self.subCubesTable.setVisible(false);

                // Adjust the border layout sizes
                self.dialogBorderLayout.getTop().setSize("100%");
                self.dialogBorderLayout.getCenter().setSize("0%");

                // Adjust dialog size
                self.dialog.setContentWidth("50%");
                if (self.grid.getContent().length > 9) {
                    self.dialog.setContentHeight("50%");
                } else {
                    self.dialog.setContentHeight("40%");
                }
            }
        });

        return this.dialog;
    },

    // Helper function to create a basic column for the table
    _createColumn: function(bindingKey, titleKey, width, visible, sortProperty, alignment) {
        var self = this;
        var visibleProperty = _.isUndefined(visible) ? true : visible;
        var alignmentProperty = _.isUndefined(alignment) ? sap.ui.core.TextAlign.Begin : alignment;
        var textProperty = {
            path: bindingKey
        };

        return new sas.hc.ui.table.Column({
            width: width,
            label: new sas.hc.m.Label({
                text: self.HprUtil.getBundle().getText(titleKey),
                textAlign: alignmentProperty
            }),
            template: new sas.risk.common.widgets.RiskColumnText({
                text: textProperty,
                textAlign: alignmentProperty
            }),
            sortProperty: sortProperty ? sortProperty : bindingKey,
            filterProperty: bindingKey,
            filterOperator: sap.ui.model.FilterOperator.Contains,
            visible: visibleProperty
        });
    },

    // Helper function to create a column for the Name property; The Name value is displayed along with a Staleness indicator
    _createNameColumn: function(textBindingKey, stalenessBindingKey, titleKey, width) {
        var self = this;

        var textProperty = {
            path: textBindingKey
        };
        var stalenessSrcProperty = {
            path: stalenessBindingKey,
            formatter: function(value) {
                return self._getStalenessIcon(value, true);
            }
        };
        var stalenessTooltipProperty = {
            path: stalenessBindingKey,
            formatter: function(value) {
                return self._getFlaggedStatusTooltipString(value);
            }
        };

        return new sas.hc.ui.table.Column({
            width: width,
            label: new sas.hc.m.Label({
                text: self.HprUtil.getBundle().getText(titleKey)
            }),
            template: new sas.risk.common.layout.BarLayout({
                content: [
                    new sas.hc.m.Image({
                        src: stalenessSrcProperty,
                        tooltip: stalenessTooltipProperty
                    }),
                    new sas.risk.common.layout.BarLayoutSpacer(),
                    new sas.risk.common.widgets.RiskColumnText({
                        text: textProperty
                    })
                ]
            }),
            sortProperty: textBindingKey,
            filterProperty: textBindingKey,
            filterOperator: sap.ui.model.FilterOperator.Contains
        });
    },

    // Create a Vertical Layout (with Label and Text fields) to display in the Grid
    _createGridElement: function(labelText, value) {
        var label = new sas.hc.m.Label({
            text: labelText
        });
        var field = new sas.hc.m.Text({
            text: value
        });

        return new sas.hc.ui.layout.VerticalLayout({
            width: "100%",
            content: [label, field]
        });
    },

    // Create a Vertical Layout for the Expiration Date field which consists of a Label and HBox (displaying a Text field and staleness image)
    _createExpDateGridElement: function(labelText, value, dataSourcePropertiesMainCube) {
        var self = this;

        var label = new sas.hc.m.Label({
            text: labelText
        });
        var field = new sas.hc.m.Text({
            text: value
        });

        var oControl = new sas.hc.m.HBox({
            alignItems: sap.m.FlexAlignItems.Center,
            justifyContent: sap.m.FlexJustifyContent.Start
        }).addStyleClass("expirationDateField");

        var oImage = new sas.hc.m.Image({});
        oImage.setSrc(self._getStalenessIcon(dataSourcePropertiesMainCube.staleness, false));
        self._setFlaggedStatusTooltip(oImage, dataSourcePropertiesMainCube.staleness);

        oControl.addItem(field);
        oControl.addItem(oImage);

        return new sas.hc.ui.layout.VerticalLayout({
            width: "100%",
            content: [label, oControl]
        });
    },

    // Helper function to determine staleness of a cube
    _hasStaleness: function(cube) {
        if (cube.isStress || (_.isUndefined(cube.sasAlertDate) && _.isUndefined(cube.sasExpirationDate))) {
            return false;
        }
        return true;
    },

    // Helper function to determine staleness icon of main cube and sub cubes
    _getStalenessIcon: function(value, displayDefaultIconIfValueMissing) {
        switch (value) {
            case 2:
                return sas.icons.HC.FLAGGEDSTATUS;
            case 1:
                return sas.icons.HC.WARNINGSTATUS;
            case 0:
            default:
                if (displayDefaultIconIfValueMissing)
                    return sas.icons.HC.COMPLETESTATUS;
                else
                    return null;
        }
    },

    // Helper function to set a custom tooltip for Flagged Status Staleness icon
    _setFlaggedStatusTooltip: function(oImage, value) {
        var self = this;
        switch (value) {
            case 2:
                oImage.setTooltip(self._getFlaggedStatusTooltipString(value));
                break;
            default:
                break;
        }
    },

    // Helper function to retrieve tooltip for a Flagged Status Staleness icon
    _getFlaggedStatusTooltipString: function(value) {
        var self = this;

        if (value === 2) {
            return self.HprUtil.getBundle().getText("DataPane.data.source.properties.flagged.status.icon.tip");
        }
    },

    _onSearchLiveChange: function(evt) {
        var self = this;
        var searchValue = evt.getSource().getValue();
        var delay = (searchValue ? 300 : 0); // no delay if value is empty

        // Execute search after user stopped typing for 300ms
        clearTimeout(this.liveChangeTimer);
        if (delay) {
            this.liveChangeTimer = setTimeout(function () {
                self._filterSubCubes(searchValue);
            }, delay);
        } else {
            self._filterSubCubes(searchValue);
        }
    },

    _filterSubCubes: function(searchValue) {
        var allFilters = null;
        if(searchValue.length > 0) {
            var columns = this.subCubesTable.getColumns();
            var filters = [];

            // create filters for visible columns only
            _.each(columns, function (column) {
                if (column.getVisible() && column.getFilterProperty() && column.getFilterOperator()) {
                    filters.push(new sap.ui.model.Filter(column.getFilterProperty(), column.getFilterOperator(), searchValue));
                }
            });

            allFilters = filters.length > 0 ? new sap.ui.model.Filter(filters) : null;
        }

        var binding = this.subCubesTable.getBinding("rows");
        binding.filter(allFilters);
    }
});