// (c) 2018, SAS Institute Inc.

jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.hc.ui.commons.logViewer.LogViewer");

/**
 * Exploration summary viewer dialog thing.
 */
sap.ui.jsfragment("sas.analytics.riskhp.fragments.ExplorationSummaryDialog", {

    HprUtil: sas.analytics.riskhp.utils.HprUtil,

    createContent: function(ctrl) {

        var self = this;

        // id variables
        var dialogId = "ExplorationSummaryDialog" + Date.now();
        var closeButtonId = dialogId + "-closeButton";

        // Grid for Field layout; Content is added later when dialog stands up
        // this.grid = new sas.hc.ui.layout.Grid({
        //     hSpacing: 1,
        //     vSpacing: 1,
        //     defaultSpan: "L4 M6 S6",
        //     height: "100%"
        // }).addStyleClass("dataSourcePropertiesDialogGrid");

        // LogViewer component which will be used to display summary information
        this.summaryViewer = new sas.hc.ui.commons.logViewer.LogViewer({ height: "100%", width: "100%" });

        // Dialog border layout
        this.dialogBorderLayout = new sas.hc.ui.commons.layout.BorderLayout({
            height: "100%",
            width: "100%",
            // top: new sas.hc.ui.commons.layout.BorderLayoutArea({
            //     content: [this.grid],
            //     overflowX: "hidden",
            //     overflowY: "hidden"
            // }),
            center: new sas.hc.ui.commons.layout.BorderLayoutArea({
                content: [this.summaryViewer],
                overflowX: "hidden",
                overflowY: "hidden"
            }).addStyleClass("sasLogDialogCenterLayout")
        });

        var onClose = function() {
            self.dialog.close();
        };

        var rightButton = new sas.hc.m.Button(closeButtonId, {
            text: self.HprUtil.getBundle().getText("hprisk.closeButton.txt"),
            press: onClose
        });

        this.dialog = new sas.hc.m.Dialog(dialogId, {
            title: "Risk Exploration Summary",
            //title: self.HprUtil.getBundle().getText("Exploration.summary.dialog.title"),
            content: [self.dialogBorderLayout],
            defaultButton: rightButton,
            initialFocus: rightButton.getId(),
            rightButton: rightButton,
            afterClose: function(evt) {
                this.destroy();
            }
        });

        // Perform some actions after the dialog opens
        this.dialog.attachAfterOpen(function() {
            var model = self.dialog.getModel();
            var summaryData = model.getProperty("/explorationSummaryData");

            // This call to LogViewer.setLog only worked in this "attachAfterOpen".  If placed in "attachBeforeOpen", the detail text never
            // rendered for the LogViewer.  This was due to some timing issues of when the detail text was built in the LogViewer code
            self.summaryViewer.setLog(self._parseSummaryData(summaryData));
        });

        // Perform some actions before the dialog opens
        this.dialog.attachBeforeOpen(function() {

            // Adjust the border layout sizes
            // self.dialogBorderLayout.getTop().setSize("10%"); // not used
            self.dialogBorderLayout.getCenter().setSize("90%");

            // Adjust dialog size
            self.dialog.setContentWidth("50%");
            self.dialog.setContentHeight("75%");
        });

        return this.dialog;
    },

    _parseSummaryData: function(sumData) {

        var logData = {
            log : sumData
        };

        return logData;
    }

});