// (c) 2017, SAS Institute Inc.

jQuery.sap.require("sas.analytics.riskhp.models.HPRiskModelManager");
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");

/**
 * Hierarchy dialog
 * @name sas.analytics.riskhp.fragments.HierarchyDialog
 */
sap.ui.jsfragment("sas.analytics.riskhp.fragments.HierarchyDialog", {

    createContent: function(oController) {

        var self = this;

        // id variables
        var hierarchyDialogId = "hierarchyDialog" + Date.now();
        var messageTextId = hierarchyDialogId + "-messageText";
        var leftButtonId = hierarchyDialogId + "-left-button";
        var middleButtonId = hierarchyDialogId + "-middle-button";
        var rightButtonId = hierarchyDialogId + "-right-button";

        var messageText = new sas.hc.m.Text({
            id: messageTextId,
            text: oController.messageText
        });

        var messageText2 = undefined;
        if (oController.messageText2 !== undefined) {
            messageText2 = new sas.hc.m.Text({
                id: messageTextId + 2,
                text: oController.messageText2
            });
            messageText2.addStyleClass("hprHierarchyDialogText2");
        }

        // Dialog layout
        var hierarchyDialogLayout = new sas.hc.ui.layout.VerticalLayout({
            id:hierarchyDialogId + '-vertLayout',
            content: [
                messageText,
                messageText2
            ]
        });

        var doCancel = function() {
            self.hierarchyDialog.close();
            if (oController.doCancel !== undefined){
                oController.doCancel();
            }
        };

        var doContinue = function() {
            self.hierarchyDialog.close();
            if (oController.doContinue !== undefined) {
                oController.doContinue();
            }
        };

        var doInsert = function() {
            self.hierarchyDialog.close();
            if (oController.doInsert !== undefined) {
                oController.doInsert();
            }
        };

        var doAppend = function() {
            self.hierarchyDialog.close();
            if (oController.doAppend !== undefined) {
                oController.doAppend();
            }
        };

        // Right dialog button will always be "cancel" operation
        var rightButton = new sas.hc.m.Button(rightButtonId, {
            text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("Hierarchy.dialog.cancel.txt"),
            press: doCancel
        });

        // Middle dialog button may be an "append" classvar
        var middleButton = undefined;
        if (oController.doAppend !== undefined) {
            middleButton = new sas.hc.m.Button(middleButtonId, {
                text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("Hierarchy.dialog.append.txt"),
                press: doAppend
            });
        }

        // Left dialog button may be a "continue" operation or "insert" classvar
        var leftButton = undefined;
        if (oController.doContinue !== undefined) {
            leftButton = new sas.hc.m.Button(leftButtonId, {
                text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("Hierarchy.dialog.continue.txt"),
                press: doContinue
            });
        }
        else if (oController.doInsert !== undefined) {
            leftButton = new sas.hc.m.Button(leftButtonId, {
                text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("Hierarchy.dialog.insert.txt"),
                press: doInsert
            });
        }

        this.hierarchyDialog = new sas.hc.m.Dialog(hierarchyDialogId, {
            icon: sas.icons.HC.WARNINGSTATUS,
            title: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("Hierarchy.dialog.title.txt"),
            content: [
                hierarchyDialogLayout
            ],
            defaultButton: leftButton,
            leftButton: leftButton,
            middleButton: middleButton,
            rightButton: rightButton,
            afterClose: function(evt) {
                this.destroy();
            }
        }).addStyleClass("hierarchyDialog");

        return this.hierarchyDialog;
    }
});