// (c) 2017, SAS Institute Inc.

jQuery.sap.require("sas.analytics.riskhp.utils.DraggableListItemDelegate");
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.analytics.riskhp.utils.ViewManager");
jQuery.sap.require("sas.hc.ui.commons.propPane.PropertyPane");

sap.ui.jsfragment("sas.analytics.riskhp.fragments.ObjectsPane", {

    _getResourceBundle: function() {
        return sas.analytics.riskhp.utils.HprUtil.getBundle();
    },

    // Helper method to customize menu based on item selected
    _fnCustomizeRowContextMenu: function(oMenu, oEvent) {
        var oControl = $(oEvent.target).control()[0]; // List item

        // Make decisions on menu item text based on the selections
        if (oControl && oControl instanceof sas.hc.m.StandardListItem) {
            // Only have one menu item now
            if (oMenu.getAggregation("items").length === 1) {
                // 'this' is in the context of the oMenu for this method so we have to use other means to get to ExplorationView and HprUtil
                var selectedPageTitle = sas.analytics.riskhp.utils.ViewManager.getExplorationView().getController().getSelectedPageTitle();
                var menuItemText = sas.analytics.riskhp.utils.HprUtil.getBundle().getText('ObjectsPane.menu.fmt', [oControl.getTitle(),selectedPageTitle]);
                var menuItem = oMenu.getAggregation("items")[0];
                menuItem.setText(menuItemText);

                // Create a custom property to keep track of the list item id for use in itemSelect method
                oMenu.listItemId = oControl.getId();
            }
        }
        return oMenu;
    },

    _createContextMenu: function() {
        // Create a basic Menu with one MenuItem (properties are filled in via '_fnCustomizeRowContextMenu' method)
        var oMenu = new sas.hc.ui.unified.Menu();
        var item = new sas.hc.ui.unified.MenuItem({});
        oMenu.addItem(item);

        oMenu.attachItemSelect(function(event) {
            var oSource = event.getSource();
            if (oSource) {
                var listItemId = oSource.listItemId; // Use the custom property off of the Menu

                // Add an appropriate panel to the selected Risk Panels View
                var rpv = sas.analytics.riskhp.utils.HprUtil.getSelectedRiskPanelsView();
                rpv.addPanel(null,sas.risk.common.panelsview.RiskPanelsViewDock.Bottom, [{id: listItemId}]);
            }
        });

        return oMenu;
    },

    _buildRow: function(model, dataType) {
        var self = this;
        var oList = new sas.hc.m.List({});

        var oItemTemplate = new sas.hc.m.StandardListItem({
            title: "{Name}",
            icon: "{ProductPicUrl}",
            customData: {key: 'type', value: "{ObjectType}"}
        });
        oItemTemplate.addStyleClass("draggableListItem");

        oList.setModel(model);
        oList.bindAggregation("items", "/" + dataType, oItemTemplate);
        oList.addDelegate(new sas.analytics.riskhp.utils.DraggableListItemDelegate(oList, {cancel:false}));

        // S1450184: Context menu for list items not accessible via mouse (modeled after HierarchiesPane.view List)
        oList.setAggregation("contextMenu", self._createContextMenu());
        oList.customizeContextMenu = self._fnCustomizeRowContextMenu;

        return oList;
    },

    _buildSection: function(titleRootId, model, dataType) {
        var section = new  sas.hc.ui.commons.windowshade.WindowShadeSection();
        section.setTitle(this._getResourceBundle().getText("ObjectsPane.section." + titleRootId + ".title.txt"));
        section.setTooltip(this._getResourceBundle().getText("ObjectsPane.section." + titleRootId + ".tooltip.txt"));
        section.addContent(this._buildRow(model, dataType));
        return section;
    },

    _buildContent: function() {
        var windowShade = new sas.hc.ui.commons.windowshade.WindowShade();
        windowShade.setWidth('100%');

        // Build model here
        var modelData = {
            tables: [
                {
                    "ProductPicUrl": "sap-icon://hc/crosstab",
                    "Name": this._getResourceBundle().getText("ObjectsPane.section.Tables.crosstab.txt"),
                    "ObjectType": HprConstants.HPR_CROSSTAB_PANEL
                }
            ]
            // graphs: [ // We don't support graphs yet
            //     {
            //         "ProductPicUrl": "sap-icon://hc/barChart",
            //         "Name": "Bar Chart"
            //     },
            //     {
            //         "ProductPicUrl": "sap-icon://hc/boxPlot",
            //         "Name": "Box Plot"
            //     },
            //     {
            //         "ProductPicUrl": "sap-icon://hc/linePlot",
            //         "Name": "Line Chart"
            //     },
            //     {
            //         "ProductPicUrl": "sap-icon://hc/timeSeriesChart",
            //         "Name": "Time Series Plot"
            //     }
            // ]
        };

        // Create sections for WindowShade
        var model = new sap.ui.model.json.JSONModel(modelData);
        windowShade.addSection(this._buildSection('Tables', model, 'tables'));
        //windowShade.addSection(this._buildSection('Graphs', model, 'graphs'));

        return new sas.hc.m.VBox({
            items: [windowShade]
        });
    },

    createContent: function (controller) {
        return new sas.hc.ui.commons.propPane.PropertyPane({
            text: this._getResourceBundle().getText("ObjectsPane.title.txt"),
            showRefresh: false,
            buttonData: {
                icon: sas.icons.HC.LINEPLOT,
                tooltip: this._getResourceBundle().getText("ObjectsPane.icon.tooltip.txt"),
                enabled: '{' + HprConstants.MODEL_ID_EXPLORATION_PANES_STATE + '>/panes/objectsPane/enabled}'

            },
            content: this._buildContent(),
            alignment: 'left'
        });
    }
});