// (c) 2018, SAS Institute Inc.

jQuery.sap.require("sas.hc.m.Dialog");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.ResponsiveToolbar");
jQuery.sap.require("sas.hc.m.ToolbarSpacer");

jQuery.sap.require("sas.hc.ui.table.Table");
jQuery.sap.require("sas.hc.ui.table.Column");

jQuery.sap.require("sas.risk.common.models.RiskJSONModel");

jQuery.sap.require("sas.analytics.riskhp.controls.HprObjectStatus");
jQuery.sap.require("sas.analytics.riskhp.datasource.DataSourceChangeManager");
jQuery.sap.require("sas.analytics.riskhp.models.DataSourceCacheManager");
jQuery.sap.require("sas.analytics.riskhp.models.DataSourceParams");
jQuery.sap.require("sas.analytics.riskhp.utils.ExplorationUtil");
jQuery.sap.require("sas.analytics.riskhp.utils.HprAPI");
jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");


/**
 * Fragment for a dialog that allows the user to replace one or more data sources in an exploration.
 *
 * @name sas.analytics.riskhp.fragments.ReplaceExplorationDataSourceDialog
 */
sap.ui.jsfragment("sas.analytics.riskhp.fragments.ReplaceExplorationDataSourceDialog", function() {

    // classes
    var DataSourceChangeManager = sas.analytics.riskhp.datasource.DataSourceChangeManager;
    var DataSourceCacheManager = sas.analytics.riskhp.models.DataSourceCacheManager;
    var DataSourceParams = sas.analytics.riskhp.models.DataSourceParams;
    var ExplorationUtil = sas.analytics.riskhp.utils.ExplorationUtil;
    var HprAPI = sas.analytics.riskhp.utils.HprAPI;
    var HprConstants = sas.analytics.riskhp.utils.HprConstants;
    var HprUtil = sas.analytics.riskhp.utils.HprUtil;

    // static vars (persistent for the life of the session)
    var rb = HprUtil.getBundle();
    var myModel = new sas.risk.common.models.RiskJSONModel();
    var dataSourceStatusMap = [];   // array of objects containing status and timestamp

    // controls
    var myController = null;
    var testButton = null;
    var replaceButton = null;
    var dataSourceTable = null;
    var nameColumn = null;
    var statusColumn = null;
    var myDialog = null;


    // private functions

    /**
     * Creates the table of exploration data sources.
     * @returns {*}
     * @private
     */
    function _createDataSourceTable() {
        dataSourceTable = new sas.hc.ui.table.Table({
            width: "100%",
            selectionMode: sas.hc.ui.table.SelectionMode.Single,
            showSelectionControls: true,
            showTableOptions: false,
            toolbar: _createDataSourceTableToolbar(),
            columns: _createDataSourceTableColumns(),
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Fixed,
            visibleRowCount: 6,     // really 5, but the table isn't calculating height correctly
            rowSelectionChange: function(event) {
                _updateForSelection(event);
            }
        });
        dataSourceTable.setModel(myModel);
        dataSourceTable.bindRows("/dataSourceMetadata");
        dataSourceTable.allowTextSelection(false);
        dataSourceTable.sort(statusColumn, sas.hc.ui.table.SortOrder.Ascending, true);
        dataSourceTable.sort(nameColumn, sas.hc.ui.table.SortOrder.Ascending, true);

        return dataSourceTable;
    }

    /**
     * Creates the table toolbar.
     * @returns {sas.hc.m.ResponsiveToolbar}
     * @private
     */
    function _createDataSourceTableToolbar() {
        var tableTitle = new sas.hc.m.Label({text: "{/exploration/name}"});

        testButton = new sas.hc.m.Button({
            icon: sas.icons.HC.VALIDATEITEM,
            tooltip: rb.getText("HPRisk.ReplaceExplorationDataSource.test.tip"),
            enabled: false,
            press: _testDataSource
        });

        replaceButton = new sas.hc.m.Button({
            icon: sas.icons.HC.REPLACE,
            tooltip: rb.getText("HPRisk.ReplaceExplorationDataSource.replace.tip"),
            enabled: false,
            press: function(evt) {
                _replaceSelectedDataSource();
            }
        });

        return new sas.hc.m.ResponsiveToolbar({
            content : [
                tableTitle,
                new sas.hc.m.ToolbarSpacer(),
                testButton,
                replaceButton
            ]
        });
    }

    /**
     * Creates the table columns.
     * @returns {*[]}
     * @private
     */
    function _createDataSourceTableColumns() {
        // create columns
        nameColumn = new sas.hc.ui.table.Column({
            label: new sas.hc.m.Label({
                text: rb.getText("HPRisk.ReplaceExplorationDataSource.dataSourceName.title.txt")
            }),
            template: new sas.risk.common.widgets.RiskColumnText({text: "{label}"}),
            sortProperty: "label",
            showFilterMenuEntry: false,
            required: true
        });

        statusColumn = new sas.hc.ui.table.Column({
            label: new sas.hc.m.Label({
                text: rb.getText("HPRisk.ReplaceExplorationDataSource.dataSourceStatus.title.txt")
            }),
            template: new sas.analytics.riskhp.controls.HprObjectStatus({
                text: {
                    parts: [
                        {path: "status"}
                    ],
                    formatter: function (status) {
                        var statusText = rb.getText("HPRisk.ReplaceExplorationDataSource.unknown.txt");
                        switch(status) {
                            case HprConstants.HPR_DATA_SOURCE_STATUS_VALID:
                                statusText = rb.getText("HPRisk.ReplaceExplorationDataSource.valid.txt");
                                break;
                            case HprConstants.HPR_DATA_SOURCE_STATUS_NO_DESCRIPTOR:
                                statusText = rb.getText("HPRisk.ReplaceExplorationDataSource.missingDescriptor.txt");
                                break;
                            case HprConstants.HPR_DATA_SOURCE_STATUS_MISSING_FILES:
                                statusText = rb.getText("HPRisk.ReplaceExplorationDataSource.missingGridFiles.txt");
                                break;
                            case HprConstants.HPR_DATA_SOURCE_STATUS_TESTING:
                                statusText = rb.getText("HPRisk.ReplaceExplorationDataSource.testing.txt");
                                break;
                            case HprConstants.HPR_DATA_SOURCE_STATUS_MISSING_TYPE_PRICE:
                                statusText = rb.getText("HPRisk.ReplaceExplorationDataSource.missingType.price.txt");
                                break;
                            case HprConstants.HPR_DATA_SOURCE_STATUS_MISSING_TYPE_STRESS:
                                statusText = rb.getText("HPRisk.ReplaceExplorationDataSource.missingType.stress.txt");
                                break;
                            case HprConstants.HPR_DATA_SOURCE_STATUS_WARNING_NOT_PRICESTRESS:
                                statusText = rb.getText("HPRisk.ReplaceExplorationDataSource.notPriceStress.txt");
                                break;
                            default:
                                break;
                        }
                        return statusText;
                    }
                },
                icon: {
                    parts: [
                        {path: "status"}
                    ],
                    formatter: function (status) {
                        var icon = sas.icons.HC.REVIEWSTATUS;
                        switch(status) {
                            case HprConstants.HPR_DATA_SOURCE_STATUS_VALID:
                                icon = sas.icons.HC.COMPLETESTATUS;
                                break;
                            case HprConstants.HPR_DATA_SOURCE_STATUS_NO_DESCRIPTOR:
                                icon = sas.icons.HC.ERRORSTATUS;
                                break;
                            case HprConstants.HPR_DATA_SOURCE_STATUS_MISSING_FILES:
                                icon = sas.icons.HC.ERRORSTATUS;
                                break;
                            case HprConstants.HPR_DATA_SOURCE_STATUS_TESTING:
                                icon = sas.icons.HC.PROGRESSSTATUS;
                                break;
                            case HprConstants.HPR_DATA_SOURCE_STATUS_MISSING_TYPE_PRICE:
                                icon = sas.icons.HC.ERRORSTATUS;
                                break;
                            case HprConstants.HPR_DATA_SOURCE_STATUS_MISSING_TYPE_STRESS:
                                icon = sas.icons.HC.ERRORSTATUS;
                                break;
                            case HprConstants.HPR_DATA_SOURCE_STATUS_WARNING_NOT_PRICESTRESS:
                                icon = sas.icons.HC.WARNINGSTATUS;
                                break;
                            default:
                                break;
                        }
                        return icon;
                    }
                }
            }),
            sortProperty: "status",
            showFilterMenuEntry: false,
            required: true
        });

        return [nameColumn, statusColumn];
    }

    /**
     * Updates the enabled states of the toolbar buttons.
     * @param event
     * @private
     */
    function _updateForSelection(event) {
        var selectedIndex = dataSourceTable.getSelectedIndex();

        // proceed if there is a selection
        if (selectedIndex >= 0) {
            // get the object represented by the selection
            var rowContext = dataSourceTable.getContextByIndex(selectedIndex);
            var rowObject = rowContext.getObject();

            // if the data source is not being tested, check its status
            if (rowObject.status !== HprConstants.HPR_DATA_SOURCE_STATUS_TESTING) {
                // test is enabled whenever the data source is not valid and not in a known error state
                testButton.setEnabled(rowObject.status === HprConstants.HPR_DATA_SOURCE_STATUS_UNKNOWN || rowObject.status === HprConstants.HPR_DATA_SOURCE_STATUS_WARNING_NOT_PRICESTRESS);

                // enabled whenever there is a selection
                replaceButton.setEnabled(true);

                // no further processing
                return;
            }
        }

        // if we made it here, disable toolbar buttons
        testButton.setEnabled(false);
        replaceButton.setEnabled(false);
    }

    /**
     * Build a unique set of data source metadata objects.
     *
     * @param dataSourceParams
     * @returns {Array}
     * @private
     */
    function _compressDataSourceParams(dataSourceParams) {
        var compressedDataSourceDataObjectsMap = {};
        _.each(dataSourceParams, function(dataSourceParamsObject) {
            if (!compressedDataSourceDataObjectsMap[dataSourceParamsObject.abstractDataSourceId]) {
                compressedDataSourceDataObjectsMap[dataSourceParamsObject.abstractDataSourceId] = {
                    abstractDataSourceId: dataSourceParamsObject.abstractDataSourceId,
                    id: dataSourceParamsObject.abstractDataSourceId,    // used to build the root params object to look up details
                    label: dataSourceParamsObject.abstractDataSourceId, // displayed to user
                    application: dataSourceParamsObject.application,
                    relPath: dataSourceParamsObject.relPath,
                    workGroup: dataSourceParamsObject.workGroup,
                    types: [dataSourceParamsObject.type],
                    savedRealDataSourceIds: [dataSourceParamsObject.realDataSourceId]
                };
            }
            else {
                compressedDataSourceDataObjectsMap[dataSourceParamsObject.abstractDataSourceId].types.push(dataSourceParamsObject.type);
                compressedDataSourceDataObjectsMap[dataSourceParamsObject.abstractDataSourceId].savedRealDataSourceIds.push(dataSourceParamsObject.realDataSourceId);
            }
            if (dataSourceStatusMap[dataSourceParamsObject.abstractDataSourceId]) {
                compressedDataSourceDataObjectsMap[dataSourceParamsObject.abstractDataSourceId].status = dataSourceStatusMap[dataSourceParamsObject.abstractDataSourceId].status;
            }
        });
        return _.values(compressedDataSourceDataObjectsMap);
    }

    /**
     * Loads the exploration and its data sources to populate the dialog.
     * @param explorationID
     * @private
     */
    function _populateData(explorationID) {
        dataSourceTable.setBusy(true);

        /**
         * Get the data source details. Failure here means the cube descriptor is missing. This is also the point
         * where we verify that the data source types (price, stress) that were previously associated with this
         * exploration are still valid.
         *
         * @param dataSourceRootParamObject
         * @returns Deferred
         * @private
         */
        var __checkDataSourceDetails = function(dataSourceDataObject) {
            var deferred = $.Deferred();

            HprAPI.getDataSourceDetails(DataSourceParams.prototype.createRootParams(dataSourceDataObject)).done(
                function (dataSourceDetails) {
                    // add the params to the cache manager (needed for testing)
                    DataSourceCacheManager.updateDataSourceParams(dataSourceDetails);

                    // If the data source has been modified from the last time we checked it, check it again
                    var currentKnownDataSourceStatus = dataSourceStatusMap[dataSourceDataObject.abstractDataSourceId];
                    if (!currentKnownDataSourceStatus || currentKnownDataSourceStatus.lastModifiedTimeStamp !== dataSourceDetails.lastModifiedTimeStamp) {
                        // start in unknown status
                        var dsStatus = HprConstants.HPR_DATA_SOURCE_STATUS_UNKNOWN;

                        // make sure that the details contains the type (price and/or stress) that was previously stored
                        var dsTypes = _.map(dataSourceDetails.dataSources, "type");
                        if (dataSourceDataObject.types) {
                            // this was price or stress only (error)
                            if (dataSourceDataObject.types.length === 1 && !_.contains(dsTypes, dataSourceDataObject.types[0])) {
                                dsStatus = dataSourceDataObject.types[0] === HprConstants.HPR_CUBE_INFORMATION_TYPE_PRICE ? HprConstants.HPR_DATA_SOURCE_STATUS_MISSING_TYPE_PRICE : HprConstants.HPR_DATA_SOURCE_STATUS_MISSING_TYPE_STRESS;
                            }
                            // this was price/stress but is now price or stress only (warning)
                            else if (dataSourceDataObject.types.length === 2 && dsTypes.length === 1) {
                                dsStatus = HprConstants.HPR_DATA_SOURCE_STATUS_WARNING_NOT_PRICESTRESS;
                            }
                        }

                        // update the map with the new status
                        dataSourceStatusMap[dataSourceDataObject.abstractDataSourceId] = {
                            status: dsStatus,
                            lastModifiedTimeStamp: dataSourceDetails.lastModifiedTimeStamp
                        };
                    }

                    dataSourceDataObject.status = dataSourceStatusMap[dataSourceDataObject.abstractDataSourceId].status;
                    dataSourceDataObject.currentRealDataSourceIds = _.map(dataSourceDetails.dataSources, function(dataSource) {
                        return dataSource.id;
                    });

                    deferred.resolve();
                }
            ).fail(
                function (jqxhr, textStatus, error) {
                    dataSourceStatusMap[dataSourceDataObject.abstractDataSourceId] = {
                        status: HprConstants.HPR_DATA_SOURCE_STATUS_NO_DESCRIPTOR
                    };
                    dataSourceDataObject.status = dataSourceStatusMap[dataSourceDataObject.abstractDataSourceId].status;
                    deferred.resolve();
                }
            );
            return deferred;
        };

        HprAPI.loadSavedExploration(explorationID, true).done(
            function(explorationData) {
                var deferreds = [];

                var analysis = $.parseJSON(explorationData.analysis); // String of JSON for Exploration
                var dataSourceMetadataObjects = _compressDataSourceParams(analysis.dataSourceParams);
                _.each(dataSourceMetadataObjects, function(compressedDataSourceDataObject) {
                    deferreds.push(__checkDataSourceDetails(compressedDataSourceDataObject));
                });

                $.when.apply($, deferreds).done(
                    function() {
                        myModel.setData(
                            {
                                exploration: {
                                    id: explorationData.id,
                                    name: explorationData.name,
                                    analysis: analysis
                                },
                                dataSourceMetadata: dataSourceMetadataObjects
                            }
                        );
                    }
                ).always(
                    function() {
                        dataSourceTable.setBusy(false);
                    }
                );
            }
        ).fail(
            function(jqxhr, textStatus, error) {
                HprUtil.displayDetailedErrorDialog(jqxhr);
                myDialog.close();
            }
        );
    }

    /**
     * Tests the cube selected in the table by attempting to open its data sources. If all open successfully,
     * the data source metadata is updated with a valid status. If not, then it is updated with an error status
     * indicating why it failed to open.
     *
     * @private
     */
    function _testDataSource() {
        // sds = Selected Data Source
        var sdsMetadataPath = dataSourceTable.getContextByIndex(dataSourceTable.getSelectedIndex()).sPath;
        var sdsStatusPath = sdsMetadataPath + "/status";
        var sdsMetadata = myModel.getProperty(sdsMetadataPath);

        // update the state of the data source to testing
        myModel.setProperty(sdsStatusPath, HprConstants.HPR_DATA_SOURCE_STATUS_TESTING);

        // update the toolbar button states
        _updateForSelection();

        var dsDeferreds = [];
        _.each(sdsMetadata.currentRealDataSourceIds, function(id) {
            dsDeferreds.push(DataSourceCacheManager.getDataSource(id));
        });

        $.when.apply($, dsDeferreds).done(
            function(data) {
                dataSourceStatusMap[sdsMetadata.abstractDataSourceId].status = HprConstants.HPR_DATA_SOURCE_STATUS_VALID;
                myModel.setProperty(sdsStatusPath, HprConstants.HPR_DATA_SOURCE_STATUS_VALID);
            }
        ).fail(
            function (jqxhr, textStatus, error) {
                dataSourceStatusMap[sdsMetadata.abstractDataSourceId].status = HprConstants.HPR_DATA_SOURCE_STATUS_MISSING_FILES;
                myModel.setProperty(sdsStatusPath, HprConstants.HPR_DATA_SOURCE_STATUS_MISSING_FILES);
            }
        ).always(
            function() {
                myModel.refresh(true);      // force the table to update and reapply sort
                _updateForSelection();      // update the enabled states for the current selection
            }
        );
    }

    /**
     * Initiates the process of replacing the selected data source, using the DataSourceChangeManager.
     * @private
     */
    function _replaceSelectedDataSource() {
        // sds = Selected Data Source
        var sdsMetadataPath = dataSourceTable.getContextByIndex(dataSourceTable.getSelectedIndex()).sPath;
        var sdsStatusPath = sdsMetadataPath + "/status";
        var sdsMetadata = myModel.getProperty(sdsMetadataPath);
        var explorationAnalysis = myModel.getProperty("/exploration/analysis");

        var __successFunc = function(analysis, newDataSourceDetails) {
            // update the exploration in the model
            myModel.setProperty("/exploration/analysis", analysis);

            // we know this cube is valid if the replace succeeded
            dataSourceStatusMap[newDataSourceDetails.id] = {
                status: HprConstants.HPR_DATA_SOURCE_STATUS_VALID,
                lastModifiedTimeStamp: new Date(newDataSourceDetails.lastModifiedTimeStamp).getTime()
            };

            // recreate the data source metadata to include the new data source
            myModel.setProperty("/dataSourceMetadata", _compressDataSourceParams(analysis.dataSourceParams));
        };

        var __failFunc = function(jqxhr) {
            HprUtil.displayDetailedErrorDialog(jqxhr);
        };

        DataSourceChangeManager.replaceDataSourceForExploration(explorationAnalysis, sdsMetadata.label, sdsMetadata.savedRealDataSourceIds, false, myController, myDialog, __successFunc, __failFunc);
    }

    /**
     * Saves the updated exploration and closes the dialog.
     * @private
     */
    function _saveExploration() {
        // extract the updated exploration from the model
        var exploration = myModel.getProperty("/exploration");

        // generate the cube list
        exploration.dataSources = ExplorationUtil.generateDataSourceCubesFromAnalysisObject(exploration.analysis);

        // save the exploration
        HprAPI.updateExploration(exploration.id, exploration).done(
            function(savedExploration) {
                myDialog.close();
            }
        ).fail(
            function(jqxhr, textStatus, error) {
                HprUtil.displayDetailedErrorDialog(jqxhr);
            }
        )
    }

    /**
     * Cleans up the DataSourceCacheManager and resets all static-ish variables.
     * @private
     */
    function _cleanup() {
        // destroy the dialog and null the control references
        myDialog.destroy();

        myDialog = null;
        testButton = null;
        replaceButton = null;
        dataSourceTable = null;
        nameColumn = null;
        statusColumn = null;
        myController = null;

        // clear model data
        myModel.setData();

        // empty the data source cache manager
        DataSourceCacheManager.releaseAllLoadedDataSources();
    }


    // public

    return {
        /**
         * Creates the dialog for replacing exploration data sources.
         * @param controller
         * @returns {sas.hc.m.Dialog}
         */
        createContent: function (controller) {
            myController = controller;

            // have to create this here to assign it as the primary button later
            var okButton = new sas.hc.m.Button({
                text: rb.getText("hprisk.okButton.txt"),
                press: function(evt) {
                    _saveExploration();
                }
            });

            myDialog = new sas.hc.m.Dialog({
                title: rb.getText("ChooseDataSourceDialog.replace.data.source.title.txt"),
                content: [_createDataSourceTable()],
                // contentHeight: "600px",
                contentWidth: "50%",
                leftButton: okButton,
                rightButton: new sas.hc.m.Button({
                    text: rb.getText("hprisk.cancelButton.txt"),
                    press: function(evt) {
                        myDialog.close();
                    }
                }),
                defaultButton: okButton,
                afterClose: function(evt) {
                    _cleanup();
                }
            });
            myDialog.setBusyIndicatorDelay(0);

            // define function for launching and populating the dialog
            myDialog.launch = function(explorationID) {
                myDialog.setBusyIndicatorDelay(0);
                dataSourceTable.setBusyIndicatorDelay(0);

                myDialog.open();

                _populateData(explorationID);
            };

            return myDialog;
        },
    };

}());