// (c) 2017, SAS Institute Inc.

jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.analytics.riskhp.utils.HPRiskPanelsViewDelegate");
jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
jQuery.sap.require("sas.ltjs.sasicu.util.SASFormatterUtil");

sap.ui.jsfragment("sas.analytics.riskhp.fragments.options.CrosstabOptions", {

    RESPONSIVE_GRID_LAYOUT_SETTINGS: {
        labelSpanL: 6,
        emptySpanL: 0,
        labelSpanM: 6,
        emptySpanM: 0,
        labelSpanS: 12,
        emptySpanS: 0
    },

    _getResourceBundle: function () {
        return sas.analytics.riskhp.utils.HprUtil.getBundle();
    },

    createContent: function (controller) {
        var self = this;
        
        // Flags to tell Crosstab that is should requery or just redraw; Only happens when certain options are changed
        this.resetCrosstabRows = false;
        this.fullRequery = false;
        this.requery = false;
        this.redraw = false;
        this.liveChangeTimer = 0;

        this.generalWindowShadeSectionId = sas.analytics.riskhp.utils.HprConstants.HPR_OPTIONS_PANE_CROSSTAB_GENERAL_SECTION_ID;
        this.totalsWindowShadeSectionId = sas.analytics.riskhp.utils.HprConstants.HPR_OPTIONS_PANE_CROSSTAB_TOTALS_SECTION_ID;
        this.advancedQueryWindowShadeSectionId = sas.analytics.riskhp.utils.HprConstants.HPR_OPTIONS_PANE_CROSSTAB_ADVANCED_QUERY_SECTION_ID;

        this.windowShade = new sas.hc.ui.commons.windowshade.WindowShade("crosstabOptions", {});
        this.windowShade.setOpenFirstSection( false );
        this.windowShade.setWidth('100%');
        this.windowShade.addStyleClass("optionsPane_windowShade");

        // Attach event listeners for open and close of the WindowShadeSections
        this.windowShade.attachSectionOpen(this._onSectionOpenOrClose, this);
        this.windowShade.attachSectionClose(this._onSectionOpenOrClose, this);

        var generalOptionsSection = new sas.hc.ui.commons.windowshade.WindowShadeSection({
            id: this.generalWindowShadeSectionId,
            collapsed: "{/options/generalOptionsSectionCollapsed}"
        }).addStyleClass("optionsPane_WindowShadeSection");
        generalOptionsSection.setTitle(this._getResourceBundle().getText("OptionsPane.generalOptionsWindowShadeSection.txt"));
        generalOptionsSection.setTooltip(this._getResourceBundle().getText("OptionsPane.generalOptionsWindowShadeSection.tooltip.txt"));
        generalOptionsSection.addContent( this._buildCrosstabGeneralOptionsSection() );
        this.windowShade.addSection( generalOptionsSection );

        var totalsOptionsSection = new sas.hc.ui.commons.windowshade.WindowShadeSection({
            id: this.totalsWindowShadeSectionId,
            collapsed: "{/options/totalsOptionsSectionCollapsed}"
        }).addStyleClass("optionsPane_WindowShadeSection_NoTopMargin");
        totalsOptionsSection.setTitle(this._getResourceBundle().getText("OptionsPane.totalsOptionsWindowShadeSection.txt"));
        totalsOptionsSection.setTooltip(this._getResourceBundle().getText("OptionsPane.totalsOptionsWindowShadeSection.tooltip.txt"));
        totalsOptionsSection.addContent( this._buildCrosstabTotalsOptionsSection() );
        this.windowShade.addSection( totalsOptionsSection );

        var advancedQueryOptionsSection = new sas.hc.ui.commons.windowshade.WindowShadeSection({
            id: this.advancedQueryWindowShadeSectionId,
            collapsed: "{/options/advancedQueryOptionsSectionCollapsed}"
        }).addStyleClass("optionsPane_WindowShadeSection");
        advancedQueryOptionsSection.setTitle(this._getResourceBundle().getText("OptionsPane.advancedQueryOptionsWindowShadeSection.txt"));
        advancedQueryOptionsSection.setTooltip(this._getResourceBundle().getText("OptionsPane.advancedQueryOptionsWindowShadeSection.tooltip.txt"));
        advancedQueryOptionsSection.addContent( this._buildCrosstabAdvancedQueryOptionsSection() );
        this.windowShade.addSection( advancedQueryOptionsSection );

        this.borderLayout = new sas.hc.ui.commons.layout.BorderLayout({
            width: "100%",
            height: "100%",
            center: new sas.hc.ui.commons.layout.BorderLayoutArea({
                overflowX: "hidden",
                overflowY: "hidden",
                content: [this.windowShade]
            }).addStyleClass("optionsPane_centerLayoutArea"),
            bottom: new sas.hc.ui.commons.layout.BorderLayoutArea({
                overflowX: "hidden",
                overflowY: "hidden",
                size: "5.0rem",
                content: [this._buildFooter()]
            })
        });

        // Workaround to set WindowShade.openedSectionsId property so it matches which sections we have open in the model
        var updateWindowShadeOpenedSectionsIdProperty = function() {
            // Gather the model and appropriate properties
            var model = self.borderLayout.getModel();
            var generalOptionsSectionCollapsed = model.getProperty("/options/generalOptionsSectionCollapsed");
            var totalsOptionsSectionCollapsed = model.getProperty("/options/totalsOptionsSectionCollapsed");
            var advancedQueryOptionsSectionCollapsed = model.getProperty("/options/advancedQueryOptionsSectionCollapsed");

            // Create a new 'openedSectionsId' array with any of our model properties that are opened (!collapsed means the section is open);
            // When finished, update the WindowShade with the new 'openedSectionsId' value
            var openedSectionsId = [];
            if (!generalOptionsSectionCollapsed) {openedSectionsId.push(self.generalWindowShadeSectionId);}
            if (!totalsOptionsSectionCollapsed) {openedSectionsId.push(self.totalsWindowShadeSectionId);}
            if (!advancedQueryOptionsSectionCollapsed) {openedSectionsId.push(self.advancedQueryWindowShadeSectionId);}
            self.windowShade.setOpenedSectionsId(openedSectionsId);
        };
        this.borderLayout.updateWindowShadeOpenedSectionsIdProperty = updateWindowShadeOpenedSectionsIdProperty;

        // Allow access to the indentRowsCheckBox
        var updateCrosstabIndent = function(indent) {
            self._onUpdateCrosstabIndent(indent);
        };
        this.borderLayout.updateCrosstabIndent = updateCrosstabIndent;

        return this.borderLayout;
    },

    _onSectionOpenOrClose: function(evt) {
        // When a WindowShadeSection is opened | closed, we need to keep the Crosstab context option properties in-sync with the model changes
        var model = $.extend(true, {}, this.borderLayout.getModel()); // Create copy

        // Determine the WindowShadeSection and whether it was opened | closed
        var sectionId = null;
        var sectionOpened = false;
        if (evt.getParameter('openSectionId')) {
            sectionId = evt.getParameter('openSectionId');
            sectionOpened = true;
        } else {
            sectionId = evt.getParameter('closeSectionId');
        }

        // Update the Crosstab context with the required properties to keep it in-sync;  We need these updated to be able to save which WindowShadeSections were last opened | closed and restore them.
        sas.analytics.riskhp.models.HPRiskModelManager.updateCrosstabOptionSectionProperties(model.getProperty("/options/id"),
            model.getProperty("/options/generalOptionsSectionCollapsed"), model.getProperty("/options/totalsOptionsSectionCollapsed"), model.getProperty("/options/advancedQueryOptionsSectionCollapsed"));

        // Publish an event that will update the dirtyState of the Exploration view
        sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {
            dirty: true
        });
    },

    _buildFooter: function() {
        var self = this;

        self.applyButton = new sas.hc.m.Button({
            id: this.createId('_crosstabOptions_applyButton'),
            text: this._getResourceBundle().getText("OptionsPane.applyButton.txt"),
            enabled: "{/options/applyEnabled}",
            press: $.proxy(self._onApplyButtonPressed, self)
        }).addStyleClass("sasPrimaryButton");

        self.cancelButton = new sas.hc.m.Button({
            id: this.createId('_crosstabOptions_cancelButton'),
            text: this._getResourceBundle().getText("hprisk.cancelButton.txt"),
            enabled: "{/options/applyEnabled}", // Defect S1387426: Should behave like Apply Button state
            press: $.proxy(self._onCancelButtonPressed, self)
        });

        var tb = new sas.hc.m.ResponsiveToolbar({
            content: [new sas.hc.m.ToolbarSpacer(), self.applyButton, self.cancelButton]
        });
        tb.addStyleClass("noBackground");

        return tb;
    },

    _buildCrosstabGeneralOptionsSection: function() {
        var self = this;

        self.nameInput = new sas.hc.m.Input(this.createId("_crosstabGeneralOptionsNameInput"), {
            label: this._getResourceBundle().getText("OptionsPane.generalOptions.name.inputLabel.txt"),
            labelPosition: sas.hc.m.InputLabelPosition.Top,
            liveChange: $.proxy(self._onNameFieldLiveChange, self),
            value: "{/options/name}",
            width: "200px",
            required: true
        });

        var RISK_CONFIDENCE_LEVEL_MAX_FRACTION_DIGITS = 15;
        var RISK_CONFIDENCE_LEVEL_DEFAULT_VALUE = 95;
        var RISK_CONFIDENCE_LEVEL_MIN_VALUE = .0001;
        var RISK_CONFIDENCE_LEVEL_MAX_VALUE = 99.9999;
        self.riskConfidenceLevelStepper = new sas.hc.m.NumericStepper(this.createId("_crosstabGeneralOptionsRiskConfidenceLevelStepper"), {
            required: true,
            stepSize: 1,
            width: "170px",
            value: {
                path: "/options/alpha/value",
                type: new sap.ui.model.type.Float({
                    maxFractionDigits: RISK_CONFIDENCE_LEVEL_MAX_FRACTION_DIGITS
                }, {
                    minimum: RISK_CONFIDENCE_LEVEL_MIN_VALUE,
                    maximum: RISK_CONFIDENCE_LEVEL_MAX_VALUE
                }),
                formatter: function(value) {
                    // Use the default value if there is no initial value
                    if (!value) return RISK_CONFIDENCE_LEVEL_DEFAULT_VALUE;

                    // Special processing is needed for 'alpha' because the UI component needs to display a percentage, but mid-tier uses decimal format (i.e. .05)
                    return self._adjustAlphaValueForUIDisplay(value);
                }
            },
            enabled: { // Not enabled for stress cube.
                path: "/options/id",
                formatter: function(id) {
                    var crosstabContext = sas.analytics.riskhp.models.HPRiskModelManager.getCrosstabContext(id);
                    if (crosstabContext)
                        return !Boolean(crosstabContext.cubeDataModel.isStressCube());
                    else
                        return true;
                }
            },
            liveChange: $.proxy(self._handleAlphaChange, self, false)
        });

        var MAXIMUM_ROWS_DEFAULT_VALUE = 25;
        var MAXIMUM_ROWS_MIN_VALUE = 5;
        var MAXIMUM_ROWS_MAX_VALUE = 10000;
        self.maximumRowsStepper = new sas.hc.m.NumericStepper(this.createId("_crosstabGeneralOptionsMaximumRowsStepper"), {
            required: true,
            stepSize: 1,
            width: "170px",
            value: {
                path: "/options/nrows/value",
                type: new sap.ui.model.type.Integer({
                    minIntegerDigits: 1, // minimal number of non-fraction digits
                    maxIntegerDigits: 5, // maximal number of non-fraction digits
                    minFractionDigits: 0, // minimal number of fraction digits
                    maxFractionDigits: 0  // maximal number of fraction digits
                }, {
                    minimum: MAXIMUM_ROWS_MIN_VALUE,
                    maximum: MAXIMUM_ROWS_MAX_VALUE
                }),
                formatter: function(value) {
                    // Use the default value when there is no initial value
                    if (!value) return MAXIMUM_ROWS_DEFAULT_VALUE;

                    return value;
                }

            },
            liveChange: $.proxy(self._handleMaxRowsChange, self, true)
        });

        // Set a default value for the Select component.  However, the real default value will be set in the Options model by
        // pulling the Display Horizons Application Setting value
        var DISPLAYHORIZONS_SELECT_DEFAULT_OPTION = {
            key: 'absolute', text: this._getResourceBundle().getText("OptionsPane.generalOptions.displayHorizons.absolute.txt")
        };
        var DISPLAYHORIZONS_SELECT_OPTIONS = [
            DISPLAYHORIZONS_SELECT_DEFAULT_OPTION,
            {key: 'relative', text: this._getResourceBundle().getText("OptionsPane.generalOptions.displayHorizons.relative.txt")}
        ];
        self.displayHorizonsSelect = new sas.hc.m.Select(this.createId("_crosstabDisplayHorizonsSelect"), {
            width: "200px",
            selectedKey: {
                path: "/options/displayHorizons/value",
                formatter: function(value) {
                    // Determine if value exists in available options, if not return default value; The real default value
                    // is determined by the Application Settings in the Options model when the Crosstab stands up
                    var objectMatch = sas.analytics.riskhp.utils.HprUtil.findObject(DISPLAYHORIZONS_SELECT_OPTIONS, value, "key");
                    if (objectMatch) return value;
                    // Return a default value, this will probably never have to be used because a default value is pulled from the Application Settings
                    return DISPLAYHORIZONS_SELECT_DEFAULT_OPTION.key;

                }
            },
            change: $.proxy(self._handleDisplayHorizonsChange, self)
        });
        sas.analytics.riskhp.utils.HprUtil.addListItems(self.displayHorizonsSelect, DISPLAYHORIZONS_SELECT_OPTIONS, "text", "key");

        self.transposeCheckBox = new sas.hc.m.CheckBox({
            id: this.createId('_crosstabTransposeCB'),
            text: this._getResourceBundle().getText("OptionsPane.generalOptions.transposeLabel.txt"),
            selected: {
                path: "/options/transpose/value",
                formatter: function(value) {
                    var valueAsBoolean = Boolean(value);
                    return valueAsBoolean;
                }
            },
            select: $.proxy(self._handleTransposeChange, self)
        }).addStyleClass("optionsPane_CB_NoMargin");

        self.indentRowsCheckBox = new sas.hc.m.CheckBox({
            id: this.createId('_crosstabIndentRowsCB'),
            text: this._getResourceBundle().getText("OptionsPane.generalOptions.indentRowsLabel.txt"),
            enabled: {
                path: "/options/transpose/value",
                formatter: function(value) {
                    return !Boolean(value);
                }
            },
            selected: {
                parts: ["/options/indentRows/value", "/options/transpose/value"],
                formatter: function(value, transposeValue) {
                    if (Boolean(transposeValue) === true) {
                        return false;
                    }
                    var valueAsBoolean = Boolean(value);
                    return valueAsBoolean;
                }
            },
            select: $.proxy(self._handleIndentRowChange, self)
        }).addStyleClass("optionsPane_CB_NoMargin");

        self.showLabelsCheckBox = new sas.hc.m.CheckBox({
            id: this.createId('_crosstabShowLabelsCB'),
            text: this._getResourceBundle().getText("OptionsPane.generalOptions.showLabelsLabel.txt"),
            enabled: true,
            selected: {
                path: "/options/showLabels/value",
                formatter: function(value) {
                    var valueAsBoolean = Boolean(value);
                    return valueAsBoolean;
                }
            },
            select: $.proxy(self._handleShowLabelsChange, self)
        }).addStyleClass("optionsPane_CB_NoMargin");

        var crosstabGeneralOptionsForm  = new sas.hc.ui.layout.form.Form({
            layout: new sas.hc.ui.layout.form.ResponsiveGridLayout(this.RESPONSIVE_GRID_LAYOUT_SETTINGS),
            formContainers: [
                new sas.hc.ui.layout.form.FormContainer({
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement(this.createId("_crosstabGeneralOptionsNameInputFormElement"), {
                            fields: [self.nameInput]
                        }),
                        new sas.hc.ui.layout.form.FormElement(this.createId("_crosstabGeneralOptionsRiskConfidenceLevelFormElement"), {
                            label: new sas.hc.m.Label({
                                text: this._getResourceBundle().getText("OptionsPane.generalOptions.riskConfidenceLevelStepperLabel.txt")
                            }),
                            fields: [self.riskConfidenceLevelStepper]
                        }),
                        new sas.hc.ui.layout.form.FormElement(this.createId("_crosstabGeneralOptionsMaximumRowsFormElement"), {
                            label: new sas.hc.m.Label({
                                text: this._getResourceBundle().getText("OptionsPane.generalOptions.maximumRowsStepperLabel.txt")
                            }),
                            fields: [self.maximumRowsStepper]
                        }),
                        new sas.hc.ui.layout.form.FormElement(this.createId("_crosstabGeneralOptionsDisplayHorizonsSelectFormElement"), {
                            label: new sas.hc.m.Label({
                                text: this._getResourceBundle().getText("OptionsPane.generalOptions.displayHorizonsLabel.txt")
                            }),
                            fields: [self.displayHorizonsSelect]
                        }),
                        new sas.hc.ui.layout.form.FormElement(this.createId("_crosstabGeneralOptionsTransposeFormElement"), {
                            fields: [self.transposeCheckBox]
                        }),
                        new sas.hc.ui.layout.form.FormElement(this.createId("_crosstabGeneralOptionsIndentRowsFormElement"), {
                            fields: [self.indentRowsCheckBox]
                        }),
                        new sas.hc.ui.layout.form.FormElement(this.createId("_crosstabGeneralOptionsShowLabelsFormElement"), {
                            fields: [self.showLabelsCheckBox]
                        })
                    ]
                })
            ]
        }).addStyleClass("sasHPRiskCommon-noFormPadding optionsPane_Form");

        return new sas.hc.ui.layout.VerticalLayout({
            content: [ crosstabGeneralOptionsForm ]
        });
    },

    _buildCrosstabTotalsOptionsSection: function() {
        var self = this;

        self.totalsCheckBox = new sas.hc.m.CheckBox({
            id: this.createId('_crosstabTotalsCB'),
            text: this._getResourceBundle().getText("OptionsPane.totalsOptionsTotalsLabel.txt"),
            selected: {
                path: "/options/totals/value",
                formatter: function(value) {
                    var valueAsBoolean = Boolean(value);
                    return valueAsBoolean;
                }
            },
            //select: $.proxy(self._enableFullRequeryChangeOrSelect, self)
            //select: $.proxy(self._enableRequeryChangeOrSelect, self)
            select: $.proxy(self._handleTotalsChange, self)
        }).addStyleClass("optionsPane_CB_NoMargin");

        self.subTotalsCheckBox = new sas.hc.m.CheckBox({
            id: this.createId('_crosstabSubTotalsCB'),
            text: this._getResourceBundle().getText("OptionsPane.totalsOptionsSubTotalsLabel.txt"),
            enabled: {
                path: "/options/indentRows/value",
                formatter: function(indentRowsValue) {
                    return !Boolean(indentRowsValue);
                }
            },
            selected: {
                path: "/options/subtotals/value",
                formatter: function (value) {
                    return Boolean(value);
                }
            },
            //select: $.proxy(self._enableFullRequeryChangeOrSelect, self)
            //select: $.proxy(self._enableRequeryChangeOrSelect, self)
            select: $.proxy(self._handleSubtotalsChange, self)
        }).addStyleClass("optionsPane_CB_NoMargin");

        self.totalsOptionsRBG = new sas.hc.m.RadioButtonGroup(this.createId("_crosstabTotalsOptionsRBG"), {
            selectedIndex: {
                path: "/options/location/value",
                formatter: function(value) {
                    return value ? 1 : 0;
                }
            },
            buttons: [
                new sas.hc.m.RadioButton({
                    text: this._getResourceBundle().getText("OptionsPane.totalsOptionsBeforeRBLabel.txt")
                }),
                new sas.hc.m.RadioButton({
                    text: this._getResourceBundle().getText("OptionsPane.totalsOptionsAfterRBLabel.txt")
                })
            ],
            select: $.proxy(self._handleLocationChange, self)
        }).addStyleClass("totalOptionsRBG");

        var crosstabTotalsOptionsForm  = new sas.hc.ui.layout.form.Form({
            layout: new sas.hc.ui.layout.form.ResponsiveGridLayout(this.RESPONSIVE_GRID_LAYOUT_SETTINGS),
            formContainers: [
                new sas.hc.ui.layout.form.FormContainer({
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement(this.createId("_totalsOptionsTotalsCBFormElement"), {
                            fields: [self.totalsCheckBox]
                        }),
                        new sas.hc.ui.layout.form.FormElement(this.createId("_totalsOptionsSubTotalsCBFormElement"), {
                            fields: [self.subTotalsCheckBox]
                        }),
                        new sas.hc.ui.layout.form.FormElement(this.createId("_totalsOptionsRBGFormElement"), {
                            label: new sas.hc.m.Label({
                                text: this._getResourceBundle().getText("OptionsPane.totalsOptionsRBGLabel.txt")
                            }),
                            fields: [self.totalsOptionsRBG]
                        })
                    ]
                })
            ]
        }).addStyleClass("sasHPRiskCommon-noFormPadding optionsPane_Form");

        return new sas.hc.ui.layout.VerticalLayout({
            content: [ crosstabTotalsOptionsForm ]
        });
    },

    _buildCrosstabAdvancedQueryOptionsSection: function() {
        var self = this;

        self.advancedQueryOptionsRBG = new sas.hc.m.RadioButtonGroup(this.createId("_crosstabAdvancedQueryOptionsRBG"), {
            selectedIndex: {
                path: "/options/pctldef/value",
                formatter: function(value) {
                    // Valid values are "0 - 4" for RadioButtonGroup; value from JSON is "1 - 5", so we have to adjust
                    var adjustedValue = value && (value >=1 && value <= 5) ? value - 1 : 0; // Use 0 as default if value not in valid range
                    return adjustedValue;
                }
            },

            buttons: [
                new sas.hc.m.RadioButton({
                    text: this._getResourceBundle().getText("OptionsPane.advancedQueryOptions.weightedAverageLeftRBLabel.txt")
                }),
                new sas.hc.m.RadioButton({
                    text: this._getResourceBundle().getText("OptionsPane.advancedQueryOptions.observationLeftRBLabel.txt")
                }),
                new sas.hc.m.RadioButton({
                    text: this._getResourceBundle().getText("OptionsPane.advancedQueryOptions.empiricalDistributionRBLabel.txt")
                }),
                new sas.hc.m.RadioButton({
                    text: this._getResourceBundle().getText("OptionsPane.advancedQueryOptions.weightedAverageRightRBLabel.txt")
                }),
                new sas.hc.m.RadioButton({
                    text: this._getResourceBundle().getText("OptionsPane.advancedQueryOptions.empiricalDistributionAverageRBLabel.txt")
                })
            ],
            enabled: { // Not enabled for stress cube.
                path: "/options/id",
                formatter: function(id) {
                    var crosstabContext = sas.analytics.riskhp.models.HPRiskModelManager.getCrosstabContext(id);
                    if (crosstabContext)
                        return !Boolean(crosstabContext.cubeDataModel.isStressCube());
                    else
                        return true;
                }
            },
            select: $.proxy(self._handlePercentileDefinitionChange, self)
        });

        var crosstabAdvancedQueryOptionsForm  = new sas.hc.ui.layout.form.Form({
            layout: new sas.hc.ui.layout.form.ResponsiveGridLayout(this.RESPONSIVE_GRID_LAYOUT_SETTINGS),
            formContainers: [
                new sas.hc.ui.layout.form.FormContainer({
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement(this.createId("_crosstabAdvancedQueryOptionsRBGFormElement"), {
                            label: new sas.hc.m.Label({
                                text: this._getResourceBundle().getText("OptionsPane.advancedQueryOptions.rbgLabel.txt")
                            }),
                            fields: [self.advancedQueryOptionsRBG]
                        })
                    ]
                })
            ]
        }).addStyleClass("sasHPRiskCommon-noFormPadding optionsPane_Form");

        return new sas.hc.ui.layout.VerticalLayout({
            content: [ crosstabAdvancedQueryOptionsForm ]
        });
    },


    //
    // S1410237: handlers for form field changes that allow us to update the model as
    // changes are made, in order to support preserving settings when switching between
    // panels.
    //

    _handleAlphaChange: function(resetCrosstab, evt) {
        this._onNumericStepperLiveChange(resetCrosstab, evt, HprConstants.HPR_OPTIONS_PANE_ALPHA_NAME);
    },

    _handleMaxRowsChange: function(resetCrosstab, evt) {
        this._onNumericStepperLiveChange(resetCrosstab, evt, HprConstants.HPR_OPTIONS_PANE_NROWS_NAME);
    },

    _handleDisplayHorizonsChange: function(evt) {
        this._enableRequeryChangeOrSelect(evt);
        this._updateModelForDisplayHorizonsChange(this.borderLayout.getModel());
    },

    _handleTransposeChange: function(evt) {
        this._transposeSelect(evt);
        this._updateModelForTransposeChange(this.borderLayout.getModel());
    },

    _handleIndentRowChange: function(evt) {
        this._indentRowSelect(evt);
        this._updateModelForIndentRowChange(this.borderLayout.getModel());
    },

    _handleShowLabelsChange: function(evt) {
        this._enableRequeryChangeOrSelect(evt);
        this._updateModelForShowLabelsChange(this.borderLayout.getModel());
    },

    _handleTotalsChange: function(evt) {
        this._enableRedrawChangeOrSelect(evt);
        this._updateModelForTotalsChange(this.borderLayout.getModel());
    },

    _handleSubtotalsChange: function(evt) {
        this._enableRedrawChangeOrSelect(evt);
        this._updateModelForSubtotalsChange(this.borderLayout.getModel());
    },

    _handleLocationChange: function(evt) {
        this._enableRedrawChangeOrSelect(evt);
        this._updateModelForLocationChange(this.borderLayout.getModel());
    },

    _handlePercentileDefinitionChange: function(evt) {
        this._enableFullRequeryChangeOrSelect(evt);
        this._updateModelForPercentileDefinitionChange(this.borderLayout.getModel());
    },

    _onNameFieldLiveChange: function(evt) {
        var enabled = false;
        var source = evt.getSource();

        // Perform validation on Name field as the user types
        source._performValidation();

        if (source.getValueState() !== sap.ui.core.ValueState.Error) {
            enabled = true;
        }

        this._enableApplyButton(enabled);
    },

    _onNumericStepperLiveChange: function(resetCrosstab, evt, control) {
        var self = this;
        var source = evt.getSource();
        var newValue = source.getValue();
        var delay = (newValue ? 300 : 0); // no delay if value is empty

        // Execute search after user stopped typing for 300ms
        clearTimeout(self.liveChangeTimer);
        if (delay) {
            self.liveChangeTimer = setTimeout(function () {
                source._performValidation();
                if (source.inputField.getValueState() !== sap.ui.core.ValueState.Error) {
                    resetCrosstab ? self._enableResetCrosstabRowsChangeOrSelect() : self._enableFullRequeryChangeOrSelect();

                    // If the Stepper is in a valid state, update the model according to which Stepper is firing event
                    if (control === HprConstants.HPR_OPTIONS_PANE_ALPHA_NAME) {
                        self._updateModelForAlphaChange(self.borderLayout.getModel());
                    } else {
                        self._updateModelForMaxRowsChange(self.borderLayout.getModel());
                    }
                }
                else {
                    self._enableApplyButton(false);
                }
            }, delay);
        }
        else {
            source._performValidation();
            if (source.inputField.getValueState() !== sap.ui.core.ValueState.Error) {
                resetCrosstab ? self._enableResetCrosstabRowsChangeOrSelect() : self._enableFullRequeryChangeOrSelect();

                // If the Stepper is in a valid state, update the model according to which Stepper is firing event
                if (control === HprConstants.HPR_OPTIONS_PANE_ALPHA_NAME) {
                    self._updateModelForAlphaChange(self.borderLayout.getModel());
                } else {
                    self._updateModelForMaxRowsChange(self.borderLayout.getModel());
                }
            }
            else {
                self._enableApplyButton(false);
            }
        }
    },

    /**
     * An event handler for Change or Select events that sets a flag so the Crosstab will know to requery.  Only options that trigger a Crosstab requery should call
     * this method.
     * @param evt The Event object
     * @private
     */
    _enableRequeryChangeOrSelect: function(evt) {
        this.requery = true;
        this._enableApplyButton(true);
    },

    _enableFullRequeryChangeOrSelect: function(evt) {
        this.fullRequery = true;
        this.requery = true;
        this._enableApplyButton(true);
    },

    _enableResetCrosstabRowsChangeOrSelect: function(evt) {
        this.resetCrosstabRows = true;
        this.fullRequery = true;
        this.requery = true;
        this._enableApplyButton(true);
    },

    _enableRedrawChangeOrSelect: function(evt) {
        this.redraw = true;
        this._enableApplyButton(true);
    },

    _indentRowSelect: function(evt) {
        // if indent then subtotal on, location before
        if (this.indentRowsCheckBox.getSelected() === true) {
            this.subTotalsCheckBox.setSelected(true);
            this._updateModelForSubtotalsChange(this.borderLayout.getModel());

            this.totalsOptionsRBG.setSelectedIndex(0);
            this._updateModelForLocationChange(this.borderLayout.getModel());
        }
        this.redraw = true;
        this._enableApplyButton(true);
    },

    _transposeSelect: function(evt) {
        if (this.transposeCheckBox.getSelected() === true) {
            this.indentRowsCheckBox.setSelected(false);
            this._updateModelForIndentRowChange(this.borderLayout.getModel());
        }
        this.redraw = true;
        this._enableApplyButton(true);
    },

    _enableApplyButton: function(enabled) {
        // One last check to make sure Name Input field or NumericStepper fields are not in Error state
        if (this.nameInput.getValueState() === sap.ui.core.ValueState.Error ||
            this.riskConfidenceLevelStepper.inputField.getValueState() === sap.ui.core.ValueState.Error ||
            this.maximumRowsStepper.inputField.getValueState() === sap.ui.core.ValueState.Error) {
            enabled = false;
        }

        this.applyButton.setEnabled(enabled);
        this.cancelButton.setEnabled(enabled);
    },

    _needHorizonDialog: function(crosstab) {
        var self = this;
        var needed = false;
        var horizons = crosstab.context.getAllHorizons();
        var displayHorizonSelectKey = this.displayHorizonsSelect.getSelectedKey();
        if (displayHorizonSelectKey === "relative"){
            needed = !_.every(horizons, function(horizon) {
                return horizon.interval !== undefined;
            });
        }
        else {
            needed = !_.every(horizons, function(horizon) {
                return horizon.horizonDate !== undefined && horizon.horizonDate.length > 0;
            });
        }
        return needed;
    },

    _doApply: function(crosstab) {
        var self = this;

        // Disable Apply button; This affects borderLayout model so do it before we make a copy
        this._enableApplyButton(false);

        //TODO--do we need to clone this? It should already be a clone.
        // NOTE: 'this.borderLayout' corresponds to this fragment's model since this fragment returns 'this.borderLayout' in createContent()
        var model = $.extend(true, {}, this.borderLayout.getModel()); // Create copy

        // Retrieve the option values from the model since they were just updated, and update the corresponding Crosstab context to keep that in-sync as well
        var options = model.getProperty("/options");
        sas.analytics.riskhp.models.HPRiskModelManager.updateCrosstabOptions(options);

        // If we should requery the Crosstab, locate the Crosstab using the current options id and rebuild the Crosstab; The Crosstab will requery
        // and pass along the options we just set via the new model
        if (self.requery) {
            //var crosstab = sap.ui.getCore().byId(options.id);
            if (self.resetCrosstabRows) {
                //crosstab.resetCrosstabRows();
                crosstab.onNrowsChange();
            }
            crosstab.rebuildCrosstab(self.fullRequery, self.resetCrosstabRows);
            self.resetCrosstabRows = false;
            self.fullRequery = false;
            self.requery = false;
            self.redraw = false;
        } else if (self.redraw) {
            //var crosstab = sap.ui.getCore().byId(options.id);
            crosstab.redraw();
            self.redraw = false;
        }

        // Publish an event that will update the name of the Crosstab
        sas.eventBus.publish(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_CROSSTAB_OPTIONS_NAME_UPDATED, {
            crosstabId: options.id,
            newCrosstabName: self.nameInput.getValue()
        });

        // Publish an event that will signal the 'Show labels' field was updated
        sas.eventBus.publish(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_CROSSTAB_OPTIONS_SHOW_LABELS_UPDATED, {
            crosstabId: options.id,
            showLabels: self.showLabelsCheckBox.getSelected()
        });

        // Publish an event that will signal the transpose field was updated
        sas.eventBus.publish(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_CROSSTAB_OPTIONS_TRANSPOSE_UPDATED, {
            crosstabId: options.id,
            transposed: self.transposeCheckBox.getSelected()
        });

        // Publish an event that will update the dirty state of the Exploration view
        sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {
            dirty: true
        });

        // Notify linked plots of the change will occur when the crosstab is refreshed
    },

    _onApplyButtonPressed: function(evt) {
        var self = this;
        var model = $.extend(true, {}, this.borderLayout.getModel()); // Create copy
        var options = model.getProperty("/options");
        var crosstab = sap.ui.getCore().byId(options.id);

        if (self._needHorizonDialog(crosstab)) {
            var horizonDialog = sap.ui.jsfragment("sas.analytics.riskhp.fragments.options.HorizonTypeChangeDialog", {
                messageText: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HorizonTypeChange.dialog.txt"),
                doContinue: function() {
                    self._doApply(crosstab);
               },
                doCancel: function() {

                }
            });
            horizonDialog.open();

        }
        else {
            self._doApply(crosstab);
        }
    },

    _onCancelButtonPressed: function(evt) {
        // Disable Apply button; This affects borderLayout model so do it before we make a copy
        this._enableApplyButton(false);

        // Revert components back to previous state by getting previous Crosstab context information;
        // We can't rely on the borderLayout model because it is bound and already updated at this point
        var model = $.extend(true, {}, this.borderLayout.getModel()); // Create copy
        var optionsId = model.getProperty("/options/id"); // id of the appropriate CrosstabContext and Crosstab
        var crosstabContext = sas.analytics.riskhp.models.HPRiskModelManager.getCrosstabContext(optionsId);

        // Set the name input field's value to the previous name
        this.nameInput.setValue(crosstabContext.getOptionName());

        // Revert all the other components
        this.maximumRowsStepper.setValue(crosstabContext.getOptionNROWS().value);
        this.displayHorizonsSelect.setSelectedKey(crosstabContext.getOptionDisplayHorizons().value);
        this.transposeCheckBox.setSelected(crosstabContext.getOptionTranspose().value);
        this.indentRowsCheckBox.setSelected(crosstabContext.getOptionIndentRows().value);
        this.showLabelsCheckBox.setSelected(crosstabContext.getOptionShowLabels().value);
        this.totalsCheckBox.setSelected(crosstabContext.getOptionTOTALS().value);
        this.subTotalsCheckBox.setSelected(crosstabContext.getOptionSUBTOTALS().value);
        this.totalsOptionsRBG.setSelectedIndex(crosstabContext.getOptionLOCATION().value);

        // Special processing is needed for 'alpha' because the UI component needs to display a percentage, but mid-tier uses decimal format (i.e. .05)
        var adjustedAlphaValue = this._adjustAlphaValueForUIDisplay(crosstabContext.getOptionALPHA().value);
        this.riskConfidenceLevelStepper.setValue(adjustedAlphaValue.toString());
        
        // pctldef requires special processing since UI expects 0-4 and mid-tier requires 1-5
        this.advancedQueryOptionsRBG.setSelectedIndex(crosstabContext.getOptionPCTLDEF().value - 1); // Take away one so UI component will show selected option correctly

        // Make sure indentRows is set and enabled properly based on transpose
        if (this.transposeCheckBox.getSelected() === true) {
            this.indentRowsCheckBox.setSelected(false);
            this.indentRowsCheckBox.setEnabled(false);
        } else {
            this.indentRowsCheckBox.setEnabled(true);
        }

        // Reset the requery and redraw flags
        this.resetCrosstabRows = false;
        this.fullRequery = false;
        this.requery = false;
        this.redraw = false;
    },

    _adjustAlphaValueForUIDisplay: function(alphaValue) {
        // Some of the locales pass this number as a String (i.e. pl); Need to massage the value so component and mid-tier can handle it
        if (_.isString(alphaValue)) {
            alphaValue = sas.ltjs.sasicu.util.SASFormatterUtil.parse(alphaValue, "BEST9.2");
        }

        var adjustedValue = Number((1 - alphaValue) * 100);
        adjustedValue = Math.round(adjustedValue * 10000) / 10000;
        return adjustedValue;
    },

    _onUpdateCrosstabIndent: function(indent) {
        this.indentRowsCheckBox.setSelected(indent);
    },


    //
    // S1410237: model update functions to enable preservation of unapplied selections when switching between panels.
    //

    _updateModelForAlphaChange: function(model) {
        // Special processing is needed for 'alpha' because the UI component displays a percentage but the mid-tier uses decimal format (i.e. .05)
        var newAlpha = Number((100.0 - sas.ltjs.sasicu.util.SASFormatterUtil.parse(this.riskConfidenceLevelStepper.getValue(), "BEST9.2")) * .01);
        model.setProperty("/options/alpha/value", newAlpha);
    },

    _updateModelForMaxRowsChange: function(model) {
        // Depending on the client locale, some locales place commas and/or whitespace in the value for the Numeric Stepper;
        // We need to clean those up before setting the value on the model so the query won't blow up
        var maxRowsStepperValue = this.maximumRowsStepper.getValue().replace(/[\s,.]+/g, "");
        model.setProperty("/options/nrows/value", sas.ltjs.sasicu.util.SASFormatterUtil.parse(maxRowsStepperValue, "BEST9"));
    },

    _updateModelForDisplayHorizonsChange: function(model) {
        model.setProperty("/options/displayHorizons/value", this.displayHorizonsSelect.getSelectedKey());
    },

    _updateModelForTransposeChange: function(model) {
        model.setProperty("/options/transpose/value", this.transposeCheckBox.getSelected());
    },

    _updateModelForIndentRowChange: function(model) {
        model.setProperty("/options/indentRows/value", this.indentRowsCheckBox.getSelected());
    },

    _updateModelForShowLabelsChange: function(model) {
        model.setProperty("/options/showLabels/value", this.showLabelsCheckBox.getSelected());
    },

    _updateModelForTotalsChange: function(model) {
        model.setProperty("/options/totals/value", this.totalsCheckBox.getSelected());
    },

    _updateModelForSubtotalsChange: function(model) {
        model.setProperty("/options/subtotals/value", this.subTotalsCheckBox.getSelected());
    },

    _updateModelForLocationChange: function(model) {
        model.setProperty("/options/location/value", this.totalsOptionsRBG.getSelectedIndex());
    },

    _updateModelForPercentileDefinitionChange: function(model) {
        // Special processing is needed for 'pctldef' because the UI component expects 0-4 but the mid-tier expects 1-5
        var adjustedAdvancedQueryOptionsRBGSelectedIndex = this.advancedQueryOptionsRBG.getSelectedIndex() + 1;
        model.setProperty("/options/pctldef/value", adjustedAdvancedQueryOptionsRBGSelectedIndex);
    }
});