// (c) 2017, SAS Institute Inc.

jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils");
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
jQuery.sap.require("sas.ltjs.sasicu.util.SASFormatterUtil");

sap.ui.jsfragment("sas.analytics.riskhp.fragments.options.PlotOptions", {

    RESPONSIVE_GRID_LAYOUT_SETTINGS: {
        labelSpanL: 6,
        emptySpanL: 0,
        labelSpanM: 6,
        emptySpanM: 0,
        labelSpanS: 12,
        emptySpanS: 0
    },

    _getResourceBundle: function() {
        return sas.analytics.riskhp.utils.HprUtil.getBundle();
    },

    createContent: function (controller) {
        var self = this;
        this.liveChangeTimer = 0;

        // Flag to tell Plot that it should requery; Only happens when certain options are changed
        this.requery = false;

        this.generalWindowShadeSectionId = sas.analytics.riskhp.utils.HprConstants.HPR_OPTIONS_PANE_PLOT_GENERAL_SECTION_ID;
        this.graphWindowShadeSectionId = sas.analytics.riskhp.utils.HprConstants.HPR_OPTIONS_PANE_PLOT_GRAPH_SECTION_ID;
        this.advancedQueryWindowShadeSectionId = sas.analytics.riskhp.utils.HprConstants.HPR_OPTIONS_PANE_PLOT_ADVANCED_QUERY_SECTION_ID;

        this.windowShade = new sas.hc.ui.commons.windowshade.WindowShade("plotOptions", {});
        this.windowShade.setOpenFirstSection( false );
        this.windowShade.setWidth('100%');
        this.windowShade.addStyleClass("optionsPane_windowShade");

        // Attach event listeners for open and close of the WindowShadeSections
        this.windowShade.attachSectionOpen(this._onSectionOpenOrClose, this);
        this.windowShade.attachSectionClose(this._onSectionOpenOrClose, this);

        this.generalOptionsSection = new sas.hc.ui.commons.windowshade.WindowShadeSection({
            id: this.generalWindowShadeSectionId,
            collapsed: "{/options/generalOptionsSectionCollapsed}"
        }).addStyleClass("optionsPane_WindowShadeSection");
        this.generalOptionsSection.setTitle(this._getResourceBundle().getText("OptionsPane.generalOptionsWindowShadeSection.txt"));
        this.generalOptionsSection.setTooltip(this._getResourceBundle().getText("OptionsPane.generalOptionsWindowShadeSection.tooltip.txt"));
        this.generalOptionsSection.addContent( this._buildPlotGeneralOptionsSection() );
        this.windowShade.addSection( this.generalOptionsSection );

        this.graphOptionsSection = new sas.hc.ui.commons.windowshade.WindowShadeSection({
            id: this.graphWindowShadeSectionId,
            visible: {
                path: "/options/plotType",
                formatter: function(plotType) {
                    // Only display this section for Density Plots
                    return plotType && sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils.isDensityPlot(plotType);
                }
            },
            collapsed: "{/options/graphOptionsSectionCollapsed}"
        }).addStyleClass("optionsPane_WindowShadeSection_NoTopMargin");
        this.graphOptionsSection.setTitle(this._getResourceBundle().getText("OptionsPane.graphOptionsWindowShadeSection.txt"));
        this.graphOptionsSection.setTooltip(this._getResourceBundle().getText("OptionsPane.graphOptionsWindowShadeSection.tooltip.txt"));
        this.graphOptionsSection.addContent( this._buildPlotGraphOptionsSection() );
        this.windowShade.addSection( this.graphOptionsSection );

        this.advancedQueryOptionsSection = new sas.hc.ui.commons.windowshade.WindowShadeSection({
            id: this.advancedQueryWindowShadeSectionId,
            visible: {
                path: "/options/plotType",
                formatter: function(plotType) {
                    // Only display this section for Density Plots
                    return plotType && sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils.isDensityPlot(plotType);
                }
            },
            collapsed: "{/options/advancedQueryOptionsSectionCollapsed}"
        }).addStyleClass("optionsPane_WindowShadeSection");
        this.advancedQueryOptionsSection.setTitle(this._getResourceBundle().getText("OptionsPane.advancedQueryOptionsWindowShadeSection.txt"));
        this.advancedQueryOptionsSection.setTooltip(this._getResourceBundle().getText("OptionsPane.advancedQueryOptionsWindowShadeSection.tooltip.txt"));
        this.advancedQueryOptionsSection.addContent( this._buildPlotAdvancedQueryOptionsSection() );
        this.windowShade.addSection( this.advancedQueryOptionsSection );

        // Need to create the linked section which we will place at the top
        this.borderLayout = new sas.hc.ui.commons.layout.BorderLayout({
            width: "100%",
            height: "100%",
            center: new sas.hc.ui.commons.layout.BorderLayoutArea({
                overflowX: "hidden",
                overflowY: "hidden",
                content: [this.windowShade]
            }).addStyleClass("optionsPane_centerLayoutArea"),
            bottom: new sas.hc.ui.commons.layout.BorderLayoutArea({
                overflowX: "hidden",
                overflowY: "hidden",
                size: "5.0rem",
                content: [this._buildFooter()]
            })
        });

        // Workaround to set NumericStepper component values; This was needed because in some cases the 'change'
        // event was not being fired for the NumericStepper because the same value was being used when the Options
        // were switched for different Plots; In other words, the 'change' event was not firing because the value
        // of the NumericStepper was the same for the Options of two Plots
        // var setRiskConfidenceLevelStepperValue = function(value) {
        //     if (self.riskConfidenceLevelStepper) {
        //         self.riskConfidenceLevelStepper.setValue(value);
        //     }
        // };
        // this.borderLayout.setRiskConfidenceLevelStepperValue = setRiskConfidenceLevelStepperValue;

        // Workaround to set WindowShade.openedSectionsId property so it matches which sections we have open in the model
        this.borderLayout.updateWindowShadeOpenedSectionsIdProperty = function() {
            // Gather the model and appropriate properties
            var model = self.borderLayout.getModel();
            var generalOptionsSectionCollapsed = model.getProperty("/options/generalOptionsSectionCollapsed");
            var graphOptionsSectionCollapsed = model.getProperty("/options/graphOptionsSectionCollapsed");
            var advancedQueryOptionsSectionCollapsed = model.getProperty("/options/advancedQueryOptionsSectionCollapsed");

            // Create a new 'openedSectionsId' array with any of our model properties that are opened (!collapsed means the section is open);
            // When finished, update the WindowShade with the new 'openedSectionsId' value
            var openedSectionsId = [];
            if (!generalOptionsSectionCollapsed) {openedSectionsId.push(self.generalWindowShadeSectionId);}
            if (!graphOptionsSectionCollapsed) {openedSectionsId.push(self.graphWindowShadeSectionId);}
            if (!advancedQueryOptionsSectionCollapsed) {openedSectionsId.push(self.advancedQueryWindowShadeSectionId);}
            self.windowShade.setOpenedSectionsId(openedSectionsId);
        };

        return this.borderLayout;
    },

    _onSectionOpenOrClose: function(evt) {
        // When a WindowShadeSection is opened | closed, we need to keep the Plot model option properties in-sync with the model changes
        var model = $.extend(true, {}, this.borderLayout.getModel()); // Create copy

        // Determine the WindowShadeSection and whether it was opened | closed
        var sectionId = null;
        var sectionOpened = false;
        if (evt.getParameter('openSectionId')) {
            sectionId = evt.getParameter('openSectionId');
            sectionOpened = true;
        } else {
            sectionId = evt.getParameter('closeSectionId');
        }

        // Update the Plot model with the required properties to keep it in-sync;  We need these updated to be able to save which WindowShadeSections were last opened | closed and restore them.
        sas.analytics.riskhp.models.HPRiskModelManager.updatePlotOptionSectionProperties(model.getProperty("/options/id"),
            model.getProperty("/options/generalOptionsSectionCollapsed"), model.getProperty("/options/graphOptionsSectionCollapsed"), model.getProperty("/options/advancedQueryOptionsSectionCollapsed"));

        // Publish an event that will update the dirtyState of the Exploration view
        sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {
            dirty: true
        });
    },

    _buildFooter: function() {
        var self = this;

        self.applyButton = new sas.hc.m.Button({
            id: this.createId('_plotOptions_applyButton'),
            text: this._getResourceBundle().getText("OptionsPane.applyButton.txt"),
            enabled: "{/options/applyEnabled}",
            press: $.proxy(self._onApplyButtonPressed, self)
        }).addStyleClass("sasPrimaryButton");

        self.cancelButton = new sas.hc.m.Button({
            id: this.createId('_plotOptions_cancelButton'),
            text: this._getResourceBundle().getText("hprisk.cancelButton.txt"),
            enabled: "{/options/applyEnabled}",  // Defect S1387426: Should behave like Apply Button state
            press: $.proxy(self._onCancelButtonPressed, self)
        });

        var tb = new sas.hc.m.ResponsiveToolbar({
            content: [new sas.hc.m.ToolbarSpacer(), self.applyButton, self.cancelButton]
        });
        tb.addStyleClass("noBackground");

        return tb;
    },

    _buildPlotGeneralOptionsSection: function() {
        var self = this;

        self.nameInput = new sas.hc.m.Input(this.createId("_plotGeneralOptionsNameInput"), {
            label: this._getResourceBundle().getText("OptionsPane.generalOptions.name.inputLabel.txt"),
            labelPosition: sas.hc.m.InputLabelPosition.Top,
            liveChange: $.proxy(self._onNameFieldLiveChange, self),
            value: "{/options/name}",
            width: "200px",
            required: true
        });

        var RISK_CONFIDENCE_LEVEL_MAX_FRACTION_DIGITS = 15;
        var RISK_CONFIDENCE_LEVEL_DEFAULT_VALUE = 95;
        var RISK_CONFIDENCE_LEVEL_MIN_VALUE = .0001;
        var RISK_CONFIDENCE_LEVEL_MAX_VALUE = 99.9999;
        self.riskConfidenceLevelStepper = new sas.hc.m.NumericStepper(this.createId("_plotGeneralOptionsRiskConfidenceLevelStepper"), {
            required: true,
            stepSize: 1,
            width: "170px",
            enabled: {
                path: "/options/linkedToCrosstab",
                formatter: function(linkedToCrosstab) {
                    return !linkedToCrosstab;
                }
            },
            value: {
                path: "/options/alpha/value",
                type: new sap.ui.model.type.Float({
                    maxFractionDigits: RISK_CONFIDENCE_LEVEL_MAX_FRACTION_DIGITS
                }, {
                    minimum: RISK_CONFIDENCE_LEVEL_MIN_VALUE,
                    maximum: RISK_CONFIDENCE_LEVEL_MAX_VALUE
                }),
                formatter: function(value) {
                    // Use the default value if there is no initial value
                    if (!value) return RISK_CONFIDENCE_LEVEL_DEFAULT_VALUE;

                    // Special processing is needed for 'alpha' because the UI component needs to display a percentage, but mid-tier uses decimal format (i.e. .05)
                    return self._adjustAlphaValueForUIDisplay(value);
                }
            },
            liveChange: $.proxy(self._handleAlphaChange, self)
        });

        // Set a default value for the Select component.  However, the real default value will be set in the Options model by
        // pulling the Display Horizons Application Setting value
        var DISPLAYHORIZONS_SELECT_DEFAULT_OPTION = {
            key: 'absolute', text: this._getResourceBundle().getText("OptionsPane.generalOptions.displayHorizons.absolute.txt")
        };
        var DISPLAYHORIZONS_SELECT_OPTIONS = [
            DISPLAYHORIZONS_SELECT_DEFAULT_OPTION,
            {key: 'relative', text: this._getResourceBundle().getText("OptionsPane.generalOptions.displayHorizons.relative.txt")}
        ];
        self.displayHorizonsSelect = new sas.hc.m.Select(this.createId("_plotDisplayHorizonsSelect"), {
            width: "200px",
            selectedKey: {
                path: "/options/displayHorizons/value",
                formatter: function(value) {
                    // Determine if value exists in available options, if not return default value; The real default value
                    // is determined by the Application Settings in the Options model when the Plot stands up
                    var objectMatch = sas.analytics.riskhp.utils.HprUtil.findObject(DISPLAYHORIZONS_SELECT_OPTIONS, value, "key");
                    if (objectMatch) return value;
                    // Return a default value, this will probably never have to be used because a default value is pulled from the Application Settings
                    return DISPLAYHORIZONS_SELECT_DEFAULT_OPTION.key;
                    
                }
            },
            change: $.proxy(self._handleDisplayHorizonsChange, self)
        });
        sas.analytics.riskhp.utils.HprUtil.addListItems(self.displayHorizonsSelect, DISPLAYHORIZONS_SELECT_OPTIONS, "text", "key");

        self.showLabelsCheckBox = new sas.hc.m.CheckBox(this.createId('_plotGeneralOptionsShowLabelsCB'), {
            text: this._getResourceBundle().getText("OptionsPane.generalOptions.showLabelsLabel.txt"),
            enabled: true,
            selected: {
                path: "/options/showLabels/value",
                formatter: function(value) {
                    return Boolean(value);
                }
            },
            select: $.proxy(self._handleShowLabelsChange, self, true)  // no need to requery, so just enable Apply button
        }).addStyleClass("optionsPane_CB_MarginBottom");

        var plotGeneralOptionsForm  = new sas.hc.ui.layout.form.Form({
            layout: new sas.hc.ui.layout.form.ResponsiveGridLayout(this.RESPONSIVE_GRID_LAYOUT_SETTINGS),
            formContainers: [
                new sas.hc.ui.layout.form.FormContainer({
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement(this.createId("_plotGeneralOptionsNameInputFormElement"), {
                            fields: [self.nameInput]
                        }),
                        new sas.hc.ui.layout.form.FormElement(this.createId("_plotGeneralOptionsRiskConfidenceLevelFormElement"), {
                            label: new sas.hc.m.Label({
                                text: this._getResourceBundle().getText("OptionsPane.generalOptions.riskConfidenceLevelStepperLabel.txt")
                            }),
                            visible: {
                                path: "/options/plotType",
                                formatter: function(plotType) {
                                    var CHART_UTILS = sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils;
                                    // Only display this section for Density Plots and Distribution by Subportfolio
                                    return plotType && (CHART_UTILS.isSubportfolioDistributionPlot(plotType) ||
                                            CHART_UTILS.isDensityPlot(plotType) || CHART_UTILS.isComparisonPlot(plotType));
                                }
                            },
                            fields: [self.riskConfidenceLevelStepper]
                        }),
                        new sas.hc.ui.layout.form.FormElement(this.createId("_plotOptionsDisplayHorizonsSelectFormElement"), {
                            label: new sas.hc.m.Label({
                                text: this._getResourceBundle().getText("OptionsPane.generalOptions.displayHorizonsLabel.txt")
                            }),
                            visible: false, // TODO: dacagl - Not visible right now until we can figure out how to handle Absolute vs. Relative Horizon labels on TimeSeries graph
                            // visible: {
                            //     path: "/options/plotType",
                            //     formatter: function(plotType) {
                            //         // Only display this section plots that contain Horizons
                            //         return plotType && sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils.isByTypeHorizon(plotType);
                            //     }
                            // },
                            fields: [self.displayHorizonsSelect]
                        }),
                        new sas.hc.ui.layout.form.FormElement(this.createId("_plotGeneralOptionsShowLabelsFormElement"), {
                            fields: [self.showLabelsCheckBox]
                        })
                    ]
                })
            ]
        }).addStyleClass("sasHPRiskCommon-noFormPadding optionsPane_Form");

        return new sas.hc.ui.layout.VerticalLayout({
            content: [ plotGeneralOptionsForm ]
        });
    },

    _buildPlotGraphOptionsSection: function() {
        var self = this;

        var DENSITY_GRAPH_POINTS_SELECT_DEFAULT_OPTION = {
            key: 32, text: "32"
        };
        var DENSITY_GRAPH_POINTS_SELECT_OPTIONS = [
            {key: 16, text: "16"},
            DENSITY_GRAPH_POINTS_SELECT_DEFAULT_OPTION,
            {key: 64, text: "64"},
            {key: 128, text: "128"},
            {key: 256, text: "256"},
            {key: 512, text: "512"},
            {key: 1024, text: "1024"}
        ];
        self.densityGraphPointsSelect = new sas.hc.m.Select(this.createId("_plotDensityGraphPointsSelect"), {
            width: "200px",
            enabled: true,
            selectedKey: {
                path: "/options/ndensity/value",
                formatter: function(value) {
                    // Determine if value exists in available options, if not return default value
                    var objectMatch = sas.analytics.riskhp.utils.HprUtil.findObject(DENSITY_GRAPH_POINTS_SELECT_OPTIONS, value, "key");
                    if (objectMatch) return value;
                    return DENSITY_GRAPH_POINTS_SELECT_DEFAULT_OPTION.key; // Return default value
                }
            },
            change: $.proxy(self._handleDensityChange, self)
        });
        sas.analytics.riskhp.utils.HprUtil.addListItems(self.densityGraphPointsSelect, DENSITY_GRAPH_POINTS_SELECT_OPTIONS, "text", "key");

        var plotGraphOptionsForm  = new sas.hc.ui.layout.form.Form({
            layout: new sas.hc.ui.layout.form.ResponsiveGridLayout(this.RESPONSIVE_GRID_LAYOUT_SETTINGS),
            formContainers: [
                new sas.hc.ui.layout.form.FormContainer({
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement(this.createId("_plotOptionsDensitySelectFormElement"), {
                            label: new sas.hc.m.Label({
                                text: this._getResourceBundle().getText("OptionsPane.graphOptions.densityPointsLabel.txt")
                            }),
                            fields: [self.densityGraphPointsSelect]
                        })
                    ]
                })
            ]
        }).addStyleClass("sasHPRiskCommon-bottomFormPadding optionsPane_Form");

        return new sas.hc.ui.layout.VerticalLayout({
            content: [ plotGraphOptionsForm ]
        });
    },

    _buildPlotAdvancedQueryOptionsSection: function() {
        var self = this;

        var KERNEL_SELECT_DEFAULT_OPTION = {
            key: 0, text: this._getResourceBundle().getText("OptionsPane.advancedQueryOptions.kernelDensity.normal.txt")
        };
        var KERNEL_SELECT_OPTIONS = [
            KERNEL_SELECT_DEFAULT_OPTION,
            {key: 1, text: this._getResourceBundle().getText("OptionsPane.advancedQueryOptions.kernelDensity.triangular.txt")},
            {key: 2, text: this._getResourceBundle().getText("OptionsPane.advancedQueryOptions.kernelDensity.quadratic.txt")}
        ];
        self.kernelDensitySelect = new sas.hc.m.Select(this.createId("_plotKernelDensitySelect"), {
            width: "200px",
            enabled: true, // S1415978: Should always be enabled for linked plots and de-linked plots
            selectedKey: {
                path: "/options/kernel/value",
                formatter: function(value) {
                    // Determine if value exists in available options, if not return default value
                    var objectMatch = sas.analytics.riskhp.utils.HprUtil.findObject(KERNEL_SELECT_OPTIONS, value, "key");
                    if (objectMatch) return value;
                    return KERNEL_SELECT_DEFAULT_OPTION.key; // Return default value
                }
            },
            change: $.proxy(self._handleKernelChange, self)
        });
        sas.analytics.riskhp.utils.HprUtil.addListItems(self.kernelDensitySelect, KERNEL_SELECT_OPTIONS, "text", "key");

        self.advancedQueryOptionsRBG = new sas.hc.m.RadioButtonGroup(this.createId("_plotAdvancedQueryOptionsRBG"), {
            enabled: true,
            selectedIndex: {
                path: "/options/pctldef/value",
                formatter: function(value) {
                    // Valid values are "0 - 4" for RadioButtonGroup; value from JSON is "1 - 5", so we have to adjust
                    var adjustedValue = value && (value >=1 && value <= 5) ? value - 1 : 0; // Use 0 as default if value not in valid range
                    return adjustedValue;
                }
            },
            buttons: [
                new sas.hc.m.RadioButton({
                    text: this._getResourceBundle().getText("OptionsPane.advancedQueryOptions.weightedAverageLeftRBLabel.txt")
                }),
                new sas.hc.m.RadioButton({
                    text: this._getResourceBundle().getText("OptionsPane.advancedQueryOptions.observationLeftRBLabel.txt")
                }),
                new sas.hc.m.RadioButton({
                    text: this._getResourceBundle().getText("OptionsPane.advancedQueryOptions.empiricalDistributionRBLabel.txt")
                }),
                new sas.hc.m.RadioButton({
                    text: this._getResourceBundle().getText("OptionsPane.advancedQueryOptions.weightedAverageRightRBLabel.txt")
                }),
                new sas.hc.m.RadioButton({
                    text: this._getResourceBundle().getText("OptionsPane.advancedQueryOptions.empiricalDistributionAverageRBLabel.txt")
                })
            ],
            select: $.proxy(self._handlePercentileDefinitionChange, self)
        });

        var plotAdvancedQueryOptionsForm  = new sas.hc.ui.layout.form.Form({
            layout: new sas.hc.ui.layout.form.ResponsiveGridLayout(this.RESPONSIVE_GRID_LAYOUT_SETTINGS),
            formContainers: [
                new sas.hc.ui.layout.form.FormContainer({
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement(this.createId("_plotAdvancedQueryOptionsKernelSelectFormElement"), {
                            label: new sas.hc.m.Label({
                                text: this._getResourceBundle().getText("OptionsPane.advancedQueryOptions.kernelDensityLabel.txt")
                            }),
                            fields: [self.kernelDensitySelect]
                        }),
                        new sas.hc.ui.layout.form.FormElement(this.createId("_plotAdvancedQueryOptionsRBGFormElement"), {
                            label: new sas.hc.m.Label({
                                text: this._getResourceBundle().getText("OptionsPane.advancedQueryOptions.rbgLabel.txt")
                            }),
                            fields: [self.advancedQueryOptionsRBG]
                        })
                    ]
                })
            ]
        }).addStyleClass("sasHPRiskCommon-noFormPadding optionsPane_Form");

        return new sas.hc.ui.layout.VerticalLayout({
            content: [ plotAdvancedQueryOptionsForm ]
        });
    },


    //
    // S1410237: handlers for form field changes that allow us to update the model as
    // changes are made, in order to support preserving settings when switching between
    // panels.
    //

    _handleAlphaChange: function(evt) {
        this._onNumericStepperLiveChange(evt, HprConstants.HPR_OPTIONS_PANE_ALPHA_NAME);
    },

    _handleKernelChange: function(evt) {
        this._enableRequeryChangeOrSelect(evt);
        this._updateModelForKernelChange(this.borderLayout.getModel());
    },

    _handleDensityChange: function(evt) {
        this._enableRequeryChangeOrSelect(evt);
        this._updateModelForDensityChange(this.borderLayout.getModel());
    },

    _handleDisplayHorizonsChange: function(evt) {
        this._enableRequeryChangeOrSelect(evt);
        this._updateModelForDisplayHorizonsChange(this.borderLayout.getModel());
    },

    _handleShowLabelsChange: function(evt) {
        this._enableApplyButton(evt);
        this._updateModelForShowLabelsChange(this.borderLayout.getModel());
    },

    _handlePercentileDefinitionChange: function(evt) {
        this._enableRequeryChangeOrSelect(evt);
        this._updateModelForPercentileDefinitionChange(this.borderLayout.getModel());
    },

    _onNameFieldLiveChange: function(evt) {
        var enabled = false;
        var source = evt.getSource();

        // Perform validation on Name field as the user types
        source._performValidation();

        if (source.getValueState() !== sap.ui.core.ValueState.Error) {
            enabled = true;
        }

        this._enableApplyButton(enabled);
    },

    _onNumericStepperLiveChange: function(evt, control) {
        var self = this;
        var source = evt.getSource();
        var newValue = source.getValue();
        var delay = (newValue ? 300 : 0); // no delay if value is empty

        // Execute search after user stopped typing for 300ms
        clearTimeout(self.liveChangeTimer);
        if (delay) {
            self.liveChangeTimer = setTimeout(function () {
                source._performValidation();
                if (source.inputField.getValueState() !== sap.ui.core.ValueState.Error) {
                    self._enableRequeryChangeOrSelect();

                    // If the Stepper is in a valid state, update the model according to which Stepper is firing event
                    if (control === HprConstants.HPR_OPTIONS_PANE_ALPHA_NAME) {
                        self._updateModelForAlphaChange(self.borderLayout.getModel());
                    }
                } else {
                    self._enableApplyButton(false);
                }
            }, delay);
        } else {
            source._performValidation();
            if (source.inputField.getValueState() !== sap.ui.core.ValueState.Error) {
                self._enableRequeryChangeOrSelect();

                // If the Stepper is in a valid state, update the model according to which Stepper is firing event
                if (control === HprConstants.HPR_OPTIONS_PANE_ALPHA_NAME) {
                    self._updateModelForAlphaChange(self.borderLayout.getModel());
                }
            } else {
                self._enableApplyButton(false);
            }
        }
    },

    _onLinkedPlotChange: function(evt) {
//        var source = evt.getSource();
        this._enableApplyButton(true);
    },

    /**
     * An event handler for Change or Select events that sets a flag so the Plot will know to requery.  Only options that trigger a Plot requery should call
     * this method.
     * @param evt The Event object
     * @private
     */
    _enableRequeryChangeOrSelect: function(evt) {
        this.requery = true;
        this._enableApplyButton(true);
    },

    _enableApplyButton: function(enabled) {
        // One last check to make sure Name Input field or NumericStepper fields are not in Error state
        if (this.nameInput.getValueState() === sap.ui.core.ValueState.Error ||
            this.riskConfidenceLevelStepper.inputField.getValueState() === sap.ui.core.ValueState.Error) {
            enabled = false;
        }

        this.applyButton.setEnabled(enabled);
        this.cancelButton.setEnabled(enabled);
    },

    _onApplyButtonPressed: function(evt) {
        var self = this;

        // Disable Apply button; This affects borderLayout model so do it before we make a copy
        this._enableApplyButton(false);

        //TODO--do we need to clone this? It should already be a clone.
        // NOTE: 'this.borderLayout' corresponds to this fragment's model since this fragment returns 'this.borderLayout' in createContent()
        var model = $.extend(true, {}, this.borderLayout.getModel());

        // Retrieve the option values from the model since they were just updated, and update the corresponding Plot model to keep that in-sync as well
        var options = model.getProperty("/options");
        sas.analytics.riskhp.models.HPRiskModelManager.updatePlotOptions(options);

        // Commented out due to decision to not track absolute/relative switch on titles
        // at 3/29/2017 UI meeting. See HprChartModel.updateOptions function

        // BEGIN: Absolute/Relative responding code
        // var plot = sap.ui.getCore().byId(options.id);
        // var chartModel = plot.getChartModel();
        // var optionName = chartModel.getOptionName();
        //
        // // If the model has a different name than what is in the nameInput field, update it with the new name
        // // this will happen if the user moved from absolute to relative horizons or vice versa and was using the
        // // default title
        // if (optionName !== options.name) {
        //     self.nameInput.setValue(optionName);
        // }
        // END: Absolute/Relative responding code

        // If we should requery the Plot, locate the Plot using the current options id and rebuild the Plot;
        // The Plot will requery and pass along the options we just set via the new model
        var plot = sap.ui.getCore().byId(options.id);
        if (self.requery) {
            // Use the rebuild function because we need to execute the query with
            // the changed options and have the plot redraw with the new options
            plot.rebuild();

            self.requery = false;
        } else {
            plot.updateChartWithCurrentModel();
        }

        // Publish an event that will update the name of the Plot
        sas.eventBus.publish(sas.analytics.riskhp.utils.HprConstants.CHANNEL_OPTIONS_PANE, sas.analytics.riskhp.utils.HprConstants.EVENT_PLOT_OPTIONS_NAME_UPDATED, {
            plotId: options.id,
            newPlotName: options.name //Absolute/Relative responding code use 'optionName' instead
        });

        // Publish an event that will signal the 'Show labels' field was updated
        sas.eventBus.publish(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_PLOT_OPTIONS_SHOW_LABELS_UPDATED, {
            plotId: options.id,
            showLabels: self.showLabelsCheckBox.getSelected()
        });

        // Publish an event that will update the dirty state of the Exploration view
        sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {
            dirty: true
        });
    },

    _onCancelButtonPressed: function(evt) {
        // Disable Apply button; This affects borderLayout model so do it before we make a copy
        this._enableApplyButton(false);

        // Revert components back to previous state by getting previous Plot model information;
        // We can't rely on the borderLayout model because it is bound and already updated at this point
        var model = $.extend(true, {}, this.borderLayout.getModel()); // Create copy
        var optionsId = model.getProperty("/options/id"); // id of the appropriate Plot model and Plot
        var plotModel = sas.analytics.riskhp.models.HPRiskModelManager.getPlotModel(optionsId);

        // Set the name input field's value to the previous name
        this.nameInput.setValue(plotModel.getOptionName());

        // Revert all the other components
        this.showLabelsCheckBox.setSelected(plotModel.getOptionShowLabels().value);
        this.kernelDensitySelect.setSelectedKey(plotModel.getOptionKERNEL().value);
        this.densityGraphPointsSelect.setSelectedKey(plotModel.getOptionNDENSITY().value);
        this.displayHorizonsSelect.setSelectedKey(plotModel.getOptionDisplayHorizons().value);

        // Special processing is needed for 'alpha' because the UI component needs to display a percentage, but mid-tier uses decimal format (i.e. .05)
        var adjustedAlphaValue = this._adjustAlphaValueForUIDisplay(plotModel.getOptionALPHA().value);
        this.riskConfidenceLevelStepper.setValue(adjustedAlphaValue.toString());

        // pctldef requires special processing since UI expects 0-4 and mid-tier requires 1-5
        this.advancedQueryOptionsRBG.setSelectedIndex(plotModel.getOptionPCTLDEF().value - 1); // Take away one so UI component will show selected option correctly

        // Reset the requery flag
        this.requery = false;
    },

    _adjustAlphaValueForUIDisplay: function(alphaValue) {
        // Some of the locales pass this number as a String (i.e. pl); Need to massage the value so component and mid-tier can handle it
        if (_.isString(alphaValue)) {
            alphaValue = sas.ltjs.sasicu.util.SASFormatterUtil.parse(alphaValue, "BEST9.2");
        }

        var adjustedValue = Number((1 - alphaValue) * 100);
        adjustedValue = Math.round(adjustedValue * 10000) / 10000;
        return adjustedValue;
    },


    //
    // S1410237: model update functions to enable preservation of unapplied selections when switching between panels.
    //

    _updateModelForAlphaChange: function(model) {
        // Special processing is needed for 'alpha' because the UI component displays a percentage but the mid-tier uses decimal format (i.e. .05)
        var newAlpha = Number((100.0 - sas.ltjs.sasicu.util.SASFormatterUtil.parse(this.riskConfidenceLevelStepper.getValue(), "BEST9.2")) * .01);
        model.setProperty("/options/alpha/value", newAlpha);
    },

    _updateModelForKernelChange: function(model) {
        model.setProperty("/options/kernel/value", parseInt(this.kernelDensitySelect.getSelectedKey()));
    },

    _updateModelForDensityChange: function(model) {
        model.setProperty("/options/ndensity/value", parseInt(this.densityGraphPointsSelect.getSelectedKey()));
    },

    _updateModelForDisplayHorizonsChange: function(model) {
        model.setProperty("/options/displayHorizons/value", this.displayHorizonsSelect.getSelectedKey());
    },

    _updateModelForShowLabelsChange: function(model) {
        model.setProperty("/options/showLabels/value", this.showLabelsCheckBox.getSelected());
    },

    _updateModelForPercentileDefinitionChange: function(model) {
        // Special processing is needed for 'pctldef' because the UI component expects 0-4 but the mid-tier expects 1-5
        var adjustedAdvancedQueryOptionsRBGSelectedIndex = this.advancedQueryOptionsRBG.getSelectedIndex() + 1;
        model.setProperty("/options/pctldef/value", adjustedAdvancedQueryOptionsRBGSelectedIndex);
    }
});