// (c) 2018, SAS Institute Inc.

jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");

/**
 * Options pane fragment for the generic TableViewer control
 */
sap.ui.jsfragment("sas.analytics.riskhp.fragments.options.TableViewerOptions", {

    RESPONSIVE_GRID_LAYOUT_SETTINGS: {
        labelSpanL: 6,
        emptySpanL: 0,
        labelSpanM: 6,
        emptySpanM: 0,
        labelSpanS: 12,
        emptySpanS: 0
    },

    _getResourceBundle: function() {
        return sas.analytics.riskhp.utils.HprUtil.getBundle();
    },

    createContent: function (controller) {
        var self = this;

        this.generalWindowShadeSectionId = sas.analytics.riskhp.utils.HprConstants.HPR_OPTIONS_PANE_TABLE_VIEWER_GENERAL_SECTION_ID;

        this.windowShade = new sas.hc.ui.commons.windowshade.WindowShade("tableViewerOptions", {});
        this.windowShade.setOpenFirstSection( false );
        this.windowShade.setWidth('100%');
        this.windowShade.addStyleClass("optionsPane_windowShade");

        // Attach event listeners for open and close of the WindowShadeSections
        this.windowShade.attachSectionOpen(this._onSectionOpenOrClose, this);
        this.windowShade.attachSectionClose(this._onSectionOpenOrClose, this);

        this.generalOptionsSection = new sas.hc.ui.commons.windowshade.WindowShadeSection({
            id: this.generalWindowShadeSectionId,
            collapsed: "{/options/generalOptionsSectionCollapsed}"
        });
        this.generalOptionsSection.setTitle(this._getResourceBundle().getText("OptionsPane.generalOptionsWindowShadeSection.txt"));
        this.generalOptionsSection.setTooltip(this._getResourceBundle().getText("OptionsPane.generalOptionsWindowShadeSection.tooltip.txt"));
        this.generalOptionsSection.addContent( this._buildTableViewerGeneralOptionsSection() );
        this.windowShade.addSection( this.generalOptionsSection );

        this.borderLayout = new sas.hc.ui.commons.layout.BorderLayout({
            width: "100%",
            height: "100%",
            center: new sas.hc.ui.commons.layout.BorderLayoutArea({
                overflowX: "hidden",
                overflowY: "hidden",
                content: [this.windowShade]
            }).addStyleClass("optionsPane_centerLayoutArea"),
            bottom: new sas.hc.ui.commons.layout.BorderLayoutArea({
                overflowX: "hidden",
                overflowY: "hidden",
                size: "5.0rem",
                content: [this._buildFooter()]
            })
        });

        // Workaround to set WindowShade.openedSectionsId property so it matches which sections we have open in the model
        var updateWindowShadeOpenedSectionsIdProperty = function() {
            // Gather the model and appropriate properties
            var model = self.borderLayout.getModel();
            var generalOptionsSectionCollapsed = model.getProperty("/options/generalOptionsSectionCollapsed");

            // Create a new 'openedSectionsId' array with any of our model properties that are opened (!collapsed means the section is open);
            // When finished, update the WindowShade with the new 'openedSectionsId' value
            var openedSectionsId = [];
            if (!generalOptionsSectionCollapsed) {openedSectionsId.push(self.generalWindowShadeSectionId);}
            self.windowShade.setOpenedSectionsId(openedSectionsId);
        };
        this.borderLayout.updateWindowShadeOpenedSectionsIdProperty = updateWindowShadeOpenedSectionsIdProperty;

        return this.borderLayout;
    },

    _onSectionOpenOrClose: function(evt) {
        // When a WindowShadeSection is opened | closed, we need to keep the TableViewer model properties in-sync with the model changes
        var model = $.extend(true, {}, this.borderLayout.getModel()); // Create copy

        // Update the TableViewer model with the required properties to keep it in-sync; These are needed to be able to
        // save which WindowShadeSections were last opened | closed and restore them.
        sas.analytics.riskhp.models.HPRiskModelManager.updateTableViewerOptionSectionProperties(model.getProperty("/options/id"),
            model.getProperty("/options/generalOptionsSectionCollapsed"));

        // Publish an event that will update the dirtyState of the Exploration view
        sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {
            dirty: true
        });
    },

    _buildFooter: function() {
        var self = this;

        self.applyButton = new sas.hc.m.Button({
            id: this.createId('_tableViewerOptions_applyButton'),
            text: this._getResourceBundle().getText("OptionsPane.applyButton.txt"),
            enabled: "{/options/applyEnabled}",
            press: $.proxy(self._onApplyButtonPressed, self)
        }).addStyleClass("sasPrimaryButton");

        self.cancelButton = new sas.hc.m.Button({
            id: this.createId('_tableViewerOptions_cancelButton'),
            text: this._getResourceBundle().getText("hprisk.cancelButton.txt"),
            enabled: "{/options/applyEnabled}", // Defect S1387426: Should behave like Apply Button state
            press: $.proxy(self._onCancelButtonPressed, self)
        });

        var tb = new sas.hc.m.ResponsiveToolbar({
            content: [new sas.hc.m.ToolbarSpacer(), self.applyButton, self.cancelButton]
        });
        tb.addStyleClass("noBackground");

        return tb;
    },

    _buildTableViewerGeneralOptionsSection: function() {
        var self = this;

        self.nameInput = new sas.hc.m.Input(this.createId("_tableViewerGeneralOptionsNameInput"), {
            label: this._getResourceBundle().getText("OptionsPane.generalOptions.name.inputLabel.txt"),
            labelPosition: sas.hc.m.InputLabelPosition.Top,
            liveChange: $.proxy(self._onNameFieldLiveChange, self),
            value: "{/options/name}",
            width: "200px",
            required: true
        });

        self.showLabelsCheckBox = new sas.hc.m.CheckBox(this.createId('_tableViewerGeneralOptionsShowLabelsCB'), {
            text: this._getResourceBundle().getText("OptionsPane.generalOptions.showLabelsLabel.txt"),
            enabled: true,
            selected: "{/options/showLabels/value}",
            select: $.proxy(self._genericChangeOrSelect, self)
        }).addStyleClass("optionsPane_CB_MarginBottom");

        var tableViewerGeneralOptionsForm  = new sas.hc.ui.layout.form.Form({
            layout: new sas.hc.ui.layout.form.ResponsiveGridLayout(this.RESPONSIVE_GRID_LAYOUT_SETTINGS),
            formContainers: [
                new sas.hc.ui.layout.form.FormContainer({
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement(this.createId("_tableViewerGeneralOptionsNameInputFormElement"), {
                            fields: [self.nameInput]
                        }),
                        new sas.hc.ui.layout.form.FormElement(this.createId("_tableViewerGeneralOptionsShowLabelsFormElement"), {
                            fields: [self.showLabelsCheckBox]
                        })
                    ]
                })
            ]
        }).addStyleClass("sasHPRiskCommon-bottomFormPadding");

        return new sas.hc.ui.layout.VerticalLayout({
            content: [ tableViewerGeneralOptionsForm ]
        });
    },

    _onNameFieldLiveChange: function(evt) {
        var enabled = false;
        var source = evt.getSource();

        // Perform validation on Name field as the user types
        source._performValidation();

        if (source.getValueState() !== sap.ui.core.ValueState.Error) {
            enabled = true;
        }

        this._enableApplyButton(enabled);
    },

    _genericChangeOrSelect: function(evt) {
        this._enableApplyButton(true);
    },

    _enableApplyButton: function(enabled) {
        // One last check to make sure Name Input field is not in Error state
        if (this.nameInput.getValueState() === sap.ui.core.ValueState.Error) {
            enabled = false;
        }

        this.applyButton.setEnabled(enabled);
        this.cancelButton.setEnabled(enabled);
    },

    _onApplyButtonPressed: function(evt) {
        var self = this;

        // Disable Apply button; This affects borderLayout model so do it before we make a copy
        this._enableApplyButton(false);

        //TODO--do we need to clone this? It should already be a clone.
        // NOTE: 'this.borderLayout' corresponds to this fragment's model since this fragment returns 'this.borderLayout' in createContent()
        var model = $.extend(true, {}, this.borderLayout.getModel()); // Create copy

        // Retrieve the option values from the model since they were just updated, and update the corresponding TableViewer model to keep that in-sync as well
        var options = model.getProperty("/options");
        sas.analytics.riskhp.models.HPRiskModelManager.updateTableViewerOptions(options);

        // For now, just publish an event that will update the name of the TableViewer
        sas.eventBus.publish(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_TABLE_VIEWER_OPTIONS_NAME_UPDATED, {
            tableViewerId: options.id,
            newTableViewerName: self.nameInput.getValue()
        });

        // Publish an event that will signal the 'Show labels' field was updated
        sas.eventBus.publish(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_TABLE_VIEWER_OPTIONS_SHOW_LABELS_UPDATED, {
            tableViewerId: options.id,
            showLabels: self.showLabelsCheckBox.getSelected()
        });

        // Publish an event that will update the dirty state of the Exploration view
        sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {
            dirty: true
        });
    },

    _onCancelButtonPressed: function(evt) {
        // Disable Apply button; This affects borderLayout model so do it before we make a copy
        this._enableApplyButton(false);

        // Revert components back to previous state by getting previous TableViewer model context information;
        // We can't rely on the borderLayout model because it is bound and already updated at this point
        var model = $.extend(true, {}, this.borderLayout.getModel()); // Create copy
        var optionsId = model.getProperty("/options/id"); // id of the appropriate TableViewer model
        var tableViewerModel = sas.analytics.riskhp.models.HPRiskModelManager.getTableViewerModel(optionsId);

        // Set the name input field's value to the previous name
        this.nameInput.setValue(tableViewerModel.getOptionName());

        // Revert all the other components
        this.showLabelsCheckBox.setSelected(tableViewerModel.getOptionShowLabels().value);
    }
});