// (c) 2018, SAS Institute Inc.
jQuery.sap.require("sas.ltjs.commons.rules.expressions.BinaryRelationalOperator");
// jQuery.sap.require("sas.analytics.riskhp.models.ColumnInfo");
jQuery.sap.require("sas.hc.m.colorpicker.ColorPicker");
jQuery.sap.require("sas.analytics.riskhp.views.displayrules.DisplayRulesUtil");

jQuery.sap.declare("sas.analytics.riskhp.models.DisplayRule");

/**
 * @typedef {Object} DisplayRule
 * @property {string} description
 * @property {string} subType
 * @property {string} name
 * @property {string} label
 * @property {string} tooltip
 * @property {string} icon
 * @property {string} format
 * @property {string} dataType
 * @property {string} id
 * @property {(string)} horizonDate - Will have this if it is one of our horizon types
 * @property {function} getId
 * @property {function} getType
 * @property {function} getLabel
 * @property {function} getName
 */

/**
 * @namespace
 * @name sas.analytics.riskhp.models.DisplayRule
 * @extends sap.ui.base.Object
 */
sap.ui.base.Object.extend("sas.analytics.riskhp.models.DisplayRule", {
    constructor : function() {
        this.id = this.createUniqueId();

        this.description = "";
        this.referenceColumn = [];
        this.operator = sas.analytics.riskhp.models.DisplayRule.operators[7]; // >=
        this.dataItem1 = {
            type: sas.analytics.riskhp.models.DisplayRule.dataItemTypes[0],
            value: 0,
            // column: []
        };
        // this.dataItem2 = {
        //     type: sas.analytics.riskhp.models.DisplayRule.dataItemTypes[0],
        //     value: 0,
        //     column: []
        // };
        this.style = {
            // fontSize: 10,
            bold: false,
            italic: false,
            underline: false,
            color: "#000000", // black text color
            backgroundColor: sas.hc.m.colorpicker.ColorPicker.NO_COLOR // transparent background color
        };
        this.appliesTo = {
            type: sas.analytics.riskhp.models.DisplayRule.appliesToTypes[0], // reference column
            // column: []
        };
    }
});

sas.analytics.riskhp.models.DisplayRule.prototype.getType = function() {
    return this.type;
};

sas.analytics.riskhp.models.DisplayRule.prototype._getNextCopyName = function(description) {
    var copyText = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.DisplayRules.Copy.txt");
    var existingRules = DisplayRulesManager.getDisplayRules();
    var trial = "";
    var copyNum = 0;
    var foundNextCopyDesc = false;
    while  (!foundNextCopyDesc) {
        trial = description + " " + copyText + (copyNum === 0 ? "" : " " + copyNum);
        if (_.find(existingRules, function(rule) {
            return rule.description === trial;
        })) {
            copyNum = copyNum + 1;
        } else {
            foundNextCopyDesc = true;
        }
    }
    return trial;
};

sas.analytics.riskhp.models.DisplayRule.prototype.copy = function() {
    var self = this;
    // var clonedDisplayRule = this.fromJSON(this);
    var clonedDisplayRule =  sas.analytics.riskhp.utils.HprUtil.hprClone(this);
    clonedDisplayRule.id = this.createUniqueId();

    clonedDisplayRule.description = self._getNextCopyName(clonedDisplayRule.description);

    return clonedDisplayRule;
};
sas.analytics.riskhp.models.DisplayRule.prototype.clone = function(displayRule) {
    // this needs to return a display rule object
    return sas.analytics.riskhp.models.DisplayRule.prototype.fromJSON(displayRule);
    // return sas.analytics.riskhp.utils.HprUtil.hprClone(displayRule);
};

sas.analytics.riskhp.models.DisplayRule.prototype.fromJSON = function(json) {
    var displayRule = new sas.analytics.riskhp.models.DisplayRule();
    displayRule.id = json.id;

    displayRule.description = json.description;
    displayRule.referenceColumn = sas.analytics.riskhp.utils.HprUtil.hprClone(json.referenceColumn);
    displayRule.operator = json.operator;

    if (json.dataItem1) {
        displayRule.dataItem1 = sas.analytics.riskhp.utils.HprUtil.hprClone(json.dataItem1);
    }
    if (json.dataItem2) {
        displayRule.dataItem2 = sas.analytics.riskhp.utils.HprUtil.hprClone(json.dataItem2);
    }
    // // the binding in the editor needs the entire item to be defined
    // displayRule.dataItem1 = {
    //     type: sas.analytics.riskhp.models.DisplayRule.dataItemTypes[0],
    //     value: 0,
    //     column: []
    // };
    // if (json.dataItem1) {
    //     displayRule.dataItem1.type = json.dataItem1.type;
    //     if (json.dataItem1.type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE) {
    //         displayRule.dataItem1.value = json.dataItem1.value;
    //     } else {
    //         displayRule.dataItem1.column = sas.analytics.riskhp.utils.HprUtil.hprClone(json.dataItem1.column);
    //     }
    // }
    //
    // // the binding in the editor needs the entire item to be defined
    // displayRule.dataItem2 = {
    //     type: sas.analytics.riskhp.models.DisplayRule.dataItemTypes[0],
    //     value: 0,
    //     column: []
    // };
    // if (json.dataItem2) {
    //     displayRule.dataItem2.type = json.dataItem2.type;
    //     if (json.dataItem2.type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE) {
    //         displayRule.dataItem2.value = json.dataItem2.value;
    //     } else {
    //         displayRule.dataItem2.column = sas.analytics.riskhp.utils.HprUtil.hprClone(json.dataItem2.column);
    //     }
    // }

    displayRule.style = sas.analytics.riskhp.utils.HprUtil.hprClone(json.style);
    displayRule.appliesTo = sas.analytics.riskhp.utils.HprUtil.hprClone(json.appliesTo);

    return displayRule;
};

sas.analytics.riskhp.models.DisplayRule.prototype.dataItemToJSON = function(dataItem) {
    var json = {
        type: dataItem.type
    };
    if (dataItem.type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE) {
        json.value = dataItem.value;
    } else {
        json.column = dataItem.column;
    }
    return json;
};

sas.analytics.riskhp.models.DisplayRule.prototype.appliesToItemToJSON = function(appliesToItem) {
    var json = {
        type: appliesToItem.type
    };
    if (appliesToItem.type === sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_COL) {
        json.column = appliesToItem.column;
    }
    return json;
};

sas.analytics.riskhp.models.DisplayRule.prototype.toJSON = function() {
    var json = {
        id: this.id,
        description: this.description,
        referenceColumn: this.referenceColumn,
        operator: this.operator,
        style: this.style,
        appliesTo: this.appliesToItemToJSON(this.appliesTo)
    };
    if (sas.analytics.riskhp.views.displayrules.DisplayRulesUtil.requiresDataItem1(this.operator)) {
        json.dataItem1 = this.dataItemToJSON(this.dataItem1);
    }
    if (sas.analytics.riskhp.views.displayrules.DisplayRulesUtil.requiresDataItem2(this.operator)) {
        json.dataItem2 = this.dataItemToJSON(this.dataItem2);
    }
    return json;
};

sas.analytics.riskhp.models.DisplayRule.prototype.createUniqueId = function() {
    return "DR" + Date.now();
};

/**
 * Returns an array of string representations of property names for this object which will be used when comparing
 * two Data Source objects.  Only the properties listed here will be compared.
 * @returns {string[]}
 */
sas.analytics.riskhp.models.DisplayRule.prototype.getValidPropertiesToCompare = function() {
    return ["id"];
};

/* Statics */
sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_COL = "dataItemColumn";
sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE = "dataItemValue";

sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_REF_COL = "appliesToReferenceColumn";
sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_ROW = "appliesToRow";
sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_COL = "appliesToColumn";

sas.analytics.riskhp.models.DisplayRule.OPERATOR_MISSING = "missing";
sas.analytics.riskhp.models.DisplayRule.OPERATOR_NOT_MISSING = "notMissing";
sas.analytics.riskhp.models.DisplayRule.OPERATOR_EQ = sas.ltjs.commons.rules.expressions.BinaryRelationalOperator.EQ;
sas.analytics.riskhp.models.DisplayRule.OPERATOR_NE = sas.ltjs.commons.rules.expressions.BinaryRelationalOperator.NE;
sas.analytics.riskhp.models.DisplayRule.OPERATOR_LT = sas.ltjs.commons.rules.expressions.BinaryRelationalOperator.LT;
sas.analytics.riskhp.models.DisplayRule.OPERATOR_LE = sas.ltjs.commons.rules.expressions.BinaryRelationalOperator.LE;
sas.analytics.riskhp.models.DisplayRule.OPERATOR_GT = sas.ltjs.commons.rules.expressions.BinaryRelationalOperator.GT;
sas.analytics.riskhp.models.DisplayRule.OPERATOR_GE = sas.ltjs.commons.rules.expressions.BinaryRelationalOperator.GE;
sas.analytics.riskhp.models.DisplayRule.OPERATOR_BETWEEN = "between";
sas.analytics.riskhp.models.DisplayRule.OPERATOR_NOT_BETWEEN = "notBetween";

sas.analytics.riskhp.models.DisplayRule.operators = [
    sas.analytics.riskhp.models.DisplayRule.OPERATOR_MISSING,
    sas.analytics.riskhp.models.DisplayRule.OPERATOR_NOT_MISSING,
    sas.analytics.riskhp.models.DisplayRule.OPERATOR_EQ,
    sas.analytics.riskhp.models.DisplayRule.OPERATOR_NE,
    sas.analytics.riskhp.models.DisplayRule.OPERATOR_LT,
    sas.analytics.riskhp.models.DisplayRule.OPERATOR_LE,
    sas.analytics.riskhp.models.DisplayRule.OPERATOR_GT,
    sas.analytics.riskhp.models.DisplayRule.OPERATOR_GE,
    sas.analytics.riskhp.models.DisplayRule.OPERATOR_BETWEEN,
    sas.analytics.riskhp.models.DisplayRule.OPERATOR_NOT_BETWEEN
];

sas.analytics.riskhp.models.DisplayRule.dataItemTypes = [
    sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE,
    sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_COL
];

sas.analytics.riskhp.models.DisplayRule.appliesToTypes = [
    sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_REF_COL,
//    sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_ROW
    sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_COL // S1457454
];