jQuery.sap.declare("sas.analytics.riskhp.models.HPRiskModelManager");

jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
jQuery.sap.require("sas.analytics.riskhp.models.DataSourceParams");
jQuery.sap.require('sas.analytics.riskhp.models.DisplayRulesManager');


/**
 * @name sas.analytics.riskhp.models.HPRiskModelManager
 */
sas.analytics.riskhp.models.HPRiskModelManager = (function () {
    "use strict";
    var currentExploration = {};
    var pageObjects = {}; // Hashmap of Page objects
    var draggableObjects = [];

    function _updateExploration(newExploration) {
        // TODO: Merge object with currently held object or replace based on current state
        currentExploration = newExploration;
        return currentExploration;
    }

    return {

        // OBJECT: "object",  // Left here as an example of how to have a public var, once we have one we can get rid of this.
        // var model = new sas.stress.wkbench.model.RiskJSONModel(json);  // TODO: Switch to RiskJSONModel

        //*****************************************************************************************************
        //*****************************************************************************************************
        // General model functions
        //*****************************************************************************************************
        //*****************************************************************************************************
        /**
         * Resets the model when we are switching out the exploration view for a new exploration view
         */
        reset: function() {
            draggableObjects = [];
            pageObjects = {};
            sas.analytics.riskhp.utils.HprUtil.resetUniquePageCount();
        },

        //*****************************************************************************************************
        //*****************************************************************************************************
        // Exploration functions
        //*****************************************************************************************************
        //*****************************************************************************************************
        /**
         * Creates an object that represents a new Exploration to be used in the UI.
         *
         * @param newExplorationName
         * @returns {{exploration: {name: String, analysis: Object, isDirty: boolean}}}
         */
        createNewExplorationObject: function(newExplorationName) {
            // Create some default 'analysis' properties for a new Exploration so the UI comes up in the desired appearance
            var newExplorationDefaultAnalysisObject = {};
            newExplorationDefaultAnalysisObject.layout = {
                leftPropertyPanel: {
                    visible: true,
                    selectedPropertyPaneIndex: 0
                },
                rightPropertyPanel: {
                    visible: true,
                    selectedPropertyPaneIndex: 0
                },
                leftSplitterPanel: {
                    size: sas.analytics.riskhp.utils.HprConstants.HPR_EXPLORATION_VIEW_LEFT_PANEL_DEFAULT_SIZE
                },
                rightSplitterPanel: {
                    size: sas.analytics.riskhp.utils.HprConstants.HPR_EXPLORATION_VIEW_RIGHT_PANEL_DEFAULT_SIZE
                },
                centerSplitterPanel: {
                    size: sas.analytics.riskhp.utils.HprConstants.HPR_EXPLORATION_VIEW_MAIN_PANEL_DEFAULT_SIZE
                }
            };

            return {
                "exploration": {
                    "name": newExplorationName,
                    "analysis": newExplorationDefaultAnalysisObject,
                    "isDirty": true
                }
            };
        },

        /**
         * Creates an object that represents a saved Exploration which will be used in the UI.
         *
         * @param explorationData The string of Exploration data to be parsed
         * @param explorationId The id of the Exploration
         * @returns {{exploration: {name: String, id: String, analysis: Object, isDirty: boolean}}}
         */
        createExplorationObject: function (explorationData, explorationId) {
            // TODO: Create new sas.analytics.riskhp.models.Exploration model object?
            // Loop through the saved DataSourceParams objects and re-create them
            var analysis = $.parseJSON(explorationData.analysis); // String of JSON for Exploration
            _.each(analysis.dataSourceParams, function(dataSourceParam){
                var paramObject = sas.analytics.riskhp.models.DataSourceParams.prototype.fromJSON(dataSourceParam);
                DataSourceCacheManager.addDataSourceParam(paramObject);
            });

            // Recall the 'selectedDataSourceId' property from the saved analysis JSON and set it for the Cache Manager
            DataSourceCacheManager.setSelectedDataSourceId(analysis.selectedDataSourceId);

            ComputedVariablesManager.setComputedVariables(analysis.computedVariables);
            DisplayRulesManager.fromJSON(analysis.displayRules);

            // If a summary block is found in the analysis, then show the user.
            if (analysis.summary) {
                var dialogModel = new sap.ui.model.json.JSONModel();
                dialogModel.setProperty("/explorationSummaryData", analysis.summary);

                // Open the ExplorationSummaryDialog
                var explorationSummaryDialog = sap.ui.jsfragment("sas.analytics.riskhp.fragments.ExplorationSummaryDialog", this);
                explorationSummaryDialog.setModel(dialogModel);
                explorationSummaryDialog.open();
            };

            return {
                "exploration" : {
                    "name": explorationData.name,
                    "id": explorationId,
                    "analysis": analysis,
                    "isDirty": false
                }
            };
        },

        changeExplorationAnalysisObject: function(changedExplorationAnalysis) {
            var explorationModelData = sas.analytics.riskhp.models.HPRiskModelManager.getExplorationModel().oData;

            var newExplorationModel = {
                "exploration" : {
                    "name": explorationModelData.exploration.name,
                    "id": explorationModelData.exploration.id,
                    "analysis": changedExplorationAnalysis,  // This is really the analysis
                    "isDirty": false // Should this automatically be true?
                }
            };

            sas.analytics.riskhp.models.HPRiskModelManager.setExplorationModel(newExplorationModel);

        },

        /**
         * Set the current Exploration to 'exploration', create a new JSONModel using the 'exploration', and then
         * set the Exploration model used by the UI to the new JSONModel
         *
         * @param exploration Object The Exploration object to track as the current Exploration and used to create
         * a new Exploration model
         */
        setExplorationModel: function(exploration) {
            var explorationModel = new sap.ui.model.json.JSONModel(_updateExploration(exploration));
            sap.ui.getCore().setModel(explorationModel, sas.analytics.riskhp.utils.HprConstants.MODEL_ID_EXPLORATION);
            //TODO: Should we fire a bus event here
        },

        /**
         * Creates a copy of the Exploration model and returns the Exploration object.
         *
         * @returns {*} Object Returns the Exploration object
         */
        getExplorationObject: function() {
            var data = sap.ui.getCore().getModel(sas.analytics.riskhp.utils.HprConstants.MODEL_ID_EXPLORATION).oData;
            return jQuery.extend(true, {}, data.exploration);
        },

        /**
         * Returns the Exploration model
         *
         * @returns {sap.ui.core.Model|sap.ui.model.Model|*|Object|sas.ltjs.commons.models.BaseModel|sap.ui.model.control.ControlModel}
         */
        getExplorationModel: function() {
            return sap.ui.getCore().getModel(sas.analytics.riskhp.utils.HprConstants.MODEL_ID_EXPLORATION);
        },

        /**
         * Generate the string which represents the view of the Exploration so it can be saved and rebuilt
         *
         * @param explorationView {sap.ui.jsview} The ExplorationView which will be used to gather layout information so it can be saved
         */
        generateExplorationAnalysisJSON: function(explorationView) {
            var self = this;
            var explorationObject = this.getExplorationObject();
            var selectedDataSourceId = DataSourceCacheManager.getSelectedDataSourceId();

            // Convert the DataSourceParams to JSON and add them to an array for inclusion in the final JSON
            var dataSourceParamsArray = [];
            _.each(DataSourceCacheManager.getDataSourceParams(), function(dataSourceParam) {
                // For the selected DataSource, delete its copy of the DataSource TreeModel since it will be
                // added to the final JSON in the DataPane section below.  This prevents two copies of the same
                // TreeTable model in the final JSON
                if (dataSourceParam.realDataSourceId === selectedDataSourceId) {
                    dataSourceParam.deleteDataSourceTreeTableModelObject(selectedDataSourceId);
                }
                
                dataSourceParamsArray.push(dataSourceParam.toJSON());
            });

            // // TODO: From Applegate... Will talk about this more later, but the UI will generate 'tag' value,
            // // TODO: it is a place holder for use in the query.  So instead of each query having a full repeat of the datasource it will just be "ds1"
            // dataSourceObject.tag = "Tag_Placeholder_1";

            // Pages/Tabs
            var pagesArray = []; // An array to store all the Pages for the Exploration
            var mainTabbar = explorationView.getMainTabbar();

            // Loop through all the Pages/Tabs on the IconTabbar
            $.each(mainTabbar.getItems(), function(index, page) {
                // 'page' is an IconTabFilter object with 'text', 'key', 'closable', 'design', 'iconColor', etc. properties;
                // Since 'page' is of type IconTabFilter, it also contains the RiskPanelsView component as its content;
                // Create a copy of the corresponding Page Object for the current 'page' whose properties should all be up-to-date
                var pageObjectToSave = $.extend(true, {}, self.getPageObject(page.getProperty('key'))); // Create copy

                // Gather the first content object (should be 'sas.risk.common.panelsview.RiskPanelsView') of the current Page (IconTabFilter)
                var riskPanelsView = page.getContent()[0];
                if (riskPanelsView) {
                    // Gather the layout data from the RiskPanelsView so we can re-create the content for the Page when the Exploration is opened again
                    pageObjectToSave.content = riskPanelsView.getDCLayoutData();
                }

                pagesArray.push(pageObjectToSave);
            });

            // Get the DataPane and the information we need to save an Exploration from it
            var dataPaneJSON = DataPaneModelManager.toJSON();

            var displayRulesJSON = DisplayRulesManager.toJSON();

            // Create an object that will hold all the analysis information
            return {
                name: explorationObject.name,
                version: sas.analytics.riskhp.utils.HprConstants.EXPLORATION_VERSION,
                dataSourceParams: dataSourceParamsArray,
                selectedDataSourceId: selectedDataSourceId,
                computedVariables: ComputedVariablesManager.getComputedVariables(),
                displayRules: displayRulesJSON,
                layout: {
                    leftPropertyPanel: explorationView.getLeftPropertyPaneJSON(),
                    leftSplitterPanel: explorationView.getLeftSplitterPanelJSON(),
                    centerSplitterPanel: explorationView.getCenterSplitterPanelJSON(),
                    rightPropertyPanel: explorationView.getRightPropertyPaneJSON(),
                    rightSplitterPanel: explorationView.getRightSplitterPanelJSON(),
                    pages: pagesArray,
                    selectedPage: mainTabbar.getSelectedKey(),
                    dataPane: dataPaneJSON
                }
            };
        },

        //*****************************************************************************************************
        //*****************************************************************************************************
        // Page object functions
        //*****************************************************************************************************
        //*****************************************************************************************************
        /**
         * Create an object that encapsulates information about a new Page.  The 'pageObject' serves two purposes:
         * The first is to represent a new Tab/Page in the Exploration view.  And the second is the model for the PageOptions pane.
         *
         * @param existingPages [] Array of existing Pages so we can determine the next best default Page name
         * @returns {{name: string, id: string, theme: string, applyEnabled: boolean, generalOptionsSectionCollapsed: boolean, themeOptionsSectionCollapsed: boolean}} Object A pageObject
         */
        createUniquePageObject: function(existingPages) {
            // Timestamp shim in case browser is non-modern
            if (!Date.now) {
                Date.now = function() { return new Date().getTime(); };
            }

            // Gather the next available unique page count and build a default page name
            var uniquePageCount = sas.analytics.riskhp.utils.HprUtil.getUniquePageCount();
            var defaultNewPageName = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("IconTabBar.defaultPageName.fmt", uniquePageCount);
            // If there are existing Pages/Tabs already present, we want to search through the names of those so we won't have a duplicate Page/Tab;
            // If there are not present, then just use the first default name we built above
            if(existingPages) {
                // Store the names of the existing Pages/Tabs in an Array for easier searching
                var pageObjectsArray = [];
                $.each(existingPages, function(index, page) {
                    pageObjectsArray.push({"name": page.getText()});
                });
                // See if there is a matching Page already
                var matchingPage = sas.analytics.riskhp.utils.HprUtil.findObject(pageObjectsArray, defaultNewPageName, "name", true);
                // While there is a matching Page, keep re-building the default name while increasing the unique page count; Loop will stop when we
                // get to a name we can use
                while (matchingPage) {
                    uniquePageCount = sas.analytics.riskhp.utils.HprUtil.getUniquePageCount();
                    defaultNewPageName = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("IconTabBar.defaultPageName.fmt", uniquePageCount);
                    matchingPage = sas.analytics.riskhp.utils.HprUtil.findObject(pageObjectsArray, defaultNewPageName, "name", true);
                }
            }

            return {
                "name": defaultNewPageName,
                "id": "mainRiskPanelsViewId_" + Date.now(),
                "theme": sas.analytics.riskhp.utils.HprUtil.getBundle().getText("OptionsPane.themeOptions.theme.umstead.txt"), // Default for all Page objects for now
                "applyEnabled": false, // Flag for the Apply button on each Options pane; Disabled by default
                "generalOptionsSectionCollapsed": false, // Open by default
                "themeOptionsSectionCollapsed": true // Closed by default
            };
        },

        /**
         * Add a 'pageObject' to the hashmap.
         *
         * @param pageObject Object A pageObject
         */
        addPageObject: function(pageObject) {
            if (!(pageObject.id in pageObjects)) {
                pageObjects[pageObject.id] = pageObject;
            }
        },

        /**
         * Delete a 'pageObject' from the hashmap if it exists.
         *
         * @param id The id of the 'pageObject' to delete
         */
        deletePageObject: function(id) {
            if ((id in pageObjects)) {
                delete pageObjects[id];
            }
        },

        /**
         * Update a 'pageObject' if it exists in the hashmap.
         *
         * @param pageObject Object A pageObject
         */
        updatePageObject: function(pageObject) {
            if (pageObject.id in pageObjects) {
                pageObjects[pageObject.id] = pageObject;
            }
        },

        /**
         * Update the properties for a Page Object which correspond to the open|close state for the WindowShadeSections.  These properties include:
         * 'generalOptionsSectionCollapsed' and 'themeOptionsSectionCollapsed'.  This will keep things in-sync because they are needed to render
         * the WindowShadeSections to their saved state.
         *
         * @param id
         * @param generalOptionsSectionCollapsed
         * @param themeOptionsSectionCollapsed
         */
        updatePageObjectSectionProperties: function(id, generalOptionsSectionCollapsed, themeOptionsSectionCollapsed) {
            if (id in pageObjects) {
                // Update the Page Object properties which correspond to the open|close state for the WindowShadeSections
                pageObjects[id].generalOptionsSectionCollapsed = generalOptionsSectionCollapsed;
                pageObjects[id].themeOptionsSectionCollapsed = themeOptionsSectionCollapsed;
            }
        },

        /**
         * Return a 'pageObject' from the hashmap if it exists.
         *
         * @param id The id of the 'pageObject' to return
         * @returns {*} Object A pageObject
         */
        getPageObject: function(id) {
            if (id in pageObjects) {
                return pageObjects[id]; // Returns a pageObject
            }
        },

        /**
         * Returns the Page Options model
         *
         * @returns {sap.ui.core.Model|sap.ui.model.Model|*|Object|sas.ltjs.commons.models.BaseModel|sap.ui.model.control.ControlModel}
         */
        getPageOptionsModel: function(pageId) {
            //return sap.ui.getCore().getModel(sas.analytics.riskhp.utils.HprConstants.MODEL_ID_PAGE_OPTIONS);

            if (pageId in pageObjects) {
                // Retrieve the corresponding pageObject
                var pageObject = pageObjects[pageId];

                // Build a JSONModel for the Page Options pane with correct data
                return new sap.ui.model.json.JSONModel({
                    options: {
                        id: pageObject.id,
                        name: pageObject.name,
                        theme: pageObject.theme,
                        applyEnabled: pageObject.applyEnabled,
                        generalOptionsSectionCollapsed: pageObject.generalOptionsSectionCollapsed,
                        themeOptionsSectionCollapsed: pageObject.themeOptionsSectionCollapsed
                    }
                });
                //sap.ui.getCore().setModel(pageOptionsModel, sas.analytics.riskhp.utils.HprConstants.MODEL_ID_PAGE_OPTIONS);
            }
        },

        //*****************************************************************************************************
        //*****************************************************************************************************
        // Plot/Chart model functions - The 'Plot' contains 'chart' and 'chartModel' properties, so those terms
        // will probably be used interchangeably throughout the code.
        //*****************************************************************************************************
        //*****************************************************************************************************
        /**
         * Return the model (aka. 'chartModel') for a Plot with an 'id'.
         *
         * @param id The id of the Plot to look for
         * @returns {*} Object The model of the Plot
         */
        getPlotModel: function(id) {
            var plotModel = null;
            var plot = sap.ui.getCore().byId(id);
            if (plot) {
                var chart = plot.chart;
                // If the 'chart' exists, use it to find the 'chartModel'; Otherwise, if the 'chart' does not exist
                // yet because it hasn't been rendered (see HprChartView.renderer() and _currentContent methods) use
                // the 'chartModel' property on the plot; 'chartModel' is set before rendering occurs so there is a
                // good chance it exists; This is useful when switching the Options pane to Plot Options when a new
                // Plot is created
                if (chart) {
                    plotModel = chart.getModel("chartModel").oData;
                } else {
                    var plotChartModel = plot.getProperty("chartModel");
                    if (plotChartModel) {
                        plotModel = plotChartModel;
                    }
                }
            }
            return plotModel;
        },

        /**
         * Update the options for a Plot, if it exists, using it's model.
         *
         * @param plotOptions Object An object with the options to update for a Plot
         */
        updatePlotOptions: function(plotOptions) {
            var plotModel = this.getPlotModel(plotOptions.id);
            if (plotModel) {
                plotModel.updateOptions(plotOptions);
            }
        },

        /**
         * Update the properties for a Plot which correspond to the open|close state for the WindowShadeSections.  These properties include:
         * 'generalOptionsSectionCollapsed', 'graphOptionsSectionCollapsed', and 'advancedQueryOptionsSectionCollapsed'.  These are needed to WindowShadeSections to their saved state.
         *
         * @param id
         * @param generalOptionsSectionCollapsed
         * @param graphOptionsSectionCollapsed
         * @param advancedQueryOptionsSectionCollapsed
         */
        updatePlotOptionSectionProperties: function(id, generalOptionsSectionCollapsed, graphOptionsSectionCollapsed, advancedQueryOptionsSectionCollapsed) {
            var plotModel = this.getPlotModel(id);
            if (plotModel) {
                // Update the Plot option properties which correspond to the open|close state for the WindowShadeSections
                plotModel.setOptionGeneralOptionsSectionCollapsed(generalOptionsSectionCollapsed);
                plotModel.setOptionGraphOptionsSectionCollapsed(graphOptionsSectionCollapsed);
                plotModel.setOptionAdvancedQueryOptionsSectionCollapsed(advancedQueryOptionsSectionCollapsed);
            }
        },

        /**
         * Causes the plot options model to be updated based on the current information held in the options attribute of
         * the plot with the passed in id.  This will force a refresh on the plotOptionsModel to ensure the plot options
         * pane is updated with the latest changes.
         * @param plotId
         */
        updatePlotOptionsModel: function(plotId) {
            var plotModel = this.getPlotModel(plotId);
            if (plotModel) {
                // Create a temp object that will be used to build the model and create an 'options' layer
                var plotOptionsModel = this.getPlotOptionsModel();
                if (!plotOptionsModel || !plotModel) {
                    return;  // Nothing to be updated.
                }
                var newOptions = plotModel.getOptions();
                plotOptionsModel.setProperty("/options/name",newOptions.name);
                plotOptionsModel.refresh();
            }
        },

        /**
         * Returns the Plot Options model
         *
         * @returns {sap.ui.core.Model|sap.ui.model.Model|*|Object|sas.ltjs.commons.models.BaseModel|sap.ui.model.control.ControlModel}
         */
        getPlotOptionsModel: function(plotId) {
            //return sap.ui.getCore().getModel(sas.analytics.riskhp.utils.HprConstants.MODEL_ID_PLOT_OPTIONS);

            var model = this.getPlotModel(plotId);
            if (model) {
                return new sap.ui.model.json.JSONModel({
                    options: $.extend(true, {}, model.getOptions())
                });
            }
        },

        /**
         * Returns the Plot Filters model
         *
         * @returns {sap.ui.core.Model|sap.ui.model.Model|*|Object|sas.ltjs.commons.models.BaseModel|sap.ui.model.control.ControlModel}
         */
        getPlotFiltersModel: function(plotId) {
            var model = this.getPlotModel(plotId);
            if (model) {
                var showLabels = (model.getOptionShowLabels() && model.getOptionShowLabels().value) ? model.getOptionShowLabels().value : false;
                // Create a new JSONModel for the plot Filters
                return new sap.ui.model.json.JSONModel({
                    plotId: plotId,
                    filters: model.getFilters(),
                    showLabels: model.getOptionShowLabels().value,
                    enableControls: !model.isLinkedToCrosstab()
                });
            }
        },

        /**
         * Update the Plot with which filter sections are expanded for persistence with the exploration.
         *
         * @param id
         * @param filters
         */
        updatePlotFilters: function(id, filters) {
            var model = this.getPlotModel(id);
            if (model) {
                model.setFilters(filters);
            }
        },

        /**
         * Update the expand/collapse state of the filters.
         * @param id
         * @param filters
         */
        updatePlotFilterSections: function(id, filters) {
            var model = this.getPlotModel(id);
            if (model) {
                model.updateFilterSections(_.indexBy(filters, "id"));
            }
        },

        //*****************************************************************************************************
        //*****************************************************************************************************
        // Crosstab context functions
        //*****************************************************************************************************
        //*****************************************************************************************************
        /**
         * Return the 'context' (model) for a Crosstab with an 'id'.
         *
         * @param id The id of the Crosstab to look for
         * @returns {*} Object The context of the Crosstab
         */
        getCrosstabContext: function(id) {
            var context = null;
            var crosstab = sap.ui.getCore().byId(id);
            if (crosstab) {
                context = crosstab.context;
            }
            return context;
        },

        /**
         * Update the options for a Crosstab, if it exists, using it's context.
         *
         * @param crosstabOptions Object An object with the options to update for a Crosstab
         */
        updateCrosstabOptions: function(crosstabOptions) {
            var context = this.getCrosstabContext(crosstabOptions.id);
            if (context) {
                context.updateOptions(crosstabOptions);
            }
        },

        /**
         * Update the properties for a Crosstab which correspond to the open|close state for the WindowShadeSections.  These properties include:
         * 'generalOptionsSectionCollapsed', 'totalsOptionsSectionCollapsed', and 'advancedQueryOptionsSectionCollapsed'.  These are needed to
         * WindowShadeSections to their saved state.
         *
         * @param id
         * @param generalOptionsSectionCollapsed
         * @param totalsOptionsSectionCollapsed
         * @param advancedQueryOptionsSectionCollapsed
         */
        updateCrosstabOptionSectionProperties: function(id, generalOptionsSectionCollapsed, totalsOptionsSectionCollapsed, advancedQueryOptionsSectionCollapsed) {
            var context = this.getCrosstabContext(id);
            if (context) {
                // Update the Crosstab option properties which correspond to the open|close state for the WindowShadeSections
                context.setOptionGeneralOptionsSectionCollapsed(generalOptionsSectionCollapsed);
                context.setOptionTotalsOptionsSectionCollapsed(totalsOptionsSectionCollapsed);
                context.setOptionAdvancedQueryOptionsSectionCollapsed(advancedQueryOptionsSectionCollapsed);
            }
        },

        /**
         * Update the Crosstab with which filter sections are expanded for persistence with the exploration.
         *
         * @param id
         * @param filters
         */
        updateCrosstabFilters: function(id, filters) {
            var context = this.getCrosstabContext(id);
            if (context) {
                context.setFilters(filters);
            }
        },

        /**
         * Update the expand/collapse state of the filters.
         * @param id
         * @param filters
         */
        updateCrosstabFilterSections: function(id, filters) {
            var context = this.getCrosstabContext(id);
            if (context) {
                context.updateFilterSections(_.indexBy(filters, "id"));
            }
        },

        /**
         * Returns the Crosstab Options model
         *
         * @returns {sap.ui.core.Model|sap.ui.model.Model|*|Object|sas.ltjs.commons.models.BaseModel|sap.ui.model.control.ControlModel}
         */
        getCrosstabOptionsModel: function(crosstabId) {
            //return sap.ui.getCore().getModel(sas.analytics.riskhp.utils.HprConstants.MODEL_ID_CROSSTAB_OPTIONS);

            var context = this.getCrosstabContext(crosstabId);
            if (context) {
                // Create a new JSONModel for the Crosstab Options
                return new sap.ui.model.json.JSONModel({options: $.extend(true, {}, context.getOptions())});
            }
        },

        /**
         * Returns the Crosstab Filters model
         *
         * @returns {sap.ui.core.Model|sap.ui.model.Model|*|Object|sas.ltjs.commons.models.BaseModel|sap.ui.model.control.ControlModel}
         */
        getCrosstabFiltersModel: function(crosstabId) {
            var context = this.getCrosstabContext(crosstabId);
            if (context) {
                var showLabels = (context.getOptionShowLabels() && context.getOptionShowLabels().value) ? context.getOptionShowLabels().value : false;
                // Create a new JSONModel for the Crosstab Filters
                return new sap.ui.model.json.JSONModel({crosstabId: crosstabId, filters: context.getFilters(), showLabels: showLabels});
            }
        },

        //*****************************************************************************************************
        //*****************************************************************************************************
        // Portfolio Details model functions
        //*****************************************************************************************************
        //*****************************************************************************************************
        /**
         * Return the model (aka. 'portfolioModel') for a Portfolio Details with an 'id'.
         *
         * @param id The id of the Portfolio Details to look for
         * @returns {*} Object The model of the Portfolio Details
         */
        getPortfolioDetailsModel: function(id) {
            var portfolioDetailsModel = null;
            var portfolioDetails = sap.ui.getCore().byId(id);
            if (portfolioDetails) {
                portfolioDetailsModel = portfolioDetails.getPortfolioModel();
            }
            return portfolioDetailsModel;
        },

        /**
         * Update the options for a Portfolio Details, if it exists, using it's model.
         *
         * @param portfolioDetailsOptions Object An object with the options to update for a Portfolio Details
         */
        updatePortfolioDetailsOptions: function(portfolioDetailsOptions) {
            var portfolioDetailsModel = this.getPortfolioDetailsModel(portfolioDetailsOptions.id);
            if (portfolioDetailsModel) {
                portfolioDetailsModel.updateOptions(portfolioDetailsOptions);
            }
        },

        /**
         * Update the properties for a Portfolio Details which correspond to the open|close state for the WindowShadeSections.  These properties include:
         * 'generalOptionsSectionCollapsed'.  These are needed to WindowShadeSections to their saved state.
         *
         * @param id
         * @param generalOptionsSectionCollapsed
         */
        updatePortfolioDetailsOptionSectionProperties: function(id, generalOptionsSectionCollapsed) {
            var portfolioDetailsModel = this.getPortfolioDetailsModel(id);
            if (portfolioDetailsModel) {
                // Update the Portfolio Details option properties which correspond to the open|close state for the WindowShadeSections
                portfolioDetailsModel.setOptionGeneralOptionsSectionCollapsed(generalOptionsSectionCollapsed);
            }
        },

        /**
         * Returns the Portfolio Details Options model
         *
         * @returns {sap.ui.core.Model}
         */
        getPortfolioDetailsOptionsModel: function(portfolioDetailsId) {
            var portfolioDetailsModel = this.getPortfolioDetailsModel(portfolioDetailsId);
            if (portfolioDetailsModel) {
                // Create a new JSONModel for the Portfolio Details Options
                return new sap.ui.model.json.JSONModel({options: $.extend(true, {}, portfolioDetailsModel.getOptions())});
            }
        },

        /**
         * Returns the Portfolio Details Filters model
         *
         * @returns {sap.ui.core.Model|sap.ui.model.Model|*|Object|sas.ltjs.commons.models.BaseModel|sap.ui.model.control.ControlModel}
         */
        getPortfolioDetailsFiltersModel: function(portfolioDetailsId) {
            var model = this.getPortfolioDetailsModel(portfolioDetailsId);
            if (model) {
                var showLabels = (model.getOptionShowLabels() && model.getOptionShowLabels().value) ? model.getOptionShowLabels().value : false;
                // Create a new JSONModel for the Portfolio Details Filters
                return new sap.ui.model.json.JSONModel({
                    portfolioDetailsId: portfolioDetailsId,
                    filters: model.getFilters(),
                    showLabels: showLabels
                });
            }
        },

        /**
         * Update the Portfolio Details with which filter sections are expanded for persistence with the exploration.
         *
         * @param id
         * @param filters
         */
        updatePortfolioDetailsFilters: function(id, filters) {
            var model = this.getPortfolioDetailsModel(id);
            if (model) {
                model.setFilters(filters);
            }
        },

        /**
         * Update the expand/collapse state of the filters.
         * @param id
         * @param filters
         */
        updatePortfolioDetailsFilterSections: function(id, filters) {
            var model = this.getPortfolioDetailsModel(id);
            if (model) {
                model.updateFilterSections(_.indexBy(filters, "id"));
            }
        },

        //*****************************************************************************************************
        //*****************************************************************************************************
        // Risk Factor Sensitivities (RFS) model functions
        //*****************************************************************************************************
        //*****************************************************************************************************
        /**
         * Return the model (aka. 'rfsModel') for a RFS table with an 'id'.
         *
         * @param id The id of the RFS table to look for
         * @returns {*} Object The model of the RFS table
         */
        getRiskFactorSensitivitiesModel: function(id) {
            var model = null;
            var rfsView = sap.ui.getCore().byId(id);
            if (rfsView) {
                model = rfsView.getRfsModel();
            }
            return model;
        },

        /**
         * Update the options for a RFS, if it exists, using it's model.
         *
         * @param detailsOptions Object An object with the options to update for a RFS
         */
        updateRiskFactorSensitivitiesOptions: function(detailsOptions) {
            var rfsModel = this.getRiskFactorSensitivitiesModel(detailsOptions.id);
            if (rfsModel) {
                rfsModel.updateOptions(detailsOptions);
            }
        },

        /**
         * Update the properties for a RFS which correspond to the open|close state for the WindowShadeSections.  These properties include:
         * 'generalOptionsSectionCollapsed'.  These are needed for WindowShadeSections to their saved state.
         *
         * @param id
         * @param generalOptionsSectionCollapsed
         */
        updateRiskFactorSensitivitiesOptionSectionProperties: function(id, generalOptionsSectionCollapsed) {
            var rfsModel = this.getRiskFactorSensitivitiesModel(id);
            if (rfsModel) {
                // Update the RFS option properties which correspond to the open|close state for the WindowShadeSections
                rfsModel.setOptionGeneralOptionsSectionCollapsed(generalOptionsSectionCollapsed);
            }
        },

        /**
         * Returns the RFS Options model
         *
         * @returns {sap.ui.core.Model}
         */
        getRiskFactorSensitivitiesOptionsModel: function(rfsId) {
            var rfsModel = this.getRiskFactorSensitivitiesModel(rfsId);
            if (rfsModel) {
                // Create a new JSONModel for the RFS Options
                return new sap.ui.model.json.JSONModel({options: $.extend(true, {}, rfsModel.getOptions())});
            }
        },

        //*****************************************************************************************************
        //*****************************************************************************************************
        // TableViewer model functions
        //*****************************************************************************************************
        //*****************************************************************************************************
        /**
         * Return the model (aka. 'tableViewerModel') for a generic TableViewer with an 'id'.
         *
         * @param id The id of the TableViewer to look for
         * @returns {*} Object The model of the TableViewer
         */
        getTableViewerModel: function(id) {
            var model = null;
            var tableViewer = sap.ui.getCore().byId(id);
            if (tableViewer) {
                model = tableViewer.getTableViewerModel();
            }
            return model;
        },

        /**
         * Update the options for a generic TableViewer, if it exists, using it's model.
         *
         * @param detailsOptions Object An object with the options to update for a TableViewer
         */
        updateTableViewerOptions: function(detailsOptions) {
            var tableViewerModel = this.getTableViewerModel(detailsOptions.id);
            if (tableViewerModel) {
                tableViewerModel.updateOptions(detailsOptions);
            }
        },

        /**
         * Update the properties for a generic TableViewer which correspond to the open|close state for the WindowShadeSections.  These properties include:
         * 'generalOptionsSectionCollapsed'.  These are needed for WindowShadeSections to their saved state.
         *
         * @param id
         * @param generalOptionsSectionCollapsed
         */
        updateTableViewerOptionSectionProperties: function(id, generalOptionsSectionCollapsed) {
            var tableViewerModel = this.getTableViewerModel(id);
            if (tableViewerModel) {
                // Update the TableViewer option properties which correspond to the open|close state for the WindowShadeSections
                tableViewerModel.setOptionGeneralOptionsSectionCollapsed(generalOptionsSectionCollapsed);
            }
        },

        /**
         * Returns the TableViewer Options model
         *
         * @param tableViewerId
         * @returns {sap.ui.core.Model}
         */
        getTableViewerOptionsModel: function(tableViewerId) {
            var tableViewerModel = this.getTableViewerModel(tableViewerId);
            if (tableViewerModel) {
                // Create a new JSONModel for the TableViewer Options
                return new sap.ui.model.json.JSONModel({options: $.extend(true, {}, tableViewerModel.getOptions())});
            }
        },

        //*****************************************************************************************************
        //*****************************************************************************************************
        // Draggable object functions
        //*****************************************************************************************************
        //*****************************************************************************************************
        /**
         * Set the list of objects being dragged.  This is intended as a TEMPORARY cache during a drag operation
         * to squirrel away the actual objects being dragged during a drag.  It is intended to be set upon the start
         * of a drag operation and used at the time of drop.  After that, what is held in this cache is no
         * longer valid.
         * @param objects
         */
        setDraggableObjects: function (objects) {
            draggableObjects = objects;
        },

        /**
         * Retrieves the list of objects being dragged.  See the discussion in the setDraggableObjects method
         * on how this is used.
         * @returns {Array}
         */
        getDraggableObjects: function () {
            return draggableObjects;
        },

        //*****************************************************************************************************
        //*****************************************************************************************************
        // Debug options model functions
        //*****************************************************************************************************
        //*****************************************************************************************************
        /**
         * Get the model for the Debug Options.
         *
         * @returns debugOptionsModel
         */
        getDebugOptions: function() {
            var debugOptionsModelObject = sap.ui.getCore().getModel(sas.analytics.riskhp.utils.HprConstants.MODEL_ID_DEBUG_OPTIONS);
            var debugOptionsModel;
            if (debugOptionsModelObject === undefined) {
                // create default debug options
                var debugOptions = {
                    cellHighlighter : true,
                    showAxisHeaders : true,
                    transpose : false,
                    crosstabStyleText : ""
                };
                debugOptionsModel = {
                    options: debugOptions
                };
            }
            else {
                debugOptionsModel = $.extend(true, {}, debugOptionsModelObject.oData);
            }

            return debugOptionsModel;
        },

        /**
         * Set the model for the Debug Options.
         *
         * @param newDebugOptions
         */
        setDebugOptions: function(newDebugOptions) {
            if (newDebugOptions) {
                // Create a temp object that will be used to build the model and create an 'options' layer
                var debugOptionsModelObject = {};
                debugOptionsModelObject.options = $.extend(true, {}, newDebugOptions.options);

                // Create a new JSONModel for the Debug Options
                var debugOptionsModel = new sap.ui.model.json.JSONModel(debugOptionsModelObject);
                sas.analytics.riskhp.utils.HprConstants.USE_DEBUG_MODE_FOR_PANELS = newDebugOptions.options.showPanelHighlighting;
                sas.analytics.riskhp.utils.HprUtil.getSelectedRiskPanelsView().setDebugMode(newDebugOptions.options.showPanelHighlighting);
                sas.analytics.riskhp.utils.HprUtil.getSelectedRiskPanelsView().refreshAllPanels({});
                sap.ui.getCore().setModel(debugOptionsModel, sas.analytics.riskhp.utils.HprConstants.MODEL_ID_DEBUG_OPTIONS);
            }
        },

        /**
         * Returns the Debug Options model
         *
         * @returns {sap.ui.core.Model}
         */
        getDebugOptionsModel: function() {
            return sap.ui.getCore().getModel(sas.analytics.riskhp.utils.HprConstants.MODEL_ID_DEBUG_OPTIONS);
        }
    };
}());