// (c) 2018, SAS Institute Inc.

jQuery.sap.declare("sas.analytics.riskhp.models.cache.ResultsCache");

(function() {
    "use strict";

    jQuery.sap.require("sas.analytics.riskhp.models.cache.CacheKey");

    jQuery.sap.require('sas.analytics.riskhp.common.riskcommons.CrosstabUtil');
    var CrosstabUtil = sas.analytics.riskhp.common.riskcommons.CrosstabUtil;

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Classvar");
    var CLASSVAR = sas.analytics.riskhp.models.dataitems.Classvar.CLASSVAR;
    var CLASSVARS = CLASSVAR + "s";

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Outvar");
    var OUTVAR = sas.analytics.riskhp.models.dataitems.Outvar.OUTVAR;
    var OUTVARS = OUTVAR + "s";
    var Outvar = sas.analytics.riskhp.models.dataitems.Outvar;

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Compvar");
    var COMPVAR = sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR;
    var COMPVARS = COMPVAR + "s";

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Benchvar");
    var BENCHVAR = sas.analytics.riskhp.models.dataitems.Benchvar.BENCHVAR;
    var BENCHVARS = BENCHVAR + "s";

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Statistic");
    var STATISTIC = sas.analytics.riskhp.models.dataitems.Statistic.STATISTIC;
    var STATISTICS = STATISTIC + "s";
    var Statistic = sas.analytics.riskhp.models.dataitems.Statistic;

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Horizon");
    var HORIZON = sas.analytics.riskhp.models.dataitems.Horizon.HORIZON;
    var HORIZONS = HORIZON + "s";
    var Horizon = sas.analytics.riskhp.models.dataitems.Horizon;

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.RelativeHorizon");
    var RELATIVEHORIZON = sas.analytics.riskhp.models.dataitems.RelativeHorizon.RELATIVEHORIZON;

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.AbsoluteHorizon");
    var ABSOLUTEHORIZON = sas.analytics.riskhp.models.dataitems.AbsoluteHorizon.ABSOLUTEHORIZON;

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.RiskCube");
    var RISKCUBE = sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE;
    var RISKCUBES = RISKCUBE + "s";

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.ResultSet");
    var RESULTSET = sas.analytics.riskhp.models.dataitems.ResultSet.RESULTSET;
    var RESULTSETS = RESULTSET + "s";

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Scenario");
    var SCENARIO = sas.analytics.riskhp.models.dataitems.Scenario.SCENARIO;
    var SCENARIOS = SCENARIO + "s";

    jQuery.sap.require("sas.analytics.riskhp.settings.ApplicationSettings");
    var ApplicationSettings = sas.analytics.riskhp.settings.ApplicationSettings;

    jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
    var HprConstants = sas.analytics.riskhp.utils.HprConstants; // No need to do this

    jQuery.sap.require('sas.analytics.riskhp.utils.HprUtil');
    var HprUtil = sas.analytics.riskhp.utils.HprUtil;

    var EXPANSION_NAME = "query";
    var EXPANSION_NAME_SEPARATOR = ">";
    var SAS_ALL = "_sas_ALL_";

    /**
     * @name sas.analytics.riskhp.models.cache.ResultsCache
     * @extends sap.ui.base.Object
     */
    sap.ui.base.Object.extend("sas.analytics.riskhp.models.cache.ResultsCache", {
        metadata: {
            publicMethods: []
        },

        constructor: function (cacheKey) {

            this.id = "cache";
            this._refreshNeeded = false;

            this._resultsTree = this._createRootNode();
            this._dataSourceId = undefined;
            this._classvarIds = []; // classvar dataItem ids
            this._cubeDataModel = undefined;
            this._sorts = undefined; // [{direction: "ascending"|"descending", type: "value"|"absolute", tuple: {name: string, dataItems:[{id: item.id, type: item.type}, ...] }]
            this._options = undefined; // [ {name: option.name, value: option.value}, ... ]
            this._filters = undefined; // [{id: aFilter.id, type: aFilter.type, includes: aFilter.includes, values: aFilter.values}]

            // External dependencies
            this._api = sas.analytics.riskhp.utils.HprAPI;
            this._i18n = sas.analytics.riskhp.utils.HprUtil.getBundle();

            // Use cacheKey to populate this object
            if (cacheKey) {
                this.id = cacheKey.hashCode();
                this._dataSourceId = cacheKey.getDataSourceId();
                this._classvarIds = cacheKey.getClassvarIds(); // classvar dataItem ids
                this._cubeDataModel = undefined;
                this._sorts = cacheKey.getSorts();
                this._options = cacheKey.getOptions();
                this._filters = cacheKey.getFilters();
            }
            else {
                // Anything to do here?
            }

        }
    });

    sas.analytics.riskhp.models.cache.ResultsCache.prototype.getId = function () {
        return this.id;
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype.setId = function (id) {
        this.id = id;
    };

//*****************************
// Refresh of data needed
//*****************************
    sas.analytics.riskhp.models.cache.ResultsCache.prototype.isRefreshNeeded = function () {
        return this._refreshNeeded;
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype.setRefreshNeeded = function (needed) {
        this._refreshNeeded = needed;
    };

//*****************************
// Localized strings here
//*****************************
    sas.analytics.riskhp.models.cache.ResultsCache.prototype.getI18NBundle = function () {
        return this._i18n;
    };

    /**
     * Method for testing purposes that allows you to override the default i18 bundle
     * in a test harness
     * @param i18nBundle
     */
    sas.analytics.riskhp.models.cache.ResultsCache.prototype.setI18NBundle = function (i18nBundle) {
        this._i18n = i18nBundle;
    };

//*****************************
// Cube's data model
//*****************************
    sas.analytics.riskhp.models.cache.ResultsCache.prototype.getCubeDataModel = function () {
        return this._cubeDataModel;
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype.setCubeDataModel = function (cubeDataModel) {
        this._cubeDataModel = cubeDataModel;
    };


//*****************************
// DataSource
//*****************************
    sas.analytics.riskhp.models.cache.ResultsCache.prototype.getDataSourceId = function () {
        return this._dataSourceId;
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype.setDataSourceId = function (dataSourceId) {
        this._dataSourceId = dataSourceId;
        this.id = "cache" + dataSourceId;
    };

//*****************************
// Class Variables    Array of dataItem IDs.
//*****************************
    sas.analytics.riskhp.models.cache.ResultsCache.prototype.getClassVariables = function () {
        var self = this;

        return self._classvarIds;
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype.setClassVariables = function (newCvIDs) {
        var self = this;

        return self.replaceClassVariables(newCvIDs);
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype.getClassVariable = function (idx) {
        var self = this;

        return self._classvarIds[idx];
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype.addClassVariable = function (cvID) {
        var self = this;

        return self.pushClassVariable(cvID);
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype.pushClassVariable = function (cvID) {
        var self = this;

        // Make sure cv is a classvar and not already in the list
        if (self._validClassVariable(cvID)) {
            var index = _.indexOf(self._classvarIds, cvID);
            if (index < 0) {
                return self._classvarIds.push(cvID);
            }
        }
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype._validClassVariable = function (cvID) {
        var self = this;

        var dataItem = self._cubeDataModel.getDataItemById(cvID);
        if (dataItem && dataItem.type === CLASSVAR)
            return dataItem;

        return undefined;
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype.removeClassVariable = function (cvID) {
        var self = this;

        for (var cvIndex = 0; cvIndex < self._classvarIds.length; cvIndex++) {
            if (self._classvarIds[cvIndex] === cvID) {
                // Copy original cv list, remove the cv from new list
                var newCvIDs = self._classvarIds.concat();
                newCvIDs.splice(cvIndex, 1);
                // Call replace to fix tree if necessary
                self.replaceClassVariables(newCvIDs);
                break;
            }
        }
    };

    /**
     * Replace the existing class variables that define this caches.
     *
     * @param newCvIDs  - array of classvar dataItem ids
     * @returns boolean
     */
    sas.analytics.riskhp.models.cache.ResultsCache.prototype.replaceClassVariables = function (newCvIDs) {
        var self = this;
        var success = true;
        var indexOfDifference = 0;

        //Validate the ids are in fact classvars
        for (var i = 0; i < newCvIDs.length; i++) {
            if (!self._validClassVariable(newCvIDs[i])) {
                success = false;
            }
        }
        //May have to update results tree so call replace.
        if (success) {

            // how are they different?  There are four possible conditions: order changed, removed one or more
            // classvars, added one or more classvars, nothing changed.
            var condition;
            for (var a = 0; a < self._classvarIds.length; a++) {
                if (a < newCvIDs.length) {
                    if (self._classvarIds[a] !== newCvIDs[a]) {
                        // order changed
                        condition = "orderChanged";
                        indexOfDifference = a;
                        break;
                    }
                }
                // we have more existing classvars than the incoming new classvars
                else {
                    condition = "lessIncomingClassvars";
                    break;
                }
            }

            // one last check, do we have added classvars?
            if (condition === undefined) {
                if (self._classvarIds.length < newCvIDs.length) {
                    condition = "moreIncomingClassvars";
                }
            }

            // if we get this far and condition is undefined
            // then incoming classvars are the same as existing
            // classvars.  Bail.
            if (condition !== undefined) {

                // order of classvars change?
                if (condition === "orderChanged") {

                    self._classvarIds = [];

                    // fix results tree - trim everything below, relative to the first different
                    // in the order of new classvars
                    self._pruneTree(indexOfDifference);

                    self._classvarIds = newCvIDs;
                }

                // removed one or more classvars?
                else if (condition === "lessIncomingClassvars") {

                    // fix results tree - trim everything below, relative to the last classvar
                    // of new classvars
                    self._pruneTree(newCvIDs.length);

                    self._classvarIds = newCvIDs;
                }

                // added one or more classvars?  Just add them to the classvar list
                else if (condition === "moreIncomingClassvars") {

                    self._classvarIds = newCvIDs;
                }
                else {
                    // shouldn't get here
                }

            }
        }  // success = true
        return success;
    };


//*****************************
// Mid-tier API
//*****************************
    sas.analytics.riskhp.models.cache.ResultsCache.prototype.getRemoteAPI = function () {
        return this._api;
    };

    /**
     * Method for testing purposes that allows you to override the default remote api
     * handler in a test harness
     * @param api
     */
    sas.analytics.riskhp.models.cache.ResultsCache.prototype.setRemoteAPI = function (api) {
        this._api = api;
    };

    // This function will take a single expansion object to query the midtier.
    sas.analytics.riskhp.models.cache.ResultsCache.prototype.queryExpansion = function (oExpansion, hprReportRequest) {
        var self = this;
        // Wrap expansion in a array and pass to handler
        return self.queryExpansions([oExpansion], hprReportRequest);
    };

    // This function will take an array of expansion objects (instead of a single expansion) to query the midtier.
    sas.analytics.riskhp.models.cache.ResultsCache.prototype.queryExpansions = function (expArray, hprReportRequest) {
        var self = this;

        if (self._api !== undefined) {

            if (hprReportRequest === undefined) {
                hprReportRequest = self._createQueryRequest(expArray);
            }

            // Save the query request.
            self.lastQueryRequest = hprReportRequest;

            // Set breakpoint here to see hprReportRequest that is sent to midtier
            // returning a deferred here
            return self._api.queryExpansion(hprReportRequest);
        }

        return $.Deferred().reject();
    };

//*****************************
// Query Request to midtier
//*****************************

    // oExpansion = { key: [ccValue], startRow: startRow, maxRows: maxRows }
    sas.analytics.riskhp.models.cache.ResultsCache.prototype._createQueryRequest = function (expArray) {
        var self = this;

        // Timestamp shim in case browser is non-modern
        if (!Date.now) {
            Date.now = function () {
                return new Date().getTime();
            };
        }

        var jsonObject = {
            version: 1,
            id: self.getId(),
            type: "crosstab",
            dataSources: self._cubeDataModel.queryJSON(),

            dataDefinitions: {
                classVariables: self.getClassVariables(),
            },

            report: {
                sorts: self.buildSortJSON(),
                options: self.getOptions(),
                filters: self.getFilters(),
                requests: self.generateRequests(expArray)
            }
        };

        return jsonObject;
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype.buildSortJSON = function () {
        var self = this;

        if (self._sorts) {
            return self._sorts;
        }
        else
            return [];
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype.generateRequests = function (expArray) {
        var self = this;
        var requests = [];

        _.each(expArray, function (oExpansion) {
            requests.push(self.generateRequest(oExpansion));
        });

        return requests;
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype.generateRequest = function (oExpansion) {
        var self = this;

        //FIXME temporary.  See if the "old" expansion is being used, if so, use it instead.
        if (!oExpansion.hasOwnProperty("name")) {
            // oExpansion = { key: [ccValue], startRow: startRow, maxRows: maxRows }
            var name = "cache";
            var expansion = _.map(oExpansion.key, function (ccValue, index) {
                name = name + ">" + ccValue;
                return {id: self._classvarIds[index], value: ccValue};
            });

            return {name: name, key: expansion, startRow: oExpansion.startRow, maxRows: oExpansion.maxRows};
        }
        else {
            return oExpansion;
        }
    };

//*****************************
// Property accessors
//*****************************

    // [{direction: "ascending"|"descending", type: "value"|"absolute", tuple: {name: string, dataItems:[{id: item.id, type: item.type}, ...] }]
    sas.analytics.riskhp.models.cache.ResultsCache.prototype.setSorts = function (sorts) {
        var self = this;
        var success = true;

        if (sorts && sorts.length > 0) {
            sorts = undefined;
        }
        self._sorts = sorts;

        return success;
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype.getSorts = function () {
        var self = this;
        return self._sorts;
    };


    // [{id: aFilter.id, type: aFilter.type, includes: aFilter.includes, values: aFilter.values}]
    sas.analytics.riskhp.models.cache.ResultsCache.prototype.setFilters = function (oFilters) {
        var self = this;
        var success = true;

        if (oFilters && oFilters.length === 0) {
            oFilters = undefined;
        }
        self._filters = oFilters;

        return success;
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype.getFilters = function () {
        var self = this;
        return self._filters;
    };

    // [ {name: option.name, value: option.value} ]
    sas.analytics.riskhp.models.cache.ResultsCache.prototype.setOptions = function (oOption) {
        var self = this;
        var success = true;
        self._options = oOption;
        return success;
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype.getOptions = function () {
        var self = this;
        return self._options;
    };


    //*****************************
    // Expansions
    //*****************************

    sas.analytics.riskhp.models.cache.ResultsCache.prototype._createExpansionQueryKey = function (valueArray, includeExpansion) {

        var queryKey = valueArray.concat(); // clone

        if (queryKey.length === 0) {
            if (includeExpansion) {
                queryKey.push(SAS_ALL);
            }
        } else {
            if ((queryKey[queryKey.length - 1] === this._i18n.getText('HPRisk.Crosstab.Total.txt')) ||
                (queryKey[queryKey.length - 1] === this._i18n.getText('HPRisk.Crosstab.Subtotal.txt'))) {
                // This is a catchall and I don't think includeExpansion pertains here.
                queryKey[queryKey.length - 1] = SAS_ALL;
            } else {
                if (includeExpansion) {
                    // Ensure the last element of key is a SAS_ALL
                    queryKey.push(SAS_ALL);
                }
            }
        }

        return queryKey;
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype._createExpansionName = function (key) {
        var name;
        if (key) {
            name = EXPANSION_NAME;
            for (var i = 0; i < key.length; i++) {
                name = name.concat(EXPANSION_NAME_SEPARATOR, key[i]);
            };
        }
        else {
            name = "toplevel";
        }
        return name;
    };


//*****************************
// Results tree
//*****************************
    // Parse the key into its parts
    sas.analytics.riskhp.models.cache.ResultsCache.prototype._parseKey = function (obj) {
        var propertyArray = Object.keys(obj);
        var cvID = propertyArray[0];
        var cvValue = obj[cvID];

        return {id: cvID, value: cvValue};
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype._createRootNode = function () {
        return {
            name: "toplevel",
            label: "toplevel",
            isSasAllResult: false,
            result: undefined,
            children: [],
            startRow: 1,
            rows: 1,
            totalRows: 1
        };
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype._createChildNode = function (result, key) {
        var self = this;

        // Get name and label from the key
        // "key": {
        //    "0018l2gsh4z2": "Americas"
        // },
        if (key === undefined) {
            key = result.key;
        }
        var oKey = self._parseKey(key);
        var cvID = oKey.id;
        var cvValue = oKey.value;

        // Return a node
        return {
            name: cvValue,
            key: key,
            isSasAllResult: false,
            result: result,
            children: [],
            startRow: 1,
            rows: 1,
            totalRows: 1
        };
    };

    /**
     * This function will take the given query results from the midtier and place the results into
     * the tree, building nodes as necessary.
     *
     * @param json - data received from mid-tier
     */
    sas.analytics.riskhp.models.cache.ResultsCache.prototype._setDataToCache = function (oResults) {
        var self = this;

        if (oResults) {

            // Find node in tree based on key given in json
            // Create nodes if necessary until we get to the final path entry
            var _findNode = function (subPath, node) {

                if (subPath.length > 0) {
                    for (var i = 0; i < node.children.length; i++) {
                        var childNode = node.children[i];
                        if (childNode.name === self._parseKey(subPath[0]).value) {
                            subPath.shift();
                            return _findNode(subPath, childNode);
                        }
                    }
                    // Didn't find the childNode, create and keep going
                    var newChildNode = self._createChildNode(undefined, subPath[0]);
                    node.children.push(newChildNode);
                    subPath.shift();
                    return _findNode(subPath, newChildNode);
                }
                return node;
            };

            // Determine if this result is from a _sas_ALL_ request.
            var _isSasAllResult = function (name) {
                var pos = name.lastIndexOf(SAS_ALL);
                return pos >= 0 ? true : false;
            };

            // Iterate over the queryResults entries.  Each entry represents one query expansion in
            // the original request sent to midtier.
            _.each(oResults.queryResults, function (qResult, index) {

                // If we have actual data results then process, else skip it.
                // (Skip it because it could have been filtered out.  S1427052)
                if (qResult && qResult.results && qResult.results.length > 0) {

                    var node = _findNode(qResult.key.concat(), self._resultsTree);

                    // Save the children nodes, we may need info from them to repopulate newer children.
                    var nodeChildren = node.children;
                    // remove existing children
                    node.children = [];

                    // S1439669
                    // Set the flag on the node to signify that results here came from a sas_all and
                    // thus a full result.  (as opposed to a targeted result such as coming from a plot)
                    node.isSasAllResult =  _isSasAllResult(qResult.name);

                    // look at each result
                    for (var resultsIndex = 0; resultsIndex < qResult.results.length; resultsIndex++) {

                        var cResult = qResult.results[resultsIndex];

                        // this result has a classvar value
                        // key = { cvid : cvValue }
                        if (cResult.key && Object.keys(cResult.key).length > 0) {
                            var newResultNode = self._createChildNode(cResult, undefined);
                            node.children.push(newResultNode);
                            // Is there an existing child with that name, if so copy over previous data
                            var name = self._parseKey(cResult.key).value;
                            var childNode = _.find(nodeChildren, function (child) {
                                return child.name === name;
                            });
                            if (childNode) {
                                newResultNode.children = childNode.children;
                                newResultNode.startRow = childNode.startRow;
                                newResultNode.rows = childNode.rows;
                                newResultNode.totalRows = childNode.totalRows;
                            }
                        }
                        // a result does not have a classvar value, thus this result is meant for this node
                        // and not a child
                        else {
                            node.result = cResult;
                        }
                    }

                    // set the paging info
                    node.startRow = qResult.startRow;
                    node.rows = qResult.rows;
                    node.totalRows = qResult.totalRows;
                }

            }); // Each queryResults entry

        } // no json argument
    };

    // Parse the key into its parts
    // Find a node based on path.  Use "name" or "label" property when comparing path entries.
    sas.analytics.riskhp.models.cache.ResultsCache.prototype._findTreeNode = function (path, nodeProperty) {
        var self = this;
        var node;

        //nodeProperty = "name" or "label" for our purposes.  So we are testing based off of if key is an array
        // of names or labels.  We can search the nodes based on either.
        if (nodeProperty === undefined) {
            nodeProperty = "name";
        }

        var _findNode = function (subPath, node) {

            if (subPath.length > 0 && subPath[0] !== SAS_ALL) {
                for (var i = 0; i < node.children.length; i++) {
                    var childNode = node.children[i];
                    //if (childNode[nodeProperty] === self._parseKey(subPath[0]).value) {
                    if (childNode[nodeProperty] === subPath[0]) {
                        subPath.shift();
                        return _findNode(subPath, childNode);
                    }
                }
                return undefined; // {}
            }
            return node;
        };

        //if (this._resultsTree.children.length > 0) { // REMOVED for path = [] should return root resultsTree
        node = _findNode(path.concat(), this._resultsTree);
        //};

        return node;
    };


    // Trim row tree to the given depth
    sas.analytics.riskhp.models.cache.ResultsCache.prototype._pruneTree = function (depth) {
        var self = this;
        var node;

        var _findNode = function (myDepth, node) {

            if (myDepth === depth) {
                node.children = [];
                //node.result = undefined;  // Leave this result alone
                node.startRow = 1;
                node.rows = 1;
                node.totalRows = 1;
            } else {
                for (var b = 0; b < node.children.length; b++) {
                    _findNode(myDepth + 1, node.children[b]);
                }
            }
        };

        if (depth === 0) {
            self._resultsTree.children = [];
            //self._resultsTree.result = undefined; // Leave this result alone
            self._resultsTree.startRow = 1;
            self._resultsTree.rows = 1;
            self._resultsTree.totalRows = 1;
        }
        else {
            for (var idx = 0; idx < self._resultsTree.children.length; idx++) {
                _findNode(1, self._resultsTree.children[idx]);
            }
        }
    };


//*****************************
// Query cache, get data
//*****************************

    /**
     * This function will return an array of data based on the expansion.
     * What data, is defined by the tuple(s).
     *
     * Note: this function will get the data from the tree, if present.  If expansion
     * not found or page not found, then the data is pulled from midtier/server.  The
     * tree is updated with this new data prior to being returned to caller.
     *
     * @param oExpansion = { key: [ccValue], startRow: startRow, maxRows: maxRows } or []
     * @param tuples = [ [dataItem, ... dataItem], ..., [dataItem, ... dataItem] ]
     * @returns deferred     results = [{expansion: oExpansion, data: rowsOfData},...]
     *                    rowsOfData = [{value: ccValue, data: [x,y,z]}, ... ]
     */
    sas.analytics.riskhp.models.cache.ResultsCache.prototype.getData = function (oExpansion, tuples) {
        var self = this;

        // Multiple expansions?
        if (Array.isArray(oExpansion)) {
            return self._getDataMultipleExpansions(oExpansion, tuples);
        }
        // Single expansion?
        else {
            return self._getDataSingleExpansion(oExpansion, tuples);
        }
    };

    // Find which expansion produce cache or page miss.
    // Those need to be queried and cache populated before returning data
    sas.analytics.riskhp.models.cache.ResultsCache.prototype._getDataMultipleExpansions = function(expArray, tuples) {
        var self = this;
        var deferred = $.Deferred();
        var expWithCacheMiss = [];

        var _doAction = function(expArray, tuples) {
            // for each expansion see if in cache
            _.each(expArray, function (oExpansion, index) {

                // if an expansion is not in cache, then add to list
                var node = self._findTreeNode(oExpansion.key, "name");

                if (node) {
                    // Got a cache hit (so far), check the data there
                    // if page of data not there then cache miss
                    if (self._isPageMiss(oExpansion, node)) {
                        expWithCacheMiss.push(oExpansion);
                    }
                }
                else {
                    expWithCacheMiss.push(oExpansion);
                }
            });

            // Query midtier for data for any expansion cache or page miss.
            if (expWithCacheMiss.length > 0) {
                $.when(
                    self.queryExpansions(expWithCacheMiss)
                ).done(function (queryData) {
                    if (!queryData || (queryData && queryData.status !== 200)) {
                        deferred.reject(queryData.status);
                    } else {
                        // take data and put it in the cache.
                        self._setDataToCache(queryData);
                        // return the data from the cache
                        var results = self._getDataFromCache(expArray, tuples);
                        deferred.resolve(results);
                    }
                }).fail(function (jqxhr) {
                    deferred.reject(jqxhr);
                });
            }
            // Just return the data from the cache
            else {
                var results = self._getDataFromCache(expArray, tuples);
                deferred.resolve(results);
            }
        };

        if (self.isRefreshNeeded()) {
            $.when(
                self.refresh()
            ).done(function (queryData) {
                _doAction(expArray, tuples);
            }).fail(function (jqxhr) {
                deferred.reject(jqxhr);
            });
        }
        else {
            _doAction(expArray, tuples);
        }

        return deferred;
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype._getDataSingleExpansion = function(oExpansion, tuples) {
        var self = this;
        var deferred = $.Deferred();

        var _doAction = function(oExpansion, tuples) {
            var node = self._findTreeNode(oExpansion.key, "name");

            if (node) {
                // Got a cache hit (so far), check the data there
                // if page of data not there then cache miss
                if (self._isPageMiss(oExpansion, node)) {

                    $.when(
                        self.queryExpansion(oExpansion)
                    ).done(function (queryData) {
                        if (!queryData || (queryData && queryData.status !== 200)) {
                            deferred.reject(queryData.status);
                        } else {
                            // take data and put it in the tree.
                            self._setDataToCache(queryData);
                            var results = self._getDataFromCache([oExpansion], tuples);
                            deferred.resolve(results);
                        }
                    }).fail(function (jqxhr) {
                        deferred.reject(jqxhr);
                    });
                }
                else {
                    var results = self._getDataFromCache([oExpansion], tuples);
                    deferred.resolve(results);
                }
            }
            else {
                // Got a cache miss, query the data.
                $.when(
                    self.queryExpansion(oExpansion)
                ).done(function (queryData) {
                    if (!queryData || (queryData && queryData.status !== 200)) {
                        deferred.reject(queryData.status);
                    } else {
                        // take data and put it in the tree.
                        self._setDataToCache(queryData);
                        var results = self._getDataFromCache([oExpansion], tuples);
                        deferred.resolve(results);
                    }
                }).fail(function (jqxhr) {
                    deferred.reject(jqxhr);
                });
            }
        };

        if (self.isRefreshNeeded()) {
            $.when(
                self.refresh()
            ).done(function (queryData) {
                _doAction(oExpansion, tuples);
            }).fail(function (jqxhr) {
                deferred.reject(jqxhr);
            });
        }
        else {
            _doAction(oExpansion, tuples);
        }

        return deferred;
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype._isPageMiss=function(oExpansion, node) {
        var self = this;
        var pageMiss = true;

        //if oExpansion DOES contain SAS_ALL, then check startRow >= node.startRow
        if (oExpansion.key[oExpansion.key.length -1] === SAS_ALL) {

            // page of results in node similar to page of results requested in expansion
            if (node.isSasAllResult === true && node.startRow === oExpansion.startRow) {
                if(node.children && node.children.length > 0)
                    pageMiss = false;
            }
        }
        // if oExpansion does not contain SAS_ALL , then no miss.
        else {
            if(node.result && !self._isEmpty(node.result))
                pageMiss = false;
        }
        return pageMiss;
    };

    //  [{value: ccValue, data: [x,y,z]}, ... ]
    //NOTE: this function assumes data has already be retrieved from midtier/server and tree populated.
    sas.analytics.riskhp.models.cache.ResultsCache.prototype._getDataFromCache=function(expArray, tuples) {
        var self = this;
        var expansionData = [];

        _.each(expArray, function (oExpansion, index) {
            var rowsOfData = [];

            var node = self._findTreeNode(oExpansion.key, "name");

            if (node) {
                // If SAS_ALL, then collect the data from each child node
                if (oExpansion.key[oExpansion.key.length - 1] === SAS_ALL) {
                    _.each(node.children, function (childNode) {
                        var data = [];
                        _.each(tuples, function (tuple) {
                            data.push(self._parseDataFromResult(tuple, childNode.result));
                        });
                        rowsOfData.push({value: childNode.key, data: data});
                    });
                    oExpansion.startRow = node.startRow;
                    oExpansion.rows = node.rows;
                    oExpansion.totalRows = node.totalRows;
                }
                // If NOT SAS_ALL, the collect the data from this node
                else {
                    var data = [];
                    _.each(tuples, function (tuple) {
                        data.push(self._parseDataFromResult(tuple, node.result));
                    });
                    rowsOfData.push({value: node.key, data: data});
                    oExpansion.startRow = 1; // Set this to 1 because only 1 result is desired/expected
                    oExpansion.rows = 1; // Set this to 1 because only 1 result is desired/expected
                    oExpansion.totalRows = node.totalRows;
                }
            }
            expansionData.push({expansion: oExpansion, data: rowsOfData});
        });

        return expansionData;
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype._parseDataFromResult=function(tuple, result) {
        var self = this;
        var data = undefined;
        var tupleItemInfo = undefined;

        // Check the simple stuff first before getting into more complicated tuples.
        if (result !== undefined) {
            // Tuples with compvar or benchvar consist of single dataItem and never used
            // with other dataItems.
            tupleItemInfo = self._getDataItemInfoFromTuple(tuple, COMPVAR);
            if (tupleItemInfo !== undefined) {
                data = result[COMPVARS][tupleItemInfo.dataItem.id];
            }
            else {
                tupleItemInfo = self._getDataItemInfoFromTuple(tuple, BENCHVAR);
                if (tupleItemInfo !== undefined) {
                    data = result[BENCHVARS][tupleItemInfo.dataItem.id];
                }
                else {
                    // Check for the the more complicated tuples with multiple dataItems

                    // Check for risk cube
                    tupleItemInfo = self._getDataItemInfoFromTuple(tuple, RISKCUBE);
                    if (tupleItemInfo !== undefined) {
                        data = self._parseDataFromRiskCube(tuple, result[RISKCUBES][tupleItemInfo.dataItem.id]);
                    }
                    else {
                        // check for result set
                        tupleItemInfo = self._getDataItemInfoFromTuple(tuple, RESULTSET);
                        if (tupleItemInfo !== undefined) {
                            data = self._parseDataFromResultSet(tuple, result[RESULTSETS][tupleItemInfo.dataItem.id]);
                        }
                        else {
                            // If all else fails
                            // row stat by itself?
                            var statItemInfo = self._getDataItemInfoFromTuple(tuple, STATISTIC);
                            data = self._parseDataForStatistics(statItemInfo.dataItem.id, result[STATISTICS]);
                        }
                    }
                }
            }
        }
        else {
            sas.log.error("Could not determine data : results = undefined");
            data = ".";
        }

        return data;
    };


    sas.analytics.riskhp.models.cache.ResultsCache.prototype._parseDataFromRiskCube=function(tuple, rcResult) {
        var self = this;
        var data = undefined;
        var tupleItemInfo = undefined;
        try {
            // We will eventually support resultsets under risk cube.  Be proactive.
            tupleItemInfo = self._getDataItemInfoFromTuple(tuple, RESULTSET);
            if (tupleItemInfo !== undefined) {
                data = self._parseDataFromResultSet(tuple, rcResult[RESULTSETS][tupleItemInfo.dataItem.id]);
            }
            else {
                // Check for typical horizon, outvar, stat
                tupleItemInfo = self._getDataItemInfoFromTuple(tuple, HORIZON);
                if (tupleItemInfo !== undefined) {
                    data = self._parseDataFromHorizon(tuple, rcResult[HORIZONS][tupleItemInfo.dataItem.id]);
                }
                else {
                    // No horizon, so there must be a row stat
                    var statItemInfo = self._getDataItemInfoFromTuple(tuple, STATISTIC);
                    data = self._parseDataForStatistics(statItemInfo.dataItem.id, rcResult[STATISTICS]);
                }
            }
        }
        catch(err) {
            sas.log.error("Could not determine data for risk cube : " + err);
            data = ".";
        }
        return data;
    };


    sas.analytics.riskhp.models.cache.ResultsCache.prototype._parseDataFromResultSet=function(tuple, rsResult) {
        var self = this;
        var data = undefined;
        var tupleItemInfo = undefined;
        try {
            tupleItemInfo = self._getDataItemInfoFromTuple(tuple, SCENARIO);
            if (tupleItemInfo !== undefined) {
                data = self._parseDataFromScenario(tuple, rsResult[SCENARIOS][tupleItemInfo.dataItem.id]);
            }
            else {
                tupleItemInfo = self._getDataItemInfoFromTuple(tuple, HORIZON);
                if (tupleItemInfo !== undefined) {
                    data = self._parseDataFromHorizon(tuple, rsResult[HORIZONS][tupleItemInfo.dataItem.id]);
                }
                else {
                    // No horizon, so there must be a row stat
                    var statItemInfo = self._getDataItemInfoFromTuple(tuple, STATISTIC);
                    data = self._parseDataForStatistics(statItemInfo.dataItem.id, rsResult[STATISTICS]);
                }
            }
        }
        catch(err) {
            sas.log.error("Could not determine data for result set : " + err);
            data = ".";
        }
        return data;
    };


    sas.analytics.riskhp.models.cache.ResultsCache.prototype._parseDataFromScenario=function(tuple, scenResult) {
        var self = this;
        var data = undefined;
        var tupleItemInfo = undefined;
        try {
            tupleItemInfo = self._getDataItemInfoFromTuple(tuple, HORIZON);
            if (tupleItemInfo !== undefined) {
                data = self._parseDataFromHorizon(tuple, scenResult[HORIZONS][tupleItemInfo.dataItem.id]);
            }
            else {
                // No horizon. Currently nothing else is supported under scenarios
                data = ".";
            }
        }
        catch(err) {
            sas.log.error("Could not determine data for scenario : " + err);
            data = ".";
        }
        return data;
    };


    sas.analytics.riskhp.models.cache.ResultsCache.prototype._parseDataFromHorizon=function(tuple, horizonResult) {
        var self = this;
        var data = undefined;
        try {
            var ovItemInfo = self._getDataItemInfoFromTuple(tuple, OUTVAR);
            data = self._parseDataFromOutvar(tuple, horizonResult[OUTVARS][ovItemInfo.dataItem.id]);
        }
        catch(err) {
            sas.log.error("Could not determine data for horizon : " + err);
            data = ".";
        }
        return data;
    };


    sas.analytics.riskhp.models.cache.ResultsCache.prototype._parseDataFromOutvar=function(tuple, outvarResult) {
        var self = this;
        var data = undefined;
        try {
            var statItemInfo = self._getDataItemInfoFromTuple(tuple, STATISTIC);
            data = self._parseDataForStatistics(statItemInfo.dataItem.id, outvarResult[STATISTICS]);
        }
        catch(err) {
            sas.log.error("Could not determine data for outvar : " + err);
            data = ".";
        }
        return data;
    };


    sas.analytics.riskhp.models.cache.ResultsCache.prototype._parseDataForStatistics=function(statisticID, statResult) {
        var self = this;
        var data = undefined;
        try {
            // statResult is an object of statisticIDs and values { id1: value1, id2: value2, ... idN: valueN }
            data = statResult[statisticID];
        }
        catch(err) {
            sas.log.error("Could not determine data for stat : " + err);
            data = ".";
        }
        return data;
    };

    /**
     * This function will force a requery/refresh of all the data in the cache.
     * Note: Useful to get new data such as activated scenario into the cache.
     * @returns deferred
     */
    sas.analytics.riskhp.models.cache.ResultsCache.prototype.refresh = function () {
        var self = this;
        var deferred = $.Deferred();
        var nrows = _.findWhere(self.getOptions(), {name: HprConstants.HPR_OPTIONS_PANE_NROWS_NAME}).value;
        var expansions = []; // example{key: [], startRow: 1, maxRows: 500}

        var _findNode = function (path, node) {
            if (node.children.length > 0) {
                var myPath = path.concat();
                myPath.push(SAS_ALL);
                expansions.push({key: myPath, startRow: node.startRow, maxRows: nrows});
                for (var b = 0; b < node.children.length; b++) {
                    var childPath = path.concat();
                    childPath.push(node.children[b].name);
                    _findNode(childPath, node.children[b]);
                }
            }
        };

        // walk the results tree and collect expansions
        expansions.push({key: [], startRow: 1, maxRows: nrows}); // top level expansion
        _findNode([], self._resultsTree);

        // query midtier with expansions
        $.when(
            self.queryExpansions(expansions)
        ).done(function (queryData) {
            if (!queryData || (queryData && queryData.status !== 200)) {
                deferred.reject(queryData.status);
            } else {
                // put the results data back into tree overwriting existing results.
                self._setDataToCache(queryData);
                self.setRefreshNeeded(false);
                deferred.resolve("true");
            }
        }).fail(function (jqxhr) {
            deferred.reject(jqxhr);
        });

        return deferred;
    };


//*****************************
// Internal Utilities and helpers
//*****************************

    // A check to see if a given object is empty, ie {}.
    sas.analytics.riskhp.models.cache.ResultsCache.prototype._isEmpty=function(obj) {
        var self = this;
        for(var key in obj) {
            if(obj.hasOwnProperty(key))
                return false;
        }
        return true;
    };


//*****************************
// External Utilities and helpers
//*****************************

    // Used by getPickInfo to convert the displayed path (based on label) with path used by mid-tier
    // (based on name).
    sas.analytics.riskhp.models.cache.ResultsCache.prototype.convertPathFromLabelToName=function(path) {
        //sas.analytics.riskhp.models.cache.ResultsCache.prototype.convertRowKeyLabelToName=function(path)
        var self = this;
        var node = self.findNodeFromPath(path, "label"); // The path entries are labels
        var convertedPath = [];

        while(node && node.parent !== undefined) {
            convertedPath.push(node["name"]); // Get the name from the node
            node = node.parent;
        }
        return convertedPath.reverse();
    };

    sas.analytics.riskhp.models.cache.ResultsCache.prototype.findNodeFromPath = function (path, nodeProperty) {
        var self = this;
        var node;

        //nodeProperty = "name" or "label" for our purposes.  So we are testing based off of if key is an array
        // of names or labels.  We can search the nodes based on either.
        if (nodeProperty === undefined) {
            nodeProperty = "name";
        }

        var _findNode = function (subPath, node) {

            if (subPath.length > 0 && subPath[0] !== SAS_ALL) {
                for (var i = 0; i < node.children.length; i++) {
                    var childNode = node.children[i];
                    //if (childNode[nodeProperty] === self._parseKey(subPath[0]).value) {
                    if (childNode[nodeProperty] === subPath[0]) {
                        subPath.shift();
                        return _findNode(subPath, childNode);
                    }
                }
                return undefined; // {}
            }
            return node;
        };

        //if (this._resultsTree.children.length > 0) { // REMOVED for path = [] should return root resultsTree
        node = _findNode(path.concat(), this._resultsTree);
        //};

        return node;
    };


    // Given a tuple and a dataItem type, return the dataItem matching the type.
    // Note: this function tries to service a tuple consisting of dataItems or consisting of IDs.
    sas.analytics.riskhp.models.cache.ResultsCache.prototype._getDataItemInfoFromTuple=function(tuple, type){
        var self = this;

        for (var idx = 0; idx < tuple.length; idx++) {
            var dataItem;
            // Test the tuple entry to see if it is a dataItem or an id.
            // If id, the fetch the dataItem.
            if (typeof tuple[idx] === "string") {
                dataItem = self._cubeDataModel.getDataItemById(tuple[idx]);
            }
            else {
                dataItem = tuple[idx];
            }
            if (dataItem && dataItem.getType() === type) {
                return {dataItem: dataItem, index: idx};
            }
        }
        return undefined;
    };

})();