// (c) 2017, SAS Institute Inc.

jQuery.sap.declare("sas.analytics.riskhp.models.dataitems.Scenario");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.BaseDataItem");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Horizon");

jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");

// noinspection JSClosureCompilerSyntax
/**
 * @name sas.analytics.riskhp.models.dataitems.Scenario
 * @extends sas.analytics.riskhp.models.dataitems.BaseDataItem
 */
sas.analytics.riskhp.models.dataitems.BaseDataItem.extend("sas.analytics.riskhp.models.dataitems.Scenario",{
	constructor:function(json){
        // call super
        sas.analytics.riskhp.models.dataitems.BaseDataItem.prototype.constructor.call(this, json);

        this.repoId = json.repoId; // Id used by scenario manager, instead of hprisk's id.
        this.workGroup = json.workGroup;
        this.scenarioVersion = json.scenarioVersion;
        this.description = json.description;

        this.dataSourceName = json.dataSourceName;

        this.baselineId = json.baselineId;
        this.baseline = json.baselineName ? json.baselineName : sas.analytics.riskhp.utils.HprUtil.getBundle().getText("DataItem.scenario.nobaseline.txt");
        this.baselineVersion = json.baselineVersion ? json.baselineVersion : "";

        var horizonsArray = [];
        _.each(json.horizons, function(json){
            horizonsArray.push(new sas.analytics.riskhp.models.dataitems.Horizon(json))
        });
        this.horizons = horizonsArray;

        if (json.modifiedTimeStamp) {
            this.modifiedTimeStamp = new Date(json.modifiedTimeStamp);
            this.modifiedTimeStampDisplayString = sas.analytics.riskhp.utils.HprUtil.formatDateTime(this.modifiedTimeStamp);
        }
        this.modifiedBy = json.modifiedBy;
        if (json.createdTimeStamp) {
            this.createdTimeStamp = new Date(json.creationTimeStamp);
            this.createdTimeStampDisplayString = sas.analytics.riskhp.utils.HprUtil.formatDateTime(this.createdTimeStamp);
        }
        this.createdBy = json.createdBy;

        this.activated = json.activated;

        // do this after activation is set
        this._updateIcon();
    }
});

sas.analytics.riskhp.models.dataitems.Scenario.prototype._updateIcon = function() {
    if (!this.activated) {
        this.icon = sas.icons.HC.DRAGINVALID;
    }
    else {
        this.icon = sas.icons.HC.SCENARIO;
    }
};

sas.analytics.riskhp.models.dataitems.Scenario.prototype.setScenarioVersion = function(scenarioVersion) {
    this.scenarioVersion = scenarioVersion;
};

sas.analytics.riskhp.models.dataitems.Scenario.prototype.getScenarioVersion = function() {
    return this.scenarioVersion;
};

sas.analytics.riskhp.models.dataitems.Scenario.prototype.getBaselineId = function() {
    return this.baselineId;
};

sas.analytics.riskhp.models.dataitems.Scenario.prototype.setHorizons = function(horizons) {
    // Always make a copy so it can't be changed out from under the object
    this.horizons = $.extend(true,[],horizons);
};

sas.analytics.riskhp.models.dataitems.Scenario.prototype.getHorizons = function() {
    // Return a copy here so consumers can't change the underlying collection
    return $.extend(true,[],this.horizons);
};

sas.analytics.riskhp.models.dataitems.Scenario.prototype.getDescription = function() {
    return this.description;
};

sas.analytics.riskhp.models.dataitems.Scenario.prototype.getBaseline = function() {
    return this.baseline;
};

sas.analytics.riskhp.models.dataitems.Scenario.prototype.getBaselineVersion = function() {
    return this.baselineVersion;
};

sas.analytics.riskhp.models.dataitems.Scenario.prototype.getModifiedTimeStamp = function() {
    return this.modifiedTimeStamp;
};

sas.analytics.riskhp.models.dataitems.Scenario.prototype.getModifiedBy = function() {
    return this.modifiedBy;
};

sas.analytics.riskhp.models.dataitems.Scenario.prototype.getCreatedTimeStamp = function() {
    return this.createdTimeStamp;
};

sas.analytics.riskhp.models.dataitems.Scenario.prototype.getCreatedBy = function() {
    return this.createdBy;
};

sas.analytics.riskhp.models.dataitems.Scenario.prototype.getDataSourceName = function() {
    return this.dataSourceName;
};

sas.analytics.riskhp.models.dataitems.Scenario.prototype.setActivated = function(activated) {
    this.activated = activated;
    this._updateIcon();
};

sas.analytics.riskhp.models.dataitems.Scenario.prototype.isActivated = function() {
    return this.activated;
};

sas.analytics.riskhp.models.dataitems.Scenario.prototype.getLabel = function() {
    return this.label;
};

/**
 * Returns an array of string representations of property names for this object which will be used when comparing
 * two Data Source objects.  Only the properties listed here will be compared.
 * @returns {string[]}
 */
sas.analytics.riskhp.models.dataitems.Scenario.prototype.getValidPropertiesToCompare = function() {
    return ["id", "version", "horizons", "baselineId"];
};

sas.analytics.riskhp.models.dataitems.Scenario.prototype.generateLabel = function(json) {
    return json.scenarioVersion ? sas.analytics.riskhp.utils.HprUtil.getBundle().getText("Scenario.Label.NameAndVersion.fmt", [json.name, json.scenarioVersion]) : json.name;
};

sas.analytics.riskhp.models.dataitems.Scenario.prototype.generateTooltip = function(json) {
    if (json.workGroup) {
        return sas.analytics.riskhp.utils.HprUtil.getBundle().getText("DataPane.data.source.select.item.workGroup.tooltip.fmt", [json.workGroup]);
    }
    return null;
};

/* Scenario Type */
sas.analytics.riskhp.models.dataitems.Scenario.SCENARIO = "scenario";