// (c) 2017, SAS Institute Inc.

jQuery.sap.declare("sas.analytics.riskhp.portfolio.PortfolioDetailsModel");

jQuery.sap.require("sas.analytics.riskhp.models.DataSourceCacheManager");
jQuery.sap.require("sas.analytics.riskhp.models.HPRiskModelManager");
jQuery.sap.require("sas.analytics.riskhp.portfolio.ColumnDescriptor");
jQuery.sap.require("sas.analytics.riskhp.portfolio.PortfolioDetailsUtil");
jQuery.sap.require("sas.analytics.riskhp.settings.ApplicationSettings");
jQuery.sap.require("sas.analytics.riskhp.utils.HprAPI");
jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");

jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Horizon");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.RiskCube");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Statistic");


/**
 *
 *
 * @name sas.analytics.riskhp.portfolio.PortfolioDetailsModel
 */
sap.ui.base.Object.extend("sas.analytics.riskhp.portfolio.PortfolioDetailsModel", {
    metadata: {
        publicMethods: [
            "initializeModel",
            "setOptionGeneralOptionsSectionCollapsed",
            "setSortColumn",
            "addFreezeColumn",
            "removeFreezeColumn",
            "setSelectedColumnOption",
            "setSelectedInstrumentTypeNames",
            "setSelectedColumnIds",
            "getDataSourceId",
            "getDrillItemLabels",
            "getQueryURL",
            "getRequestData",
            "getOptions",
            "getOptionName",
            "setOptionName",
            "getOptionShowLabels",
            "setOptionShowLabels",
            "getRequiredColumnIds",
            "getInstrumentTypeNames",
            "getColumnDescriptor",
            'getColumnDescriptors',
            "getAvailableColumnIds",
            "getSortColumn",
            "getFreezeColumns",
            "getSelectedColumnOption",
            "getSelectedInstrumentTypeNames",
            "getSelectedColumnIds",
            "getTuples",
            "areHorizonsAbsolute",
            "toJSON",
            "areDataSourceIdsBeingReferenced",
            'removeDataSource',
            "replaceDataSource",
            "toOptionsJSON",
            "fromOptionsJSON",
            "updateOptions",
            'setFilters',
            'updateFilterSections',
            'getFilters',
            'isModelInvalid'
        ]
    },

    // pseudo-imports
    DataSourceCacheManager: sas.analytics.riskhp.models.DataSourceCacheManager,
    HPRiskModelManager: sas.analytics.riskhp.models.HPRiskModelManager,
    PortfolioDetailsUtil: sas.analytics.riskhp.portfolio.PortfolioDetailsUtil,
    HprAPI: sas.analytics.riskhp.utils.HprAPI,
    HprConstants: sas.analytics.riskhp.utils.HprConstants,
    HprUtil: sas.analytics.riskhp.utils.HprUtil,
    ApplicationSettings: sas.analytics.riskhp.settings.ApplicationSettings,


    constructor: function(id, contextData, isRestoration) {
        this.id = id;
        this.isNewModel = !(isRestoration === true);

        // Options properties
        this.options = {};
        this.options.id = this.id;
        this.options.name = null;
        this.options.nameChanged = false;
        this.options.showLabels = {};
        this.options.applyEnabled = false; // Flag for the Apply button on each Options pane; Disabled by default
        this.options.generalOptionsSectionCollapsed = false; // Open by default

        // selected column options
        this.selectedColumnOption = null;
        this.selectedInstrumentTypeNames = [];
        this.selectedColumnIds = [];
        this.sortColumn = null;
        this.freezeColumns = [];

        // filters: array of {collapsed, name (id?), type, values, include}
        this.filters = [];
        //this.filtersModified = false;

        // set the context data
        if (contextData) {
            this.requests = contextData.requests;

            this.dataSourceId = contextData.dataSourceId;
            this.type = contextData.type;
            this.drill = contextData.drill;
            this.displayHorizons = contextData.displayHorizons;
            this.tuples = contextData.tuples;

            if (isRestoration === true) {
                this._restoreModelData(contextData);
            }
            else {
                this._setContextData(contextData);
            }
        }

        // update the valid state of the model
        this.invalid = !!!contextData || contextData.invalid === true ? true : false;

        // capture of most recent request
        this.currentRequestData = null;
    },

    _setContextData: function(contextData) {
        // risk cube ID
        var riskCube = _.find(this.tuples[0], function(dataItem) {
            return dataItem.type === sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE;
        });
        this.riskCubeId = riskCube ? riskCube.id : null;

        // options
        var showLabelsObject = {};
        showLabelsObject.name = this.HprConstants.HPR_OPTIONS_PANE_SHOW_LABELS_NAME;
        // For the Portfolio Detail's Show Labels option, the value should come from the Crosstab that launched it
        showLabelsObject.value = contextData.showLabels;
        this.options.showLabels = showLabelsObject;

        // default filters from cube
        if (contextData.filters) {
            this.filters = contextData.filters;
        }

        // query data
        this._buildQueryData(contextData);
    },

    _restoreModelData: function(modelData) {
        // risk cube
        this.riskCubeId = modelData.riskCubeId;

        // column selections
        this.selectedColumnOption = modelData.selectedColumnOption;
        this.selectedInstrumentTypeNames = modelData.selectedInstrumentTypeNames;
        this.selectedColumnIds = modelData.selectedColumnIds;

        // column data
        this.sortColumn = modelData.sortColumn;
        this.freezeColumns = modelData.freezeColumns ? modelData.freezeColumns : [];

        // options
        this.fromOptionsJSON(modelData.options);

        // filters
        this.filters = modelData.filters;
        //this.filtersModified = modelData.filtersModified;

        // data query
        this._buildQueryData(modelData);

        this.invalid = modelData.invalid;
    },

    _buildQueryData: function(contextData) {
        var self = this;

        // query data
        self.queryData = {};
        self.queryData.requestData = {};
        self.queryData.requestData.name = self.id;
        self.queryData.requestData.version = 1;
        self.queryData.requestData.type = self.HprConstants.HPR_PORTFOLIO_DETAILS_QUERY_TYPE;
        self.queryData.requestData.orientation = "row";

        self.queryData.requestData.dataDefinitions = {};
        self.queryData.requestData.dataDefinitions.tuples = [];
        _.each(contextData.tuples, function(dataItems) {
            var tuple = {};
            tuple.dataItems = [];
            _.each(dataItems, function(dataItem) {
                tuple.dataItems.push({
                    id: dataItem.id,
                    type: dataItem.type
                });
            });
            self.queryData.requestData.dataDefinitions.tuples.push(tuple);
        });

        self.queryData.requestData.report = {};
        self.queryData.requestData.report.requests = [];
        self.queryData.requestData.report.requests.push({});
        //self.queryData.requestData.report.requests[0].key = contextData.requests[0].key;
        self.queryData.requestData.report.requests[0].key = [];
        self.queryData.requestData.report.sorts = [];
        self.queryData.requestData.report.options = [];
        self.queryData.requestData.report.filters = [];
    },

    /**
     * This method must be called before the model can be used to manage/access portfolio
     * columns and the associated data source. It is meant to be called once by the consumer
     * (see PortfolioDetailsViewer). If calls to functions such as getAvailableColumns occur
     * before this function has successfully resolved,
     *
     * @returns {Deferred}
     */
    initializeModel: function() {
        var self = this;
        var deferred = $.Deferred();

        // load the data source and column metadata
        self._loadDataSource(self.dataSourceId).done(
            function (dataSource) {
                self.dataSource = dataSource;
                self._setColumnMetadata(dataSource);

                // for new models, build initial filters and set selected columns
                if (self.isNewModel === true) {
                    // create the existing filters for the new model
                    self._setFiltersFromContextData(self);

                    // set the default columns as the selected columns
                    self.selectedColumnIds = dataSource.getDefaultInstrumentVariables();
                }

                deferred.resolve();
            }
        ).fail(
            function (jqxhr, textStatus, error) {
                self.invalidate();
                deferred.reject(jqxhr, textStatus, error);
            }
        );

        return deferred.promise();
    },

    toJSON: function() {
        return {
            name: this.id,
            type: this.type,
            dataSourceId: this.dataSourceId,
            riskCubeId: this.riskCubeId,
            tuples: this._dehydrateTuples(this.tuples),
            drill: this.drill,
            displayHorizons: this.displayHorizons,

            requests: this.requests,

            selectedColumnOption: this.selectedColumnOption,
            selectedInstrumentTypeNames: this.selectedInstrumentTypeNames,
            selectedColumnIds: this.selectedColumnIds,

            sortColumn: this.sortColumn,
            freezeColumns: this.freezeColumns,

            options: this.toOptionsJSON(),

            filters: this._toFiltersJSON(),
            //filtersModified: this.filtersModified

            invalid: this.invalid
        };
    },

    getDataSourceId: function() {
        return this.dataSourceId;
    },

    getRiskCubeId: function() {
        return this.riskCubeId;
    },

    getDrillItemLabels: function() {
        return this.drill;
    },

    areHorizonsAbsolute: function() {
        return this.displayHorizons !== HprConstants.DISPLAY_HORIZONS_RELATIVE_VALUE
    },

    isValid: function() {
        return !this.invalid;
    },

    invalidate: function(valid) {
        this.invalid = true;
    },

    /**
     * Creates the string that represents the crosstab drill path from which this portfolio viewer was launched.
     * @returns {String}
     * @private
     */
    createPathString: function() {
        var self = this;
        //if (self.areFiltersModified() === true) {
        //    return self.HprUtil.getBundle().getText("HPRisk.panel.pathSeeFilters.txt");
        //}

        if (!self.isPathFilteringApplied()) {
            return "";
        }

        var classVariables = self.dataSource.getDataItemsForType(sas.analytics.riskhp.models.dataitems.Classvar.CLASSVAR);
        var keys = self.requests[0].key;

        var pathString = "";
        var appendToPathString = function (itemLabel) {
            if (pathString.length > 0) {
                pathString += " > ";
            }
            pathString += "[" + itemLabel + "]";
        };

        var classVariable = null;
        if (keys.length === 0) {
            classVariable = classVariables[0];
            appendToPathString(classVariable.label + ":" + self.HprUtil.getBundle().getText("HPRisk.plot.path.value.all.txt"));
        } else {
            _.each(keys, function (key) {
                classVariable = _.find(classVariables, function(classVar) {
                    return classVar.id === key.id;
                });

                if (classVariable) {
                    var valueToUse = key.value;
                    if (classVariable.getDataType() !== HprConstants.HPR_DATA_TYPE_STRING) {
                        valueToUse = sas.ltjs.sasicu.util.SASFormatterUtil.format(Number(valueToUse), classVariable.getFormat());
                    }
                    appendToPathString(classVariable.label + ":" + valueToUse);
                }
            });
        }
        return pathString ? this.HprUtil.getBundle().getText("HPRisk.panel.genericPath.fmt", [pathString]) : null;
    },

    /**
     * A method which checks to see if the existing content object references any of the data source ids by looking
     * at the "dataSourceId" property.
     * @param { string } currentContentData - The JSON object representing the current state of a panel
     * @param { [string] } dataSourceIds - A list of data source ids
     */
    areDataSourceIdsBeingReferenced: function(currentContentData, dataSourceIds) {
        return _.contains(dataSourceIds, currentContentData.dataSourceId);
    },

    /**
     * pseudo static method that takes a list of Data Source ids which are being removed from an Exploration and determines
     * if this model references one of them; If it does a Zero State object is returned
     * @param { string } currentContentData - The JSON object representing the current state of a panel
     * @param { [string] } dataSourceIdsBeingRemoved - A list of data source ids that are being removed
     * @returns {*}
     */
    removeDataSource: function(currentContentData, dataSourceIdsBeingRemoved) {
        var deferred = $.Deferred();

        // Check to make sure the current data source id is contained in the list of data source ids being replaced,
        // otherwise there is nothing to do so return the provided currentContentData.
        var currentDataSourceId = currentContentData.dataSourceId;
        if (!_.contains(dataSourceIdsBeingRemoved, currentDataSourceId)) {
            deferred.resolve(currentContentData);   // this model is unaffected as it doesn't reference the removed data sources
        } else {
            // Define the basic model to signify a ZeroState PortfolioDetails Viewer
            var updatedModelJSON = {
                name: currentContentData.name,
                dataSourceId: "",
                options: {
                    // So the name of the ZeroState PortfolioDetails Viewer will be the same
                    name: currentContentData.options.name
                },
                type: currentContentData.type,
                invalid: true
            };

            deferred.resolve(updatedModelJSON);
        }

        return deferred.promise();
    },

    /**
     * pseudo static method that takes an existing content object and changes it in a way to represent a new data source.
     * This function should select an equivalent data source id from the list of newDataSourceIds for the data source
     * id the currentContentData is representing.
     *
     * This method must return a jQuery.Deferred object that behaves as follows:
     * - Returns the JSON to represent the panel's new state regardless of whether the panel should change because of the new data source
     * - ALWAYS resolve successfully.
     * - May make subsequent/wrapped ajax calls but when it returns all processing should be complete.
     * - May not change the state of the model instance or attempt to interact with the UI.
     *
     * @param { string } currentContentData - The JSON object representing the current state of a panel
     * @param { [string] } dataSourceIdsBeingReplaced - A list of data source ids that are being replaced
     * @param { [string] } newDataSourceIds - A list of replacement data source ids
     * @returns {*}
     */
    replaceDataSource: function(currentContentData, dataSourceIdsBeingReplaced, newDataSourceIds) {
        var self = this;
        var deferred = $.Deferred();

        // Check to make sure the current data source id is contained in the list of data source ids being replaced,
        // otherwise there is nothing to do so return the provided currentContentData.
        var currentDataSourceId = currentContentData.dataSourceId;
        if (!_.contains(dataSourceIdsBeingReplaced, currentDataSourceId)) {
            deferred.resolve(currentContentData);   // this model is unaffected as it doesn't reference the replaced data source
        }
        else if (currentContentData.invalid === true) {
            deferred.resolve(currentContentData);   // already invalid, so nothing we can do
        }
        else {
            // Store any deferreds from requests needed to update for the new data source here. After all items are
            // processed, this set of deferreds will be passed into a when function, whereupon their completion, the
            // master deferred will be resolved (if we get that far, of course).
            var pendingRequests = [];

            // Get a compatible data source id from the list of newDataSourceIds.  If one is not found then there is
            // nothing to do so return the provided currentContentData
            var replacementDataSourceId = self.HprUtil.getCompatibleDataSourceId(currentContentData.dataSourceId, newDataSourceIds);

            // define the basic model
            var updatedModelJSON = {
                name: currentContentData.name,
                dataSourceId: replacementDataSourceId,
                displayHorizons: currentContentData.displayHorizons,
                drill: currentContentData.drill,
                options: currentContentData.options,
                type: currentContentData.type,
                requests: currentContentData.requests,

                // other defaults (may be replaced by subsequent processing)
                selectedColumnOption: self.HprConstants.HPR_PORTFOLIO_COLS_ALL_OPTION_KEY,
                selectedInstrumentTypeNames: [],
                selectedColumnIds: [],
                freezeColumns: [],
                filters: []
            };

            self._loadDataSource(replacementDataSourceId).done(
                function (newDataSource) {
                    // If going from a side-by-side data source to a single cube source, replace the current risk cube
                    // with the new risk cube. However, if going from a single cube data source to a side-by-side, or
                    // from one side-by-side to another, make sure the current risk cube exists in the new data source.
                    if (newDataSource.isSideBySide() === true) {
                        var newRiskCube = newDataSource.getDataItemById(currentContentData.riskCubeId);
                        if (!newRiskCube) {
                            updatedModelJSON.invalid = true;
                            deferred.resolve(updatedModelJSON);
                            return;
                        }
                        else {
                            updatedModelJSON.riskCubeId = currentContentData.riskCubeId;
                        }
                    }
                    else {
                        updatedModelJSON.riskCubeId = newDataSource.getAllRiskCubes()[0].id;
                    }

                    // now that we have the risk cube sorted out, update the title if the user has not yet changed it
                    if (updatedModelJSON.options.nameChanged !== true) {
                        updatedModelJSON.options.name = self.PortfolioDetailsUtil.generateDefaultPortfolioTitle(newDataSource, updatedModelJSON.riskCubeId);
                    }

                    // compare tuples
                    var remainingTuples = [];
                    _.each(currentContentData.tuples, function(tupleDataItems) {
                        var remainingTupleDataItems = _.filter(tupleDataItems, function(tupleDataItem) {
                            // make sure for risk cubes we are in sync with the updated model risk cube
                            if (tupleDataItem.type === sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE) {
                                tupleDataItem.id = updatedModelJSON.riskCubeId;
                                return true;
                            }
                            return !!newDataSource.getDataItemById(tupleDataItem.id);
                        });
                        if (remainingTupleDataItems && remainingTupleDataItems.length > 0) {
                            remainingTuples.push(remainingTupleDataItems);
                        }
                    });
                    if (remainingTuples.length === 0) {
                        updatedModelJSON.invalid = true;
                        deferred.resolve(updatedModelJSON);
                        return;
                    }
                    updatedModelJSON.tuples = remainingTuples;

                    // build column descriptor map from the new data source
                    var columnDescriptorMap = {};
                    _.each(newDataSource.getInstrumentVariables(), function(portfolioColumn) {
                        columnDescriptorMap[portfolioColumn.id] = new sas.analytics.riskhp.portfolio.ColumnDescriptor(portfolioColumn);
                    });

                    // compare columns
                    var remainingColumns = _.filter(currentContentData.selectedColumnIds, function(columnId) {
                        return !!columnDescriptorMap[columnId];
                    });
                    // if no remaining columns, invalidate the model
                    if (remainingColumns.length === 0) {
                        updatedModelJSON.invalid = true;
                        deferred.resolve(updatedModelJSON);
                        return;
                    }
                    else if (remainingColumns.length === currentContentData.selectedColumnIds.length) {
                        // only if all columns match will we attempt to preserve the option and instrument selections
                        if (currentContentData.selectedInstrumentTypeNames && currentContentData.selectedInstrumentTypeNames.length > 0) {
                            var instrumentTypeMap = _.indexBy(newDataSource.getInstrumentTypes(), "name");
                            var remainingInstrumentTypeNames = _.filter(currentContentData.selectedInstrumentTypeNames, function(instrumentTypeName) {
                                return !!instrumentTypeMap[instrumentTypeName];
                            });
                            if (remainingInstrumentTypeNames.length === 0) {
                                updatedModelJSON.invalid = true;
                                deferred.resolve(updatedModelJSON);
                                return;
                            }

                            // all instrument type selections accounted for, so preserve option selection. Otherwise default to all.
                            if (remainingInstrumentTypeNames === currentContentData.selectedInstrumentTypeNames.length) {
                                updatedModelJSON.selectedColumnOption = currentContentData.selectedColumnOption;
                                updatedModelJSON.selectedInstrumentTypeNames = remainingInstrumentTypeNames;
                            }
                        }
                        else {
                            updatedModelJSON.selectedColumnOption = currentContentData.selectedColumnOption;
                        }
                    }
                    updatedModelJSON.selectedColumnIds = remainingColumns;

                    // compare freeze columns (if none left, consider this non-fatal)
                    var remainingFreezeColumns = _.filter(currentContentData.freezeColumns, function(columnId) {
                        return !!columnDescriptorMap[columnId];
                    });
                    updatedModelJSON.freezeColumns = remainingFreezeColumns;

                    // check sort column
                    if (currentContentData.sortColumn && !!columnDescriptorMap[currentContentData.sortColumn]) {
                        updatedModelJSON.sortColumn = currentContentData.sortColumn;
                    }

                    // validate the filters
                    var validateFiltersDeferred = self.HprUtil.validateFilters(currentContentData.filters, newDataSource, columnDescriptorMap).done(
                        function(filters) {
                            updatedModelJSON.filters = filters;
                        }
                    );
                    pendingRequests.push(validateFiltersDeferred);

                    // once all the requests have finished, we can return
                    $.when.apply(self, pendingRequests).always(
                        function() {
                            deferred.resolve(updatedModelJSON);
                        }
                    );
                }
            ).fail(
                function (jqxhr, textStatus, error) {
                    updatedModelJSON.invalid = true;
                    deferred.resolve(updatedModelJSON);
                }
            );
        }

        return deferred.promise();
    },


    //*****************************
    // Query
    //*****************************

    getRequestKeys: function() {
        return this.queryData.requestData.report.requests[0].key;
    },

    getQueryURL: function() {
        if (!this.queryData.url) {
            this.queryData.url = this._buildQueryURL(this.dataSourceId);
        }
        return this.queryData.url;
    },

    getRequestData: function(start, limit, sortBy) {
        var self = this;

        // clone the initial request data
        var requestData = $.extend(true, {}, this.queryData.requestData);

        // get the data source information
        var dataSourceParams = self.DataSourceCacheManager.getDataSourceParam(self.getDataSourceId());
        requestData.dataSource = {
            id: dataSourceParams.realDataSourceId,
            application: dataSourceParams.application,
            workGroup: dataSourceParams.workGroup,
            relPath: dataSourceParams.relPath,
            type: dataSourceParams.type
        };

        requestData.dataDefinitions.instrumentVariables = this.getSelectedColumnIds();

        requestData.report.requests[0].startRow = start;
        requestData.report.requests[0].maxRows = limit;

        if (sortBy) {
            requestData.report.sorts = [this._translateSortBy(sortBy)];
        }

        // Any options that need to be passed down on query would go here (i.e. ndetails, alpha, nrows, etc.)
        // 'name', 'applyEnabled', 'generalOptionsSectionCollapsed' do not go along with query though
        requestData.report.options = this._buildQueryOptions(); // See HprChartModel & CrosstabContext

        // filters
        requestData.report.filters = this._buildQueryFilterJSON();

        // capture this request
        this.currentRequestData = requestData;

        return requestData;
    },

    getExportRequestData: function() {
        // if we have not yet created a request, nothing to export
        if (!this.currentRequestData) {
            return {};
        }

        // clone the most recent request and add export information to it
        var self = this;
        var requestData = $.extend(true, {}, self.currentRequestData);

        // update the request with the current page size
        self.PortfolioDetailsUtil.updateRequestPageSize(requestData, self.ApplicationSettings.getPortfolioDetailsPageSize());

        // create the export object
        requestData.export = {
            name: self.HprUtil.getBundle().getText("Hpr.Export.WorksheetName.label.fmt", [1]),  // always sheet 1 for now
            type: self.HprConstants.HPR_EXPORT_FORMAT_EXCEL,
            headers: []
        };

        // add column data with format and label to the export object
        var columnDescriptor,
            classVariableDataItem,
            instrumentVariables = [];
        _.each(this.getSelectedColumnIds(), function(selectedColumnId) {
            columnDescriptor = self.getColumnDescriptor(selectedColumnId);
            classVariableDataItem = columnDescriptor.classVariableId ? self.dataSource.getDataItemById(columnDescriptor.classVariableId) : null;
            instrumentVariables.push({
                id: columnDescriptor.id,
                format: columnDescriptor.format,
                label: classVariableDataItem ? classVariableDataItem.getDisplayLabel(self.options.showLabels.value) : columnDescriptor.label ? columnDescriptor.label : columnDescriptor.name
            });
        });
        requestData.export.instrumentVariables = instrumentVariables;

        // add the data source name
        requestData.export.headers.push({
            key: self.HprConstants.HPR_EXPORT_TABLE_NAME_KEY,
            name: self.HprUtil.getBundle().getText("Hpr.Export.TableName.label.txt"),
            value: self.options.name
        });

        // add the table type
        requestData.export.headers.push({
            key: self.HprConstants.HPR_EXPORT_TABLE_TYPE_KEY,
            name: self.HprUtil.getBundle().getText("Hpr.Export.TableType.label.txt"),
            value: self.HprUtil.getBundle().getText("Hpr.Export.TableType.Portfolio.label.txt"),
        });

        // add the data source name
        requestData.export.headers.push({
            key: self.HprConstants.HPR_EXPORT_DATA_SOURCE_NAME_KEY,
            name: self.HprUtil.getBundle().getText("Hpr.Export.DataSourceName.label.txt"),
            value: self.dataSource.getName()
        });

        // add the creation date
        requestData.export.headers.push({
            key: self.HprConstants.HPR_EXPORT_DATASOURCE_CREATION_DATE_KEY,
            name: self.HprUtil.getBundle().getText("Hpr.Export.DataSourceCreationDate.label.txt"),
            value: self.HprUtil.formatDateTime(self.dataSource.getCreatedDate())
        });

        requestData.export.headers.push({
            key: HprConstants.HPR_EXPORT_EXPORT_CREATION_DATE_KEY,
            name: self.HprUtil.getBundle().getText("Hpr.Export.ExportCreationDate.label.txt"),
            value: self.HprUtil.formatDateTime(Date.now())
        });

        // add the path value
        requestData.export.headers.push({
            key: self.HprConstants.HPR_EXPORT_PORTFOLIO_DRILL_KEY,
            name: self.HprUtil.getBundle().getText("Hpr.Export.PortfolioDrill.label.txt"),
            value: self.createPathString()
        });

        // add the risk cube if this is a side-by-side
        if (self.dataSource.isSideBySide()) {
            var riskCubeDataItem = self.dataSource.getDataItemById(self.riskCubeId);
            requestData.export.headers.push({
                key: self.HprConstants.HPR_EXPORT_RISK_CUBE_KEY,
                name: self.HprUtil.getBundle().getText("Hpr.Export.RiskCube.label.txt"),
                value: riskCubeDataItem.getDisplayLabel(self.options.showLabels.value)
            });
        }

        // This entry must be last.
        requestData.export.headers.push({
            key: HprConstants.HPR_EXPORT_TABLE_DATA_KEY,
            name: self.HprUtil.getBundle().getText("Hpr.Export.TableData.label.txt"),
            value: ""
        });

        return requestData;
    },

    getTuples: function() {
        if (this.tuples && this.tuples.length > 0) {
            if (this.dataSource) {
                this.tuples = this._hydrateTuples(this.tuples);
            }
            return this.tuples;
        }
        return [];
    },


    //*****************************
    // Columns
    //*****************************

    _setColumnMetadata: function(dataSource) {
        var self = this;

        // parse the instrument types
        self.instrumentTypes = dataSource.getInstrumentTypes();
        self.instrumentTypesMap = _.indexBy(self.instrumentTypes, "name");
        self.instrumentTypeNames = _.pluck(self.instrumentTypes, "name");

        // set the instrument type labels for display
        _.each(self.instrumentTypes, function(instrumentType) {
            if (instrumentType.name === self.HprConstants.HPR_PORTFOLIO_INSTTYPE_POSITION_KEY) {
                instrumentType.label = self.HprUtil.getBundle().getText("HPRisk.PortfolioDetails.ColumnSelector.Positions.txt");
            }
            else if (instrumentType.name === self.HprConstants.HPR_PORTFOLIO_INSTTYPE_QUADRATIC_KEY) {
                instrumentType.label = self.HprUtil.getBundle().getText("HPRisk.PortfolioDetails.ColumnSelector.Quadratics.txt");
            }
            else if (!instrumentType.label) {
                instrumentType.label = instrumentType.name;
            }
        });

        // get the required (common) column set
        self.requiredColumnIds = dataSource.getRequiredInstrumentVariables();

        // get the default column set
        self.defaultColumnIds = dataSource.getDefaultInstrumentVariables();

        // build a map of column descriptors
        self.columnDescriptorMap = {};
        _.each(dataSource.getInstrumentVariables(), function(portfolioColumn) {
            self.columnDescriptorMap[portfolioColumn.id] = new sas.analytics.riskhp.portfolio.ColumnDescriptor(portfolioColumn);
        });
    },

    getAvailableColumnIds: function(selectedOptionKey, selectedInstrumentTypeNames) {
        var self = this;

        // clone the required columns array as a starting point
        var tableColumnSet = $.extend(true, [], self.getRequiredColumnIds());

        // collection of instrument types to add columns for (if any)
        var columnInstrumentTypes = [];

        // option not passed in, then get it from the saved selections
        if (!selectedOptionKey) {
            selectedOptionKey = self.selectedColumnOption;
            selectedInstrumentTypeNames = self.selectedInstrumentTypeNames;
        }

        // determine applicable instrument types for columns--for now, default to all columns if no option is specified
        if (!selectedOptionKey || selectedOptionKey === self.HprConstants.HPR_PORTFOLIO_COLS_ALL_OPTION_KEY) {
            columnInstrumentTypes = $.extend(true, [], self.instrumentTypes);
        }
        else if (selectedOptionKey === self.HprConstants.HPR_PORTFOLIO_COLS_INST_TYPE_OPTION_KEY) {
            _.each(selectedInstrumentTypeNames, function (instrumentTypeName) {
                columnInstrumentTypes.push(self.instrumentTypesMap[instrumentTypeName]);
            });
        }

        // harvest the column names from the applicable instrument types
        _.each(columnInstrumentTypes, function(instrumentType) {
            _.each(instrumentType.instrumentVariables, function (columnId) {
                if (_.indexOf(tableColumnSet, columnId) === -1) {
                    tableColumnSet.push(columnId);
                }
            });
        });

        // sort the table column set by selection order and return that
        var availableColumnIds = _.intersection(self.getSelectedColumnIds(), tableColumnSet);
        _.each(tableColumnSet, function(columnId) {
            if (_.indexOf(availableColumnIds, columnId) === -1) {
                availableColumnIds.push(columnId);
            }
        });
        return availableColumnIds;
    },

    setSortColumn: function(columnId, sortOrder) {
        // if no sort order, then it has been removed, so clear the metadata
        if (!sortOrder) {
            this.sortColumn = null;
        }
        else {
            this.sortColumn = {
                columnId: columnId,
                sortOrder: sortOrder
            };
        }
    },

    addFreezeColumn: function(columnId) {
        this.freezeColumns.push(columnId);
    },

    removeFreezeColumn: function(columnId) {
        _.reject(this.freezeColumns, function(freezeColumnName) {
            return freezeColumnName === columnId;
        });
    },

    setSelectedColumnOption: function(columnOption) {
        this.selectedColumnOption = columnOption;
        this.availableColumnIds = null;
    },

    setSelectedInstrumentTypeNames: function(instrumentTypeNames) {
        this.selectedInstrumentTypeNames = $.extend(true, [], instrumentTypeNames);
        this.availableColumnIds = null;
    },

    setSelectedColumnIds: function(columnIds) {
        this.selectedColumnIds = $.extend([], columnIds);
        this.availableColumnIds = null;
    },

    getInstrumentTypeNames: function() {
        return this.instrumentTypeNames;
    },

    getInstrumentType: function(instrumentTypeName) {
        return this.instrumentTypesMap[instrumentTypeName];
    },

    getRequiredColumnIds: function() {
        return this.requiredColumnIds;
    },

    getColumnDescriptors: function() {
        return _.values(this.columnDescriptorMap);
    },

    getColumnDescriptor: function(columnId) {
        return this.columnDescriptorMap[columnId];
    },

    getSelectedColumnOption: function() {
        if (!this.selectedColumnOption) {
            return this.HprConstants.HPR_PORTFOLIO_COLS_ALL_OPTION_KEY;
        }
        return this.selectedColumnOption;
    },

    getSelectedInstrumentTypeNames: function() {
        return this.selectedInstrumentTypeNames;
    },

    getSelectedColumnIds: function() {
        return $.extend([], this.selectedColumnIds);
    },

    getSortColumn: function() {
        return this.sortColumn;
    },

    getFreezeColumns: function() {
        return this.freezeColumns;
    },


    //*****************************
    // Options
    //*****************************

    setOptionGeneralOptionsSectionCollapsed: function(generalOptionsSectionCollapsed) {
        this.options.generalOptionsSectionCollapsed = generalOptionsSectionCollapsed;
    },

    getOptions: function () {
        return $.extend(true, {}, this.options);
    },

    getOptionName: function() {
        return this.options.name;
    },

    setOptionName: function(name) {
        this.options.name = name;
    },

    getOptionsNameChanged: function() {
        return this.options.nameChanged;
    },

    getOptionShowLabels: function() {
        return this.options.showLabels;
    },

    setOptionShowLabels: function(showLabels) {
        this.options.showLabels = showLabels;
    },

    toOptionsJSON: function() {
        return {
            id: this.options.id,
            name : this.options.name,
            nameChanged : this.options.nameChanged,
            showLabels : this.options.showLabels,
            applyEnabled : this.options.applyEnabled,
            generalOptionsSectionCollapsed : this.options.generalOptionsSectionCollapsed
        };
    },

    fromOptionsJSON: function(opts) {
        this.options.id = opts.id;
        this.options.name = opts.name;
        this.options.nameChanged = opts.nameChanged;
        this.options.showLabels = opts.showLabels;
        this.options.applyEnabled = opts.applyEnabled;
        this.options.generalOptionsSectionCollapsed = opts.generalOptionsSectionCollapsed;
    },

    updateOptions: function(optionsData) {
        // check if the name changed
        if (this.options.name !== optionsData.name) {
            this.options.nameChanged = true;
        }

        // Update the option properties with data passed in
        this.options.name = optionsData.name;
        this.options.showLabels = optionsData.showLabels;
        this.options.applyEnabled = optionsData.applyEnabled;
        this.options.generalOptionsSectionCollapsed = optionsData.generalOptionsSectionCollapsed;
    },


    //*****************************
    // Filters
    //*****************************

    setFilters: function(filters) {
        this.filters = this.HprUtil.hprClone(filters);
        //this.filtersModified = true;
    },

    updateFilterSections: function(indexedFilterSectionData) {
        var self = this;
        var filterSection;
        _.each(self.filters, function(filter) {
            filterSection = indexedFilterSectionData[filter.id];
            if (filterSection) {
                filter.collapsed = filterSection.collapsed;
            }
        });
    },

    getFilters: function() {
        return this.HprUtil.hprClone(this.filters);
    },

    //areFiltersModified: function() {
    //    return this.filtersModified;
    //},

    _toFiltersJSON: function() {
        return this.filters;
    },

    _fromFiltersJSON: function(filterJSON) {
        this.filters = filterJSON;
    },

    _buildQueryFilterJSON: function() {
        var self = this;
        var filters = [];
        
        var whereClause = "";
        var columnDescriptor = {};

        _.each(self.getFilters(), function(filter){
            if (filter.type === this.HprConstants.CLASS_VARIABLE_DATA_ITEM_TYPE) {
                // push filter object with just the required fields
                filters.push({
                    id: filter.id,
                    includes: filter.includes,
                    type: filter.type,
                    values: filter.values
                });
            } else {  // portfolio column filters get put in whereClause
                columnDescriptor = self.getColumnDescriptor(filter.id);
                if (columnDescriptor && (columnDescriptor.getClassVariableId() || columnDescriptor.getDataType() === self.HprConstants.HPR_DATA_TYPE_STRING)) {
                    var operator = filter.includes === true ? " in " : " notin ";
                    var valuesString = "";
                    _.each(filter.values, function (value) {
                        if (valuesString.length > 0) {
                            valuesString += ",";
                        }
                        valuesString += columnDescriptor.dataType === "string" ? "'" + value + "'" : value;
                    });

                    whereClause += (whereClause.length > 0 ? " and " : "") + (columnDescriptor.getName() + operator + "(" + valuesString + ")");
                }
                else {
                    if (!isNaN(filter.minValue) && !isNaN(filter.maxValue)) {
                        whereClause += (whereClause.length > 0 ? " and " : "") + "(" + (columnDescriptor.getName() + " >= " + filter.minValue + " and " + columnDescriptor.getName() + " <= " + filter.maxValue + (filter.includeMissingValues ? (" or " + columnDescriptor.getName() + " = .") : "")) + ")";
                    }
                }
            }
        });

        if (whereClause.length > 0) {
            filters.push({
                type: "whereClause",
                where: whereClause
            });
        }
        
        return filters;
    },


    /**
     * When launching from a crosstab, the filters for class variables in the crosstab are based
     * on the cell the user launched portfolio details from. Therefore, we must create filters
     * from the request key, and ignore filters for the same class variable(s) applied on the
     * crosstab. Filters applied to class variables not in the crosstab will be added to the filter
     * collection as-is.
     *
     * @param contextData
     * @private
     */
    _setFiltersFromContextData: function(contextData) {
        var self = this;

        // if context data doesn't contain requests, no filters to resolve
        if (!contextData.requests) {
            return;
        }

        // create the inherited filters from the request key
        var filters = [];
        var key = contextData.requests[0].key;
        var keyItemIds = [];
        _.each(key, function(keyItem) {
            // log the ids we've created filters from
            keyItemIds.push(keyItem.id);

            // create the filter
            filters.push({
                id: keyItem.id,
                type: self.HprConstants.CLASS_VARIABLE_DATA_ITEM_TYPE,
                includes: true,
                collapsed: false,
                values: [self._convertKeyItemValue(keyItem)]
            });
        });

        // add the inherited filters for class variables not in the crosstab
        _.each(contextData.filters, function(filter) {
            if (keyItemIds.indexOf(filter.id) === -1) {
                filters.push(filter);
            }
        });

        self.filters = filters;
    },

    /**
     * Returns true if the filters collection still contains any and all filters created from
     * the context request key that indicates the crosstab cell from which the user launched
     * the portfolio details table.
     */
    isPathFilteringApplied: function() {
        var self = this;
        var key = self.requests[0].key;

        // optimization--if the filter set contains fewer items than the key, then the filters cannot
        // contain all filters produced by the key.
        if (self.filters.length < key.length) {
            return false;
        }

        // otherwise, check each key and ensure that there is a corresponding filter created for it,
        // and that the filter is an includes filter (all key-based filters are includes)
        var matchedKeyItems = _.filter(key, function(keyItem) {
            return !!_.find(self.filters, function(filter) {
                return filter.id === keyItem.id && filter.values.length === 1 && filter.values[0] === self._convertKeyItemValue(keyItem) && filter.includes === true;
            });
        });
        return matchedKeyItems.length === key.length;
    },

    /**
     * Returns true if the filters have been modified such that the path filters are no longer
     * fully represented in the filter collection, or if the path filters are represented but
     * additional filtering is also present.
     */
    isAdditionalFilteringApplied: function() {
        return this.filters.length > 0 && !this.isPathFilteringApplied() || this.requests[0].key.length < this.filters.length;
    },

    /**
     * S1390667 -- ensure that numeric/date values are represented consistently. For numeric
     * and date data item types, convert the string to a number and back to a string to eliminate
     * space padding added by the server.
     *
     * Future consideration--should we just trim this?
     *
     * @param keyItem
     * @returns {*}
     * @private
     */
    _convertKeyItemValue: function(keyItem) {
        var parsedFilterValue = keyItem.value;
        var filterDataItem = this.dataSource.getDataItemById(keyItem.id);
        if (filterDataItem) {
            var filterItemDataType = filterDataItem.getDataType();
            if (filterItemDataType === this.HprConstants.HPR_DATA_TYPE_NUMBER || filterItemDataType === this.HprConstants.HPR_DATA_TYPE_DATE) {
                parsedFilterValue = Number(keyItem.value).toString();
            }
        }
        return parsedFilterValue;
    },


    //*****************************
    // Utils
    //*****************************

    _hydrateTuples: function(tuples) {
        if (tuples && tuples.length > 0) {
            if (!_.isFunction(tuples[0].getId) && this.dataSource) {
                return this.dataSource.hydrateTuples(tuples);
            }
        }
        return tuples;
    },

    _dehydrateTuples: function(tuples) {
        if (tuples && tuples.length > 0) {
            if(_.isFunction(tuples[0].getId) && this.dataSource) {
                return this.dataSource.dehydrateTuples(tuples);
            }
        }
        return tuples;
    },

    _loadDataSource: function(dataSourceId) {
        var self = this;
        var deferred = $.Deferred();

        // load the data source
        self.DataSourceCacheManager.getDataSource(dataSourceId).done(
            function (dataSource) {
                deferred.resolve(dataSource);
            }
        ).fail(
            function (jqxhr, textStatus, error) {
                deferred.reject(jqxhr, textStatus, error);
            }
        );

        return deferred.promise();
    },

    _buildQueryURL: function(dataSourceId) {
        var url = this.HprConstants.HPR_PORTFOLIO_DETAILS_QUERY_URL + "?" + this.HprAPI.GUID_PARAM_STRING;
        // S1439890: Encode the data source id to guard against special characters
        url = url.replace(this.HprConstants.HPR_CUBE_ID_TOKEN, encodeURIComponent(dataSourceId));
        return url;
    },

    _buildQueryOptions: function() {
        var queryOptions = [];

        // Create an array of options that should be sent on query
        // NOTE: NDETAILS is stored on Preferences dialog and pulled from AppSettings for HTML5 UI
        queryOptions.push({"name": "NDETAILS", "value": this.ApplicationSettings.getPortfolioDetailsPageSize()});

        return queryOptions;
    },

    _translateSortBy: function(sortBy) {
        var sortByProperties = sortBy.split(":");
        var columnDescriptor = this.columnDescriptorMap[sortByProperties[0]];

        var sortObject = {};
        sortObject.tuple = {};
        sortObject.tuple.dataItems = [
            {
                type: columnDescriptor.type,
                id: sortByProperties[0]
            }
        ];
        sortObject.direction = sortByProperties[1];
        sortObject.type = "value";

        return sortObject;
    }
});