// (c) 2017, SAS Institute Inc.

jQuery.sap.declare("sas.analytics.riskhp.portfolio.PortfolioDetailsViewer");

jQuery.sap.require("sas.hc.ui.commons.dualselector.DualSelector");
jQuery.sap.require("sas.hc.ui.layout.HorizontalLayout");
jQuery.sap.require("sas.hc.ui.layout.VerticalLayout");
jQuery.sap.require("sas.hc.ui.model.PageableJSONModel");
jQuery.sap.require("sas.hc.x.WideTable");

jQuery.sap.require("sas.ltjs.sasicu.util.SASFormatterUtil");

jQuery.sap.require("sas.hc.m.Dialog");
jQuery.sap.require("sas.hc.m.Heading");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.List");
jQuery.sap.require("sas.hc.m.Menu");
jQuery.sap.require("sas.hc.m.MenuItem");
jQuery.sap.require("sas.hc.m.RadioButton");
jQuery.sap.require("sas.hc.m.RadioButtonGroup");
jQuery.sap.require("sas.hc.m.SearchField");
jQuery.sap.require("sas.hc.m.StandardListItem");
jQuery.sap.require("sas.hc.m.ResponsiveToolbar");
jQuery.sap.require("sas.hc.m.ToolbarSeparator");
jQuery.sap.require("sas.hc.m.ToolbarSpacer");

jQuery.sap.require("sas.analytics.riskhp.settings.ApplicationSettings");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Classvar");
jQuery.sap.require("sas.analytics.riskhp.portfolio.PortfolioDetailsColumn");
jQuery.sap.require("sas.analytics.riskhp.portfolio.PortfolioDetailsModel");
jQuery.sap.require("sas.analytics.riskhp.portfolio.PortfolioDetailsColumnMenu");
jQuery.sap.require("sas.analytics.riskhp.portfolio.PortfolioDetailsColumnSelectionDialog");
jQuery.sap.require("sas.analytics.riskhp.utils.HprAPI");
jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");

jQuery.sap.require("sas.risk.common.layout.BarLayout");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.risk.common.panelsview.DivWrapper");
jQuery.sap.require("sas.risk.common.utils.CommonUtil");
jQuery.sap.require("sas.risk.common.widgets.RiskColumnText");
jQuery.sap.require("sas.risk.common.widgets.ZeroStateLayout");

jQuery.sap.require("sas.ltjs.sasicu.util.CalendarUtil");
jQuery.sap.require("sas.ltjs.sasicu.util.SASFormatterUtil");

/**
 * @name sas.analytics.riskhp.portfolio.PortfolioDetailsViewer
 */
sap.ui.core.Control.extend("sas.analytics.riskhp.portfolio.PortfolioDetailsViewer", {

    metadata: {
        properties: {
            width: {type: "sap.ui.core.CSSSize", defaultValue: null},
            height: {type: "sap.ui.core.CSSSize", defaultValue: null},
            portfolioModel: {type: "sas.analytics.riskhp.portfolio.PortfolioDetailsModel"}
        },
        events: {
            dirtyChange: {}
        }
    },

    ApplicationSettings: sas.analytics.riskhp.settings.ApplicationSettings,
    CommonUtil: sas.risk.common.utils.CommonUtil,
    HprAPI: sas.analytics.riskhp.utils.HprAPI,
    HprConstants: sas.analytics.riskhp.utils.HprConstants,
    HPRiskModelManager: sas.analytics.riskhp.models.HPRiskModelManager,
    HprUtil: sas.analytics.riskhp.utils.HprUtil,
    PortfolioDetailsUtil: sas.analytics.riskhp.portfolio.PortfolioDetailsUtil,
    CLASSVAR: sas.analytics.riskhp.models.dataitems.Classvar.CLASSVAR,

    wrapper: null,
    table: null,
    tableColumnSet: [],
    tableModel: null,
    pageSizeChangeDetected: false,

    renderer: function(rm, control) {
        if(!control.getVisible()) return;

        rm.write("<div");
        rm.writeControlData(control);
        if(control.getWidth()) {
            rm.addStyle("width", control.getWidth());
        }
        if(control.getHeight()) {
            rm.addStyle("height", control.getHeight());
        }

        rm.writeStyles();
        rm.writeClasses();
        rm.write(">");

        // render the details table
        if (!_.isUndefined(control.tableWrapper) && control.tableWrapper) {
            control.addDependent(control.tableWrapper);
            rm.renderControl(control.tableWrapper);
        }

        rm.write("</div>");
    },

    constructor: function(id, mSettings) {
        // construct super
        sap.ui.core.Control.call(this, id, mSettings);

        // declare local vars
        var self = this;
        var portfolioModel = self.getPortfolioModel();

        // mark the control as busy
        self.setBusy(true);

        // If the model is not valid there is not much we can do, so display the ZeroStateLayout
        if (!self.isModelValid()) {
            self.setBusy(false);
            self._createZeroStateLayout(self.HprUtil.getBundle().getText("HPRisk.PortfolioDetails.zeroState.query.problem.txt"));
            return;
        }

        // We need the wrapper so we can properly contain the table by positioning
        // it absolutely without layering on top of the popout menu for the panel
        // (the swizzle).
        self.tableWrapper = new sas.risk.common.panelsview.DivWrapper({
            height: "100%",
            width: "100%",
            content: self._createPortfolioTable()
        });
        self.tableWrapper.addStyleClass("hprBoundedControlContainer");

        // add a JSON model for metadata such as row count since it won't work properly
        // if added to the PageableJSONModel
        self.table.setModel(new sas.risk.common.models.RiskJSONModel(), "metadata");

        // separate model for available columns, otherwise updates to the count in metadata
        // trigger the table to cycle through the columns again causing duplicate ID errors
        self.table.setModel(new sas.risk.common.models.RiskJSONModel(), "availableColumns");

        // load the data source (required for column creation, so make sure it happens before we bind columns)
        portfolioModel.initializeModel().done(
            function () {
                self.restoringTableState = true;

                // now that the dataSource is in the model, update path label and the filter indicator
                self._updatePathLabel();
                self._updateFilterIndicator();

                // add the table columns
                self.table.getModel("availableColumns").setProperty("/", portfolioModel.getAvailableColumnIds());
                self.table.getBinding("columns").refresh();

                // configure the table model
                self.table.setModel(self._createTableDataModel());
                self.table.bindRows("/data");

                // restore sort order
                var sortColumn = portfolioModel.getSortColumn();
                if (sortColumn) {
                    var tableColumn = _.find(self.table.getColumns(), function (column) {
                        var columnId = _.find(column.getCustomData(), function (customData) {
                            return customData.getKey() === "columnId";
                        }).getValue();

                        return columnId === sortColumn.columnId;
                    });

                    self.table.sort(tableColumn, sortColumn.sortOrder, false);
                }

                // frozen columns
                var frozenColumnIds = portfolioModel.getFreezeColumns();
                if (frozenColumnIds && frozenColumnIds.length > 0) {
                    _.each(frozenColumnIds, function (frozenColumnName) {
                        var tableColumn = _.find(self.table.getColumns(), function (column) {
                            var columnId = _.find(column.getCustomData(), function (customData) {
                                return customData.getKey() === "columnId";
                            }).getValue();

                            return columnId === frozenColumnName;
                        });

                        self.table.freezeColumn(tableColumn);
                    });
                }

                // rerender the table to show changes
                self._rerenderTable();

                self.restoringTableState = false;
            }
        ).fail(
            function (jqxhr, textStatus, error) {
                // No metadata, display the ZeroStateLayout
                //TODO notify user instead of log?
                console.log("Portfolio metadata failed to load: " + jqxhr);
                self.setBusy(false);
                self._createZeroStateLayout(self.HprUtil.getBundle().getText("HPRisk.PortfolioDetails.zeroState.query.problem.txt"));
            }
        );

        // Subscribe to an event that is fired when the label for a Data Item is updated
        sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_DATA_ITEM_LABEL_UPDATED, self._onDataItemLabelUpdate, self);

        // Subscribe to an event that is fired when the 'Show labels' option field was updated
        sas.eventBus.subscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_PORTFOLIO_DETAILS_OPTIONS_SHOW_LABELS_UPDATED, this._onShowLabelsChange, this);

        // Subscribe to an event that is fired when the Portfolio Details page size preference (aka. ndetails) is updated
        sas.eventBus.subscribe(HprConstants.CHANNEL_APP_SETTINGS, HprConstants.EVENT_PORTFOLIO_DETAILS_PAGE_SIZE_CHANGED, self._onPortfolioDetailsPageSizeChange, self);
    },

    exit: function() {
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_DATA_ITEM_LABEL_UPDATED, this._onDataItemLabelUpdate, this);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_PORTFOLIO_DETAILS_OPTIONS_SHOW_LABELS_UPDATED, this._onShowLabelsChange, this);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_APP_SETTINGS, HprConstants.EVENT_PORTFOLIO_DETAILS_PAGE_SIZE_CHANGED, this._onPortfolioDetailsPageSizeChange, this);
    },


    // public functions

    /**
     * Called from HPRiskPanelsViewDelegate.getContentMetadata() when saving the control.
     * @returns {Object} the query data for this control
     */
    getQueryData: function() {
        return this.getPortfolioModel().toJSON();
    },

    /**
     * Returns the toolbar actions for the view.
     * @returns {Array}
     */
    getToolbarActions: function() {
        var toolbarActions = [
            {
                text: this.HprUtil.getBundle().getText("HPRisk.PortfolioDetails.OptionsMenu.Columns.txt"),
                tooltip: this.HprUtil.getBundle().getText("HPRisk.PortfolioDetails.OptionsMenu.Columns.txt"),
                press: $.proxy(this._handleLaunchColumnSelectorDialog, this)
            },
            {
                text: this.HprUtil.getBundle().getText("HPRisk.PortfolioDetails.OptionsMenu.Export.txt"),
                tooltip: this.HprUtil.getBundle().getText("HPRisk.PortfolioDetails.OptionsMenu.Export.txt"),
                startsSection: true,
                press: $.proxy(this._handleExportToExcel, this),
            }
        ];

        // Return the actions if the model is valid
        if (this.getPortfolioModel() && this.getPortfolioModel().isValid()) {
            return toolbarActions;
        } else {
            return [];
        }
    },

    handleResize: function() {
        this._rerenderTable();
    },

    /**
     * Requeries the data for the portfolio details.
     */
    rebuild: function() {
        // save the deferred on the PortfolioDetailsViewer so it can be resolved/rejected after the model query returns
        this.rebuildDeferred = $.Deferred();

        this.table.getModel().clearData();
        this._rerenderTable();
        this._updatePathLabel();
        this._updateFilterIndicator();

        return this.rebuildDeferred;
    },


    // private functions

    _updatePathLabel: function() {
        this.pathLabel.setText(this.getPortfolioModel().createPathString());
        this.pathSeparator.setVisible(this.pathLabel.getText() !== "" || this.getPortfolioModel().isAdditionalFilteringApplied());
    },

    _updateFilterIndicator: function() {
        this.filterIndicatorIcon.setVisible(this.getPortfolioModel().isAdditionalFilteringApplied());
    },

    /**
     * This method performs actions that make this viewer's PortfolioDetails model invalid.  Currently it sets the 'tuples' to an empty array.
     * The point of the method is to make the PortfolioDetails model invalid so a ZeroStateLayout is displayed as the content.
     * @private
     */
    _invalidatePortfolioDetailsModel: function() {
        this.getPortfolioModel().invalidate();
    },

    /**
     * Checks to see if this viewer's PortfolioDetails model is valid or not.  It currently checks the model for an empty 'tuples' array.
     * @returns {boolean} Returns true if this viewer's model is valid; otherwise, it returns false
     */
    isModelValid: function() {
        var valid = true;
        var portfolioModel = this.getPortfolioModel();

        if (_.isUndefined(portfolioModel) || !portfolioModel) {
            valid = false;
        }

        if (valid && portfolioModel.isValid() === false/*_.isEmpty(portfolioModel.tuples)*/) {
            valid = false;
        }

        return valid;
    },

    /**
     * Pseudo interface method to return the dataSourceId the component is using in a common way
     * @returns String
     */
    getDataSourceId: function() {
        return this.getPortfolioModel().dataSourceId;
    },

    /**
     * Pseudo interface method to return the type of control this is in a common way
     * @returns String
     */
    getPanelType: function() {
        return HprConstants.HPR_PORTFOLIO_DETAILS_PANEL;
    },

    /**
     * This method replaces the currently displayed WideTable with a ZeroStateLayout with a detailed message (if provided).  The 'zeroStateText'
     * string is displayed in the ZerStateLayout if it is provided; otherwise, a default message is displayed.
     * @param zeroStateText String Message to display within the ZeroStateLayout
     * @private
     */
    _createZeroStateLayout: function(zeroStateText) {
        // Determine the correct text to display in the ZeroState
        var zeroStateLayoutText = zeroStateText ? zeroStateText : this.HprUtil.getBundle().getText("HPRisk.PortfolioDetails.zeroState.default.txt");

        // Construct a ZeroStateLayout to display inside the RiskPanelsView
        var zeroStateLayout = new sas.risk.common.widgets.ZeroStateLayout({
            icon: sas.icons.HC.ZEROSTATETABLE,
            title: zeroStateLayoutText,
            buttons: []
        });

        if (!_.isUndefined(this.tableWrapper) && this.tableWrapper) {
            // Destroy the content of the tableWrapper so we can replace it with the ZeroStateLayout
            this.tableWrapper.destroyContent();
        } else {
            // tableWrapper does not exist yet, have to create it
            this.tableWrapper = new sas.risk.common.panelsview.DivWrapper({
                height: "100%",
                width: "100%"
            });
            this.tableWrapper.addStyleClass("hprBoundedControlContainer");
        }

        this.tableWrapper.setContent(zeroStateLayout);
    },

    /**
     * Creates the table displayed by this control.
     * @private
     */
    _createPortfolioTable: function() {
        this.table = new sas.hc.x.WideTable({
            id: this.getId() + "-table",
            showSelectionControls: false,
            showTableOptions: false,
            selectionMode: sas.hc.ui.table.SelectionMode.None,
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            minAutoRowCount: 1, // required for scrolling with < 5 rows
            allowReflow: false,
            toolbar: this._createPortfolioTableToolbar(),
            sort: $.proxy(this._handleColumnSort, this),
            columnFreeze: $.proxy(this._handleColumnFreeze, this),
            columnMove: $.proxy(this._handleColumnMove, this),
        });
        this.table.bindColumns({
            path: "availableColumns>/",
            factory: $.proxy(this._createPortfolioTableColumn, this)
        });
        this.table.addStyleClass("hprBoundedControl");
        this.table.addStyleClass("hprPortfolioTable");

        return this.table;
    },

    /**
     * Create the toolbar for the table.
     * @private
     */
    _createPortfolioTableToolbar: function() {
        var self = this;
        var toolbarContent = [];

        // label pair for total row count
        toolbarContent.push(new sas.hc.m.Label({
            text: {
                path: "metadata>/count",
                formatter: function(count) {
                    if (!count) {
                        count = 0;
                    }
                    return self.HprUtil.getBundle().getText("HPRisk.PortfolioDetails.Total.fmt", [count]);
                }
            }
        }));

        self.pathSeparator = new sas.hc.m.ToolbarSeparator();
        self.pathLabel = new sas.hc.m.Label();
        self.filterIndicatorIcon = sas.icons.pool.createControlByURI({
            src: sas.icons.HC.INFORMATIONSTATUS,
            tooltip: self.HprUtil.getBundle().getText("HPRisk.PortfolioDetails.Filtered.tooltip.txt"),
            visible: false
        });

        toolbarContent.push(self.pathSeparator);
        toolbarContent.push(self.pathLabel);
        toolbarContent.push(self.filterIndicatorIcon);


        return new sas.risk.common.layout.BarLayout({content: toolbarContent}).addStyleClass("hprPortfolioTableBar");
    },

    /**
     * Factory function for creating table a column.
     * @param id
     * @param context
     * @returns {sas.analytics.riskhp.portfolio.PortfolioDetailsColumn}
     * @private
     */
    _createPortfolioTableColumn: function(id, context) {
        var self = this;

        var dataSource = self.getPortfolioModel().dataSource;
        var selectedColumnIds = self.getPortfolioModel().getSelectedColumnIds();
        var columnDescriptor = self.getPortfolioModel().getColumnDescriptor(context.getProperty(context.sPath));
        var visible = !!(!selectedColumnIds || selectedColumnIds.length === 0 || _.contains(selectedColumnIds, columnDescriptor.id));

        // The labels for the columns which represent the Class Variable Data Items must update for a couple of scenarios;
        // First, the actual label for the Data Item could be updated via the Data Pane.  Also, the showLabels option for the
        // PortfolioDetailsViewer might be set to 'false' which indicates the name of the Data Item should be displayed instead
        // of the label.  Both scenarios need to be reflected when the column is constructed;
        // Use the classVariableId from the columnDescriptor and see if there a Class Variable Data Item by
        // that id, if so we want to construct the correct label for the column
        if (columnDescriptor.classVariableId) {
            var dataItem = dataSource.getDataItemById(columnDescriptor.classVariableId);
            if (dataItem) {
                // Create the label for the column based on the showLabels option setting
                var showLabels = self.getPortfolioModel().getOptionShowLabels().value;
                columnDescriptor.label = dataItem.getDisplayLabel(showLabels);
            }
        }

        var columnAlign = columnDescriptor.dataType === self.HprConstants.HPR_DATA_TYPE_NUMBER ? sap.ui.core.TextAlign.Right : sap.ui.core.TextAlign.Begin;

        var column = new sas.analytics.riskhp.portfolio.PortfolioDetailsColumn(self.getId() + "-" + columnDescriptor.id, {
            width: "10rem",
            label: new sas.hc.m.Label({
                textAlign: columnAlign,
                text: columnDescriptor.label ? columnDescriptor.label : columnDescriptor.name
            }),
            template: new sas.risk.common.widgets.RiskColumnText({
                textAlign: columnAlign,
                text: {
                    parts: [{path: columnDescriptor.id}],
                    formatter: function (value) {
                        var displayText = value;

                        // if date or numeric, apply formatter
                        switch (columnDescriptor.dataType) {
                            case self.HprConstants.HPR_DATA_TYPE_DATE:
                                displayText = sas.ltjs.sasicu.util.SASFormatterUtil.format(Number(value), columnDescriptor.format);
                                break;
                            case self.HprConstants.HPR_DATA_TYPE_NUMBER:
                                displayText = sas.ltjs.sasicu.util.SASFormatterUtil.format(Number(value), columnDescriptor.format);
                                break;
                        }

                        // return the resolved cell text
                        return displayText;
                    }
                }
            }),
            sortProperty: columnDescriptor.id,
            showFilterMenuEntry: false,
            visible: !!(!selectedColumnIds || selectedColumnIds.length === 0 || _.contains(selectedColumnIds, columnDescriptor.id)),
            customData: new sas.hc.ui.core.CustomData({key: "columnId", value: columnDescriptor.id})
        });

        // For a column representing a ClassVariable, add another CustomData object so we can update the label appropriately
        if (columnDescriptor.classVariableId) {
            column.addCustomData(new sas.hc.ui.core.CustomData({key: "classVariableId", value: columnDescriptor.classVariableId}));
        }

        return column;
    },

    _getResourceBundle: function() {
        return sas.analytics.riskhp.utils.HprUtil.getBundle();
    },

    _createTableDataModel: function() {
        var self = this;

        var oService = {
            sURL : self.getPortfolioModel().getQueryURL(),
            get: function(requestOptions) {
                sas.log.debug("Service->get()", requestOptions);
                var service = this;
                return $.Deferred(function(defer) {
                    self.setBusy(true);

                    self.currentRequestData = self.getPortfolioModel().getRequestData(requestOptions.data.start, requestOptions.data.limit, requestOptions.data.sortBy);
                    if (self.pageSizeChangeDetected === true) {
                        self.model.setPageSize(self.ApplicationSettings.getPortfolioDetailsPageSize());
                        self.PortfolioDetailsUtil.updateRequestPageSize(self.currentRequestData, self.ApplicationSettings.getPortfolioDetailsPageSize());
                        self.pageSizeChangeDetected = false;
                    }

                    var headers = {};
                    headers["Accept"] = "application/vnd.sas.risk.portfolio.query+json;charset=utf-8";
                    headers["Content-Type"] = "application/vnd.sas.risk.query.parms+json;charset=utf-8";
                    var request = sas.ajax({
                        async: true,
                        cache: false,
                        method: "POST",
                        headers: headers,
                        data: JSON.stringify(self.currentRequestData),
                        url: service.sURL
                    }).done(function(data, textStatus, jqXHR) {
                        // The server returns a 204 No Content response if the query is valid but has no results.  It does
                        // not return an empty JSON model which is resulting in an exception in the PageableJSONModel (S1387411)
                        // If we get a No Content response, provide an empty JSON model
                        if (jqXHR.status === 204) {
                            // self.HprUtil.displayGenericErrorDialog(self._getResourceBundle().getText('HPRisk.ErrorCode.204.txt'));

                            data = {
                                id: "",
                                limit: 200,
                                name: self.sId,
                                version: 1,
                                count: 0,
                                start: 0,
                                data: null // displays no data message in table
                            };
                        }

                        defer.resolve(data);
                    }).fail(function(XMLHttpRequest, textStatus, errorThrown) {
                        defer.reject(XMLHttpRequest, textStatus, errorThrown);
                    }).always(function() {
                        self.setBusy(false);
                    });
                }).promise();
            }
        };

        self.model = new sas.hc.ui.model.PageableJSONModel({
            service: oService,
            bindId: "data",
            cacheSize: 3,
            pageSize: Number(self.ApplicationSettings.getPortfolioDetailsPageSize())
        });

        // process successful request
        self.model.attachRequestCompleted(function(evt) {
            var data = evt.getParameter("json");
            if (data) {
                self.table.getModel("metadata").setProperty("/count", data.count);
            }
            if (self.rebuildDeferred) {
                self.rebuildDeferred.resolve();
                self.rebuildDeferred = null;
            }
        });

        // handle request failure
        self.model.attachRequestFailed(function(jqxhr) {
            sas.analytics.riskhp.utils.HprUtil.displayDetailedErrorDialog(jqxhr);

            // PageableJSONModel request failed, display the ZeroStateLayout
            self._invalidatePortfolioDetailsModel();
            self._createZeroStateLayout(self.HprUtil.getBundle().getText("HPRisk.PortfolioDetails.zeroState.query.problem.txt"));
            if (self.rebuildDeferred) {
                self.rebuildDeferred.reject();
                self.rebuildDeferred = null;
            }
        });

        //// define an intercept function to modify the request
        //self.model.fnUpdateUrlOptions = function(requestOptions) {
        //    self.setBusy(true);
        //
        //    requestOptions.type = "POST";
        //    requestOptions.data = self.getPortfolioModel().getRequestData(requestOptions.data.start, requestOptions.data.limit, requestOptions.data.sortBy);
        //
        //    if (!requestOptions.headers) {
        //        requestOptions.headers = {};
        //    }
        //    requestOptions.headers["Accept"] = "application/vnd.sas.risk.portfolio.query+json;charset=utf-8";
        //    requestOptions.headers["Content-Type"] = "application/vnd.sas.risk.query.parms+json;charset=utf-8";
        //};

        return self.model;
    },

    /**
     * When a label for a Data Item is updated, see if any of the column labels representing a Class Variable Data Item need to be updated.
     *
     * @param channel
     * @param event
     * @param data
     * @private
     */
    _onDataItemLabelUpdate: function(channel, event, data) {
        var self = this;

        // Only try to update the label if the id's of the DataSource being updated and DataSource for this PortfolioDetailsViewer match
        if (data && data.dataSourceId) {
            if (data.dataSourceId != self.getPortfolioModel().dataSourceId) return;
        }

        // Only try to update the label for a column if the event was for a Class Variable Data Item
        if (data && data.dataItem) {
            if (data.dataItem.getType() != self.CLASSVAR) return;
        }

        // Only try to update the column label if PortfolioDetailsViewer is displaying labels (and not names) for Data Items
        var showLabels = self.getPortfolioModel().getOptionShowLabels().value;
        if (!showLabels) return;

        if (self.table && data && data.dataItem) {
            // Go through the columns and update the one that had a label change if it exists
            _.each(self.table.getColumns(), function(column) {
                // Use the custom data added for the column to get its classVariableId
                var customData = column.getCustomData();
                // Columns representing ClassVariables have more than 1 CustomData objects
                if (customData && customData.length > 1) {
                    // Take the classVariableId and compare it to the Data Item the label change event was fired for
                    var columnId = customData[1].getProperty('value');
                    if (columnId === data.dataItem.getId()) {
                        // There is a column by that id, try to find the real Data Item for one last check
                        var dataItem = self.getPortfolioModel().dataSource.getDataItemById(columnId);
                        if (dataItem) {
                            var newLabel = data.newLabel;

                            // A Class Variable Data Item exists by that id, so update the label for the Column
                            column.setLabel(newLabel);

                            // We also need to update the column descriptor with the correct label so the
                            // available columns in the PortfolioDetailsColumnSelectionDialog will be correct
                            var columnBindingContext = column.getBindingContext("availableColumns");
                            var contextId = columnBindingContext.getModel().getProperty(columnBindingContext.sPath);
                            self.getPortfolioModel().getColumnDescriptor(contextId).setLabel(newLabel);
                        }
                    }
                }
            });
        }

        // update the path string
        self._updatePathLabel();
    },

    /**
     * When the "Show labels" option is updated, update the table column labels appropriately.
     *
     * @param channel
     * @param event
     * @param data
     * @private
     */
    _onShowLabelsChange: function(channel, event, data) {
        var self = this;

        // Only try to update the column labels if the id of this PortfolioDetailsViewer matches the one that fired the event (think multiple PortfolioDetailsViewers could be on a page)
        if (data && data.portfolioDetailsId) {
            if (data.portfolioDetailsId != self.getPortfolioModel().id) return;
        }

        if (self.table && data) {
            // Go through the columns and try to update the ones that correspond to Class Variables
            _.each(self.table.getColumns(), function(column) {
                // Use the custom data added for the column to get its classVariableId
                var customData = column.getCustomData();
                // Columns representing ClassVariables have more than 1 CustomData objects
                if (customData && customData.length > 1) {
                    // Take the id (corresponds to a Class Variable) and find the appropriate Data Item
                    var columnId = customData[1].getProperty('value');
                    var dataItem = self.getPortfolioModel().dataSource.getDataItemById(columnId);
                    if (dataItem) {
                        // Retrieve the correct label to display (label vs. name)
                        var displayLabel = dataItem.getDisplayLabel(data.showLabels);

                        // A Class Variable Data Item exists by that id, so update the label for the Column appropriately
                        column.setLabel(displayLabel);

                        // We also need to update the column descriptor with the correct label so the
                        // available columns in the PortfolioDetailsColumnSelectionDialog will be correct
                        var columnBindingContext = column.getBindingContext("availableColumns");
                        var contextId = columnBindingContext.getModel().getProperty(columnBindingContext.sPath);
                        self.getPortfolioModel().getColumnDescriptor(contextId).setLabel(displayLabel);
                    }
                }
            });
        }
    },

    /**
     * When the Portfolio Details Page Size preference setting (NDETAILS) is updated, set the page size appropriately on the model.
     *
     * @param channel
     * @param event
     * @param data
     * @private
     */
    _onPortfolioDetailsPageSizeChange: function(channel, event, data) {
        var self = this;

        if (self.model) {
            self.pageSizeChangeDetected = true;
            //self.model.setPageSize(self.ApplicationSettings.getPortfolioDetailsPageSize());
        }
    },

    /**
     * Handle the user sorting a column in the table.
     * @param event
     * @private
     */
    _handleColumnSort: function(event) {
        if (this.restoringTableState === true) {
            return;
        }

        var sortOrder = event.getParameter("sortOrder");
        var columnId = _.find(event.getParameter("column").getCustomData(), function(customData) {
            return customData.getKey() === "columnId";
        }).getValue();

        this.getPortfolioModel().setSortColumn(columnId, sortOrder);

        this.fireDirtyChange();
    },

    /**
     * Handle the user freezing a column in the table.
     * @param event
     * @private
     */
    _handleColumnFreeze: function(event) {
        if (this.restoringTableState === true) {
            return;
        }

        var column = event.getParameter("column");
        var columnId = _.find(column.getCustomData(), function(customData) {
            return customData.getKey() === "columnId";
        }).getValue();

        if (event.getParameter("isFreeze") === true) {
            this.getPortfolioModel().addFreezeColumn(columnId);
        }
        else {
            this.getPortfolioModel().removeFreezeColumn(columnId);
        }

        this._rerenderTable();

        this.fireDirtyChange();
    },

    /**
     * Handle the user moving a column in the table.
     * @param event
     * @private
     */
    _handleColumnMove: function(event) {
        if (this.restoringTableState === true) {
            return;
        }

        // Move event is fired before the column is actually moved. Therefore,
        // we need to delay processing the move until the table has finished
        // and rerendered itself.
        var self = this;
        self._rerenderTable().done(function() {
            var selectedColumnIds = [];
            _.each(self.table.getColumns(), function (tableColumn) {
                selectedColumnIds.push(
                    _.find(tableColumn.getCustomData(), function (customData) {
                        return customData.getKey() === "columnId";
                    }).getValue()
                );
            });

            self.getPortfolioModel().setSelectedColumnIds(selectedColumnIds);

            self.fireDirtyChange();
        });
    },


    //
    // Menu handling functions
    //

    _handleLaunchColumnSelectorDialog: function() {
        var self = this;

        var dialog = new sas.analytics.riskhp.portfolio.PortfolioDetailsColumnSelectionDialog({
            portfolioModel: self.getPortfolioModel(),
            table: self.table,
            afterClose: function () {
                dialog.destroy();
            },
            okPressed: function() {
                self.fireDirtyChange();
            }
        });
        dialog.open();
    },

    _handleExportToExcel: function() {
        var self = this;

        self.setBusy(true);

        // if no export request, nothing to export
        var exportRequestData = self.getPortfolioModel().getExportRequestData();
        if (!exportRequestData) {
            //TODO show message
            return;
        }

        // export the data represented by the request
        self.HprAPI.exportPanel(exportRequestData).done(
            function(exportResult) {
                console.debug("Portfolio export successful.");
            }
        ).fail(
            function (jqxhr, textStatus, error) {
                console.debug("Portfolio export failed.");
            }
        ).always(
            function() {
                self.setBusy(false);
            }
        );
    },


    // utilities

    /**
     * Rerenders the table and ensures that the sort decoration is reapplied.
     * @private
     */
    _rerenderTable: function() {
        var self = this;

        // If there is no table, don't try and re-render
        if (!self.table) { return; }

        var deferred = $.Deferred();
        var rerenderFunctionInternal = function() {
            // rerender table
            self.table.rerender();

            // reapply sort icon
            var sortColumn = self.getPortfolioModel().getSortColumn();
            if (sortColumn) {
                var tableColumn = _.find(self.table.getColumns(), function (column) {
                    var columnId = _.find(column.getCustomData(), function (customData) {
                        return customData.getKey() === "columnId";
                    }).getValue();

                    return columnId === sortColumn.columnId;
                });

                // resize can be fired before the table columns are set
                if (tableColumn) {
                    tableColumn._updateIcons();
                }
            }

            deferred.resolve();
        };

        // Using a 0 timeout moves this execution after the current
        // block of code that triggered this function has finished
        // processing.
        setTimeout(rerenderFunctionInternal, 0);

        return deferred;
    },
});