/*
 * '(c) 2017, SAS Institute Inc.'
 */

// (c) 2017, SAS Institute Inc.

jQuery.sap.declare("sas.analytics.riskhp.sensitivities.RiskFactorSensitivitiesModel");

jQuery.sap.require("sas.analytics.riskhp.models.HPRiskModelManager");
jQuery.sap.require("sas.analytics.riskhp.models.DataSourceCacheManager");

jQuery.sap.require("sas.analytics.riskhp.settings.ApplicationSettings");
jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");

jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Horizon");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.RiskCube");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Statistic");

jQuery.sap.require("sas.ltjs.sasicu.util.SASFormatterUtil");
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");

/**
 *
 *
 * @name sas.analytics.riskhp.sensitivities.RiskFactorSensitivitiesModel
 */
sap.ui.base.Object.extend("sas.analytics.riskhp.sensitivities.RiskFactorSensitivitiesModel", {
    metadata: {
        publicMethods: [
            "isValid",
            "executeQuery",
            "configure",
            "areDataSourceIdsBeingReferenced",
            "removeDataSource",
            "replaceDataSource",
            "getOptionName",
            "setOptionName",
            "getOptionShowLabels",
            "setOptionShowLabels"
        ]
    },

    // pseudo-imports
    DataSourceCacheManager: sas.analytics.riskhp.models.DataSourceCacheManager,
    SASFormatterUtil: sas.ltjs.sasicu.util.SASFormatterUtil,
    HprUtil: sas.analytics.riskhp.utils.HprUtil,

    // not much setup needed here as configure will be called when creating a new rfs view, and FromJSON called when opening persisted rfs view.
    constructor: function(id) {
        this.id = id;
    },


    // public setters

    getId: function() {
        return this.id;
    },

    executeQuery: function(successCallback, failureCallback) {
        var self = this;

        // Do not execute the query if the model is invalid
        if (!self.isValid()) {
            failureCallback();
            return;
        }

        var dataSourceDeferred = DataSourceCacheManager.getDataSource(self.dataSourceId);
        dataSourceDeferred.done(function(dataSource) {
            self.dataSource = dataSource;

            self.query = self._buildQuery(dataSource);
            console.log(self.query);
            self._executeQuery(self.query,successCallback,failureCallback);
        });
    },

    _executeQuery: function(queryRequest,successCallback,failureCallback) {
        var self = this;
        console.log("Executing query request:\n" + JSON.stringify(queryRequest));
        sas.analytics.riskhp.utils.HprAPI.query(queryRequest).done(function (queryResults) {

            // If there are no query results, set the model to an invalid state and call the failureCallBack function
            if (!queryResults || _.isEmpty(queryResults)) {
//                self._invalidateChartModel();
                failureCallback();
                return;
            }

            console.log("RAW queryResults:\n" + JSON.stringify(queryResults));

            // format the theta line in the results
            var dateResult = _.find(queryResults.results.data, function(result) {
               return result.id === "_date_";
            });
            if (dateResult) {
                dateResult.id = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.Sensitivities.cell.value.date.txt");
                dateResult.baseValue = self.SASFormatterUtil.format(Number(dateResult.baseValue), "DATE9.");
            }

            self.queryResults = queryResults;
            self.results = queryResults.results; // TODO: null check?
            self.data = queryResults.results.data; // TODO: null check?
            if (HprConstants.ENABLE_DEBUG) {
                console.log("Model is:\n" + JSON.stringify(self));
            }
            successCallback();
        }).fail(function (jqxhr, textStatus, error) {
            // If there are no query results, set the chart model to an invalid state and call the failureCallBack function
            //self._invalidateChartModel();
            failureCallback();
        });
    },

    /**
     * Called when a new risk factor sensitivities view is being opened.  FromJSON is called when opening a rfs view that was persisted.
     * @param contextData - context information from the crosstab selection and risk factors from the selection dialog
     */
    configure: function(contextData) {
        var self = this;
        if (HprConstants.ENABLE_DEBUG) {
            console.log("Query Data:\n" + JSON.stringify(contextData));
        }

        // save showLabels value from the dialog to be used as the showLabels value in the options pane
        var showLabels = contextData.showLabels;

        self.dataSourceId = contextData.dataSourceId;
        var deferred = DataSourceCacheManager.getDataSource(self.dataSourceId);
        deferred.done(function(cubeData) {
            var optionsModel = cubeData.getOptions();
            // Pass in data to populate the default options
            self.options = self._buildDefaultOptionsForOptionsPanel(optionsModel);
            self.options.showLabels = showLabels;
        });

        self.type = contextData.type;
        self.classVariables = contextData.classVariables;
        self.drill = contextData.drill;
        self.displayHorizons = contextData.displayHorizons;
        self.tuples = contextData.tuples;
        self.riskFactors = contextData.riskFactors;

        self.report = {
            requests: contextData.requests,
            sorts: contextData.sorts,
            options: contextData.options,
            filters: contextData.filters
        };
    },


    // public getters
    getTuples: function() {
        if (this.tuples && this.tuples.length > 0) {
            if (this.dataSource) {
                this.tuples = this.dataSource.hydrateTuples(this.tuples);
            }
            return this.tuples;
        }
        return [];
    },

    areHorizonsAbsolute: function() {
        return this.displayHorizons !== HprConstants.DISPLAY_HORIZONS_RELATIVE_VALUE;
    },

    getData: function() {
        return this.data;
    },

    // public functions

    toJSON: function() {
        return {
            name: this.id,
            type: this.type,
            classVariables: this.classVariables,
            dataSourceId: this.dataSourceId,
            tuples: this.tuples,
            drill: this.drill,
            displayHorizons: this.displayHorizons,
            queryData: this.queryData,
            options: this.toOptionsJSON(),
            riskFactors: this.riskFactors,
            report: this.report
        };
    },

    // Called when restoring a saved sensitivity
    fromJSON: function(expJson, obj) {
        var self = this;

        // Safety check to make sure we have an Object
        var exp;
        if (obj) {
            exp = obj;
        } else {
            exp = JSON.parse(expJson);
        }

        // Have to set the dataSourceId first as hydration will need to use it.
        self.name = exp.name;
        self.type = exp.type;
        self.dataSourceId = exp.dataSourceId;

        self.classVariables = exp.classVariables;
        self.drill = exp.drill;
        self.displayHorizons = exp.displayHorizons;
        self.tuples = exp.tuples;
        self.queryData = exp.queryData;

        // Re-construct RFS Options from JSON
        if (exp.options) {
            self.fromOptionsJSON(exp.options);
        }

        self.riskFactors = exp.riskFactors;
        self.report = exp.report;
    },

    /**
     * A method which checks to see if the existing content object references any of the data source ids by looking
     * at the "dataSourceId" property.
     * @param { string } currentContentData - The JSON object representing the current state of a panel
     * @param { [string] } dataSourceIds - A list of data source ids
     */
    areDataSourceIdsBeingReferenced: function(currentContentData, dataSourceIds) {
        return _.contains(dataSourceIds, currentContentData.dataSourceId);
    },

    /**
     * pseudo static method that takes a list of Data Source ids which are being removed from an Exploration and determines
     * if this model references one of them; If it does a Zero State object is returned
     * @param { string } currentContentData - The JSON object representing the current state of a panel
     * @param { [string] } dataSourceIdsBeingRemoved - A list of data source ids that are being removed
     * @returns {*}
     */
    removeDataSource: function(currentContentData, dataSourceIdsBeingRemoved) {

        // Create the wrapping deferred
        var returnDeferred = $.Deferred();

        var currentModelJSON = currentContentData;
        if (HprConstants.ENABLE_DEBUG) {
            console.log("removeDataSource currentModelJSON:\n" + JSON.stringify(currentModelJSON));
        }

        // Check to make sure the current data source id is contained in the list of data source ids being removed,
        // otherwise there is nothing to do so return the provided currentContentData.
        if (!_.contains(dataSourceIdsBeingRemoved, currentModelJSON.dataSourceId)) {
            returnDeferred.resolve(currentModelJSON);
            return returnDeferred;
        }

        // Initial setup checks are done, create the new content data object
        var updatedModelJSON = {
            name: currentModelJSON.name,
            type: currentModelJSON.type,
            dataSourceId: "",
            options: {
                // So the name of the ZeroState RFS will be the same
                name: currentModelJSON.options.name
            }
        };

        returnDeferred.resolve(updatedModelJSON);
        return returnDeferred;
    },

    /**
     * pseudo static method that takes an existing content object and changes it in a way to represent a new data source.
     * This function should select an equivalent data source id from the list of newDataSourceIds for the data source
     * id the currentContentData is representing.
     *
     * This method must return a jQuery.Deferred object that behaves as follows:
     * - Returns the JSON to represent the panel's new state regardless of whether the panel should change because of the new data source
     * - ALWAYS resolve successfully.
     * - May make subsequent/wrapped ajax calls but when it returns all processing should be complete.
     * - May not change the state of the model instance or attempt to interact with the UI.
     *
     * @param { string } currentContentData - The JSON object representing the current state of a panel
     * @param { [string] } dataSourceIdsBeingReplaced - A list of data source ids that are being replaced
     * @param { [string] } newDataSourceIds - A list of replacement data source ids
     * @returns {*}
     */
    replaceDataSource: function(currentContentData, dataSourceIdsBeingReplaced, newDataSourceIds) {

        // Create the wrapping deferred
        var returnDeferred = $.Deferred();

        var currentModelJSON = currentContentData;
        if (HprConstants.ENABLE_DEBUG) {
            console.log("replaceDataSource currentModelJSON:\n" + JSON.stringify(currentModelJSON));
        }

        // Check to make sure the current data source id is contained in the list of data source ids being replaced,
        // otherwise there is nothing to do so return the provided currentContentData.
        if (!_.contains(dataSourceIdsBeingReplaced, currentModelJSON.dataSourceId)) {
            returnDeferred.resolve(currentModelJSON);
            return returnDeferred;
        }

        // Store any deferreds from requests needed to update for the new data source here. After all items are
        // processed, this set of deferreds will be passed into a when function, whereupon their completion, the
        // master deferred will be resolved (if we get that far, of course).
        var pendingRequests = [];

        var newDataSourceId = sas.analytics.riskhp.utils.HprUtil.getCompatibleDataSourceId(currentModelJSON.dataSourceId, newDataSourceIds);

        // Initial setup checks are done, create the new content data object
        var updatedModelJSON = {
            name: currentModelJSON.name,
            type: currentModelJSON.type,
            dataSourceId: newDataSourceId,
            drill: currentModelJSON.drill,
            displayHorizons: currentModelJSON.displayHorizons,
            options: currentModelJSON.options,
            report: currentModelJSON.report,

            // these may be changed in further processing below
            tuples: [],
            classVariables: [],
            riskFactors: []
        };

        // Get a compatible data source id from the list of newDataSourceIds.  If one is not found then there is
        // nothing to do so return the provided currentContentData
        if (!newDataSourceId) {
            returnDeferred.resolve(updatedModelJSON);
            return returnDeferred;
        }

        // Grab the data source - should already be loaded
        var deferred = DataSourceCacheManager.getDataSource(newDataSourceId);
        deferred.done(function(targetDataSource) {

            // flag to capture the overall status of the validation
            var passedValidation = true;

            // variable to capture the first error message for why validation failed
            var firstErrorMessage = null;

            /**
             * Internal helper function to collect the two steps of setting the error message and setting the overall validation
             * status when an error condition is detected
             * @param message
             */
            var setErrorMessage = function(message) {
                if (firstErrorMessage === null) {
                    firstErrorMessage = message;
                    passedValidation = false;
                }
            };

            // Validate the capabilities
            var capabilities = targetDataSource.capabilities;
            if (!capabilities.sensitivitiesSupported) {
                setErrorMessage("The new data source does not support sensitivities");
            }

            // validate that there is only 1 risk cube in the new data source.  Otherwise we cannot replace the
            // risk cube in the horizon tuple for the new data source
            var riskCubes = targetDataSource.getAllRiskCubes();
            if (!riskCubes || riskCubes.length === 0 || riskCubes.length > 1) {
                setErrorMessage("The new data source does not have exactly one risk cube from which to get base case horizon");
            }

            // validate base case horizon
            if (targetDataSource.horizon && targetDataSource.horizon.length > 0) {
                var horizon = targetDataSource.horizon[0];
                if (horizon.horizonNumber !== -1  || horizon.interval !== sas.analytics.riskhp.models.dataitems.Horizon.BASECASE) {
                    setErrorMessage("Could not find base case horizon in new data source");
                } else {
                    var newDataItems = [];
                    _.each(currentModelJSON.tuples[0], function(dataItem) {
                        if (dataItem.type === sas.analytics.riskhp.models.dataitems.Horizon.HORIZON) {
                            newDataItems.push(horizon);
                        } else if (dataItem.type === sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE) {
                            var targetRiskCube = targetDataSource.getDataItemById(riskCubes[0].id);
                            if (targetRiskCube) {
                                newDataItems.push(targetRiskCube);
                            }
                        } else { // should we ignore result sets and risk cubes?
                            var dataItem = targetDataSource.getDataItemById(dataItem.id);
                            if (dataItem) {
                                newDataItems.push(dataItem);
                            }
                        }
                    });
                    if (currentModelJSON.tuples[0].length === newDataItems.length) {
                        updatedModelJSON.tuples = [newDataItems];
                    } else {
                        setErrorMessage("Missing horizon data in new data source");
                    }
                }
            } else {
                setErrorMessage("Could not find base case horizon in new data source");
            }

            // validate class variables in request (pickinfo expansion) exist
            var currentClassVars = currentModelJSON.classVariables;
            var allClassVarsFound = true;
            _.each(currentClassVars, function (currentCV) {
               var newCV = targetDataSource.getDataItemById(currentCV.id);
               if (!newCV) {
                   allClassVarsFound = false;
               }
            });
            if (allClassVarsFound) {
                updatedModelJSON.classVariables = currentModelJSON.classVariables;
            } else {
                setErrorMessage("The new data source does not contain all class variables from the sensitivity analysis");
            }

            // validate that at least one of the selected risk factors is available in the new data source
            var currentRfNames = [];
            _.each(currentModelJSON.riskFactors, function(rf) {
                currentRfNames.push(rf.name);
            });
            var riskFactorsDeferred = sas.analytics.riskhp.utils.HprAPI.getVariableDefinitions(targetDataSource, currentRfNames).done(function (data){
                _.each(data.items, function(rf) {
                    updatedModelJSON.riskFactors.push(rf);
                });
            });
            pendingRequests.push(riskFactorsDeferred);

            // once all the requests have finished, we can return
            $.when.apply(self, pendingRequests).always(
                function() {
                    if (!passedValidation) {
                        console.error(firstErrorMessage);

                        // empty out the data.  This makes it so you can not replace data source back...
                        updatedModelJSON.tuples = [];
                        updatedModelJSON.classVariables = [];
                        updatedModelJSON.riskFactors = [];
                    }
                    if (HprConstants.ENABLE_DEBUG) {
                        console.log("replaceDataSource updatedModelJSON:\n" + JSON.stringify(updatedModelJSON));
                    }
                    // Always resolve the deferred whether it passed validation or not.
                    returnDeferred.resolve(updatedModelJSON);
                }
            );
        });
        deferred.fail(function (jqxhr, textStatus, error) {
            // Failure case where we create the zero state json and resolve the deferred successfully
            jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
            updatedModelJSON.type = HprConstants.PlotType.ZeroState;
            returnDeferred.resolve(updatedModelJSON);
        });
        return returnDeferred;
    },

    isValid: function() {
        return !_.isEmpty(this.tuples) && !_.isEmpty(this.classVariables) && !_.isEmpty(this.riskFactors);
    },


    isDeltaNormal: function() {
        var self = this;
        var result = false;
        // is it better to use the analysisType of the riskCube to determine type?
        var dataSource = self.dataSource;

        var riskCube = _.find(self.tuples[0], function(tupleItem) {
            return tupleItem.type === "riskCube";
        });
        if (riskCube) {
            // get the inflated/full data item
            var riskCubeItem = dataSource.getDataItemById(riskCube.id);
            if (riskCubeItem) {
                result = !riskCubeItem.capabilities.stressCube
            }
        }
        return result;
    },

    // private functions

    _buildQuery: function(dataSource) {
        var self = this;

        var dataDefinitions = {
            tuples: [],
            riskFactors: []
        };
        _.each(self.tuples, function(dataItems) {
            var tuple = {};
            tuple.dataItems = [];
            _.each(dataItems, function(dataItem) {
                tuple.dataItems.push({
                    id: dataItem.id,
                    type: dataItem.type
                });
            });
            dataDefinitions.tuples.push(tuple);
        });
        var riskFactorIds = [];
        _.each(self.riskFactors, function(rf) {
           riskFactorIds.push(rf.id);
        });
        dataDefinitions.riskFactors = riskFactorIds;

        var requestType = self.isDeltaNormal() ? "deltaNormalSensitivity" : "sensitivity";

        return {
            version: 1,
            id: self.getId(),
            type: requestType,
            dataSources: {
                id: dataSource.dataSource.id,
                application: dataSource.dataSource.application,
                workGroup: dataSource.dataSource.workGroup,
                relPath: dataSource.dataSource.relPath,
                type: dataSource.dataSource.type
            },
            options: self.options,
            dataDefinitions: dataDefinitions,
            report: self.report
        };
    },


    //*********************************
    // Options panel support methods
    //*********************************
    getOptions: function () {
        var optionsObject = $.extend(true, {}, this.options);
        optionsObject.id = this.id;
        return optionsObject;
    },

    setOptionGeneralOptionsSectionCollapsed: function(generalOptionsSectionCollapsed) {
        this.options.generalOptionsSectionCollapsed = generalOptionsSectionCollapsed;
    },

    getOptionName: function() {
        return this.options.name;
    },

    setOptionName: function(name) {
        this.options.name = name;
    },

    getOptionShowLabels: function() {
        return this.options.showLabels;
    },

    setOptionShowLabels: function(showLabels) {
        this.options.showLabels = showLabels;
    },

    _buildDefaultOptionsForOptionsPanel: function(optionsData) {
        var options = {};
        // Initialize these properties so the UI components are initialized correctly
        options.applyEnabled = false; // Flag for the Apply button on each Options pane; Disabled by default
        options.showLabels = true;//{value: true};
        options.generalOptionsSectionCollapsed = false; // Open by default

        return options;
    },

    createPathString: function() {
        var self = this;
        var pathString = "";

        var appendToPathString = function(itemLabel) {
            if (pathString.length > 0) {
                pathString += " > ";
            }
            pathString += "[" + itemLabel + "]";
        };

        var paths = self.report.requests[0].key;
        var dataSource = self.dataSource;

        if (paths.length === 0) {
            var classVar = dataSource.getDataItemById(self.classVariables[0].id);
            var classVarStr = self.options.showLabels && classVar.label ? classVar.label : classVar.name;
            appendToPathString(classVarStr + " : " + sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.plot.path.value.all.txt"));
        } else {
            _.each(paths, function (path) {
                var classVar = dataSource.getDataItemById(path.id);
                if (classVar) {
                    var classVarStr = self.options.showLabels && classVar.label ? classVar.label : classVar.name;
                    appendToPathString(classVarStr + " : " + path.value);
                }
            });
        }

        return self.HprUtil.getBundle().getText("HPRisk.panel.genericPath.fmt", [pathString]);
    },

    _loadDataSource: function(dataSourceId) {
        var self = this;
        var deferred = $.Deferred();

        // load the data source
        self.DataSourceCacheManager.getDataSource(dataSourceId).done(
            function (dataSource) {
                deferred.resolve(dataSource);
            }
        ).fail(
            function (jqxhr, textStatus, error) {
                deferred.reject(jqxhr, textStatus, error);
            }
        );

        return deferred.promise();
    },

    initializeModel: function() {
        var self = this;
        var deferred = $.Deferred();

        // load the data source and column metadata
        self._loadDataSource(self.dataSourceId).done(
            function (dataSource) {
                self.dataSource = dataSource;

                deferred.resolve();
            }
        ).fail(
            function (jqxhr, textStatus, error) {
                self.invalidate();
                deferred.reject(jqxhr, textStatus, error);
            }
        );

        return deferred.promise();
    },



    toOptionsJSON: function() {
        return $.extend(true,{},this.options);
    },

    fromOptionsJSON: function(opts) {
        this.options = $.extend(true,{},opts);
    },

    updateOptions: function(optionsData) {
        this.options = $.extend(true,{},optionsData);
    }

});