/*
 * '(c) 2017, SAS Institute Inc.'
 */
jQuery.sap.require("sas.hc.ui.table.Table");
jQuery.sap.require("sas.hc.ui.table.Column");

jQuery.sap.require("sas.hc.m.ResponsiveToolbar");
jQuery.sap.require("sas.hc.m.SearchField");

jQuery.sap.require("sas.risk.common.widgets.RiskColumnText");

jQuery.sap.require('sas.analytics.riskhp.models.HPRiskModelManager');
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.analytics.riskhp.utils.ViewManager");

jQuery.sap.require("sas.analytics.riskhp.utils.HprAPI");
jQuery.sap.require("sas.hc.ui.table.TreeTable");
jQuery.sap.require("sas.hc.ui.table.VisibleRowCountMode");
jQuery.sap.require("sas.hc.ui.table.SelectionBehavior");
jQuery.sap.require("sas.hc.ui.table.SelectionMode");
jQuery.sap.require("sas.hc.ui.model.PageableJSONTreeModel");
jQuery.sap.require("sas.hc.ui.layout.Splitter");
jQuery.sap.require("sas.hc.ui.layout.SplitterLayoutData");
jQuery.sap.require("sap.ui.core.Orientation");
jQuery.sap.require("sas.risk.common.widgets.ZeroStateLayout");

jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.risk.common.utils.CommonUtil");
jQuery.sap.require("sas.analytics.riskhp.models.DataSourceCacheManager");


/**
 * Add Scenarios dialog.
 */
sap.ui.jsfragment("sas.analytics.riskhp.sensitivities.RiskFactorSensitivitiesSelectorDialog", {

    HprAPI: sas.analytics.riskhp.utils.HprAPI,
    CommonUtil: sas.risk.common.utils.CommonUtil,
    HprUtil: sas.analytics.riskhp.utils.HprUtil,

    successCallBack: null,
    _isAddingAvailableRiskFactors: true, // true when the available risk factors tree is selected.  Controls dual selector button state

    _onSearchAvailableRiskFactors: function (oEvent) {
        var self = this;

        var value = oEvent.getSource().getValue() || "";
        var nameFilter = new sap.ui.model.Filter("name", sap.ui.model.FilterOperator.Contains, value);
        var binding = self.riskFactorsTreeTable.getBinding("rows");
        binding.filter(nameFilter);
        self.riskFactorsTreeTable.expandToLevel(sas.risk.common.utils.CommonConstants.MAX_INTEGER);
    },

    _createRiskFactorsTreeTableToolbar: function () {
        var self = this;

        var searchField = new sas.hc.m.SearchField({
            liveChange: $.proxy(self._onSearchAvailableRiskFactors, self), //$.proxy(ctrl.onInstrumentVariableSourcesFilterChanged, ctrl),
            filterMode: true,
            // ariaLabel: this.rb.getText('instrumentVariableSourceTable.filter.aria.txt'),
            search: $.proxy(self._onSearchAvailableRiskFactors, self) // $.proxy(ctrl.onInstrumentVariableSourcesFilterChanged, ctrl)
        });//.addStyleClass('riskToolbarSearchField');
        self.CommonUtil.setSearchFieldLandmarkLabel(searchField, self.HprUtil.getBundle().getText("HPRisk.Sensitivities.SearchAvailableRiskFactors.landmark.txt"));

        return new sas.hc.m.ResponsiveToolbar({
            content: [searchField]
        });
    },

    _updateButtonStates: function() {
        var enableSingleButton;
        var enableAllButton;
        var self = this;

        if (self._isAddingAvailableRiskFactors) {
            enableSingleButton = self.riskFactorsTreeTable.getSelectedIndices().length > 0;
            self.singleItemButton.setIcon(sas.icons.HC.ARROWRIGHTADDDUALSELECTOR);
            self.singleItemButton.setEnabled(enableSingleButton);

            enableAllButton = self.riskFactorsTreeTable.getBinding("rows").getLength() > 0 ;// only when have data
            self.allItemsButton.setIcon(sas.icons.HC.ARROWRIGHTADDALLDUALSELECTOR);
            self.allItemsButton.setEnabled(enableAllButton);
        } else {
            enableSingleButton = self.riskFactorsList.getSelectedIndices().length > 0;
            self.singleItemButton.setIcon(sas.icons.HC.ARROWLEFTREMOVEDUALSELECTOR);
            self.singleItemButton.setEnabled(enableSingleButton);

            enableAllButton = self.riskFactorsList.getModel().getData().length > 0;
            self.allItemsButton.setIcon(sas.icons.HC.ARROWLEFTREMOVEALLDUALSELECTOR);
            self.allItemsButton.setEnabled(enableAllButton);
        }
    },

    _handleAvailableRiskFactorSelection: function(event) {
        var self = this;

        if (event.getParameter("userInteraction")) {
            self._isAddingAvailableRiskFactors = true;

            self.riskFactorsList.clearSelection();
            self._updateButtonStates();
        }
    },

    _handleSelectedRiskFactorSelection: function(event) {
        var self = this;

        if (event.getParameter("userInteraction")) {
            self._isAddingAvailableRiskFactors = false;

            self.riskFactorsTreeTable.clearSelection();
            self._updateButtonStates();
        }
    },

    _addRiskFactorTreeItemToSelectedList: function(item) {
        var self = this;
        if (item.type === "riskFactor") {
            var selectedRiskFactorsModel = self.riskFactorsList.getModel();
            var selectedRiskFactors = selectedRiskFactorsModel.getData();
            if (!_.find(selectedRiskFactors, function(riskFactor){
                return item.name === riskFactor.name;
            })){
                selectedRiskFactors.push(item);
                selectedRiskFactorsModel.updateBindings();
            }
        } else {
            // recursively load the items members from the mid-tier
            this.HprAPI.getRiskFactorMembers(item).done(function(data) {
                _.each(data.riskFactors, function(item) {
                    self._addRiskFactorTreeItemToSelectedList(item);
                });
            }).fail(function() {
                console.error("Failed to load risk factor information from server: " + item.name);
            });
        }
    },

    _addRiskFactorsToSelectedList: function(items) {
        var self = this;

        var selectedRiskFactorsModel = self.riskFactorsList.getModel();
        var selectedRiskFactors = selectedRiskFactorsModel.getData();
        _.each(items, function(item) {
            if (item.type === "riskFactor") {

                if (!_.find(selectedRiskFactors, function (riskFactor) {
                        return item.name === riskFactor.name;
                    })) {
                    selectedRiskFactors.push(item);
                }
            }
        });
        selectedRiskFactorsModel.updateBindings();
    },

    _handleSingleItemButtonPressed: function() {
        var self = this;
        if (self._isAddingAvailableRiskFactors) {
            _.each(self.riskFactorsTreeTable.getSelectedIndices(), function(index){
                var item = self.riskFactorsTreeTable.getContextByIndex(index).getObject();
                self._addRiskFactorTreeItemToSelectedList(item);
            });
        } else {
            var model = self.riskFactorsList.getModel();
            var items = model.getData();
            var selectedItems = [];
            var selectedIndices = self.riskFactorsList.getSelectedIndices();
            _.each(selectedIndices, function(index) {
                selectedItems.push(self.riskFactorsList.getContextByIndex(index).getObject());
            });
            _.each(selectedItems, function(item) {
                var itemIndex = items.indexOf(item);
                if (itemIndex > -1) {
                    items.splice(itemIndex, 1);
                }
            });
            model.setData(items);
            model.updateBindings();
            self.riskFactorsList.clearSelection();
        }
    },

    _handleAllItemsButtonPressed: function() {
        var self = this;
        if (self._isAddingAvailableRiskFactors) {
            self.HprAPI.getAllRiskFactors().done(function (data) {
                if (data.total > data.threshold) {
                    self.HprUtil.displayGenericWarningDialog(self._getResourceBundle().getText('HPRisk.Sensitivities.Error.RiskFactor.Threshold.fmt',
                        [data.threshold, data.total]));
                }
                self._addRiskFactorsToSelectedList(data.riskFactors);
                self._updateButtonStates();
            }).fail(function (error) {
                self.HprUtil.displayGenericErrorDialog(self._getResourceBundle().getText('HPRisk.Sensitivities.Error.Retrieving.RiskFactors.txt'));
            });
        } else {
            var model = self.riskFactorsList.getModel();
            model.setData([]);
            model.updateBindings();
            self.riskFactorsList.clearSelection();
            self._updateButtonStates();
        }
    },

    _getSelectedRiskFactorsArray: function() {
        var self = this;
        return self.riskFactorsList.getModel().getData();
    },

    _handleRiskFactorTreeDoubleClick: function(event) {
        var self = this;

        self._isAddingAvailableRiskFactors = true;

        var addItem = event.getParameter("rowBindingContext").getObject();  // double-clicked risk factor tree item
        self._addRiskFactorTreeItemToSelectedList(addItem);
    },

    _handleRiskFactorsListDoubleClick: function(event) {
        var self = this;

        self._isAddingAvailableRiskFactors = false;

        var model = self.riskFactorsList.getModel();
        var items = model.getData();

        var deleteItem = event.getParameter("rowBindingContext").getObject();  // double-clicked risk factor item
        var itemIndex = items.indexOf(deleteItem);
        if (itemIndex > -1) {
            items.splice(itemIndex, 1);

            model.setData(items);
            model.updateBindings();
            self.riskFactorsList.clearSelection();
            self._updateButtonStates();
        }
    },

    _getResourceBundle: function() {
        return sas.analytics.riskhp.utils.HprUtil.getBundle();
    },

    _getDisplayName: function (name, label) {
        var self = this;
        var showLabels = self.dialog.getModel().getData().showLabels;
        if (showLabels && label && label.length > 0) {
            return label;
        } else {
            return name;
        }
    },

    createContent: function(ctrl) {
        var self = this;
        this.liveChangeTimer = 0;

        self.riskFactorsTreeTable = new sas.hc.ui.table.TreeTable({
            title: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.Sensitivities.Available.txt"),
            toolbar: this._createRiskFactorsTreeTableToolbar(self),
            columnHeaderVisible: false,
            columns: [
                new sas.hc.ui.table.Column({
                    width: "100%",
                    template: new sas.hc.m.Label({
                        text: {
                            parts: ["name", "label"],
                            formatter: $.proxy(self._getDisplayName, self)
                        },
                        // had to bind to "/" to get triggered when riskFactorsTreeTable.getModel().refresh(true) is called when items are added/remove from the riskFactorsList
                        enabled: {
                            parts: ["name", "/"],
                            formatter: function (name, whole) {
                                var isSelected = _.find(self.selectedRiskFactorsModel.getData(), function (rf) {
                                    return rf.name === name;
                                });
                                return !isSelected;
                            }
                        },
                        // tooltip: "{name}",
                        wrapping: false
                    }),
                    name: "name",
                    sortProperty: "name",
                    resizable: false
                })
            ],
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            minAutoRowCount: 1, // required for scrolling with < 5 rows
            enableColumnReordering: false,
            allowTextSelection: false,
            selectionBehavior: sas.hc.ui.table.SelectionBehavior.RowOnly,
            selectionMode: sas.hc.ui.table.SelectionMode.MultiToggle,
            rowSelectionChange: $.proxy(self._handleAvailableRiskFactorSelection, self),
            noData: new sas.risk.common.widgets.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATEALERT,
                text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.Sensitivities.ZeroState.txt")
            }),
            layoutData: new sap.m.FlexItemData({growFactor: 1})
        });
        self.riskFactorsTreeTable.attachDoubleClick(null, $.proxy(self._handleRiskFactorTreeDoubleClick, self));

        self.singleItemButton = new sas.hc.m.Button({
            text: "",
            icon: sas.icons.HC.ARROWRIGHTADDDUALSELECTOR,
            enabled: false,
            press: $.proxy(this._handleSingleItemButtonPressed, this)
        });

        self.allItemsButton = new sas.hc.m.Button({
            text: "",
            icon: sas.icons.HC.ARROWRIGHTADDALLDUALSELECTOR,
            enabled: true,
            press: $.proxy(this._handleAllItemsButtonPressed, this)
        });

        var buttonsFlexBox = new sas.hc.m.FlexBox({
            height: "100%",
            direction: sap.m.FlexDirection.Column,
            alignItems: sap.m.FlexAlignItems.Start,
            justifyContent: sap.m.FlexJustifyContent.Center,
            items: [self.singleItemButton, self.allItemsButton]
        });

        var availableLayout = new sas.hc.ui.commons.layout.BorderLayout({
            end: new sas.hc.ui.commons.layout.BorderLayoutArea({
                size: "2rem",
                content: [buttonsFlexBox]
            }),
            center: new sas.hc.ui.commons.layout.BorderLayoutArea({
                content: [self.riskFactorsTreeTable ]
            })
        });

        var selectedRiskFactorsSearchField = new sas.hc.m.SearchField({
            filterMode: true,
            liveChange: function (evt) {
                var filters = [
                    new sap.ui.model.Filter("name", sap.ui.model.FilterOperator.Contains, evt.getParameter("newValue"))
                ];
                self.riskFactorsList.getBinding("rows").filter(filters);
            }
        });
        self.CommonUtil.setSearchFieldLandmarkLabel(selectedRiskFactorsSearchField, self.HprUtil.getBundle().getText("HPRisk.Sensitivities.SearchSelectedRiskFactors.landmark.txt"));

        self.selectedRiskFactorsModel = new sas.risk.common.models.RiskJSONModel();
        self.selectedRiskFactorsModel.setData([]);
        self.riskFactorsList = new sas.hc.ui.table.Table({
            title: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.Sensitivities.Selected.txt"),
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            selectionMode: sas.hc.ui.table.SelectionMode.MultiToggle,
            selectionBehavior: sap.ui.table.SelectionBehavior.RowOnly,
            allowTextSelection: false,
            columnHeaderVisible: false,
            showColumnVisibilityMenu: false,
            enableColumnFreeze: false,
            toolbar: new sas.hc.m.Toolbar({
                content: [
                    new sas.hc.m.Toolbar({
                        content: [selectedRiskFactorsSearchField]
                    })
                ]
            }),
            rowSelectionChange: $.proxy(self._handleSelectedRiskFactorSelection, self),
            columns: [
                new sas.hc.ui.table.Column({
                    width: "100%",
                    template: new sas.hc.m.Label({
                        text: {
                            parts: ["name", "label"],
                            formatter: $.proxy(self._getDisplayName, self)
                        },
                        // tooltip: {
                        //     parts: ["name", "label"],
                        //     formatter: $.proxy(self._getDisplayName, self)
                        // },
                        wrapping: false
                    }),
                    name: "name",
                    sortProperty: "name",
                    resizable: false
                })
            ],
            layoutData: new sap.m.FlexItemData({growFactor: 1})
        }).addStyleClass("selectedRiskFactorsList");
        self.riskFactorsList.attachDoubleClick(null,$.proxy(self._handleRiskFactorsListDoubleClick, self));
        self.riskFactorsList.setModel(self.selectedRiskFactorsModel);
        self.riskFactorsList.setShowSelectionControls(false);
        var sorter = new sap.ui.model.Sorter("name", false); // sort ascending
        self.riskFactorsList.bindRows("/", null, sorter);
        self.riskFactorsList.allowTextSelection(false);
        self.riskFactorsList.getBinding().attachChange(function() {
            // enable/disable run button based on data in riskFactorsList
            runButton.setEnabled(self.riskFactorsList.getModel().getData().length > 0);

            // force the riskFactorsTreeTable to update visible nodes to indicate presence in riskFactorsList
            self.riskFactorsTreeTable.getModel().refresh(true);

        });
        var riskFactorsListLayout = new sas.hc.ui.commons.layout.BorderLayout({
            center: new sas.hc.ui.commons.layout.BorderLayoutArea({
                content: [self.riskFactorsList ]
            })
        });

        var splitter = new sas.hc.ui.layout.Splitter({
            contentAreas: [
                availableLayout,
                riskFactorsListLayout
            ],
            orientation: sap.ui.core.Orientation.Horizontal,
            width: "100%",
            height: "100%"
        });

        var showLabelsCheckBox = new sas.hc.m.CheckBox({
            text: this._getResourceBundle().getText("OptionsPane.generalOptions.showLabelsLabel.txt"),
            selected: "{/options/showLabels}",
            select: function(event) {
                var showLabels = event.getParameter("selected");
                self.dialog.getModel().getData().showLabels = showLabels;

                self.riskFactorsList.getModel().refresh(true);
                self.riskFactorsTreeTable.getModel().refresh(true);
            }
        });

        var editorLayout = new sas.hc.ui.commons.layout.BorderLayout({
            width: "100%",
            height: "100%",
            center: new sas.hc.ui.commons.layout.BorderLayoutArea({
                overflowX: "auto",
                overflowY: "auto",
                content: [splitter]
            }).addStyleClass("sasPropertyPaneLayout_paneArea_centerLayoutArea"),
            bottom: new sas.hc.ui.commons.layout.BorderLayoutArea({
                size: "2rem",
                content: [showLabelsCheckBox]
            })
        });


        var onCancel = function() {
            self.dialog.close();
        };

        var runButton = new sas.hc.m.Button({
            text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.Sensitivities.runButton.txt"),
            press: function() {
                self.dialog.setBusy(false);
                self.dialog.close();
                if (self.dialog.successCallback) {
                    self.dialog.successCallback(self._getSelectedRiskFactorsArray());
                }
            },
            enabled: false
        });

        this.dialog = new sas.hc.m.Dialog({
            title: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.Sensitivities.Dialog.title.txt"),
            content: [editorLayout],
            contentHeight: "75%",
            contentWidth: "75%",
            defaultButton: runButton,
            leftButton: runButton,
            rightButton: new sas.hc.m.Button({
                text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("hprisk.cancelButton.txt"),
                press: onCancel
            }),
            afterOpen: function(evt) {
                var dataSourceId = self.dialog.getModel().getData().dataSourceId;
                sas.analytics.riskhp.models.DataSourceCacheManager.getDataSource(dataSourceId).done(function(dataSource) {
                    var link = _.find(dataSource.json.links, {rel: "riskFactorTree"});

                    var model = new sas.hc.ui.model.PageableJSONTreeModel({
                        useHref: true,
                        url: link.href
                    });
                    self.riskFactorsTreeTable.setModel(model);
                    self.riskFactorsTreeTable.bindRows("/");
                });

                // showLabels is set to the value of showLabels of the options pane of the crosstab used for running rf sensitivity
                showLabelsCheckBox.setSelected(self.dialog.getModel().getData().showLabels);
            },
            afterClose: function(evt) {
                this.destroy();
            }
        });

        return this.dialog;
    }
});