// (c) 2017, SAS Institute Inc.

jQuery.sap.declare("sas.analytics.riskhp.sensitivities.RiskFactorSensitivitiesView");

jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.risk.common.panelsview.DivWrapper");
jQuery.sap.require("sas.risk.common.widgets.RiskColumnText");


/**
 * @name sas.analytics.riskhp.sensitivities.RiskFactorSensitivitiesView
 */
sap.ui.core.Control.extend("sas.analytics.riskhp.sensitivities.RiskFactorSensitivitiesView", {

    metadata: {
        properties: {
            width: {type: "sap.ui.core.CSSSize", defaultValue: null},
            height: {type: "sap.ui.core.CSSSize", defaultValue: null},
            rfsModel: {type: "sas.analytics.riskhp.sensitivities.RiskFactorSensitivitiesModel"}
        },
        events: {
            dirtyChange: {}
        }
    },

    HprUtil: sas.analytics.riskhp.utils.HprUtil,
    comma4: new sap.ui.model.type.Float({ decimals: 4, groupingEnabled: true }),
    table: null,

    renderer: function(rm, control) {
        if(!control.getVisible()) return;

        rm.write("<div");
        rm.writeControlData(control);
        if(control.getWidth()) {
            rm.addStyle("width", control.getWidth());
        }
        if(control.getHeight()) {
            rm.addStyle("height", control.getHeight());
        }

        rm.writeStyles();
        rm.writeClasses();
        rm.write(">");

        // render the RiskFactorSensitivities table
        if (!_.isUndefined(control.tableWrapper) && control.tableWrapper) {
            control.addDependent(control.tableWrapper);
            rm.renderControl(control.tableWrapper);
        }

        rm.write("</div>");
    },

    // the path and title may have changed
    _onDataItemLabelUpdate: function(channel, event, data) {
        var self = this;

        self._updatePathLabel();
    },

    _onShowLabelsChange: function(channel, event, data) {
        var self = this;

        self._updatePathLabel();
        self.table.getModel().refresh(true);
    },

    constructor: function(id, mSettings) {
        var self = this;

        // construct super
        sap.ui.core.Control.call(self, id, mSettings);

        // If the model is not valid there is not much we can do, so display the ZeroStateLayout
        if (!self.isModelValid()) {
            self._createZeroStateLayout(self.HprUtil.getBundle().getText("HPRisk.Sensitivities.zeroState.query.problem.txt"));
            return;
        }
        self.getRfsModel().initializeModel().done(
            function () {
                // We need the wrapper so we can properly contain the table by positioning
                // it absolutely without layering on top of the popout menu for the panel
                // (the swizzle).
                self.tableWrapper = new sas.risk.common.panelsview.DivWrapper({
                    height: "100%",
                    width: "100%",
                    content: self._createTable()
                });
                self.tableWrapper.addStyleClass("hprBoundedControlContainer");


                // now that the dataSource is in the model, update path label
                self._updatePathLabel();
            }
        ).fail(
            function (jqxhr, textStatus, error) {
                console.log("Risk Factor Sensitivities model failed to load: " + jqxhr);
                // self.setBusy(false);
                self._createZeroStateLayout(self.HprUtil.getBundle().getText("HPRisk.Sensitivities.zeroState.query.problem.txt"));
            }
        );

        //
        // Subscribe to an event that is fired when the label for a Data Item is updated
        sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_DATA_ITEM_LABEL_UPDATED, self._onDataItemLabelUpdate, self);

        // Subscribe to an event that is fired when the 'Show labels' option field was updated
        sas.eventBus.subscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_RISK_FACTOR_SENSITIVITIES_OPTIONS_SHOW_LABELS_UPDATED, this._onShowLabelsChange, this);

    },

    exit: function() {
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_DATA_ITEM_LABEL_UPDATED, this._onDataItemLabelUpdate, this);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_RISK_FACTOR_SENSITIVITIES_OPTIONS_SHOW_LABELS_UPDATED, this._onShowLabelsChange, this);
    },


    // public functions

    /**
     * Pseudo interface method to return the dataSourceId the component is using in a common way
     * @returns String
     */
    getDataSourceId: function() {
        return this.getRfsModel().dataSourceId;
    },

    /**
     * Pseudo interface method to return the type of control this is in a common way
     * @returns String
     */
    getPanelType: function() {
        return HprConstants.HPR_RISK_FACTOR_SENSITIVITIES_PANEL;
    },

    /**
     * Called from HPRiskPanelsViewDelegate.getContentMetadata() when saving the control.
     * @returns {Object} the query data for this control
     */
    getQueryData: function() {
        return this.getRfsModel().toJSON();
    },

    /**
     * Returns the toolbar actions for the view.
     * @returns {Array}
     */
    getToolbarActions: function() {
        return [];  // no additional actions to contribute yet
    },

    handleResize: function() {
    },

    /**
     * Requeries the data
     */
    // rebuild: function() {
    //     this.table.getModel().clearData();
    // },

    // private functions



    _updatePathLabel: function() {
        var self = this;
        self.pathLabel.setText(self.getRfsModel().createPathString());
    },

    /**
     *
     * @private
     */
    _createSensitivitiesTableToolbar: function() {
        var self = this;

        self.pathLabel = new sas.hc.m.Label();
        self._updatePathLabel();

        return new sas.hc.m.ResponsiveToolbar({
            content: self.pathLabel
        }).addStyleClass("hprPortfolioDetailsPathBar");
    },


    /**
     * Checks to see if this view's model is valid or not.  It currently checks the model for an empty 'tuples' array.
     * @returns {boolean} Returns true if this view's model is valid; otherwise, it returns false
     */
    isModelValid: function() {
        var model = this.getRfsModel();

        if (_.isUndefined(model) || !model) {
            return false;
        }

        return model.isValid();
    },

    /**
     * This method replaces the currently displayed Table with a ZeroStateLayout with a detailed message (if provided).  The 'zeroStateText'
     * string is displayed in the ZeroStateLayout if it is provided; otherwise, a default message is displayed.
     * @param zeroStateText String Message to display within the ZeroStateLayout
     * @private
     */
    _createZeroStateLayout: function(zeroStateText) {
        // Determine the correct text to display in the ZeroState
        var zeroStateLayoutText = zeroStateText ? zeroStateText : this.HprUtil.getBundle().getText("HPRisk.Sensitivities.zeroState.default.txt");

        // Construct a ZeroStateLayout to display inside the RiskPanelsView
        var zeroStateLayout = new sas.risk.common.widgets.ZeroStateLayout({
            icon: sas.icons.HC.ZEROSTATETABLE,
            title: zeroStateLayoutText,
            buttons: []
        });

        if (!_.isUndefined(this.tableWrapper) && this.tableWrapper) {
            // Destroy the content of the tableWrapper so we can replace it with the ZeroStateLayout
            this.tableWrapper.destroyContent();
        } else {
            // tableWrapper does not exist yet, have to create it
            this.tableWrapper = new sas.risk.common.panelsview.DivWrapper({
                height: "100%",
                width: "100%"
            });
            this.tableWrapper.addStyleClass("hprBoundedControlContainer");
        }

        this.tableWrapper.setContent(zeroStateLayout);
    },


    /**
     * Creates the table displayed by this control.
     * @private
     */
    _createTable: function() {
        var aColumns = [];
        var self = this;

        aColumns.push(this._createColumn("id", "HPRisk.Sensitivities.column.title.name.txt", sap.ui.core.TextAlign.Begin));
        aColumns.push(this._createColumn("baseValue", "HPRisk.Sensitivities.column.title.basevalue.txt", sap.ui.core.TextAlign.Right));
        aColumns.push(this._createColumn("delta", "HPRisk.Sensitivities.column.title.delta.txt", sap.ui.core.TextAlign.Right));
        if (!self.getRfsModel().isDeltaNormal()) {
            // don't get gamma back for delta normal data sources so remove column
            aColumns.push(this._createColumn("gamma", "HPRisk.Sensitivities.column.title.gamma.txt", sap.ui.core.TextAlign.Right));
        }
        this.table = new sas.hc.ui.table.Table({
            toolbar: self._createSensitivitiesTableToolbar(),
            selectionMode: sas.hc.ui.table.SelectionMode.None,
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            enableColumnFreeze: false,
            minAutoRowCount: 1, // required for scrolling with < 5 rows
            showTableOptions: false,
            editable: false,
            columns: aColumns
        });
        this.table.addStyleClass("hprBoundedControl");
        this.table.addStyleClass("hprPortfolioTable");

        this.table.setModel(this._buildTableModel());
        this.table.bindRows("/values");

        this.table.setBusy(true);
        var successCallback = function() {
            self._rebuildTable();
            self.table.setBusy(false);
        };
        var failureCallback = function() {
            self.rerender();
            self.table.setBusy(false);
        };
        var rfsModel = this.getRfsModel();
        rfsModel.executeQuery(successCallback, failureCallback);
        return this.table;
     },

    _buildTableModel: function() {
       return new sap.ui.model.json.JSONModel({"values":this.getRfsModel().getData()})
    },

    // called after successful query execution
    _rebuildTable: function() {
        // If the model for this view is invalid we can't rebuild the table
        if (!this.isModelValid()) {
            return;
        }
        this.table.setModel(this._buildTableModel());
    },

    /**
     * Factory function for creating table a column.
     * @param bindingKey
     * @param titleKey
     * @returns {sas.analytics.riskhp.portfolio.PortfolioDetailsColumn}
     * @private
     */
    _createColumn: function(bindingKey, titleKey, hAlign) {
        var self = this;
        return new sas.hc.ui.table.Column({
            label: new sas.hc.m.Label({
                textAlign: hAlign,
                text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText(titleKey),
            }),
            template: new sas.risk.common.widgets.RiskColumnText({
                textAlign: hAlign,
                text: {
                    path: bindingKey,
                    formatter: function(value) {
                        switch(bindingKey) {
                            case "id":
                                var rfs = self.getRfsModel().riskFactors;
                                var riskFactor = _.find(rfs, function(rf){
                                    return rf.id === value;
                                });
                                if (riskFactor) {
                                    if (self.getRfsModel().getOptionShowLabels() && riskFactor.label && riskFactor.label.length > 0) {
                                        return riskFactor.label;
                                    } else {
                                        return riskFactor.name;
                                    }
                                } else {
                                    return value;
                                }
                            case "baseValue":
                            case "delta":
                            case "gamma":
                                if (isNaN(value)) {
                                    return value;
                                } else {
                                    return self.comma4.formatValue(value, 'string');
                                }
                            default:
                                return "default";
                        }
                    }
                }
            }),
            sortProperty: bindingKey
        });
    }
});