// (c) 2017, SAS Institute Inc.

jQuery.sap.declare("sas.analytics.riskhp.settings.SettingsProvider");

jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
jQuery.sap.require('sas.analytics.riskhp.utils.HprUtil');
jQuery.sap.require("sas.hc.ui.commons.formControl.Field");
jQuery.sap.require("sas.hc.ui.commons.settings.Settings");
jQuery.sap.require("sas.hc.ui.commons.settings.providers.AbstractSettingsProvider");

(function() {

    var Field = sas.hc.ui.commons.formControl.Field,
        SETTINGS_PREFIX_ID = "hpriskui-settings";

    sas.hc.ui.commons.settings.providers.AbstractSettingsProvider.extend("sas.analytics.riskhp.settings.SettingsProvider", {

        rb: sas.analytics.riskhp.utils.HprUtil.getBundle(),

        getPreferences: function() {
            return this.createGroup(this.rb.getText("hprisk.settings.group.label"), [], [this._createPreferencesSubGroup()], SETTINGS_PREFIX_ID + "-preferences-subgroup");
        },

        _createPreferencesSubGroup: function() {
            return this.createGroup(this.rb.getText('hprisk.settings.preferences.subGroup.label'), this._createSettingsFields(), []);
        },

        _createSettingsFields: function() {
            var self = this;

            /* TODO: Comment out for now, not part of initial Polling feature (HPR-1842)
            var showStalenessIndication = this.createField(HprConstants.HPR_SETTINGS_SHOW_STALENESS_NOTIFICATION, Field.BOOLEAN_TYPE, this.rb.getText('hprisk.settings.staleness.notification.label'), false);
            showStalenessIndication.setSelectionType("multiple");*/

            var showLabels = new Field({
                id: HprConstants.HPR_SETTINGS_SHOW_LABELS,
                label: self.rb.getText('hprisk.settings.show.labels.label'),
                defaultValue: HprConstants.HPR_SETTINGS_SHOW_LABELS_DEFAULT_VALUE,
                type: Field.BOOLEAN_TYPE,
                selectionType: 'multiple'
            });

            /* TODO: Comment out for now, Priority 3
            var includePlotImage = this.createField(HprConstants.HPR_SETTINGS_INCLUDE_PLOT_IMAGE, Field.BOOLEAN_TYPE, this.rb.getText('hprisk.settings.include.plot.image.label'), false);
            includePlotImage.setSelectionType("multiple");

            var plotWidthDimension = new Field({
                id: HprConstants.HPR_SETTINGS_PLOT_WIDTH_DIMENSION,
                label: self.rb.getText('hprisk.settings.export.width.dimension.label'),
                labelAlignment: "top",
                styleClass: "ApplicationSettings_Field_With_Left_Bottom_Adjustment", // FormControl appends 'Cntr' to the end of the style class
                description: self.rb.getText('hprisk.settings.export.width.dimension.desc.txt'),
                type: Field.INTEGER_TYPE,
                width: "75px",
                defaultValue: HprConstants.HPR_SETTINGS_PLOT_WIDTH_DIMENSION_DEFAULT_VALUE,
                constraints: {min: HprConstants.HPR_SETTINGS_PLOT_WIDTH_DIMENSION_MIN_VALUE, max: HprConstants.HPR_SETTINGS_PLOT_WIDTH_DIMENSION_MAX_VALUE}
            });

            var plotHeightDimension = new Field({
                id: HprConstants.HPR_SETTINGS_PLOT_HEIGHT_DIMENSION,
                label: self.rb.getText('hprisk.settings.export.height.dimension.label'),
                labelAlignment: "top",
                styleClass: "ApplicationSettings_Field_With_Left_Adjustment", // FormControl appends 'Cntr' to the end of the style class
                description: self.rb.getText('hprisk.settings.export.height.dimension.desc.txt'),
                type: Field.INTEGER_TYPE,
                width: "75px",
                defaultValue: HprConstants.HPR_SETTINGS_PLOT_HEIGHT_DIMENSION_DEFAULT_VALUE,
                constraints: {min: HprConstants.HPR_SETTINGS_PLOT_HEIGHT_DIMENSION_MIN_VALUE, max: HprConstants.HPR_SETTINGS_PLOT_HEIGHT_DIMENSION_MAX_VALUE}
            });*/

            /* /* TODO: Comment out for now, not part of initial Polling feature (HPR-1842)
            var cubePolling = this.createField(HprConstants.HPR_SETTINGS_CUBE_POLLING, Field.BOOLEAN_TYPE, this.rb.getText('hprisk.settings.cube.polling.label'), false);
            cubePolling.setSelectionType("multiple");

            var cubePollingInterval = new Field({
                id: HprConstants.HPR_SETTINGS_CUBE_POLLING_INTERVAL,
                label: self.rb.getText('hprisk.settings.cube.polling.interval.label'),
                labelAlignment: "top",
                styleClass: "ApplicationSettings_Field_With_Left_Adjustment", // FormControl appends 'Cntr' to the end of the style class
                defaultValue: HprConstants.HPR_SETTINGS_CUBE_POLLING_INTERVAL_DEFAULT_VALUE,
                uiConfig: new sas.hc.ui.commons.formControl.uiconfig.ClassInstanceUIConfig({
                    generator: function(oData, oDataProvider, oConstraints, sSelectionType) {
                        // Build a NumericStepper
                        // NOTE: 'change' event is picked up correctly by ApplicationSettings.setCubePollingInterval() when
                        // the value is changed.  I guess the Field/FormControl does that because of the binding of 'value'
                        var oControl = new sas.hc.m.NumericStepper({
                            stepSize: 1,
                            value: {
                                path: oData.value,
                                type: new sap.ui.model.type.Integer(null, {
                                    minimum: HprConstants.HPR_SETTINGS_CUBE_POLLING_INTERVAL_MIN_VALUE,
                                    maximum: HprConstants.HPR_SETTINGS_CUBE_POLLING_INTERVAL_MAX_VALUE
                                })
                            },
                            liveChange: function(evt) {
                                // S1380415 - Perform validation as they type so an error will be
                                // present if they hit the Close button
                                var source = evt.getSource();
                                source._performValidation();
                            }
                        });
                        return oControl;
                    }
                })
            });*/

            var displayHorizons = new Field({
                id: HprConstants.HPR_SETTINGS_DISPLAY_HORIZONS,
                label: self.rb.getText('hprisk.settings.display.horizons.label'),
                labelAlignment: "top",
                styleClass: "ApplicationSettings_Field_With_Top_Bottom_Adjustment",
                defaultValue: HprConstants.DISPLAY_HORIZONS_ABSOLUTE_VALUE,
                dataProvider: [
                    { key: HprConstants.DISPLAY_HORIZONS_ABSOLUTE_VALUE, text: self.rb.getText("hprisk.settings.display.horizons.absolute.txt") },
                    { key: HprConstants.DISPLAY_HORIZONS_RELATIVE_VALUE, text: self.rb.getText("hprisk.settings.display.horizons.relative.txt") }
                ]
            });

            // This field maps to the old NDETAILS preference field used in Flex version; However, the values
            // for that field are contained in the "options" data of the cube; Since these Application Settings can be displayed
            // in the new HTML5 UI before we have a cube open, we must fake out the values
            var portfolioDetailsPageSize = new sas.hc.ui.commons.formControl.Field({
                id: HprConstants.HPR_SETTINGS_PORTFOLIO_DETAILS_PAGE_SIZE,
                label: self.rb.getText("hprisk.settings.portfolio.details.page.size.label"),
                labelAlignment: "top",
                defaultValue: HprConstants.HPR_SETTINGS_PORTFOLIO_DETAILS_PAGE_SIZE_DEFAULT_VALUE,
                width: "170px",
                uiConfig: new sas.hc.ui.commons.formControl.uiconfig.ClassInstanceUIConfig({
                    generator: function(oData, oDataProvider, oConstraints, sSelectionType) {
                        // Build a NumericStepper
                        // NOTE: 'change' event is picked up correctly by ApplicationSettings.setPortfolioDetailsPageSize() when
                        // the value is changed.  I guess the Field/FormControl does that because of the binding of 'value'
                        var oControl = new sas.hc.m.NumericStepper({
                            stepSize: 1,
                            value: {
                                path: oData.value,
                                type: new sap.ui.model.type.Integer(null, {
                                    minimum: HprConstants.HPR_SETTINGS_PORTFOLIO_DETAILS_PAGE_SIZE_MIN_VALUE,
                                    maximum: HprConstants.HPR_SETTINGS_PORTFOLIO_DETAILS_PAGE_SIZE_MAX_VALUE
                                })
                            },
                            liveChange: function(evt) {
                                // S1380415 - Perform validation as they type so an error will be
                                // present if they hit the Close button
                                var source = evt.getSource();
                                source._performValidation();
                            }
                        });
                        return oControl;
                    }
                })
            });

            //return [ showStalenessIndication, showLabels, /*includePlotImage, plotWidthDimension, plotHeightDimension,*/
            //    cubePolling, cubePollingInterval, displayHorizons, portfolioDetailsPageSize ];
            return [ showLabels, displayHorizons, portfolioDetailsPageSize ];
        },

        loadSettings: function() {
            var service = this.getSettingsService();
            if (service) {
                return service.get({
                    "data": {
                        "name": [
                            HprConstants.HPR_SETTINGS_SHOW_STALENESS_NOTIFICATION,
                            HprConstants.HPR_SETTINGS_SHOW_LABELS,
                            HprConstants.HPR_SETTINGS_INCLUDE_PLOT_IMAGE,
                            HprConstants.HPR_SETTINGS_PLOT_WIDTH_DIMENSION,
                            HprConstants.HPR_SETTINGS_PLOT_HEIGHT_DIMENSION,
                            HprConstants.HPR_SETTINGS_CUBE_POLLING,
                            HprConstants.HPR_SETTINGS_CUBE_POLLING_INTERVAL,
                            HprConstants.HPR_SETTINGS_DISPLAY_HORIZONS,
                            HprConstants.HPR_SETTINGS_PORTFOLIO_DETAILS_PAGE_SIZE
                        ]
                    },
                    "async": false
                });
            }
            else {
                var deferred = $.Deferred();
                deferred.reject();
                return deferred;
            }
        },

        getSettingsService: function() {
            if (sas.hc.ui.commons.settings.getSettingsManager &&
                sas.hc.ui.commons.settings.getSettingsManager().isServiceAvailable()) {
                return sas.hc.ui.commons.settings.getSettingsManager().getService();
            }
        }
    });
})();