jQuery.sap.declare("sas.analytics.riskhp.utils.HPRiskPanelsViewDelegate");

jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.crosstab.HprCrosstab");
jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprChartModel");
jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprChartView");
jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils");
jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.table.TableViewer");
jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.table.TableViewerUtil");

jQuery.sap.require("sas.analytics.riskhp.models.HPRiskModelManager");
jQuery.sap.require("sas.analytics.riskhp.portfolio.PortfolioDetailsViewer");
jQuery.sap.require("sas.analytics.riskhp.portfolio.PortfolioDetailsModel");
jQuery.sap.require("sas.analytics.riskhp.portfolio.PortfolioDetailsUtil");
jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.analytics.riskhp.sensitivities.RiskFactorSensitivitiesModel");
jQuery.sap.require("sas.analytics.riskhp.sensitivities.RiskFactorSensitivitiesView");

jQuery.sap.require("sas.risk.common.panelsview.DivWrapper");
jQuery.sap.require("sas.risk.common.panelsview.RiskPanelsViewDelegate");
jQuery.sap.require("sas.risk.common.widgets.ZeroStateLayout");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");

/**
 * @name sas.analytics.riskhp.utils.HPRiskPanelsViewDelegate
 */
sas.risk.common.panelsview.RiskPanelsViewDelegate.extend("sas.analytics.riskhp.utils.HPRiskPanelsViewDelegate", {

    metadata: {
        publicMethods: [
            // Something goes here
        ]
    },

    // pseudo-imports
    HprUtil: sas.analytics.riskhp.utils.HprUtil,
    HprAPI: sas.analytics.riskhp.utils.HprAPI,

    //*******************************************
    // RiskPanelsViewDelegate implementation methods
    //*******************************************

    /**
     * Constructor which creates some useful variables that are needed to keep track of content and titles for this Delegate.
     *
     * @param sId
     * @param mProperties
     */
    constructor: function(sId, mProperties) {
        this.contentTitleMap = {}; // An object that holds all the titles for this Delegate's content
        this.contentFooterMap = {}; // An object that holds all the footers for this Delegate's content
        this.contentCountMap = {};  // An object that holds the count of each type of this Delegate's content
        this.crosstabIds = []; // Solution specific so the view can know the ids of the Crosstabs being displayed
        this.plotIds = []; // Solution specific so the view can know the ids of the Plots being displayed
        this.portfolioDetailsIds = []; // Solution specific so the view can know the ids of the Portfolio Details being displayed
        this.tableIds = []; // Solution specific so the view can know the ids of the tables being displayed
        this.riskFactorSensitivitiesIds = [];
        this.selectedContentId = null;
        this.renderCounts = {};
    },

    /**
     * Create the correct component based on the content of 'newContentData'.  This method is called when re-creating components from a
     * saved Exploration, as well as, when creating new Plots or dragging Data Items from the Data pane.  Re-creating a Crosstab and Plot
     * sends a string, new Plots send an object, and dragging a Data Item sends an array of table rows (always length of 1).
     * This method will either create a Crosstab, Plot, or return null.
     *
     * @param newContentData Either a String, Object, or Array
     * @param event
     * @returns {*} A Crosstab, Plot, or null
    */
    // FOR_REVIEW: DroppableContainer delegate method names need to be reviewed and probably changed to indicate their real intent
    createContent: function(newContentData, event) {
        var self = this;
        var content = null;

        if (_.isUndefined(newContentData)) {
            return this.createDefaultCrosstab();
        }

        // Since this method is ran in different scenarios (i.e. when re-creating components from a saved Exploration and
        // when creating new Plots, dragging Data Items, etc.), protect ourselves because saved Crosstabs and Plots send a
        // string, new Plots send an object, and dragging Data Items send an Array
        var newContentDataObject = newContentData;
        // If we have a String, convert to Object; Don't worry about the Array just yet
        if (_.isString(newContentData)) {
            newContentDataObject = JSON.parse(newContentData);
        }

        // Create a Crosstab
        if (newContentDataObject.type === HprConstants.HPR_CROSSTAB_PANEL) {
            return self._createOrRestoreCrosstab(newContentDataObject);
        }

        // Create a RiskFactorSensitivities table
        if (newContentDataObject.type === HprConstants.HPR_RISK_FACTOR_SENSITIVITIES_PANEL) {
            return self._createOrRestoreRiskFactorSensitivities(newContentDataObject);
        }

        // Create a Plot
        if (newContentDataObject.type === HprConstants.HPR_PLOT_PANEL) {
            return self._createOrRestorePlot(newContentDataObject);
        }

        // portfolio details
        if (newContentDataObject.type === HprConstants.HPR_PORTFOLIO_DETAILS_PANEL) {
            var isPanelRestoration = false;
            var portfolioDetailsId = HprConstants.HPR_PORTFOLIO_DETAILS_COMPONENT_ID + Date.now();
            if (newContentDataObject.name) {
                isPanelRestoration = true;
                portfolioDetailsId = newContentDataObject.name;
            }

            // The model id needs to match the component id in order for the Options to work correctly
            var portfolioModel = new sas.analytics.riskhp.portfolio.PortfolioDetailsModel(portfolioDetailsId, newContentDataObject, isPanelRestoration);

            // Add a placeholder  title to hashmap
            this.contentTitleMap[portfolioDetailsId] = "";

            var portfolioDetails = new sas.analytics.riskhp.portfolio.PortfolioDetailsViewer(portfolioDetailsId, {
                portfolioModel: portfolioModel,
                dirtyChange: $.proxy(self.onPortfolioDetailsDirtyChange, self)
            });
            self._renderCounter(portfolioDetailsId);

            // Add title and footer to internal maps
            var dataSourceId = newContentDataObject.dataSourceId;
            var deferred = DataSourceCacheManager.getDataSource(dataSourceId);
            deferred.done(function(dataSource) {
                if (isPanelRestoration === false) {
                    var title = sas.analytics.riskhp.portfolio.PortfolioDetailsUtil.generateDefaultPortfolioTitle(dataSource, portfolioModel.getRiskCubeId());
                    self.contentTitleMap[portfolioDetailsId] = title;
                    portfolioModel.setOptionName(title);
                }
                else {
                    self.contentTitleMap[portfolioDetailsId] = portfolioModel.getOptionName();
                }
                var dataSourceParam = DataSourceCacheManager.getDataSourceParam(dataSourceId);
                if (dataSourceParam) {
                    self.contentFooterMap[portfolioDetailsId] = dataSourceParam.label;
                }
                sas.analytics.riskhp.utils.HprUtil.getSelectedRiskPanelsView().refreshPanelWithId(portfolioDetailsId, false, null);
            });
            deferred.fail(function() {
                self.contentTitleMap[portfolioDetailsId] = portfolioModel.getOptionName();
            });

            // Only add the portfolio details id if it doesn't exist in the array yet
            if (!_.contains(this.portfolioDetailsIds, portfolioDetailsId)) {
                this.portfolioDetailsIds.push(portfolioDetailsId);
            }

            return portfolioDetails;
        }

        // Simulated Values
        if (newContentDataObject.type === HprConstants.HPR_SIMULATED_VALUES_PANEL) {
            var isPanelRestoration = false;
            var simulatedValuesId = HprConstants.HPR_SIMULATED_VALUES_COMPONENT_ID + Date.now();
            if (newContentDataObject.name) {
                isPanelRestoration = true;
                simulatedValuesId = newContentDataObject.name;
            }

            // The model id needs to match the component id in order for the Options to work correctly
            var tableViewerModel = new sas.analytics.riskhp.common.riskcommons.table.TableViewerModel(simulatedValuesId, newContentDataObject, isPanelRestoration);

            // Add a placeholder  title to hashmap
            this.contentTitleMap[simulatedValuesId] = "";

            var simulatedValuesTableViewer = new sas.analytics.riskhp.common.riskcommons.table.TableViewer(simulatedValuesId, {
                tableViewerModel: tableViewerModel,
                dirtyChange: $.proxy(self.onTableViewerDirtyChange, self)
            });
            self._renderCounter(simulatedValuesId);

            // Add title and footer to internal maps
            var dataSourceId = newContentDataObject.dataSourceId;
            var deferred = DataSourceCacheManager.getDataSource(dataSourceId);
            deferred.done(function(dataSource) {
                if (isPanelRestoration === false || tableViewerModel.getOptionsNameChanged() === false) {
                    var title = sas.analytics.riskhp.common.riskcommons.table.TableViewerUtil.generateDefaultTableTitle(dataSource, tableViewerModel);
                    self.contentTitleMap[simulatedValuesId] = title;
                    tableViewerModel.setOptionName(title);
                }
                else {
                   self.contentTitleMap[simulatedValuesId] = tableViewerModel.getOptionName();
                }
                var dataSourceParam = DataSourceCacheManager.getDataSourceParam(dataSourceId);
                if (dataSourceParam) {
                    self.contentFooterMap[simulatedValuesId] = dataSourceParam.label;
                }
                sas.analytics.riskhp.utils.HprUtil.getSelectedRiskPanelsView().refreshPanelWithId(simulatedValuesId, false, null);
            });
            deferred.fail(function() {
                self.contentTitleMap[simulatedValuesId] = tableViewerModel.getOptionName();
            });

            // Only add the portfolio details id if it doesn't exist in the array yet
            if (!_.contains(this.tableIds, simulatedValuesId)) {
                this.tableIds.push(simulatedValuesId);
            }

            return simulatedValuesTableViewer;
        }

        // If we got here, we have dragged a Data Item(s) from the Data Pane or an item from the content pane
        var control = undefined;
        if (newContentData && newContentData[0] && newContentData[0].id) {
            var id = newContentData[0].id;
            control = sap.ui.getCore().byId(id);
        }
        if (_.isUndefined(control)) {
            return null;
        }

        // At this point we expect the control to have a type attribute attached to it's custom data so we know
        // what to build with it.
        var data = control.data();
        switch (data['type']) {
            case HprConstants.HPR_CROSSTAB_PANEL:
                content = this.createDefaultCrosstab();
                break;
            case HprConstants.HPR_LIST_TABLE_PANEL:
                content = new sas.risk.common.panelsview.DivWrapper({});
                this.contentTitleMap[content.getId()] = 'List Table';
                content.addStyleClass('testDivRed');
                break;
        }

        return content;
    },

    /**
     * Called when the RiskPanelsView is told to refresh it's panels, whether that be one panel or all of them.
     * @param id Id of the content to refresh
     * @param actionData Any data that was created when the refresh action was called to be passed to this method.
     */
    refreshContent: function(id, actionData) {
        var control = sap.ui.getCore().byId(id);
        var self = this;
        if (!control) {
            return;
        }

        if (_.indexOf(this.plotIds, id) !== -1) {
            console.log("Refreshing plot with id: " + id + " with data: " + JSON.stringify(actionData));

            var chartModel = control.getChartModel();
            self.contentTitleMap[chartModel.getId()] = chartModel.getOptionName();

            // Make sure the control is really a Plot and call the rebuildChart method
            if ($.isFunction(control.getChart)) {
                control.rebuild();
            }
        }
        else if (_.indexOf(this.crosstabIds, id) !== -1) {
            console.log("Refreshing crosstab with id: " + id + " with data: " + JSON.stringify(actionData));

            // Make sure the control is really a Crosstab and call the redraw method
            if ($.isFunction(control.getCrosstab)) {
                control.redraw();
                control.rerender(); // Redraw method for Crosstab does not rerender, so explicitly call it in case we need to update
            }
        }
    },

    /**
     * Decide whether this control can handle the array of draggables.  Note this only is consulted by the panels and
     * the drop zones.  Individual controls will need to handle the drop life cycle on thier own.
     *
     * @param draggables An array of draggable items in the current drag context that are being dragged around the window
     * @returns {boolean} Whether this control can handle the list of draggables
     */
    canAcceptDrop: function(draggables) {
        if (draggables && draggables.length === 1) {
            var id = draggables[0].id;
            var control = sap.ui.getCore().byId(id);
            if (_.isUndefined(control)) {
                return false;
            }
            var data = control.data();
            switch (data['type']) {
                case HprConstants.HPR_CROSSTAB_PANEL:
                    return true;
                case HprConstants.HPR_LIST_TABLE_PANEL:
                    return true;
                default:
                    return false;
            }
        }
        return false;
    },

    setSelectedContentId: function(id) {
        this.selectedContentId = id;
    },

    /**
     * @description Changes the title for the content with the passed in panelId to the passed in newTitle.  If this
     * delegate instance does not know about a panel with the given panelId then this is a no-op.
     * @param {string} newTitle
     * @param {string} panelId
     */
    setTitleForContent: function(newTitle, panelId) {
        if (_.has(this.contentTitleMap,panelId)) {
            this.contentTitleMap[panelId] = newTitle;
        }
    },

    getTitleForContent: function(id) {
        var title = this.contentTitleMap[id];
        var titleQualifier = "";

        // If this panel contains a crosstab, then we also need to indicate Forced Hierarchy mode (if ON).
        if (_.find(this.crosstabIds, function(crosstabId) { return crosstabId === id; })) {
            var crosstab = sap.ui.getCore().byId(id);
            if (crosstab !== undefined) {
                var context = crosstab.context;
                if (context && context.isForcedHierarchyMode()) {
                    titleQualifier = " (" + sas.analytics.riskhp.utils.HprUtil.getBundle().getText('Hierarchy.dialog.title.txt') + ")";
                }
            }
        }
        // Allow empty strings as titles
        if (!_.isUndefined(title)) {
            return title + titleQualifier;
        }
        return sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.unknown.content.title.txt');
    },

    getFooterForContent: function(id) {
        return this.contentFooterMap[id];
    },

    getToolbarButtonsForContentWithId: function(id) {
        if (this.plotIds.indexOf(id) !== -1) {
            var chartView = sap.ui.getCore().byId(id);
            if (chartView) {
                return chartView.getToolbarActions();
            }
        }
        if (this.tableIds.indexOf(id) !== -1) {
            var tableView = sap.ui.getCore().byId(id);
            if (tableView) {
                return tableView.getToolbarActions();
            }
        }
        return [];
    },

    getOverflowMenuActionsForContentWithId: function(id) {
        if (this.plotIds.indexOf(id) !== -1) {
            var chartView = sap.ui.getCore().byId(id);
            if (chartView) {
                return chartView.getOverflowMenuActions();
            }
        }
        if (this.portfolioDetailsIds.indexOf(id) !== -1) {
            var portfolioDetailsView = sap.ui.getCore().byId(id);
            if (portfolioDetailsView) {
                return portfolioDetailsView.getToolbarActions();
            }
        }
        if (this.crosstabIds.indexOf(id) !== -1) {
            var crosstabView = sap.ui.getCore().byId(id);
            if (crosstabView) {
                return crosstabView.getToolbarActions();
            }
        }
        if (this.riskFactorSensitivitiesIds.indexOf(id) !== -1) {
            var riskFactorSensitivitiesView = sap.ui.getCore().byId(id);
            if (riskFactorSensitivitiesView) {
                return riskFactorSensitivitiesView.getToolbarActions();
            }
        }
        if (this.tableIds.indexOf(id) !== -1) {
            var tableView = sap.ui.getCore().byId(id);
            if (tableView) {
                return tableView.getOverflowMenuActions();
            }
        }
        return [];
    },

    createZeroStateContent: function() {
        // Update the right-side panels to account for a Zero State
        sas.analytics.riskhp.utils.HprUtil.updateRightPanelsForZeroState(true);

        // Timestamp shim in case browser is non-modern
        if (!Date.now) {
            Date.now = function() { return new Date().getTime(); };
        }

        var zeroStateLayout = new sas.risk.common.widgets.ZeroStateLayout({
            icon: sas.icons.HC.ZEROSTATETABLE,
            text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText('HPRisk.zeroState.default.txt'),
            buttons: []
        });

        // Create a DivWrapper with an unique id that will hold the ZeroStateLayout
        var zeroStateId = HprConstants.HPR_ZERO_STATE_COMPONENT_ID + Date.now();
        var zeroStateWrapper = new sas.risk.common.panelsview.DivWrapper({
            id: zeroStateId,
            height: "100%",
            width: "100%"
        });
        zeroStateWrapper.addStyleClass("hprBoundedControlContainer"); // Needed so content stays within RiskPanelsView
        zeroStateWrapper.setContent(zeroStateLayout);

        this.contentTitleMap[zeroStateId] = 'Zero State'; // TODO: Need better title

        this.selectedContentId = zeroStateId; // Set the selected content for this Delegate

        return zeroStateWrapper;
    },

    createInitialContent: function() {
        return this.createDefaultCrosstab();
    },

    // addedContent: function() {
    //     var panelsView = sas.analytics.riskhp.utils.HprUtil.getSelectedRiskPanelsView();
    //     var parent = panelsView.getParent();
    //     var _isSplitter = function (control) {
    //         if (_.isUndefined(control) || control === null) {
    //             return false;
    //         }
    //         var metadata = control.getMetadata();
    //         console.log(metadata["_sClassName"]);
    //         return (metadata["_sClassName"].indexOf('Splitter') !== -1);
    //     };
    //
    //     while (!_isSplitter(parent)) {
    //         console.log("Traversing container with id: "+parent.getId());
    //         parent = parent.getParent();
    //     }
    //
    //     parent.rerender();
    // },

    /**
     * Callback method when content is removed from the panels view.  We need to clean up any cache's here
     * @param id
     */
    removedContentWithId: function(id) {
        delete this.contentTitleMap[id];
        delete this.contentFooterMap[id];

        this.contentCountMap = {};  // An object that holds the count of each type of this Delegate's content
        var index = _.indexOf(this.crosstabIds,id);
        if (index > -1) {
            this.crosstabIds.splice(index,1);
            sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_CROSSTAB_DELETED, {
                id: id
            });
        }
        index = _.indexOf(this.plotIds,id);
        if (index > -1) {
            this.plotIds.splice(index,1);
        }
        index = _.indexOf(this.portfolioDetailsIds,id);
        if (index > -1) {
            this.portfolioDetailsIds.splice(index,1);
        }

        index = _.indexOf(this.tableIds,id);
        if (index > -1) {
            this.tableIds.splice(index,1);
        }

        // If we are deleting the content that we had selected, make sure the right-side panels update to Zero State
        if (id === this.selectedContentId) {
            sas.analytics.riskhp.utils.HprUtil.updateRightPanelsForZeroState();
        }

        // Finally destroy the content
        var content = sap.ui.getCore().byId(id);
        if (content) {
            content.destroy();
        }
    },

    /**
     * This method is called when the RiskPanelsView and DroppableContainer code save the Exploration.  The metadata
     * returned will serve as the content to fully restore the component when rebuilt.
     *
     * @param id The id of the control to find the metadata for
     * @returns {*}
     */
    getContentMetadata: function(id) {
        var control = sap.ui.getCore().byId(id);
        // Note: Depending on what we are holding in the various panels, we might ask the control for it's
        // metadata using a different method.  The point here is to return an object that when sent to the
        // createContent function of the delegate the content could be rebuilt.
        if(typeof control.getQueryData === 'function') {
            console.log("getQueryData getting called for control: " + control.getId());
            return control.getQueryData();
        } else if (typeof control.getQueryData === 'undefined') {
            console.log("getQueryData is undefined for control: " + control.getId());
            return {};
        } else {
            console.log("Could not determine type for call to getQueryData for control: " + JSON.stringify(control));
        }
    },

    //*******************************************
    // Private helper methods
    //*******************************************

    /**
     * Creates a unique id for the new content.  This is done by checking the passed in metadata object for the "name"
     * property.  If this is found that is the id returned as we are in a restore workflow, otherwise the id is
     * generated by appending the current date to the passed in defaultPrefix parameter
     * @param {} metadata
     * @param {string} defaultPrefix
     * @returns {string}
     * @private
     */
    _createUniqueId: function(metadata, defaultPrefix) {
        // Generate a default unique id
        var id =  defaultPrefix + Date.now();

        // If the metadata has a name attribute then we are dealing with saved content so the id should be that name
        if (metadata.name) {
            id = metadata.name;
        }
        return id;
    },

    _renderCounter: function(id){
        var countObject = this.renderCounts;
        var count = countObject[id];
        if (count === undefined) {
            countObject[id] = 1;
        }
        else {
            countObject[id] = count +1;
        }

        var keys = _.keys(countObject);
        console.log("Current Render Counts:");
        _.each(keys, function(key){
            console.log("    " + key +": "+ countObject[key]);
        });
    },
    //*******************************************
    // Public helper methods
    //*******************************************

    /**
     * Returns the id of the currently selected panel
     * @returns {string|*|null}
     */
    getSelectedContainerId: function() {
        return this.selectedContentId;
    },

    // _getPanelType: function(id) {
    //     if (_.contains(this.crosstabIds, id)) {
    //         return HprConstants.HPR_CROSSTAB_PANEL;
    //     }
    //     else if (_.contains(this.plotIds, id)) {
    //         return HPR_PLOT_PANEL;
    //     }
    //     else if (_.contains(this.portfolioDetailsIds, id)) {
    //         return HPR_PORTFOLIO_DETAILS_PANEL;
    //     }
    //     else if (_.contains(this.riskFactorSensitivitiesIds, id)) {
    //         return HPR_RISK_FACTOR_SENSITIVITIES_PANEL;
    //     }
    //     return HPR_UNKNOWN_PANEL;
    // },

    _updateRightPanelsForSelectedControl: function(panelType, id) {
        switch (panelType) {
            case HprConstants.HPR_CROSSTAB_PANEL:
                sas.analytics.riskhp.utils.HprUtil.updateRightPanelsForSelectedCrosstab(id);
                break;
            case HprConstants.HPR_PLOT_PANEL:
                sas.analytics.riskhp.utils.HprUtil.updateRightPanelsForSelectedPlot(id, HprConstants.HPR_OPTIONS_PANE_PLOT_TYPE, HprConstants.HPR_FILTERS_PANE_PLOT_TYPE);
                break;
            case HprConstants.HPR_PORTFOLIO_DETAILS_PANEL:
                sas.analytics.riskhp.utils.HprUtil.updateRightPanelsForSelectedPortfolioDetails(id, HprConstants.HPR_OPTIONS_PANE_PORTFOLIO_DETAILS_TYPE, HprConstants.HPR_FILTERS_PANE_PORTFOLIO_DETAILS_TYPE);
                break;
            case HprConstants.HPR_RISK_FACTOR_SENSITIVITIES_PANEL:
                sas.analytics.riskhp.utils.HprUtil.updateRightPanelsForSelectedRiskFactorSensitivities(id, HprConstants.HPR_OPTIONS_PANE_RISK_FACTOR_SENSITIVITIES_TYPE, HprConstants.HPR_FILTERS_PANE_RISK_FACTOR_SENSITIVITIES_TYPE);
                break;
            case HprConstants.HPR_SIMULATED_VALUES_PANEL:
                sas.analytics.riskhp.utils.HprUtil.updateRightPanelsForSelectedTableViewer(id);
                break;
        }
    },

    _updateRightPanelsForZeroState: function(panelType, id) {
        switch (panelType) {
            case HprConstants.HPR_CROSSTAB_PANEL:
                sas.analytics.riskhp.utils.HprUtil.updateRightPanelsForZeroStateCrosstab(id);
                break;
            case HprConstants.HPR_PLOT_PANEL:
                // Try to keep Plot filters fix for S1387714 intact, but show Zero State for Plot options when Plot is in Zero State
                var filtersType = HprConstants.HPR_FILTERS_PANE_ZERO_STATE_TYPE;
                var control = sap.ui.getCore().byId(id);
                if (control) {
                    var chartModel = control.getChartModel();
                    // Check to see if the model has filters, if so, display Filter pane so user has a chance to change the filter because
                    // the Plot may become valid again when they do
                    if (chartModel && chartModel.getFilters().length > 0) {
                        filtersType = HprConstants.HPR_FILTERS_PANE_PLOT_TYPE;
                    }
                }
                sas.analytics.riskhp.utils.HprUtil.updateRightPanelsForSelectedPlot(id, HprConstants.HPR_OPTIONS_PANE_ZERO_STATE_TYPE, filtersType);
                break;
            case HprConstants.HPR_PORTFOLIO_DETAILS_PANEL:
                sas.analytics.riskhp.utils.HprUtil.updateRightPanelsForSelectedPortfolioDetails(id, HprConstants.HPR_OPTIONS_PANE_ZERO_STATE_TYPE, HprConstants.HPR_FILTERS_PANE_ZERO_STATE_TYPE);
                break;
            case HprConstants.HPR_RISK_FACTOR_SENSITIVITIES_PANEL:
                sas.analytics.riskhp.utils.HprUtil.updateRightPanelsForSelectedRiskFactorSensitivities(id, HprConstants.HPR_OPTIONS_PANE_ZERO_STATE_TYPE, HprConstants.HPR_FILTERS_PANE_ZERO_STATE_TYPE);
                break;
            case HprConstants.HPR_SIMULATED_VALUES_PANEL:
            default:
                sas.analytics.riskhp.utils.HprUtil.updateRightPanelsForZeroState();
                break;
        }
    },

    /**
     * When a panel in the RiskPanelsView is selected this delegate method is called with the id of the content
     * contained within the selected panel. We should update other UI components based on the selected content.
     * @param id Id of the contained content (this will be the id of the content object supplied earlier by a call to createContent)
     */
    setSelectedContainerId: function(id) {
        console.log("setSelectedContainerId: selected container id is " + id);

        var control = sap.ui.getCore().byId(id);
        if (!control) {
            // Update the right-side panels to account for a Zero State
            this._updateRightPanelsForZeroState(HprConstants.HPR_UNKNOWN_PANEL,id);
            return;
        }
        console.log("setSelectedContainerId: selected control is " + control);

        if (this.selectedContentId !== id) {
            // Things are going to happen below, so set dirty state of Exploration view
            sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {
                dirty: true
            });
        }

        // See comment in DroppableContainer.onAfterRendering about whether this is a good idea
        this.selectedContentId = id;

        var panelType = control.getPanelType();
        if (control.isModelValid()) {
            this._updateRightPanelsForSelectedControl(panelType,id);
        }
        else {
            this._updateRightPanelsForZeroState(panelType,id);
        }

        // Store the dataSourceId value for the selected control
        var dataSourceIdForSelectedControl = control.getDataSourceId();

        // If the selected control has a DataSource id associated with it, publish an event so other components can react to the selection.
        // For example, the DataPane may need to change the selection in the DataSource Select component to match the selected control's
        // DataSource id
        if (dataSourceIdForSelectedControl !== null) {
            sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_DELEGATE_SELECTED_CONTROL_CHANGED, {
                dataSourceId: dataSourceIdForSelectedControl
            });
        }
    },

    updateTitleForContent: function(id, newTitle) {
        if (id in this.contentTitleMap) {
            this.contentTitleMap[id] = newTitle;
        }
    },

    updateFooterForContent: function(id, newFooterText) {
        if (id in this.contentFooterMap) {
            this.contentFooterMap[id] = newFooterText;
        }
    },

    /**
     * Retrieve the count (int) that should be used to generate a unique name for the type of content.  The count is
     * also stored in the 'contentCountMap' object.  Default count is 1 if this is the first time for the type of content.
     *
     * @param contentType The type of content to find the count for (i.e. "crosstab", "SubportfolioComparison")
     * @returns {int} The count for a type of content
     */
    getUniqueContentCount: function(contentType) {
        var currentCount = 0;
        if (this.contentCountMap.hasOwnProperty(contentType)) {
            currentCount = this.contentCountMap[contentType];
        }
        this.contentCountMap[contentType] = currentCount + 1;
        return this.contentCountMap[contentType];
    },

    /**
     * Resets the count (int) to 0 for the type of content.  Count is stored in the 'contentCountMap' object.
     *
     * @param contentType The type of content to reset the count for (i.e. "crosstab", "SubportfolioComparison")
     */
    resetUniqueContentCount: function(contentType) {
        this.contentCountMap[contentType] = 0;
    },

    //*******************************************
    // Crosstab methods
    //*******************************************

    /**
     * When a Crosstab needs to be created from the createContent method
     * this function does the job of doing that.  The newCrosstabMetadata object
     * is used to either get the CrosstabContext directly (in the case a Crosstab
     * is being created as a duplicate of an already existing crosstab) or it is
     * a JSON representation of a context and needs to be turned into a "real"
     * CrosstabContext.  This function takes care of this and then starts the
     * process of getting the CubeData object from the CacheManager and ultimately
     * calling the _updateCrosstabAfterDataSourceLoad to update the CrosstabContext
     *
     * @param newCrosstabMetadata
     * @returns {*} - A Crosstab
     * @private
     */
    _createOrRestoreCrosstab: function(newCrosstabMetadata) {
        var contextId,
            context = null,
            self = this;

        // Make sure we have a unique id, but since this is a saved Crosstab the saved 'name' should be used
        contextId = self._createUniqueId(newCrosstabMetadata, HprConstants.HPR_CROSSTAB_COMPONENT_ID);

        // Add a placeholder title to hashmap
        this.contentTitleMap[contextId] = "";

        // Pull the id of the DataSource
        var dataSourceId = newCrosstabMetadata.dataSourceId;
        if (newCrosstabMetadata.context) {
            // Here newCrosstabMetadata has an embedded CrosstabContext in it's
            // context property.  We can use this directly as the CrosstabContext for the
            // Crosstab we are going to create.  Additionally there is no need to apply
            // any modifications to it as it is "presumed" to be correct for the
            // Crosstab we are about to show.
            context = newCrosstabMetadata.context;
        }
        else {
            // Here newCrosstabMetadata represents the data for the context, but not a real context object.
            // This happens in the restore scenario, so we create a default context that will
            // be populated after the dataSource is loaded.
            // Create context object, set the necessities, fill in the data from json.
            // CONTEXT DEFINED
            context = new sas.analytics.riskhp.models.CrosstabContext(contextId);
            context.setDataSourceId(dataSourceId);
        }

        // Create the Crosstab to return
        var crosstab = self._createNewCrosstab(context.getId());
        self._renderCounter(context.getId());
        crosstab.setBusy(true);
        // Make sure the DataSource used by the Crosstab is loaded, and once loaded use it to
        // finish configuring the CrosstabContext and ultimately the Crosstab
        var deferred = DataSourceCacheManager.getDataSource(dataSourceId);
        deferred.done(function(cube) {
           self._updateCrosstabAfterDataSourceLoad(cube, context, crosstab, newCrosstabMetadata, false, null);
        });
        deferred.fail(function() {
            self._updateCrosstabAfterDataSourceLoad(undefined, context, crosstab, newCrosstabMetadata, false, null);
        });

        return crosstab;
    },

    /**
     * Creates a new crosstab object with the given id
     * @param {String} id - The id for the crosstab, MUST be the id of the CrosstabContext that will be the
     * model for this crosstab
     * @private
     */
    _createNewCrosstab: function(id) {
        var self = this;
        // Hack to stop a duplicate id problem, not convinced this is a best practice.
        var oldCrosstab = sap.ui.getCore().byId(id);
        if (oldCrosstab) {
            return oldCrosstab;
        }
        // Create a new Crosstab; id must be same as the CrosstabContext
        var crosstab = new sas.analytics.riskhp.common.riskcommons.crosstab.HprCrosstab(id, {
            dirtyChange: $.proxy(self.onCrosstabDirtyChange, self)
        });
        self.crosstabIds.push(crosstab.getId());
        return crosstab;
    },

    /**
     * A very heavily parameterized and complex method to do the final configuration(s) on a crosstab
     * after we know the cube has been opened.
     *
     * @param cube
     * @param context
     * @param crosstab
     * @param jsonToUpdateWith - JSON object to update the CrosstabContext after the cube is loaded
     * @param resetToDefaults - Should the CrosstabContext be reset to it's defaults
     * @param crosstabTitle - Default title to use for the crosstab when the CrosstabContext is reset to it's defaults
     * @private
     */
    _updateCrosstabAfterDataSourceLoad: function(cube, context, crosstab, jsonToUpdateWith, resetToDefaults, crosstabTitle) {
        // Grab the datasourceId from the context as that is what everything is based on
        var dataSourceId = context.getDataSourceId();

        // Set the cube model on the context and start updating/rebuilding
        context.setCubeDataModel(cube);

        var self = this;
        var finishUpdateOfCrosstab = function(oCube, oContext, oCrosstab, bResetToDefaults, sCrosstabTitle) {
            // If creating a default context we need to set the context back to it's default settings
            if (bResetToDefaults) {
                oContext.setDefaultOptions(oCube.getOptions(), sCrosstabTitle);
                oContext.setDefaultFilters(oCube.getDefaultFilters());

                // Do this last to ensure everything is ready for the internal structures to be populated properly.
                oContext.resetToDefaults();
            }
            // At this point the CrosstabContext is correct for the crosstab and all that
            // is left to do is update data the panels view delegate needs.

            // Add Crosstab title to the contentTitleMap
            self.contentTitleMap[oContext.getId()] = oContext.getOptionName();

            // Add the title for the footer to the contentFooterMap
            var dataSourceParam = DataSourceCacheManager.getDataSourceParam(dataSourceId);
            if (dataSourceParam) {
                self.contentFooterMap[oContext.getId()] = dataSourceParam.label;
            }

            // Set the model of the Crosstab to a JSON representation of the context
            if (oCrosstab) {
                // We appear to be converting the context to a JSONModel here so it can be
                // set on the control the Crosstab extends.
                oCrosstab.setModel(new sas.risk.common.models.RiskJSONModel(oContext), oContext.getId());
            }

            // Since this deferred might complete after the Crosstab has been returned to the window manager,
            // we need to refresh the panel to make sure to pick up the correct panel title and footer
            sas.analytics.riskhp.utils.HprUtil.getSelectedRiskPanelsView().refreshPanelWithId(oContext.getId(), true, null);
            oCrosstab.setBusy(false);
        };

        // Only need to update the context with the supplied json if we got some json;
        if (jsonToUpdateWith) {
            // Update from JSON might take a couple of seconds since we might have to hit the mid-tier, so we need to wait
            $.when(
                context.fromJSON(null, jsonToUpdateWith)
            ).done(function() {
                finishUpdateOfCrosstab(cube, context, crosstab, resetToDefaults, crosstabTitle);
            });
        } else {
            finishUpdateOfCrosstab(cube, context, crosstab, resetToDefaults, crosstabTitle);
        }
    },

    /**
     * Use this method to create the default crosstab when first opening a
     * new exploration.
     * @returns {*}
     */
    createDefaultCrosstab: function() {
        var self = this;

        // Build a default context for the new Crosstab

        // Timestamp shim in case browser is non-modern
        if (!Date.now) {
            Date.now = function() { return new Date().getTime(); };
        }

        // id of CrosstabContext and Crosstab have to be unique and the same in order for Options to work correctly
        var contextId = HprConstants.HPR_CROSSTAB_COMPONENT_ID + Date.now();

        // Instantiate the CrosstabContext for the new Crosstab
        var context = new sas.analytics.riskhp.models.CrosstabContext(contextId);

        // Build a unique title for the Crosstab
        var uniqueCrosstabCount = self.getUniqueContentCount("crosstab");
        var defaultNewCrosstabName = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.Crosstab.defaultName.fmt", uniqueCrosstabCount);
        var contains = _.contains(_.values(self.contentTitleMap), defaultNewCrosstabName);
        // While the 'contentTitleMap' contains the generated new Crosstab name, keep generating new names to try
        while (contains) {
            uniqueCrosstabCount = self.getUniqueContentCount("crosstab");
            defaultNewCrosstabName = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.Crosstab.defaultName.fmt", uniqueCrosstabCount);
            contains = _.contains(_.values(self.contentTitleMap), defaultNewCrosstabName);
        }

        self.resetUniqueContentCount("crosstab"); // Reset the Crosstab count after each new Crosstab is created so we can start at 0 again
        // End of building a unique title

        // Datasource is ASSUMED to be the currently selected DataSource
        var dataSourceId = DataSourceCacheManager.getSelectedDataSourceId();
        context.setDataSourceId(dataSourceId);

        // Create the crosstab we will return
        var crosstab = self._createNewCrosstab(context.getId());
        crosstab.setBusy(true);

        // Make sure the DataSource used by the Crosstab is loaded, and once loaded use it to
        // finish configuring the CrosstabContext and ultimately the Crosstab
        var deferred = DataSourceCacheManager.getDataSource(dataSourceId);
        deferred.done(function(cube) {
            self.selectedContentId = context.getId(); // Set the selected content for this Delegate

            self._updateCrosstabAfterDataSourceLoad(cube, context, crosstab, null, true, defaultNewCrosstabName);
        });

        return crosstab;
    },

    onCrosstabDirtyChange: function(evt) {
        // Publish an event that will update the dirtyState of the Exploration view based on the dirty
        // parameter passed by the Crosstab
        var isCrosstabDirty = evt.getParameter("dirty");
        sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {
            dirty: isCrosstabDirty ? isCrosstabDirty : false
        });
    },

    //*******************************************
    // Plot methods
    //*******************************************

    /**
     * When a Plot needs to be created from the createContent method
     * this function does the job of doing that.  The newContentMetadata object
     * is used to either configure the chartModel directly (in the case a Plot
     * is being created as a duplicate of an already existing Plot) or it is
     * a JSON representation of a Plot and needs to be turned into a "real"
     * chartModel.  This function takes care of this and then starts the
     * process of getting the CubeData object from the CacheManager and ultimately
     * finish configuring the Plot correctly.
     *
     * @param newContentMetadata
     * @returns {*} - A Plot
     * @private
     */
    _createOrRestorePlot: function(newContentMetadata) {
        var plotId,
            chartModel = null,
            self = this,
            content = null;

        // Will do different actions based on whether this is a new Plot or saved Plot
        var newPlot = true;

        // Make sure we have a unique id for the Plot.  In the case of a saved Plot, use the saved 'name' property
        plotId = self._createUniqueId(newContentMetadata, HprConstants.HPR_PLOT_COMPONENT_ID);
        if (newContentMetadata.name) {
            newPlot = false;
        }

        // Create the Plot model object, and fill in the necessities based on whether it is a new Plot or saved Plot
        chartModel = new sas.analytics.riskhp.common.riskcommons.plot.HprChartModel(plotId);

        // If this is a new Plot, have to do some default property creation; Otherwise, take the JSON from the saved Exploration
        // and populate the Plot/Chart model appropriately so the UI will stand up correctly
        if (newPlot) {
            chartModel.configure(newContentMetadata);
        } else {
            // It is important to note that after this method is called the model does NOT have a it's dataSource set
            chartModel.fromJSON(null, newContentMetadata);
        }

        // Add Plot title to hashmap
        self.contentTitleMap[chartModel.getId()] = "";

        // TODO: May need to attach to DSCM for dataSourceId that is loading to force a refresh of panel title, like is done in crosstab code above. TBD
        // Create a new Plot/Chart; id must be same as the model
        content = new sas.analytics.riskhp.common.riskcommons.plot.HprChartView(chartModel.getId(), {
            chartModel: chartModel
        });
        self._renderCounter(chartModel.getId());

        // Add title and footer to internal maps
        var dataSourceId = newContentMetadata.dataSourceId;
        var deferred = DataSourceCacheManager.getDataSource(dataSourceId);
        deferred.done(function(dataSource) {
            if (newPlot) {
                var title = sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils.generateDefaultPanelTitle(dataSource, chartModel);
                self.contentTitleMap[chartModel.getId()] = title;
                chartModel.setDefaultTitle(title);
            }
            else {
                chartModel.dataSource = dataSource;
//                if (chartModel.getModelInvalidReasonString() && !chartModel.isUsingCustomTitle()) {
                if (!chartModel.isUsingCustomTitle()) {
                    chartModel.setDefaultTitle(sas.analytics.riskhp.common.riskcommons.plot.HprChartUtils.generateDefaultPanelTitle(dataSource,chartModel));
                }
                self.contentTitleMap[chartModel.getId()] = chartModel.getOptionName(); // Move the above check to this method?
            }
            var dataSourceParam = DataSourceCacheManager.getDataSourceParam(dataSourceId);
            if (dataSourceParam) {
                self.contentFooterMap[chartModel.getId()] = dataSourceParam.label;
            }
            sas.analytics.riskhp.utils.HprUtil.getSelectedRiskPanelsView().refreshPanelWithId(chartModel.getId(), false, null);
        });
        deferred.fail(function() {
            self.contentTitleMap[chartModel.getId()] = chartModel.getOptionName();
        });

        // Only add the plot id if it doesn't exist in the array yet
        if (!_.contains(self.plotIds, chartModel.getId())) {
            self.plotIds.push(chartModel.getId());
        }

        // TODO: dacagl - Should wait on Mark's fix to RiskPanelsView where it will select the new "View" by default; Will probably have to re-work this code
        // Publish an event that will update the Options pane only for a new Plot; In this case, the 'chart'
        // hasn't been rendered yet, so more than likely the Options pane will have to rely on 'chartModel'
        // property being set in order to fill out the Options pane correctly
        // if (newPlot) {
        //     sas.eventBus.publish("optionsPane", "changeOptionsPane", {
        //         optionsType: HprConstants.HPR_OPTIONS_PANE_PLOT_TYPE,
        //         plotId: chartModel.getId()
        //     });
        // }

        return content;
    },

    //*******************************************
    // Risk Factor Sensitivities methods
    //*******************************************

    /**
     * When a Plot needs to be created from the createContent method
     * this function does the job of doing that.  The newContentMetadata object
     * is used to either configure the chartModel directly (in the case a Plot
     * is being created as a duplicate of an already existing Plot) or it is
     * a JSON representation of a Plot and needs to be turned into a "real"
     * chartModel.  This function takes care of this and then starts the
     * process of getting the CubeData object from the CacheManager and ultimately
     * finish configuring the Plot correctly.
     *
     * @param newContentMetadata
     * @returns {*} - A Plot
     * @private
     */
    _createOrRestoreRiskFactorSensitivities: function(newContentMetadata) {
        var componentId,
            rfsModel = null,
            isNew = true,
            self = this,
            content = null;

        // Make sure we have a unique id for the table.  In the case of a saved RFS, use the saved 'name' property
        componentId = self._createUniqueId(newContentMetadata, HprConstants.HPR_RISK_FACTOR_SENSITIVITIES_COMPONENT_ID);
        if (newContentMetadata.name) {
            isNew = false;
        }

        // Create the RFS model object, and fill in the details based on whether it is new or saved
        rfsModel = new sas.analytics.riskhp.sensitivities.RiskFactorSensitivitiesModel(componentId);

        // If this is new, have to do some default property creation; Otherwise, take the JSON from the saved Exploration
        // and populate the model appropriately so the UI will stand up correctly
        if (isNew) {
            rfsModel.configure(newContentMetadata);
        } else {
            rfsModel.fromJSON(null, newContentMetadata);
        }

        // Add title to hashmap
        self.contentTitleMap[rfsModel.getId()] = "";

        // Create a new RFS view; id must be same as the model
        content = new sas.analytics.riskhp.sensitivities.RiskFactorSensitivitiesView(rfsModel.getId(), {
            rfsModel: rfsModel
        });
        self._renderCounter(rfsModel.getId());

        // Add title and footer to internal maps
        var dataSourceId = newContentMetadata.dataSourceId;
        var deferred = DataSourceCacheManager.getDataSource(dataSourceId);
        deferred.done(function(dataSource) {
            if (isNew) {
                var title = sas.analytics.riskhp.sensitivities.RiskFactorSensitivitiesUtil.generateDefaultSensitivitiesTitle(rfsModel, dataSource);
                self.contentTitleMap[rfsModel.getId()] = title;
                rfsModel.setOptionName(title);
            }
            else {
                self.contentTitleMap[rfsModel.getId()] = rfsModel.getOptionName();
            }
            var dataSourceParam = DataSourceCacheManager.getDataSourceParam(dataSourceId);
            if (dataSourceParam) {
                self.contentFooterMap[rfsModel.getId()] = dataSourceParam.label;
            }
            sas.analytics.riskhp.utils.HprUtil.getSelectedRiskPanelsView().refreshPanelWithId(rfsModel.getId(), false, null);
        });
        deferred.fail(function() {
            self.contentTitleMap[rfsModel.getId()] = rfsModel.getOptionName();
        });

        // Only add the rfs id if it doesn't exist in the array yet
        if (!_.contains(self.riskFactorSensitivitiesIds, rfsModel.getId())) {
           self.riskFactorSensitivitiesIds.push(rfsModel.getId());
        }

        return content;
    },

    //*******************************************
    // Table Viewer methods
    //*******************************************

    onTableViewerDirtyChange: function(evt) {
        // Publish an event that will update the dirtyState of the Exploration view
        sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {dirty: true});
    },

    //*******************************************
    // PortfolioDetails methods
    //*******************************************

    onPortfolioDetailsDirtyChange: function(evt) {
        // Publish an event that will update the dirtyState of the Exploration view
        sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {dirty: true});
    }
});