// (c) 2017, SAS Institute Inc.

jQuery.sap.require('sas.analytics.riskhp.models.DataSourceCacheManager');
jQuery.sap.require('sas.analytics.riskhp.models.DataSourceUtil');
jQuery.sap.require('sas.analytics.riskhp.models.HPRiskModelManager');
jQuery.sap.require('sas.analytics.riskhp.models.DisplayRulesManager');
jQuery.sap.require('sas.analytics.riskhp.utils.HprAPI');
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
jQuery.sap.require("sas.analytics.riskhp.utils.HPRiskPanelsViewDelegate");
jQuery.sap.require("sas.analytics.riskhp.utils.ViewManager");
jQuery.sap.require("sas.hc.m.Dialog");
jQuery.sap.require("sas.hc.m.CheckBox");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.ui.core.settings.GlobalSettingsHelper");
jQuery.sap.require("sas.hc.ui.services.navigation.ContentSelector");
jQuery.sap.require("sas.hc.ui.services.navigation.classes.NavigatorToolBar");
jQuery.sap.require("sas.hc.ui.services.navigation.config.FilterItem");
jQuery.sap.require("sas.hc.ui.services.navigation.config.FilteredItemsConfig");
jQuery.sap.require("sas.hc.ui.services.navigation.config.ModelConfig");
jQuery.sap.require("sas.hc.ui.services.navigation.config.ServiceConfig");
jQuery.sap.require("sas.hc.ui.services.navigation.types.ObjectTypeConstants");
jQuery.sap.require("sas.risk.common.panelsview.RiskPanelsView");
jQuery.sap.require("sas.risk.common.utils.NavChecker");
jQuery.sap.require("sap.ui.model.json.JSONModel");
jQuery.sap.require('sap.ui.thirdparty.jqueryui.jquery-ui-draggable');
jQuery.sap.require('sap.ui.thirdparty.jqueryui.jquery-ui-droppable');
jQuery.sap.require('sas.analytics.riskhp.common.riskcommons.CrosstabUtil');

sap.ui.controller('sas.analytics.riskhp.views.ExplorationView', {
    CrosstabUtil: sas.analytics.riskhp.common.riskcommons.CrosstabUtil,
    ExplorationUtil: sas.analytics.riskhp.utils.ExplorationUtil,
    HprConstants: sas.analytics.riskhp.utils.HprConstants,
    initialExplorationRenderDone: false, // Flag to let us know when the Exploration has rendered
    restoringSavedExploration: false, // Flag to let us know if we are restoring a saved Exploration
    resetExplorationEventsAllowed: true, // Flag to stop some events from firing during reset method as controls are added/changed
    addNewPageToIconTabBarDone: true, // Flag that signals that a new Page is done being added
    lastDataItemLabelRefreshDate: Date.now(), // int used to store the last Date of when a Data Item label was updated
    computedVarEditorView: null, // View for creating computed variables
    computedVarCrosstabPanel: null, // The panel that was holding the crosstab when the computed variable editor was opened

    setResetExplorationEventsAllowed: function(allowed) {
        this.resetExplorationEventsAllowed = allowed;
    },
    
    reset: function(isOpeningExploration) {
        var ctrl = this;
        var view = ctrl.getView();
        var shouldStartDirty = false;

        // Set the Exploration model on the View
        var explorationModel = sas.analytics.riskhp.models.HPRiskModelManager.getExplorationModel();
        // NOTE: In a change data source workflow, the returned exploration model will not have hydrated dataitems in it.
        // The next line will cause the datapane's tree table to refresh with the non-hydrated data items and it will DIE!!!
        view.setModel(explorationModel);

        // When the Exploration view stands up, see if we are going to use a saved layout or if we are
        // a new Exploration (i.e. no pages in model)
        var mainTabBar = view.getMainTabbar();
        mainTabBar.destroyItems();

        var explorationPages = explorationModel.getProperty("/exploration/analysis/layout/pages");
        if (explorationPages && explorationPages.length > 0) {
            this.restoringSavedExploration = true; // Restoring a saved Exploration

            // If we have Pages/Tabs, make sure the IconTabbar is rendered in the correct state
            var tabClosable = true; // Should new Page/Tab be closable
            if (explorationPages.length > 1) {
                // First tab should be closable when there is more than one tab
                mainTabBar.setFirstTabClosable(true);
            } else {
                tabClosable = false;
            }

            // Loop through the Pages and re-create the tabs for each one.  This includes the content that was on the tab
            $.each(explorationPages, function(index, page) {
                // 'page' is already the Page object that we need, just add it back to model manager so we can track it and the Page options will work
                sas.analytics.riskhp.models.HPRiskModelManager.addPageObject(page);

                // Create a new IconTabFilter for the page
                var tabFilter = new sas.hc.m.IconTabFilter({
                    design: "Horizontal",
                    iconColor: "Neutral",
                    text: page.name,
                    key: page.id,
                    closable: tabClosable
                });

                // Re-create the RiskPanelsView with the original content and add it to the IconTabFilter and IconTabbar
                // Create copy of page content to be safe
                var pageContent = $.extend(true, {}, page.content); // Create a copy to guarantee safety
                var currentView = new sas.risk.common.panelsview.RiskPanelsView(page.id, {
                    delegate: new sas.analytics.riskhp.utils.HPRiskPanelsViewDelegate(),
                    dirtyChange: $.proxy(ctrl._onRiskPanelsViewDirtyChange, ctrl),
                    debugMode: HprConstants.USE_DEBUG_MODE_FOR_PANELS,
                    originalLayout: pageContent
                }, false);
                tabFilter.addContent(currentView);

                mainTabBar.addItem(tabFilter);
            });

            // Set the selected key to the saved Page; This will set focus to the Page
            mainTabBar.setSelectedKey(explorationModel.getProperty("/exploration/analysis/layout/selectedPage"));

            // Update the right-side panels for the saved selected Page
            sas.analytics.riskhp.utils.HprUtil.updateRightPanelsForSelectedPage(this._getSelectedPage(mainTabBar));
        } else {
            // No saved Pages in model, so create an initial RiskPanelsView
            // Create a unique Page object to add to the middle part of the View
            var uniquePageObject = sas.analytics.riskhp.models.HPRiskModelManager.createUniquePageObject();

            // Add the new Page object to the model manager to keep track of it
            sas.analytics.riskhp.models.HPRiskModelManager.addPageObject(uniquePageObject);

            // Create a new IconTabFilter for the new unique Page
            var tabFilter = new sas.hc.m.IconTabFilter({
                design: "Horizontal",
                iconColor: "Neutral",
                text: uniquePageObject.name,
                key: uniquePageObject.id,
                closable: false
            });

            // Create a new RiskPanelsView and add it to the IconTabFilter
            var currentView = new sas.risk.common.panelsview.RiskPanelsView(uniquePageObject.id, {
                delegate: new sas.analytics.riskhp.utils.HPRiskPanelsViewDelegate(),
                dirtyChange: $.proxy(ctrl._onRiskPanelsViewDirtyChange, ctrl),
                debugMode: HprConstants.USE_DEBUG_MODE_FOR_PANELS
            }, true);
            tabFilter.addContent(currentView);

            mainTabBar.addItem(tabFilter);
            mainTabBar.setSelectedKey(uniquePageObject.id); // Need to set selected key so IconTabBar will redraw

            shouldStartDirty = true;
        }

        // Call the DataPane.controller.reset method to get the DataPane back in a default state
        // Call the DataPane.controller.reset method to get the DataPane back in a default state
        view.getDataPane().getDataPaneView().getController().reset(isOpeningExploration);

        // call the FiltersPane.controller.reset method to react to the changes
        view.getFiltersPane().getFiltersPaneView().getController().reset();

        // Make sure the NavChecker's dirty state is up to date with the Exploration view
        explorationModel.setProperty("/exploration/isDirty", shouldStartDirty);
        this.navChecker.setDirty(shouldStartDirty);
    },

    onInit: function () {
        var ctrl = this;
        var view = ctrl.getView();

        // Set the Exploration model on the View
        var explorationModel = sas.analytics.riskhp.models.HPRiskModelManager.getExplorationModel();
        view.setModel(explorationModel);

        var explorationPanesStateModel = sap.ui.getCore().getModel(HprConstants.MODEL_ID_EXPLORATION_PANES_STATE);
        if (!explorationPanesStateModel) {
            explorationPanesStateModel = new sap.ui.model.json.JSONModel();
            sap.ui.getCore().setModel(explorationPanesStateModel, HprConstants.MODEL_ID_EXPLORATION_PANES_STATE);
        }
        var explorationPanesStateModelData = {
            computedVariableEditorMode: false,
            displayRuleEditorMode: false,
            panes: {
                objectsPane: {
                    "enabled": true
                },
                dataPane: {
                    "enabled": true,
                    "dataSourceToolbarEnabled": true
                },
                optionsPane: {
                    "enabled": true
                },
                filtersPane: {
                    "enabled": true
                },
                hierarchiesPane: {
                    "enabled": true
                },
                displayRulesPane: {
                    "enabled": true
                }
            }
        };
        explorationPanesStateModel.setData(explorationPanesStateModelData);

        // When the Exploration view stands up, see if we are going to use a saved layout or if we are
        // a new Exploration (i.e. no pages in model)
        var explorationPages = explorationModel.getProperty("/exploration/analysis/layout/pages");
        if (explorationPages && explorationPages.length > 0) {

            this.restoringSavedExploration = true; // Restoring a saved Exploration

            // If we have Pages/Tabs, make sure the IconTabbar is rendered in the correct state
            var mainTabbar = view.getMainTabbar();
            var tabClosable = true; // Should new Page/Tab be closable
            if (explorationPages.length > 1) {
                // First tab should be closable when there is more than one tab
                mainTabbar.setFirstTabClosable(true);
            } else {
                tabClosable = false;
            }

            // Loop through the Pages and re-create the tabs for each one.  This includes the content that was on the tab
            $.each(explorationPages, function(index, page) {
                // 'page' is already the Page object that we need, just add it back to model manager so we can track it and the Page options will work
                sas.analytics.riskhp.models.HPRiskModelManager.addPageObject(page);

                // Create a new IconTabFilter for the page
                var tabFilter = new sas.hc.m.IconTabFilter({
                    design: "Horizontal",
                    iconColor: "Neutral",
                    text: page.name,
                    key: page.id,
                    closable: tabClosable
                });

                // Re-create the RiskPanelsView with the original content and add it to the IconTabFilter and IconTabbar
                // Create copy of page content to be safe
                var pageContent = $.extend(true, {}, page.content); // Create a copy to guarantee safety
                var currentView = new sas.risk.common.panelsview.RiskPanelsView(page.id, {
                    delegate: new sas.analytics.riskhp.utils.HPRiskPanelsViewDelegate(),
                    dirtyChange: $.proxy(ctrl._onRiskPanelsViewDirtyChange, ctrl),
                    debugMode: HprConstants.USE_DEBUG_MODE_FOR_PANELS,
                    originalLayout: pageContent
                }, false);
                tabFilter.addContent(currentView);

                mainTabbar.addItem(tabFilter);
            });

            // Set the selected key to the saved Page; This will set focus to the Page
            mainTabbar.setSelectedKey(explorationModel.getProperty("/exploration/analysis/layout/selectedPage"));
        } else {
            // No saved Pages in model, so create an initial RiskPanelsView
            // Create a unique Page object to add to the middle part of the View
            var uniquePageObject = sas.analytics.riskhp.models.HPRiskModelManager.createUniquePageObject();

            // Add the new Page object to the model manager to keep track of it
            sas.analytics.riskhp.models.HPRiskModelManager.addPageObject(uniquePageObject);

            // Create a new IconTabFilter for the new unique Page
            var tabFilter = new sas.hc.m.IconTabFilter({
                design: "Horizontal",
                iconColor: "Neutral",
                text: uniquePageObject.name,
                key: uniquePageObject.id,
                closable: false
            });

            // Create a new RiskPanelsView and add it to the IconTabFilter
            var currentView = new sas.risk.common.panelsview.RiskPanelsView(uniquePageObject.id, {
                delegate: new sas.analytics.riskhp.utils.HPRiskPanelsViewDelegate(),
                debugMode: HprConstants.USE_DEBUG_MODE_FOR_PANELS,
                dirtyChange: $.proxy(ctrl._onRiskPanelsViewDirtyChange, ctrl)
            }, true);
            tabFilter.addContent(currentView);

            view.getMainTabbar().addItem(tabFilter);
        }

        // Build overflow menu
        view.getOverflowButton().setMenu(this._createOverflowMenu());

        // NavChecker setup; NavChecker detects navigation requests for a page and warns/prompts
        // a user before proceeding if the page has unsaved changes.
        this.navChecker = sas.analytics.riskhp.utils.HprUtil.createNavChecker(true, view);

        // Subscribe to an event that is published when the Name field for a Page is updated on the Options pane
        sas.eventBus.subscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_PAGE_OPTIONS_NAME_UPDATED, this.onPageOptionsNameUpdate, this);

        // Subscribe to an event that is published when the Name field for a Crosstab is updated on the Options pane
        sas.eventBus.subscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_CROSSTAB_OPTIONS_NAME_UPDATED, this.onCrosstabOptionsNameUpdate, this);

        // Subscribe to an event that is published when the Name field for a Plot is updated on the Options pane
        sas.eventBus.subscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_PLOT_OPTIONS_NAME_UPDATED, this.onPlotOptionsNameUpdate, this);

        // Subscribe to an event that is published when the Name field for a Portfolio Details is updated on the Options pane
        sas.eventBus.subscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_PORTFOLIO_DETAILS_OPTIONS_NAME_UPDATED, this.onPortfolioDetailsOptionsNameUpdate, this);

        // Subscribe to an event that is published when the Name field for a Risk Factor Sensitivities table is updated on the Options pane
        sas.eventBus.subscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_RISK_FACTOR_SENSITIVITIES_OPTIONS_NAME_UPDATED, this.onRiskFactorSensitivitiesOptionsNameUpdate, this);

        // Subscribe to an event that is published when the Name field for a TableViewer is updated on the Options pane
        sas.eventBus.subscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_TABLE_VIEWER_OPTIONS_NAME_UPDATED, this.onTableViewerOptionsNameUpdate, this);

        // Subscribe to an event that signifies the dirty state for the Exploration should be updated
        sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, this.onUpdateDirtyState, this);

        // Subscribe to an event that is fired when the Settings change
        sas.eventBus.subscribe(sas.hc.ui.core.SETTINGS_CHANNEL, sas.hc.ui.core.SETTINGS_CHANGE, this.onApplicationSettingsChange, this);

        // Subscribe to an event that is fired when the label for a Data Item is updated
        sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_DATA_ITEM_LABEL_UPDATED, this.onDataItemLabelUpdate, this);
    },

    onExit: function() {
        // Release the loaded cube resources
        DataSourceCacheManager.releaseAllLoadedDataSources();

        // Reset the page count to 0 when this view is destroyed; Otherwise, page count will continue to grow since we implement per session
        sas.analytics.riskhp.utils.HprUtil.resetUniquePageCount();

        this.navChecker.stop(); // Stop handling page navigation requests for this View

        sas.eventBus.unsubscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_PAGE_OPTIONS_NAME_UPDATED, this.onPageOptionsNameUpdate, this);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_CROSSTAB_OPTIONS_NAME_UPDATED, this.onCrosstabOptionsNameUpdate, this);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_PLOT_OPTIONS_NAME_UPDATED, this.onPlotOptionsNameUpdate, this);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_PORTFOLIO_DETAILS_OPTIONS_NAME_UPDATED, this.onPortfolioDetailsOptionsNameUpdate, this);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_RISK_FACTOR_SENSITIVITIES_OPTIONS_NAME_UPDATED, this.onRiskFactorSensitivitiesOptionsNameUpdate, this);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_TABLE_VIEWER_OPTIONS_NAME_UPDATED, this.onTableViewerOptionsNameUpdate, this);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, this.onUpdateDirtyState, this);
        sas.eventBus.unsubscribe(sas.hc.ui.core.SETTINGS_CHANNEL, sas.hc.ui.core.SETTINGS_CHANGE, this.onApplicationSettingsChange, this);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_DATA_ITEM_LABEL_UPDATED, this.onDataItemLabelUpdate, this);
    },

    /**
     * Called when the View has been rendered (so its HTML is part of the document). Post-rendering manipulations of
     * the HTML could be done here. This hook is the same one that SAPUI5 controls get after being rendered.
     * @returns {void}
     */
    onAfterRendering: function () {
        var ctrl = this;
        var view = ctrl.getView();
        var model = view.getModel();

        // Make sure the NavChecker's dirty state is up to date with the Exploration view
        var explorationIsDirty = model.getProperty("/exploration/isDirty");
        this.navChecker.setDirty(explorationIsDirty);

        // Find the selected Tab/Page from the IconTabBar
        var mainTabBar = view.getMainTabbar();
        var selectedPage = ctrl._getSelectedPage(mainTabBar);

        // When restoring a saved Exploration, make sure to update the right-side panels with the selected Page
        if (this.restoringSavedExploration && selectedPage) {
            sas.analytics.riskhp.utils.HprUtil.updateRightPanelsForSelectedPage(selectedPage);
        }

        this.initialExplorationRenderDone = true;
    },

    //*****************************************************************************************************
    //*****************************************************************************************************
    // Button handlers
    //*****************************************************************************************************
    //*****************************************************************************************************

    onBack:function() {
        // Navigate back to Welcome screen; NavChecker will display a confirm dialog if there
        // are unsaved changes (i.e. view is in dirty state)
        sas.analytics.riskhp.utils.ViewManager.loadHprWelcomeView();
    },

    /**
     * Core method to save an exploration.  If the id is given then it is assumed to be a 'Save' operation, if
     * explorationId is 'null' then it is assumed to be a 'Save As' operation
     * @param explorationId The id of the exploration (must be null if this is a 'Save As' operation)
     * @param explorationName The name for the exploration
     * @private
     */
    _saveExploration:function(explorationId, explorationName) {
        var ctrl = this;
        var view = ctrl.getView();
        var explorationModel = view.getModel();

        // for external callers (see HprApplicationSwitcherController)
        var deferred = $.Deferred();

        var callback = function(savedExplorationObject) {
            // After a save, update the 'id' if it didn't exist yet (i.e. new Exploration)
            if (!explorationId) {
                explorationModel.setProperty("/exploration/id", savedExplorationObject.id);
            }

            // User may have updated name on ContentSelector, must update model
            if (savedExplorationObject && savedExplorationObject.name) {
                explorationModel.setProperty("/exploration/name", savedExplorationObject.name);
            }

            // Update the dirty state for the Exploration (calls explorationModel.refresh())
            ctrl._updateDirtyState(false);
        };

        // Based on whether the Exploration has an id, either just update the Exploration or display a ContentSelector Save Dialog
        if (explorationModel && explorationId) {
            // Exploration has been saved before, just update it; Construct an Exploration object
            // with the necessary pieces for a successful update
            var analysisJSON = sas.analytics.riskhp.models.HPRiskModelManager.generateExplorationAnalysisJSON(view);
            var explorationObject = {
                // version: ctrl.HprConstants.EXPLORATION_VERSION,
                id: explorationId,
                analysis: analysisJSON,
                dataSources: ctrl.ExplorationUtil.generateDataSourceCubesFromAnalysisObject(analysisJSON)
            };

            if (HprConstants.ENABLE_DEBUG) {
                console.log(JSON.stringify(analysisJSON));
            }
            sas.analytics.riskhp.utils.HprAPI.updateExploration(explorationId, explorationObject).done(
                function() {
                    callback();
                    deferred.resolve();
                }
            ).fail(
                function(jqxhr) {
                    //TODO surface an error message?
                    deferred.reject(jqxhr);
                }
            );
        } else {
            // S1422616: The ContentSelector Save dialog has Breadcrumb links with an href of '#'.  This causes problems for our
            // Router and NavChecker code in that '#' triggers the Router code into thinking it is a hash and view change.  We have
            // to stop the NavChecker and Router before the ContentSelector dialog appears so we do not change views when the
            // Breadcrumb links are clicked.  The NavChecker is re-started in the ContentSelector.closeHandler code below.
            ctrl.navChecker.stop();
            sas.analytics.riskhp.utils.ViewManager.getRouter().stop();

            sas.analytics.riskhp.utils.HprUtil.showContentSelector(
                ctrl.createId("-contentSelectorSaveDlg"),
                sas.hc.ui.services.navigation.ContentSelector.Mode.SAVE,
                function (result, dataDescriptor) {
                    // S1422616: When the ContentSelector is closed, restart the NavChecker (which ultimately ends up restarting the Router) so
                    // we can safely navigate to views and properly receive NavChecker discard dialogs
                    var navCheckerCallback = $.proxy(view.destroy, view);
                    ctrl.navChecker.start(navCheckerCallback);

                    // User clicked 'Save'
                    if ( result.detail === sas.hc.ui.services.navigation.ContentSelector.CloseDetail.COMMIT ) {
                        // Create an Exploration object which will be saved with the necessary information
                        var analysisObject = sas.analytics.riskhp.models.HPRiskModelManager.generateExplorationAnalysisJSON(view);
                        var explorationObject = {
                            // version: ctrl.HprConstants.EXPLORATION_VERSION,
                            name: result.saveItemName,
                            folderEntityKey: dataDescriptor.getId(result.containerItem),
                            folderPath: dataDescriptor.getAbsolutePath(result.containerItem),
                            analysis: analysisObject,
                            dataSources: ctrl.ExplorationUtil.generateDataSourceCubesFromAnalysisObject(analysisObject)
                        };

                        sas.analytics.riskhp.utils.HprAPI.createExploration(explorationObject).done(
                            function(data) {
                                callback(data);
                                deferred.resolve();
                            }
                        ).fail(
                            function(jqxhr) {
                                //TODO surface an error message?
                                deferred.reject(jqxhr);
                            }
                        );
                    }
                    else {
                        deferred.resolve();
                    }
                },
                explorationName
            );
        }

        return deferred;
    },

    /**
     * Called when the user chooses to save the changes they have made to the current exploration
     */
    onSaveExploration:function() {
        var ctrl = this;
        var view = ctrl.getView();
        var explorationModel = view.getModel();
        var explorationId = explorationModel.getProperty("/exploration/id");
        var explorationName = explorationModel.getProperty("/exploration/name");

        // Returns a deferred that resolves when the exploration is saved. Used for external callers (see HprApplicationSwitcherController).
        return this._saveExploration(explorationId, explorationName);
    },

    /**
     * Called when the user chooses to save the current exploration as with a new name.  The existing exploration name
     * will be appended with a phrase to indicate it is a copy, but the user can change it should they chose to do so.
     */
    onSaveExplorationAs:function() {
        var ctrl = this;
        var view = ctrl.getView();
        var explorationModel = view.getModel();
        var explorationName = explorationModel.getProperty("/exploration/name");
        var newExplorationName = sas.analytics.riskhp.utils.HprUtil.getBundle().getText("SaveExplorationDialog.saveAsName.fmt", [explorationName]);
        this._saveExploration(null, newExplorationName);
    },

    // onRefreshExploration:function() {
    //     var ctrl = this;
    //     var view = ctrl.getView();
    // },

    // onExportExploration:function() {
    //     var ctrl = this;
    //     var view = ctrl.getView();
    //     // TODO: Should look at Global Settings for 'Include plot image in export' & 'Plot dimensions (width & height)'
    //     // var AppSettings = sas.analytics.riskhp.settings.ApplicationSettings;
    //     // AppSettings.getIncludePlotImage();
    //     // AppSettings.getPlotWidthDimension();
    //     // AppSettings.getPlotHeightDimension();
    // },

    onDebug: function() {
        var dialogModel = new sap.ui.model.json.JSONModel();
        var debugDialog = sap.ui.jsfragment("sas.analytics.riskhp.fragments.DebugDialog", this);
        debugDialog.setModel(dialogModel);
        debugDialog.open();
    },

    //*****************************************************************************************************
    //*****************************************************************************************************
    // Other handlers
    //*****************************************************************************************************
    //*****************************************************************************************************

    /**
     * Returns the currently selected Page (aka. Tab) for the IconTabBar.
     *
     * @param iconTabBar The iconTabBar object which holds the tabs
     * @returns {sas.hc.m.IconTabFilter} The selected Page (aka. Tab) or null
     * @private
     */
    _getSelectedPage: function(iconTabBar) {
        var selectedKey = iconTabBar.getSelectedKey();
        var selectedPage = null;

        // Loop through the IconTabBar and find the selected Tab/Page and capture the item which signifies the selected Page
        $.each(iconTabBar.getItems(), function(index, item) {
            var itemKey = item.getKey();
            if (itemKey === selectedKey) {
                selectedPage = item;
                return false;
            }
        });

        return selectedPage;
    },

    /**
     * Returns the currently selected Page (aka. Tab) for the Exploration view's IconTabBar.
     *
     * @returns {sas.hc.m.IconTabFilter} The selected Page (aka. Tab) or null
     */
    getSelectedPage: function() {
        // Find the selected Tab/Page from the IconTabBar
        var mainTabBar = this.getView().getMainTabbar();
        return this._getSelectedPage(mainTabBar);
    },

    /**
     * Returns the Page (aka. Tab) represented by the pageId
     *
     * @param pageId The id of the Page to find
     * @returns {sas.hc.m.IconTabFilter} The Page (aka. Tab) or null
     */
    getPage: function(pageId) {
        // Find the Tab/Page from the IconTabBar
        var mainTabBar = this.getView().getMainTabbar();
        var page = null;

        // Loop through the IconTabBar and find the desired Tab/Page
        $.each(mainTabBar.getItems(), function(index, item) {
            var itemKey = item.getKey();
            if (itemKey === pageId) {
                page = item;
                return false;
            }
        });

        return page;
    },

    /**
     * Convenience method to return the title of the currently selected page
     * @returns {string}
     */
    getSelectedPageTitle: function() {
        var mainTabBar = this.getView().getMainTabbar();
        var selectedPage = this._getSelectedPage(mainTabBar);
        var selectedPageTitle = selectedPage.getText();
        if (selectedPageTitle) {
            return selectedPageTitle;
        }
        return sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.unknown.content.title.txt");
    },

    /**
     * This method is called when the user hits the Apply button on the Page Options pane.  The text for the
     * Tab/Page identified by 'data.pageId' will be updated.
     *
     * @param channelId
     * @param eventId
     * @param data Object which contains the 'newPageName' value
     */
    onPageOptionsNameUpdate: function(channelId, eventId, data) {
        var ctrl = this;

        // Update the text and tooltip for the Page
        var page = ctrl.getPage(data.pageId);
        if (page) {
            page.setText(data.newPageName);
        }
    },

    /**
     * This method is called when the user hits the Apply button on the Crosstab Options pane.  The title for the selected
     * Crosstab will be updated.
     *
     * @param channelId
     * @param eventId
     * @param data Object which contains the id and new name for the Crosstab
     */
    onCrosstabOptionsNameUpdate: function(channelId, eventId, data) {
        var ctrl = this;
        ctrl._onOptionsNameUpdate(data.crosstabId, data.newCrosstabName);
    },

    /**
     * This method is called when the user hits the Apply button on the Plot Options pane.  The title for the selected
     * Plot will be updated.
     *
     * @param channelId
     * @param eventId
     * @param data Object which contains the id and new name for the Plot
     */
    onPlotOptionsNameUpdate: function(channelId, eventId, data) {
        var ctrl = this;
        ctrl._onOptionsNameUpdate(data.plotId, data.newPlotName);
    },

    /**
     * This method is called when the user hits the Apply button on the Portfolio Details Options pane.  The title for the selected
     * Portfolio Details will be updated.
     *
     * @param channelId
     * @param eventId
     * @param data Object which contains the id and new name for the Portfolio Details
     */
    onPortfolioDetailsOptionsNameUpdate: function(channelId, eventId, data) {
        var ctrl = this;
        ctrl._onOptionsNameUpdate(data.portfolioDetailsId, data.newPortfolioDetailsName);
    },

    /**
     * This method is called when the user hits the Apply button on the Risk Factor Sensitivities Options pane.  The title for the selected
     * Risk Factor Sensitivities table will be updated.
     *
     * @param channelId
     * @param eventId
     * @param data Object which contains the id and new name for the Portfolio Details
     */
    onRiskFactorSensitivitiesOptionsNameUpdate: function(channelId, eventId, data) {
        var ctrl = this;
        ctrl._onOptionsNameUpdate(data.rfsId, data.newRiskFactorSensitivitiesName);
    },

    /**
     * This method is called when the user hits the Apply button on the TableViewer Options pane.  The title for the selected
     * TableViewer will be updated.
     *
     * @param channelId
     * @param eventId
     * @param data Object which contains the id and new name for the TableViewer
     */
    onTableViewerOptionsNameUpdate: function(channelId, eventId, data) {
        var ctrl = this;
        ctrl._onOptionsNameUpdate(data.tableViewerId, data.newTableViewerName);
    },

    /**
     * Helper method to update the name of the component/content using the delegate and RiskPanelsView methods.
     * This method will update the name displayed at the top of the tile and refresh the tile show the new name
     * will be rendered.
     * 
     * @param contentId The id of the component/content to update the name for
     * @param newName The new name
     * @private
     */
    _onOptionsNameUpdate: function(contentId, newName) {
        var riskPanelsView = this._getActivePanelsView();
        if (riskPanelsView !== null) {
            // Gather the delegate ('sas.analytics.riskhp.utils.HPRiskPanelsViewDelegate')
            var delegate = riskPanelsView.getDelegate();
            // Update the title for the current component/content object
            delegate.updateTitleForContent(contentId, newName);
            // Refresh the RiskPanelsView so the title will display correctly
            riskPanelsView.refreshPanelWithId(contentId);
        }
    },

    /**
     * This method is called by the EventBus whenever the dirty state for the Exploration should be updated.  All the work
     * is really done by the '_updateDirtyState' method.
     *
     * @param channelId
     * @param eventId
     * @param data Object which contains the new 'dirty' value
     */
    onUpdateDirtyState: function(channelId, eventId, data) {
        this._updateDirtyState(data.dirty);
    },

    /**
     * Returns a boolean value indicating if the exploration is dirty or not.
     * @returns {Boolean}
     */
    isExplorationDirty: function() {
        return sas.analytics.riskhp.models.HPRiskModelManager.getExplorationModel().getProperty("/exploration/isDirty");
    },

    /**
     * Update the dirty state for the Exploration and keep the NavChecker's dirty state in-sync.
     *
     * @param isDirty Boolean Whether the Exploration is in a dirty state
     * @private
     */
    _updateDirtyState: function(isDirty) {
        // If events fired during reset of Exploration UI are not allowed, stop this function
        if (!this.resetExplorationEventsAllowed) {
            return;
        }

        // Update the 'isDirty' model property so the bindings will work
        var explorationModel = sas.analytics.riskhp.models.HPRiskModelManager.getExplorationModel();
        explorationModel.setProperty("/exploration/isDirty", isDirty);
        explorationModel.refresh();

        // Keep the NavChecker dirty state in-sync with the Explorations
        if (this.navChecker) {
            this.navChecker.setDirty(isDirty);
        }
    },

    /**
     * Set the dirty state of the Exploration when the RiskPanelsView signals it's dirty state has changed.
     *
     * @param evt Event object which has 'changeType' (reason the event was fired), 'componentId' (which component
     * fired the event), and 'id' (which RiskPanelsView is affected) properties
     * @private
     */
    _onRiskPanelsViewDirtyChange: function(evt) {
        // Always assume the dirty state should be set to 'true' when RiskPanelsView sends this event
        this._updateDirtyState(true);
    },

    /**
     * This method is called when the Application Settings change.  This will allow us to
     * react to the settings change in an appropriate way.
     *
     * @param channelId
     * @param eventId
     * @param data
     */
    onApplicationSettingsChange: function(channelId, eventId, data) {
        if (data) {
            _.each(data, function (field) {
                // TODO: Need to add code to react to the settings change in an appropriate way for each (if applicable)
                switch (field.id) {
                    case HprConstants.HPR_SETTINGS_SHOW_STALENESS_NOTIFICATION:
                        break;
                    case HprConstants.HPR_SETTINGS_SHOW_LABELS:
                        // Fire an event that signals that the preference for whether to show DataItem labels or names has been updated
                        sas.eventBus.publish(HprConstants.CHANNEL_APP_SETTINGS, HprConstants.EVENT_SHOW_LABELS_CHANGED, {
                            newValue: field.newValue,
                            oldValue: field.oldValue
                        });

                        break;
                    case HprConstants.HPR_SETTINGS_INCLUDE_PLOT_IMAGE:
                        break;
                    case HprConstants.HPR_SETTINGS_PLOT_WIDTH_DIMENSION:
                        break;
                    case HprConstants.HPR_SETTINGS_PLOT_HEIGHT_DIMENSION:
                        break;
                    case HprConstants.HPR_SETTINGS_CUBE_POLLING:
                        break;
                    case HprConstants.HPR_SETTINGS_CUBE_POLLING_INTERVAL:
                        break;
                    case HprConstants.HPR_SETTINGS_DISPLAY_HORIZONS:
                        // Fire an event that signals that the preference for how to display Horizon labels has been updated
                        sas.eventBus.publish(HprConstants.CHANNEL_APP_SETTINGS, HprConstants.EVENT_DISPLAY_HORIZONS_CHANGED, {
                            newValue: field.newValue,
                            oldValue: field.oldValue
                        });

                        break;
                    case HprConstants.HPR_SETTINGS_PORTFOLIO_DETAILS_PAGE_SIZE:
                        // This field maps to the old 'NDETAILS' Preferences field used in the Flex version; However, in the HTML5 UI
                        // the 'NDETAILS' data comes from the 'options' data for an opened cube.  But, the Application Settings for the HTML5 UI can
                        // be shown before a cube is loaded.  So we do not use the 'NDETAILS' data from the opened cube, but rely on the value
                        // set in the Preferences.

                        // Fire an event that signals that the Portfolio Details page size preference (NDETAILS) has been updated
                        sas.eventBus.publish(HprConstants.CHANNEL_APP_SETTINGS, HprConstants.EVENT_PORTFOLIO_DETAILS_PAGE_SIZE_CHANGED, {
                            newValue: field.newValue,
                            oldValue: field.oldValue
                        });

                        break;
                }
            });
        }
    },

    /**
     * This method is called when a label for a Data Item has been updated
     *
     * @param channelId
     * @param eventId
     * @param data Object which contains the Data Item, new label value, and old label value
     */
    onDataItemLabelUpdate: function(channelId, eventId, data) {
        // Find the RiskPanelsView that is selected/active and refresh all the panels on it so they will pick up the new label;
        // Store the last Date of when a Data Item label was updated for comparison later when switching Pages/Tabs
        var activeRiskPanelsView = this._getActivePanelsView();
        if (activeRiskPanelsView !== null) {

            // Retrieve the DataSourceParams object corresponding to this Data Item label update
            var dataSourceParam = DataSourceCacheManager.getDataSourceParam(data.dataSourceId);
            if (dataSourceParam !== null) {
                // Refresh all the panels on the current RiskPanelsView
                activeRiskPanelsView.refreshAllPanels(data);

                // Store a timestamp to compare later when switching Pages/Tabs to see if we need to refresh that RiskPanelsView
                this.lastDataItemLabelRefreshDate = Date.now();

                // Store away the data for the Data Item label update
                dataSourceParam.addDataItemChangedLabelObject(data);
            }
        }
    },

    _openComputedvariableEditor: function(crosstabId) {
        var ctrl = this;
        var view = ctrl.getView();

        if (!ctrl.computedVarEditorView) {
            // Create a View that will be able to swap out the appropriate Hierarchies pane based on selection in ExplorationView
            ctrl.computedVarEditorView = sap.ui.view({
                // id: this.createId('_computedColumnEditor'),
                type: sap.ui.core.mvc.ViewType.JS,
                height: "100%",
                viewName: "sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableEditor"
            });
        }

        var panelsView = ctrl._getActivePanelsView();
        var crosstabPanel = panelsView.findPanelContainingContentWithId(crosstabId);

        this.computedVarCrosstabPanel = crosstabPanel;

        var centerArea = view.getCenterSplitterPanel().getCenter();
        centerArea.removeAllContent();
        centerArea.addContent(this.computedVarEditorView);
        view.getCenterSplitterPanel().setCenter(centerArea);

        var explorationPanesStateModel = sap.ui.getCore().getModel(HprConstants.MODEL_ID_EXPLORATION_PANES_STATE);
        explorationPanesStateModel.setProperty("/computedVariableEditorMode", true);

        // var leftPropertyPane = view.getLeftPropertyPaneLayout();
        // leftPropertyPane.setPropertyPaneVisible(false);
        // leftPropertyPane.getAggregation("_leftExpandCollapseButton").setEnabled(false);  // disable opening it with chevron
        // explorationPanesStateModel.setProperty("/panes/dataPane/enabled", true); // false);
        explorationPanesStateModel.setProperty("/panes/dataPane/dataSourceToolbarEnabled", false);
        explorationPanesStateModel.setProperty("/panes/objectsPane/enabled", false);

        var rightPropertyPane = view.getRightPropertyPaneLayout();
        rightPropertyPane.setPropertyPaneVisible(false);
        // rightPropertyPane.getAggregation("_rightExpandCollapseButton").setEnabled(false);
        // rightPropertyPane.getAggregation("_expandCollapseButton").setEnabled(false);
        explorationPanesStateModel.setProperty("/panes/optionsPane/enabled", false);
        explorationPanesStateModel.setProperty("/panes/filtersPane/enabled", false);
        explorationPanesStateModel.setProperty("/panes/hierarchiesPane/enabled", false);
        explorationPanesStateModel.setProperty("/panes/displayRulesPane/enabled", false);

        view.getSaveButton().setEnabled(false);
        view.getOverflowButton().setEnabled(false);
        view.getBackButton().setEnabled(false);

        // Subscribe to an event that is fired when the computed variable editor is being closed
        sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_CLOSE_COMPUTED_VARIABLE_EDITOR, ctrl.onComputedVariableEditorClose, ctrl);
    },

    getComputedVariableEditorCrosstabId: function() {
        var self = this;
        if (self.computedVarEditorView) {
            return self.computedVarEditorView.getController().crosstabId;
        }
    },

    createComputedVariable: function(crosstabId, pickinfo) {
        var self = this;
        self._openComputedvariableEditor(crosstabId);

        self.computedVarEditorView.getController().configureForCreating(crosstabId, pickinfo);
    },

    editComputedVariable: function(crosstab, dataItem) {
        var self = this;
        var compVar = sas.analytics.riskhp.models.ComputedVariablesManager.getComputedVariableByName(dataItem[0].name);
        self._openComputedvariableEditor(crosstab.sId);

        self.computedVarEditorView.getController().configureForEditing(crosstab.sId, compVar);
    },

    copyComputedVariable: function(crosstab, dataItem) {
        var self = this;

        var compVar = sas.analytics.riskhp.models.ComputedVariablesManager.getComputedVariableByName(dataItem[0].name);
        self._openComputedvariableEditor(crosstab.sId);

        self.computedVarEditorView.getController().configureForCopying(crosstab.sId, compVar);
    },

    onComputedVariableEditorClose: function(channelId, eventId, data) {
        var ctrl = this;
        var view = ctrl.getView();

        sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_CLOSE_COMPUTED_VARIABLE_EDITOR, ctrl.onComputedVariableEditorClose, ctrl);

        var context = sas.analytics.riskhp.models.HPRiskModelManager.getCrosstabContext(data.crosstabId);
        var crosstab = context.crosstab;

        var mainContent = this.computedVarCrosstabPanel.getAggregation("_mainContentWrapper");
        mainContent.setAggregation("content", crosstab);
        // unset this.computedVarCrosstabPanel
        
        var centerArea = view.getCenterSplitterPanel().getCenter();
        centerArea.removeAllContent();
        centerArea.addContent(view.getMainTabbar());
        view.getCenterSplitterPanel().setCenter(centerArea);

        var explorationPanesStateModel = sap.ui.getCore().getModel(HprConstants.MODEL_ID_EXPLORATION_PANES_STATE);
        explorationPanesStateModel.setProperty("/computedVariableEditorMode", false);

        explorationPanesStateModel.setProperty("/panes/dataPane/dataSourceToolbarEnabled", true);

        explorationPanesStateModel.setProperty("/panes/objectsPane/enabled", true);
        explorationPanesStateModel.setProperty("/panes/optionsPane/enabled", true);
        explorationPanesStateModel.setProperty("/panes/filtersPane/enabled", true);
        explorationPanesStateModel.setProperty("/panes/hierarchiesPane/enabled", true);
        explorationPanesStateModel.setProperty("/panes/displayRulesPane/enabled", true);

        var rightPropertyPane = view.getRightPropertyPaneLayout();
        rightPropertyPane.setPropertyPaneVisible(true);
        // rightPropertyPane.getAggregation("_rightExpandCollapseButton").setEnabled(true);

        view.getSaveButton().setEnabled(true);
        view.getOverflowButton().setEnabled(true);
        view.getBackButton().setEnabled(true);

    },

    _openDisplayRuleEditor: function(crosstabId) {
        var ctrl = this;
        var view = ctrl.getView();

        if (!ctrl.displayRuleEditorView) {
            // Create a View that will be able to swap out the appropriate display rules pane based on selection in ExplorationView
            ctrl.displayRuleEditorView = sap.ui.view({
                type: sap.ui.core.mvc.ViewType.JS,
                height: "100%",
                viewName: "sas.analytics.riskhp.views.displayrules.DisplayRuleEditor"
            });
        }

        var panelsView = ctrl._getActivePanelsView();
        var crosstabPanel = panelsView.findPanelContainingContentWithId(crosstabId);

        this.displayRulesCrosstabPanel = crosstabPanel;

        var centerArea = view.getCenterSplitterPanel().getCenter();
        centerArea.removeAllContent();
        centerArea.addContent(this.displayRuleEditorView);
        view.getCenterSplitterPanel().setCenter(centerArea);

        var explorationPanesStateModel = sap.ui.getCore().getModel(HprConstants.MODEL_ID_EXPLORATION_PANES_STATE);
        explorationPanesStateModel.setProperty("/displayRuleEditorMode", true);

        explorationPanesStateModel.setProperty("/panes/dataPane/dataSourceToolbarEnabled", false);
        explorationPanesStateModel.setProperty("/panes/objectsPane/enabled", false);

        // var rightPropertyPane = view.getRightPropertyPaneLayout();
        // rightPropertyPane.setPropertyPaneVisible(false);

        explorationPanesStateModel.setProperty("/panes/optionsPane/enabled", false);
        explorationPanesStateModel.setProperty("/panes/filtersPane/enabled", false);
        explorationPanesStateModel.setProperty("/panes/hierarchiesPane/enabled", false);
        explorationPanesStateModel.setProperty("/panes/displayRulesPane/enabled", true);


        view.getSaveButton().setEnabled(false);
        view.getOverflowButton().setEnabled(false);
        view.getBackButton().setEnabled(false);

        // Subscribe to an event that is fired when the display rules editor is being closed
        sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_CLOSE_DISPLAY_RULE_EDITOR, ctrl.onDisplayRuleEditorClose, ctrl);
    },

    createDisplayRule: function(crosstabId, pickinfo) {
        var self = this;
        self._openDisplayRuleEditor(crosstabId);

        self.displayRuleEditorView.getController().configureForCreating(crosstabId, pickinfo);
    },

    editDisplayRule: function(crosstabId, displayRuleId) {
        var self = this;
        var displayRule = sas.analytics.riskhp.models.DisplayRulesManager.getDisplayRuleById(displayRuleId);
        self._openDisplayRuleEditor(crosstabId);

        self.displayRuleEditorView.getController().configureForEditing(crosstabId, displayRule);
    },

    copyDisplayRule: function(crosstabId, displayRuleId) {
        var self = this;

        self._openDisplayRuleEditor(crosstabId);

        var displayRule = sas.analytics.riskhp.models.DisplayRulesManager.getDisplayRuleById(displayRuleId);
        self.displayRuleEditorView.getController().configureForCopying(crosstabId, displayRule);
    },

    onDisplayRuleEditorClose: function(channelId, eventId, data) {
        var ctrl = this;
        var view = ctrl.getView();

        sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_CLOSE_DISPLAY_RULE_EDITOR, ctrl.onDisplayRuleEditorClose, ctrl);

        var context = sas.analytics.riskhp.models.HPRiskModelManager.getCrosstabContext(data.crosstabId);
        var crosstab = context.crosstab;

        var mainContent = this.displayRulesCrosstabPanel.getAggregation("_mainContentWrapper");
        mainContent.setAggregation("content", crosstab);
        // unset this.computedVarCrosstabPanel

        var centerArea = view.getCenterSplitterPanel().getCenter();
        centerArea.removeAllContent();
        centerArea.addContent(view.getMainTabbar());
        view.getCenterSplitterPanel().setCenter(centerArea);

        var explorationPanesStateModel = sap.ui.getCore().getModel(HprConstants.MODEL_ID_EXPLORATION_PANES_STATE);
        explorationPanesStateModel.setProperty("/computedVariableEditorMode", false);
        explorationPanesStateModel.setProperty("/displayRuleEditorMode", false);

        explorationPanesStateModel.setProperty("/panes/dataPane/dataSourceToolbarEnabled", true);

        explorationPanesStateModel.setProperty("/panes/objectsPane/enabled", true);
        explorationPanesStateModel.setProperty("/panes/optionsPane/enabled", true);
        explorationPanesStateModel.setProperty("/panes/filtersPane/enabled", true);
        explorationPanesStateModel.setProperty("/panes/hierarchiesPane/enabled", true);
        explorationPanesStateModel.setProperty("/panes/displayRulesPane/enabled", true);

        // var rightPropertyPane = view.getRightPropertyPaneLayout();
        // rightPropertyPane.setPropertyPaneVisible(true);
        // rightPropertyPane.getAggregation("_rightExpandCollapseButton").setEnabled(true);

        view.getSaveButton().setEnabled(true);
        view.getOverflowButton().setEnabled(true);
        view.getBackButton().setEnabled(true);

    },

    onIconTabBarSelectPage: function(evt) {
        // IconTabBar 'select' event is fired when new Tabs/Pages are added, as well as, when user changes Tabs/Pages;
        // So, do not perform any actions if there is just one page, if initial render has not been done, or reset method
        // is updating/adding controls (this is because we add any number of Tabs/Pages from saved Explorations or when we
        // create a new Exploration)
        var source = evt.oSource;
        if (source.getItems().length === 1 || !this.initialExplorationRenderDone || !this.resetExplorationEventsAllowed || !this.addNewPageToIconTabBarDone) {
            return;
        }

        // Get the selected Page and use the first content object (should be 'sas.risk.common.panelsview.RiskPanelsView') of the selected Page (IconTabFilter)
        var selectedPage = evt.getParameters().item;
        var riskPanelsView = selectedPage.getContent()[0];

        // If a Data Item label has been updated, we need to check the RiskPanelsView the user is trying to navigate to.  We need to determine if the RiskPanelsView
        // should be refreshed so it will pick up the change.
        // NOTE: The original code checked the lastDataItemLabelRefreshDate with the last time the riskpanels view was refreshed and
        // would only refresh the panels if riskPanelsViewLastRefreshDate < this.lastDataItemLabelRefreshDate this appeared
        // to conflict with rendering when coming back to a tab (particularly in MS Edge).  David C. and I looked at this
        // and this code appeared (as best we could remember) to be an attempt to stop aggressive refreshes.  However,
        // upon further thought, this might have been erroneous on our part, so to make rendering and Edge happy I am
        // removing the check.  The end result of this is when a tab is changed the panels will be refreshed, regardless.
        // One unintended side affect of this is it 'might' cause additional queries, although with the advent of the
        // CacheManager, (which was introduced after this design was put in place) that might not be as big of a problem.
        // See DEFECT S1454850
        if (riskPanelsView) {
            riskPanelsView.refreshAllPanels({});
        }

        // Update the right-side panels for the new selected Page
        sas.analytics.riskhp.utils.HprUtil.updateRightPanelsForSelectedPage(selectedPage);

        // Update the dirty state of the Exploration after changing pages/tabs
        this._updateDirtyState(true);
    },

    onIconTabBarPageOptionsPress: function(evt) {
        var ctrl = this;
        var view = ctrl.getView();

        // Quick check to see if user clicked the "Close page" menu item, if so, return out
        var oItem = evt.getParameter("item");
        if (oItem) {
            var key = oItem.getProperty("key");
            var selected = oItem.getProperty("selected");
            if (key === "close" && selected) {
                return;
            }
        }

        // Find the tab/page the user is working with and show Page options
        var oTab = evt.getParameter("tab");
        if (oTab) {
            // Make sure the Options pane is showing
            var rightPropertyPane = view.getRightPropertyPaneLayout();
            rightPropertyPane.setSelectedPropertyPaneIndex(0);
            rightPropertyPane.setPropertyPaneVisible(true);

            // Update the right-side panels for the new selected Page; In a timeout function
            // because setting PropertyPane to visible causes a re-render; We want the correct
            // Page options for the Tab/Page the user selected to actually show up
            setTimeout(function () {
                sas.analytics.riskhp.utils.HprUtil.updateRightPanelsForSelectedPage(oTab);
            }, 0);
        }
    },

    onIconTabBarAddPage: function(evt) {
        this.addPageToExploration(false);
    },

    addPageToExploration: function(bAddDefaultCrosstab) {
        // Set a flag that signals that a new Page is done being added; This helps block the code in the 'select' event from running on an Add event
        this.addNewPageToIconTabBarDone = false;

        var ctrl = this;
        var view = ctrl.getView();
        var mainTabbar = view.getMainTabbar();

        // Create a unique Page object to add to the middle part of the View; Pass in the existing Page/Tabs so we won't have a duplicate
        var uniquePageObject = sas.analytics.riskhp.models.HPRiskModelManager.createUniquePageObject(mainTabbar.getItems());

        // Add the new Page object to the model manager to keep track of it
        sas.analytics.riskhp.models.HPRiskModelManager.addPageObject(uniquePageObject);

        // Create a new IconTabFilter for the new unique Page
        var tabFilter = new sas.hc.m.IconTabFilter({
            design: "Horizontal",
            iconColor: "Neutral",
            text: uniquePageObject.name,
            key: uniquePageObject.id,
            closable: true
        });

        // Create a new RiskPanelsView and add it to the IconTabFilter
        var newView = new sas.risk.common.panelsview.RiskPanelsView(uniquePageObject.id, {
            delegate: new sas.analytics.riskhp.utils.HPRiskPanelsViewDelegate(),
            dirtyChange: $.proxy(ctrl._onRiskPanelsViewDirtyChange, ctrl),
        }, bAddDefaultCrosstab);
        tabFilter.addContent(newView);

        // Add the new IconTabFilter to the main tabbar
        mainTabbar.addItem(tabFilter);
        mainTabbar.setSelectedKey(uniquePageObject.id); // This sets focus to new Page and fires 'select' event on IconTabBar

        // When adding a new Page/Tab, the first tab should become closable
        mainTabbar.setFirstTabClosable(true);
        var firstTab = mainTabbar.getItems()[0];
        firstTab.setClosable(true);

        // Update the dirty state of the Exploration after adding a new page/tab
        this._updateDirtyState(true);

        this.addNewPageToIconTabBarDone = true; // Update the Add page flag
    },

    onIconTabBarClosePage: function(evt) {
        var self = this;
        var source = evt.oSource;
        var tab = evt.getParameters().tab; // Get the tab to close

        var closeTab = function() {
            source.closeTab(tab);

            // Delete the PageObject from the ModelManager
            sas.analytics.riskhp.models.HPRiskModelManager.deletePageObject(tab.getKey());

            // When there is only one tab left, don't allow it to be closed
            if (source.getItems().length === 1) {
                source.setFirstTabClosable(false);
                var firstTab = source.getItems()[0];
                firstTab.setClosable(false);

                // Make sure first tab has Page options showing
                sas.analytics.riskhp.utils.HprUtil.updateRightPanelsForSelectedPage(firstTab);
            } else {
                // Find the page options model which is currently being displayed
                var currentPageOptionsModel = self.getView().getOptionsPane().getOptionsPaneView().getOptionsPanelModel(HprConstants.HPR_OPTIONS_PANE_PAGE_TYPE);

                // If the tab we are closing is currently the one displayed on the Page Options, we need to update Options pane to selected tab
                if (currentPageOptionsModel && currentPageOptionsModel.getData() && (tab.getProperty("key") === currentPageOptionsModel.getData().options.id)) {
                    var selectedTab = source.getSelectedItem();
                    sas.analytics.riskhp.utils.HprUtil.updateRightPanelsForSelectedPage(selectedTab);
                }
            }

            // Update the dirty state of the Exploration after deleting a page/tab
            self._updateDirtyState(true);
        };

        var closeButton = new sas.hc.m.Button({
            text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("IconTabBar.close.page.confirmation.dialog.close.button.txt"),
            press: function (evt) {
                closeTab();
            }
        });

        // Create an id that will be used by all Tabs so each one can be suppressed by the Global Settings
        var closeTabConfirmationDialogId = "closeTabConfirmationDialog";
        // Check the global settings to see if the dialog is being blocked from opening
        if (sas.hc.ui.core.settings.GlobalSettingsHelper.getSuppressedMessages().indexOf(closeTabConfirmationDialogId) === -1) {
            // Dialog is not being blocked so construct a new instance to prompt the user
            var closeTabConfirmationDialog = new sas.hc.m.MessageDialog({
                id: closeTabConfirmationDialogId,
                type: sas.hc.m.MessageDialogType.Warning,
                text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("IconTabBar.close.page.confirmation.dialog.warning.fmt", [tab.getText()]),
                title: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("IconTabBar.close.page.confirmation.dialog.title"),
                includeDontShowCheckBox: true,
                buttonActions: [
                    closeButton,
                    {
                        text: sas.hc.m.MessageDialogAction.Cancel,
                        initialFocus: true,
                        defaultButton: true
                    }
                ],
                afterClose: function(evt) {
                    this.destroy();
                }
            });
            closeTabConfirmationDialog.open();
        } else {
            // Dialog is blocked from opening, but we still need to close the tab
            closeTab();
        }
    },

    /**
     * Returns the active panels view for the exploration view
     * @private
     */
    _getActivePanelsView: function() {
        var ctrl = this;
        var view = ctrl.getView();
        var mainTabBar = view.getMainTabbar();
        var selectedPage = ctrl._getSelectedPage(mainTabBar);
    
        // Update the title for the selected component/content
        if (selectedPage) {
            // Gather the first content object (should be 'sas.risk.common.panelsview.RiskPanelsView') of the selected Page (IconTabFilter)
            return selectedPage.getContent()[0];
        }
        return null;
    },

    //*****************************************************************************************************
    //*****************************************************************************************************
    // Splitter handlers
    //*****************************************************************************************************
    //*****************************************************************************************************

    _onLeftPropertyPanelLayoutSplitterUpdate: function(evt) {
        var ctrl = this;
        var view = ctrl.getView();
        var leftPropertyPanelLayout = evt.getSource();
        var leftSplitterPanel = view.getLeftSplitterPanel();
        var mainLayout = view.getMainLayout();

        // When the PropertyPane is updated and needs to redraw (i.e closed or opened) we also need to move the Splitter
        if (leftPropertyPanelLayout.getLeftPropertyPaneVisible()) {
            // When the PropertyPane is opened, only move the Splitter back to the default position if it is currently at the min size (i.e. closed);
            // This prevents the Splitter from moving when another PropertyPane button is pushed since this event fires then too
            if (leftSplitterPanel.getLayoutData().getSize() === HprConstants.HPR_EXPLORATION_VIEW_SIDE_PANEL_MIN_PIXEL_SIZE) {
                leftSplitterPanel.getLayoutData().setSize(HprConstants.HPR_EXPLORATION_VIEW_LEFT_PANEL_DEFAULT_SIZE);
                leftSplitterPanel.getLayoutData().setResizable(true);
                mainLayout.invalidate();

                // Check to see if the Data Pane tree is hidden; if so, show it but do it when there is time (avoids a infinite running script problem on IE - S1390297)
                var dataSourceTreeTable = view.getDataPane().getDataPaneView().getDataSourceTreeTable();
                var dataSourceTreeTableVisible = dataSourceTreeTable.getVisible();
                if (!dataSourceTreeTableVisible) {
                    setTimeout(function () {
                        dataSourceTreeTable.setVisible(true);
                    }, 0);
                }
            }
        } else {
            // Hide Data Pane tree (did this to get around a problem on IE where the Data Pane tree gets into an infinite running script problem - S1390297)
            view.getDataPane().getDataPaneView().getDataSourceTreeTable().setVisible(false);

            // When the PropertyPane is closed, close the Splitter too and remove the ability to resize
            leftSplitterPanel.getLayoutData().setSize(HprConstants.HPR_EXPLORATION_VIEW_SIDE_PANEL_MIN_PIXEL_SIZE);
            leftSplitterPanel.getLayoutData().setResizable(false);
            mainLayout.invalidate();
        }

        // Update the dirty state of the Exploration after updating the left panel
        if (ctrl.initialExplorationRenderDone && ctrl.resetExplorationEventsAllowed) {
            ctrl._updateDirtyState(true);
        }
    },

    _onRightPropertyPanelLayoutSplitterUpdate: function(evt) {
        var ctrl = this;
        var view = ctrl.getView();
        var rightPropertyPanelLayout = evt.getSource();
        var rightSplitterPanel = view.getRightSplitterPanel();
        var mainLayout = view.getMainLayout();

        // When the PropertyPane is updated and needs to redraw (i.e closed or opened) we also need to move the Splitter
        if(rightPropertyPanelLayout.getRightPropertyPaneVisible()) {
            // When the PropertyPane is opened, only move the Splitter back to the default position if it is currently at the min size (i.e. closed);
            // This prevents the Splitter from moving when another PropertyPane button is pushed since this event fires then too
            if (rightSplitterPanel.getLayoutData().getSize() === HprConstants.HPR_EXPLORATION_VIEW_SIDE_PANEL_MIN_PIXEL_SIZE) {
                rightSplitterPanel.getLayoutData().setSize(HprConstants.HPR_EXPLORATION_VIEW_RIGHT_PANEL_DEFAULT_SIZE);
                rightSplitterPanel.getLayoutData().setResizable(true);
                mainLayout.invalidate();
            }
        } else {
            // When the PropertyPane is closed, close the Splitter too and remove the ability to resize
            rightSplitterPanel.getLayoutData().setSize(HprConstants.HPR_EXPLORATION_VIEW_SIDE_PANEL_MIN_PIXEL_SIZE);
            rightSplitterPanel.getLayoutData().setResizable(false);
            mainLayout.invalidate();
        }

        // Update the dirty state of the Exploration after updating the right panel
        if (ctrl.initialExplorationRenderDone && ctrl.resetExplorationEventsAllowed) {
            ctrl._updateDirtyState(true);
        }
    },

    _onSplitterResize: function(evt) {
        // This method is fired when resizing of the Splitter stops
        var ctrl = this;
        var view = ctrl.getView();

        // Gather new panel sizes passed in from event (0 is left, 1 is center, 2 is right)
        var newLeftPaneSize = evt.getParameters().newSizes[0];
        var newRightPaneSize = evt.getParameters().newSizes[2];

        // If the left or right side have been re-sized to the minimum size, set the propertyPaneVisible
        // value to 'false' so proper PropertyPane methods run (i.e. updateControls, buttons are switched)
        if (newLeftPaneSize <= HprConstants.HPR_EXPLORATION_VIEW_SIDE_PANEL_MIN_SIZE) {
            view.getLeftPropertyPaneLayout().setLeftPropertyPaneVisible(false);
        }
        if (newRightPaneSize <= HprConstants.HPR_EXPLORATION_VIEW_SIDE_PANEL_MIN_SIZE) {
            view.getRightPropertyPaneLayout().setRightPropertyPaneVisible(false);
        }

        // Update the dirty state of the Exploration after resizing of the Splitter
        // _onSplitterResize runs during initial render of ExplorationView
        if (this.initialExplorationRenderDone && this.resetExplorationEventsAllowed) {
            this._updateDirtyState(true);
        }
    },

    //*****************************************************************************************************
    //*****************************************************************************************************
    // OverflowMenu creation methods
    //*****************************************************************************************************
    //*****************************************************************************************************

    _createOverflowMenu: function() {
        var ctrl = this;
        var menu = new sas.hc.m.Menu({id: ctrl.createId('-overflow-menu')});
        menu.addItem(ctrl._createNewOverflowMenuItem());
        menu.addItem(ctrl._createFindOverflowMenuItem());
        menu.addItem(ctrl._createSaveOverflowMenuItem());
        menu.addItem(ctrl._createSaveAsOverflowMenuItem());
        //menu.addItem(ctrl._createResetDataPanelOverflowMenuItem());
        //menu.addItem(ctrl._createApplyViewFromOverflowMenuItem());
        //menu.addItem(ctrl._createExportToExcelOverflowMenuItem());
        menu.addItem(ctrl._createCopyLinkOverflowMenuItem());

        // only display this if the URL parameter is present
        if (this.HprConstants.ENABLE_DEBUG === true) {
            menu.addItem(ctrl._createDebugOverflowMenuItem());
        }

        return menu;
    },

    _createNewOverflowMenuItem: function() {
        var self = this;
        var resourceKey = "hprisk.new.menu.action.label";
        var menuItem = new sas.hc.m.MenuItem({
            //id: self.createId(id),
            text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText(resourceKey)
        });
        menuItem.attachPress($.proxy(sas.analytics.riskhp.utils.HprUtil.handleNewExploration, this));
        return menuItem;
    },

    _createFindOverflowMenuItem: function() {
        var self = this;
        var resourceKey = "hprisk.find.menu.action.label";
        var menuItem = new sas.hc.m.MenuItem({
            //id: self.createId(id),
            text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText(resourceKey)
        });
        menuItem.attachPress($.proxy(sas.analytics.riskhp.utils.HprUtil.handleFindExploration, this));
        return menuItem;
    },

    _createSaveOverflowMenuItem: function() {
        var self = this;
        var resourceKey = "hprisk.save.menu.action.label";
        var menuItem = new sas.hc.m.MenuItem({
            //id: self.createId(id),
            text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText(resourceKey),
            enabled: "{/exploration/isDirty}",
            startsSection: true
        });
        menuItem.attachPress($.proxy(this.onSaveExploration, this));
        return menuItem;
    },

    _createSaveAsOverflowMenuItem: function() {
        var self = this;
        var resourceKey = "hprisk.saveas.menu.action.label";
        var menuItem = new sas.hc.m.MenuItem({
            //id: self.createId(id),
            text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText(resourceKey)
        });
        menuItem.attachPress($.proxy(this.onSaveExplorationAs, this));
        return menuItem;
    },

    //_createResetDataPanelOverflowMenuItem: function() {
    //     var self = this;
    //     var resourceKey = "hprisk.resetDataPanel.menu.action.label";
    //     var menuItem = new sas.hc.m.MenuItem({
    //         //id: self.createId(id),
    //         text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText(resourceKey),
    //         enabled: false, // Disable until working
    //         startsSection: true
    //     });
    //     menuItem.attachPress($.proxy(this._onResetDataPanelOverflowMenuItemSelectChange, this));
    //     return menuItem;
    //},

    //_createApplyViewFromOverflowMenuItem: function() {
    //     var self = this;
    //     var resourceKey = "hprisk.applyViewFrom.menu.action.label";
    //     var menuItem = new sas.hc.m.MenuItem({
    //         //id: self.createId(id),
    //         text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText(resourceKey),
    //         enabled: false, // Disable until working
    //         startsSection: true
    //     });
    //     menuItem.attachPress($.proxy(this._onApplyViewFromOverflowMenuItemSelectChange, this));
    //     return menuItem;
    //},

    //_createExportToExcelOverflowMenuItem: function() {
    //     var self = this;
    //     var resourceKey = "hprisk.exportToExcel.menu.action.label";
    //     var menuItem = new sas.hc.m.MenuItem({
    //         //id: self.createId(id),
    //         text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText(resourceKey),
    //         enabled: false // Disable until working
    //     });
    //     menuItem.attachPress($.proxy(this._onExportToExcelOverflowMenuItemSelectChange, this));
    //     return menuItem;
    //},

    _createCopyLinkOverflowMenuItem: function() {
        var self = this;
        var resourceKey = "hprisk.copyLink.menu.action.label";
        var menuItem = new sas.hc.m.MenuItem({
            //id: self.createId(id),
            text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText(resourceKey),
            enabled: true,
            startsSection: true,
            tooltip: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("hprisk.copyLink.menu.action.tooltip.txt"),
        });
        menuItem.attachPress($.proxy(this._onCopyLinkOverflowMenuItemSelectChange, this));
        return menuItem;
    },

    _createDebugOverflowMenuItem: function() {
        var self = this;
        var resourceKey = "hprisk.debug.menu.action.label";
        var menuItem = new sas.hc.m.MenuItem({
            //id: self.createId(id),
            text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText(resourceKey)
        });
        menuItem.attachPress($.proxy(this.onDebug, this));
        return menuItem;
    },

    //_onResetDataPanelOverflowMenuItemSelectChange: function(evt) {},
    //_onApplyViewFromOverflowMenuItemSelectChange: function(evt) {},
    // _onExportToExcelOverflowMenuItemSelectChange: function(evt) {},

    _getURLForExploration: function (explorationId) {
        var newQuery = "";
        var query = window.location.search.substring(1);  // returns everything after the ?
        var vars = query.split("&");
        var hasExplorationParameter = false;
        var hasInitialViewParameter = false;

        for (var i = 0; i < vars.length; i++) {
            var pair = vars[i].split("=");
            if (pair[0] === HprConstants.HPR_URL_PARAMETER_EXPLORATION) {
                pair[1] = explorationId;
                hasExplorationParameter = true;
            } else if (pair[0] === HprConstants.HPR_URL_PARAMETER_INITIAL_VIEW) {
                pair[1] = HprConstants.HPR_EXPLORATION_VIEW_ID;
                hasInitialViewParameter = true;
            }

            if (pair[0] !== HprConstants.HPR_URL_PARAMETER_LAUNCHED_FROM_APP_SWITCHER) {
                newQuery = newQuery + (newQuery.length === 0 ? "?" : "&") + pair[0] + "=" + pair[1];
            }
        }

        if (!hasInitialViewParameter) {
            newQuery = newQuery + (newQuery.length === 0 ? "?" : "&") + HprConstants.HPR_URL_PARAMETER_INITIAL_VIEW + "=" + HprConstants.HPR_EXPLORATION_VIEW_ID;
        }
        // if there wasn't already an exploration parameter, add it
        if (!hasExplorationParameter) {
            newQuery =  newQuery + "&" + HprConstants.HPR_URL_PARAMETER_EXPLORATION + "=" + explorationId;
        }
        var newurl = window.location.protocol + "//" + window.location.host + window.location.pathname + newQuery;
        return newurl;
    },

    _onCopyLinkOverflowMenuItemSelectChange: function() {
        var self = this;
        var explorationId = sas.analytics.riskhp.models.HPRiskModelManager.getExplorationModel().getData().exploration.id;
        if (explorationId) {
            var explorationUrl = self._getURLForExploration(explorationId);
            sas.analytics.riskhp.utils.HprUtil.copyTextToClipboard(explorationUrl);
            sas.hc.m.MessageToast.show(sas.analytics.riskhp.utils.HprUtil.getBundle().getText('ExplorationView.copy.link.toast.txt'));
        } else {
            sas.analytics.riskhp.utils.HprUtil.displayGenericErrorDialog(sas.analytics.riskhp.utils.HprUtil.getBundle().getText('ExplorationView.copy.link.exploration.must.be.saved.first.txt'));
        }
    }
});