// (c) 2017, SAS Institute Inc.

jQuery.sap.require("sas.hc.ui.commons.propPane.PropertyPaneLayout");

jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.risk.common.widgets.WidgetFactory");

sap.ui.jsview('sas.analytics.riskhp.views.ExplorationView', (function() {

    var semanticId = 'ExplorationView';
    var hprUiToolbar;
    var backButton;
    var explorationNameLabel;
    var saveButton;
    //var refreshButton;
    //var exportButton;
    var overflowButton;
    var mainLayout;
    var leftSplitterPanel;
    var leftPropertyPaneLayout;
    var dataPane;
    var centerSplitterPanel;
    var rightSplitterPanel;
    var rightPropertyPaneLayout;
    var optionsPane;
    var filtersPane;
    var hierarchiesPane;
    var displayRulesPane;
    var mainTabbar;
    var splitter;

    // Create shortcut variables for commonly used classes
    var HprConstants = sas.analytics.riskhp.utils.HprConstants;

    function _createHprUiToolbar(ctrl) {
        backButton = new sas.hc.m.Button({
            id: semanticId + '-back-button',
            icon: sas.icons.HC.BACKTOLIST,
            press: $.proxy(ctrl.onBack, ctrl),
            tooltip: sas.analytics.riskhp.utils.HprUtil.getBundle().getText('hprisk.backButton.tip')
        }).addStyleClass("sasHPRiskUI-toolbarHomeButton");

        explorationNameLabel = new sas.hc.m.Label({
            id: semanticId + '-explorationName-label',
            text: "{/exploration/name}"
        }).addStyleClass("sasHPRiskCommon-leftMargin sasHPRiskUI-explorationNameLabel");

        // refreshButton = new sas.hc.m.Button({
        //     id: semanticId + '-refresh-button',
        //     icon: sas.icons.HC.REFRESH,
        //     press: $.proxy(ctrl.onRefreshExploration, ctrl),
        //     enabled: false,
        //     tooltip: sas.analytics.riskhp.utils.HprUtil.getBundle().getText('hprisk.refreshExplorationButton.tip')
        // });

        // exportButton = new sas.hc.m.Button({
        //     id: semanticId + '-export-button',
        //     icon: sas.icons.HC.EXPORTTOEXCEL,
        //     press: $.proxy(ctrl.onExportExploration, ctrl),
        //     enabled: false,
        //     tooltip: sas.analytics.riskhp.utils.HprUtil.getBundle().getText('hprisk.exportExplorationButton.tip')
        // });

        saveButton = new sas.hc.m.Button({
            id: semanticId + '-save-button',
            icon: sas.icons.HC.SAVE,
            enabled: {
                parts: [{path: "/exploration/isDirty"},
                        {path:  HprConstants.MODEL_ID_EXPLORATION_PANES_STATE + ">/computedVariableEditorMode"}],
                formatter: function(isDirty, computedVarEditorMode) {
                    return isDirty && !computedVarEditorMode;
                }
            },
            press: $.proxy(ctrl.onSaveExploration, ctrl),
            tooltip: sas.analytics.riskhp.utils.HprUtil.getBundle().getText('hprisk.saveExplorationButton.tip'),
            layoutData: new sas.hc.m.OverflowToolbarLayoutData({
                // Do not show this on overflow MenuButton when screen is small (i.e. Save already in menu there)
                priority: sap.m.OverflowToolbarPriority.Disappear
            })
        });

        // Menu is populated in controller
        overflowButton = new sas.hc.m.MenuButton({
            id: semanticId + '-overflow-button',
            icon: sas.icons.HC.OVERFLOW,
            enabled: true,
            tooltip: sas.analytics.riskhp.utils.HprUtil.getBundle().getText('hprisk.overflowButton.tip')
        }).addStyleClass("sasHPRiskUI-overflowMenuButton");

        hprUiToolbar = new sas.hc.m.ResponsiveToolbar({
            id: semanticId + '-toolbar',
            content : [
                backButton,
                new sas.hc.m.ToolbarSeparator({
                    id: semanticId + '-toolbar-back-separator'
                }),
                explorationNameLabel,
                new sas.hc.m.ToolbarSpacer({
                    id: semanticId + '-toolbar-spacer'
                }),
                //refreshButton,
                //exportButton,
                saveButton,
                overflowButton

            ]
        }).addStyleClass("explorationView-toolbar");

        return hprUiToolbar;
    }

    function _createLeftPropertyPaneLayout(ctrl) {
        // Model for DataPane is set in controller or taken care of when model is set on View
        dataPane = sap.ui.jsfragment("sas.analytics.riskhp.views.datapane.DataPane", ctrl);
        var objectsPane = sap.ui.jsfragment("sas.analytics.riskhp.fragments.ObjectsPane", ctrl);

        var leftPropertyPane = new sas.hc.ui.commons.propPane.PropertyPaneLayout({
            id: semanticId + '-leftPropertyPaneLayout',
            propertyPanes: [dataPane, objectsPane],
            leftPropertyPaneVisible: "{/exploration/analysis/layout/leftPropertyPanel/visible}",
            leftSelectedPropertyPaneIndex: "{/exploration/analysis/layout/leftPropertyPanel/selectedPropertyPaneIndex}",
            propertyPaneVisibleChange: $.proxy(ctrl._onLeftPropertyPanelLayoutSplitterUpdate, ctrl),
            selectionChange: $.proxy(ctrl._onLeftPropertyPanelLayoutSplitterUpdate, ctrl),
            enablePin: false,
            defaultPin: true
        });
        leftPropertyPane.addStyleClass("hprPropertyPane");
        leftPropertyPane.addStyleClass("left");
        return leftPropertyPane;
    }

    function _createRightPropertyPaneLayout(ctrl) {
        optionsPane = sap.ui.jsfragment("sas.analytics.riskhp.fragments.options.OptionsPane", ctrl);
        filtersPane = sap.ui.jsfragment("sas.analytics.riskhp.fragments.filters.FiltersPane", ctrl);
        hierarchiesPane = sap.ui.jsfragment("sas.analytics.riskhp.fragments.HierarchiesPane", ctrl);
        //var commentsPane = sap.ui.jsfragment("sas.analytics.riskhp.fragments.CommentsPane", ctrl); // TODO: Remove for now
        displayRulesPane = sap.ui.jsfragment("sas.analytics.riskhp.fragments.DisplayRulesPane", ctrl);

        var rightPropertyPaneLayout = new sas.hc.ui.commons.propPane.PropertyPaneLayout({
            id: semanticId + '-rightPropertyPaneLayout',
            propertyPanes: [optionsPane, filtersPane, hierarchiesPane, displayRulesPane],
            rightPropertyPaneVisible: "{/exploration/analysis/layout/rightPropertyPanel/visible}",
            rightSelectedPropertyPaneIndex: "{/exploration/analysis/layout/rightPropertyPanel/selectedPropertyPaneIndex}",
            propertyPaneVisibleChange: $.proxy(ctrl._onRightPropertyPanelLayoutSplitterUpdate, ctrl),
            selectionChange: $.proxy(ctrl._onRightPropertyPanelLayoutSplitterUpdate, ctrl),
            enablePin: false,
            defaultPin: true
        });
        rightPropertyPaneLayout.addStyleClass("hprPropertyPane");
        rightPropertyPaneLayout.addStyleClass("right");
        return rightPropertyPaneLayout;
    }

    function _createMainIconTabBar(ctrl) {
        return new sas.hc.m.IconTabBar(semanticId + '-mainTabBar', {
            firstTabClosable: false,
            showOverflowPopOver: false,
            enableDynamicStyle: true,
            //enableTabReordering: true, // TODO: dacagl - Experimental, eval when S1418785 resolved
            showNewTabButton: true,
            addNewTab: false,
            newTabButtonTooltip: sas.analytics.riskhp.utils.HprUtil.getBundle().getText('IconTabBar.add.button.tooltip.txt'),
            newTabAdding: $.proxy(ctrl.onIconTabBarAddPage, ctrl),
            select: $.proxy(ctrl.onIconTabBarSelectPage, ctrl),
            close: $.proxy(ctrl.onIconTabBarClosePage, ctrl),
            menuItems: [
                new sas.hc.m.MenuItem({
                    text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText('IconTabBar.page.options.menuitem.txt')
                })
            ],
            closeMenuItemText: sas.analytics.riskhp.utils.HprUtil.getBundle().getText('IconTabBar.close.page.menuitem.txt'),
            tabMenuItemSelected: $.proxy(ctrl.onIconTabBarPageOptionsPress, ctrl)
        });
    }

    return {

        getControllerName: function() {
            return 'sas.analytics.riskhp.views.ExplorationView';
        },

        createContent: function(ctrl) {
            this.setHeight('100%');
            this.setWidth('100%');

            // Create the left side of Splitter
            leftPropertyPaneLayout = _createLeftPropertyPaneLayout(ctrl);
            leftSplitterPanel = sas.risk.common.widgets.WidgetFactory.buildPanelNoPadding({content: leftPropertyPaneLayout});
            leftSplitterPanel.setLayoutData(new sas.hc.ui.layout.SplitterLayoutData({
                size: "{/exploration/analysis/layout/leftSplitterPanel/size}",
                minSize: HprConstants.HPR_EXPLORATION_VIEW_SIDE_PANEL_MIN_SIZE,
                resizable: true
            }));

            // Create a IconTabBar for the main part of the View; The content is added in ExplorationView.controller.onInit()
            mainTabbar = _createMainIconTabBar(ctrl);

            // Create a panel for the main part of the view
            centerSplitterPanel = sas.risk.common.widgets.WidgetFactory.buildPanelNoPadding({content: mainTabbar});
            centerSplitterPanel.setLayoutData(new sas.hc.ui.layout.SplitterLayoutData({
                size: "{/exploration/analysis/layout/centerSplitterPanel/size}",
                minSize: HprConstants.HPR_EXPLORATION_VIEW_MAIN_PANEL_MIN_SIZE,
                resizable: true
            }));

            // Create the right side of Splitter
            rightPropertyPaneLayout = _createRightPropertyPaneLayout(ctrl);
            rightSplitterPanel = sas.risk.common.widgets.WidgetFactory.buildPanelNoPadding({content: rightPropertyPaneLayout});
            rightSplitterPanel.setLayoutData(new sas.hc.ui.layout.SplitterLayoutData({
                size: "{/exploration/analysis/layout/rightSplitterPanel/size}",
                minSize: HprConstants.HPR_EXPLORATION_VIEW_SIDE_PANEL_MIN_SIZE,
                resizable: true
            }));

            // Splitter
            splitter = new sas.hc.ui.layout.Splitter({
                id: semanticId + '-splitter',
                height: "100%",
                width: "100%",
                contentAreas: [leftSplitterPanel, centerSplitterPanel, rightSplitterPanel]
            });
            splitter.attachResize($.proxy(ctrl._onSplitterResize, ctrl)); // Attach resize handler method
            splitter.addStyleClass('defaultBackground');

            // Main BorderLayout
            mainLayout = new sas.hc.ui.commons.layout.BorderLayout({
                height: '100%',
                width: '100%',
                top: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    contentAlign: "center",
                    size: '2.5rem',
                    content: [_createHprUiToolbar(ctrl)]
                }),
                center: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    contentAlign: "center",
                    overflowX: "hidden",
                    overflowY: "hidden",
                    content: [splitter]
                })
            });

            return mainLayout;
        },

        getMainTabbar: function() {
            return mainTabbar;
        },

        setActivePage: function(pageId) {
            mainTabbar.setSelectedKey(pageId);
        },

        getOverflowButton: function() {
            return overflowButton;
        },

        getBackButton: function() {
            return backButton;
        },

        getSaveButton: function() {
            return saveButton;
        },

        getLeftPropertyPaneLayout: function() {
            return leftPropertyPaneLayout;
        },

        getLeftPropertyPaneJSON: function() {
            return  {
                visible: leftPropertyPaneLayout.getLeftPropertyPaneVisible(),
                selectedPropertyPaneIndex: leftPropertyPaneLayout.getLeftSelectedPropertyPaneIndex()
            }
        },

        getDataPane: function() {
            return dataPane;
        },

        getDataPaneHorizonNodeChildrenCache: function() {
            return jQuery.extend(true, {}, dataPane.getDataPaneView().getController().getHorizonNodeChildrenCache());
        },

        getLeftSplitterPanel: function() {
            return leftSplitterPanel;
        },

        getLeftSplitterPanelJSON: function() {
            return {
                size: leftSplitterPanel.getLayoutData().getSize()
            }
        },

        getCenterSplitterPanel: function() {
            return centerSplitterPanel;
        },

        getCenterSplitterPanelJSON: function() {
            return {
                size: centerSplitterPanel.getLayoutData().getSize()
            }
        },

        getRightPropertyPaneLayout: function() {
            return rightPropertyPaneLayout;
        },

        getRightPropertyPaneJSON: function() {
            return  {
                visible: rightPropertyPaneLayout.getRightPropertyPaneVisible(),
                selectedPropertyPaneIndex: rightPropertyPaneLayout.getRightSelectedPropertyPaneIndex()
            }
        },

        getOptionsPane: function() {
            return optionsPane;
        },

        getFiltersPane: function() {
            return filtersPane;
        },

        getHierarchiesPane: function() {
            return hierarchiesPane;
        },

        getRightSplitterPanel: function() {
            return rightSplitterPanel;
        },

        getRightSplitterPanelJSON: function() {
            return {
                size: rightSplitterPanel.getLayoutData().getSize()
            }
        },

        getMainLayout: function() {
            return mainLayout;
        },

        /**
         * Implementation of the destroy function. This function must clear the view to prevent the router
         * from caching the destroyed view after the user navigates away. Without clearing the view, the
         * router will attempt to load the cached destroyed view and an error will occur.
         */
        destroy: function () {
            sas.analytics.riskhp.utils.ViewManager.getRouter().getViews().setView("sas.analytics.riskhp.views.ExplorationView", null);
            sap.ui.core.mvc.JSView.prototype.destroy.apply(this, arguments);
        }
    }; // return
}()));