// (c) 2017, SAS Institute Inc.

jQuery.sap.require("sas.analytics.riskhp.utils.ViewManager");
jQuery.sap.require("sas.analytics.riskhp.utils.HprAPI");
jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");

sap.ui.controller('sas.analytics.riskhp.views.HprWelcome', {

    onInit: function() {
        var ctrl = this;
        var view = ctrl.getView();

        // Create model for selectType
        var selectTypeData = {
            FilterCollection: [
                {name: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("hprisk.recentSelectOption.txt"), id: "recent"},
                {name: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("hprisk.favoritesSelectOption.txt"), id: "favorite"}
            ]
        };
        var selectTypeModel = new sap.ui.model.json.JSONModel(selectTypeData);
        view.getSelectType().setModel(selectTypeModel);

        // Use the Router to call the '_init' method every-time this view is navigated to
        this.hprRouter = sap.ui.core.routing.Router.getRouter(sas.analytics.riskhp.utils.HprConstants.HPR_ROUTER_ID);
        this.hprRouter.attachExactRouteMatched(sas.analytics.riskhp.utils.HprConstants.HPR_WELCOME_VIEW_ID, this._init, this);

        // NavChecker setup; NavChecker detects navigation requests for a page and warns/prompts
        // a user before proceeding if the page has unsaved changes. It also actively destroys
        // the view when the user navigates away.
        this.navChecker = sas.analytics.riskhp.utils.HprUtil.createNavChecker(true, view);
    },

    _init: function(event) {
        var ctrl = this;
        var view = ctrl.getView();

        // Show busy indicator immediately
        view.setBusyIndicatorDelay(0);
        view.setBusy(true);

        $.when (
            // Load the explorations
            sas.analytics.riskhp.utils.HprAPI.getExplorations()
        ).done(function(explorations) {
            if (explorations && explorations.items) {
                ctrl.explorations = explorations.items; // Save off the list of explorations
            }

            // Create the model for the explorations
            var explorationsModel = new sap.ui.model.json.JSONModel();
            explorationsModel.setData(explorations);

            // Bind the table to the model
            view.getExplorationTable().setModel(explorationsModel);
            view.getExplorationTable().bindRows("/items");

            // Initially sort the table
            view.getExplorationTable().sort(view.getExplorationTable().getColumns()[0]);

            // This code works better here than in View createContent(); This is because we are reloading the Explorations
            // when the Welcome view is shown everytime and if this ZeroStateLayout was shown everytime the view was loaded
            // until the Explorations were read in and the table populated; Putting the code here gets rid of that
            view.getExplorationTable().setNoData(new sas.risk.common.widgets.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATEADD,
                text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("hprisk.explorationTableZeroState.txt"),
                buttons: [
                    new sas.hc.m.Button({
                        enabled: true,
                        press: $.proxy(ctrl._onNewExploration, ctrl),
                        text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("hprisk.newButton.txt"),
                        visible: true
                    }).addStyleClass("sasPrimaryButton"),
                    new sas.hc.m.Button({
                        enabled: true,
                        press: $.proxy(sas.analytics.riskhp.utils.HprUtil.handleFindExploration, ctrl),
                        text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("hprisk.findEllipsisButton.txt"),
                        visible: true
                    })
                ]
            }));

            // // Setup masonry data, model, and content aggregation
            // view.explorationMasonry.setModel(explorationsModel);
            // view.explorationMasonry.bindAggregation('content', '/items', new sas.analytics.riskhp.controls.ExplorationMasonryTile({
            //     explorationName: "{name}",
            //     explorationId: "{id}",
            //     author: "{createdBy}",
            //     lastUpdatedDate: "{modifiedDate}"
            // }));

            // Find the selected item and build a filter
            var selectedItem = view.getSelectType().getSelectedItem();
            ctrl._selectTypeFilterHelper(selectedItem);

            view.updateForSelectionChange();  // update button states

            view.setBusy(false);
        }).fail(function() {
            view.setBusy(false);
            sas.analytics.riskhp.utils.HprUtil.displayGenericErrorDialog();
        });
    },

    onExit: function() {
        this.hprRouter.detachExactRouteMatched(sas.analytics.riskhp.utils.HprConstants.HPR_WELCOME_VIEW_ID, this._init, this);
        this.navChecker.stop(); // Stop handling page navigation requests for this View
    },

    //*****************************************************************************************************
    //*****************************************************************************************************
    // Button handlers
    //*****************************************************************************************************
    //*****************************************************************************************************

    _onNewExploration: function(evt) {
        var ctrl = this;
        var view = ctrl.getView();

        // Set Welcome view to busy state so user can't click anything else on the view while cubes are loading
        view.setBusyIndicatorDelay(0);
        view.setBusy(true);

        $.when (
            // Load the cubes
            sas.analytics.riskhp.utils.HprAPI.getCubes()
        ).done(function(cubes) {
            sas.analytics.riskhp.utils.HprUtil.openNewExplorationDialog(cubes, ctrl.explorations, false, ctrl);
        }).fail(function() {
            sas.analytics.riskhp.utils.HprUtil.displayGenericErrorDialog();
        }).always(function() {
            view.setBusy(false);
        });
    },

    _onOpenExploration: function(evt) {
        var ctrl = this;
        var view = ctrl.getView();

        var explorationTable = view.getExplorationTable();
        var selectedIndices = explorationTable.getSelectedIndices();
        if (selectedIndices && selectedIndices.length === 1) {
            var rowContext = explorationTable.getContextByIndex(selectedIndices[0]);
            if (rowContext) {
                ctrl._onOpenExplorationHelper(rowContext, view);
            }
        }
    },

    _onOpenExplorationHelper: function(rowContext, view) {
        var rowObject = rowContext.getObject();

        // Once an exploration is selected, open it
        view.setBusy(true);

        // Open the selected Exploration; view setBusy is handled in handler functions
        var promise = sas.analytics.riskhp.utils.ViewManager.loadSavedExploration(rowObject.id);
        promise.done(sas.analytics.riskhp.utils.HprUtil.createSuccessHandler(view, null));
        promise.fail(sas.analytics.riskhp.utils.HprUtil.createFailureHandler(view, true, null));
    },

    _onCopyExploration: function() {
        var self = this;
        var view = self.getView();
        var explorationTable = view.getExplorationTable();

        var rowContext = explorationTable.getContextByIndex(explorationTable.getSelectedIndex());
        var explorationData = rowContext.getObject();

        sas.analytics.riskhp.utils.HprUtil.handleCopyExploration(explorationData.id, explorationData.name, this).done(
            function(newExploration) {
                self._onRefreshExplorationList();   // refresh when copy completes
            }
        ).fail(
            function(jqxhr, textStatus, error) {
                sas.analytics.riskhp.utils.HprUtil.displayDetailedErrorDialog(jqxhr);
            }
        );
    },

    _onRefreshExplorationList: function() {
        this._init();
    },
    
    _onDeleteExploration: function() {
        var ctrl = this;
        var view = ctrl.getView();

        var explorationTable = view.getExplorationTable();
        var selectedIndices = explorationTable.getSelectedIndices();
        if (selectedIndices && selectedIndices.length > 0) {
            // Based on the number of Explorations to be deleted, build an appropriate MessageDialog
            var deleteMessageDialog = null;
            if (selectedIndices.length === 1) {
                var rowContext = explorationTable.getContextByIndex(selectedIndices[0]);
                var rowObject = rowContext.getObject();
                deleteMessageDialog = sas.hc.m.MessageDialog.createDeleteItem(rowObject.name);
            } else {
                deleteMessageDialog = sas.hc.m.MessageDialog.createDeleteSelectedItems();
            }
            deleteMessageDialog.attachButtonPress(function(evt) {
                var oButton = evt.getParameter("origin");
                if (oButton.getMessageDialogAction() === sas.hc.m.MessageDialogAction.Delete) {
                    // User confirmed Delete, show busy indicator immediately
                    view.setBusyIndicatorDelay(0);
                    view.setBusy(true);

                    // Construct an array of temporary Exploration Objects to delete which contain the 'id' and 'name' properties
                    var explorationObjectsToDeleteArray = [];
                    $.each(selectedIndices, function(index, selectedIndex) {
                        var rowContext = explorationTable.getContextByIndex(selectedIndex);
                        var rowObject = rowContext.getObject();
                        explorationObjectsToDeleteArray.push({id: rowObject.id, name: rowObject.name});
                    });

                    $.when (
                        // Delete the selected Explorations
                        sas.analytics.riskhp.utils.HprAPI.deleteExplorations(explorationObjectsToDeleteArray)
                    ).done(function() {
                        ctrl._init(); // setBusy taken care of in called method
                    }).fail(function() {
                        sas.analytics.riskhp.utils.HprUtil.displayGenericErrorDialog();
                        view.setBusy(false);
                    });
                }
            });
        }
    },

    /**
     * Launch the dialog for replacing an exploration data source.
     * @param evt
     * @private
     */
    _onReplaceExplorationDataSource: function(evt) {
        sap.ui.jsfragment("sas.analytics.riskhp.fragments.ReplaceExplorationDataSourceDialog", this).launch(this.getView().getSelectedRowContexts()[0].getObject().id);
    },

    // // This method runs when the view is initialized and SegmentButton is created via SegmentButton.setSelectedButton()
    // _onViewButtonsSelect: function(evt) {
    //     var ctrl = this;
    //     var view = ctrl.getView();
    //     var isTableViewSelected = sas.analytics.riskhp.utils.HprUtil.stringEndsWith(evt.getParameters().id, "-tableViewButton");
    //
    //     view.getExplorationTable().setVisible(isTableViewSelected);
    //     view.masonryContainer.setVisible(!isTableViewSelected);
    // },

    //*****************************************************************************************************
    //*****************************************************************************************************
    // Select handlers
    //*****************************************************************************************************
    //*****************************************************************************************************

    _onSelectTypeChange: function (evt) {
        if (evt) {
            var ctrl = this;
            var selectedItem = evt.getSource().getSelectedItem();
            ctrl._selectTypeFilterHelper(selectedItem);
        }
    },

    _selectTypeFilterHelper: function(selectedItem) {
        var ctrl = this;
        var view = ctrl.getView();

        if (selectedItem) {
            var group = selectedItem.data("group");

            view.getExplorationTable().clearSelection();

            // Build a filter based on the group
            if (group) {
                this.groupFilter = new sap.ui.model.Filter({
                    path: "group",
                    operator: sap.ui.model.FilterOperator.EQ,
                    value1: group
                });
            } else {
                this.groupFilter = null;
            }

            this._applyFilters();
        }
    },
    
    //*****************************************************************************************************
    //*****************************************************************************************************
    // Table handlers
    //*****************************************************************************************************
    //*****************************************************************************************************

    _onExplorationTableRowSelectionChange: function(evt) {
        var ctrl = this;
        var view = ctrl.getView();

        var source = evt.getSource();
        if (source) {
            view.updateForSelectionChange(source.getSelectedIndices());
        }
    },

    _onExplorationTableDoubleClick: function(evt) {
        var ctrl = this;
        var view = ctrl.getView();

        // Find if the row is selected and navigate to associated risk exploration
        var rowIndex = evt.getParameters().rowIndex;
        if (rowIndex > -1) {
            var rowContext = view.getExplorationTable().getContextByIndex(rowIndex);
            if (rowContext) {
                ctrl._onOpenExplorationHelper(rowContext, view);
            }
        }
    },

    //*****************************************************************************************************
    //*****************************************************************************************************
    // Masonry handlers
    //*****************************************************************************************************
    //*****************************************************************************************************

    _onExplorationMasonryOpenItem: function(evt) {
        var ctrl = this;
        var view = ctrl.getView();
        view.setBusy(true);
        evt.cancelBubble();

        // Open the selected Exploration
        var tile = evt.getSource();
        sas.analytics.riskhp.utils.ViewManager.loadSavedExploration(
            tile.getProperty("explorationId"),
            sas.analytics.riskhp.utils.HprUtil.createSuccessHandler(view,null),
            sas.analytics.riskhp.utils.HprUtil.createFailureHandler(view,true,null));
    },

    //*****************************************************************************************************
    //*****************************************************************************************************
    // Search handlers
    //*****************************************************************************************************
    //*****************************************************************************************************

    _onSearchLiveChange: function(evt) {
        var self = this;
        var searchValue = evt.getSource().getValue();
        var delay = (searchValue ? 300 : 0); // no delay if value is empty
        var view = this.getView();

        view.getExplorationTable().clearSelection();

        // Execute search after user stopped typing for 300ms
        clearTimeout(this.liveChangeTimer);
        if (delay) {
            this.liveChangeTimer = setTimeout(function () {
                self._filterTable(searchValue);
            }, delay);
        } else {
            self._filterTable(searchValue);
        }
    },

    _filterTable: function(searchValue) {
        if(searchValue && searchValue.length > 0) {
            this.searchFilter = new sap.ui.model.Filter({
                path: "name",
                operator: sap.ui.model.FilterOperator.Contains,
                value1: searchValue
            });
        } else {
            this.searchFilter = null;
        }

        this._applyFilters();
    },

    _applyFilters: function(evt) {
        var ctrl = this;
        var view = ctrl.getView();
        var filters = [];

        if(ctrl.groupFilter) {
            filters.push(ctrl.groupFilter);
        }
        if(ctrl.searchFilter) {
            filters.push(ctrl.searchFilter);
        }

        if(view.getExplorationTable().isBound("rows")) {
            // Apply the search filter to the table
            view.getExplorationTable().getBinding("rows").filter(filters);
        }

        // if(view.explorationMasonry.isBound("content")) {
        //     // Apply the search filter to the masonry
        //     view.explorationMasonry.getBinding("content").filter(filters);
        // }
    }
});