// (c) 2017, SAS Institute Inc.

jQuery.sap.require("sas.hc.ui.layout.MasonryLayoutMode");

jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.analytics.riskhp.utils.ViewManager");

jQuery.sap.require("sas.risk.common.layout.BarLayout");
jQuery.sap.require("sas.risk.common.layout.BarLayoutSpacer");
jQuery.sap.require("sas.risk.common.widgets.ZeroStateLayout");

jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.MenuButton");
jQuery.sap.require("sas.hc.m.MenuItem");
jQuery.sap.require("sas.hc.m.Select");
jQuery.sap.require("sas.hc.m.SearchField");

jQuery.sap.require("sas.hc.ui.commons.layout.BorderLayout");
jQuery.sap.require("sas.hc.ui.commons.layout.BorderLayoutArea");
jQuery.sap.require("sas.hc.ui.table.Table");
jQuery.sap.require("sas.hc.ui.table.Column");


sap.ui.jsview('sas.analytics.riskhp.views.HprWelcome', (function() {

    var semanticId = 'HprWelcome';

    // controls
    var selectType;
    var newButton;
    var copyButton;
    var deleteButton;
    var refreshButton;
    var openExplorationMenuItem;
    var replaceDataSourceMenuItem;
    var explorationTable;


    function _createHprWelcomeContent(ctrl) {

        var introText = new sas.hc.m.Label({
            text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("hprisk.introText.txt")
        }).addStyleClass("sasHPRiskWelcome-introText");

        var findButton = new sas.hc.m.Button({
            id: semanticId + '-find-button',
            text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("hprisk.findEllipsisButton.txt"),
            press: $.proxy(sas.analytics.riskhp.utils.HprUtil.handleFindExploration, ctrl)
        });

        var topBarLayout = new sas.risk.common.layout.BarLayout({
            id: semanticId + '-topBarLayout',
            content: [introText, new sas.risk.common.layout.BarLayoutSpacer(), findButton]
        }).addStyleClass("sasHPRiskWelcome-topBarLayout");

        var searchField = new sas.hc.m.SearchField({
            id: semanticId + '-searchField',
            filterMode: true,
            liveChange: $.proxy(ctrl._onSearchLiveChange, ctrl) // Filter results when user types after slight delay
        });

        selectType = new sas.hc.m.Select({
            id: semanticId + '-selectField',
            items: {
                path: "/FilterCollection",
                template: new sas.hc.ui.core.Item({
                    key: '{id}',
                    text: '{name}',
                    customData: new sas.hc.ui.core.CustomData({key: "group", value: '{id}'})
                })
            },
            change: $.proxy(ctrl._onSelectTypeChange, ctrl)
        }).addStyleClass("sasHPRiskWelcome-selectType");

        /* TODO: Comment out Masonry view because it is currently not used
        var viewButtons = new sas.hc.m.SegmentedButton({
            id: semanticId + '-viewButtons',
            select: $.proxy(ctrl._onViewButtonsSelect, ctrl),
            buttons: [
                new sas.hc.m.Button({
                    id: semanticId + '-tableViewButton',
                    icon: sas.icons.HC.LISTTABLE
                }),
                new sas.hc.m.Button({
                    id: semanticId + '-masonryViewButton',
                    icon: sas.icons.HC.VIEWTILES
                })
            ]
        }).addStyleClass("sasHPRiskWelcome-viewButtons");*/

        newButton = new sas.hc.m.Button({
            id: semanticId + '-new-button',
            icon: sas.icons.HC.NEWSTARBURST,
            tooltip: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("hprisk.newButton.tip"),
            press: $.proxy(ctrl._onNewExploration, ctrl)
        });

        copyButton = new sas.hc.m.Button({
            id: semanticId + '-copy-button',
            icon: sas.icons.HC.COPY,
            tooltip: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("hprisk.copyButton.tip"),
            enabled: false,
            press: $.proxy(ctrl._onCopyExploration, ctrl)
        });

        deleteButton = new sas.hc.m.Button({
            id: semanticId + '-delete-button',
            icon: sas.icons.HC.DELETE,
            tooltip: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("hprisk.deleteButton.tip"),
            enabled: false,
            press: $.proxy(ctrl._onDeleteExploration, ctrl)
        });

        var refreshButton = new sas.hc.m.Button({
            id: semanticId + '-refresh-button',
            icon: sas.icons.HC.REFRESH,
            tooltip: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("hprisk.refreshButton.tip"),
            press: $.proxy(ctrl._onRefreshExplorationList, ctrl)
        });

        var overflowMenuButton = new sas.hc.m.MenuButton({
            overflowPopupBehavior: sas.hc.m.MenuButton.OverflowPopupBehavior.Expand,
            layoutData: new sas.hc.m.OverflowToolbarLayoutData({
                priority: sap.m.OverflowToolbarPriority.AlwaysOverflow
            }),
            menu:   new sas.hc.m.Menu({
                items: _createOverflowMenuItems(ctrl)
            })
        });

        var centerControlsToolbar = new sas.hc.m.ResponsiveToolbar({
            content: [searchField, selectType/*, viewButtons*/, new sas.hc.m.ToolbarSpacer(), newButton, copyButton, deleteButton, refreshButton, overflowMenuButton]
        });

        explorationTable = new sas.hc.ui.table.Table({
            id: semanticId + '-explorationTable',
            //visible: false, // Made visible via SegmentButton.select() method
            selectionMode: sas.hc.ui.table.SelectionMode.MultiToggle,
            selectionBehavior: sas.hc.ui.table.SelectionBehavior.RowSelector,
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            minAutoRowCount: 9,
            showTableOptions: false,
            editable: false,
            rowSelectionChange: $.proxy(ctrl._onExplorationTableRowSelectionChange, ctrl),
            doubleClick: $.proxy(ctrl._onExplorationTableDoubleClick, ctrl),
            toolbar: centerControlsToolbar,
            columns: [
                new sas.hc.ui.table.Column({
                    label: new sas.hc.m.Label({
                        text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("hprisk.explorationTableNameColumn.txt")
                    }),
                    template: new sas.hc.m.HBox({
                        items: [
                            new sas.hc.ui.core.Icon({
                                size: "1rem",
                                src: sas.icons.HC.REPORT
                            }).addStyleClass("sasHPRiskCommon-rightMargin"),
                            new sas.hc.m.Text().bindProperty("text", "name")
                        ],
                        alignItems: sap.m.FlexAlignItems.Center
                    }),
                    sortProperty: "name",
                    width: "60%"
                }),
                new sas.hc.ui.table.Column({
                    label: new sas.hc.m.Label({
                        text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("hprisk.explorationTableAuthorNameColumn.txt")
                    }),
                    template: new sas.hc.m.Text().bindProperty("text", "createdBy"),
                    sortProperty: "createdBy",
                    width: "40%"
                })
                // S1448754, S1393818: Hide 'Last Updated' column because it does not update correctly
                // new sas.hc.ui.table.Column({
                //     label: new sas.hc.m.Label({
                //         text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("hprisk.explorationTableAuthorLastUpdatedColumn.txt")
                //     }),
                //     template: new sas.hc.m.Text({
                //         text: {
                //             path:"modifiedDate",
                //             formatter: sas.analytics.riskhp.utils.HprUtil.formatDateTime
                //         }
                //     }),
                //     sortProperty: "modifiedDate",
                //     width: "20%"
                // })
            ]
        });

        /* TODO: dacagl - Once this is put back in place, it will need some work for deletion of Explorations
        // NOTE: Currently there is no way to individually select a tile (no checkbox)
        this.explorationMasonry = new sas.hc.ui.layout.Masonry({
            id: semanticId + '-explorationMasonry',
            visible: true,
            layoutMode: sas.hc.ui.layout.MasonryLayoutMode.Magnitude3,
            page: true,
            pageSize: 10,
            options: {
                useWrapper: true,
                transitionDuration: 0
            }
        }).addStyleClass("sasHPRiskWelcome-explorationMasonry");

        // ExplorationMasonryTile fires an 'openItem' event when the tile is clicked, capture the event and handle it
        this.explorationMasonry.attachEvent("openItem", $.proxy(ctrl._onExplorationMasonryOpenItem, ctrl));

        this.masonryContainer = new sas.hc.m.ScrollContainer({
            horizontal: false,
            vertical: true,
            visible: false, // Made visible via SegmentButton.select() method
            height: "100%",
            content: [this.explorationMasonry]
        });*/

        var centerLayout = new sas.hc.ui.commons.layout.BorderLayout({
            width: "100%",
            height: "100%",
            center: new sas.hc.ui.commons.layout.BorderLayoutArea({
                overflowX: "hidden",
                overflowY: "hidden",
                content: [explorationTable/*, this.masonryContainer*/]
            }).addStyleClass("sasHPRiskWelcome-centerBorderLayout")
        });

        var contentLayout = new sas.hc.ui.commons.layout.BorderLayout({
            width: "100%",
            height: "100%",
            top: new sas.hc.ui.commons.layout.BorderLayoutArea({
                size: "2.5rem",
                overflowX: "hidden",
                overflowY: "hidden",
                content: [topBarLayout]
            }),
            center: new sas.hc.ui.commons.layout.BorderLayoutArea({
                overflowX: "hidden",
                overflowY: "hidden",
                content: [centerLayout]
            })
        });

        return contentLayout;
    }

    function _createOverflowMenuItems(ctrl) {
        var overflowMenuItems = [];

        openExplorationMenuItem = new sas.hc.m.MenuItem({
            id: semanticId + '-open-menuitem',
            text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("hprisk.openButton.tip"),
            tooltip: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("hprisk.openButton.tip"),
            press: $.proxy(ctrl._onOpenExploration, ctrl),
            enabled: false
        });
        overflowMenuItems.push(openExplorationMenuItem);

        replaceDataSourceMenuItem = new sas.hc.m.MenuItem({
            id: semanticId + '-replaceDataSource-menuitem',
            text: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("DataPane.overflow.menu.replace.data.source.txt"),
            tooltip: sas.analytics.riskhp.utils.HprUtil.getBundle().getText("DataPane.overflow.menu.replace.data.source.txt"),
            press: $.proxy(ctrl._onReplaceExplorationDataSource, ctrl),
            enabled: false
        });
        overflowMenuItems.push(replaceDataSourceMenuItem);

        return overflowMenuItems;
    }

    return {
        getControllerName: function() {
            return 'sas.analytics.riskhp.views.HprWelcome';
        },

        createContent: function(ctrl) {
            this.setHeight('100%');

            var viewLayout = _createHprWelcomeContent(ctrl);

            return viewLayout;
        },

        /**
         * Implementation of the destroy function. This function must clear the view to prevent the router
         * from caching the destroyed view after the user navigates away. Without clearing the view, the
         * router will attempt to load the cached destroyed view and an error will occur.
         */
        destroy: function () {
            // clear control references for GC purposes
            selectType = null;
            newButton = null;
            copyButton = null;
            deleteButton = null;
            refreshButton = null;
            openExplorationMenuItem = null;
            replaceDataSourceMenuItem = null;
            explorationTable = null;

            // update the router reference to this view
            sas.analytics.riskhp.utils.ViewManager.getRouter().getViews().setView("sas.analytics.riskhp.views.HprWelcome", null);

            // call super
            sap.ui.core.mvc.JSView.prototype.destroy.apply(this, arguments);
        },


        // content accessor functions

        updateForSelectionChange: function(selectedIndices) {
            copyButton.setEnabled(!!(selectedIndices && selectedIndices.length === 1));
            deleteButton.setEnabled(!!(selectedIndices && selectedIndices.length > 0));
            openExplorationMenuItem.setEnabled(!!(selectedIndices && selectedIndices.length === 1));
            replaceDataSourceMenuItem.setEnabled(!!(selectedIndices && selectedIndices.length === 1));
        },

        getSelectType: function() {
            return selectType;
        },

        getNewButton: function() {
            return newButton;
        },

        getExplorationTable: function() {
            return explorationTable;
        },

        getSelectedRowContexts: function() {
            var selectedIndices = explorationTable.getSelectedIndices();
            var selectedRowContexts = [];

            var selectedRowContext = null;
            _.each(selectedIndices, function(selectedIndex) {
                selectedRowContext = explorationTable.getContextByIndex(selectedIndex);
                if (selectedRowContext) {
                    selectedRowContexts.push(selectedRowContext);
                }
            });

            return selectedRowContexts;
        }

    }; // return
}()));