// (c) 2017, SAS Institute Inc.

jQuery.sap.require("sas.analytics.riskhp.models.HPRiskModelManager");
jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");

sap.ui.controller('sas.analytics.riskhp.views.OptionsPane', {
    HprConstants: sas.analytics.riskhp.utils.HprConstants,

    onInit: function() {
        var ctrl = this;
        var view = ctrl.getView();

        // Subscribe to an event that is published when the Options panes should be updated
        sas.eventBus.subscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_OPTIONS_PANE_CHANGED, this.onChangeOptionsPane, this);
    },

    onExit: function() {
        // Un-subscribe from events
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_OPTIONS_PANE_CHANGED, this.onChangeOptionsPane, this);
    },

    setModelCache: function(modelCache) {
        this.modelCache = modelCache;
    },

    getModelCache: function() {
        if (!this.modelCache) {
            this.setModelCache({});
        }
        return this.modelCache;
    },

    // Update the Option panes depending on what type of Object is selected in the Exploration view
    onChangeOptionsPane: function(channelId, eventId, data) {
        var ctrl = this;
        var view = ctrl.getView();
        var optionsTypeUpper = data.optionsType.toUpperCase();

        // tell the view to show the correct options panel
        view.showOptionsPanel(optionsTypeUpper);

        switch (optionsTypeUpper) {
            case this.HprConstants.HPR_OPTIONS_PANE_PAGE_TYPE:
                // Check to see if the Page in the model is the same one the user
                // clicked on, if so, just return and don't set model again
                var pageOptionsModel = view.getOptionsPanelModel(this.HprConstants.HPR_OPTIONS_PANE_PAGE_TYPE);
                var pageModelId = pageOptionsModel.getProperty('/options/id');
                if (pageModelId === data.pageId) {
                    return;
                }
                else if (!this.getModelCache()[data.pageId]) {
                    // add the model to the cache
                    this.getModelCache()[data.pageId] = sas.analytics.riskhp.models.HPRiskModelManager.getPageOptionsModel(data.pageId);
                }

                // Set the model on the Page options container
                view.setOptionsPanelModel(
                    this.getModelCache()[data.pageId],
                    this.HprConstants.HPR_OPTIONS_PANE_PAGE_TYPE
                );

                // Keep the WindowShade openedSectionsId property in-sync with open sections in our model
                view.getOptionsPanel(this.HprConstants.HPR_OPTIONS_PANE_PAGE_TYPE).updateWindowShadeOpenedSectionsIdProperty();

                break;
            case this.HprConstants.HPR_OPTIONS_PANE_CROSSTAB_TYPE:
                // In some cases for the Crosstab, we want to update the Options model and Panel even though the current
                // Crosstab is the one displayed and stored in the Crosstab Options model (i.e. when HprCrosstab._resetToDefaults is called)
                var updateOptionsModel = data.updateOptionsModel;

                // Check to see if the Crosstab in the model is the same one the user
                // clicked on, if so, just return and don't set model again
                var crosstabOptionsModel = view.getOptionsPanelModel(this.HprConstants.HPR_OPTIONS_PANE_CROSSTAB_TYPE);
                var crosstabModelId = crosstabOptionsModel.getProperty('/options/id');
                if (!updateOptionsModel && crosstabModelId === data.crosstabId) {
                    return;
                }
                else if (!this.getModelCache()[data.crosstabId] || updateOptionsModel) {
                    // add the model to the cache
                    this.getModelCache()[data.crosstabId] = sas.analytics.riskhp.models.HPRiskModelManager.getCrosstabOptionsModel(data.crosstabId);
                }

                // Set the model on the Crosstab options container
                view.setOptionsPanelModel(
                    this.getModelCache()[data.crosstabId],
                    this.HprConstants.HPR_OPTIONS_PANE_CROSSTAB_TYPE
                );

                // Keep the WindowShade openedSectionsId property in-sync with open sections in our model
                view.getOptionsPanel(this.HprConstants.HPR_OPTIONS_PANE_CROSSTAB_TYPE).updateWindowShadeOpenedSectionsIdProperty();

                break;
            case this.HprConstants.HPR_OPTIONS_PANE_PLOT_TYPE:
                // Check to see if the Plot in the model is the same one the user
                // clicked on, if so, just return and don't set model again
                var plotOptionsModel = view.getOptionsPanelModel(this.HprConstants.HPR_OPTIONS_PANE_PLOT_TYPE);
                var plotModelId = plotOptionsModel.getProperty('/options/id');
                // Commented out as I don't think this is needed, but we need to monitor it.  Waiting till
                // updating of filters is fixed - MCA 1/31/2018
                if (plotModelId === data.plotId && !data.forceUpdate) {
                    return;
                }
                else if (!this.getModelCache()[data.plotId] || data.forceUpdate) {
                    // add the model to the cache
                    this.getModelCache()[data.plotId] = sas.analytics.riskhp.models.HPRiskModelManager.getPlotOptionsModel(data.plotId);
                }

                // Set the model on the Plot options container
                view.setOptionsPanelModel(
                    this.getModelCache()[data.plotId],
                    this.HprConstants.HPR_OPTIONS_PANE_PLOT_TYPE
                );

                // Keep the WindowShade openedSectionsId property in-sync with open sections in our model
                view.getOptionsPanel(this.HprConstants.HPR_OPTIONS_PANE_PLOT_TYPE).updateWindowShadeOpenedSectionsIdProperty();

                break;
            case this.HprConstants.HPR_OPTIONS_PANE_PORTFOLIO_DETAILS_TYPE:
                // Check to see if the PortfolioDetails in the model is the same one the user
                // clicked on, if so, just return and don't set model again
                var portfolioDetailsOptionsModel = view.getOptionsPanelModel(this.HprConstants.HPR_OPTIONS_PANE_PORTFOLIO_DETAILS_TYPE);
                var portfolioDetailsModelId = portfolioDetailsOptionsModel.getProperty('/options/id');
                if (portfolioDetailsModelId === data.portfolioDetailsId) {
                    return;
                }
                else if (!this.getModelCache()[data.portfolioDetailsId]) {
                    // add the model to the cache
                    this.getModelCache()[data.portfolioDetailsId] = sas.analytics.riskhp.models.HPRiskModelManager.getPortfolioDetailsOptionsModel(data.portfolioDetailsId);
                }

                // Set the model on the Portfolio options container
                view.setOptionsPanelModel(
                    this.getModelCache()[data.portfolioDetailsId],
                    this.HprConstants.HPR_OPTIONS_PANE_PORTFOLIO_DETAILS_TYPE
                );

                // Keep the WindowShade openedSectionsId property in-sync with open sections in our model
                view.getOptionsPanel(this.HprConstants.HPR_OPTIONS_PANE_PORTFOLIO_DETAILS_TYPE).updateWindowShadeOpenedSectionsIdProperty();

                break;
            case this.HprConstants.HPR_OPTIONS_PANE_RISK_FACTOR_SENSITIVITIES_TYPE:
                // Check to see if the PortfolioDetails in the model is the same one the user
                // clicked on, if so, just return and don't set model again
                var optionsModel = view.getOptionsPanelModel(this.HprConstants.HPR_OPTIONS_PANE_RISK_FACTOR_SENSITIVITIES_TYPE);
                var modelId = optionsModel.getProperty('/options/id');
                if (modelId === data.rfsId) {
                    return;
                }
                else if (!this.getModelCache()[data.rfsId]) {
                    // add the model to the cache
                    this.getModelCache()[data.rfsId] = sas.analytics.riskhp.models.HPRiskModelManager.getRiskFactorSensitivitiesOptionsModel(data.rfsId);
                }

                // Set the model on the Risk Factor Sensitivities options container
                view.setOptionsPanelModel(
                    this.getModelCache()[data.rfsId],
                    this.HprConstants.HPR_OPTIONS_PANE_RISK_FACTOR_SENSITIVITIES_TYPE
                );

                // Keep the WindowShade openedSectionsId property in-sync with open sections in our model
                view.getOptionsPanel(this.HprConstants.HPR_OPTIONS_PANE_RISK_FACTOR_SENSITIVITIES_TYPE).updateWindowShadeOpenedSectionsIdProperty();

                break;
            case this.HprConstants.HPR_OPTIONS_PANE_TABLE_VIEWER_TYPE:
                // Check to see if the TableViewer in the model is the same one the user
                // clicked on, if so, just return and don't set model again
                var tableViewerOptionsModel = view.getOptionsPanelModel(this.HprConstants.HPR_OPTIONS_PANE_TABLE_VIEWER_TYPE);
                var tableViewerModelId = tableViewerOptionsModel.getProperty('/options/id');
                if (tableViewerModelId === data.tableViewerId) {
                    return;
                }
                else if (!this.getModelCache()[data.tableViewerId]) {
                    // add the model to the cache
                    this.getModelCache()[data.tableViewerId] = sas.analytics.riskhp.models.HPRiskModelManager.getTableViewerOptionsModel(data.tableViewerId);
                }

                // Set the model on the TableViewer options container
                view.setOptionsPanelModel(
                    this.getModelCache()[data.tableViewerId],
                    this.HprConstants.HPR_OPTIONS_PANE_TABLE_VIEWER_TYPE
                );

                // Keep the WindowShade openedSectionsId property in-sync with open sections in our model
                view.getOptionsPanel(this.HprConstants.HPR_OPTIONS_PANE_TABLE_VIEWER_TYPE).updateWindowShadeOpenedSectionsIdProperty();

                break;
        }
    }
});