// (c) 2017, SAS Institute Inc.

jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.risk.common.panelsview.DivWrapper");

sap.ui.jsview('sas.analytics.riskhp.views.OptionsPane', (function() {
    // pseudo-imports
    var HprConstants = sas.analytics.riskhp.utils.HprConstants;
    var HprUtil = sas.analytics.riskhp.utils.HprUtil;

    // ui controls
    var optionsPaneLayout = null;
    var pageOptionsPanel = null;
    var crosstabOptionsPanel = null;
    var plotOptionsPanel = null;
    var portfolioDetailsOptionsPanel = null;
    var rfsOptionsPanel = null;
    var tableViewerOptionsPanel = null;
    var zeroStatePage = null;

    return {
        getControllerName: function() {
            return 'sas.analytics.riskhp.views.OptionsPane';
        },

        createContent: function(ctrl) {
            // Create Page Options fragment
            pageOptionsPanel = sap.ui.jsfragment("sas.analytics.riskhp.fragments.options.PageOptions", ctrl);

            // Create Crosstab Options fragment
            crosstabOptionsPanel = sap.ui.jsfragment("sas.analytics.riskhp.fragments.options.CrosstabOptions", ctrl);

            // Create Plot Options fragment
            plotOptionsPanel = sap.ui.jsfragment("sas.analytics.riskhp.fragments.options.PlotOptions", ctrl);

            // Create Portfolio Details fragment
            portfolioDetailsOptionsPanel = sap.ui.jsfragment("sas.analytics.riskhp.fragments.options.PortfolioDetailsOptions", ctrl);

            // Create RiskFactorSensitivities Table fragment
            rfsOptionsPanel = sap.ui.jsfragment("sas.analytics.riskhp.fragments.options.RiskFactorSensitivitiesOptions", ctrl);

            // Create an Options fragment which covers the generic TableViewer component
            tableViewerOptionsPanel = sap.ui.jsfragment("sas.analytics.riskhp.fragments.options.TableViewerOptions", ctrl);

            zeroStatePage = new sas.hc.m.MessagePage({
                text: "", // Must leave empty, causes big font text
                description: HprUtil.getBundle().getText("OptionsPane.select.control.txt"),
                icon: sas.icons.HC.ZEROSTATEALERT,
                showHeader: false
            });

            optionsPaneLayout = new sas.risk.common.panelsview.DivWrapper({
                width: "100%",
                height: "100%"
            });

            return optionsPaneLayout;
        },

        destroy: function() {
            // options fragments may be un-parented, so manually clean them up
            pageOptionsPanel.destroy();
            crosstabOptionsPanel.destroy();
            plotOptionsPanel.destroy();
            portfolioDetailsOptionsPanel.destroy();
            rfsOptionsPanel.destroy();
            tableViewerOptionsPanel.destroy();

            sap.ui.core.mvc.JSView.prototype.destroy.apply(this, arguments);
        },

        setOptionsPanelModel: function(model, optionsType) {
            switch (optionsType) {
                case HprConstants.HPR_OPTIONS_PANE_PAGE_TYPE:
                    pageOptionsPanel.setModel(model);
                    break;
                case HprConstants.HPR_OPTIONS_PANE_CROSSTAB_TYPE:
                    crosstabOptionsPanel.setModel(model);
                    break;
                case HprConstants.HPR_OPTIONS_PANE_PLOT_TYPE:
                    plotOptionsPanel.setModel(model);
                    break;
                case HprConstants.HPR_OPTIONS_PANE_PORTFOLIO_DETAILS_TYPE:
                    portfolioDetailsOptionsPanel.setModel(model);
                    break;
                case HprConstants.HPR_OPTIONS_PANE_RISK_FACTOR_SENSITIVITIES_TYPE:
                    rfsOptionsPanel.setModel(model);
                    break;
                case HprConstants.HPR_OPTIONS_PANE_TABLE_VIEWER_TYPE:
                    tableViewerOptionsPanel.setModel(model);
                    break;
            }
        },

        getOptionsPanelModel: function(optionsType) {
            switch (optionsType) {
                case HprConstants.HPR_OPTIONS_PANE_PAGE_TYPE:
                    return pageOptionsPanel.getModel();
                case HprConstants.HPR_OPTIONS_PANE_CROSSTAB_TYPE:
                    return crosstabOptionsPanel.getModel();
                case HprConstants.HPR_OPTIONS_PANE_PLOT_TYPE:
                    return plotOptionsPanel.getModel();
                case HprConstants.HPR_OPTIONS_PANE_PORTFOLIO_DETAILS_TYPE:
                    return portfolioDetailsOptionsPanel.getModel();
                case HprConstants.HPR_OPTIONS_PANE_RISK_FACTOR_SENSITIVITIES_TYPE:
                    return rfsOptionsPanel.getModel();
                case HprConstants.HPR_OPTIONS_PANE_TABLE_VIEWER_TYPE:
                    return tableViewerOptionsPanel.getModel();
                case HprConstants.HPR_OPTIONS_PANE_ZERO_STATE_TYPE:
                default:
                    return null;
            }
        },

        showOptionsPanel: function(optionsType) {
            switch (optionsType) {
                case HprConstants.HPR_OPTIONS_PANE_PAGE_TYPE:
                    optionsPaneLayout.setContent(pageOptionsPanel);
                    break;
                case HprConstants.HPR_OPTIONS_PANE_CROSSTAB_TYPE:
                    optionsPaneLayout.setContent(crosstabOptionsPanel);
                    break;
                case HprConstants.HPR_OPTIONS_PANE_PLOT_TYPE:
                    optionsPaneLayout.setContent(plotOptionsPanel);
                    break;
                case HprConstants.HPR_OPTIONS_PANE_PORTFOLIO_DETAILS_TYPE:
                    optionsPaneLayout.setContent(portfolioDetailsOptionsPanel);
                    break;
                case HprConstants.HPR_OPTIONS_PANE_RISK_FACTOR_SENSITIVITIES_TYPE:
                    optionsPaneLayout.setContent(rfsOptionsPanel);
                    break;
                case HprConstants.HPR_OPTIONS_PANE_TABLE_VIEWER_TYPE:
                    optionsPaneLayout.setContent(tableViewerOptionsPanel);
                    break;
                case HprConstants.HPR_OPTIONS_PANE_ZERO_STATE_TYPE:
                default:
                    optionsPaneLayout.setContent(zeroStatePage);
                    break;
            }
        },

        getOptionsPanel: function(optionsType) {
            switch (optionsType) {
                case HprConstants.HPR_OPTIONS_PANE_PAGE_TYPE:
                    return pageOptionsPanel;
                case HprConstants.HPR_OPTIONS_PANE_CROSSTAB_TYPE:
                    return crosstabOptionsPanel;
                case HprConstants.HPR_OPTIONS_PANE_PLOT_TYPE:
                    return plotOptionsPanel;
                case HprConstants.HPR_OPTIONS_PANE_PORTFOLIO_DETAILS_TYPE:
                    return portfolioDetailsOptionsPanel;
                case HprConstants.HPR_OPTIONS_PANE_RISK_FACTOR_SENSITIVITIES_TYPE:
                    return rfsOptionsPanel;
                case HprConstants.HPR_OPTIONS_PANE_TABLE_VIEWER_TYPE:
                    return tableViewerOptionsPanel;
                case HprConstants.HPR_OPTIONS_PANE_ZERO_STATE_TYPE:
                default:
                    return null;
            }
        },

        updateWindowShadeOpenedSectionsIdProperty: function(optionsType) {
            switch (optionsType) {
                case HprConstants.HPR_OPTIONS_PANE_PAGE_TYPE:
                    pageOptionsPanel.updateWindowShadeOpenedSectionsIdProperty();
                    break;
                case HprConstants.HPR_OPTIONS_PANE_CROSSTAB_TYPE:
                    crosstabOptionsPanel.updateWindowShadeOpenedSectionsIdProperty();
                    break;
                case HprConstants.HPR_OPTIONS_PANE_PLOT_TYPE:
                    plotOptionsPanel.updateWindowShadeOpenedSectionsIdProperty();
                    break;
                case HprConstants.HPR_OPTIONS_PANE_PORTFOLIO_DETAILS_TYPE:
                    portfolioDetailsOptionsPanel.updateWindowShadeOpenedSectionsIdProperty();
                    break;
                case HprConstants.HPR_OPTIONS_PANE_RISK_FACTOR_SENSITIVITIES_TYPE:
                    rfsOptionsPanel.updateWindowShadeOpenedSectionsIdProperty();
                    break;
                case HprConstants.HPR_OPTIONS_PANE_TABLE_VIEWER_TYPE:
                    tableViewerOptionsPanel.updateWindowShadeOpenedSectionsIdProperty();
                    break;
            }
        }
    };
}()));