// (c) 2017, SAS Institute Inc.

jQuery.sap.declare("sas.analytics.riskhp.views.computedvariableeditor.ComputedVarCrosstabContextMenuProvider");

jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.crosstab.HprCrosstabContextMenuProvider");
jQuery.sap.require("sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil");

// noinspection JSClosureCompilerSyntax
/**
 * Crosstab context menu provider to be used with the computed variable editor
 * @name sas.analytics.riskhp.views.computedvariableeditor.ComputedVarCrosstabContextMenuProvider
 * @extends sas.analytics.riskhp.common.riskcommons.crosstab.HprCrosstabContextMenuProvider
 */
sas.analytics.riskhp.common.riskcommons.crosstab.HprCrosstabContextMenuProvider.extend("sas.analytics.riskhp.views.computedvariableeditor.ComputedVarCrosstabContextMenuProvider", {

    computedVariableEditorController: null,
    CompVarUtil: sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil,

    editor : undefined,

    constructor: function (computedVarEditorController) {
        this.computedVariableEditorController = computedVarEditorController;
    },

    _getResourceBundle: function() {
        return sas.analytics.riskhp.utils.HprUtil.getBundle();
    },

    _buildMeasureHeaderContextMenu: function(oMenu, crosstab, info) {
        var self = this;
        self._buildContextMenuFromPickInfo(oMenu, crosstab, info);
    },

    _buildColumnHeaderMenu: function(oMenu, crosstab, info) {
        // var context = crosstab.context;
        // var node = context.findColumnTreeNode(info.keys);
    },

    _buildContextMenuFromPickInfo:function(oMenu, crosstab, info) {
        var self = this;

        if (self.editor && self.editor.isValidCompVarDataItemColumn(info)) {
            var items = self.computedVariableEditorController.getItems();

            var columnStr = self.CompVarUtil.getColumnStringFromPickInfo(info);
            var dataItems = self.CompVarUtil.getTuplesFromPickInfo(info);

            var computedVariableItemsSubMenu = new sas.hc.ui.unified.Menu();
            _.each(items, function (item) {
                var computedVariableItemMenuItem = self._createMenuItem(
                    item.name,
                    item.name,
                    function () {
                        item.column = columnStr;
                        item.dataItems = dataItems;
                        self.computedVariableEditorController.getView().getModel().refresh();

                        self.editor.setNeedsValidation();
                        self.editor.validateItems();
                        // self.computedVariableEditorController.setNeedsValidation.call(self.computedVariableEditorController);
                    });

                computedVariableItemsSubMenu.addItem(computedVariableItemMenuItem);
            });

            var newItemMenuItem =self._createMenuItem(
                self._getResourceBundle().getText("HPRisk.ComputedVariables.New.Item.txt"),
                self._getResourceBundle().getText("HPRisk.ComputedVariables.New.Item.txt"),
                function() {
                    self.computedVariableEditorController.createNewItem.call(self.computedVariableEditorController, columnStr, dataItems);
                }
            );
            computedVariableItemsSubMenu.addItem(newItemMenuItem);

            var computedVariableItemsMenu = self._createMenuItem(
                self._getResourceBundle().getText("HPRisk.ComputedVariables.Crosstab.Context.Menu.Column.Root.txt"),
                self._getResourceBundle().getText("HPRisk.ComputedVariables.Crosstab.Context.Menu.Column.Root.txt"),
                function () {
                }
            );
            computedVariableItemsMenu.setSubmenu(computedVariableItemsSubMenu);

            oMenu.addItem(computedVariableItemsMenu);

            if (info.dataResultPathObject) {
                var value = info.dataResultPathObject.node.results[info.columnIndex];
                oMenu.addItem(self._createMenuItem(
                    self._getResourceBundle().getText("HPRisk.Crosstab.Context.Menu.CopyToClipboard.txt"),
                    "copyValue",
                    function () {
                        self.CROSSTAB_UTIL.copyTextToClipboard(value);
                    }
                ));
            }
        }
    },

    /**
     * Adds the appropriate MenuItem and sub-menus for the currently selected cell as identified by the entry and
     * info objects.
     * @param oMenu - the menu to add the menu items to
     * @param crosstab - The crosstab for this menu
     * @param info - the info object extracted from the pick results
     * @param entry
     */
    _buildDataCellContextMenu: function (oMenu, crosstab, info, entry) {
        var self = this;
        if (self.editor.isValidCompVarDataItemColumn(info)) {
            self._buildContextMenuFromPickInfo(oMenu, crosstab, info);
        }
    },

    setEditor: function(controller) {
        var self = this;
        self.editor = controller;
    }
});
