// (c) 2017, SAS Institute Inc.
jQuery.sap.require('sas.analytics.riskhp.common.riskcommons.CrosstabUtil');
jQuery.sap.require('sas.analytics.riskhp.utils.HprAPI');
jQuery.sap.require('sas.analytics.riskhp.views.computedvariableeditor.ComputedVarCrosstabContextMenuProvider');
jQuery.sap.require("sas.analytics.riskhp.models.DataSourceCacheManager");
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil");
jQuery.sap.require("sas.analytics.riskhp.views.datapane.DataPaneUtil");
jQuery.sap.require("sas.analytics.riskhp.models.ComputedVariablesManager");
jQuery.sap.require("sas.hc.m.MessageToast");
jQuery.sap.require("sas.analytics.riskhp.utils.ViewManager");
jQuery.sap.require("sas.analytics.riskhp.utils.ExplorationUtil");
jQuery.sap.require("sas.analytics.riskhp.models.DisplayRulesManager");
jQuery.sap.require("sas.hc.m.MessageDialog");

sap.ui.controller('sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableEditor', {
    CrosstabUtil : sas.analytics.riskhp.common.riskcommons.CrosstabUtil,
    HprAPI: sas.analytics.riskhp.utils.HprAPI,
    DataSourceCacheManager: sas.analytics.riskhp.models.DataSourceCacheManager,
    HprUtil: sas.analytics.riskhp.utils.HprUtil,
    CompVarMgr: sas.analytics.riskhp.models.ComputedVariablesManager,
    CompVarUtil: sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil,
    DataPaneUtil: sas.analytics.riskhp.views.datapane.DataPaneUtil,
    ViewManager: sas.analytics.riskhp.utils.ViewManager,
    ExplorationUtil: sas.analytics.riskhp.utils.ExplorationUtil,
    ComputedVariablesManager: sas.analytics.riskhp.models.ComputedVariablesManager,
    DisplayRulesManager: sas.analytics.riskhp.models.DisplayRulesManager,

    MODEL_EDITOR_STATE: "editorState",
    FORMAT_NUMERIC: "numeric",
    FORMAT_PERCENT: "percent",
    FORMAT_STRING: "string",
    NAME_PLACEHOLDER: "_placeholder_",
    
    onInit: function () {
        var self = this;
        var view = self.getView();

        // expressions model
        var expressionsModel = new sap.ui.model.json.JSONModel();
        // Agreed not to translate ITEM as it would be hard to change the text of the expression if locale changed
        expressionsModel.setData({
            expressions: [
                    {name: "ITEM1"},
                    {name: "ITEM1 + ITEM2"},
                    {name: "ITEM1 - ITEM2"},
                    {name: "ITEM1 * ITEM2"},
                    {name: "ITEM1 / ITEM2"},
                    {name: "(ITEM1 - ITEM2) / ITEM2"},
                    {name: "ABS(ITEM1 - ITEM2)"}
            ]});
        view.setModel(expressionsModel, 'expressions');

        var editorStateModel = new sap.ui.model.json.JSONModel();
        editorStateModel.setData({
            isEdit: false,
            isValidated: false,
            isApplied: false,
            formattedExpression: "",
            hasInvalidItems: false
        });
        view.setModel(editorStateModel, self.MODEL_EDITOR_STATE);

        // view model
        var model = new sap.ui.model.json.JSONModel();
        view.setModel(model);

        // attach change listener for when the items aggregation changes (items added/removed)
        view.itemsList.getBinding("items").attachChange($.proxy(self.setNeedsValidation, self));

        // attach change listener to when the contents of an item in the items list change
        var itemBinding = new sap.ui.model.Binding(model, "/items", model.getContext("/items"));
        itemBinding.attachChange($.proxy(self.setNeedsValidation, self));

        // i18N model
        var i18N = new sap.ui.model.resource.ResourceModel({ bundleName: 'sas.analytics.riskhp.nls.messagebundle' });
        this.rb = i18N.getResourceBundle();
        view.setModel(i18N, 'i18N');
    },

    onBeforeRendering: function() {
        var self = this;
        var view = self.getView();

        // clear out prior selection
        view.expressionSelect.setSelectedItem(null, false);
    },

    onAfterRendering: function () {
        var self = this;
        var view = self.getView();

        // fix splitter sizing/rendering issue when opening editor again
        view.itemsSplitter.invalidate();
    },

    cancelEditing: function() {
        var self = this;

        // set original menu provider back on crosstab
        self.crosstab.setMenuProvider(self.origMenuProvider);

        // restore the original drag handler on crosstab.uiVIew
        self.crosstab.uiView.setDragHandler(self.origDragHandler );

        sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_DATA_ITEM_LABEL_UPDATED, this.onDataItemLabelUpdate, this);


        // Publish an event to the exploration view to close the computed variable editor for the crosstab
        sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_CLOSE_COMPUTED_VARIABLE_EDITOR, {
            crosstabId: this.crosstabId
        });

    },

    onDataItemLabelUpdate: function(channelId, eventId, data) {
        var self = this;
        var view = self.getView();
        var viewModel = view.getModel();
        var dataItem = data.dataItem;

        if (dataItem.type === sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR &&
            dataItem.subType === "expression" &&
            view.nameInput.getEnabled() === false && // this comp var has been applied
            dataItem.name === viewModel.getData().name) {
            viewModel.setProperty("/label", data.newLabel);
        }

    },

    _determineNextDefaultExpressionName: function() {
        var expressions = sas.analytics.riskhp.models.ComputedVariablesManager.getComputedVariables();
        // cannot externalize the prefix since only A-Za-z0-9 characters are allowed
        var prefix = "EXPRESSION_";

        var re = new RegExp("^" + prefix + "([0-9]+)$");
        var maxItem = 0;
        _.each(expressions, function(expression){
            if (re.test(expression.name)) {
                var value = parseInt(expression.name.substring(prefix.length));
                if (value > maxItem) {
                    maxItem = value;
                }
            }
        });
        return prefix + (maxItem + 1);
    },

    /**
     *
     * @param crosstabId
     */
    configure: function(crosstabId){
        var self = this;
        var view = self.getView();

        var context = sas.analytics.riskhp.models.HPRiskModelManager.getCrosstabContext(crosstabId);
        self.crosstab = context.crosstab;
        self.crosstabId = self.crosstab.sId;

        self.origMenuProvider = self.crosstab.getMenuProvider();  // save off the crosstab's menu provider
        var computedVarMenuProvider = new sas.analytics.riskhp.views.computedvariableeditor.ComputedVarCrosstabContextMenuProvider(self);
        computedVarMenuProvider.setEditor(self);
        self.crosstab.setMenuProvider(computedVarMenuProvider);

        view.topSplitterPanel.removeAllContent(sap.ui.commons.layout.BorderLayoutAreaTypes.top);
        view.topSplitterPanel.removeAllContent(sap.ui.commons.layout.BorderLayoutAreaTypes.center);

        var selectedRiskPanelsView = self.HprUtil.getSelectedRiskPanelsView();
        var riskPanelsViewDelegate = selectedRiskPanelsView.getDelegate();
        var crosstabTitle = riskPanelsViewDelegate.getTitleForContent(crosstabId);

        var topLabel = new sas.hc.m.Label({
            text:  crosstabTitle
        }).addStyleClass("computedVariableEditorCrosstabLabel");
        view.topSplitterPanel.addContent(sap.ui.commons.layout.BorderLayoutAreaTypes.top, topLabel);
        view.topSplitterPanel.setAreaData(sap.ui.commons.layout.BorderLayoutAreaTypes.top, {
            size: "1.5rem"
        });

        view.topSplitterPanel.addContent(sap.ui.commons.layout.BorderLayoutAreaTypes.center, this.crosstab);

        setTimeout(function() {
            // crosstab calculates its height when there is a breadcrumb (drillpath) so redraw after added to the layout to get correct crosstab height
            self.crosstab.redraw();
        },1);

        self.origDragHandler = self.crosstab.uiView.getDragHandler();
        self.crosstab.uiView.setDragHandler(self);

        sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_DATA_ITEM_LABEL_UPDATED, self.onDataItemLabelUpdate, self);

        self.isNameValid = true;
        view.nameInput.setValueState(sap.ui.core.ValueState.None);
    },

    configureForCreating: function(crosstabId, pickinfo) {
        var self = this;
        var view = self.getView();

        self.configure(crosstabId);

        var editorStateModel = view.getModel(self.MODEL_EDITOR_STATE);
        editorStateModel.setProperty("/isEdit", false);
        editorStateModel.setProperty("/isValidated", false);
        editorStateModel.setProperty("/isApplied", false);
        editorStateModel.setProperty("/formattedExpression", "");
        editorStateModel.setProperty("/hasInvalidItems", false);


        var nextExpressionName = self._determineNextDefaultExpressionName();
        var cannedExpressions = view.getModel('expressions').getData().expressions;

        var item1 = {
            name: "ITEM1",
            column: ""
        };
        if (pickinfo) {
            item1.column = sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil.getColumnStringFromPickInfo(pickinfo);
            item1.dataItems = sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil.getTuplesFromPickInfo(pickinfo);
        }

        // view model
        view.setModel(new sap.ui.model.json.JSONModel({
            name: nextExpressionName,
            label: nextExpressionName,
            dataType: self.FORMAT_NUMERIC,
            dataLength: "32",  // only used for "string" dataType
            items: [
                item1
            ],
            expression: nextExpressionName + " = " + cannedExpressions[0].name + ";"
        }));

        self.syncNameAndLabel = true;
    },


    validateItems: function(warnIfInvalidItems) {
        var self = this;
        var view = self.getView();
        var model = view.getModel();
        var cubeData = self.crosstab.context.cubeDataModel;

        var items = model.getProperty("/items");
        var invalidItemNames = [];
        _.each(items, function(item) {
            if (!self.CompVarUtil._isValidColumn(cubeData, item.dataItems)){
                invalidItemNames.push(item.name);
            }
        });

        var editorStateModel = view.getModel(self.MODEL_EDITOR_STATE);
        editorStateModel.setProperty("/hasInvalidItems", invalidItemNames.length !== 0);


        // display message that items are invalid
        if (warnIfInvalidItems && invalidItemNames.length > 0) {
            var dialogText = null;
            // var detailText = null;

            if (invalidItemNames.length === 1) {
                dialogText = self.rb.getText("HPRisk.ComputedVariables.Editor.Invalid.Item.Single.fmt", invalidItemNames[0]);
            } else {
                dialogText = self.rb.getText("HPRisk.ComputedVariables.Editor.Invalid.Items.Multiple.txt");
                // detailText = self.rb.getText("HPRisk.ComputedVariables.Editor.Invalid.Items.Details.txt") + "\n\n";
                // _.each(invalidItemNames, function (name) {
                //     detailText += "\t" + name + "\n";
                // });
            }

            var compVarWarningDialogId = "compVarEditorWarning";

            if (sas.hc.ui.core.settings.GlobalSettingsHelper.getSuppressedMessages().indexOf(compVarWarningDialogId) === -1) {

                // warning dialogs do not show detail text
                var warning = sas.hc.m.MessageDialog.warning({
                    id: compVarWarningDialogId,
                    text: dialogText,
                    title: self.rb.getText("HPRisk.ComputedVariables.Editor.Invalid.Items.Window.title"),
                    // detailText: detailText ? detailText : null,
                    includeDontShowCheckBox: true
                });
                warning.attachAfterClose(function(e) {
                    warning.destroy();
                });
            }
        }
    },

    configureForEditing: function(crosstabId, compVarData) {
        var self = this;
        var view = self.getView();

        self.configure(crosstabId);

        var dataItem = self._getComputedVariableDataItem(compVarData.name);
        var editorStateModel = view.getModel(self.MODEL_EDITOR_STATE);
        // had to set isEdit before setting the model to keep name Input component from turning red as if in error condition
        editorStateModel.setProperty("/isEdit", true);

        // clone it and set it back in the model only when applied
        var compVar = sas.analytics.riskhp.utils.HprUtil.hprClone(compVarData);
        view.setModel(new sap.ui.model.json.JSONModel(compVar));
        self.syncNameAndLabel = compVar.name === compVar.label;

        editorStateModel.setProperty("/isValidated", true);
        editorStateModel.setProperty("/isApplied", true);
        editorStateModel.setProperty("/formattedExpression", dataItem.expression);

        self.validateItems(true);
    },

    _replaceNameInExpression:function(originalName, newName) {
        var self = this;
        var view = self.getView();

        var expression = view.getModel().getProperty("/expression");
        var myRexEx = new RegExp("\\b" + originalName + "\\b", "g");
        var newExpression = expression.replace(myRexEx, newName);
        console.log(expression + " >> " + newExpression);
        view.getModel().setProperty("/expression", newExpression);
    },

    configureForCopying: function(crosstabId, compVarData) {
        var self = this;
        var view = self.getView();

        var editorStateModel = view.getModel(self.MODEL_EDITOR_STATE);
        editorStateModel.setProperty("/isEdit", false);
        editorStateModel.setProperty("/isValidated", false);
        editorStateModel.setProperty("/isApplied", false);
        editorStateModel.setProperty("/formattedExpression", "");

        self.configure(crosstabId);

        // if editing or copying an existing computed variable, clone it and set it back in the model only when applied
        var compVar = sas.analytics.riskhp.utils.HprUtil.hprClone(compVarData);

        var originalName = compVar.name;  // save off original name for the regex to replace the name in the expression

        compVar.name = compVar.name + sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.ComputedVariables.Default.Copied.Expression.Name.Suffix.txt");
        if (compVar.label && compVar.label.length > 0) {
            compVar.label = compVar.label + sas.analytics.riskhp.utils.HprUtil.getBundle().getText("HPRisk.ComputedVariables.Default.Copied.Expression.Name.Suffix.txt");
        }

        // set the model before replacing name in expression
        view.setModel(new sap.ui.model.json.JSONModel(compVar));
        // the name is part of the expression, so replace the name tokens in the expression
        self._replaceNameInExpression(originalName, compVar.name);

        // view.getModel().setData(compVar);
        self.syncNameAndLabel = compVar.name === compVar.label;

        self.validateItems(true);
    },

    handleExternalDragStartEvent: function (event, element) {
        var self = this;
        var pickResults = self.crosstab.uiView.getPickResults(event.offsetX, event.offsetY);
        if (pickResults && pickResults.length > 0) {
            var info = sas.analytics.riskhp.common.riskcommons.CrosstabUtil.getPickInfo(pickResults, self.crosstab.context);
            // $(this).data('dragColumnInfo', info);
            element.data('dragColumnInfo', info);

        }
    },

    handleExternalDragHelperEvent: function (event) {
        var self = this;
        var view = self.getView();

        var pickResults = self.crosstab.uiView.getPickResults(event.offsetX, event.offsetY);
        var columnStr = "";

        if (pickResults && pickResults.length > 0) {
            var info = sas.analytics.riskhp.common.riskcommons.CrosstabUtil.getPickInfo(pickResults, self.crosstab.context);

            if (info) {
                _.each(info.column, function (col) {
                    columnStr += col.label + " ";
                });
            }
        }

        var helper = document.getElementById("DandDHelper_div");
        if (!helper) {
            helper = document.createElement('div');
            helper.id = 'DandDHelper_div';
            document.getElementById(view.mainContentArea.getId()).appendChild(helper);
        }
        helper = document.getElementById("DandDHelper_div");

        helper.style.position = "absolute";
        helper.style.zIndex = 100;
        helper.innerHTML = columnStr;
        return helper;
    },


    _getNextItemNumber: function() {
        var self = this;
        var model = self.getView().getModel();

        var items = model.getProperty("/items");
        var maxItem = 0;
        _.each(items, function(item){
            // if (item.name.match(/^ITEM([0-9]+)$/)) {
                var value = parseInt(item.name.substring(4));
                if (value > maxItem) {
                    maxItem = value;
                }
            // }
        });
        return maxItem +1;
    },

    addItemButtonPress: function() {
        var self = this;
        var model = self.getView().getModel();
        var items = model.getProperty("/items");
        var nextItemNumber = self._getNextItemNumber();
        items.push({name: "ITEM" + nextItemNumber,
                    column: ""});
        model.updateBindings();
    },

    createNewItem: function (columnStr, dataItems) {
        var self = this;
        var model = self.getView().getModel();
        var items = model.getProperty("/items");
        var nextItemNumber = self._getNextItemNumber();
        items.push({
            name: "ITEM" + nextItemNumber,
            column: columnStr,
            dataItems: dataItems
        });
        model.updateBindings();

        self.setNeedsValidation();
        self.validateItems();
    },

    handleLabelChange: function(event) {
        var self = this;
        var model = self.getView().getModel();
        var currentName = model.getProperty("/name");
        var currentLabel = model.getProperty("/label");

        self.syncNameAndLabel = currentName === currentLabel
    },

    handleNameChange: function(event) {
        var self = this;
        var model = self.getView().getModel();
        var currentName = model.getProperty("/name");
        var currentLabel = model.getProperty("/label");

        self.syncNameAndLabel = currentName === currentLabel
    },

    _validateName: function (newName) {
        var self = this;
        var view = self.getView();

        // name is required
        if (newName.length === 0) {
            view.nameInput.setValueState(sap.ui.core.ValueState.Error);
            view.nameInput.setValueStateText(self.rb.getText("HPRisk.ComputedVariables.Error.Name.Required.txt"));
            self.isNameValid = false;
            return;
        }

        // allow only alphanumerics and underscore
        var invalidCharsRegEx = new RegExp(/[^a-zA-Z0-9_]+/g);
        if (invalidCharsRegEx.test(newName)) {
            view.nameInput.setValueState(sap.ui.core.ValueState.Error);
            view.nameInput.setValueStateText(self.rb.getText("HPRisk.ComputedVariables.Error.Name.Must.Be.Alphanumeric.txt"));
            self.isNameValid = false;
            return;
        }

        // name cannot start with numeric digit
        var startsWithNumRegEx = new RegExp(/^[0-9].*$/g);
        if (startsWithNumRegEx.test(newName)) {
            view.nameInput.setValueState(sap.ui.core.ValueState.Error);
            view.nameInput.setValueStateText(self.rb.getText("HPRisk.ComputedVariables.Error.Name.Must.Start.With.alpha.txt"));
            self.isNameValid = false;
            return;
        }

        // do not allow name to start with ITEM followed by a number
        var startsWithITEMRegEx = new RegExp(/^ITEM[0-9]+/g);
        if (startsWithITEMRegEx.test(newName)) {
            view.nameInput.setValueState(sap.ui.core.ValueState.Error);
            view.nameInput.setValueStateText(self.rb.getText("HPRisk.ComputedVariables.Error.Name.Must.Not.Start.With.ITEM.txt"));
            self.isNameValid = false;
            return;
        }

        // name must be unique
        if (!self._isComputedVariableNameUnique(newName)) {
            view.nameInput.setValueState(sap.ui.core.ValueState.Error);
            view.nameInput.setValueStateText(self.rb.getText("HPRisk.ComputedVariables.Error.Name.Must.Be.Unique.txt"));
            self.isNameValid = false;
            return;
        }

        // ok, name is valid
        self.isNameValid = true;
        view.nameInput.setValueState(sap.ui.core.ValueState.None);
    },

    handleLabelLiveChange: function(event) {
        var self = this;
        self.setNeedsValidation();
    },

    handleNameLiveChange: function(event) {
        var self = this;
        var view = self.getView();

        self.setNeedsValidation();

        var model = view.getModel();
        var origName = model.getProperty("/name");
        var newName = event.getParameter("newValue").trim();


        var wasValid = self.isNameValid;

        self._validateName(newName);

        // console.log(origName + " --> " + cleanedNewName);
        model.setProperty("/name", newName);
        view.nameInput.setValue(newName);

        if (newName === "") {
            newName = self.NAME_PLACEHOLDER;
        }
        if (origName === "") {
            origName = self.NAME_PLACEHOLDER;
        }

        var replaceNewName = self.isNameValid ? newName : self.NAME_PLACEHOLDER;
        self._replaceNameInExpression(wasValid ? origName : self.NAME_PLACEHOLDER, replaceNewName);

        // if the user has not edited the label to be different than the name, keep the label in sync with the name change
        if (self.syncNameAndLabel) {
            model.setProperty("/label", replaceNewName === self.NAME_PLACEHOLDER ? "" : replaceNewName);
         }
    },

    handleExpressionSelection: function(event) {
        var self = this;
        var viewModel = self.getView().getModel();
        var name = viewModel.getProperty("/name");
        var selectedExpression = event.getParameter("selectedItem").getProperty("text");

        self.setNeedsValidation();

        viewModel.setProperty("/expression", (self.isNameValid ? name : self.NAME_PLACEHOLDER) + " = " + selectedExpression + ";");
    },

    setNeedsValidation: function() {
        var self = this;

        var stateModel = self.getView().getModel(self.MODEL_EDITOR_STATE);
        stateModel.setProperty("/isValidated", false);
        stateModel.setProperty("/isApplied", false);
        stateModel.setProperty("/formattedExpression","");

        self.getView().editorTabBar.setSelectedKey("editor");
    },

    handleValidatePressed: function() {
        var self = this;
        var view = self.getView();
        var stateModel = view.getModel(self.MODEL_EDITOR_STATE);

        var compVarData = self.getView().getModel().getData();
        // var dataSource = self.crosstab.context.cubeDataModel.dataSource;
        var request = self.CompVarUtil.buildComputedVarRequest(compVarData, self.crosstab.context.cubeDataModel);

        // send request to server
        var deferred = sas.analytics.riskhp.utils.HprAPI.validateComputedVariable(self.crosstab.context.cubeDataModel, [request]);
        deferred.done(function (computedVar) {
            if (computedVar.items[0].error != 0) {
                sas.hc.m.MessageToast.show(self.rb.getText('HPRisk.ComputedVariables.Expression.Validation.Failed.txt'));
            } else {
                stateModel.setProperty("/isValidated", true);
                stateModel.setProperty("/formattedExpression", computedVar.items[0].expression);
            }
        });
        deferred.fail(function (jqxhr, textStatus, error) {
            console.error("failed to validate computed variable " + jqxhr);
            stateModel.setProperty("/isValidated", false);
            stateModel.setProperty("/formattedExpression", "");

        });
    },

    handleApplyPressed: function() {
        var self = this;
        var isEdit = self.getView().getModel(self.MODEL_EDITOR_STATE).getProperty("/isEdit");
        if (isEdit) {
            self.updateComputedVariable();
        } else {
            self.createComputedVariable();
        }
    },

    updateCompVarOnDataSource: function(dataSource, compVarData) {
        var self = this;
        var deferred = $.Deferred();

        var request = self.CompVarUtil.buildComputedVarRequest(compVarData, dataSource);
        var origFormat = dataSource.getDataItemById(compVarData.serverId).format;
        var existingDataItem = dataSource.getDataItemByNameAndType(compVarData.name, sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR);
        var wasValid = existingDataItem.isValid();
        var isValid = self.CompVarUtil.isValidComputedVariable(dataSource, compVarData);

        if (wasValid) {
            if (isValid) {
                // update comp var on dataSource
                sas.analytics.riskhp.utils.HprAPI.updateComputedVariable(dataSource, request).done(function (computedVar) {
                    var stateModel = self.getView().getModel(self.MODEL_EDITOR_STATE);
                    stateModel.setProperty("/isApplied", true);

                    if (computedVar.error != 0) {
                        // results in messageToast and editor remaining open
                        deferred.fail();
                    } else {
                        var dataItem = dataSource.getDataItemByNameAndType(compVarData.name, sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR);

                        // update the dataItem with results from server creation
                        dataItem.label = dataItem.generateLabel(computedVar);
                        dataItem.tooltip = dataItem.generateTooltip(computedVar);
                        dataItem.format = computedVar.format;
                        dataItem.dataType = computedVar.dataType;

                        CacheManager.invalidate(dataSource.dataSource.id);

                        var dataPaneView = self.ViewManager.getExplorationView().getDataPane().getDataPaneView();
                        var dataPaneController = dataPaneView.getController();
                        self.DataPaneUtil.updateComputedVariable(dataSource, compVarData, computedVar, dataPaneController);

                        var crosstabIds = self.CompVarUtil.getCrosstabIdsUsingDataSource(dataSource.dataSource);
                        if (dataSource.dataSource.id === self.crosstab.context.dataSourceId) {
                            crosstabIds.push(self.crosstab.sId);
                        }
                        _.each(crosstabIds, function(id) {
                            var crosstab = sap.ui.getCore().byId(id);
                            if (crosstab && crosstab.context) { // extra check if crosstab is hydrated.  redundant?
                                // var dataItem = dataSource.getDataItemByNameAndType(compVarData.name, sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR);

                                // crosstab keeps track of the column format outside of the dataitem (columnFormatOverride),
                                // so we have to explicitly tell crosstab if the format changes
                                if (origFormat !== computedVar.format) {
                                    self.crosstab.context.clearColumnFormatOverride([dataItem]);
                                }

                                // even on update, the computed variable might not have been in the crosstab, so add it
                                self.crosstab.rebuildCrosstab();
                                // self.DataPaneUtil.addAllDataItemsToCrosstab(self.crosstab,[dataItem]);
                            }
                        });
                        deferred.resolve();
                    }
                }).fail(function (jqxhr, textStatus, error) {
                    console.error("failed to update computed variable " + jqxhr);
                    deferred.fail();
                });
            } else {
                // mark comp var dataitem and all dependent comp var data items invalid, and remove invalid comp vars from crosstabs based on dataSource
                var dataPaneView = self.ViewManager.getExplorationView().getDataPane().getDataPaneView();
                var dataPaneController = dataPaneView.getController();
                self.DataPaneUtil.setCompVarInvalid(existingDataItem, dataSource, dataPaneController, true).done(function(){
                    CacheManager.invalidate(dataSource.dataSource.id);
                    deferred.resolve();
                });

            }
        } else {
            if (isValid) {
                // mark comp var dataitem valid (and create it on server) and revalidate all dependent comp var data items which may become valid now
                var dataPaneView = self.ViewManager.getExplorationView().getDataPane().getDataPaneView();
                var dataPaneController = dataPaneView.getController();
                self.DataPaneUtil.setCompVarValid(existingDataItem, dataSource, dataPaneController, true, compVarData).done(function(){
                    CacheManager.invalidate(dataSource.dataSource.id);
                    deferred.resolve();
                });
            } else {
                // still not valid, no need to do anything
                deferred.resolve();
            }
        }
        return deferred;
    },



    updateComputedVariable: function() {
        var self = this;
        var compVarData = self.getView().getModel().getData();

        // first attempt to create the comp var on the data source of the crosstab used by the editor
        var editorDataSource = self.crosstab.context.cubeDataModel;
        self.updateCompVarOnDataSource(editorDataSource, compVarData).done(function() {

            self.ComputedVariablesManager.updateComputedVariable(compVarData);

            // automatically add the edited computed variable column to the editor crosstab
            var dataItem = editorDataSource.getDataItemByNameAndType(compVarData.name, sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR);
            self.DataPaneUtil.addAllDataItemsToCrosstab(self.crosstab,[dataItem]);
            self.crosstab.rebuildCrosstab();

            self.DisplayRulesManager.validateDisplayRules(editorDataSource);

            // updated comp var on data source of editor, now try on all other loaded data sources
            var loadedDataSourceIds = DataSourceCacheManager.getLoadedDataSourceIds();
            if (loadedDataSourceIds.length > 1) {
                // already did the editors data source, so reject it
                var otherDataSourceIds = _.reject(loadedDataSourceIds, function(id) {
                    return id === editorDataSource.dataSource.id;
                });

                _.each(otherDataSourceIds, function(dataSourceId) {
                    DataSourceCacheManager.getDataSource(dataSourceId).done(function(dataSource) {
                        self.updateCompVarOnDataSource(dataSource, compVarData).done(function() {
                            self.DisplayRulesManager.validateDisplayRules(dataSource);
                        });
                    });
                });
            }

            self.cancelEditing();
        }).fail(function() {
            sas.hc.m.MessageToast.show(self.rb.getText('HPRisk.ComputedVariables.Expression.Update.Failed.txt'));
        });
    },




    createCompVarOnDataSource: function(dataSource, compVarData) {
        var self = this;
        var deferred = $.Deferred();

        var dataPaneController = sas.analytics.riskhp.utils.ViewManager.getExplorationView().getDataPane().getDataPaneView().getController();

        var isValid = self.CompVarUtil.isValidComputedVariable(dataSource, compVarData);
        if (isValid) {
            var request = self.CompVarUtil.buildComputedVarRequest(compVarData, dataSource);
            // validate the computed variable on the server.  Always succeeds, but comp var items returned with error != 0 failed
            var validateDeferred = sas.analytics.riskhp.utils.HprAPI.validateComputedVariable(dataSource, [request]);
            validateDeferred.done(function(cvItems){
                if (cvItems.items[0].error === 0) {
                    // validation succeeded, now create CV on server
                    self.HprAPI.createComputedVariable(dataSource, [request]).done(function (computedVars) {
                        // update editor state (validate/apply buttons an name can no longer be edited)
                        var stateModel = self.getView().getModel(self.MODEL_EDITOR_STATE);
                        stateModel.setProperty("/isApplied", true);

                        var computedVar = computedVars.items[0];
                        if (computedVar.error === 0) {

                            // datapane items need an id.  save the server id in the exploration in case it fails to validate when reopening exploration
                            compVarData.serverId = computedVar.id;
                            // addComputedVariable ignores duplicates
                            sas.analytics.riskhp.models.ComputedVariablesManager.addComputedVariable(compVarData);

                            self.DataPaneUtil.addComputedVariable(dataSource, computedVar, dataPaneController);

                            CacheManager.invalidate(dataSource.dataSource.id);

                            deferred.resolve();
                        } else { // server failed to create the comp var for this data source
                            // add comp var to datapane in invalid state so it can be edited/corrected
                            self.DataPaneUtil.addComputedVariable(dataSource, computedVar, dataPaneController);
                            deferred.fail();
                        }
                    });
                } else {
                    // there was a validation error on the server
                    // add comp var to datapane in invalid state so it can be edited/corrected
                    self.DataPaneUtil.addComputedVariable(dataSource, cvItems.items[0], dataPaneController);
                    deferred.fail();
                }
            });
        } else {
            // not a valid comp var for this data source.  Add comp var to the datapane in invalid state so it can be
            // edited/corrected
            //
            // The comp var editor prevents the user from creating a comp var that is invalid for the dataSource of the
            // editor's crosstab, so this case can only be reached for an incompatible dataSource after the comp var
            // exists for the dataSource of the editor's crosstab
            var invalidCV = self.HprUtil.hprClone(self.crosstab.context.cubeDataModel.getDataItemByNameAndType(compVarData.name, sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR));
            invalidCV.error = 1;
            self.DataPaneUtil.addComputedVariable(dataSource, invalidCV , dataPaneController);
            var cvDataItem = dataSource.getDataItemById(invalidCV.id);
            cvDataItem.setValid(false);

            deferred.fail();
        }
        return deferred;
    },

    createComputedVariable: function() {
        var self = this;
        var compVarData = self.getView().getModel().getData();

        // first attempt to create the comp var on the data source of the crosstab used by the editor
        var editorDataSource = self.crosstab.context.cubeDataModel;
        self.createCompVarOnDataSource(editorDataSource, compVarData).done(function() {
            // add the computed variable on the crosstab used in the editor
            var dataItem = editorDataSource.getDataItemByNameAndType(compVarData.name, sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR);
            self.DataPaneUtil.addAllDataItemsToCrosstab(self.crosstab, [dataItem]);

            self.DisplayRulesManager.validateDisplayRules(editorDataSource);
            // created comp var on data source of editor, now try on all other loaded data sources
            var loadedDataSourceIds = DataSourceCacheManager.getLoadedDataSourceIds();
            if (loadedDataSourceIds.length > 1) {
                // already did the editors data source, so reject it
                var otherDataSourceIds = _.reject(loadedDataSourceIds, function(id) {
                    return id === editorDataSource.dataSource.id;
                });

                _.each(otherDataSourceIds, function(dataSourceId) {
                    DataSourceCacheManager.getDataSource(dataSourceId).done(function(dataSource) {
                        self.createCompVarOnDataSource(dataSource, compVarData).done(function() {
                            self.DisplayRulesManager.validateDisplayRules(dataSource);
                        });
                    });
                });
            }

            self.cancelEditing();
        }).fail(function() {
            // failed to add comp var on data source
            sas.hc.m.MessageToast.show(self.rb.getText('HPRisk.ComputedVariables.Expression.Creation.Failed.txt'));
        });
    },


    /**
     * returns true if the column can be used as an item in a computed variable expression
     * @param tuple
     */
    isValidCompVarDataItemColumn: function(info) {
        var self = this;

        if (!info.tuple) {
            return false;
        }
        // can't have this computed variable as an item in itself
        var isMyself = false;
        var isDefinedLater = false;
        if (self.getView().getModel(self.MODEL_EDITOR_STATE).getProperty("/isEdit")) {
            var tuple = info.tuple[0];
            if (tuple.type === "computedVariable" && tuple.subType === "expression") {
                isMyself = tuple.name === self.getView().getModel().getProperty("/name");

                // When editing/updating, the server can only handle items that have already been defined.
                // It cannot handle editing an expression to be based on an expression defined after itself.  This is how the flex client worked.
                // An alternative would be to delete all expressions and reapply them in the correct order (or fix the server).
                var expressions = sas.analytics.riskhp.models.ComputedVariablesManager.getComputedVariables();
                var myIndex = _.findIndex(expressions, function (expression) {
                    return expression.name === self.getView().getModel().getProperty("/name")
                });
                var tupleIndex = _.findIndex(expressions, function (expression) {
                    return expression.name === tuple.name;
                });
                isDefinedLater = myIndex < tupleIndex;
            }
        }

        // server does not know about STALENESS so cannot use in computed variables
        var staleness =_.find(info.tuple, function(tupleItem) {
            return tupleItem.type === "statistic" && (tupleItem.name === "STALENESS" || tupleItem.name === "ALERTDATETIME" || tupleItem.name === "EXPIREDATETIME");
        });
        return !isMyself && !staleness && !isDefinedLater;
    },

    _findItemsInExpression: function(expression) {
        var myRe = /\bITEM([0-9]+)\b/g;
        var foundItems = [];
        var items;
        while (items = myRe.exec(expression)) {
            foundItems.push(items[0]);
            // console.log("Found " + items[0]);
        }
        return foundItems;
    },


    _getComputedVariableDataItem: function(name) {
        var self = this;
        var dataSource = self.crosstab.context.cubeDataModel;

        var computedVars = dataSource.computedVariable;
        return _.find(computedVars, function(cv) {
            return cv.name === name;
        });
    },

    _isComputedVariableNameUnique: function(newName) {
        var self = this;
        var dataSource = self.crosstab.context.cubeDataModel;

        var computedVars = dataSource.computedVariable;
        return _.find(computedVars, function(cv) {
            return cv.name === newName;
        }) === undefined;
    },

    isReadyForValidation: function(isValidated, name, expression, hasInvalidItems) {
        var self = this;

        if (isValidated || !expression) {
            return false;
        }

        // could check for tuples for items in expression

        // self._findItemsInExpression(expression);
        var expressionIsValid = expression.length >= (name.length + 3); // minimum valid expression is name=X;

        return self.isNameValid && expressionIsValid && !hasInvalidItems;
    },

    handleExpressionChange: function(event) {
        var self = this;
        var view = self.getView();

        // after edit has occurred, mark the cv as invalid
        // could track last validated expression and mark it validated if they undo editing
        self.setNeedsValidation();
    },

    deleteItem: function(event) {
        var self = this;
        var model = self.getView().getModel();
        var items = model.getProperty("/items");

        var listItem = event.getParameter("listItem");
        var item = listItem.getBindingContext().getObject();

        items.splice( items.indexOf(item), 1);
        model.updateBindings();
    },

    // left over from when we supported multi-select of items
    deleteItems: function() {
        var self = this;
        var view = self.getView();
        var model = self.getView().getModel();
        var items = model.getProperty("/items");

        _.each(view.itemsList.getSelectedContexts(), function(context) {
            var item = context.getObject();
            items.splice( items.indexOf(item), 1);
        });
        view.itemsList.removeSelections();
        model.refresh();
    },

    // left over from when we supported multi-select of items
    handleItemSelectionChange: function() {
        var self = this;
        var view = self.getView();

        view.removeMenuItem.setEnabled(view.itemsList.getSelectedItems().length > 0);
    },

    getItems: function() {
        var self = this;
        var model = self.getView().getModel();
        var items = model.getProperty("/items");
        return items;
    },

    handleOpenSelectFormatDialog: function() {
        var self = this;

        var okButton = new sas.hc.m.Button({
            text: self.rb.getText("hprisk.okButton.txt"),
            press: function() {
                var viewModel = self.getView().getModel();

                if (formatsRBG.getSelectedIndex() === 0) {
                    viewModel.setProperty("/dataType", self.FORMAT_NUMERIC);
                } else if (formatsRBG.getSelectedIndex() === 1) {
                    viewModel.setProperty("/dataType", self.FORMAT_PERCENT);
                } else {
                    viewModel.setProperty("/dataType", self.FORMAT_STRING);
                }

                viewModel.setProperty("/dataLength", stringLenStepper.getValue());
                dialog.close();
            }
        });

        var isValidInteger = function(str) {
            var n = Math.floor(Number(str));
            return n !== Infinity && String(n) === str && n >= 8; // 8 is min string column width
        };

        var updateOkButtonState =  function() {
            var val = stringLenStepper.getValue().replace(/^0+/, '');  // get string value and remove leading 0s
            okButton.setEnabled(isValidInteger(val));
        };

        var cancelButton = new sas.hc.m.Button({
            text: self.rb.getText("hprisk.cancelButton.txt"),
            press: function() {
                dialog.close();
            }
        });

        var layout = new sas.hc.m.FlexBox({
            direction: sap.m.FlexDirection.Row,
            fitContainer: true,
            alignItems: sap.m.FlexAlignItems.End,
            alignContent: sap.m.FlexAlignContent.End
        });

        var formatsRBG = new sas.hc.m.RadioButtonGroup({
            selectedIndex: {
                path: "/dataType",
                formatter: function(type) {
                    if (type === self.FORMAT_NUMERIC) {
                        return 0;
                    } else if (type === self.FORMAT_PERCENT) {
                        return 1;
                    } else {
                        return 2;
                    }
                }
            },
            select : function(event) {
                if (event.getParameter("selectedIndex") === 0) {
                    dialog.getModel().setProperty("/dataType", self.FORMAT_NUMERIC);
                } else if (event.getParameter("selectedIndex") === 1) {
                    dialog.getModel().setProperty("/dataType", self.FORMAT_PERCENT);
                } else {
                    dialog.getModel().setProperty("/dataType", self.FORMAT_STRING);
                }

                if (stringLenStepper.getValue() === "") {
                    stringLenStepper.setValue(32);
                }

                updateOkButtonState();
            },
            buttons: [
                new sas.hc.m.RadioButton({
                    text: self.rb.getText('HPRisk.ComputedVariables.DataType.Numeric.txt'),
                    tooltip : self.rb.getText('HPRisk.ComputedVariables.DataType.Numeric.txt'),
                    groupName: "formats"
                }),
                new sas.hc.m.RadioButton({
                    text: self.rb.getText('HPRisk.ComputedVariables.DataType.Percent.txt'),
                    tooltip : self.rb.getText('HPRisk.ComputedVariables.DataType.Percent.txt'),
                    groupName: "formats"
                }),
                new sas.hc.m.RadioButton({
                    text: self.rb.getText('HPRisk.ComputedVariables.DataType.String.txt'),
                    tooltip : self.rb.getText('HPRisk.ComputedVariables.DataType.String.txt'),
                    groupName: "formats"
                })
            ]
        });
        layout.addItem(formatsRBG);

        var stringLenStepper = new sas.hc.m.NumericStepper({
            required: true,
            stepSize: 1,
            pageSize: 32,
            enabled: {
                path: "/dataType",
                formatter: function(dataType) {
                    return dataType === self.FORMAT_STRING;
                }
            },
            value: {
                path: '/dataLength',
                type: new sap.ui.model.type.Integer({
                    minIntegerDigits: 1, // minimal number of non-fraction digits
                    maxIntegerDigits: 4, // maximal number of non-fraction digits
                    minFractionDigits: 0, // minimal number of fraction digits
                    maxFractionDigits: 0  // maximal number of fraction digits
                }, {
                    minimum: 8,
                    maximum: 1024
                }),
                formatter: function(value) {
                    // Use the default value when there is no initial value
                    if (!value) {
                        return 32;
                    }
                    return value;
                }
            },
            liveChange: function(evt){
                var source = evt.getSource();
                source._performValidation();

                updateOkButtonState();
            }
        }).addStyleClass("computedVarFormatStepper");
        layout.addItem(stringLenStepper);

        var dialog = new sas.hc.m.Dialog({
            title: self.rb.getText('HPRisk.ComputedVariables.DataType.Select.Format.title.txt'),
            content: layout,
            leftButton: okButton,
            rightButton: cancelButton
        });

        var viewModel = self.getView().getModel();

        var formatModel = new sap.ui.model.json.JSONModel();
        formatModel.setData({
            dataType: viewModel.getProperty("/dataType"),
            dataLength: viewModel.getProperty("/dataLength")
        });
        dialog.setModel(formatModel);
        dialog.open();

    }

});