// (c) 2017, SAS Institute Inc.

jQuery.sap.require("sas.analytics.riskhp.views.computedvariableeditor.DroppableListItemDelegate");
jQuery.sap.require("sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil");

sap.ui.jsview('sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableEditor', {

    getControllerName: function () {
        return 'sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableEditor';
    },

    createOverflowMenu: function (ctrl) {
        var view = this;

        var overflowMenu = new sas.hc.ui.unified.Menu();

        //Create the items and add them to the menu
        view.removeMenuItem = new sas.hc.ui.unified.MenuItem({
            text: '{i18N>HPRisk.ComputedVariables.Remove.Items.txt}',
            tooltip: '{i18N>HPRisk.ComputedVariables.Remove.Items.txt}',
            enabled: false,
            select: $.proxy(ctrl.deleteItems, ctrl)
        });
        overflowMenu.addItem(view.removeMenuItem);

        return overflowMenu;
    },

    /**
     * Returns the ResourceBundle used
     * @returns {*|jQuery.sap.util.ResourceBundle}
     * @private
     */
    _getResourceBundle: function () {
        return sas.analytics.riskhp.utils.HprUtil.getBundle();
    },


    createContent: function (ctrl) {
        var view = this;

        var topLabel = new sas.hc.m.Label({
            text:  'Top'
        });

        this.topSplitterPanel = sas.risk.common.widgets.WidgetFactory.buildPanel({
            content: [topLabel],
            topPadding: "0rem",
            bottomPadding: "0rem",
            leftPadding: "0.5rem",
            rightPadding: "0.5rem"
        });

        var helpPopup = new sas.hc.m.HelpPopup({
            library: HprConstants.HPR_EUA_LIBRARY,
            layoutData: new sas.hc.m.OverflowToolbarLayoutData({
                priority: sap.m.OverflowToolbarPriority.High
            }),
            key: HprConstants.HPR_EUA_COMPUTED_VARIABLE_KEY
        });

        var addItemButton = new sas.hc.m.Button({
            icon: sas.icons.HC.ADD,
            text: '{i18N>HPRisk.ComputedVariables.Add.Item.txt}',
            press: $.proxy(ctrl.addItemButtonPress, ctrl)
        });
        var overflowButton = new sas.hc.ui.commons.MenuButton({
            icon: sas.icons.HC.OVERFLOW,
            menu: this.createOverflowMenu(ctrl)
        });
        var itemsToolbar = new sas.hc.m.Toolbar({
            height: "100%",
            content: [addItemButton, new sas.hc.m.ToolbarSpacer(), helpPopup]//, overflowButton]
        });
        
        var oItemTemplate = new sas.hc.m.CustomListItem({
            type: sap.m.ListType.Active,
            content: [
                new sas.hc.m.VBox({
                    items: [
                        new sas.hc.m.Label({
                            text: {
                                path: 'name',
                                formatter: function (name) {
                                    return name + ":";
                                }
                            }
                        }),
                        new sas.hc.m.Input({
                            value: {
                                path: 'dataItems',
                                formatter: function(dataItems) {
                                    var columnStr = "";
                                    var sortedDataItems = ctrl.crosstab.context.reconcileColumnAgainstHierarchy(dataItems);
                                    _.each(sortedDataItems, function(di) {
                                        var dataItem = ctrl.crosstab.context.cubeDataModel.getDataItemById(di.id);
                                        columnStr += dataItem.label + " ";
                                    });
                                    return columnStr;
                                }
                            },
                            disableValidation: false,
                            valueState: {
                                path: 'dataItems',
                                formatter: function(dataItems) {
                                    var isValid = sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil._isValidColumn(ctrl.crosstab.context.cubeDataModel,dataItems);
                                    if (!isValid) {
                                        return sap.ui.core.ValueState.Error;
                                    }
                                    return sap.ui.core.ValueState.None;
                                }
                            },
                            // valueStateText: doesn't show because the Input is not editable
                            placeholder: '{i18N>HPRisk.ComputedVariables.Item.Input.Placeholder.txt}',
                            editable: false
                        })
                    ]
                })
            ]
        }).addStyleClass("computedVarItem");

        view.itemsList = new sas.hc.m.List({
            mode: sap.m.ListMode.Delete,
            // selectionChange: $.proxy(ctrl.handleItemSelectionChange, ctrl),
            "delete": $.proxy(ctrl.deleteItem, ctrl)
        });

        // itemsList.addDelegate(new sas.analytics.riskhp.utils.DraggableListItemDelegate(itemsList, {cancel:false}));
        var droppableDelegate = new sas.analytics.riskhp.views.computedvariableeditor.DroppableListItemDelegate(view.itemsList, {});
        droppableDelegate.setEditor(ctrl);
        view.itemsList.addDelegate(droppableDelegate);
        view.itemsList.bindAggregation("items", "/items", oItemTemplate);

        var itemsLayout = new sap.ui.commons.layout.BorderLayout({
            width: "100%",
            height: "100%"
        });
        itemsLayout.createArea(sap.ui.commons.layout.BorderLayoutAreaTypes.top, itemsToolbar).addStyleClass("itemsListTop");
        itemsLayout.createArea(sap.ui.commons.layout.BorderLayoutAreaTypes.center, view.itemsList).addStyleClass("itemsListCenter");
        itemsLayout.setLayoutData(new sap.ui.layout.SplitterLayoutData({
            size: "320px"
        }));
        view.editorTabBar = new sas.hc.m.IconTabBar({
            id: ctrl.createId("editorTabBar"),
            firstTabClosable: false,
            showOverflowPopOver: false
        }).addStyleClass("computedVarTabBar");

        var editorTextArea = new sas.hc.m.TextArea({
            // required: true,
            editable: true,
            value: '{/expression}',
            autoExpandable: false,
            liveChange: $.proxy(ctrl.handleExpressionChange, ctrl)
        }).addStyleClass("forceFullHeight");
        var editorTab = new sas.hc.m.IconTabFilter({
            key: "editor",
            text: '{i18N>HPRisk.ComputedVariables.Editor.Tab.title.txt}',
            closable: false
        });

        editorTab.addContent(editorTextArea);
        view.editorTabBar.addItem(editorTab);

        var formattedTextArea = new sas.hc.m.TextArea({
            editable: false,
            value: '{editorState>/formattedExpression}'
        }).addStyleClass("forceFullHeight");
        var formattedTab = new sas.hc.m.IconTabFilter({
            key: "formatted",
            text: '{i18N>HPRisk.ComputedVariables.Formatted.Tab.title.txt}',
            closable: false,
            enabled: '{editorState>/isValidated}'
        });
        formattedTab.addContent(formattedTextArea);
        view.editorTabBar.addItem(formattedTab);

        var editorLayout = sas.risk.common.widgets.WidgetFactory.buildPanel({
            leftPadding: 0,
            rightPadding: "0.5rem",
            topPadding: 0,
            bottomPadding: 0,
            content: [ view.editorTabBar ]
        });

        view.itemsSplitter = new sas.hc.ui.layout.Splitter({
            orientation: sap.ui.core.Orientation.Horizontal,
            height: "100%",
            width: "100%",
            contentAreas: [itemsLayout, editorLayout]
        });



        //
        // Name
        //

        var nameLabel = new sas.hc.m.Label({
            text: '{i18N>HPRisk.ComputedVariables.Name.Field.label}',
            required: true
        });

        this.nameInput = new sas.hc.m.Input({
            liveChange: $.proxy(ctrl.handleNameLiveChange, ctrl),
            change: $.proxy(ctrl.handleNameChange, ctrl),
            minLength: 1,
            maxLength: HprConstants.SAS_NAME_MAX_LENGTH,
            enabled: {
                path: "editorState>/isEdit",
                formatter: function(isEdit) {
                    return !isEdit;
                }
            },
            value: '{/name}',
            disableValidation: false
        });

        //
        // Label
        //

        var labelLabel = new sas.hc.m.Label({
            text: '{i18N>HPRisk.ComputedVariables.Label.Field.label}'
        });

        this.labelInput = new sas.hc.m.Input({
            value: '{/label}',
            maxLength: 256,
            change: $.proxy(ctrl.handleLabelChange, ctrl),
            liveChange: $.proxy(ctrl.handleLabelLiveChange, ctrl),
        });

        //
        // Format
        //

        var formatLabel = new sas.hc.m.Label({
            text: '{i18N>HPRisk.ComputedVariables.Format.label}'
        });

        var formatInput = new sas.hc.m.CompositeInput({
            inputProperties: {
                type: sap.m.InputType.text,
                editable: false,
                value: {
                    parts: ["/dataType", "/dataLength"],
                    formatter: function(dataType, dataLength) {
                        if (dataType === "numeric") {
                            return view._getResourceBundle().getText("HPRisk.ComputedVariables.DataType.Numeric.txt")
                        } else if (dataType === "percent") {
                            return view._getResourceBundle().getText("HPRisk.ComputedVariables.DataType.Percent.txt")
                        } else {
                            return view._getResourceBundle().getText("HPRisk.ComputedVariables.DataType.String.fmt", [dataLength]);
                        }
                    }
                }
            },
            buttonProperties: {
                icon: sas.icons.HC.OPEN,
                tooltip: '{i18N>HPRisk.ComputedVariables.Configure.DataType.tooltip.txt}',
                press: $.proxy(ctrl.handleOpenSelectFormatDialog, ctrl)
            }
        });


        //
        // Expressions
        //

        var expressionLabel = new sas.hc.m.Label({
            text: '{i18N>HPRisk.ComputedVariables.Expression.Form.label}'
        });

        view.expressionSelect = new sas.hc.m.Select({
            change: $.proxy(ctrl.handleExpressionSelection, ctrl)
        });
        var expressionItemTemplate = new sap.ui.core.ListItem({
            text: "{expressions>name}",
            tooltip: "{expressions>name}"
        });
        view.expressionSelect.bindItems("expressions>/expressions", expressionItemTemplate, null);

        
         var nameLayout = new sas.hc.ui.layout.VerticalLayout({
            content: [
                nameLabel,
                this.nameInput
            ]
        });
        var labelLayout = new sas.hc.ui.layout.VerticalLayout({
            content: [
                labelLabel,
                this.labelInput
            ]
        });
        var formatLayout = new sas.hc.ui.layout.VerticalLayout({
            content: [
                formatLabel,
                formatInput
            ]
        });
        var expressionLayout = new sas.hc.ui.layout.VerticalLayout({
            content: [
                expressionLabel,
                view.expressionSelect
            ]
        });


        var topToolbar = new sas.hc.m.Toolbar({
            content : [
                nameLayout,
                labelLayout,
                formatLayout,
                new sas.hc.m.ToolbarSpacer(),
                expressionLayout
            ]
        }).addStyleClass("computedVarTop");
        var bottomLayout = new sap.ui.commons.layout.BorderLayout({
            width: "100%",
            height: "100%"
        });
        bottomLayout.createArea(sap.ui.commons.layout.BorderLayoutAreaTypes.top, topToolbar).addStyleClass("computedVarTop");
        bottomLayout.createArea(sap.ui.commons.layout.BorderLayoutAreaTypes.center, view.itemsSplitter).addStyleClass("computedVarCenter");


        this.bottomPanel = sas.risk.common.widgets.WidgetFactory.buildPanelNoPadding({
            content: [ bottomLayout ]
        });

        var splitter = new sas.hc.ui.layout.Splitter({
            orientation: sap.ui.core.Orientation.Vertical,
            height: "100%",
            width: "100%",
            contentAreas: [this.topSplitterPanel, this.bottomPanel]
        });

        var toolbar = this._createToolbar(ctrl);

        this.mainContentArea = new sas.hc.ui.commons.layout.BorderLayoutArea({
            contentAlign: "center",
            overflowX: "hidden",
            overflowY: "hidden",
            content: [splitter]
        }).addStyleClass("computedVariableEditor");

        var mainLayout = new sas.hc.ui.commons.layout.BorderLayout({
            height: '100%',
            width: '100%',
            center: this.mainContentArea,
            bottom: new sas.hc.ui.commons.layout.BorderLayoutArea({
                size: "2.75rem",
                content: [toolbar]
            })
        });

        return mainLayout;
    },

    _createToolbar: function (ctrl) {

        var validateButton = new sas.hc.m.Button({
            text: '{i18N>hprisk.validateButton.txt}',
            enabled: {
                parts: ["editorState>/isValidated", "/name", "/expression", "editorState>/hasInvalidItems"],
                formatter: $.proxy(ctrl.isReadyForValidation, ctrl)
            },
            press: $.proxy(ctrl.handleValidatePressed, ctrl),
            tooltip: '{i18N>hprisk.validateButton.txt}'
        }).addStyleClass("sasPrimaryButton");

        var applyButton = new sas.hc.m.Button({
            text: '{i18N>hprisk.applyButton.txt}',
            enabled: {
                parts: ['editorState>/isValidated', 'editorState>/isApplied'],
                formatter: function(isValidated, isApplied) {
                    return isValidated && !isApplied;
                }
            },
            press: $.proxy(ctrl.handleApplyPressed, ctrl),
            tooltip: '{i18N>hprisk.applyButton.txt}'
        }).addStyleClass("sasPrimaryButton");

        var cancelButton = new sas.hc.m.Button({
            text: '{i18N>hprisk.cancelButton.txt}',
            press: $.proxy(ctrl.cancelEditing, ctrl),
            tooltip: '{i18N>hprisk.cancelButton.txt}'
        });

        var toolbar = new sas.hc.m.ResponsiveToolbar({
            content : [
                new sas.hc.m.ToolbarSpacer(),
                validateButton,
                applyButton,
                cancelButton
            ]
        });

        return toolbar;
    }
});