// (c) 2018, SAS Institute Inc.

/*
 * '(c) 2017, SAS Institute Inc.'
 */
jQuery.sap.declare("sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil");

jQuery.sap.require("sas.analytics.riskhp.models.DataSourceCacheManager");
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
jQuery.sap.require("sas.analytics.riskhp.utils.HprAPI");
jQuery.sap.require("sas.analytics.riskhp.utils.ViewManager");
jQuery.sap.require("sas.analytics.riskhp.utils.ExplorationUtil");
jQuery.sap.require("sas.analytics.riskhp.models.ComputedVariablesManager");

/**
 * @name sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil
 */
sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil = {
    HprUtil: sas.analytics.riskhp.utils.HprUtil,
    ExplorationUtil: sas.analytics.riskhp.utils.ExplorationUtil,
    ComputedVariablesManager: sas.analytics.riskhp.models.ComputedVariablesManager,


buildComputedVarRequest: function(data, cubeDataModel) {
        var dataSource = cubeDataModel.dataSource;

        // get the tuples from the items
        var tuples = [];
        _.each(data.items, function(item) {
            if (item.dataItems) {
                tuples.push({
                    name: item.name,
                    dataItems: item.dataItems
                });
            }
        });

        var request = {
            version: 1,
            name: data.name,
            label: data.label,
            dataType: data.dataType,
            dataLength: data.dataLength,
            dataSource: {
                id: dataSource.id,
                name: dataSource.name,
                label: dataSource.label,
                type: dataSource.type,
                lastModifiedTimeStamp: dataSource.lastModifiedTimeStamp
            },
            dataDefinitions: {
                tuples: tuples,
                rawExpression: data.expression
            }
        };

        // set the id on the request if this is an update of an existing computed variable
        var existingCV = cubeDataModel.getDataItemByNameAndType(data.name, sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR);
        if (existingCV &&  !existingCV.error) {
            request.id = existingCV.id;
        }

        return request;
    },

    _getValidCrosstabIds: function() {
        var self = this;

        // Gather all the Crosstab Ids on the selected RiskPanelsView
        var selectedRiskPanelsView = self.HprUtil.getSelectedRiskPanelsView();
        var riskPanelsViewDelegate = selectedRiskPanelsView.getDelegate();
        var crosstabIds = riskPanelsViewDelegate.crosstabIds;

         // Filter down all the Crosstab Ids to only valid ones
        return _.filter(crosstabIds, function(crosstabId) {
            var crosstab = sap.ui.getCore().byId(crosstabId);
            if (crosstab && crosstab.context) {
                // If the Crosstab Data Source id does not match the DataPane Data Source id, then Crosstab is not valid
                return crosstab.context.getDataSourceId() === DataPaneModelManager.getSelectedDataSourceId();
            }
            return false; // If Crosstab does not have a context, it is not valid
        });
    },

    /**
     * Controller hook to launch the computed variable editor
     */
    _onNewComputedvariable: function(crosstabContextId) {
        var expCtrl = sas.analytics.riskhp.utils.ViewManager.getExplorationView().getController();
        expCtrl.createComputedVariable.call(expCtrl, crosstabContextId);
    },

    createCreateCompVarMenuItem: function(startsSection) {
        var self = this;

        if (_.isUndefined(startsSection)) {
            startsSection = false;
        }

        var selectedRiskPanelsView = self.HprUtil.getSelectedRiskPanelsView();
        var riskPanelsViewDelegate = selectedRiskPanelsView.getDelegate();

        // Retrieve the list of crosstabIds that are valid for the datapane
        var validCrosstabIds = self._getValidCrosstabIds({
            folderType: sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR
        });

        // need a crosstab to be able to open comp var editor
        if (validCrosstabIds.length > 0) {
            // For the menu item displayed from the DataPane, use the ApplicationSettings showLabels property so
            // the menu item will match the DataPane.  The submenus will be populated using the showLabels option
            // value from the Crosstab
            var showLabels = sas.analytics.riskhp.settings.ApplicationSettings.getShowLabels();

            // If only one crosstab was valid there is no need to show the crosstab submenu
            if (validCrosstabIds.length === 1) {
                // This crosstab variable is lifted to method scope
                var crosstab = sap.ui.getCore().byId(validCrosstabIds[0]);
                var crosstabTitle = riskPanelsViewDelegate.getTitleForContent(validCrosstabIds[0]);

                if (crosstab && crosstab.context) {
                    var createComputedVarMenuItem = new sas.hc.ui.unified.MenuItem({
                        enabled: {
                            path: 'EXPLORATION_PANES_STATE_MODEL>/computedVariableEditorMode',
                            formatter: function(cvEditorMode) {
                                return !cvEditorMode;
                            }
                        },
                        startsSection: startsSection
                    });
                    createComputedVarMenuItem.setText(self.HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.create.computed.variable.fmt", [crosstabTitle]));
                    createComputedVarMenuItem.attachSelect($.proxy(function () {
                        self._onNewComputedvariable(crosstab.context.id);
                    }, self));
                    return createComputedVarMenuItem;
                }
            }
            else {
                // If there is more than one valid crosstab we need to create a crosstab submenu
                var subMenu = new sas.hc.ui.unified.Menu();

                _.each(validCrosstabIds, function (id) {
                    // This crosstab variable is lifted to method scope
                    var crosstab = sap.ui.getCore().byId(id);

                    if (crosstab && crosstab.context) {
                        var subMenuItem = new sas.hc.ui.unified.MenuItem({
                            text: riskPanelsViewDelegate.getTitleForContent(id)
                        });
                        subMenuItem.attachSelect($.proxy(function () {
                            self._onNewComputedvariable(crosstab.context.id);
                        }, self));

                        subMenu.addItem(subMenuItem);
                    }
                });

                if (subMenu.getItems().length > 0) {
                    var createComputedVarMenuItem = new sas.hc.ui.unified.MenuItem({
                        enabled: {
                            path: 'EXPLORATION_PANES_STATE_MODEL>/computedVariableEditorMode',
                            formatter: function(cvEditorMode) {
                                return !cvEditorMode;
                            }
                        }
                    });
                    createComputedVarMenuItem.setText(self.HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.create.computed.variable.txt"));
                    createComputedVarMenuItem.setSubmenu(subMenu);
                    return createComputedVarMenuItem;
                }
            }
        }
    },

    _isValidColumn: function(cubeData, column) {
        var self = this;
        var isValid = true;

        _.each(column, function (dataItem) {
            var item =cubeData.getDataItemById(dataItem.id);
            if (item) {
                if (item.type === sas.analytics.riskhp.models.dataitems.Scenario.SCENARIO) {
                    if (!item.isActivated()) {
                        isValid = false;
                    }
                } else if (item.type === sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR) {
                    if (!item.isValid()) {
                        isValid = false;
                    }
                }
            } else {
                isValid = false;
            }
        });

        return isValid;
    },

    /**
     * determines if computed variable is valid based on all data items being available in the cube, all required
     * scenarios being activated, and all required computed variables being valid.
     *
     * @param cubeData
     * @param compVarData
     */
    isValidComputedVariable: function(cubeData, compVarData) {
        var self = this;

        var isValid = true;
        _.each(compVarData.items, function(item) {
            if (isValid) {
                isValid = self._isValidColumn(cubeData, item.dataItems);
            }
        });

        return isValid;
    },



    /**
     * Loop through the Crosstabs and delete the computed variable from the Crosstabs that contain it in one of it's columns (i.e. tuples)
     * @param crosstabIds
     * @param compVars
     * @private
     */
    removeComputedVariablesFromCrosstabs: function(crosstabIds, compVars) {
        _.each(crosstabIds, function (id) {
            var crosstab = sap.ui.getCore().byId(id);

            if (crosstab && crosstab.context) {
                var rebuildCrosstabFlag = false;
                var tuples = crosstab.context.getTuples();

                // Loop through the computed variables and see if the Crosstab contains it; if so, remove it
                _.each(compVars, function(compVar) {
                    var compVarTuples = _.filter(tuples, function(aTuple) {
                        var compVarDataItem = crosstab.context.getDataItemInfoFromTuple(aTuple, compVar.getType());
                        return compVarDataItem && compVarDataItem.dataItem.id === compVar.id;
                    });

                    if (compVarTuples && compVarTuples.length > 0) {
                        var labelOrNameToUse = crosstab.context.getOptionShowLabels().value ? compVar.getLabel(): compVar.getName();
                        crosstab.context.removeColumnsWithNameAndType(labelOrNameToUse, compVar.getType());
                        rebuildCrosstabFlag = true;
                    }
                });

                // Only rebuild the Crosstab if we have too
                if (rebuildCrosstabFlag) {
                    crosstab.rebuildCrosstab();
                }
            }
        });
    },


    /**
     * recursively find the computed variables used in defining compVar (in compVar's items) and add them to the ancestors array
     * @param ancestors - array of names of computed variables used in defining compVar
     * @param compVar - computed variable for which to find ancestors
     * @param computedVariables - list of exploration computed variables
     * @private
     */
    getComputedVariableAncestors: function(ancestors, compVar, computedVariables, cubeData) {
        var self = this;
        _.each(compVar.items, function(item){
            _.each(item.dataItems, function(dataItem) {
                if (dataItem.type === "computedVariable") {
                    // if it is an exploration cv push it to the ancestors array and recursively add its computed variable ancestors
                    var ancestor = _.find(computedVariables, function (cv) {
                        return cv.name === dataItem.name;
                    });
                    if (ancestor) {
                        ancestors.push(ancestor.name);
                        self.getComputedVariableAncestors(ancestors, ancestor, computedVariables, cubeData);
                    }
                }
            })
        });
    },

    /**
     * recursive method to return the list of all computed variables that require compVar in order to be created.  In
     * other words, all computed variables that have compVar as an ancestor.
     * @param descendants
     * @param compVar
     * @param computedVariables
     * @param cubeData
     */
    getComputedVariableDescendants: function(descendants, compVar, computedVariables, cubeData) {
        var self = this;
        _.each(computedVariables, function(cv) {
            var ancestors = [];
            self.getComputedVariableAncestors(ancestors, cv, computedVariables, cubeData);
            if (_.contains(ancestors, compVar.name)) {
                descendants.push(cv.name);
            }
        });
    },

    getCrosstabIdsUsingDataSource: function(dataSource) {
        var self = this;
        var crosstabIds = [];

        // Find and group all the Crosstabs which use the same Data Source by the Exploration page ids
        var explorationView = sas.analytics.riskhp.utils.ViewManager.getExplorationView();
        var sameDataSourceCrosstabs = self.ExplorationUtil.getCrosstabsUsingDataSource(explorationView, dataSource);
        var groupSameDataSourceCrosstabsByPage = _.groupBy(sameDataSourceCrosstabs, function(crosstab) {
            return crosstab.pageId;
        });

        // Since we have all the Crosstabs which use the same Data Source grouped by pages loop through them and see
        // if the first Crosstab we come across is instantiated.  This means the Page is active and the control exists
        $.each(_.values(groupSameDataSourceCrosstabsByPage), function(index, crosstabs) {
            var firstCrosstabId = crosstabs[0].id;
            var firstCrosstab = sap.ui.getCore().byId(firstCrosstabId);
            // Check the first Crosstab to see if it is in DOM, this means Page is active and control exists; If the first Crosstab
            // can not be found, that means the Page has not been activated (or clicked on), so the controls don't exist yet;  In
            // this case, when the Page/Tab is clicked on, the CrosstabContext code (spec. the 'fromColumnHeaderJSON' method) will trim
            // out the Data items and columns that don't exist anymore
            if (firstCrosstab && firstCrosstab.context) {
                // Pull all the Crosstab ids and try to remove the Computed Variables from the Crosstabs
                // var crosstabIds = _.pluck(crosstabs, 'id');
                // self._removeComputedVariablesFromCrosstabs(crosstabIds, compVars);
                // return true; // Get out, done with the Crosstabs on this Page
                crosstabIds = _.union(crosstabIds, _.pluck(crosstabs, 'id'));
            }
        });

        return crosstabIds;
    },

    /**
     * returns a string representation of the crosstab column
     * @param info - crosstab pickinfo
     * @returns {string}
     */
    getColumnStringFromPickInfo: function(info) {
        var columnStr = "";
        if (info) {
            // info.column has the visible column info needed for the item details
            _.each(info.column, function(col) {
                columnStr += col.label + " ";
            });
        }
        return columnStr;
    },

    /**
     * returns the minimum representation of the tuples for the crosstab column
     * @param info - crosstab pickinfo
     * @returns {Array}
     */
    getTuplesFromPickInfo: function(info) {
        var dataItems = [];
        if (info) {
            // info.tuple contains all the corrected information for communicating with the mid-tier. Corrected meaning
            // benchmark/compute variable headers removed, absolut/relative horizons replaces with base horizon, etc.
            _.each(info.tuple, function(tuple){
                dataItems.push({
                    id: tuple.id,
                    type: tuple.type,
                    name: tuple.name
                });
            });
        }
        return dataItems;
    }


};