// (c) 2017, SAS Institute Inc.
jQuery.sap.require("sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil");

sap.ui.define(['jquery.sap.global', 'sap/ui/base/Object', 'sas/hc/ui/core/delegate/Droppable'],
    function(jQuery, BaseObject, Droppable) {
        "use strict";

        var $ = jQuery;

        var editor = undefined;
        /**
         * Creates a Droppable delegate for the list of items in the computed variable editor.  This is a heavily customized version of the
         * sas core Droppable version.
         * @class Delegate for dropping (i.e. moving) rows in the DataPane.
         * @author maasti
         * @extends sas.hc.ui.core.delegate.Droppable
         * @constructor
         * @name sas.analytics.riskhp.views.computedcolumneditor.DroppableListItemDelegate
         * @public
         */

        var RowDroppable = Droppable.extend("sas.analytics.riskhp.views.computedcolumneditor.DroppableListItemDelegate", {

            /**
             * The constructor.
             *
             * @param {sap.ui.core.Control} oControl the Control of which this Droppable is the delegate
             * @param {object} oDropOptions options to utilize while dropping. Based on jQuery UI droppable options.
             * @param {boolean} bEnableTouch set to false to disable touch dropping.
             */
            constructor: function (oControl, oDropOptions, bEnableTouch) {
                this._oControl = oControl; // the list
                var self = this;
                var oConfig = $.extend({},
                    {
                        tolerance: "pointer", // This must be "pointer" because of the layout of the HprTreeTableRow
                        over: function(event,ui) {
                            // Remove the dataPaneRowDropZoneHovered style from anything else
                            // $(".dataPaneRowDropZoneHovered").removeClass("dataPaneRowDropZoneHovered");
                            //
                            // // Figure out if we have a valid drop area and style it if so.
                            var targetRowId = event.target.id;
                            // var draggableId = ui.draggable[0].id;
                            // var model = DataPaneUtil.getDataPaneTreeTableSelectionModel();
                            // if (model.validDropArea(droppableId, draggableId)) {
                            //     $(".hprrow_dropzone",$("#"+droppableId)).addClass("dataPaneRowDropZoneHovered");
                            // }
                            $("#"+targetRowId).addClass("computedVarRowDropZoneHovered");
                            return true;
                        },
                        out: function(event,ui) {
                            var targetRowId = event.target.id;
                            // Remove the computedVarRowDropZoneHovered style from the item
                            $("#"+targetRowId).removeClass("computedVarRowDropZoneHovered");
                            return true;
                        },
                        deactivate: function(event,ui) {
                            var targetRowId = event.target.id;
                            $("#"+targetRowId).removeClass("computedVarRowDropZoneHovered");
                            return true;
                        },

                        drop: function(event,ui) {

                            var targetRowId = event.target.id;
                            var rowInfo = self.getRowData(targetRowId);

                            var info = ui.draggable.data('dragColumnInfo');

                            var columnStr = sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil.getColumnStringFromPickInfo(info);
                            var dataItems = sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil.getTuplesFromPickInfo(info);

                            rowInfo.model.setProperty(rowInfo.path + "/column", columnStr);
                            rowInfo.model.setProperty(rowInfo.path + "/dataItems", dataItems);
                            $("#"+targetRowId).removeClass("computedVarRowDropZoneHovered");

                            if (editor) {
                                editor.setNeedsValidation();
                                editor.validateItems();
                            }
                        },
                        accept: self._canAcceptDrop
                    }, oDropOptions);
                Droppable.call(this, oControl, oConfig, bEnableTouch);
            },
            _canAcceptDrop: function(draggable) {
                var info = draggable.data('dragColumnInfo');

                if (editor) {
                    return info != null && editor.isValidCompVarDataItemColumn.call(editor, info);
                } else {
                    return false;
                }
            },

            /**
             * Must be set to droppable in onAfterRendering function.
             */
            onAfterRendering: function () {
                var oConfig = $.extend({}, this._oDropOptions);

                var itemsList = this._oControl;
                var items = itemsList.getItems();
                items.forEach(function (item) {
                    $(item.getDomRef()).droppable(oConfig);
                    // $domRefs = getDomRefs(oRow);
                    // oConfig = $.extend({}, oDropOptions, {
                    //     disabled: (!oRow.getBindingContext())
                    // });
                    // $(".hprrow_dropzone",$domRefs).droppable(oConfig);
                    // $(".hprrow_dropzone",$domRefs).data("sasTableRow", oRow);
                });
                // var oTable = this._oControl;
                // this._hasRendered = true;
                // var aRows = oTable.getRows(),
                //     oDropOptions = this._oDropOptions,
                //     oDelegate = this,
                //     oConfig,
                //     $domRefs;
                //
                // // Use closure to keep a distinct copy of row and domrefs
                // function makeContextChangeCallback(oRow, $domRefs) {
                //     return function (oEvent) {
                //         $domRefs.droppable("option", 'disabled', (!oRow.getBindingContext()));
                //     };
                // }
                //
                // function getDomRefs(oRow) {
                //     var $refs = oRow.getDomRef("fixed");
                //     $refs = $($refs).add(oRow.getDomRef());
                //     return $refs;
                // }
                //
                // // For each row, configure the draggable options for it
                // aRows.forEach(function (oRow) {
                //     $domRefs = getDomRefs(oRow);
                //     oConfig = $.extend({}, oDropOptions, {
                //         disabled: (!oRow.getBindingContext())
                //     });
                //     $(".hprrow_dropzone",$domRefs).droppable(oConfig);
                //     $(".hprrow_dropzone",$domRefs).data("sasTableRow", oRow);
                // });
                //
                // // Instead of listening as each row's context is updated, we'll listen to the table for '_rowsUpdated' events
                // oTable.attachEvent("_rowsUpdated", function (oEvent) {
                //     // Loop through all rows to enable/disable drag state
                //     var myRows = oTable.getRows();
                //     myRows.forEach(function (oRow) {
                //         // Add the draggable options again for each row
                //         // NOTE: This solves a problem with hidden rows in TreeTable being able to be dragged
                //         var $refs = getDomRefs(oRow);
                //         oConfig = $.extend({}, oDropOptions);
                //         $refs.droppable(oConfig);
                //         $refs.droppable("option", 'disabled', (!oRow.getBindingContext()));
                //     });
                // });
            },
            /**
             * Finds the row that is associated with the given elementId.  The elementId is assumed to be
             * a child in the hierarchy of the row. The resultant id can be used by sap.ui.getCore().byId() to retrieve the UI5 component.
             * @param elementId
             * @private
             */
            _findRowId: function(elementId) {
                // Find element in the DOM that matches the given id
                var matches = $('#' + elementId);
                // If we got at least one and it is a 'tr' element, return it's id
                if (matches.length > 0 && $(matches[0]).is('li')) {
                    return $(matches[0]).attr('id');
                }
                else {
                    // Otherwise find the closest one up the DOM tree that is a 'table'
                    matches = $('#' + elementId).closest('li');
                    if (matches.length > 0) {
                        return $(matches[0]).attr('id');
                    }
                }
                // If nothing was found return null.
                return null;
            },

            /**
             * Finds the row with the given id and returns it's UI5 object representation
             * @param elementId
             * @returns {*}
             * @private
             */
            _getUI5RowWithId: function(elementId) {
                // // Get the id of the table
                // var tableId = this._findTableId(elementId);
                // if (!tableId) {
                //     return null;
                // }
                //
                // // Get the table as a UI5 object
                // var table = sap.ui.getCore().byId(tableId);
                // if (!table) {
                //     return null;
                // }

                // Get the real id of the row
                var rowId =  this._findRowId(elementId);
                if (!rowId) {
                    return null;
                }

                // Get the row as a UI5 object
                var row = sap.ui.getCore().byId(rowId);
                if (!row) {
                    return null;
                }
                return row;
            },

            /**
             * Given an id of a row, returns the path into the underlying tree table model where that
             * row's data can be found and the row's model as a hash.  If the row cannot be found then returns null.
             * @param rowId
             * @returns {*} Returns null if the row cannot be found, otherwise returns an object with the path key
             * representing the path into the model for the data and the model key representing the actual model
             * for the row.
             */
            getRowData: function(rowId){
                var row = this._getUI5RowWithId(rowId);
                if (!row) {
                    return null;
                }
                var context = row.getBindingContext();
                if (!context) {
                    return null;
                }
                var rowPath = context.getPath();
                var model = row.getModel();
                return {path: rowPath, model: model};
            },

            setEditor: function(controller) {
                editor = controller;
            }

        });
        return RowDroppable;
    },
    true
);