// (c) 2017, SAS Institute Inc.

jQuery.sap.require("sas.hc.m.BusyDialog");
jQuery.sap.require("sas.hc.m.MessageToast");
jQuery.sap.require("sas.hc.ui.unified.Menu");

jQuery.sap.require("sas.analytics.riskhp.datasource.DataSourceChangeManager");
jQuery.sap.require("sas.analytics.riskhp.datasource.DataSourceChanger");
jQuery.sap.require("sas.analytics.riskhp.models.DataSourceCacheManager");
jQuery.sap.require("sas.analytics.riskhp.models.DataSourceParams");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Horizon");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.RelativeHorizon");
jQuery.sap.require("sas.analytics.riskhp.models.HPRiskModelManager");
jQuery.sap.require("sas.analytics.riskhp.settings.ApplicationSettings");
jQuery.sap.require("sas.analytics.riskhp.utils.CompatibilityCheck");
jQuery.sap.require("sas.analytics.riskhp.utils.HprAPI");
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
jQuery.sap.require("sas.analytics.riskhp.utils.ViewManager");
jQuery.sap.require("sas.analytics.riskhp.views.datapane.DataPaneUtil");
jQuery.sap.require("sas.analytics.riskhp.views.datapane.DataPaneModelManager");
jQuery.sap.require("sas.analytics.riskhp.views.datapane.HprDataPaneContextMenuProvider");
jQuery.sap.require("sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil");
jQuery.sap.require("sas.analytics.riskhp.models.DisplayRulesManager");


sap.ui.controller('sas.analytics.riskhp.views.datapane.DataPane', {
    liveChangeTimer: 0,
    initialDataPaneRenderDone: false,
    dataSourceTreeTableContextMenu: null,
    DATA_SOURCE_TREE_TABLE_MODEL_PROPERTY_CHILDREN_PATH: "/dataSourceTreeTableModel/children",
    DATA_SOURCE_TREE_TABLE_MODEL_PROPERTY_PATH: "/dataSourceTreeTableModel",
    HPRiskModelManager: sas.analytics.riskhp.models.HPRiskModelManager,
    DataPaneUtil: sas.analytics.riskhp.views.datapane.DataPaneUtil,
    ComputedVariableUtil: sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil,
    DataSourceChangeManager: sas.analytics.riskhp.datasource.DataSourceChangeManager,
    DataSourceChanger: sas.analytics.riskhp.datasource.DataSourceChanger,
    DataSourceCacheManager: sas.analytics.riskhp.models.DataSourceCacheManager,
    DataSourceParams: sas.analytics.riskhp.models.DataSourceParams,
    ViewManager: sas.analytics.riskhp.utils.ViewManager,
    HprAPI: sas.analytics.riskhp.utils.HprAPI,
    HprUtil: sas.analytics.riskhp.utils.HprUtil,
    HprDataPaneContextMenuProvider: sas.analytics.riskhp.views.datapane.HprDataPaneContextMenuProvider,
    CompatibilityCheck: sas.analytics.riskhp.utils.CompatibilityCheck,
    ApplicationSettings: sas.analytics.riskhp.settings.ApplicationSettings,
    HORIZON: sas.analytics.riskhp.models.dataitems.Horizon.HORIZON,
    RELATIVEHORIZON: sas.analytics.riskhp.models.dataitems.RelativeHorizon.RELATIVEHORIZON,
    DisplayRulesManager: sas.analytics.riskhp.models.DisplayRulesManager,

    logger: sas.log.getLogger("sas.analytics.riskhp.views.datapane.DataPane_controller"),

    onInit: function() {
        // Reset the cached selected items any time this component is rebuilt
        this.DataPaneUtil.getDataPaneTreeTableSelectionModel().clearSelectedRows();

        // Subscribe to an event that is fired when the Display Horizons preference is updated
        sas.eventBus.subscribe(HprConstants.CHANNEL_APP_SETTINGS, HprConstants.EVENT_DISPLAY_HORIZONS_CHANGED, this._onDisplayHorizonsChange, this);

        // Subscribe to an event that is fired when the Show Labels preference is updated
        sas.eventBus.subscribe(HprConstants.CHANNEL_APP_SETTINGS, HprConstants.EVENT_SHOW_LABELS_CHANGED, this._onShowLabelsChange, this);

        // Subscribe to an event that is fired when a Data Item has their 'count' property updated on another panel/view
        sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_DATA_ITEM_COUNT_UPDATED, this._onDataItemCountUpdate, this);

        // Subscribe to an event that is fired when a control is set to selected in the HPRiskPanelsViewDelegate
        sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_DELEGATE_SELECTED_CONTROL_CHANGED, this._onDelegateSelectedControlChanged, this);

        // subscribe to scenario events
        sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_SCENARIO_UPDATED, this._onScenarioUpdated, this);
        sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_SCENARIO_DELETED, this._onScenarioDeleted, this);
        sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_SCENARIO_ADDED, this._onScenarioAdded, this);

    },

    onExit: function() {
        if (this.dataSourceTreeTableContextMenu) {
            this.dataSourceTreeTableContextMenu.destroy();
        }

        sas.eventBus.unsubscribe(HprConstants.CHANNEL_APP_SETTINGS, HprConstants.EVENT_DISPLAY_HORIZONS_CHANGED, this._onDisplayHorizonsChange, this);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_APP_SETTINGS, HprConstants.EVENT_SHOW_LABELS_CHANGED, this._onShowLabelsChange, this);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_DATA_ITEM_COUNT_UPDATED, this._onDataItemCountUpdate, this);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_DELEGATE_SELECTED_CONTROL_CHANGED, this._onDelegateSelectedControlChanged, this);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_SCENARIO_UPDATED, this._onScenarioUpdated, this);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_SCENARIO_DELETED, this._onScenarioDeleted, this);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_SCENARIO_ADDED, this._onScenarioAdded, this);

    },

    onBeforeRendering: function () {
        // This logic replaces 'onBeforeShow' method which VA DataPanel.controller used; They added a hook
        // to their controller so 'onBeforeShow' is called once before 'onBeforeRendering' & 'onAfterRendering'
        if (!this._initialized) {
            var ctrl = this;
            var view = ctrl.getView();

            // Build the model for the Multiple Data Source drop-down
            ctrl.buildDataSourceSelect(true);

            // Ensure the DataPaneModelManager is in an initial state
            DataPaneModelManager.initDataPaneModel(true);

            var deferred = DataSourceCacheManager.getDataSource(DataPaneModelManager.getSelectedDataSourceId());
            deferred.done(function(dataSource) {

                // Pass the loaded DataSource to the DataPaneModelManager so it can build the model
                DataPaneModelManager.setSelectedDataSource(dataSource);

                // Retrieve the tree table model from the DataPaneModelManager and place it on the treeTable
                var treeTableModel = DataPaneModelManager.getTreeTableModel();
                var oModel = new sap.ui.model.json.JSONModel(treeTableModel);
                view.dataSourceTreeTable.setModel(oModel);
                view.dataSourceTreeTable.bindRows({ path: "/", parameters: { arrayNames: ['children'] } });

                // Expand Class Variables folder node by default
                // S1433189 - Request by MIP folks that Class Variable node is not expanded by default
                //ctrl.DataPaneUtil.expandClassVariablesFolder(ctrl);

                // TODO: Expanded nodes is currently not saved when saving an Exploration; If that code is added back, we will probably
                // have to update this code to account for the new way of doing selection on the TreeTable and HprDataPanelTreeTableSelectionModel
                // Restore the state of the TreeTable by looping through the array of saved expanded tree node ids and locating the
                // corresponding tree node via the TreeTable model; We could not use the index of the row because the row indexes
                // change when a node is expanded or collapsed, so we key off an 'id' for the node
                //ctrl.HPRiskModelManager.resetDataPaneExpandedNodeIds(); // Make sure our array is cleared out
                // TODO: Not too sure about how this code should/could work it would need to go to the DataPaneModelManager to be supported.
                /*DataPaneModelManager.resetExpandedNodeIds();
                var dataSourceTreeTableExpandedNodeIdsString = "/expandedNodeIds";
                var expandedNodes = view.getModel().getProperty(dataSourceTreeTableExpandedNodeIdsString);
                if (expandedNodes && expandedNodes.length > 0) {
                    // Sadly we have to search for each 'id' in the TreeTable model and expand it if we find it
                    var rowBinding = view.dataSourceTreeTable.getBinding("rows");
                    var treeModel = view.dataSourceTreeTable.getModel();
                    for (var i = 0; i < expandedNodes.length; ++i) {
                        // Loop through the rows looking for the 'id'
                        for (var j = 0; j < rowBinding.getLength(); ++j) {
                            var context = view.dataSourceTreeTable.getContextByIndex(j);
                            var nodePath = context.getPath();
                            var node = treeModel.getProperty(nodePath);
                            // If we find the 'id', expand the node and update our array to keep it in-sync
                            if (node && node.id === expandedNodes[i]) {
                                view.dataSourceTreeTable.expand(j);
                                DataPaneModelManager.updateExpandedNodeIds(expandedNodes[i], true);
                                break;
                            }
                        }
                    }
                }*/

                ctrl._initialized = true;
            });
        }
    },

    onAfterRendering: function () {
        this.initialDataPaneRenderDone = true;
    },

    /**
     * Reset the DataPane by rebuilding the DataSource drop-down, initializing the DataPane model, and rebuilding the TreeTable model
     * @param isOpeningExploration Boolean which signals whether we are opening an Exploration
     */
    reset: function(isOpeningExploration) {
        var self = this;

        // Rebuild the Multiple Data Source drop-down and the TreeTable because the cube might have changed
        self.buildDataSourceSelect(false);

        // Make sure the DataPaneModelManager is ready to build a new tree table
        DataPaneModelManager.initDataPaneModel(isOpeningExploration);

        // Retrieve the selected DataSource and rebuild the TreeTable model
        var deferred = DataSourceCacheManager.getDataSource(DataPaneModelManager.getSelectedDataSourceId());
        deferred.done(function(dataSource) {

            var view = self.getView();
            DataPaneModelManager.setSelectedDataSource(dataSource);

            var newModelData = DataPaneModelManager.getTreeTableModel();
            var oModel = new sap.ui.model.json.JSONModel(newModelData);
            view.dataSourceTreeTable.setModel(oModel);
            view.dataSourceTreeTable.bindRows({ path: "/", parameters: { arrayNames: ['children'] } }); // Note: May not need this

            // Expand the Class Variables folder node after resetting the Data pane
            // S1433189 - Request by MIP folks that Class Variable node is not expanded by default
            //self.DataPaneUtil.expandClassVariablesFolder(self);

            // Reset the cached selected rows any time this component is rebuilt
            self.DataPaneUtil.getDataPaneTreeTableSelectionModel().clearSelectedRows();
        });
    },

    buildDataSourceSelect: function(addButton) {
        var ctrl = this;
        var view = ctrl.getView();

        // Gather the DataSourceParam objects that the cache currently knows about so we can populate the drop-down
        var dataSourceParams = DataSourceCacheManager.getDataSourceParams();

        // Define the model for the drop-down
        var oData = { dataSources: dataSourceParams };
        var oModel = new sap.ui.model.json.JSONModel(oData);
        view.dataSourceSelect.setModel(oModel);
        
        // Find the currently selected Data Source and set the drop-down to it's id (key corresponds to 'realDataSourceId')
        view.dataSourceSelect.setSelectedKey(DataSourceCacheManager.getSelectedDataSourceId());

        // If the Add DataSource button should be added
        if (addButton) {
            var actionSelectAddDSButtonId = "ActionSelect-addDataSourceButton";

            // Clean-up previous buttons
            if( sap.ui.getCore().byId(actionSelectAddDSButtonId) ) {
                sap.ui.getCore().byId(actionSelectAddDSButtonId).destroy();
            }

            view.dataSourceSelect.addButton(
                new sas.hc.m.Button(actionSelectAddDSButtonId, {
                    text: ctrl.HprUtil.getBundle().getText('DataPane.add.data.source.button.txt'),
                    icon: sas.icons.HC.ADD,
                    design: sas.hc.m.Button.Design.SmallNeutral,
                    press: $.proxy(ctrl._onAddDataSourceButtonPress, ctrl)
                }).addStyleClass("hprAddDataSourceButton")
            );
        }
    },

    dataSourceSelectChange: function(evt) {
        var ctrl = this;
        var view = ctrl.getView();
        var source = evt.getSource();
        var selectedItem = source.getSelectedItem();
        var previousSelectedDataSourceId = DataSourceCacheManager.getSelectedDataSourceId();

        // In some cases, we don't want to clear the selected panels in the RiskPanelsView
        var skipClearSelectedPanels = evt.getParameter("skipClearSelectedPanels");

        if (source && selectedItem) {
            var contextPath = selectedItem.getBindingContext().getPath();
            var selectedDataSource = source.getModel().getProperty(contextPath);

            // Set the DataPane view to busy while we load the new DataSource
            view.setBusyIndicatorDelay(0);
            view.setBusy(true);

            // Use the DataSourceCacheManager to load the new DataSource
            var deferred = DataSourceCacheManager.getDataSource(selectedDataSource.realDataSourceId);
            deferred.done(function(newDataSource) {
                // After the newly selected DataSource loads, we want to stash away the TreeTable model from the previous DataSource
                // so we will be able to keep the order of the Data Items if the user has dragged them around, as well as, label changes
                var previousSelectedDataSourceTreeTableModelData = DataPaneModelManager.toJSON();
                var previousSelectedDataSourceParam = DataSourceCacheManager.getSelectedDataSourceParam();
                previousSelectedDataSourceParam.addDataSourceTreeTableModelObject(previousSelectedDataSourceId, previousSelectedDataSourceTreeTableModelData);

                // Set the necessary properties and models on the Data Pane using the newly selected Data Source
                ctrl.setDataPaneDataSource(newDataSource, skipClearSelectedPanels);
            });
            deferred.fail(function() {
                // We get here when a data source fails to load; Revert the select component back to the previously selected key
                view.dataSourceSelect.setSelectedKey(previousSelectedDataSourceId);

                // Display the error dialog with a meaningful message
                ctrl.HprUtil.displayGenericErrorDialog(ctrl.HprUtil.getBundle().getText('DataPane.data.source.load.error.fmt', selectedItem.getProperty('text')));

                view.setBusy(false); // Revert the view back
            });
        }
    },

    /**
     * Sets the selected data source on the data pane.
     * @param dataSource
     * @param skipClearSelectedPanels
     */
    setDataPaneDataSource: function(dataSource, skipClearSelectedPanels) {
        var ctrl = this;
        var view = ctrl.getView();

        var previousSelectedDataSourceId = DataSourceCacheManager.getSelectedDataSourceId();
        if (previousSelectedDataSourceId === dataSource.getId()) {
            view.setBusy(false); // Make sure the view is not in busy state
            return; // no change, so no-op
        }

        // We want the TreeTable to reflect the new cubeData we just loaded, so set the appropriate TreeTable model
        // and refresh.  Note: This works in both cases of working with a New Exploration and Saved Exploration when
        // the TreeTable model uses 'children' as the arrayNames
        view.getModel().setProperty(ctrl.DATA_SOURCE_TREE_TABLE_MODEL_PROPERTY_PATH, dataSource);
        view.getModel().refresh();

        // Set the selected DataSource id on the cache to the new DataSource
        DataSourceCacheManager.setSelectedDataSourceId(dataSource.getId());

        ctrl.reset(false); // Reset the DataPane controller so the drop-down and TreeTable will rebuild

        // Clear the search field and reset the 'filteredModel' property for the selection model
        view.searchField.setValue("");
        ctrl.DataPaneUtil.getDataPaneTreeTableSelectionModel().resetFilteredModel();

        // Skip the call to clear the selected panels in some cases
        if (!!!skipClearSelectedPanels) {
            ctrl.HprUtil.getSelectedRiskPanelsView().clearSelectedPanels(); // Clear the selected RiskPanelsView
        }

        view.setBusy(false); // Hide the busy indicator

        // Update the dirty state of the Exploration view
        sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {
            dirty: true
        });
    },

    dataSourceTreeTableRowSelectionChange: function(evt) {
        var ctrl = this;

        // no-op if we are in the midst of a selection
        var dataPaneTreeTableSelectionModel = ctrl.DataPaneUtil.getDataPaneTreeTableSelectionModel();
        if (!dataPaneTreeTableSelectionModel.getSelecting()) {
            return;
        }

        var view = ctrl.getView();
        var tree = evt.getSource();
        var rowIndices = evt.getParameter("rowIndices");

        ctrl.logger.debug("----Start dataSourceTreeTableRowSelectionChange----");

        var eventIndicesSelected = [];
        var eventIndicesDeselected = [];
        if (rowIndices && rowIndices.length > 0) {
            _.each(rowIndices, function (index) {
                tree.isIndexSelected(index) ? eventIndicesSelected.push(index) : eventIndicesDeselected.push(index);
                ctrl.logger.debug("Row " + index + " is" + (tree.isIndexSelected(index) ? " " : " NOT ") + "selected...");
            });
            ctrl.logger.debug("Selected indices: " + eventIndicesSelected);
            ctrl.logger.debug("Deselected indices: " + eventIndicesDeselected);
        }

        var selectedRowIndices;
        if (eventIndicesSelected && eventIndicesSelected.length > 0) {
            selectedRowIndices = dataPaneTreeTableSelectionModel.selectRowsByIndex(eventIndicesSelected);
        }

        var deselectedRowIndices;
        if (eventIndicesDeselected && eventIndicesDeselected.length > 0) {
            deselectedRowIndices = dataPaneTreeTableSelectionModel.deselectRowsByIndex(eventIndicesDeselected);
        }

        var invalidSelectionIndices = _.difference(eventIndicesSelected, selectedRowIndices);
        _.each(invalidSelectionIndices, function(index) {
            tree.removeSelectionInterval(index, index);
        });

        // get the selected row paths
        var selectedRowPaths = dataPaneTreeTableSelectionModel.getSelectedRows();

        // debug logging
        ctrl.logger.debug("Selected row paths (" + selectedRowPaths.length + "):");
        _.each(selectedRowPaths, function(selectedRowPath) {
            ctrl.logger.debug("    " + selectedRowPath);
        });

        // update the view toolbar for the latest selections
        var treeModel = view.dataSourceTreeTable.getModel();
        var rowObjects = [];
        _.each(selectedRowPaths, function(path) {
            rowObjects.push(treeModel.getProperty(path));
        });
        var selectedDataItems = [];
        _.each(rowObjects, function(rowObject) {
            selectedDataItems.push(rowObject.dataItem);
        });
        view.updateDataPaneToolbarForSelections(selectedDataItems);

        // we can now refresh the tree with the changes
        tree.getModel().refresh(true);

        ctrl.logger.debug("----End dataSourceTreeTableRowSelectionChange----");
    },

    dataSourceTreeTableToggleOpenState: function(evt) {
        var controller = this;
        var view = controller.getView();

        var rowContext = evt.getParameter("rowContext");
        if (rowContext) {
            var selectedRowObject = rowContext.getObject();
            // Case where row is being expanded
            if (evt.getParameter("expanded")) {
                var expandedRowPath = rowContext.sPath;
                var treeTableSelectionModel = controller.DataPaneUtil.getDataPaneTreeTableSelectionModel();
                var selectedRowPaths = treeTableSelectionModel.getSelectedRows();

                // Get the row paths that are children of the row being expanded
                var filteredRowPaths = _.filter(selectedRowPaths, function(rowPath){
                    return rowPath.startsWith(expandedRowPath);
                });

                // Build a list of absolute indexes for the selected rows who are children of the row being expanded
                var selectedRowIndex = evt.getParameter("rowIndex");
                var indexes = _.map(filteredRowPaths, function(rowPath) {
                    var index = rowPath.lastIndexOf('/');
                    // Here is the magic math: (relative index of selected row + index of the row being expanded + 1)
                    return parseInt(rowPath.substring(index+1)) + selectedRowIndex + 1;
                });

                // We have delay selecting the rows on the TreeTable until it has fully rendered, so
                // this is how we do it.
                setTimeout(function() {
                    // Turn off responding to row selection events
                    treeTableSelectionModel.setSelecting(false);

                    // Add the selections to the TreeTable
                    _.forEach(indexes, function(index) {
                        view.dataSourceTreeTable.addSelectionInterval(index,index);
                    });

                    // When all that is done and the TreeTable has finished rerendering selections
                    // turn the row selection event listening back on
                    setTimeout(function() {
                        treeTableSelectionModel.setSelecting(true);
                    },1);
                },1);
            }

            if (selectedRowObject) {
                // Update the expanded node ids array so the saved TreeTable state can be restored
                DataPaneModelManager.updateExpandedNodeIds(selectedRowObject.id, evt.getParameter("expanded"));

                // Update the dirty state of the Exploration view
                sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {
                    dirty: true
                });
            }
        }
    },

    _showContextMenu: function(event, controller) {
        event.preventDefault();

        var self = this,
            menuProvider = new this.HprDataPaneContextMenuProvider(),
            oTableCellControl = event.getParameter("cellControl"),
            oTableCellDomRef = event.getParameter("cellDomRef");

        if (oTableCellControl === null) {
            return;
        }

        // Find the appropriate TreeTable node
        var oTableCellControlBindingContext = oTableCellControl.getBindingContext();
        if (oTableCellControlBindingContext) {
            var sTableCellControlPath = oTableCellControlBindingContext.getPath();
            var oTableTreeNode = oTableCellControl.getModel().getProperty(sTableCellControlPath);

            if (!this.dataSourceTreeTableContextMenu) {
                this.dataSourceTreeTableContextMenu = new sas.hc.ui.unified.Menu();
            }

            this.dataSourceTreeTableContextMenu.destroyItems();

            // Build an appropriate context menu for the TreeTable node
            menuProvider.buildContextMenu(this.dataSourceTreeTableContextMenu, oTableTreeNode, oTableCellDomRef, self);
        }
    },

    //*****************************************************************************************************
    //*****************************************************************************************************
    // EventBus helper functions
    //*****************************************************************************************************
    //*****************************************************************************************************
    /**
     * This method is ran when the Display Horizons preference is updated.  Determine if we need to update the Horizons TreeNode in the TreeTable.
     * @param channelId
     * @param eventId
     * @param data
     * @private
     */
    _onDisplayHorizonsChange: function(channelId, eventId, data) {
        // Don't change the Horizons for a Side By Side cube
        if (DataPaneModelManager.getSelectedDataSource().isSideBySide()) {
            return;
        }

        var ctrl = this;
        var view = ctrl.getView();

        // TODO: This code could be cleaned up a good bit using the DataPaneModelManager
        // Loop through the tree model to find the 'Horizons' top-level node
        // NOTE: Both New and Saved Explorations have same TreeTable model here
        var treeModel = view.dataSourceTreeTable.getModel();
        var treeModelData = treeModel.getData();
        for (var i = 0; i < treeModelData.children.length; ++i) {
            var node = treeModelData.children[i];
            if (node && node.id === HprConstants.HPR_DATA_PANE_HORIZONS_NODE_ID) {
                // We found the 'Horizons' top-level node, now switch the type of children nodes
                if (node.children && node.children.length > 0) {

                    // Switch out the children nodes for the Horizon parent node
                    DataPaneModelManager.setHorizonsCache(node.children,data.oldValue);

                    // Go find suitable Horizon nodes to display in the TreeTable
                    ctrl.DataPaneUtil.dataPaneTreeFindSuitableHorizonNodes(view.getModel(), data.newValue, node);

                    // Update the 'folderType' property of the folder node to reflect which type of Horizons are displayed
                    node.folderType = data.newValue === HprConstants.DISPLAY_HORIZONS_ABSOLUTE_VALUE ? ctrl.HORIZON : ctrl.RELATIVEHORIZON;

                    // Refresh the tree model so the new Horizon nodes will show up
                    treeModel.refresh();
                }
                break; // Done with updating the Horizon children nodes, break out of rowBinding for loop
            }
            else if (node && node.id === HprConstants.HPR_DATA_PANE_SCENARIOS_NODE_ID) {
                // should always have children, but JIC
                if (node.children && node.children.length > 0) {
                    DataPaneModelManager.updateStressCubeHorizonsForDisplayType(treeModelData);
                }

                // Refresh the tree model so the new Horizon nodes will show up
                treeModel.refresh();

                break;
            }
        }

        // Update the dirty state of the Exploration view
        sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {
            dirty: true
        });
    },

    /**
     * This method is called when a control is set to selected in the HPRiskPanelsViewDelegate.  The DataPane might have to react by switching the
     * selected DataSource and update the TreeTable.
     * @param channelId
     * @param eventId
     * @param data An Object that contains information about the control that was selected
     * @private
     */
    _onDelegateSelectedControlChanged: function(channelId, eventId, data) {
        var ctrl = this;
        var view = ctrl.getView();

        if (data && data.dataSourceId) {
            // See if we need to change the selection in the drop-down by comparing the currently selected DataSource id in the CacheManager
            // with the selected control's 'dataSourceId'
            var cacheManagerSelectedDataSourceId = DataSourceCacheManager.getSelectedDataSourceId();
            if (cacheManagerSelectedDataSourceId !== data.dataSourceId) {
                // If the id's are different, set the selected key to the new id and fire the change event; In this case,
                // we don't want the RiskPanelsView to clear the selected panels, so pass an event parameter to help with that
                view.dataSourceSelect.setSelectedKey(data.dataSourceId);
                view.dataSourceSelect.fireChange({skipClearSelectedPanels: true});
            }
        }
    },

    /**
     * This method is called when the 'count' property for a Data Item has been updated on another Panel/View and that change should
     * be reflected in the DataPane TreeTable.
     * @param channelId
     * @param eventId
     * @param data An Object that contains information about the Data Item which should be updated (dataSourceId, dataItemId, dataItemCount)
     * @private
     */
    _onDataItemCountUpdate: function(channelId, eventId, data) {
        var ctrl = this;
        var view = ctrl.getView();

        var dataSourceDeferred = DataSourceCacheManager.getDataSource(data.dataSourceId);
        dataSourceDeferred.done(function(dataSource) {
            var item = dataSource.getDataItemById(data.dataItemId);
            if (item) {
                // Update the count and updateCount properties for the Data Item
                item.setCount(data.dataItemCount);
                item.setUpdateCount(false);

                // Refresh the TreeTable model
                var dataSourceTreeTable = view.dataSourceTreeTable;
                var dataSourceTreeTableModel = dataSourceTreeTable.getModel();
                dataSourceTreeTableModel.refresh();

                // Create a temp object with information about the Data Item and the count; Store it in the correct DataSoureParam
                var dataSourceParam = DataSourceCacheManager.getDataSourceParam(data.dataSourceId);
                var dataItemChangedCountObject = {dataSourceId: data.dataSourceId, dataItem: item};
                dataSourceParam.addDataItemChangedCountObject(dataItemChangedCountObject);
            }
        });
    },
    
    _onShowLabelsChange: function(channelId, eventId, data) {
        var ctrl = this;
        var view = ctrl.getView();

        // Clear the search field and reset the 'filteredModel' property for the selection model
        view.searchField.setValue("");
        ctrl.DataPaneUtil.getDataPaneTreeTableSelectionModel().resetFilteredModel();

        // Clear the selections in the TreeTable
        ctrl.DataPaneUtil.clearSelections();

        // Brute force to rebuild the TreeTable model; The HprTreeTableRow class will
        // re-render the rows and pick up the correct displayLabels for the DataItems which
        // are bound to the model
        var dataSourceTreeTable = view.dataSourceTreeTable;
        var dataSourceTreeTableModel = dataSourceTreeTable.getModel();
        var newDataSourceTreeTableModel = new sap.ui.model.json.JSONModel(dataSourceTreeTableModel.oData);
        dataSourceTreeTable.setModel(newDataSourceTreeTableModel);
        dataSourceTreeTable.bindRows({ path: "/", parameters: { arrayNames: ['children'] } });

        // Update the dirty state of the Exploration view
        sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {
            dirty: true
        });
    },

    _onScenarioAdded: function(channelId, eventId, data) {
        var self = this;
        // The scenario has already been added to the cube, so just rebuild the nodes
        this.DataPaneUtil.updateScenarioNodes(DataPaneModelManager.getSelectedDataSource(), this);

        this.DataPaneUtil.revalidateAllComputedVariables(data.dataSourceId, this).done(function() {
            DataSourceCacheManager.getDataSource(data.dataSourceId).done(function(dataSource) {
                self.DisplayRulesManager.validateDisplayRules(dataSource);
            });
        });
    },

    _onScenarioUpdated: function(channelId, eventId, data) {
        var self = this;

        // The scenario has already been updated in the cube, so just rebuild the nodes
        this.DataPaneUtil.updateScenarioNodes(DataPaneModelManager.getSelectedDataSource(), this);

        this.DataPaneUtil.revalidateAllComputedVariables(data.dataSourceId, this).done(function() {
            DataSourceCacheManager.getDataSource(data.dataSourceId).done(function(dataSource) {
                self.DisplayRulesManager.validateDisplayRules(dataSource);
            });
        });
    },

    _onScenarioDeleted: function(channelId, eventId, data) {
        var self = this;

        // The scenario has already been removed from the cube, so just rebuild the nodes
        this.DataPaneUtil.updateScenarioNodes(DataPaneModelManager.getSelectedDataSource(), this);

        this.DataPaneUtil.revalidateAllComputedVariables(data.dataSourceId, this).done(function() {
            DataSourceCacheManager.getDataSource(data.dataSourceId).done(function(dataSource) {
                self.DisplayRulesManager.validateDisplayRules(dataSource);
            });
        });
    },

    //*****************************************************************************************************
    //*****************************************************************************************************
    // Overflow menu functions
    //*****************************************************************************************************
    //*****************************************************************************************************
    onTopOverflowButtonItemSelected: function(evt) {
        var ctrl = this;
        var view = ctrl.getView();
        var treeTable = view.dataSourceTreeTable;
        if (treeTable) {
            var selectedItem = evt.getSource();
            var selectedKey = selectedItem.data("key");
            ctrl._openNodes(selectedKey === "expand");
        }
    },

    _openNodes: function(openNodes) {
        var view = this.getView();
        if (openNodes) {
            // March through the row binding and expand top-level nodes
            var binding = view.dataSourceTreeTable.getBinding("rows");
            var model = view.dataSourceTreeTable.getModel();
            // NOTE: Binding length continues to grow as we expand more parent/top-level nodes
            for (var i = 0; i < binding.getLength(); ++i) {
                var context = view.dataSourceTreeTable.getContextByIndex(i);
                var nodePath = context.getPath();
                var node = model.getProperty(nodePath);
                if (node.children && node.children.length > 0) {
                    // Expand the node
                    view.dataSourceTreeTable.expand(i);

                    // Fire the toggleOpenState event when we expand the node so logic that selects children nodes is ran.
                    // The event expects the following event parameters: 'rowIndex', 'rowContext', and 'expanded'
                    view.dataSourceTreeTable.fireToggleOpenState({"rowIndex": i, "rowContext": context, "expanded": true});

                    // NOTE: Old code to remind us to keep track of expanded nodes (if that code is resurrected)
                    // Must keep the array of expanded tree node id's in-sync
                    // this.HPRiskModelManager.updateDataPaneExpandedNodeIds(node.id, true);
                }
            }
        }
        else {
            view.dataSourceTreeTable.collapseAll();

            // Just clear our array of expanded tree node id's
            DataPaneModelManager.resetExpandedNodeIds();
        }

        // Update the dirty state of the Exploration view
        sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {
            dirty: true
        });
    },


    _handleOpenDataSourceOverflowMenu: function(event) {
        var self = this;
        var source = event.oSource;
        var menu = self._createDataSourceOverflowMenu();
        var eDock = sap.ui.core.Popup.Dock;
        menu.open(
            false, // First item already highlighted
            source.getFocusDomRef(), // Dom reference which gets the focus back when the menu is closed
            eDock.BeginTop, // "Edge" of the menu (see sap.ui.core.Popup)
            eDock.BeginBottom, // "Edge" of the related opener position (see sap.ui.core.Popup)
            source.getDomRef() // Related opener position (see sap.ui.core.Popup)
        );
    },

    _createDataSourceOverflowMenu: function() {
        var self = this;
        var menu = new sas.hc.ui.unified.Menu();
        menu.addItem(self._createViewPropertiesOverflowMenuItem("viewProperties"));
        menu.addItem(self._createShowSASLogOverflowMenuItem("showLog"));
        menu.addItem(self._createUpdateCountOverflowMenuItem("updateCount"));

        // call method requires a 'this' as first argument followed by the rest of the arguments
        var createComputedVariableMenuItem = self.ComputedVariableUtil.createCreateCompVarMenuItem.call(self.ComputedVariableUtil, true);
        if (createComputedVariableMenuItem) {
            menu.addItem(createComputedVariableMenuItem);
        }

        menu.addItem(self._createReplaceDataSourceOverflowMenuItem("replaceDataSource"));
        menu.addItem(self._createRemoveDataSourceOverflowMenuItem("removeDataSource"));

        return menu;
    },

    _createReplaceDataSourceOverflowMenuItem: function(id) {
        var ctrl = this;
        var menuItem = new sas.hc.ui.unified.MenuItem({
            text: ctrl.HprUtil.getBundle().getText('DataPane.overflow.menu.replace.data.source.txt'),
            select: $.proxy(ctrl._onReplaceDataSource, ctrl),
            startsSection: true
        });
        return menuItem;
    },

    _onReplaceDataSource: function() {
        var ctrl = this;
        var explorationView = ctrl.ViewManager.getExplorationView();

        // Method which is ran after updating the Exploration with replace data source changes
        var completionHandler = function(updatedExploration) {
            // Use the new cube data and update the Data Pane; This will update the TreeTable and drop-down selector with the new cube data
            ctrl.reset(false); // Here is where the treetable's nodes are rebuilt.  At this point the datapane is good.

            // Update the Exploration model and Exploration analysis with the changes
            ctrl.HPRiskModelManager.changeExplorationAnalysisObject(updatedExploration);

            // Force a reset on the Exploration view which will get everything back in shape for use
            explorationView.getController().reset(false);

            // Update the dirty state of the Exploration view
            sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {
                dirty: true
            });
        };

        var currentAnalysisObject = ctrl.HPRiskModelManager.generateExplorationAnalysisJSON(explorationView);

        // Using the currently selected DataSourceParams object, find the type.  This is the type
        // of DataSource we are going to try and replace the current one with
        var previouslySelectedDataSourceParam = ctrl.DataSourceCacheManager.getSelectedDataSourceParam();

        // Find matching DataSourceParam objects to the one we had selected; This allows us to find which ones we need to replace
        var dataSourceParamsToRemove = ctrl.DataSourceCacheManager.getMatchingDataSourceParamsUsingParams(previouslySelectedDataSourceParam);

        // In order to remove them, we need to build an array of 'realDataSourceId' ids from 'dataSourceParamsToRemove'
        var dataSourceParamsToRemoveIds = _.map(dataSourceParamsToRemove, function (dataSourceParam, key) {
            return dataSourceParam.realDataSourceId;
        });

        ctrl.DataSourceChangeManager.replaceDataSourceForExploration(currentAnalysisObject, dataSourceParamsToRemove[0].abstractDataSourceId, dataSourceParamsToRemove, true, this, explorationView, completionHandler);
    },

    _createRemoveDataSourceOverflowMenuItem: function(id) {
        var ctrl = this;

        // Get the count of the unique DataSourceParams so we know whether to enable menu item
        var uniqueDSParamsLength = DataSourceCacheManager.getCountOfUniqueDataSourceParams();

        var menuItem = new sas.hc.ui.unified.MenuItem({
            text: ctrl.HprUtil.getBundle().getText('DataPane.overflow.menu.remove.data.source.txt'),
            enabled: uniqueDSParamsLength > 1,
            select: $.proxy(ctrl._onRemoveDataSource, ctrl)
        });
        return menuItem;
    },

    _onRemoveDataSource: function() {
        var ctrl = this;

        // Collect the current analysis JSON for the Exploration so we can modify it with changes from the different components when
        // they do their remove data source
        var explorationView = ctrl.ViewManager.getExplorationView();
        var currentAnalysisObject = ctrl.HPRiskModelManager.generateExplorationAnalysisJSON(explorationView);

        // Find matching DataSourceParam objects to the one we had selected; This allows us to find which ones we need to remove
        var previouslySelectedDataSourceParam = DataSourceCacheManager.getSelectedDataSourceParam();
        var dataSourceParamsToRemove = DataSourceCacheManager.getMatchingDataSourceParamsUsingParams(previouslySelectedDataSourceParam);
        // In order to remove them, we need to build an array of 'realDataSourceId' ids from 'dataSourceParamsToRemove'
        var dataSourceParamsToRemoveIds = _.map(dataSourceParamsToRemove, function (dataSourceParam, key) {
            return dataSourceParam.realDataSourceId;
        });

        // Create a Remove button for the Warning dialog
        var removeButton = new sas.hc.m.Button({
            text: ctrl.HprUtil.getBundle().getText("DataPane.remove.data.source.confirmation.dialog.remove.button.txt"),
            press: function (evt) {
                ctrl._removeDataSourceImpl(explorationView, currentAnalysisObject, dataSourceParamsToRemoveIds);
            }
        });

        // Determine if any of the data source ids which are being removed are referenced by any panel in the analysis JSON, if so, a warning dialog will appear
        var areDataSourceIdsBeingRemovedReferenced = ctrl.DataSourceChanger.areDataSourceIdsBeingReferenced(currentAnalysisObject, dataSourceParamsToRemoveIds);
        if (areDataSourceIdsBeingRemovedReferenced) {
            var selectedDataSource = DataSourceCacheManager.getSelectedDataSource();

            // Create an id that will be used by the Remove DataSource confirmation dialog so it can be suppressed by the Global Settings
            var removeDataSourceConfirmationDialogId = "removeDataSourceConfirmationDialog";
            // Check the global settings to see if the dialog is being blocked from opening
            if (sas.hc.ui.core.settings.GlobalSettingsHelper.getSuppressedMessages().indexOf(removeDataSourceConfirmationDialogId) === -1) {
                // Dialog is not being blocked so construct a new instance to prompt the user
                var removeDataSourceConfirmationDialog = new sas.hc.m.MessageDialog({
                    id: removeDataSourceConfirmationDialogId,
                    type: sas.hc.m.MessageDialogType.Warning,
                    text: ctrl.HprUtil.getBundle().getText("DataPane.remove.data.source.confirmation.dialog.warning.fmt", [selectedDataSource.getCubeDisplayLabel()]),
                    title: ctrl.HprUtil.getBundle().getText("DataPane.remove.data.source.confirmation.dialog.title"),
                    includeDontShowCheckBox: true,
                    buttonActions: [
                        removeButton,
                        {
                            text: sas.hc.m.MessageDialogAction.Cancel,
                            initialFocus: true,
                            defaultButton: true
                        }
                    ],
                    afterClose: function(evt) {
                        this.destroy();
                    }
                });
                removeDataSourceConfirmationDialog.open();
            } else {
                // Dialog is blocked from opening, but we still need to remove the Data Source
                ctrl._removeDataSourceImpl(explorationView, currentAnalysisObject, dataSourceParamsToRemoveIds);
            }
        } else {
            // Data Sources are not referenced so we don't need a confirmation dialog, but we still need to remove them
            ctrl._removeDataSourceImpl(explorationView, currentAnalysisObject, dataSourceParamsToRemoveIds);
        }
    },

    /**
     * This method actually does the removing of a DataSource.
     * @param explorationView - The exploration view object
     * @param currentAnalysisObject - The analysis/exploration as it stands right now
     * @param dataSourceParamsToRemoveIds - The list of DataSourceParams ids which are going to be removed
     * @private
     */
    _removeDataSourceImpl: function(explorationView, currentAnalysisObject, dataSourceParamsToRemoveIds) {
        var ctrl = this;
        var view = ctrl.getView();

        // Set the Exploration view to busy because a lot may happen when we remove a Data Source
        explorationView.setBusyIndicatorDelay(0);
        explorationView.setBusy(true);

        // Keep track of the previously selected Data Source id in case things fail
        var previousSelectedDataSourceId = DataSourceCacheManager.getSelectedDataSourceId();

        // Create two arrays which keep track of the ids and labels from the sorted items in the Data pane drop-down representing DataSourceParam objects
        var dataSourceSelectItemIds = [];
        var dataSourceSelectItemLabels = [];
        _.each(view.dataSourceSelect.getItems(), function(dataSourceSelectItem) {  // Items will be sorted like they show in UI
            dataSourceSelectItemIds.push(dataSourceSelectItem.getProperty("key"));
            dataSourceSelectItemLabels.push(dataSourceSelectItem.getProperty("text"));
        });

        // Find the first DataSourceParam object id that will not be deleted when the data sources are removed
        var idsNotToDeleted = _.difference(dataSourceSelectItemIds, dataSourceParamsToRemoveIds);
        var firstDSIdNotToBeDeleted = (idsNotToDeleted && idsNotToDeleted.length > 0) ? idsNotToDeleted[0] : undefined;
        if (firstDSIdNotToBeDeleted) {
            // After the removal of the data sources we are going to select the first remaining DataSourceParam object so we must try and load it first
            var promise = DataSourceCacheManager.getDataSource(firstDSIdNotToBeDeleted);
            promise.done(function(dataSource) {
                // Use the new cube data and update the Data Pane; This will update the TreeTable and drop-down selector with the new cube data
                view.getModel().setProperty(ctrl.DATA_SOURCE_TREE_TABLE_MODEL_PROPERTY_PATH, dataSource); // Temporarily put the data source into the model so when the refresh kicks in the treetable nodes will be rebuilt - UGH!!!
                view.getModel().refresh(); // Force a refresh on the model?

                // Set the selected DataSource id on the cache to the new DataSource
                DataSourceCacheManager.setSelectedDataSourceId(dataSource.getId());

                ctrl.reset(false); // Here is where the treetable's nodes are rebuilt.  At this point the datapane is good.

                var failureHandler = ctrl.HprUtil.createFailureHandler(explorationView, true, null); // Create a failure handler function in case something goes wrong

                // Method which is ran after updating the Exploration with removal of data sources
                var completionHandler = function(updatedExploration) {
                    // Update the Exploration model and Exploration analysis with the changes
                    ctrl.HPRiskModelManager.changeExplorationAnalysisObject(updatedExploration);

                    $.when(
                        // When the new data source is finished loading, release the old dataSourceParam objects from the mid-tier to free up memory
                        DataSourceCacheManager.releaseDataSourcesById(dataSourceParamsToRemoveIds)
                    ).done(function() {
                        // Force a reset on the Exploration view which will get everything back in shape for use
                        explorationView.getController().reset(false);

                        // Remove busy state from Exploration view
                        explorationView.setBusy(false);

                        // Update the dirty state of the Exploration view
                        sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {
                            dirty: true
                        });
                    }).fail(failureHandler);
                };

                // Here we update the exploration json with the data source removal
                ctrl.DataSourceChanger.updateExploration(false, true, currentAnalysisObject, dataSourceParamsToRemoveIds, null, completionHandler);
            });
            promise.fail(function() {
                // We get here when a data source fails to load; Revert the select component back to the previously selected key
                view.dataSourceSelect.setSelectedKey(previousSelectedDataSourceId);

                // Display the error dialog with a meaningful message using the label of the first item in the select dropdown
                ctrl.HprUtil.displayGenericErrorDialog(ctrl.HprUtil.getBundle().getText('DataPane.data.source.load.error.fmt', dataSourceSelectItemLabels[0]));

                explorationView.setBusy(false); // Revert the exploration view back
            });
        }
    },

    _createViewPropertiesOverflowMenuItem: function(id) {
        var ctrl = this;
        var menuItem = new sas.hc.ui.unified.MenuItem({
            text: ctrl.HprUtil.getBundle().getText('DataPane.overflow.menu.view.properties.txt'),
            select: $.proxy(this._onViewProperties, this)
        });
        return menuItem;
    },

    _onViewProperties: function(evt) {
        var ctrl = this;

        // Gather the cube data and use the 'dataSource.info' property to build the model for the View Data Source Properties dialog
        var cubeData = DataPaneModelManager.getSelectedDataSource();
        if (cubeData && cubeData.dataSource && cubeData.dataSource.info) {
            var dialogModel = new sap.ui.model.json.JSONModel();

            // Process 'cubeData.dataSource.info' into the form we need for the dialog
            var dataSourcePropertiesMainCube = $.extend(true, {}, cubeData.dataSource.info);

            // Over-write what was in the 'cubeData.dataSource.info.label' property here (always seems empty anyway) with displayable label for the cube
            dataSourcePropertiesMainCube.label = cubeData.getCubeDisplayLabel();

            dataSourcePropertiesMainCube.isStress = cubeData.isStressCube(); // Is the main cube a stress cube
            dataSourcePropertiesMainCube.cubePath = cubeData.getCubePath(); // Store the path of the main cube
            dialogModel.setProperty("/dataSourcePropertiesMainCube", dataSourcePropertiesMainCube);

            var subCubes = [];
            if (dataSourcePropertiesMainCube.numSubCubes > 0) {
                for (var i=0;i < dataSourcePropertiesMainCube.subCubes.length;i++) {
                    var subCube = $.extend(true, {}, dataSourcePropertiesMainCube.subCubes[i]);

                    // Convert the following to String values so we can filter on the columns in the dialog
                    subCube.index = String(i+1); // Create a new property to track index of sub-cube
                    subCube.positions = String(subCube.positions);
                    subCube.numSamples = String(subCube.numSamples);

                    // For the column sort to work correctly, have to create these as Numbers and use for 'sortProperty' on column
                    subCube.positionsAsNumber = Number(subCube.positions);
                    subCube.numSamplesAsNumber = Number(subCube.numSamples);

                    // Massage 'analysisType' property to something more readable
                    var analysisTypeResourceString = (_.isUndefined(subCube.analysisType) || _.isNull(subCube.analysisType)) ? "none" : subCube.analysisType;
                    subCube.analysisTypeDisplayString = ctrl.HprUtil.getBundle().getText("DataPane.data.source.properties.analysis.type." + analysisTypeResourceString + ".txt");

                    // Massage dates to correct formats and to String values so we can filter on the columns in the dialog
                    subCube.runDateDisplayString = subCube.runDate ? ctrl.HprUtil.formatISODateString(subCube.runDate) : ""; // Comes in as "2015-01-25" and we need "25Jan2015"
                    subCube.sasCreationDateDisplayString = subCube.sasCreationDate ? ctrl.HprUtil.formatDateTime(new Date(subCube.sasCreationDate)) : "";
                    subCube.sasAlertDateDisplayString = subCube.sasAlertDate ? ctrl.HprUtil.formatDateTime(new Date(subCube.sasAlertDate)) : "";
                    subCube.sasExpirationDateDisplayString = subCube.sasExpirationDate ? ctrl.HprUtil.formatDateTime(new Date(subCube.sasExpirationDate)) : "";

                    subCubes.push(subCube);
                }
            }
            dialogModel.setProperty("/dataSourcePropertiesSubCubesPath", "/dataSourcePropertiesSubCubes");
            dialogModel.setProperty("/dataSourcePropertiesSubCubes", subCubes);

            // Open the View Data Source Properties Dialog
            var viewPropertiesDialog = sap.ui.jsfragment("sas.analytics.riskhp.fragments.DataSourcePropertiesDialog", this);
            viewPropertiesDialog.setModel(dialogModel);
            viewPropertiesDialog.open();
        }
    },

    _createShowSASLogOverflowMenuItem: function(id) {
        var ctrl = this;
        var menuItem = new sas.hc.ui.unified.MenuItem({
            text: ctrl.HprUtil.getBundle().getText("DataPane.overflow.menu.show.sas.log.txt"),
            select: $.proxy(this._onShowSASLog, this)
        });
        return menuItem;
    },

    _onShowSASLog: function(evt) {
        var ctrl = this;

        // Set the Exploration view to busy
        var explorationView = ctrl.ViewManager.getExplorationView();
        explorationView.setBusyIndicatorDelay(0);
        explorationView.setBusy(true);

        // Gather the cube data
        var cubeData = DataPaneModelManager.getSelectedDataSource();

        $.when (
            // Load the SAS Log for the selected Data Source
            ctrl.HprAPI.getSASLog(cubeData.getId())
        ).done(function(logData) {
            if (logData && logData.length > 0) {
                ctrl._openSASLogDialog(cubeData, logData);
            } else {
                // If there is no log data, display an informational MessageDialog instead
                sas.hc.m.MessageDialog.information({
                    text: ctrl.HprUtil.getBundle().getText("DataPane.data.source.show.sas.log.dialog.no.log.txt"),
                    title: ctrl.HprUtil.getBundle().getText("DataPane.data.source.show.sas.log.dialog.title")
                });
            }
        }).fail(function() {
            ctrl.HprUtil.displayGenericErrorDialog();
        }).always(function() {
            explorationView.setBusy(false);
        });
    },

    _openSASLogDialog: function(cubeData, logData) {
        // Gather the cube data and use the 'dataSource.info' property to build the model for the SAS Log dialog
        if (cubeData && cubeData.dataSource && cubeData.dataSource.info) {
            var dialogModel = new sap.ui.model.json.JSONModel();

            // Process 'cubeData.dataSource.info' into the form we need for the dialog
            var dataSourceSASLogMainCube = $.extend(true, {}, cubeData.dataSource.info);
            dataSourceSASLogMainCube.isStress = cubeData.isStressCube(); // Is the main cube a stress cube
            dialogModel.setProperty("/dataSourceSASLogMainCube", dataSourceSASLogMainCube);
            dialogModel.setProperty("/dataSourceSASLogData", logData);

            // Open the SASLogDialog
            var sasLogDialog = sap.ui.jsfragment("sas.analytics.riskhp.fragments.SASLogDialog", this);
            sasLogDialog.setModel(dialogModel);
            sasLogDialog.open();
        }
    },

    _createUpdateCountOverflowMenuItem: function(id) {
        var ctrl = this;

        var view = ctrl.getView();
        var dataSourceTreeTable = view.dataSourceTreeTable;
        var dataSourceTreeTableModel = dataSourceTreeTable.getModel().oData;
        var menuItemEnabled = DataPaneModelManager.shouldClassVariableCountsBeUpdated(dataSourceTreeTableModel);

        var menuItem = new sas.hc.ui.unified.MenuItem({
            text: ctrl.HprUtil.getBundle().getText('DataPane.overflow.menu.update.count.txt'),
            enabled: menuItemEnabled,
            select: $.proxy(this._onUpdateCount, this)
        });
        return menuItem;
    },

    /**
     * Update the 'count' property for the ClassVariable DataItems that need it.  This method marches through the ClassVariable
     * TreeNodes and determines which nodes need to have their 'count' updated and applies it to the corresponding DataItem.
     * @param evt
     * @private
     */
    _onUpdateCount: function(evt) {
        var ctrl = this;
        var classVariableNodeChildren = this.DataPaneUtil.getClassVariableNodeChildren(ctrl);
        var classVariableNodeChildrenToUpdate = [];

        // March through the ClassVariable nodes and determine if any need their 'count' property updated
        _.each(classVariableNodeChildren, function(child) {
            if (child.dataItem.getUpdateCount()) {
                var classVariableObject = {
                    "version": 1,
                    "id": child.dataItem.getId(),
                    "type": child.dataItem.getType(),
                    "countOnly": true
                };
                classVariableNodeChildrenToUpdate.push(classVariableObject);
            }
        });

        if (classVariableNodeChildrenToUpdate.length > 0) {
            ctrl._onUpdateCountHelper(classVariableNodeChildrenToUpdate);
        }

        evt.getSource().setEnabled(false); // Disable the Update Count menuitem
    },

    onDataItemUpdateCountSelect: function(aCVDataItems) {
        var ctrl = this;
        var classVariableDataItemsToUpdate = [];

        // March through the ClassVariable nodes and determine if any need their 'count' property updated
        _.each(aCVDataItems, function(cvDataItem) {
            if (cvDataItem.getUpdateCount()) {
                var classVariableObject = {
                    "version": 1,
                    "id": cvDataItem.getId(),
                    "type": cvDataItem.getType(),
                    "countOnly": true
                };
                classVariableDataItemsToUpdate.push(classVariableObject);
            }
        });

        if (classVariableDataItemsToUpdate.length > 0) {
            ctrl._onUpdateCountHelper(classVariableDataItemsToUpdate);
        }
    },

    _onUpdateCountHelper: function(classVariableObjectsToUpdate) {
        var ctrl = this;
        var view = ctrl.getView();

        // Set the view to busy state since some of the count updates might take a while
        view.setBusyIndicatorDelay(0);
        view.setBusy(true);

        var dataSourceId = DataSourceCacheManager.getSelectedDataSourceId();
        // Use the dataSourceId to call the HprAPT.updateCubeValues to update the counts
        ctrl.HprAPI.updateCubeValues(dataSourceId, classVariableObjectsToUpdate)
            .done(function(results) {
                // If there are results returned, process them
                if (results && results.items && results.count > 0) {
                    var selectedDataSourceParam = DataSourceCacheManager.getSelectedDataSourceParam();
                    var selectedDataSource = DataSourceCacheManager.getSelectedDataSource();

                    _.each(results.items, function (item) {
                        // Update data item and refresh Tree model so changes will appear
                        var dataItem = selectedDataSource.getDataItemById(item.id);
                        dataItem.setCount(item.count);
                        dataItem.setUpdateCount(false);

                        // Create a temp object to pass information to the selected DataSourceParam to keep track of
                        // a Data Item where the count was updated
                        var data = {dataSourceId: dataSourceId, dataItem: dataItem};
                        selectedDataSourceParam.addDataItemChangedCountObject(data);
                    });
                    view.dataSourceTreeTable.getModel().refresh();
                }
                view.setBusy(false); // Revert the view back
            })
            .fail(function() {
                view.setBusy(false); // Revert the view back
                ctrl.HprUtil.displayGenericErrorDialog();
            })
    },

    onDataItemRenameHelper: function(dataItem) {
        var ctrl = this;

        if (dataItem) {
            // Open the DataItemRenameDialog
            var dialogModel = new sap.ui.model.json.JSONModel();
            dialogModel.setProperty("/selectedDataItem", dataItem);
            var dataItemRenameDialog = sap.ui.jsfragment("sas.analytics.riskhp.fragments.DataItemRenameDialog", ctrl);
            dataItemRenameDialog.setModel(dialogModel);
            dataItemRenameDialog.open();
        }
    },

    /**
     * Method which moves one or more Data Items to the correct position in the current folder on the Data Pane Tree Table.
     * To explain further, the Data Item(s) are moved within the folder there are already in to the correct position determined
     * by the 'moveToPosition' parameter.
     * @param moveToPosition A constant string which specifies which direction and how far to move the Data Item(s)
     * @param targetRowPath (optional) A string which specifies the row path the user right-clicked on or undefined
     * if multiple selected Data Items
     */
    onDataItemContextMenuMove: function(moveToPosition, targetRowPath) {
        var ctrl = this;
        var view = ctrl.getView();

        // Get the Data Pane TreeTable and model
        var dataPaneTable = view.getDataSourceTreeTable();
        var treeModel = dataPaneTable.getModel();

        // Populate the selected rows from the DataPaneTreeTableSelectionModel if the 'targetRowPath' is not specified or undefined
        var selectedRows = [];
        if (!targetRowPath) {
            var dataPaneTreeTableSelectionModel = ctrl.DataPaneUtil.getDataPaneTreeTableSelectionModel();
            selectedRows = dataPaneTreeTableSelectionModel.getSelectedRows();
            targetRowPath = selectedRows[0]; // Already a path string
        } else {
            selectedRows.push(targetRowPath);
        }

        // Use the targetRowPath to get the path of its parent's children (i.e. "/children/4/children/0/children")
        var parentsChildrenPathOfTargetRowPath = ctrl.DataPaneUtil.getParentsChildrenPathFromPath(targetRowPath);

        // March through the children and save their expansion state so we can re-apply after nodes have been moved to give the
        // illusion that their state has been maintained after moving
        var expansionStateObject = ctrl.DataPaneUtil.createChildrenPathExpansionStateObject(parentsChildrenPathOfTargetRowPath);

        // itemsToMove will hold an array of the objects from the table model that need to be moved
        var itemsToMove = [];

        var sortedSelectedRows = ctrl.DataPaneUtil.getSortedSelectedRowPaths(selectedRows); // Need to sort the selected rows
        // Accumulate the objects from the TreeTable's model that will be moved in their sorted order.
        _.each(sortedSelectedRows, function(selectedItemPath){
            var item = treeModel.getProperty(selectedItemPath);
            if (item !== null){
                itemsToMove.push(item);
            }
        });

        // Make sure we have something to move
        if (itemsToMove.length > 0) {
            // Remove the rows we are going to move, this will leave null "placeholders" in the model
            _.each(sortedSelectedRows, function(selectedItemPath){
                treeModel.setProperty(selectedItemPath, null);
            });

            // Get the current state of the list the target row is sitting in.  The idea is we'll get
            // this array correct and put it back into the model (after it is fixed)
            var parentsChildrenOfTargetRowArray = treeModel.getProperty(parentsChildrenPathOfTargetRowPath);

            // Helper function that increases the index upward (i.e. 0, 1, 2, etc.) until a desirable index is found;
            // The idea here is to move the data item around another data item of a different type
            var adjustMoveToIndexUpwardIfNeeded = function(iStartIndex, aChildrenArray, sDesiredType) {
                var adjustedIndex = iStartIndex;

                do {
                    var dataItem = null;
                    var item = aChildrenArray[adjustedIndex]; // Can be 'null' if it is item being moved
                    if (item) {
                        dataItem = item.dataItem;
                    }
                    // If the data item is not the desired type, increase the adjustedIndex to get around it
                    if (dataItem && dataItem.getType() !== sDesiredType) {
                        adjustedIndex++;
                    }
                } while (dataItem && dataItem.getType() !== sDesiredType && adjustedIndex < aChildrenArray.length);

                return adjustedIndex;
            };

            // Helper function that decreases the index (i.e. 4, 3, 2, 1, 0) until a desirable index is found;
            // The idea here is to move the data item around another data item of a different type
            var adjustMoveToIndexDownwardIfNeeded = function(iStartIndex, aChildrenArray, sDesiredType) {
                var adjustedIndex = iStartIndex;

                do {
                    var dataItem = null;
                    var item = aChildrenArray[adjustedIndex]; // Can be 'null' if it is item being moved
                    if (item) {
                        dataItem = item.dataItem;
                    }
                    // If the data item is not the desired type, decrease the adjustedIndex to get around it
                    if (dataItem && dataItem.getType() !== sDesiredType) {
                        adjustedIndex--;
                    }
                } while (dataItem && dataItem.getType() !== sDesiredType && adjustedIndex > 0);

                return adjustedIndex;
            };

            // Based on the "moveToPosition" parameter slice up the parentsChildrenOfTargetRowArray so we can move the data items around correctly
            var moveToIndex, startIndex = 0; // Default index values
            var desiredType = itemsToMove[0].dataItem.getType(); // The desired type of Data Item we are working with
            var head = [];
            var tail = [];
            switch(moveToPosition) {
                case "MOVETOTOP":
                    moveToIndex = adjustMoveToIndexUpwardIfNeeded(startIndex, parentsChildrenOfTargetRowArray, desiredType);
                    head = parentsChildrenOfTargetRowArray.slice(0, moveToIndex);
                    tail = parentsChildrenOfTargetRowArray.slice(moveToIndex, parentsChildrenOfTargetRowArray.length);
                    break;
                case "MOVEUP":
                    startIndex = parseInt(_.last(targetRowPath.split('/'))) - 1;
                    if (startIndex === 0) {
                        moveToIndex = adjustMoveToIndexUpwardIfNeeded(startIndex, parentsChildrenOfTargetRowArray, desiredType);
                    } else {
                        moveToIndex = adjustMoveToIndexDownwardIfNeeded(startIndex, parentsChildrenOfTargetRowArray, desiredType);
                    }
                    head = parentsChildrenOfTargetRowArray.slice(0, moveToIndex);
                    tail = parentsChildrenOfTargetRowArray.slice(moveToIndex, parentsChildrenOfTargetRowArray.length);
                    break;
                case "MOVETOBOTTOM":
                    startIndex = parentsChildrenOfTargetRowArray.length - 1;
                    moveToIndex = adjustMoveToIndexDownwardIfNeeded(startIndex, parentsChildrenOfTargetRowArray, desiredType);
                    head = parentsChildrenOfTargetRowArray.slice(0, moveToIndex + 1);
                    tail = parentsChildrenOfTargetRowArray.slice(moveToIndex + 1, parentsChildrenOfTargetRowArray.length);
                    break;
                case "MOVEDOWN":
                    startIndex = parseInt(_.last(targetRowPath.split('/'))) + 1;
                    if (startIndex === parentsChildrenOfTargetRowArray.length - 1) {
                        moveToIndex = adjustMoveToIndexDownwardIfNeeded(startIndex, parentsChildrenOfTargetRowArray, desiredType);
                    } else {
                        moveToIndex = adjustMoveToIndexUpwardIfNeeded(startIndex, parentsChildrenOfTargetRowArray, desiredType);
                    }
                    head = parentsChildrenOfTargetRowArray.slice(0, moveToIndex + 1);
                    tail = parentsChildrenOfTargetRowArray.slice(moveToIndex + 1, parentsChildrenOfTargetRowArray.length);
                    break;
            }

            // Merge everything back to one array
            var newArray = head.concat(itemsToMove, tail);

            // Clean up any nulls in the resultant array by filtering them out
            newArray = _.filter(newArray, function(item){
                return item !== null;
            });

            // Finally push the updated and cleaned up array back into the tableModel which will cause
            // a refresh on the tree table.
            treeModel.setProperty(parentsChildrenPathOfTargetRowPath, newArray);

            // Re-apply the expansion state so sub-folder will be expanded or collapsed like before the move
            ctrl.DataPaneUtil.applyExpansionStateObject(parentsChildrenPathOfTargetRowPath, expansionStateObject);

            // Clear the table's selections.  If this is not desired then we will need to run another
            // process to figure out what rows are now selected.  How we will set that, I have no
            // idea since the TreeTable doesn't seem to support that.  Maybe we could do it view jQuery?
            ctrl.DataPaneUtil.clearSelections(false); // Don't scroll the TreeTable to the top after a drop

            // Publish an event that will update the dirtyState of the Exploration view
            sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {
                dirty: true
            });
        }
    },

    _onAddDataSourceButtonPress: function(evt) {
        var ctrl = this;
        var view = ctrl.getView();

        view.dataSourceSelect.close();

        $.when (
            // Load the cubes
            ctrl.HprAPI.getCubes()
        ).done(function(cubes) {
            ctrl._openAddDataSourceDialog(cubes);
        }).fail(function() {
            ctrl.HprUtil.displayGenericErrorDialog();
        });
    },

    /**
     * Opens the ChooseDataSourceDialog in the Add Data Source mode which will add the different types of a data source to the Data Source Select component
     * @param cubes
     */
    _openAddDataSourceDialog: function(cubes) {
        var ctrl = this;

        // Open the ChooseDataSourceDialog with the list of cubes
        var dialogModel = new sap.ui.model.json.JSONModel();
        dialogModel.setProperty("/dataSourceItemsPath", "/dataSourceItems");
        dialogModel.setProperty("/dataSourceItems", cubes);
        dialogModel.setProperty("/dialogTitle", ctrl.HprUtil.getBundle().getText('ChooseDataSourceDialog.add.data.source.title.txt'));
        dialogModel.setProperty("/dialogMode", HprConstants.HPR_CHOOSEDATASOURCE_DIALOG_ADD_DATA_SOURCE_MODE);
        var chooseDataSourceDialog = sap.ui.jsfragment("sas.analytics.riskhp.fragments.ChooseDataSourceDialog", this);
        chooseDataSourceDialog.setModel(dialogModel);
        chooseDataSourceDialog.open();
    },

    /**
     * This method is called after the user chooses to Add a Data Source in the ChooseDataSourceDialog.  A DataSourceParams object
     * is created and the DataSourceDetails are gathered for it.  Next, the DataSourceCacheManager is updated with the new
     * DataSourceDetails and the new DataSourceDetails are added to the Data Source drop-down.
     * @param dataSourceObject The DataSource object
     */
    addDataSourceOnOpen: function(dataSourceObject) {
        var ctrl = this;

        // Create a DataSourceParams object that we can use to find details about it
        var dataSourceParams = ctrl.DataSourceParams.prototype.createRootParams(dataSourceObject);

        // Get the DataSource details so we can populate the Select component with different 'types'
        var promise = ctrl.HprAPI.getDataSourceDetails(dataSourceParams);
        promise.done(function(dataSourceDetails) {
            if (dataSourceDetails && dataSourceDetails.dataSources && dataSourceDetails.dataSources.length > 0) {
                // TODO: Remove this snippet of code when 'portfolio' data source will be used
                // Remove the 'portfolio' data source because it is not used yet
                dataSourceDetails.dataSources = _.filter(dataSourceDetails.dataSources, function (dataSource) {
                    return dataSource.type === 'price' || dataSource.type === 'stress';
                });

                // Have the DataSourceCacheManager update the DataSourceParams objects it tracks
                DataSourceCacheManager.updateDataSourceParams(dataSourceDetails);

                // Rebuild the Select component and its model
                ctrl.buildDataSourceSelect(false);

                // Based on the number of DataSources we are adding, output a useful message
                var messageToastOptionsObject = {
                    'my': sap.ui.core.Popup.Dock.CenterTop,
                    'at': sap.ui.core.Popup.Dock.CenterTop
                };
                if (dataSourceDetails.dataSources.length === 1) {
                    var dsDisplayLabel = ctrl.HprUtil.getCubeDisplayLabel(dataSourceDetails.dataSources[0].label, dataSourceDetails.name, dataSourceDetails.dataSources[0].type);
                    sas.hc.m.MessageToast.show(ctrl.HprUtil.getBundle().getText("ChooseDataSourceDialog.successful.datasource.add.fmt", [dsDisplayLabel]), messageToastOptionsObject);
                } else {
                    sas.hc.m.MessageToast.show(ctrl.HprUtil.getBundle().getText("ChooseDataSourceDialog.successful.multiple.datasources.add.fmt", [dataSourceDetails.dataSources.length]), messageToastOptionsObject);
                }

                // Update the dirty state of the Exploration view
                sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {
                    dirty: true
                });
            } else {
                // Display error dialog
                ctrl.HprUtil.displayGenericErrorDialog(ctrl.HprUtil.getBundle().getText('HPRisk.detailed.error.cube.information.failed.to.load.txt'));
            }
        });
    },

    clearSelectionsButtonPress: function(evt) {
        // Clear selections in Tree Table
        this.DataPaneUtil.clearSelections();
    },

    propertiesButtonPress: function(evt) {
        var ctrl = this;
        var view = ctrl.getView();

        // Use the DataPaneTreeSelectionModel to find if any rows are selected
        var treeTableSelectionModel = ctrl.DataPaneUtil.getDataPaneTreeTableSelectionModel();
        var selectedRowPaths = treeTableSelectionModel.getSelectedRows();
        if (selectedRowPaths.length === 1) {
            var treeModel = view.dataSourceTreeTable.getModel();
            var selectedRowObject = treeModel.getProperty(selectedRowPaths[0]);
            // Open the data item properties dialog for the one selected row
            if (selectedRowObject) {
                ctrl.openDataItemPropertiesDialogHelper(selectedRowObject.dataItem);
            }
        }
    },

    /**
     * Display the DataItemProperties Dialog for the specified Data Item
     * @param dataItem The Data Item to display properties for
     */
    openDataItemPropertiesDialogHelper: function(dataItem) {
        var ctrl = this;

        if (dataItem) {
            // Open the DataItemPropertiesDialog
            var dialogModel = new sap.ui.model.json.JSONModel();
            dialogModel.setProperty("/selectedDataItem", dataItem);
            var dataItemPropertiesDialog = sap.ui.jsfragment("sas.analytics.riskhp.fragments.DataItemPropertiesDialog", ctrl);
            dataItemPropertiesDialog.setModel(dialogModel);
            dataItemPropertiesDialog.open();
        }
    },

    //*****************************************************************************************************
    //*****************************************************************************************************
    // Search & Filter functions
    //*****************************************************************************************************
    //*****************************************************************************************************
    _onSearchLiveChange: function(evt) {
        var self = this;
        var searchValue = evt.getSource().getValue();
        var delay = (searchValue ? 300 : 0); // no delay if value is empty

        // Execute search after user stopped typing for 300ms
        clearTimeout(this.liveChangeTimer);
        if (delay) {
            this.liveChangeTimer = setTimeout(function () {
                self._filterDataSources(searchValue);
            }, delay);
        } else {
            self._filterDataSources(searchValue);
        }
    },

    _filterDataSources: function(searchValue) {
        var showLabels = this.ApplicationSettings.getShowLabels();

        if(searchValue && searchValue.length > 0) {
            // The DataPane TreeTable could be displaying labels or names of the DataItems, we
            // need to filter on the correct property
            var diLabelFilterPath = showLabels ? "dataItem/label" : "dataItem/name";

            this.dataItemLabelFilter = new sap.ui.model.Filter({
                path: diLabelFilterPath,
                operator: sap.ui.model.FilterOperator.Contains,
                value1: searchValue
            });
            this.labelFilter = new sap.ui.model.Filter({
                path: "lineage",
                operator: sap.ui.model.FilterOperator.Contains,
                value1: searchValue
            });
            this.searchFilter = new sap.ui.model.Filter({
                filters: [this.dataItemLabelFilter, this.labelFilter],
                and: false
            });
        } else {
            this.searchFilter = null;
        }

        this._applyFilters(searchValue, showLabels);
    },

    _applyFilters: function(searchValue, showLabels) {
        var ctrl = this;
        var view = ctrl.getView();
        var filters = [];

        if (this.searchFilter) {
            filters.push(this.searchFilter);
        }

        if (view.dataSourceTreeTable.isBound("rows")) {
            // apply the filters to the selection model
            this.DataPaneUtil.getDataPaneTreeTableSelectionModel().applyFilters(searchValue, showLabels);

            // filter the tree
            view.dataSourceTreeTable.getBinding("rows").filter(filters);
        }
    }
});