// (c) 2017, SAS Institute Inc.

jQuery.sap.require("sas.analytics.riskhp.views.datapane.DataPaneUtil");
jQuery.sap.require("sas.analytics.riskhp.views.datapane.DraggableRowDelegate");
jQuery.sap.require("sas.analytics.riskhp.views.datapane.DroppableRowDelegate");
jQuery.sap.require("sas.analytics.riskhp.views.datapane.HprTreeTableColumn");
jQuery.sap.require('sas.analytics.riskhp.views.datapane.HprTreeTableRow');
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.hc.m.Link");
jQuery.sap.require("sas.risk.common.layout.BarLayout");
jQuery.sap.require("sas.risk.common.layout.BarLayoutSpacer");

/**
 * A View that builds the Data Pane UI.
 * @name sas.analytics.riskhp.views.datapane.DataPane
 */
sap.ui.jsview('sas.analytics.riskhp.views.datapane.DataPane', (function() {

    return {
        DataPaneUtil: sas.analytics.riskhp.views.datapane.DataPaneUtil,
        HprUtil: sas.analytics.riskhp.utils.HprUtil,
        HprTreeTableColumn: sas.analytics.riskhp.views.datapane.HprTreeTableColumn,
        HprTreeTableRow: sas.analytics.riskhp.views.datapane.HprTreeTableRow,
        DraggableRowDelegate: sas.analytics.riskhp.views.datapane.DraggableRowDelegate,
        DroppableRowDelegate: sas.analytics.riskhp.views.datapane.DroppableRowDelegate,

        resourceBundle: sas.analytics.riskhp.utils.HprUtil.getBundle(),
        dataSourceTreeTable: null,
        dataPanePropertiesToolbar: null,
        dataSourceOverflowButton: null,

        logger: sas.log.getLogger("sas.analytics.riskhp.views.datapane.DataPane_view"),

        getControllerName: function() {
            return 'sas.analytics.riskhp.views.datapane.DataPane';
        },

        createContent: function(controller) {
            this._createDataSourceTreeTable(controller);
            this._createDataPanePropertiesToolbar(controller);

            return this._createView(controller); // Returns a BorderLayout
        },

        _createDataSourceTreeTable: function(controller) {
            var self = this;

            self.dataSourceTreeTable = new sas.hc.ui.table.TreeTable({
                expandFirstLevel: false,
                columnHeaderVisible: false,
                visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
                minAutoRowCount: 1,
                showTableOptions: false,
                editable: false,
                enableColumnReordering: false,
                enableColumnFreeze: false,
                selectionBehavior: sas.hc.ui.table.SelectionBehavior.RowOnly,
                selectionMode: sas.hc.ui.table.SelectionMode.MultiToggle,
                rowSelectionChange: $.proxy(controller.dataSourceTreeTableRowSelectionChange, controller),
                toggleOpenState: $.proxy(controller.dataSourceTreeTableToggleOpenState, controller),
                columns: [
                    new self.HprTreeTableColumn({
                        template: new self.HprTreeTableRow({})
                    })
                ]
            }).addStyleClass("treeTable").addStyleClass("dataSourceTreeTable");

            // Make the rows in the risk sources tree draggable
            // Note: addDelegate is a private function on the Element object.  It adds a delegate that listens to events
            // fired BY this element, NOT fired ON this element (which is done by addEventDelegate).
            self.dataSourceTreeTable.addDelegate(new self.DraggableRowDelegate(self.dataSourceTreeTable, {}));
            // Adds drop
            self.dataSourceTreeTable.addDelegate(new self.DroppableRowDelegate(self.dataSourceTreeTable, {}));
            // Add context menu for right click
            self.dataSourceTreeTable.attachCellContextmenu({},function(event) {
                controller._showContextMenu(event, controller);
            });

            // Add key listener to support up, down and selection (i.e. check)
            // TODO: This is broke right now next on the list.
            self.dataSourceTreeTable.attachBrowserEvent('keyup', function(event) {
                switch (event.keyCode) {
                    case 40: // down arrow
                        self.logger.debug("TreeTable: Got Down");
                    //     self._moveSelection(table, true, model);
                        break;
                    case 38: // up arrow
                        self.logger.debug("TreeTable: Got Up");
                    //     self._moveSelection(table, false, model);
                        break;
                    case 32: // space key
                        self.logger.debug("TreeTable: Got space");
                        // if (_.isFunction(event.getSource)) {
                        //     controller.dataSourceTreeTableRowSelectionChange(event);
                        // }
                        break;
                }
                // var model = sas.analytics.riskhp.views.datapane.DataPaneUtil.getDataPaneTreeTableSelectionModel();
                // model.handleKeyEvent(self.dataSourceTreeTable,event);
            });
        },

        _createDataPanePropertiesToolbar: function(controller) {
            this.clearSelectionsText = new sas.hc.m.Text({
                text: this.resourceBundle.getText('DataPane.properties.toolbar.dataItem.noSelections.txt')
            });

            this.clearSelectionsLink = new sas.hc.m.Link({
                text: this.resourceBundle.getText('DataPane.properties.toolbar.dataItem.clearSelections.fmt', [0]),
                press: $.proxy(controller.clearSelectionsButtonPress, controller),
                visible: false // Hidden by default
            });

            this.propertiesButton = new sas.hc.m.Button({
                icon: sas.icons.HC.PROPERTIES,
                enabled: false,
                press: $.proxy(controller.propertiesButtonPress, controller)
            });

            this.dataPanePropertiesToolbar = new sas.risk.common.layout.BarLayout({
                height: "100%",
                content: [
                    this.clearSelectionsText,
                    this.clearSelectionsLink,
                    new sas.risk.common.layout.BarLayoutSpacer(),
                    this.propertiesButton
                ]
            });
        },

        _createView: function (controller) {
            // Create a BorderLayout for this View to help with scrolling
            return new sas.hc.ui.commons.layout.BorderLayout({
                width: "100%",
                height: "100%",
                top: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    overflowX: "hidden",
                    overflowY: "hidden",
                    size: HprConstants.HPR_DATA_PANE_TOP_BORDERLAYOUTAREA_DEFAULT_SIZE,
                    content: [this._buildHeader(controller)]
                }),
                center: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    overflowX: "auto",
                    overflowY: "auto",
                    size: "auto",
                    content: [this.dataSourceTreeTable]
                }),
                bottom: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    overflowX: "hidden",
                    overflowY: "hidden",
                    size: HprConstants.HPR_DATA_PANE_BOTTOM_BORDERLAYOUTAREA_TOOLBAR_SIZE,
                    content: [this.dataPanePropertiesToolbar]
                }).addStyleClass("dataPaneMainBottomBorderLayout")
            }).addStyleClass("dataPaneMainBorderLayout");
        },

        _buildHeader: function(controller) {
            var self = this;

            // TODO: dacagl - ActionSelect is scheduled to be deprecated in HC9.0 (via https://rndjira.sas.com/browse/HTMLCOMMONS-7830)
            self.dataSourceSelect = new sas.hc.m.ActionSelect({
                width: "100%",
                items: {
                    path: '/dataSources',
                    sorter: new sap.ui.model.Sorter("label", false),
                    template:  new sas.hc.ui.core.Item ({
                        key: '{realDataSourceId}',
                        text: '{label}',
                        tooltip: {
                            parts: ["workGroup", "relPath"],
                            formatter: function(workGroup, relPath) {
                                // Construct a multi-line tooltip with the workGroup and path information for the Data Source
                                var path = self.HprUtil.getCubePath(workGroup, relPath);
                                return self.resourceBundle.getText('DataPane.data.source.select.item.workGroup.tooltip.fmt', [workGroup]) + '\n' +
                                    self.resourceBundle.getText('DataPane.data.source.select.item.path.tooltip.fmt', [path]);
                            }
                        }
                    })
                },
                enabled: '{' + HprConstants.MODEL_ID_EXPLORATION_PANES_STATE + '>/panes/dataPane/dataSourceToolbarEnabled}',
                change: $.proxy(controller.dataSourceSelectChange, controller)
            }).addStyleClass('dataSourceSelect');

            self.dataSourceOverflowButton = new sas.hc.m.Button({
                icon: sas.icons.HC.INDICATORDATA,
                press: $.proxy(controller._handleOpenDataSourceOverflowMenu, controller),
                enabled: '{' + HprConstants.MODEL_ID_EXPLORATION_PANES_STATE + '>/panes/dataPane/dataSourceToolbarEnabled}',
                tooltip: self.resourceBundle.getText('hprisk.dataSourceOverflowButton.tip')
            });
            self.dataSourceOverflowButton.addStyleClass("lastComponent dataSourceOverflowButton");

            // S1446358: This was a 'sas.hc.m.Toolbar' but caused some accessibility problems with the ActionSelect when the down arrow key was hit
            var dataSourceToolbar = new sas.risk.common.layout.BarLayout({
                height: "100%",
                content: [self.dataSourceSelect, new sas.risk.common.layout.BarLayoutSpacer(), self.dataSourceOverflowButton]
            });
            dataSourceToolbar.addStyleClass("noBackground");

            // Search field
            self.searchField = new sas.hc.m.SearchField({
                width: "100%",
                filterMode: true,
                liveChange: $.proxy(controller._onSearchLiveChange, controller) // Filter results when user types after slight delay
            }).addStyleClass("dataSourceSearchField");

            var headerLayout = new sas.hc.ui.layout.VerticalLayout({
                content:[dataSourceToolbar, self.searchField]
            });
            headerLayout.addStyleClass("fullWidth");

            return headerLayout;
        },

        getDataSourceTreeTable: function() {
            return this.dataSourceTreeTable;
        },

        updateDataPaneToolbarForSelections: function(selectedDataItems) {
            // update clear selections link text
            if (selectedDataItems && selectedDataItems.length > 0) {
                this.clearSelectionsText.setVisible(false);
                this.clearSelectionsLink.setVisible(true);
                if (selectedDataItems.length === 1) {
                    this.clearSelectionsLink.setText(this.resourceBundle.getText('DataPane.properties.toolbar.dataItem.oneSelection.txt'));
                } else {
                    this.clearSelectionsLink.setText(this.resourceBundle.getText('DataPane.properties.toolbar.dataItem.clearSelections.fmt', [selectedDataItems.length]));
                }
            } else {
                this.clearSelectionsText.setVisible(true);
                this.clearSelectionsLink.setVisible(false);
            }

            // update properties button enabled state
            this.propertiesButton.setEnabled(selectedDataItems && selectedDataItems.length === 1);
        }
    };
}()));
