// (c) 2017, SAS Institute Inc.

jQuery.sap.require("sas.analytics.riskhp.models.ComputedVariablesManager");
jQuery.sap.require("sas.analytics.riskhp.models.DataSourceCacheManager");
jQuery.sap.require("sas.analytics.riskhp.models.DataSourceUtil");
jQuery.sap.require("sas.analytics.riskhp.models.HPRiskModelManager");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Classvar");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Compvar");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Hierarchy");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Scenario");
jQuery.sap.require("sas.analytics.riskhp.models.dataitems.ResultSet");
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
jQuery.sap.require("sas.analytics.riskhp.views.datapane.DataPaneModelManager");
jQuery.sap.require("sas.analytics.riskhp.views.datapane.HprDataPanelTreeTableSelectionModel");
jQuery.sap.require("sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil");
jQuery.sap.require("sas.analytics.riskhp.utils.ExplorationUtil");
jQuery.sap.require("sas.analytics.riskhp.utils.HprAPI");
jQuery.sap.require("sas.analytics.riskhp.models.cache.CacheManager");

/**
 * @name sas.analytics.riskhp.views.datapane.DataPaneUtil
 */
sas.analytics.riskhp.views.datapane.DataPaneUtil = {

    HprUtil: sas.analytics.riskhp.utils.HprUtil,
    DataPaneModelManager: sas.analytics.riskhp.views.datapane.DataPaneModelManager,
    DataSourceUtil: sas.analytics.riskhp.models.DataSourceUtil,
    HPRiskModelManager: sas.analytics.riskhp.models.HPRiskModelManager,
    HprDataPanelTreeTableSelectionModel: sas.analytics.riskhp.views.datapane.HprDataPanelTreeTableSelectionModel,
    ComputedVariablesManager: sas.analytics.riskhp.models.ComputedVariablesManager,
    CLASSVAR: sas.analytics.riskhp.models.dataitems.Classvar.CLASSVAR,
    HIERARCHY: sas.analytics.riskhp.models.dataitems.Hierarchy.HIERARCHY,
    SCENARIO: sas.analytics.riskhp.models.dataitems.Scenario.SCENARIO,
    RESULTSET: sas.analytics.riskhp.models.dataitems.ResultSet.RESULTSET,
    COMPVAR: sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR,
    CompVarUtil: sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil,
    ExplorationUtil: sas.analytics.riskhp.utils.ExplorationUtil,
    HprAPI: sas.analytics.riskhp.utils.HprAPI,
    HprConstants: sas.analytics.riskhp.utils.HprConstants,

    //*****************************************************************************************************
    //*****************************************************************************************************
    // TreeTable helper functions
    //*****************************************************************************************************
    //*****************************************************************************************************

    /**
     * Finds the row that is associated with the given elementId.  The elementId is assumed to be
     * a child in the hierarchy of the row. The resultant id can be used by sap.ui.getCore().byId() to retrieve the UI5 component.
     * @param elementId
     * @private
     */
    _findRowId: function(elementId) {
        // Find element in the DOM that matches the given id
        var matches = $('#' + elementId);
        // If we got at least one and it is a 'tr' element, return it's id
        if (matches.length > 0 && $(matches[0]).is('tr')) {
            return $(matches[0]).attr('id');
        }
        else {
            // Otherwise find the closest one up the DOM tree that is a 'table'
            matches = $('#' + elementId).closest('tr');
            if (matches.length > 0) {
                return $(matches[0]).attr('id');
            }
        }
        // If nothing was found return null.
        return null;
    },

    /**
     * Given a rowPath and a TreeTable return the index for that row, if it is visible
     * @param rowPath
     * @param treeTableControl
     * @returns {Number} -1 if not found
     */
    getRowIndexForVisibleRowPath: function(rowPath, treeTableControl) {
        if (!treeTableControl) {
            treeTableControl = this._findDataPaneTable();
        }

        var rows = treeTableControl.getRows();
        return _.findIndex(rows, function(row) {
            var bindingContext = row.getBindingContext();
            if (bindingContext) {
                return bindingContext.sPath === rowPath;
            }
            return false;
        });
    },

    /**
     * Finds the id of the table that is associated with the given elementId.  The elementId is assumed to be
     * a child in the hierarchy of the table. The resultant id can be used by sap.ui.getCore().byId() to retrieve the UI5 component.
     * @param elementId
     * @private
     */
    _findTableId: function(elementId) {
        // Find element in the DOM that matches the given id
        var matches = $('#' + elementId);
        // If we got at least one and it is a 'table' element, return it's id, FIXME: Does this branch really work? Probably not! MCA
        if (matches.length > 0 && $(matches[0]).is('table')) {
            return $(matches[0]).attr('id');
        }
        else {
            // Otherwise find the closest one up the DOM tree that is a 'table'
            matches = $('#' + elementId).closest('.treeTable');
            if (matches.length > 0) {
                return $(matches[0]).attr('id');
            }
        }
        // If nothing was found return null.
        return null;
    },

    /**
     * Given an elementId finds the table control containing that element
     * @param elementId
     * @returns {*}
     * @private
     */
    _findTableForElementWithId: function(elementId) {
        // Get the id of the table
        var tableId = this._findTableId(elementId);
        if (!tableId) {
            return null;
        }

        // Get the table as a UI5 object
        var table = sap.ui.getCore().byId(tableId);
        if (!table) {
            return null;
        }
        return table;
    },

    /**
     * Convience method to find the single tree table held in the data pane
     * @returns {*}
     * @private
     */
    _findDataPaneTable: function() {
        var tableId = $(".dataSourceTreeTable").attr('id');
        return sap.ui.getCore().byId(tableId);
    },

    /**
     * Finds the row with the given id and returns it's UI5 object representation
     * @param elementId
     * @returns {*}
     * @private
     */
    _getUI5RowWithId: function(elementId) {
        // Get the id of the table
        var tableId = this._findTableId(elementId);
        if (!tableId) {
            return null;
        }

        // Get the table as a UI5 object
        var table = sap.ui.getCore().byId(tableId);
        if (!table) {
            return null;
        }

        // Get the real id of the row
        var rowId =  this._findRowId(elementId);
        if (!rowId) {
            return null;
        }

        // Get the row as a UI5 object
        var row = sap.ui.getCore().byId(rowId);
        if (!row) {
            return null;
        }
        return row;
    },

    /**
     * Given an id of a row, returns the path into the underlying tree table model where that
     * row's data can be found and the row's model as a hash.  If the row cannot be found then returns null.
     * @param rowId
     * @returns {*} Returns null if the row cannot be found, otherwise returns an object with the path key
     * representing the path into the model for the data and the model key representing the actual model
     * for the row.
     */
    getRowData: function(rowId){
        var row = this._getUI5RowWithId(rowId);
        if (!row) {
            return null;
        }
        var context = row.getBindingContext();
        if (!context) {
            return null;
        }
        var rowPath = context.getPath();
        var model = row.getModel();
        return {path: rowPath, model: model};
    },

    /**
     * Takes a path string (from a tree table model) and returns the path to its top-level folder (i.e "/children/4")
     * @param path
     * @returns {string}
     */
    getTopLevelFolderPath: function(path) {
        return path.substring(0, 11); // Want to substring to "/children/4"
    },

    /**
     * Takes a path string (from a tree table model) and returns the path to the parent (i.e "/children/4/children/1")
     * @param path
     * @returns {string}
     */
    getParentsPathFromPath: function(path) {
        var pathElementsArray = path.split("/");
        return pathElementsArray.slice(0,pathElementsArray.length-2).join("/");
    },

    /**
     * Takes a path string (from a tree table model) and returns the path to the parent's children (i.e. "/children/4/children/0/children")
     * @param path
     * @returns {string}
     */
    getParentsChildrenPathFromPath: function(path) {
        var pathElementsArray = path.split("/");
        return pathElementsArray.slice(0,pathElementsArray.length-1).join("/");
    },

    /**
     * Given a list of row paths that are or should be selected in the table, returns the list
     * sorted in the order they are actually shown in the TreeTable.  Note: This can be an expensive operation so be
     * sure you really need the items sorted before calling this.  Also this has not been tested with nested items, in
     * that case you may not get what you wanted, TBD. Finally, this will only work with lists that have less than
     * 1000000 items. FYI, I wouldn't try it with anything remotely close to that (if you value your user or your sanity!!),
     * but I do know once you hit 1000000 items the sorting algorithm will fail.
     * @param selectedRowPaths
     * @returns {Array}
     */
    getSortedSelectedRowPaths: function(selectedRowPaths) {
        // Create an array to "temporarily" hold row paths after we pad the indexes, this will be the sort array
        var sortableRowPathsArray = [];

        // walk through each path in selectedRowPaths, padding indexes as you find them and accumulating the resultant
        // paths in the sortableRowPathsArray
        _.each(selectedRowPaths, function(path){
            var elementsWithPaddedIndexes = [];
            // Split the path into it's parts, pad the indexes and build a new padded path
            _.each(path.split("/"), function (element){
                // Strings that can be numbers which are not NaN will pass the isFinite test
                if (_.isFinite(element)) {
                    elementsWithPaddedIndexes.push(("000000" + element).slice(-6));
                }
                else {
                    elementsWithPaddedIndexes.push(element);
                }
            });
            // Combine the (now padded) elements back into a path string
            var paddedPath = elementsWithPaddedIndexes.join("/");

            // Add the padded path string to the array that will be sorted
            sortableRowPathsArray.push(paddedPath);
        });

        // Sort the array of row paths
        sortableRowPathsArray.sort();

        // Pull out the padded indexes and rebuild the selectedRowPaths array
        selectedRowPaths = [];
        _.each(sortableRowPathsArray, function(path){
            var elementsWithUnPaddedIndexes = [];
            // Split the path into it's parts, unpad the indexes and build a new padded path
            _.each(path.split("/"), function (element){
                var elementAsInt = parseInt(element);
                if (_.isFinite(elementAsInt)) {
                    elementsWithUnPaddedIndexes.push("" + elementAsInt);
                }
                else {
                    elementsWithUnPaddedIndexes.push(element);
                }
            });
            // Combine the (now padded) elements back into a path string
            var unPaddedPath = elementsWithUnPaddedIndexes.join("/");

            // Add the padded path string to the array that will be sorted
            selectedRowPaths.push(unPaddedPath);
        });
        // Return the now sorted paths
        return selectedRowPaths;
    },

    /**
     * Given a rowId returns the associated path in the TreeTable's model for that row.
     * @param rowId
     * @returns {*}
     */
    getRowPathForRowId: function(rowId) {
        var row = sap.ui.getCore().byId(rowId);
        if (row) {
            var bindingContext = row.getBindingContext();
            if (bindingContext) {
                return bindingContext.sPath;
            }
        }
        return null;
    },

    /**
     * Given a rowId, finds the associated DataItem for that row in the DataPane's TreeTable
     * @param rowId
     * @returns {*}
     */
    getDataItemForRowId: function(rowId) {
        var row = sap.ui.getCore().byId(rowId);
        if (row) {
            var dataPaneTable = this._findDataPaneTable();

            var rowPath = row.getBindingContext().sPath;

            // Get the model of the table
            var model = dataPaneTable.getModel();
            return model.getProperty(rowPath);
        }
        return null;
    },

    /**
     * Given a rowId, determine if it is the first child in a TreeTable folder
     * @param rowId HTML id of the table row
     * @param oTable (optional) TreeTable object, will use DataPane TreeTable if not specified
     * @returns {boolean} true | false
     */
    isRowIdFirstChildOfFolder: function(rowId, oTable) {
        if (!oTable) {
            oTable = this._findDataPaneTable();
        }

        var targetRowData = this.getRowData(rowId);
        var targetRowPath = targetRowData.path;
        var parentsChildrenPathOfTargetRowPath = this.getParentsChildrenPathFromPath(targetRowPath);
        var tableModel = oTable.getModel();
        var targetRowObject = tableModel.getProperty(targetRowPath);
        var parentsChildrenOfTargetRowArray = tableModel.getProperty(parentsChildrenPathOfTargetRowPath);

        return _.isEqual(targetRowObject, _.first(parentsChildrenOfTargetRowArray));
    },

    /**
     * Given a rowId, determine if it is the last child in a TreeTable folder
     * @param rowId HTML id of the table row
     * @param oTable (optional) TreeTable object, will use DataPane TreeTable if not specified
     * @returns {boolean} true | false
     */
    isRowIdLastChildOfFolder: function(rowId, oTable) {
        if (!oTable) {
            oTable = this._findDataPaneTable();
        }

        var targetRowData = this.getRowData(rowId);
        var targetRowPath = targetRowData.path;
        var parentsChildrenPathOfTargetRowPath = this.getParentsChildrenPathFromPath(targetRowPath);
        var tableModel = oTable.getModel();
        var targetRowObject = tableModel.getProperty(targetRowPath);
        var parentsChildrenOfTargetRowArray = tableModel.getProperty(parentsChildrenPathOfTargetRowPath);

        return _.isEqual(targetRowObject, _.last(parentsChildrenOfTargetRowArray));
    },

    /**
     * Given a rowId, determines if the corresponding TreeNode is expanded.  rowId must be visible
     * @param rowId HTML id of the visible table row
     * @param oTable (optional) TreeTable object, will use DataPane TreeTable if not specified
     * @returns {boolean} true | false True if TreeNode is expanded, false if it is collapsed
     */
    isTreeNodeExpanded: function(rowId, oTable) {
        if (!oTable) {
            oTable = this._findDataPaneTable();
        }

        var targetRowData = this.getRowData(rowId);
        var rowIndex = this.getRowIndexForVisibleRowPath(targetRowData.path);

        return oTable.isExpanded(rowIndex);
    },

    /**
     * Given a rowId, determines if the corresponding TreeNode is a data item for a sub-folder and if its parent folder is expanded.
     * @param rowId HTML id of the visible table row
     * @param oTable (optional) TreeTable object, will use DataPane TreeTable if not specified
     * @returns {boolean} true | false
     */
    isSubItemAndParentFolderExpanded: function(rowId, oTable) {
        if (!oTable) {
            oTable = this._findDataPaneTable();
        }

        var isExpanded = false;
        var targetRowData = this.getRowData(rowId);
        var targetRowPath = targetRowData.path;
        var targetRowObject = oTable.getModel().getProperty(targetRowPath);

        // The 'dataItemChild' property signifies this Data Item is a child of a sub-folder
        if (targetRowObject.dataItemChild) {
            var parentTreeNodePath = this.getParentsPathFromPath(targetRowPath);
            var rowIndex = this.getRowIndexForVisibleRowPath(parentTreeNodePath);
            isExpanded = oTable.isExpanded(rowIndex);
        }

        return isExpanded;
    },

    /**
     * Given a rowId, determines if the corresponding TreeNode is the last data item for a sub-folder.
     * @param rowId HTML id of the visible table row
     * @param oTable (optional) TreeTable object, will use DataPane TreeTable if not specified
     * @returns {boolean} true | false
     */
    isLastSubItemForFolder: function (rowId, oTable) {
        if (!oTable) {
            oTable = this._findDataPaneTable();
        }

        var isLast = false;
        var targetRowData = this.getRowData(rowId);
        var targetRowPath = targetRowData.path;
        var targetRowObject = oTable.getModel().getProperty(targetRowPath);

        // The 'dataItemChild' property signifies this Data Item is a child of a sub-folder
        if (targetRowObject.dataItemChild) {
            var topLevelFolderPath = this.getTopLevelFolderPath(targetRowPath);
            var pathToLastSubItem = this.findPathToLastSubItem(topLevelFolderPath);
            isLast = targetRowPath === pathToLastSubItem;
        }

        return isLast;
    },

    /**
     * Generates the path to the last TreeNode/Data Item in a folder.  'folderPath' can be the path for a top-level
     * folder or a sub-folder.
     * @param folderPath Path for a top-level folder or sub-folder
     * @param oTable (optional) TreeTable object, will use DataPane TreeTable if not specified
     * @returns {String} String representation of the last TreeNode path in a folder
     */
    findPathToLastSubItem: function(folderPath, oTable) {
        if (!oTable) {
            oTable = this._findDataPaneTable();
        }

        var path = folderPath;
        var tableModel = oTable.getModel();
        var targetRowData = tableModel.getProperty(folderPath);

        // Keep generating a path as long as the targetRowData has children nodes
        while (targetRowData && targetRowData.children && targetRowData.children.length > 0) {
            // Append the path to the last child onto the 'path' variable (i.e. "/children/4")
            path += "/children/" + String(targetRowData.children.length - 1);
            targetRowData = tableModel.getProperty(path);
        }

        return path;
    },

    /**
     * Marches through the children for a path in the TreeTable and collapses all the sub-folders if they are expanded
     * @param childrenPath Path to children for a folder (i.e. "/children/0/children")
     * @param oTable (optional) TreeTable object, will use DataPane TreeTable if not specified
     */
    collapseAllSubFoldersForPath: function(childrenPath, oTable) {
        if (!childrenPath) return;

        if (!oTable) {
            oTable = this._findDataPaneTable();
        }

        var firstChildRowPath = childrenPath + "/0";
        var startIndex = this.getRowIndexForVisibleRowPath(firstChildRowPath);
        var tableModel = oTable.getModel();
        var childrenArray = tableModel.getProperty(childrenPath);
        var stopIndex = startIndex + (childrenArray.length - 1);

        for (var i = startIndex; i <= stopIndex; i++) {
            if (oTable.isExpanded(i)) {
                oTable.collapse(i);
            }
        }
    },

    /**
     * Marches through the children for a path in the TreeTable and creates an Object that stores the expansion state for sub-folders.  An Object is returned
     * if the children for the path contains sub-folder(s).  In this case regular nodes will get an expansion state of 'false' which is fine.
     * @param childrenPath Path to children for a folder (i.e. "/children/0/children")
     * @param oTable (optional) TreeTable object, will use DataPane TreeTable if not specified
     * @returns {Object} Object will contain 'id' (id of DataItem) and 'isExpanded' properties; Empty Object is returned for paths that have no sub-folder(s)
     */
    createChildrenPathExpansionStateObject: function(childrenPath, oTable) {
        var expansionStateObject = {};
        var hasSubFolder = false;

        if (!childrenPath) return expansionStateObject;

        if (!oTable) {
            oTable = this._findDataPaneTable();
        }

        // Locate the children in the Tree first
        var firstChildRowPath = childrenPath + "/0";
        var treeRowStartIndex = this.getRowIndexForVisibleRowPath(firstChildRowPath);
        var tableModel = oTable.getModel();
        var childrenArray = tableModel.getProperty(childrenPath);

        // Run over the children array which is full of the Data Items and gather the necessary information (i.e. id and isExpanded);
        // We must take into account if the node is expanded when trying to find out the expansion state because the TreeTable relies
        // on a 'rowIndex' for those methods
        for (var childrenArrayCounter = 0, treeRowCounter = treeRowStartIndex; childrenArrayCounter <= childrenArray.length - 1; childrenArrayCounter++) {
            var dataItemExpansionStateObject = {};

            // Find the id
            dataItemExpansionStateObject.id = childrenArray[childrenArrayCounter].dataItem.getId();

            // Find the expansion
            var isExpanded = oTable.isExpanded(treeRowCounter);
            dataItemExpansionStateObject.isExpanded = isExpanded;

            // Store away state
            expansionStateObject[dataItemExpansionStateObject.id] = dataItemExpansionStateObject;

            // Increase "treeRowCounter" based on if the node is expanded or collapsed
            var nodeChildren = childrenArray[childrenArrayCounter].children;

            // Check to see if there is children for the node we are working with
            if (nodeChildren && nodeChildren.length > 0) {
                hasSubFolder = true; // Flag that the sub-folder has children, we want to create the expansionStateObject in this case

                if (isExpanded) {
                    // Must increase treeRowCounter by children length + 1
                    treeRowCounter = treeRowCounter + (nodeChildren.length + 1);
                } else {
                    // If collapsed, only have to increase by 1
                    treeRowCounter++;
                }
            }
        }

        // If there is not a sub-folder in the children, return an empty object so we don't have to march over the object when we re-apply after the nodes have moved
        if (!hasSubFolder) {
            expansionStateObject = {};
        }

        return expansionStateObject;
    },

    /**
     * Marches through the children for a path in the TreeTable and re-applies the expansion state for the TreeNodes which
     * correspond to Data Item id's.
     * @param childrenPath Path to children for a folder (i.e. "/children/0/children")
     * @param expansionStateObject Object which contains the expansion state for TreeNodes based on 'id' of corresponding Data Item
     * @param oTable (optional) TreeTable object, will use DataPane TreeTable if not specified
     */
    applyExpansionStateObject: function(childrenPath, expansionStateObject, oTable) {
        if (!childrenPath || _.isEmpty(expansionStateObject)) return;

        if (!oTable) {
            oTable = this._findDataPaneTable();
        }

        // Locate the children in the Tree first
        var firstChildRowPath = childrenPath + "/0";
        var treeRowStartIndex = this.getRowIndexForVisibleRowPath(firstChildRowPath);
        var tableModel = oTable.getModel();
        var childrenArray = tableModel.getProperty(childrenPath);

        // Run over the children array which is full of the Data Items and re-apply the expansion state of the TreeNodes
        for (var childrenArrayCounter = 0, treeRowCounter = treeRowStartIndex; childrenArrayCounter <= childrenArray.length - 1; childrenArrayCounter++) {
            // Find the object corresponding to the current Data Item we are working with
            var dataItemId = childrenArray[childrenArrayCounter].dataItem.getId();
            var dataItemExpansionObject = expansionStateObject[dataItemId];

            // Re-apply the expansion state to the TreeNode
            var isExpanded = dataItemExpansionObject.isExpanded;
            if (isExpanded) {
                oTable.expand(treeRowCounter);
            } else {
                oTable.collapse(treeRowCounter);
            }

            // Increase "treeRowCounter" based on if the node is expanded or collapsed
            var nodeChildren = childrenArray[childrenArrayCounter].children;
            if (nodeChildren && nodeChildren.length > 0 && isExpanded) {
                // Must increase treeRowCounter by children length + 1
                treeRowCounter = treeRowCounter + (nodeChildren.length + 1);
            } else {
                // If collapsed, only have to increase by 1
                treeRowCounter++;
            }
        }
    },

    /**
     * Determines if the TreeNodes corresponding to the id's are the same Data Item type
     * @param draggableId HTML id of the draggable
     * @param droppableId HTML id of the droppable
     * @returns {boolean} true | false
     */
    areTreeNodesSameType: function(draggableId, droppableId) {
        // Make sure we have two values to check
        if (!draggableId || !droppableId) {
            return false;
        }

        // Get the rowData for both ids and make sure we have them both
        var draggableRowData = this.getRowData(draggableId);
        var droppableRowData = this.getRowData(droppableId);
        if (!draggableRowData || ! droppableRowData) {
            return false;
        }

        // Grab the nodeType for both objects and make sure they are the same
        var draggableType = draggableRowData.model.getProperty(draggableRowData.path + "/nodeType");
        var droppableType = droppableRowData.model.getProperty(droppableRowData.path + "/nodeType");

        return draggableType === droppableType;
    },

    /**
     * Determines if all the selected rows for the Data Pane TreeTable are the same type (i.e. Class Variable,
     * Output Variable, etc.).  If 'selectedRows' is not passed in, the selected rows are retrieved from the
     * HprDataPanelTreeTableSelectionModel class.
     * @param selectedRows (Optional) An array of row paths for the selected rows in the Data Pane TreeTable
     * @returns {boolean} Returns true for selected rows of length 0 or 1; Returns true | false otherwise
     */
    areSelectedNodesSameType: function(selectedRows) {
        // Get list of selected row paths that have been accumulated (if not passed in)
        if (!selectedRows) {
            var dataPaneTreeTableSelectionModel = this.getDataPaneTreeTableSelectionModel();
            selectedRows = dataPaneTreeTableSelectionModel.getSelectedRows();
        }

        // Get the Data Pane TreeTable and model
        var dataPaneTable = this._findDataPaneTable();
        var model = dataPaneTable.getModel();

        var result = true; // Default to true in case we have 0 or 1 selected row
        if (selectedRows.length > 1) {
            // Get the type of node to test against from the first selected row
            var baseLineType = model.getProperty(selectedRows[0] + "/nodeType");
            // Returns true | false if every selected row is the same node type
            result = _.every(selectedRows, function(selectedRow) {
                return model.getProperty(selectedRow + "/nodeType") === baseLineType;
            });
        }

        return result;
    },

    /**
     * Determines if all the selected rows for the Data Pane TreeTable have the same parent folder.  If
     * 'selectedRows' is not passed in, the selected rows are retrieved from the HprDataPanelTreeTableSelectionModel class.
     * @param selectedRows (Optional) An array of row paths for the selected rows in the Data Pane TreeTable
     * @returns {boolean} Returns true for selected rows of length 0 or 1; Returns true | false otherwise
     */
    doSelectedNodesHaveSameParent: function(selectedRows) {
        var self = this;

        // Get list of selected row paths that have been accumulated (if not passed in)
        if (!selectedRows) {
            var dataPaneTreeTableSelectionModel = this.getDataPaneTreeTableSelectionModel();
            selectedRows = dataPaneTreeTableSelectionModel.getSelectedRows();
        }

        var result = true; // Default to true in case we have 0 or 1 selected row
        if (selectedRows.length > 1) {
            // Get the row path for the parent of the first selected row to test against
            var baseLineParentPath = this.getParentsPathFromPath(selectedRows[0]);
            // Returns true | false if every selected row has the same parent
            result = _.every(selectedRows, function(selectedRow) {
                return self.getParentsPathFromPath(selectedRow) === baseLineParentPath;
            });
        }

        return result;
    },

    /**
     * A method which determines if all selected nodes for the DataPane TreeTable can be moved up or down (depending on the 'bMoveUp' parameter).
     * @param bMoveUp A boolean which signifies the direction to determine the result for; 'true' means the up direction
     * @returns {boolean} true | false Returns boolean if all selected nodes can be moved up or down (depending on the 'bMoveUp' parameter)
     */
    canAllSelectedNodesBeMoved: function(bMoveUp) {
        var result = false;

        // Get the Data Pane TreeTable Selection model and the selected rows
        var dataPaneTreeTableSelectionModel = this.getDataPaneTreeTableSelectionModel();
        var selectedRows = dataPaneTreeTableSelectionModel.getSelectedRows();

        // For the result to be valid, the selected nodes have to be the same type and contained under the same parent folder
        var areSelectedNodesSameType = this.areSelectedNodesSameType(selectedRows);
        var doSelectedNodesHaveSameParent = this.doSelectedNodesHaveSameParent(selectedRows);
        if (areSelectedNodesSameType && doSelectedNodesHaveSameParent) {
            // Get the Data Pane TreeTable and model
            var dataPaneTable = this._findDataPaneTable();
            var treeModel = dataPaneTable.getModel();

            // Get the row path for the parents children and retrieve all the parents children nodes
            var baseLineParentsChildrenPath = this.getParentsChildrenPathFromPath(selectedRows[0]);
            var parentsChildrenArray = treeModel.getProperty(baseLineParentsChildrenPath);

            // If the number of selected rows does not equal the length of the parents children array, then
            // we have some work to do in order to determine the result; If the two lengths are equal then
            // take the default of 'false' for the result because none of the selected nodes can be moved since
            // all of them are selected
            if (selectedRows.length !== parentsChildrenArray.length) {
                // Retrieve the indexes of the selected rows by parsing the row path string and turning the last part into an Integer
                // (i.e. "/children/0/children/3" is stored as 3 and "/children/0/children/2" is stored as 2)
                var indexesOfSelectedRows = [];
                _.each(selectedRows, function (selectedRow) {
                    indexesOfSelectedRows.push(parseInt(_.last(selectedRow.split('/'))));
                });

                // Create a range of Integer values based on whether we are trying to determine 'Move Up' or 'Move Down' numbers
                var range = [];
                if (bMoveUp) {
                    range = _.range(0, selectedRows.length); // range should be 0 up to the length of the selected rows (i.e. 0, 1, 2)
                    // Have to get selected indexes in sorted ascending order
                    indexesOfSelectedRows = _.sortBy(indexesOfSelectedRows, function(num) {
                        return num;
                    });
                } else {
                    // For 'Move Down' we need to start the range at the uppermost Integer and work our way backwards by a count of 1
                    // (i.e. 4, 3, 2)
                    range = _.range(parentsChildrenArray.length - 1, (parentsChildrenArray.length - 1) - selectedRows.length, -1);
                    // Have to get selected indexes in sorted descending order
                    indexesOfSelectedRows = _.sortBy(indexesOfSelectedRows, function(num) {
                        return -num;
                    });
                }

                // The result being returned signifies if all selected nodes can be moved; So if the 'range' and 'indexesOfSelectedRows'
                // are equal then all selected nodes can not be moved; Basically this is saying the selected nodes are at the top
                // or bottom (depending on 'bMoveUp' flag) of the parents children array; therefore, they can't be moved
                result = !(_.isEqual(range, indexesOfSelectedRows));
            }
        }

        return result;
    },

    ///**
    // * Given a top-level tree node, hydrate the dataItem property of the children with the real Data Item objects
    // * @param topLevelFolder
    // */
    //dataPaneTreeHydrateDataItemsForTreeNode: function(topLevelFolder) {
    //    var selectedDataSource = this.DataPaneModelManager.getSelectedDataSource();
    //
    //    // Loop through the children of the top-level folder and convert the dataItem property back into a real Data Item object
    //    _.each(topLevelFolder.children, function(dataItemNode) {
    //        if (dataItemNode.dataItem) {
    //            // Convert dataItemNode.dataItem back into actual Data Item class by locating Data Item by the id
    //            dataItemNode.dataItem = selectedDataSource.getDataItemById(dataItemNode.dataItem.id); // Have to use properties, no functions at this point
    //        }
    //    });
    //},

    /**
     * This method searches through different models to find the correct type of Horizon node to display in the Data Pane Tree based on the 'horizonType'
     * parameter passed in.  The end result is that the children tree nodes for the 'horizonFolderNode' are set to the correct Horizon nodes.
     * @param viewModel
     * @param horizonType
     * @param horizonFolderNode
     */
    dataPaneTreeFindSuitableHorizonNodes: function(viewModel, horizonType, horizonFolderNode) {
        var cubeData, dataTypeItems;

        // If we need absolute (normal) horizons, we can just get the horizons from the data source. Otherwise,
        // we need to generate relative horizons from the data source horizons, since those do not include the
        // scenario horizons (the data source relative and absolute horizon collection includes scenario horizons).
        cubeData = this.DataPaneModelManager.getSelectedDataSource();
        if (horizonType === HprConstants.DISPLAY_HORIZONS_RELATIVE_VALUE) {
            dataTypeItems = this.DataSourceUtil.generateRelativeHorizonDataItems(cubeData.getHorizons(), true, cubeData.getId());
        }
        else {
            dataTypeItems = cubeData.getHorizons();
        }

        // Construct the children nodes for this Horizon folder node
        this.DataPaneModelManager.dataPaneTreeConstructChildrenNodes(horizonFolderNode, dataTypeItems, horizonType);
    },

    /**
     * Returns the list of objects associated with the currently selected rows in the DataPane's TreeTable.
     * For now top level folders are not included, but we might want to revisit this, maybe.
     * @returns {Array} An Array of Objects that are currently selected in the DataPane's TreeTable.  Included in
     * the Object is the actual Data Item object.
     */
    getSelectedTableRowObjects: function() {
        var dataPaneTable = this._findDataPaneTable();

        // get list of selected row paths that have been accumulated
        var dataPaneTreeTableModel = this.getDataPaneTreeTableSelectionModel();
        var selectedRowPaths = dataPaneTreeTableModel.getSelectedRows();

        // Get the model of the table
        var model = dataPaneTable.getModel();

        var selectedRowObjects = [];
        _.each(selectedRowPaths, function(path){
            var object = model.getProperty(path);
            if (object && object.type !== HprConstants.HPR_TREETABLE_LEVEL_TOP) {
                selectedRowObjects.push(object);
            }
        });
        return selectedRowObjects;
    },

    /**
     * Uses the HprDataPanelTreeTableSelectionModel class to retrieve the real Data Items for each selected row in the Data Pane TreeTable
     * @returns {Array} Returns an Array of the selected Data Items in the Data Pane TreeTable
     */
    getSelectedDataItems: function() {
        var selectedDataItems = [];

        // Get the selected Objects the table knows are selected
        _.each(this.getSelectedTableRowObjects(), function(rowObject) {
            if (rowObject.dataItem) {
                selectedDataItems.push(rowObject.dataItem);
            }
        });

        return selectedDataItems;
    },

    /**
     * Clears the selections on the DataPane's TreeTable
     * @param bScrollToTop (optional) Scroll the DataPane TreeTable to the top after clearing selections;
     * Default is false.  If not specified, the DataPane TreeTable will not scroll to the top.
     */
    clearSelections: function(bScrollToTop) {
        if (_.isUndefined(bScrollToTop) || _.isNull(bScrollToTop)) {
            bScrollToTop = false;
        }

        var table = this._findDataPaneTable();
        if (table) {
            table.clearSelection();

            if (bScrollToTop) {
                table.setFirstVisibleRow(0);
            }
        }
        this.HPRiskModelManager.setDraggableObjects([]);
        this.getDataPaneTreeTableSelectionModel().clearSelectedRows();
    },

    /**
     * Returns the model that is used to manage the selection of rows for the TreeTable in the DataPane.  If the model doesn't exist in core it
     * will be created and placed in it.
     * @returns {*|Object|sap.ui.base.ManagedObject}
     */
    getDataPaneTreeTableSelectionModel: function() {
        var dataPaneTreeTableSelectionModel = sap.ui.getCore().getModel('HprDataPanelTreeTableSelectionModel');
        if (!dataPaneTreeTableSelectionModel) {
            dataPaneTreeTableSelectionModel = new this.HprDataPanelTreeTableSelectionModel();
            sap.ui.getCore().setModel(dataPaneTreeTableSelectionModel,'HprDataPanelTreeTableSelectionModel');
        }
        return dataPaneTreeTableSelectionModel;
    },

    /**
     * Locates the top-level ClassVariable TreeNode and returns its children.
     * @returns {Array} Array of children nodes for the ClassVarible top-level TreeNode
     * @param controller
     */
    getClassVariableNodeChildren: function(controller) {
        var view = controller.getView();
        var dataSourceTreeTable = view.dataSourceTreeTable;
        var dataSourceTreeTableModel = dataSourceTreeTable.getModel();
        var branch = this.DataPaneModelManager.getClassVariableBranch(dataSourceTreeTableModel.oData);
        if (branch) {
            return branch.children;
        }
        return [];
    },

    // /**
    //  * Expands the Class Variables folder node in the Tree Table
    //  * @param controller
    //  */
    // S1433189 - Request by MIP folks that Class Variable node is not expanded by default
    // expandClassVariablesFolder: function(controller) {
    //     // Locate and expand the Class Variables folder if it exists
    //     var view = controller.getView();
    //     var rowBinding = view.dataSourceTreeTable.getBinding("rows");
    //     var treeModel = view.dataSourceTreeTable.getModel();
    //     for (var i = 0; i < rowBinding.getLength(); ++i) {
    //         var context = view.dataSourceTreeTable.getContextByIndex(i);
    //         var nodePath = context.getPath();
    //         var node = treeModel.getProperty(nodePath);
    //         if (node && node.id === HprConstants.HPR_DATA_PANE_CLASS_VARIABLES_NODE_ID) {
    //             view.dataSourceTreeTable.expand(i);
    //             this.DataPaneModelManager.updateExpandedNodeIds(node.id, true);
    //             break;
    //         }
    //     }
    // },

    //*****************************************************************************************************
    //*****************************************************************************************************
    // Scenarios helper functions
    //*****************************************************************************************************
    //*****************************************************************************************************

    addScenarios: function(scenarios, controller) {
        // Get the cube data object
        var cubeData = DataSourceCacheManager.getSelectedDataSource();
        _.each(scenarios, jQuery.proxy(cubeData.addScenario,cubeData));

        // TODO: DSCM - This seems like a wierd way to update the dataSourceParam.  Might be better on save to update it then from the current cube data
        var dataSourceParam = DataSourceCacheManager.getSelectedDataSourceParam();
        var addedScenarioIds = [];
        _.each(scenarios, function(scenario) {
            dataSourceParam.addScenarioId(scenario.id);
            addedScenarioIds.push(scenario.id);
        });

        this.updateScenarioNodes(cubeData, controller);

        sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_SCENARIO_ADDED, {
            dataSourceId: cubeData.getId(),
            scenarioIds: addedScenarioIds
        });

    },

    /**
     * Updates the scenarios node list in the tree table based on the current state of the passed in CubeData object
     * @param dataSource
     * @param controller
     */
    updateScenarioNodes: function(dataSource, controller) {
        var self = this;

        var view = controller.getView();
        var treeTableModel = view.dataSourceTreeTable.getModel();

        // result sets are the immediate children of the scenarios folder
        var resultSets = dataSource.getDataItemsForType(self.RESULTSET);

        // get the top level nodes
        var topLevelFolders = treeTableModel.getProperty("/children");

        // find the scenarios node and rebuild its children
        if (topLevelFolders && topLevelFolders.length > 0) {
            // find the scenarios folder:
            var scenarioFolder = _.find(topLevelFolders, function(folder){
                return folder.folderType === self.SCENARIO;
            });

            // rebuild the scenario folder children
            if (scenarioFolder) {
                // add the scenarios to the scenario folder - might consider sorting them here
                self.DataPaneModelManager.dataPaneTreeConstructScenarioChildrenNodes(scenarioFolder, resultSets);
            }
            treeTableModel.updateBindings(true);

            // Publish an event that will update the dirtyState of the Exploration view
            sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {
                dirty: true
            });
        }
    },

    //*****************************************************************************************************
    //*****************************************************************************************************
    // Computed Variables helper functions
    //*****************************************************************************************************
    //*****************************************************************************************************
    /**
     * Add the computed variable to the dataSource and exploration
     * @param compVarData - editor/request information to save to exploration
     * @param computedVar - dataItem information received from server on Apply
     * @param controller
     */
    addComputedVariable: function(dataSource, computedVar, controller) {
        var self = this;

        dataSource.addComputedVariable.call(dataSource, computedVar);
        this.updateComputedVariableNodes(DataSourceCacheManager.getSelectedDataSource(), controller);
    },


    /**
     *
     * @param compVarData editor information needed to make requests to server
     * @param computedVar data item returned from server
     * @param controller data pane controller
     */
    updateComputedVariable: function(dataSource, compVarData, computedVar, controller) {
        // update computed var in the dataSource, then update data pane nodes based on dataSource
        dataSource.updateComputedVariable.call(dataSource, computedVar);
        this.updateComputedVariableNodes(DataSourceCacheManager.getSelectedDataSource(), controller);
    },

    /**
     * Updates the computed variable node list in the tree table based on the current state of the passed in dataSource object
     * @param dataSource
     * @param controller
     */
    updateComputedVariableNodes: function(dataSource, controller) {
        var self = this;
        var view = controller.getView();
        var treeTableModel = view.dataSourceTreeTable.getModel();

        var computedVars = dataSource.computedVariable;
        // update the nodes in the tree table
        var topLevelFolders = treeTableModel.getProperty("/children");
        if (topLevelFolders && topLevelFolders.length > 0) {
            // find the computed variables folder:
            var computedVariablesFolder = _.find(topLevelFolders, function(folder){
                return folder.folderType === self.COMPVAR;
            });

            if (!computedVariablesFolder) {
                computedVariablesFolder = {
                    label: self.HprUtil.dataTypeTitle(self.COMPVAR),
                    nodeType: HprConstants.HPR_TREETABLE_LEVEL_TOP,
                    folderType: self.COMPVAR,
                    id: HprConstants.HPR_DATA_PANE_COMPUTED_VARIABLES_NODE_ID,
                    isHprMoveable: false,
                    isHprDraggable: false,
                    isHprSelectable: false,
                    isHprEditable: false
                };

                // Add a lineage property which will help with search filtering on nested folders
                computedVariablesFolder.lineage = self.HprUtil.dataTypeTitle(self.COMPVAR);// treeTableModel.label;
                topLevelFolders.push(computedVariablesFolder);
            }

            // add the computed variables to the folder - might consider sorting them here
            self.DataPaneModelManager.dataPaneTreeConstructChildrenNodes(computedVariablesFolder, computedVars);

            treeTableModel.updateBindings(true);

            // Publish an event that will update the dirtyState of the Exploration view
            sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_UPDATE_DIRTY_STATE, {
                dirty: true
            });
        }
    },

    _getCrosstabIdsUsingDataSource: function(dataSource) {
        var self = this;
        var crosstabIds = [];

        // Find and group all the Crosstabs which use the same Data Source by the Exploration page ids
        var explorationView = sas.analytics.riskhp.utils.ViewManager.getExplorationView();
        var sameDataSourceCrosstabs = self.ExplorationUtil.getCrosstabsUsingDataSource(explorationView, dataSource);
        var groupSameDataSourceCrosstabsByPage = _.groupBy(sameDataSourceCrosstabs, function(crosstab) {
            return crosstab.pageId;
        });

        // Since we have all the Crosstabs which use the same Data Source grouped by pages loop through them and see
        // if the first Crosstab we come across is instantiated.  This means the Page is active and the control exists
        $.each(_.values(groupSameDataSourceCrosstabsByPage), function(index, crosstabs) {
            var firstCrosstabId = crosstabs[0].id;
            var firstCrosstab = sap.ui.getCore().byId(firstCrosstabId);
            // Check the first Crosstab to see if it is in DOM, this means Page is active and control exists; If the first Crosstab
            // can not be found, that means the Page has not been activated (or clicked on), so the controls don't exist yet;  In
            // this case, when the Page/Tab is clicked on, the CrosstabContext code (spec. the 'fromColumnHeaderJSON' method) will trim
            // out the Data items and columns that don't exist anymore
            if (firstCrosstab && firstCrosstab.context) {
                // Pull all the Crosstab ids and try to remove the Computed Variables from the Crosstabs
                // var crosstabIds = _.pluck(crosstabs, 'id');
                // self._removeComputedVariablesFromCrosstabs(crosstabIds, compVars);
                // return true; // Get out, done with the Crosstabs on this Page
                crosstabIds = _.union(crosstabIds, _.pluck(crosstabs, 'id'));
            }
        });

        return crosstabIds;
    },



    /**
     *  sets compVar dataitem and all dependent computed variable data items invalid, and removes the invalid computed
     *  variables from all crosstabs based on dataSource
     * @param compVar computed variable dataItem to set invalid
     * @param dataSource dataSource for which the computed variable is being marked invalid
     */
    setCompVarInvalid: function(compVar, dataSource, controller, processDescendants, compVarData) {
        var self = this;
        var deferred = $.Deferred();

        var computedVariable = compVarData;
        if (!computedVariable) {
            computedVariable = self.ComputedVariablesManager.getComputedVariableByName(compVar.name);
        }

        if (compVar.isValid()) { // just being defensive

            // remove from dataSource on server
            self.HprAPI.deleteComputedVariable(dataSource, [compVar]).done(
                function(data){
                    if (data.items[0].error !== 0) {
                        console.log("error deleting computed variable " + compVar.name + " from " + dataSource.dataSource.id);
                    }
                }
            ).always(
                function () {
                    // console.log(compVar.name + " is set invalid on server");
                    compVar.setValid(false);
                    // update data pane
                    sas.analytics.riskhp.views.datapane.DataPaneUtil.updateComputedVariableNodes(DataSourceCacheManager.getSelectedDataSource(), controller);

                    var crosstabIds = self._getCrosstabIdsUsingDataSource(dataSource.dataSource);
                    self.CompVarUtil.removeComputedVariablesFromCrosstabs(crosstabIds, [compVar]);

                    if (processDescendants) {
                        var descendantNames = []; // array of names of all descendant computed variables
                        self.CompVarUtil.getComputedVariableDescendants(descendantNames, computedVariable, sas.analytics.riskhp.models.ComputedVariablesManager.getComputedVariables(), dataSource);
                        if (descendantNames.length > 0) {
                            var descendants = [];
                            _.each(descendantNames, function(name) {
                                descendants.push(dataSource.getDataItemByNameAndType(name, sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR));
                            });
                            var invalidateDeferreds = [];
                            _.each(descendants, function(descendant){
                                invalidateDeferreds.push(self.setCompVarInvalid(descendant, dataSource, controller));
                            });
                            // wait for all descendants to be set invalid before resolving
                            $.when.apply($, invalidateDeferreds).then(function() {
                                deferred.resolve();
                            });
                        } else {
                            deferred.resolve();
                        }
                    } else {
                        deferred.resolve();
                    }
                }
            );
        }
        return deferred;
    },

    /**
     * set the compVar dataItem valid and recursively call setCompVarValid on set any descendants that become valid
     * @param compVar computed variable dataItem to set valid
     * @param dataSource
     */
    setCompVarValid: function(compVar, dataSource, controller, processDescendants, compVarData) {
        var self = this;
        var deferred = $.Deferred();

        if (!compVar.isValid()) { // just being defensive
            var computedVariable = compVarData;
            if (!computedVariable) {
                computedVariable = self.ComputedVariablesManager.getComputedVariableByName(compVar.name);
            }
            // console.log("setCompVarValid about to send request to server for " + compVar.name);

            // create computedvariable on the server
            var request = self.CompVarUtil.buildComputedVarRequest(computedVariable, dataSource);
            self.HprAPI.createComputedVariable(dataSource, [request]).done(function (computedVars) {
                if (computedVars.items[0].error === 0) {
                    // console.log(compVar.name + " is set valid on server");

                    // if the computed variable was created as invalid on this dataSource it will not have these properties set
                    compVar.label = compVar.generateLabel(computedVars.items[0]);
                    compVar.tooltip = compVar.generateTooltip(computedVars.items[0]);
                    compVar.format = computedVars.items[0].format;
                    compVar.dataType = computedVars.items[0].dataType;
                    compVar.setValid(true);
                    self.updateComputedVariableNodes(DataSourceCacheManager.getSelectedDataSource(), controller);

                    if (processDescendants) {
                        // revalidate all descendants
                        var descendantNames = []; // array of names of descendant computedvariables
                        self.CompVarUtil.getComputedVariableDescendants(descendantNames, computedVariable, self.ComputedVariablesManager.getComputedVariables(), dataSource);

                        if (descendantNames.length > 0) {
                            var descendants = [];
                            _.each(descendantNames, function(name) {
                                descendants.push(self.ComputedVariablesManager.getComputedVariableByName(name));
                            });
                            self.revalidateComputedVariables(dataSource.dataSource.id, descendants, controller).done(function() {
                                deferred.resolve();
                            })
                        } else { // no descendants, we're done
                            deferred.resolve();
                        }
                    } else { // not processing dependents, we're done
                        deferred.resolve();
                    }
                } else {
                    console.error("Failed to create computed variable on the server");
                    deferred.fail();
                }
            });
        } else { // was already valid.  This case should not happen
            deferred.resolve();
        }
        return deferred;
    },

    revalidateAllComputedVariables: function(dataSourceId, controller) {
        var self = this;
        var computedVariables = self.ComputedVariablesManager.getComputedVariables();
        return self.revalidateComputedVariables(dataSourceId, computedVariables, controller);
    },

    /**
     * toggles computed variables valid/invalid for the dataSource
     *
     * Called when something changes that could cause the validity of computed variables to change, such as
     * adding/deleting scenarios.
     *
     * @param dataSourceId
     */
    revalidateComputedVariables: function(dataSourceId, computedVariables, controller) {
        var self = this;
        var deferred = $.Deferred();

        if (computedVariables.length > 0) {
            DataSourceCacheManager.getDataSource(dataSourceId).done(function (dataSource) {
                var clonedCompVars = sas.analytics.riskhp.utils.HprUtil.hprClone(computedVariables);

                // sort clonedCompVars according to items.  Computed variables may be created or updated to be based on
                // other computed variables, so they need to be created on the server in order.
                clonedCompVars.sort(function (a, b) {
                    var aAncestors = [];  // array of names of computed variables used in the creation of computed variable a
                    var bAncestors = [];  // array of names of computed variables used in the creation of computed variable b

                    self.CompVarUtil.getComputedVariableAncestors(aAncestors, a, clonedCompVars, dataSource);
                    self.CompVarUtil.getComputedVariableAncestors(bAncestors, b, clonedCompVars, dataSource);

                    if (_.contains(aAncestors, b.name)) {
                        return 1;
                    } else if (_.contains(bAncestors, a.name)) {
                        return -1;
                    }
                    return 0;
                });

                function doTask(computedVariable, next) {
                    var compVar = dataSource.getDataItemByNameAndType(computedVariable.name, sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR);
                    var wasValid = compVar.isValid();
                    var isValid = self.CompVarUtil.isValidComputedVariable(dataSource, computedVariable);
                    if (wasValid && !isValid) {
                        // console.log("revalidateComputedVariables: setting invalid " + computedVariable.name);
                        sas.analytics.riskhp.views.datapane.DataPaneUtil.setCompVarInvalid(compVar, dataSource, controller, false).done(next);
                    } else if (!wasValid && isValid) {
                        // console.log("revalidateComputedVariables: setting valid " + computedVariable.name);
                        sas.analytics.riskhp.views.datapane.DataPaneUtil.setCompVarValid(compVar, dataSource, controller, false).done(next);
                    } else {
                        next();
                    }
                }

                function createTask(computedVariable) {
                    return function (next) {
                        doTask(computedVariable, next);
                    }
                }

                _.each(clonedCompVars, function (computedVariable) {
                    $(document).queue(self.HprConstants.HPR_QUEUE_REVALIDATE_COMP_VARS_TASKS, createTask(computedVariable));
                });

                // after all comp vars have been revalidated, refresh the cache of all crosstabs using the dataSource
                $(document).queue(self.HprConstants.HPR_QUEUE_REVALIDATE_COMP_VARS_TASKS, function () {
                    // wait for the comp vars to be created/deleted from the dataSource on the server, then mark the
                    // cache for the dataSource invalid
                    CacheManager.invalidate(dataSourceId);

                    // no need to wait for all of the crosstab caches to be refreshed
                    deferred.resolve();
                });

                $(document).dequeue(self.HprConstants.HPR_QUEUE_REVALIDATE_COMP_VARS_TASKS);

            });
        } else {
            deferred.resolve();
        }
        return deferred;
    },


    //*****************************************************************************************************
    //*****************************************************************************************************
    // Other helper functions
    //*****************************************************************************************************
    //*****************************************************************************************************


    /**
     * Given an array of Data Items, adds them to the Crosstab
     * @param crosstab
     * @param dataItems
     */
    addAllDataItemsToCrosstab: function(crosstab, dataItems) {
        var self = this;

        // Set Crosstab to busy state
        crosstab.setBusyIndicatorDelay(0);
        crosstab.setBusy(true);

        var dataItemType = dataItems[0].type;
        var otherDataItems = _.filter(dataItems, function (dataItem) {
            return (dataItem.type !== dataItemType);
        });

        // Are all the dataItems of the same type?
        if (otherDataItems.length === 0) {
            if (dataItemType === self.CLASSVAR) {
                crosstab.doDropRowHeader(dataItems);
            }
            else if (dataItemType === self.HIERARCHY) {
                if (dataItems.length === 1) {
                    crosstab.doDropRowHeader(dataItems); // Only support adding one hierarchy at a time.
                } else {
                    // Open an informational MessageDialog to alert the user why no data items were added
                    sas.hc.m.MessageDialog.information({
                        text: self.HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.add.selected.items.multiple.hierarchies.information.txt")
                    });
                }
            } else {
                crosstab.doDropColumnHeader(dataItems); // Don't pass 'info' object argument in this case
            }
        } else {
            // Open an informational MessageDialog to alert the user why no data items were added
            sas.hc.m.MessageDialog.information({
                text: self.HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.add.selected.items.information.txt")
            });
        }

        crosstab.setBusy(false);

        // Clear out the selections
        self.clearSelections();
    },

    getNodeDataItemId: function(node) {
        return node.dataItem ? node.dataItem.id : node.dataItemId;
    },

    /**
     * Determines if all dataItems passed in can be deleted.  Currently the method only returns true if all dataItems
     * are Scenarios and have valid id's.
     * @param dataItems An Array of data items
     * @returns {boolean} true if all dataItems are Scenarios and have a valid id; false, otherwise
     */
    areDataItemsDeletable: function(dataItems) {
        var self = this;

        if (!dataItems || dataItems.length === 0) {
            return false;
        }
        return _.every(dataItems, function (dataItem) {
            return dataItem.type === self.SCENARIO && dataItem.id >= 0;   //TODO include computed variables?
        });
    }
};