// (c) 2017, SAS Institute Inc.

jQuery.sap.declare("sas.analytics.riskhp.views.datapane.HprDataPaneContextMenuProvider");

/**
 * @name sas.analytics.riskhp.views.datapane.HprDataPaneContextMenuProvider
 */
(function() {
    "use strict";

    jQuery.sap.require("sas.analytics.riskhp.models.DataSourceCacheManager");
    jQuery.sap.require("sas.analytics.riskhp.models.ComputedVariablesManager");
    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Scenario");
    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Classvar");
    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Compvar");
    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Benchvar");
    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.ResultSet");
    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Hierarchy");
    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Horizon");
    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.RelativeHorizon");
    jQuery.sap.require("sas.analytics.riskhp.settings.ApplicationSettings");
    jQuery.sap.require("sas.analytics.riskhp.utils.ExplorationUtil");
    jQuery.sap.require("sas.analytics.riskhp.utils.HprAPI");
    jQuery.sap.require("sas.analytics.riskhp.utils.HprApplicationSwitcherController");
    jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
    jQuery.sap.require("sas.analytics.riskhp.utils.ViewManager");
    jQuery.sap.require("sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil");
    jQuery.sap.require("sas.analytics.riskhp.views.datapane.DataPaneModelManager");
    jQuery.sap.require("sas.analytics.riskhp.views.datapane.DataPaneUtil");
    jQuery.sap.require("sas.hc.m.BusyDialog");
    jQuery.sap.require("sas.hc.ui.unified.Menu");
    jQuery.sap.require("sas.hc.ui.unified.MenuItem");
    jQuery.sap.require("sas.analytics.riskhp.models.DisplayRulesManager");

    var DataSourceCacheManager = sas.analytics.riskhp.models.DataSourceCacheManager;
    var ComputedVariablesManager = sas.analytics.riskhp.models.ComputedVariablesManager;
    var DataPaneModelManager = sas.analytics.riskhp.views.datapane.DataPaneModelManager;
    var DataPaneUtil = sas.analytics.riskhp.views.datapane.DataPaneUtil;
    var HprUtil = sas.analytics.riskhp.utils.HprUtil;
    var HprAPI = sas.analytics.riskhp.utils.HprAPI;
    var ExplorationUtil = sas.analytics.riskhp.utils.ExplorationUtil;
    var HprApplicationSwitcherController = sas.analytics.riskhp.utils.HprApplicationSwitcherController;
    var ViewManager = sas.analytics.riskhp.utils.ViewManager;
    var ComputedVariableUtil = sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil;
    var ApplicationSettings = sas.analytics.riskhp.settings.ApplicationSettings;
    var CLASSVAR = sas.analytics.riskhp.models.dataitems.Classvar.CLASSVAR;
    var SCENARIO = sas.analytics.riskhp.models.dataitems.Scenario.SCENARIO;
    var COMPVAR = sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR;
    var BENCHVAR = sas.analytics.riskhp.models.dataitems.Benchvar.BENCHVAR;
    var HIERARCHY = sas.analytics.riskhp.models.dataitems.Hierarchy.HIERARCHY;
    var RESULTSET = sas.analytics.riskhp.models.dataitems.ResultSet.RESULTSET;
    var HORIZON = sas.analytics.riskhp.models.dataitems.Horizon.HORIZON;
    var RELATIVEHORIZON = sas.analytics.riskhp.models.dataitems.RelativeHorizon.RELATIVEHORIZON;

    /**
     * @typedef {Object} ScenarioUpdatedObject - An object passed to listeners that indicates a scenario has been updated
     * @property {string} dataSourceId - The id of the datasource where the scenario is attached to
     * @property {string} scenarioId - The id of the scenario that was updated
     * @property {string[]} addedHorizonIds - Ids of horizons that were added as a result of the update
     * @property {string[]} deletedHorizonIds - Ids of horizons that were removed as a result of the update
     */

    /**
     * @typedef {Object} ScenarioAddedObject - An object passed to listeners that indicates a scenario(s) was added
     * @property {string} dataSourceId - The id of the datasource where the scenarios were attached to
     * @property {string[]} scenarioIds - The ids of the scenarios that were added
     */

    /**
     * @typedef {Object} ScenarioDeletedObject - An object passed to listeners that indicates a scenario(s) was deleted
     * @property {string} dataSourceId - The id of the datasource where the scenarios were attached to
     * @property {string[]} scenarioIds - The ids of the scenarios that were deleted
     */

    sap.ui.base.Object.extend("sas.analytics.riskhp.views.datapane.HprDataPaneContextMenuProvider", {
        metadata: {
            publicMethods: [
                'buildContextMenu'
            ]
        },

        constructor: function () {},

        //*****************************************************************************************************
        //*****************************************************************************************************
        // Public methods
        //*****************************************************************************************************
        //*****************************************************************************************************

        /**
         * Builds the context menu for the TreeNode passed in based on the type of node it is.  The context menu is
         * added to the oMenu and the oMenu is opened if it has menu items.
         * @param oMenu
         * @param oTableTreeNode
         * @param oTableCellDomRef
         * @param controller
         */
        buildContextMenu: function(oMenu, oTableTreeNode, oTableCellDomRef, controller) {
            var self = this;
            var topLevelFolderMenuItems = []; // Menu items for the top-level folder nodes
            var multipleSelectionMenuItems = []; // Menu items if there are multiple selections in the TreeTable
            var dataItemMenuItems = []; // Menu items for the current TreeTable node representing the data item the user right-clicked on
            var defaultDataItemMenuItems = []; // Menu items which are consistent across the different data item types

            // Gather some information about the TreeTable node
            var sTableTreeNodeFolderType = oTableTreeNode.folderType;
            var sTableTreeNodeNodeType = oTableTreeNode.nodeType;

            // Don't display a context menu if the tree node is deemed un-selectable and is a Class Variable (NOTE: This takes care
            // of Class Variable data items underneath the Hierarchy folder, we don't want a context menu for those); Other tree nodes
            // which are deemed un-selectable do need a context menu displayed though unfortunately
            if (!oTableTreeNode.isHprSelectable && sTableTreeNodeNodeType === CLASSVAR) {
                return;
            }

            // Keep track of the selected data items
            var selectedDataItems = DataPaneUtil.getSelectedDataItems();

            // Determine if the Data Pane TreeTable has a filtered selection model
            var filteredModel = DataPaneUtil.getDataPaneTreeTableSelectionModel().isFilteredModel();

            // Determine if the TreeNode we right-clicked on is selected
            var isTreeNodeSelected = _.contains(selectedDataItems, oTableTreeNode.dataItem);

            // Gather the appropriate Menu Items for the TreeNode which was passed in based on the type of node; Type
            // of node here implies whether it is a top-level node and the type of Data Item the TreeNode represents
            if (sTableTreeNodeNodeType === HprConstants.HPR_TREETABLE_LEVEL_TOP) {
                if (!filteredModel) {
                    // Only display top-level folder context menus if selection model is not filtered
                    if (sTableTreeNodeFolderType === CLASSVAR) {
                        topLevelFolderMenuItems = self._getClassVarFolderContextMenuItems(oTableTreeNode);
                    } else if (sTableTreeNodeFolderType === SCENARIO) {
                        topLevelFolderMenuItems = self._getScenariosFolderContextMenuItems(oTableTreeNode, controller);
                    } else if (sTableTreeNodeFolderType === COMPVAR) {
                        topLevelFolderMenuItems = self._getComputedvariablesFolderContextMenuItems(oTableTreeNode, controller);
                    } else if (sTableTreeNodeFolderType !== HIERARCHY) {
                        // All other top-level folders except Hierarchy
                        topLevelFolderMenuItems = self._getDefaultFolderContextMenuItems(oTableTreeNode, controller, sTableTreeNodeFolderType);
                    }
                }
            } else {
                // Data Item categories context menus; Display these Data Item category context menus if the node we
                // right-clicked on is not selected or we have one selection and the current node is it
                if (!isTreeNodeSelected || (isTreeNodeSelected && selectedDataItems.length === 1)) {
                    if (sTableTreeNodeNodeType === CLASSVAR) {
                        dataItemMenuItems = self._getClassVarHierarchyContextMenuItems(oTableTreeNode);
                    } else if (sTableTreeNodeNodeType === HIERARCHY) {
                        dataItemMenuItems = self._getClassVarHierarchyContextMenuItems(oTableTreeNode);
                    } else if(sTableTreeNodeNodeType === RESULTSET) {
                        dataItemMenuItems = self._getResultSetContextMenuItems(oTableTreeNode, controller);
                    } else if (sTableTreeNodeNodeType === SCENARIO) {
                        dataItemMenuItems = self._getScenarioContextMenuItems(oTableTreeNode, controller);
                    } else if (sTableTreeNodeNodeType === COMPVAR) {
                        dataItemMenuItems = self._getComputedVariableContextMenuItems(oTableTreeNode, controller);
                    } else {
                        dataItemMenuItems = self._getDefaultAddToCrosstabContextMenuItems(oTableTreeNode);
                    }
                }
            }

            // Multiple selections (but not including top-level folders), need to add more menu items for them
            if (sTableTreeNodeNodeType !== HprConstants.HPR_TREETABLE_LEVEL_TOP && selectedDataItems.length > 1) {
                multipleSelectionMenuItems = self._getMultipleSelectionContextMenu(oTableTreeNode, isTreeNodeSelected, selectedDataItems);
            }

            // Add the default data item menus (but not on top-level folders)
            if (sTableTreeNodeNodeType !== HprConstants.HPR_TREETABLE_LEVEL_TOP) {
                // Need to determine if the 'Rename' menu item should start a new section for the menu
                var bShouldRenameStartSection = topLevelFolderMenuItems.length > 0 || dataItemMenuItems.length > 0 || multipleSelectionMenuItems.length > 0;
                defaultDataItemMenuItems = self._getDefaultDataItemMenuItems(bShouldRenameStartSection, oTableTreeNode, oTableCellDomRef, isTreeNodeSelected, selectedDataItems, filteredModel, controller);
            }

            // If there are menu items to display, add them to the Menu and open the Menu
            if (topLevelFolderMenuItems.length > 0 || dataItemMenuItems.length > 0 || multipleSelectionMenuItems.length > 0 || defaultDataItemMenuItems.length > 0) {

                // Make sure the Menu and the Sub-Menu items are closed after every selection
                oMenu.attachItemSelect(function() {
                    this.close();
                });

                // Add the menu items for the top-level folder nodes
                _.each(topLevelFolderMenuItems, function (menuItem) {
                    oMenu.addItem(menuItem);
                });

                // Add the menu items for the current TreeTable node
                _.each(dataItemMenuItems, function (menuItem) {
                    oMenu.addItem(menuItem);
                });

                // Add the menu items which represent multiple selections in the TreeTable
                _.each(multipleSelectionMenuItems, function (menuItem) {
                    // Don't add the menuItem here if it is marked to be moved, it will be taken care of next
                    if (!menuItem.moveMenu) {
                        oMenu.addItem(menuItem);
                    }
                });

                // Loop back over the multiple selection menu items and move the ones marked for move; Have to do this for some of the
                // data items so the menus will look ok and not unorganized (i.e. 'Add' menu items in totally different positions in menu)
                _.each(multipleSelectionMenuItems, function (menuItem) {
                    if (menuItem.moveMenu && oMenu.getItems().length > 0) {
                        oMenu.insertItem(menuItem, 1); // Insert at the second position
                    } else if (menuItem.moveMenu && oMenu.getItems().length === 0) {
                        oMenu.addItem(menuItem); // Just add it
                    }
                });

                // Add the default data item menu items
                _.each(defaultDataItemMenuItems, function (menuItem) {
                    oMenu.addItem(menuItem);
                });

                var eDock = sap.ui.core.Popup.Dock;
                oMenu.open(false, oTableCellDomRef, eDock.BeginCenter, eDock.CenterBottom, oTableCellDomRef, "none none");
            }
        },

        //*****************************************************************************************************
        //*****************************************************************************************************
        // ContextMenu helper functions
        //*****************************************************************************************************
        //*****************************************************************************************************

        /**
         * Creates an Array of MenuItems for actions which are consistent across the different data item types
         * @param bShouldRenameStartSection
         * @param node
         * @param oTableCellDomRef
         * @param isTreeNodeSelected
         * @param selectedDataItems
         * @param filteredModel
         * @param controller
         * @returns {Array} An Array of MenuItems
         * @private
         */
        _getDefaultDataItemMenuItems: function(bShouldRenameStartSection, node, oTableCellDomRef, isTreeNodeSelected, selectedDataItems, filteredModel, controller) {
            var self = this;
            var menuItems = [];

            // How many selected data items do we have
            var numberOfSelectedDataItems = selectedDataItems ? selectedDataItems.length : 0;

            // Determine if node represents a BaseCase Horizon
            var isDataItemBaseCaseHorizon = (node.dataItem.getType() === HORIZON || node.dataItem.getType() === RELATIVEHORIZON) && ($.isFunction(node.dataItem.isBasecase) && node.dataItem.isBasecase());

            // Rename menu item is enabled for non-basecase Horizons && if the number of selected data items is 1 or 0 (this is because controller.onDataItemRenameHelper function will work with node.dataItem) or
            // if the number of selected data items is > 1 and the tree node we right-clicked on is not selected (i.e. user should be able to rename a data item that is not selected)
            var isRenameMenuItemEnabled = !isDataItemBaseCaseHorizon && (numberOfSelectedDataItems <= 1 || (numberOfSelectedDataItems > 1 && !isTreeNodeSelected));
            menuItems.push(self._createMenuItem(
                HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.data.item.rename.txt"),
                function() {
                    controller.onDataItemRenameHelper(node.dataItem);
                },
                bShouldRenameStartSection,
                isRenameMenuItemEnabled
            ));

            // Properties menu item is enabled if the number of selected data items is 1 or 0 (this is because controller.openDataItemPropertiesDialogHelper function will work with node.dataItem) or
            // if the number of selected data items is > 1 and the tree node we right-clicked on is not selected (i.e. user should be able to view properties for a data item that is not selected)
            var isPropertiesMenuItemEnabled = numberOfSelectedDataItems <= 1 || (numberOfSelectedDataItems > 1 && !isTreeNodeSelected);
            menuItems.push(self._createMenuItem(
                HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.data.item.properties.txt"),
                function() {
                    controller.openDataItemPropertiesDialogHelper(node.dataItem);
                },
                undefined,
                isPropertiesMenuItemEnabled
            ));

            // Update count menu item is enabled if any of the selected data items are Class Variables and its 'updateCount' flag is true
            var dataItemsForUpdateCountMenu = [];
            if (numberOfSelectedDataItems <= 1 || (numberOfSelectedDataItems > 1 && !isTreeNodeSelected)) {
                dataItemsForUpdateCountMenu.push(node.dataItem);
            } else {
                dataItemsForUpdateCountMenu = [].concat(selectedDataItems);
            }
            var classVarThatNeedsCountUpdated = _.filter(dataItemsForUpdateCountMenu, function(dataItem) {
                return (dataItem.getType() === CLASSVAR && ($.isFunction(dataItem.getUpdateCount) && dataItem.getUpdateCount()));
            });
            var shouldUpdateCountBeEnabled = classVarThatNeedsCountUpdated.length > 0;
            menuItems.push(self._createMenuItem(
                HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.data.item.updateCount.txt"),
                function() {
                    controller.onDataItemUpdateCountSelect(classVarThatNeedsCountUpdated);
                },
                undefined,
                shouldUpdateCountBeEnabled
            ));

            // Move menu items are enabled/disabled based on a lot of logic and conditions
            var isMoveToTopMenuItemEnabled = false;
            var isMoveUpMenuItemEnabled = false;
            var isMoveToBottomMenuItemEnabled = false;
            var isMoveDownMenuItemEnabled = false;

            // The row path string of the node the user right-clicked on; Only used for one selected node or right-click on un-selected nodes
            var rowPath = undefined;

            if (numberOfSelectedDataItems > 1 && isTreeNodeSelected) { // More than one row selected and right-clicked on one that is selected
                // For this case just leave the 'isMoveUpMenuItemEnabled' and 'isMoveDownMenuItemEnabled' disabled (no idea where to move
                // selected items too... biggest problem is when selected items are not continuous);
                // The Data Pane Tree selection model should be un-filtered too
                isMoveToTopMenuItemEnabled = DataPaneUtil.canAllSelectedNodesBeMoved(true) && !filteredModel;
                isMoveToBottomMenuItemEnabled = DataPaneUtil.canAllSelectedNodesBeMoved() && !filteredModel;
            } else {
                // Determine the rowPath by using the table row that right-click happened on
                var rowId = oTableCellDomRef.id;
                var rowData = DataPaneUtil.getRowData(rowId);
                rowPath = rowData.path;

                // Get the parent folders path and actual parents folder from the model
                var parentsPath = DataPaneUtil.getParentsPathFromPath(rowPath);
                var parentsNode = rowData.model.getProperty(parentsPath);

                // Move menus are disabled for Base Case Horizons inside of Result Set folder and if the selection model is filtered
                var areMoveMenuItemsEnabled = !(isDataItemBaseCaseHorizon && parentsNode.nodeType === RESULTSET) && !filteredModel;

                // Appropriate move menus are also disabled if node is first or last child in the parent's folder
                isMoveToTopMenuItemEnabled = areMoveMenuItemsEnabled && !DataPaneUtil.isRowIdFirstChildOfFolder(rowId, null);
                isMoveToBottomMenuItemEnabled = areMoveMenuItemsEnabled && !DataPaneUtil.isRowIdLastChildOfFolder(rowId, null);
                isMoveUpMenuItemEnabled = isMoveToTopMenuItemEnabled;
                isMoveDownMenuItemEnabled = isMoveToBottomMenuItemEnabled;
            }

            menuItems.push(self._createMenuItem(
                HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.data.item.moveToTop.txt"),
                function() {
                    controller.onDataItemContextMenuMove("MOVETOTOP", rowPath);
                },
                true,
                isMoveToTopMenuItemEnabled
            ));

            menuItems.push(self._createMenuItem(
                HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.data.item.moveUp.txt"),
                function() {
                    controller.onDataItemContextMenuMove("MOVEUP", rowPath);
                },
                undefined,
                isMoveUpMenuItemEnabled
            ));

            menuItems.push(self._createMenuItem(
                HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.data.item.moveDown.txt"),
                function() {
                    controller.onDataItemContextMenuMove("MOVEDOWN", rowPath);
                },
                undefined,
                isMoveDownMenuItemEnabled
            ));

            menuItems.push(self._createMenuItem(
                HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.data.item.moveToBottom.txt"),
                function() {
                    controller.onDataItemContextMenuMove("MOVETOBOTTOM", rowPath);
                },
                undefined,
                isMoveToBottomMenuItemEnabled
            ));

            return menuItems;
        },

        /**
         * Creates an Array of MenuItems for multiple selections in the TreeTable
         * @param node
         * @param isTreeNodeSelected
         * @param selectedDataItems
         * @returns {Array} An Array of MenuItems
         * @private
         */
        _getMultipleSelectionContextMenu: function(node, isTreeNodeSelected, selectedDataItems) {
            var self = this;
            var multipleSelectionMenuItems = [];

            // Get the RiskPanelsViewDelegate to use
            var selectedRiskPanelsView = HprUtil.getSelectedRiskPanelsView();
            var riskPanelsViewDelegate = selectedRiskPanelsView.getDelegate();

            // If the current node is not selected, just return an empty array; No multiple selection menu in that case
            if (!isTreeNodeSelected) {
                return [];
            }

            // Retrieve the list of crosstabIds that are valid for the datapane
            var validCrosstabIds = self._getValidCrosstabIds(node, selectedDataItems);

            // If no Crosstabs are valid for the date pane bail out
            if (validCrosstabIds.length === 0) {
                return [];
            }

            // Create the root menu items (NOTE: text filled in later)
            var multipleSelectionMenuItem = new sas.hc.ui.unified.MenuItem({});

            // If only one crosstab was valid there is no need to show the crosstab submenu
            if (validCrosstabIds.length === 1) {
                var crosstab = sap.ui.getCore().byId(validCrosstabIds[0]);
                var crosstabTitle = riskPanelsViewDelegate.getTitleForContent(validCrosstabIds[0]);

                if (crosstab && crosstab.context) {
                    // Set the text of the menu item
                    multipleSelectionMenuItem.setText(HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.add.selected.items.to.one.crosstab.fmt", [crosstabTitle]));

                    // Create a Select event for the root menu item
                    multipleSelectionMenuItem.attachSelect($.proxy(function () {
                        DataPaneUtil.addAllDataItemsToCrosstab(crosstab, selectedDataItems);
                    }, self));
                }
            } else {
                // If there is more than one valid crosstab we need to create a crosstab submenu
                var subMenu = new sas.hc.ui.unified.Menu();

                _.each(validCrosstabIds, function(id){
                    var crosstab = sap.ui.getCore().byId(id);

                    if (crosstab && crosstab.context) {
                        var subMenuItem = new sas.hc.ui.unified.MenuItem({
                            text: riskPanelsViewDelegate.getTitleForContent(id)
                        });
                        subMenuItem.attachSelect(function () {
                            DataPaneUtil.addAllDataItemsToCrosstab(crosstab, selectedDataItems);
                        });
                        subMenu.addItem(subMenuItem);
                    }
                });

                // Set the text of the root menu items
                multipleSelectionMenuItem.setText(HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.add.selected.items.to.multiple.crosstab.txt"));

                // Set the submenu for the Add All root menu
                multipleSelectionMenuItem.setSubmenu(subMenu);
            }

            // For Scenarios and Computed Variables, we want to move the multiSelection menu to a different location before display, so mark it
            if (node.dataItem && (node.dataItem.getType() === SCENARIO
                    || node.dataItem.getType() === COMPVAR)) {
                multipleSelectionMenuItem.moveMenu = true;
            }

            multipleSelectionMenuItems.push(multipleSelectionMenuItem);

            return multipleSelectionMenuItems;
        },

        /**
         * Creates an Array of default MenuItems which will be used to populate a context-menu for a folder node
         * @param node
         * @param controller
         * @param folderType
         * @returns {Array} An Array of default MenuItems
         * @private
         */
        _getDefaultFolderContextMenuItems: function(node, controller, folderType) {
            var self = this;
            var menuItems = [];

            // for scenarios folder, the immediate children are actually result sets
            var dataItemsType = folderType === SCENARIO ? RESULTSET : folderType;

            // Retrieve the list of Data Items pertaining to the type
            var dataSource = DataPaneModelManager.getSelectedDataSource();
            var dataItems = dataSource.getDataItemsForType(dataItemsType);

            // Get the RiskPanelsViewDelegate to use
            var selectedRiskPanelsView = HprUtil.getSelectedRiskPanelsView();
            var riskPanelsViewDelegate = selectedRiskPanelsView.getDelegate();

            // Retrieve the list of crosstabIds that are valid for the datapane
            var validCrosstabIds = self._getValidCrosstabIds(node, dataItems);

            // If no Crosstabs are valid for the date pane bail out
            if (validCrosstabIds.length === 0) {
                return [];
            }

            // Create the Add All and Remove All root menu items (NOTE: text filled in later)
            var addAllDefaultRootMenuItem = new sas.hc.ui.unified.MenuItem({
                enabled: dataItems && dataItems.length > 0,
                startsSection: folderType === SCENARIO
            });
            // TODO: dacagl - Remove for now
            // var removeAllDefaultRootMenuItem = new sas.hc.ui.unified.MenuItem({
            //     enabled: false
            // });

            // If only one crosstab was valid there is no need to show the crosstab submenu
            if (validCrosstabIds.length === 1) {
                // This crosstab variable is lifted to method scope
                var crosstab = sap.ui.getCore().byId(validCrosstabIds[0]);
                var crosstabTitle = riskPanelsViewDelegate.getTitleForContent(validCrosstabIds[0]);

                if (crosstab && crosstab.context) {
                    // Set the text of the root menu items
                    addAllDefaultRootMenuItem.setText(HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.addall.to.one.crosstab.fmt", [HprUtil.dataTypeTitle(folderType), crosstabTitle]));
                    //removeAllDefaultRootMenuItem.setText(HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.removeall.from.one.crosstab.fmt", [HprUtil.dataTypeTitle(folderType), crosstabTitle]));

                    // Create a Select event for the Add All root menu item
                    addAllDefaultRootMenuItem.attachSelect($.proxy(function () {
                        DataPaneUtil.addAllDataItemsToCrosstab(crosstab, dataItems);
                    }, self));
                }
            } else {
                // If there is more than one valid crosstab we need to create a crosstab submenu
                var subMenu = new sas.hc.ui.unified.Menu();

                _.each(validCrosstabIds, function(id){
                    // This crosstab variable is lifted to this anonymous function
                    var crosstab = sap.ui.getCore().byId(id);

                    if (crosstab && crosstab.context) {
                        var subMenuItem = new sas.hc.ui.unified.MenuItem({
                            text: riskPanelsViewDelegate.getTitleForContent(id)
                        });
                        subMenuItem.attachSelect(function () {
                            DataPaneUtil.addAllDataItemsToCrosstab(crosstab, dataItems);
                        });
                        subMenu.addItem(subMenuItem);
                    }
                });

                // Set the text of the root menu items
                addAllDefaultRootMenuItem.setText(HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.addall.to.multiple.crosstab.fmt", [HprUtil.dataTypeTitle(folderType)]));
                //removeAllDefaultRootMenuItem.setText(HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.removeall.from.multiple.crosstab.fmt", [HprUtil.dataTypeTitle(folderType)]));

                // Set the submenu for the Add All root menu
                addAllDefaultRootMenuItem.setSubmenu(subMenu);
            }

            // add folder menu option - Note: Not available in November 2017 release - MCA
            // var menuItem = new sas.hc.ui.unified.MenuItem({
            //     text: HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.addfolder.txt"),
            //     startsSection: true
            // });
            // menuItem.attachSelect($.proxy(function() {
            //     controller.addFolder();
            // }, this));
            // menuItems.push(menuItem);

            menuItems.push(addAllDefaultRootMenuItem);
            //menuItems.push(removeAllDefaultRootMenuItem);

            return menuItems;
        },

        /**
         * Returns the default context menu items for adding data items to a Crosstab
         * @param node
         * @returns {Array}
         */
        _getDefaultAddToCrosstabContextMenuItems: function(node) {
            var self = this;
            var menuItems = [];

            // Get the RiskPanelsViewDelegate to use
            var selectedRiskPanelsView = HprUtil.getSelectedRiskPanelsView();
            var riskPanelsViewDelegate = selectedRiskPanelsView.getDelegate();

            // Retrieve the list of crosstabIds that are valid for the datapane
            var validCrosstabIds = self._getValidCrosstabIds(node);

            // If no Crosstabs are valid for the date pane bail out
            if (validCrosstabIds.length === 0) {
                return [];
            }

            // For the menu item displayed from the DataPane, use the ApplicationSettings showLabels property so
            // the menu item will match the DataPane.  The submenus will be populated using the showLabels option
            // value from the Crosstab
            var showLabels = ApplicationSettings.getShowLabels();

            // Create the root menu item (NOTE: text filled in later)
            var menuItem = new sas.hc.ui.unified.MenuItem({});

            // If only one crosstab was valid there is no need to show the crosstab submenu
            if (validCrosstabIds.length === 1) {
                // This crosstab variable is lifted to method scope
                var crosstab = sap.ui.getCore().byId(validCrosstabIds[0]);
                var crosstabTitle = riskPanelsViewDelegate.getTitleForContent(validCrosstabIds[0]);

                if (crosstab && crosstab.context) {
                    // Set the text of the root menu item
                    menuItem.setText(HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.add.to.one.crosstab.fmt", [node.dataItem.getDisplayLabel(showLabels), crosstabTitle]));

                    // Construct an appropriate context menu for the Data Item based on Crosstab logic
                    menuItem.attachSelect($.proxy(function () {
                        DataPaneUtil.addAllDataItemsToCrosstab(crosstab, [node.dataItem]);
                    }, self));
                }
            }
            else {
                // If there is more than one valid crosstab we need to create a crosstab submenu
                var subMenu = new sas.hc.ui.unified.Menu();

                _.each(validCrosstabIds, function(id){
                    // This crosstab variable is lifted to this anonymous function
                    var crosstab = sap.ui.getCore().byId(id);

                    if (crosstab && crosstab.context) {
                        var subMenuItem = new sas.hc.ui.unified.MenuItem({
                            text: riskPanelsViewDelegate.getTitleForContent(id)
                        });
                        subMenuItem.attachSelect($.proxy(function () {
                            DataPaneUtil.addAllDataItemsToCrosstab(crosstab, [node.dataItem]);
                        }, self));
                        subMenu.addItem(subMenuItem);
                    }
                });

                // Set the text and submenu of the root menu item
                menuItem.setText(HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.add.to.multiple.crosstab.fmt", [node.dataItem.getDisplayLabel(showLabels)]));
                menuItem.setSubmenu(subMenu);
            }

            menuItems.push(menuItem);

            return menuItems;
        },

        /**
         * Create a 'sas.hc.ui.unified.MenuItem' with the properties passed in
         * @param label
         * @param selectHandler
         * @param startsSection
         * @param enabled
         * @returns {sas.hc.ui.unified.MenuItem}
         * @private
         */
        _createMenuItem: function(label, selectHandler, startsSection, enabled) {
            var menuItem = new sas.hc.ui.unified.MenuItem({
                text: label,
                select: selectHandler,
                startsSection: startsSection,
                enabled: enabled
            });
            return menuItem;
        },

        /**
         * Returns the ids of the Crosstabs in the currently selected RiskPanelsView that have the same Data Source id as
         * the currently selected one in the DataPaneModelManager.  Also, a Crosstab is deemed valid if it does not contain
         * all of the "dataItems" we are looking for.
         * @param node The node that was selected.  Could be a single TreeNode corresponding to a Data Item or a folder node
         * @param dataItems (Optional)  An array of Data Items usually filled in with the children of a folder node or selected
         * data items
         * @returns [] Returns an Array of valid Crosstab Ids or an empty Array if none are found
         * @private
         */
        _getValidCrosstabIds: function(node, dataItems) {
            var self = this;

            // If there is not a node to work with, return an empty Array
            if (!node) {
                return [];
            }

            // Since we can be working with a single node or a folder node full of Data Items here,
            // make sure our 'dataItems' Array has a similar structure for the single node case
            if (!dataItems) {
                dataItems = [node.dataItem];
            }

            // Since the Scenarios top level folder contains Result Sets, we need to check whether the Scenarios and Horizons of each Result Set
            // is already contained in the Crosstab in order to consider it valid
            if (node.nodeType === HprConstants.HPR_TREETABLE_LEVEL_TOP && node.folderType === SCENARIO) {
                var resultSetScenariosAndHorizons = [];
                // Loop through each dataItem (in this case Result Sets) and build an array containing each Scenario and Horizon
                _.each(dataItems, function (dataItem) {
                    if (dataItem && dataItem.scenarios) {
                        resultSetScenariosAndHorizons = resultSetScenariosAndHorizons.concat(dataItem.scenarios);
                    }
                    if (dataItem && dataItem.horizon) {
                        resultSetScenariosAndHorizons = resultSetScenariosAndHorizons.concat(dataItem.horizon);
                    }
                });
                // Build the big array of Data Items to search for in order to determine if the Crosstab is valid
                dataItems = dataItems.concat(resultSetScenariosAndHorizons);
            }
            // For the Result Sets node, we need to check its Scenarios and Horizons to see if the Crosstab is valid
            else if (node.nodeType === RESULTSET) {
                if (node.dataItem && node.dataItem.scenarios) {
                    dataItems = dataItems.concat(node.dataItem.scenarios);
                }
                if (node.dataItem && node.dataItem.horizon) {
                    dataItems = dataItems.concat(node.dataItem.horizon);
                }
            }

            // Gather all the Crosstab Ids on the selected RiskPanelsView
            var selectedRiskPanelsView = HprUtil.getSelectedRiskPanelsView();
            var riskPanelsViewDelegate = selectedRiskPanelsView.getDelegate();
            var crosstabIds = riskPanelsViewDelegate.crosstabIds;

            // Filter down all the Crosstab Ids to only valid ones
            return _.filter(crosstabIds, function(crosstabId) {
                var crosstab = sap.ui.getCore().byId(crosstabId);
                if (crosstab && crosstab.context) {
                    // This is a valid Crosstab if all the "dataItems" we are looking for are NOT already in the Crosstab (this spans all types of Data Items); Also, the
                    // DataSource for the Crosstab has to be the same as the DataPane
                    return crosstab.isModelValid() && self._areAllDataItemsInCrosstab(crosstab, dataItems) === false && crosstab.context.getDataSourceId() === DataPaneModelManager.getSelectedDataSourceId();
                }
                return false; // If Crosstab does not have a context, it is not valid
            });
        },

        /**
         * Helper method that determines if all "dataItems" exist in the Crosstab.  The "dataItems" array can contain any type of Data Item, but
         * only Class Variables, Computed Variables, and Benchmark Variables are checked at this time.  For the other types of Data Items, this
         * method currently does not care if they already exist in the Crosstab and always returns false.
         * @param crosstab The Crosstab we are currently working with
         * @param dataItems An array of Data Items usually filled in with the children of a folder node or selected data items
         * @returns {boolean} Returns true or false for Class Variables, Computed Variables, and Benchmark Variables if all 'dataItems' exist in the Crosstab;
         * false is returned for all other types of Data Items
         * @private
         */
        _areAllDataItemsInCrosstab: function(crosstab, dataItems) {
            // Group the data items passed in by the 'type' property; This signifies the "selected" data items the user is trying to add to the Crosstab
            var selectedDataItemsGroupedByType = _.groupBy(dataItems, "type");

            // Helper method to determine if all data items passed in exist in big array representing Crosstab Class Vars or Tuples
            var _areAllDataItemsInCrosstabHelper = function(aDataItems, currentCrosstabDataItemsAsOneArray) {
                // NOTE: _.every returns true if length of aDataItems === 0; Will return true if every Data Item in aDataItems exist in currentCrosstabDataItemsAsOneArray
                return _.every(aDataItems, function(dataItem) {
                    return _.findWhere(currentCrosstabDataItemsAsOneArray, {id: dataItem.getId()}) !== undefined;
                });
            };

            // Gather the Class Variables currently used in the Crosstab and the currently selected Class Variables (if any)
            var currentCrosstabClassVars = crosstab.context.getClassVariables();
            var currentSelectedClassVars = _.isUndefined(selectedDataItemsGroupedByType[CLASSVAR]) ? [] : selectedDataItemsGroupedByType[CLASSVAR];
            // If lengths match, we have all Class Vars selected, just return value here of whether all the selected Class Vars exist in Crosstab
            if (dataItems.length === currentSelectedClassVars.length) {
                return _areAllDataItemsInCrosstabHelper(currentSelectedClassVars, currentCrosstabClassVars);
            }

            // For all other types of Data Items, we need to ask the crosstab for all the other Data Items it has in it's tuples (with no duplicates)
            var crosstabTuplesAsOneArray = crosstab.context.getTuplesAsOneArray();

            // Gather the selected Computed Variables (if any)
            var currentSelectedCompVars = _.isUndefined(selectedDataItemsGroupedByType[COMPVAR]) ? [] : selectedDataItemsGroupedByType[COMPVAR];
            // If lengths match, we have all Comp Vars selected, just return value here of whether all the selected Comp Vars exist in Crosstab
            if (dataItems.length === currentSelectedCompVars.length) {
                return _areAllDataItemsInCrosstabHelper(currentSelectedCompVars, crosstabTuplesAsOneArray);
            }

            // Gather the selected Benchmark Variables (if any)
            var currentSelectedBenchVars = _.isUndefined(selectedDataItemsGroupedByType[BENCHVAR]) ? [] : selectedDataItemsGroupedByType[BENCHVAR];
            // If lengths match, we have all Bench Vars selected, just return value here of whether all the selected Bench Vars exist in Crosstab
            if (dataItems.length === currentSelectedBenchVars.length) {
                return _areAllDataItemsInCrosstabHelper(currentSelectedBenchVars, crosstabTuplesAsOneArray);
            }

            // For all other types of data items, we do not care how many times they are present in the Crosstab so return false
            return false;
        },

        /**
         * Given a folderType removes all the Data Items of that type from a Crosstab
         * @param node
         * @param folderType
         */
        _removeAllDataItemsFromCrosstab: function(node, folderType) {
            // TODO: dacagl - Implement this
            console.log("removeAll: " + folderType);
        },

        _clearSelectionsHelper: function() {
            // Clear out the selections
            DataPaneUtil.clearSelections();
        },

        //*****************************************************************************************************
        //*****************************************************************************************************
        // Class Variables helper functions
        //*****************************************************************************************************
        //*****************************************************************************************************

        /**
         * Creates an Array of MenuItems which will be used to populate the context-menu for the Class Variables folder node
         * @param node
         * @returns {Array} An Array of menu items
         * @private
         */
        _getClassVarFolderContextMenuItems: function(node) {
            var self = this;
            var menuItems = [];
            var folderType = CLASSVAR;

            // Retrieve the list of Data Items pertaining to the type
            var dataSource = DataPaneModelManager.getSelectedDataSource();
            var dataItems = dataSource.getDataItemsForType(folderType);

            // Get the RiskPanelsViewDelegate to use
            var selectedRiskPanelsView = HprUtil.getSelectedRiskPanelsView();
            var riskPanelsViewDelegate = selectedRiskPanelsView.getDelegate();

            // Retrieve the list of crosstabIds that are valid for the datapane
            var validCrosstabIds = self._getValidCrosstabIds(node, dataItems);

            // If no Crosstabs are valid for the date pane bail out
            if (validCrosstabIds.length === 0) {
                return [];
            }

            // Create the Add All and Remove All root menu items (NOTE: text filled in later)
            var addAllClassVarRootMenuItem = new sas.hc.ui.unified.MenuItem({});
            // TODO: dacagl - Remove for now
            // var removeAllClassVarRootMenuItem = new sas.hc.ui.unified.MenuItem({
            //     enabled: false
            // });

            // If only one crosstab was valid there is no need to show the crosstab submenu
            if (validCrosstabIds.length === 1) {
                // This crosstab variable is lifted to method scope
                var crosstab = sap.ui.getCore().byId(validCrosstabIds[0]);
                var crosstabTitle = riskPanelsViewDelegate.getTitleForContent(validCrosstabIds[0]);

                if (crosstab && crosstab.context) {
                    // Set the text of the root menu items
                    addAllClassVarRootMenuItem.setText(HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.addall.to.one.crosstab.fmt", [HprUtil.dataTypeTitle(folderType), crosstabTitle]));
                    //removeAllClassVarRootMenuItem.setText(HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.removeall.from.one.crosstab.fmt", [HprUtil.dataTypeTitle(folderType), crosstabTitle]));

                    // Create a Select event for the Add All root menu item
                    addAllClassVarRootMenuItem.attachSelect($.proxy(function () {
                        crosstab.doDropRowHeader(dataItems);
                        self._clearSelectionsHelper();
                    }, self));
                }
            } else {
                // If there is more than one valid crosstab we need to create a crosstab submenu
                var subMenu = new sas.hc.ui.unified.Menu();

                _.each(validCrosstabIds, function(id){
                    // This crosstab variable is lifted to this anonymous function
                    var crosstab = sap.ui.getCore().byId(id);

                    if (crosstab && crosstab.context) {
                        var subMenuItem = new sas.hc.ui.unified.MenuItem({
                            text: riskPanelsViewDelegate.getTitleForContent(id)
                        });
                        subMenuItem.attachSelect(function () {
                            crosstab.doDropRowHeader(dataItems);
                            self._clearSelectionsHelper();
                        });
                        subMenu.addItem(subMenuItem);
                    }
                });

                // Set the text of the root menu items
                addAllClassVarRootMenuItem.setText(HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.addall.to.multiple.crosstab.fmt", [HprUtil.dataTypeTitle(folderType)]));
                //removeAllClassVarRootMenuItem.setText(HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.removeall.from.multiple.crosstab.fmt", [HprUtil.dataTypeTitle(folderType)]));

                // Set the submenu for the Add All root menu
                addAllClassVarRootMenuItem.setSubmenu(subMenu);
            }

            menuItems.push(addAllClassVarRootMenuItem);
            //menuItems.push(removeAllClassVarRootMenuItem);

            return menuItems;
        },


        /**
         * Returns the context menu items for Classvar and Hierarchy dataitems
         * @param node
         * @returns {Array}
         */
        _getClassVarHierarchyContextMenuItems: function(node) {
            var self = this;
            var menuItems = [];

            // Get the RiskPanelsViewDelegate to use
            var selectedRiskPanelsView = HprUtil.getSelectedRiskPanelsView();
            var riskPanelsViewDelegate = selectedRiskPanelsView.getDelegate();

            // Retrieve the list of crosstabIds that are valid for the datapane
            var validCrosstabIds = self._getValidCrosstabIds(node);

            // If no Crosstabs are valid for the date pane bail out
            if (validCrosstabIds.length === 0) {
                return [];
            }

            // For the menu item displayed from the DataPane, use the ApplicationSettings showLabels property so
            // the menu item will match the DataPane.  The submenus will be populated using the showLabels option
            // value from the Crosstab
            var showLabels = ApplicationSettings.getShowLabels();

            // Create the root menu item (NOTE: text filled in later)
            var menuItem = new sas.hc.ui.unified.MenuItem({});

            // If only one crosstab was valid there is no need to show the crosstab submenu
            if (validCrosstabIds.length === 1) {
                // This crosstab variable is lifted to method scope
                var crosstab = sap.ui.getCore().byId(validCrosstabIds[0]);
                var crosstabTitle = riskPanelsViewDelegate.getTitleForContent(validCrosstabIds[0]);

                if (crosstab && crosstab.context) {
                    // Set the text of the root menu item
                    menuItem.setText(HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.add.to.one.crosstab.fmt", [node.dataItem.getDisplayLabel(showLabels), crosstabTitle]));

                    // Create a Select event for the root menu item
                    menuItem.attachSelect($.proxy(function () {
                        crosstab.doDropRowHeader([node.dataItem]);
                        self._clearSelectionsHelper();
                    }, self));
                }
            }
            else {
                // If there is more than one valid crosstab we need to create a crosstab submenu
                var subMenu = new sas.hc.ui.unified.Menu();

                _.each(validCrosstabIds, function(id){
                    // This crosstab variable is lifted to this anonymous function
                    var crosstab = sap.ui.getCore().byId(id);

                    if (crosstab && crosstab.context) {
                        var subMenuItem = new sas.hc.ui.unified.MenuItem({
                            text: riskPanelsViewDelegate.getTitleForContent(id)
                        });
                        subMenuItem.attachSelect(function () {
                            crosstab.doDropRowHeader([node.dataItem]);
                            self._clearSelectionsHelper();
                        });
                        subMenu.addItem(subMenuItem);
                    }
                });

                // Set the text and submenu of the root menu item
                menuItem.setText(HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.add.to.multiple.crosstab.fmt", [node.dataItem.getDisplayLabel(showLabels)]));
                menuItem.setSubmenu(subMenu);
            }

            menuItems.push(menuItem);

            return menuItems;
        },

        //*****************************************************************************************************
        //*****************************************************************************************************
        // Computed Variables helper functions
        //*****************************************************************************************************
        //*****************************************************************************************************

        /**
         * Creates an Array of MenuItems which will be used to populate a context-menu for the computed variables folder node
         * @param node
         * @param controller
         * @returns {Array} An Array of MenuItems
         * @private
         */
        _getComputedvariablesFolderContextMenuItems: function(node, controller) {
            var self = this;
            var menuItems = [];
            var folderType = COMPVAR;

            var createCompVarMenuItem = ComputedVariableUtil.createCreateCompVarMenuItem();
            if (createCompVarMenuItem) {
                menuItems.push(createCompVarMenuItem);
            }

            // Add the default folder context menu items to this folder root menu
            var defaultMenuItems = self._getDefaultFolderContextMenuItems(node, controller, folderType);
            _.each(defaultMenuItems, function(defaultMenuItem) {
                menuItems.push(defaultMenuItem);
            });

            return menuItems;
        },

        /**
         * Controller hook to launch the computed variable editor
         */
        _onNewComputedvariable: function(crosstabContextId) {
            var expCtrl = ViewManager.getExplorationView().getController();
            expCtrl.createComputedVariable.call(expCtrl, crosstabContextId);
        },


        /**
         * Returns the context menu items for Computed Variable dataitems
         * @param node
         * @param controller
         * @returns {Array}
         */
        _getComputedVariableContextMenuItems: function(node, controller) {
            var self = this;
            var menuItems = [];

            // Get the RiskPanelsViewDelegate to use
            var selectedRiskPanelsView = HprUtil.getSelectedRiskPanelsView();
            var riskPanelsViewDelegate = selectedRiskPanelsView.getDelegate();

            // In computed var editor mode, disable edit, copy, delete, new computed var, and only the crosstab used in the editor is valid
            var explorationPanesStateModel = sap.ui.getCore().getModel(HprConstants.MODEL_ID_EXPLORATION_PANES_STATE);
            var inComputedVarEditor = explorationPanesStateModel.getProperty("/computedVariableEditorMode");
            var explorationCtrl = ViewManager.getExplorationView().getController();

            // Retrieve the list of crosstabIds that are valid for the datapane for the different type of menu items;
            // Valid Crosstab for Add menu item depends on CV not already being in Crosstab and Data Source id's matching;
            // For other CV menu items, just need to check for matching Data Source id's
            var validCrosstabIdsForAddMenuItem = self._getValidCrosstabIds(node);
            var validCrosstabIdsForOtherMenuItems = ComputedVariableUtil._getValidCrosstabIds();

            // For the menu item displayed from the DataPane, use the ApplicationSettings showLabels property so
            // the menu item will match the DataPane.  The submenus will be populated using the showLabels option
            // value from the Crosstab
            var showLabels = ApplicationSettings.getShowLabels();
            var shouldAddMenuItems = node.dataItem.subType === "expression" && !inComputedVarEditor;

            // add computed variable to crosstab menu option (NOTE: text filled in later)
            var addToCrosstabMenuItem = new sas.hc.ui.unified.MenuItem({
                enabled: validCrosstabIdsForAddMenuItem.length > 0 && !node.dataItem.error
            });

            var editCompVarMenuItem = new sas.hc.ui.unified.MenuItem({
                enabled:  !inComputedVarEditor && node.dataItem.subType === "expression",
                startsSection: validCrosstabIdsForAddMenuItem.length > 0 // Only start a new section if Add menu item appears
            });

            var copyCompVarMenuItem = new sas.hc.ui.unified.MenuItem({
                enabled:  !inComputedVarEditor && node.dataItem.subType === "expression"
            });

            // Add Computed Variable menu item; If no Crosstabs are valid, no need to add submenu
            if (validCrosstabIdsForAddMenuItem.length > 0) {
                // If only one crosstab was valid there is no need to show the crosstab submenu
                if (validCrosstabIdsForAddMenuItem.length === 1) {
                    // This crosstab variable is lifted to method scope
                    var crosstab = sap.ui.getCore().byId(validCrosstabIdsForAddMenuItem[0]);
                    var crosstabTitle = riskPanelsViewDelegate.getTitleForContent(validCrosstabIdsForAddMenuItem[0]);

                    if (crosstab && crosstab.context) {
                        addToCrosstabMenuItem.setText(HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.add.to.one.crosstab.fmt", [node.dataItem.getDisplayLabel(showLabels), crosstabTitle]));
                        addToCrosstabMenuItem.attachSelect($.proxy(function () {
                            DataPaneUtil.addAllDataItemsToCrosstab(crosstab, [node.dataItem]);
                        }, self));
                    }
                }
                else {
                    // If there is more than one valid crosstab we need to create a crosstab submenu
                    var subMenu = new sas.hc.ui.unified.Menu();

                    _.each(validCrosstabIdsForAddMenuItem, function (id) {
                        // This crosstab variable is lifted to method scope
                        var crosstab = sap.ui.getCore().byId(id);

                        if (crosstab && crosstab.context) {
                            var subMenuItem = new sas.hc.ui.unified.MenuItem({
                                text: riskPanelsViewDelegate.getTitleForContent(id)
                            });
                            subMenuItem.attachSelect($.proxy(function () {
                                DataPaneUtil.addAllDataItemsToCrosstab(crosstab, [node.dataItem]);
                            }, self));

                            subMenu.addItem(subMenuItem);
                        }
                    });

                    // Set the text and submenu of the root menu item
                    addToCrosstabMenuItem.setText(HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.add.to.multiple.crosstab.fmt", [node.dataItem.getDisplayLabel(showLabels)]));
                    addToCrosstabMenuItem.setSubmenu(subMenu);
                }
                menuItems.push(addToCrosstabMenuItem);
            }

            // Edit/Copy Computed Variable menu items
            if (validCrosstabIdsForOtherMenuItems.length > 0) {
                if (validCrosstabIdsForOtherMenuItems.length === 1) {
                    var crosstab = sap.ui.getCore().byId(validCrosstabIdsForOtherMenuItems[0]);
                    var crosstabTitle = riskPanelsViewDelegate.getTitleForContent(validCrosstabIdsForOtherMenuItems[0]);

                    if (crosstab && crosstab.context) {
                        editCompVarMenuItem.setText(HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.edit.computed.variable.single.crosstab.fmt", [node.dataItem.getDisplayLabel(showLabels), crosstabTitle]));
                        editCompVarMenuItem.attachSelect($.proxy(explorationCtrl.editComputedVariable, explorationCtrl, crosstab, [node.dataItem]));

                        copyCompVarMenuItem.setText(HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.copy.computed.variable.single.crosstab.fmt", [node.dataItem.getDisplayLabel(showLabels), crosstabTitle]));
                        copyCompVarMenuItem.attachSelect($.proxy(explorationCtrl.copyComputedVariable, explorationCtrl, crosstab, [node.dataItem]));
                    }
                } else {
                    var editSubMenu = new sas.hc.ui.unified.Menu();
                    var copySubMenu = new sas.hc.ui.unified.Menu();

                    _.each(validCrosstabIdsForOtherMenuItems, function (id) {
                        var crosstab = sap.ui.getCore().byId(id);

                        if (crosstab && crosstab.context) {
                            var editSubMenuItem = new sas.hc.ui.unified.MenuItem({
                                text: riskPanelsViewDelegate.getTitleForContent(id)
                            });
                            editSubMenuItem.attachSelect($.proxy(explorationCtrl.editComputedVariable, explorationCtrl, crosstab, [node.dataItem]));
                            editSubMenu.addItem(editSubMenuItem);

                            var copySubMenuItem = new sas.hc.ui.unified.MenuItem({
                                text: riskPanelsViewDelegate.getTitleForContent(id)
                            });
                            copySubMenuItem.attachSelect($.proxy(explorationCtrl.copyComputedVariable, explorationCtrl, crosstab, [node.dataItem]));
                            copySubMenu.addItem(copySubMenuItem);
                        }
                    });

                    editCompVarMenuItem.setText(HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.edit.computed.variable.multiple.crosstabs.fmt", [node.dataItem.getDisplayLabel(showLabels)]));
                    editCompVarMenuItem.setSubmenu(editSubMenu);

                    copyCompVarMenuItem.setText(HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.copy.computed.variable.multiple.crosstabs.fmt", [node.dataItem.getDisplayLabel(showLabels)]));
                    copyCompVarMenuItem.setSubmenu(copySubMenu);
                }

                if (shouldAddMenuItems) {
                    menuItems.push(editCompVarMenuItem);
                    menuItems.push(copyCompVarMenuItem);
                }
            }

            // Delete Computed Variable menu item
            if (shouldAddMenuItems) {
                // Delete menu option
                menuItems.push(self._createMenuItem(
                    HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.delete.fmt.txt", node.dataItem.getDisplayLabel(showLabels)),
                    function() {
                        self._removeComputedVariableWithId(node.dataItem.getId(), controller);
                    },
                    true // startsSection
                ));
            }

            return menuItems;
        },


        /**
         * Removes the ComputedVariable with the given id, and all of its dependent computed variables, from the
         * data source, exploration, data pane, and all crosstabs that use it.
         * @param compVarId
         * @param controller
         * @private
         */
        _removeComputedVariableWithId: function(compVarId, controller) {
            var self = this;
            var rb = HprUtil.getBundle();

            var showLabels = ApplicationSettings.getShowLabels();

            // Get the cube data object
            var cubeData = DataPaneModelManager.getSelectedDataSource();

            // Get the individual computed variable
            var compVar = cubeData.getDataItemById(compVarId);

            // determine all dependent computed variables that would also need to be deleted
            var compVarItemsToDelete = ComputedVariablesManager.getAllRelatedComputedVariableDataItems(cubeData, compVar);

            // confirmation dialog.  The Flex client deleted the dependent CVs so we will too
            var confirmMessage = "";
            if (compVarItemsToDelete.length === 1) {
                confirmMessage = rb.getText("DataPane.Computed.Variable.Delete.Single.fmt", compVar.getDisplayLabel(showLabels));
            } else {
                confirmMessage = rb.getText("DataPane.Computed.Variable.Delete.Multiple.fmt", compVar.getDisplayLabel(showLabels));

                confirmMessage += "\n\n";
                for (var i = 1; i < compVarItemsToDelete.length; i++){
                    confirmMessage += "\t" + compVarItemsToDelete[i].getDisplayLabel(showLabels) + "\n";
                }

                confirmMessage += "\n" + rb.getText("DataPane.Computed.Variable.Delete.Multiple.Items.txt");
            }

            var confirmationDialog = new sas.hc.m.MessageDialog({
                type: sas.hc.m.MessageDialogType.Warning,
                title: rb.getText("DataPane.Computed.Variable.Delete.title.txt"),
                text: confirmMessage,
                buttonActions: [
                    new sas.hc.m.Button({
                        text: rb.getText("hprisk.deleteButton.txt"),
                        press: function (evt) {
                            // delete comp var from all data Sources where it is valid
                            var loadedDataSourceIds = DataSourceCacheManager.getLoadedDataSourceIds();
                            _.each(loadedDataSourceIds, function(dataSourceId) {
                                DataSourceCacheManager.getDataSource(dataSourceId).done(function(dataSource) {
                                    compVar = dataSource.getDataItemById(compVarId);

                                    if (compVar.isValid()) {
                                        // delete computed variables on server.
                                        HprAPI.deleteComputedVariable(dataSource, compVarItemsToDelete).done(function (data) {
                                            // var numToDelete = compVarItemsToDelete.length;
                                            // var failedToDelete = [];
                                            //
                                            // _.each(data.items, function (item) {
                                            //     // if not deleter from the server, add to error message and remove from compVarItemsToDelete before removing from the data pane and exploration
                                            //     if (item.error !== 0) {
                                            //
                                            //         failedToDelete = _.filter(compVarItemsToDelete, function (compVarItem) {
                                            //             return compVarItem.id === item.id;
                                            //         });
                                            //     }
                                            // });
                                            //
                                            // if (failedToDelete.length > 0) {
                                            //     var dialogText = null;
                                            //     var detailText = null;
                                            //
                                            //     if (failedToDelete.length === 1) {
                                            //         dialogText = rb.getText("DataPane.Computed.Variable.Delete.Error.Single.fmt", failedToDelete[0].getDisplayLabel(showLabels));
                                            //     } else {
                                            //         dialogText = rb.getText("DataPane.Computed.Variable.Delete.Error.Multiple.fmt", failedToDelete.length, numToDelete);
                                            //         detailText = rb.getText("DataPane.Computed.Variable.Delete.Error.Detail.txt") + "\n\n";
                                            //         _.each(failedToDelete, function (failed) {
                                            //             detailText += "\t" + failed.name + "\n";
                                            //         });
                                            //     }
                                            //     compVarItemsToDelete = _.difference(compVarItemsToDelete, failedToDelete);
                                            //
                                            //     sas.hc.m.MessageDialog.error({
                                            //         text: dialogText,
                                            //         title: rb.getText("error.window.title"),
                                            //         detailText: detailText ? detailText : null
                                            //     });
                                            // }

                                            // delete from the data pane and exploration those computed variables that were deleted from the server
                                            // if (compVarItemsToDelete.length > 0) {
                                                self._removeComputedVariablesHelper(compVarItemsToDelete, dataSource, controller);
                                            // }
                                        });
                                    } else {
                                        // if (compVarItemsToDelete.length > 0) {
                                            self._removeComputedVariablesHelper(compVarItemsToDelete, dataSource, controller);
                                        // }
                                    }
                                });
                            });
                        }
                    }),
                    {
                        text: sas.hc.m.MessageDialogAction.Cancel,
                        initialFocus: true,
                        defaultButton: true
                    }
                ]
            });
            confirmationDialog.open();
        },


        /**
         * Helper function for removing Computed Variables from the CubeData object, Data Pane, and Crosstabs
         * @param compVars
         * @param cubeData
         * @param controller
         * @private
         */
        _removeComputedVariablesHelper: function(compVars, cubeData, controller) {
            var self = this;

            DataSourceCacheManager.getDataSource(cubeData.dataSource.id).done(function(dataSource) {
                _.each(compVars, function(compVar){
                    dataSource.removeComputedVariableWithId(compVar.id);
                    ComputedVariablesManager.removeComputedVariable(compVar);
                });
                DisplayRulesManager.validateDisplayRules(dataSource);
            });

            // Remove the Computed Variables from the Data pane
            DataPaneUtil.updateComputedVariableNodes(cubeData, controller);

            // Find and group all the Crosstabs which use the same Data Source by the Exploration page ids
            var explorationView = ViewManager.getExplorationView();
            // var sameDataSourceCrosstabs = ExplorationUtil.getCrosstabsUsingSameDataSource(explorationView);
            var sameDataSourceCrosstabs = ExplorationUtil.getAllCrosstabs(explorationView);
            var groupSameDataSourceCrosstabsByPage = _.groupBy(sameDataSourceCrosstabs, function(crosstab) {
                return crosstab.pageId;
            });

            // Since we have all the Crosstabs which use the same Data Source grouped by pages loop through them and see
            // if the first Crosstab we come across is instantiated.  This means the Page is active and the control exists
            $.each(_.values(groupSameDataSourceCrosstabsByPage), function(index, crosstabs) {
                var firstCrosstabId = crosstabs[0].id;
                var firstCrosstab = sap.ui.getCore().byId(firstCrosstabId);
                // Check the first Crosstab to see if it is in DOM, this means Page is active and control exists; If the first Crosstab
                // can not be found, that means the Page has not been activated (or clicked on), so the controls don't exist yet;  In
                // this case, when the Page/Tab is clicked on, the CrosstabContext code (spec. the 'fromColumnHeaderJSON' method) will trim
                // out the Data items and columns that don't exist anymore
                if (firstCrosstab && firstCrosstab.context) {
                    // Pull all the Crosstab ids and try to remove the Computed Variables from the Crosstabs
                    var crosstabIds = _.pluck(crosstabs, 'id');
                    ComputedVariableUtil.removeComputedVariablesFromCrosstabs(crosstabIds, compVars);
                    return true; // Get out, done with the Crosstabs on this Page
                }
            });

            // Further clean up the Data pane and its controls
            DataPaneUtil.clearSelections();
        },


        //*****************************************************************************************************
        //*****************************************************************************************************
        // Scenarios helper functions
        //*****************************************************************************************************
        //*****************************************************************************************************

        /**
         * Creates an Array of MenuItems which will be used to populate a context-menu for the Scenarios folder node
         * @param node
         * @param controller
         * @returns {Array} An Array of MenuItems
         * @private
         */
        _getScenariosFolderContextMenuItems: function(node, controller) {
            var self = this;
            var menuItems = [];
            var folderType = SCENARIO;

            //// add scenario to data pane menu option
            //menuItems.push(self._createMenuItem(
            //    HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.add.scenarios.to.data.pane.txt"),
            //    function() {
            //        self._onShowAddScenariosDialog(controller);
            //    },
            //    true
            //));
            //
            //// remove all scenarios from data pane menu option; only enabled if there are Scenarios displayed in Data Pane
            //menuItems.push(self._createMenuItem(
            //    HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.removeall.scenarios.from.data.pane.txt"),
            //    function() {
            //        self._removeAllScenariosFromDataPane(controller);
            //    },
            //    false,
            //    menuItemEnabled
            //));

            // new scenario menu option
            if (HprApplicationSwitcherController.isScenarioManagerAvailable()) {
                menuItems.push(self._createMenuItem(
                    HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.newscenario.txt"),
                    function () {
                        self._onNewScenario();
                    }
                ));
            }

            // Add the default folder context menu items to this Scenario folder root menu
            var defaultMenuItems = self._getDefaultFolderContextMenuItems(node, controller, folderType);
            _.each(defaultMenuItems, function(defaultMenuItem) {
                menuItems.push(defaultMenuItem);
            });

            return menuItems;
        },

        _getResultSetContextMenuItems: function(node, controller) {
            var self = this;
            var resultSet = node.dataItem;
            var menuItems = [];

            // for default result set, we need to be able to add/remove scenarios
            if (resultSet.isDefault()) {
                // If there are no Scenarios displayed in the Data Pane, we need to disable the remove menu item
                var currentScenariosInDataPane = DataPaneModelManager.getSelectedDataSource().scenarios;
                var removeEnabled = currentScenariosInDataPane && currentScenariosInDataPane.length > 0;

                // add scenario to data pane menu option (only show if we can access scenario manager--otherwise this will result in error)
                if (HprApplicationSwitcherController.isScenarioManagerAvailable()) {
                    menuItems.push(self._createMenuItem(
                        HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.add.scenarios.to.data.pane.txt"),
                        function () {
                            self._onShowAddScenariosDialog(controller);
                        },
                        false
                    ));
                }

                // remove all scenarios from data pane menu option; only enabled if there are Scenarios displayed in Data Pane
                menuItems.push(self._createMenuItem(
                    HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.removeall.scenarios.from.data.pane.txt"),
                    function () {
                        self._removeAllScenariosFromDataPane(controller);
                    },
                    false,
                    removeEnabled
                ));
            }

            // add the default result set menu items
            var defaultMenuItems = self._getDefaultAddToCrosstabContextMenuItems(node);
            if (defaultMenuItems && defaultMenuItems.length > 0) {
                // default menu items are a distinct section
                defaultMenuItems[0].setStartsSection(resultSet.isDefault());

                // add the menu items
                _.each(defaultMenuItems, function(defaultMenuItem) {
                    menuItems.push(defaultMenuItem);
                });
            }

            return menuItems;
        },

        /**
         * Returns the context menu items for Scenario dataitems
         * @param node
         * @param controller
         * @returns {Array}
         */
        _getScenarioContextMenuItems: function(node, controller) {
            var self = this;
            var menuItems = [];

            // Get the RiskPanelsViewDelegate to use
            var selectedRiskPanelsView = HprUtil.getSelectedRiskPanelsView();
            var riskPanelsViewDelegate = selectedRiskPanelsView.getDelegate();

            // Retrieve the list of crosstabIds that are valid for the datapane
            var validCrosstabIds = self._getValidCrosstabIds(node);

            // For the menu item displayed from the DataPane, use the ApplicationSettings showLabels property so
            // the menu item will match the DataPane.  The submenus will be populated using the showLabels option
            // value from the Crosstab
            var showLabels = ApplicationSettings.getShowLabels();

            // add scenario to crosstab menu option (NOTE: text filled in later)
            var menuItem = new sas.hc.ui.unified.MenuItem({
                enabled: (node.dataItem.horizons && node.dataItem.horizons.length > 0) && validCrosstabIds.length > 0
            });

            // If no Crosstabs are valid for the scenario, no need to add submenu
            if (validCrosstabIds.length > 0) {
                // If only one crosstab was valid there is no need to show the crosstab submenu
                if (validCrosstabIds.length === 1) {
                    // This crosstab variable is lifted to method scope
                    var crosstab = sap.ui.getCore().byId(validCrosstabIds[0]);
                    var crosstabTitle = riskPanelsViewDelegate.getTitleForContent(validCrosstabIds[0]);

                    if (crosstab && crosstab.context) {
                        // Set the text of the root menu item
                        menuItem.setText(HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.add.to.one.crosstab.fmt", [node.dataItem.getDisplayLabel(showLabels), crosstabTitle]));
                        // Construct an appropriate context menu for the Data Item based on Crosstab logic
                        menuItem.attachSelect($.proxy(function () {
                            DataPaneUtil.addAllDataItemsToCrosstab(crosstab, [node.dataItem]);
                        }, self));
                    }
                }
                else {
                    // If there is more than one valid crosstab we need to create a crosstab submenu
                    var subMenu = new sas.hc.ui.unified.Menu();

                    _.each(validCrosstabIds, function (id) {
                        // This crosstab variable is lifted to method scope
                        var crosstab = sap.ui.getCore().byId(id);

                        if (crosstab && crosstab.context) {
                            var subMenuItem = new sas.hc.ui.unified.MenuItem({
                                text: riskPanelsViewDelegate.getTitleForContent(id)
                            });
                            subMenuItem.attachSelect($.proxy(function () {
                                DataPaneUtil.addAllDataItemsToCrosstab(crosstab, [node.dataItem]);
                            }, self));

                            subMenu.addItem(subMenuItem);
                        }
                    });

                    // Set the text and submenu of the root menu item
                    menuItem.setText(HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.add.to.multiple.crosstab.fmt", [node.dataItem.getDisplayLabel(showLabels)]));
                    menuItem.setSubmenu(subMenu);
                }
                menuItems.push(menuItem);
            }

            // only surface menu items that redirect to scenario manager if we can find it
            if (HprApplicationSwitcherController.isScenarioManagerAvailable()) {
                // construct appropriate View and Edit menu item text which tries to use the official name of Scenario Manager
                var viewMenuText = HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.viewscenario.fmt", [node.dataItem.getDisplayLabel(showLabels)]); // default to regular text w/o app label
                var editMenuText = HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.editscenario.fmt", [node.dataItem.getDisplayLabel(showLabels)]); // default to regular text w/o app label
                var scenarioManagerAppObject = sas.applicationSwitcher.findApplication(function compare(oApp) {
                    return oApp.appId === HprApplicationSwitcherController.getScenarioManagerAppId();
                });
                if (scenarioManagerAppObject) {
                    // found the Scenario Manager app object which contains data about the app; use the label
                    viewMenuText = HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.viewscenario.with.product.name.fmt", [node.dataItem.getDisplayLabel(showLabels), scenarioManagerAppObject.label]);
                    editMenuText = HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.editscenario.with.product.name.fmt", [node.dataItem.getDisplayLabel(showLabels), scenarioManagerAppObject.label]);
                }

                // view scenario menu option
                menuItems.push(self._createMenuItem(
                    viewMenuText,
                    function () {
                        self._onViewScenario(node.dataItem.getId());
                    },
                    validCrosstabIds.length > 0 // startsSection - Only start a new section if first menu item above was displayed
                ));

                // Determine if we need to remove the Edit and Remove menu items; They should only be displayed for Scenarios in the Scenarios folder on the Data Pane
                // We could probably key off node.dataItem.resultSetId here, but not sure if all Scenarios in Scenarios folder will have the same resultSetId
                var shouldAddMenuItems = _.find(DataPaneModelManager.getSelectedDataSource().scenarios, function (scenario) {
                    return scenario.id === node.dataItem.id; // returns undefined if nothing found
                });
                if (shouldAddMenuItems) {
                    // edit scenario menu option
                    menuItems.push(self._createMenuItem(
                        editMenuText,
                        function () {
                            self._onEditScenario(node.dataItem.getId());
                        }
                    ));

                    // Remove scenario menu option
                    menuItems.push(self._createMenuItem(
                        HprUtil.getBundle().getText("DataPane.treetable.cellcontextmenu.remove.fmt", [node.dataItem.getDisplayLabel(showLabels)]),
                        function () {
                            self._removeScenarioWithId(node.dataItem.getId(), controller);
                        },
                        true
                    ));
                }
            }

            return menuItems;
        },

        /**
         * Controller hook to launch Scenario Manager via the application switcher controller in order
         * to create a scenario .
         */
        _onNewScenario: function() {
            DataSourceCacheManager.getDataSource(DataPaneModelManager.getSelectedDataSourceId()).done(
                function(dataSource) {
                    HprApplicationSwitcherController.createScenario(dataSource.getId(), DataSourceCacheManager.getDataSourceParam(dataSource.getId()));
                }
            ).fail(
                function() {
                    //TODO show error
                }
            );
        },

        /**
         * Controller hook to launch Scenario Manager via the application switcher controller in order
         * to view a scenario.
         *
         * @param scenarioId {number} the ID of the scenario to view
         */
        _onViewScenario: function(scenarioId) {
            DataSourceCacheManager.getDataSource(DataPaneModelManager.getSelectedDataSourceId()).done(
                function(dataSource) {
                    HprApplicationSwitcherController.viewScenario(dataSource.getDataItemById(scenarioId), DataSourceCacheManager.getDataSourceParam(dataSource.getId()));
                }
            ).fail(
                function() {
                    //TODO show error
                }
            );
        },

        /**
         * Controller hook to launch Scenario Manager via the application switcher controller in order
         * to edit a scenario.
         *
         * @param scenarioId {number} the ID of the scenario to edit
         */
        _onEditScenario: function(scenarioId) {
            DataSourceCacheManager.getDataSource(DataPaneModelManager.getSelectedDataSourceId()).done(
                function(dataSource) {
                    HprApplicationSwitcherController.editScenario(dataSource.getDataItemById(scenarioId), DataSourceCacheManager.getDataSourceParam(dataSource.getId()));
                }
            ).fail(
                function() {
                    //TODO show error
                }
            );
        },

        /**
         * Launches dialog to add scenarios to the exploration
         * @param controller
         */
        _onShowAddScenariosDialog: function(controller) {
            var rb = HprUtil.getBundle();

            // put up a busy dialog so the user knows something is happening while
            // we wait for the scenarios to be loaded.
            var busyDialog = new sas.hc.m.BusyDialog({
                title: rb.getText("AddScenariosDialog.busyDialog.txt"),
                showCancelButton: false
            });
            busyDialog.open();

            // Load the scenarios
            var dataSource = DataPaneModelManager.getSelectedDataSource();
            HprAPI.getScenarioList(dataSource).done(
                function(scenarios) {
                    var scenariosToSelect = _.reject(scenarios, function(scenario) {
                        var foundScenario = _.find(dataSource.scenarios, function(existingCubeDataScenario) {
                            return existingCubeDataScenario.getId() === scenario.getId();
                        });
                        return (foundScenario !== undefined);
                    });
                    var dialogModel = new sap.ui.model.json.JSONModel();
                    dialogModel.setProperty("/scenarios", scenariosToSelect);
                    var dialog = sap.ui.jsfragment("sas.analytics.riskhp.fragments.AddScenariosDialog", controller);
                    dialog.setModel(dialogModel);
                    dialog.open();
                }
            ).fail(
                function(jqxhr) {
                    HprUtil.displayGenericErrorDialog();
                }
            ).always(
                function() {
                    busyDialog.close();
                }
            );
        },

        /**
         * Removes the Scenario with the given id from the list of Scenarios this CubeData object knows about, removes the Scenario from
         * the Data Pane, and also removes the Scenario from all Crosstabs that use it. No-op if the passed in scenarioId is not being tracked by this object.
         * @param scenarioId
         * @param controller
         * @private
         */
        _removeScenarioWithId: function(scenarioId, controller) {
            var self = this;

            // Get the cube data object
            var cubeData = DataPaneModelManager.getSelectedDataSource();

            // Get the individual scenario
            var scenario = cubeData.getDataItemById(scenarioId);

            // De-activate the scenario if it is active
            if (scenario && scenario.isActivated() === true) {
                HprAPI.deactivateScenarios(cubeData.dataSource.id, [scenarioId]);
            }

            self._removeScenariosHelper([scenario], cubeData, controller);
        },

        /**
         * Removes all the Scenarios from the Data Pane.  In doing this, we also need to remove them from the CubeData object and all Crosstabs
         * @param controller
         * @private
         */
        _removeAllScenariosFromDataPane: function(controller) {
            var self = this;

            // Get the cube data object
            var cubeData = DataPaneModelManager.getSelectedDataSource();

            // Get the scenarios
            var scenarios = cubeData.getDataItemsForType(SCENARIO);
            var activeScenarios = _.filter(scenarios, function(scenario) {
                return scenario.isActivated() === true;
            });

            // De-activate the scenarios
            if (activeScenarios && activeScenarios.length > 0) {
                HprAPI.deactivateScenarios(cubeData.dataSource.id, _.pluck(activeScenarios, "id"));
            }

            self._removeScenariosHelper(scenarios, cubeData, controller);
        },

        /**
         * Helper function for removing Scenarios from the CubeData object, Data Pane, and Crosstabs
         * @param scenarios
         * @param cubeData
         * @param controller
         * @private
         */
        _removeScenariosHelper: function(scenarios, cubeData, controller) {
            var self = this;

            // Remove the scenarios from the Data Source
            var removedAScenario = false;
            if (scenarios.length > 1) {
                removedAScenario = cubeData.removeAllScenarios();
            } else {
                removedAScenario = cubeData.removeScenarioWithId(scenarios[0].getId());
            }

            // If at least one scenario was not removed then we have nothing to do so abort
            if (!removedAScenario) {
                return;
            }

            // TODO: DSCM - This seems like a weird way to update the dataSourceParam.  Might be better on save to update it then from the current cube data
            // Remove the Scenarios from the DataSourceParam object
            var dataSourceParam = DataSourceCacheManager.getSelectedDataSourceParam();
            if (scenarios.length > 1) {
                dataSourceParam.removeAllScenarioIds();
            } else {
                dataSourceParam.removeScenarioId(scenarios[0].getId());
            }

            var scenarioIds = _.map(scenarios, function(scenario) {
                return scenario.getId();
            });
            // Remove the Scenarios from the Data pane
            DataPaneUtil.updateScenarioNodes(cubeData, controller);

            // Find and group all the Crosstabs which use the same Data Source by the Exploration page ids
           var explorationView = ViewManager.getExplorationView();
           var sameDataSourceCrosstabs = ExplorationUtil.getCrosstabsUsingSameDataSource(explorationView);
            var groupSameDataSourceCrosstabsByPage = _.groupBy(sameDataSourceCrosstabs, function(crosstab) {
                return crosstab.pageId;
            });

            // Since we have all the Crosstabs which use the same Data Source grouped by pages loop through them and see
            // if the first Crosstab we come across is instantiated.  This means the Page is active and the control exists
            $.each(_.values(groupSameDataSourceCrosstabsByPage), function(index, crosstabs) {
                var firstCrosstabId = crosstabs[0].id;
                var firstCrosstab = sap.ui.getCore().byId(firstCrosstabId);
                // Check the first Crosstab to see if it is in DOM, this means Page is active and control exists; If the first Crosstab
                // can not be found, that means the Page has not been activated (or clicked on), so the controls don't exist yet;  In
                // this case, when the Page/Tab is clicked on, the CrosstabContext code (spec. the 'fromColumnHeaderJSON' method) will trim
                // out the Data items and columns that don't exist anymore
                if (firstCrosstab && firstCrosstab.context) {
                    // Pull all the Crosstab ids and try to remove the Scenarios from the Crosstabs
                    var crosstabIds = _.pluck(crosstabs, 'id');
                    self._removeScenariosFromCrosstabs(crosstabIds, scenarios);
                    return true; // Get out, done with the Crosstabs on this Page
                }
            });
            sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_SCENARIO_DELETED, {dataSourceId: cubeData.getId(), scenarioIds: scenarioIds});

            // Further clean up the Data pane and its controls
            DataPaneUtil.clearSelections();
        },

        /**
         * Loop through the Crosstabs and delete the Scenario from the Crosstabs that contain it in one of it's columns (i.e. tuples)
         * @param crosstabIds
         * @param scenarios
         * @private
         */
        _removeScenariosFromCrosstabs: function(crosstabIds, scenarios) {
            _.each(crosstabIds, function (id) {
                var crosstab = sap.ui.getCore().byId(id);

                if (crosstab && crosstab.context) {
                    var rebuildCrosstabFlag = false;
                    var tuples = crosstab.context.getTuples();

                    // Loop through the Scenarios and see if the Crosstab contains it; if so, remove it
                    _.each(scenarios, function(scenario) {
                        var scenarioTuples = _.filter(tuples, function(aTuple) {
                            var scenarioDataItem = crosstab.context.getDataItemInfoFromTuple(aTuple, scenario.getType());
                            return scenarioDataItem && scenarioDataItem.dataItem.id === scenario.id;
                        });

                        if (scenarioTuples && scenarioTuples.length > 0) {
                            var labelOrNameToUse = crosstab.context.getOptionShowLabels().value ? scenario.getLabel(): scenario.getName();
                            crosstab.context.removeColumnsWithNameAndType(labelOrNameToUse, scenario.getType());
                            rebuildCrosstabFlag = true;
                        }
                    });

                    // Only rebuild the Crosstab if we have to
                    if (rebuildCrosstabFlag) {
                        crosstab.rebuildCrosstab();
                    }
                }
            });
        }
    });
})();