// (c) 2018, SAS Institute Inc.
jQuery.sap.require('sas.analytics.riskhp.common.riskcommons.CrosstabUtil');
jQuery.sap.require('sas.analytics.riskhp.utils.HprAPI');
jQuery.sap.require('sas.analytics.riskhp.views.displayrules.DisplayRulesCrosstabContextMenuProvider');
jQuery.sap.require("sas.analytics.riskhp.models.DataSourceCacheManager");
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.analytics.riskhp.views.displayrules.DisplayRulesUtil");
jQuery.sap.require("sas.analytics.riskhp.views.datapane.DataPaneUtil");
jQuery.sap.require("sas.analytics.riskhp.models.DisplayRule");
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.hc.m.MessageToast");

sap.ui.controller('sas.analytics.riskhp.views.displayrules.DisplayRuleEditor', {
    CrosstabUtil: sas.analytics.riskhp.common.riskcommons.CrosstabUtil,
    HprAPI: sas.analytics.riskhp.utils.HprAPI,
    DataSourceCacheManager: sas.analytics.riskhp.models.DataSourceCacheManager,
    HprUtil: sas.analytics.riskhp.utils.HprUtil,
    CompVarUtil: sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil,
    DataPaneUtil: sas.analytics.riskhp.views.datapane.DataPaneUtil,
    DisplayRulesUtil: sas.analytics.riskhp.views.displayrules.DisplayRulesUtil,

    onInit: function () {
        var self = this;
        var view = self.getView();

        // operators model
        var operatorsModel = new sap.ui.model.json.JSONModel();
        operatorsModel.setData({operators: sas.analytics.riskhp.models.DisplayRule.operators});
        view.setModel(operatorsModel, 'operators');

        // dataItemTypes model
        var dataItemTypesModel = new sap.ui.model.json.JSONModel();
        dataItemTypesModel.setData({dataItemTypes: sas.analytics.riskhp.models.DisplayRule.dataItemTypes});
        view.setModel(dataItemTypesModel, 'dataItemTypes');

        // appliesToTypes model
        var appliesToTypesModel = new sap.ui.model.json.JSONModel();
        appliesToTypesModel.setData({appliesToTypes: sas.analytics.riskhp.models.DisplayRule.appliesToTypes});
        view.setModel(appliesToTypesModel, 'appliesToTypes');

        // view model
        var model = new sap.ui.model.json.JSONModel();
        view.setModel(model);

        // i18N model
        var i18N = new sap.ui.model.resource.ResourceModel({bundleName: 'sas.analytics.riskhp.nls.messagebundle'});
        view.setModel(i18N, 'i18N');

        sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_DATA_ITEM_LABEL_UPDATED, this.onDataItemLabelUpdate, this);
    },

    onExit: function() {
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_DATA_ITEM_LABEL_UPDATED, this.onDataItemLabelUpdate, this);
    },

    // Update the display rules when a label for a Data Item is updated.
    onDataItemLabelUpdate: function(channelId, eventId, data) {
        var self = this;
        var view = self.getView();
       if (self.crosstab.context.getOptionShowLabels().value) {
           self.crosstab.redraw();
            view.getModel().refresh(true);
       }
    },


    onBeforeRendering: function () {
        var self = this;
        var view = self.getView();
    },

    onAfterRendering: function () {
        var self = this;
        var view = self.getView();
        self.updatePreviewTextAreaStyle();
    },

    _getResourceBundle: function () {
        return sas.analytics.riskhp.utils.HprUtil.getBundle();
    },

    cancelEditing: function () {
        var self = this;
        // set original menu provider back on crosstab
        self.crosstab.setMenuProvider(self.origMenuProvider);

        // restore the original drag handler on crosstab.uiVIew
        self.crosstab.uiView.setDragHandler(self.origDragHandler);

        // Publish an event to the exploration view to close the computed variable editor for the crosstab
        sas.eventBus.publish(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_CLOSE_DISPLAY_RULE_EDITOR, {
            crosstabId: this.crosstabId
        });
    },

    /**
     *
     * @param crosstabId
     */
    configure: function (crosstabId) {
        var self = this;
        var view = self.getView();

        var context = sas.analytics.riskhp.models.HPRiskModelManager.getCrosstabContext(crosstabId);
        self.crosstab = context.crosstab;
        self.crosstabId = self.crosstab.sId;

        // force the display rules pane to be for this crosstab since it is still enabled
        sas.analytics.riskhp.utils.HprUtil.updateRightPanelsForSelectedCrosstab(self.crosstabId);


        self.origMenuProvider = self.crosstab.getMenuProvider();  // save off the crosstab's menu provider
        var crosstabMenuProvider = new sas.analytics.riskhp.views.displayrules.DisplayRulesCrosstabContextMenuProvider(self);
        self.crosstab.setMenuProvider(crosstabMenuProvider);

        view.topSplitterPanel.removeAllContent(sap.ui.commons.layout.BorderLayoutAreaTypes.top);
        view.topSplitterPanel.removeAllContent(sap.ui.commons.layout.BorderLayoutAreaTypes.center);

        var selectedRiskPanelsView = self.HprUtil.getSelectedRiskPanelsView();
        var riskPanelsViewDelegate = selectedRiskPanelsView.getDelegate();
        var crosstabTitle = riskPanelsViewDelegate.getTitleForContent(crosstabId);

        var topLabel = new sas.hc.m.Label({
            text: crosstabTitle
        }).addStyleClass("computedVariableEditorCrosstabLabel");
        view.topSplitterPanel.addContent(sap.ui.commons.layout.BorderLayoutAreaTypes.top, topLabel);
        view.topSplitterPanel.setAreaData(sap.ui.commons.layout.BorderLayoutAreaTypes.top, {
            size: "1.5rem"
        });

        view.topSplitterPanel.addContent(sap.ui.commons.layout.BorderLayoutAreaTypes.center, this.crosstab);
        view.topSplitterPanel.setAreaData(sap.ui.commons.layout.BorderLayoutAreaTypes.center, {
            "margin-left": "0.5rem",
            "margin-right": "0.5rem"
        });
        setTimeout(function () {
            // crosstab calculates its height when there is a breadcrumb (drillpath) so redraw after added to the layout to get correct crosstab height
            self.crosstab.redraw();
        }, 1);

        self.origDragHandler = self.crosstab.uiView.getDragHandler();
        self.crosstab.uiView.setDragHandler(self);

        self.isNameValid = true;
        view.descriptionInput.setValueState(sap.ui.core.ValueState.None);
    },

    configureForCreating: function (crosstabId, pickinfo) {
        var self = this;
        var view = self.getView();

        self.configure(crosstabId);

        var displayRule = new sas.analytics.riskhp.models.DisplayRule();

        if (pickinfo) {
            displayRule.referenceColumn = sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil.getTuplesFromPickInfo(pickinfo);
        }

        view.setModel(new sap.ui.model.json.JSONModel(displayRule));

        view.dataItem1TypeSelect.setSelectedKey(sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE);
        view.dataItem2TypeSelect.setSelectedKey(sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE);
        view.appliesToTypeSelect.setSelectedKey(sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_REF_COL);
        view.operatorSelect.setSelectedKey(sas.analytics.riskhp.models.DisplayRule.operators[7]);

        view.applyButton.setEnabled(self.isEditorValid());
    },

    validateColumnItems: function(displayRule) {
        var self = this;
        var self = this;
        var cubeData = self.crosstab.context.cubeDataModel;

        var numInvalidColumns = 0;
        if (!DisplayRulesManager.isValidColumn(cubeData, displayRule.referenceColumn)) {
            numInvalidColumns++;
        }

        if (sas.analytics.riskhp.views.displayrules.DisplayRulesUtil.requiresDataItem1(displayRule.operator) &&
            displayRule.dataItem1.type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_COL &&
            !DisplayRulesManager.isValidColumn(cubeData, displayRule.dataItem1.column)) {
            numInvalidColumns++;
        }
        if (sas.analytics.riskhp.views.displayrules.DisplayRulesUtil.requiresDataItem2(displayRule.operator) &&
            displayRule.dataItem2.type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_COL &&
            !DisplayRulesManager.isValidColumn(cubeData, displayRule.dataItem2.column)) {
            numInvalidColumns++;
        }

        if (displayRule.appliesTo.type === sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_COL &&
            !DisplayRulesManager.isValidColumn(cubeData, displayRule.appliesTo.column)) {
            numInvalidColumns++;
        }

        // display message that items are invalid
        if (numInvalidColumns > 0) {
            var dialogText = self.HprUtil.getBundle().getText("HPRisk.DisplayRules.Editor.Columns.Marked.Invalid.txt");

            var displayRuleWarningDialogId = "displayRuleWarningDialog";
            // warning dialogs do not show detail text
            if (sas.hc.ui.core.settings.GlobalSettingsHelper.getSuppressedMessages().indexOf(displayRuleWarningDialogId) === -1) {
                var warning = sas.hc.m.MessageDialog.warning({
                    id: displayRuleWarningDialogId,
                    text: dialogText,
                    title: self.HprUtil.getBundle().getText("HPRisk.DisplayRules.Editor.Invalid.Items.Window.title"),
                    includeDontShowCheckBox: true
                });
                warning.attachAfterClose(function(e) {
                    warning.destroy();
                });
            }
        }
    },

    clearInvalidColumns: function (displayRule) {
        var self = this;
        var cubeData = self.crosstab.context.cubeDataModel;

        if (!DisplayRulesManager.isValidColumn(cubeData, displayRule.referenceColumn)) {
            displayRule.referenceColumn = [];
        }

        if (sas.analytics.riskhp.views.displayrules.DisplayRulesUtil.requiresDataItem1(displayRule.operator) &&
            displayRule.dataItem1.type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_COL &&
            !DisplayRulesManager.isValidColumn(cubeData, displayRule.dataItem1.column)) {
            displayRule.dataItem1.column = [];
        }
        if (sas.analytics.riskhp.views.displayrules.DisplayRulesUtil.requiresDataItem2(displayRule.operator) &&
            displayRule.dataItem2.type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_COL &&
            !DisplayRulesManager.isValidColumn(cubeData, displayRule.dataItem2.column)) {
            displayRule.dataItem2.column = [];
        }

        if (displayRule.appliesTo.type === sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_COL &&
            !DisplayRulesManager.isValidColumn(cubeData, displayRule.appliesTo.column)) {
            displayRule.appliesTo.column = [];
        }
    },


    configureForEditing: function (crosstabId, displayRule) {
        var self = this;
        var view = self.getView();

        self.configure(crosstabId);

        var clonedDisplayRule = sas.analytics.riskhp.models.DisplayRule.prototype.clone(displayRule);
        // self.clearInvalidColumns(clonedDisplayRule);

        // fluff up model for editing (we only persist what is necessary, but editor needs whole model
        self._fluffUpModelForEditing(clonedDisplayRule);


        view.setModel(new sap.ui.model.json.JSONModel(clonedDisplayRule));

        var opIndex = _.findIndex(sas.analytics.riskhp.models.DisplayRule.operators, function (op) {
            return clonedDisplayRule.operator === op;
        });

        view.fontChooser.setFormatting({
            isItalic: clonedDisplayRule.style.italic ? "italic" : "none",
            isBold: clonedDisplayRule.style.bold ? "bold" : "normal",
            isUnderLine: clonedDisplayRule.style.underline ? "underline" : "none",
            // fontSize: clonedDisplayRule.style.fontSize + "px",
            currentColor: clonedDisplayRule.style.color.replace("0x", "#"),
            fontName: clonedDisplayRule.style.fontName
        });
        view.operatorSelect.setSelectedKey(sas.analytics.riskhp.models.DisplayRule.operators[opIndex]);
        view.foregroundCP.setCurrentColor(clonedDisplayRule.style.color.replace("0x", "#"));
        view.backgroundCP.setCurrentColor(clonedDisplayRule.style.backgroundColor.replace("0x", "#"));
        view.dataItem1TypeSelect.setSelectedKey(clonedDisplayRule.dataItem1.type);
        view.dataItem2TypeSelect.setSelectedKey(clonedDisplayRule.dataItem2.type);
        view.appliesToTypeSelect.setSelectedKey(clonedDisplayRule.appliesTo.type);

        // when editing, always start with Apply disabled until something changes
        view.applyButton.setEnabled(false);
        self.validateColumnItems(clonedDisplayRule);
    },

    _fluffUpModelForEditing: function(displayRule) {
        if (self.DisplayRulesUtil.requiresDataItem1(displayRule.operator)) {
            if (displayRule.dataItem1.type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE) {
                displayRule.dataItem1.column = [];
            } else {
                displayRule.dataItem1.value = 0;
            }
        } else {
            displayRule.dataItem1 = {
                type: sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE,
                value: 0,
                column: []
            };
        }
        if (self.DisplayRulesUtil.requiresDataItem2(displayRule.operator)) {
            if (displayRule.dataItem2.type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE) {
                displayRule.dataItem2.column = [];
            } else {
                displayRule.dataItem2.value = 0;
            }
        } else {
            displayRule.dataItem2 = {
                type: sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE,
                value: 0,
                column: []
            };
        }
        if (displayRule.appliesTo.type !== sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_COL) {
            displayRule.appliesTo.column = [];
        }
    },

    configureForCopying: function (crosstabId, displayRule) {
        var self = this;
        var view = self.getView();

        self.configure(crosstabId);

        var copyOfDisplayRule = displayRule.copy();//sas.analytics.riskhp.models.DisplayRule.prototype.copy(displayRule);
        // self.clearInvalidColumns(clonedDisplayRule);
        // fluff up model for editing (we only persist what is necessary, but editor needs whole model
        self._fluffUpModelForEditing(copyOfDisplayRule);
        view.setModel(new sap.ui.model.json.JSONModel(copyOfDisplayRule));

        var opIndex = _.findIndex(sas.analytics.riskhp.models.DisplayRule.operators, function (op) {
            return copyOfDisplayRule.operator === op;
        });
        view.fontChooser.setFormatting({
            isItalic: copyOfDisplayRule.style.italic ? "italic" : "none",
            isBold: copyOfDisplayRule.style.bold ? "bold" : "normal",
            isUnderLine: copyOfDisplayRule.style.underline ? "underline" : "none",
            // fontSize: clonedDisplayRule.style.fontSize + "px",
            currentColor: copyOfDisplayRule.style.color.replace("0x", "#"),
            fontName: copyOfDisplayRule.style.fontName
        });
        view.operatorSelect.setSelectedKey(sas.analytics.riskhp.models.DisplayRule.operators[opIndex]);
        view.foregroundCP.setCurrentColor(copyOfDisplayRule.style.color.replace("0x", "#"));
        view.backgroundCP.setCurrentColor(copyOfDisplayRule.style.backgroundColor.replace("0x", "#"));
        view.dataItem1TypeSelect.setSelectedKey(copyOfDisplayRule.dataItem1 ? copyOfDisplayRule.dataItem1.type : sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE);
        view.dataItem2TypeSelect.setSelectedKey(copyOfDisplayRule.dataItem2 ? copyOfDisplayRule.dataItem2.type : sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE);

        view.appliesToTypeSelect.setSelectedKey(copyOfDisplayRule.appliesTo.type);
        // when copying, always start with Apply enabled
        view.applyButton.setEnabled(true);
        self.validateColumnItems(copyOfDisplayRule);
    },

    handleApplyPressed: function () {
        var self = this;
        var view = self.getView();
        var displayRule = view.getModel().getData();

        // clean up model for persisting
        var clonedDisplayRule = sas.analytics.riskhp.models.DisplayRule.prototype.clone(displayRule);
        clonedDisplayRule.valid = true;  // must be valid in order to create it.  Validity can change when replacing dataSource
        if (self.requiresDataItem1()) {
            if (clonedDisplayRule.dataItem1.type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE) {
                delete clonedDisplayRule.dataItem1.column;
            } else {
                delete clonedDisplayRule.dataItem1.value;
            }
        } else {
            delete clonedDisplayRule.dataItem1;
        }
        if (self.requiresDataItem2()) {
            if (clonedDisplayRule.dataItem2.type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE) {
                delete clonedDisplayRule.dataItem2.column;
            } else {
                delete clonedDisplayRule.dataItem2.value;
            }
        } else {
            delete clonedDisplayRule.dataItem2;
        }
        if (clonedDisplayRule.appliesTo.type !== sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_COL) {
            delete clonedDisplayRule.appliesTo.column;
        }

        if (DisplayRulesManager.getDisplayRuleById(clonedDisplayRule.id)) {
            DisplayRulesManager.updateDisplayRule(clonedDisplayRule);
        } else {
            DisplayRulesManager.addDisplayRule(clonedDisplayRule, self.crosstabId);
        }

        self.cancelEditing();
    },

    handleDescriptionLiveChange: function (event) {
        var self = this;
        var view = self.getView();
        var model = view.getModel();
        var newValue = event.getParameter("newValue");

        // the binding to the model has not happened at this point so
        // force the value in the model so it is available for the validation
        model.setProperty("/description", newValue);
        view.descriptionInput.setValue(newValue);

        self.updateApplyButtonState();
    },

    handleDataItem1ValueLiveChange: function (event) {
        var self = this;
        var view = self.getView();
        var model = view.getModel();

        var newValue = event.getParameter("newValue");

        // the binding to the model has not happened at this point so
        // force the value in the model so it is available for the validation
        model.setProperty("/dataItem1/value", newValue);
        view.dataItem1ValueInput.setValue(newValue);

        self.updateApplyButtonState();
    },

    handleDataItem2ValueLiveChange: function (event) {
        var self = this;
        var view = self.getView();
        var model = view.getModel();

        var newValue = event.getParameter("newValue");

        // the binding to the model has not happened at this point so
        // force the value in the model so it is available for the validation
        model.setProperty("/dataItem2/value", newValue);
        view.dataItem2ValueInput.setValue(newValue);

        self.updateApplyButtonState();
    },

    handleColorChange: function (oEvent) {
        var self = this;
        var view = self.getView();
        var model = view.getModel();
        var selectionType = oEvent.getParameter("selectionType");
        if (selectionType === sas.hc.m.colorpicker.layout.ColorPickerContainer.prototype.selectionType.NO_COLOR) {
            model.setProperty("/style/color", sas.hc.m.colorpicker.layout.ColorPickerContainer.prototype.selectionType.NO_COLOR);
        } else {
            var colorHex =  oEvent.getParameter("newValue").toLowerCase().replace("#", "0x");
            model.setProperty("/style/color", colorHex);
        }
        self.updatePreviewTextAreaStyle();
        self.updateApplyButtonState();
    },

    handleBackgroundColorChange: function (oEvent) {
        var self = this;
        var view = self.getView();
        var model = view.getModel();
        var selectionType = oEvent.getParameter("selectionType");
        if (selectionType === sas.hc.m.colorpicker.layout.ColorPickerContainer.prototype.selectionType.NO_COLOR) {
            model.setProperty("/style/backgroundColor", sas.hc.m.colorpicker.ColorPicker.NO_COLOR);
        } else {
            var colorHex =  oEvent.getParameter("newValue").toLowerCase().replace("#", "0x");
            model.setProperty("/style/backgroundColor", colorHex);
        }
        self.updatePreviewTextAreaStyle();
        self.updateApplyButtonState();
    },

    handleBoldChange: function(oEvent) {
        var self = this;
        var view = self.getView();
        var model = view.getModel();

        model.setProperty("/style/bold", oEvent.getParameter("pressed"));
        self.updatePreviewTextAreaStyle();
        self.updateApplyButtonState();
    },

    handleItalicChange: function(oEvent) {
        var self = this;
        var view = self.getView();
        var model = view.getModel();

        model.setProperty("/style/italic", oEvent.getParameter("pressed"));
        self.updatePreviewTextAreaStyle();
        self.updateApplyButtonState();
    },

    handleUnderlineChange: function(oEvent) {
        var self = this;
        var view = self.getView();
        var model = view.getModel();

        model.setProperty("/style/underline", oEvent.getParameter("pressed"));
        self.updatePreviewTextAreaStyle();
        self.updateApplyButtonState();
    },

    handleFontChange: function(formatting) {
        var self = this;
        var view = self.getView();
        var model = view.getModel();

        model.setProperty("/style/fontName", formatting.getParameter("fontName"));
        self.updatePreviewTextAreaStyle();
        self.updateApplyButtonState();
    },

    _getPreviewAreaDomref: function() {
        var view = this.getView();
        var $preview;
        if (view.previewTextArea.getDomRef()) {
            $preview = view.previewTextArea.$().find("textarea");
        }
        return $preview;
    },

    updatePreviewTextAreaStyle: function() {
        var previewTextAreaDomRef = this._getPreviewAreaDomref();

        var view = this.getView();
        var displayRule = view.getModel().getData();
        var style = displayRule.style;

        previewTextAreaDomRef.css('font-family', style.fontName);
        if (style.bold) {
            previewTextAreaDomRef.addClass("displayRulesBold");
        } else {
            previewTextAreaDomRef.removeClass("displayRulesBold");
        }
        if (style.italic) {
            previewTextAreaDomRef.addClass("displayRulesItalic");
        } else {
            previewTextAreaDomRef.removeClass("displayRulesItalic");
        }
        if (style.underline) {
            previewTextAreaDomRef.addClass("displayRulesUnderline");
        } else {
            previewTextAreaDomRef.removeClass("displayRulesUnderline");
        }

        var color = style.color.toLowerCase().replace("0x", "#");
        previewTextAreaDomRef.css('color', color);
        previewTextAreaDomRef.css('background-color', style.backgroundColor.toLowerCase().replace("0x", "#"));
    },

    handleOperatorSelection: function(event) {
        var self = this;
        var view = self.getView();
        var model = view.getModel();
        var operatorKey = event.getParameter("selectedItem").getProperty("key");

        var operator = _.find(sas.analytics.riskhp.models.DisplayRule.operators, function(op) {
            return op == operatorKey;
        });

        if (operator === sas.analytics.riskhp.models.DisplayRule.OPERATOR_BETWEEN || operator === sas.analytics.riskhp.models.DisplayRule.OPERATOR_NOT_BETWEEN) {
            if (!model.getProperty("/dataItem2")) {
                model.setProperty("/dataItem2", {
                    type: sas.analytics.riskhp.models.DisplayRule.dataItemTypes[0],
                    value: 0,
                    column: []
                });
            }
        }

        model.setProperty("/operator", operator);
        self.updateApplyButtonState();
    },

    handleDataItem1TypeSelection: function(event) {
        var self = this;
        var view = self.getView();
        var model = view.getModel();
        var di1TypeKey = event.getParameter("selectedItem").getProperty("key");

        var di1Type = _.find(sas.analytics.riskhp.models.DisplayRule.dataItemTypes, function(dataItemType) {
            return dataItemType == di1TypeKey;
        });

        model.setProperty("/dataItem1/type", di1Type);
        self.updateApplyButtonState();
    },

    handleDataItem2TypeSelection: function(event) {
        var self = this;
        var view = self.getView();
        var model = view.getModel();
        var di2TypeKey = event.getParameter("selectedItem").getProperty("key");

        var di2Type = _.find(sas.analytics.riskhp.models.DisplayRule.dataItemTypes, function(dataItemType) {
            return dataItemType == di2TypeKey;
        });

        model.setProperty("/dataItem2/type", di2Type);
        self.updateApplyButtonState();
    },

    handleAppliesToTypeSelection: function(event) {
        var self = this;
        var view = self.getView();
        var model = view.getModel();
        var appliesToTypeKey = event.getParameter("selectedItem").getProperty("key");

        var appliesToType = _.find(sas.analytics.riskhp.models.DisplayRule.appliesToTypes, function(atType) {
            return atType == appliesToTypeKey;
        });

        model.setProperty("/appliesTo/type", appliesToType);
        self.updateApplyButtonState();
    },

    setReferenceColumn: function (dataItems) {
        var self = this;
        var view = self.getView();
        var model = view.getModel();

        model.setProperty("/referenceColumn",
            dataItems
        );
        self.updateApplyButtonState();
    },

    /**
     * returns a string representation of the crosstab column
     * @param info - crosstab pickinfo
     * @returns {string}
     */
    getLabelForColumn: function(column) {
        var self = this;
        var columnStr = "";
        if (self.crosstab) {
            var dataSource = self.crosstab.context.cubeDataModel;

            if (column) {
                _.each(column, function (colItem) {
                    var dataItem = dataSource.getDataItemById(colItem.id);
                    columnStr += (dataItem ? dataItem.getDisplayLabel(true) : colItem.name) + " ";
                });
            }
        }
        return columnStr;
    },


    requiresDataItem1: function() {
        var self = this;
        var operator = self.getView().getModel().getProperty("/operator");
        return self.DisplayRulesUtil.requiresDataItem1(operator);
    },

    setDataItem1: function (newValue) {
        var self = this;
        var view = self.getView();
        var model = view.getModel();

        if (Array.isArray(newValue)) {
            model.setProperty("/dataItem1/type", sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_COL);
            view.dataItem1TypeSelect.setSelectedKey(sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_COL);

            model.setProperty("/dataItem1/column",
                newValue
            );
        } else {
            // must be a value
            model.setProperty("/dataItem1/type", sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE);
            view.dataItem1TypeSelect.setSelectedKey(sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE);
            model.setProperty("/dataItem1/value",
                newValue
            );
        }
        self.updateApplyButtonState();
    },

    updateApplyButtonState: function() {
        var self = this;
        var view = self.getView();
        view.applyButton.setEnabled(self.isEditorValid());
    },

    isEditorValid: function() {
        var self = this;
        var view = self.getView();
        var model = view.getModel();

        var cubeData = self.crosstab.context.cubeDataModel;

        var description = model.getProperty("/description");
        if (description.length === 0) {
            return false;
        }

        var refCol = model.getProperty("/referenceColumn");
        if (!refCol || refCol.length === 0 || !DisplayRulesManager.isValidColumn(cubeData, refCol)) {
            return false;
        }

        if (self.requiresDataItem1()) {
            var dataItem1 = model.getProperty("/dataItem1");
            if (dataItem1.type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE) {
                if (dataItem1.value.length === 0) {
                    return false;
                }
            } else {
                if (!dataItem1.column || dataItem1.column.length === 0 || !DisplayRulesManager.isValidColumn(cubeData, dataItem1.column)) {
                    return false;
                }
            }
        }

        if (self.requiresDataItem2()) {
            var dataItem2 = model.getProperty("/dataItem2");
            if (dataItem2.type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE) {
                if (dataItem2.value.length === 0) {
                    return false;
                }
            } else {
                if (!dataItem2.column || dataItem2.column.length === 0  || !DisplayRulesManager.isValidColumn(cubeData, dataItem2.column)) {
                    return false;
                }
            }
        }
        var appliesToType = model.getProperty("/appliesTo/type");
        if (appliesToType === sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_COL) {
            var appliesToColumn = model.getProperty("/appliesTo/column");
            if (!appliesToColumn || appliesToColumn.length === 0 || !DisplayRulesManager.isValidColumn(cubeData, appliesToColumn)) {
                return false;
            }
        }

        return true;
    },


    requiresDataItem2: function() {
        var self = this;
        var operator = self.getView().getModel().getProperty("/operator");
        return self.DisplayRulesUtil.requiresDataItem2(operator);
    },

    setDataItem2: function (newValue) {
        var self = this;
        var view = self.getView();
        var model = view.getModel();

        if (Array.isArray(newValue)) {
            model.setProperty("/dataItem2/type", sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_COL);
            view.dataItem2TypeSelect.setSelectedKey(sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_COL);

            model.setProperty("/dataItem2/column",
                newValue
            );
        } else {
            // must be a value
            model.setProperty("/dataItem2/type", sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE);
            view.dataItem2TypeSelect.setSelectedKey(sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE);
            model.setProperty("/dataItem2/value",
                newValue
            );
        }
        self.updateApplyButtonState();
    },

    setAppliesToColumn: function (dataItems){
        var self = this;
        var view = self.getView();
        var model = view.getModel();

        model.setProperty("/appliesTo/type", sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_COL);
        view.appliesToTypeSelect.setSelectedKey(sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_COL);

        model.setProperty("/appliesTo/column",
            dataItems
        );
        self.updateApplyButtonState();
    },


    getDataItemTypeLabel: function(dataItemType) {
        var self = this;
        if (dataItemType === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE) {
            return self._getResourceBundle().getText("HPRisk.DisplayRules.Data.Item.Type.Value.txt");
        } else {
            return self._getResourceBundle().getText("HPRisk.DisplayRules.Data.Item.Type.Column.txt");
        }
    },

    getAppliesToTypeLabel: function(appliesToType) {
        var self = this;
        if (appliesToType === sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_REF_COL) {
            return self._getResourceBundle().getText("HPRisk.DisplayRules.Applies.To.Type.Reference.Column.txt");
        } else if (appliesToType === sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_ROW) {
            return self._getResourceBundle().getText("HPRisk.DisplayRules.Applies.To.Type.Row.txt");
        } else {
            return self._getResourceBundle().getText("HPRisk.DisplayRules.Applies.To.Type.Column.txt");
        }
    },

    // handleFontChooserValueCommit: function(formatting) {
    //     var self = this;
    //     var view = self.getView();
    //     var displayRule = view.getModel().getData();
    //     var style = displayRule.style;
    //
    //     style.font = formatting.getParameter("fontName");
    //     style.fontSize = Number(formatting.getParameter("fontSize").replace("px", ""));
    //     style.fontName = formatting.getParameter("fontName");
    //
    //     style.bold = formatting.getParameter("isBold") === "bold";
    //     style.italic = formatting.getParameter("isItalic") === "italic";
    //     style.underline = formatting.getParameter("isUnderLine") === "underline";
    //     var currentColor = formatting.getParameter("currentColor");
    //
    //     if (currentColor !== sas.hc.m.colorpicker.layout.ColorPickerContainer.prototype.selectionType.NO_COLOR) {
    //         currentColor = currentColor.toLowerCase().replace("#", "0x");
    //     }
    //     style.color = currentColor;
    // },

    handleExternalDragStartEvent: function (event, element) {
        var self = this;
        var pickResults = self.crosstab.uiView.getPickResults(event.offsetX, event.offsetY);
        if (pickResults && pickResults.length > 0) {
            var info = sas.analytics.riskhp.common.riskcommons.CrosstabUtil.getPickInfo(pickResults, self.crosstab.context);
            // $(this).data('dragColumnInfo', info);
            element.data('dragColumnInfo', info);

        }
    },

    handleExternalDragHelperEvent: function (event) {
        var self = this;
        var view = self.getView();

        var pickResults = self.crosstab.uiView.getPickResults(event.offsetX, event.offsetY);
        var columnStr = "";

        if (pickResults && pickResults.length > 0) {
            var info = sas.analytics.riskhp.common.riskcommons.CrosstabUtil.getPickInfo(pickResults, self.crosstab.context);

            if (info) {
                _.each(info.column, function (col) {
                    columnStr += col.label + " ";
                });
                if (info.dataResultPathObject) {
                    var value = info.dataResultPathObject.node.results[info.columnIndex];
                    columnStr += "<br>" + value;
                }
            }
        }

        var helper = document.getElementById("DandDHelper_div");
        if (!helper) {
            helper = document.createElement('div');
            helper.id = 'DandDHelper_div';
            document.getElementById(view.mainContentArea.getId()).appendChild(helper);
        }
        helper = document.getElementById("DandDHelper_div");

        helper.style.position = "absolute";
        helper.style.zIndex = 100;
        helper.innerHTML = columnStr;
        return helper;
    }
});