// (c) 2018, SAS Institute Inc.

jQuery.sap.require("sas.analytics.riskhp.views.displayrules.DisplayRulesUtil");
jQuery.sap.require("sas.hc.ui.commons.HorizontalDivider");
jQuery.sap.require("sas.hc.m.colorpicker.ColorPicker");

sap.ui.jsview('sas.analytics.riskhp.views.displayrules.DisplayRuleEditor', {

    getControllerName: function () {
        return 'sas.analytics.riskhp.views.displayrules.DisplayRuleEditor';
    },

    /**
     * Returns the ResourceBundle used
     * @returns {*|jQuery.sap.util.ResourceBundle}
     * @private
     */
    _getResourceBundle: function () {
        return sas.analytics.riskhp.utils.HprUtil.getBundle();
    },


    createContent: function (ctrl) {
        var view = this;

        var topLabel = new sas.hc.m.Label({
            text:  'Top'
        });

        this.topSplitterPanel = sas.risk.common.widgets.WidgetFactory.buildPanel({
            content: [topLabel],
            topPadding: "0rem",
            bottomPadding: "0rem",
            leftPadding: "0.5rem",
            rightPadding: "0.5rem"
        });

        //
        // Description
        //

        var descriptionLabel = new sas.hc.m.Label({
            text: '{i18N>HPRisk.DisplayRules.Description.label}',
            required: true
        });
        this.descriptionInput = new sas.hc.m.Input({
            liveChange: $.proxy(ctrl.handleDescriptionLiveChange, ctrl),
            minLength: 1,
            maxLength: 128, // HprConstants.SAS_NAME_MAX_LENGTH,
            value: '{/description}',
            disableValidation: false,
            layoutData: new sas.hc.m.FlexItemData({growFactor: 1})

        });

        //
        // Reference Column
        //

        var referenceColumnLabel = new sas.hc.m.Label({
            text: '{i18N>HPRisk.DisplayRules.Reference.Column.label}',
            required: true
        });
        view.referenceColumn = new sas.hc.m.Input({
            placeholder: '{i18N>HPRisk.ComputedVariables.Item.Input.Placeholder.txt}',
            value: {
                path: "/referenceColumn",
                formatter: function (refcol) {
                    return ctrl.getLabelForColumn(refcol);
                }
            },
            editable: false,
            valueState: {
                path: '/referenceColumn',
                formatter: function(column) {
                    var state = sap.ui.core.ValueState.None;
                    if (ctrl.crosstab) {
                        state = DisplayRulesManager.isValidColumn(ctrl.crosstab.context.cubeDataModel, column) ?
                            sap.ui.core.ValueState.None :
                            sap.ui.core.ValueState.Error;
                    }
                    return state;
                }
            },
        }).addStyleClass("forceLightGrayBorder");
        view.referenceColumn.addDelegate(
            new sas.hc.ui.core.delegate.Droppable(
                view.referenceColumn,
                {
                    tolerance: "pointer",
                    drop: function(event,ui) {
                        var info = ui.draggable.data('dragColumnInfo');
                        var dataItems = sas.analytics.riskhp.views.displayrules.DisplayRulesUtil.getTuplesFromPickInfo(info);

                        ctrl.setReferenceColumn(dataItems);
                    }
                }
            )
        );

        //
        // Operator
        //

        var operatorLabel = new sas.hc.m.Label({
            text: '{i18N>HPRisk.DisplayRules.Operator.label}'
        });

        view.operatorSelect = new sas.hc.m.Select({
            change: jQuery.proxy(ctrl.handleOperatorSelection, ctrl)
        });
        var operatorItemTemplate = new sap.ui.core.ListItem({
            text: {
                path: "operators>",
                formatter: sas.analytics.riskhp.views.displayrules.DisplayRulesUtil.getOperatorLabel
            },
            key: "{operators>}",
            tooltip: {
                path: "operators>",
                formatter: sas.analytics.riskhp.views.displayrules.DisplayRulesUtil.getOperatorLabel
            }
        });
        view.operatorSelect.bindItems("operators>/operators", operatorItemTemplate, null);

        //
        // DataItem1
        //

        var di1Label = new sas.hc.m.Label({
            text: '{i18N>HPRisk.DisplayRules.DataItem1.label}'
        });

        view.dataItem1TypeSelect = new sas.hc.m.Select({
            change: jQuery.proxy(ctrl.handleDataItem1TypeSelection, ctrl)
        }).addStyleClass("displayRulesEditorRightMargin");
        var dataItemItemTypeTemplate = new sap.ui.core.ListItem({
            text: {
                path: "dataItemTypes>",
                formatter: $.proxy(ctrl.getDataItemTypeLabel, ctrl)
            },
            key: "{dataItemTypes>}",
            tooltip: {
                path: "dataItemTypes>",
                formatter: $.proxy(ctrl.getDataItemTypeLabel, ctrl)
            }
        });
        view.dataItem1TypeSelect.bindItems("dataItemTypes>/dataItemTypes", dataItemItemTypeTemplate, null);

        view.dataItem1 = new sas.hc.m.Input({
            placeholder: '{i18N>HPRisk.ComputedVariables.Item.Input.Placeholder.txt}',
            value: {
                path: "/dataItem1/column",
                formatter: function (col) {
                    return ctrl.getLabelForColumn(col);
                }
            },
            editable: false,
            valueState: {
                path: '/dataItem1/column',
                formatter: function(column) {
                    var state = sap.ui.core.ValueState.None;
                    if (ctrl.crosstab) {
                        state = DisplayRulesManager.isValidColumn(ctrl.crosstab.context.cubeDataModel, column) ?
                            sap.ui.core.ValueState.None :
                            sap.ui.core.ValueState.Error;
                    }
                    return state;
                }
            },
            visible: {
                parts: [{path: "/operator"},
                        {path: "/dataItem1/type"}],
                formatter: function(operator, di1Type) {
                    if (!operator || !di1Type) {
                        return false;
                    }
                    return ctrl.requiresDataItem1() && di1Type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_COL;
                }
            },
            layoutData: new sas.hc.m.FlexItemData({growFactor: 1})
        }).addStyleClass("forceLightGrayBorder");
        view.dataItem1.addDelegate(
            new sas.hc.ui.core.delegate.Droppable(
                view.dataItem1,
                {
                    tolerance: "pointer",
                    drop: function(event,ui) {
                        var info = ui.draggable.data('dragColumnInfo');
                        var dataItems = sas.analytics.riskhp.views.displayrules.DisplayRulesUtil.getTuplesFromPickInfo(info);

                        ctrl.setDataItem1(dataItems);
                    }
                }
            )
        );

        view.dataItem1ValueInput = new sas.hc.m.Input({
            value: '{/dataItem1/value}',
            liveChange: $.proxy(ctrl.handleDataItem1ValueLiveChange, ctrl),
            formatValue: false,
            enableGroupingSeparator: false,
            autoFormatBeforeDisplay: false,
            type: {
                path: "/referenceColumn",
                formatter: function(referenceColumn) {
                    if (referenceColumn && ctrl.crosstab) {
                        var isNumeric = sas.analytics.riskhp.views.displayrules.DisplayRulesUtil.isNumericColumn(ctrl.crosstab, referenceColumn);
                        return (isNumeric ? sap.m.InputType.Number : sap.m.InputType.Text);
                    } else {
                        return sap.m.InputType.Text;
                    }
                }
            },
            visible: {
                parts: [{path: "/operator"},
                        {path:  "/dataItem1/type"}],
                formatter: function(operator, di1Type) {
                    if (!operator || !di1Type) {
                        return false;
                    }
                    return ctrl.requiresDataItem1() && di1Type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE;
                }
            },
            layoutData: new sas.hc.m.FlexItemData({growFactor: 1})
        });
        view.dataItem1ValueInput.addDelegate(
            new sas.hc.ui.core.delegate.Droppable(
                view.dataItem1ValueInput,
                {
                    tolerance: "pointer",
                    drop: function(event,ui) {
                        var info = ui.draggable.data('dragColumnInfo');
                        // ignore the drop if it's a column header (there is no value)
                        if (info.dataResultPathObject) {
                            var value = info.dataResultPathObject.node.results[info.columnIndex];
                            ctrl.setDataItem1(value);
                        }
                    }
                }
            )
        );

        //
        // Data Item 2
        //

        var di2Label = new sas.hc.m.Label({
            text: '{i18N>HPRisk.DisplayRules.DataItem2.label}'
        });

        view.dataItem2TypeSelect = new sas.hc.m.Select({
            change: jQuery.proxy(ctrl.handleDataItem2TypeSelection, ctrl)
        }).addStyleClass("displayRulesEditorRightMargin");
        var dataItemItemType2Template = new sap.ui.core.ListItem({
            text: {
                path: "dataItemTypes>",
                formatter: $.proxy(ctrl.getDataItemTypeLabel, ctrl)
            },
            key: "{dataItemTypes>}",
            tooltip: {
                path: "dataItemTypes>",
                formatter: $.proxy(ctrl.getDataItemTypeLabel, ctrl)
            }
        });
        view.dataItem2TypeSelect.bindItems("dataItemTypes>/dataItemTypes", dataItemItemType2Template, null);

        view.dataItem2 = new sas.hc.m.Input({
            placeholder: '{i18N>HPRisk.ComputedVariables.Item.Input.Placeholder.txt}',
            value: {
                path: "/dataItem2/column",
                formatter: function (col) {
                    return ctrl.getLabelForColumn(col);
                }
            },
            editable: false,
            valueState: {
                path: '/dataItem2/column',
                formatter: function(column) {
                    var state = sap.ui.core.ValueState.None;
                    if (ctrl.crosstab) {
                        state = DisplayRulesManager.isValidColumn(ctrl.crosstab.context.cubeDataModel, column) ?
                            sap.ui.core.ValueState.None :
                            sap.ui.core.ValueState.Error;
                    }
                    return state;
                }
            },
            visible: {
                parts: [{path: "/operator"},
                    {path: "/dataItem2/type"}],
                formatter: function(operator, di2Type) {
                    if (!operator || !di2Type) {
                        return false;
                    }
                    return ctrl.requiresDataItem2() && di2Type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_COL;
                }
            },
            layoutData: new sas.hc.m.FlexItemData({growFactor: 1})
        }).addStyleClass("forceLightGrayBorder");
        view.dataItem2.addDelegate(
            new sas.hc.ui.core.delegate.Droppable(
                view.dataItem2,
                {
                    tolerance: "pointer",
                    drop: function(event,ui) {
                        var info = ui.draggable.data('dragColumnInfo');
                        var dataItems = sas.analytics.riskhp.views.displayrules.DisplayRulesUtil.getTuplesFromPickInfo(info);

                        ctrl.setDataItem2(dataItems);
                    }
                }
            )
        );

        view.dataItem2ValueInput = new sas.hc.m.Input({
            value: '{/dataItem2/value}',
            type: sap.m.InputType.Number,  // we don't support between operator for strings, only numerics
            formatValue: false,
            enableGroupingSeparator: false,
            autoFormatBeforeDisplay: false,
            liveChange: $.proxy(ctrl.handleDataItem2ValueLiveChange, ctrl),
            visible: {
                parts: [{path: "/operator"},
                    {path:  "/dataItem2/type"}],
                formatter: function(operator, di2Type) {
                    if (!operator || !di2Type) {
                        return false;
                    }
                    return ctrl.requiresDataItem2() && di2Type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE;
                }
            },
            layoutData: new sas.hc.m.FlexItemData({growFactor: 1})
        });
        view.dataItem2ValueInput.addDelegate(
            new sas.hc.ui.core.delegate.Droppable(
                view.dataItem2ValueInput,
                {
                    tolerance: "pointer",
                    drop: function(event,ui) {
                        var info = ui.draggable.data('dragColumnInfo');
                        // ignore the drop if it's a column header (there is no value)
                        if (info.dataResultPathObject) {
                            var value = info.dataResultPathObject.node.results[info.columnIndex];
                            ctrl.setDataItem2(value);
                        }
                    }
                }
            )
        );
        //
        // Style
        //

        var styleLabel = new sas.hc.m.Label({
            text: '{i18N>HPRisk.DisplayRules.Style.label}'
        });

        view.fontChooser = new sas.hc.m.fontChooser.FontChooser({
            formatting: {
                isItalic: "none",
                isBold: "normal",
                isUnderLine: "none",
                fontSize: "10px",
                currentColor: "#2998FF"//,
                // fontName: "AvenirNext"
            },
            showCustomFonts: false,
            showBoldButton: false,
            showItalicButton: false,
            showUnderLineButton: false,
            showColorPicker: false,
            showAlignmentButton: false,
            showFontFamily: true,
            showFontSize: false,
            previewText: rb.getText("samples.FontChooser.preview.Title.txt"),
            showPreview: false,
            valueCommit: $.proxy(ctrl.handleFontChange, ctrl)
        });

        view.boldTB = new sas.hc.m.ToggleButton({
            icon: sas.icons.HC.BOLD,
            pressed: '{/style/bold}',
            press: $.proxy(ctrl.handleBoldChange, ctrl)
        });

        view.italicTB = new sas.hc.m.ToggleButton({
            icon: sas.icons.HC.ITALIC,
            pressed: '{/style/italic}',
            press: $.proxy(ctrl.handleItalicChange, ctrl)
        });

        view.underlineTB = new sas.hc.m.ToggleButton({
            icon: sas.icons.HC.UNDERLINE,
            pressed: '{/style/underline}',
            press: $.proxy(ctrl.handleUnderlineChange, ctrl)
        });

        view.foregroundCP = new sas.hc.m.colorpicker.ColorPicker({
            currentColor: '{/style/color}',
            colorChange: $.proxy(ctrl.handleColorChange, ctrl)
        }).addStyleClass("displayRulesColorPicker");

        view.backgroundCP = new sas.hc.m.colorpicker.ColorPicker({
            currentColor: '{/style/backgroundColor}',
            colorChange: $.proxy(ctrl.handleBackgroundColorChange, ctrl)
        }).addStyleClass("displayRulesColorPicker").addStyleClass("displayRulesBackgroundColorPicker");

        //
        // Applies To
        //

        var appliesToLabel = new sas.hc.m.Label({
            text: '{i18N>HPRisk.DisplayRules.Applies.To.label}'
        });

        view.appliesToTypeSelect = new sas.hc.m.Select({
            change: jQuery.proxy(ctrl.handleAppliesToTypeSelection, ctrl),
            enabled: true
        }).addStyleClass("displayRulesEditorRightMargin");
        var appliesToTypeTemplate = new sap.ui.core.ListItem({
            text: {
                path: "appliesToTypes>",
                formatter: $.proxy(ctrl.getAppliesToTypeLabel, ctrl)
            },
            key: "{appliesToTypes>}",
            tooltip: {
                path: "appliesToTypes>",
                formatter: $.proxy(ctrl.getAppliesToTypeLabel, ctrl)
            }
        });
        view.appliesToTypeSelect.bindItems("appliesToTypes>/appliesToTypes", appliesToTypeTemplate, null);

        view.appliesToDataItem = new sas.hc.m.Input({
            placeholder: '{i18N>HPRisk.ComputedVariables.Item.Input.Placeholder.txt}',
            value: {
                path: "/appliesTo/column",
                formatter: function (col) {
                    return ctrl.getLabelForColumn(col);
                }
            },
            valueState: {
                path: '/appliesTo/column',
                formatter: function(column) {
                    var state = sap.ui.core.ValueState.None;
                    if (ctrl.crosstab) {
                        state = DisplayRulesManager.isValidColumn(ctrl.crosstab.context.cubeDataModel, column) ?
                            sap.ui.core.ValueState.None :
                            sap.ui.core.ValueState.Error;
                    }
                    return state;
                }
            },
            editable: false,
            layoutData: new sas.hc.m.FlexItemData({growFactor: 1}),
            visible: {
                path: "/appliesTo/type",
                formatter: function(appliesToType) {
                    if (!appliesToType) {
                        return false;
                    }
                    return appliesToType === sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_COL;
                }
            }
        }).addStyleClass("forceLightGrayBorder");
        view.appliesToDataItem.addDelegate(
            new sas.hc.ui.core.delegate.Droppable(
                view.appliesToDataItem,
                {
                    tolerance: "pointer",
                    drop: function(event,ui) {
                        var info = ui.draggable.data('dragColumnInfo');
                        var dataItems = sas.analytics.riskhp.views.displayrules.DisplayRulesUtil.getTuplesFromPickInfo(info);

                        ctrl.setAppliesToColumn(dataItems);
                    }
                }
            )
        );

        //
        // Preview style
        //

        var previewLabel = new sas.hc.m.Label({
            text: '{i18N>HPRisk.DisplayRules.Preview.label}'
        });
        view.previewTextArea = new sas.hc.m.TextArea({
            value: '{i18N>HPRisk.DisplayRules.Preview.Text.label}',
            editable: false//,
            // width: '100%'
        });

        //
        // Layout
        //

        var editorLayout = new sas.hc.ui.layout.VerticalLayout({
            content: [
                new sas.hc.ui.layout.HorizontalLayout({
                    content: [
                        new sas.hc.ui.layout.VerticalLayout({
                            content: [
                                descriptionLabel,
                                view.descriptionInput
                            ]
                        }).addStyleClass("displayRulesLeftCol")
                    ]
                }).addStyleClass("fullWidth").addStyleClass("displayRulesRow"),
                new sas.hc.ui.layout.HorizontalLayout({
                    content: [
                        new sas.hc.ui.layout.VerticalLayout({
                            content: [
                                new sas.hc.m.HBox({
                                    items: [
                                        new sas.hc.ui.layout.VerticalLayout({
                                            content: [
                                                referenceColumnLabel,
                                                view.referenceColumn // dropRefColPanel
                                            ],
                                            layoutData: new sas.hc.m.FlexItemData({growFactor: 1})
                                        }).addStyleClass("displayRulesRefColLayout"),
                                        new sas.hc.ui.layout.VerticalLayout({
                                            content: [
                                                operatorLabel,
                                                view.operatorSelect
                                            ]
                                        }).addStyleClass("displayRulesEditorLeftMargin")
                                    ]
                                }).addStyleClass("fullWidth")
                            ]
                        }).addStyleClass("displayRulesLeftCol")
                    ]
                }).addStyleClass("fullWidth").addStyleClass("displayRulesRow"),
                new sas.hc.ui.layout.HorizontalLayout({
                    content: [
                        new sas.hc.ui.layout.VerticalLayout({
                            content: [
                                di1Label,
                                new sas.hc.m.HBox({
                                    items: [
                                        view.dataItem1TypeSelect,
                                        view.dataItem1ValueInput,
                                        view.dataItem1
                                    ]
                                })
                            ],
                            visible: {
                                path: "/operator",
                                formatter: $.proxy(ctrl.requiresDataItem1, ctrl)
                            }
                        }).addStyleClass("displayRulesLeftCol"),
                        new sas.hc.ui.layout.VerticalLayout({
                            content: [
                                di2Label,
                                new sas.hc.m.HBox({
                                    items: [
                                        view.dataItem2TypeSelect,
                                        view.dataItem2ValueInput,
                                        view.dataItem2
                                    ]
                                })
                            ],
                            visible: {
                                path: "/operator",
                                formatter: $.proxy(ctrl.requiresDataItem2, ctrl)
                            }
                        }).addStyleClass("displayRulesRightCol")
                    ],
                    visible: {
                        path: "/operator",
                        formatter: $.proxy(ctrl.requiresDataItem1, ctrl)
                    }
                }).addStyleClass("fullWidth").addStyleClass("displayRulesRow"),
                new sas.hc.ui.commons.HorizontalDivider().addStyleClass("thinHorizDivider"),
                new sas.hc.ui.layout.HorizontalLayout({
                    content: [
                        new sas.hc.ui.layout.VerticalLayout({
                            content: [
                                styleLabel,
                                new sas.hc.m.HBox({
                                    items: [
                                        view.fontChooser,
                                        new sas.hc.m.HBox({
                                            items: [
                                                view.boldTB,
                                                view.italicTB,
                                                view.underlineTB,
                                                view.foregroundCP,
                                                view.backgroundCP
                                            ]
                                        })
                                    ]
                                }).addStyleClass("allowFlexWrap")
                            ]
                        }).addStyleClass("displayRulesLeftCol"),
                        new sas.hc.ui.layout.VerticalLayout({
                            content: [
                                appliesToLabel,
                                new sas.hc.m.HBox({
                                    items: [
                                        view.appliesToTypeSelect,
                                        view.appliesToDataItem
                                    ]
                                })
                            ]
                        }).addStyleClass("displayRulesRightCol")
                    ]
                }).addStyleClass("fullWidth").addStyleClass("displayRulesRow"),
                new sas.hc.ui.layout.HorizontalLayout({
                    content: [
                        new sas.hc.ui.layout.VerticalLayout({
                            content: [
                                previewLabel,
                                view.previewTextArea
                            ]
                        }).addStyleClass("displayRulesLeftCol")
                    ]
                }).addStyleClass("fullWidth").addStyleClass("displayRulesRow")
            ]
        }).addStyleClass("fullWidth");

        this.bottomPanel = sas.risk.common.widgets.WidgetFactory.buildPanel({
            content: [ editorLayout],
            topPadding: "0rem",
            bottomPadding: "0rem",
            leftPadding: "0.5rem",
            rightPadding: "0.5rem"
        });

        var splitter = new sas.hc.ui.layout.Splitter({
            orientation: sap.ui.core.Orientation.Vertical,
            height: "100%",
            width: "100%",
            contentAreas: [this.topSplitterPanel, this.bottomPanel ]
        });

        var toolbar = this._createToolbar(ctrl);

        this.mainContentArea = new sas.hc.ui.commons.layout.BorderLayoutArea({
            contentAlign: "center",
            overflowX: "hidden",
            overflowY: "hidden",
            content: [splitter]
        }).addStyleClass("computedVariableEditor");

        var mainLayout = new sas.hc.ui.commons.layout.BorderLayout({
            height: '100%',
            width: '100%',
            center: this.mainContentArea,
            bottom: new sas.hc.ui.commons.layout.BorderLayoutArea({
                size: "2.75rem",
                content: [toolbar]
            })
        });

        return mainLayout;
    },

    _createToolbar: function (ctrl) {
        var view = this;
        view.applyButton = new sas.hc.m.Button({
            text: '{i18N>hprisk.applyButton.txt}',
            press: $.proxy(ctrl.handleApplyPressed, ctrl),
            tooltip: '{i18N>hprisk.applyButton.txt}'
        }).addStyleClass("sasPrimaryButton");

        var cancelButton = new sas.hc.m.Button({
            text: '{i18N>hprisk.cancelButton.txt}',
            press: $.proxy(ctrl.cancelEditing, ctrl),
            tooltip: '{i18N>hprisk.cancelButton.txt}'
        });

        var toolbar = new sas.hc.m.ResponsiveToolbar({
            content : [
                new sas.hc.m.ToolbarSpacer(),
                view.applyButton,
                cancelButton
            ]
        });

        return toolbar;
    }
});