// (c) 2018, SAS Institute Inc.

jQuery.sap.declare("sas.analytics.riskhp.views.displayrules.DisplayRulesCrosstabContextMenuProvider");

jQuery.sap.require("sas.analytics.riskhp.common.riskcommons.crosstab.HprCrosstabContextMenuProvider");
jQuery.sap.require("sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil");

// noinspection JSClosureCompilerSyntax
/**
 * Crosstab context menu provider to be used with the display rules editor
 * @name sas.analytics.riskhp.views.computedvariableeditor.ComputedVarCrosstabContextMenuProvider
 * @extends sas.analytics.riskhp.common.riskcommons.crosstab.HprCrosstabContextMenuProvider
 */
sas.analytics.riskhp.common.riskcommons.crosstab.HprCrosstabContextMenuProvider.extend("sas.analytics.riskhp.views.displayrules.DisplayRulesCrosstabContextMenuProvider", {

    displayRuleEditorController: null,
    CompVarUtil: sas.analytics.riskhp.views.computedvariableeditor.ComputedVariableUtil,

    constructor: function (displayRuleEditorController) {
        this.displayRuleEditorController = displayRuleEditorController;
    },

    _getResourceBundle: function() {
        return sas.analytics.riskhp.utils.HprUtil.getBundle();
    },

    _buildMeasureHeaderContextMenu: function(oMenu, crosstab, info) {
        var self = this;
        self._buildContextMenuFromPickInfo(oMenu, crosstab, info);
    },

    _buildColumnHeaderMenu: function(oMenu, crosstab, info) {
        // var context = crosstab.context;
        // var node = context.findColumnTreeNode(info.keys);
    },

    _buildContextMenuFromPickInfo:function(oMenu, crosstab, info) {
        var self = this;

        if (self.displayRuleEditorController) {

            var dataItems = self.CompVarUtil.getTuplesFromPickInfo(info);

            var columnSubMenu = new sas.hc.ui.unified.Menu();
            columnSubMenu.addItem(self._createMenuItem(
                self._getResourceBundle().getText("HPRisk.DisplayRules.Reference.Column.menu.action.label"),
                self._getResourceBundle().getText("HPRisk.DisplayRules.Reference.Column.menu.action.label"),
                function () {
                    self.displayRuleEditorController.setReferenceColumn(dataItems);
                }
            ));
            if (self.displayRuleEditorController.requiresDataItem1()) {
                columnSubMenu.addItem(self._createMenuItem(
                    self._getResourceBundle().getText("HPRisk.DisplayRules.DataItem1.menu.action.label"),
                    self._getResourceBundle().getText("HPRisk.DisplayRules.DataItem1.menu.action.label"),
                    function () {
                        self.displayRuleEditorController.setDataItem1(dataItems);
                    }
                ));
            }
            if (self.displayRuleEditorController.requiresDataItem2()) {
                columnSubMenu.addItem(self._createMenuItem(
                    self._getResourceBundle().getText("HPRisk.DisplayRules.DataItem2.menu.action.label"),
                    self._getResourceBundle().getText("HPRisk.DisplayRules.DataItem2.menu.action.label"),
                    function () {
                        self.displayRuleEditorController.setDataItem2(dataItems);
                    }
                ));
            }
            columnSubMenu.addItem(self._createMenuItem(
                self._getResourceBundle().getText("HPRisk.DisplayRules.Applies.To.Column.menu.action.label"),
                self._getResourceBundle().getText("HPRisk.DisplayRules.Applies.To.Column.menu.action.label"),
                function () {
                    self.displayRuleEditorController.setAppliesToColumn(dataItems);
                }
            ));

            var columnMenu = self._createMenuItem(
                self._getResourceBundle().getText("HPRisk.ComputedVariables.Crosstab.Context.Menu.Column.Root.txt"),
                self._getResourceBundle().getText("HPRisk.ComputedVariables.Crosstab.Context.Menu.Column.Root.txt"),
                function () {
                }
            );
            columnMenu.setSubmenu(columnSubMenu);
            oMenu.addItem(columnMenu);
            // if right-clicked on a data cell
            if (info.dataResultPathObject) {
                if (self.displayRuleEditorController.requiresDataItem1() || self.displayRuleEditorController.requiresDataItem2()) {
                    var valueMenu = self._createMenuItem(
                        self._getResourceBundle().getText("HPRisk.ComputedVariables.Crosstab.Context.Menu.Value.Root.txt"),
                        self._getResourceBundle().getText("HPRisk.ComputedVariables.Crosstab.Context.Menu.Value.Root.txt"),
                        function () {
                        }
                    );
                    var valueSubMenu = new sas.hc.ui.unified.Menu();
                    if (self.displayRuleEditorController.requiresDataItem1()) {
                        valueSubMenu.addItem(self._createMenuItem(
                            self._getResourceBundle().getText("HPRisk.DisplayRules.DataItem1.menu.action.label"),
                            self._getResourceBundle().getText("HPRisk.DisplayRules.DataItem1.menu.action.label"),
                            function () {
                                value = info.dataResultPathObject.node.results[info.columnIndex];
                                self.displayRuleEditorController.setDataItem1(value);
                            }
                        ));
                    }
                    if (self.displayRuleEditorController.requiresDataItem2()) {
                        valueSubMenu.addItem(self._createMenuItem(
                            self._getResourceBundle().getText("HPRisk.DisplayRules.DataItem2.menu.action.label"),
                            self._getResourceBundle().getText("HPRisk.DisplayRules.DataItem2.menu.action.label"),
                            function () {
                                value = info.dataResultPathObject.node.results[info.columnIndex];
                                self.displayRuleEditorController.setDataItem2(value);
                            }
                        ));
                    }
                    valueMenu.setSubmenu(valueSubMenu);
                    oMenu.addItem(valueMenu);
                }

                var value = info.dataResultPathObject.node.results[info.columnIndex];
                oMenu.addItem(self._createMenuItem(
                    self._getResourceBundle().getText("HPRisk.Crosstab.Context.Menu.CopyToClipboard.txt"),
                    "copyValue",
                    function () {
                        self.CROSSTAB_UTIL.copyTextToClipboard(value);
                    }
                ));
            }
        }
    },

    /**
     * Adds the appropriate MenuItem and sub-menus for the currently selected cell as identified by the entry and
     * info objects.
     * @param oMenu - the menu to add the menu items to
     * @param crosstab - The crosstab for this menu
     * @param info - the info object extracted from the pick results
     * @param entry
     */
    _buildDataCellContextMenu: function (oMenu, crosstab, info, entry) {
        var self = this;
        self._buildContextMenuFromPickInfo(oMenu, crosstab, info);
    }

});
