// (c) 2018, SAS Institute Inc.

jQuery.sap.require('sas.analytics.riskhp.common.riskcommons.CrosstabUtil');
jQuery.sap.require("sas.analytics.riskhp.models.HPRiskModelManager");
jQuery.sap.require("sas.analytics.riskhp.utils.ViewManager");
jQuery.sap.require("sas.analytics.riskhp.models.DisplayRulesManager");

sap.ui.controller('sas.analytics.riskhp.views.displayrules.DisplayRulesPane', {

    CrosstabUtil: sas.analytics.riskhp.common.riskcommons.CrosstabUtil,
    HprUtil: sas.analytics.riskhp.utils.HprUtil,
    ViewManager: sas.analytics.riskhp.utils.ViewManager,

    onInit: function() {
        sas.eventBus.subscribe(HprConstants.CHANNEL_DISPLAY_RULES_PANE, HprConstants.EVENT_DISPLAY_RULES_PANE_CHANGED, this.onChangeDisplayRulesPane, this);
        sas.eventBus.subscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_CROSSTAB_OPTIONS_NAME_UPDATED, this.onParentPanelOptionsNameUpdate, this);
        sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_DISPLAY_RULES_CHANGED, this.handleDisplayRulesChanged, this);
        sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_DISPLAY_RULES_VALIDATED, this.handleDisplayRulesValidated, this);
        sas.eventBus.subscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_CLOSE_DISPLAY_RULE_EDITOR, this.onDisplayRuleEditorClose, this);
    },

    onExit: function() {
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_DISPLAY_RULES_PANE, HprConstants.EVENT_DISPLAY_RULES_PANE_CHANGED, this.onChangeDisplayRulesPane, this);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_CROSSTAB_OPTIONS_NAME_UPDATED, this.onParentPanelOptionsNameUpdate, this);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_DISPLAY_RULES_CHANGED, this.handleDisplayRulesChanged, this);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_DISPLAY_RULES_VALIDATED, this.handleDisplayRulesValidated, this);
        sas.eventBus.unsubscribe(HprConstants.CHANNEL_EXPLORATION, HprConstants.EVENT_CLOSE_DISPLAY_RULE_EDITOR, this.onDisplayRuleEditorClose, this);

    },

    onBeforeRendering: function () {
    },

    onAfterRendering: function () {
    },

    _getResourceBundle: function () {
        return sas.analytics.riskhp.utils.HprUtil.getBundle();
    },



    /**
     * Update title text based on parent panel
     * @param titleStr
     */
    updateParentPanelTitle: function(titleStr) {
        var ctrl = this;

        if (!titleStr) {
            var displayRulesModel = $.extend(true, {}, ctrl._getDisplayRulesModel()); // Create copy
            var crosstabId = displayRulesModel.getProperty("/crosstabId");
            var selectedRiskPanelsView = this.HprUtil.getSelectedRiskPanelsView();
            var riskPanelsViewDelegate = selectedRiskPanelsView.getDelegate();
            titleStr = riskPanelsViewDelegate.getTitleForContent(crosstabId);
        }
        ctrl.getView().parentPanelTitleText.setText(titleStr);
    },



    /**
     * Returns the model for the HierarchiesPane view which holds the Crosstab properties to make
     * the HierarchiesPane function correctly.
     * @returns {*}
     * @private
     */
    _getDisplayRulesModel: function() {
        var ctrl = this;
        var view = ctrl.getView();
        return view.getModel();
    },


    updateActionStates: function() {
        var ctrl = this;
        var view = ctrl.getView();

        var explorationPanesStateModel = sap.ui.getCore().getModel(HprConstants.MODEL_ID_EXPLORATION_PANES_STATE);
        var inDisplayRuleEditor = explorationPanesStateModel.getProperty("/displayRuleEditorMode");

        var selectedItem = view.displayRulesList.getSelectedItem();
        view.editMenuItem.setEnabled(!inDisplayRuleEditor && selectedItem != null);
        view.copyMenuItem.setEnabled(!inDisplayRuleEditor && selectedItem != null);
        view.removeMenuItem.setEnabled(!inDisplayRuleEditor && selectedItem != null);

        if (selectedItem === null) {
            view.upButton.setEnabled(false);
            view.downButton.setEnabled(false);
        } else {
            var ruleId = selectedItem.getCustomData()[0].getValue();

            var rules = view.getModel().getData().rules;
            var index = _.findIndex(rules, function (rule) {
                return rule.ruleId === ruleId;
            });
            view.upButton.setEnabled(index > 0);
            view.downButton.setEnabled(index < rules.length - 1);
        }
    },

    onUpButtonPressed : function(buttonEvent) {
        var ctrl = this;
        var view = ctrl.getView();

        var selectedItem = view.displayRulesList.getSelectedItem();
        if (selectedItem) {
            var ruleId = selectedItem.getCustomData()[0].getValue();
            var crosstabId = view.getModel().getProperty("/crosstabId");

            DisplayRulesManager.moveDisplayRule(true, ruleId, crosstabId);
        }
    },

    onDownButtonPressed : function(buttonEvent) {
        var ctrl = this;
        var view = ctrl.getView();

        var selectedItem = view.displayRulesList.getSelectedItem();
        if (selectedItem) {
            var ruleId = selectedItem.getCustomData()[0].getValue();
            var crosstabId = view.getModel().getProperty("/crosstabId");

            DisplayRulesManager.moveDisplayRule(false, ruleId, crosstabId);
        }
    },

    // enable/disable remove menu item, up button, down button based on selection
    onDisplayRuleSelectionChange : function(selectionChangeEvent) {
        var ctrl = this;
        ctrl.updateActionStates();
    },

    onNewDisplayRule: function() {
        var self = this;
        var view = self.getView();
        var crosstabId = view.getModel().getProperty("/crosstabId");

        var expCtrl = sas.analytics.riskhp.utils.ViewManager.getExplorationView().getController();
        expCtrl.createDisplayRule.call(expCtrl, crosstabId);
    },

    editDisplayRule: function() {
        var self = this;
        var view = self.getView();
        var selectedItem = view.displayRulesList.getSelectedItem();
        var ruleId = selectedItem.getCustomData()[0].getValue();
        var crosstabId = view.getModel().getProperty("/crosstabId");

        var expCtrl = sas.analytics.riskhp.utils.ViewManager.getExplorationView().getController();
        expCtrl.editDisplayRule.call(expCtrl, crosstabId, ruleId);
    },

    copyDisplayRule: function() {
        var self = this;
        var view = self.getView();
        var selectedItem = view.displayRulesList.getSelectedItem();
        var ruleId = selectedItem.getCustomData()[0].getValue();
        var crosstabId = view.getModel().getProperty("/crosstabId");

        var expCtrl = sas.analytics.riskhp.utils.ViewManager.getExplorationView().getController();
        expCtrl.copyDisplayRule.call(expCtrl, crosstabId, ruleId);
    },

    deleteDisplayRule: function() {
        var self = this;
        var rb = self._getResourceBundle();

        var view = self.getView();
        var selectedItem = view.displayRulesList.getSelectedItem();
        var ruleId = selectedItem.getCustomData()[0].getValue();

        var confirmMessage = rb.getText("HPRisk.DisplayRules.Delete.Confirm.Unused.txt");
        // Confirm deletion including presenting the user with a list of crosstabs with this display rule active.
        var crosstabs = DisplayRulesManager.getCrosstabsWithRuleActive(ruleId);
        if (crosstabs.length > 0) {
            // talked with Mark A.  Can't get crosstab titles for pages that have not yet been rendered, so for now only
            // display the number of crosstabs that have the display rule active
            confirmMessage = (crosstabs.length === 1 ? rb.getText("HPRisk.DisplayRules.Delete.Single.txt") : rb.getText("HPRisk.DisplayRules.Delete.Multiple.fmt", crosstabs.length)) +
                "  " + rb.getText("HPRisk.DisplayRules.Delete.Confirm.txt");
        }
        var confirmationDialog = new sas.hc.m.MessageDialog({
            type: sas.hc.m.MessageDialogType.Warning,
            title: rb.getText("HPRisk.DisplayRules.Delete.title.txt"),
            text: confirmMessage,
            buttonActions: [
                new sas.hc.m.Button({
                    text: rb.getText("hprisk.deleteButton.txt"),
                    press: function (evt) {
                        DisplayRulesManager.deleteDisplayRule(ruleId);
                    }
                }),
                {
                    text: sas.hc.m.MessageDialogAction.Cancel,
                    initialFocus: true,
                    defaultButton: true
                }
            ]
        });
        confirmationDialog.open();
    },

    onParentPanelOptionsNameUpdate: function(channelId, eventId, data) {
        var ctrl = this;
        var view = ctrl.getView();
        var displayRulesModel = view.getModel();
        var crosstabModelId = displayRulesModel.getProperty('/crosstabId');
        if (crosstabModelId === data.crosstabId) {
            ctrl.updateParentPanelTitle(data.newCrosstabName);
        }
    },

    // Change the Display Rules panes depending on what crosstab is selected in the Exploration view
    onChangeDisplayRulesPane: function(channelId, eventId, data) {
        var self = this;
        var view = self.getView();

        // Check to see if the Crosstab in the model is the same one the user
        // clicked on, if so, just return and don't set model again
        var displayRulesModel = view.getModel();
        var crosstabModelId = displayRulesModel.getProperty('/crosstabId');

        if (!data.crosstabId || data.displayRulesType.toUpperCase() === HprConstants.HPR_DISPLAY_RULES_PANE_ZERO_STATE_TYPE) {
            view.navContainer.to(view.zeroStatePage, "show");
        } else {
            view.navContainer.to(view.defaultPage, "show");

            if (crosstabModelId === data.crosstabId) {
                return;
            }
            else {
                // the formatter for the actionIcons depends on DataSourceCacheManager.getSelectedDataSource which is
                // updated after the EVENT_DISPLAY_RULES_PANE_CHANGED event is received, so using setTimeout to delay
                // updating the model until the selected data source has changed
                setTimeout(function() {
                    var newModel = new sas.risk.common.models.RiskJSONModel();
                    newModel.setData({
                        crosstabId: data.crosstabId,
                        rules: DisplayRulesManager.getDisplayRulesForCrosstab(data.crosstabId)
                    });
                    view.setModel(newModel);

                    self.updateParentPanelTitle();
                }, 1);
            }
        }
    },

    onDisplayRuleEditorClose: function(channelId, eventId, data) {
        var self = this;
        setTimeout(function() {
            self.updateActionStates();
        },1);
    },

    handleDisplayRulesValidated: function(channelId, eventId, data) {
        var self = this;
        var view = self.getView();
        view.getModel().refresh(true);
    },

    handleDisplayRulesChanged: function(channelId, eventId, data) {
        var self = this;
        var view = self.getView();
        var crosstabId = view.getModel().getProperty("/crosstabId");

        var selectedRuleId = null;
        var selectedItem = view.displayRulesList.getSelectedItem();
        if (selectedItem) {
            selectedRuleId = selectedItem.getBindingContext().getObject().ruleId;
        }
        // Setting the property did not refresh the list when a display rule description had changed
        // view.getModel().setProperty("/rules", DisplayRulesManager.getDisplayRulesForCrosstab(crosstabId));

        var newModel = new sas.risk.common.models.RiskJSONModel();
        newModel.setData({
            crosstabId: crosstabId,
            rules: DisplayRulesManager.getDisplayRulesForCrosstab(crosstabId)
        });
        view.setModel(newModel);
        setTimeout(function() {
            self.updateActionStates();
        },1);
        if (selectedItem) {
            self.selectDisplayRuleById(selectedRuleId);
        }
    },

    selectDisplayRuleById: function(ruleId) {
        var self = this;
        var view = self.getView();

        var selectedItem = _.find(view.displayRulesList.getItems(), function(item) {
            return item.getBindingContext().getObject().ruleId === ruleId;
        });
        if (selectedItem) {
            view.displayRulesList.setSelectedItem(selectedItem);
        }
    },

    handleEnableDisableDisplayrule: function (oEvent) {
        var self = this;
        var view = self.getView();
        var viewModel = view.getModel();

        var item = oEvent.getSource().getParent();
        var cd = item.getCustomData()[0];
        var ruleId = cd.getValue();

        var crosstabId = viewModel.getProperty("/crosstabId");
        DisplayRulesManager.toggleDisplayRule(ruleId, crosstabId);
    }
});