// (c) 2017, SAS Institute Inc.

jQuery.sap.require("sas.hc.m.Menu");
jQuery.sap.require("sas.hc.m.MenuItem");
jQuery.sap.require("sas.hc.m.ResponsiveToolbar");
jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");


sap.ui.jsview('sas.analytics.riskhp.views.displayrules.DisplayRulesPane', (function() {

    return {

        COLUMN : "column",
        REMOVE : "remove",
        MOVECOLUMNTOTOP : "moveColumnToTop",
        MOVECOLUMNTOBOTTOM : "moveColumnToBottom",
        MOVECOLUMNUP : "moveColumnUp",
        MOVECOLUMNDOWN : "moveColumnDown",

        getControllerName: function() {
            return 'sas.analytics.riskhp.views.displayrules.DisplayRulesPane';
        },


        _createToolbarOverflowMenu: function (ctrl) {
            var view = this;

            var overflowMenu = new sas.hc.m.Menu();

            view.editMenuItem = new sas.hc.m.MenuItem({
                text: ctrl._getResourceBundle().getText('HPRisk.DisplayRules.Edit.menu.action.label'),
                tooltip: ctrl._getResourceBundle().getText('HPRisk.DisplayRules.Edit.menu.action.label'),
                enabled: false, // only enabled by selection
                press: $.proxy(ctrl.editDisplayRule, ctrl)
            });
            overflowMenu.addItem(view.editMenuItem);

            view.copyMenuItem = new sas.hc.m.MenuItem({
                text: ctrl._getResourceBundle().getText('HPRisk.DisplayRules.Copy.menu.action.label'),
                tooltip: ctrl._getResourceBundle().getText('HPRisk.DisplayRules.Copy.menu.action.label'),
                enabled: false, // only enabled by selection
                press: $.proxy(ctrl.copyDisplayRule, ctrl)
            });
            overflowMenu.addItem(view.copyMenuItem);

            view.removeMenuItem = new sas.hc.m.MenuItem({
                text: ctrl._getResourceBundle().getText('HPRisk.DisplayRules.Delete.menu.action.label'),
                tooltip: ctrl._getResourceBundle().getText('HPRisk.DisplayRules.Delete.menu.action.label'),
                enabled: false, // only enabled by selection
                // press: $.proxy(ctrl.removeDisplayRuleFromCrosstab, ctrl)
                press: $.proxy(ctrl.deleteDisplayRule, ctrl)
            });
            overflowMenu.addItem(view.removeMenuItem);

            return overflowMenu;
        },

        _createToolbar: function (ctrl) {
            var view = this;

            view.createButton = new sas.hc.m.Button({
                icon: sas.icons.HC.ADD,
                text: ctrl._getResourceBundle().getText('HPRisk.DisplayRules.New.Rule.action.label'),
                tooltip: ctrl._getResourceBundle().getText('HPRisk.DisplayRules.New.Rule.action.label'),
                enabled: {
                    // disable when in display rules editor
                    path: sas.analytics.riskhp.utils.HprConstants.MODEL_ID_EXPLORATION_PANES_STATE + '>/displayRuleEditorMode',
                    formatter: function (displayRulesEditorMode) {
                        return !displayRulesEditorMode;
                    }
                },
                press: $.proxy(ctrl.onNewDisplayRule, ctrl)
            });

            view.upButton = new sas.hc.m.Button({
                icon: sas.icons.HC.ARROWSMALLUP,
                enabled: false,
                press: $.proxy(ctrl.onUpButtonPressed, ctrl),
                layoutData: new sas.hc.m.OverflowToolbarLayoutData({
                    priority: sap.m.OverflowToolbarPriority.High
                })
            });

            view.downButton = new sas.hc.m.Button({
                icon: sas.icons.HC.ARROWSMALLDOWN,
                enabled: false,
                press: $.proxy(ctrl.onDownButtonPressed, ctrl),
                layoutData: new sas.hc.m.OverflowToolbarLayoutData({
                    priority: sap.m.OverflowToolbarPriority.High
                })
            });

            var overflowButton = new sas.hc.m.MenuButton({
                icon: sas.icons.HC.OVERFLOW,

                overflowPopupBehavior: sas.hc.m.MenuButton.OverflowPopupBehavior.Expand,
                layoutData: new sas.hc.m.OverflowToolbarLayoutData({
                    priority: sap.m.OverflowToolbarPriority.AlwaysOverflow
                }),
                menu: view._createToolbarOverflowMenu(ctrl)
            });

            var toolbar = new sas.hc.m.ResponsiveToolbar({
                height: "100%",
                content: [view.createButton, new sas.hc.m.ToolbarSpacer(), view.upButton, view.downButton, new sas.hc.m.ToolbarSeparator(), overflowButton]
            });

            return toolbar;
        },

        _createDisplayRulesList: function(ctrl) {
            var view = this;
            view.displayRulesList = new sas.hc.m.List({
                headerToolbar: this._createToolbar(ctrl),
                mode: sap.m.ListMode.SingleSelectMaster,
                selectionChange: $.proxy(ctrl.onDisplayRuleSelectionChange, ctrl)
            });

            var oItemTemplate = new sas.hc.m.StandardListItem({
                title: {
                    path: "",
                    formatter: function (dr) {
                        var label = "";
                        if (dr) {
                            var rule = DisplayRulesManager.getDisplayRuleById(dr.ruleId);
                            if (rule) {
                                label = rule.description.length > 0 ? rule.description : sas.analytics.riskhp.views.displayrules.DisplayRulesUtil.generateWhenLabel(rule);
                            }
                        }
                        return label;
                    }
                },
                type: sap.m.ListType.Active,
                actionIcons: [
                    new sas.hc.ui.core.Icon({
                        src: {
                            parts: ["ruleId", "active"],
                            formatter: function(ruleId, active) {
                                // can't enable/disable Icons so have to use different icon for active/not-active
                                var cubeData = DataSourceCacheManager.getSelectedDataSource();

                                var rule = DisplayRulesManager.getDisplayRuleById(ruleId);
                                if (!rule || !cubeData) {
                                    return sas.icons.HC.BLANK;
                                }
                                var isValid = DisplayRulesManager.isDisplayRuleValidForDataSource(cubeData.dataSource.id, ruleId);
                                if (!isValid) {
                                    return sas.icons.HC.ALERT;
                                } else if (active) {
                                    return sas.icons.HC.VISIBLE;
                                } else {
                                    return sas.icons.HC.HIDDEN;
                                }
                            }
                        },
                        press: $.proxy(ctrl.handleEnableDisableDisplayrule, ctrl)
                    })
                ]
            });
            var ruleIdCustomData = new sap.ui.core.CustomData({key:"ruleId"});
            ruleIdCustomData.bindProperty("value", "ruleId");
            oItemTemplate.addCustomData(ruleIdCustomData);

            view.displayRulesList.bindAggregation("items", {path: "/rules", template: oItemTemplate} );
        },

        createContent: function(controller) {

            this._createDisplayRulesList(controller);

            this.parentPanelTitleText = new sas.hc.m.Text({
                wrapping: false,
                text: ''
            });

            var borderLayout = new sas.hc.ui.commons.layout.BorderLayout({
                width: "100%",
                height: "100%",
                top: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    size: "2rem",
                    overflowX: "hidden",
                    overflowY: "hidden",
                    content: [this.parentPanelTitleText]
                }),
                center: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    overflowX: "hidden",
                    overflowY: "hidden",
                    content: [this.displayRulesList]
                })
            });

            this._createColumnContextMenu(controller);

            this.zeroStatePage = new sas.hc.m.MessagePage({
                text: "", // Must leave empty, causes big font text
                description: this._getResourceBundle().getText("HPRisk.DisplayRules.Zero.State.txt"),
                icon: sas.icons.HC.ZEROSTATEALERT,
                showHeader: false
            });

            this.defaultPage = new sas.hc.m.Page({
                showHeader: false,
                enableScrolling: false,
                content: [borderLayout]//this._createDefaultPageBorderLayout(controller)]
            });
            this.navContainer = new sas.hc.m.NavContainer({
                height: "100%",
                width: "100%",
                pages: [this.zeroStatePage, this.defaultPage]
            });

            return this.navContainer;
        },

        _getResourceBundle: function () {
            return sas.analytics.riskhp.utils.HprUtil.getBundle();
        },

        _createColumnContextMenu: function (controller) {
            // var view = this;
            //
            // var moveToTopColumnMenuItem = new sas.hc.ui.unified.MenuItem({
            //     id: view.MOVECOLUMNTOTOP,
            //     text: view._getResourceBundle().getText("HierarchiesPane.ContextMenu.MoveToTop.txt")
            // });
            // var moveToBottomColumnMenuItem = new sas.hc.ui.unified.MenuItem({
            //     id: view.MOVECOLUMNTOBOTTOM,
            //     text: view._getResourceBundle().getText("HierarchiesPane.ContextMenu.MoveToBottom.txt")
            // });
            // var moveUpColumnMenuItem = new sas.hc.ui.unified.MenuItem({
            //     id: view.MOVECOLUMNUP,
            //     text: view._getResourceBundle().getText("HierarchiesPane.ContextMenu.MoveUp.txt")
            // });
            // var moveDownColumnMenuItem = new sas.hc.ui.unified.MenuItem({
            //     id: view.MOVECOLUMNDOWN,
            //     text: view._getResourceBundle().getText("HierarchiesPane.ContextMenu.MoveDown.txt")
            // });
            //
            // var fnCustomizeColumnContextMenu = function (oMenu, oEvent) {
            //     var oControl = $(oEvent.target).control()[0];
            //
            //     // make decisions on what menu item to show based on the selections
            //     if (oControl && oControl instanceof sas.hc.m.StandardListItem) {
            //         //oMenu.setVisible(true);
            //         oMenu.baseItemTitle = oControl.getTitle();
            //         oMenu.baseItemId = oControl.getId();
            //         var idx = oControl.getId().lastIndexOf("-");
            //         oMenu.baseItemIndex = oControl.getId().slice(idx + 1);
            //     }
            //     else {
            //         oMenu.baseItemTitle = undefined;
            //         oMenu.baseItemId = undefined;
            //         oMenu.baseItemIndex = undefined;
            //     }
            //     return oMenu;
            // };
            //
            // this.columnContextMenu = new sas.hc.ui.unified.Menu({
            //     items: [
            //         moveToTopColumnMenuItem,
            //         moveToBottomColumnMenuItem,
            //         moveUpColumnMenuItem,
            //         moveDownColumnMenuItem
            //     ],
            //     visible : false,
            //     itemSelect: $.proxy(controller.onColumnItemSelectFunction, controller)
            // });
            //
            // this.displayRulesList.setAggregation("contextMenu", this.columnContextMenu);
            // this.displayRulesList.customizeContextMenu = fnCustomizeColumnContextMenu;
        }
    };
}()));