// (c) 2018, SAS Institute Inc.

jQuery.sap.declare("sas.analytics.riskhp.views.displayrules.DisplayRulesUtil");

// rule making requirements
jQuery.sap.require("sas.ltjs.commons.rules.data.RuleDataItemMap");
jQuery.sap.require("sas.ltjs.commons.rules.expressions.CompoundBooleanExpression");
jQuery.sap.require("sas.ltjs.commons.rules.expressions.CompoundBooleanOperator");
jQuery.sap.require("sas.ltjs.commons.rules.expressions.BinaryRelationalExpression");
jQuery.sap.require("sas.ltjs.commons.rules.expressions.ExpressionConditionVariable");
jQuery.sap.require("sas.ltjs.commons.rules.expressions.Expression");
jQuery.sap.require("sas.ltjs.commons.rules.expressions.BinaryRelationalOperator");
jQuery.sap.require("sas.ltjs.commons.rules.expressions.ConstantNumberExpression");
jQuery.sap.require("sas.ltjs.commons.rules.expressions.ConstantTextExpression");
jQuery.sap.require("sas.ltjs.commons.rules.expressions.DataTypeExt");
jQuery.sap.require("sas.ltjs.commons.rules.actions.AttributeBundleTarget");
jQuery.sap.require("sas.ltjs.commons.styles.AttributeBundle");
jQuery.sap.require("sas.ltjs.commons.rules.data.CurrentDataItem");
jQuery.sap.require("sas.ltjs.commons.styles.AttributeNames");
jQuery.sap.require("sas.ltjs.commons.rules.actions.AttributeBundleAction");
jQuery.sap.require("sas.ltjs.commons.rules.conditions.TableCondition");
jQuery.sap.require("sas.ltjs.commons.rules.interfaces.Rule");
jQuery.sap.require("sas.ltjs.commons.rules.ConditionalRule");
jQuery.sap.require("sas.ltjs.commons.rules.interfaces.RuleContext");
jQuery.sap.require("sas.ltjs.commons.rules.contexts.RelationalDataRuleContext");
jQuery.sap.require("sas.ltjs.commons.rules.data.ColumnDataItem");
jQuery.sap.require("sas.ltjs.commons.styles.TextDecoration");
jQuery.sap.require("sas.ltjs.commons.rules.expressions.DataItemExpression");
jQuery.sap.require("sas.ltjs.commons.models.fonts.FontManifest");
jQuery.sap.require("sas.ltjs.commons.models.fonts.FontFile");
jQuery.sap.require("sas.ltjs.commons.rules.expressions.IsMissingExpression");
jQuery.sap.require("sas.ltjs.commons.rules.conditions.BooleanExpressionDataCondition");
jQuery.sap.require("sas.ltjs.commons.rules.expressions.NegatedExpression");
jQuery.sap.require("sas.ltjs.commons.rules.expressions.BetweenExpression");
jQuery.sap.require("sas.ltjs.commons.rules.CompoundRule");
jQuery.sap.require("sas.ltjs.commons.models.datagrid.DataGridColumnInfo");
jQuery.sap.require("sas.ltjs.commons.util.Dimension");
jQuery.sap.require("sas.ltjs.commons.util.DimensionUnit");

jQuery.sap.require("sas.analytics.riskhp.models.DataSourceCacheManager");
jQuery.sap.require("sas.analytics.riskhp.utils.HprUtil");
jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
jQuery.sap.require("sas.analytics.riskhp.utils.HprAPI");
jQuery.sap.require("sas.analytics.riskhp.utils.ViewManager");

jQuery.sap.require("sas.analytics.riskhp.models.DisplayRulesManager");

/**
 * @name sas.analytics.riskhp.views.displayrules.DisplayRulesUtil
 */
sas.analytics.riskhp.views.displayrules.DisplayRulesUtil = {
    HprUtil: sas.analytics.riskhp.utils.HprUtil,
    DisplayRulesUtil: sas.analytics.riskhp.views.displayrules.DisplayRulesUtil,

    createConditionComparison: function(crosstab, displayRule) {
        var self = this;
        var op = displayRule.operator;
        var dataSet = crosstab.crosstab.getDataSet();

        var parms = new sas.ltjs.commons.rules.data.RuleDataItemMap();
        var ruleExpression = new sas.ltjs.commons.rules.expressions.CompoundBooleanExpression().retain();
        ruleExpression.setOperator( sas.ltjs.commons.rules.expressions.CompoundBooleanOperator.AND );

        var binExp = new sas.ltjs.commons.rules.expressions.BinaryRelationalExpression();
        binExp.setOperator(op);

        // reference column
        var refColIndex = crosstab.context.getColumnIndexFromTuple(displayRule.referenceColumn, true);
        var refColVar = new sas.ltjs.commons.rules.expressions.ExpressionConditionVariable();
        var refColName = dataSet.getColumnAtIndex(refColIndex).getName();
        refColVar.setName(refColName);
        var ltjsRefCol = dataSet.getColumnAtIndex(refColIndex);
        refColVar.setDataType(ltjsRefCol.getDataType() === sas.ltjs.commons.models.data.dataset.DataType.NUMBER ? sas.ltjs.commons.rules.expressions.DataTypeExt.NUMERIC : sas.ltjs.commons.rules.expressions.DataTypeExt.CHARACTER);
        var refCol = new sas.ltjs.commons.rules.data.ColumnDataItem(ltjsRefCol);
        parms.setDataItemForName(refColName, refCol);

        // dataItem1
        if (displayRule.dataItem1.type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE) {
            var di1Var = ltjsRefCol.getDataType() === sas.ltjs.commons.models.data.dataset.DataType.NUMBER ? new sas.ltjs.commons.rules.expressions.ConstantNumberExpression(Number(displayRule.dataItem1.value)) : new sas.ltjs.commons.rules.expressions.ConstantTextExpression(displayRule.dataItem1.value);
        } else {
            var dataItem1Index = crosstab.context.getColumnIndexFromTuple(displayRule.dataItem1.column, true);
            var di1Var = new sas.ltjs.commons.rules.expressions.ExpressionConditionVariable();
            var di1Name = dataSet.getColumnAtIndex(dataItem1Index).getName();
            di1Var.setName(di1Name);
            var ltjsDi1Col = dataSet.getColumnAtIndex(dataItem1Index);
            di1Var.setDataType(ltjsDi1Col.getDataType() === sas.ltjs.commons.models.data.dataset.DataType.NUMBER ? sas.ltjs.commons.rules.expressions.DataTypeExt.NUMERIC : sas.ltjs.commons.rules.expressions.DataTypeExt.CHARACTER);
            var di1 = new sas.ltjs.commons.rules.data.ColumnDataItem(ltjsDi1Col);
            parms.setDataItemForName(di1Name, di1);
        }

        binExp.setLeftOperand(refColVar);
        binExp.setRightOperand(di1Var);
        var operands = [binExp.retain()];
        ruleExpression.setOperands(operands);
        ruleExpression.resolveParameters(parms);

        var condition = new sas.ltjs.commons.rules.conditions.TableCondition(ruleExpression);
        // condition.setEvaluatesNumeric(true);
        // condition.setHasWildcard(true);

        return condition;
    },


    createConditionBetween: function(crosstab, displayRule) {
        var self = this;
        var op = displayRule.operator;
        var dataSet = crosstab.crosstab.getDataSet();

        var betweenExpression = new sas.ltjs.commons.rules.expressions.BetweenExpression();

        // reference column
        var refColIndex = crosstab.context.getColumnIndexFromTuple(displayRule.referenceColumn, true);
        var refCol = dataSet.getColumnAtIndex(refColIndex);
        var refColDataItem = new sas.ltjs.commons.rules.data.ColumnDataItem(refCol);
        var refColExpr = new sas.ltjs.commons.rules.expressions.DataItemExpression(refColDataItem);
        betweenExpression.setItemToCompare(refColExpr);

        // dataitem 1
        var di1Expr = null;
        if (displayRule.dataItem1.type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE) {
            di1Expr = new sas.ltjs.commons.rules.expressions.ConstantNumberExpression(Number(displayRule.dataItem1.value));
        } else {
            var di1ColIndex = crosstab.context.getColumnIndexFromTuple(displayRule.dataItem1.column, true);
            var di1Col = dataSet.getColumnAtIndex(di1ColIndex);
            var di1DataItem = new sas.ltjs.commons.rules.data.ColumnDataItem(di1Col);
            di1Expr = new sas.ltjs.commons.rules.expressions.DataItemExpression(di1DataItem);
        }
        betweenExpression.setLeftBoundary(di1Expr);

        // dataitem 2
        var di2Expr = null;
        if (displayRule.dataItem2.type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE) {
            di2Expr = new sas.ltjs.commons.rules.expressions.ConstantNumberExpression(Number(displayRule.dataItem2.value));
        } else {
            var di2ColIndex = crosstab.context.getColumnIndexFromTuple(displayRule.dataItem2.column, true);
            var di2Col = dataSet.getColumnAtIndex(di2ColIndex);
            var di2DataItem = new sas.ltjs.commons.rules.data.ColumnDataItem(di2Col);
            di2Expr = new sas.ltjs.commons.rules.expressions.DataItemExpression(di2DataItem);
        }
        betweenExpression.setRightBoundary(di2Expr);

        betweenExpression.setIsInvertedToMatchOutside(op === sas.analytics.riskhp.models.DisplayRule.OPERATOR_NOT_BETWEEN);
        // return new sas.ltjs.commons.rules.conditions.BooleanExpressionDataCondition(betweenExpression);

        //  The between expression is false if the value of the item to compare is between the values of the left and
        // right boundaries and the value of the left boundary is larger than the value of the right boundary.  So we
        // have to check with the boundaries reversed as well.
        var reverseBetweenExpression = new sas.ltjs.commons.rules.expressions.BetweenExpression();
        reverseBetweenExpression.setItemToCompare(refColExpr);
        reverseBetweenExpression.setLeftBoundary(di2Expr);
        reverseBetweenExpression.setRightBoundary(di1Expr);
        reverseBetweenExpression.setIsInvertedToMatchOutside(op === sas.analytics.riskhp.models.DisplayRule.OPERATOR_NOT_BETWEEN);

        var ruleExpression = new sas.ltjs.commons.rules.expressions.CompoundBooleanExpression().retain();
        ruleExpression.setOperator( op === sas.analytics.riskhp.models.DisplayRule.OPERATOR_NOT_BETWEEN ?
            sas.ltjs.commons.rules.expressions.CompoundBooleanOperator.AND :
            sas.ltjs.commons.rules.expressions.CompoundBooleanOperator.OR
        );

        var operands = [betweenExpression.retain(), reverseBetweenExpression.retain()];
        ruleExpression.setOperands(operands);
        // ruleExpression.resolveParameters(parms);

        return new sas.ltjs.commons.rules.conditions.BooleanExpressionDataCondition(ruleExpression);
    },

    createConditionMissing: function(crosstab, displayRule) {
        var self = this;
        var op = displayRule.operator;
        var dataSet = crosstab.crosstab.getDataSet();

        var missingExpression = new sas.ltjs.commons.rules.expressions.IsMissingExpression();

        // reference column
        var refColIndex = crosstab.context.getColumnIndexFromTuple(displayRule.referenceColumn, true);
        var valCol = dataSet.getColumnAtIndex(refColIndex);
        var valColDataItem = new sas.ltjs.commons.rules.data.ColumnDataItem(valCol);
        var valueExpr = new sas.ltjs.commons.rules.expressions.DataItemExpression(valColDataItem);

        missingExpression.setSubExpression(valueExpr);

        var ruleExpression = null;
        if (op === sas.analytics.riskhp.models.DisplayRule.OPERATOR_NOT_MISSING) {
            ruleExpression = new sas.ltjs.commons.rules.expressions.NegatedExpression();
            ruleExpression.setSubExpression(missingExpression);
        }  else {
            ruleExpression = missingExpression;
        }
        return new sas.ltjs.commons.rules.conditions.BooleanExpressionDataCondition(ruleExpression);
    },

    createDisplayRuleCondition: function(crosstab, displayRule) {
        var self = this;
        var op = displayRule.operator;

        if (op === sas.analytics.riskhp.models.DisplayRule.OPERATOR_MISSING ||
            op === sas.analytics.riskhp.models.DisplayRule.OPERATOR_NOT_MISSING) {
            return self.createConditionMissing(crosstab, displayRule);
        } else if (op === sas.analytics.riskhp.models.DisplayRule.OPERATOR_BETWEEN ||
            op === sas.analytics.riskhp.models.DisplayRule.OPERATOR_NOT_BETWEEN) {
            return self.createConditionBetween(crosstab, displayRule);
        } else {
            return self.createConditionComparison(crosstab, displayRule);
        }
    },


    createStyleAction: function(displayRule) {
        var style = displayRule.style;

        var ruleAttributeBundle = new sas.ltjs.commons.rules.actions.AttributeBundleTarget( new sas.ltjs.commons.styles.AttributeBundle() );
        var attrBundle =  ruleAttributeBundle.getAttributeBundle();

        if (style.backgroundColor != sas.hc.m.colorpicker.ColorPicker.NO_COLOR) {
            attrBundle.setUnsignedAttribute(sas.ltjs.commons.styles.AttributeNames.BACKGROUND_COLOR, style.backgroundColor.replace("#", "0x"));// 0xffff00);
        }
        if (style.color != sas.hc.m.colorpicker.ColorPicker.NO_COLOR) {
            attrBundle.setUnsignedAttribute(sas.ltjs.commons.styles.AttributeNames.COLOR, style.color.replace("#", "0x"));// 0xff0000);
        }

        // attrBundle.setDimensionAttribute(sas.ltjs.commons.styles.AttributeNames.FONT_SIZE, new sas.ltjs.commons.util.Dimension(style.fontSize, sas.ltjs.commons.util.DimensionUnit.POINTS));
        attrBundle.setStringAttribute(sas.ltjs.commons.styles.AttributeNames.FONT_FAMILY, style.fontName);

        if (style.italic) {
            attrBundle.setUnsignedAttribute(sas.ltjs.commons.styles.AttributeNames.FONT_STYLE, sas.ltjs.commons.styles.FontStyle.ITALIC);
        }
        if (style.bold) {
            attrBundle.setUnsignedAttribute(sas.ltjs.commons.styles.AttributeNames.FONT_WEIGHT, sas.ltjs.commons.styles.FontWeight.BOLD);
        }
        if (style.underline) {
            attrBundle.setUnsignedAttribute(sas.ltjs.commons.styles.AttributeNames.TEXT_DECORATION, sas.ltjs.commons.styles.TextDecoration.UNDERLINE);
        }

        return new sas.ltjs.commons.rules.actions.AttributeBundleAction(attrBundle);
    },


    generateLTJSDisplayRule: function(crosstab, displayRule) {
        var self = this;
        var dataSet = crosstab.crosstab.getDataSet();

        var condition = self.createDisplayRuleCondition(crosstab, displayRule);
        var action = self.createStyleAction(displayRule);

        var rule = new sas.ltjs.commons.rules.ConditionalRule(condition, action);
        var context = new sas.ltjs.commons.rules.contexts.RelationalDataRuleContext(dataSet);
        context.setIsDebugLoggingEnabled(true);
        rule.setBoundContext(context);
        return rule;
    },

    _addRuleToColIndex: function (crosstab, colIndex, rule) {
        var dataSet = crosstab.crosstab.getDataSet();

        var existingDisplayRule = crosstab.crosstab.getDisplayRule(colIndex);
        if (existingDisplayRule) {
            existingDisplayRule.addRule(rule);
        } else {
            // add rule to new compound rule
            existingDisplayRule = new sas.ltjs.commons.rules.CompoundRule();

            // var dataSet = crosstab._createDataSetFromResults().dataSet;
            var context = new sas.ltjs.commons.rules.contexts.RelationalDataRuleContext(dataSet);
            existingDisplayRule.setBoundContext(context);

            existingDisplayRule.addRule(rule);
            crosstab.crosstab.setDisplayRule(colIndex, existingDisplayRule);
        }
    },

    applyLTJSDisplayRule: function (crosstab, displayRule) {
        var self = this;

        var rule = self.generateLTJSDisplayRule(crosstab, displayRule);

        if (displayRule.appliesTo.type === sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_ROW) {
            var numCols = crosstab.context.getNumberOfColumns();
            for (var i = 0; i < numCols; i++) {
                self._addRuleToColIndex(crosstab, i,rule);
            }
        } else {
            var appliesToIndex = crosstab.context.getColumnIndexFromTuple(displayRule.referenceColumn);
            if (displayRule.appliesTo.type === sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_COL) {
                appliesToIndex = crosstab.context.getColumnIndexFromTuple(displayRule.appliesTo.column);
            }
            self._addRuleToColIndex(crosstab, appliesToIndex, rule);
        }
    },


    clearDisplayRuleFromCrosstabIndex: function(crosstab, index) {
        var colInfo = crosstab.crosstab.getColumnInfo(index);
        if (colInfo) {
            colInfo.setDisplayRule(null);
            crosstab.crosstab.setColumnInfo(index, colInfo);
        }
    },

    clearDisplayRulesFromCrosstabTuple: function(crosstab, tuple) {
        var self = this;
        var index = crosstab.context.getColumnIndexFromTuple(tuple);
        if (index != -1) {
            self.clearDisplayRuleFromCrosstabIndex(crosstab, index);
        }
    },

    // need to clear rules from all columns as rules might have been deleted or made inactive
    clearDisplayRulesFromCrosstab: function(crosstab) {
        var self = this;
        var numCols = crosstab.context.getNumberOfColumns();
        for (var i = 0; i < numCols; i++) {
            self.clearDisplayRuleFromCrosstabIndex(crosstab, index);
        }
    },


    applyLTJSDisplayRulesForCrosstabTuple: function(crosstab, tuple) {
        var self = this;

        // self.clearDisplayRulesFromCrosstab(crosstab);

        var crosstabRules = DisplayRulesManager.getActiveDisplayRulesForCrosstabTuple(crosstab.sId, tuple);
        if (crosstabRules) {
            // apply rules in reverse order since the top has highest priority
            var reverseRules = self.HprUtil.hprClone(crosstabRules).reverse();
            _.each(reverseRules, function (crosstabRule) {
                if (crosstabRule.active ) {
                    var rule = DisplayRulesManager.getDisplayRuleById(crosstabRule.ruleId);
                    var isValid = DisplayRulesManager.isDisplayRuleValidForDataSource(crosstab.context.dataSourceId, rule.id);
                    if (isValid &&  // valid and the applies to column is in crosstab
                        !(rule.appliesTo.type === sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_COL && crosstab.context.getColumnIndexFromTuple(rule.appliesTo.column) === -1)) {
                        self.applyLTJSDisplayRule(crosstab, rule);
                    }
                }
            });
        }
    },

    applyLTJSDisplayRulesForCrosstab: function(crosstab) {
        var self = this;

        // self.clearDisplayRulesFromCrosstab(crosstab);

        var crosstabRules = DisplayRulesManager.getDisplayRulesForCrosstab(crosstab.sId);
        if (crosstabRules) {
            // apply rules in reverse order since the top has highest priority
            var reverseRules = self.HprUtil.hprClone(crosstabRules).reverse();
            _.each(reverseRules, function (crosstabRule) {
                if (crosstabRule.active ) {
                    var rule = DisplayRulesManager.getDisplayRuleById(crosstabRule.ruleId);
                    var isValid = DisplayRulesManager.isDisplayRuleValidForDataSource(crosstab.context.dataSourceId, rule.id);
                    if (isValid &&  // valid and the applies to column is in crosstab
                        !(rule.appliesTo.type === sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_COL && crosstab.context.getColumnIndexFromTuple(rule.appliesTo.column) === -1)) {
                        self.applyLTJSDisplayRule(crosstab, rule);
                    }
                }
            });
        }
    },

    getOperatorLabel: function(op) {
        var rb = sas.analytics.riskhp.utils.HprUtil.getBundle();
        switch (op) {
            case sas.analytics.riskhp.models.DisplayRule.OPERATOR_MISSING:
                return rb.getText("HPRisk.DisplayRules.Operator.Is.Missing.label");
            case sas.analytics.riskhp.models.DisplayRule.OPERATOR_NOT_MISSING:
                return rb.getText("HPRisk.DisplayRules.Operator.Is.Not.Missing.label");
            case sas.analytics.riskhp.models.DisplayRule.OPERATOR_EQ:
                return "=";
            case sas.analytics.riskhp.models.DisplayRule.OPERATOR_NE:
                return "^=";
            case sas.analytics.riskhp.models.DisplayRule.OPERATOR_LT:
                return "<";
            case sas.analytics.riskhp.models.DisplayRule.OPERATOR_LE:
                return "<=";
            case sas.analytics.riskhp.models.DisplayRule.OPERATOR_GT:
                return ">";
            case sas.analytics.riskhp.models.DisplayRule.OPERATOR_GE:
                return ">=";
            case sas.analytics.riskhp.models.DisplayRule.OPERATOR_BETWEEN:
                return rb.getText("HPRisk.DisplayRules.Operator.Is.Between.label");
            case sas.analytics.riskhp.models.DisplayRule.OPERATOR_NOT_BETWEEN:
                return rb.getText("HPRisk.DisplayRules.Operator.Is.Not.Between.label");
            // default:
            //     return "MISSING OPERATOR";
        }
    },

    generateWhenLabel: function(displayRule) {
        var rb = sas.analytics.riskhp.utils.HprUtil.getBundle();
        var label = displayRule.referenceColumn.label + " " + this.getOperatorLabel(displayRule.operator);

        if (this.requiresDataItem1(displayRule.operator)) {
            label += " " + (displayRule.dataItem1.type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE ? displayRule.dataItem1.value :
                displayRule.dataItem1.column.label);
        }
        if (this.requiresDataItem2(displayRule.operator)) {
            label += " " + rb.getText("HPRisk.DisplayRules.And.txt") + " " + (displayRule.dataItem2.type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_VALUE ? displayRule.dataItem2.value :
                displayRule.dataItem2.column.label);
        }
        return label;
    },

    requiresDataItem1: function(operator) {
        return operator && !(operator == sas.analytics.riskhp.models.DisplayRule.OPERATOR_MISSING || operator == sas.analytics.riskhp.models.DisplayRule.OPERATOR_NOT_MISSING);
    },

    requiresDataItem2: function(operator) {
        return operator && (operator == sas.analytics.riskhp.models.DisplayRule.OPERATOR_BETWEEN || operator == sas.analytics.riskhp.models.DisplayRule.OPERATOR_NOT_BETWEEN);
    },

    /**
     * returns the minimum representation of the tuples for the crosstab column
     * @param info - crosstab pickinfo
     * @returns {Array}
     */
    getTuplesFromPickInfo: function(info) {
        var dataItems = [];
        if (info) {
            // info.tuple contains all the corrected information for communicating with the mid-tier. Corrected meaning
            // benchmark/compute variable headers removed, absolut/relative horizons replaces with base horizon, etc.
            _.each(info.tuple, function(tuple){
                dataItems.push({
                    id: tuple.id,
                    type: tuple.type, // don't need
                    name: tuple.name // don't need
                });
            });
        }
        return dataItems;
    },

    isNumericColumn: function(crosstab, tuple) {
        var isNumeric = true;
        if (tuple && tuple.length > 0) {
            var dataSet = crosstab.crosstab.getDataSet();
            var refColIndex = crosstab.context.getColumnIndexFromTuple(tuple, true);
            var ltjsRefCol = dataSet.getColumnAtIndex(refColIndex);
            if (ltjsRefCol) {
                isNumeric = ltjsRefCol.getDataType() === sas.ltjs.commons.models.data.dataset.DataType.NUMBER;
            }
        }
        return isNumeric;
    }
};

DisplayRulesUtil = sas.analytics.riskhp.views.displayrules.DisplayRulesUtil;