// (c) 2017, SAS Institute Inc.

sap.ui.jsview('sas.analytics.riskhp.views.hierarchies.HierarchiesPane', (function() {

    return {

        ROW : "row",
        COLUMN : "column",
        MOVEROWTOTOP : "moveRowToTop",
        MOVEROWTOBOTTOM : "moveRowToBottom",
        MOVEROWUP : "moveRowUp",
        MOVEROWDOWN : "moveRowDown",
        REMOVE : "remove",
        MOVECOLUMNTOTOP : "moveColumnToTop",
        MOVECOLUMNTOBOTTOM : "moveColumnToBottom",
        MOVECOLUMNUP : "moveColumnUp",
        MOVECOLUMNDOWN : "moveColumnDown",

        getControllerName: function() {
            return 'sas.analytics.riskhp.views.hierarchies.HierarchiesPane';
        },

        createContent: function(controller) {
            this._createRowHierarchy(controller);
            this._createColumnHierarchy(controller);
            this._createRowContextMenu(controller);
            this._createColumnContextMenu(controller);

            this.zeroStatePage = new sas.hc.m.MessagePage({
                id: this.createId('_hierarchiesPane_zeroStatePage'),
                text: "", // Must leave empty, causes big font text
                description: this._getResourceBundle().getText("HierarchiesPane.select.crosstab.txt"),
                icon: sas.icons.HC.ZEROSTATEALERT,
                showHeader: false
            });

            this.defaultPage = new sas.hc.m.Page({
                id: this.createId('_hierarchiesPane_defaultPage'),
                showHeader: false,
                enableScrolling: false,
                content: [this._createDefaultPageBorderLayout(controller)]
            });

            this.navContainer = new sas.hc.m.NavContainer({
                height: "100%",
                width: "100%",
                pages: [this.zeroStatePage, this.defaultPage]
            });

            return this.navContainer;
        },

        _getResourceBundle: function () {
            return sas.analytics.riskhp.utils.HprUtil.getBundle();
        },

        _createRowContextMenu: function (controller) {
            var view = this;

            var moveToTopRowMenuItem = new sas.hc.ui.unified.MenuItem({
                id: view.MOVEROWTOTOP,
                text: view._getResourceBundle().getText("HierarchiesPane.ContextMenu.MoveToTop.txt")
            });
            var moveToBottomRowMenuItem = new sas.hc.ui.unified.MenuItem({
                id: view.MOVEROWTOBOTTOM,
                text: view._getResourceBundle().getText("HierarchiesPane.ContextMenu.MoveToBottom.txt")
            });
            var moveUpRowMenuItem = new sas.hc.ui.unified.MenuItem({
                id: view.MOVEROWUP,
                text: view._getResourceBundle().getText("HierarchiesPane.ContextMenu.MoveUp.txt")
            });
            var moveDownRowMenuItem = new sas.hc.ui.unified.MenuItem({
                id: view.MOVEROWDOWN,
                text: view._getResourceBundle().getText("HierarchiesPane.ContextMenu.MoveDown.txt")
            });
            var removeRowMenuItem = new sas.hc.ui.unified.MenuItem({
                id: view.REMOVE,
                text: view._getResourceBundle().getText("HierarchiesPane.ContextMenu.Remove.txt"),
                startsSection: true
            });

            var fnCustomizeRowContextMenu = function (oMenu, oEvent) {
                var oControl = $(oEvent.target).control()[0];

                // make decisions on what menu item to show based on the selections
                if (oControl && oControl instanceof sas.hc.m.StandardListItem) {
                    // Don't show the menu if there is only 1 item; 'this' is the "rowHierarchyList"
                    if (this.getAggregation("items").length === 1) {
                        return;
                    }

                    oMenu.baseItemTitle = oControl.getTitle();
                    oMenu.baseItemId = oControl.getId();
                    var idx = oControl.getId().lastIndexOf("-");
                    oMenu.baseItemIndex = oControl.getId().slice(idx + 1);
                }
                else {
                    oMenu.baseItemTitle = undefined;
                    oMenu.baseItemId = undefined;
                    oMenu.baseItemIndex = undefined;
                }
                return oMenu;
            };

            this.rowContextMenu = new sas.hc.ui.unified.Menu({
                items: [
                    moveToTopRowMenuItem,
                    moveUpRowMenuItem,
                    moveDownRowMenuItem,
                    moveToBottomRowMenuItem,
                    removeRowMenuItem
                ],
                visible : false,
                itemSelect: $.proxy(controller.onRowItemSelectFunction, controller)
            });

            this.rowHierarchyList.setAggregation("contextMenu", this.rowContextMenu);
            this.rowHierarchyList.customizeContextMenu = fnCustomizeRowContextMenu;
        },

        _createColumnContextMenu: function (controller) {
            var view = this;

            var moveToTopColumnMenuItem = new sas.hc.ui.unified.MenuItem({
                id: view.MOVECOLUMNTOTOP,
                text: view._getResourceBundle().getText("HierarchiesPane.ContextMenu.MoveToTop.txt")
            });
            var moveToBottomColumnMenuItem = new sas.hc.ui.unified.MenuItem({
                id: view.MOVECOLUMNTOBOTTOM,
                text: view._getResourceBundle().getText("HierarchiesPane.ContextMenu.MoveToBottom.txt")
            });
            var moveUpColumnMenuItem = new sas.hc.ui.unified.MenuItem({
                id: view.MOVECOLUMNUP,
                text: view._getResourceBundle().getText("HierarchiesPane.ContextMenu.MoveUp.txt")
            });
            var moveDownColumnMenuItem = new sas.hc.ui.unified.MenuItem({
                id: view.MOVECOLUMNDOWN,
                text: view._getResourceBundle().getText("HierarchiesPane.ContextMenu.MoveDown.txt")
            });

            var fnCustomizeColumnContextMenu = function (oMenu, oEvent) {
                var oControl = $(oEvent.target).control()[0];

                // make decisions on what menu item to show based on the selections
                if (oControl && oControl instanceof sas.hc.m.StandardListItem) {
                    //oMenu.setVisible(true);
                    oMenu.baseItemTitle = oControl.getTitle();
                    oMenu.baseItemId = oControl.getId();
                    var idx = oControl.getId().lastIndexOf("-");
                    oMenu.baseItemIndex = oControl.getId().slice(idx + 1);
                }
                else {
                    oMenu.baseItemTitle = undefined;
                    oMenu.baseItemId = undefined;
                    oMenu.baseItemIndex = undefined;
                }
                return oMenu;
            };

            this.columnContextMenu = new sas.hc.ui.unified.Menu({
                items: [
                    moveToTopColumnMenuItem,
                    moveUpColumnMenuItem,
                    moveDownColumnMenuItem,
                    moveToBottomColumnMenuItem
                ],
                visible : false,
                itemSelect: $.proxy(controller.onColumnItemSelectFunction, controller)
            });

            this.columnHierarchyList.setAggregation("contextMenu", this.columnContextMenu);
            this.columnHierarchyList.customizeContextMenu = fnCustomizeColumnContextMenu;
        },

        _createRowHierarchyHeader: function(controller) {
            this.rowHierarchyUpButton = new sas.hc.m.Button({
                id: 'rowHierarchy-list-up-Button',
                icon: sas.icons.HC.ARROWSMALLUP,
                enabled: true,
                press: $.proxy(controller.onRowHierarchyUpButtonPressed, controller)
            });

            this.rowHierarchyDownButton = new sas.hc.m.Button({
                id: 'rowHierarchy-list-down-Button',
                icon: sas.icons.HC.ARROWSMALLDOWN,
                enabled: true,
                press: $.proxy(controller.onRowHierarchyDownButtonPressed, controller)
            });

            this.warnIcon = sas.icons.pool.createControlByURI(sas.icons.HC.WARNINGSTATUS);
            this.warnIcon.setTooltip(this._getResourceBundle().getText("HierarchiesPane.row.warning.tooltip.txt"));
            this.warnIcon.setVisible(false);

            var rowHeaderText = this._getResourceBundle().getText("HierarchiesPane.row.txt");
            this.rowHierarchyListHeaderText = new sas.hc.m.Label({
                text: this.rowHeaderText
            });
            this.rowHierarchyListHeader  = new sas.hc.ui.commons.layout.BorderLayout({
                begin: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    size: "65%",
                    contentAlign: "left",
                    visible: true,
                    content: [ this.rowHierarchyListHeaderText ]
                }),
                end: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    size: "35%",
                    contentAlign: "right",
                    visible: true,
                    content: [ this.warnIcon, this.rowHierarchyUpButton, this.rowHierarchyDownButton ]
                })
            });
        },

        _createRowHierarchyList: function(controller) {
            var self = this;

            this.rowHierarchyList = new sas.hc.m.List({
                id: 'rowHierarchy-list',
                mode: sap.m.ListMode.SingleSelectMaster,
                //mode: sap.m.ListMode.Delete,
                //"delete": $.proxy(controller.onRowHierarchyDelete, controller),
                selectionChange: $.proxy(controller.onRowHierarchySelectionChange, controller)
            });

            // this.rowHierarchyList.addEventDelegate({
            //     onBeforeRendering: function(oEvent) {
            //         self.rowHierarchyList.$().find(".sapMLIBSelectD").hide();
            //     },
            //     onAfterRendering: function(oEvent) {
            //         self.rowHierarchyList.$().find(".sapMLIBSelectD").hide();
            //     },
            //     ontap: function(oEvent) {
            //         self.rowHierarchyList.$().find(".sapMLIBSelectD").hide();
            //         oEvent.srcControl.$().find(".sapMLIBSelectD").show();
            //     }
            //     // other events beside tap?
            // });


            // var showLabels = false;
            // var crosstabContext = controller.getCrosstabContext();
            // if (crosstabContext !== undefined) {
            //     showLabels = crosstabContext.getOptionShowLabels().value;
            // }

            var oItemTemplate = new sas.hc.m.StandardListItem({
                title: {parts: ["name", "label"], formatter: function(name, label) {
                    var showLabels = false;
                    var crosstabContext = controller.getCrosstabContext();
                    if (crosstabContext !== undefined) {
                        showLabels = crosstabContext.getOptionShowLabels().value;
                    }
                    return showLabels ? label : name
                }},
                type: sap.m.ListType.Active
            });
            this.rowHierarchyList.bindAggregation("items", {path: "/crosstab/row", template: oItemTemplate} );
        },

        _createRowHierarchyLayout: function(controller) {
            this.rowHierarchyLayout = new sas.hc.ui.commons.layout.BorderLayout({
                width: "100%",
                height: "100%",
                top: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    size: "2em",
                    overflowX: "visible",
                    content: [this.rowHierarchyListHeader]
                }),
                center: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    overflowX: "hidden",
                    content: [this.rowHierarchyList]
                }),
            });
        },

        _createRowHierarchy: function(controller) {
            this._createRowHierarchyList(controller);
            this._createRowHierarchyHeader(controller);
            this._createRowHierarchyLayout(controller);
        },

        _createColumnHierarchyHeader: function(controller) {
            this.columnHierarchyUpButton = new sas.hc.m.Button({
                id: 'columnHierarchy-list-up-Button',
                icon: sas.icons.HC.ARROWSMALLUP,
                enabled: true,
                press: $.proxy(controller.onColumnHierarchyUpButtonPressed, controller)
            });

            this.columnHierarchyDownButton = new sas.hc.m.Button({
                id: 'columnHierarchy-list-down-Button',
                icon: sas.icons.HC.ARROWSMALLDOWN,
                enabled: true,
                press: $.proxy(controller.onColumnHierarchyDownButtonPressed, controller)
            });

            var columnHeaderText = this._getResourceBundle().getText("HierarchiesPane.column.txt");
            this.columnHierarchyListHeaderText = new sas.hc.m.Label({
                text: this.columnHeaderText
            });
            this.columnHierarchyListHeader  = new sas.hc.ui.commons.layout.BorderLayout({
                begin: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    size: "70%",
                    contentAlign: "left",
                    visible: true,
                    content: [
                        this.columnHierarchyListHeaderText
                    ]
                }),
                end: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    size: "30%",
                    contentAlign: "right",
                    visible: true,
                    content: [ this.columnHierarchyUpButton, this.columnHierarchyDownButton]
                })
            });
        },

        _createColumnHierarchyList: function(controller) {
            this.columnHierarchyList = new sas.hc.m.List({
                //headerText: this._getResourceBundle().getText("HierarchiesPane.column.txt"),
                id: 'columnHierarchy-list',
                mode: sap.m.ListMode.SingleSelectMaster,
                selectionChange: $.proxy(controller.onColumnHierarchySelectionChange, controller)
            });

            var oItemTemplate = new sas.hc.m.StandardListItem({
                title: "{label}",
                type: sap.m.ListType.Active,
                visible: "{visible}"
            });
            this.columnHierarchyList.bindAggregation("items", {path: "/crosstab/column", template: oItemTemplate} );
        },

        _createColumnHierarchyLayout: function(controller) {
            this.columnHierarchyLayout = new sas.hc.ui.commons.layout.BorderLayout({
                width: "100%",
                height: "100%",
                top: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    size: "2em",
                    overflowX: "visible",
                    content: [this.columnHierarchyListHeader]
                }),
                center: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    overflowX: "hidden",
                    content: [this.columnHierarchyList]
                }),
            });
        },

        _createColumnHierarchy: function(controller) {
            this._createColumnHierarchyList(controller);
            this._createColumnHierarchyHeader(controller);
            this._createColumnHierarchyLayout(controller);
        },

        _createDefaultPageBorderLayout: function(controller) {
            // Create a BorderLayout to help with scrolling
            var borderLayout = new sas.hc.ui.commons.layout.BorderLayout({
                width: "100%",
                height: "100%",
                center: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    overflowX: "hidden",
                    overflowY: "hidden",
                    content: [this._buildCenter(controller)]
                }).addStyleClass("sasHierarchyPaneLayout_paneArea_centerLayoutArea"),
                bottom: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    overflowX: "hidden",
                    overflowY: "hidden",
                    size: "5.0rem",
                    content: [this._buildFooter(controller)]
                })
            });

            return borderLayout;
        },

        _buildCenter: function(controller) {

            this.parentPanelTitleText = new sas.hc.m.Text({
                wrapping: false,
                text: ''
            });

            var panelContentLayout = new sas.hc.ui.commons.layout.BorderLayout({
                width: "100%",
                height: "100%",
                top: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    size: "50%",
                    overflowX: "hidden",
                    overflowY: "hidden",
                    content: [this.rowHierarchyLayout]
                }),
                center: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    size: "5%",
                    overflowX: "hidden",
                    overflowY: "hidden",
                    content: [new sas.hc.m.Text({
                        text: ''
                    })]
                }),
                bottom: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    size: "45%",
                    overflowX: "hidden",
                    overflowY: "hidden",
                    content: [this.columnHierarchyLayout]
                })
            });

            var centerLayout = new sas.hc.ui.commons.layout.BorderLayout({
                width: "100%",
                height: "100%",
                top: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    size: "2rem",
                    overflowX: "hidden",
                    overflowY: "hidden",
                    content: [this.parentPanelTitleText]
                }),
                center: new sas.hc.ui.commons.layout.BorderLayoutArea({
                    overflowX: "hidden",
                    overflowY: "hidden",
                    content: [panelContentLayout]
                })
            });

            return centerLayout;
        },

        _buildFooter: function(controller) {
            var self = this;

            self.applyButton = new sas.hc.m.Button({
                id: this.createId('_hierarchiesPane_applyButton'),
                text: this._getResourceBundle().getText("HierarchiesPane.applyButton.txt"),
                enabled: false,
                press: $.proxy(controller._onApplyButtonPressed, controller)
            }).addStyleClass("sasPrimaryButton");

            self.cancelButton = new sas.hc.m.Button({
                id: this.createId('_hierarchiesPane_cancelButton'),
                text: this._getResourceBundle().getText("HierarchiesPane.cancelButton.txt"),
                enabled: false, // Defect S1387426: Should behave like Apply Button state
                press: $.proxy(controller._onCancelButtonPressed, controller)
            });

            var tb = new sas.hc.m.ResponsiveToolbar({
                content: [new sas.hc.m.ToolbarSpacer(), self.applyButton, self.cancelButton]
            });
            tb.addStyleClass("noBackground");

            return tb;
        }
    };
}()));