// (c) 2017, SAS Institute Inc.

jQuery.sap.require("sas.risk.common.panelsview.PanelsViewUtil");


/**
 * @name sas.risk.common.panelsview.PanelsViewUtil
 */
sas.risk.common.panelsview.PanelsViewUtil = (function () {

    function _isDroppableContainer(control) {
        if (_.isUndefined(control) || control === null) {
            return false;
        }
        var metadata = control.getMetadata();
        return (metadata["_sClassName"].indexOf('DroppableContainer') !== -1);
    }

    /**
     * detect IE
     * https://codepen.io/gapcode/pen/vEJNZN
     * returns version of IE or false, if browser is not Internet Explorer
     */
    function detectIE() {
        var ua = window.navigator.userAgent;

        // Test values; Uncomment to check result …

        // IE 10
        // ua = 'Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0)';

        // IE 11
        // ua = 'Mozilla/5.0 (Windows NT 6.3; Trident/7.0; rv:11.0) like Gecko';

        // Edge 12 (Spartan)
        // ua = 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.71 Safari/537.36 Edge/12.0';

        // Edge 13
        // ua = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/46.0.2486.0 Safari/537.36 Edge/13.10586';

        var msie = ua.indexOf('MSIE ');
        if (msie > 0) {
            // IE 10 or older => return version number
            return parseInt(ua.substring(msie + 5, ua.indexOf('.', msie)), 10);
        }

        var trident = ua.indexOf('Trident/');
        if (trident > 0) {
            // IE 11 => return version number
            var rv = ua.indexOf('rv:');
            return parseInt(ua.substring(rv + 3, ua.indexOf('.', rv)), 10);
        }

        var edge = ua.indexOf('Edge/');
        if (edge > 0) {
            // Edge (IE 12+) => return version number
            return parseInt(ua.substring(edge + 5, ua.indexOf('.', edge)), 10);
        }

        // other browser
        return false;
    }

    return {
        /**
         * Iterates over the specified controls and calls the handleResize function on
         * each if defined.
         * @param controls
         */
        notifyControlsOfResize: function(controls) {
            if (detectIE()) {
                var riskPanelsView = jQuery('.riskPanelsView');
                if (!riskPanelsView ) {
                    return;
                }
                var panelsViewId = riskPanelsView.attr('id');

                var panelsView =  sap.ui.getCore().byId(panelsViewId);
                if (!panelsView) {
                    return;
                }
                setTimeout(function() {
                    panelsView.refreshAllPanels({});
                },0);
                return;
            }
            _.each(controls, function(control) {
                if (_isDroppableContainer(control)) {
                    if (control.getAggregation("_mainContentWrapper").getContent()) {
                        if (typeof control.getAggregation("_mainContentWrapper").getContent().handleResize === 'function') {
                            control.getAggregation("_mainContentWrapper").getContent().handleResize();
                        }
                    }
                }
            });
        },

        /**
         * Helper to determine if the passed in control is a DroppableContainer
         *  REVIEW: Is there a better class-like method for this?
         * @param control The control to check
         * @returns {boolean}
         * @private
         */
        isDroppableContainer: function (control) {
            return _isDroppableContainer(control);
        },

        /**
         * Common method to build a splitter control for use inside a DroppableContainer.  Note: There
         * might be a better way to encapsulate this method, but at the time the actual purpose of this util
         * class is still in flux.  If this ultimately is used in a global way then this method (and
         * probably others) should be "hidden" inside RiskPanelsView or DroppableContainer, but if this
         * util object only serves the needs of the DroppableContainer and RiskPanelsView then this method
         * could stay here
         * @param orientation The orientation of the splitter, expected to be one of the sap.ui.core.Orientation constants
         * @param content The content to be contained in the splitter, This must be an array of length 2 and both contents must have the SplitterLayoutData layoutData applied
         * @param debugMode Should the splitter be styled for debug mode
         * @param resizeHandler A callback function that gets called anytime the splitter's contents is resized.
         * @returns {sas.hc.ui.layout.Splitter}
         */
        buildSplitterForDC: function(orientation, content, debugMode, resizeHandler) {
            var splitterStyleClass = (debugMode) ? "droppablePanelHorizontalSplitterDebug" : "droppablePanelHorizontalSplitter";
            if (orientation === sap.ui.core.Orientation.Vertical) {
                splitterStyleClass = (debugMode) ? "droppablePanelVerticalSplitterDebug" : "droppablePanelVerticalSplitter";
            }

            var splitter = new sas.hc.ui.layout.Splitter({
                height: "100%",
                width: "100%",
                orientation: orientation,
                contentAreas: content,
                resize: resizeHandler
            });
            splitter.addStyleClass('defaultBackground');
            splitter.addStyleClass(splitterStyleClass);
            return splitter;

        }
    };
})();