jQuery.sap.declare('sas.risk.scenariomanager.ScenarioAPI');
jQuery.sap.require("sas.risk.scenariomanager.model.Scenario");
jQuery.sap.require("sas.risk.scenariomanager.utils.ScenarioEditorUtil");
jQuery.sap.require("sas.hc.m.MessageDialog");

sas.risk.scenariomanager.ScenarioAPI = {

    _scenarioLinks: [],

    getScenarios: function (workGroupName) {
        var api = this;
        var deferred = $.Deferred();
        var workGroupQuery = "";
        if (workGroupName) {
            workGroupQuery = "?workGroup=" + workGroupName;
        }

        var link = api.findRequiredLink(deferred, api.getRootLinks(), { rel: 'scenarios' });
        if (link) {
            $.getJSON(link.href + workGroupQuery, function (data) {
                api._scenarioLinks = data.links;
                // Transform undefined workGroup into the public work group
                _.each(data.items, function (scenario) {
                    // if (scenario.workGroup === undefined || scenario.workGroup === null || scenario.workGroup === "") {
                    //     var publicWorkGroup = api.getPublicWorkGroup();
                    //     if (publicWorkGroup) {
                    //         scenario.workGroup = publicWorkGroup.name;
                    //     }
                    // }
                    scenario.displayName = sas.risk.scenariomanager.utils.ScenarioEditorUtil.formatSourceName(scenario.name, scenario.label);
                    scenario.usingLabel = sas.risk.scenariomanager.settings.ApplicationSettings.getShowRiskFactorLabel();
                });
                deferred.resolve(data.items);
            }).fail(function (jqxhr) {
                jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }

        return deferred;
    },

    getScenariosForBaseline: function (baselineId) {
        var api = this;
        var deferred = $.Deferred();

        var link = api.findRequiredLink(deferred, api.getRootLinks(), { rel: 'scenarios' });
        if (link) {
            $.ajax(
                {
                    type: 'GET',
                    cache: false,
                    url: link.href + '?baselineId=' + baselineId,
                    dataType: 'json',
                    accepts: { json: link.type },
                    beforeSend: function (xhr) {
                        xhr.setRequestHeader('Accept-Item', link.itemType);
                    }
                }
            ).done(function (data) {
                // Transform undefined workGroup into the public work group
                _.each(data.items, function (scenario) {
                    // if (scenario.workGroup === undefined) {
                    //     var publicWorkGroup = api.getPublicWorkGroup();
                    //     if (publicWorkGroup) {
                    //         scenario.workGroup = publicWorkGroup.name;
                    //     }
                    // }
                });
                deferred.resolve(data.items);
            }).fail(function (jqxhr) {
                jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }

        return deferred;
    },

    getScenarioHorizons: function (workGroupName) {
        var api = this;
        var deferred = $.Deferred();
        var linkDesc = {
            rel: 'scenarios',
            type: 'application/vnd.sas.collection+json',
            itemType: 'application/vnd.sas.risk.scenario+json;charset=utf-8'
        };

        var workGroupQuery = "";
        if (workGroupName) {
            workGroupQuery = "&workGroup=" + workGroupName;
        }

        var link = api.findRequiredLink(deferred, api.getRootLinks(), linkDesc);
        if (link) {

            $.ajax(
                {
                    type: 'GET',
                    cache: false,
                    url: link.href + '?fields=periods' + workGroupQuery,
                    dataType: 'json',
                    accepts: { json: link.type },
                    beforeSend: function (xhr) {
                        xhr.setRequestHeader('Accept-Item', link.itemType);
                    }
                }
            ).done(function (data) {
                _.each(data.items, function (scenario) {
                    scenario.periods = sas.risk.scenariomanager.utils.ScenarioEditorUtil.mapPeriods(scenario.periods);
                    // Transform undefined work group into the public work group
                    // if(scenario.workGroup === undefined) {
                    //     var publicWorkGroup = api.getPublicWorkGroup();
                    //     if (publicWorkGroup) {
                    //         scenario.workGroup = publicWorkGroup.name;
                    //     }
                    // }
                });
                deferred.resolve(data.items);
            }).fail(function (jqxhr) {
                jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }

        return deferred;
    },

    // NOTE: this is called by ViewManager during routing, in response to edit on manager page.  Currently, the router
    // does not pass the selected scenario(s) through; the route matches arguments from the first scenario, and an args
    // object is built that is passed to the editor's exactRouteMatched method.  In that case, we're grabbing root
    // collection link & appending id to that

    // NOTE: also called when setting the baseline of a scenario to load the baseline and copy risk factors; in that
    // case, we have a scenario summary object with horizons but no stresses.
    // TODO: if we ever get a real scenario object, short-circuit the read if it's a full scenario, not a summary
    getScenarioData: function (scenario) {
        var api = this;
        var deferred = $.Deferred();
        // this one should be there if we got an actual scenario instead of the args object passed by the router
        var linkDesc = {
            rel: 'self',
            type: 'application/vnd.sas.risk.scenario+json;charset=utf-8'
        };
        var link = this.findOptionalLink(scenario.links, linkDesc);
        if (!link) {
            // ... didn't find the self link; assume we got a dummy object from ViewManager/router
            var rootLink = this.findRequiredLink(deferred, api.getRootLinks(), { rel: 'scenarios' });
            if (rootLink) {
                link = sas.risk.scenariomanager.utils.ScenarioEditorUtil.deepCopy(rootLink);
                link.href += '/' + scenario.id;
            }
        }

        if (link) {
            return this.getScenarioFromLink(link);
        }

        return deferred;
    },

    getScenarioFromLink: function (link) {
        var api = this;
        var deferred = $.Deferred();

        $.ajax(
            {
                type: 'GET',
                cache: false,
                url: link.href,
                dataType: 'json',
                accepts: { json: 'application/vnd.sas.risk.scenario+json;charset=utf-8' }
            }
        ).done(function (data) {
            // S1257470 work around.
            var hasNoHorizons = (!data.hasOwnProperty('periods') ||
                                 data.periods.length === 0);
            if (hasNoHorizons) {
                data.periods = [sas.risk.scenariomanager.utils.ScenarioEditorUtil.getNewPeriod()];
            } else {
                data.periods = sas.risk.scenariomanager.utils.ScenarioEditorUtil.mapPeriods(data.periods);
            }

            data.displayName = sas.risk.scenariomanager.utils.ScenarioEditorUtil.formatSourceName(data.name, data.label);

            _.each(data.stresses, function (stress) {
                stress.perturbations = sas.risk.scenariomanager.utils.ScenarioEditorUtil.mapPerturbations(stress.perturbations);
            });
            // if (data.workGroup === undefined) {
            //     var publicWorkGroup = api.getPublicWorkGroup();
            //     if (publicWorkGroup) {
            //         data.workGroup = publicWorkGroup.name;
            //     }
            // }
            deferred.resolve(data);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
            deferred.reject(jqxhr);
        });

        return deferred;
    },

    saveScenarios: function (scenarios) {
        var api = this;
        var deferred = $.Deferred();
        var linkDesc = {
            rel: 'importScenarios',
            type: 'application/vnd.sas.collection+json',
            itemType: 'application/vnd.sas.risk.scenario+json;charset=utf-8',
            responseType: 'application/vnd.sas.collection+json',
            responseItemType: 'application/vnd.sas.risk.scenario.summary+json;charset=utf-8'
        };
        _.each(scenarios, function (scenario) {
            for (var index = 0; index < scenario.stresses.length; ++index) {
                scenario.stresses[index].perturbations = sas.risk.scenariomanager.utils.ScenarioEditorUtil.unmapPerturbations(scenario.stresses[index].perturbations);
                scenario.stresses[index] = _.omit(scenario.stresses[index], ['parent', 'children']);
            }
            if (scenario.periods) {
                scenario.periods = sas.risk.scenariomanager.utils.ScenarioEditorUtil.unmapPeriods(scenario.periods);
            }
            if (scenario.dataSourceArgs)
                scenario.dataSourceArgs = sas.risk.scenariomanager.ScenarioAPI.decodeArgsAsQuery(scenario.dataSourceArgs);
        });
        var link = api.findRequiredLink(deferred, api.getRootLinks(), linkDesc);
        if (link) {
            $.ajax(
                {
                    type: 'POST',
                    url: link.href,
                    dataType: 'json',
                    accepts: { json: link.responseType },
                    contentType: link.type,
                    beforeSend: function (xhr) {
                        xhr.setRequestHeader('Content-Item', link.itemType);
                        xhr.setRequestHeader('Accept-Item', link.responseItemType);
                    },
                    data: JSON.stringify(scenarios)
                }
            ).done(function (data) {
                // Clear the current context after an import is successful, this will force it to reload the next time the editor is opened.
                api._crntContext = null;
                _.each(scenarios, function (scenario) {
                    for (var index = 0; index < scenario.stresses.length; ++index) {
                        scenario.stresses[index].perturbations = sas.risk.scenariomanager.utils.ScenarioEditorUtil.mapPerturbations(scenario.stresses[index].perturbations);
                    }
                    scenario.periods = sas.risk.scenariomanager.utils.ScenarioEditorUtil.mapPeriods(scenario.periods);
                    // if (scenario.workGroup === undefined) {
                    //     var publicWorkGroup = api.getPublicWorkGroup();
                    //     if (publicWorkGroup) {
                    //         scenario.workGroup = publicWorkGroup.name;
                    //     }
                    // }
                });

                deferred.resolve(data);
            }).fail(function (jqxhr) {
                _.each(scenarios, function (scenario) {
                    // If it failed, we need to put it back as it was
                    if (scenario.stresses) {
                        for (var index = 0; index < scenario.stresses.length; ++index) {
                            scenario.stresses[index].perturbations = sas.risk.scenariomanager.utils.ScenarioEditorUtil.mapPerturbations(scenario.stresses[index].perturbations);
                        }
                    }
                    scenario.periods = sas.risk.scenariomanager.utils.ScenarioEditorUtil.mapPeriods(scenario.periods);
                });
                jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }

        return deferred;
    },

    createScenario: function (scenario) {
        var api = this;
        var deferred = $.Deferred();

        var linkDesc = {
            rel: 'createScenario',
            type: 'application/vnd.sas.risk.scenario+json;charset=utf-8',
            responseType: 'application/vnd.sas.risk.scenario+json;charset=utf-8'
        };

        var link = api.findRequiredLink(deferred, api.getRootLinks(), linkDesc);
        if (link) {
            if (scenario.stresses) {
                for (var index = 0; index < scenario.stresses.length; ++index) {
                    scenario.stresses[index].perturbations = sas.risk.scenariomanager.utils.ScenarioEditorUtil.unmapPerturbations(scenario.stresses[index].perturbations);
                    scenario.stresses[index] = _.omit(scenario.stresses[index], ['parent', 'children']);
                }
            }
            if (scenario.periods) {
                scenario.periods = sas.risk.scenariomanager.utils.ScenarioEditorUtil.unmapPeriods(scenario.periods);
            }
            if (scenario.dataSourceArgs)
                scenario.dataSourceArgs = this.decodeArgsAsQuery(scenario.dataSourceArgs);
            $.ajax(
                {
                    type: 'POST',
                    url: link.href,
                    dataType: 'json',
                    accepts: { json: link.responseType },
                    contentType: link.type,
                    data: JSON.stringify(scenario)
                }
            ).done(function (data) {
                _.each(data.stresses, function (stress) {
                    stress.perturbations = sas.risk.scenariomanager.utils.ScenarioEditorUtil.mapPerturbations(stress.perturbations);
                });
                sas.risk.scenariomanager.utils.ScenarioEditorUtil.mapPeriods(data.periods);
                // if (data.workGroup === undefined) {
                //     var publicWorkGroup = api.getPublicWorkGroup();
                //     if (publicWorkGroup) {
                //         data.workGroup = publicWorkGroup.name;
                //     }
                // }
                deferred.resolve(data);
            }).fail(function (jqxhr) {
                // If it failed, we need to put it back as it was
                if (scenario.stresses) {
                    for (var index = 0; index < scenario.stresses.length; ++index) {
                        scenario.stresses[index].perturbations = sas.risk.scenariomanager.utils.ScenarioEditorUtil.mapPerturbations(scenario.stresses[index].perturbations);
                    }
                }
                scenario.periods = sas.risk.scenariomanager.utils.ScenarioEditorUtil.mapPeriods(scenario.periods);
                jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }

        return deferred;
    },

    putScenario: function (scenario, linkRel) {
        var api = this;
        var deferred = $.Deferred();
        var link = api.findRequiredLink(deferred, scenario.links, { rel: linkRel });
        if (link) {
            if (scenario.stresses) {
                for (var index = 0; index < scenario.stresses.length; ++index) {
                    scenario.stresses[index].perturbations = sas.risk.scenariomanager.utils.ScenarioEditorUtil.unmapPerturbations(scenario.stresses[index].perturbations);
                    scenario.stresses[index] = _.omit(scenario.stresses[index], ['parent', 'children']);
                }
            }
            if (scenario.periods) {
                scenario.periods = sas.risk.scenariomanager.utils.ScenarioEditorUtil.unmapPeriods(scenario.periods);
            }
            if (scenario.dataSourceArgs)
                scenario.dataSourceArgs = this.decodeArgsAsQuery(scenario.dataSourceArgs);
            $.ajax(
                {
                    type: 'PUT',
                    url: link.href,
                    contentType: link.type,
                    beforeSend: function (xhr) {
                        xhr.setRequestHeader('If-Unmodified-Since', new Date(scenario.modifiedTimeStamp).toUTCString());
                    },
                    data: JSON.stringify(scenario)
                }
            ).done(function (data) {
                _.each(data.stresses, function (stress) {
                    stress.perturbations = sas.risk.scenariomanager.utils.ScenarioEditorUtil.mapPerturbations(stress.perturbations);
                });
                sas.risk.scenariomanager.utils.ScenarioEditorUtil.mapPeriods(data.periods);
                // if (data.workGroup === undefined) {
                //     var publicWorkGroup = api.getPublicWorkGroup();
                //     if (publicWorkGroup) {
                //         data.workGroup = publicWorkGroup.name;
                //     }
                // }
                deferred.resolve(data);
            }).fail(function (jqxhr) {
                // If it failed, we need to put it back as it was
                if (scenario.stresses) {
                    for (var index = 0; index < scenario.stresses.length; ++index) {
                        scenario.stresses[index].perturbations = sas.risk.scenariomanager.utils.ScenarioEditorUtil.mapPerturbations(scenario.stresses[index].perturbations);
                    }
                }
                scenario.periods = sas.risk.scenariomanager.utils.ScenarioEditorUtil.mapPeriods(scenario.periods);

                jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }

        return deferred;
    },

    deleteScenarios: function (scenarios, cascade) {
        var api = this;
        var deferred = $.Deferred();

        var patchData = [];
        scenarios = _.sortBy(scenarios, function (scen) {
            return 0 - scen.baselineId;
        });

        for (var i = 0; i < scenarios.length; i++) {
            patchData.push("{\"op\":\"remove\", \"path\":\"/" + scenarios[i].id + "\"}");
        }
        var patchStr = "[" + patchData.toString() + "]";
        var rel = cascade ? 'multiDeleteCascade' : 'multiDelete';
        var link = this.findRequiredLink(deferred, api._scenarioLinks, { rel: rel });

        if (link) {
            $.ajax(
                {
                    type: 'PATCH',
                    url: link.href,
                    dataType: 'json',
                    accepts: { json: link.responseType },
                    contentType: link.type,
                    beforeSend: function (xhr) {
                        xhr.setRequestHeader('Accept-Item', link.responseItemType);
                    },
                    data: patchStr
                }
            ).done(function () {
                deferred.resolve();
            }).fail(function (jqxhr) {
                jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }

        return deferred;
    },

    getWorkGroups: function () {
        if (!this._workGroups) {
            this._workGroups = [];
        }
        return this._workGroups;
    },

    loadWorkGroups: function () {
        var that = this;
        var deferred = $.Deferred();
        var rootLinks = sas.risk.scenariomanager.ScenarioAPI.getRootLinks();

        var workGroupsLink = sas.risk.scenariomanager.ScenarioAPI.findRequiredLink(deferred, rootLinks, { rel: 'workGroups' });
        if (workGroupsLink) {
            // Only
            if (this._workGroups === undefined) {
                $.getJSON(workGroupsLink.href + "?solutions=hprisksvrc,stresssvrc", function (data) {
                    that._workGroups = data.items;
                    for (var i = 0; i < data.items.length; ++i) {
                        if (data.items[i].recommendedPrimaryGroup === true) {
                            that._recommendedPrimaryWorkGroup = data.items[i];
                            break;
                        }
                    }
                    deferred.resolve(data.items);
                }).fail(function (jqxhr) {
                    jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
                    deferred.reject(jqxhr);
                });
            } else {
                deferred.resolve(this._workGroups);
            }
        } else {
            deferred.reject();
        }
        return deferred;
    },

    getRecommendedPrimaryWorkGroup: function () {
        return this._recommendedPrimaryWorkGroup;
    },

    getCubesVisible: function () {
        var api = this;
        var link = this.findOptionalLink(api.getRootLinks(), { rel: 'cubes' });
        return !!link;
    },

    getCubes: function (workGroups) {
        var api = this;
        var deferred = $.Deferred();
        api.getCubesRoot(workGroups)
            .done(function(response){
                deferred.resolve(response.items);
            })
            .fail(function(jqxhr){
                deferred.reject(jqxhr);
            });

        return deferred;
    },

    // Returns the root collection of cubes, with all resource collection properties intact
    getCubesRoot: function (workGroups) {
        var api = this;
        var deferred = $.Deferred();
        var link = this.findOptionalLink(api.getRootLinks(), { rel: 'cubes' });
        if (link) {

            var workGroupQuery = "";
            if (workGroups.length > 0) {
                workGroupQuery += "?workGroup=";
            }
            for (var i = 0; i < workGroups.length; ++i) {
                if (i > 0) {
                    workGroupQuery += "%7c"; // '|' must apparently be encoded, now
                }
                workGroupQuery += workGroups[i];
            }

            $.getJSON(link.href + workGroupQuery, function (response) {
                deferred.resolve(response);
            }).fail(function (jqxhr) {
                jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        } else {
            deferred.resolve({items:[]});
        }

        return deferred;
    },

    // Ideally, HPRisk would support GET .../cubes?query where we could specify .../cubes?cubeId=cubeid
    getCube: function (cubeid, cubeParms) {
        var api = this;
        var deferred = $.Deferred();
        var link = this.findOptionalLink(api.getRootLinks(), { rel: 'queryCubes' });
        if (link) {
            var url = link.href + '/' + cubeid;
            if (cubeParms) {
                if (typeof cubeParms !== 'string')
                    cubeParms = api.encodeArgsAsQuery(cubeParms);

                url += '?' + cubeParms;
            }
            $.getJSON(url, function (data) {
                deferred.resolve(data);
            }).fail(function (jqxhr) {
                jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        } else {
            deferred.reject();
        }

        return deferred;
    },

    getCubeSelf: function (cube) {
        var api = this;
        var deferred = $.Deferred();
        if (cube) {
            var link = this.findRequiredLink(deferred, cube.links, { rel: 'self' });
            if (link) {
                $.getJSON(link.href, function (context) {
                    deferred.resolve(context);
                }).fail(function (jqxhr) {
                    jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
                    deferred.reject(jqxhr);
                });
            }
        } else {
            deferred.resolve(cube);
        }

        return deferred;
    },

    _crntContext: null,

    _isSameContext: function (newContextLink) {
        var api = this;
        // Is there a saved context already and a link for the new context?
        if (api._crntContext && newContextLink) {
            var currentContextLink = this.findOptionalLink(api._crntContext.links, {rel: 'self'});
            // Does the saved context have a link?
            if (currentContextLink) {
                // Get the data source args for the current context
                var currentContextTokens = currentContextLink.uri.split("?");
                // Get the data source args for the new context
                var newContextTokens = newContextLink.uri.split("?");

                if (currentContextTokens.length === 2 && newContextTokens.length === 2) {
                    // Is the cube id the same?
                    if (currentContextTokens[0] === newContextTokens[0]) {
                        var currentDataSourceParameters = api.encodeArgsAsObject(currentContextTokens[1]);
                        var newDataSourceParameters = api.encodeArgsAsObject(newContextTokens[1]);
                        return _.isEqual(currentDataSourceParameters, newDataSourceParameters);
                    }
                }
            }
        }
            return false;
    },

    maybeCloseContext: function(context) {
        var api = this;
        if (context) {
            var link = this.findOptionalLink(context.links, {rel: 'close'});
            if (link) {
                $.ajax(
                    {
                        type: 'DELETE',
                        cache: false,
                        url: link.href
                    }
                ).done(function(data){
                }).fail(function (jqxhr) {
                    jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
                });
            }
        }
    },

    getCubeContext: function (cube) {
        var api = this;
        var deferred = $.Deferred();
        var link = this.findOptionalLink(cube.links, { rel: 'stressContext' });
        if (link) {
            if (api._isSameContext(link)) {
                deferred.resolve(api._crntContext);
            } else {
                api.maybeCloseContext(api._crntContext);
                sas.risk.scenariomanager.utils.ScenarioEditorUtil.closeCubeKeepAlive();
                $.getJSON(api._appendContextFields(link.href), function (context) {
                    api._crntContext = context;
                    //Call keepAlive method here
                    sas.risk.scenariomanager.utils.ScenarioEditorUtil.updateCubeKeepAlive(context);
                    deferred.resolve(context);
                }).fail(function (jqxhr) {
                    jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
                    deferred.reject(jqxhr);
                });
            }
        }
        else {
            var jqxhr = deferred.promise();
            deferred.reject(jqxhr);
        }

        return deferred;
    },

    getScenarioVariables: function(scenario, context) {
        var api = this;
        var names = _.union(_.pluck(scenario.stresses, 'variable'), _.pluck(scenario.variableStresses, 'variable'));
        if (scenario.baseline != null) {
            names = _.union(names, _.pluck(scenario.baseline.stresses, 'variable'), _.pluck(scenario.baseline.variableStresses))
        }

        var deferred = api.getVariableDefinitions(context, names);
        deferred.done(function(data){ api.updateStressTypes(scenario, context); });
        return deferred;
    },

    // When we get variable definitions from the midtier, we may find that the given scenario has some
    // risk factor stresses in the instrument variable list or vice versa (due to missing definitions or
    // from variables that are defined as one thing in one cube and a different thing in another cube.
    // Shuffle them off to the right list.
    updateStressTypes: function(scenario, context) {
        var isInstrumentVariable = function(type) {
            return type.toUpperCase() == sas.risk.scenariomanager.model.RiskVarTypeConstants.INSTRUMENTVARIABLE_OBJTYPE;
        };

        if (scenario.stresses) {
            for (var i = scenario.stresses.length - 1; i >= 0; --i) {
                var stress = scenario.stresses[i],
                    definition = context.getObject(stress.variable);

                if (definition && isInstrumentVariable(definition.type) && stress.perturbations.length == 1) {
                    if (scenario.variableStresses) {
                        scenario.variableStresses.push({
                            variable: stress.variable,
                            perturbType: stress.perturbType,
                            changeValue: stress.perturbations[0].value.toString()
                        })
                    }
                    else {
                        scenario.variableStresses = [{
                            variable: stress.variable,
                            perturbType: stress.perturbType,
                            changeValue: stress.perturbations[0].value.toString()
                        }];
                    }

                    scenario.stresses.splice(i, 1);
                }
            }
        }

        // This case will be far less common.  Q: can we move a variableStress over to stresses if it occurs multiple
        // times and one (or more) variableStresses have a ruleType, ruleValue, or string shock???
        if (scenario.variableStresses) {
            for (i = scenario.variableStresses.length - 1; i >= 0; --i){
                stress = scenario.variableStresses[i];
                definition = context.getObject(stress.variable);

                if (definition && !isInstrumentVariable(definition.type)) {
                    // If the stress has a ruleType, ruleValue, or character change, skip it
                    if ((stress.hasOwnProperty('ruleType') && stress.ruleType) ||
                        (stress.hasOwnProperty('ruleValue') && stress.ruleValue) ||
                        (stress.hasOwnProperty('changeValue') && !jQuery.isNumeric(stress.changeValue))
                    ){
                        continue;
                    }

                    if (scenario.stresses) {
                        scenario.stresses.push({
                            variable: stress.variable,
                            perturbType: stress.perturbType,
                            perturbations: [{
                                value: Number(stress.changeValue),
                                overrideInfo: null
                            }]
                        })
                    }
                    else {
                        scenario.stresses = [{
                            variable: stress.variable,
                            perturbType: stress.perturbType,
                            perturbations: [{
                                value: Number(stress.changeValue),
                                overrideInfo: null
                            }]
                        }];
                    }
                    scenario.variableStresses.splice(i, 1);
                }
            }
        }
    },

    getVariableDefinitions: function(context, names) {
        var deferred = $.Deferred();
        if (names == null || names.length == 0) {
            deferred.resolve({items:[]});
            return deferred;
        }

        var link = this.findRequiredLink(deferred, context.getLinks(), {rel: 'variableDefinitions'});
        $.ajax({
            type: 'POST',
            url: link.href,
            dataType: 'json',
            accepts: { json: link.responseType },
            contentType: link.type,
            beforeSend: function (xhr) {
                xhr.setRequestHeader('Accept-Item', link.responseItemType);
            },
            data: JSON.stringify(names)
        }).done(function (data) {
            // Populate the scenario context with the variables loaded
            if (data && data.items) {
                _.each(data.items, jQuery.proxy(context.updateIndex, context));
            }
            deferred.resolve(data);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
            deferred.reject(jqxhr);
        });

        return deferred;
    },

    getScenarioContext: function (scenario) {
        var api = this;
        var deferred = $.Deferred();
        var link = this.findOptionalLink(scenario.links, { rel: 'stressContext' });
        if (link) {
            if (api._isSameContext(link)) {
                deferred.resolve(api._crntContext);
            } else {
                api.maybeCloseContext(api._crntContext);
                sas.risk.scenariomanager.utils.ScenarioEditorUtil.closeCubeKeepAlive();
                $.getJSON(api._appendContextFields(link.href), function (context) {
                    api._crntContext = context;
                    //Call keepAlive method here
                    sas.risk.scenariomanager.utils.ScenarioEditorUtil.updateCubeKeepAlive(context);
                    deferred.resolve(context);
                }).fail(function (jqxhr) {
                    jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);

                    // The cube failed to load, warn the user and fallback to the scenario repository
                    var rb = sap.ui.getCore().getLibraryResourceBundle("sas.risk.scenariomanager.nls");
                    sas.hc.m.MessageDialog.warning({
                            text: rb.getText('scenarioEditor.cubeNotFound.warning.fmt',
                                             [scenario.dataSourceName.replace(new RegExp("\.rskcdesc$", "i"), "")]),
                            title: rb.getText('scenarioEditor.cubeNotFound.warning.title')
                        });
                    api.getRepositoryContext(deferred);
                });
            }

            return deferred;
        }

        return api.getRepositoryContext(deferred);
    },

    getRepositoryContext: function (deferred) {
        var api = this;
        if (!deferred) {
            deferred = $.Deferred();
        }
        var link = this.findRequiredLink(deferred, api.getRootLinks(), { rel: 'stressContext' });
        if (api._isSameContext(link)) {
            deferred.resolve(api._crntContext);
        } else {
            api.maybeCloseContext(api._crntContext);
            $.getJSON(api._appendContextFields(link.href), function (context) {
                api._crntContext = context;
                deferred.resolve(context);
            }).fail(function (jqxhr) {
                jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }

        return deferred;
    },

    _appendContextFields: function(href) {
        var conj = (href.indexOf("?") > 0) ? '&' : '?';
        return href + conj + 'fields=id,name,label,type,runDate,filters';
    },

    getHistoryLink: function(context) {
        return this.findOptionalLink(context.getLinks(), {
            rel: 'history',
            type: 'application/json',
            responseType: 'application/vnd.sas.risk.scenario+json;charset=utf-8'
        });
    },

    getVariableHistory: function (context, varNames) {
        var api = this;
        var deferred = $.Deferred();
        var link = this.findRequiredLink(deferred, context.getLinks(),
                                         {
                                             rel: 'history',
                                             type: 'application/json',
                                             responseType: 'application/vnd.sas.risk.scenario+json;charset=utf-8'
                                         });

        if (link) {
            $.ajax(
                {
                    type: 'POST',
                    url: link.href,
                    dataType: 'json',
                    accepts: { json: link.responseType },
                    contentType: 'application/json', //'application/json;charset=utf-8'
                    data: JSON.stringify(varNames)
                }
            ).done(function (data) {
                deferred.resolve(data);
            }).fail(function (jqxhr) {
                jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }

        return deferred;
    },

    _userName: '',

    loadUserName: function () {
        var api = this;
        var deferred = $.Deferred();
        $.getJSON('rest/commons/apps/userdetails', function (user) {
            api.setUserName(user.displayName);
            deferred.resolve(user);
            // NOTE:we can grab the roles here
        });
        return deferred;
    },

    setUserName: function (userName) {
        this._userName = userName;
    },

    getUserName: function () {
        var oUserInfo = sas.hc.ui.core.util.UserInfo.instance;

        // I don't know what object property to use. Assuming name.
        return (oUserInfo !== undefined) ? oUserInfo.name : this._userName;
    },

    _fillInWorkGroupParams: function(href, workGroupName){
        var url = href.replace("{0}", "WORKGROUP");
        if (!workGroupName) {
            workGroupName = "";
        }
        url = url.replace("{1}", workGroupName);
        return url;
    },

    loadServerFiles: function (workGroupName) {
        var api = this;
        var deferred = $.Deferred();
        var link = this.findOptionalLink(api.getRootLinks(), { rel: 'importExportFiles' });
        if (link) {
            var url = api._fillInWorkGroupParams(link.href, workGroupName);
            $.getJSON(url, function (data) {
                deferred.resolve(data);
            }).fail(function (jqxhr) {
                jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);

                var rb = sap.ui.getCore().getLibraryResourceBundle("sas.risk.scenariomanager.nls");
                var title = rb.getText('error.window.title');
                var response = jqxhr.responseJSON;
                if (response) {
                    sas.risk.common.widgets.MessageDetailsBox.error(response, title);
                } else {
                    sas.hc.m.MessageDialog.error({ text: rb.getText('error.message.txt'), title: title });
                }

                deferred.resolve([]);
            });
        } else {
            deferred.resolve([]);
        }

        return deferred;
    },

    importFile: function (fileName, workGroupName, historyDate) {
        var api = this;
        var deferred = $.Deferred();
        var link = this.findRequiredLink(deferred, api.getRootLinks(), { rel: 'reportScenariosForLocation' });

        if (link) {
            var url = api._fillInWorkGroupParams(link.href, workGroupName).replace("{2}", fileName);
            if (historyDate)
                url += "&historyDate="+historyDate;

            $.ajax(
                {
                    type: 'POST',
                    url: url,
                    dataType: 'json',
                    accepts: { json: link.responseType },
                    beforeSend: function (xhr) {
                        xhr.setRequestHeader('Accept-Item', link.responseItemType);
                    },
                    data: {}
                }
            ).done(function (data) {
                // Clear the current context after an import is successful, this will force it to reload the next time the editor is opened.
                api._crntContext = null;
                deferred.resolve(data);
            }).fail(function (jqxhr) {
                jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }

        return deferred;
    },

    exportScenarios: function (scenarioIds, datasetName, workGroupName, useServerWorkGroupDir) {
        var api = this;
        var deferred = $.Deferred();

        // if not set to use the work group directory, then we will write to the temporary work directory on the server
        var toServerWorkGroupDir = useServerWorkGroupDir !== false;
        var link = this.findRequiredLink(deferred, api.getRootLinks(), { rel: 'exportScenariosByIDToLocation' });

        if (link) {
            var url = link.href;
            if (toServerWorkGroupDir) {
                url = api._fillInWorkGroupParams(url, workGroupName).replace("{2}", datasetName);
            } else { // to temporary work directory, to be downloaded by subsequent call
                url = url.replace("{0}", "LIBNAME").replace("{1}", "WORK").replace("{2}", datasetName);
            }

            var ajaxParmsToServer = {
                type: link.method,
                url: url,
                dataType: 'json',
                accepts: { json: link.responseType },
                contentType: link.type,
                beforeSend: function (xhr) {
                    xhr.setRequestHeader('Content-Item', link.itemType);
                    xhr.setRequestHeader('Accept-Item', link.responseItemType);
                },
                data: JSON.stringify(scenarioIds)
            };

            $.ajax(ajaxParmsToServer).done(function (data) {
                deferred.resolve(data);
            }).fail(function (jqxhr) {
                jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }

        return deferred;
    },

    generateSensitivityAnalysis: function (msaConfiguration) {
        var api = this;
        var deferred = $.Deferred();
        var link = this.findRequiredLink(deferred, api.getRootLinks(), { rel: 'msa' });
        if (link) {
            $.ajax(
                {
                    type: 'POST',
                    url: link.href,
                    dataType: 'text',
                    accepts: {
                        text: 'text/plain'// returns text but link is not set up with a responseType
                    },
                    contentType: link.type,
                    data: JSON.stringify(msaConfiguration)
                }
            ).done(function (data) {
                deferred.resolve(data);
            }).fail(function (jqxhr) {
                jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }

        return deferred;
    },

    resolveDates: function (interval, horizons, context) {
        var api = this,
            deferred = $.Deferred(),
            linkDesc = {
                rel: 'computeIntervals',
                type: 'application/vnd.sas.collection+json',
                itemType: 'text/plain'
            },
            link = api.findOptionalLink(api.getRootLinks(), linkDesc);

        // TODO: if context has a 'computeIntervals' link, use it instead (it should correspond to an HPRisk cube
        // that provides similar functionality, but might be able to resolve custom intervals that the repo's sas
        // connection can't)

        if (link) {
            link.href = link.href.replace("{intervalType}", interval)
                                 .replace("{histDate}", context.getRunDate())
                                 .replace("{start}", horizons[0])
                                 .replace("{end}", horizons[horizons.length - 1]);

            $.getJSON(link.href, function (data) {
                deferred.resolve(data);
            }).fail(function (jqxhr) {
                jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        } else {
            deferred.resolve();
        }

        return deferred;
    },

    resolveHorizonDates: function (scenario, context) {
        var Scenario = sas.risk.scenariomanager.model.Scenario,
            api = this,
            deferred = $.Deferred(),

            linkDesc = {
                rel: 'computeIntervals',
                type: 'application/vnd.sas.collection+json',
                itemType: 'text/plain'
            },
            link = api.findOptionalLink(scenario.links, linkDesc);

        // If we've already resolved the horizons for this context, there is no need to continue
        if (Scenario.hasResolvedDates(scenario, context)) {
            deferred.resolve();
            return;
        }

        // TODO: if context has a 'computeIntervals' link, use it instead (it should correspond to an HPRisk cube
        // that provides similar functionality, but might be able to resolve custom intervals that the repo's sas
        // connection can't)

        if (link) {
            $.getJSON(link.href + "&historyDate=" + context.getRunDate(), function (data) {
                var firstHz = scenario.periods[0].horizon;
                var i = 0;
                scenario.periods = scenario.periods.map(function (period) {
                    period.index = ++i;
                    period.date = data.items[period.horizon - firstHz];
                    period.matchString = sas.risk.scenariomanager.utils.ScenarioEditorUtil.getHorizonMatchString(period);
                    return period;
                });
                scenario[Scenario.HORIZON_SOURCE] = context.getId()
                deferred.resolve();
            }).fail(function (jqxhr) {
                jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        } else {
            deferred.resolve();
        }

        return deferred;
    },

    encodeArgsAsQuery: function(cubeParms) {
        var conj = '', str = '';
        for (var key in cubeParms) {
            if (cubeParms.hasOwnProperty(key)) {
                if (cubeParms[key] !== null && cubeParms[key] !== undefined) {
                    str += conj + key + '=' + encodeURIComponent(cubeParms[key]);
                    conj = '&';
                }
            }
        }

        return str;
    },

    encodeArgsAsObject: function(cubeParms) {
        var parmsObj = {};
        if (!cubeParms)
            return parmsObj;

        var parmsArray = decodeURIComponent(cubeParms).split('&');
        for (var idx = 0; idx < parmsArray.length; ++idx) {
            var keyVal = parmsArray[idx].split('=');
            if (keyVal[1])
                parmsObj[keyVal[0]] = decodeURIComponent(keyVal[1].replace(new RegExp("\\+", 'g'), " "));
        }

        return parmsObj;
    },

    decodeArgsAsQuery: function(cubeParms) {
        var conj = '', str = '';
        if (!cubeParms)
            return parmsObj;

        var parmsArray = decodeURIComponent(cubeParms).split('&');
        for (var idx = 0; idx < parmsArray.length; ++idx) {
            var keyVal = parmsArray[idx].split('=');
            str+= conj+keyVal[0] +'='+keyVal[1];
            conj = '&';
        }

        return str;
    },

    _rootLinks: [],

    loadRootLinks: function () {
        var api = this;
        var deferred = $.Deferred();
        $.getJSON('rest', function (data) {
            api._rootLinks = data.links;
            deferred.resolve(data);
        });
        return deferred;
    },

    getRootLinks: function () {
        return this._rootLinks;
    },

    findOptionalLink: function (links, properties) {
        var link = _.findWhere(links, properties);
        return (link) ? sas.risk.scenariomanager.utils.ScenarioEditorUtil.deepCopy(link) : link;
    },

    findRequiredLink: function (deferred, links, properties) {
        var link = this.findOptionalLink(links, properties);
        if (!link) {
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.risk.scenariomanager.nls");
            var msg = rb.getText('error.missingLink.fmt', JSON.stringify(properties));
            deferred.reject(
                {
                    responseText: msg,
                    responseJSON: {
                        message: msg,
                        errorCode: 400
                    }
                }
            );
        }

        return link;
    },


    cubeKeepAlive:function(cube){
        var deferred = $.Deferred();
        var keepLiveURL = cube.keepLiveURL;

        var stringifiedData = JSON.stringify(cube);

        // NOTE: dataType must be set to "text" because no data is being returned. dataType = "json" throws a
        // json parsing error.
        // https://stackoverflow.com/questions/37057711/unexpected-end-of-json-input-from-an-ajax-call
        var ajaxRequest = {
            headers: {
                "Content-Type": "application/json",
                "Content-Item": "application/vnd.sas.risk.cube.summary+json;charset=utf-8"
            },
            contentType: "application/json",
            method: "POST",
            url: keepLiveURL,
            data: stringifiedData,
            dataType: "text"
        };

        // Get the logger for this object and output stuff
        var logger = sas.log.getLogger("sas.risk.scenariomanager.ScenarioAPI");
        logger.debug('ScenarioAPI.cubeKeepAlive ' + keepLiveURL);
        logger.trace('ScenarioAPI.cubeKeepAlive ' + JSON.stringify(ajaxRequest));

        $.ajax(ajaxRequest)
            .done(function (data, textStatus, jqxhr) {
                logger.debug('ScenarioAPI.cubeKeepAlive DONE');
                deferred.resolve({status: jqxhr.status});
            })
            .fail(function (jqxhr, textStatus, error) {
                jQuery.sap.log.error("ScenarioAPI.cubeKeepAlive  Request Failed. " + textStatus + " Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        return deferred;
    },

    releaseDataSource:function(){
        var api = this;
        var loadedDataSource = api._crntContext;

        if (loadedDataSource && loadedDataSource.name!=="Scenario Repository"){
            var deferred = api.maybeCloseContext(loadedDataSource);
            deferred.done(function(){
                sas.risk.scenariomanager.utils.ScenarioEditorUtil.closeCubeKeepAlive();
                deferred.resolve();
            }).fail(function(){
                jQuery.sap.log.error("ScenarioAPI.releaseDataSource  Failed. " );
                deferred.reject();
            });
        }


    }

};