// (c) 2015, SAS Institute Inc.

jQuery.sap.declare("sas.analytics.riskhp.models.CrosstabContext");
jQuery.sap.require("sas.hc.ui.unified.Menu");
jQuery.sap.require("sas.hc.ui.unified.MenuItem");

(function() {
    "use strict";

    jQuery.sap.require('sas.analytics.riskhp.common.riskcommons.CrosstabUtil');
    var CrosstabUtil = sas.analytics.riskhp.common.riskcommons.CrosstabUtil;

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Classvar");
    var CLASSVAR = sas.analytics.riskhp.models.dataitems.Classvar.CLASSVAR;

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Hierarchy");
    var HIERARCHY = sas.analytics.riskhp.models.dataitems.Hierarchy.HIERARCHY;

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Outvar");
    var OUTVAR = sas.analytics.riskhp.models.dataitems.Outvar.OUTVAR;
    var Outvar = sas.analytics.riskhp.models.dataitems.Outvar;

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Compvar");
    var COMPVAR = sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR;

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Benchvar");
    var BENCHVAR = sas.analytics.riskhp.models.dataitems.Benchvar.BENCHVAR;

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Statistic");
    var STATISTIC = sas.analytics.riskhp.models.dataitems.Statistic.STATISTIC;
    var Statistic = sas.analytics.riskhp.models.dataitems.Statistic;

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Horizon");
    var HORIZON = sas.analytics.riskhp.models.dataitems.Horizon.HORIZON;
    var Horizon = sas.analytics.riskhp.models.dataitems.Horizon;

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.RelativeHorizon");
    var RELATIVEHORIZON = sas.analytics.riskhp.models.dataitems.RelativeHorizon.RELATIVEHORIZON;

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.AbsoluteHorizon");
    var ABSOLUTEHORIZON = sas.analytics.riskhp.models.dataitems.AbsoluteHorizon.ABSOLUTEHORIZON;

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.RiskCube");
    var RISKCUBE = sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE;

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.ResultSet");
    var RESULTSET = sas.analytics.riskhp.models.dataitems.ResultSet.RESULTSET;

    jQuery.sap.require("sas.analytics.riskhp.models.dataitems.Scenario");
    var SCENARIO = sas.analytics.riskhp.models.dataitems.Scenario.SCENARIO;

    jQuery.sap.require("sas.analytics.riskhp.settings.ApplicationSettings");
    var ApplicationSettings = sas.analytics.riskhp.settings.ApplicationSettings;

    jQuery.sap.require("sas.analytics.riskhp.utils.HprConstants");
    var HprConstants = sas.analytics.riskhp.utils.HprConstants; // No need to do this

    jQuery.sap.require('sas.analytics.riskhp.utils.HprUtil');
    var HprUtil = sas.analytics.riskhp.utils.HprUtil;

    jQuery.sap.require("sas.analytics.riskhp.models.cache.CacheManager");

    jQuery.sap.require("sas.analytics.riskhp.models.cache.CacheKey");
    var CacheKey = sas.analytics.riskhp.models.cache.CacheKey;

    jQuery.sap.require("sas.analytics.riskhp.views.displayrules.DisplayRulesUtil");
    var DisplayRulesUtil = sas.analytics.riskhp.views.displayrules.DisplayRulesUtil;

    var EXPANSION_NAME = "query";
    var EXPANSION_NAME_SEPARATOR = ">";
    var SAS_ALL = "_sas_ALL_";

    // noinspection JSClosureCompilerSyntax
    /**
     * @name sas.analytics.riskhp.models.CrosstabContext
     * @extends sap.ui.base.Object
     */
    sap.ui.base.Object.extend("sas.analytics.riskhp.models.CrosstabContext", {
        metadata: {
            publicMethods: []
        },

        constructor: function (id) {
            this.id = id;

            // External dependencies
            this.api = sas.analytics.riskhp.utils.HprAPI;
            this.i18n = sas.analytics.riskhp.utils.HprUtil.getBundle();
            this.cubeDataModel = {};
            this.crosstab = {};
            this.crosstabScrollPosition = undefined;

            this.lastQueryRequest = undefined;
            this.lastQueryResponse = undefined;

            // Act zero state
            this.zeroState = true;

            // Default data items from cube
            this.defaultClassVariables = []; // dataItems
            this.defaultTuples = []; // dataItems

            // Axes to be displayed in crosstab
            this.classVariables = []; // dataItems
            this.columnTree = {};
            this.reportResultsTree = {
                name: "toplevel",
                label: "toplevel",
                parent: undefined,
                results: [],
                expanded: true,
                children: [],
                startRow: 1,
                rows: 1,
                totalRows: 1
            };


            // Modifications to crosstab display
            this.rowIndent = false;
            this.columnIndent = false;

            this.drillpath = []; // "classvar values"

            this.sorts = [];  // {direction, type, column number}

            // column formats
            this.columnFormatOverride = []; // [{key: [dataItem, ...], format: "formatstring"}, ...]

            // filters
            this.filters = [];  // {collapsed, name (id?), type, values, include}

            this.rowHierarchyName = "rowHierarchy";
            this.columnHierarchyName = "columnHierarchy";
            this.columnHierarchyMembers = [];

            // Options
            this.optionName = null;
            this.optionALPHA = {};
            this.optionPCTLDEF = {};
            this.optionNROWS = {};
            this.optionHIERARCHY = {};
            this.optionTOTALS = {};
            this.optionSUBTOTALS = {};
            this.optionLOCATION = {};
            this.optionApplyEnabled = false; // Flag for the Apply button on each Options pane; Disabled by default
            this.optionGeneralOptionsSectionCollapsed = false; // Open by default
            this.optionTotalsOptionsSectionCollapsed = true; // Closed by default
            this.optionAdvancedQueryOptionsSectionCollapsed = true; // Closed by default

            this.optionTranspose = {};
            this.optionIndentRows = {};
            this.optionIndentColumns = {}; //NOT USED CURRENTLY may be needed in future
            this.optionShowLabels = {};
            this.optionDisplayHorizons = {};

            // Experimental but candidates for real options.
            this._optionShowAxisHeaders = true;

            // Our key to the cache.
            this.cacheKey = undefined;
        }
    });

    sas.analytics.riskhp.models.CrosstabContext.prototype.close = function () {
        CacheManager.deregister(this.cacheKey, this.getId());
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.exit = function () {
        CacheManager.deregister(this.cacheKey, this.getId());
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getId = function () {
        return this.id;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setId = function (id) {
        this.id = id;
    };

//*****************************
// Localized strings here
//*****************************
    sas.analytics.riskhp.models.CrosstabContext.prototype.getI18NBundle = function () {
        return this.i18n;
    };

    /**
     * Method for testing purposes that allows you to override the default i18 bundle
     * in a test harness
     * @param i18nBundle
     */
    sas.analytics.riskhp.models.CrosstabContext.prototype.setI18NBundle = function (i18nBundle) {
        this.i18n = i18nBundle;
    };

//*****************************
// Cube's data model
//*****************************
    sas.analytics.riskhp.models.CrosstabContext.prototype.getCubeDataModel = function () {
        return this.cubeDataModel;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setCubeDataModel = function (cubeDataModel) {
        this.cubeDataModel = cubeDataModel;
        if (cubeDataModel !== undefined) {
            this.zeroState = false;
            this.rebuildInternalModelsFromDefaults();
        } else {
            this.zeroState = true;
        }
    };

    /**
     * Is this model in zero state?
     * @returns {boolean} zero state
     */
    sas.analytics.riskhp.models.CrosstabContext.prototype.isZeroState = function () {
        return this.zeroState;
    };

    /**
     * Set the zero state flag
     */
    sas.analytics.riskhp.models.CrosstabContext.prototype.setZeroState = function (value) {
        if (value === undefined) {
            this.zeroState = true;
        } else {
            this.zeroState = value;
        }
    };

    /**
     * Is this model in a valid state?
     * @returns {boolean}
     */
    sas.analytics.riskhp.models.CrosstabContext.prototype.isValidModel = function () {
        var self = this;
        var valid = true;

        if (self.columnTree === undefined || self.columnTree.children === undefined || self.columnTree.children.length < 1) {
            valid = false;
        }
        return valid;
    };


//*****************************
// DataSource
//*****************************
    sas.analytics.riskhp.models.CrosstabContext.prototype.getDataSource = function () {
        return this.cubeDataModel.dataSource;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getDataSourceId = function () {
        return this.dataSourceId;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setDataSourceId = function (dataSourceId) {
        this.dataSourceId = dataSourceId;
    };


//*****************************
// crosstab scroll position
//*****************************
    sas.analytics.riskhp.models.CrosstabContext.prototype.getScrollPosition = function () {
        var self = this;

        if (self.crosstab && self.crosstab.uiView) {
            var crosstabView = self.crosstab.uiView.getBaseView();
            if (crosstabView) {
                self.crosstabScrollPosition = crosstabView.getScrollPosition();
            }
        }
        return self.crosstabScrollPosition;
    };

//*****************************
// Mid-tier API
//*****************************
    sas.analytics.riskhp.models.CrosstabContext.prototype.getRemoteAPI = function () {
        return this.api;
    };

    /**
     * Method for testing purposes that allows you to override the default remote api
     * handler in a test harness
     * @param api
     */

    sas.analytics.riskhp.models.CrosstabContext.prototype.setRemoteAPI = function (api) {
        this.api = api;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.buildCacheKey = function () {
        var self = this;

        self.cacheKey = new sas.analytics.riskhp.models.cache.CacheKey();
        self.cacheKey.setDataSourceId(self.getDataSourceId());
        self.cacheKey.setClassvarIds(self.getClassVariableIds());
        self.cacheKey.setALPHA(self.optionALPHA.value);
        self.cacheKey.setPCTLDEF(self.optionPCTLDEF.value);
        self.cacheKey.setHIERARCHY(self.optionHIERARCHY.value);
        self.cacheKey.setNROWS(self.optionNROWS.value);
        self.cacheKey.setFilters(self.buildQueryFilterObjects());
        self.cacheKey.setSorts(self.buildSortJSON(false));

        return self.cacheKey;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.buildCacheExpansions = function (forceFullQuery) {
        var self = this;

        // Collect all the expansions.
        var nrows = self.getOptionNROWS().value;
        var expansions = []; // example{key: [], startRow: 1, maxRows: 500}

        var _findNode = function (path, node) {
            if (node.expanded === true || (forceFullQuery && node.children.length > 0)) {
                //if (node.children.length > 0 && (node.expanded === true || forceFullQuery)) {
                //if (node.expanded === true || forceFullQuery) {
                var myPath = path.concat();
                myPath.push(SAS_ALL);
                expansions.push({key: myPath, startRow: node.startRow, maxRows: nrows});
                // Search each child for expansion, BUT not more than requested nrows
                // S1421144
                for (var b = 0; b < node.children.length && b < nrows; b++) {
                    var childPath = path.concat();
                    childPath.push(node.children[b].name);
                    _findNode(childPath, node.children[b]);
                }
            }
        };

        // walk the results tree and collect expansions
        expansions.push({key: [], startRow: 1, maxRows: nrows}); // top level expansion
        _findNode([], self.reportResultsTree);

        return expansions;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.cacheRefresh = function () {
        var self = this;

        return CacheManager.refresh(self.cacheKey, self.getId(), false);
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.queryCache = function (forceFullQuery) {
        var self = this;

        if (forceFullQuery === undefined) {
            forceFullQuery = false;
        }

        // Build the cache key
        self.cacheKey = self.buildCacheKey();

        // Collect all the expansions.
        var expansions = self.buildCacheExpansions(forceFullQuery); // example{key: [], startRow: 1, maxRows: 500}

        // convert dehydrated hidden tuples to tuples of real dataitems
        var hiddenTuples = [];
        _.each(self.getHiddenTuples(), function(tuple) {
            var hiddenTuple = [];
            _.each(tuple, function(item){
                hiddenTuple.push(self.cubeDataModel.getDataItemById(item.id));
            });
            hiddenTuples.push(hiddenTuple);
        });

        return CacheManager.getData(self.cacheKey, self.getId(), expansions, self.getTuples().concat(hiddenTuples));
    };


//*****************************
// Transpose
//*****************************
    sas.analytics.riskhp.models.CrosstabContext.prototype.getTranspose = function () {
        return this.optionTranspose.value;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setTranspose = function (trans) {
        this.optionTranspose.value = trans;
    };


//*****************************
// Indention
//*****************************
    sas.analytics.riskhp.models.CrosstabContext.prototype.getRowIndent = function () {
        return this.optionIndentRows.value;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setRowIndent = function (indent) {
        this.optionIndentRows.value = indent;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getColumnIndent = function () {
        return this.optionIndentColumns.value;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setColumnIndent = function (indent) {
        this.optionIndentColumns.value = indent;
    };


//*****************************
// Total/subtotal
//*****************************
    sas.analytics.riskhp.models.CrosstabContext.prototype.isTotalEnabled = function () {
        return this.optionTOTALS.value;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setTotalEnabled = function (totalEnabled) {
        this.optionTOTALS.value = totalEnabled;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.isSubtotalsEnabled = function () {
        return this.optionSUBTOTALS.value;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setSubtotalsEnabled = function (subtotalsEnabled) {
        this.optionSUBTOTALS.value = subtotalsEnabled;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getTotalsLocation = function () {
        return this.optionLOCATION.value;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setTotalsLocation = function (totalsLocation) {
        this.optionLOCATION.value = totalsLocation;
    };

//*****************************
// Drillpath  [string, string, etc]
//*****************************

    sas.analytics.riskhp.models.CrosstabContext.prototype.pushDrillPath = function (path) {
        if (this.drillpath.length > 0) {
            this.drillpath = this.drillpath.concat(path);
            this.drillnode = this.findRowTreeNode(this.drillpath, "label");
            this.drillnode.expanded = true;
        } else {
            this.drillpath = path;
            this.drillnode = this.findRowTreeNode(path, "label");
            this.drillnode.expanded = true;
        }
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getDrillPath = function () {
        return this.drillpath;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setDrillPath = function (newDrillpath) {
        // shorten new drill path if same length or more than number of classvars
        if (newDrillpath.length >= this.classVariables.length) {
            for (var a = 0; a < newDrillpath.length - this.classVariables.length + 1; a++) {
                newDrillpath.pop();
            }
        }
        this.drillpath = newDrillpath;
        this.drillnode = this.findRowTreeNode(newDrillpath, "label");
        this.drillnode.expanded = true;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.clearDrillPath = function () {
        this.drillpath = [];
        this.drillnode = this.reportResultsTree;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.popDrillPath = function (drillMember) {
        if (this.drillpath.length > 0) {
            var drillMemberIndex = _.lastIndexOf(this.drillpath, drillMember);
            if (drillMemberIndex >= 0) {
                for (var i = this.drillpath.length - 1; i >= drillMemberIndex; i--) {
                    var poppedDrillMember = this.drillpath.pop();
                    this.drillnode = this.drillnode.parent;
                }
            }
        }
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.updateDrill = function () {
        var self = this;

        var findChildIndex = function (node, name) {
            return _.findIndex(node.children, function (child) {
                return child.name === name;
            });
        };

        if (this.drillpath.length > 0) {
            // Check drill path for nRows impact.
            var node = self.drillnode;
            while (node.parent) {
                var childIndex = findChildIndex(node.parent, node.name);
                if (childIndex > node.parent.startRow + self.optionNROWS.value) {
                    self.popDrillPath(node.name);
                }
                node = node.parent;
            }
        }
    };


//*****************************
// Class Variables
//*****************************
    sas.analytics.riskhp.models.CrosstabContext.prototype.getClassVariables = function () {
        return this.classVariables;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getClassVariableIds = function () {
        return _.map(this.classVariables, function (cv) {
            return cv.getId();
        });
    };


    sas.analytics.riskhp.models.CrosstabContext.prototype.getDrilledClassVariables = function () {
        var dcvs = [];
        for (var idx = this.drillpath.length; idx < this.classVariables.length; idx++) {
            dcvs.push(this.classVariables[idx]);
        }
        return dcvs;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getDrillClassVariables = function () {
        var dcvs = [];
        for (var idx = 0; idx < this.drillpath.length; idx++) {
            dcvs.push(this.classVariables[idx]);
        }
        return dcvs;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setClassVariables = function (classVariables) {
        this.classVariables = classVariables;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getClassVariable = function (idx) {
        return this.classVariables[idx];
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.addClassVariable = function (cv) {
        return this.pushClassVariable(cv);
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.pushClassVariable = function (cv) {
        // Make sure cv is a classvar and not already in the list
        if (cv && cv.type === CLASSVAR) {
            if (!_.findWhere(this.classVariables, {id: cv.id})) {
                return this.classVariables.push(cv);
            }
        }
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.insertClassVariable = function (cv) {
        // Make sure cv is a classvar and not already in the list
        if (cv && cv.type === CLASSVAR) {
            if (!_.findWhere(this.classVariables, {id: cv.id})) {
                return this.classVariables.unshift(cv);
            }
        }
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.removeClassVariable = function (searchParameter) {
        var self = this;

        if (self.classVariables.length > 1) {
            var key = Object.keys(searchParameter)[0];

            for (var cvIndex = 0; cvIndex < self.classVariables.length; cvIndex++) {

                // Actual function that removes a classvar
                var removeFunc =  function () {
                    var cvItems = self.classVariables.concat();
                    cvItems.splice(cvIndex, 1);
                    //self.clearAllExpansions();
                    //self.resetReportResultsTree();
                    var newCvIds = _.map(cvItems, function (cv) {
                        return cv.getId();
                    });
                    self.replaceClassVariables(newCvIds);
                };

                // Only look at the first property of searchKey
                if ((key === "name" && self.classVariables[cvIndex].name === searchParameter.name) ||
                    (key === "label" && self.classVariables[cvIndex].label === searchParameter.label) ||
                    (key === "id" && self.classVariables[cvIndex].id === searchParameter.id)){
                    removeFunc();
                    break;
                }
            }
        }
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.replaceClassVariables = function (newCvIds) {
        var self = this;
        var indexOfDifference = 0;

        // Get existing classvar ids
        var cvIds = self.getClassVariableIds();

        // how are they different?  There are four possible conditions: order changed, removed one or more
        // classvars, added one or more classvars, nothing changed.
        var condition;
        for (var a = 0; a < cvIds.length; a++) {
            if (a < newCvIds.length) {
                if (cvIds[a] !== newCvIds[a]) {
                    // order changed
                    condition = "orderChanged";
                    indexOfDifference = a;
                    break;
                }
            }
            // we have more existing classvars than the incoming new classvars
            else {
                condition = "lessIncomingClassvars";
                break;
            }
        }

        // one last check, do we have added classvars?
        if (condition === undefined) {
            if (cvIds.length < newCvIds.length) {
                condition = "moreIncomingClassvars";
            }
        }

        // if we get this far and condition is undefined
        // then incoming classvars are the same as existing
        // classvars.  Bail.
        if (condition !== undefined) {

            // order of classvars change?  Dump everything and start over with new classvars
            if (condition === "orderChanged") {
                var newDrill = [].concat(self.getDrillPath());

                self.classVariables = [];

                // fix results tree
                self.trimRowTree(indexOfDifference);

                _.each(newCvIds, function (element, index, list) {
                    var cv = _.findWhere(self.cubeDataModel.classVariable, {id: element});
                    if (cv !== undefined) {
                        self.classVariables.push(cv);
                    }
                });

                // fix drill
                if (newDrill.length >= indexOfDifference) {
                    var numPops = newDrill.length - indexOfDifference;
                    for (var c = 0; c < numPops; c++) {
                        newDrill.pop();
                    }
                    self.setDrillPath(newDrill);
                }
            }

            // removed one or more classvars?
            else if (condition === "lessIncomingClassvars") {

                //var numCVs = cvIds.length - newCvIds.length;

                // fix results tree
                self.trimRowTree(newCvIds.length);

                self.classVariables = [];
                _.each(newCvIds, function (element, index, list) {
                    var cv = _.findWhere(self.cubeDataModel.classVariable, {id: element});
                    if (cv !== undefined) {
                        self.classVariables.push(cv);
                    }
                });

                // fix drill
                var newDrill = [].concat(self.getDrillPath());
                if (newDrill.length >= newCvIds.length) {
                    var numPops = newDrill.length - newCvIds.length + 1;
                    for (var b = 0; b < numPops; b++) {
                        newDrill.pop();
                    }
                    self.setDrillPath(newDrill);
                }
            }

            // added one or more classvars?  Just add them to the classvar list
            else if (condition === "moreIncomingClassvars") {
                var startIndex = cvIds.length;
                for (var b = startIndex; b < newCvIds.length; b++) {
                    var addedCv = _.findWhere(self.cubeDataModel.classVariable, {id: newCvIds[b]});
                    if (addedCv !== undefined) {
                        self.classVariables.push(addedCv);
                    }
                }
            }
            else {
                // shouldn't get here
            }
        }
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getClassVariableLabels = function (ctrl) {
        if (ctrl) {
            var displayedCVs = this.classVariables.concat();
            displayedCVs.splice(0, this.drillpath.length); // Remove drilled CVs
            var displayedCVsLength = displayedCVs.length;
            var numClassVariablesDisplayed = this.getExpansionDepth();

            if (ctrl === "drilledAndDisplayed") {
                for (var i = 0; i < (displayedCVsLength - numClassVariablesDisplayed); i++) {
                    displayedCVs.pop(); // Remove non-displayed CVs
                }
                return this._getDataItemLabels(displayedCVs);
            } else if (ctrl === "drilled") {
                return this._getDataItemLabels(displayedCVs);
            }
        }
        return this._getDataItemLabels(this.classVariables);
    };


//*****************************
// Columns  - Array of dataitems.  ex [horz,ov3,stat9] where each is a dataitem
//*****************************

    sas.analytics.riskhp.models.CrosstabContext.prototype.getColumnDecorations = function(){

        var currentIndex = -1;
        var columnDecorations = [];

        var _recurseTree = function(decorations, node) {
            if (node) {
                if (node.children.length === 0) {
                    currentIndex++;
                    var decorationType = {};
                    if (node.sort !== undefined) {
                        decorationType.sort = node.sort.direction;
                    }
                    if (node.width !== undefined  && node.width !== HprConstants.DEFAULT_CROSSTAB_WIDTH) {
                        decorationType.width = node.width;
                    }
                    if (node.item.name === sas.analytics.riskhp.models.dataitems.Statistic.STALENESS) {
                        decorationType.staleness =  true;
                    }
                    if (Object.keys(decorationType).length !== 0) {
                        decorations.push({index: currentIndex, type: decorationType});
                    }
                } else {
                    for (var i = 0; i < node.children.length; i++) {
                        _recurseTree(decorations, node.children[i]);
                    }
                }
            }
            return false;
        };

        if (this.columnTree.children.length > 0) {
            for (var i = 0; i <this.columnTree.children.length; i++) {
                _recurseTree(columnDecorations, this.columnTree.children[i]);
            }
        }

        return columnDecorations;
    };



    sas.analytics.riskhp.models.CrosstabContext.prototype.getMaxColumnDepth = function () {
        var self = this;
        var depth = 0;

        var _walkColumnsForDepth = function (nDepth, node) {
            if (node) {
                depth = Math.max(nDepth + 1, depth);
                for (var idx = 0; idx < node.children.length; idx++) {
                    _walkColumnsForDepth(nDepth + 1, node.children[idx]);
                }
            }
        };

        if (self.columnTree.children.length > 0) {
            for (var idx = 0; idx < self.columnTree.children.length; idx++) {
                _walkColumnsForDepth(0, self.columnTree.children[idx]);
            }
        }

        return depth;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getColumn = function (idx, withHeaders) {
        if (withHeaders === undefined) {
            //withHeaders = false;
            withHeaders = true;
        }
        var columnNodes = this.getColumnNodes(idx);

        // Convert array of column nodes to array of data items
        var column = [];
        for (var i = 0; i < columnNodes.length; i++) {

            var dataItem = columnNodes[i].item;

            // Remove header nodes.  (ie computed, benchmark, etc)
            if ((!CrosstabUtil.isHeaderDataItem(dataItem.type)) || (withHeaders)) {
                column.push(dataItem);
            }
        }
        return column;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getColumnNodes = function (desiredColumnIndex) {

        var currentIndex = -1;
        var columnNodePath = [];

        var _findColumn = function (path, node) {
            if (node) {
                if (node.children.length === 0) {
                    currentIndex++;
                    if (currentIndex === desiredColumnIndex) {
                        columnNodePath = path.concat([node]);
                        return true;
                    }
                } else {
                    for (var i = 0; i < node.children.length; i++) {
                        if (_findColumn(path.concat([node]), node.children[i])) {
                            return true;
                        }
                    }
                }
            }
            return false;
        };

        if (this.columnTree.children.length > 0) {
            for (var i = 0; i < this.columnTree.children.length; i++) {
                if (_findColumn([], this.columnTree.children[i])) {
                    break; // get out
                }
            }
        }

        return columnNodePath;
    };


    sas.analytics.riskhp.models.CrosstabContext.prototype.removeColumn = function (desiredColumnIndex) {
        var self = this;
        var currentIndex = -1;
        var columnNodePath = [];

        var _findColumn = function (path, node) {
            if (node) {
                if (node.children.length === 0) {
                    currentIndex++;
                    if (currentIndex === desiredColumnIndex) {
                        return true;
                    }
                } else {
                    for (var i = 0; i < node.children.length; i++) {
                        if (_findColumn(path.concat([node]), node.children[i])) {
                            node.children.splice(i, 1);
                            if (node.children.length === 0) {
                                // if no other children then we get rid of parent as well
                                return true;
                            } else {
                                // there are other children, so just bail
                                break;
                            }
                        } else {
                            if (currentIndex === desiredColumnIndex) {
                                break;
                            }
                        }
                    }
                }
            }
            return false;
        };

        if (this.columnTree.children.length > 0) {
            for (var i = 0; i < this.columnTree.children.length; i++) {
                if (_findColumn([], this.columnTree.children[i])) {
                    this.columnTree.children.splice(i, 1);
                    break; // get out
                } else {
                    if (currentIndex === desiredColumnIndex) {
                        break;
                    }
                }
            }
        }

        return columnNodePath;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.validTuple = function (tuple) {
        var self = this;
        // Reconcile column against the heirarchy and rearrange if necessary
        var reconciledTuple = self.reconcileColumnAgainstHierarchy(tuple);
        if (reconciledTuple) {
            // Do the dataItems together make a valid column?  Restrictions check here.
            var isCompatible = self.cubeDataModel.areDataItemsCompatible(reconciledTuple);
            return isCompatible;
        }
        return false;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.createValidTuplesForDataItem = function (dataItem, info) {
        var self = this;
        var potentialTuples = self.createTuplesForDataItem(dataItem, info);
        var validPotentialTuples = _.some(potentialTuples, function (tuple) {
            return self.validTuple(tuple);
        });

        return validPotentialTuples;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.createTuplesForDataItem = function (dataItem, info) {
        var self = this;
        var rc = true;
        var newTuples = [];

        // Can't add classvars to column
        if ((dataItem.type === CLASSVAR) || (dataItem.type === HIERARCHY)) {
            return;
        }

        // Special case dataItems
        else if ((dataItem.type === COMPVAR) || (dataItem.type === BENCHVAR)) {
            // addColumn will find or create special header for these.
            newTuples.push([dataItem]);
        }

        // Else we have a data item that will require more processing, ie need column dataitems to create a new (and proper)
        // column/tuple
        else if (info) {
            newTuples = self.createNewTuplesBasedOnInfoObject(dataItem, info);
        }

        if (newTuples.length === 0) {
            // Create tuples from the root of the columnTree.  The effect here is to put the dataItem
            // everywhere possible in the existing crosstab.
            newTuples = self.createNewTuplesFromRoot(dataItem);

            // Still not able to create tuples?  Make new one based on restrictions.
            if (newTuples.length === 0) {
                // Call CagleFunction2000()
                var validColumns = self.cubeDataModel.findValidColumnsForDataItem(dataItem, -1, true, false, self);
                if (validColumns && validColumns.length > 0) {
                    newTuples = validColumns;
                }
            }
        }

        return newTuples;
    };

    // Based on the info given, get the node in the column header tree for "context".  This "context" will be
    // used to create similar columns using the dataItem.
    sas.analytics.riskhp.models.CrosstabContext.prototype.createNewTuplesBasedOnInfoObject = function (dataItem, info) {
        var self = this;
        var rc = true;
        var newTuples = [];

        var targetColumnDataItems = [];
        var keys = [];
        for (var i = 0; i < info.keys.length; i++) {
            //Skip over headers
            if (!CrosstabUtil.isHeaderDataItem(info.keys[i])) {
                var infoKeyDataItem = self.cubeDataModel.getDataItemById(info.keys[i]);
                targetColumnDataItems.push(infoKeyDataItem);
                keys.push(info.keys[i]);
            }
        }

        // Convert info.keys to dataItems to make a column.
        var startNode = self.findColumnTreeNode(keys);

        if (startNode) {

            // Get column tuples based on the "context"
            var tuples = self.getTuples(startNode, targetColumnDataItems);
            var convertedTuples = self.convertTuplesToUseDataItem(tuples, dataItem);

            // Validate against the restrictions
            newTuples = _.filter(convertedTuples, function (tuple) {
                return self.cubeDataModel.areDataItemsCompatible(tuple);
            });

        }
        return newTuples;
    };

    // Based on the root of the column tree, create similar columns using the dataItem.
    sas.analytics.riskhp.models.CrosstabContext.prototype.createNewTuplesFromRoot = function (dataItem) {
        var self = this;
        var rc = true;
        var newTuples = [];

        // Get column tuples based from the root of column tree
        var tuples = self.getTuples();
        newTuples = self.convertTuplesToUseDataItem(tuples, dataItem);

        // Validate against the restrictions
        var validTuples = _.filter(newTuples, function (tuple) {
            return self.cubeDataModel.areDataItemsCompatible(tuple);
        });

        return validTuples;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.convertTuplesToUseDataItem = function (tuples, dataItem) {
        var self = this;
        var newTuples = [];

        var __replaceRiskCubeFunction = function(col, index) {
            var riskCubeDataItem = CrosstabUtil.getDataItemById(dataItem.riskCubeId, RISKCUBE, self.cubeDataModel);
            if (riskCubeDataItem) {
                if (index !== -1) {
                    col[index] = riskCubeDataItem;
                }
                else {
                    col.push(riskCubeDataItem);
                }
            }
        };

        if (tuples.length > 0) {

            // If we are dealing with a row stat, then pull out the riskcube (if available) and
            // create a new single column with it and the row stat. If it is not available, create
            // a column with just the Row Statistic
            if (self.cubeDataModel.isRowStatistic(dataItem)) {
                for (var a = 0; a < tuples.length; a++) {
                    var newRowStatColumn = [];

                    var riskCubeObject = self.getDataItemInfoFromTuple(tuples[a], RISKCUBE);
                    if (riskCubeObject) {
                        newRowStatColumn.push(riskCubeObject.dataItem);
                    }
                    newRowStatColumn.push(dataItem);
                    newTuples.push(newRowStatColumn);
                }
            }

            // Handle resultSet
            else if (dataItem.type === RESULTSET) {

                var rsColumns = [];

                for (var columnIndex = 0; columnIndex < tuples.length; columnIndex++) {
                    var col = tuples[columnIndex];
                    var riskCubeObject = self.getDataItemInfoFromTuple(col, RISKCUBE);
                    var resultSetDataItemObject = self.getDataItemInfoFromTuple(col, RESULTSET);
                    var scenarioDataItemObject = self.getDataItemInfoFromTuple(col, SCENARIO);
                    var horizonDataItemObject = self.getAnyHorizonInfo(col);

                    // S1427431 - If the resultset has a basecase horizon then display basecase horizon; This code
                    // was added back to undo an old defect (S1407030) where the BaseCase Horizon was not shown
                    if (dataItem.horizon !== undefined) {
                        var newRsColumn = col.concat();

                        // replace the horizon
                        if (horizonDataItemObject) {
                            newRsColumn[horizonDataItemObject.index] = dataItem.horizon;
                        }
                        else {
                            newRsColumn.unshift(dataItem.horizon);
                        }

                        // add or replace the result set
                        if (resultSetDataItemObject) {
                            newRsColumn[resultSetDataItemObject.index] = dataItem;
                        }
                        else {
                            newRsColumn.unshift(dataItem);
                        }
                        // Add or replace riskcube
                        __replaceRiskCubeFunction(newRsColumn, riskCubeObject ? riskCubeObject.index : -1);

                        // remove the scenario (always do this last to avoid screwing up the indexes)
                        if (scenarioDataItemObject) {
                            newRsColumn.splice(scenarioDataItemObject.index, 1);
                        }

                        rsColumns.push(newRsColumn);
                    }

                    for (var scenIndex = 0; scenIndex < dataItem.scenarios.length; scenIndex++) {
                        var scenarioDataItem = dataItem.scenarios[scenIndex];
                        if (scenarioDataItem.isActivated()) {
                            for (var horizonIndex = 0; horizonIndex < scenarioDataItem.horizons.length; horizonIndex++) {
                                // Create a new column from base column
                                var newRsColumn = col.concat();
                                // Replace horizon or add to new column
                                if (horizonDataItemObject === undefined) {
                                    newRsColumn.push(scenarioDataItem.horizons[horizonIndex]);
                                } else {
                                    newRsColumn[horizonDataItemObject.index] = scenarioDataItem.horizons[horizonIndex];
                                }
                                // Add or replace scenario
                                if (scenarioDataItemObject === undefined) {
                                    newRsColumn.push(scenarioDataItem);
                                }
                                else {
                                    newRsColumn[scenarioDataItemObject.index] = scenarioDataItem;
                                }
                                // Add or replace resultset
                                if (resultSetDataItemObject === undefined) {
                                    newRsColumn.push(dataItem);
                                }
                                else {
                                    newRsColumn[resultSetDataItemObject.index] = dataItem;
                                }
                                // Add or replace riskcube
                                __replaceRiskCubeFunction(newRsColumn, riskCubeObject ? riskCubeObject.index : -1);
                                rsColumns.push(newRsColumn);
                            }
                        }
                        else {
                            // scenario not activated
                            sas.log.warning("Scenario " + dataItem.getLabel() + " can not be displayed because it has not been activated");
                            sas.log.debug("Scenario " + dataItem.getLabel() + " is " + dataItem.getId() + " NOT ACTIVE");
                        }
                    }
                }

                //Add our new columns to the crosstab.  Invalid columns will be rejected by addColumn();
                _.each(rsColumns, function (col, index) {
                    newTuples.push(col);
                });
            }

            else if (dataItem.type === SCENARIO) {
                if (dataItem.isActivated()) {
                    var resultSetDataItem = CrosstabUtil.getDataItemById(dataItem.resultSetId, RESULTSET, self.cubeDataModel);
                    var riskCubeDataItem = undefined;
                    if (resultSetDataItem)
                        riskCubeDataItem = CrosstabUtil.getDataItemById(resultSetDataItem.riskCubeId, RISKCUBE, self.cubeDataModel);
                    var scenColumns = [];

                    for (var columnIndex = 0; columnIndex < tuples.length; columnIndex++) {

                        var col = tuples[columnIndex];
                        var outvarDataItemObject = self.getDataItemInfoFromTuple(col, OUTVAR);
                        var statDataItemObject = self.getDataItemInfoFromTuple(col, STATISTIC);
                        var riskCubeObject = self.getDataItemInfoFromTuple(col, RISKCUBE);

                        for (var horizonIndex = 0; horizonIndex < dataItem.horizons.length; horizonIndex++) {

                            var scenHorizon = dataItem.horizons[horizonIndex];

                            // Check that this scenario horizon supports the crosstab display type option.
                            if (self._canDisplayHorizon(scenHorizon)) {
                                // Create a new column from base column
                                var newRsColumn = [];
                                if (riskCubeDataItem) {
                                    newRsColumn.push(riskCubeDataItem);
                                } else {
                                    newRsColumn.push(riskCubeObject.dataItem);
                                }
                                if (resultSetDataItem) {
                                    newRsColumn.push(resultSetDataItem);  //resultSet
                                }
                                newRsColumn.push(dataItem);  //Scenario
                                newRsColumn.push(scenHorizon);
                                if (outvarDataItemObject) {
                                    newRsColumn.push(outvarDataItemObject.dataItem);
                                }
                                if (statDataItemObject) {
                                    newRsColumn.push(statDataItemObject.dataItem);
                                }

                                scenColumns.push(newRsColumn);
                            }
                            else {
                                // Bad horizon
                                sas.log.error("Horizon " + scenHorizon.getLabel() + " can not be displayed per crosstab option: " + this.optionDisplayHorizons.value);
                                sas.log.debug("Horizon " + scenHorizon.getLabel() + " is " + scenHorizon.getId());
                            }
                        }
                    }

                    //Add our new columns to the crosstab.  Invalid columns will be rejected by addColumn();
                    _.each(scenColumns, function (col, index) {
                        newTuples.push(col);
                    });
                }
                else {
                    // scenario not activated
                    sas.log.warning("Scenario " + dataItem.getLabel() + " can not be displayed because it has not been activated");
                    sas.log.debug("Scenario " + dataItem.getLabel() + " is " + dataItem.getId() + " NOT ACTIVE");
                }
            }

            // Handle riskCube
            else if (dataItem.type === RISKCUBE) {

                var rcColumns = [];

                // Clone each tuple and swap dataItems as nesessary.
                for (var columnIndex = 0; columnIndex < tuples.length; columnIndex++) {
                    // Create a new column from base column
                    var newRcColumn = tuples[columnIndex].concat();

                    //FIXME no support for riskcube in stess yet as there is only one.  This code will have to
                    // change somewhat when stress supports multiple risk cubes.

                    // Replace riskcube in tuple with the new one or if one does not exist, use the one we are working with
                    var riskCubeDataItemObject = self.getDataItemInfoFromTuple(newRcColumn, RISKCUBE);
                    if (riskCubeDataItemObject) {
                        newRcColumn[riskCubeDataItemObject.index] = dataItem;
                    } else {
                        newRcColumn[0] = dataItem;
                    }

                    // If present, fix the horizon to one compatible with new riskcube
                    var horizonDataItemObject = self.getAnyHorizonInfo(newRcColumn);
                    if (horizonDataItemObject) {
                        var newHorizon;
                        if (horizonDataItemObject.dataItem.isBasecase()) {
                            newHorizon = dataItem.getBasecaseHorizon();
                        }
                        else {
                            var searchKey = {
                                horizonNumber: horizonDataItemObject.dataItem.horizonNumber,
                                interval: horizonDataItemObject.dataItem.interval
                            };
                            newHorizon = _.findWhere(dataItem.horizons, searchKey);
                            if (newHorizon === undefined) {
                                // Last ditch effort to find a compatible horizon
                                searchKey = {horizonDate: horizonDataItemObject.dataItem.horizonDate};
                                newHorizon = _.findWhere(dataItem.horizons, searchKey);
                            }
                            if (newHorizon === undefined) {
                                continue;  // Still no horizon, get OUT.
                            }
                        }
                        newRcColumn[horizonDataItemObject.index] = newHorizon;
                    }

                    rcColumns.push(newRcColumn);
                }

                //Add our new columns to the crosstab.  Invalid columns will be rejected by addColumn();
                _.each(rcColumns, function (col, index) {
                    newTuples.push(col);
                });
            }

            // Handle everything else
            else {
                //replace corresponding dataitem in columns with our dataItem
                var newColumns = _.map(tuples, function (tuple, index) {
                    if (self.isAnyHorizonType(dataItem)) {
                        var horizonDataItemObject = self.getAnyHorizonInfo(tuple);
                        if (horizonDataItemObject !== undefined) {
                            tuple[horizonDataItemObject.index] = dataItem;

                            // If the horizon is not part of a scenario, then if the tuple has scenario and result
                            // set then remove them.
                            if (self.cubeDataModel.isStressCube()) {
                                if (dataItem.scenarioId === undefined) {
                                    // remove the scenario from the tuple
                                    var scenDataItemObject = self.getDataItemInfoFromTuple(tuple, SCENARIO);
                                    if (scenDataItemObject !== undefined) {
                                        tuple.splice(scenDataItemObject.index, 1);
                                    }

                                    // replace the result set with the one referenced by the horizon
                                    var horizonResultSet = self.cubeDataModel.getDataItemById(dataItem.resultSetId);
                                    var resultSetTupleObject = self.getDataItemInfoFromTuple(tuple, sas.analytics.riskhp.models.dataitems.ResultSet.RESULTSET);
                                    if (horizonResultSet && resultSetTupleObject) {
                                        tuple.splice(resultSetTupleObject.index, 1, horizonResultSet);
                                    }
                                }
                                else {
                                    // replace the scenario with the one referenced by the horizon
                                    var horizonScenario = self.cubeDataModel.getDataItemById(dataItem.scenarioId);
                                    var scenarioTupleObject = self.getDataItemInfoFromTuple(tuple, sas.analytics.riskhp.models.dataitems.Scenario.SCENARIO);
                                    if (horizonScenario && scenarioTupleObject) {
                                        tuple.splice(scenarioTupleObject.index, 1, horizonScenario);
                                    }

                                    // replace the result set with the one referenced by the scenario
                                    var scenarioResultSet = self.cubeDataModel.getDataItemById(horizonScenario.resultSetId);
                                    var resultSetTupleObject = self.getDataItemInfoFromTuple(tuple, sas.analytics.riskhp.models.dataitems.ResultSet.RESULTSET);
                                    if (scenarioResultSet && resultSetTupleObject) {
                                        tuple.splice(resultSetTupleObject.index, 1, scenarioResultSet);
                                    }
                                }
                            }
                            // Horizon in price cube
                            else {

                            }
                        }
                        else {
                            tuple = [];
                        }
                    }
                    // non-horizon dataItem
                    else {
                        var dataItemObject = self.getDataItemInfoFromTuple(tuple, dataItem.getType());
                        if (dataItemObject !== undefined) {
                            tuple[dataItemObject.index] = dataItem;
                        }
                        else {
                            tuple = [];
                        }
                    }
                    self.forceHorizonToBaseHorizon(tuple);// Set horizon to original base horizon

                    // if basecase is indicated then each tuple will contain a basecase horizon.
                    // we need to fix the outvar and stat, use cagleFunction2000() to do that.
                    if (dataItem.basecase && dataItem.basecase === true) {
                        var basecaseObject = self.getDataItemInfoFromTuple(tuple, HORIZON);
                        if (basecaseObject) {
                            var validColumns = self.cubeDataModel.findValidColumnsForDataItem(basecaseObject.dataItem, 1, false, false, self); // Can return no valid columns and empty array
                            if (validColumns.length > 0) {
                                tuple = validColumns[0];
                            }
                        }
                        else {
                            tuple = [];
                        }
                    }
                    return tuple;
                });

                //Add our new columns to the crosstab.  Invalid columns will be rejected by addColumn();
                _.each(newColumns, function (tuple, index) {
                    if (tuple.length > 0) {
                        newTuples.push(tuple);
                    }
                });
            }

        } // if (columns.length > 0)
        return newTuples;
    };

    /**
     * This function, given a single dataItem and possibly an info object, will attempt to
     * place it in the crosstab.
     * @param dataItem
     * @param info
     * @returns success
     */
    sas.analytics.riskhp.models.CrosstabContext.prototype.addDataItemToColumns = function (dataItem, info) {
        var self = this;
        var numColumnsGenerated = 0;

        // S1392674
        // check the horizon intervals with respect to crosstab horizon display type
        var horizonDisplayTypeChanged = false;
        if (self.optionDisplayHorizons.value === HprConstants.DISPLAY_HORIZONS_RELATIVE_VALUE) {
            var __checkScenarioHorizons = function(scenario) {
                var notRelativeHorizons = _.filter(scenario.horizons, function (hz) {
                    return hz.interval === undefined;
                });
                if (scenario.horizons.length === notRelativeHorizons.length) {
                    // All the horizons are absolute, so set the crosstab to absolute mode.
                    self.optionDisplayHorizons.value = HprConstants.DISPLAY_HORIZONS_ABSOLUTE_VALUE;
                    horizonDisplayTypeChanged = true;
                }
            };

            // if a result set or scenario, examine the scenario horizons to see if the display type should change
            if (dataItem && dataItem.type === RESULTSET) {
                if (dataItem.scenarios && dataItem.scenarios.length > 0) {
                    for (var i = 0; i < dataItem.scenarios.length; i++) {
                        __checkScenarioHorizons(dataItem.scenarios[i]);

                        // if the display type changed, we can stop processing
                        if (horizonDisplayTypeChanged) {
                            break;
                        }
                    }
                }
            }
            else if (dataItem && dataItem.type === SCENARIO) {
                __checkScenarioHorizons(dataItem);
            }
        }

        var tuples = self.createTuplesForDataItem(dataItem, info);

        if (tuples && tuples.length > 0) {
            _.each(tuples, function (col, index) {
                numColumnsGenerated += (self.addColumn(col) ? 1 : 0);
            });
        }
        else {
            // Something bad with the dataItems.
            sas.log.error("addDataItemToColumns  No columns generated");
            numColumnsGenerated = 0;
        }

        if (horizonDisplayTypeChanged === true) {
            // update the options pane
            sas.eventBus.publish(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_OPTIONS_PANE_CHANGED, {
                optionsType: HprConstants.HPR_OPTIONS_PANE_CROSSTAB_TYPE,
                crosstabId: self.getId(),
                updateOptionsModel: true
            });

            // reconstruct the columns to match the new display type
            self.updateColumnTree();
        }

        return numColumnsGenerated;
    };

    /**
     * This function, given an array of dataItem and possibly a sort object, will attempt to
     * place it in the crosstab.
     * @param {array} tuple Array of dataItems
     * @param {object} sort
     * @param {object} width
     * @param {boolean} forceColumnOrder Whether to enforce column ordering, ie basecase horizons first/leftmost.
     */
    sas.analytics.riskhp.models.CrosstabContext.prototype.addColumn = function (tuple, sort, width, forceColumnOrder) {
        var self = this;
        var success = false;

        // Assume columns should be place in crosstab based on preference order, ie basecase horizons are first/leftmost,
        // and not in insertion order.  (A simple redraw should not force column order and just honor insertion order.)
        if (forceColumnOrder === undefined) {
            forceColumnOrder = true;
        }

        var column = self.convertTupleToColumn(tuple);

        if (column && column.length > 0) {

            // Ok, add nodes to internal node tree for the column headers.
            var node = self.columnTree;

            // Process each dataItem making up the reconciled column tuple.
            // Find nodes or create new nodes as necessary to column tree.
            for (var i = 0; i < column.length; i++) {

                var childNode = undefined;
                switch (column[i].type) {
                    case RELATIVEHORIZON:
                    case ABSOLUTEHORIZON:
                        childNode = self._findOrCreateChildNode(node, column[i], forceColumnOrder);
                        break;
                    default:
                        childNode = self._findOrCreateChildNode(node, column[i], forceColumnOrder);
                        break;
                }
                node = childNode;
            }

            // Set the sort on the node
            if (sort) {
                node.sort = sort;
            }
            // Set the width on the node
            if (width) {
                node.width = width;
            }
            success = true;
        }
        return success;
    };

    // The function will take a dataItemArray (column or tuple) and force the horizon to be it's base (ie not
    // relative or absolute).
    sas.analytics.riskhp.models.CrosstabContext.prototype.forceHorizonToBaseHorizon = function (dataItemArray) {

        var proxyHorizonObject = this.getAnyHorizonInfo(dataItemArray);
        if (proxyHorizonObject !== undefined) {
            var horizon = CrosstabUtil.findBaseHorizon(dataItemArray, proxyHorizonObject.index, this.cubeDataModel);
            if (horizon !== undefined) {
                dataItemArray[proxyHorizonObject.index] = horizon;
            }
            else {
                sas.log.error("forceHorizonToBaseHorizon  Unable to determine base horizon");
            }
        }
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype._createChildNode = function (node, dataItem) {
        return {
            name: dataItem.id,
            item: dataItem,
            parent: node,
            children: []
        };
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype._findChildNode = function (node, dataItem) {
        return _.find(node.children, function (cNode) {
            if ((cNode.item.type === dataItem.type) &&
                (cNode.item.id === dataItem.id)) {
                if (cNode.item.basecase !== undefined) {
                    if (dataItem.basecase !== undefined  && cNode.item.basecase === dataItem.basecase) {
                        return cNode;
                    }
                } else {
                    return cNode;
                }
            }
        });
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype._findOrCreateChildNode = function (node, dataItem, forceColumnOrder) {
        var self = this;

        var childNode = this._findChildNode(node, dataItem);
        if (childNode === undefined) {
            childNode = this._createChildNode(node, dataItem);

            //Add child into the tree
            if (forceColumnOrder === true) {
                if (dataItem.type === CrosstabUtil.COMPVARHEADER) {
                    // Make compvar header first
                    node.children.unshift(childNode);
                }
                else if (dataItem.type === CrosstabUtil.ROWSTATHEADER) {
                    //if (node.item.id === this.COLUMNROOT &&
                    if (node.children.length > 0 &&
                        node.children[0].item.id === CrosstabUtil.COMPVARHEADER) {
                        // If we have compvars then insert after
                        node.children.splice(1, 0, childNode);
                    }
                    else {
                        // Make row stat header first
                        node.children.unshift(childNode);
                    }
                } else {
                    //If basecase horizon, then put it after rowstat and compvar headers.
                    if (self.isAnyHorizonType(dataItem) && dataItem.basecase) {
                        //if (node.item.id === this.COLUMNROOT &&
                        if (node.children.length > 0) {
                            var i = 0;
                            while (i < node.children.length && (node.children[i].item.id === CrosstabUtil.COMPVARHEADER ||
                            node.children[i].item.id === CrosstabUtil.ROWSTATHEADER)) {
                                i++;
                            }
                            // If we have compvar or rowstat headers then insert after
                            node.children.splice(i, 0, childNode);
                        }
                        else node.children.unshift(childNode);
                    }
                    else {
                        node.children.push(childNode);
                    }

                }
            }
            else {
                // Just add to child array, ie right most.
                node.children.push(childNode);
            }
        }
        return childNode;
    };

    // Can we display this horizon based on the crosstab's Horizon display type: Absolute or Relative
    sas.analytics.riskhp.models.CrosstabContext.prototype._canDisplayHorizon = function (horizon) {

        // Basecase
        // if (horizon.isBasecase()) {
        //     return true;
        // }
        //else
        // Option says absolute
        if (this.optionDisplayHorizons.value === HprConstants.DISPLAY_HORIZONS_ABSOLUTE_VALUE) {
            var absHorizon = this.cubeDataModel.findAbsoluteHorizon(horizon.horizonDate, horizon.label, horizon.basecase);
            if (absHorizon !== undefined) {
                return true;
            }
        }
        // Option says relative
        else {
            var relHorizon = this.cubeDataModel.findRelativeHorizon(horizon.getInterval(), horizon.getHorizonNumber(), horizon.basecase);
            if (relHorizon !== undefined) {
                return true;
            }
        }
        return false;
    };

// Returns real dataItems

    sas.analytics.riskhp.models.CrosstabContext.prototype.getTuple = function (columnIndex) {
        var self = this;
        return self.getTuples()[columnIndex];
    };

    /**
     * This method combines all the tuple arrays into one big array.  All Data Items should be unique and not
     * repeated.  This is a good way to see all the Data Items a Crosstab contains in the tuples at a given moment.
     */
    sas.analytics.riskhp.models.CrosstabContext.prototype.getTuplesAsOneArray = function () {
        var self = this;
        // Use apply so we can loop over all the 'tuples' because it is an Array of Arrays;
        // This is because "_.union" method needs to know all the arrays to work properly
        return _.union.apply(null, self.getTuples());
    };


    // Hidden tuples are tuples that are required by active display rules but are not in the crosstab,  These tuples
    // must be added to the crosstab in a hidden state.
    sas.analytics.riskhp.models.CrosstabContext.prototype.getHiddenTuples = function() {
        var self = this;
        var displayRules = DisplayRulesManager.getDisplayRulesForCrosstab(self.crosstab.sId);
        var hiddenTuples = [];

        _.each(displayRules, function(crosstabDisplayRule) {
            if (crosstabDisplayRule.active) {
                var displayRule = DisplayRulesManager.getDisplayRuleById(crosstabDisplayRule.ruleId);

                // skip if appliesTo col is not in crosstab or rule is not valid
                var isValid = DisplayRulesManager.isDisplayRuleValidForDataSource(self.dataSourceId, displayRule.id);
                if (isValid &&
                    !(displayRule.appliesTo.type === sas.analytics.riskhp.models.DisplayRule.APPLIES_TO_TYPE_COL && self.getColumnIndexFromTuple(displayRule.appliesTo.column) === -1)) {

                    if (self.getColumnIndexFromTuple(displayRule.referenceColumn) === -1) {
                        hiddenTuples.push(displayRule.referenceColumn);
                    }

                    if (DisplayRulesUtil.requiresDataItem1(displayRule.operator) && displayRule.dataItem1.type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_COL) {
                        if (self.getColumnIndexFromTuple(displayRule.dataItem1.column) === -1) {
                            hiddenTuples.push(displayRule.dataItem1.column);
                        }
                    }

                    if (DisplayRulesUtil.requiresDataItem2(displayRule.operator) && displayRule.dataItem2.type === sas.analytics.riskhp.models.DisplayRule.DATA_ITEM_TYPE_COL) {
                        if (self.getColumnIndexFromTuple(displayRule.dataItem2.column) === -1) {
                            hiddenTuples.push(displayRule.dataItem2.column);
                        }
                    }
                }
            }
        });
        return hiddenTuples;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getTuples = function (startNode, startPath) {
        var self = this;
        var tuples = [];
        var tupleObjects = self.getTupleObjects(startNode, startPath);
        _.each(tupleObjects, function (tupleObject) {
            tuples.push(tupleObject.tuple);
        });

        return tuples;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getTupleObjects = function (startNode, startPath, dehydrated) {
        var self = this;
        var tupleObjects = [];

        if(dehydrated === undefined) {
            dehydrated = false;
        };

        var columnObjects = this.getColumnObjects(startNode, startPath, false);
        _.each(columnObjects, function (columnObject) {
            self.forceHorizonToBaseHorizon(columnObject.column); // replace abs/rel horizon with real horizon

            //FORCE a risk cube into the tuple if one is not found and this is not a compvar or benchvar tuple.
            var compvar = self.getDataItemInfoFromTuple(columnObject.column, sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR);
            var benchvar = self.getDataItemInfoFromTuple(columnObject.column, sas.analytics.riskhp.models.dataitems.Benchvar.BENCHVAR);
            if (!compvar && !benchvar) {
                var riskCubeId = undefined;
                var riskcubeDataItemObject = self.getDataItemInfoFromTuple(columnObject.column,
                    sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE);
                if (riskcubeDataItemObject === undefined) {
                    var resultSetDataItemObject = self.getDataItemInfoFromTuple(columnObject.column,
                        sas.analytics.riskhp.models.dataitems.ResultSet.RESULTSET);
                    if (resultSetDataItemObject === undefined) {
                        var horizonDataItemObject = self.getDataItemInfoFromTuple(columnObject.column,
                            sas.analytics.riskhp.models.dataitems.Horizon.HORIZON);
                        if (horizonDataItemObject === undefined) {
                            // Something is hosed
                            riskCubeId = undefined;  // ummm... fixing below
                        }
                        else {
                            riskCubeId = horizonDataItemObject.dataItem.riskCubeId;
                        }
                    }
                    else {
                        riskCubeId = resultSetDataItemObject.dataItem.riskCubeId;
                    }
                    // Something is hosed, default to the first riskcube object in the datasource
                    if (riskCubeId === undefined) {
                        riskCubeId = self.cubeDataModel.riskCube[0].id;
                    }
                    var riskCube = CrosstabUtil.getDataItemById(riskCubeId,
                        sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE,
                        self.cubeDataModel);
                    columnObject.column.push(riskCube);
                }
            }
            var tuple = undefined;
            if (dehydrated) {
                tuple = self._dehydrateDataItems(columnObject.column);
            }
            else {
                tuple = columnObject.column
            };
            tupleObjects.push({tuple: tuple, sort: columnObject.sort, width: columnObject.width});
        });

        return tupleObjects;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getColumnObjects = function (startNode, startPath, withHeaders) {
        var self = this;

        if (withHeaders === undefined) {
            //withHeaders = false;
            withHeaders = true;
        }
        var columnObjects = [];

        // Flatten tree to produce individual columns
        var recurseDeep = function (node, path) {
            if (node) {
                if (node.children.length > 0) {
                    _.each(node.children, function (child) {
                        var newPath = [];
                        $.extend(true, newPath, path);
                        // Remove header nodes.  (ie computed and benchmark variables)
                        if (child) {
                            if (!CrosstabUtil.isHeaderDataItem(child.item.type) || withHeaders) {
                                newPath.push(child.item);
                            }
                            recurseDeep(child, newPath);
                        }
                        else {
                            var bork = 0;
                        }
                    });
                } else {
                    // Catch the case where there are no columns.  node is pointing to the "columnRoot" and no
                    // children.  We should not create a column object.
                    if (node.name !== "columnRoot") {
                        var col = [];
                        $.extend(true, col, path); // Cant modify path
                        var newColumnObject = {column: col};
                        if (node.sort) {
                            newColumnObject.sort = node.sort;
                        }
                        if (node.width) {
                            newColumnObject.width = node.width;
                        }
                        else {
                            newColumnObject.width = HprConstants.DEFAULT_CROSSTAB_WIDTH;
                        }
                        columnObjects.push(newColumnObject);
                    }
                }
            }
        };

        // Set default to start at the top of the tree and not a subtree
        if (startNode === undefined) {
            startNode = this.columnTree;
        }
        if (startPath === undefined) {
            startPath = [];
        }

        recurseDeep(startNode, startPath);
        return columnObjects;
    };

    /**
     * Returns the index of the column for the tuple, or -1 if not found.
     * @param tuple
     * @returns {number}
     */
    sas.analytics.riskhp.models.CrosstabContext.prototype.getColumnIndexFromTuple = function (tuple, includeHiddenCols) {
        var self = this;
        for (var index = 0; index < self.getNumberOfColumns(); index++) {
            var columnTuple = self.getTuple(index);
            if (columnTuple && columnTuple.length === tuple.length) {
                var isEqual = _.every(columnTuple, function(colDataItem) {
                    return _.find(tuple, function(dataItem){
                        return dataItem.id === colDataItem.id;
                    })
                });
                if (isEqual) {
                    return index;
                }
            }
        }
        if (includeHiddenCols) {
            // if you get here it's not a visible column in the crosstab.  Look for hidden columns
            var hiddenTuples = self.getHiddenTuples();
            for (var hiddenIndex = 0; hiddenIndex < hiddenTuples.length; hiddenIndex++) {
                var columnTuple = hiddenTuples[hiddenIndex];
                if (columnTuple && columnTuple.length === tuple.length) {
                    var isEqual = _.every(columnTuple, function (colDataItem) {
                        return _.find(tuple, function (dataItem) {
                            return dataItem.id === colDataItem.id;
                        })
                    });
                    if (isEqual) {
                        return self.getNumberOfColumns() + hiddenIndex;
                    }
                }
            }
        }
        return -1;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getLeafIndexesFromNode = function (node) {

        var preLeafCount = 0;
        var leafCount = 0;

        var countToNode = function (obj) {
            var found = false;
            if (obj) {
                if (obj !== node) {
                    if (obj.children.length > 0) {
                        for(var x = 0; x < obj.children.length; x++) {
                            var found = countToNode(obj.children[x]);
                            if (found) {
                                return true;
                            }
                        }
                    } else {
                        preLeafCount++;
                    }
                } else {
                    return true;
                }
            }
            return found;
        };

        var countLeaves = function (obj) {
            if (obj) {
                if (obj.children.length > 0) {
                    _.each(obj.children, function (child) {
                        countLeaves(child);
                    });
                } else {
                    leafCount++;
                }
            }
        };

        countToNode(this.columnTree);

        countLeaves(node);

        var indexArray = [];
        var preLeafIndex = preLeafCount - 1;
        if (preLeafIndex !== -1) {
            indexArray.push(preLeafIndex);  // column prior
        }
        for(var i = 1; i <= leafCount; i++) {
            indexArray.push(preLeafIndex + i);
        }
        indexArray.push(preLeafIndex + leafCount + 1); // column after
        return indexArray;
    };

    /**
     * Find the number of columns a node contains.  If a node is not passed in, then the total number of columns for this context is determined.
     * @param node (Optional) A particular tree node on the columnTree
     * @returns {number} The number of columns a node contains or the total number of columns for the context
     */
    sas.analytics.riskhp.models.CrosstabContext.prototype.getNumberOfColumns = function (node) {

        var numberOfColumns = 0;

        var recurseDeep = function (obj) {
            if (obj) {
                if (obj.children.length > 0) {
                    _.each(obj.children, function (child) {
                        recurseDeep(child);
                    });
                } else {
                    numberOfColumns++;
                }
            }
        };

        // If a node was passed in use it to find the number of columns it has; otherwise use the whole column tree
        recurseDeep(node ? node : this.columnTree);

        return numberOfColumns;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getAllHorizons = function () {
        var self = this;
        var horizonDataItems = [];
        var tupleObjects = self.getTupleObjects();

        for (var idx = 0; idx < tupleObjects.length; idx++) {
            var horizonInfo = self.getDataItemInfoFromTuple(tupleObjects[idx].tuple, HORIZON);
            if (horizonInfo) {
                horizonDataItems.push(horizonInfo.dataItem);
            }
        }
        return horizonDataItems;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getDataItemInfoFromTuple = function (tuple, type) {

        for (var idx = 0; idx < tuple.length; idx++) {
            if (tuple[idx].getType() === type) {
                return {dataItem: tuple[idx], index: idx};
            }
        }
        return undefined;
    };


    sas.analytics.riskhp.models.CrosstabContext.prototype.fixColumnToBaseHorizon = function (column) {
        var tempColumn = $.extend(true, [], column);
        var proxyHorizonObject = this.getAnyHorizonInfo(column);
        if (proxyHorizonObject) {
            tempColumn[proxyHorizonObject.index] = CrosstabUtil.findBaseHorizon(tempColumn, proxyHorizonObject.index, this.cubeDataModel);
        }
        return tempColumn;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getBaseHorizon = function (column, dataItem) {
        var proxyHorizonObject = this.getAnyHorizonInfo(column);
        if (proxyHorizonObject) {
            // If basecase horizon, then just return it.
            if (proxyHorizonObject.dataItem.isBasecase()) {
                return proxyHorizonObject.dataItem;
            }
            var tempColumn = $.extend(true, [], column);
            if (dataItem !== undefined) {
                tempColumn[proxyHorizonObject.index] = dataItem;
            }
            return CrosstabUtil.findBaseHorizon(tempColumn, proxyHorizonObject.index, this.cubeDataModel);
        }
        return undefined;
    };

    // This function will return information about any horizon in the dataItem array
    sas.analytics.riskhp.models.CrosstabContext.prototype.getAnyHorizonInfo = function (dataItemArray) {
        var self = this;

        for (var idx = 0; idx < dataItemArray.length; idx++) {
            if (self.isAnyHorizonType(dataItemArray[idx])) {
                return {dataItem: dataItemArray[idx], index: idx, type: dataItemArray[idx].getType()};
            }
        }
        return undefined;
    };

    // This function will test if dataItem is any horizon type.
    sas.analytics.riskhp.models.CrosstabContext.prototype.isAnyHorizonType = function (dataItem) {
        var type = dataItem.getType();
        if (type === HORIZON || type === RELATIVEHORIZON || type === ABSOLUTEHORIZON) {
            return true;
        }
        return false;
    };


    sas.analytics.riskhp.models.CrosstabContext.prototype.getFormatFromColumn = function (column) {
        var self = this;

        var formatDataItem = self.getFormatDataItemFromColumn(column);
        if (formatDataItem) {
            var format = formatDataItem.format;
            var formatDecomp = format.split(".");
            return {type: formatDecomp[0], decimal: formatDecomp[1]};
        }
        return undefined;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getFormatDataItemFromColumn = function (column) {

        for (var idx = 0; idx < column.length; idx++) {
            switch (column[idx].getType()) {
                case sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR:
                case sas.analytics.riskhp.models.dataitems.Benchvar.BENCHVAR:
                case sas.analytics.riskhp.models.dataitems.Statistic.STATISTIC:
                    return column[idx];
                default:
                    break;
            }
        }
        return undefined;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getStatisticFromColumn = function (column) {

        for (var idx = 0; idx < column.length; idx++) {
            switch (column[idx].getType()) {
                case sas.analytics.riskhp.models.dataitems.Statistic.STATISTIC:
                    return {index: idx, dataItem: column[idx]};
                default:
                    break;
            }
        }
        return undefined;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getRowStatisticFromColumn = function (column) {
        var self = this;

        var statDataItemObject = this.getStatisticFromColumn(column);
        if (statDataItemObject) {
            if (self.cubeDataModel.isRowStatistic(statDataItemObject.dataItem)) {
                return statDataItemObject;
            }
        }
        return undefined;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.removeColumnsWithKey = function (key, dataItemType) {
        var self = this;
        var columnNodePath = [];

        var _removeNode = function (key, node) {
            if (key && key.length > 0) {
                if (node && node.children.length > 0) {

                    var nextNode;
                    var idx = 0;
                    for (idx = 0; idx < node.children.length; idx++) {
                        //if (key[0] == node.children[idx].name) {
                        //if (key[0] == node.children[idx].item.getLabel()) {
                        if (key[0] === node.children[idx].item.id) {
                            nextNode = node.children[idx];
                            break; // bail on for loop
                        }
                    }
                    if (nextNode) {
                        if (key.length > 1) {
                            // keys so go deeper
                            var paredKey = key.concat();
                            paredKey.splice(0, 1);
                            var rc = _removeNode(paredKey, nextNode);

                            // if there are no children anymore then remove this branch.
                            if (nextNode.children.length === 0) {
                                node.children.splice(idx, 1);
                            }
                            return rc;
                        } else {
                            // we have our node, so remove it.
                            node.children.splice(idx, 1);
                            return true;
                        }
                    }
                }
            }
            return false;
        };

        var isRemoved;
        if (this.columnTree.children.length > 0) {
            isRemoved = _removeNode(key, this.columnTree);
        }

        return isRemoved;
    };


    sas.analytics.riskhp.models.CrosstabContext.prototype.removeColumnsWithNameAndType = function (name, dataItemType) {
        var self = this;

        var tuples = self.getTupleObjects(); // Get tuples as opposed to columns.  Tuples do not have headers.

        // At least leave one column
        for (var i = tuples.length - 1; i >= 0 && tuples.length > 1; i--) {
            var dataItemObject;
            // Dealing with tuples so if horizon we need to check for absolute and relative horizon dataItems.
            if (dataItemType === HORIZON) {
                dataItemObject = self.getAnyHorizonInfo(tuples[i].tuple);
            } else {
                dataItemObject = self.getDataItemInfoFromTuple(tuples[i].tuple, dataItemType);
            }
            if (dataItemObject) {
                var dataItemLabel = dataItemObject.dataItem.getDisplayLabel(self.getOptionShowLabels().value);
                if (dataItemLabel === name) {
                    self.removeColumnsBasedOnDataItem(dataItemObject.dataItem);
                    return;
                }
            }
        }
    };


    sas.analytics.riskhp.models.CrosstabContext.prototype.removeColumnsBasedOnInfo = function (info) {
        var self = this;

        // Get targetNode to determine the dataItem Id.  This will be used to find nodes to remove.
        var targetNode = self.findColumnTreeNode(info.keys);
        if (targetNode && targetNode.item) {
            var targetDataItem = targetNode.item;
            if (targetDataItem) {
                self.removeColumnsBasedOnDataItem(targetDataItem);
            }
        }
    };


    sas.analytics.riskhp.models.CrosstabContext.prototype.removeColumnsBasedOnDataItem = function (targetDataItem) {
        var self = this;

        // NOTE: the following internal functions may be valuable external to this function.
        // FIXME whenever we get around to refactoring this file perhaps.

        var _countNode = function (node, targetingData) {
            if (node && targetingData) {
                for (var i = node.children.length - 1; i >= 0; i--) {
                    var childNode = node.children[i];
                    if (targetingData.id && childNode.item.id === targetingData.id) {
                        targetingData.count++;
                    }
                    else if (targetingData.basecase  && childNode.item.basecase === targetingData.basecase) {
                        targetingData.count++;
                    }
                    if (childNode.children.length > 0) {
                        _countNode(childNode, targetingData);
                    }
                }
            }
        };

        var _findNode = function (node, targetingData) {
            if (node && targetingData && targetingData.count > 0) {
                for (var i = node.children.length - 1; i >= 0; i--) {
                    var childNode = node.children[i];
                    if (targetingData.id && childNode.item.id === targetingData.id) {
                        return childNode;
                    }
                    else if (targetingData.basecase  && childNode.item.basecase === targetingData.basecase) {
                        return childNode;
                    }
                    else if (childNode.children.length > 0) {
                        var childChild = _findNode(childNode, targetingData);
                        if (childChild !== undefined)
                            return childChild;
                    }
                }
            }
            return undefined;
        };

        // Remove target from children of node
        var _pruneChild = function(node, targetingData) {
            if (node && targetingData) {
                var prunedChildren = _.reject(node.children, function (child) {
                    if (targetingData.basecase) {
                        return targetingData.basecase === child.item.basecase ? true : false;
                    }
                    else {
                        return targetingData.id === child.item.id ? true : false;
                    }
                });
                node.children = prunedChildren;
                targetingData.count--; // safety valve
            }
        };

        // Keep looking thru column tree for the target and remove node where ever found.
        // Also handle parent with no children after removal.
        var _nukeNode = function(targetingData) {
            var nuked = false;
            var node = self.columnTree;
            var nukeNode = _findNode(self.columnTree, targetingData);
            while (nukeNode !== undefined) {
                var parentNode = nukeNode.parent;
                if (parentNode !== undefined) {
                    _pruneChild(parentNode, targetingData);

                    // Check parents on up for empty children
                    while (parentNode !== undefined && parentNode.children.length === 0) {
                        var childId = parentNode.item.id;
                        parentNode = parentNode.parent;
                        _pruneChild(parentNode, {id: childId, count: Number.MAX_VALUE});
                    }
                    nukeNode = _findNode(self.columnTree, targetingData);
                }
            }
            return nuked;
        };

        if (targetDataItem) {

            // Create our targeting data. Basically what we are looking for in the crosstab
            // columns to remove, be it based on dataItem id or basecase horizon.
            var targetingData = {id: targetDataItem.id};
            if (self.isAnyHorizonType(targetDataItem)) {
                var basecase = targetDataItem.basecase;
                if (basecase) {
                    targetingData = {basecase : true};
                }
            }

            // This is our "safety valve" so we do not remove ALL columns.  Must leave at least one column up.
            targetingData.count = self.getNumberOfColumns() - 1;
            _nukeNode(targetingData);
        }
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.removeColumnsBasedOnId = function (id) {
        var self = this;

        // NOTE: the following internal functions may be valuable external to this function.
        //   whenever we get around to refactoring this file perhaps.

        var _countNode = function (node, targetingData) {
            if (node && targetingData) {
                for (var i = node.children.length - 1; i >= 0; i--) {
                    var childNode = node.children[i];
                    if (targetingData.id && childNode.item.id === targetingData.id) {
                        targetingData.count++;
                    }
                    else if (targetingData.basecase  && childNode.item.basecase === targetingData.basecase) {
                        targetingData.count++;
                    }
                    if (childNode.children.length > 0) {
                        _countNode(childNode, targetingData);
                    }
                }
            }
        };

        var _findNode = function (node, targetingData) {
            if (node && targetingData && targetingData.count > 0) {
                for (var i = node.children.length - 1; i >= 0; i--) {
                    var childNode = node.children[i];
                    if (targetingData.id && childNode.item.id === targetingData.id) {
                        return childNode;
                    }
                    else if (targetingData.basecase  && childNode.item.basecase === targetingData.basecase) {
                        return childNode;
                    }
                    else if (childNode.children.length > 0) {
                        var childChild = _findNode(childNode, targetingData);
                        if (childChild !== undefined)
                            return childChild;
                    }
                }
            }
            return undefined;
        };

        // Remove target from children of node
        var _pruneChild = function(node, targetingData) {
            if (node && targetingData) {
                var prunedChildren = _.reject(node.children, function (child) {
                    if (targetingData.basecase) {
                        return targetingData.basecase === child.item.basecase ? true : false;
                    }
                    else {
                        return targetingData.id === child.item.id ? true : false;
                    }
                });
                node.children = prunedChildren;
                targetingData.count--; // safety valve
            }
        };

        // Keep looking thru column tree for the target and remove node where ever found.
        // Also handle parent with no children after removal.
        var _nukeNode = function(targetingData) {
            var nuked = false;
            var nukeNode = _findNode(self.columnTree, targetingData);
            while (nukeNode !== undefined) {
                var parentNode = nukeNode.parent;
                if (parentNode !== undefined) {
                    _pruneChild(parentNode, targetingData);

                    // Check parents on up for empty children
                    while (parentNode !== undefined && parentNode.children.length === 0) {
                        var childId = parentNode.item.id;
                        parentNode = parentNode.parent;
                        _pruneChild(parentNode, {id: childId, count: Number.MAX_VALUE});
                    }
                    nukeNode = _findNode(self.columnTree, targetingData);
                }
            }
            return nuked;
        };

        if (id) {

            // Create our targeting data. Basically what we are looking for in the crosstab
            // columns to remove, be it based on dataItem id or basecase horizon.
            var targetingData = {id: id};

            // This is our "safety valve" so we do not remove ALL columns.  Must leave at least one column up.
            targetingData.count = self.getNumberOfColumns() - 1;
            _nukeNode(targetingData);
        }
    };


    sas.analytics.riskhp.models.CrosstabContext.prototype.findColumnTreeNode = function (path) {
        var node;

        var _findNode = function (colPath, node) {

            if (colPath.length > 0) {
                var subColumnPath = colPath.concat();
                for (var i = 0; i < node.children.length; i++) {
                    if (node.children[i].item.id === subColumnPath[0]) {
                        subColumnPath.shift();
                        return _findNode(subColumnPath, node.children[i]);
                    }

                    // Need to skip header dataItems and check the children.
                    else if (CrosstabUtil.isHeaderDataItem(node.children[i].item.type)) {
                        var nodeUnderHeader = _findNode(subColumnPath, node.children[i]);
                        // If we get an empty object then this sub-tree under the header will be skipped
                        // and we continue on with other peer children.
                        if (nodeUnderHeader.name !== undefined) {
                            return nodeUnderHeader;
                        }
                    }
                }
                return {};  // Return empty object if we can't continue any farther
            }
            return node;
        };

        if (this.columnTree.children.length > 0) {
            node = _findNode(path.concat(), this.columnTree);
        }

        return node;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.findNodeChildFromId = function (node, id) {

        if (node && id) {
            for (var i = 0; i < node.children.length; i++) {
                if (node.children[i].item.id === id) {
                    return node.children[i];
                }
            }
        }
        return undefined;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.convertTupleToColumn = function (tuple) {
        var self = this;
        var col = tuple.concat(); // copy

        // If scenario present and not activated, bail...
        var scenarioDataItemObject = self.getDataItemInfoFromTuple(col, sas.analytics.riskhp.models.dataitems.Scenario.SCENARIO);
        if (scenarioDataItemObject && !scenarioDataItemObject.dataItem.isActivated()) {
            return[];
        }

        // This addColumn function may be called externally so as a safety check,
        // swap rel/abs horizon to real one.  This is need to ensure proper horizon
        // dataItem is present in the column tuple for the compatibility check
        self.forceHorizonToBaseHorizon(col);

        // Test for any dataItem headers (such as rowstat) in the
        // incoming column and remove if found.  This will prevent a valid column
        // from failing restrictions check.
        self._removeHeaders(col);

        // Reconcile column against the heirarchy and rearrange if necessary
        var reconciledColumn = self.reconcileColumnAgainstHierarchy(col);

        if (reconciledColumn) {

            // Do the dataItems together make a valid column?  Restrictions check here.
            var isCompatible = self.cubeDataModel.areDataItemsCompatible(reconciledColumn);

            if (isCompatible) {
                var horizonConvertSuccess = true;

                // Now fix the column tuple to be displayable.
                // 1) add in propery "fake" headers for groupings
                // 2) swap out the horizon for relative/absolute based on crosstab option

                // Add in "fake" header dataItems to create groupings for compvars and benchvars.
                if (reconciledColumn.length === 1) {
                    if (reconciledColumn[0].type === COMPVAR) {
                        reconciledColumn.unshift(CrosstabUtil.getDataItem(CrosstabUtil.COMPVARHEADER, CrosstabUtil.COMPVARHEADER, self.cubeDataModel));
                    }
                    else if (reconciledColumn[0].type === BENCHVAR) {
                        reconciledColumn.unshift(CrosstabUtil.getDataItem(CrosstabUtil.BENCHVARHEADER, CrosstabUtil.BENCHVARHEADER, self.cubeDataModel));
                    }
                }

                // For row stats, need to modify the horizon to make it unique and separate from
                // the same horizon with outvars and non-row stats
                var rowStatObject = self.getRowStatisticFromColumn(reconciledColumn);
                if (rowStatObject) {
                    reconciledColumn.splice(rowStatObject.index, 0, CrosstabUtil.getDataItem(CrosstabUtil.ROWSTATHEADER, CrosstabUtil.ROWSTATHEADER, self.cubeDataModel));
                }

                // Remove the riskcube if there is only one in the cube data model. (no point in showing it)
                var riskCubeDataItemObject = self.getDataItemInfoFromTuple(reconciledColumn, sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE);
                if (riskCubeDataItemObject && self.cubeDataModel.riskCube.length < 2) {
                    reconciledColumn.splice(riskCubeDataItemObject.index, 1);
                }

                // Process the horizon prior to display in the crosstab.
                var proxyHorizonObject = self.getAnyHorizonInfo(reconciledColumn);
                if (proxyHorizonObject) {
                    var horizon = proxyHorizonObject.dataItem;

                    // Swap out the horizon for absolute or relative based on crosstab display option
                    if (this.optionDisplayHorizons.value === HprConstants.DISPLAY_HORIZONS_ABSOLUTE_VALUE) {
                        var absHorizon = this.cubeDataModel.findAbsoluteHorizon(horizon.horizonDate, horizon.label, horizon.basecase);
                        if (absHorizon !== undefined) {
                            // Horizon to absolute
                            reconciledColumn[proxyHorizonObject.index] = absHorizon;
                        }
                        else {
                            // Can't display this column, display error later
                            horizonConvertSuccess = false;
                        }
                    }
                    else {
                        var relHorizon = this.cubeDataModel.findRelativeHorizon(horizon.getInterval(), horizon.getHorizonNumber(), horizon.basecase);
                        if (relHorizon !== undefined) {
                            // horizon to relative
                            reconciledColumn[proxyHorizonObject.index] = relHorizon;
                        }
                        else {
                            // Can't display this column, display error later
                            horizonConvertSuccess = false;
                        }
                    }
                    // If basecase then skip the horizon abs/rel substitution.  Add basecase header.
                    if (self.cubeDataModel.isStressCube() && horizon.basecase === true) {
                        // putting basecase header at the same location a scenario would go, for alignment purposes
                        var baseCaseIndex = 0;
                        var scenarioIndex = self.getColumnMemberIndex(SCENARIO);    // this should ALWAYS be here
                        if (scenarioIndex) {
                            for (var i = scenarioIndex-1; i >= 0; i--) {
                                var chm = self.columnHierarchyMembers[i];
                                var chmIndex = _.findIndex(reconciledColumn, function(rcDataItem) {
                                    if (chm.type === HORIZON) {
                                        return self.isAnyHorizonType(rcDataItem);
                                    }
                                    else {
                                        return rcDataItem.getType() === chm.type;
                                    }
                                });
                                if (chmIndex !== -1) {
                                    baseCaseIndex = chmIndex + 1;   // AFTER the found data item
                                    break;
                                }
                            }
                        }
                        reconciledColumn.splice(baseCaseIndex, 0, CrosstabUtil.getDataItem(CrosstabUtil.BASECASEHEADER, CrosstabUtil.BASECASEHEADER, self.cubeDataModel));
                    }
                }

                if (horizonConvertSuccess !== true) {
                    // Bad horizon
                    sas.log.error("Horizon " + horizon.getLabel() + " can not be displayed per crosstab option: " + this.optionDisplayHorizons.value);
                    sas.log.debug("Horizon " + horizon.getLabel() + " is " + horizon.getId());
                    return [];
                }
            } else {
                // The dataItems in the column are not compatible
                var itemString = "";
                _.each(reconciledColumn, function (dataItem) {
                    itemString += "  " + dataItem.getType() + ":" + dataItem.getLabel();
                });
                sas.log.error("Failed restrictions check: " + itemString);
                return [];
            }
        } else {
            // Error, the column doesn't contain the necessary dataItems.
            sas.log.error("Column tuple does not contain necessary dataItems");
            return [];
        }

        return reconciledColumn;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype._removeHeaders=function(tuple){
        var self = this;

        for (var idx = tuple.length -1; idx >= 0; idx--) {
            if (tuple[idx].type === CrosstabUtil.BASECASEHEADER || tuple[idx].type === CrosstabUtil.ROWSTATHEADER) {
                tuple.splice(idx, 1);
            }
        }
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.findColumnTreeLeafNode=function(dataItemArray, dataItemArrayType){
        var self = this;
        var node;
        var tuple = dataItemArray.concat(); // tuple or column

        var _findNode = function(colItem, node) {

            if (colItem.length > 0) {
                for (var i = 0; i < node.children.length; i++) {
                    var childNode = node.children[i];

                    // Handle horizons different because of abs/rel horizons.
                    if (self.isAnyHorizonType(childNode.item)) {
                        var childNodeHorizon = childNode.item;
                        if (dataItemArrayType === "tuple") {

                            //Get the abs/rel horizon of column item (first item in dataItemArray).
                            if (self.isAnyHorizonType(colItem[0])) {
                                var columnHorizon;
                                var nodeHorizonType = childNode.item.type;
                                if (nodeHorizonType === ABSOLUTEHORIZON) {
                                    columnHorizon = self.cubeDataModel.findAbsoluteHorizon(colItem[0].horizonDate, colItem[0].label, colItem[0].basecase);
                                } else if (nodeHorizonType === RELATIVEHORIZON) {
                                    columnHorizon = self.cubeDataModel.findRelativeHorizon(colItem[0].interval, colItem[0].horizonNumber, colItem[0].basecase);
                                } else {
                                    columnHorizon = colItem[0];
                                }

                                // test the abs/rel ids
                                if (columnHorizon && columnHorizon.id === childNodeHorizon.id) {
                                    colItem.shift();
                                    return _findNode(colItem, childNode);
                                }
                            }
                        } else {
                            if (childNodeHorizon.id === colItem[0].id) {
                                colItem.shift();
                                return _findNode(colItem, childNode);
                            }
                        }
                        // if (childNodeHorizon.id === colItem[0].id) {
                        //     colItem.shift();
                        //     return _findNode(colItem, childNode);
                        // }
                    } else if (childNode.name === colItem[0].id) {
                        colItem.shift();
                        return _findNode(colItem, childNode);
                        // compvar, benchvar,
                    } else if (CrosstabUtil.isHeaderDataItem(childNode.name)) {
                        var foundViaHeaderNode = _findNode(colItem, childNode);
                        // If we found what we are looking for under the header, then we can return.  Else,
                        // keep going with other children of node.
                        if (foundViaHeaderNode) {
                            return foundViaHeaderNode;
                        }
                    }
                }
                return undefined;
            }
            return node;
        };

        // Remove the riskcube if there is only one in the cube data model, therefor not displayed, thus not in columnTree.
        var riskCubeDataItemObject = self.getDataItemInfoFromTuple(tuple, sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE);
        if (riskCubeDataItemObject && self.cubeDataModel.riskCube.length < 2) {
            tuple.splice(riskCubeDataItemObject.index, 1);
        }

        if (this.columnTree.children.length > 0) {
            node = _findNode(tuple.concat(), this.columnTree);
        }

        return node;
    };

// This will reorder the dataitems in the candidate column to align with the heirarchy
    sas.analytics.riskhp.models.CrosstabContext.prototype.reconcileColumnAgainstHierarchy=function(candidateColumn) {

        var self = this;
        var validColumn = [];

        if (candidateColumn) {

            // Has benchmark?
            var benchvarDataItem = _.findWhere(candidateColumn, {type: sas.analytics.riskhp.models.dataitems.Benchvar.BENCHVAR});
            if (benchvarDataItem) {
                return [benchvarDataItem];
            }

            // Has computed variable?
            var compvarDataItem = _.findWhere(candidateColumn, {type: sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR});
            if (compvarDataItem) {
                return [compvarDataItem];
            }

            // Now rearrange dataItems based on column hierarchy.
            for(var i = 0; i < self.columnHierarchyMembers.length; i++) {
                var dataItem = _.findWhere(candidateColumn, {type: self.columnHierarchyMembers[i].type});
                if (dataItem) {
                    validColumn.push(dataItem);
                } else {
                    // Based on the column hierarchy, there is a missing a dataItem from the candidate column
                    // This may or may not be a problem.
                }
            }
            return validColumn;
        }
        return undefined;
    };



    sas.analytics.riskhp.models.CrosstabContext.prototype.isValidColumn = function(col) {

        var self = this;
        var validColumn = true;

        if (col) {

            // Has benchmark?
            var benchvarDataItem = _.findWhere(col, {type: sas.analytics.riskhp.models.dataitems.Benchvar.BENCHVAR});
            if (benchvarDataItem && col.length === 1) {
                return true;
            }

            // Has computed variable?
            var compvarDataItem = _.findWhere(col, {type: sas.analytics.riskhp.models.dataitems.Compvar.COMPVAR});
            if (compvarDataItem && col.length === 1) {
                return true;
            }

            // Has row stat?
            var statDataItem = _.findWhere(col, {type: sas.analytics.riskhp.models.dataitems.Statistic.STATISTIC});
            if (statDataItem  && self.cubeDataModel.isRowStatistic(statDataItem)) {
                if (col.length < 3) {
                    var rcDataItem = _.findWhere(col, {type: sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE});
                    if (rcDataItem === undefined) {
                        if (self.cubeDataModel.riskCube.length < 2) {
                            return true;
                        }
                    } else {
                        return true;
                    }
                    return false;
                }
                else {
                    return false;
                }
            }

            // Now test dataItems based on column hierarchy.
            for(var i = 0; i < self.columnHierarchyMembers.length; i++) {
                var dataItem = _.findWhere(col, {type: self.columnHierarchyMembers[i].type});
                if (dataItem === undefined) {
                    // Based on the column hierarchy, there is a missing dataItem from the column.
                    // This may or may not be a problem.
                    if (self.columnHierarchyMembers[i].type === RISKCUBE) {
                        if (self.cubeDataModel.riskCube.length > 1) {
                            validColumn = false;
                            break;
                        }
                    }
                    else if (self.columnHierarchyMembers[i].type === SCENARIO) {
                        var horizonObject = self.getAnyHorizonInfo(col);
                        if (horizonObject !== undefined) {
                            // If we are missing a scenario, then the horizon better be basecase.
                            if (horizonObject.dataItem.basecase === false) {
                                validColumn = false;
                                break;
                            }
                        } else {
                            validColumn = false;
                            break;
                        }
                    }
                    else if (self.columnHierarchyMembers[i].type === HORIZON) {
                        // if rel/abs, then ok
                        var horizonObject = self.getAnyHorizonInfo(col);
                        if (horizonObject === undefined) {
                            validColumn = false;
                            break;
                        }
                    }
                    else {
                        validColumn = false;
                        break;
                    }
                }
            }
            return validColumn;
        }
        return false;
    };


    sas.analytics.riskhp.models.CrosstabContext.prototype.updateColumnTree = function() {
        var self = this;
        var forceDefaultColumnOrdering = false; // Just use the existing column order.  Do not reorder.

        var tupleObjects = this.getTupleObjects();
        this.clearColumnTree();
        _.each(tupleObjects, function(tupleObject) {
            self.addColumn(tupleObject.tuple, tupleObject.sort, tupleObject.width, forceDefaultColumnOrdering);
        });
        return this.columnTree;
    };


    sas.analytics.riskhp.models.CrosstabContext.prototype.COLUMNROOT = "columnRoot";

    sas.analytics.riskhp.models.CrosstabContext.prototype.clearColumnTree = function() {
        this.columnTree = {
            name: this.COLUMNROOT,
            // Fake dataItem for columnTree root node.  This is needed to make processing the
            // nodes similar and simple.
            item: {
                type: this.COLUMNROOT,
                name: this.COLUMNROOT,
                id: this.COLUMNROOT,
                getType: function () {
                    return this.COLUMNROOT;
                },
                label: this.COLUMNROOT,
                getLabel: function () {
                    return this.COLUMNROOT;
                }
            },
            parent: undefined,
            children: []
        };
        return this.columnTree;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.replaceScenario = function(scenario, deletedHorizonIds, addedHorizonIds) {
        var self = this;

        // get the remaining tuples
        var tupleObjects = this.getTupleObjects();

        // reset the column tree
        this.clearColumnTree();

        // Process each tuple: If the scenario is activated, replace it in the tuples that
        // contain it. If not, then the scenario cannot be activated for this crosstab so
        // we will remove the tuple containing it entirely.
        var scenarioFound = false;
        _.each(tupleObjects, function (tupleObject) {
            var scenarioIndex = _.findIndex(tupleObject.tuple, function (tupleItem) {
                return tupleItem.id === scenario.getId();
            });

            var tupleHorizon = self.getDataItemInfoFromTuple(tupleObject.tuple, sas.analytics.riskhp.models.dataitems.Horizon.HORIZON);

            if (scenarioIndex !== -1) {
                if (scenario.isActivated() === true && tupleHorizon && _.contains(deletedHorizonIds, tupleHorizon.dataItem.id) !== true) {
                    tupleObject.tuple.splice(scenarioIndex, 1, scenario);
                    self.addColumn(tupleObject.tuple, tupleObject.sort, tupleObject.width);
                    scenarioFound = true;
                }
            }
            else {
                self.addColumn(tupleObject.tuple, tupleObject.sort, tupleObject.width);
            }
        });

        // add any horizons that were added to the scenario
        if (scenarioFound === true && addedHorizonIds && addedHorizonIds.length > 0) {
            _.each(addedHorizonIds, function(horizonId) {
                var newHorizon = _.find(scenario.horizons, function(horizon) {return horizon.id === horizonId;});
                if (newHorizon) {
                    self.addDataItemToColumns(newHorizon);
                }
            });
        }

        // rebuild the column tree to be sure all tuples are valid
        self.updateColumnTree();
    };

//*****************************
// For CrosstabModelBuilder
//*****************************

    sas.analytics.riskhp.models.CrosstabContext.prototype.getRowMemberNames=function(){
        //return _.pluck(this.classVariables, 'label');
        return _.map(this.classVariables, function(dataItem){
            return dataItem.getLabel();
        });
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getColumnMembers=function(){
        return this.columnHierarchyMembers;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getColumnMemberIndex=function(type){
        var self = this;
        var idx = self.columnHierarchyMembers.length - 1;

        for(; idx >= 0; idx--) {
            if (self.columnHierarchyMembers[idx].type === type) {
                return idx;
            }
        }
        return undefined;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setColumnMembers=function(hierarchyMembers){

        var self = this;

        this.columnHierarchyMembers = _.map(hierarchyMembers, function(item) {
            var label = "";
            switch(item){
                case CLASSVAR:
                    label = self.i18n.getText("DataItem.classVariable.txt");
                    break;
                case HORIZON:
                    label = self.i18n.getText("DataItem.horizon.txt");
                    break;
                case RISKCUBE:
                    label = self.i18n.getText("DataItem.riskCube.txt");
                    break;
                case OUTVAR:
                    label = self.i18n.getText("DataItem.outputVariable.txt");
                    break;
                case BENCHVAR:
                    label = self.i18n.getText("DataItem.benchmarkVariable.txt");
                    break;
                case STATISTIC:
                    label = self.i18n.getText("DataItem.statistic.txt");
                    break;
                case RESULTSET:
                    label = self.i18n.getText("DataItem.resultSet.txt");
                    break;
                case SCENARIO:
                    label = self.i18n.getText("DataItem.scenario.txt");
                    break;
                default:
                    break;
            }
            return {label: label, type: item};
        });
    };

    // /**
    //  * Method to get tuples from the crosstab that meet criteria from template.
    //  * Template made up of one or more objects, each representing a test of a dataItem.
    //  * More templates will yield a more specific search.  Undefined is used as a wildcard.
    //  * @param template = [{type: dataItemType, dataItem: undefined|dataItem}, {}, ... {}]
    //  * @param option
    //  * @returns [tuple]
    //  */
    // sas.analytics.riskhp.models.CrosstabContext.prototype.getTuplesBasedOnTemplate=function(template, options) {
    //     var self = this;
    //     var matchingTuples = [];
    //     var option = [];
    //     option["strict"] = false; // Have to have the same number of dataItems
    //     option["ordered"] = false; // DataItems in the template and tuple have to be in the same order
    //
    //     if (options === undefined) {
    //         option["strict"] = true;
    //         option["ordered"] = true;
    //     }
    //     else {
    //         _.each(options, function(opt) {
    //             option[opt] = true;
    //         });
    //     }
    //
    //     var tuples = self.getTuples();
    //     _.each(tuples, function(tuple) {
    //         // do we have the proper number of dataItems or don't care (strict = false)?
    //         if ((template.length === tuple.length) ||
    //             ((option["strict"] === false) && (template.length < tuple.length))){
    //             var match = true;
    //             _.each(template, function(templateItem, index) {
    //                 var tupleItem;
    //                 if (option["ordered"] === true) {
    //                     // Get the dataItem at the same index as the template index.
    //                     tupleItem = tuple[index];
    //                 }
    //                 else {
    //                     // Get the dataItemtype from the tuple regardless of order.
    //                     tupleItem = self.getDataItemInfoFromTuple(tuple, templateItem.type);
    //                 }
    //                 if (tupleItem) {
    //                     // undefined is treated as a wildcard
    //                     if (templateItem.dataItem !== undefined) {
    //                         if (templateItem.dataItem.id !== tupleItem.dataItem.id) {
    //                             match = false;
    //                         }
    //                     }
    //                 }
    //                 else {
    //                     match = false;
    //                 }
    //             });
    //             if (match) {
    //                 matchingTuples.push([].concat(tuple));
    //             }
    //         }
    //     });
    //     return matchingTuples;
    // };

    /**
     * @typedef {Object} TemplateItem - an object that represents a DataItem in a column template
     * @property {DataItem} item - The DataItem for the template, may be undefined to match any DataItems of the type specified in the type property
     * @property {string} type - The type of data item that should be matched to this row in the column
     */

    /**
     * @typedef {Object} ColumnQueryOption
     * @property {boolean} [strict=true] - true if the template must have the same number of dataItems as the crosstab?
     * @property {boolean} [ordered=true] - true if DataItems in the template and tuple have to be in the same order
     */
    /**
     * Method to get tuples from the crosstab that meet criteria from template.
     * Template made up of one or more dataItems, undefined is used as a wildcard.
     * @param {TemplateItem} template
     * @param {ColumnQueryOption} options    (NOTE: Experimental)
     * @returns [tuple]
     */
    sas.analytics.riskhp.models.CrosstabContext.prototype.getTuplesFromColumnTemplate=function(template, options) {
        var self = this;

        var matchingColumns = [];
        var option = [];
        option["strict"] = false; // Have to have the same number of dataItems
        option["ordered"] = false; // DataItems in the template and tuple have to be in the same order

        if (options === undefined) {
            option["strict"] = true;
            option["ordered"] = true;
        }
        else {
            _.each(_.keys(options), function(key) {
                option[key] = options[key];
            });
        }

        var columnObjects = self.getColumnObjects();
        _.each(columnObjects, function(columnObject) {
            var aColumn = columnObject.column;

            // do we have the proper number of dataItems or don't care (strict = false)?
            if ((template.length === aColumn.length) ||
                ((option["strict"] === false) && (template.length < aColumn.length))){
                var match = true;
                _.each(template, function(templateItem, index) {
                    var tupleItem;
                    if (option["ordered"] === true) {
                        // Get the dataItem at the same index as the template index.
                        tupleItem = aColumn[index];
                    }
                    else {
                        // Get the dataItemtype from the tuple regardless of order.
                        tupleItem = self.getDataItemInfoFromTuple(aColumn, templateItem.type);
                        if (tupleItem) {
                            tupleItem = tupleItem.dataItem;
                        }
                    }
                    if (tupleItem) {
                        // undefined is treated as a wildcard
                        if (templateItem.item !== undefined) {
                            if (templateItem.item.id !== tupleItem.id) {
                                match = false;
                            }
                        } else if (templateItem.type !== tupleItem.type) {
                            match = false;
                        }
                    }
                    else {
                        match = false;
                    }
                });
                if (match) {
                    matchingColumns.push([].concat(aColumn));
                }
            }
        });
        //Convert to base horizon in each tuple, remove header
        var verifiedTuples = _.each(matchingColumns, function(c) {
            CrosstabUtil.removeHeaderFromColumn(c);
            self.forceHorizonToBaseHorizon(c);
            self.replaceRiskCube(c);
        });

        return verifiedTuples;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.replaceRiskCube = function(col) {
        var self = this;
        var riskCubeId = undefined;
        var riskcubeDataItemObject = self.getDataItemInfoFromTuple(col, sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE);
        if (riskcubeDataItemObject === undefined) {
            var resultSetDataItemObject = self.getDataItemInfoFromTuple(col, sas.analytics.riskhp.models.dataitems.ResultSet.RESULTSET);
            if (resultSetDataItemObject === undefined) {
                var horizonDataItemObject = self.getDataItemInfoFromTuple(col, sas.analytics.riskhp.models.dataitems.Horizon.HORIZON);
                if (horizonDataItemObject === undefined) {
                    riskCubeId = undefined;  // ummm... fixing below
                }
                else {
                    riskCubeId = horizonDataItemObject.dataItem.riskCubeId;
                }
            }
            else {
                riskCubeId = resultSetDataItemObject.dataItem.riskCubeId;
            }

            // Default to the first riskcube object in the datasource
            if (riskCubeId === undefined) {
                riskCubeId = self.cubeDataModel.riskCube[0].id;
            }
            var riskCube = CrosstabUtil.getDataItemById(riskCubeId,
                sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE,
                self.cubeDataModel);
            if (riskCube) {
                // Lets put the riskcube in the same position as columnHierarchy
                var riskCubeIndex = _.findIndex(self.columnHierarchyMembers, function(member) {
                    return (member.type === sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE ? true : false);
                });
                if (riskCubeIndex < 0) {
                    col.push(riskCube);
                }
                else {
                    col.splice(riskCubeIndex, 0, riskCube);
                }
            }
        }
    };


    sas.analytics.riskhp.models.CrosstabContext.prototype.getRowHierarchyName=function(){
        return this.rowHierarchyName;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setRowHierarchyName=function(name){
        this.rowHierarchyName = name;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getColumnHierarchyName=function(){
        return this.columnHierarchyName;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setColumnHierarchyName=function(name){
        this.columnHierarchyName = name;
    };



//*****************************
// Sort
//*****************************
    sas.analytics.riskhp.models.CrosstabContext.SORT_DIRECTION_ASCENDING = "ascending";
    sas.analytics.riskhp.models.CrosstabContext.SORT_DIRECTION_DESCENDING = "descending";

    sas.analytics.riskhp.models.CrosstabContext.SORT_TYPE_VALUE = "value";
    sas.analytics.riskhp.models.CrosstabContext.SORT_TYPE_ABSOLUTE = "absolute";

    sas.analytics.riskhp.models.CrosstabContext.prototype.setSortColumn = function(colNumber, direction, type){
        var self = this;

        //Clear out any old sort(s)
        var sortIndexes = self.getSortColumnIndexes();
        for (var i = 0; i < sortIndexes.length; i++) {
            self.clearSortColumn(sortIndexes[i]);
        }

        var columnNodes = this.getColumnNodes(colNumber);
        if (columnNodes && columnNodes.length > 0) {
            columnNodes[columnNodes.length - 1].sort = {direction: direction, type: type};
        }
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.clearSortColumn = function(colNumber){
        var self = this;
        var columnNodes = this.getColumnNodes(colNumber);
        if (columnNodes && columnNodes.length > 0) {
            delete columnNodes[columnNodes.length - 1].sort;
        }
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getSortColumn = function(colNumber){
        var columnNodes = this.getColumnNodes(colNumber);
        if (columnNodes && columnNodes.length > 0) {
            return columnNodes[columnNodes.length - 1].sort;
        }
        return undefined;
    };


    sas.analytics.riskhp.models.CrosstabContext.prototype.getSortColumnIndexes = function(){

        var currentIndex = -1;
        var columnIndexes = [];

        var _recurseTree = function(indexes, node) {
            if (node) {
                if (node.children.length === 0) {
                    currentIndex++;
                    if (node.sort !== undefined) {
                        indexes.push(currentIndex);
                    }
                } else {
                    for (var i = 0; i < node.children.length; i++) {
                        _recurseTree(indexes, node.children[i]);
                    }
                }
            }
            return false;
        };

        if (this.columnTree.children.length > 0) {
            for (var i = 0; i <this.columnTree.children.length; i++) {
                _recurseTree(columnIndexes, this.columnTree.children[i]);
            }
        }

        return columnIndexes;
    };


    sas.analytics.riskhp.models.CrosstabContext.prototype.getSort = function(){

        var currentIndex = -1;
        var columnIndexes = [];

        var _recurseTree = function(indexes, node) {
            if (node) {
                if (node.children.length === 0) {
                    currentIndex++;
                    if (node.sort !== undefined) {
                        indexes.push(currentIndex);
                    }
                } else {
                    for (var i = 0; i < node.children.length; i++) {
                        _recurseTree(indexes, node.children[i]);
                    }
                }
            }
            return false;
        };

        if (this.columnTree.children.length > 0) {
            for (var i = 0; i <this.columnTree.children.length; i++) {
                _recurseTree(columnIndexes, this.columnTree.children[i]);
            }
        }

        return {};
    };


//*****************************
// Column format overrides.
//*****************************

    //NOTE: this.columnFormatOverride = []; // [{outvar: id, statistic: id, format: "formatstring"}, ...]

    sas.analytics.riskhp.models.CrosstabContext.prototype.setColumnFormatOverride = function(tuple, format) {
        var self = this;
        var outvarDataItemObject = self.getDataItemInfoFromTuple(tuple, OUTVAR);
        if (outvarDataItemObject) {
            var statDataItemObject = self.getDataItemInfoFromTuple(tuple, STATISTIC);
            if (statDataItemObject) {
                //NOTE:  What about row stat formats?  Currently filtered prior to call in menu provider.
                // Remove format if it already exists
                self.clearColumnFormatOverride(tuple);

                // Add format to list
                self.columnFormatOverride.push({
                    outvar: outvarDataItemObject.dataItem.id,
                    statistic: statDataItemObject.dataItem.id, format: format
                });
            }
        } else {
            var compvarDataItemObject = self.getDataItemInfoFromTuple(tuple, COMPVAR);
            var benchvarDataItemObject = self.getDataItemInfoFromTuple(tuple, BENCHVAR);

            //FIXME  What about string format in column?

            if (compvarDataItemObject &&
                compvarDataItemObject.dataItem.dataType === HprConstants.HPR_DATA_TYPE_NUMBER) {

                // Remove format if it already exists
                self.clearColumnFormatOverride(tuple);

                // Add format to list
                self.columnFormatOverride.push({compvar: compvarDataItemObject.dataItem.id, format: format});

            } else if (benchvarDataItemObject &&
                benchvarDataItemObject.dataItem.dataType === HprConstants.HPR_DATA_TYPE_NUMBER) {

                // Remove format if it already exists
                self.clearColumnFormatOverride(tuple);

                // Add format to list
                self.columnFormatOverride.push({benchvar: benchvarDataItemObject.dataItem.id, format: format});
            }
        }
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.clearColumnFormatOverride = function(tuple){
        var self = this;
        var filteredList = self.columnFormatOverride;

        var outvarDataItemObject = self.getDataItemInfoFromTuple(tuple, OUTVAR);
        if (outvarDataItemObject) {
            var statDataItemObject = self.getDataItemInfoFromTuple(tuple, STATISTIC);
            if (statDataItemObject) {
                filteredList = _.reject(self.columnFormatOverride, function (aColumnFormat) {
                    return (aColumnFormat.outvar === outvarDataItemObject.dataItem.id &&
                    aColumnFormat.statistic === statDataItemObject.dataItem.id ? true : false);
                });
            }
        } else {
            var compvarDataItemObject = self.getDataItemInfoFromTuple(tuple, COMPVAR);
            var benchvarDataItemObject = self.getDataItemInfoFromTuple(tuple, BENCHVAR);

            if (compvarDataItemObject) {
                filteredList = _.reject(self.columnFormatOverride, function(aColumnFormat) {
                    return (aColumnFormat.compvar === compvarDataItemObject.dataItem.id ? true : false);
                });
            } else if (benchvarDataItemObject) {
                filteredList = _.reject(self.columnFormatOverride, function(aColumnFormat) {
                    return (aColumnFormat.benchvar === benchvarDataItemObject.dataItem.id ? true : false);
                });
            }
        }
        self.columnFormatOverride = filteredList;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getColumnFormatOverride = function(tuple){
        var self = this;
        var format = undefined;
        var formatEntry = undefined;

        var outvarDataItemObject = self.getDataItemInfoFromTuple(tuple, OUTVAR);
        if (outvarDataItemObject) {
            var statDataItemObject = self.getDataItemInfoFromTuple(tuple, STATISTIC);
            if (statDataItemObject) {
                formatEntry = _.findWhere(self.columnFormatOverride, {
                    outvar: outvarDataItemObject.dataItem.id,
                    statistic: statDataItemObject.dataItem.id
                });
            }
        } else {
            var compvarDataItemObject = self.getDataItemInfoFromTuple(tuple, COMPVAR);
            var benchvarDataItemObject = self.getDataItemInfoFromTuple(tuple, BENCHVAR);

            if (compvarDataItemObject) {
                formatEntry = _.findWhere(self.columnFormatOverride, {compvar: compvarDataItemObject.dataItem.id});
            } else if (benchvarDataItemObject) {
                formatEntry = _.findWhere(self.columnFormatOverride, {benchvar: benchvarDataItemObject.dataItem.id});
            }
        }
        if (formatEntry) {
            format = formatEntry.format;
        }
        return format;
    };


//*****************************
// Rebuild internal models, etc
//*****************************

    sas.analytics.riskhp.models.CrosstabContext.prototype.rebuildInternalModelsFromDefaults = function() {
        var self = this;

        // Convert default tuples to dataItem arrays (columns).
        this.defaultTuples = _.map(self.cubeDataModel.defaults.tuples, function (tuple, tIndex) {
            var t = tuple.dataItems;
            return _.map(t, function (dataItem, dIndex) {
                var d = dataItem;
                return  CrosstabUtil.getDataItemById(d.id, d.type, self.cubeDataModel);
            });
        });

        // Convert default classvars to default classvar dataitems
        this.defaultClassVariables = _.map(self.cubeDataModel.defaults.classVariables, function (id) {
            return _.findWhere(self.cubeDataModel.classVariable, {id: id});
        });

        // Set the heirarchy from defaults.
        this.columnHierarchyMembers = _.map(self.cubeDataModel.defaults.tupleHierarchy, function(type, tIndex) {
            return {
                label: self.i18n.getText('DataItem.' + type + '.txt'),
                type: type
            };
        });
    };

// Sets the columns and rows to default values
    sas.analytics.riskhp.models.CrosstabContext.prototype.resetToDefaults = function() {
        var self = this;

        self.zeroState = false;

        // Set the crosstab Columns based on default tuples
        self.clearColumnTree();

        // Set the crosstab row based on default classvar(s) and clear drill
        //self.clearDrillPath();
        self.resetReportResultsTree();

        // Set the heirarchy from defaults.
        this.columnHierarchyMembers = _.map(self.cubeDataModel.defaults.tupleHierarchy, function(type, tIndex) {
            return {
                label: self.i18n.getText('DataItem.' + type + '.txt'),
                type: type
            };
        });

        _.each(this.defaultTuples, function(col, index, list) {
            self.addColumn([].concat(col));
        });

        // Set the default classvars in this context
        self.classVariables = []; // reset the classvars
        for (var i = 0; i < this.defaultClassVariables.length; i++) {
            self.pushClassVariable(this.defaultClassVariables[i]);
        }

    };


    //*****************************
    // Expansions
    //*****************************

    sas.analytics.riskhp.models.CrosstabContext.prototype.createExpansionQueryKey = function(valueArray, includeExpansion ) {

        var queryKey = valueArray.concat(); // clone

        if (queryKey.length === 0) {
            if (includeExpansion) {
                queryKey.push(SAS_ALL);
            }
        } else {
            if ((queryKey[queryKey.length - 1] === this.i18n.getText('HPRisk.Crosstab.Total.txt')) ||
                (queryKey[queryKey.length - 1] === this.i18n.getText('HPRisk.Crosstab.Subtotal.txt'))) {
                // This is a catchall and I don't think includeExpansion pertains here.
                queryKey[queryKey.length - 1] = SAS_ALL;
            } else {
                if (includeExpansion) {
                    // Ensure the last element of key is a SAS_ALL
                    queryKey.push(SAS_ALL);
                }
            }
        }

        return queryKey;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.createExpansionName = function(key) {
        var name;
        if (key) {
            name = EXPANSION_NAME;
            for (var i = 0; i < key.length; i++) {
                name = name.concat(EXPANSION_NAME_SEPARATOR, key[i]);
            }
        }
        else {
            name = "toplevel";
        }
        return name;
    };


    sas.analytics.riskhp.models.CrosstabContext.prototype.clearAllExpansions=function(){
        var self = this;

        var _walkExpansionsForClearing = function (node) {
            var self = this;

            // {name: "root", results: [], expanded: false, children: []}
            if (node) {

                node.expanded = false;

                for (var idx = 0; idx < node.children.length; idx++) {
                    _walkExpansionsForClearing(node.children[idx]);
                }
            }
        };

        if (self.reportResultsTree.children.length > 0) {
            for (var idx = 0; idx < self.reportResultsTree.children.length; idx++) {
                _walkExpansionsForClearing(self.reportResultsTree.children[idx]);
            }
        }
    };


    sas.analytics.riskhp.models.CrosstabContext.prototype.findRowTreeNode = function (key, nodeProperty){
        var self = this;
        var node;

        if (nodeProperty === undefined) {
            nodeProperty = "label";
        }
        var expansionNode = self.reportResultsTree;

        var _walkExpansionsForNode = function (key, node) {
            var self = this;

            // {name: "root", results: [], expanded: false, children: []}
            if (node) {

                // We need to check for indented mode display tree hack!  There are nodes
                // that have a non breaking space at the start of the key value.  (This is done
                // to ITJS display tree nodes, NOT the crosstab's row tree nodes!)  So, we need
                // to check to see if the key(s) we are give have them as the key may have been
                // received from ITJS.
                //var scrubbedKey = (key[key.length - 1][0] === '\xa0' ? key[key.length - 1].slice(1) : key[key.length - 1]);
                if (node[nodeProperty] === key[key.length - 1]) {
                    //if (node[nodeProperty] === scrubbedKey) {

                    if (key.length <= 1) {
                        return node;
                    } else {

                        var rKey = key.concat();
                        rKey.pop();

                        for (var idx = 0; idx < node.children.length; idx++) {
                            var expNode = _walkExpansionsForNode(rKey, node.children[idx]);
                            if (expNode) {
                                return expNode;
                            }
                        }
                    }
                }
            }
            return;
        };

        if ((key) && (key.length > 0)) {

            // We will search using a reversed key.
            var rKey = key.concat();
            while ((rKey[rKey.length -1] === SAS_ALL) ||
            (rKey[rKey.length -1] === -100)){
                rKey.pop(); // remove the sas_all as this really represents all the children of the node
                if (rKey.length <= 0) {
                    break;
                }
            }
            rKey.reverse();

            if ((self.reportResultsTree.children) && (self.reportResultsTree.children.length > 0) && (rKey.length > 0)) {
                for (var idx = 0; idx < self.reportResultsTree.children.length; idx++) {
                    var eNode = _walkExpansionsForNode(rKey, self.reportResultsTree.children[idx]);
                    if (eNode) {
                        expansionNode = eNode;
                        break;
                    }
                }
            }
        }

        return expansionNode;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.findRowTreePath = function (node, nodeProperty){
        var self = this;
        var aNode = node;
        var path = [];

        if (node === undefined) {
            return [];
        }
        if (nodeProperty === undefined) {
            nodeProperty = "name";
        }

        while(aNode !== undefined) {
            path.push(aNode[nodeProperty]);
            aNode = aNode.parent;
        }

        path.reverse(); // because these are in reverse order
        path.shift(); // to remove "toplevel"

        return path;
    };

    // Find the index of the child from the given node's children array
    sas.analytics.riskhp.models.CrosstabContext.prototype.findChildIndex = function (node, name) {
        if (node && name) {
            for (var i = 0; i < node.children.length; i++) {
                if (node.children[i].name === name) {
                    return i;
                }
            }
        }
        return -1;
    };


    sas.analytics.riskhp.models.CrosstabContext.prototype.getExpandExpressionKeys = function () {
        var self = this;
        var expansionkeys = [];

        var _walkExpansionsForKeys = function (expansionKeys, path, node) {
            if (node) {
                if (node.expanded === true)  {

                    // Take current path in the the tree and add in this node's label to create new expansion.
                    // Note: In indented mode, the last key does not contain the added space at the end.
                    var key = path.concat(["\'" + node.label + "\'"]);

                    // Need to exclude any drill path elements from key.  We need just the path elements that are
                    // visible in crosstab.
                    // NOTE: if key.length is less than drillpath.length, then this expansion isn't showing in
                    // crosstab, so skip it.
                    if (key.length > self.drillpath.length) {
                        for (var i = 0; i < self.drillpath.length; i++) {
                            key.shift();
                        }
                        expansionKeys.push(key);
                    }

                    // For indented mode, the key must include a '&' at the end to represent the
                    // label of the dummy/invisible parent displayTreeNode.  "hackish"
                    var nodePath = path.concat(["\'" + node.label + (self.getRowIndent() ? '&' : "") + "\'"]);

                    for (var idx = 0; idx < node.children.length; idx++) {
                        _walkExpansionsForKeys(expansionKeys, nodePath, node.children[idx]);
                    }
                }
            }
        };

        // if (self.reportResultsTree.children.length > 0) {
        //     for (var idx = 0; idx < self.reportResultsTree.children.length; idx++) {
        //         _walkExpansionsForKeys(expansionkeys, [], self.reportResultsTree.children[idx]);
        //     }
        // }
        if (self.drillnode.children.length > 0) {
            for (var idx = 0; idx < self.drillnode.children.length; idx++) {
                _walkExpansionsForKeys(expansionkeys, self.drillpath, self.drillnode.children[idx]);
            }
        }

        return expansionkeys;
    };

    // Return the depth of expansions based on the start node of the row tree.  Default to
    // the top of the row tree (self.reportResultsTree).
    sas.analytics.riskhp.models.CrosstabContext.prototype.getExpansionDepth = function (startNode) {
        var self = this;

        if (startNode === undefined) {
            startNode = self.reportResultsTree;
        }
        var expansionDepth = 1;

        var _walkExpansionsForDepth = function (depth, node) {
            if (node) {
                if (node.expanded === true)  {
                    expansionDepth = Math.max(depth + 1, expansionDepth);
                    for (var idx = 0; idx < node.children.length; idx++) {
                        _walkExpansionsForDepth(depth +1, node.children[idx]);
                    }
                }
            }
        };

        if (startNode.children.length > 0) {
            for (var idx = 0; idx < startNode.children.length; idx++) {
                _walkExpansionsForDepth(1, startNode.children[idx]);
            }
        }

        return expansionDepth;
    };

    // Return the depth of expansions based on the drill node of the row tree.  This effectively is
    // the expansion depth of what is currently displayed in the crosstab.
    sas.analytics.riskhp.models.CrosstabContext.prototype.getDrilledExpansionDepth = function () {
        var self = this;
        return self.getExpansionDepth(self.drillnode);
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getDataResultExpansionObject = function(idx) {
        var self = this;

        var expansion = {};
        var dataResultPathObject = this.getTreeNodePath(idx, this.drillnode);
        var fullpath = [].concat(dataResultPathObject.fullpath);
        var node = dataResultPathObject.node;
        var key = [];

        if (node) {
            if (node.name !== "toplevel") {
                for (var i = 0; i < fullpath.length; i++) {
                    var name = fullpath[i];
                    key.push({"name": self.classVariables[i].name, "id": self.classVariables[i].id, "value": name});
                }
            } else {
                fullpath = [node.name];
            }

            expansion = {
                name: this.createExpansionName(fullpath),
                key: key,
                startRow: node.startRow,
                maxRows: this.optionNROWS.value
            };
        }

        return {expansion: expansion, dataResultPathObject: dataResultPathObject};
    };


//*****************************
// Report/Query Results
//*****************************
//     sas.analytics.riskhp.models.CrosstabContext.prototype._createChildRowNode = function(node, key) {
//         return {
//             name: key,
//             label: key,
//             results: [],
//             expanded: false,
//             children: [],
//             parent: node,
//             startRow: 1,
//             rows : 1,
//             totalRows: 1
//         };
//     };
//
//     sas.analytics.riskhp.models.CrosstabContext.prototype._findChildRowNode = function(node, key) {
//         return _.find(node.children, function (cNode) {
//             if ((cNode.name === key)){
//                 return cNode;
//             }
//         });
//     };
//
//     sas.analytics.riskhp.models.CrosstabContext.prototype._findOrCreateChildRowNode = function(node, key) {
//         var childNode = this._findChildRowNode(node, key);
//         if (childNode === undefined) {
//             childNode = this._createChildRowNode(node, key);
//             //Add child into the tree
//             node.children.push(childNode);
//             //node.expanded = true;
//         }
//         return childNode;
//     };

    // Parse the key into its parts
    sas.analytics.riskhp.models.CrosstabContext.prototype._parseKey = function (obj) {
        var propertyArray = Object.keys(obj);
        var cvID = propertyArray[0];
        var cvValue = obj[cvID];

        return {id: cvID, value: cvValue};
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype._createChildResultsNode = function (result, key, parentNode) {
        var self = this;

        // Get name and label from the key
        // "key": {
        //    "0018l2gsh4z2": "Americas"
        // },
        if (key === undefined) {
            key = result.value;
        }
        var oKey = self._parseKey(key);
        var cvID = oKey.id;
        var cvValue = oKey.value;

        // extra label processing!!!!
        // if date or numeric, apply formatter
        var classvar = self.cubeDataModel.getDataItemById(cvID);
        var displayText = cvValue;
        switch (classvar.dataType) {
            case HprConstants.HPR_DATA_TYPE_DATE:
                displayText = sas.ltjs.sasicu.util.SASFormatterUtil.format(Number(cvValue), classvar.format);
                break;
            case HprConstants.HPR_DATA_TYPE_NUMBER:
                displayText = sas.ltjs.sasicu.util.SASFormatterUtil.format(Number(cvValue), classvar.format);
                break;
        }

        // Return a node
        return {
            name: cvValue,
            label: displayText,
            classvarID : cvID,
            parent : parentNode,
            results: (result ? result.data : undefined),
            expanded : false,
            children: [],
            startRow: 1,
            rows: 1,
            totalRows: 1
        };
    };


    sas.analytics.riskhp.models.CrosstabContext.prototype.setResults = function(results, rebuildResultsTree) {
        var self = this;

        if (results) {

            // Find the node associated with the path.  subPath is array of classvar values.
            var _findNode = function (subPath, node, cvIndex) {
                if (subPath.length > 0) {
                    //var oKey = self._parseKey(subPath);
                    if (subPath[0] !== SAS_ALL) {
                        for (var i = 0; i < node.children.length; i++) {
                            var childNode = node.children[i];
                            if (childNode.name === subPath[0]) {
                                subPath.shift();
                                return _findNode(subPath, childNode, ++cvIndex);
                            }
                        }
                        // Didn't find the childNode, create node and keep going
                        var key = {};
                        key[self.classVariables[cvIndex].id] = subPath[0];
                        var newChildNode = self._createChildResultsNode(undefined, key, node);
                        subPath.shift();
                        return _findNode(subPath, newChildNode, ++cvIndex);
                    }
                }
                return node;
            };

            // Did we get all the info that was requested?

            // Check to make sure that we have data for populating a drilled crosstab.  A filter may have
            // invalidated the drill.  S1376911.
            var drillIsSafe = false;
            for (var idx = 0; idx < results.length; idx++) {
                var path = results[idx].expansion.key;
                if (path[path.length -1] === SAS_ALL) {
                    path.pop(); // remove SAS_ALL
                    var node = self.findRowTreeNode(path);
                    var dNode = self.drillnode;
                    if (node === dNode) {
                        drillIsSafe = true;
                        break;
                    }
                }
            }

            //var numCVColumns = self.getClassVariables().length;

            // Reset the report results tree so it will be rebuilt fresh based on the results coming in.
            // Note: the query that created these new results is based on the report results tree that will now
            // be tossed.  This will be used when a column sort is changed and (possibly) rows will move around
            // and tossing the existing tree and rebuilding from scratch is the simplest way to do it.
            var savedExpansions = [];
            var savedDrillPath = [];
            if (rebuildResultsTree) {
                //Save expansions and drill
                savedExpansions = self.getExpansionPaths();
                if (drillIsSafe)
                    savedDrillPath = self.getDrillPath();
                self.resetReportResultsTree();
            }

            // Process each expansion
            for (var eIdx = 0; eIdx < results.length; eIdx++) {

                var expansionResult = results[eIdx];

                // Skip expansions that do not have data results.  Filtered out?
                if (expansionResult.data.length > 0) {
                    // expansion.key is [cvValue, cvValue, ...] NOT [{id: cvID, value: cvValue}, {id: cvID, value: cvValue},...]
                    var path = expansionResult.expansion.key.concat();

                    // Start at the root of the resultsTree and find node based on path
                    var resultsNode = _findNode(path, self.reportResultsTree, 0);

                    // Process each row of data from the expansion
                    for (var rIdx = 0; rIdx < expansionResult.data.length; rIdx++) {

                        if (expansionResult.data[rIdx].value) {
                            var ccValue = self._parseKey(expansionResult.data[rIdx].value).value;

                            // find the child that matches this row
                            var childNode = _.findWhere(resultsNode.children, {name: ccValue});
                            // Found child node so update the data in the child
                            if (childNode) {
                                childNode.results = expansionResult.data[rIdx].data;
                            }
                            // Didn't find the child node so create a new one
                            else {
                                childNode = self._createChildResultsNode(expansionResult.data[rIdx], undefined, resultsNode);
                                //resultsNode.children.push(childNode);
                                //This is backport from d4risk191 to f0risk181
                                resultsNode.children.splice(rIdx, 0, childNode);
                            }
                        }
                        // else value = undefined thus signifying "toplevel", store data in node
                        else {
                            resultsNode.results = expansionResult.data[rIdx].data;
                        }
                    } // each row of data

                    resultsNode.startRow = expansionResult.expansion.startRow;
                    resultsNode.rows = expansionResult.data.length;
                    resultsNode.totalRows = expansionResult.expansion.totalRows;
                }
            } // process each dataResults row

            //Apply saved expansions and drill
            if (rebuildResultsTree) {
                self.setExpansionPaths(savedExpansions);
                self.setDrillPath(savedDrillPath);
            }
        } // results
    };

    // sas.analytics.riskhp.models.CrosstabContext.prototype.setReportResults = function(reportResults, rebuildResultsTree) {
    //     var self = this;
    //
    //     if (reportResults) {
    //         var dataResults = reportResults.results.data;
    //         var dataLabels = reportResults.results.labels;
    //         var cardinalities = reportResults.results.cardinalities;
    //
    //         if ((dataResults) && (dataLabels)) {
    //
    //             // Did we get all the info that was requested?
    //
    //             // Check to make sure that we have data for populating a drilled crosstab.  A filter may have
    //             // invalidated the drill.  S1376911.
    //             var drillIsSafe = false;
    //             for (var cardIdx = 0; cardIdx < cardinalities.length; cardIdx++) {
    //                 var path = cardinalities[cardIdx].name.split(EXPANSION_NAME_SEPARATOR);
    //                 path.shift(); // remove the EXPANSION_NAME
    //                 if (path[path.length -1] === SAS_ALL) {
    //                     path.pop(); // remove SAS_ALL
    //                     var node = self.findRowTreeNode(path);
    //                     var dNode = self.drillnode;
    //                     if (node === dNode) {
    //                         drillIsSafe = true;
    //                     }
    //                 }
    //             }
    //
    //             var numCVColumns = self.getClassVariables().length;
    //
    //             // Reset the report results tree so it will be rebuilt fresh based on the results coming in.
    //             // Note: the query that created these new results is based on the report results tree that will now
    //             // be tossed.  This will be used when a column sort is changed and (possibly) rows will move around
    //             // and tossing the existing tree and rebuilding from scratch is the simplest way to do it.
    //             var savedExpansions = [];
    //             var savedDrillPath = [];
    //             if (rebuildResultsTree) {
    //                 //Save expansions and drill
    //                 savedExpansions = self.getExpansionPaths();
    //                 if (drillIsSafe)
    //                     savedDrillPath = self.getDrillPath();
    //                 self.resetReportResultsTree();
    //             }
    //
    //             // Process each row of data
    //             for (var rIdx = 0; rIdx < dataResults.length; rIdx++) {
    //                 var resultsRow = dataResults[rIdx].concat();
    //
    //                 // rowTree navigation key
    //                 var key = resultsRow.slice(0, numCVColumns);
    //
    //                 // Just the data, with navigation key removed.
    //                 resultsRow.splice(0, numCVColumns);
    //
    //                 // Start at the root of the rowTree
    //                 var rowNode = self.reportResultsTree;
    //                 for (var keyIdx = 0; keyIdx < key.length; keyIdx++) {
    //
    //                     var childNode = rowNode;
    //
    //                     // Stay on this node if a total/subtotal
    //                     if (key[keyIdx] !== -100) {
    //                         //Convert key to label
    //                         key[keyIdx] = dataLabels[key[keyIdx]];
    //
    //                         // extra label processing!!!!
    //                         // if date or numeric, apply formatter
    //                         var classvar = self.classVariables[keyIdx];
    //                         var displayText = key[keyIdx];
    //                         switch (classvar.dataType) {
    //                             case HprConstants.HPR_DATA_TYPE_DATE:
    //                                 displayText = sas.ltjs.sasicu.util.SASFormatterUtil.format(Number(displayText), classvar.format);
    //                                 break;
    //                             case HprConstants.HPR_DATA_TYPE_NUMBER:
    //                                 displayText = sas.ltjs.sasicu.util.SASFormatterUtil.format(Number(displayText), classvar.format);
    //                                 break;
    //                         }
    //
    //                         // Find the node for this key or create it.
    //                         childNode = self._findOrCreateChildRowNode(rowNode, key[keyIdx]);
    //
    //                         // Set the label
    //                         childNode.label = displayText;
    //                     }
    //
    //                     if (key[keyIdx] === -100) {
    //                         break;
    //                     }
    //
    //                     // set up for the next key to be processed (do it at this node)
    //                     rowNode = childNode;
    //                 }
    //
    //                 // Set the data results
    //                 rowNode.results = resultsRow;
    //
    //             } // process each dataResults row
    //
    //             //Apply saved expansions and drill
    //             if (rebuildResultsTree) {
    //                 self.setExpansionPaths(savedExpansions);
    //                 self.setDrillPath(savedDrillPath);
    //             }
    //
    //         }
    //
    //         if (cardinalities) {
    //             for (var cardIdx = 0; cardIdx < cardinalities.length; cardIdx++) {
    //                 var path = cardinalities[cardIdx].name.split(EXPANSION_NAME_SEPARATOR);
    //                 path.shift(); // remove the EXPANSION_NAME
    //                 var node = self.findRowTreeNode(path);
    //                 node.startRow = cardinalities[cardIdx].startRow;
    //                 node.rows = cardinalities[cardIdx].rows;
    //                 node.totalRows = cardinalities[cardIdx].totalRows;
    //             }
    //         }
    //
    //     }
    //
    //     var xyz = 0; //TEMP  here for a breakpoint.  Ok to remove.
    // };

    sas.analytics.riskhp.models.CrosstabContext.prototype.resetReportResultsTree = function () {
        var self = this;

        //self.reportResultsTree = {name: "toplevel", label: "toplevel", parent: undefined, results: [], expanded: false, children: [], startRow: 1, rows : 1, totalRows: 1};
        self.reportResultsTree = {name: "toplevel", label: "toplevel", parent: undefined, results: [], expanded: true, children: [], startRow: 1, rows : 1, totalRows: 1};
        self.clearDrillPath();
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getExpansionPaths = function () {
        var self = this;
        var expansions = [];

        var _walkNodesForExpansions = function (node, path, expansions) {
            if (node) {
                if (node.expanded === true)  {
                    expansions.push(path.concat());

                    for (var idx = 0; idx < node.children.length; idx++) {
                        var childPath = path.concat([node.children[idx].name]);
                        _walkNodesForExpansions(node.children[idx], childPath, expansions);
                    }
                }
            }
        };

        // Create a top level expansion
        var node = self.reportResultsTree;
        var path = [];
        expansions.push(path.concat());

        if (node.children) {
            for (var idx = 0; idx < node.children.length; idx++) {
                var childPath = path.concat([node.children[idx].name]);
                _walkNodesForExpansions(node.children[idx], childPath, expansions);
            }
        }

        return expansions;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setExpansionPaths = function (expansions) {
        var self = this;
        _.each(expansions, function(exp) {
            var node = self.findRowTreeNode(exp);
            if (node) {
                node.expanded = true;
            }
        });
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.convertRowKeyLabelToName=function(key) {
        var self = this;
        var node = self.findTreeNode(key, "label"); // The keys are labels
        var convertedKey = [];

        while(node && node.parent !== undefined) {
            convertedKey.push(node["name"]); // Get the name from the node
            node = node.parent;
        }
        return convertedKey.reverse();
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.findTreeNode=function(key, nodeProperty){
        var self = this;
        var node;

        //nodeProperty = "name" or "label" for our purposes.  So we are testing based off of if key is an array
        // of names or labels.  We can search the nodes based on either.
        if (nodeProperty === undefined) {
            nodeProperty = "name";
        }

        var _findNode = function(rowkeys, node) {

            if (rowkeys.length > 0) {
                for (var i = 0; i < node.children.length; i++) {
                    var childNode = node.children[i];
                    if (childNode[nodeProperty] === rowkeys[0]) {
                        rowkeys.shift();
                        return _findNode(rowkeys, childNode);
                    }
                }
                return undefined; // {}
            }
            return node;
        };

        if (this.reportResultsTree.children.length > 0) {
            node = _findNode(key.concat(), this.reportResultsTree);
        }

        return node;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getTreeNodePath=function(index, tree){

        var self = this;
        var currentIndex = 0;
        var nodePath = [];
        var nodeAtIndex = undefined;

        var _findNodeWithTotalsBefore = function(path, node) {
            if (node) {
                // If subtotals enabled or no subtotals be we are a leaf, then check
                if (self.isSubtotalsEnabled() || (!self.isSubtotalsEnabled() && node.children.length === 0)) {
                    if (currentIndex === index) {
                        nodePath = path.concat([node.name]);
                        nodeAtIndex = node;
                        return true;
                    }
                    currentIndex++;
                }
                if (node.children.length > 0 && node.expanded === true) {
                    for (var i = 0; i < node.children.length; i++) {
                        if (_findNodeWithTotalsBefore(path.concat([node.name]), node.children[i])) {
                            return true;
                        }
                    }
                }
            }
            return false;
        };

        var _findNodeWithTotalsAfter = function(path, node) {
            if (node) {
                if (node.children.length > 0 && node.expanded === true) {
                    for (var i = 0; i < node.children.length; i++) {
                        if (_findNodeWithTotalsAfter(path.concat([node.name]), node.children[i])) {
                            return true;
                        }
                    }
                }

                // If subtotals enabled or no subtotals be we are a leaf, then check
                if (self.isSubtotalsEnabled() || (!self.isSubtotalsEnabled() && node.children.length === 0)) {
                    if (currentIndex === index) {
                        nodePath = path.concat([node.name]);
                        nodeAtIndex = node;
                        return true;
                    }
                    currentIndex++;
                }
            }
            return false;
        };

        // Need to check total now?
        if (self.isTotalEnabled() && self.getTotalsLocation() === CrosstabUtil.TOTALS_LOCATION_BEFORE) {
            if (currentIndex === index) {
                nodeAtIndex = tree;
                return {path:[tree.name], fullpath:self.findRowTreePath(tree), node:nodeAtIndex};
            }
            currentIndex++;
        }

        // Go deep
        var found = false;
        for (var i = 0; i < tree.children.length; i++) {
            if (self.getTotalsLocation() === CrosstabUtil.TOTALS_LOCATION_BEFORE) {
                if (_findNodeWithTotalsBefore([].concat([tree.name]), tree.children[i])) {
                    found = true;
                    break;
                }
            }
            else {
                if (_findNodeWithTotalsAfter([].concat([tree.name]), tree.children[i])) {
                    found = true;
                    break;
                }
            }
        }

        // Need to check total now?
        if ((found === false) && self.isTotalEnabled() && (self.getTotalsLocation() === CrosstabUtil.TOTALS_LOCATION_AFTER)) {
            if (currentIndex === index) {
                nodePath = nodePath.concat([tree.name]);
                nodeAtIndex = tree;
            }
        }

        var fullpath = self.findRowTreePath(nodeAtIndex, "name");

        return {path:nodePath, fullpath: fullpath, node:nodeAtIndex};
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getDataResultObjects=function(){

        var self = this;
        var currentIndex = 0;
        var nodePath = [];
        var resultsObjects = [];
        var nodeAtIndex = undefined;

        var _findNodeWithTotalsBefore = function(path, node) {
            if (node) {
                // If subtotals enabled or no subtotals but we are a leaf, then check
                if (self.isSubtotalsEnabled() ||
                    (!self.isSubtotalsEnabled() && ((node.expanded === false) || (node.children.length === 0)))) {
                    nodePath = path.concat([node.name]);
                    nodeAtIndex = node;
                    resultsObjects.push({path: nodePath, index: currentIndex, results:node.results});
                    currentIndex++;
                }
                if (node.children.length > 0 && node.expanded === true) {
                    for (var i = 0; i < node.children.length; i++) {
                        _findNodeWithTotalsBefore(path.concat([node.name]), node.children[i]);
                    }
                }
            }
        };

        var _findNodeWithTotalsAfter = function(path, node) {
            if (node) {
                if (node.children.length > 0 && node.expanded === true) {
                    for (var i = 0; i < node.children.length; i++) {
                        _findNodeWithTotalsAfter(path.concat([node.name]), node.children[i]);
                    }
                }

                // If subtotals enabled or no subtotals but we are a leaf, then check
                if (self.isSubtotalsEnabled() ||
                    (!self.isSubtotalsEnabled() && ((node.expanded === false) || (node.children.length === 0)))) {
                    nodePath = path.concat([node.name]);
                    nodeAtIndex = node;
                    resultsObjects.push({path: nodePath, index: currentIndex, results:node.results});
                    currentIndex++;
                }
            }
        };

        // Handle the total/subtotal row if location is before.
        if (self.drillpath.length > 0) {
            if (self.isSubtotalsEnabled() && self.getTotalsLocation() === CrosstabUtil.TOTALS_LOCATION_BEFORE) {
                resultsObjects.push({path: [], index: currentIndex, results: self.drillnode.results});
                currentIndex++;
            }
        }
        else {
            if (self.isTotalEnabled() && self.getTotalsLocation() === CrosstabUtil.TOTALS_LOCATION_BEFORE) {
                resultsObjects.push({path: [], index: currentIndex, results:self.drillnode.results});
                currentIndex++;
            }
        }

        for (var i = 0; i < self.drillnode.children.length; i++) {
            if ((self.getTotalsLocation() === CrosstabUtil.TOTALS_LOCATION_BEFORE) || (self.getRowIndent() === true)) {
                _findNodeWithTotalsBefore([], self.drillnode.children[i]);
            }
            else {
                _findNodeWithTotalsAfter([], self.drillnode.children[i]);
            }
        }

        // Handle the total/subtotal row if location is after.
        if (self.drillpath.length > 0) {
            if (self.isSubtotalsEnabled() && self.getTotalsLocation() === CrosstabUtil.TOTALS_LOCATION_AFTER) {
                resultsObjects.push({path: [], index: currentIndex, results: self.drillnode.results});
                currentIndex++;
            }
        }
        else {
            if (self.isTotalEnabled() && self.getTotalsLocation() === CrosstabUtil.TOTALS_LOCATION_AFTER) {
                resultsObjects.push({path: [], index: currentIndex, results:self.drillnode.results});
                currentIndex++;
            }
        }

        return resultsObjects;
    };

    // Trim row tree to the given depth
    sas.analytics.riskhp.models.CrosstabContext.prototype.trimRowTree=function(depth){
        var self = this;
        var node;

        var _findNode = function(myDepth, node) {

            if (myDepth === depth) {
                node.children = [];
                node.expanded = false;
                //node.results = [];
                node.startRow = 1;
                node.rows = 1;
                node.totalRows=1;
            } else {
                for (var b = 0; b < node.children.length; b++) {
                    _findNode(myDepth + 1, node.children[b]);
                }
            }
        };

        if (depth === 0) {
            self.reportResultsTree.children = [];
            self.reportResultsTree.expanded = true;
            self.reportResultsTree.results = [];
            self.reportResultsTree.startRow = 1;
            self.reportResultsTree.rows = 1;
            self.reportResultsTree.totalRows=1;
        }
        else {
            for (var idx = 0; idx < self.reportResultsTree.children.length; idx++) {
                _findNode(1, self.reportResultsTree.children[idx]);
            }
        }
        //return node;
    };



//*****************************
// Helpers
//*****************************
    sas.analytics.riskhp.models.CrosstabContext.prototype._getDataItemLabels = function(dataItemArray) {
        // Retrieve the showLabels option setting so the labels displayed to the user will be based on the option
        var showLabels = this.getOptionShowLabels().value;
        return _.map(dataItemArray, function(dataItem){return dataItem.getDisplayLabel(showLabels);});
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype._getDataItemNames = function(dataItemArray) {
        return _.map(dataItemArray, function(dataItem){return dataItem.name;});
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype._getDataItemIds = function(dataItemArray) {
        return _.map(dataItemArray, function(dataItem){return dataItem.id;});
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype._createDataItemObjects = function(dataItemArray) {
        var self = this;
        return _.map(dataItemArray, function(dataItem){
            return self._createDataItemObject(dataItem);
        });
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype._createDataItemObject = function(dataItem) {
        if (dataItem) {
            var obj = {name: dataItem.name, id: dataItem.id, type: dataItem.type};
            if (dataItem.hasOwnProperty(HprConstants.BASECASE_PROPERTY)) {
                obj[HprConstants.BASECASE_PROPERTY] = dataItem.basecase;
            }
            return obj;
        } else return {};
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype._createDataItemObjectsWithId = function(dataItemArray, addLabel) {
        var self = this;
        if (addLabel === undefined) {
            addLabel = true;
        }
        return _.map(dataItemArray, function(dataItem){
            return self._createDataItemObjectWithId(dataItem, addLabel);
        });
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype._createDataItemObjectWithId = function(dataItem, addLabel) {
        var self = this;

        if (addLabel === undefined) {
            addLabel = true;
        }

        if (dataItem) {
            if (addLabel) {
                var showLabels = self.getOptionShowLabels().value;
                var label = dataItem.getDisplayLabel(showLabels);
                return {id: dataItem.id, label: label, type: dataItem.type};
            }
            else {
                return {id: dataItem.id, type: dataItem.type};
            }
        } else return {};
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getCashFlowLegs = function() {
        return this.cubeDataModel.cashFlowLeg;
    };




//*****************************
// Clone
//*****************************
    sas.analytics.riskhp.models.CrosstabContext.prototype.clone = function(newId){
        var self = this;
        var deferred = $.Deferred();

        var cloneContext = new sas.analytics.riskhp.models.CrosstabContext(newId);

        var json = this.toJSON();
        var dataSourceId = this.getDataSourceId();
        var dataModel = this.getCubeDataModel();
        var newOptionName = self.i18n.getText("HPRisk.Crosstab.cloned.txt") + " " + this.getOptionName();

        cloneContext.setDataSourceId(dataSourceId);
        cloneContext.setCubeDataModel(dataModel);

        $.when(
            // fromJSON may take a couple of seconds, so we need to wait on it
            cloneContext.fromJSON(null, json)
        ).done(function() {
                cloneContext.setOptionName(newOptionName);
                deferred.resolve(cloneContext);
            });

        return deferred.promise();
    };


//*****************************
// Options
//*****************************

    sas.analytics.riskhp.models.CrosstabContext.prototype.setDefaultOptions = function(defaultOptionsData, optionsName) {
        var optionsData = $.extend(true, {}, defaultOptionsData);

        // Default option properties provided by default Options cube model data
        this.optionALPHA = optionsData.alpha;
        this.optionPCTLDEF = optionsData.pctldef;
        this.optionNROWS = optionsData.nrows;
        this.optionHIERARCHY = optionsData.hierarchy;

        // Other properties needed for display in Options pane (others are initialized in constructor)
        this.optionName = optionsName;

        // totals, subtotals, and location are not in the default cube options either but they are needed
        // for queries, so we need to initialize them appropriately so they are created correctly for the UI model
        var totalsObject = {};
        totalsObject.name = HprConstants.HPR_OPTIONS_PANE_TOTALS_NAME;
        totalsObject.value = true;
        this.optionTOTALS = totalsObject;

        var subtotalsObject = {};
        subtotalsObject.name = HprConstants.HPR_OPTIONS_PANE_SUBTOTALS_NAME;
        subtotalsObject.value = true;
        this.optionSUBTOTALS = subtotalsObject;

        var locationObject = {};
        locationObject.name = HprConstants.HPR_OPTIONS_PANE_LOCATION_NAME;
        locationObject.value = 0;
        this.optionLOCATION = locationObject;

        // UI only options
        var transposeObject = {};
        transposeObject.name = HprConstants.HPR_OPTIONS_PANE_TRANSPOSE_NAME;
        transposeObject.value = false;
        this.optionTranspose = transposeObject;

        var indentRowsObject = {};
        indentRowsObject.name = HprConstants.HPR_OPTIONS_PANE_INDENT_ROWS_NAME;
        indentRowsObject.value = false;
        this.optionIndentRows = indentRowsObject;

        var showLabelsObject = {};
        showLabelsObject.name = HprConstants.HPR_OPTIONS_PANE_SHOW_LABELS_NAME;
        // For the Crosstab's default options, the default Show Labels value should come from the Application Settings
        showLabelsObject.value = ApplicationSettings.getShowLabels();
        this.optionShowLabels = showLabelsObject;

        var displayHorizonsObject = {};
        displayHorizonsObject.name = HprConstants.HPR_OPTIONS_PANE_DISPLAY_HORIZONS_NAME;
        // S1395005: For the Crosstab's default options, the default Display Horizons value should come from the
        // Application Settings except for Side by Side cubes which should default to Relative
        if (this.getCubeDataModel() && this.getCubeDataModel().isSideBySide()) {
            displayHorizonsObject.value = HprConstants.DISPLAY_HORIZONS_RELATIVE_VALUE;
        } else {
            displayHorizonsObject.value = ApplicationSettings.getDisplayHorizons();
        }
        this.optionDisplayHorizons = displayHorizonsObject;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.updateOptions = function(optionsData) {
        // Update the option properties with data passed in
        this.optionALPHA = optionsData.alpha;
        this.optionPCTLDEF = optionsData.pctldef;
        this.optionNROWS = optionsData.nrows;
        //this.optionHIERARCHY = optionsData.hierarchy;
        this.optionTOTALS = optionsData.totals;
        this.optionSUBTOTALS = optionsData.subtotals;
        this.optionLOCATION = optionsData.location;

        this.optionTranspose = optionsData.transpose;
        this.optionIndentRows = optionsData.indentRows;
        this.optionShowLabels = optionsData.showLabels;

        this.optionName = optionsData.name;
        this.optionApplyEnabled = optionsData.applyEnabled;
        this.optionGeneralOptionsSectionCollapsed = optionsData.generalOptionsSectionCollapsed;
        this.optionTotalsOptionsSectionCollapsed = optionsData.totalsOptionsSectionCollapsed;
        this.optionAdvancedQueryOptionsSectionCollapsed = optionsData.advancedQueryOptionsSectionCollapsed;

        if (this.optionDisplayHorizons.value !== optionsData.displayHorizons.value) {
            this.optionDisplayHorizons = optionsData.displayHorizons;
            // The horizons being displayed need to change from rel to abs or visa versa.
            // This will change the column tree accordingly.  Rebuild or redraw should be called
            // to see the results.
            this.updateColumnTree();
        }
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getOptions = function() {
        // Construct an object that mimics what the UI model needs in order to display properly
        var optionsObject = {};

        optionsObject.id = this.id;
        optionsObject.name = this.optionName;
        optionsObject.applyEnabled = this.optionApplyEnabled;
        optionsObject.generalOptionsSectionCollapsed = this.optionGeneralOptionsSectionCollapsed;
        optionsObject.totalsOptionsSectionCollapsed = this.optionTotalsOptionsSectionCollapsed;
        optionsObject.advancedQueryOptionsSectionCollapsed = this.optionAdvancedQueryOptionsSectionCollapsed;

        optionsObject.nrows = this.optionNROWS;
        optionsObject.alpha = this.optionALPHA;
        optionsObject.hierarchy = this.optionHIERARCHY;
        optionsObject.totals = this.optionTOTALS;
        optionsObject.subtotals = this.optionSUBTOTALS;
        optionsObject.location = this.optionLOCATION;
        optionsObject.pctldef = this.optionPCTLDEF;

        optionsObject.transpose = this.optionTranspose;
        optionsObject.indentRows = this.optionIndentRows;
        optionsObject.showLabels = this.optionShowLabels;

        optionsObject.displayHorizons = this.optionDisplayHorizons;

        return optionsObject;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getOptionName = function() {
        return this.optionName;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setOptionName = function(optionName) {
        this.optionName = optionName;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getOptionNROWS = function() {
        return this.optionNROWS;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getOptionHIERARCHY = function() {
        return this.optionHIERARCHY;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setOptionHIERARCHY = function(optionHierarchy) {
        this.optionHIERARCHY = optionHierarchy;

        // Publish an event that will update the name of the Crosstab
        // This will also remove the (Forced Hierarchy) indicator
        sas.eventBus.publish(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_CROSSTAB_OPTIONS_NAME_UPDATED, {
            crosstabId: this.id,
            newCrosstabName: this.getOptionName()
        });
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.isForcedHierarchyMode = function() {
        return (this.optionHIERARCHY && this.optionHIERARCHY.value && this.optionHIERARCHY.value.length > 0);
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setForcedHierarchyModeOff = function() {
        this.optionHIERARCHY.value = "";

        // Publish an event that will update the name of the Crosstab
        // This will also remove the (Forced Hierarchy) indicator
        sas.eventBus.publish(HprConstants.CHANNEL_OPTIONS_PANE, HprConstants.EVENT_CROSSTAB_OPTIONS_NAME_UPDATED, {
            crosstabId: this.id,
            newCrosstabName: this.getOptionName()
        });
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getOptionTOTALS = function() {
        return this.optionTOTALS;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getOptionSUBTOTALS = function() {
        return this.optionSUBTOTALS;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getOptionLOCATION = function() {
        return this.optionLOCATION;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getOptionALPHA = function() {
        return this.optionALPHA;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getOptionPCTLDEF = function() {
        return this.optionPCTLDEF;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getOptionTranspose = function() {
        return this.optionTranspose;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getOptionIndentRows = function() {
        return this.optionIndentRows;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setOptionIndentRows = function(optionIndentRows) {
        this.optionIndentRows = optionIndentRows;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getOptionShowLabels = function() {
        return this.optionShowLabels;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getOptionDisplayHorizons = function() {
        return this.optionDisplayHorizons;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setOptionGeneralOptionsSectionCollapsed = function(generalOptionsSectionCollapsed) {
        this.optionGeneralOptionsSectionCollapsed = generalOptionsSectionCollapsed;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setOptionTotalsOptionsSectionCollapsed = function(totalsOptionsSectionCollapsed) {
        this.optionTotalsOptionsSectionCollapsed = totalsOptionsSectionCollapsed;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setOptionAdvancedQueryOptionsSectionCollapsed = function(advancedQueryOptionsSectionCollapsed) {
        this.optionAdvancedQueryOptionsSectionCollapsed = advancedQueryOptionsSectionCollapsed;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getShowAxisHeaders = function() {
        var debugOptionsModel = sas.analytics.riskhp.models.HPRiskModelManager.getDebugOptions();
        if (debugOptionsModel === undefined) {
            return false;
        }
        return debugOptionsModel.options.showAxisHeaders;
    };

// sas.analytics.riskhp.models.CrosstabContext.prototype.getTranspose = function() {
//     var debugOptionsModel = sas.analytics.riskhp.models.HPRiskModelManager.getDebugOptions();
//     if (debugOptionsModel === undefined) {
//         return false;
//     }
//     return debugOptionsModel.options.transpose;
// }

//*****************************
// Filters
//*****************************

    sas.analytics.riskhp.models.CrosstabContext.prototype.setDefaultFilters = function(defaultFilters) {
        var processedDefaultFilters = $.extend(true, [], defaultFilters);

        _.each(processedDefaultFilters, function(clonedFilter) {
            clonedFilter.collapsed = false;
        });

        this.filters = processedDefaultFilters;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.setFilters = function(filters) {
        this.filters = filters;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.updateFilterSections = function(indexedFilterSectionData) {
        var self = this;
        var filterSection;
        _.each(self.filters, function(filter) {
            filterSection = indexedFilterSectionData[filter.id];
            if (filterSection) {
                filter.collapsed = filterSection.collapsed;
            }
        });
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.getFilters = function() {
        return $.extend(true, [], this.filters);
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.toFiltersJSON = function() {
        return this.filters;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.fromFiltersJSON = function(filterJSON) {
        this.filters = filterJSON;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.buildQueryFilterObjects = function() {
        var self = this;
        return _.map(self.filters, function(aFilter, index, list){
            return {id: aFilter.id, type: aFilter.type, includes: aFilter.includes, values: aFilter.values};
        });
    };


//*****************************
// JSON
//*****************************


//*****************************
// For saving to exploration
//*****************************

    sas.analytics.riskhp.models.CrosstabContext.prototype.toJSON = function() {

        // Timestamp shim in case browser is non-modern
        if (!Date.now) {
            Date.now = function() { return new Date().getTime(); };
        }

        var self = this;
        var jsonObject = {
            type : HprConstants.HPR_CROSSTAB_PANEL,
            name: self.id,
            dataSourceId: self.dataSourceId,

            hierarchy: {
                row: self.toRowHierarchyJSON(),
                column: self.toColumnHierarchyJSON()
            },

            headers : {
                row: self.toRowHeadersJSON(this.reportResultsTree),
                //column: self.toColumnHeadersJSON(this.columnTree)
                tuples: self.getTupleObjects(undefined, undefined, true)
            },

            classVariables : self.toClassVariablesJSON(),
            drillpath : self.toDrillPathJSON(),
            scrollPosition: self.getScrollPosition(),
            columnFormatOverride: self.toColumnFormatOverrideJSON(),
            options: self.toOptionsJSON(),
            filters: self.toFiltersJSON()
        };

        return jsonObject;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.toRowHierarchyJSON = function() {

        var rowHierarchyJSON = {
            name : this.getRowHierarchyName(),
            values : this._createDataItemObjects(this.getDrilledClassVariables())
        };

        return rowHierarchyJSON;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.toColumnHierarchyJSON = function() {

        var columnHierarchyJSON = {
            name : this.getColumnHierarchyName(),
            values : _.pluck(this.columnHierarchyMembers, "type")
        };

        return columnHierarchyJSON;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.toClassVariablesJSON = function() {

        return this._createDataItemObjects(this.getClassVariables());
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.toDrillPathJSON = function() {

        return this.getDrillPath();
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.toRowHeadersJSON = function(node) {

        //This is back ported from d4risk191 to f0risk181 for Mr. Applegate's
        //Flex exploration exporter
        var self = this;
        var nodeJSON = undefined;
        var childArray = [];

        if (node) {
            if (node.children.length > 0) {
                for (var i = 0; i < node.children.length; i++) {
                    var child = self.toRowHeadersJSON(node.children[i]);
                    if (child !== undefined) {
                        childArray.push(child);
                    }
                }
            }

            if (node.expanded === true) {
                nodeJSON = {
                    name: node.name,
                    label: node.label,
                    expanded: node.expanded,
                    children: childArray,
                    startRow: node.startRow,
                    rows: node.rows,
                    totalRows: node.totalRows,
                };
            }
        }

        return nodeJSON;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.toColumnHeadersJSON = function(node) {

        var self = this;
        var nodeJSON = {};
        var childArray = [];

        if (node) {
            if (node.children.length > 0) {
                for (var i = 0; i < node.children.length; i++) {
                    childArray.push(self.toColumnHeadersJSON(node.children[i]));
                }
            }

            nodeJSON = {
                name: node.name,
                item: self._createDataItemObject(node.item),
                children: childArray,
                sort: node.sort,
                width: node.width
            };
        }

        return nodeJSON;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.toOptionsJSON = function() {
        return {
            nrows : this.optionNROWS,
            alpha : this.optionALPHA,
            hierarchy : this.optionHIERARCHY,
            totals : this.optionTOTALS,
            subtotals : this.optionSUBTOTALS,
            location : this.optionLOCATION,
            pctldef : this.optionPCTLDEF,
            transpose : this.optionTranspose,
            indentRows : this.optionIndentRows,
            indentColumns : this.optionIndentColumns,
            showLabels : this.optionShowLabels,
            displayHorizons : this.optionDisplayHorizons,
            name : this.optionName,
            applyEnabled : this.optionApplyEnabled,
            generalOptionsSectionCollapsed : this.optionGeneralOptionsSectionCollapsed,
            totalsOptionsSectionCollapsed : this.optionTotalsOptionsSectionCollapsed,
            advancedQueryOptionsSectionCollapsed : this.optionAdvancedQueryOptionsSectionCollapsed
        };
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.toColumnFormatOverrideJSON = function() {

        var self = this;
        var columnFormatOverrideJSON = [];

        if (self.columnFormatOverride.length > 0) {
            for (var i = 0; i < self.columnFormatOverride.length; i++) {
                var override = self.columnFormatOverride[i];
                var overrideJSON = {
                    format: override.format
                };
                if (override.hasOwnProperty("compvar")) {
                    overrideJSON.compvar = override.compvar;
                } else if (override.hasOwnProperty("benchvar")) {
                    overrideJSON.benchvar = override.benchvar;
                } else {
                    overrideJSON.outvar = override.outvar;
                    overrideJSON.statistic = override.statistic;
                }
                columnFormatOverrideJSON.push(overrideJSON);
            }
        }

        return columnFormatOverrideJSON;
    };


//*****************************
// For recreating from exploration
//*****************************

    sas.analytics.riskhp.models.CrosstabContext.prototype.fromJSON = function(expJson, obj) {
        var self = this;
        var deferred = $.Deferred();

        var exp;
        if (obj) {
            exp = obj;
        } else {
            exp = JSON.parse(expJson);
        }

        if (exp.classVariables) {
            self.classVariables = [];
            _.each(exp.classVariables, function(item) {
                var classVar = CrosstabUtil.getDataItemById(item.id, item.type, self.cubeDataModel);
                if (classVar) {
                    self.classVariables.push(classVar);
                }
            });
        }

        if (exp.hierarchy) {
            self.setRowHierarchyName(exp.hierarchy.row.name);
            self.setColumnHierarchyName(exp.hierarchy.column.name);
            self.setColumnMembers(exp.hierarchy.column.values);
        }

        if (exp.headers) {
            self.reportResultsTree = self.fromRowHeadersJSON(exp.headers.row, undefined);

            // Must set the options before processing the tuple header as it will call addColumn
            // to create the column headers.  (displaying absolute or relative horizons based on
            // that option)
            if (exp.options) {
                self.fromOptionsJSON(exp.options);
            }

            // 'fromColumnHeaderJSON' my take a couple of seconds since he might have to hit the mid-tier, so we need to wait
            $.when(
                //self.fromColumnHeaderJSON(exp.headers.column)
                self.fromColumnTupleHeaderJSON(exp.headers)
            ).done(function(results) {
                if (exp.columnFormatOverride) {
                    self.fromColumnFormatOverrideJSON(exp.columnFormatOverride);
                }
                if (exp.scrollPosition) {
                    self.crosstabScrollPosition = exp.scrollPosition;
                }

                if (exp.filters) {
                    self.fromFiltersJSON(exp.filters);
                }

                if (exp.drillpath) {
                    self.setDrillPath(exp.drillpath);
                }

                deferred.resolve();
            }).fail(function() {
            // Needed for Remove Data Source so Crosstab name will appear correctly on redraw
            if (exp.options) {
                self.fromOptionsJSON(exp.options);
            }

            // Just resolve the Deferred here and let calling code deal with failure
                deferred.resolve();
            });
        }

        // In a clone situation, none of the properties exist but we still need to resolve the deferred
        if (!exp.classVariables && !exp.headers) {
            deferred.resolve();
        }

        return deferred.promise();
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.fromRowHeadersJSON = function(node, parent) {

        var self = this;
        var nodeJSON = {};
        var childArray = [];

        if (node) {
            nodeJSON = {
                name: node.name,
                label: node.label,
                expanded: node.expanded,
                parent: parent,
                startRow: node.startRow,
                rows: node.rows,
                totalRows: node.totalRows,
                results: []
            };

            if (node.children.length > 0) {
                for (var i = 0; i < node.children.length; i++) {
                    childArray.push(self.fromRowHeadersJSON(node.children[i], nodeJSON));
                }
            }

            nodeJSON.children = childArray;
        }

        return nodeJSON;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.fromColumnHeaderJSON = function(jsonColumnTree) {
        var self = this;

        // Create the wrapping deferred
        var returnDeferred = $.Deferred();

        var columnObjects = [];
        var sort = undefined;
        var width = undefined;
        var hidden = undefined;

        // Flatten tree to produce individual columns
        var _flattenTree = function (jNode, protoColumn) {
            if (jNode) {
                // TODO: dacagl - This could probably be extended or re-worked when we do work to compare the Data items
                // saved in an Exploration with the current state of the Data Source when we re-open an Exploration
                // Check to see if the Data item exists before adding it

                // WORKAROUND.  Since we don't have an ExplorationMigrationFunction(tm) as of f0risk181, we need to
                // ensure that basecase absolute horizons use the new id format with a prepended "basecase".
                // ie absoluteHorizon_2018-01-25  becomes basecaseabsoluteHorizon_2018-01-25.
                // See S1441466.
                if ((jNode.item.type === sas.analytics.riskhp.models.dataitems.AbsoluteHorizon.ABSOLUTEHORIZON)
                    && (jNode.item.basecase === true)) {
                    if (jNode.item.id.indexOf("basecase") < 0) {
                        jNode.item.id = "basecase" + jNode.item.id;
                        jNode.name = jNode.item.id;
                    }
                }

                // sort sort if the entry is not empty
                if ((jNode.sort !== undefined) && (jNode.sort.direction !== undefined)) {
                    sort = {direction: jNode.sort.direction, type: jNode.sort.type};
                }

                // width, if the entry is not empty
                if (jNode.width !== undefined) {
                    width = jNode.width;
                }

                // hidden, if the entry is not empty
                if (jNode.hidden !== undefined) {
                    hidden = true;
                }

                // Get the dataItem
                var dataItem = CrosstabUtil.getDataItemByIdType(jNode.item.id, jNode.item.type, jNode.item.basecase, self.cubeDataModel);
                if (dataItem) {
                    // Add dataItem to protoColumn to buildup to a full column
                    protoColumn.push(dataItem);

                    if (jNode.children.length > 0) {
                        _.each(jNode.children, function (child) {
                            var newProtoColumn = [];
                            $.extend(true, newProtoColumn, protoColumn);
                            _flattenTree(child, newProtoColumn);
                        });
                    }
                    else {
                        // Reached the bottom of a subtree so save the column information
                        columnObjects.push({column: protoColumn, sort: sort, width: width, hidden: hidden});
                        sort = undefined;
                        width = undefined;
                        hidden = undefined;
                    }
                }
                else {
                    // warning here?
                }
            }
        };

        // Start with just a root node of the column tree
        self.clearColumnTree();
        //self.columnTree = self.resetColumnModel();

        if (jsonColumnTree && jsonColumnTree.children && jsonColumnTree.children.length > 0) {
            // Ignore head node of the tree and process each child which will generate one or more columns
            $.each(jsonColumnTree.children, function(index, jChild) {
                _flattenTree(jChild, []);
                if (columnObjects && columnObjects.length > 0) {
                    // We have a batch of columns so add them to the crosstab.
                    $.each(columnObjects, function (index, columnObject) {
                        // Do not force default column ordering
                        self.addColumn(columnObject.column, columnObject.sort, columnObject.width, false);
                    });
                }
                columnObjects = [];  // reset for another child/subtree
            });
            returnDeferred.resolve();
        } else {
            // If there is no 'jsonColumnTree' to work with, just reject the Deferred; This can happen on replace data source
            returnDeferred.reject();
        }

        return returnDeferred;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.fromOptionsJSON = function(optionsJSON) {

        var opts = $.extend(true, {}, optionsJSON); // Make a copy of the original Options

        this.optionNROWS = opts.nrows;
        this.optionALPHA = opts.alpha;
        this.optionHIERARCHY = opts.hierarchy;
        this.optionTOTALS = opts.totals;
        this.optionSUBTOTALS = opts.subtotals;
        this.optionLOCATION = opts.location;
        this.optionPCTLDEF = opts.pctldef;
        this.optionTranspose = opts.transpose;
        this.optionIndentRows = opts.indentRows;
        this.optionIndentColumns = opts.indentColumns;
        this.optionShowLabels = opts.showLabels;
        this.optionDisplayHorizons = opts.displayHorizons;
        this.optionName = opts.name;
        this.optionApplyEnabled = opts.applyEnabled;
        this.optionGeneralOptionsSectionCollapsed = opts.generalOptionsSectionCollapsed;
        this.optionTotalsOptionsSectionCollapsed = opts.totalsOptionsSectionCollapsed;
        this.optionAdvancedQueryOptionsSectionCollapsed = opts.advancedQueryOptionsSectionCollapsed;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.fromColumnFormatOverrideJSON = function(columnFormatOverrideJSON) {

        var self = this;
        self.columnFormatOverride = [];

        if (columnFormatOverrideJSON.length > 0) {
            for (var i = 0; i < columnFormatOverrideJSON.length; i++) {
                var override = columnFormatOverrideJSON[i];
                var overrideJSON = {
                    format: override.format
                };
                if (override.hasOwnProperty("compvar")) {
                    overrideJSON.compvar = override.compvar;
                } else if (override.hasOwnProperty("benchvar")) {
                    overrideJSON.benchvar = override.benchvar;
                } else {
                    overrideJSON.outvar = override.outvar;
                    overrideJSON.statistic = override.statistic;
                }
                self.columnFormatOverride.push(overrideJSON);
            }
        }

    };

    /**
     * A method which checks to see if the existing content object references any of the data source ids by looking
     * at the "dataSourceId" property.
     * @param { string } currentContentData - The JSON object representing the current state of a panel
     * @param { [string] } dataSourceIds - A list of data source ids
     */
    sas.analytics.riskhp.models.CrosstabContext.prototype.areDataSourceIdsBeingReferenced = function(currentContentData, dataSourceIds) {
        return _.contains(dataSourceIds, currentContentData.dataSourceId);
    };

    /**
     * pseudo static method that takes a list of Data Source ids which are being removed from an Exploration and determines
     * if this model references one of them; If it does a Zero State object is returned
     * @param { string } currentContentData - The JSON object representing the current state of a panel
     * @param { [string] } dataSourceIdsBeingRemoved - A list of data source ids that are being removed
     * @returns {*}
     */
    sas.analytics.riskhp.models.CrosstabContext.prototype.removeDataSource = function(currentContentData, dataSourceIdsBeingRemoved) {
        // Create the wrapping deferred
        var returnDeferred = $.Deferred();

        // Check to make sure the current data source id is contained in the list of data source ids being replaced,
        // otherwise there is nothing to do so return the provided currentContentData.
        var currentDataSourceId = currentContentData.dataSourceId;
        if (!_.contains(dataSourceIdsBeingRemoved, currentDataSourceId)) {
            returnDeferred.resolve(currentContentData);
            return returnDeferred;
        }

        // Construct an object which represents the JSON for a Crosstab which will display as a Zero State control;
        // The most important part is that this JSON should allow the control to make it through all the code and model checks
        // to actually display something; That is why all properties are represented even though they are empty
        var zeroStateObject = {
            type: HprConstants.HPR_CROSSTAB_PANEL,
            name: currentContentData.name,
            dataSourceId: "",
            classVariables: [],
            hierarchy: {
                row: { name: "", values: [] },
                column: { name: "", values: [] }
            },
            headers: {
                row: { children: [] },
                column: { children: [] }
            },
            columnFormatOverride: [],
            drillpath: [],
            options: {
                // So the name of the ZeroState Crosstab will be the same
                name: currentContentData.options.name,

                // These options are needed to build a valid cache key
                alpha: { name: "", value: "" },
                pctldef: { name: "", value: "" },
                hierarchy: { name: "", value: "" },
                nrows: { name: "", value: "" }
            },
            filters: []
        };

        returnDeferred.resolve(zeroStateObject);
        return returnDeferred;
    };

    /**
     * pseudo static method that takes an existing content object and changes it in a way to represent a new data source.
     * This function should select an equivalent data source id from the list of newDataSourceIds for the data source
     * id the currentContentData is representing.
     *
     * This method must return a jQuery.Deferred object that behaves as follows:
     * - Returns the JSON to represent the panel's new state regardless of whether the panel should change because of the new data source
     * - ALWAYS resolve successfully.
     * - May make subsequent/wrapped ajax calls but when it returns all processing should be complete.
     * - May not change the state of the model instance or attempt to interact with the UI.
     *
     * @param { string } currentContentData - The JSON object representing the current state of a panel
     * @param { [string] } dataSourceIdsBeingReplaced - A list of data source ids that are being replaced
     * @param { [string] } newDataSourceIds - A list of replacement data source ids
     * @returns {*}
     */
    sas.analytics.riskhp.models.CrosstabContext.prototype.replaceDataSource = function(currentContentData, dataSourceIdsBeingReplaced, newDataSourceIds) {
        // Create the wrapping deferred
        var returnDeferred = $.Deferred();
        var self = this;

        // Check to make sure the current data source id is contained in the list of data source ids being replaced,
        // otherwise there is nothing to do so return the provided currentContentData.
        var currentDataSourceId = currentContentData.dataSourceId;
        if (!_.contains(dataSourceIdsBeingReplaced, currentDataSourceId)) {
            returnDeferred.resolve(currentContentData);
            return returnDeferred;
        }

        // Construct an object which represents the JSON for a Crosstab which will display as a Zero State control;
        // The most important part is that this JSON should allow the control to make it through all the code and model checks
        // to actually display something; That is why all properties are represented even though they are empty
        var zeroStateObject = {
            type: HprConstants.HPR_CROSSTAB_PANEL,
            name: currentContentData.name,
            dataSourceId: "",
            classVariables: [],
            hierarchy: {
                row: { name: "", values: [] },
                column: { name: "", values: [] }
            },
            headers: {
                row: { children: [] },
                column: { children: [] },
                tuples: []
            },
            columnFormatOverride: [],
            drillpath: [],
            options: {
                // So the name of the ZeroState Crosstab will be the same
                name: currentContentData.options.name,

                // These options are needed to build a valid cache key
                alpha: { name: "", value: "" },
                pctldef: { name: "", value: "" },
                hierarchy: { name: "", value: "" },
                nrows: { name: "", value: "" }
            },
            filters: []
        };

        // Get a compatible data source id from the list of newDataSourceIds.  If one is not found then there is
        // nothing to do so return a ZeroState object
        var newDataSourceId = HprUtil.getCompatibleDataSourceId(currentDataSourceId, newDataSourceIds);
        if (!newDataSourceId) {
            returnDeferred.resolve(zeroStateObject);
            return returnDeferred;
        }

        // Store any deferreds from requests needed to update for the new data source here. After all items are
        // processed, this set of deferreds will be passed into a when function, whereupon their completion, the
        // master deferred will be resolved (if we get that far, of course).
        var pendingRequests = [];

        // Initial setup checks are done, create the new content data object
        var newContentData = {};

        // 'type' stays the same
        newContentData.type = currentContentData.type;

        // 'name' stays the same
        newContentData.name = currentContentData.name;

        // Change to use new Data Source id
        newContentData.dataSourceId = newDataSourceId;

        // Grab the data source - should already be loaded
        var deferred = DataSourceCacheManager.getDataSource(newDataSourceId);
        deferred.done(function(targetDataSource) {

            // flag to capture the overall status of the validation
            var passedValidation = true;

            // variable to capture the first error message for why validation failed
            var firstErrorMessage = null;

            /**
             * Internal helper function to collect the two steps of setting the error message and setting the overall validation
             * status when an error condition is detected
             * @param message
             */
            var setErrorMessage = function(message) {
                if (firstErrorMessage === null) {
                    firstErrorMessage = message;
                    passedValidation = false;
                }
            };

            // Validate Class Variables
            // Helper function to determine if class variables exist in the new target data source
            var findValidClassVariables = function(targetDS, dataItemList) {
                var validClassVariables = [];

                // Return empty array if no targetDS or dataItemList
                if (!targetDS || !dataItemList) {
                    return validClassVariables;
                }

                // See if the class variable data item exists in targetDS; if so, use it
                _.each(dataItemList, function(candidateDataItem) {
                    var dataItem = targetDS.getDataItemById(candidateDataItem.id);
                    if (dataItem) {
                        validClassVariables.push(candidateDataItem); // Return JSON object and not real Data Item
                    }
                });

                return validClassVariables;
            };
            // Use the class variables list from the current content to see if they exist in the new target data source
            var validClassVariables = findValidClassVariables(targetDataSource, currentContentData.classVariables);
            if (validClassVariables.length > 0) {
                // Found some common class variables used in both the current data source and new target data source
                newContentData.classVariables = validClassVariables;
            } else {
                setErrorMessage("Could not find compatible class variables in new data source, using default class variables for new data source");
                // If no Class Variables were found, use the default Class Variables from the new Data Source
                newContentData.classVariables = _.map(targetDataSource.defaults.classVariables, function (id) {
                    // Don't want to use real Data Item here, just a shorthand representation like 'currentContentData.classVariables' would be
                    var dataItem = targetDataSource.getDataItemById(id);
                    return {name: dataItem.getName(), id: dataItem.getId(), type: dataItem.getType()};
                });
            }

            // If there were no compatible class variables found, we can't continue with anything else so return Zero State
            if (!newContentData.classVariables || (newContentData.classVariables && newContentData.classVariables.length === 0)) {
                console.error("No compatible class variables found, returning zero state");
                returnDeferred.resolve(zeroStateObject);
                return returnDeferred;
            }

            // Validate hierarchy
            // No real validation needed so just construct basic hierarchy object
            newContentData.hierarchy = {};
            newContentData.hierarchy.row = {};
            newContentData.hierarchy.row.name = currentContentData.hierarchy.row.name;
            newContentData.hierarchy.row.values = self._createDataItemObjects(newContentData.classVariables);
            newContentData.hierarchy.column = {};
            newContentData.hierarchy.column.name = currentContentData.hierarchy.column.name;
            // When saving a Crosstab, it appears 'hierarchy.column.values' is always the default tuple hierarchy no matter what the columns are so just use that here
            newContentData.hierarchy.column.values = targetDataSource.defaults.tupleHierarchy;

            // Validate headers

            // Helper function which takes 'headers' from the current content and tries to produce valid tuples JSON for the new target data source
            var generateValidHeadersTuplesJSON = function(targetDS, currentCDHeaders, currentCDOptions) {

                var validTuplesForTargetDS = [];
                var defaultTuplesForTargetDS = [];
                // Determine if we can use the headers column json passed in;
                // If not just return defaults of new data source.
                if (!currentCDHeaders || !currentCDHeaders.tuples) {
                    _.each(targetDS.defaults.tuples, function (defaultTuple) {
                        defaultTuplesForTargetDS.push({tuple: [].concat(defaultTuple.dataItems)});
                    });
                    return defaultTuplesForTargetDS;
                }

                // Loop through the array of tuples to try and produce valid tuples for the new target data source
                _.each(currentCDHeaders.tuples, function(tuple) {

                    var newTuple = [];
                    var newHydratedTuple = [];
                    var riskCubeAdded = false;

                    // Loop through the individual dehydrated dataItem which consist of data item objects {id: "id", type: "type"}
                    _.each(tuple.tuple, function(dehydratedDataItem) {

                        // See if data item exists in target data source
                        var hydratedDataItem = CrosstabUtil.getDataItemByIdType(dehydratedDataItem.id, dehydratedDataItem.type, undefined, targetDS);
                        if (hydratedDataItem) {
                            var targetDSDataItem = {id: hydratedDataItem.id, type: hydratedDataItem.type};

                            // Since all columns (except Computed Variables and Benchmarks) require a Risk Cube as the first data item in a column,
                            // add the first Risk Cube in the target data source to the new column (this sets up the column correctly for Side by Side cubes)
                            if (!riskCubeAdded && (dehydratedDataItem.type !== COMPVAR && dehydratedDataItem.type !== BENCHVAR)) {
                                // If the data item is a Risk Cube and it exists in the target data source, use it; otherwise, just use the first Risk Cube
                                if (dehydratedDataItem.type === RISKCUBE && hydratedDataItem) {
                                    newTuple.push(dehydratedDataItem);
                                    newHydratedTuple.push(hydratedDataItem);
                                } else {
                                    newTuple.push({id: targetDS.riskCube[0].id, type: targetDS.riskCube[0].type}); // Take the first one
                                    newHydratedTuple.push(targetDS.riskCube[0]);
                                }
                                riskCubeAdded = true;
                            }

                            // Data item can be undefined but try to look for an existing Computed Variable or Benchmark in the target data source to use
                            if (!hydratedDataItem && (dehydratedDataItem.type === RISKCUBE ||
                                dehydratedDataItem.type === COMPVAR || dehydratedDataItem.type === BENCHVAR)) {
                                var dataItemsForType = targetDS.getDataItemsForType(dehydratedDataItem.type);
                                if (dataItemsForType && dataItemsForType.length > 0) {
                                    targetDSDataItem = {id: dataItemsForType[0].id, type: dataItemsForType[0].type}; // Take the first one
                                    newHydratedTuple.push(dataItemsForType[0]);
                                } else {
                                    targetDSDataItem = undefined;
                                }
                            }

                            // If we are looking to add a Risk Cube, make sure we have not added one already and that we have more than 1 in target data source;
                            // Otherwise, we get unwanted default Risk Cube in new column
                            if (dehydratedDataItem.type === RISKCUBE) {
                                var validRiskCubes = targetDS.getValidRiskCubes();
                                if (!riskCubeAdded && validRiskCubes && validRiskCubes.length > 1) {
                                    // This will by-pass adding a Data Item for the targetDS when it only has 1 Risk Cube (aka. default Risk Cube)
                                    newTuple.push(targetDSDataItem);
                                }
                            } else {
                                // Push data item to new column even if undefined; They will be removed later
                                newTuple.push(targetDSDataItem);
                                newHydratedTuple.push(hydratedDataItem);
                            }
                        }
                    });

                    // Check each data item for the new column to make sure all exists; This will remove new columns with undefined data items
                    var newTupleHasAllDataItems = _.every(newTuple, function(dataItem) { return dataItem && dataItem.id; });
                    if (newTupleHasAllDataItems) {

                        // Check the restrictions for the new column to make sure it passes for the new target data source
                        var passRestrictionCheck = targetDS.areDataItemsCompatible(newHydratedTuple);
                        if (passRestrictionCheck) {
                            // Only use the new column if it passes restriction check
                            validTuplesForTargetDS.push({tuple: newTuple, width: tuple.width});
                        }
                    }
                });

                // No valid tuple could be found for target data source so return defaults - UNCOMMENT if desired
                if (validTuplesForTargetDS.length === 0) {
                    jQuery.sap.log.warning('Could not propogate crosstab columns to new data source, using default columns');
                //     _.each(targetDS.defaults.tuples, function (defaultTuple) {
                //         defaultTuplesForTargetDS.push({tuple: [].concat(defaultTuple.dataItems)});
                //     });
                //     return defaultTuplesForTargetDS;
                }

                return validTuplesForTargetDS;
            };

            // Create a headers object
            newContentData.headers = {};

            // Generate a basic header row; Taken from resetReportResultsTree()
            newContentData.headers.row = {name: "toplevel", label: "toplevel", parent: undefined, results: [], expanded: true, children: [], startRow: 1, rows : 1, totalRows: 1};

            // Generate the header columns by using the target data source and headers from current content
            newContentData.headers.tuples = generateValidHeadersTuplesJSON(targetDataSource, currentContentData.headers, currentContentData.options);

            // Validate columnFormatOverride
            // Ex: "[{"format":"COMMA.10","outvar":"046hkoffqd","statistic":"096hkoffqd"},{...}]"
            var columnFormatOverrideValidator = function(targetDS, columnFormatOverride) {
                var validOverrides = _.filter(columnFormatOverride, function(override) {
                    if (override.hasOwnProperty("compvar")) {
                        return targetDS.getDataItemById(override.compvar);
                    } else if (override.hasOwnProperty("benchvar")) {
                        return targetDS.getDataItemById(override.benchvar);
                    } else {
                        return targetDS.getDataItemById(override.outvar) && targetDS.getDataItemById(override.statistic);
                    }
                });
                return validOverrides;
            };
            newContentData.columnFormatOverride = columnFormatOverrideValidator(targetDataSource, currentContentData.columnFormatOverride);

            // Not sure how this will ever work; The value looks like data for a Class Variable but we would have to query and get data back for the new Crosstab in order
            // to see if we could use the same drillpath; Just set to empty Array for now
            // Ex: currentContentData.drillPath = ["Calgary"];
            newContentData.drillpath = [];

            // options stay the same except hierarchy (S1406391), we need to check it
            var hierarchyOptionsValueValidator = function(optionsHierarchy) {
                var value = optionsHierarchy.value;
                var hierarchyDataItem = targetDataSource.getDataItemByNameAndType(optionsHierarchy.value, sas.analytics.riskhp.models.dataitems.Hierarchy.HIERARCHY);
                if (!hierarchyDataItem) {
                    // If hierarchy data item does not exist in targetDataSource, use empty string to signal no forced hierarchy
                    value = "";
                }
                return value;
            };
            newContentData.options = HprUtil.hprClone(currentContentData.options);
            newContentData.options.id = currentContentData.name; // Have to set Options id so Options model can be found
            newContentData.options.hierarchy.value = hierarchyOptionsValueValidator(currentContentData.options.hierarchy);

            // validate the filters
            var validateFiltersDeferred = HprUtil.validateFilters(currentContentData.filters, targetDataSource, {}).done(
                function(filters) {
                    newContentData.filters = filters;
                }
            );
            pendingRequests.push(validateFiltersDeferred);

            if (!passedValidation) {
                // Just output the error message
                console.error(firstErrorMessage);
            }

            // Always resolve the deferred whether it passed validation or not.
            // once all the requests have finished, we can return
            $.when.apply(self, pendingRequests).always(
                function() {
                    returnDeferred.resolve(newContentData);
                }
            );
        });
        deferred.fail(function (jqxhr, textStatus, error) {
            // Failure case where we create the zero state json and resolve the deferred successfully
            jQuery.sap.log.error('Request Failed. Response text: ' + jqxhr.responseText);
            returnDeferred.resolve(zeroStateObject);
        });

        return returnDeferred;
    };

//*****************************
// Create the json for export
//*****************************

    sas.analytics.riskhp.models.CrosstabContext.prototype.exportJSONRequest = function(forceQueryAll) {
        var self = this;
        var exportRequest = {};

        if (forceQueryAll === undefined) {
            forceQueryAll = false;
        }

        // For now use the query request as the export JSON.  The query request has been modified to carry
        // label and format information that is need for export, yet does not impact query.
        exportRequest = self.createQueryRequest();

        // Need export "header information"
        exportRequest.export = {
            name: HprUtil.getBundle().getText("Hpr.Export.WorksheetName.label.fmt", [1]),  // always sheet 1 for now
            type: HprConstants.HPR_EXPORT_FORMAT_EXCEL,
            classVariables : self.buildExportCVLabelsJSON(),
            tuples : self.buildExportColumnsJSON(),
            drill : self._buildDrillExportJSON(),
            options : {
                queryAll : forceQueryAll,
                totals: self.optionTOTALS.value,
                subTotals: self.optionSUBTOTALS.value,
                location: self.optionLOCATION.value,
                transpose: self.optionTranspose.value,
                indented: self.optionIndentRows.value,
                totalsLabel : HprUtil.getBundle().getText("HPRisk.Crosstab.Total.txt"),
                subTotalsLabel : HprUtil.getBundle().getText("HPRisk.Crosstab.Subtotal.txt")
            },
            headers : []

        };

        exportRequest.export.headers.push({
            key: HprConstants.HPR_EXPORT_TABLE_NAME_KEY,
            name: HprUtil.getBundle().getText("Hpr.Export.TableName.label.txt"),
            value: self.optionName
        });
        exportRequest.export.headers.push({
            key: HprConstants.HPR_EXPORT_TABLE_TYPE_KEY,
            name: HprUtil.getBundle().getText("Hpr.Export.TableType.label.txt"),
            value: HprUtil.getBundle().getText("Hpr.Export.TableType.Crosstab.label.txt")
        });
        exportRequest.export.headers.push({
            key: HprConstants.HPR_EXPORT_DATA_SOURCE_NAME_KEY,
            name: HprUtil.getBundle().getText("Hpr.Export.DataSourceName.label.txt"),
            value: self.cubeDataModel.dataSource.label
        });
        exportRequest.export.headers.push({
            key: HprConstants.HPR_EXPORT_DATASOURCE_CREATION_DATE_KEY,
            name: HprUtil.getBundle().getText("Hpr.Export.DataSourceCreationDate.label.txt"),
            value: HprUtil.formatDateTime(self.cubeDataModel.getCreatedDate())
        });
        exportRequest.export.headers.push({
            key: HprConstants.HPR_EXPORT_EXPORT_CREATION_DATE_KEY,
            name: HprUtil.getBundle().getText("Hpr.Export.ExportCreationDate.label.txt"),
            value: HprUtil.formatDateTime(Date.now())
        });
        var drillValues = self._buildPortfolioDrillHeader();
        if (drillValues !== undefined && drillValues.length > 0) {
            exportRequest.export.headers.push({
                key: HprConstants.HPR_EXPORT_PORTFOLIO_DRILL_KEY,
                name: HprUtil.getBundle().getText("Hpr.Export.PortfolioDrill.label.txt"),
                value: drillValues
            });
        }
        // This entry must be last.
        exportRequest.export.headers.push({
            key: HprConstants.HPR_EXPORT_TABLE_DATA_KEY,
            name: HprUtil.getBundle().getText("Hpr.Export.TableData.label.txt"),
            value: ""
        });


        return exportRequest;
    };

//*****************************
// Create the request to populate crosstab
//*****************************

    sas.analytics.riskhp.models.CrosstabContext.prototype.createQueryRequest = function(forceFullQuery) {

        // Timestamp shim in case browser is non-modern
        if (!Date.now) {
            Date.now = function() { return new Date().getTime(); };
        }

        var self = this;

        // Guard against making a query if some key values are missing; These are possible when replace data source on Crosstab is performed
        if (!self.classVariables || (self.classVariables && self.classVariables.length === 0) || _.isUndefined(self.optionTranspose.value)) {
            return null;
        }

        var jsonObject = {
            "version": 1,
            "id": self.id,
            "cacheId": self.id,
            "type": "crosstab",
            "orientation": (self.optionTranspose.value ? "row" : "column"),
            "dataSource": self.cubeDataModel.queryJSON(),

            "dataDefinitions": {
                "classVariables": self.buildClassVariablesJSON(),
            },

            "report": {
                "sorts": self.buildSortJSON(),
                "options": self.buildOptionsJSON(false),
                "filters": self.buildQueryFilterObjects(),
                "requests": self.buildExpansionsJSON(forceFullQuery),
            }
        };

        return jsonObject;
    };


//*****************************
// Helpers for query and export
//*****************************

    sas.analytics.riskhp.models.CrosstabContext.prototype._buildPortfolioDrillHeader = function() {
        var self = this;
        var exportDrill = "";

        var drillPath = self.getDrillPath();
        for (var i = 0; i < drillPath.length; i++) {
            exportDrill = exportDrill + "[" + self.getClassVariable(i).getDisplayLabel(self.optionShowLabels.value) + ":" + drillPath[i] + "]";
            if (i < drillPath.length - 1) {
                exportDrill = exportDrill + ">";
            }
        }

        return exportDrill;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype._buildDrillExportJSON = function() {
        var self = this;
        var exportDrill = [];

        var drillPath = self.getDrillPath();
        for (var i = 0; i < drillPath.length; i++) {
            exportDrill.push({id : self.getClassVariable(i).id,  value:  drillPath[i]});
        }

        return exportDrill;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.buildClassVariablesJSON = function() {
        var self = this;
        return self._getDataItemIds(self.classVariables);
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.buildExportCVLabelsJSON = function() {
        var self = this;
        var labels = [];
        _.each(self.classVariables, function(cv) {
            var item = {};
            item[cv.getId()] = cv.getDisplayLabel(self.optionShowLabels.value);
            labels.push({
                id: cv.getId(),
                label: cv.getDisplayLabel(self.optionShowLabels.value),
                format: cv.getFormat(),
            });
        });
        return labels;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.buildColumnsJSON= function() {
        var self = this;
        var columnObjects = self.getColumnObjects();

        return _.map(columnObjects, function(columnObject, index, list){
            var format = sas.analytics.riskhp.models.dataitems.BaseDataItem.DEFAULT_FORMAT;
            var column = self.fixColumnToBaseHorizon(columnObject.column);
            var formatDataItem = self.getFormatDataItemFromColumn(column);
            if (formatDataItem) {
                format = formatDataItem.format;
            }
            return { "name" : "column" + index, "format" : format, "width" : columnObject.width,
                "dataItems" : self._createDataItemObjectsWithId(column)};
        });
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.buildExportColumnsJSON= function() {
        var self = this;
        var columnObjects = self.getColumnObjects();

        return _.map(columnObjects, function(columnObject, index, list){
            var format = sas.analytics.riskhp.models.dataitems.BaseDataItem.DEFAULT_FORMAT;
            var column = self.fixColumnToBaseHorizon(columnObject.column);

            // Make sure there is a risk cube data item in the column
            var includeRCLabel = true;
            var riskCubeId = undefined;
            var riskcubeDataItemObject = self.getDataItemInfoFromTuple(column,
                sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE);
            if (riskcubeDataItemObject === undefined) {
                riskCubeId = self.cubeDataModel.riskCube[0].id;
                var riskCube = CrosstabUtil.getDataItemById(riskCubeId,
                    sas.analytics.riskhp.models.dataitems.RiskCube.RISKCUBE,
                    self.cubeDataModel);
                column.unshift(riskCube);
                includeRCLabel = false;
            }

            var formatOverride = self.getColumnFormatOverride(column);
            if (formatOverride !== undefined) {
                format = formatOverride;
            }
            else {
                var formatDataItem = self.getFormatDataItemFromColumn(column);
                if (formatDataItem !== undefined) {
                    format = formatDataItem.format;
                }
            }
            return { "name" : "column" + index, "format" : format, "width" : columnObject.width,
                "dataItems" : self._createExportDataItemObjects(column, includeRCLabel)};
        });
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype._createExportDataItemObjects = function(dataItemArray, includeRiskCubeLabel) {
        var self = this;
        if (includeRiskCubeLabel === undefined) {
            includeRiskCubeLabel = true;
        }
        return _.map(dataItemArray, function(dataItem){
            return self._createExportDataItemObject(dataItem, (dataItem.type === RISKCUBE ? includeRiskCubeLabel : true));
        });
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype._createExportDataItemObject = function(dataItem, includeLabel) {
        var self = this;

        if (includeLabel === undefined) {
            includeLabel = true;
        }

        if (dataItem) {
            if (includeLabel) {
                var showLabels = self.getOptionShowLabels().value;
                var label = dataItem.getDisplayLabel(showLabels);
                if (dataItem.type === HORIZON) {
                    if (this.optionDisplayHorizons.value === HprConstants.DISPLAY_HORIZONS_RELATIVE_VALUE) {
                        if (dataItem.isBasecase()) {
                            label = dataItem.constructBaseCaseString();
                        }
                        else {
                            label = dataItem.constructIntervalString(dataItem.interval, dataItem.horizonNumber);
                        }
                    }
                }
                return {id: dataItem.id, label: label, type: dataItem.type};
            }
            else {
                return {id: dataItem.id, type: dataItem.type};
            }
        } else return {};
    };

    /**
     * Does the display horizons option indicate that horizons should be displayed as relative
     * @returns {boolean}
     */
    sas.analytics.riskhp.models.CrosstabContext.prototype.isInRelativeMode = function() {
        return this.optionDisplayHorizons.value === HprConstants.DISPLAY_HORIZONS_RELATIVE_VALUE;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype._createDataItemLabelArray = function(dataItemArray) {
        var self = this;
        return _.map(dataItemArray, function(dataItem){
            return dataItem.getDisplayLabel(self.getOptionShowLabels().value);
        });
    };

    // Search thru column tree for any and all sort indications.  Currently, only one sort is
    // supported in the server.
    // If/When server support multicolumn sorts, this function is ready to go.
    sas.analytics.riskhp.models.CrosstabContext.prototype.buildSortJSON = function(includeLabel) {
        var self = this;

        var columnIndexes = self.getSortColumnIndexes();
        return _.map(columnIndexes, function(columnIndex, index, list){
            // var column = self.getColumn(columnIndex, false);
            // self.forceHorizonToBaseHorizon(column);
            var tuple = self.getTuple(columnIndex);
            var sortInfo = self.getSortColumn(columnIndex);
            return {direction: sortInfo.direction,
                type: sortInfo.type,
                tuple: {
                    name : "column" + columnIndex,
                    dataItems : self._createDataItemObjectsWithId(tuple, includeLabel)
                }
            };
        });
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.buildExpansionsJSON = function (forceFullQuery) {
        var self = this;
        var expansionJson = [];

        var _walkNodesForExpansionJson = function (node, path, expansionJson) {
            if (node) {
                if (node.expanded === true || (forceFullQuery === true && node.children.length > 0))  {
                    var qKey = self.createExpansionQueryKey(path, true);
                    expansionJson.push({
                        name : self.createExpansionName(qKey),
                        key: self.buildExpansionKeyJSON(qKey),
                        startRow: node.startRow,
                        maxRows: self.optionNROWS.value
                    });

                    for (var idx = 0; idx < node.children.length; idx++) {
                        var childPath = path.concat([node.children[idx].name]);
                        _walkNodesForExpansionJson(node.children[idx], childPath, expansionJson);
                    }
                }
            }
        };

        // Create a top level expansion
        var node;
        if (forceFullQuery) {
            node = self.reportResultsTree;
        }
        else {
            node = self.drillnode;
        }
        var path = self.findRowTreePath(node, "name");
        var topLevelKey = self.createExpansionQueryKey(path, false);

        expansionJson.push({
            name : self.createExpansionName(topLevelKey),
            key: self.buildExpansionKeyJSON(topLevelKey),
            startRow: 1,
            maxRows: self.optionNROWS.value
        });

        if (node) {
            // Create first expansion
            var qKey = self.createExpansionQueryKey(path, true);
            expansionJson.push({
                name : self.createExpansionName(qKey),
                key: self.buildExpansionKeyJSON(qKey),
                startRow: node.startRow,
                maxRows: self.optionNROWS.value
            });

            if (node.children) {
                for (var idx = 0; idx < node.children.length; idx++) {
                    var childPath = path.concat([node.children[idx].name]);
                    _walkNodesForExpansionJson(node.children[idx], childPath, expansionJson);
                }
            }
        }

        return expansionJson;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.buildExpansionJSON = function(anExpansion) {
        var self = this;
        return {name : anExpansion.name, key: self.buildExpansionKeyJSON(anExpansion.key), startRow: anExpansion.startRow, maxRows: anExpansion.maxRows};
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.buildExpansionKeyJSON = function(key) {
        var self = this;
        var newKey = [];
        for (var i = 0; i < key.length; i++) {
            var name = key[i];
            if ((name === self.i18n.getText('HPRisk.Crosstab.Total.txt')) ||
                (name === self.i18n.getText('HPRisk.Crosstab.Subtotal.txt'))) {
                name = SAS_ALL;
            }
            newKey.push({"id": self.classVariables[i].id, "value": name});
        }
        return newKey;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.buildOptionsJSON = function(full) {
        var crosstabOptions = [];
        if (full === undefined) {
            full = false;
        }

        // Create an array of options that should be sent on query
        crosstabOptions.push({"name": this.optionALPHA.name, "value": this.optionALPHA.value});
        crosstabOptions.push({"name": this.optionHIERARCHY.name, "value": this.optionHIERARCHY.value});
        crosstabOptions.push({"name": this.optionPCTLDEF.name, "value": this.optionPCTLDEF.value});
        crosstabOptions.push({"name": this.optionNROWS.name, "value": this.optionNROWS.value});

        // Necessary for export to excel
        if (full) {
            crosstabOptions.push({"name": this.optionTOTALS.name, "value": this.optionTOTALS.value});
            crosstabOptions.push({"name": this.optionSUBTOTALS.name, "value": this.optionSUBTOTALS.value});
            crosstabOptions.push({"name": this.optionLOCATION.name, "value": this.optionLOCATION.value});
            crosstabOptions.push({"name": this.optionTranspose.name, "value": this.optionTranspose.value});
            crosstabOptions.push({"name": this.optionIndentRows.name, "value": this.optionIndentRows.value});
        }

        return crosstabOptions;
    };

    //*****************************
    // Utils
    //*****************************

    sas.analytics.riskhp.models.CrosstabContext.prototype._hydrateDataItem = function(dataItem) {
        if (dataItem && this.cubeDataModel) {
            return this.cubeDataModel.getDataItemById(dataItem);
        }
        return dataItem;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype._dehydrateDataItem = function(dataItem) {
        if (dataItem) {
            return {id: dataItem.id, type: dataItem.type};
        }
        return dataItem;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype._hydrateDataItems = function(dehydratedDataItems) {
        var self = this;
        if (dehydratedDataItems && dehydratedDataItems.length > 0 && this.cubeDataModel) {
            var hydratedDataItems = [];
            _.each(dehydratedDataItems, function(dehydratedDataItem){
                var dataItem = self._hydrateDataItem(dehydratedDataItem.id);
                if (dataItem !== null) {
                    hydratedDataItems.push(dataItem);
                }
            });
            return hydratedDataItems;
        }
        return dehydratedDataItems;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype._dehydrateDataItems = function(hydratedDataItems) {
        var self = this;
        if (hydratedDataItems && hydratedDataItems.length > 0) {
            var dehydratedDataItems = [];
            _.each(hydratedDataItems, function(hydratedDataItem) {
                dehydratedDataItems.push(self._dehydrateDataItem(hydratedDataItem));
            });
            return dehydratedDataItems;
        }
        return hydratedDataItems;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype._hydrateTuples = function(tuples) {
        if (tuples && tuples.length > 0) {
            if (this.cubeDataModel) {
                return this.cubeDataModel.hydrateTuples(tuples);
            }
        }
        return tuples;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype._dehydrateTuples = function(tuples) {
        if (tuples && tuples.length > 0) {
            if(this.cubeDataModel) {
                return this.cubeDataModel.dehydrateTuples(tuples);
            }
        }
        return tuples;
    };

    sas.analytics.riskhp.models.CrosstabContext.prototype.fromTupleHeaderJSON = function(tupleObjects) {
        var self = this;

        // Start with just a root node of the column tree
        self.clearColumnTree();

        $.each(tupleObjects, function (index, tupleObject) {
            //hydrate
            var hydratedTuple = self._hydrateDataItems(tupleObject.tuple, self.cubeDataModel);

            // Do not force default column ordering
            self.addColumn(hydratedTuple, tupleObject.sort, tupleObject.width, false);
        });
    };

    // This function will restore the column header from JSON.  It will use the tuples if they are
    // present else it will fall back to the old method of using the column header tree.
    sas.analytics.riskhp.models.CrosstabContext.prototype.fromColumnTupleHeaderJSON = function(headers) {
        var self = this;

        // Create the wrapping deferred
        var returnDeferred = $.Deferred();

        if (headers.tuples && headers.tuples.length > 0) {
            $.when (
                self.fromTupleHeaderJSON(headers.tuples)
            ).done(function(){
                returnDeferred.resolve();
            });
        }
        else {
            if (headers.column) {
                $.when(
                    self.fromColumnHeaderJSON(headers.column)
                ).done(function () {
                    returnDeferred.resolve();
                });
            }
            else {
                returnDeferred.resolve();
            }
        }
        return returnDeferred;
    };


})();